@@ -1,9 +1,9 @@
 /** @file
- * VBoxGuest - VirtualBox Guest Additions Interface, 16-bit (OS/2) header.
+ * VBoxGuest - VirtualBox Guest Additions Driver Interface, 16-bit (OS/2) header.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,19 +21,12 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBox_VBoxGuest16_h
 #define ___VBox_VBoxGuest16_h
 
-#define RT_BIT(bit)                                (1UL << (bit))
-
-
-#define VMMDEV_VERSION                          0x00010004UL
+#define RT_BIT(bit)                             (1UL << (bit))
 
 #define VBOXGUEST_DEVICE_NAME                   "vboxgst$"
 
@@ -43,7 +36,7 @@ typedef struct VBGOS2IDC
     unsigned long u32Version;
     unsigned long u32Session;
     unsigned long pfnServiceEP;
-    short (__cdecl __far *fpfnServiceEP)(unsigned long u32Session, unsigned short iFunction, 
+    short (__cdecl __far *fpfnServiceEP)(unsigned long u32Session, unsigned short iFunction,
                                          void __far *fpvData, unsigned short cbData, unsigned short __far *pcbDataReturned);
     unsigned long fpfnServiceAsmEP;
 } VBGOS2IDC;
@@ -53,14 +46,6 @@ typedef VBGOS2IDC *PVBGOS2IDC;
 #define VBOXGUEST_IOCTL_VMMREQUEST              3
 #define VBOXGUEST_IOCTL_OS2_IDC_DISCONNECT      48
 
-
-#define VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED RT_BIT(0)
-#define VMMDEV_EVENT_HGCM                       RT_BIT(1)
-#define VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST     RT_BIT(2)
-#define VMMDEV_EVENT_JUDGE_CREDENTIALS          RT_BIT(3)
-#define VMMDEV_EVENT_RESTORED                   RT_BIT(4)
-
-
 #define VBOXGUEST_WAITEVENT_OK                  0
 #define VBOXGUEST_WAITEVENT_TIMEOUT             1
 #define VBOXGUEST_WAITEVENT_INTERRUPTED         2
@@ -90,10 +75,10 @@ typedef struct
 #define VMMDevReq_SetMouseStatus                2
 #define VMMDevReq_CtlGuestFilterMask            42
 
-#define VBOXGUEST_MOUSE_GUEST_CAN_ABSOLUTE      RT_BIT(0)
-#define VBOXGUEST_MOUSE_HOST_CAN_ABSOLUTE       RT_BIT(1)
-#define VBOXGUEST_MOUSE_GUEST_NEEDS_HOST_CURSOR RT_BIT(2)
-#define VBOXGUEST_MOUSE_HOST_CANNOT_HWPOINTER   RT_BIT(3)
+#define VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE      RT_BIT(0)
+#define VMMDEV_MOUSE_HOST_CAN_ABSOLUTE       RT_BIT(1)
+#define VMMDEV_MOUSE_GUEST_NEEDS_HOST_CURSOR RT_BIT(2)
+#define VMMDEV_MOUSE_HOST_CANNOT_HWPOINTER   RT_BIT(3)
 
 typedef struct
 {
@@ -110,5 +95,15 @@ typedef struct
     unsigned long u32NotMask;
 } VMMDevCtlGuestFilterMask;
 
+
+/* From VMMDev.h: */
+#define VMMDEV_VERSION                          0x00010004UL
+
+#define VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED RT_BIT(0)
+#define VMMDEV_EVENT_HGCM                       RT_BIT(1)
+#define VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST     RT_BIT(2)
+#define VMMDEV_EVENT_JUDGE_CREDENTIALS          RT_BIT(3)
+#define VMMDEV_EVENT_RESTORED                   RT_BIT(4)
+
 #endif
 
