@@ -1,9 +1,9 @@
 /** @file
- * DBGF - Debugger Facility.
+ * DBGF - Debugger Facility. (VMM)
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBox_dbgf_h
@@ -118,6 +114,7 @@ typedef const DBGFADDRESS *PCDBGFADDRESS;
 /** @} */
 
 VMMR3DECL(int)          DBGFR3AddrFromSelOff(PVM pVM, VMCPUID idCpu, PDBGFADDRESS pAddress, RTSEL Sel, RTUINTPTR off);
+VMMR3DECL(int)          DBGFR3AddrFromSelInfoOff(PVM pVM, PDBGFADDRESS pAddress, PCDBGFSELINFO pSelInfo, RTUINTPTR off);
 VMMR3DECL(PDBGFADDRESS) DBGFR3AddrFromFlat(PVM pVM, PDBGFADDRESS pAddress, RTGCUINTPTR FlatPtr);
 VMMR3DECL(PDBGFADDRESS) DBGFR3AddrFromPhys(PVM pVM, PDBGFADDRESS pAddress, RTGCPHYS PhysAddr);
 VMMR3DECL(bool)         DBGFR3AddrIsValid(PVM pVM, PCDBGFADDRESS pAddress);
@@ -292,7 +289,7 @@ VMMR3DECL(int)  DBGFR3EventAssertion(PVM pVM, DBGFEVENTTYPE enmEvent, const char
 VMMR3DECL(int)  DBGFR3EventBreakpoint(PVM pVM, DBGFEVENTTYPE enmEvent);
 VMMR3DECL(int)  DBGFR3Attach(PVM pVM);
 VMMR3DECL(int)  DBGFR3Detach(PVM pVM);
-VMMR3DECL(int)  DBGFR3EventWait(PVM pVM, unsigned cMillies, PCDBGFEVENT *ppEvent);
+VMMR3DECL(int)  DBGFR3EventWait(PVM pVM, RTMSINTERVAL cMillies, PCDBGFEVENT *ppEvent);
 VMMR3DECL(int)  DBGFR3Halt(PVM pVM);
 VMMR3DECL(bool) DBGFR3IsHalted(PVM pVM);
 VMMR3DECL(bool) DBGFR3CanWait(PVM pVM);
@@ -411,6 +408,10 @@ VMMDECL(bool)           DBGFIsStepping(PVMCPU pVCpu);
 
 
 
+VMMR3DECL(CPUMMODE)     DBGFR3CpuGetMode(PVM pVM, VMCPUID idCpu);
+
+
+
 
 /** Pointer to a info helper callback structure. */
 typedef struct DBGFINFOHLP *PDBGFINFOHLP;
@@ -445,7 +446,7 @@ typedef struct DBGFINFOHLP
 /**
  * Info handler, device version.
  *
- * @param   pDevIns     Device instance which registered the info.
+ * @param   pDevIns     The device instance which registered the info.
  * @param   pHlp        Callback functions for doing output.
  * @param   pszArgs     Argument string. Optional and specific to the handler.
  */
@@ -453,10 +454,21 @@ typedef DECLCALLBACK(void) FNDBGFHANDLERDEV(PPDMDEVINS pDevIns, PCDBGFINFOHLP pH
 /** Pointer to a FNDBGFHANDLERDEV function. */
 typedef FNDBGFHANDLERDEV  *PFNDBGFHANDLERDEV;
 
+/**
+ * Info handler, USB device version.
+ *
+ * @param   pUsbIns     The USB device instance which registered the info.
+ * @param   pHlp        Callback functions for doing output.
+ * @param   pszArgs     Argument string. Optional and specific to the handler.
+ */
+typedef DECLCALLBACK(void) FNDBGFHANDLERUSB(PPDMUSBINS pUsbIns, PCDBGFINFOHLP pHlp, const char *pszArgs);
+/** Pointer to a FNDBGFHANDLERUSB function. */
+typedef FNDBGFHANDLERUSB  *PFNDBGFHANDLERUSB;
+
 /**
  * Info handler, driver version.
  *
- * @param   pDrvIns     Driver instance which registered the info.
+ * @param   pDrvIns     The driver instance which registered the info.
  * @param   pHlp        Callback functions for doing output.
  * @param   pszArgs     Argument string. Optional and specific to the handler.
  */
@@ -637,6 +649,10 @@ VMMR3DECL(int)          DBGFR3AsLoadImage(PVM pVM, RTDBGAS hDbgAs, const char *p
 VMMR3DECL(int)          DBGFR3AsLoadMap(PVM pVM, RTDBGAS hDbgAs, const char *pszFilename, const char *pszModName, PCDBGFADDRESS pModAddress, RTDBGSEGIDX iModSeg, RTGCUINTPTR uSubtrahend, uint32_t fFlags);
 VMMR3DECL(int)          DBGFR3AsLinkModule(PVM pVM, RTDBGAS hDbgAs, RTDBGMOD hMod, PCDBGFADDRESS pModAddress, RTDBGSEGIDX iModSeg, uint32_t fFlags);
 
+VMMR3DECL(int)          DBGFR3AsSymbolByAddr(PVM pVM, RTDBGAS hDbgAs, PCDBGFADDRESS pAddress, PRTGCINTPTR poffDisp, PRTDBGSYMBOL pSymbol, PRTDBGMOD phMod);
+VMMR3DECL(PRTDBGSYMBOL) DBGFR3AsSymbolByAddrA(PVM pVM, RTDBGAS hDbgAs, PCDBGFADDRESS pAddress, PRTGCINTPTR poffDisp, PRTDBGMOD phMod);
+VMMR3DECL(int)          DBGFR3AsSymbolByName(PVM pVM, RTDBGAS hDbgAs, const char *pszSymbol, PRTDBGSYMBOL pSymbol, PRTDBGMOD phMod);
+
 /* The following are soon to be obsoleted: */
 VMMR3DECL(int)          DBGFR3ModuleLoad(PVM pVM, const char *pszFilename, RTGCUINTPTR AddressDelta, const char *pszName, RTGCUINTPTR ModuleAddress, unsigned cbImage);
 VMMR3DECL(void)         DBGFR3ModuleRelocate(PVM pVM, RTGCUINTPTR OldImageBase, RTGCUINTPTR NewImageBase, RTGCUINTPTR cbImage,
@@ -644,9 +660,7 @@ VMMR3DECL(void)         DBGFR3ModuleRelocate(PVM pVM, RTGCUINTPTR OldImageBase,
 VMMR3DECL(int)          DBGFR3SymbolAdd(PVM pVM, RTGCUINTPTR ModuleAddress, RTGCUINTPTR SymbolAddress, RTUINT cbSymbol, const char *pszSymbol);
 VMMR3DECL(int)          DBGFR3SymbolByAddr(PVM pVM, RTGCUINTPTR Address, PRTGCINTPTR poffDisplacement, PDBGFSYMBOL pSymbol);
 VMMR3DECL(int)          DBGFR3SymbolByName(PVM pVM, const char *pszSymbol, PDBGFSYMBOL pSymbol);
-VMMR3DECL(PDBGFSYMBOL)  DBGFR3SymbolByAddrAlloc(PVM pVM, RTGCUINTPTR Address, PRTGCINTPTR poffDisplacement);
-VMMR3DECL(PDBGFSYMBOL)  DBGFR3SymbolByNameAlloc(PVM pVM, const char *pszSymbol);
-VMMR3DECL(void)         DBGFR3SymbolFree(PDBGFSYMBOL pSymbol);
+
 VMMR3DECL(int)          DBGFR3LineByAddr(PVM pVM, RTGCUINTPTR Address, PRTGCINTPTR poffDisplacement, PDBGFLINE pLine);
 VMMR3DECL(PDBGFLINE)    DBGFR3LineByAddrAlloc(PVM pVM, RTGCUINTPTR Address, PRTGCINTPTR poffDisplacement);
 VMMR3DECL(void)         DBGFR3LineFree(PDBGFLINE pLine);
@@ -724,9 +738,9 @@ typedef struct DBGFSTACKFRAME const  *PCDBGFSTACKFRAME;
 typedef struct DBGFSTACKFRAME
 {
     /** Frame number. */
-    RTUINT          iFrame;
+    uint32_t        iFrame;
     /** Frame flags. */
-    RTUINT          fFlags;
+    uint32_t        fFlags;
     /** The frame address.
      * The off member is [e|r]bp and the Sel member is ss. */
     DBGFADDRESS     AddrFrame;
@@ -737,7 +751,7 @@ typedef struct DBGFSTACKFRAME
      * The off member is [e|r]ip and the Sel member is cs. */
     DBGFADDRESS     AddrPC;
     /** Pointer to the symbol nearest the program counter (PC). NULL if not found. */
-    PDBGFSYMBOL     pSymPC;
+    PRTDBGSYMBOL    pSymPC;
     /** Pointer to the linnumber nearest the program counter (PC). NULL if not found. */
     PDBGFLINE       pLinePC;
 
@@ -754,7 +768,7 @@ typedef struct DBGFSTACKFRAME
      * The off member is [e|r]ip and the Sel member is cs. */
     DBGFADDRESS     AddrReturnPC;
     /** Pointer to the symbol nearest the return PC. NULL if not found. */
-    PDBGFSYMBOL     pSymReturnPC;
+    PRTDBGSYMBOL    pSymReturnPC;
     /** Pointer to the linnumber nearest the return PC. NULL if not found. */
     PDBGFLINE       pLineReturnPC;
 
@@ -791,6 +805,12 @@ typedef struct DBGFSTACKFRAME
 #define DBGFSTACKFRAME_FLAGS_LOOP       RT_BIT(2)
 /** This is the last record because we reached the maximum depth. */
 #define DBGFSTACKFRAME_FLAGS_MAX_DEPTH  RT_BIT(3)
+/** 16-bit frame. */
+#define DBGFSTACKFRAME_FLAGS_16BIT      RT_BIT(4)
+/** 32-bit frame. */
+#define DBGFSTACKFRAME_FLAGS_32BIT      RT_BIT(5)
+/** 64-bit frame. */
+#define DBGFSTACKFRAME_FLAGS_64BIT      RT_BIT(6)
 /** @} */
 
 /** @name DBGFCODETYPE
@@ -875,7 +895,8 @@ VMMR3DECL(int) DBGFR3DisasInstrLogInternal(PVMCPU pVCpu, RTSEL Sel, RTGCPTR GCPt
 #endif
 
 
-VMMR3DECL(int) DBGFR3MemScan(PVM pVM, VMCPUID idCpu, PCDBGFADDRESS pAddress, RTGCUINTPTR cbRange, const uint8_t *pabNeedle, size_t cbNeedle, PDBGFADDRESS pHitAddress);
+VMMR3DECL(int) DBGFR3MemScan(PVM pVM, VMCPUID idCpu, PCDBGFADDRESS pAddress, RTGCUINTPTR cbRange, RTGCUINTPTR uAlign,
+                             const void *pvNeedle, size_t cbNeedle, PDBGFADDRESS pHitAddress);
 VMMR3DECL(int) DBGFR3MemRead(PVM pVM, VMCPUID idCpu, PCDBGFADDRESS pAddress, void *pvBuf, size_t cbRead);
 VMMR3DECL(int) DBGFR3MemReadString(PVM pVM, VMCPUID idCpu, PCDBGFADDRESS pAddress, char *pszBuf, size_t cbBuf);
 VMMR3DECL(int) DBGFR3MemWrite(PVM pVM, VMCPUID idCpu, PCDBGFADDRESS pAddress, void const *pvBuf, size_t cbRead);
@@ -884,10 +905,12 @@ VMMR3DECL(int) DBGFR3MemWrite(PVM pVM, VMCPUID idCpu, PCDBGFADDRESS pAddress, vo
 /** @name DBGFR3SelQueryInfo flags.
  * @{ */
 /** Get the info from the guest descriptor table. */
-#define DBGFSELQI_FLAGS_DT_GUEST        UINT32_C(0)
+#define DBGFSELQI_FLAGS_DT_GUEST            UINT32_C(0)
 /** Get the info from the shadow descriptor table.
  * Only works in raw-mode.  */
-#define DBGFSELQI_FLAGS_DT_SHADOW       UINT32_C(1)
+#define DBGFSELQI_FLAGS_DT_SHADOW           UINT32_C(1)
+/** If currently executing in in 64-bit mode, blow up data selectors. */
+#define DBGFSELQI_FLAGS_DT_ADJ_64BIT_MODE   UINT32_C(2)
 /** @} */
 VMMR3DECL(int) DBGFR3SelQueryInfo(PVM pVM, VMCPUID idCpu, RTSEL Sel, uint32_t fFlags, PDBGFSELINFO pSelInfo);
 
