@@ -1,9 +1,9 @@
 /** @file
- * PATM - Dynamic Guest OS Patching Manager
+ * PATM - Dynamic Guest OS Patching Manager. (VMM)
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -21,10 +21,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBox_patm_h
@@ -35,7 +31,7 @@
 #include <VBox/dis.h>
 
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 /** @defgroup grp_patm      The Patch Manager API
  * @{
@@ -193,7 +189,7 @@ VMMDECL(RCPTRTYPE(PPATMGCSTATE)) PATMQueryGCState(PVM pVM);
  * @param   pVM         The VM to operate on.
  * @param   pAddr       Guest context address
  */
-VMMDECL(bool) PATMIsPatchGCAddr(PVM pVM, RTRCPTR pAddr);
+VMMDECL(bool) PATMIsPatchGCAddr(PVM pVM, RTRCUINTPTR pAddr);
 
 /**
  * Check if we must use raw mode (patch code being executed or marked safe for IF=0)
@@ -670,7 +666,7 @@ VMMR3DECL(bool) PATMIsSpinlockReleasePatch(PVM pVM, RTRCPTR pCallTargetGC);
 
 
 /** @} */
-__END_DECLS
+RT_C_DECLS_END
 
 
 #endif
