@@ -4,7 +4,7 @@
  *
  * Display driver screen draw entry points.
  *
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /* The driver operates in 3 modes:
@@ -143,16 +139,16 @@ BOOL bIsScreenSurface (SURFOBJ *pso)
         {                                                              \
             vbva##__fn __a;                                            \
                                                                        \
-            if (  ppdev->pVBVA->u32HostEvents                           \
+            if (  ppdev->pVBVA->hostFlags.u32HostEvents                \
                 & VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET)            \
             {                                                          \
                 vrdpReset (ppdev);                                     \
                                                                        \
-                ppdev->pVBVA->u32HostEvents &=                          \
+                ppdev->pVBVA->hostFlags.u32HostEvents &=               \
                           ~VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET;   \
             }                                                          \
                                                                        \
-            if (ppdev->pVBVA->u32HostEvents                             \
+            if (ppdev->pVBVA->hostFlags.u32HostEvents                  \
                 & VBVA_F_MODE_VRDP)                                    \
             {                                                          \
                 vrdp##__fn __a;                                        \
@@ -365,7 +361,7 @@ BOOL APIENTRY DrvCopyBits(
         DISPDBG((1, "offscreen->screen\n"));
 
         if (   ppdev->pVBVA
-            && (ppdev->pVBVA->u32HostEvents & VBVA_F_MODE_ENABLED))
+            && (ppdev->pVBVA->hostFlags.u32HostEvents & VBVA_F_MODE_ENABLED))
 #endif /* VBOX_WITH_HGSMI */
         {
             if (   (psoSrc->fjBitmap & BMF_DONTCACHE) != 0
@@ -792,18 +788,18 @@ BOOL APIENTRY DrvRealizeBrush(
         }
 #else
         if (   ppdev->pVBVA
-            && (ppdev->pVBVA->u32HostEvents & VBVA_F_MODE_ENABLED))
+            && (ppdev->pVBVA->hostFlags.u32HostEvents & VBVA_F_MODE_ENABLED))
         {
-            if (ppdev->pVBVA->u32HostEvents
+            if (ppdev->pVBVA->hostFlags.u32HostEvents
                 & VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET)
             {
                 vrdpReset (ppdev);
 
-                ppdev->pVBVA->u32HostEvents &=
+                ppdev->pVBVA->hostFlags.u32HostEvents &=
                     ~VBOX_VIDEO_INFO_HOST_EVENTS_F_VRDP_RESET;
             }
 
-            if (ppdev->pVBVA->u32HostEvents
+            if (ppdev->pVBVA->hostFlags.u32HostEvents
                 & VBVA_F_MODE_VRDP)
             {
                 bRc = vrdpRealizeBrush (pbo, psoTarget, psoPattern, psoMask, pxlo, iHatch);
