@@ -1,10 +1,10 @@
+/* $Id: vbox.c 29260 2010-05-09 18:51:24Z vboxsync $ */
 /** @file
- *
- * VBoxGuest -- VirtualBox Win 2000/XP guest display driver
- *
- * VBox support functions.
- *
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Display - VirtualBox Win 2000/XP guest display driver, support functions.
+ */
+
+/*
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,17 +13,15 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "driver.h"
 
+#include <VBox/VMMDev.h>
 #include <VBox/VBoxGuest.h>
 #include <VBox/err.h>
 #include <iprt/asm.h>
+#include <iprt/asm-amd64-x86.h>
 
 /*
  * There is a hardware ring buffer in the VBox VMMDev PCI memory space.
@@ -343,8 +341,9 @@ static void vboxHGSMIBufferSubmit (PPDEV ppdev, void *p)
 {
     HGSMIOFFSET offBuffer = HGSMIHeapBufferOffset (&ppdev->hgsmiDisplayHeap, p);
 
-    ASMOutU16 (VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_VBVA_GUEST);
-    ASMOutU32 (VBE_DISPI_IOPORT_DATA, offBuffer);
+    DISPDBG((0, "VBoxDISP::vboxHGSMIBufferSubmit: offset 0x%x\n", offBuffer));
+
+    ASMOutU32(ppdev->IOPortGuestCommand, offBuffer);
 }
 
 static BOOL vboxVBVAInformHost (PPDEV ppdev, BOOL bEnable)
@@ -367,12 +366,20 @@ static BOOL vboxVBVAInformHost (PPDEV ppdev, BOOL bEnable)
 
             pEnable->u32Flags  = bEnable? VBVA_F_ENABLE: VBVA_F_DISABLE;
             pEnable->u32Offset = ppdev->layout.offVBVABuffer;
+            pEnable->i32Result = VERR_NOT_SUPPORTED;
 
             vboxHGSMIBufferSubmit (ppdev, p);
 
-            HGSMIHeapFree (&ppdev->hgsmiDisplayHeap, p);
+            if (bEnable)
+            {
+                bRc = RT_SUCCESS(pEnable->i32Result);
+            }
+            else
+            {
+                bRc = TRUE;
+            }
 
-            bRc = TRUE;
+            HGSMIHeapFree (&ppdev->hgsmiDisplayHeap, p);
         }
     }
 
@@ -386,14 +393,16 @@ BOOL vboxVbvaEnable (PPDEV ppdev)
 {
     BOOL bRc = FALSE;
 
-    DISPDBG((1, "VBoxDisp::vboxVbvaEnable called\n"));
+    DISPDBG((1, "VBoxDisp::vboxVbvaEnable called ppdev %p, hgsmi %d, vbva %p\n", ppdev, ppdev->bHGSMISupported, ppdev->pVBVA));
 
     if (ppdev->bHGSMISupported)
     {
         VBVABUFFER *pVBVA = (VBVABUFFER *)((uint8_t *)ppdev->pjScreen + ppdev->layout.offVBVABuffer);
 
-        pVBVA->u32HostEvents      = 0;
-        pVBVA->u32SupportedOrders = 0;
+        DISPDBG((1, "VBoxDisp::vboxVbvaEnable screen %p vbva off 0x%x\n", ppdev->pjScreen, ppdev->layout.offVBVABuffer));
+
+        pVBVA->hostFlags.u32HostEvents      = 0;
+        pVBVA->hostFlags.u32SupportedOrders = 0;
         pVBVA->off32Data          = 0;
         pVBVA->off32Free          = 0;
         RtlZeroMemory (pVBVA->aRecords, sizeof (pVBVA->aRecords));
@@ -438,7 +447,7 @@ BOOL vboxHwBufferBeginUpdate (PPDEV ppdev)
     //          ppdev->pVBVA? ppdev->pVBVA->u32HostEvents: -1));
 
     if (   ppdev->pVBVA
-        && (ppdev->pVBVA->u32HostEvents & VBVA_F_MODE_ENABLED))
+        && (ppdev->pVBVA->hostFlags.u32HostEvents & VBVA_F_MODE_ENABLED))
     {
         uint32_t indexRecordNext;
 
@@ -514,7 +523,7 @@ static void vboxHwBufferFlush (PPDEV ppdev)
 {
     /* Issue the flush command. */
     void *p = HGSMIHeapAlloc (&ppdev->hgsmiDisplayHeap,
-                              sizeof (VBVA_FLUSH),
+                              sizeof (VBVAFLUSH),
                               HGSMI_CH_VBVA,
                               VBVA_FLUSH);
     if (!p)
@@ -646,7 +655,7 @@ BOOL vboxOrderSupported (PPDEV ppdev, unsigned code)
         return FALSE;
     }
 
-    if (pVBVA->u32SupportedOrders & (1 << code))
+    if (pVBVA->hostFlags.u32SupportedOrders & (1 << code))
     {
         return TRUE;
     }
@@ -665,7 +674,7 @@ void VBoxProcessDisplayInfo (PPDEV ppdev)
                                   VBVA_INFO_SCREEN);
         if (!p)
         {
-            DISPDBG((0, "VBoxDISP::vboxHwBufferFlush: HGSMIHeapAlloc failed\n"));
+            DISPDBG((0, "VBoxDISP::VBoxProcessDisplayInfo: HGSMIHeapAlloc failed\n"));
         }
         else
         {
@@ -674,6 +683,7 @@ void VBoxProcessDisplayInfo (PPDEV ppdev)
             pScreen->u32ViewIndex    = ppdev->iDevice;
             pScreen->i32OriginX      = ppdev->ptlDevOrg.x;
             pScreen->i32OriginY      = ppdev->ptlDevOrg.y;
+            pScreen->u32StartOffset  = 0;
             pScreen->u32LineSize     = ppdev->lDeltaScreen > 0?ppdev->lDeltaScreen: -ppdev->lDeltaScreen;
             pScreen->u32Width        = ppdev->cxScreen;
             pScreen->u32Height       = ppdev->cyScreen;
@@ -707,6 +717,7 @@ VBOXVHWACMD* vboxVHWACommandCreate (PPDEV ppdev, VBOXVHWACMD_TYPE enmCmd, VBOXVH
         pHdr->iDisplay = ppdev->iDevice;
         pHdr->rc = VERR_GENERAL_FAILURE;
         pHdr->enmCmd = enmCmd;
+        pHdr->cRefs = 1;
     }
 
     /* temporary hack */
@@ -722,8 +733,8 @@ void vboxVHWACommandFree (PPDEV ppdev, VBOXVHWACMD* pCmd)
 
 static DECLCALLBACK(void) vboxVHWACommandCompletionCallbackEvent(PPDEV ppdev, VBOXVHWACMD * pCmd, void * pContext)
 {
-    PEVENT pEvent = (PEVENT)pContext;
-    LONG oldState = EngSetEvent(pEvent);
+    VBOXPEVENT pEvent = (VBOXPEVENT)pContext;
+    LONG oldState = ppdev->VideoPortProcs.pfnSetEvent(ppdev->pVideoPortContext, pEvent);
     Assert(!oldState);
 }
 
@@ -755,6 +766,7 @@ static void vboxVBVAHostCommandHanlder(PPDEV ppdev, VBVAHOSTCMD * pCmd)
 # endif
         default:
         {
+            Assert(0);
             vboxVBVAHostCommandComplete(ppdev, pCmd);
         }
     }
@@ -762,54 +774,60 @@ static void vboxVBVAHostCommandHanlder(PPDEV ppdev, VBVAHOSTCMD * pCmd)
 
 void vboxVHWACommandCheckHostCmds(PPDEV ppdev)
 {
-    VBVAHOSTCMD * pCmd;
+    VBVAHOSTCMD * pCmd, * pNextCmd;
     int rc = ppdev->pfnHGSMIRequestCommands(ppdev->hMpHGSMI, HGSMI_CH_VBVA, &pCmd);
+    /* don't assert here, otherwise NT4 will be unhappy */
     if(RT_SUCCESS(rc))
     {
-        for(; pCmd; pCmd = pCmd->u.pNext)
+        for(;pCmd; pCmd = pNextCmd)
         {
+            pNextCmd = pCmd->u.pNext;
             vboxVBVAHostCommandHanlder(ppdev, pCmd);
         }
     }
 }
 
-void vboxVHWACommandSubmitAsynchByEvent (PPDEV ppdev, VBOXVHWACMD* pCmd, PEVENT pEvent)
+void vboxVHWACommandSubmitAsynchByEvent (PPDEV ppdev, VBOXVHWACMD* pCmd, VBOXPEVENT pEvent)
 {
 //    Assert(0);
     pCmd->GuestVBVAReserved1 = (uintptr_t)pEvent;
     pCmd->GuestVBVAReserved2 = 0;
+    /* ensure the command is not removed until we're processing it */
+    vbvaVHWACommandRetain(ppdev, pCmd);
 
     /* complete it asynchronously by setting event */
-    pCmd->Flags = VBOXVHWACMD_FLAG_ASYNCH_EVENT;
+    pCmd->Flags |= VBOXVHWACMD_FLAG_GH_ASYNCH_EVENT;
     vboxHGSMIBufferSubmit (ppdev, pCmd);
 
-    if(!(ASMAtomicReadU32((volatile uint32_t *)&pCmd->Flags)  & VBOXVHWACMD_FLAG_ASYNCH))
+    if(!(ASMAtomicReadU32((volatile uint32_t *)&pCmd->Flags)  & VBOXVHWACMD_FLAG_HG_ASYNCH))
     {
         /* the command is completed */
-        EngSetEvent(pEvent);
+        ppdev->VideoPortProcs.pfnSetEvent(ppdev->pVideoPortContext, pEvent);
     }
+
+    vbvaVHWACommandRelease(ppdev, pCmd);
 }
 
 BOOL vboxVHWACommandSubmit (PPDEV ppdev, VBOXVHWACMD* pCmd)
 {
-    PEVENT pEvent;
-    BOOL brc = EngCreateEvent(&pEvent);
-    Assert(brc);
-
-    if(brc)
+    VBOXPEVENT pEvent;
+    VBOXVP_STATUS rc = ppdev->VideoPortProcs.pfnCreateEvent(ppdev->pVideoPortContext, VBOXNOTIFICATION_EVENT, NULL, &pEvent);
+    /* don't assert here, otherwise NT4 will be unhappy */
+    if(rc == VBOXNO_ERROR)
     {
+        pCmd->Flags |= VBOXVHWACMD_FLAG_GH_ASYNCH_IRQ;
         vboxVHWACommandSubmitAsynchByEvent (ppdev, pCmd, pEvent);
 
-        brc = EngWaitForSingleObject(pEvent,
+        rc = ppdev->VideoPortProcs.pfnWaitForSingleObject(ppdev->pVideoPortContext, pEvent,
                 NULL /*IN PLARGE_INTEGER  pTimeOut*/
                 );
-        Assert(brc);
-        if(brc)
+        Assert(rc == VBOXNO_ERROR);
+        if(rc == VBOXNO_ERROR)
         {
-            EngDeleteEvent(pEvent);
+            ppdev->VideoPortProcs.pfnDeleteEvent(ppdev->pVideoPortContext, pEvent);
         }
     }
-    return brc;
+    return rc == VBOXNO_ERROR;
 }
 
 /* do not wait for completion */
@@ -818,38 +836,79 @@ void vboxVHWACommandSubmitAsynch (PPDEV ppdev, VBOXVHWACMD* pCmd, PFNVBOXVHWACMD
 //    Assert(0);
     pCmd->GuestVBVAReserved1 = (uintptr_t)pfnCompletion;
     pCmd->GuestVBVAReserved2 = (uintptr_t)pContext;
+    vbvaVHWACommandRetain(ppdev, pCmd);
 
-    pCmd->Flags = 0;
     vboxHGSMIBufferSubmit (ppdev, pCmd);
 
-    if(!(pCmd->Flags  & VBOXVHWACMD_FLAG_ASYNCH))
+    if(!(pCmd->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH))
     {
         /* the command is completed */
         pfnCompletion(ppdev, pCmd, pContext);
     }
+
+    vbvaVHWACommandRelease(ppdev, pCmd);
+}
+
+static DECLCALLBACK(void) vboxVHWAFreeCmdCompletion(PPDEV ppdev, VBOXVHWACMD * pCmd, void * pContext)
+{
+    vbvaVHWACommandRelease(ppdev, pCmd);
+}
+
+void vboxVHWACommandSubmitAsynchAndComplete (PPDEV ppdev, VBOXVHWACMD* pCmd)
+{
+//    Assert(0);
+    pCmd->GuestVBVAReserved1 = (uintptr_t)vboxVHWAFreeCmdCompletion;
+
+    vbvaVHWACommandRetain(ppdev, pCmd);
+
+    pCmd->Flags |= VBOXVHWACMD_FLAG_GH_ASYNCH_NOCOMPLETION;
+
+    vboxHGSMIBufferSubmit (ppdev, pCmd);
+
+    if(!(pCmd->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH)
+            || pCmd->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH_RETURNED)
+    {
+        /* the command is completed */
+        vboxVHWAFreeCmdCompletion(ppdev, pCmd, NULL);
+    }
+
+    vbvaVHWACommandRelease(ppdev, pCmd);
 }
 
 void vboxVHWAFreeHostInfo1(PPDEV ppdev, VBOXVHWACMD_QUERYINFO1* pInfo)
 {
     VBOXVHWACMD* pCmd = VBOXVHWACMD_HEAD(pInfo);
-    vboxVHWACommandFree (ppdev, pCmd);
+    vbvaVHWACommandRelease (ppdev, pCmd);
 }
 
 void vboxVHWAFreeHostInfo2(PPDEV ppdev, VBOXVHWACMD_QUERYINFO2* pInfo)
 {
     VBOXVHWACMD* pCmd = VBOXVHWACMD_HEAD(pInfo);
-    vboxVHWACommandFree (ppdev, pCmd);
+    vbvaVHWACommandRelease (ppdev, pCmd);
 }
 
 VBOXVHWACMD_QUERYINFO1* vboxVHWAQueryHostInfo1(PPDEV ppdev)
 {
     VBOXVHWACMD* pCmd = vboxVHWACommandCreate (ppdev, VBOXVHWACMD_TYPE_QUERY_INFO1, sizeof(VBOXVHWACMD_QUERYINFO1));
+    VBOXVHWACMD_QUERYINFO1 *pInfo1;
     if (!pCmd)
     {
         DISPDBG((0, "VBoxDISP::vboxVHWAQueryHostInfo1: vboxVHWACommandCreate failed\n"));
         return NULL;
     }
 
+    if (!pCmd)
+    {
+        DISPDBG((0, "VBoxDISP::vboxVHWAQueryHostInfo1: vboxVHWACommandCreate failed\n"));
+        return NULL;
+    }
+
+    pInfo1 = VBOXVHWACMD_BODY(pCmd, VBOXVHWACMD_QUERYINFO1);
+    pInfo1->u.in.guestVersion.maj = VBOXVHWA_VERSION_MAJ;
+    pInfo1->u.in.guestVersion.min = VBOXVHWA_VERSION_MIN;
+    pInfo1->u.in.guestVersion.bld = VBOXVHWA_VERSION_BLD;
+    pInfo1->u.in.guestVersion.reserved = VBOXVHWA_VERSION_RSV;
+
     if(vboxVHWACommandSubmit (ppdev, pCmd))
     {
         if(RT_SUCCESS(pCmd->rc))
@@ -858,17 +917,17 @@ VBOXVHWACMD_QUERYINFO1* vboxVHWAQueryHostInfo1(PPDEV ppdev)
         }
     }
 
-    vboxVHWACommandFree (ppdev, pCmd);
+    vbvaVHWACommandRelease (ppdev, pCmd);
     return NULL;
 }
 
 VBOXVHWACMD_QUERYINFO2* vboxVHWAQueryHostInfo2(PPDEV ppdev, uint32_t numFourCC)
 {
-    VBOXVHWACMD* pCmd = vboxVHWACommandCreate (ppdev, VBOXVHWACMD_TYPE_QUERY_INFO2, sizeof(VBOXVHWACMD_QUERYINFO2));
+    VBOXVHWACMD* pCmd = vboxVHWACommandCreate (ppdev, VBOXVHWACMD_TYPE_QUERY_INFO2, VBOXVHWAINFO2_SIZE(numFourCC));
     VBOXVHWACMD_QUERYINFO2 *pInfo2;
     if (!pCmd)
     {
-        DISPDBG((0, "VBoxDISP::vboxVHWAQueryHostInfo1: vboxVHWACommandCreate failed\n"));
+        DISPDBG((0, "VBoxDISP::vboxVHWAQueryHostInfo2: vboxVHWACommandCreate failed\n"));
         return NULL;
     }
 
@@ -886,37 +945,48 @@ VBOXVHWACMD_QUERYINFO2* vboxVHWAQueryHostInfo2(PPDEV ppdev, uint32_t numFourCC)
         }
     }
 
-    vboxVHWACommandFree (ppdev, pCmd);
+    vbvaVHWACommandRelease (ppdev, pCmd);
     return NULL;
 }
 
 int vboxVHWAInitHostInfo1(PPDEV ppdev)
 {
-    VBOXVHWACMD_QUERYINFO1* pInfo = vboxVHWAQueryHostInfo1(ppdev);
+    VBOXVHWACMD_QUERYINFO1* pInfo;
+
+    if (!ppdev->bHGSMISupported)
+        return VERR_NOT_SUPPORTED;
+
+    pInfo = vboxVHWAQueryHostInfo1(ppdev);
     if(!pInfo)
     {
         ppdev->vhwaInfo.bVHWAEnabled = false;
         return VERR_OUT_OF_RESOURCES;
     }
 
-    ppdev->vhwaInfo.caps = pInfo->caps;
-    ppdev->vhwaInfo.colorKeyCaps = pInfo->colorKeyCaps;
-    ppdev->vhwaInfo.stretchCaps = pInfo->stretchCaps;
-    ppdev->vhwaInfo.surfaceCaps = pInfo->surfaceCaps;
-    ppdev->vhwaInfo.numOverlays = pInfo->numOverlays;
-    ppdev->vhwaInfo.numFourCC = pInfo->numFourCC;
-    ppdev->vhwaInfo.bVHWAEnabled = (pInfo->cfgFlags & VBOXVHWA_CFG_ENABLED);
-
+    ppdev->vhwaInfo.caps = pInfo->u.out.caps;
+    ppdev->vhwaInfo.caps2 = pInfo->u.out.caps2;
+    ppdev->vhwaInfo.colorKeyCaps = pInfo->u.out.colorKeyCaps;
+    ppdev->vhwaInfo.stretchCaps = pInfo->u.out.stretchCaps;
+    ppdev->vhwaInfo.surfaceCaps = pInfo->u.out.surfaceCaps;
+    ppdev->vhwaInfo.numOverlays = pInfo->u.out.numOverlays;
+    ppdev->vhwaInfo.numFourCC = pInfo->u.out.numFourCC;
+    ppdev->vhwaInfo.bVHWAEnabled = (pInfo->u.out.cfgFlags & VBOXVHWA_CFG_ENABLED);
     vboxVHWAFreeHostInfo1(ppdev, pInfo);
     return VINF_SUCCESS;
 }
 
 int vboxVHWAInitHostInfo2(PPDEV ppdev, DWORD *pFourCC)
 {
-    VBOXVHWACMD_QUERYINFO2* pInfo = vboxVHWAQueryHostInfo2(ppdev, ppdev->vhwaInfo.numFourCC);
+    VBOXVHWACMD_QUERYINFO2* pInfo;
     int rc = VINF_SUCCESS;
 
-    if(pInfo)
+    if (!ppdev->bHGSMISupported)
+        return VERR_NOT_SUPPORTED;
+
+    pInfo = vboxVHWAQueryHostInfo2(ppdev, ppdev->vhwaInfo.numFourCC);
+
+    Assert(pInfo);
+    if(!pInfo)
         return VERR_OUT_OF_RESOURCES;
 
     if(ppdev->vhwaInfo.numFourCC)
@@ -937,11 +1007,16 @@ int vboxVHWAInitHostInfo2(PPDEV ppdev, DWORD *pFourCC)
 int vboxVHWAEnable(PPDEV ppdev)
 {
     int rc = VERR_GENERAL_FAILURE;
-    VBOXVHWACMD* pCmd = vboxVHWACommandCreate (ppdev, VBOXVHWACMD_TYPE_ENABLE, 0);
+    VBOXVHWACMD* pCmd;
+
+    if (!ppdev->bHGSMISupported)
+        return VERR_NOT_SUPPORTED;
+
+    pCmd = vboxVHWACommandCreate (ppdev, VBOXVHWACMD_TYPE_ENABLE, 0);
     if (!pCmd)
     {
-        DISPDBG((0, "VBoxDISP::vboxVHWAQueryHostInfo1: vboxVHWACommandCreate failed\n"));
-        return NULL;
+        DISPDBG((0, "VBoxDISP::vboxVHWAEnable: vboxVHWACommandCreate failed\n"));
+        return rc;
     }
 
     if(vboxVHWACommandSubmit (ppdev, pCmd))
@@ -952,18 +1027,23 @@ int vboxVHWAEnable(PPDEV ppdev)
         }
     }
 
-    vboxVHWACommandFree (ppdev, pCmd);
+    vbvaVHWACommandRelease (ppdev, pCmd);
     return rc;
 }
 
 int vboxVHWADisable(PPDEV ppdev)
 {
     int rc = VERR_GENERAL_FAILURE;
-    VBOXVHWACMD* pCmd = vboxVHWACommandCreate (ppdev, VBOXVHWACMD_TYPE_DISABLE, 0);
+    VBOXVHWACMD* pCmd;
+
+    if (!ppdev->bHGSMISupported)
+        return VERR_NOT_SUPPORTED;
+
+    pCmd = vboxVHWACommandCreate (ppdev, VBOXVHWACMD_TYPE_DISABLE, 0);
     if (!pCmd)
     {
-        DISPDBG((0, "VBoxDISP::vboxVHWAQueryHostInfo1: vboxVHWACommandCreate failed\n"));
-        return NULL;
+        DISPDBG((0, "VBoxDISP::vboxVHWADisable: vboxVHWACommandCreate failed\n"));
+        return rc;
     }
 
     if(vboxVHWACommandSubmit (ppdev, pCmd))
@@ -974,13 +1054,38 @@ int vboxVHWADisable(PPDEV ppdev)
         }
     }
 
-    vboxVHWACommandFree (ppdev, pCmd);
+    vbvaVHWACommandRelease (ppdev, pCmd);
 
     vboxVHWACommandCheckHostCmds(ppdev);
 
     return rc;
 }
 
+void vboxVHWAInit(PPDEV ppdev)
+{
+    VHWAQUERYINFO info;
+    DWORD returnedDataLength;
+    DWORD err;
+
+    memset(&info, 0, sizeof (info));
+
+    err = EngDeviceIoControl(ppdev->hDriver,
+            IOCTL_VIDEO_VHWA_QUERY_INFO,
+            NULL,
+            0,
+            &info,
+            sizeof(info),
+            &returnedDataLength);
+    Assert(!err);
+    if(!err)
+    {
+        ppdev->vhwaInfo.offVramBase = info.offVramBase;
+        ppdev->vhwaInfo.bVHWAInited = TRUE;
+    }
+    else
+        ppdev->vhwaInfo.bVHWAInited = FALSE;
+}
+
 # endif
 
 void vboxVBVAHostCommandComplete(PPDEV ppdev, VBVAHOSTCMD * pCmd)
