@@ -2,7 +2,7 @@
  *
  * VBoxGuest -- VirtualBox Win 2000/XP guest video driver
  *
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -11,24 +11,24 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 // enable backdoor logging
 //#define LOG_ENABLED
-
+#ifndef VBOXWDDM
 extern "C"
 {
-#include <ntddk.h>
+# include <ntddk.h>
 }
+#else
+# include "VBoxVideo.h"
+#endif
 
 #include <VBox/err.h>
 
 #include <VBox/VBoxGuestLib.h>
 
+#ifndef VBOXWDDM
 /* the video miniport headers not compatible with the NT DDK headers */
 typedef struct _VIDEO_POINTER_ATTRIBUTES
 {
@@ -43,9 +43,15 @@ typedef struct _VIDEO_POINTER_ATTRIBUTES
 } VIDEO_POINTER_ATTRIBUTES, *PVIDEO_POINTER_ATTRIBUTES;
 #define VIDEO_MODE_COLOR_POINTER  0x04 // 1 if a color hardware pointer is
                                        // supported.
+#endif /* #ifndef VBOXWDDM */
 
 #include "Helper.h"
 
+#ifdef DEBUG_misha
+bool g_bVBoxVDbgBreakF = true;
+bool g_bVBoxVDbgBreakFv = false;
+#endif
+
 /**
  * Globals
  */
@@ -58,15 +64,15 @@ typedef struct _VIDEO_POINTER_ATTRIBUTES
 #pragma alloc_text(PAGE, vboxQueryHostWantsAbsolute)
 #pragma alloc_text(PAGE, vboxQueryWinVersion)
 
-BOOLEAN vboxQueryDisplayRequest(uint32_t *xres, uint32_t *yres, uint32_t *bpp)
+BOOLEAN vboxQueryDisplayRequest(uint32_t *xres, uint32_t *yres, uint32_t *bpp, uint32_t *pDisplayId)
 {
     BOOLEAN bRC = FALSE;
 
-    dprintf(("VBoxVideo::vboxQueryDisplayRequest: xres = %p, yres = %p bpp = %p\n", xres, yres, bpp));
+    dprintf(("VBoxVideo::vboxQueryDisplayRequest: xres = 0x%p, yres = 0x%p bpp = 0x%p\n", xres, yres, bpp));
 
-    VMMDevDisplayChangeRequest *req = NULL;
+    VMMDevDisplayChangeRequest2 *req = NULL;
 
-    int rc = VbglGRAlloc ((VMMDevRequestHeader **)&req, sizeof (VMMDevDisplayChangeRequest), VMMDevReq_GetDisplayChangeRequest);
+    int rc = VbglGRAlloc ((VMMDevRequestHeader **)&req, sizeof (VMMDevDisplayChangeRequest2), VMMDevReq_GetDisplayChangeRequest2);
 
     if (RT_FAILURE(rc))
     {
@@ -78,7 +84,7 @@ BOOLEAN vboxQueryDisplayRequest(uint32_t *xres, uint32_t *yres, uint32_t *bpp)
 
         rc = VbglGRPerform (&req->header);
 
-        if (RT_SUCCESS(rc) && RT_SUCCESS(req->header.rc))
+        if (RT_SUCCESS(rc))
         {
             if (xres)
                 *xres = req->xres;
@@ -86,14 +92,16 @@ BOOLEAN vboxQueryDisplayRequest(uint32_t *xres, uint32_t *yres, uint32_t *bpp)
                 *yres = req->yres;
             if (bpp)
                 *bpp  = req->bpp;
-            dprintf(("VBoxVideo::vboxQueryDisplayRequest: returning %d x %d @ %d\n",
-                     req->xres, req->yres, req->bpp));
+            if (pDisplayId)
+                *pDisplayId  = req->display;
+            dprintf(("VBoxVideo::vboxQueryDisplayRequest: returning %d x %d @ %d for %d\n",
+                     req->xres, req->yres, req->bpp, req->display));
             bRC = TRUE;
         }
         else
         {
-            dprintf(("VBoxVideo::vboxQueryDisplayRequest: ERROR querying display request from VMMDev."
-                     "rc = %Rrc, VMMDev rc = %Rrc\n", rc, req->header.rc));
+            dprintf(("VBoxVideo::vboxQueryDisplayRequest: ERROR querying display request from VMMDev. "
+                     "rc = %Rrc\n", rc));
         }
 
         VbglGRFree (&req->header);
@@ -102,13 +110,13 @@ BOOLEAN vboxQueryDisplayRequest(uint32_t *xres, uint32_t *yres, uint32_t *bpp)
     return bRC;
 }
 
-BOOLEAN vboxLikesVideoMode(uint32_t width, uint32_t height, uint32_t bpp)
+BOOLEAN vboxLikesVideoMode(uint32_t display, uint32_t width, uint32_t height, uint32_t bpp)
 {
     BOOLEAN bRC = FALSE;
 
-    VMMDevVideoModeSupportedRequest *req = NULL;
+    VMMDevVideoModeSupportedRequest2 *req2 = NULL;
 
-    int rc = VbglGRAlloc((VMMDevRequestHeader**)&req, sizeof(VMMDevVideoModeSupportedRequest), VMMDevReq_VideoModeSupported);
+    int rc = VbglGRAlloc((VMMDevRequestHeader**)&req2, sizeof(VMMDevVideoModeSupportedRequest2), VMMDevReq_VideoModeSupported2);
     if (RT_FAILURE(rc))
     {
         dprintf(("VBoxVideo::vboxLikesVideoMode: ERROR allocating request, rc = %Rrc\n", rc));
@@ -119,23 +127,46 @@ BOOLEAN vboxLikesVideoMode(uint32_t width, uint32_t height, uint32_t bpp)
     }
     else
     {
-        req->width  = width;
-        req->height = height;
-        req->bpp    = bpp;
-        rc = VbglGRPerform(&req->header);
-        if (RT_SUCCESS(rc) && RT_SUCCESS(req->header.rc))
+        req2->display = display;
+        req2->width  = width;
+        req2->height = height;
+        req2->bpp    = bpp;
+        rc = VbglGRPerform(&req2->header);
+        if (RT_SUCCESS(rc))
         {
-            bRC = req->fSupported;
+            bRC = req2->fSupported;
         }
         else
         {
-            dprintf(("VBoxVideo::vboxLikesVideoMode: ERROR querying video mode supported status from VMMDev."
-                     "rc = %Rrc, VMMDev rc = %Rrc\n", rc, req->header.rc));
+            /* Retry using old inteface. */
+            AssertCompile(sizeof(VMMDevVideoModeSupportedRequest2) >= sizeof(VMMDevVideoModeSupportedRequest));
+            VMMDevVideoModeSupportedRequest *req = (VMMDevVideoModeSupportedRequest *)req2;
+            req->header.size        = sizeof(VMMDevVideoModeSupportedRequest);
+            req->header.version     = VMMDEV_REQUEST_HEADER_VERSION;
+            req->header.requestType = VMMDevReq_VideoModeSupported;
+            req->header.rc          = VERR_GENERAL_FAILURE;
+            req->header.reserved1   = 0;
+            req->header.reserved2   = 0;
+            req->width  = width;
+            req->height = height;
+            req->bpp    = bpp;
+
+            rc = VbglGRPerform(&req->header);
+            if (RT_SUCCESS(rc))
+            {
+                bRC = req->fSupported;
+            }
+            else
+            {
+                dprintf(("VBoxVideo::vboxLikesVideoMode: ERROR querying video mode supported status from VMMDev."
+                         "rc = %Rrc\n", rc));
+            }
         }
-        VbglGRFree(&req->header);
+        VbglGRFree(&req2->header);
     }
 
     dprintf(("VBoxVideo::vboxLikesVideoMode: width: %d, height: %d, bpp: %d -> %s\n", width, height, bpp, (bRC == 1) ? "OK" : "FALSE"));
+
     return bRC;
 }
 
@@ -155,14 +186,14 @@ ULONG vboxGetHeightReduction()
     else
     {
         rc = VbglGRPerform(&req->header);
-        if (RT_SUCCESS(rc) && RT_SUCCESS(req->header.rc))
+        if (RT_SUCCESS(rc))
         {
             retHeight = (ULONG)req->heightReduction;
         }
         else
         {
-            dprintf(("VBoxVideo::vboxGetHeightReduction: ERROR querying height reduction value from VMMDev."
-                     "rc = %Rrc, VMMDev rc = %Rrc\n", rc, req->header.rc));
+            dprintf(("VBoxVideo::vboxGetHeightReduction: ERROR querying height reduction value from VMMDev. "
+                     "rc = %Rrc\n", rc));
         }
         VbglGRFree(&req->header);
     }
@@ -190,9 +221,9 @@ static BOOLEAN vboxQueryPointerPosInternal (uint16_t *pointerXPos, uint16_t *poi
     {
         rc = VbglGRPerform (&req->header);
 
-        if (RT_SUCCESS(rc) && RT_SUCCESS(req->header.rc))
+        if (RT_SUCCESS(rc))
         {
-            if (req->mouseFeatures & VBOXGUEST_MOUSE_HOST_CAN_ABSOLUTE)
+            if (req->mouseFeatures & VMMDEV_MOUSE_HOST_CAN_ABSOLUTE)
             {
                 if (pointerXPos)
                 {
@@ -209,8 +240,8 @@ static BOOLEAN vboxQueryPointerPosInternal (uint16_t *pointerXPos, uint16_t *poi
         }
         else
         {
-            dprintf(("VBoxVideo::vboxQueryPointerPosInternal: ERROR querying mouse capabilities from VMMDev."
-                     "rc = %Rrc, VMMDev rc = %Rrc\n", rc, req->header.rc));
+            dprintf(("VBoxVideo::vboxQueryPointerPosInternal: ERROR querying mouse capabilities from VMMDev. "
+                     "rc = %Rrc\n", rc));
         }
 
         VbglGRFree (&req->header);
@@ -261,7 +292,6 @@ winVersion_t vboxQueryWinVersion()
 
     dprintf(("VBoxVideo::vboxQueryWinVersion: running on Windows NT version %d.%d, build %d\n",
              majorVersion, minorVersion, buildNumber));
-
     if(majorVersion == 6)
     {
         if (minorVersion == 1)
@@ -348,14 +378,14 @@ BOOLEAN vboxUpdatePointerShape(PVIDEO_POINTER_ATTRIBUTES pointerAttr, uint32_t c
 
         rc = VbglGRPerform (&req->header);
 
-        if (RT_SUCCESS(rc) && RT_SUCCESS(req->header.rc))
+        if (RT_SUCCESS(rc))
         {
             bRC = TRUE;
         }
         else
         {
-            dprintf(("VBoxVideo::vboxUpdatePointerShape: ERROR querying mouse capabilities from VMMDev."
-                     "rc = %Rrc, VMMDev rc = %Rrc\n", rc, req->header.rc));
+            dprintf(("VBoxVideo::vboxUpdatePointerShape: ERROR querying mouse capabilities from VMMDev. "
+                     "rc = %Rrc\n", rc));
         }
 
         dprintf(("VBoxVideo::vboxUpdatePointerShape: req->u32Version = %08X\n", req->header.version));
