@@ -18,6 +18,9 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+/* IDirect3DVertexDeclaration8 is internal to our implementation.
+ * It's not visible in the API. */
+
 /*
  * Sun LGPL Disclaimer: For the avoidance of doubt, except that if any license choice
  * other than GPL or LGPL is available it will apply instead, Sun elects to use only
@@ -27,9 +30,6 @@
  * of the LGPL is applied is otherwise unspecified.
  */
 
-/* IDirect3DVertexDeclaration8 is internal to our implementation.
- * It's not visible in the API. */
-
 #include "config.h"
 #include "d3d8_private.h"
 
@@ -38,7 +38,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(d3d8);
 /* IUnknown */
 static HRESULT WINAPI IDirect3DVertexDeclaration8Impl_QueryInterface(IDirect3DVertexDeclaration8 *iface, REFIID riid, void **obj_ptr)
 {
-    TRACE("(%p)->(%s, %p)\n", iface, debugstr_guid(riid), obj_ptr);
+    TRACE("iface %p, riid %s, object %p.\n", iface, debugstr_guid(riid), obj_ptr);
 
     if (IsEqualGUID(riid, &IID_IUnknown)
             || IsEqualGUID(riid, &IID_IDirect3DVertexDeclaration8))
@@ -55,9 +55,16 @@ static HRESULT WINAPI IDirect3DVertexDeclaration8Impl_QueryInterface(IDirect3DVe
 static ULONG WINAPI IDirect3DVertexDeclaration8Impl_AddRef(IDirect3DVertexDeclaration8 *iface)
 {
     IDirect3DVertexDeclaration8Impl *This = (IDirect3DVertexDeclaration8Impl *)iface;
-
     ULONG ref_count = InterlockedIncrement(&This->ref_count);
-    TRACE("(%p) : AddRef increasing to %d\n", This, ref_count);
+
+    TRACE("%p increasing refcount to %u.\n", iface, ref_count);
+
+    if (ref_count == 1)
+    {
+        wined3d_mutex_lock();
+        IWineD3DVertexDeclaration_AddRef(This->wined3d_vertex_declaration);
+        wined3d_mutex_unlock();
+    }
 
     return ref_count;
 }
@@ -65,16 +72,14 @@ static ULONG WINAPI IDirect3DVertexDeclaration8Impl_AddRef(IDirect3DVertexDeclar
 static ULONG WINAPI IDirect3DVertexDeclaration8Impl_Release(IDirect3DVertexDeclaration8 *iface)
 {
     IDirect3DVertexDeclaration8Impl *This = (IDirect3DVertexDeclaration8Impl *)iface;
-
     ULONG ref_count = InterlockedDecrement(&This->ref_count);
-    TRACE("(%p) : Releasing to %d\n", This, ref_count);
+
+    TRACE("%p decreasing refcount to %u.\n", iface, ref_count);
 
     if (!ref_count) {
-        EnterCriticalSection(&d3d8_cs);
+        wined3d_mutex_lock();
         IWineD3DVertexDeclaration_Release(This->wined3d_vertex_declaration);
-        LeaveCriticalSection(&d3d8_cs);
-        HeapFree(GetProcessHeap(), 0, This->elements);
-        HeapFree(GetProcessHeap(), 0, This);
+        wined3d_mutex_unlock();
     }
 
     return ref_count;
@@ -264,7 +269,7 @@ static const WINED3DFORMAT wined3d_format_lookup[] =
     /*WINED3DDECLTYPE_FLOAT2*/    WINED3DFMT_R32G32_FLOAT,
     /*WINED3DDECLTYPE_FLOAT3*/    WINED3DFMT_R32G32B32_FLOAT,
     /*WINED3DDECLTYPE_FLOAT4*/    WINED3DFMT_R32G32B32A32_FLOAT,
-    /*WINED3DDECLTYPE_D3DCOLOR*/  WINED3DFMT_A8R8G8B8,
+    /*WINED3DDECLTYPE_D3DCOLOR*/  WINED3DFMT_B8G8R8A8_UNORM,
     /*WINED3DDECLTYPE_UBYTE4*/    WINED3DFMT_R8G8B8A8_UINT,
     /*WINED3DDECLTYPE_SHORT2*/    WINED3DFMT_R16G16_SINT,
     /*WINED3DDECLTYPE_SHORT4*/    WINED3DFMT_R16G16B16A16_SINT,
@@ -305,7 +310,8 @@ static const wined3d_usage_t wined3d_usage_lookup[] = {
 };
 
 /* TODO: find out where rhw (or positionT) is for declaration8 */
-UINT convert_to_wined3d_declaration(const DWORD *d3d8_elements, DWORD *d3d8_elements_size, WINED3DVERTEXELEMENT **wined3d_elements)
+static UINT convert_to_wined3d_declaration(const DWORD *d3d8_elements, DWORD *d3d8_elements_size,
+        WINED3DVERTEXELEMENT **wined3d_elements)
 {
     const DWORD *token = d3d8_elements;
     WINED3DVERTEXELEMENT *element;
@@ -360,9 +366,82 @@ UINT convert_to_wined3d_declaration(const DWORD *d3d8_elements, DWORD *d3d8_elem
     return element_count;
 }
 
-const IDirect3DVertexDeclaration8Vtbl Direct3DVertexDeclaration8_Vtbl =
+static const IDirect3DVertexDeclaration8Vtbl Direct3DVertexDeclaration8_Vtbl =
 {
     IDirect3DVertexDeclaration8Impl_QueryInterface,
     IDirect3DVertexDeclaration8Impl_AddRef,
     IDirect3DVertexDeclaration8Impl_Release
 };
+
+static void STDMETHODCALLTYPE d3d8_vertexdeclaration_wined3d_object_destroyed(void *parent)
+{
+    IDirect3DVertexDeclaration8Impl *declaration = parent;
+    HeapFree(GetProcessHeap(), 0, declaration->elements);
+    HeapFree(GetProcessHeap(), 0, declaration);
+}
+
+static const struct wined3d_parent_ops d3d8_vertexdeclaration_wined3d_parent_ops =
+{
+    d3d8_vertexdeclaration_wined3d_object_destroyed,
+};
+
+HRESULT vertexdeclaration_init(IDirect3DVertexDeclaration8Impl *declaration,
+        IDirect3DDevice8Impl *device, const DWORD *elements, DWORD shader_handle)
+{
+    WINED3DVERTEXELEMENT *wined3d_elements;
+    UINT wined3d_element_count;
+    HRESULT hr;
+
+    declaration->lpVtbl = &Direct3DVertexDeclaration8_Vtbl;
+    declaration->ref_count = 1;
+    declaration->shader_handle = shader_handle;
+
+    wined3d_element_count = convert_to_wined3d_declaration(elements, &declaration->elements_size, &wined3d_elements);
+    declaration->elements = HeapAlloc(GetProcessHeap(), 0, declaration->elements_size);
+    if (!declaration->elements)
+    {
+        ERR("Failed to allocate vertex declaration elements memory.\n");
+        HeapFree(GetProcessHeap(), 0, wined3d_elements);
+        return E_OUTOFMEMORY;
+    }
+
+    memcpy(declaration->elements, elements, declaration->elements_size);
+
+    wined3d_mutex_lock();
+    hr = IWineD3DDevice_CreateVertexDeclaration(device->WineD3DDevice, &declaration->wined3d_vertex_declaration,
+            (IUnknown *)declaration, &d3d8_vertexdeclaration_wined3d_parent_ops,
+            wined3d_elements, wined3d_element_count);
+    wined3d_mutex_unlock();
+    HeapFree(GetProcessHeap(), 0, wined3d_elements);
+    if (FAILED(hr))
+    {
+        WARN("Failed to create wined3d vertex declaration, hr %#x.\n", hr);
+        HeapFree(GetProcessHeap(), 0, declaration->elements);
+        return hr;
+    }
+
+    return D3D_OK;
+}
+
+HRESULT vertexdeclaration_init_fvf(IDirect3DVertexDeclaration8Impl *declaration,
+        IDirect3DDevice8Impl *device, DWORD fvf)
+{
+    HRESULT hr;
+
+    declaration->ref_count = 1;
+    declaration->lpVtbl = &Direct3DVertexDeclaration8_Vtbl;
+    declaration->elements = NULL;
+    declaration->elements_size = 0;
+    declaration->shader_handle = fvf;
+
+    hr = IWineD3DDevice_CreateVertexDeclarationFromFVF(device->WineD3DDevice,
+            &declaration->wined3d_vertex_declaration, (IUnknown *)declaration,
+            &d3d8_vertexdeclaration_wined3d_parent_ops, fvf);
+    if (FAILED(hr))
+    {
+        WARN("Failed to create wined3d vertex declaration, hr %#x.\n", hr);
+        return hr;
+    }
+
+    return D3D_OK;
+}
