@@ -273,7 +273,7 @@ DECL_WINELIB_TYPE_AW(LPFINDREPLACE)
 #define FR_MATCHKASHIDA                 0x40000000
 #define FR_MATCHALEFHAMZA               0x80000000
 
-typedef UINT (CALLBACK *LPCFHOOKPROC)(HWND,UINT,WPARAM,LPARAM);
+typedef UINT_PTR (CALLBACK *LPCFHOOKPROC)(HWND,UINT,WPARAM,LPARAM);
 
 typedef struct tagCHOOSEFONTA
 {
