@@ -1566,4 +1566,16 @@ typedef enum _D3DCOMPOSERECTSOP{
 } D3DCOMPOSERECTSOP;
 #endif /* D3D_DISABLE_9EX */
 
+typedef enum _D3DSHADER_COMPARISON
+{
+    D3DSPC_RESERVED0 = 0,
+    D3DSPC_GT,
+    D3DSPC_EQ,
+    D3DSPC_GE,
+    D3DSPC_LT,
+    D3DSPC_NE,
+    D3DSPC_LE,
+    D3DSPC_RESERVED1,
+} D3DSHADER_COMPARISON;
+
 #endif /* __WINE_D3D9TYPES_H */
