@@ -181,11 +181,6 @@ enum MatrixOrder
 {
     MatrixOrderPrepend = 0,
     MatrixOrderAppend  = 1
-enum WarpMode {
-    WarpModePerspective,
-    WarpModeBilinear
-};
-
 };
 
 enum ImageType
@@ -195,6 +190,11 @@ enum ImageType
     ImageTypeMetafile
 };
 
+enum WarpMode {
+    WarpModePerspective,
+    WarpModeBilinear
+};
+
 enum WrapMode
 {
     WrapModeTile,
@@ -300,6 +300,13 @@ enum HotkeyPrefix
     HotkeyPrefixHide   = 2
 };
 
+enum PaletteFlags
+{
+    PaletteFlagsHasAlpha        = 1,
+    PaletteFlagsGrayScale       = 2,
+    PaletteFlagsHalftone        = 4
+};
+
 enum ImageCodecFlags
 {
     ImageCodecFlagsEncoder          = 1,
@@ -441,7 +448,6 @@ typedef enum LineCap LineCap;
 typedef enum PathPointType PathPointType;
 typedef enum LineJoin LineJoin;
 typedef enum QualityMode QualityMode;
-typedef enum WarpMode WarpMode;
 typedef enum SmoothingMode SmoothingMode;
 typedef enum CompositingQuality CompositingQuality;
 typedef enum InterpolationMode InterpolationMode;
@@ -451,6 +457,7 @@ typedef enum DashStyle DashStyle;
 typedef enum MatrixOrder MatrixOrder;
 typedef enum ImageType ImageType;
 typedef enum ImageFlags ImageFlags;
+typedef enum WarpMode WarpMode;
 typedef enum WrapMode WrapMode;
 typedef enum MetafileType MetafileType;
 typedef enum LinearGradientMode LinearGradientMode;
@@ -464,6 +471,7 @@ typedef enum FontStyle FontStyle;
 typedef enum StringFormatFlags StringFormatFlags;
 typedef enum HotkeyPrefix HotkeyPrefix;
 typedef enum PenAlignment GpPenAlignment;
+typedef enum PaletteFlags PaletteFlags;
 typedef enum ImageCodecFlags ImageCodecFlags;
 typedef enum CombineMode CombineMode;
 typedef enum FlushIntention FlushIntention;
