@@ -310,7 +310,7 @@ UINT WINAPI MsiSummaryInfoGetPropertyCount(MSIHANDLE,PUINT);
 
 UINT WINAPI MsiEnableUIPreview(MSIHANDLE, MSIHANDLE*);
 BOOL WINAPI MsiGetMode(MSIHANDLE, MSIRUNMODE);
-BOOL WINAPI MsiSetMode(MSIHANDLE, MSIRUNMODE, BOOL);
+UINT WINAPI MsiSetMode(MSIHANDLE, MSIRUNMODE, BOOL);
 
 UINT WINAPI MsiViewModify(MSIHANDLE, MSIMODIFY, MSIHANDLE);
 
