@@ -56,7 +56,7 @@ typedef int _off_t;
 typedef unsigned char u_char;
 typedef unsigned short u_short;
 typedef unsigned int  u_int;
-typedef unsigned long u_long;
+typedef __msvcrt_ulong u_long;
 #define _BSDTYPES_DEFINED
 #endif
 
