@@ -403,5 +403,87 @@
 #define DISPID_MX_READER_CONTROL_RESUME                 0x00000577
 #define DISPID_MX_READER_CONTROL_SUSPEND                0x00000578
 
+#define DISPID_MX_SCHEMADECLHANDLER                     0x0000057a
+#define DISPID_MX_SCHEMADECLHANDLER_SCHEMAELEMENTDECL   0x0000057b
+
+#define DISPID_MX_NSMGR                                 0x0000057d
+#define DISPID_MX_NSMGR_ALLOWOVERRIDE                   0x0000057e
+#define DISPID_MX_NSMGR_RESET                           0x0000057f
+#define DISPID_MX_NSMGR_PUSHCONTEXT                     0x00000580
+#define DISPID_MX_NSMGR_PUSHNODECONTEXT                 0x00000581
+#define DISPID_MX_NSMGR_POPCONTEXT                      0x00000582
+#define DISPID_MX_NSMGR_DECLAREPREFIX                   0x00000583
+#define DISPID_MX_NSMGR_GETDECLAREDPREFIXES             0x00000584
+#define DISPID_MX_NSMGR_GETPREFIXES                     0x00000585
+#define DISPID_MX_NSMGR_GETURI                          0x00000586
+#define DISPID_MX_NSMGR_GETURIFROMNODE                  0x00000587
+#define DISPID_MX_NSMGR_LENGTH                          0x00000588
+
+#define DISPID_SOM_VALIDATE                             0x0000058b
+#define DISPID_SOM_VALIDATEONLOAD                       0x0000058c
+#define DISPID_SOM_GETSCHEMA                            0x0000058d
+#define DISPID_SOM_GETDECLARATION                       0x0000058e
+#define DISPID_SOM_ITEMBYNAME                           0x0000058f
+#define DISPID_SOM_ITEMBYQNAME                          0x00000590
+#define DISPID_SOM_ANYATTRIBUTE                         0x00000591
+#define DISPID_SOM_ATTRIBUTEGROUPS                      0x00000592
+#define DISPID_SOM_ATTRIBUTES                           0x00000593
+#define DISPID_SOM_BASETYPES                            0x00000594
+#define DISPID_SOM_CONTENTMODEL                         0x00000595
+#define DISPID_SOM_CONTENTTYPE                          0x00000596
+#define DISPID_SOM_DEFAULTVALUE                         0x00000597
+#define DISPID_SOM_DERIVEDBY                            0x00000598
+#define DISPID_SOM_DISALLOWED                           0x00000599
+#define DISPID_SOM_ELEMENTS                             0x0000059a
+#define DISPID_SOM_ENUMERATION                          0x0000059b
+#define DISPID_SOM_FIELDS                               0x0000059c
+#define DISPID_SOM_FINAL                                0x0000059d
+#define DISPID_SOM_FIXEDVALUE                           0x0000059e
+#define DISPID_SOM_FRACTIONDIGITS                       0x0000059f
+#define DISPID_SOM_ID                                   0x000005a0
+#define DISPID_SOM_IDCONSTRAINTS                        0x000005a1
+#define DISPID_SOM_ISABSTRACT                           0x000005a2
+#define DISPID_SOM_ISNILLABLE                           0x000005a3
+#define DISPID_SOM_ISREFERENCE                          0x000005a4
+#define DISPID_SOM_ISVALID                              0x000005a5
+#define DISPID_SOM_ITEMTYPE                             0x000005a6
+#define DISPID_SOM_LENGTH                               0x000005a7
+#define DISPID_SOM_MAXEXCLUSIVE                         0x000005a8
+#define DISPID_SOM_MAXINCLUSIVE                         0x000005a9
+#define DISPID_SOM_MAXLENGTH                            0x000005aa
+#define DISPID_SOM_MAXOCCURS                            0x000005ab
+#define DISPID_SOM_MINEXCLUSIVE                         0x000005ac
+#define DISPID_SOM_MININCLUSIVE                         0x000005ad
+#define DISPID_SOM_MINLENGTH                            0x000005ae
+#define DISPID_SOM_MINOCCURS                            0x000005af
+#define DISPID_SOM_MODELGROUPS                          0x000005b0
+#define DISPID_SOM_NAME                                 0x000005b1
+#define DISPID_SOM_NAMESPACES                           0x000005b2
+#define DISPID_SOM_NAMESPACEURI                         0x000005b3
+#define DISPID_SOM_NOTATIONS                            0x000005b4
+#define DISPID_SOM_PARTICLES                            0x000005b5
+#define DISPID_SOM_PATTERNS                             0x000005b6
+#define DISPID_SOM_PROCESSCONTENTS                      0x000005b7
+#define DISPID_SOM_PROHIBITED                           0x000005b8
+#define DISPID_SOM_PUBLICIDENTIFIER                     0x000005b9
+#define DISPID_SOM_REFERENCEDKEY                        0x000005ba
+#define DISPID_SOM_SCHEMA                               0x000005bb
+#define DISPID_SOM_SCHEMALOCATIONS                      0x000005bc
+#define DISPID_SOM_SCOPE                                0x000005bd
+#define DISPID_SOM_SELECTOR                             0x000005be
+#define DISPID_SOM_SUBSTITUTIONGROUP                    0x000005bf
+#define DISPID_SOM_EXCLUSIONS                           0x000005c0
+#define DISPID_SOM_SYSTEMIDENTIFIER                     0x000005c1
+#define DISPID_SOM_TARGETNAMESPACE                      0x000005c2
+#define DISPID_SOM_TOTALDIGITS                          0x000005c3
+#define DISPID_SOM_TYPE                                 0x000005c4
+#define DISPID_SOM_TYPES                                0x000005c5
+#define DISPID_SOM_UNHANDLEDATTRS                       0x000005c6
+#define DISPID_SOM_USE                                  0x000005c7
+#define DISPID_SOM_VARIETY                              0x000005c8
+#define DISPID_SOM_VERSION                              0x000005c9
+#define DISPID_SOM_WHITESPACE                           0x000005ca
+#define DISPID_SOM_WRITEANNOTATION                      0x000005cb
+
 
 #endif /* __MSXML2DID_H__ */
