@@ -914,6 +914,89 @@
 #define STATUS_AUTHENTICATION_FIREWALL_FAILED ((NTSTATUS) 0xC0000413)
 #define STATUS_VDM_DISALLOWED            ((NTSTATUS) 0xC0000414)
 #define STATUS_HUNG_DISPLAY_DRIVER_THREAD ((NTSTATUS) 0xC0000415)
+#define STATUS_INSUFFICIENT_RESOURCE_FOR_SPECIFIED_SHARED_SECTION_SIZE ((NTSTATUS) 0xC0000416)
+#define STATUS_INVALID_CRUNTIME_PARAMETER   ((NTSTATUS) 0xC0000417)
+#define STATUS_NTLM_BLOCKED                 ((NTSTATUS) 0xC0000418)
+#define STATUS_ASSERTION_FAILURE            ((NTSTATUS) 0xC0000420)
+#define STATUS_VERIFIER_STOP                ((NTSTATUS) 0xC0000421)
+#define STATUS_CALLBACK_POP_STACK           ((NTSTATUS) 0xC0000423)
+#define STATUS_INCOMPATIBLE_DRIVER_BLOCKED  ((NTSTATUS) 0xC0000424)
+#define STATUS_HIVE_UNLOADED                ((NTSTATUS) 0xC0000425)
+#define STATUS_COMPRESSION_DISABLED         ((NTSTATUS) 0xC0000426)
+#define STATUS_FILE_SYSTEM_LIMITATION       ((NTSTATUS) 0xC0000427)
+#define STATUS_INVALID_IMAGE_HASH           ((NTSTATUS) 0xC0000428)
+#define STATUS_NOT_CAPABLE                  ((NTSTATUS) 0xC0000429)
+#define STATUS_REQUEST_OUT_OF_SEQUENCE      ((NTSTATUS) 0xC000042A)
+#define STATUS_IMPLEMENTATION_LIMIT         ((NTSTATUS) 0xC000042B)
+#define STATUS_ELEVATION_REQUIRED           ((NTSTATUS) 0xC000042C)
+#define STATUS_BEYOND_VDL                   ((NTSTATUS) 0xC0000432)
+#define STATUS_ENCOUNTERED_WRITE_IN_PROGRESS ((NTSTATUS) 0xC0000433)
+#define STATUS_PTE_CHANGED                  ((NTSTATUS) 0xC0000434)
+#define STATUS_PURGE_FAILED                 ((NTSTATUS) 0xC0000435)
+#define STATUS_CRED_REQUIRES_CONFIRMATION   ((NTSTATUS) 0xC0000440)
+#define STATUS_CS_ENCRYPTION_INVALID_SERVER_RESPONSE ((NTSTATUS) 0xC0000441)
+#define STATUS_CS_ENCRYPTION_UNSUPPORTED_SERVER ((NTSTATUS) 0xC0000442)
+#define STATUS_CS_ENCRYPTION_EXISTING_ENCRYPTED_FILE ((NTSTATUS) 0xC0000443)
+#define STATUS_CS_ENCRYPTION_NEW_ENCRYPTED_FILE ((NTSTATUS) 0xC0000444)
+#define STATUS_CS_ENCRYPTION_FILE_NOT_CSE   ((NTSTATUS) 0xC0000445)
+#define STATUS_INVALID_LABEL                ((NTSTATUS) 0xC0000446)
+#define STATUS_DRIVER_PROCESS_TERMINATED    ((NTSTATUS) 0xC0000450)
+#define STATUS_AMBIGUOUS_SYSTEM_DEVICE      ((NTSTATUS) 0xC0000451)
+#define STATUS_SYSTEM_DEVICE_NOT_FOUND      ((NTSTATUS) 0xC0000452)
+#define STATUS_RESTART_BOOT_APPLICATION     ((NTSTATUS) 0xC0000453)
+#define STATUS_INVALID_TASK_NAME            ((NTSTATUS) 0xC0000500)
+#define STATUS_INVALID_TASK_INDEX           ((NTSTATUS) 0xC0000501)
+#define STATUS_THREAD_ALREADY_IN_TASK       ((NTSTATUS) 0xC0000502)
+#define STATUS_CALLBACK_BYPASS              ((NTSTATUS) 0xC0000503)
+#define STATUS_PORT_CLOSED                  ((NTSTATUS) 0xC0000700)
+#define STATUS_MESSAGE_LOST                 ((NTSTATUS) 0xC0000701)
+#define STATUS_INVALID_MESSAGE              ((NTSTATUS) 0xC0000702)
+#define STATUS_REQUEST_CANCELED             ((NTSTATUS) 0xC0000703)
+#define STATUS_RECURSIVE_DISPATCH           ((NTSTATUS) 0xC0000704)
+#define STATUS_LPC_RECEIVE_BUFFER_EXPECTED  ((NTSTATUS) 0xC0000705)
+#define STATUS_LPC_INVALID_CONNECTION_USAGE ((NTSTATUS) 0xC0000706)
+#define STATUS_LPC_REQUESTS_NOT_ALLOWED     ((NTSTATUS) 0xC0000707)
+#define STATUS_RESOURCE_IN_USE              ((NTSTATUS) 0xC0000708)
+#define STATUS_HARDWARE_MEMORY_ERROR        ((NTSTATUS) 0xC0000709)
+#define STATUS_THREADPOOL_HANDLE_EXCEPTION  ((NTSTATUS) 0xC000070A)
+#define STATUS_THREADPOOL_SET_EVENT_ON_COMPLETION_FAILED ((NTSTATUS) 0xC000070B)
+#define STATUS_THREADPOOL_RELEASE_SEMAPHORE_ON_COMPLETION_FAILED ((NTSTATUS) 0xC000070C)
+#define STATUS_THREADPOOL_RELEASE_MUTEX_ON_COMPLETION_FAILED ((NTSTATUS) 0xC000070D)
+#define STATUS_THREADPOOL_FREE_LIBRARY_ON_COMPLETION_FAILED ((NTSTATUS) 0xC000070E)
+#define STATUS_THREADPOOL_RELEASED_DURING_OPERATION ((NTSTATUS) 0xC000070F)
+#define STATUS_CALLBACK_RETURNED_WHILE_IMPERSONATING ((NTSTATUS) 0xC0000710)
+#define STATUS_APC_RETURNED_WHILE_IMPERSONATING ((NTSTATUS) 0xC0000711)
+#define STATUS_PROCESS_IS_PROTECTED         ((NTSTATUS) 0xC0000712)
+#define STATUS_MCA_EXCEPTION                ((NTSTATUS) 0xC0000713)
+#define STATUS_CERTIFICATE_MAPPING_NOT_UNIQUE ((NTSTATUS) 0xC0000714)
+#define STATUS_SYMLINK_CLASS_DISABLED       ((NTSTATUS) 0xC0000715)
+#define STATUS_INVALID_IDN_NORMALIZATION    ((NTSTATUS) 0xC0000716)
+#define STATUS_NO_UNICODE_TRANSLATION       ((NTSTATUS) 0xC0000717)
+#define STATUS_ALREADY_REGISTERED           ((NTSTATUS) 0xC0000718)
+#define STATUS_CONTEXT_MISMATCH             ((NTSTATUS) 0xC0000719)
+#define STATUS_PORT_ALREADY_HAS_COMPLETION_LIST ((NTSTATUS) 0xC000071A)
+#define STATUS_CALLBACK_RETURNED_THREAD_PRIORITY ((NTSTATUS) 0xC000071B)
+#define STATUS_INVALID_THREAD               ((NTSTATUS) 0xC000071C)
+#define STATUS_CALLBACK_RETURNED_TRANSACTION ((NTSTATUS) 0xC000071D)
+#define STATUS_CALLBACK_RETURNED_LDR_LOCK   ((NTSTATUS) 0xC000071E)
+#define STATUS_CALLBACK_RETURNED_LANG       ((NTSTATUS) 0xC000071F)
+#define STATUS_CALLBACK_RETURNED_PRI_BACK   ((NTSTATUS) 0xC0000720)
+#define STATUS_CALLBACK_RETURNED_THREAD_AFFINITY ((NTSTATUS) 0xC0000721)
+#define STATUS_DISK_REPAIR_DISABLED         ((NTSTATUS) 0xC0000800)
+#define STATUS_DS_DOMAIN_RENAME_IN_PROGRESS ((NTSTATUS) 0xC0000801)
+#define STATUS_DISK_QUOTA_EXCEEDED          ((NTSTATUS) 0xC0000802)
+#define STATUS_CONTENT_BLOCKED              ((NTSTATUS) 0xC0000804)
+#define STATUS_BAD_CLUSTERS                 ((NTSTATUS) 0xC0000805)
+#define STATUS_VOLUME_DIRTY                 ((NTSTATUS) 0xC0000806)
+#define STATUS_FILE_CHECKED_OUT             ((NTSTATUS) 0xC0000901)
+#define STATUS_CHECKOUT_REQUIRED            ((NTSTATUS) 0xC0000902)
+#define STATUS_BAD_FILE_TYPE                ((NTSTATUS) 0xC0000903)
+#define STATUS_FILE_TOO_LARGE               ((NTSTATUS) 0xC0000904)
+#define STATUS_FORMS_AUTH_REQUIRED          ((NTSTATUS) 0xC0000905)
+#define STATUS_VIRUS_INFECTED               ((NTSTATUS) 0xC0000906)
+#define STATUS_VIRUS_DELETED                ((NTSTATUS) 0xC0000907)
+#define STATUS_BAD_MCFG_TABLE               ((NTSTATUS) 0xC0000908)
+
 #define STATUS_WOW_ASSERTION             ((NTSTATUS) 0xC0009898)
 
 #define RPC_NT_INVALID_STRING_BINDING    ((NTSTATUS) 0xC0020001)
