@@ -46,6 +46,10 @@ typedef HANDLE PDH_HQUERY;
 typedef HANDLE PDH_HCOUNTER;
 typedef HANDLE PDH_HLOG;
 
+#define PDH_CVERSION_WIN40  0x0400
+#define PDH_CVERSION_WIN50  0x0500
+#define PDH_VERSION         0x0503
+
 #define PDH_MAX_SCALE 7
 #define PDH_MIN_SCALE (-7)
 
@@ -58,6 +62,10 @@ typedef HANDLE PDH_HLOG;
 #define PDH_FMT_1000        0x00002000
 #define PDH_FMT_NOCAP100    0x00008000
 
+#define DATA_SOURCE_REGISTRY    0x00000001
+#define DATA_SOURCE_LOGFILE     0x00000002
+#define DATA_SOURCE_WBEM        0x00000004
+
 typedef struct _PDH_FMT_COUNTERVALUE
 {
     DWORD CStatus;
@@ -191,6 +199,7 @@ PDH_STATUS WINAPI PdhGetCounterInfoA(PDH_HCOUNTER, BOOLEAN, LPDWORD, PPDH_COUNTE
 PDH_STATUS WINAPI PdhGetCounterInfoW(PDH_HCOUNTER, BOOLEAN, LPDWORD, PPDH_COUNTER_INFO_W);
 #define    PdhGetCounterInfo WINELIB_NAME_AW(PdhGetCounterInfo)
 PDH_STATUS WINAPI PdhGetCounterTimeBase(PDH_HCOUNTER, LONGLONG *);
+PDH_STATUS WINAPI PdhGetDllVersion(LPDWORD);
 PDH_STATUS WINAPI PdhGetFormattedCounterValue(PDH_HCOUNTER, DWORD, LPDWORD, PPDH_FMT_COUNTERVALUE);
 PDH_STATUS WINAPI PdhGetRawCounterValue(PDH_HCOUNTER, LPDWORD, PPDH_RAW_COUNTER);
 PDH_STATUS WINAPI PdhLookupPerfIndexByNameA(LPCSTR, LPCSTR, LPDWORD);
@@ -207,6 +216,7 @@ PDH_STATUS WINAPI PdhOpenQueryW(LPCWSTR, DWORD_PTR, PDH_HQUERY *);
 #define    PdhOpenQuery WINELIB_NAME_AW(PdhOpenQuery)
 PDH_STATUS WINAPI PdhRemoveCounter(PDH_HCOUNTER);
 PDH_STATUS WINAPI PdhSetCounterScaleFactor(PDH_HCOUNTER, LONG);
+PDH_STATUS WINAPI PdhSetDefaultRealTimeDataSource(DWORD);
 PDH_STATUS WINAPI PdhValidatePathA(LPCSTR);
 PDH_STATUS WINAPI PdhValidatePathW(LPCWSTR);
 #define    PdhValidatePath WINELIB_NAME_AW(PdhValidatePath)
