@@ -868,7 +868,9 @@ typedef struct varstring_tag {
 
 /* line functions */
 DWORD WINAPI lineAccept(HCALL,LPCSTR,DWORD);
-DWORD WINAPI lineAddProvider(LPCSTR,HWND,LPDWORD);
+DWORD WINAPI lineAddProviderA(LPCSTR,HWND,LPDWORD);
+DWORD WINAPI lineAddProviderW(LPCWSTR,HWND,LPDWORD);
+#define      lineAddProvider WINELIB_NAME_AW(lineAddProvider)
 DWORD WINAPI lineAddToConference(HCALL,HCALL);
 DWORD WINAPI lineAnswer(HCALL,LPCSTR,DWORD);
 DWORD WINAPI lineBlindTransfer(HCALL,LPCSTR,DWORD);
@@ -895,14 +897,20 @@ DWORD WINAPI lineGetCallInfo(HCALL,LPLINECALLINFO);
 DWORD WINAPI lineGetCallStatus(HCALL,LPLINECALLSTATUS);
 DWORD WINAPI lineGetConfRelatedCalls(HCALL,LPLINECALLLIST);
 DWORD WINAPI lineGetCountry(DWORD,DWORD,LPLINECOUNTRYLIST);
-DWORD WINAPI lineGetDevCaps(HLINEAPP,DWORD,DWORD,DWORD,LPLINEDEVCAPS);
+DWORD WINAPI lineGetDevCapsA(HLINEAPP,DWORD,DWORD,DWORD,LPLINEDEVCAPS);
+DWORD WINAPI lineGetDevCapsW(HLINEAPP,DWORD,DWORD,DWORD,LPLINEDEVCAPS);
+#define      lineGetDevCaps WINELIB_NAME_AW(lineGetDevCaps);
 DWORD WINAPI lineGetDevConfig(DWORD,LPVARSTRING,LPCSTR);
-DWORD WINAPI lineGetID(HLINE,DWORD,HCALL,DWORD,LPVARSTRING,LPCSTR);
+DWORD WINAPI lineGetIDA(HLINE,DWORD,HCALL,DWORD,LPVARSTRING,LPCSTR);
+DWORD WINAPI lineGetIDW(HLINE,DWORD,HCALL,DWORD,LPVARSTRING,LPCWSTR);
+#define      lineGetID WINELIB_NAME_AW(lineGetID);
 DWORD WINAPI lineGetIcon(DWORD,LPCSTR,HICON *);
 DWORD WINAPI lineGetLineDevStatus(HLINE,LPLINEDEVSTATUS);
 DWORD WINAPI lineGetNewCalls(HLINE,DWORD,DWORD,LPLINECALLLIST);
 DWORD WINAPI lineGetNumRings(HLINE,DWORD,LPDWORD);
-DWORD WINAPI lineGetProviderList(DWORD dwAPIVersion,LPLINEPROVIDERLIST);
+DWORD WINAPI lineGetProviderListA(DWORD,LPLINEPROVIDERLIST);
+DWORD WINAPI lineGetProviderListW(DWORD,LPLINEPROVIDERLIST);
+#define      lineGetProviderList WINELIB_NAME_AW(lineGetProviderList)
 DWORD WINAPI lineGetRequest(HLINEAPP,DWORD,LPVOID);
 DWORD WINAPI lineGetStatusMessages(HLINE,LPDWORD,LPDWORD);
 DWORD WINAPI lineGetTranslateCaps(HLINEAPP,DWORD,LPLINETRANSLATECAPS);
@@ -912,7 +920,9 @@ DWORD WINAPI lineInitialize(LPHLINEAPP,HINSTANCE,LINECALLBACK,LPCSTR,LPDWORD);
 LONG  WINAPI lineInitializeExA(LPHLINEAPP,HINSTANCE,LINECALLBACK,LPCSTR,LPDWORD,LPDWORD,LPLINEINITIALIZEEXPARAMS);
 LONG  WINAPI lineInitializeExW(LPHLINEAPP,HINSTANCE,LINECALLBACK,LPCWSTR,LPDWORD,LPDWORD,LPLINEINITIALIZEEXPARAMS);
 #define      lineInitializeEx WINELIB_NAME_AW(lineInitializeEx)
-DWORD WINAPI lineMakeCall(HLINE,LPHCALL,LPCSTR,DWORD,LPLINECALLPARAMS);
+DWORD WINAPI lineMakeCallA(HLINE,LPHCALL,LPCSTR,DWORD,LPLINECALLPARAMS);
+DWORD WINAPI lineMakeCallW(HLINE,LPHCALL,LPCWSTR,DWORD,LPLINECALLPARAMS);
+#define      lineMakeCall WINELIB_NAMEAW(lineMakeCall);
 DWORD WINAPI lineMonitorDigits(HCALL,DWORD);
 DWORD WINAPI lineMonitorMedia(HCALL,DWORD);
 DWORD WINAPI lineMonitorTones(HCALL,LPLINEMONITORTONE,DWORD);
@@ -951,7 +961,6 @@ DWORD WINAPI lineUncompleteCall(HLINE,DWORD);
 DWORD WINAPI lineUnHold(HCALL);
 DWORD WINAPI lineUnpark(HLINE,DWORD,LPHCALL,LPCSTR);
 
-DWORD WINAPI lineAddProviderA(LPCSTR,HWND,LPDWORD);
 DWORD WINAPI lineBlindTransferA(HCALL,LPCSTR,DWORD);
 DWORD WINAPI lineConfigDialogA(DWORD,HWND,LPCSTR);
 DWORD WINAPI lineConfigDialogEditA(DWORD,HWND,LPCSTR,LPVOID const,DWORD,LPVARSTRING);
@@ -965,16 +974,12 @@ DWORD WINAPI lineGetAddressStatusA(HLINE,DWORD,LPLINEADDRESSSTATUS);
 DWORD WINAPI lineGetAppPriorityA(LPCSTR,DWORD,LPLINEEXTENSIONID const,DWORD,LPVARSTRING,LPDWORD);
 DWORD WINAPI lineGetCallInfoA(HCALL,LPLINECALLINFO);
 DWORD WINAPI lineGetCountryA(DWORD,DWORD,LPLINECOUNTRYLIST);
-DWORD WINAPI lineGetDevCapsA(HLINEAPP,DWORD,DWORD,DWORD,LPLINEDEVCAPS);
 DWORD WINAPI lineGetDevConfigA(DWORD,LPVARSTRING,LPCSTR);
-DWORD WINAPI lineGetIDA(HLINE,DWORD,HCALL,DWORD,LPVARSTRING,LPCSTR);
 DWORD WINAPI lineGetIconA(DWORD,LPCSTR,HICON *);
 DWORD WINAPI lineGetLineDevStatusA(HLINE,LPLINEDEVSTATUS);
-DWORD WINAPI lineGetProviderListA(DWORD dwAPIVersion,LPLINEPROVIDERLIST);
 DWORD WINAPI lineGetRequestA(HLINEAPP,DWORD,LPVOID);
 DWORD WINAPI lineGetTranslateCapsA(HLINEAPP,DWORD,LPLINETRANSLATECAPS);
 DWORD WINAPI lineHandoffA(HCALL,LPCSTR,DWORD);
-DWORD WINAPI lineMakeCallA(HLINE,LPHCALL,LPCSTR,DWORD,LPLINECALLPARAMS);
 DWORD WINAPI lineOpenA(HLINEAPP,DWORD,LPHLINE,DWORD,DWORD,DWORD,DWORD,DWORD,LPLINECALLPARAMS);
 DWORD WINAPI lineParkA(HCALL,DWORD,LPCSTR,LPVARSTRING);
 DWORD WINAPI linePickupA(HLINE,DWORD,LPHCALL,LPCSTR,LPCSTR);
