@@ -103,6 +103,25 @@ extern "C" {
 #define siglongjmp(buf,val) longjmp(buf,val)
 #endif
 
+extern void __wine_rtl_unwind( EXCEPTION_REGISTRATION_RECORD* frame, EXCEPTION_RECORD *record,
+                               void (*target)(void) ) DECLSPEC_HIDDEN DECLSPEC_NORETURN;
+extern DWORD __wine_exception_handler( EXCEPTION_RECORD *record,
+                                       EXCEPTION_REGISTRATION_RECORD *frame,
+                                       CONTEXT *context,
+                                       EXCEPTION_REGISTRATION_RECORD **pdispatcher ) DECLSPEC_HIDDEN;
+extern DWORD __wine_exception_handler_page_fault( EXCEPTION_RECORD *record,
+                                                  EXCEPTION_REGISTRATION_RECORD *frame,
+                                                  CONTEXT *context,
+                                                  EXCEPTION_REGISTRATION_RECORD **pdispatcher ) DECLSPEC_HIDDEN;
+extern DWORD __wine_exception_handler_all( EXCEPTION_RECORD *record,
+                                           EXCEPTION_REGISTRATION_RECORD *frame,
+                                           CONTEXT *context,
+                                           EXCEPTION_REGISTRATION_RECORD **pdispatcher ) DECLSPEC_HIDDEN;
+extern DWORD __wine_finally_handler( EXCEPTION_RECORD *record,
+                                     EXCEPTION_REGISTRATION_RECORD *frame,
+                                     CONTEXT *context,
+                                     EXCEPTION_REGISTRATION_RECORD **pdispatcher ) DECLSPEC_HIDDEN;
+
 #define __TRY \
     do { __WINE_FRAME __f; \
          int __first = 1; \
@@ -222,6 +241,18 @@ static inline EXCEPTION_REGISTRATION_RECORD *__wine_pop_frame( EXCEPTION_REGISTR
 #endif
 }
 
+static inline EXCEPTION_REGISTRATION_RECORD *__wine_get_frame(void)
+{
+#if defined(__GNUC__) && defined(__i386__)
+    EXCEPTION_REGISTRATION_RECORD *ret;
+    __asm__ __volatile__(".byte 0x64\n\tmovl (0),%0" : "=r" (ret) );
+    return ret;
+#else
+    NT_TIB *teb = (NT_TIB *)NtCurrentTeb();
+    return teb->ExceptionList;
+#endif
+}
+
 /* Exception handling flags - from OS/2 2.0 exception handling */
 
 /* Win32 seems to use the same flags as ExceptionFlags in an EXCEPTION_RECORD */
@@ -243,108 +274,6 @@ static inline EXCEPTION_REGISTRATION_RECORD *__wine_pop_frame( EXCEPTION_REGISTR
 
 extern void __wine_enter_vm86( CONTEXT *context );
 
-#ifndef USE_COMPILER_EXCEPTIONS
-
-NTSYSAPI void WINAPI RtlUnwind(PVOID,PVOID,PEXCEPTION_RECORD,PVOID);
-
-/* wrapper for RtlUnwind since it clobbers registers on Windows */
-static inline void __wine_rtl_unwind( EXCEPTION_REGISTRATION_RECORD* frame, EXCEPTION_RECORD *record )
-{
-#if defined(__GNUC__) && defined(__i386__)
-    int dummy1, dummy2, dummy3;
-    __asm__ __volatile__("pushl %%ebp\n\t"
-                         "pushl %%ebx\n\t"
-                         "pushl $0\n\t"
-                         "pushl %2\n\t"
-                         "pushl $0\n\t"
-                         "pushl %1\n\t"
-                         "call *%0\n\t"
-                         "popl %%ebx\n\t"
-                         "popl %%ebp"
-                         : "=a" (dummy1), "=S" (dummy2), "=D" (dummy3)
-                         : "0" (RtlUnwind), "1" (frame), "2" (record)
-                         : "ecx", "edx", "memory" );
-#else
-    RtlUnwind( frame, 0, record, 0 );
-#endif
-}
-
-static inline void DECLSPEC_NORETURN __wine_unwind_frame( EXCEPTION_RECORD *record,
-                                                          EXCEPTION_REGISTRATION_RECORD *frame )
-{
-    __WINE_FRAME *wine_frame = (__WINE_FRAME *)frame;
-
-    /* hack to make GetExceptionCode() work in handler */
-    wine_frame->ExceptionCode   = record->ExceptionCode;
-    wine_frame->ExceptionRecord = wine_frame;
-
-    __wine_rtl_unwind( frame, record );
-    __wine_pop_frame( frame );
-    siglongjmp( wine_frame->jmp, 1 );
-}
-
-static inline DWORD __wine_exception_handler( EXCEPTION_RECORD *record,
-                                              EXCEPTION_REGISTRATION_RECORD *frame,
-                                              CONTEXT *context,
-                                              EXCEPTION_REGISTRATION_RECORD **pdispatcher )
-{
-    __WINE_FRAME *wine_frame = (__WINE_FRAME *)frame;
-    EXCEPTION_POINTERS ptrs;
-
-    if (record->ExceptionFlags & (EH_UNWINDING | EH_EXIT_UNWIND | EH_NESTED_CALL))
-        return ExceptionContinueSearch;
-
-    ptrs.ExceptionRecord = record;
-    ptrs.ContextRecord = context;
-    switch(wine_frame->u.filter( &ptrs ))
-    {
-    case EXCEPTION_CONTINUE_SEARCH:
-        return ExceptionContinueSearch;
-    case EXCEPTION_CONTINUE_EXECUTION:
-        return ExceptionContinueExecution;
-    case EXCEPTION_EXECUTE_HANDLER:
-        break;
-    }
-    __wine_unwind_frame( record, frame );
-}
-
-static inline DWORD __wine_exception_handler_page_fault( EXCEPTION_RECORD *record,
-                                                         EXCEPTION_REGISTRATION_RECORD *frame,
-                                                         CONTEXT *context,
-                                                         EXCEPTION_REGISTRATION_RECORD **pdispatcher )
-{
-    if (record->ExceptionFlags & (EH_UNWINDING | EH_EXIT_UNWIND | EH_NESTED_CALL))
-        return ExceptionContinueSearch;
-    if (record->ExceptionCode != STATUS_ACCESS_VIOLATION)
-        return ExceptionContinueSearch;
-    __wine_unwind_frame( record, frame );
-}
-
-static inline DWORD __wine_exception_handler_all( EXCEPTION_RECORD *record,
-                                                  EXCEPTION_REGISTRATION_RECORD *frame,
-                                                  CONTEXT *context,
-                                                  EXCEPTION_REGISTRATION_RECORD **pdispatcher )
-{
-    if (record->ExceptionFlags & (EH_UNWINDING | EH_EXIT_UNWIND | EH_NESTED_CALL))
-        return ExceptionContinueSearch;
-    __wine_unwind_frame( record, frame );
-}
-
-static inline DWORD __wine_finally_handler( EXCEPTION_RECORD *record,
-                                            EXCEPTION_REGISTRATION_RECORD *frame,
-                                            CONTEXT *context,
-                                            EXCEPTION_REGISTRATION_RECORD **pdispatcher )
-{
-    if (record->ExceptionFlags & (EH_UNWINDING | EH_EXIT_UNWIND))
-    {
-        __WINE_FRAME *wine_frame = (__WINE_FRAME *)frame;
-        wine_frame->u.finally_func( FALSE );
-    }
-    return ExceptionContinueSearch;
-}
-
-#endif /* USE_COMPILER_EXCEPTIONS */
-
 #ifdef __cplusplus
 }
 #endif
