@@ -71,6 +71,17 @@ extern void winetest_end_todo( const char* platform );
 extern int winetest_get_mainargs( char*** pargv );
 extern void winetest_wait_child_process( HANDLE process );
 
+extern const char *wine_dbgstr_wn( const WCHAR *str, int n );
+static inline const char *wine_dbgstr_w( const WCHAR *s ) { return wine_dbgstr_wn( s, -1 ); }
+
+/* strcmpW is avaiable for tests compiled under Wine, but not in standalone
+ * builds under Windows, so we reimplement it under a different name. */
+static inline int winetest_strcmpW( const WCHAR *str1, const WCHAR *str2 )
+{
+    while (*str1 && (*str1 == *str2)) { str1++; str2++; }
+    return *str1 - *str2;
+}
+
 #ifdef STANDALONE
 #define START_TEST(name) \
   static void func_##name(void); \
@@ -80,27 +91,35 @@ extern void winetest_wait_child_process( HANDLE process );
 #define START_TEST(name) void func_##name(void)
 #endif
 
+#if defined(__x86_64__) && defined(__GNUC__) && defined(__WINE_USE_MSVCRT)
+#define __winetest_cdecl __cdecl
+#define __winetest_va_list __builtin_ms_va_list
+#else
+#define __winetest_cdecl
+#define __winetest_va_list va_list
+#endif
+
 extern int broken( int condition );
-extern int winetest_vok( int condition, const char *msg, va_list ap );
-extern void winetest_vskip( const char *msg, va_list ap );
+extern int winetest_vok( int condition, const char *msg, __winetest_va_list ap );
+extern void winetest_vskip( const char *msg, __winetest_va_list ap );
 
 #ifdef __GNUC__
 
-extern int winetest_ok( int condition, const char *msg, ... ) __attribute__((format (printf,2,3) ));
-extern void winetest_skip( const char *msg, ... ) __attribute__((format (printf,1,2)));
-extern void winetest_win_skip( const char *msg, ... ) __attribute__((format (printf,1,2)));
-extern void winetest_trace( const char *msg, ... ) __attribute__((format (printf,1,2)));
+extern void __winetest_cdecl winetest_ok( int condition, const char *msg, ... ) __attribute__((format (printf,2,3) ));
+extern void __winetest_cdecl winetest_skip( const char *msg, ... ) __attribute__((format (printf,1,2)));
+extern void __winetest_cdecl winetest_win_skip( const char *msg, ... ) __attribute__((format (printf,1,2)));
+extern void __winetest_cdecl winetest_trace( const char *msg, ... ) __attribute__((format (printf,1,2)));
 
 #else /* __GNUC__ */
 
-extern int winetest_ok( int condition, const char *msg, ... );
-extern void winetest_skip( const char *msg, ... );
-extern void winetest_win_skip( const char *msg, ... );
-extern void winetest_trace( const char *msg, ... );
+extern void __winetest_cdecl winetest_ok( int condition, const char *msg, ... );
+extern void __winetest_cdecl winetest_skip( const char *msg, ... );
+extern void __winetest_cdecl winetest_win_skip( const char *msg, ... );
+extern void __winetest_cdecl winetest_trace( const char *msg, ... );
 
 #endif /* __GNUC__ */
 
-#define ok_(file, line)       (winetest_set_location(file, line), 0) ? 0 : winetest_ok
+#define ok_(file, line)       (winetest_set_location(file, line), 0) ? (void)0 : winetest_ok
 #define skip_(file, line)     (winetest_set_location(file, line), 0) ? (void)0 : winetest_skip
 #define win_skip_(file, line) (winetest_set_location(file, line), 0) ? (void)0 : winetest_win_skip
 #define trace_(file, line)    (winetest_set_location(file, line), 0) ? (void)0 : winetest_trace
@@ -166,6 +185,14 @@ extern void winetest_trace( const char *msg, ... );
 
 #include <stdio.h>
 
+#if defined(__x86_64__) && defined(__GNUC__) && defined(__WINE_USE_MSVCRT)
+# define __winetest_va_start(list,arg) __builtin_ms_va_start(list,arg)
+# define __winetest_va_end(list) __builtin_ms_va_end(list)
+#else
+# define __winetest_va_start(list,arg) va_start(list,arg)
+# define __winetest_va_end(list) va_end(list)
+#endif
+
 struct test
 {
     const char *name;
@@ -205,6 +232,8 @@ typedef struct
     int current_line;                /* line of current check */
     int todo_level;                  /* current todo nesting level */
     int todo_do_loop;
+    char *str_pos;                   /* position in debug buffer */
+    char strings[2000];              /* buffer for debug strings */
 } tls_data;
 static DWORD tls_index;
 
@@ -217,13 +246,33 @@ static tls_data* get_tls_data(void)
     data=TlsGetValue(tls_index);
     if (!data)
     {
-        data=HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(tls_data));
+        data=HeapAlloc(GetProcessHeap(), 0, sizeof(tls_data));
+        data->todo_level = 0;
+        data->str_pos = data->strings;
         TlsSetValue(tls_index,data);
     }
     SetLastError(last_error);
     return data;
 }
 
+/* allocate some tmp space for a string */
+static char *get_temp_buffer( size_t n )
+{
+    tls_data *data = get_tls_data();
+    char *res = data->str_pos;
+
+    if (res + n >= &data->strings[sizeof(data->strings)]) res = data->strings;
+    data->str_pos = res + n;
+    return res;
+}
+
+/* release extra space that we requested in gimme1() */
+static void release_temp_buffer( char *ptr, size_t size )
+{
+    tls_data *data = get_tls_data();
+    data->str_pos = ptr + size;
+}
+
 static void exit_process( int code )
 {
     fflush( stdout );
@@ -259,7 +308,7 @@ int broken( int condition )
  * Return:
  *   0 if condition does not have the expected value, 1 otherwise
  */
-int winetest_vok( int condition, const char *msg, va_list args )
+int winetest_vok( int condition, const char *msg, __winetest_va_list args )
 {
     tls_data* data=get_tls_data();
 
@@ -306,32 +355,30 @@ int winetest_vok( int condition, const char *msg, va_list args )
     }
 }
 
-int winetest_ok( int condition, const char *msg, ... )
+void __winetest_cdecl winetest_ok( int condition, const char *msg, ... )
 {
-    va_list valist;
-    int rc;
+    __winetest_va_list valist;
 
-    va_start(valist, msg);
-    rc=winetest_vok(condition, msg, valist);
-    va_end(valist);
-    return rc;
+    __winetest_va_start(valist, msg);
+    winetest_vok(condition, msg, valist);
+    __winetest_va_end(valist);
 }
 
-void winetest_trace( const char *msg, ... )
+void __winetest_cdecl winetest_trace( const char *msg, ... )
 {
-    va_list valist;
+    __winetest_va_list valist;
     tls_data* data=get_tls_data();
 
     if (winetest_debug > 0)
     {
         fprintf( stdout, "%s:%d: ", data->current_file, data->current_line );
-        va_start(valist, msg);
+        __winetest_va_start(valist, msg);
         vfprintf(stdout, msg, valist);
-        va_end(valist);
+        __winetest_va_end(valist);
     }
 }
 
-void winetest_vskip( const char *msg, va_list args )
+void winetest_vskip( const char *msg, __winetest_va_list args )
 {
     tls_data* data=get_tls_data();
 
@@ -340,23 +387,23 @@ void winetest_vskip( const char *msg, va_list args )
     skipped++;
 }
 
-void winetest_skip( const char *msg, ... )
+void __winetest_cdecl winetest_skip( const char *msg, ... )
 {
-    va_list valist;
-    va_start(valist, msg);
+    __winetest_va_list valist;
+    __winetest_va_start(valist, msg);
     winetest_vskip(msg, valist);
-    va_end(valist);
+    __winetest_va_end(valist);
 }
 
-void winetest_win_skip( const char *msg, ... )
+void __winetest_cdecl winetest_win_skip( const char *msg, ... )
 {
-    va_list valist;
-    va_start(valist, msg);
+    __winetest_va_list valist;
+    __winetest_va_start(valist, msg);
     if (strcmp(winetest_platform, "windows") == 0)
         winetest_vskip(msg, valist);
     else
         winetest_vok(0, msg, valist);
-    va_end(valist);
+    __winetest_va_end(valist);
 }
 
 void winetest_start_todo( const char* platform )
@@ -416,6 +463,62 @@ void winetest_wait_child_process( HANDLE process )
     }
 }
 
+const char *wine_dbgstr_wn( const WCHAR *str, int n )
+{
+    char *dst, *res;
+    size_t size;
+
+    if (!((ULONG_PTR)str >> 16))
+    {
+        if (!str) return "(null)";
+        res = get_temp_buffer( 6 );
+        sprintf( res, "#%04x", LOWORD(str) );
+        return res;
+    }
+    if (n == -1)
+    {
+        const WCHAR *end = str;
+        while (*end) end++;
+        n = end - str;
+    }
+    if (n < 0) n = 0;
+    size = 12 + min( 300, n * 5 );
+    dst = res = get_temp_buffer( size );
+    *dst++ = 'L';
+    *dst++ = '"';
+    while (n-- > 0 && dst <= res + size - 10)
+    {
+        WCHAR c = *str++;
+        switch (c)
+        {
+        case '\n': *dst++ = '\\'; *dst++ = 'n'; break;
+        case '\r': *dst++ = '\\'; *dst++ = 'r'; break;
+        case '\t': *dst++ = '\\'; *dst++ = 't'; break;
+        case '"':  *dst++ = '\\'; *dst++ = '"'; break;
+        case '\\': *dst++ = '\\'; *dst++ = '\\'; break;
+        default:
+            if (c >= ' ' && c <= 126)
+                *dst++ = c;
+            else
+            {
+                *dst++ = '\\';
+                sprintf(dst,"%04x",c);
+                dst+=4;
+            }
+        }
+    }
+    *dst++ = '"';
+    if (n > 0)
+    {
+        *dst++ = '.';
+        *dst++ = '.';
+        *dst++ = '.';
+    }
+    *dst++ = 0;
+    release_temp_buffer( res, dst - res );
+    return res;
+}
+
 /* Find a test by name */
 static const struct test *find_test( const char *name )
 {
