@@ -410,9 +410,6 @@ typedef struct
 #define SBM_SETRANGE16           (WM_USER+2)
 #define SBM_GETRANGE16           (WM_USER+3)
 #define SBM_ENABLE_ARROWS16      (WM_USER+4)
-#define SBM_SETRANGEREDRAW16     WM_NULL  /* Not in Win16 */
-#define SBM_SETSCROLLINFO16      WM_NULL  /* Not in Win16 */
-#define SBM_GETSCROLLINFO16      WM_NULL  /* Not in Win16 */
 
   /* CBT hook structures */
 
@@ -458,9 +455,6 @@ typedef struct
 #define BM_GETSTATE16          (WM_USER+2)
 #define BM_SETSTATE16          (WM_USER+3)
 #define BM_SETSTYLE16          (WM_USER+4)
-#define BM_CLICK16             WM_NULL  /* Does not exist in Win16 */
-#define BM_GETIMAGE16          WM_NULL  /* Does not exist in Win16 */
-#define BM_SETIMAGE16          WM_NULL  /* Does not exist in Win16 */
 
 /* Static Control Messages */
 #define STM_SETICON16       (WM_USER+0)
@@ -498,15 +492,6 @@ typedef struct
 #define EM_SETWORDBREAKPROC16      (WM_USER+32)
 #define EM_GETWORDBREAKPROC16      (WM_USER+33)
 #define EM_GETPASSWORDCHAR16       (WM_USER+34)
-/*
-  not in win16:
-    EM_SETMARGINS16            WM_NULL
-    EM_GETMARGINS16            WM_NULL
-    EM_GETLIMITTEXT16          WM_NULL
-    EM_POSFROMCHAR16           WM_NULL
-    EM_CHARFROMPOS16           WM_NULL
-    EM_SETLIMITTEXT16          WM_NULL - no name change in win16
-*/
 
 typedef struct
 {
@@ -970,12 +955,7 @@ INT16       WINAPI lstrcmpi16(LPCSTR,LPCSTR);
 
 /* undocumented functions */
 
-typedef VOID (*SYSTEMTIMERPROC)(WORD);
-
 void        WINAPI ConvertDialog32To16(LPCVOID,DWORD,LPVOID);
-WORD        WINAPI CreateSystemTimer(WORD,SYSTEMTIMERPROC);
-VOID        WINAPI DisableSystemTimers16(void);
-VOID        WINAPI EnableSystemTimers16(void);
 BOOL16      WINAPI EnumTaskWindows16(HTASK16,WNDENUMPROC16,LPARAM);
 BOOL16      WINAPI GrayString16(HDC16,HBRUSH16,GRAYSTRINGPROC16,LPARAM,
                                 INT16,INT16,INT16,INT16,INT16);
