@@ -380,6 +380,19 @@ typedef struct _INTERNET_PER_CONN_OPTION_LISTW {
 DECL_WINELIB_TYPE_AW(INTERNET_PER_CONN_OPTION_LIST)
 DECL_WINELIB_TYPE_AW(LPINTERNET_PER_CONN_OPTION_LIST)
 
+typedef struct _INTERNET_DIAGNOSTIC_SOCKET_INFO
+{
+    DWORD_PTR Socket;
+    DWORD     SourcePort;
+    DWORD     DestPort;
+    DWORD     Flags;
+} INTERNET_DIAGNOSTIC_SOCKET_INFO, *LPINTERNET_DIAGNOSTIC_SOCKET_INFO;
+
+#define IDSI_FLAG_KEEP_ALIVE 0x00000001
+#define IDSI_FLAG_SECURE     0x00000002
+#define IDSI_FLAG_PROXY      0x00000004
+#define IDSI_FLAG_TUNNEL     0x00000008
+
 BOOLAPI InternetTimeFromSystemTimeA(CONST SYSTEMTIME *,DWORD ,LPSTR ,DWORD);
 BOOLAPI InternetTimeFromSystemTimeW(CONST SYSTEMTIME *,DWORD ,LPWSTR ,DWORD);
 #define InternetTimeFromSystemTime WINELIB_NAME_AW(InternetTimeFromSystemTime)
@@ -1545,6 +1558,12 @@ typedef struct _INTERNET_CACHE_ENTRY_INFOW {
 DECL_WINELIB_TYPE_AW(INTERNET_CACHE_ENTRY_INFO)
 DECL_WINELIB_TYPE_AW(LPINTERNET_CACHE_ENTRY_INFO)
 
+typedef struct _INTERNET_CACHE_TIMESTAMPS
+{
+    FILETIME ftExpires;
+    FILETIME ftLastModified;
+} INTERNET_CACHE_TIMESTAMPS, *LPINTERNET_CACHE_TIMESTAMPS;
+
 BOOLAPI CreateUrlCacheEntryA(LPCSTR ,DWORD ,LPCSTR ,LPSTR ,DWORD);
 BOOLAPI CreateUrlCacheEntryW(LPCWSTR ,DWORD ,LPCWSTR ,LPWSTR ,DWORD);
 #define CreateUrlCacheEntry  WINELIB_NAME_AW(CreateUrlCacheEntry)
