@@ -113,6 +113,9 @@ WINADVAPI LSTATUS   WINAPI RegCreateKeyExW(HKEY,LPCWSTR,DWORD,LPWSTR,DWORD,REGSA
 WINADVAPI LSTATUS   WINAPI RegDeleteKeyA(HKEY,LPCSTR);
 WINADVAPI LSTATUS   WINAPI RegDeleteKeyW(HKEY,LPCWSTR);
 #define                    RegDeleteKey WINELIB_NAME_AW(RegDeleteKey)
+WINADVAPI LSTATUS   WINAPI RegDeleteKeyExA(HKEY,LPCSTR,REGSAM,DWORD);
+WINADVAPI LSTATUS   WINAPI RegDeleteKeyExW(HKEY,LPCWSTR,REGSAM,DWORD);
+#define                    RegDeleteKeyEx WINELIB_NAME_AW(RegDeleteKeyEx)
 WINADVAPI LSTATUS   WINAPI RegDeleteKeyValueA(HKEY,LPCSTR,LPCSTR);
 WINADVAPI LSTATUS   WINAPI RegDeleteKeyValueW(HKEY,LPCWSTR,LPCWSTR);
 #define                    RegDeleteKeyValue WINELIB_NAME_AW(RegDeleteKeyValue)
