@@ -27,6 +27,9 @@
  * of the LGPL is applied is otherwise unspecified.
  */
 
+#include "config.h"
+#include "wine/port.h"
+
 #include <assert.h>
 #include <limits.h>
 #include <stdio.h>
@@ -415,6 +418,7 @@ int vsnprintfW(WCHAR *str, size_t len, const WCHAR *format, va_list valist)
                     sprintf(buffiter, "%d", fieldlen);
                     while (*buffiter)
                         *fmta++ = *buffiter++;
+                    iter++;
                 }
                 else
                     while (isdigit(*iter))
