@@ -4,7 +4,7 @@
  *
  * Based on a Microsoft DDK sample
  *
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 // #define LOG_ENABLED
@@ -25,7 +21,6 @@
 
 // VBOX start
 #include <VBox/err.h>
-#include <VBox/VBoxGuest.h>
 #include <VBox/VBoxGuestLib.h>
 // VBOX end
 
@@ -278,10 +273,10 @@ static void vboxDeviceRemoved (PDEVICE_EXTENSION devExt)
 
             vboxRC = VbglGRPerform (&req->header);
 
-            if (RT_FAILURE(vboxRC) || RT_FAILURE(req->header.rc))
+            if (RT_FAILURE(vboxRC))
             {
                 dprintf(("VBoxMouse::vboxDeviceRemoved: ERROR communicating new mouse capabilities to VMMDev.\n"
-                         "rc = %d, VMMDev rc = %Rrc\n", vboxRC, req->header.rc));
+                         "rc = %Rrc\n", vboxRC));
             }
 
             VbglGRFree (&req->header);
@@ -332,16 +327,16 @@ static void vboxInformHost (PDEVICE_EXTENSION devExt)
             if (RT_SUCCESS(vboxRC))
             {
                 /* Inform host that we support absolute */
-                req->mouseFeatures = VBOXGUEST_MOUSE_GUEST_CAN_ABSOLUTE;
+                req->mouseFeatures = VMMDEV_MOUSE_GUEST_CAN_ABSOLUTE;
                 req->pointerXPos = 0;
                 req->pointerYPos = 0;
 
                 vboxRC = VbglGRPerform (&req->header);
 
-                if (RT_FAILURE(vboxRC) || RT_FAILURE(req->header.rc))
+                if (RT_FAILURE(vboxRC))
                 {
-                    dprintf(("VBoxMouse::vboxInformHost: ERROR communicating new mouse capabilities to VMMDev."
-                              "rc = %d, VMMDev rc = %Rrc\n", vboxRC, req->header.rc));
+                    dprintf(("VBoxMouse::vboxInformHost: ERROR communicating new mouse capabilities to VMMDev. "
+                             "rc = %Rrc\n", vboxRC));
                 }
                 else
                 {
@@ -1116,9 +1111,9 @@ Return Value:
     {
         int rc = VbglGRPerform (&req->header);
 
-        if (RT_SUCCESS(rc) && RT_SUCCESS(req->header.rc))
+        if (RT_SUCCESS(rc))
         {
-            if (req->mouseFeatures & VBOXGUEST_MOUSE_HOST_CAN_ABSOLUTE)
+            if (req->mouseFeatures & VMMDEV_MOUSE_HOST_CAN_ABSOLUTE)
             {
                 PMOUSE_INPUT_DATA inputData = InputDataStart;
                 while (inputData < InputDataEnd)
@@ -1133,8 +1128,8 @@ Return Value:
         }
         else
         {
-            dprintf(("VBoxMouse::VBoxMouse_ServiceCallback: ERROR querying mouse capabilities from VMMDev."
-                     "rc = %Rrc, VMMDev rc = %Rrc\n", rc, req->header.rc));
+            dprintf(("VBoxMouse::VBoxMouse_ServiceCallback: ERROR querying mouse capabilities from VMMDev. "
+                     "rc = %Rrc\n", rc));
         }
     }
 // VBOX end
