@@ -2,7 +2,7 @@
  *
  * VBoxGINA -- Windows Logon DLL for VirtualBox Dialog Code
  *
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -11,13 +11,10 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <windows.h>
+#include <stdio.h>      /* Needed for swprintf() */
 #include "Dialog.h"
 #include "WinWlx.h"
 #include "Helper.h"
@@ -108,6 +105,74 @@ void hookDialogBoxes(PVOID pWinlogonFunctions, DWORD dwWlxVersion)
 
 #define CREDPOLL_TIMERID 0x1243
 
+BOOL credentialsToUI(HWND hwndUserId, HWND hwndPassword, HWND hwndDomain)
+{
+    BOOL bIsFQDN = FALSE;
+    wchar_t szUserFQDN[512]; /* VMMDEV_CREDENTIALS_STRLEN + 255 bytes max. for FQDN */
+    if (hwndDomain)
+    {
+        /* search the domain combo box for our required domain and select it */
+        Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Trying to find domain entry in combo box ...\n"));
+        DWORD dwIndex = (DWORD) SendMessage(hwndDomain, CB_FINDSTRING,
+                                            0, (LPARAM)g_Domain);
+        if (dwIndex != CB_ERR)
+        {
+            Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Found domain at combo box pos %ld\n", dwIndex));
+            SendMessage(hwndDomain, CB_SETCURSEL, (WPARAM) dwIndex, 0);
+            EnableWindow(hwndDomain, FALSE);
+        }
+        else
+        {
+            Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Domain not found in combo box ...\n"));
+
+            /* If the domain value has a dot (.) in it, it is a FQDN (Fully Qualified Domain Name)
+             * which will not work with the combo box selection because Windows only keeps the
+             * NETBIOS names to the left most part of the domain name there. Of course a FQDN
+             * then will not be found by the search in the block above.
+             *
+             * To solve this problem the FQDN domain value will be appended at the user name value
+             * (Kerberos style) using an "@", e.g. "<user-name>@full.qualified.domain".
+             *
+             */
+            size_t l = wcslen(g_Domain);
+            if (l > 255)
+                Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Warning! FQDN is too long (max 255 bytes), will be truncated!\n"));
+
+            if (wcslen(g_Username) > 0) /* We need a user name that we can use in caes of a FQDN */
+            {
+                if (l > 16) /* Domain name is longer than 16 chars, cannot be a NetBIOS name anymore */
+                {
+                    Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Domain seems to be a FQDN (length)!\n"));
+                    bIsFQDN = TRUE;
+                }
+                else if (   l > 0
+                         && wcsstr(g_Domain, L".") != NULL) /* if we found a dot (.) in the domain name, this has to be a FQDN */
+                {
+                    Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Domain seems to be a FQDN (dot)!\n"));
+                    bIsFQDN = TRUE;
+                }
+
+                if (bIsFQDN)
+                {
+                    swprintf(szUserFQDN, sizeof(szUserFQDN) / sizeof(wchar_t), L"%s@%s", g_Username, g_Domain);
+                    Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: FQDN user name is now: %s!\n", szUserFQDN));
+                }
+            }
+        }
+    }
+    if (hwndUserId)
+    {
+        if (!bIsFQDN)
+            SendMessage(hwndUserId, WM_SETTEXT, 0, (LPARAM)g_Username);
+        else
+            SendMessage(hwndUserId, WM_SETTEXT, 0, (LPARAM)szUserFQDN);
+    }
+    if (hwndPassword)
+        SendMessage(hwndPassword, WM_SETTEXT, 0, (LPARAM)g_Password);
+
+    return TRUE;
+}
+
 INT_PTR CALLBACK MyWlxLoggedOutSASDlgProc(HWND   hwndDlg,  // handle to dialog box
                                           UINT   uMsg,     // message
                                           WPARAM wParam,   // first message parameter
@@ -155,24 +220,8 @@ INT_PTR CALLBACK MyWlxLoggedOutSASDlgProc(HWND   hwndDlg,  // handle to dialog b
                 /* query the credentials from VBox */
                 if (credentialsRetrieve())
                 {
-                    if (hwndUserId)
-                        SendMessage(hwndUserId, WM_SETTEXT, 0, (LPARAM)g_Username);
-                    if (hwndPassword)
-                        SendMessage(hwndPassword, WM_SETTEXT, 0, (LPARAM)g_Password);
-                    if (hwndDomain)
-                    {
-                        /* search the domain combo box for our required domain and select it */
-                        Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Find domain entry ...\n"));
-                        DWORD dwIndex = (DWORD) SendMessage(hwndDomain, CB_FINDSTRING,
-                                                            0, (LPARAM)g_Domain);
-                        if (dwIndex != CB_ERR)
-                        {
-                            Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Found domain at pos %ld\n", dwIndex));
-                            SendMessage(hwndDomain, CB_SETCURSEL, (WPARAM) dwIndex, 0);
-                            EnableWindow(hwndDomain, FALSE);
-                        }
-                        else Log(("VBoxGINA::MyWlxLoggedOutSASDlgProc: Domain entry not found!"));
-                    }
+                    /* fill in credentials to appropriate UI elements */
+                    credentialsToUI(hwndUserId, hwndPassword, hwndDomain);
 
                     /* we got the credentials, null them out */
                     credentialsReset();
@@ -207,12 +256,8 @@ INT_PTR CALLBACK MyWlxLoggedOutSASDlgProc(HWND   hwndDlg,  // handle to dialog b
                 {
                     if (credentialsRetrieve())
                     {
-                        if (hwndUserId)
-                            SendMessage(hwndUserId, WM_SETTEXT, 0, (LPARAM)g_Username);
-                        if (hwndPassword)
-                            SendMessage(hwndPassword, WM_SETTEXT, 0, (LPARAM)g_Password);
-                        if (hwndDomain)
-                            SendMessage(hwndDomain, WM_SETTEXT, 0, (LPARAM)g_Domain);
+                        /* fill in credentials to appropriate UI elements */
+                        credentialsToUI(hwndUserId, hwndPassword, hwndDomain);
 
                         /* we got the credentials, null them out */
                         credentialsReset();
