@@ -2,7 +2,7 @@
  *
  * VBoxGINA -- Windows Logon DLL for VirtualBox
  *
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -11,10 +11,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef __H_VBOXGINA
@@ -23,7 +19,6 @@
 #include <iprt/initterm.h>
 #include <iprt/log.h>
 
-#include <VBox/VBoxGuest.h>
 #include <VBox/Log.h>
 
 /** Handle to Winlogon service */
@@ -63,4 +58,4 @@ typedef VOID (WINAPI *PGWLXDISCONNECTNOTIFY)(PVOID);
 
 
 
-#endif /* __H_VBOXGINA */
\ No newline at end of file
+#endif /* __H_VBOXGINA */
