@@ -3,7 +3,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -12,19 +12,17 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 //#define LOG_ENABLED
 
 #include "VBoxGuest_Internal.h"
 #include "Helper.h"
+#include <iprt/string.h>
 #include <VBox/err.h>
 #include <VBox/log.h>
 #include <VBox/VBoxGuestLib.h>
+#include <VBox/version.h>
 
 #ifdef ALLOC_PRAGMA
 #pragma alloc_text (PAGE, VBoxScanPCIResourceList)
@@ -192,56 +190,80 @@ void hlpVBoxUnmapVMMDevMemory (PVBOXGUESTDEVEXT pDevExt)
     pDevExt->memoryLength = 0;
 }
 
+/** @todo maybe we should drop this routine entirely later because we detecting
+ *        the running OS via VBoxService in ring 3 using guest properties since a while. */
 NTSTATUS hlpVBoxReportGuestInfo (PVBOXGUESTDEVEXT pDevExt)
 {
-    VMMDevReportGuestInfo *req = NULL;
-
-    int rc = VbglGRAlloc ((VMMDevRequestHeader **)&req, sizeof (VMMDevReportGuestInfo), VMMDevReq_ReportGuestInfo);
-
+    VMMDevReportGuestInfo *pReq = NULL;
+    int rc = VbglGRAlloc ((VMMDevRequestHeader **)&pReq, sizeof (VMMDevReportGuestInfo), VMMDevReq_ReportGuestInfo);
     dprintf(("hlpVBoxReportGuestInfo: VbglGRAlloc rc = %d\n", rc));
-
     if (RT_SUCCESS(rc))
     {
-        req->guestInfo.additionsVersion = VMMDEV_VERSION;
+        pReq->guestInfo.additionsVersion = VMMDEV_VERSION;
 
         /* we've already determined the Windows product before */
         switch (winVersion)
         {
             case WINNT4:
-                req->guestInfo.osType = VBOXOSTYPE_WinNT4;
+                pReq->guestInfo.osType = VBOXOSTYPE_WinNT4;
                 break;
             case WIN2K:
-                req->guestInfo.osType = VBOXOSTYPE_Win2k;
+                pReq->guestInfo.osType = VBOXOSTYPE_Win2k;
                 break;
             case WINXP:
-                req->guestInfo.osType = VBOXOSTYPE_WinXP;
+                pReq->guestInfo.osType = VBOXOSTYPE_WinXP;
                 break;
             case WIN2K3:
-                req->guestInfo.osType = VBOXOSTYPE_Win2k3;
+                pReq->guestInfo.osType = VBOXOSTYPE_Win2k3;
                 break;
             case WINVISTA:
-                req->guestInfo.osType = VBOXOSTYPE_WinVista;
+                pReq->guestInfo.osType = VBOXOSTYPE_WinVista;
+                break;
+            case WIN7:
+                pReq->guestInfo.osType = VBOXOSTYPE_Win7;
                 break;
             default:
                 /* we don't know, therefore NT family */
-                req->guestInfo.osType = VBOXOSTYPE_WinNT;
+                pReq->guestInfo.osType = VBOXOSTYPE_WinNT;
                 break;
         }
 
         /** @todo registry lookup for additional information */
 
-
-        rc = VbglGRPerform (&req->header);
-
-        if (RT_FAILURE(rc) || RT_FAILURE(req->header.rc))
+        rc = VbglGRPerform (&pReq->header);
+        if (RT_FAILURE(rc))
         {
-            dprintf(("VBoxGuest::hlpVBoxReportGuestInfo: error reporting guest info to VMMDev."
-                      "rc = %d, VMMDev rc = %Rrc\n", rc, req->header.rc));
+            dprintf(("VBoxGuest::hlpVBoxReportGuestInfo: error reporting guest info to VMMDev. "
+                     "rc = %Rrc\n", rc));
         }
 
-        rc = RT_SUCCESS(rc) ? req->header.rc : rc;
+        VbglGRFree (&pReq->header);
+    }
 
-        VbglGRFree (&req->header);
+    VMMDevReportGuestInfo2 *pReq2 = NULL;
+    if (RT_SUCCESS(rc))
+        rc = VbglGRAlloc ((VMMDevRequestHeader **)&pReq2, sizeof (VMMDevReportGuestInfo2), VMMDevReq_ReportGuestInfo2);
+    dprintf(("hlpVBoxReportGuestInfo2: VbglGRAlloc rc = %d\n", rc));
+
+    if (RT_SUCCESS(rc))
+    {
+        pReq2->guestInfo.additionsMajor = VBOX_VERSION_MAJOR;
+        pReq2->guestInfo.additionsMinor = VBOX_VERSION_MINOR;
+        pReq2->guestInfo.additionsBuild = VBOX_VERSION_BUILD;
+        pReq2->guestInfo.additionsRevision = VBOX_SVN_REV;
+        pReq2->guestInfo.additionsFeatures = 0;
+        RTStrCopy(pReq2->guestInfo.szName, sizeof(pReq2->guestInfo.szName), VBOX_VERSION_STRING);
+
+        rc = VbglGRPerform (&pReq2->header);
+        if (RT_FAILURE(rc))
+        {
+            dprintf(("VBoxGuest::hlpVBoxReportGuestInfo: error reporting guest info to VMMDev. "
+                     "rc = %Rrc\n", rc));
+        }
+        if (rc == VERR_NOT_IMPLEMENTED)
+            rc = VINF_SUCCESS;
+
+        VbglGRFree (&pReq2->header);
     }
 
     return RT_FAILURE(rc) ? STATUS_UNSUCCESSFUL : STATUS_SUCCESS;
