@@ -2,7 +2,7 @@
  *
  * VBoxGuest -- VirtualBox Win32 guest support driver PnP code
  *
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -11,10 +11,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 // enable backdoor logging
@@ -558,10 +554,10 @@ NTSTATUS VBoxGuestPower(PDEVICE_OBJECT pDevObj, PIRP pIrp)
 
                                 rc = VbglGRPerform (&req->header);
 
-                                if (RT_FAILURE(rc) || RT_FAILURE(req->header.rc))
+                                if (RT_FAILURE(rc))
                                 {
-                                    dprintf(("VBoxGuest::PowerStateRequest: error communicating new power status to VMMDev."
-                                             "rc = %d, VMMDev rc = %Rrc\n", rc, req->header.rc));
+                                    dprintf(("VBoxGuest::PowerStateRequest: error communicating new power status to VMMDev. "
+                                             "rc = %Rrc\n", rc));
                                 }
 
                                 VbglGRFree (&req->header);
@@ -586,10 +582,10 @@ NTSTATUS VBoxGuestPower(PDEVICE_OBJECT pDevObj, PIRP pIrp)
 
                                     int rc = VbglGRPerform (&req->header);
 
-                                    if (RT_FAILURE(rc) || RT_FAILURE(req->header.rc))
+                                    if (RT_FAILURE(rc))
                                     {
-                                        dprintf(("VBoxGuest::PowerStateRequest: error communicating new power status to VMMDev."
-                                                 "rc = %d, VMMDev rc = %Rrc\n", rc, req->header.rc));
+                                        dprintf(("VBoxGuest::PowerStateRequest: error communicating new power status to VMMDev. "
+                                                 "rc = %Rrc\n", rc));
                                     }
                                 }
                             }
