@@ -1,11 +1,10 @@
+/* $Id: VBoxDisplay.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
- *
- * VBoxSeamless - Display notifications
- *
+ * VBoxSeamless - Display notifications.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 #define _WIN32_WINNT 0x0500
 #include <windows.h>
@@ -25,7 +20,7 @@
 #include "VBoxSeamless.h"
 #include <VBoxHook.h>
 #include <VBoxDisplay.h>
-#include <VBox/VBoxDev.h>
+#include <VBox/VMMDev.h>
 #include <iprt/assert.h>
 #include "helpers.h"
 #include <malloc.h>
@@ -39,11 +34,34 @@ typedef struct _VBOXDISPLAYCONTEXT
 
     /* EnumDisplayDevices does not exist in NT. isVBoxDisplayDriverActive et al. are using these functions. */
     BOOL (WINAPI * pfnEnumDisplayDevices)(IN LPCSTR lpDevice, IN DWORD iDevNum, OUT PDISPLAY_DEVICEA lpDisplayDevice, IN DWORD dwFlags);
-
 } VBOXDISPLAYCONTEXT;
 
 static VBOXDISPLAYCONTEXT gCtx = {0};
 
+#ifdef VBOXWDDM
+static bool vboxWddmReinitVideoModes(VBOXDISPLAYCONTEXT *pCtx)
+{
+    VBOXDISPIFESCAPE escape = {0};
+    escape.escapeCode = VBOXESC_REINITVIDEOMODES;
+    DWORD err = VBoxDispIfEscape(&pCtx->pEnv->dispIf, &escape, 0);
+    if (err != NO_ERROR)
+    {
+        Log((__FUNCTION__": VBoxDispIfEscape failed with err (%d)\n", err));
+        return false;
+    }
+    return true;
+}
+
+typedef enum
+{
+    VBOXDISPLAY_DRIVER_TYPE_UNKNOWN = 0,
+    VBOXDISPLAY_DRIVER_TYPE_XPDM    = 1,
+    VBOXDISPLAY_DRIVER_TYPE_WDDM    = 2
+} VBOXDISPLAY_DRIVER_TYPE;
+
+static VBOXDISPLAY_DRIVER_TYPE getVBoxDisplayDriverType (VBOXDISPLAYCONTEXT *pCtx);
+#endif
+
 int VBoxDisplayInit(const VBOXSERVICEENV *pEnv, void **ppInstance, bool *pfStartThread)
 {
     OSVERSIONINFO OSinfo;
@@ -57,15 +75,34 @@ int VBoxDisplayInit(const VBOXSERVICEENV *pEnv, void **ppInstance, bool *pfStart
     if (NULL == hUser)
     {
         Log(("VBoxTray: Could not get module handle of USER32.DLL!\n"));
-        return VERR_NOT_IMPLEMENTED; 
+        return VERR_NOT_IMPLEMENTED;
     }
     else if (OSinfo.dwMajorVersion >= 5)        /* APIs available only on W2K and up! */
     {
-        *(uintptr_t *)&gCtx.pfnChangeDisplaySettingsEx = (uintptr_t)GetProcAddress(hUser, "ChangeDisplaySettingsExA"); 
+        *(uintptr_t *)&gCtx.pfnChangeDisplaySettingsEx = (uintptr_t)GetProcAddress(hUser, "ChangeDisplaySettingsExA");
         Log(("VBoxTray: pfnChangeDisplaySettingsEx = %p\n", gCtx.pfnChangeDisplaySettingsEx));
 
-        *(uintptr_t *)&gCtx.pfnEnumDisplayDevices = (uintptr_t)GetProcAddress(hUser, "EnumDisplayDevicesA"); 
+        *(uintptr_t *)&gCtx.pfnEnumDisplayDevices = (uintptr_t)GetProcAddress(hUser, "EnumDisplayDevicesA");
         Log(("VBoxTray: pfnEnumDisplayDevices = %p\n", gCtx.pfnEnumDisplayDevices));
+
+#ifdef VBOXWDDM
+        if (OSinfo.dwMajorVersion >= 6)
+        {
+            /* this is vista and up, check if we need to switch the display driver if to WDDM mode */
+            Log(("VBoxTray: this is vista and up\n"));
+            VBOXDISPLAY_DRIVER_TYPE enmType = getVBoxDisplayDriverType (&gCtx);
+            if (enmType == VBOXDISPLAY_DRIVER_TYPE_WDDM)
+            {
+                Log(("VBoxTray: WDDM driver is installed, switching display driver if to WDDM mode\n"));
+                /* this is hacky, but the most easiest way */
+                DWORD err = VBoxDispIfSwitchMode(const_cast<PVBOXDISPIF>(&pEnv->dispIf), VBOXDISPIF_MODE_WDDM, NULL /* old mode, we don't care about it */);
+                if (err == NO_ERROR)
+                    Log(("VBoxTray: DispIf switched to WDDM mode successfully\n"));
+                else
+                    Log(("VBoxTray: failed to switch DispIf to WDDM mode, err (%d)\n", err));
+            }
+        }
+#endif
     }
     else if (OSinfo.dwMajorVersion <= 4)            /* Windows NT 4.0 */
     {
@@ -74,7 +111,7 @@ int VBoxDisplayInit(const VBOXSERVICEENV *pEnv, void **ppInstance, bool *pfStart
     else                                /* Unsupported platform */
     {
         Log(("VBoxTray: Warning, display for platform not handled yet!\n"));
-        return VERR_NOT_IMPLEMENTED; 
+        return VERR_NOT_IMPLEMENTED;
     }
 
     Log(("VBoxTray: Display init successful.\n"));
@@ -89,11 +126,19 @@ void VBoxDisplayDestroy (const VBOXSERVICEENV *pEnv, void *pInstance)
     return;
 }
 
+#ifdef VBOXWDDM
+static VBOXDISPLAY_DRIVER_TYPE getVBoxDisplayDriverType (VBOXDISPLAYCONTEXT *pCtx)
+#else
 static bool isVBoxDisplayDriverActive (VBOXDISPLAYCONTEXT *pCtx)
+#endif
 {
+#ifdef VBOXWDDM
+    VBOXDISPLAY_DRIVER_TYPE enmType = VBOXDISPLAY_DRIVER_TYPE_UNKNOWN;
+#else
     bool result = false;
+#endif
 
-    if( pCtx->pfnEnumDisplayDevices ) 
+    if( pCtx->pfnEnumDisplayDevices )
     {
         INT devNum = 0;
         DISPLAY_DEVICE dispDevice;
@@ -114,26 +159,31 @@ static bool isVBoxDisplayDriverActive (VBOXDISPLAYCONTEXT *pCtx)
                           &dispDevice.DeviceID[0],
                           &dispDevice.DeviceKey[0],
                           dispDevice.StateFlags));
-    
+
             if (dispDevice.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE)
             {
                 Log(("Primary device.\n"));
-    
+
                 if (strcmp(&dispDevice.DeviceString[0], "VirtualBox Graphics Adapter") == 0)
+#ifndef VBOXWDDM
                     result = true;
-    
+#else
+                    enmType = VBOXDISPLAY_DRIVER_TYPE_XPDM;
+                else if (strcmp(&dispDevice.DeviceString[0], "VirtualBox Graphics Adapter (Microsoft Corporation - WDDM)") == 0)
+                    enmType = VBOXDISPLAY_DRIVER_TYPE_WDDM;
+#endif
                 break;
             }
-    
+
             FillMemory(&dispDevice, sizeof(DISPLAY_DEVICE), 0);
-    
+
             dispDevice.cb = sizeof(DISPLAY_DEVICE);
-    
+
             devNum++;
         }
     }
     else    /* This must be NT 4 or something really old, so don't use EnumDisplayDevices() here  ... */
-    {       
+    {
         Log(("Checking for active VBox display driver (NT or older)...\n"));
 
         DEVMODE tempDevMode;
@@ -143,27 +193,37 @@ static bool isVBoxDisplayDriverActive (VBOXDISPLAYCONTEXT *pCtx)
 
         /* Check for the short name, because all long stuff would be truncated */
         if (strcmp((char*)&tempDevMode.dmDeviceName[0], "VBoxDisp") == 0)
+#ifndef VBOXWDDM
             result = true;
+#else
+            enmType = VBOXDISPLAY_DRIVER_TYPE_XPDM;
+#endif
     }
 
+#ifndef VBOXWDDM
     return result;
+#else
+    return enmType;
+#endif
 }
 
 /* Returns TRUE to try again. */
-static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsPerPixel)
+static BOOL ResizeDisplayDevice(
+        ULONG Id, DWORD Width, DWORD Height, DWORD BitsPerPixel
+        )
 {
     BOOL fModeReset = (Width == 0 && Height == 0 && BitsPerPixel == 0);
-    
+
     DISPLAY_DEVICE DisplayDevice;
 
     ZeroMemory(&DisplayDevice, sizeof(DisplayDevice));
     DisplayDevice.cb = sizeof(DisplayDevice);
-    
+
     /* Find out how many display devices the system has */
     DWORD NumDevices = 0;
     DWORD i = 0;
     while (EnumDisplayDevices (NULL, i, &DisplayDevice, 0))
-    { 
+    {
         Log(("[%d] %s\n", i, DisplayDevice.DeviceName));
 
         if (DisplayDevice.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE)
@@ -173,40 +233,40 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
         }
         else if (!(DisplayDevice.StateFlags & DISPLAY_DEVICE_MIRRORING_DRIVER))
         {
-            
+
             Log(("ResizeDisplayDevice: Found secondary device. err %d\n", GetLastError ()));
             NumDevices++;
         }
-        
+
         ZeroMemory(&DisplayDevice, sizeof(DisplayDevice));
         DisplayDevice.cb = sizeof(DisplayDevice);
         i++;
     }
-    
+
     Log(("ResizeDisplayDevice: Found total %d devices. err %d\n", NumDevices, GetLastError ()));
-    
+
     if (NumDevices == 0 || Id >= NumDevices)
     {
         Log(("ResizeDisplayDevice: Requested identifier %d is invalid. err %d\n", Id, GetLastError ()));
         return FALSE;
     }
-    
+
     DISPLAY_DEVICE *paDisplayDevices = (DISPLAY_DEVICE *)alloca (sizeof (DISPLAY_DEVICE) * NumDevices);
     DEVMODE *paDeviceModes = (DEVMODE *)alloca (sizeof (DEVMODE) * NumDevices);
     RECTL *paRects = (RECTL *)alloca (sizeof (RECTL) * NumDevices);
-    
+
     /* Fetch information about current devices and modes. */
     DWORD DevNum = 0;
     DWORD DevPrimaryNum = 0;
-    
+
     ZeroMemory(&DisplayDevice, sizeof(DISPLAY_DEVICE));
     DisplayDevice.cb = sizeof(DISPLAY_DEVICE);
-    
+
     i = 0;
     while (EnumDisplayDevices (NULL, i, &DisplayDevice, 0))
-    { 
+    {
         Log(("ResizeDisplayDevice: [%d(%d)] %s\n", i, DevNum, DisplayDevice.DeviceName));
-        
+
         BOOL bFetchDevice = FALSE;
 
         if (DisplayDevice.StateFlags & DISPLAY_DEVICE_PRIMARY_DEVICE)
@@ -217,11 +277,11 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
         }
         else if (!(DisplayDevice.StateFlags & DISPLAY_DEVICE_MIRRORING_DRIVER))
         {
-            
+
             Log(("ResizeDisplayDevice: Found secondary device. err %d\n", GetLastError ()));
             bFetchDevice = TRUE;
         }
-        
+
         if (bFetchDevice)
         {
             if (DevNum >= NumDevices)
@@ -229,9 +289,9 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
                 Log(("ResizeDisplayDevice: %d >= %d\n", NumDevices, DevNum));
                 return FALSE;
             }
-        
+
             paDisplayDevices[DevNum] = DisplayDevice;
-            
+
             /* First try to get the video mode stored in registry (ENUM_REGISTRY_SETTINGS).
              * A secondary display could be not active at the moment and would not have
              * a current video mode (ENUM_CURRENT_SETTINGS).
@@ -244,7 +304,7 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
                 Log(("ResizeDisplayDevice: EnumDisplaySettings err %d\n", GetLastError ()));
                 return FALSE;
             }
-            
+
             if (   paDeviceModes[DevNum].dmPelsWidth == 0
                 || paDeviceModes[DevNum].dmPelsHeight == 0)
             {
@@ -264,26 +324,26 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
                     ZeroMemory(&paDeviceModes[DevNum], sizeof(DEVMODE));
                 }
             }
-            
+
             Log(("ResizeDisplayDevice: %dx%dx%d at %d,%d\n",
                     paDeviceModes[DevNum].dmPelsWidth,
                     paDeviceModes[DevNum].dmPelsHeight,
                     paDeviceModes[DevNum].dmBitsPerPel,
                     paDeviceModes[DevNum].dmPosition.x,
                     paDeviceModes[DevNum].dmPosition.y));
-                    
+
             paRects[DevNum].left   = paDeviceModes[DevNum].dmPosition.x;
             paRects[DevNum].top    = paDeviceModes[DevNum].dmPosition.y;
             paRects[DevNum].right  = paDeviceModes[DevNum].dmPosition.x + paDeviceModes[DevNum].dmPelsWidth;
             paRects[DevNum].bottom = paDeviceModes[DevNum].dmPosition.y + paDeviceModes[DevNum].dmPelsHeight;
             DevNum++;
         }
-        
+
         ZeroMemory(&DisplayDevice, sizeof(DISPLAY_DEVICE));
         DisplayDevice.cb = sizeof(DISPLAY_DEVICE);
         i++;
     }
-    
+
     /* Width, height equal to 0 means that this value must be not changed.
      * Update input parameters if necessary.
      * Note: BitsPerPixel is taken into account later, when new rectangles
@@ -319,14 +379,18 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
                 paRects[i].bottom - paRects[i].top));
     }
 #endif /* Log */
-    
+
     /* Without this, Windows will not ask the miniport for its
      * mode table but uses an internal cache instead.
      */
-    DEVMODE tempDevMode;
-    ZeroMemory (&tempDevMode, sizeof (tempDevMode));
-    tempDevMode.dmSize = sizeof(DEVMODE);
-    EnumDisplaySettings(NULL, 0xffffff, &tempDevMode);
+    for (i = 0; i < NumDevices; i++)
+    {
+        DEVMODE tempDevMode;
+        ZeroMemory (&tempDevMode, sizeof (tempDevMode));
+        tempDevMode.dmSize = sizeof(DEVMODE);
+        EnumDisplaySettings((LPSTR)paDisplayDevices[i].DeviceName, 0xffffff, &tempDevMode);
+        Log(("ResizeDisplayDevice: EnumDisplaySettings last error %d\n", GetLastError ()));
+    }
 
     /* Assign the new rectangles to displays. */
     for (i = 0; i < NumDevices; i++)
@@ -335,12 +399,12 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
         paDeviceModes[i].dmPosition.y = paRects[i].top;
         paDeviceModes[i].dmPelsWidth  = paRects[i].right - paRects[i].left;
         paDeviceModes[i].dmPelsHeight = paRects[i].bottom - paRects[i].top;
-        
+
         /* On Vista one must specify DM_BITSPERPEL.
          * Note that the current mode dmBitsPerPel is already in the DEVMODE structure.
          */
         paDeviceModes[i].dmFields = DM_POSITION | DM_PELSHEIGHT | DM_PELSWIDTH | DM_BITSPERPEL;
-        
+
         if (   i == Id
             && BitsPerPixel != 0)
         {
@@ -356,13 +420,13 @@ static BOOL ResizeDisplayDevice(ULONG Id, DWORD Width, DWORD Height, DWORD BitsP
               paDeviceModes[i].dmPosition.x,
               paDeviceModes[i].dmPosition.y));
 
-        gCtx.pfnChangeDisplaySettingsEx((LPSTR)paDisplayDevices[i].DeviceName,
+        LONG status = gCtx.pfnChangeDisplaySettingsEx((LPSTR)paDisplayDevices[i].DeviceName,
                                         &paDeviceModes[i], NULL, CDS_NORESET | CDS_UPDATEREGISTRY, NULL);
-        Log(("ResizeDisplayDevice: ChangeDisplaySettingsEx position err %d\n", GetLastError ()));
+        Log(("ResizeDisplayDevice: ChangeDisplaySettingsEx position status %d, err %d\n", status, GetLastError ()));
     }
-    
+
     /* A second call to ChangeDisplaySettings updates the monitor. */
-    LONG status = ChangeDisplaySettings(NULL, 0); 
+    LONG status = gCtx.pfnChangeDisplaySettingsEx(NULL, NULL, NULL, 0, NULL);
     Log(("ResizeDisplayDevice: ChangeDisplaySettings update status %d\n", status));
     if (status == DISP_CHANGE_SUCCESSFUL || status == DISP_CHANGE_BADMODE)
     {
@@ -385,7 +449,7 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
     bool fTerminate = false;
     VBoxGuestFilterMaskInfo maskInfo;
     DWORD cbReturned;
-    
+
     maskInfo.u32OrMask = VMMDEV_EVENT_DISPLAY_CHANGE_REQUEST | VMMDEV_EVENT_MOUSE_CAPABILITIES_CHANGED;
     maskInfo.u32NotMask = 0;
     if (DeviceIoControl (gVBoxDriver, VBOXGUEST_IOCTL_CTL_FILTER_MASK, &maskInfo, sizeof (maskInfo), NULL, 0, &cbReturned, NULL))
@@ -465,7 +529,16 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
                         /*
                          * Only try to change video mode if the active display driver is VBox additions.
                          */
+#ifdef VBOXWDDM
+                        VBOXDISPLAY_DRIVER_TYPE enmDriverType = getVBoxDisplayDriverType (pCtx);
+
+                        if (enmDriverType == VBOXDISPLAY_DRIVER_TYPE_WDDM)
+                            Log(("VBoxDisplayThread : Detected WDDM Driver\n"));
+
+                        if (enmDriverType != VBOXDISPLAY_DRIVER_TYPE_UNKNOWN)
+#else
                         if (isVBoxDisplayDriverActive (pCtx))
+#endif
                         {
                             Log(("VBoxDisplayThread : Display driver is active!\n"));
 
@@ -473,11 +546,28 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
                             {
                                 Log(("VBoxDisplayThread : Detected W2K or later.\n"));
 
+#ifdef  VBOXWDDM
+                                if (enmDriverType == VBOXDISPLAY_DRIVER_TYPE_WDDM)
+                                {
+                                    DWORD err = VBoxDispIfResize(&pCtx->pEnv->dispIf,
+                                                        displayChangeRequest.display,
+                                                        displayChangeRequest.xres,
+                                                        displayChangeRequest.yres,
+                                                        displayChangeRequest.bpp);
+                                    if (err == NO_ERROR)
+                                    {
+                                        Log(("VBoxDisplayThread : VBoxDispIfResize succeeded\n"));
+                                        break;
+                                    }
+                                    Log(("VBoxDisplayThread : VBoxDispIfResize failed err(%d)\n", err));
+                                }
+#endif
                                 /* W2K or later. */
                                 if (!ResizeDisplayDevice(displayChangeRequest.display,
                                                          displayChangeRequest.xres,
                                                          displayChangeRequest.yres,
-                                                         displayChangeRequest.bpp))
+                                                         displayChangeRequest.bpp
+                                                         ))
                                 {
                                     break;
                                 }
@@ -488,7 +578,7 @@ unsigned __stdcall VBoxDisplayThread  (void *pInstance)
 
                                 /* Single monitor NT. */
                                 DEVMODE devMode;
-                                memset (&devMode, 0, sizeof (devMode));
+                                RT_ZERO(devMode);
                                 devMode.dmSize = sizeof(DEVMODE);
 
                                 /* get the current screen setup */
