@@ -1,10 +1,10 @@
-/** $Id: VBoxControl.cpp 18509 2009-03-29 02:48:19Z vboxsync $ */
+/* $Id: VBoxControl.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
- * VBoxControl - Guest Additions Command Line Management Interface
+ * VBoxControl - Guest Additions Command Line Management Interface.
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2008-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,34 +13,28 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
-
-
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
+#include <iprt/alloca.h>
+#include <iprt/cpp/autores.h>
+#include <iprt/buildconfig.h>
+#include <iprt/initterm.h>
 #include <iprt/mem.h>
+#include <iprt/path.h>
 #include <iprt/string.h>
 #include <iprt/stream.h>
-#include <iprt/path.h>
-#include <iprt/initterm.h>
-#include <iprt/autores.h>
 #include <VBox/log.h>
-#include <VBox/VBoxGuest.h>
 #include <VBox/version.h>
+#include <VBox/VBoxGuestLib.h>
 #ifdef RT_OS_WINDOWS
-# include <windows.h>
-# include <malloc.h>  /* for alloca */
+# include <Windows.h>
 #endif
 #ifdef VBOX_WITH_GUEST_PROPS
 # include <VBox/HostServices/GuestPropertySvc.h>
 #endif
-#include "VBoxControl.h"
 
 /*******************************************************************************
 *   Global Variables                                                           *
@@ -62,7 +56,11 @@ int g_cVerbosity = 0;
 /** Helper function */
 static void doUsage(char const *line, char const *name = "", char const *command = "")
 {
-    RTPrintf("%s %-*s%s", name, 32 - strlen(name), command, line);
+    /* Allow for up to 15 characters command name length (VBoxControl.exe) with
+     * perfect column alignment. Beyond that there's at least one space between
+     * the command if there are command line parameters. */
+    RTPrintf("%s %-*s%s%s\n", name, strlen(line) ? 35 - strlen(name) : 1,
+                              command, strlen(line) ? " " : "", line);
 }
 
 /** Enumerate the different parts of the usage we might want to print out */
@@ -85,33 +83,35 @@ enum g_eUsage
 static void usage(g_eUsage eWhich = USAGE_ALL)
 {
     RTPrintf("Usage:\n\n");
-    RTPrintf("%s [-v|-version]        print version number and exit\n", g_pszProgName);
-    RTPrintf("%s -nologo ...          suppress the logo\n\n", g_pszProgName);
+    doUsage("print version number and exit", g_pszProgName, "[-v|-version]");
+    doUsage("suppress the logo", g_pszProgName, "-nologo ...");
+    RTPrintf("\n");
 
 /* Exclude the Windows bits from the test version.  Anyone who needs to test
  * them can fix this. */
 #if defined(RT_OS_WINDOWS) && !defined(VBOX_CONTROL_TEST)
     if ((GET_VIDEO_ACCEL == eWhich) || (USAGE_ALL == eWhich))
-        doUsage("\n", g_pszProgName, "getvideoacceleration");
+        doUsage("", g_pszProgName, "getvideoacceleration");
     if ((SET_VIDEO_ACCEL == eWhich) || (USAGE_ALL == eWhich))
-        doUsage("<on|off>\n", g_pszProgName, "setvideoacceleration");
+        doUsage("<on|off>", g_pszProgName, "setvideoacceleration");
     if ((LIST_CUST_MODES == eWhich) || (USAGE_ALL == eWhich))
-        doUsage("\n", g_pszProgName, "listcustommodes");
+        doUsage("", g_pszProgName, "listcustommodes");
     if ((ADD_CUST_MODE == eWhich) || (USAGE_ALL == eWhich))
-        doUsage("<width> <height> <bpp>\n", g_pszProgName, "addcustommode");
+        doUsage("<width> <height> <bpp>", g_pszProgName, "addcustommode");
     if ((REMOVE_CUST_MODE == eWhich) || (USAGE_ALL == eWhich))
-        doUsage("<width> <height> <bpp>\n", g_pszProgName, "removecustommode");
+        doUsage("<width> <height> <bpp>", g_pszProgName, "removecustommode");
     if ((SET_VIDEO_MODE == eWhich) || (USAGE_ALL == eWhich))
-        doUsage("<width> <height> <bpp> <screen>\n", g_pszProgName, "setvideomode");
+        doUsage("<width> <height> <bpp> <screen>", g_pszProgName, "setvideomode");
 #endif
 #ifdef VBOX_WITH_GUEST_PROPS
     if ((GUEST_PROP == eWhich) || (USAGE_ALL == eWhich))
     {
-        doUsage("get <property> [-verbose]\n", g_pszProgName, "guestproperty");
-        doUsage("set <property> [<value> [-flags <flags>]]\n", g_pszProgName, "guestproperty");
-        doUsage("enumerate [-patterns <patterns>]\n", g_pszProgName, "guestproperty");
-        doUsage("wait <patterns> [-timestamp <last timestamp>]\n", g_pszProgName, "guestproperty");
-        doUsage("[-timeout <timeout in ms>\n");
+        doUsage("get <property> [-verbose]", g_pszProgName, "guestproperty");
+        doUsage("set <property> [<value> [-flags <flags>]]", g_pszProgName, "guestproperty");
+        doUsage("enumerate [-patterns <patterns>]", g_pszProgName, "guestproperty");
+        doUsage("wait <patterns>", g_pszProgName, "guestproperty");
+        doUsage("[-timestamp <last timestamp>]");
+        doUsage("[-timeout <timeout in ms>");
     }
 #endif
 }
@@ -652,7 +652,7 @@ void getCustomModes(HKEY hkeyVideo)
     int curMode = 0;
 
     /* null out the table */
-    memset(customModes, 0, sizeof(customModes));
+    RT_ZERO(customModes);
 
     do
     {
@@ -859,7 +859,7 @@ int handleRemoveCustomMode(int argc, char *argv[])
                 && (customModes[i].bpp  == bpp))
             {
                 RTPrintf("found mode at index %d\n", i);
-                memset(&customModes[i], 0, sizeof(customModes[i]));
+                RT_ZERO(customModes[i]);
                 break;
             }
         }
@@ -1313,7 +1313,7 @@ int main(int argc, char **argv)
            )
             {
                 /* Print version number, and do nothing else. */
-                RTPrintf("%sr%d\n", VBOX_VERSION_STRING, VBoxSVNRev ());
+                RTPrintf("%sr%u\n", VBOX_VERSION_STRING, RTBldCfgRevision());
                 onlyinfo = true;
                 showlogo = false;
                 done = true;
@@ -1341,9 +1341,9 @@ int main(int argc, char **argv)
 
     g_pszProgName = RTPathFilename(argv[0]);
     if (showlogo)
-        RTPrintf("VirtualBox Guest Additions Command Line Management Interface Version "
+        RTPrintf(VBOX_PRODUCT " Guest Additions Command Line Management Interface Version "
                  VBOX_VERSION_STRING "\n"
-                 "(C) 2008 Sun Microsystems, Inc.\n"
+                 "(C) 2008-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
                  "All rights reserved.\n\n");
     if (dohelp)
         usage();
@@ -1434,3 +1434,4 @@ int main(int argc, char **argv)
 
     return rc;
 }
+
