@@ -1,4 +1,4 @@
-/* $Id: VBoxGuest-os2.cpp 14297 2008-11-18 12:40:43Z vboxsync $ */
+/* $Id: VBoxGuest-os2.cpp 28854 2010-04-27 19:41:12Z vboxsync $ */
 /** @file
  * VBoxGuest - OS/2 specifics.
  */
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -54,8 +50,6 @@
 #include <os2ddk/bsekee.h>
 
 #include "VBoxGuestInternal.h"
-#include <VBox/VBoxGuest.h>
-#include <VBox/VBoxDev.h>               /* VMMDEV_RAM_SIZE */
 #include <VBox/version.h>
 #include <iprt/initterm.h>
 #include <iprt/string.h>
@@ -87,7 +81,7 @@ static PVBOXGUESTSESSION    g_apSessionHashTab[19];
 /** Calculates the index into g_apSessionHashTab.*/
 #define SESSION_HASH(sfn) ((sfn) % RT_ELEMENTS(g_apSessionHashTab))
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 /* Defined in VBoxGuestA-os2.asm */
 extern uint32_t             g_PhysMMIOBase;
 extern uint32_t             g_cbMMIO; /* 0 currently not set. */
@@ -108,7 +102,7 @@ extern char                 g_szLog[];
 extern char                 g_szInitText[];
 extern uint16_t             g_cchInitText;
 extern uint16_t             g_cchInitTextMax;
-__END_DECLS
+RT_C_DECLS_END
 
 
 /*******************************************************************************
@@ -158,10 +152,12 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
                 rc = VBoxGuestInitDevExt(&g_DevExt, g_IOPortBase,
                                          RTR0MemObjAddress(g_MemMapMMIO),
                                          RTR0MemObjSize(g_MemMapMMIO),
-                                         vboxGuestOS2DetectVersion());
+                                         vboxGuestOS2DetectVersion(),
+                                         0);
             else
                 rc = VBoxGuestInitDevExt(&g_DevExt, g_IOPortBase, NULL, 0,
-                                         vboxGuestOS2DetectVersion());
+                                         vboxGuestOS2DetectVersion(),
+                                         0);
             if (RT_SUCCESS(rc))
             {
                 /*
@@ -192,7 +188,7 @@ DECLASM(int) VBoxGuestOS2Init(const char *pszArgs)
                             strcpy(&g_szInitText[0],
                                    "\r\n"
                                    "VirtualBox Guest Additions Driver for OS/2 version " VBOX_VERSION_STRING "\r\n"
-                                   "Copyright (C) 2008 Sun Microsystems, Inc.\r\n");
+                                   "Copyright (C) 2008-2010 Oracle Corporation\r\n");
                             g_cchInitText = strlen(&g_szInitText[0]);
                         }
                         Log(("VBoxGuestOS2Init: Successfully loaded\n%s", g_szInitText));
@@ -253,11 +249,11 @@ static int vboxGuestOS2MapMemory(void)
      * a qualified guess using the VMMDEV_RAM_SIZE.
      */
     size_t cb = RT_ALIGN_Z(VMMDEV_RAM_SIZE, PAGE_SIZE);
-    int rc = RTR0MemObjEnterPhys(&g_MemObjMMIO, PhysMMIOBase, cb);
+    int rc = RTR0MemObjEnterPhys(&g_MemObjMMIO, PhysMMIOBase, cb, RTMEM_CACHE_POLICY_DONT_CARE);
     if (RT_FAILURE(rc))
     {
         cb = _4K;
-        rc = RTR0MemObjEnterPhys(&g_MemObjMMIO, PhysMMIOBase, cb);
+        rc = RTR0MemObjEnterPhys(&g_MemObjMMIO, PhysMMIOBase, cb, RTMEM_CACHE_POLICY_DONT_CARE);
     }
     if (RT_FAILURE(rc))
     {
@@ -299,7 +295,7 @@ static int vboxGuestOS2MapMemory(void)
             rc = RTR0MemObjFree(g_MemObjMMIO, true); AssertRC(rc);
             g_MemObjMMIO = g_MemMapMMIO = NIL_RTR0MEMOBJ;
 
-            rc = RTR0MemObjEnterPhys(&g_MemObjMMIO, PhysMMIOBase, cb);
+            rc = RTR0MemObjEnterPhys(&g_MemObjMMIO, PhysMMIOBase, cb, RTMEM_CACHE_POLICY_DONT_CARE);
             if (RT_SUCCESS(rc))
             {
                 rc = RTR0MemObjMapKernel(&g_MemMapMMIO, g_MemObjMMIO, (void *)-1, 0,
@@ -606,9 +602,8 @@ DECLASM(int) VBoxGuestOS2IOCtl(uint16_t sfn, uint8_t iCat, uint8_t iFunction, vo
         /*
          * Process the IOCtl.
          */
-        size_t cbDataReturned = 0;
-        rc = VBoxGuestCommonIOCtl(iFunction, &g_DevExt, pSession,
-                                  pvParm, *pcbParm, &cbDataReturned);
+        size_t cbDataReturned;
+        rc = VBoxGuestCommonIOCtl(iFunction, &g_DevExt, pSession, pvParm, *pcbParm, &cbDataReturned);
 
         /*
          * Unlock the buffers.
@@ -651,6 +646,13 @@ DECLASM(bool) VBoxGuestOS2ISR(void)
 }
 
 
+void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt)
+{
+    /* No polling on OS/2 */
+    NOREF(pDevExt);
+}
+
+
 #ifdef DEBUG_READ /** @todo figure out this one once and for all... */
 
 /**
