@@ -1,10 +1,10 @@
-/* $Id: VBoxGuestInternal.h $ */
+/* $Id: VBoxGuestInternal.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxGuest - Guest Additions Driver.
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VBoxGuestInternal_h
@@ -25,10 +21,14 @@
 #include <iprt/types.h>
 #include <iprt/semaphore.h>
 #include <iprt/spinlock.h>
+#include <VBox/VMMDev.h>
 #include <VBox/VBoxGuest.h>
 #include <VBox/VBoxGuestLib.h>
 
 
+/** Pointer to the VBoxGuest per session data. */
+typedef struct VBOXGUESTSESSION *PVBOXGUESTSESSION;
+
 /** Pointer to a wait-for-event entry. */
 typedef struct VBOXGUESTWAIT *PVBOXGUESTWAIT;
 
@@ -50,6 +50,8 @@ typedef struct VBOXGUESTWAIT
     uint32_t                    fReqEvents;
     /** The events we received. */
     uint32_t volatile           fResEvents;
+    /** The session that's waiting. */
+    PVBOXGUESTSESSION           pSession;
 #ifdef VBOX_WITH_HGCM
     /** The HGCM request we're waiting for to complete. */
     VMMDevHGCMRequestHeader volatile *pHGCMReq;
@@ -70,6 +72,33 @@ typedef struct VBOXGUESTWAITLIST
 typedef VBOXGUESTWAITLIST *PVBOXGUESTWAITLIST;
 
 
+/**
+ * VBox guest memory balloon.
+ */
+typedef struct VBOXGUESTMEMBALLOON
+{
+    /** Mutext protecting the members below from concurrent access.. */
+    RTSEMFASTMUTEX              hMtx;
+    /** The current number of chunks in the balloon. */
+    uint32_t                    cChunks;
+    /** The maximum number of chunks in the balloon (typically the amount of guest
+     * memory / chunksize). */
+    uint32_t                    cMaxChunks;
+    /** This is true if we are using RTR0MemObjAllocPhysNC() / RTR0MemObjGetPagePhysAddr()
+     * and false otherwise. */
+    bool                        fUseKernelAPI;
+    /** The current owner of the balloon.
+     * This is automatically assigned to the first session using the ballooning
+     * API and first released when the session closes. */
+    PVBOXGUESTSESSION           pOwner;
+    /** The pointer to the array of memory objects holding the chunks of the
+     *  balloon.  This array is cMaxChunks in size when present. */
+    PRTR0MEMOBJ                 paMemObj;
+} VBOXGUESTMEMBALLOON;
+/** Pointer to a memory balloon. */
+typedef VBOXGUESTMEMBALLOON *PVBOXGUESTMEMBALLOON;
+
+
 /**
  * VBox guest device (data) extension.
  */
@@ -79,11 +108,18 @@ typedef struct VBOXGUESTDEVEXT
     RTIOPORT                    IOPortBase;
     /** Pointer to the mapping of the VMMDev adapter memory. */
     VMMDevMemory volatile      *pVMMDevMemory;
+    /** Events we won't permit anyone to filter out. */
+    uint32_t                    fFixedEvents;
+    /** The memory object reserving space for the guest mappings. */
+    RTR0MEMOBJ                  hGuestMappings;
 
+    /** Spinlock protecting the signaling and resetting of the wait-for-event
+     * semaphores as well as the event acking in the ISR. */
+    RTSPINLOCK                  EventSpinlock;
     /** Preallocated VMMDevEvents for the IRQ handler. */
     VMMDevEvents               *pIrqAckEvents;
-    /** Spinlock protecting the signaling and resetting of the wait-for-event semaphores. */
-    RTSPINLOCK                  WaitSpinlock;
+    /** The physical address of pIrqAckEvents. */
+    RTCCPHYS                    PhysIrqAckEvents;
     /** Wait-for-event list for threads waiting for multiple events. */
     VBOXGUESTWAITLIST           WaitList;
 #ifdef VBOX_WITH_HGCM
@@ -106,14 +142,15 @@ typedef struct VBOXGUESTDEVEXT
     /** The current clipboard client ID, 0 if no client.
      * For implementing the VBOXGUEST_IOCTL_CLIPBOARD_CONNECT interface. */
     uint32_t                    u32ClipboardClientId;
+
+    /** Memory balloon information for RTR0MemObjAllocPhysNC(). */
+    VBOXGUESTMEMBALLOON         MemBalloon;
+
 } VBOXGUESTDEVEXT;
 /** Pointer to the VBoxGuest driver data. */
 typedef VBOXGUESTDEVEXT *PVBOXGUESTDEVEXT;
 
 
-/** Pointer to the VBoxGuest per session data. */
-typedef struct VBOXGUESTSESSION *PVBOXGUESTSESSION;
-
 /**
  * The VBoxGuest per session data.
  *
@@ -149,6 +186,7 @@ typedef struct VBOXGUESTSESSION
     /** The last consumed VMMDEV_EVENT_MOUSE_POSITION_CHANGED sequence number.
      * Used to implement polling.  */
     uint32_t volatile           u32MousePosChangedSeq;
+
 } VBOXGUESTSESSION;
 
 RT_C_DECLS_BEGIN
@@ -166,9 +204,20 @@ int  VBoxGuestCommonIOCtlFast(unsigned iFunction, PVBOXGUESTDEVEXT pDevExt, PVBO
 int  VBoxGuestCommonIOCtl(unsigned iFunction, PVBOXGUESTDEVEXT pDevExt, PVBOXGUESTSESSION pSession,
                           void *pvData, size_t cbData, size_t *pcbDataReturned);
 
+#if defined(RT_OS_SOLARIS) \
+ || defined(RT_OS_FREEBSD) \
+ || defined(RT_OS_LINUX)
+DECLVBGL(void *) VBoxGuestNativeServiceOpen(uint32_t *pu32Version);
+DECLVBGL(void)   VBoxGuestNativeServiceClose(void *pvOpaque);
+DECLVBGL(int)    VBoxGuestNativeServiceCall(void *pvOpaque, unsigned int iCmd, void *pvData, size_t cbSize, size_t *pcbReturn);
+#endif
+
 /**
  * ISR callback for notifying threads polling for mouse events.
  *
+ * This is called at the end of the ISR, after leaving the event spinlock, if
+ * VMMDEV_EVENT_MOUSE_POSITION_CHANGED was raised by the host.
+ *
  * @param   pDevExt     The device extension.
  */
 void VBoxGuestNativeISRMousePollEvent(PVBOXGUESTDEVEXT pDevExt);
