@@ -1,11 +1,10 @@
+/* $Revision: 29360 $ */
 /** @file
- *
- * VBoxGuestLib - A support library for VirtualBox guest additions:
- * Generic VMMDev request management
+ * VBoxGuestLibR0 - Generic VMMDev request management.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,16 +14,21 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
  */
 
-#include <VBox/VBoxGuestLib.h>
 #include "VBGLInternal.h"
 #include <iprt/asm.h>
-#include <iprt/string.h>
+#include <iprt/asm-amd64-x86.h>
 #include <iprt/assert.h>
+#include <iprt/string.h>
 
 DECLVBGL(int) VbglGRVerify (const VMMDevRequestHeader *pReq, size_t cbReq)
 {
@@ -64,7 +68,7 @@ DECLVBGL(int) VbglGRVerify (const VMMDevRequestHeader *pReq, size_t cbReq)
     }
 
     /* This can be a variable size request. Check the request type and limit the size
-     * to VMMDEV_MAX_VMMDEVREQ_SIZE, which is max size supported by the host. 
+     * to VMMDEV_MAX_VMMDEVREQ_SIZE, which is max size supported by the host.
      */
     if (   pReq->requestType == VMMDevReq_LogString
         || pReq->requestType == VMMDevReq_VideoSetVisibleRegion
@@ -75,7 +79,8 @@ DECLVBGL(int) VbglGRVerify (const VMMDevRequestHeader *pReq, size_t cbReq)
 #else
         || pReq->requestType == VMMDevReq_HGCMCall
 #endif /* VBOX_WITH_64_BITS_GUESTS */
-        || pReq->requestType == VMMDevReq_ChangeMemBalloon)
+        || pReq->requestType == VMMDevReq_ChangeMemBalloon
+        || pReq->requestType == VMMDevReq_RegisterSharedModule)
     {
         if (cbReq > VMMDEV_MAX_VMMDEVREQ_SIZE)
         {
@@ -95,7 +100,7 @@ DECLVBGL(int) VbglGRVerify (const VMMDevRequestHeader *pReq, size_t cbReq)
 DECLVBGL(int) VbglGRAlloc (VMMDevRequestHeader **ppReq, uint32_t cbSize, VMMDevRequestType reqType)
 {
     VMMDevRequestHeader *pReq;
-    int rc = VbglEnter ();
+    int rc = vbglR0Enter ();
 
     if (RT_FAILURE(rc))
         return rc;
@@ -132,7 +137,7 @@ DECLVBGL(int) VbglGRAlloc (VMMDevRequestHeader **ppReq, uint32_t cbSize, VMMDevR
 DECLVBGL(int) VbglGRPerform (VMMDevRequestHeader *pReq)
 {
     RTCCPHYS physaddr;
-    int rc = VbglEnter ();
+    int rc = vbglR0Enter ();
 
     if (RT_FAILURE(rc))
         return rc;
@@ -148,7 +153,7 @@ DECLVBGL(int) VbglGRPerform (VMMDevRequestHeader *pReq)
     }
     else
     {
-        ASMOutU32(g_vbgldata.portVMMDev + PORT_VMMDEV_REQUEST_OFFSET, (uint32_t)physaddr);
+        ASMOutU32(g_vbgldata.portVMMDev + VMMDEV_PORT_OFF_REQUEST, (uint32_t)physaddr);
         /* Make the compiler aware that the host has changed memory. */
         ASMCompilerBarrier();
         rc = pReq->rc;
@@ -158,10 +163,11 @@ DECLVBGL(int) VbglGRPerform (VMMDevRequestHeader *pReq)
 
 DECLVBGL(void) VbglGRFree (VMMDevRequestHeader *pReq)
 {
-    int rc = VbglEnter ();
+    int rc = vbglR0Enter ();
 
     if (RT_FAILURE(rc))
         return;
 
     VbglPhysHeapFree (pReq);
 }
+
