@@ -1,10 +1,10 @@
-/* $Id: VBoxGuestR3Lib.cpp $ */
+/* $Id: VBoxGuestR3Lib.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxGuestR3Lib - Ring-3 Support Library for VirtualBox guest additions, Core.
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,9 +14,14 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
  */
 
 
@@ -31,20 +36,24 @@
 # define INCL_ERRORS
 # include <os2.h>
 
-#elif defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
+#elif defined(RT_OS_FREEBSD) \
+   || defined(RT_OS_LINUX) \
+   || defined(RT_OS_SOLARIS)
 # include <sys/types.h>
 # include <sys/stat.h>
+# if defined(RT_OS_LINUX) /** @todo check this on solaris+freebsd as well. */
+#  include <sys/ioctl.h>
+# endif
 # include <errno.h>
 # include <unistd.h>
 #endif
 
-#include <iprt/time.h>
-#include <iprt/asm.h>
-#include <iprt/string.h>
-#include <iprt/file.h>
 #include <iprt/assert.h>
+#include <iprt/asm.h>
+#include <iprt/file.h>
+#include <iprt/time.h>
+#include <iprt/string.h>
 #include <iprt/thread.h>
-#include <VBox/VBoxGuest.h>
 #include <VBox/log.h>
 #include "VBGLR3Internal.h"
 
@@ -85,9 +94,7 @@ static uint32_t volatile g_cInits = 0;
 static int vbglR3Init(const char *pszDeviceName)
 {
     uint32_t cInits = ASMAtomicIncU32(&g_cInits);
-#ifndef VBOX_VBGLR3_XFREE86
     Assert(cInits > 0);
-#endif
     if (cInits > 1)
     {
         /*
@@ -182,42 +189,7 @@ static int vbglR3Init(const char *pszDeviceName)
     }
     g_File = hf;
 
-#elif defined(RT_OS_FREEBSD)
-    /*
-     * Try open the BSD device. The device cloning makes this a bit of work.
-     */
-# if defined(VBOX_VBGLR3_XFREE86)
-    int File = 0;
-# else
-    RTFILE File = 0;
-# endif
-    int rc;
-    char szDevice[RT_MAX(sizeof(VBOXGUEST_DEVICE_NAME), sizeof(VBOXGUEST_USER_DEVICE_NAME)) + 16];
-    for (unsigned iUnit = 0; iUnit < 1024; iUnit++)
-    {
-        RTStrPrintf(szDevice, sizeof(szDevice), pszDeviceName "%d", iUnit);
-# if defined(VBOX_VBGLR3_XFREE86)
-        File = xf86open(szDevice, XF86_O_RDWR);
-        if (File >= 0)
-            break;
-# else
-        rc = RTFileOpen(&File, szDevice, RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
-        if (RT_SUCCESS(rc))
-            break;
-# endif
-    }
-
-# if defined(VBOX_VBGLR3_XFREE86)
-    if (File == -1)
-        return VERR_OPEN_FAILED;
-# else
-    if (RT_FAILURE(rc))
-        return rc;
-# endif
-
-    g_File = File;
-
-#elif defined(VBOX_VBGLR3_XFREE86) && !defined(RT_OS_FREEBSD)
+#elif defined(VBOX_VBGLR3_XFREE86)
     int File = xf86open(pszDeviceName, XF86_O_RDWR);
     if (File == -1)
         return VERR_OPEN_FAILED;
@@ -225,7 +197,7 @@ static int vbglR3Init(const char *pszDeviceName)
 
 #else
 
-    /* The default implemenation. (linux, solaris) */
+    /* The default implemenation. (linux, solaris, freebsd) */
     RTFILE File;
     int rc = RTFileOpen(&File, pszDeviceName, RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_NONE);
     if (RT_FAILURE(rc))
@@ -234,6 +206,7 @@ static int vbglR3Init(const char *pszDeviceName)
 
 #endif
 
+#ifndef VBOX_VBGLR3_XFREE86
     /*
      * Create release logger
      */
@@ -245,6 +218,7 @@ static int vbglR3Init(const char *pszDeviceName)
     /* This may legitimately fail if we are using the mini-runtime. */
     if (RT_SUCCESS(rc2))
         RTLogRelSetDefaultInstance(pReleaseLogger);
+#endif
 
     return VINF_SUCCESS;
 }
@@ -365,7 +339,11 @@ int vbglR3DoIOCtl(unsigned iFunction, void *pvData, size_t cbData)
     Hdr.u32Padding = 0;
 # endif
 
-/** @todo test status code passing! */
+/** @todo test status code passing! Check that the kernel doesn't do any
+ *        error checks using specific errno values, and just pass an VBox
+ *        error instead of an errno.h one. Alternatively, extend/redefine the
+ *        header with an error code return field (much better alternative
+ *        actually). */
     int rc = ioctl((int)g_File, iFunction, &Hdr);
     if (rc == -1)
     {
@@ -374,6 +352,27 @@ int vbglR3DoIOCtl(unsigned iFunction, void *pvData, size_t cbData)
     }
     return VINF_SUCCESS;
 
+#elif defined(RT_OS_LINUX)
+# ifdef VBOX_VBGLR3_XFREE86
+    int rc = xf86ioctl((int)g_File, iFunction, pvData);
+# else
+    int rc = ioctl((int)g_File, iFunction, pvData);
+# endif
+    if (RT_LIKELY(rc == 0))
+        return VINF_SUCCESS;
+
+    /* Positive values are negated VBox error status codes. */
+    if (rc > 0)
+        rc = -rc;
+    else
+# ifdef VBOX_VBGLR3_XFREE86
+        rc = VERR_FILE_IO_ERROR;
+#  else
+        rc = RTErrConvertFromErrno(errno);
+# endif
+    NOREF(cbData);
+    return rc;
+
 #elif defined(VBOX_VBGLR3_XFREE86)
     /* PORTME - This is preferred over the RTFileIOCtl variant below, just be careful with the (int). */
 /** @todo test status code passing! */
