@@ -1,11 +1,10 @@
-/* $Id: VBoxGuestR3LibGuestProp.cpp $ */
+/* $Id: VBoxGuestR3LibGuestProp.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
- * VBoxGuestR3Lib - Ring-3 Support Library for VirtualBox guest additions,
- * guest properties.
+ * VBoxGuestR3Lib - Ring-3 Support Library for VirtualBox guest additions, guest properties.
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,9 +14,14 @@
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
+ * The contents of this file may alternatively be used under the terms
+ * of the Common Development and Distribution License Version 1.0
+ * (CDDL) only, as it comes in the "COPYING.CDDL" file of the
+ * VirtualBox OSE distribution, in which case the provisions of the
+ * CDDL are applicable instead of those of the GPL.
+ *
+ * You may elect to license modified versions of this file under the
+ * terms and conditions of either the GPL or the CDDL or both.
  */
 
 
@@ -27,13 +31,23 @@
 #include <iprt/string.h>
 #include <iprt/mem.h>
 #include <iprt/assert.h>
-#include <iprt/autores.h>
+#include <iprt/cpp/autores.h>
 #include <iprt/stdarg.h>
 #include <VBox/log.h>
 #include <VBox/HostServices/GuestPropertySvc.h>
 
 #include "VBGLR3Internal.h"
 
+#ifdef VBOX_VBGLR3_XFREE86
+/* Rather than try to resolve all the header file conflicts, I will just
+   prototype what we need here. */
+extern "C" char* xf86strcpy(char*,const char*);
+# undef strcpy
+# define strcpy xf86strcpy
+extern "C" void* xf86memchr(const void*,int,xf86size_t);
+# undef memchr
+# define memchr xf86memchr
+#endif /* VBOX_VBGLR3_XFREE86 */
 
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
@@ -41,6 +55,8 @@
 /**
  * Structure containing information needed to enumerate through guest
  * properties.
+ *
+ * @remarks typedef in VBoxGuestLib.h.
  */
 struct VBGLR3GUESTPROPENUM
 {
@@ -65,9 +81,9 @@ using namespace guestProp;
 VBGLR3DECL(int) VbglR3GuestPropConnect(uint32_t *pu32ClientId)
 {
     VBoxGuestHGCMConnectInfo Info;
-    Info.result = (uint32_t)VERR_WRONG_ORDER; /** @todo drop the cast when the result type has been fixed! */
+    Info.result = VERR_WRONG_ORDER;
     Info.Loc.type = VMMDevHGCMLoc_LocalHost_Existing;
-    memset(&Info.Loc.u, 0, sizeof(Info.Loc.u));
+    RT_ZERO(Info.Loc.u);
     strcpy(Info.Loc.u.host.achName, "VBoxGuestPropSvc");
     Info.u32ClientID = UINT32_MAX;  /* try make valgrid shut up. */
 
@@ -91,7 +107,7 @@ VBGLR3DECL(int) VbglR3GuestPropConnect(uint32_t *pu32ClientId)
 VBGLR3DECL(int) VbglR3GuestPropDisconnect(uint32_t u32ClientId)
 {
     VBoxGuestHGCMDisconnectInfo Info;
-    Info.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+    Info.result = VERR_WRONG_ORDER;
     Info.u32ClientID = u32ClientId;
 
     int rc = vbglR3DoIOCtl(VBOXGUEST_IOCTL_HGCM_DISCONNECT, &Info, sizeof(Info));
@@ -119,7 +135,7 @@ VBGLR3DECL(int) VbglR3GuestPropWrite(uint32_t u32ClientId, const char *pszName,
     {
         SetProperty Msg;
 
-        Msg.hdr.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+        Msg.hdr.result = VERR_WRONG_ORDER;
         Msg.hdr.u32ClientID = u32ClientId;
         Msg.hdr.u32Function = SET_PROP_VALUE;
         Msg.hdr.cParms = 3;
@@ -134,7 +150,7 @@ VBGLR3DECL(int) VbglR3GuestPropWrite(uint32_t u32ClientId, const char *pszName,
     {
         DelProperty Msg;
 
-        Msg.hdr.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+        Msg.hdr.result = VERR_WRONG_ORDER;
         Msg.hdr.u32ClientID = u32ClientId;
         Msg.hdr.u32Function = DEL_PROP;
         Msg.hdr.cParms = 1;
@@ -168,7 +184,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValue(uint32_t u32ClientId, const char *pszN
     {
         SetPropertyValue Msg;
 
-        Msg.hdr.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+        Msg.hdr.result = VERR_WRONG_ORDER;
         Msg.hdr.u32ClientID = u32ClientId;
         Msg.hdr.u32Function = SET_PROP_VALUE;
         Msg.hdr.cParms = 2;
@@ -182,7 +198,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValue(uint32_t u32ClientId, const char *pszN
     {
         DelProperty Msg;
 
-        Msg.hdr.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+        Msg.hdr.result = VERR_WRONG_ORDER;
         Msg.hdr.u32ClientID = u32ClientId;
         Msg.hdr.u32Function = DEL_PROP;
         Msg.hdr.cParms = 1;
@@ -194,7 +210,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValue(uint32_t u32ClientId, const char *pszN
     return rc;
 }
 
-
+#ifndef VBOX_VBGLR3_XFREE86
 /**
  * Write a property value where the value is formatted in RTStrPrintfV fashion.
  *
@@ -212,7 +228,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValueV(uint32_t u32ClientId, const char *psz
      */
     int rc = VERR_NO_STR_MEMORY;
     char *pszValue;
-    if (RTStrAPrintfV(&pszValue, pszValueFormat, va) < 0)
+    if (RTStrAPrintfV(&pszValue, pszValueFormat, va) >= 0)
     {
         rc = VbglR3GuestPropWriteValue(u32ClientId, pszName, pszValue);
         RTStrFree(pszValue);
@@ -239,7 +255,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValueF(uint32_t u32ClientId, const char *psz
     va_end(va);
     return rc;
 }
-
+#endif /* VBOX_VBGLR3_XFREE86 */
 
 /**
  * Retrieve a property.
@@ -258,7 +274,7 @@ VBGLR3DECL(int) VbglR3GuestPropWriteValueF(uint32_t u32ClientId, const char *psz
  *                          The returned data is only valid for it's lifetime.
  *                          @a ppszValue will point to the start of this buffer.
  * @param   cbBuf           The size of @a pcBuf
- * @param   pszValue        Where to store the pointer to the value retrieved.
+ * @param   ppszValue       Where to store the pointer to the value retrieved.
  *                          Optional.
  * @param   pu64Timestamp   Where to store the timestamp.  Optional.
  * @param   pszFlags        Where to store the pointer to the flags.  Optional.
@@ -276,7 +292,7 @@ VBGLR3DECL(int) VbglR3GuestPropRead(uint32_t u32ClientId, const char *pszName,
      */
     GetProperty Msg;
 
-    Msg.hdr.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+    Msg.hdr.result = VERR_WRONG_ORDER;
     Msg.hdr.u32ClientID = u32ClientId;
     Msg.hdr.u32Function = GET_PROP;
     Msg.hdr.cParms = 4;
@@ -336,7 +352,7 @@ VBGLR3DECL(int) VbglR3GuestPropRead(uint32_t u32ClientId, const char *pszName,
     return VINF_SUCCESS;
 }
 
-
+#ifndef VBOX_VBGLR3_XFREE86
 /**
  * Retrieve a property value, allocating space for it.
  *
@@ -416,7 +432,7 @@ VBGLR3DECL(void) VbglR3GuestPropReadValueFree(char *pszValue)
 {
     RTMemFree(pszValue);
 }
-
+#endif /* VBOX_VBGLR3_XFREE86 */
 
 /**
  * Retrieve a property value, using a user-provided buffer to store it.
@@ -441,25 +457,17 @@ VBGLR3DECL(int) VbglR3GuestPropReadValue(uint32_t u32ClientId, const char *pszNa
                                          char *pszValue, uint32_t cchValue,
                                          uint32_t *pcchValueActual)
 {
-    char *pszBuf = NULL;
-    int rc = VbglR3GuestPropReadValueAlloc(u32ClientId, pszName, &pszBuf);
-    if (RT_SUCCESS(rc))
-    {
-        size_t cchValueActual = strlen(pszBuf) + 1;
-        if (cchValueActual <= cchValue)
-            memcpy(pszValue, pszBuf, cchValueActual);
-        else
-        {
-            if (pcchValueActual != NULL)
-                *pcchValueActual = (uint32_t)cchValueActual;
-            rc = VERR_BUFFER_OVERFLOW;
-        }
-    }
-    VbglR3GuestPropReadValueFree(pszBuf);
+    void *pvBuf = pszValue;
+    uint32_t cchValueActual;
+    int rc = VbglR3GuestPropRead(u32ClientId, pszName, pvBuf, cchValue,
+                                 &pszValue, NULL, NULL, &cchValueActual);
+    if (pcchValueActual != NULL)
+        *pcchValueActual = cchValueActual;
     return rc;
 }
 
 
+#ifndef VBOX_VBGLR3_XFREE86
 /**
  * Raw API for enumerating guest properties which match a given pattern.
  *
@@ -489,7 +497,7 @@ VBGLR3DECL(int) VbglR3GuestPropEnumRaw(uint32_t u32ClientId,
 {
     EnumProperties Msg;
 
-    Msg.hdr.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+    Msg.hdr.result = VERR_WRONG_ORDER;
     Msg.hdr.u32ClientID = u32ClientId;
     Msg.hdr.u32Function = ENUM_PROPS;
     Msg.hdr.cParms = 3;
@@ -809,7 +817,7 @@ VBGLR3DECL(int) VbglR3GuestPropDelSet(uint32_t u32ClientId,
  * @param   cbBuf           The size of @a pvBuf
  * @param   u64Timestamp    The timestamp of the last event seen.  Pass zero
  *                          to wait for the next event.
- * @param   u32Timeout      Timeout in milliseconds.  Use RT_INDEFINITE_WAIT
+ * @param   cMillies        Timeout in milliseconds.  Use RT_INDEFINITE_WAIT
  *                          to wait indefinitely.
  * @param   ppszName        Where to store the pointer to the name retrieved.
  *                          Optional.
@@ -823,7 +831,7 @@ VBGLR3DECL(int) VbglR3GuestPropDelSet(uint32_t u32ClientId,
 VBGLR3DECL(int) VbglR3GuestPropWait(uint32_t u32ClientId,
                                     const char *pszPatterns,
                                     void *pvBuf, uint32_t cbBuf,
-                                    uint64_t u64Timestamp, uint32_t u32Timeout,
+                                    uint64_t u64Timestamp, uint32_t cMillies,
                                     char ** ppszName, char **ppszValue,
                                     uint64_t *pu64Timestamp, char **ppszFlags,
                                     uint32_t *pcbBufActual)
@@ -833,9 +841,9 @@ VBGLR3DECL(int) VbglR3GuestPropWait(uint32_t u32ClientId,
      */
     GetNotification Msg;
 
-    Msg.hdr.u32Timeout = u32Timeout;
+    Msg.hdr.u32Timeout = cMillies;
     Msg.hdr.fInterruptible = true;
-    Msg.hdr.info.result = (uint32_t)VERR_WRONG_ORDER;  /** @todo drop the cast when the result type has been fixed! */
+    Msg.hdr.info.result = VERR_WRONG_ORDER;
     Msg.hdr.info.u32ClientID = u32ClientId;
     Msg.hdr.info.u32Function = GET_NOTIFICATION;
     Msg.hdr.info.cParms = 4;
@@ -901,4 +909,4 @@ VBGLR3DECL(int) VbglR3GuestPropWait(uint32_t u32ClientId,
 
     return VINF_SUCCESS;
 }
-
+#endif /* VBOX_VBGLR3_XFREE86 */
