@@ -1,11 +1,11 @@
-/* $Id: icd_drv.c $ */
+/* $Id: icd_drv.c 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  * VBox OpenGL windows ICD driver functions
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,16 +14,13 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "cr_error.h"
 #include "icd_drv.h"
 #include "cr_gl.h"
 #include "stub.h"
+#include "cr_mem.h"
 
 #include <windows.h>
 
@@ -77,7 +74,7 @@ static GLuint ComputeVisBits( HDC hdc )
 
 void APIENTRY DrvReleaseContext(HGLRC hglrc)
 {
-    /*crDebug( "DrvReleaseContext(%x) called", hglrc );*/
+    /*crDebug( "DrvReleaseContext(0x%x) called", hglrc );*/
     stubMakeCurrent( NULL, NULL );
 }
 
@@ -99,7 +96,7 @@ PICDTABLE APIENTRY DrvSetContext(HDC hdc, HGLRC hglrc, void *callback)
     ContextInfo *context;
     WindowInfo *window;
 
-    /*crDebug( "DrvSetContext called(%x, %x)", hdc, hglrc );*/
+    /*crDebug( "DrvSetContext called(0x%x, 0x%x)", hdc, hglrc );*/
     (void) (callback);
 
     context = (ContextInfo *) crHashtableSearch(stub.contextTable, (unsigned long) hglrc);
@@ -115,10 +112,10 @@ PICDTABLE APIENTRY DrvSetContext(HDC hdc, HGLRC hglrc, void *callback)
 
 BOOL APIENTRY DrvSetPixelFormat(HDC hdc, int iPixelFormat)
 {
-    crDebug( "DrvSetPixelFormat called.\n" );
+    crDebug( "DrvSetPixelFormat(0x%x, %i) called.", hdc, iPixelFormat );
 
-    if ( iPixelFormat != 1 ) {
-        crError( "wglSetPixelFormat: iPixelFormat=%d?\n", iPixelFormat );
+    if ( (iPixelFormat<1) || (iPixelFormat>2) ) {
+        crError( "wglSetPixelFormat: iPixelFormat=%d?", iPixelFormat );
     }
 
     return 1;
@@ -129,7 +126,7 @@ HGLRC APIENTRY DrvCreateContext(HDC hdc)
     char dpyName[MAX_DPY_NAME];
     ContextInfo *context;
 
-    crDebug( "DrvCreateContext called.\n" );
+    crDebug( "DrvCreateContext(0x%x) called.", hdc);
 
     stubInit();
 
@@ -148,6 +145,7 @@ HGLRC APIENTRY DrvCreateContext(HDC hdc)
 
 HGLRC APIENTRY DrvCreateLayerContext(HDC hdc, int iLayerPlane)
 {
+    crDebug( "DrvCreateLayerContext(0x%x, %i) called.", hdc, iLayerPlane);
     //We don't support more than 1 layers.
     if (iLayerPlane == 0) {
         return DrvCreateContext(hdc);
@@ -179,50 +177,86 @@ int APIENTRY DrvGetLayerPaletteEntries(HDC hdc, int iLayerPlane,
 int APIENTRY DrvDescribePixelFormat(HDC hdc, int iPixelFormat, UINT nBytes, LPPIXELFORMATDESCRIPTOR pfd)
 {
     if ( !pfd ) {
-        return 1; /* There's only one, baby */
+        return 2;
     }
 
     if ( nBytes != sizeof(*pfd) ) {
-        crWarning( "DrvDescribePixelFormat: nBytes=%u?\n", nBytes );
-        return 1; /* There's only one, baby */
+        crWarning( "DrvDescribePixelFormat: nBytes=%u?", nBytes );
+        return 2;
     }
 
-    pfd->nSize           = sizeof(*pfd);
-    pfd->nVersion        = 1;
-    pfd->dwFlags         = ( PFD_DRAW_TO_WINDOW |
-                 PFD_SUPPORT_GDI    |
-                 PFD_SUPPORT_OPENGL |
-                 PFD_DOUBLEBUFFER );
-    pfd->iPixelType      = PFD_TYPE_RGBA;
-    pfd->cColorBits      = 32;
-    pfd->cRedBits        = 8;
-    pfd->cRedShift       = 24;
-    pfd->cGreenBits      = 8;
-    pfd->cGreenShift     = 16;
-    pfd->cBlueBits       = 8;
-    pfd->cBlueShift      = 8;
-    pfd->cAlphaBits      = 8;
-    pfd->cAlphaShift     = 0;
-    pfd->cAccumBits      = 0;
-    pfd->cAccumRedBits   = 0;
-    pfd->cAccumGreenBits = 0;
-    pfd->cAccumBlueBits  = 0;
-    pfd->cAccumAlphaBits = 0;
-    pfd->cDepthBits      = 32;
-    pfd->cStencilBits    = 8;
-    pfd->cAuxBuffers     = 0;
-    pfd->iLayerType      = PFD_MAIN_PLANE;
-    pfd->bReserved       = 0;
-    pfd->dwLayerMask     = 0;
-    pfd->dwVisibleMask   = 0;
-    pfd->dwDamageMask    = 0;
+    if (iPixelFormat==1)
+    {
+        crMemZero(pfd, sizeof(*pfd));
+
+        pfd->nSize           = sizeof(*pfd);
+        pfd->nVersion        = 1;
+        pfd->dwFlags         = (PFD_DRAW_TO_WINDOW |
+                                PFD_SUPPORT_OPENGL |
+                                PFD_DOUBLEBUFFER);
+        pfd->iPixelType      = PFD_TYPE_RGBA;
+        pfd->cColorBits      = 32;
+        pfd->cRedBits        = 8;
+        pfd->cRedShift       = 24;
+        pfd->cGreenBits      = 8;
+        pfd->cGreenShift     = 16;
+        pfd->cBlueBits       = 8;
+        pfd->cBlueShift      = 8;
+        pfd->cAlphaBits      = 8;
+        pfd->cAlphaShift     = 0;
+        pfd->cAccumBits      = 0;
+        pfd->cAccumRedBits   = 0;
+        pfd->cAccumGreenBits = 0;
+        pfd->cAccumBlueBits  = 0;
+        pfd->cAccumAlphaBits = 0;
+        pfd->cDepthBits      = 32;
+        pfd->cStencilBits    = 8;
+        pfd->cAuxBuffers     = 0;
+        pfd->iLayerType      = PFD_MAIN_PLANE;
+        pfd->bReserved       = 0;
+        pfd->dwLayerMask     = 0;
+        pfd->dwVisibleMask   = 0;
+        pfd->dwDamageMask    = 0;
+    }
+    else
+    {
+        crMemZero(pfd, sizeof(*pfd));
+        pfd->nVersion        = 1;
+        pfd->dwFlags         = (PFD_DRAW_TO_WINDOW|
+                                PFD_SUPPORT_OPENGL);
+
+        pfd->iPixelType      = PFD_TYPE_RGBA;
+        pfd->cColorBits      = 32;
+        pfd->cRedBits        = 8;
+        pfd->cRedShift       = 16;
+        pfd->cGreenBits      = 8;
+        pfd->cGreenShift     = 8;
+        pfd->cBlueBits       = 8;
+        pfd->cBlueShift      = 0;
+        pfd->cAlphaBits      = 0;
+        pfd->cAlphaShift     = 0;
+        pfd->cAccumBits      = 64;
+        pfd->cAccumRedBits   = 16;
+        pfd->cAccumGreenBits = 16;
+        pfd->cAccumBlueBits  = 16;
+        pfd->cAccumAlphaBits = 0;
+        pfd->cDepthBits      = 16;
+        pfd->cStencilBits    = 8;
+        pfd->cAuxBuffers     = 0;
+        pfd->iLayerType      = PFD_MAIN_PLANE;
+        pfd->bReserved       = 0;
+        pfd->dwLayerMask     = 0;
+        pfd->dwVisibleMask   = 0;
+        pfd->dwDamageMask    = 0;
+    }
 
     /* the max PFD index */
-    return 1;
+    return 2;
 }
 
 BOOL APIENTRY DrvDeleteContext(HGLRC hglrc)
 {
+    /*crDebug( "DrvDeleteContext(0x%x) called", hglrc );*/
     stubDestroyContext( (unsigned long) hglrc );
     return 1;
 }
@@ -236,7 +270,7 @@ BOOL APIENTRY DrvCopyContext(HGLRC hglrcSrc, HGLRC hglrcDst, UINT mask)
 BOOL APIENTRY DrvShareLists(HGLRC hglrc1, HGLRC hglrc2)
 {
     crWarning( "DrvShareLists: unsupported" );
-    return 0;
+    return 1;
 }
 
 int APIENTRY DrvSetLayerPaletteEntries(HDC hdc, int iLayerPlane,
@@ -272,7 +306,7 @@ BOOL APIENTRY DrvSwapLayerBuffers(HDC hdc, UINT fuPlanes)
 BOOL APIENTRY DrvSwapBuffers(HDC hdc)
 {
     const WindowInfo *window;
-    //crDebug( "DrvSwapBuffers(%x) called", hdc );
+    /*crDebug( "DrvSwapBuffers(0x%x) called", hdc );*/
     window = stubGetWindowInfo(hdc);    
     stubSwapBuffers( window, 0 );
     return 1;
