@@ -1,10 +1,10 @@
-/** @file $Id: vboxvideo_drm.c $
+/** @file $Id: vboxvideo_drm.c 28800 2010-04-27 08:22:32Z vboxsync $
  *
  * VirtualBox Additions Linux kernel driver, DRM support
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -53,9 +49,12 @@
  *    Gareth Hughes <gareth@valinux.com>
  */
 
-#include <linux/autoconf.h>
+#ifndef AUTOCONF_INCLUDED
+# include <linux/autoconf.h>
+#endif
 #include <linux/version.h>
 #include <linux/module.h>
+#include "version-generated.h"
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
 
@@ -66,8 +65,18 @@ static struct pci_device_id pciidlist[] = {
 	vboxvideo_PCI_IDS
 };
 
+int vboxvideo_driver_load(struct drm_device * dev, unsigned long flags)
+{
+# if LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 28)
+	return drm_vblank_init(dev, 1);
+#else
+    return 0;
+#endif
+}
+
 static struct drm_driver driver = {
 	/* .driver_features = DRIVER_USE_MTRR, */
+	.load = vboxvideo_driver_load,
 	.reclaim_buffers = drm_core_reclaim_buffers,
 	.get_map_ofs = drm_core_get_map_ofs,
 	.get_reg_ofs = drm_core_get_reg_ofs,
@@ -75,7 +84,13 @@ static struct drm_driver driver = {
 		 .owner = THIS_MODULE,
 		 .open = drm_open,
 		 .release = drm_release,
+                 /* This was changed with Linux 2.6.33 but Fedora backported this
+                  * change to their 2.6.32 kernel. */
+#if defined(DRM_UNLOCKED) || LINUX_VERSION_CODE >= KERNEL_VERSION (2, 6, 33)
+		 .unlocked_ioctl = drm_ioctl,
+#else
 		 .ioctl = drm_ioctl,
+#endif
 		 .mmap = drm_mmap,
 		 .poll = drm_poll,
 		 .fasync = drm_fasync,
@@ -111,4 +126,7 @@ MODULE_DESCRIPTION(DRIVER_DESC);
 
 #endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27) */
 
+#ifdef MODULE_VERSION
+MODULE_VERSION(VBOX_VERSION_STRING);
+#endif
 MODULE_LICENSE("GPL and additional rights");
