@@ -1,11 +1,11 @@
 /** @file
  *
- * vboxvfs -- VirtualBox Guest Additions for Linux:
+ * vboxsf -- VirtualBox Guest Additions for Linux:
  * Regular file inode and file operations
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +14,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*
@@ -26,7 +22,38 @@
 
 #include "vfsmod.h"
 
-#define CHUNK_SIZE 4096
+
+static void *alloc_bounch_buffer (size_t *tmp_sizep, PRTCCPHYS physp, size_t xfer_size, const char *caller)
+{
+    size_t tmp_size;
+    void *tmp;
+
+    /* try for big first. */
+    tmp_size = RT_ALIGN_Z(xfer_size, PAGE_SIZE);
+    if (tmp_size > 16U*_1K)
+        tmp_size = 16U*_1K;
+    tmp = kmalloc (tmp_size, GFP_KERNEL);
+    if (!tmp) {
+
+        /* fall back on a page sized buffer. */
+        tmp = kmalloc (PAGE_SIZE, GFP_KERNEL);
+        if (!tmp) {
+            LogRel(("%s: could not allocate bounce buffer for xfer_size=%zu %s\n", caller, xfer_size));
+            return NULL;
+        }
+        tmp_size = PAGE_SIZE;
+    }
+
+    *tmp_sizep = tmp_size;
+    *physp = virt_to_phys(tmp);
+    return tmp;
+}
+
+static void free_bounch_buffer (void *tmp)
+{
+    kfree (tmp);
+}
+
 
 /* fops */
 static int
@@ -34,6 +61,9 @@ sf_reg_read_aux (const char *caller, struct sf_glob_info *sf_g,
                  struct sf_reg_info *sf_r, void *buf, uint32_t *nread,
                  uint64_t pos)
 {
+        /** @todo bird: yes, kmap() and kmalloc() input only. Since the buffer is
+         *        contiguous in physical memory (kmalloc or single page), we should
+         *        use a physical address here to speed things up. */
         int rc = vboxCallRead (&client_handle, &sf_g->map, sf_r->handle,
                                pos, nread, buf, false /* already locked? */);
         if (RT_FAILURE (rc)) {
@@ -49,6 +79,9 @@ sf_reg_write_aux (const char *caller, struct sf_glob_info *sf_g,
                   struct sf_reg_info *sf_r, void *buf, uint32_t *nwritten,
                   uint64_t pos)
 {
+        /** @todo bird: yes, kmap() and kmalloc() input only. Since the buffer is
+         *        contiguous in physical memory (kmalloc or single page), we should
+         *        use a physical address here to speed things up. */
         int rc = vboxCallWrite (&client_handle, &sf_g->map, sf_r->handle,
                                 pos, nwritten, buf, false /* already locked? */);
         if (RT_FAILURE (rc)) {
@@ -64,6 +97,8 @@ sf_reg_read (struct file *file, char *buf, size_t size, loff_t *off)
 {
         int err;
         void *tmp;
+        RTCCPHYS tmp_phys;
+        size_t tmp_size;
         size_t left = size;
         ssize_t total_bytes_read = 0;
         struct inode *inode = file->f_dentry->d_inode;
@@ -83,16 +118,14 @@ sf_reg_read (struct file *file, char *buf, size_t size, loff_t *off)
                 return 0;
         }
 
-        tmp = kmalloc (CHUNK_SIZE, GFP_KERNEL);
-        if (!tmp) {
-                LogRelFunc(("could not allocate bounce buffer memory %d bytes\n", CHUNK_SIZE));
-                return -ENOMEM;
-        }
+        tmp = alloc_bounch_buffer (&tmp_size, &tmp_phys, size, __PRETTY_FUNCTION__);
+        if (!tmp)
+            return -ENOMEM;
 
         while (left) {
                 uint32_t to_read, nread;
 
-                to_read = CHUNK_SIZE;
+                to_read = tmp_size;
                 if (to_read > left) {
                         to_read = (uint32_t) left;
                 }
@@ -117,11 +150,11 @@ sf_reg_read (struct file *file, char *buf, size_t size, loff_t *off)
         }
 
         *off += total_bytes_read;
-        kfree (tmp);
+        free_bounch_buffer (tmp);
         return total_bytes_read;
 
  fail:
-        kfree (tmp);
+        free_bounch_buffer (tmp);
         return err;
 }
 
@@ -130,6 +163,8 @@ sf_reg_write (struct file *file, const char *buf, size_t size, loff_t *off)
 {
         int err;
         void *tmp;
+        RTCCPHYS tmp_phys;
+        size_t tmp_size;
         size_t left = size;
         ssize_t total_bytes_written = 0;
         struct inode *inode = file->f_dentry->d_inode;
@@ -160,16 +195,14 @@ sf_reg_write (struct file *file, const char *buf, size_t size, loff_t *off)
         if (!size)
                 return 0;
 
-        tmp = kmalloc (CHUNK_SIZE, GFP_KERNEL);
-        if (!tmp) {
-                LogRelFunc(("could not allocate bounce buffer memory %d bytes\n", CHUNK_SIZE));
+        tmp = alloc_bounch_buffer (&tmp_size, &tmp_phys, size, __PRETTY_FUNCTION__);
+        if (!tmp)
                 return -ENOMEM;
-        }
 
         while (left) {
                 uint32_t to_write, nwritten;
 
-                to_write = CHUNK_SIZE;
+                to_write = tmp_size;
                 if (to_write > left) {
                         to_write = (uint32_t) left;
                 }
@@ -180,7 +213,14 @@ sf_reg_write (struct file *file, const char *buf, size_t size, loff_t *off)
                         goto fail;
                 }
 
-                err = sf_reg_write_aux (__func__, sf_g, sf_r, tmp, &nwritten, pos);
+#if 1
+                if (VbglR0CanUsePhysPageList()) {
+                    err = VbglR0SfWritePhysCont (&client_handle, &sf_g->map, sf_r->handle,
+                                                 pos, &nwritten, tmp_phys);
+                    err = RT_FAILURE(err) ? -EPROTO : 0;
+                } else
+#endif
+                    err = sf_reg_write_aux (__func__, sf_g, sf_r, tmp, &nwritten, pos);
                 if (err)
                         goto fail;
 
@@ -197,11 +237,11 @@ sf_reg_write (struct file *file, const char *buf, size_t size, loff_t *off)
                 inode->i_size = *off;
 
         sf_i->force_restat = 1;
-        kfree (tmp);
+        free_bounch_buffer (tmp);
         return total_bytes_written;
 
  fail:
-        kfree (tmp);
+        free_bounch_buffer (tmp);
         return err;
 }
 
@@ -226,7 +266,23 @@ sf_reg_open (struct inode *inode, struct file *file)
                 return -ENOMEM;
         }
 
-        memset(&params, 0, sizeof(params));
+        /* Already open? */
+        if (sf_i->handle != SHFL_HANDLE_NIL)
+        {
+            /*
+             * This inode was created with sf_create_aux(). Check the CreateFlags:
+             * O_CREAT, O_TRUNC: inherent true (file was just created). Not sure
+             * about the access flags (SHFL_CF_ACCESS_*).
+             */
+            sf_i->force_restat = 1;
+            sf_r->handle = sf_i->handle;
+            sf_i->handle = SHFL_HANDLE_NIL;
+            sf_i->file = file;
+            file->private_data = sf_r;
+            return 0;
+        }
+
+        RT_ZERO(params);
         params.Handle = SHFL_HANDLE_NIL;
         /* We check the value of params.Handle afterwards to find out if
          * the call succeeded or failed, as the API does not seem to cleanly
@@ -277,8 +333,13 @@ sf_reg_open (struct inode *inode, struct file *file)
                 }
         }
 
+        if (file->f_flags & O_APPEND) {
+                LogFunc(("O_APPEND set\n"));
+                params.CreateFlags |= SHFL_CF_ACCESS_APPEND;
+        }
+
         params.Info.Attr.fMode = inode->i_mode;
-        LogFunc(("sf_reg_open: calling vboxCallCreate, file %s, flags=%d, %#x\n",
+        LogFunc(("sf_reg_open: calling vboxCallCreate, file %s, flags=%#x, %#x\n",
                  sf_i->path->String.utf8 , file->f_flags, params.CreateFlags));
         rc = vboxCallCreate (&client_handle, &sf_g->map, sf_i->path, &params);
 
@@ -332,6 +393,7 @@ sf_reg_release (struct inode *inode, struct file *file)
 
         kfree (sf_r);
         sf_i->file = NULL;
+        sf_i->handle = SHFL_HANDLE_NIL;
         file->private_data = NULL;
         return 0;
 }
