@@ -1,10 +1,10 @@
-/** @file $Id: vboxvideo_dri.c $
+/** @file $Id: vboxvideo_dri.c 28800 2010-04-27 08:22:32Z vboxsync $
  *
  * VirtualBox X11 Additions graphics driver, DRI support
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -174,6 +170,16 @@ Bool VBOXDRIScreenInit(int scrnIndex, ScreenPtr pScreen, VBOXPtr pVBox)
                    "DRI is only available in 16bpp or 32bpp graphics modes.\n");
         rc = FALSE;
     }
+    /* Assertion */
+    if (   (pScrn->displayWidth == 0)
+        || (pVBox->pciInfo == NULL)
+        || (pVBox->base == NULL)
+        || (pVBox->mapSize == 0))
+    {
+        xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "%s: preconditions failed\n",
+                   __PRETTY_FUNCTION__);
+        rc = FALSE;
+    }
     /* Check that the GLX, DRI, and DRM modules have been loaded by testing for
      * canonical symbols in each module, the way all existing _dri drivers do.
      */
@@ -248,9 +254,10 @@ Bool VBOXDRIScreenInit(int scrnIndex, ScreenPtr pScreen, VBOXPtr pVBox)
         pDRIInfo->ddxDriverPatchVersion = 0;
         pDRIInfo->ddxDrawableTableEntry = VBOX_MAX_DRAWABLES;
         pDRIInfo->maxDrawableTableEntry = VBOX_MAX_DRAWABLES;
-        pDRIInfo->frameBufferPhysicalAddress = pVBox->base;
+        pDRIInfo->frameBufferPhysicalAddress = (pointer)pVBox->mapPhys;
         pDRIInfo->frameBufferSize = pVBox->mapSize;
-        pDRIInfo->frameBufferStride = pScrn->displayWidth;
+        pDRIInfo->frameBufferStride =   pScrn->displayWidth
+                                      * pScrn->bitsPerPixel / 8;
         pDRIInfo->SAREASize = SAREA_MAX;  /* we have no private bits yet. */
         /* This can't be zero, as the server callocs this size and checks for
          * non-NULL... */
@@ -304,6 +311,13 @@ Bool VBOXDRIScreenInit(int scrnIndex, ScreenPtr pScreen, VBOXPtr pVBox)
     return rc;
 }
 
+void VBOXDRIUpdateStride(ScrnInfoPtr pScrn, VBOXPtr pVBox)
+{
+    DRIInfoPtr pDRIInfo = pVBox->pDRIInfo;
+    pDRIInfo->frameBufferStride =   pScrn->displayWidth
+                                  * pScrn->bitsPerPixel / 8;
+}
+
 void
 VBOXDRICloseScreen(ScreenPtr pScreen, VBOXPtr pVBox)
 {
