@@ -1,10 +1,10 @@
-/* $Id: DevIchAc97.cpp 18645 2009-04-02 15:38:31Z vboxsync $ */
+/* $Id: DevIchAc97.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DevIchAc97 - VBox ICH AC97 Audio Controller.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -205,7 +201,7 @@ typedef struct AC97LinkState
     PPDMIAUDIOCONNECTOR     pDrv;
     /** Pointer to the attached audio driver. */
     PPDMIBASE               pDrvBase;
-    /** The base interface. */
+    /** The base interface for LUN\#0. */
     PDMIBASE                IBase;
     /** Base port of the I/O space region. */
     RTIOPORT                IOPortBase[2];
@@ -1457,21 +1453,19 @@ static DECLCALLBACK(int) ichac97SaveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHan
  * @returns VBox status code.
  * @param   pDevIns     The device instance.
  * @param   pSSMHandle  The handle to the saved state.
- * @param   u32Version  The data unit version number.
+ * @param   uVersion    The data unit version number.
+ * @param   uPass       The data pass.
  */
 static DECLCALLBACK(int) ichac97LoadExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle,
-                                          uint32_t u32Version)
+                                          uint32_t uVersion, uint32_t uPass)
 {
     PCIAC97LinkState *pThis = PDMINS_2_DATA(pDevIns, PCIAC97LinkState *);
-    size_t  i;
-    uint8_t active[LAST_INDEX];
     AC97LinkState *s = &pThis->ac97;
+    uint8_t active[LAST_INDEX];
+    size_t  i;
 
-    if (u32Version != AC97_SSM_VERSION)
-    {
-        AssertFailed();
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
-    }
+    AssertMsgReturn (uVersion == AC97_SSM_VERSION, ("%d\n", uVersion), VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION);
+    Assert (uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     SSMR3GetU32 (pSSMHandle, &s->glob_cnt);
     SSMR3GetU32 (pSSMHandle, &s->glob_sta);
@@ -1539,47 +1533,20 @@ static DECLCALLBACK(void)  ac97Reset (PPDMDEVINS pDevIns)
 }
 
 /**
- * Queries an interface to the driver.
- *
- * @returns Pointer to interface.
- * @returns NULL if the interface was not supported by the driver.
- * @param   pInterface          Pointer to this interface structure.
- * @param   enmInterface        The requested interface identification.
- * @thread  Any thread.
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
  */
 static DECLCALLBACK(void *) ichac97QueryInterface (struct PDMIBASE *pInterface,
-                                                   PDMINTERFACE enmInterface)
+                                                   const char *pszIID)
 {
-    PCIAC97LinkState *pThis =
-        (PCIAC97LinkState *)((uintptr_t)pInterface
-                             - RT_OFFSETOF(PCIAC97LinkState, ac97.IBase));
+    PCIAC97LinkState *pThis = RT_FROM_MEMBER(pInterface, PCIAC97LinkState, ac97.IBase);
     Assert(&pThis->ac97.IBase == pInterface);
-    switch (enmInterface)
-    {
-        case PDMINTERFACE_BASE:
-            return &pThis->ac97.IBase;
-        default:
-            return NULL;
-    }
+
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pThis->ac97.IBase);
+    return NULL;
 }
 
 /**
- * Construct a device instance for a VM.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- *                      If the registration structure is needed,
- *                      pDevIns->pDevReg points to it.
- * @param   iInstance   Instance number. Use this to figure out which
- *                      registers and such to use.
- *                      The device number is also found in pDevIns->iInstance,
- *                      but since it's likely to be freqently used PDM passes
- *                      it as parameter.
- * @param   pCfgHandle  Configuration node handle for the device.
- *                      Use this to obtain the configuration
- *                      of the device instance. It's also found in
- *                      pDevIns->pCfgHandle, but like iInstance it's expected
- *                      to be used a bit in this function.
+ * @interface_method_impl{PDMDEVREG,pfnConstruct}
  */
 static DECLCALLBACK(int) ichac97Construct (PPDMDEVINS pDevIns, int iInstance,
                                            PCFGMNODE pCfgHandle)
@@ -1589,6 +1556,14 @@ static DECLCALLBACK(int) ichac97Construct (PPDMDEVINS pDevIns, int iInstance,
     int               rc;
 
     Assert(iInstance == 0);
+    PDMDEV_CHECK_VERSIONS_RETURN(pDevIns);
+
+    /*
+     * Validations.
+     */
+    if (!CFGMR3AreValuesValid (pCfgHandle, "\0"))
+        return PDMDEV_SET_ERROR (pDevIns, VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES,
+                                 N_ ("Invalid configuration for the AC97 device"));
 
     /*
      * Initialize data (most of it anyway).
@@ -1634,10 +1609,7 @@ static DECLCALLBACK(int) ichac97Construct (PPDMDEVINS pDevIns, int iInstance,
     if (RT_FAILURE (rc))
         return rc;
 
-    rc = PDMDevHlpSSMRegister (pDevIns, pDevIns->pDevReg->szDeviceName,
-                               iInstance, AC97_SSM_VERSION, sizeof(*pThis),
-                               NULL, ichac97SaveExec, NULL,
-                               NULL, ichac97LoadExec, NULL);
+    rc = PDMDevHlpSSMRegister (pDevIns, AC97_SSM_VERSION, sizeof(*pThis), ichac97SaveExec, ichac97LoadExec);
     if (RT_FAILURE (rc))
         return rc;
 
@@ -1658,12 +1630,10 @@ static DECLCALLBACK(int) ichac97Construct (PPDMDEVINS pDevIns, int iInstance,
 
     ac97Reset (pDevIns);
 
-#ifndef RT_OS_DARWIN /* coreaudio doesn't supply these. */
     if (!s->voice_pi)
         LogRel (("AC97: WARNING: Unable to open PCM IN!\n"));
     if (!s->voice_mc)
         LogRel (("AC97: WARNING: Unable to open PCM MC!\n"));
-#endif
     if (!s->voice_po)
         LogRel (("AC97: WARNING: Unable to open PCM OUT!\n"));
 
@@ -1683,7 +1653,6 @@ static DECLCALLBACK(int) ichac97Construct (PPDMDEVINS pDevIns, int iInstance,
             N_ ("No audio devices could be opened. Selecting the NULL audio backend "
                 "with the consequence that no sound is audible"));
     }
-#ifndef RT_OS_DARWIN
     else if (!s->voice_pi || !s->voice_po || !s->voice_mc)
     {
         char   szMissingVoices[128];
@@ -1701,7 +1670,6 @@ static DECLCALLBACK(int) ichac97Construct (PPDMDEVINS pDevIns, int iInstance,
                 "is working properly. Check the logfile for error messages of the audio "
                 "subsystem"), szMissingVoices);
     }
-#endif
 
     return VINF_SUCCESS;
 }
@@ -1713,7 +1681,7 @@ const PDMDEVREG g_DeviceICHAC97 =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "ichac97",
     /* szRCMod */
     "",
