@@ -1,4 +1,4 @@
-/* $Id: DevSB16.cpp $ */
+/* $Id: DevSB16.cpp 26165 2010-02-02 19:50:31Z vboxsync $ */
 /** @file
  * DevSB16 - VBox SB16 Audio Controller.
  *
@@ -35,14 +35,13 @@
 #include <VBox/pdmdev.h>
 #include <iprt/assert.h>
 #include <iprt/string.h>
+#include <iprt/uuid.h>
 #include "../vl_vbox.h"
 
 extern "C" {
 #include "audio.h"
 }
 
-#define SB16_SSM_VERSION 1
-
 #ifndef VBOX
 
 #define LENOFA(a) ((int) (sizeof(a)/sizeof(a[0])))
@@ -60,6 +59,12 @@ extern "C" {
 
 #else /* VBOX */
 
+/** Current saved state version. */
+#define SB16_SAVE_STATE_VERSION         2
+/** The version used in VirtualBox version 3.0 and earlier. This didn't include
+ * the config dump. */
+#define SB16_SAVE_STATE_VERSION_VBOX_30 1
+
 DECLINLINE(void) dolog (const char *fmt, ...)
 {
     va_list ap;
@@ -121,6 +126,13 @@ typedef struct SB16State {
     QEMUSoundCard card;
 #ifndef VBOX
     qemu_irq *pic;
+#endif
+#ifdef VBOX /* lazy bird */
+    int irqCfg;
+    int dmaCfg;
+    int hdmaCfg;
+    int portCfg;
+    int verCfg;
 #endif
     int irq;
     int dma;
@@ -176,6 +188,7 @@ typedef struct SB16State {
 #else
     PTMTIMER  pTimer;
     PPDMIBASE pDrvBase;
+    /** LUN\#0: Base interface. */
     PDMIBASE  IBase;
 #endif
     /* mixer state */
@@ -1476,15 +1489,12 @@ static void SB_audio_callback (void *opaque, int free)
 #endif
 }
 
-#ifndef VBOX
 static void SB_save (QEMUFile *f, void *opaque)
 {
+#ifndef VBOX
     SB16State *s = opaque;
 #else
-static DECLCALLBACK(int) SaveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
-{
-    SB16State *s = PDMINS_2_DATA (pDevIns, SB16State *);
-    QEMUFile *f = pSSMHandle;
+    SB16State *s = (SB16State *)opaque;
 #endif
 
     qemu_put_be32 (f, s->irq);
@@ -1534,33 +1544,19 @@ static DECLCALLBACK(int) SaveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
 
     qemu_put_be32 (f, s->mixer_nreg);
     qemu_put_buffer (f, s->mixer_regs, 256);
-
-#ifdef VBOX
-    return VINF_SUCCESS;
-#endif
 }
 
-#ifndef VBOX
 static int SB_load (QEMUFile *f, void *opaque, int version_id)
 {
+#ifndef VBOX
     SB16State *s = opaque;
 
     if (version_id != 1) {
         return -EINVAL;
     }
-#else  /* VBOX */
-static DECLCALLBACK(int) LoadExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle,
-                                   uint32_t u32Version)
-{
-    SB16State *s = PDMINS_2_DATA (pDevIns, SB16State *);
-    QEMUFile *f = pSSMHandle;
-
-    if (u32Version != SB16_SSM_VERSION)
-    {
-        AssertMsgFailed(("u32Version=%d\n", u32Version));
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
-    }
-#endif /* VBOX */
+#else
+    SB16State *s = (SB16State *)opaque;
+#endif
 
     s->irq=qemu_get_be32 (f);
     s->dma=qemu_get_be32 (f);
@@ -1710,19 +1706,82 @@ int SB16_init (AudioState *audio, qemu_irq *pic)
 
 #else /* VBOX */
 
-static DECLCALLBACK(void *) sb16QueryInterface (struct PDMIBASE *pInterface,
-                                                PDMINTERFACE enmInterface)
+
+static DECLCALLBACK(int) sb16LiveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t uPass)
 {
-    SB16State *pThis = (SB16State *)((uintptr_t)pInterface
-                     - RT_OFFSETOF(SB16State, IBase));
-    Assert(&pThis->IBase == pInterface);
-    switch (enmInterface)
+    SB16State *pThis = PDMINS_2_DATA (pDevIns, SB16State *);
+
+    SSMR3PutS32(pSSM, pThis->irqCfg);
+    SSMR3PutS32(pSSM, pThis->dmaCfg);
+    SSMR3PutS32(pSSM, pThis->hdmaCfg);
+    SSMR3PutS32(pSSM, pThis->portCfg);
+    SSMR3PutS32(pSSM, pThis->verCfg);
+    return VINF_SSM_DONT_CALL_AGAIN;
+}
+
+static DECLCALLBACK(int) sb16SaveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM)
+{
+    SB16State *pThis = PDMINS_2_DATA (pDevIns, SB16State *);
+
+    sb16LiveExec (pDevIns, pSSM, 0);
+    SB_save (pSSM, pThis);
+    return VINF_SUCCESS;
+}
+
+static DECLCALLBACK(int) sb16LoadExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM,
+                                       uint32_t uVersion, uint32_t uPass)
+{
+    SB16State *pThis = PDMINS_2_DATA (pDevIns, SB16State *);
+
+    AssertMsgReturn(    uVersion == SB16_SAVE_STATE_VERSION
+                    ||  uVersion == SB16_SAVE_STATE_VERSION_VBOX_30,
+                    ("%u\n", uVersion),
+                    VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION);
+    if (uVersion > SB16_SAVE_STATE_VERSION_VBOX_30)
     {
-        case PDMINTERFACE_BASE:
-            return &pThis->IBase;
-        default:
-            return NULL;
+        int32_t irq;
+        SSMR3GetS32 (pSSM, &irq);
+        int32_t dma;
+        SSMR3GetS32 (pSSM, &dma);
+        int32_t hdma;
+        SSMR3GetS32 (pSSM, &hdma);
+        int32_t port;
+        SSMR3GetS32 (pSSM, &port);
+        int32_t ver;
+        int rc = SSMR3GetS32 (pSSM, &ver);
+        AssertRCReturn (rc, rc);
+
+        if (   irq  != pThis->irqCfg
+            || dma  != pThis->dmaCfg
+            || hdma != pThis->hdmaCfg
+            || port != pThis->portCfg
+            || ver  != pThis->verCfg )
+            return SSMR3SetCfgError(pSSM, RT_SRC_POS,
+                                    N_("config changed: irq=%x/%x dma=%x/%x hdma=%x/%x port=%x/%x ver=%x/%x (saved/config)"),
+                                    irq,  pThis->irqCfg,
+                                    dma,  pThis->dmaCfg,
+                                    hdma, pThis->hdmaCfg,
+                                    port, pThis->portCfg,
+                                    ver,  pThis->verCfg);
     }
+    if (uPass != SSM_PASS_FINAL)
+        return VINF_SUCCESS;
+
+    SB_load(pSSM, pThis, uVersion);
+    return VINF_SUCCESS;
+}
+
+/**
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
+ */
+static DECLCALLBACK(void *) sb16QueryInterface (struct PDMIBASE *pInterface,
+                                                const char *pszIID)
+{
+    SB16State *pThis = RT_FROM_MEMBER(pInterface, SB16State, IBase);
+    Assert(&pThis->IBase == pInterface);
+
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pThis->IBase);
+    return NULL;
 }
 
 static DECLCALLBACK(int) sb16Construct (PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfgHandle)
@@ -1734,6 +1793,7 @@ static DECLCALLBACK(int) sb16Construct (PPDMDEVINS pDevIns, int iInstance, PCFGM
      * Validations.
      */
     Assert(iInstance == 0);
+    PDMDEV_CHECK_VERSIONS_RETURN(pDevIns);
     if (!CFGMR3AreValuesValid(pCfgHandle,
                               "IRQ\0"
                               "DMA\0"
@@ -1741,7 +1801,7 @@ static DECLCALLBACK(int) sb16Construct (PPDMDEVINS pDevIns, int iInstance, PCFGM
                               "Port\0"
                               "Version\0"))
         return PDMDEV_SET_ERROR(pDevIns, VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES,
-                                N_("Invalid configuraton for sb16 device"));
+                                N_("Invalid configuration for sb16 device"));
 
     /*
      * Read config data.
@@ -1750,30 +1810,35 @@ static DECLCALLBACK(int) sb16Construct (PPDMDEVINS pDevIns, int iInstance, PCFGM
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"IRQ\" value"));
+    s->irqCfg  = s->irq;
 
     rc = CFGMR3QuerySIntDef(pCfgHandle, "DMA", &s->dma, 1);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"DMA\" value"));
+    s->dmaCfg  = s->dma;
 
     rc = CFGMR3QuerySIntDef(pCfgHandle, "DMA16", &s->hdma, 5);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"DMA16\" value"));
+    s->hdmaCfg = s->hdma;
 
     RTIOPORT Port;
     rc = CFGMR3QueryPortDef(pCfgHandle, "Port", &Port, 0x220);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"Port\" value"));
-    s->port = Port;
+    s->port    = Port;
+    s->portCfg = Port;
 
     uint16_t u16Version;
     rc = CFGMR3QueryU16Def(pCfgHandle, "Version", &u16Version, 0x0405);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"Version\" value"));
-    s->ver = u16Version;
+    s->ver     = u16Version;
+    s->verCfg  = u16Version;
 
     /*
      * Init instance data.
@@ -1817,8 +1882,7 @@ static DECLCALLBACK(int) sb16Construct (PPDMDEVINS pDevIns, int iInstance, PCFGM
 
     s->can_write = 1;
 
-    rc = PDMDevHlpSSMRegister(pDevIns, pDevIns->pDevReg->szDeviceName, iInstance, SB16_SSM_VERSION,
-                              sizeof(*s), NULL, SaveExec, NULL, NULL, LoadExec, NULL);
+    rc = PDMDevHlpSSMRegister3(pDevIns, SB16_SAVE_STATE_VERSION, sizeof(*s), sb16LiveExec, sb16SaveExec, sb16LoadExec);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -1847,7 +1911,7 @@ const PDMDEVREG g_DeviceSB16 =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "sb16",
     /* szRCMod */
     "",
