@@ -1,10 +1,12 @@
-/* $Id: Builtins2.h $ */
+/* $Id: Builtins2.h 29563 2010-05-17 15:15:02Z vboxsync $ */
 /** @file
- * Built-in drivers & devices (part 2) header.
+ * Built-in drivers & devices part 2 header.
+ *
+ * These drivers and devices are in separate modules because of LGPL.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___Builtins2_h
@@ -24,7 +22,7 @@
 
 #include <VBox/pdm.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 #ifdef IN_VBOXDD2
 extern DECLEXPORT(const unsigned char)  g_abPcBiosBinary[];
@@ -35,10 +33,6 @@ extern DECLEXPORT(const unsigned char)  g_abNetBiosBinary[];
 extern DECLEXPORT(const unsigned)       g_cbNetBiosBinary;
 extern DECLEXPORT(const unsigned char)  g_abVmiBiosBinary[];
 extern DECLEXPORT(const unsigned)       g_cbVmiBiosBinary;
-# ifdef VBOX_WITH_EFI
-extern DECLEXPORT(const unsigned char)  g_abEfiThunkBinary[];
-extern DECLEXPORT(const unsigned)       g_cbEfiThunkBinary;
-# endif
 #else  /* !IN_VBOXDD2 */
 extern DECLIMPORT(const unsigned char)  g_abPcBiosBinary[];
 extern DECLIMPORT(const unsigned)       g_cbPcBiosBinary;
@@ -48,14 +42,16 @@ extern DECLIMPORT(const unsigned char)  g_abNetBiosBinary[];
 extern DECLIMPORT(const unsigned)       g_cbNetBiosBinary;
 extern DECLIMPORT(const unsigned char)  g_abVmiBiosBinary[];
 extern DECLIMPORT(const unsigned)       g_cbVmiBiosBinary;
-# ifdef VBOX_WITH_EFI
-extern DECLIMPORT(const unsigned char)  g_abEfiThunkBinary[];
-extern DECLIMPORT(const unsigned)       g_cbEfiThunkBinary;
-# endif
 #endif /* !IN_VBOXDD2 */
 extern const PDMDEVREG g_DeviceAPIC;
 extern const PDMDEVREG g_DeviceIOAPIC;
+#ifdef VBOX_WITH_SMC
+extern const PDMDEVREG g_DeviceSMC;
+#endif
+#ifdef VBOX_WITH_LPC
+extern const PDMDEVREG g_DeviceLPC;
+#endif
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
