@@ -1,10 +1,10 @@
-/* $Id: DevVGA.h $ */
+/* $Id: DevVGA.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DevVGA - VBox VGA/VESA device, internal header.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -43,10 +39,19 @@
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  * THE SOFTWARE.
  */
+#ifdef VBOX
+/** The default amount of VRAM. */
+# define VGA_VRAM_DEFAULT    (_4M)
+/** The maximum amount of VRAM. Limited by VBOX_MAX_ALLOC_PAGE_COUNT. */
+# define VGA_VRAM_MAX        (256 * _1M)
+/** The minimum amount of VRAM. */
+# define VGA_VRAM_MIN        (_1M)
+#endif
 
 #ifdef VBOX_WITH_HGSMI
-#include "HGSMI/HGSMIHost.h"
+# include "HGSMI/HGSMIHost.h"
 #endif /* VBOX_WITH_HGSMI */
+#include "DevVGASavedState.h"
 
 #define MSR_COLOR_EMULATION 0x01
 #define MSR_PAGE_SELECT     0x20
@@ -77,14 +82,7 @@
 #define VBE_DISPI_INDEX_X_OFFSET        0x8
 #define VBE_DISPI_INDEX_Y_OFFSET        0x9
 #define VBE_DISPI_INDEX_VBOX_VIDEO      0xa
-#ifdef VBOX_WITH_HGSMI
-/* @todo this will break saved state and is inefficient. use 2 PCI io ports. */
-#define VBE_DISPI_INDEX_VBVA_HOST       0xb
-#define VBE_DISPI_INDEX_VBVA_GUEST      0xc
-#define VBE_DISPI_INDEX_NB              0xd
-#else
 #define VBE_DISPI_INDEX_NB              0xb
-#endif /* !VBOX_WITH_HGSMI */
 
 #define VBE_DISPI_ID0                   0xB0C0
 #define VBE_DISPI_ID1                   0xB0C1
@@ -96,9 +94,7 @@
 /* The VBOX interface id. Indicates support for VBE_DISPI_INDEX_VBOX_VIDEO. */
 #define VBE_DISPI_ID_VBOX_VIDEO         0xBE00
 #ifdef VBOX_WITH_HGSMI
-/* The VBOX interface id. Indicates support for VBVA shared memory interface,
- * VBE_DISPI_INDEX_VBVA_GUEST_CMD and VBE_DISPI_INDEX_VBVA_HOST_CMD VBE indexes.
- */
+/* The VBOX interface id. Indicates support for VBVA shared memory interface. */
 #define VBE_DISPI_ID_HGSMI              0xBE01
 #endif /* VBOX_WITH_HGSMI */
 #endif /* VBOX */
@@ -259,6 +255,10 @@ typedef void FNCURSORDRAWLINE(struct VGAState *s, uint8_t *d, int y);
 
 #endif /* VBOX */
 
+#ifdef VBOXVDMA
+typedef struct VBOXVDMAHOST *PVBOXVDMAHOST;
+#endif
+
 typedef struct VGAState {
     VGA_STATE_COMMON
 #ifdef VBOX
@@ -282,13 +282,19 @@ typedef struct VGAState {
     bool                        fRemappedVGA;
     /** Whether to render the guest VRAM to the framebuffer memory. False only for some LFB modes. */
     bool                        fRenderVRAM;
-    bool                        padding9[2];
+    bool                        Padding1[2];
+
+    uint32_t                    cMonitors;
 
 #ifdef VBOX_WITH_HGSMI
     R3PTRTYPE(PHGSMIINSTANCE)   pHGSMI;
 #endif /* VBOX_WITH_HGSMI */
+#ifdef VBOXVDMA
+    R3PTRTYPE(PVBOXVDMAHOST)    pVdma;
+#endif
 
     /** Current refresh timer interval. */
+    uint32_t                    Padding2;
     uint32_t                    cMilliesRefreshInterval;
     /** Refresh timer handle - HC. */
     PTMTIMERR3                  RefreshTimer;
@@ -304,10 +310,21 @@ typedef struct VGAState {
     /** Pointer to the device instance - R0 Ptr. */
     PPDMDEVINSR0                pDevInsR0;
 
-    /** The display port base interface. */
-    PDMIBASE                    Base;
-    /** The display port interface. */
-    PDMIDISPLAYPORT             Port;
+    /** The critical section. */
+    PDMCRITSECT                 lock;
+
+    /** LUN\#0: The display port base interface. */
+    PDMIBASE                    IBase;
+    /** LUN\#0: The display port interface. */
+    PDMIDISPLAYPORT             IPort;
+#if defined(VBOX_WITH_HGSMI) && defined(VBOX_WITH_VIDEOHWACCEL)
+    /** LUN\#0: VBVA callbacks interface */
+    PDMIDISPLAYVBVACALLBACKS    IVBVACallbacks;
+#else
+# if HC_ARCH_BITS == 32
+    uint32_t                    Padding3;
+# endif
+#endif
     /** Pointer to base interface of the driver. */
     R3PTRTYPE(PPDMIBASE)        pDrvBase;
     /** Pointer to display connector interface of the driver. */
@@ -340,9 +357,9 @@ typedef struct VGAState {
 # ifdef VBE_NEW_DYN_LIST
     /** VBE Extra Data write address one byte buffer */
     uint8_t                     cbWriteVBEExtraAddress;
-    uint8_t                     Padding6;       /**< Alignment padding. */
+    uint8_t                     Padding4;
 # else
-    uint8_t                     Padding6[2];    /**< Alignment padding. */
+    uint8_t                     Padding4[2];
 # endif
 #endif
 
@@ -353,7 +370,7 @@ typedef struct VGAState {
     uint16_t                    cbVBEExtraData;
     /** The VBE BIOS current memory address. */
     uint16_t                    u16VBEExtraAddress;
-    uint16_t                    Padding7[2];    /**< Alignment padding. */
+    uint16_t                    Padding5[2];
 #endif
     /** Current logo data offset. */
     uint32_t                    offLogoData;
@@ -383,10 +400,15 @@ typedef struct VGAState {
     uint16_t                    cLogoPalEntries;
     /** Clear screen flag. */
     uint8_t                     fLogoClearScreen;
-    uint8_t                     Padding8[7];    /**< Alignment padding. */
+    uint8_t                     Padding6[7];
     /** Palette data. */
     uint32_t                    au32LogoPalette[256];
 #endif /* VBOX */
+#ifdef VBOX_WITH_HGSMI
+    /** Base port in the assigned PCI I/O space. */
+    RTIOPORT                    IOPortBase;
+    uint8_t                     Padding7[6];
+#endif /* VBOX_WITH_HGSMI */
 } VGAState;
 #ifdef VBOX
 /** VGA state. */
@@ -442,6 +464,37 @@ static inline int c6_to_8(int v)
 int      VBVAInit       (PVGASTATE pVGAState);
 void     VBVADestroy    (PVGASTATE pVGAState);
 int      VBVAUpdateDisplay (PVGASTATE pVGAState);
+void     VBVAReset (PVGASTATE pVGAState);
+
+bool VBVAIsEnabled(PVGASTATE pVGAState);
+
+/* @return host-guest flags that were set on reset
+ * this allows the caller to make further cleaning when needed,
+ * e.g. reset the IRQ */
+uint32_t HGSMIReset (PHGSMIINSTANCE pIns);
+
+# ifdef VBOX_WITH_VIDEOHWACCEL
+int vbvaVHWACommandCompleteAsynch(PPDMIDISPLAYVBVACALLBACKS pInterface, PVBOXVHWACMD pCmd);
+int vbvaVHWAConstruct (PVGASTATE pVGAState);
+int vbvaVHWADisable (PVGASTATE pVGAState);
+int vbvaVHWAReset (PVGASTATE pVGAState);
+
+int vboxVBVASaveStatePrep (PPDMDEVINS pDevIns, PSSMHANDLE pSSM);
+# endif
+
+int vboxVBVASaveStateExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM);
+int vboxVBVALoadStateExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t u32Version);
+int vboxVBVALoadStateDone (PPDMDEVINS pDevIns, PSSMHANDLE pSSM);
+
+# ifdef VBOXVDMA
+typedef struct VBOXVDMAHOST *PVBOXVDMAHOST;
+int vboxVDMAConstruct(PVGASTATE pVGAState, struct VBOXVDMAHOST **ppVdma, uint32_t cPipeElements);
+int vboxVDMADestruct(PVBOXVDMAHOST pVdma);
+void vboxVDMAControl(PVBOXVDMAHOST pVdma, PVBOXVDMA_CTL pCmd);
+void vboxVDMACommand(PVBOXVDMAHOST pVdma, PVBOXVDMACBUF_DR pCmd);
+bool vboxVDMAIsEnabled(PVBOXVDMAHOST pVdma);
+# endif /* VBOXVDMA */
+
 #endif /* VBOX_WITH_HGSMI */
 
 #ifndef VBOX
