@@ -1,10 +1,10 @@
-/** $Id: sys_arch.c $ */
+/** $Id: sys_arch.c 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * System dependent parts of lwIP, implemented with IPRT.
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -152,7 +148,7 @@ void sys_sem_signal(sys_sem_t sem)
 u32_t sys_arch_sem_wait(sys_sem_t sem, u32_t timeout)
 {
     int rc;
-    unsigned cMillies;
+    RTMSINTERVAL cMillies;
     uint64_t tsStart, tsEnd;
 
     tsStart = RTTimeMilliTS();
@@ -265,7 +261,7 @@ void sys_mbox_post(sys_mbox_t mbox, void *msg)
 u32_t sys_arch_mbox_fetch(sys_mbox_t mbox, void **msg, u32_t timeout)
 {
     int rc;
-    unsigned cMillies;
+    RTMSINTERVAL cMillies;
     uint64_t tsStart, tsEnd;
 
     Assert(mbox != NULL);
