@@ -1,4 +1,24 @@
+/* $Id: debug.c 28800 2010-04-27 08:22:32Z vboxsync $ */
+/** @file
+ * NAT - debug helpers.
+ */
+
 /*
+ * Copyright (C) 2006-2010 Oracle Corporation
+ *
+ * This file is part of VirtualBox Open Source Edition (OSE), as
+ * available from http://www.virtualbox.org. This file is free software;
+ * you can redistribute it and/or modify it under the terms of the GNU
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ */
+
+
+/*
+ * This code is based on:
+ *
  * Copyright (c) 1995 Danny Gasparovski.
  * Portions copyright (c) 2000 Kelly Price.
  *
@@ -23,7 +43,7 @@ void dump_packet(void *, int);
 void
 dump_packet(void *dat, int n)
 {
-    Log(("nat: PACKET DUMPED:\n%.*Vhxd\n", n, dat));
+    Log(("nat: PACKET DUMPED:\n%.*Rhxd\n", n, dat));
 }
 #endif
 
@@ -158,6 +178,12 @@ icmpstats(PNATState pData)
 void
 mbufstats(PNATState pData)
 {
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
+    /*
+     * (vvl) this static code can't work with mbuf zone anymore
+     * @todo: make statistic correct
+     */
+#if 0
     struct mbuf *m;
     int i;
 
@@ -177,13 +203,15 @@ mbufstats(PNATState pData)
         i++;
     lprint("  %6d mbufs on used list\n",  i);
     lprint("  %6d mbufs queued as packets\n\n", if_queued);
+#endif
+#endif
 }
 
 void
 sockstats(PNATState pData)
 {
     char buff[256];
-    int n;
+    size_t n;
     struct socket *so, *so_next;
 
     lprint(" \n");
@@ -193,28 +221,28 @@ sockstats(PNATState pData)
 
     QSOCKET_FOREACH(so, so_next, tcp)
     /* { */
-        n = sprintf(buff, "tcp[%s]", so->so_tcpcb?tcpstates[so->so_tcpcb->t_state]:"NONE");
+        n = RTStrPrintf(buff, sizeof(buff), "tcp[%s]", so->so_tcpcb?tcpstates[so->so_tcpcb->t_state]:"NONE");
         while (n < 17)
             buff[n++] = ' ';
         buff[17] = 0;
         lprint("%s %3d   %15s %5d ",
-               buff, so->s, inet_ntoa(so->so_laddr), ntohs(so->so_lport));
+               buff, so->s, inet_ntoa(so->so_laddr), RT_N2H_U16(so->so_lport));
         lprint("%15s %5d %5d %5d\n",
-                inet_ntoa(so->so_faddr), ntohs(so->so_fport),
+                inet_ntoa(so->so_faddr), RT_N2H_U16(so->so_fport),
                 so->so_rcv.sb_cc, so->so_snd.sb_cc);
     LOOP_LABEL(tcp, so, so_next);
     }
 
     QSOCKET_FOREACH(so, so_next, udp)
     /* { */
-        n = sprintf(buff, "udp[%d sec]", (so->so_expire - curtime) / 1000);
+        n = RTStrPrintf(buff, sizeof(buff), "udp[%d sec]", (so->so_expire - curtime) / 1000);
         while (n < 17)
             buff[n++] = ' ';
         buff[17] = 0;
         lprint("%s %3d  %15s %5d  ",
-                buff, so->s, inet_ntoa(so->so_laddr), ntohs(so->so_lport));
+                buff, so->s, inet_ntoa(so->so_laddr), RT_N2H_U16(so->so_lport));
         lprint("%15s %5d %5d %5d\n",
-                inet_ntoa(so->so_faddr), ntohs(so->so_fport),
+                inet_ntoa(so->so_faddr), RT_N2H_U16(so->so_fport),
                 so->so_rcv.sb_cc, so->so_snd.sb_cc);
     LOOP_LABEL(udp, so, so_next);
     }
@@ -231,7 +259,7 @@ print_ipv4_address(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput,
 
     AssertReturn(strcmp(pszType, "IP4") == 0, 0);
 
-    ip = ntohl(*(uint32_t*)pvValue);
+    ip = RT_N2H_U32(*(uint32_t*)pvValue);
     return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, IP4_ADDR_PRINTF_FORMAT,
            IP4_ADDR_PRINTF_DECOMP(ip));
 }
@@ -246,9 +274,9 @@ print_ether_address(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput,
 
     AssertReturn(strcmp(pszType, "ether") == 0, 0);
     if (ether != NULL)
-        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, 
+        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
             "[ether %hhx:%hhx:%hhx:%hhx:%hhx:%hhx]",
-            ether[0], ether[1], ether[2], 
+            ether[0], ether[1], ether[2],
             ether[3], ether[4], ether[5]);
     else
         return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "[ether null]");
@@ -268,29 +296,29 @@ print_socket(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput,
     int status = 0;
 
     AssertReturn(strcmp(pszType, "natsock") == 0, 0);
-    if (so == NULL) 
-        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, 
+    if (so == NULL)
+        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
                 "socket is null");
-    if (so->so_state == SS_NOFDREF || so->s == -1) 
-        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, 
+    if (so->so_state == SS_NOFDREF || so->s == -1)
+        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
                 "socket(%d) SS_NODREF",so->s);
     status = getsockname(so->s, &addr, &socklen);
 
     if(status != 0 || addr.sa_family != AF_INET)
     {
-        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, 
+        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
                 "socket(%d) is invalid(probably closed)",so->s);
     }
 
     in_addr = (struct sockaddr_in *)&addr;
-    ip = ntohl(so->so_faddr.s_addr);
+    ip = RT_N2H_U32(so->so_faddr.s_addr);
     return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "socket %4d:(proto:%u) "
             "state=%04x ip=" IP4_ADDR_PRINTF_FORMAT ":%d "
             "name=" IP4_ADDR_PRINTF_FORMAT ":%d",
-            so->s, so->so_type, so->so_state, IP4_ADDR_PRINTF_DECOMP(ip), 
-            ntohs(so->so_fport), 
-            IP4_ADDR_PRINTF_DECOMP(ntohl(in_addr->sin_addr.s_addr)),
-            ntohs(in_addr->sin_port));
+            so->s, so->so_type, so->so_state, IP4_ADDR_PRINTF_DECOMP(ip),
+            RT_N2H_U16(so->so_fport),
+            IP4_ADDR_PRINTF_DECOMP(RT_N2H_U32(in_addr->sin_addr.s_addr)),
+            RT_N2H_U16(in_addr->sin_port));
 }
 
 static DECLCALLBACK(size_t)
@@ -306,7 +334,7 @@ print_networkevents(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput,
 
     AssertReturn(strcmp(pszType, "natwinnetevents") == 0, 0);
 
-    cb += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "events=%02x (", 
+    cb += RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "events=%02x (",
             pNetworkEvents->lNetworkEvents);
 # define DO_BIT(bit) \
     if (pNetworkEvents->lNetworkEvents & FD_ ## bit)                        \
@@ -351,7 +379,7 @@ debug_init()
     if (!g_fFormatRegistered)
     {
         /*
-         * XXX(r - frank): Move this to IPRT using RTNETADDRIPV4. 
+         * XXX(r - frank): Move this to IPRT using RTNETADDRIPV4.
          * Use the specifier %RNAipv4.
          */
         rc = RTStrFormatTypeRegister("IP4", print_ipv4_address, NULL);
@@ -360,7 +388,7 @@ debug_init()
         AssertRC(rc);
         rc = RTStrFormatTypeRegister("natsock", print_socket, NULL);
         AssertRC(rc);
-        rc = RTStrFormatTypeRegister("natwinnetevents", 
+        rc = RTStrFormatTypeRegister("natwinnetevents",
             print_networkevents, NULL);
         AssertRC(rc);
         g_fFormatRegistered = 1;
