@@ -1,4 +1,4 @@
-/* $Id: dnsproxy.c $ */
+/* $Id: dnsproxy.c 26495 2010-02-14 07:59:48Z vboxsync $ */
 /*
  * Copyright (c) 2003,2004,2005 Armin Wolfermann
  *
@@ -99,9 +99,9 @@ timeout(int fd, short event, void *arg)
     free((struct request *)arg);
     ++removed_queries;
 }
-#else
+#else /* VBOX */
 static void
-timeout(PNATState pData, struct socket *so, void *arg) 
+timeout(PNATState pData, struct socket *so, void *arg)
 {
     struct request *req = (struct request *)arg;
     struct dns_entry *de;
@@ -118,7 +118,7 @@ timeout(PNATState pData, struct socket *so, void *arg)
         struct ip *ip;
         struct udphdr *udp;
         int iphlen;
-        struct socket *so1 = socreate(); 
+        struct socket *so1 = socreate();
         struct mbuf *m = NULL;
         char *data;
         if (so1 == NULL)
@@ -132,7 +132,11 @@ timeout(PNATState pData, struct socket *so, void *arg)
             sofree(pData, so1);
             return;
         }
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
         m = m_get(pData);
+#else
+        m = m_getcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR);
+#endif
         if (m == NULL)
         {
             LogRel(("NAT: Can't allocate mbuf\n"));
@@ -141,7 +145,7 @@ timeout(PNATState pData, struct socket *so, void *arg)
         }
         /* mbuf initialization */
         m->m_data += if_maxlinkhdr;
-        ip = mtod(m, struct ip *); 
+        ip = mtod(m, struct ip *);
         udp = (struct udphdr *)&ip[1]; /* ip attributes */
         data = (char *)&udp[1];
         iphlen = sizeof(struct ip);
@@ -149,7 +153,7 @@ timeout(PNATState pData, struct socket *so, void *arg)
         m->m_len += sizeof(struct udphdr);
         m->m_len += req->nbyte;
         ip->ip_src.s_addr = so->so_laddr.s_addr;
-        ip->ip_dst.s_addr = htonl(ntohl(special_addr.s_addr) | CTL_DNS);
+        ip->ip_dst.s_addr = RT_H2N_U32(RT_N2H_U32(pData->special_addr.s_addr) | CTL_DNS);
         udp->uh_dport = ntohs(53);
         udp->uh_sport = so->so_lport;
         memcpy(data, req->byte, req->nbyte); /* coping initial req */
@@ -164,13 +168,13 @@ timeout(PNATState pData, struct socket *so, void *arg)
         dnsproxy_query(pData, so1, m, iphlen);
     }
 }
-#endif
+#endif /* VBOX */
 
 /* do_query -- Called by the event loop when a packet arrives at our
  * listening socket. Read the packet, create a new query, append it to the
  * queue and send it to the correct server.
  *
- * Slirp: this routine should be called from udp_input 
+ * Slirp: this routine should be called from udp_input
  * socket is Slirp's construction (here we should set expiration time for socket)
  * mbuf points on ip header to easy fetch information about source and destination.
  * iphlen - len of ip header
@@ -197,7 +201,11 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
 #endif
     struct sockaddr_in addr;
     struct request *req = NULL;
+#ifndef VBOX
     struct sockaddr_in fromaddr;
+#else
+    struct sockaddr_in fromaddr = { 0, };
+#endif
     int byte = 0;
 
     ++all_queries;
@@ -259,30 +267,30 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
     }
 
     /* fill the request structure */
-    if (so->so_timeout_arg == NULL) 
+    if (so->so_timeout_arg == NULL)
     {
         req->id = QUERYID;
         memcpy(&req->client, &fromaddr, sizeof(struct sockaddr_in));
         memcpy(&req->clientid, &buf[0], 2);
-        req->dns_server = TAILQ_LAST(&pData->dns_list_head, dns_list_head);
+        req->dns_server = TAILQ_LAST(&pData->pDnsList, dns_list_head);
         if (req->dns_server == NULL)
         {
             static int fail_counter = 0;
             RTMemFree(req);
             if (fail_counter == 0)
                 LogRel(("NAT/dnsproxy: Empty DNS entry (suppressed 100 times)\n"));
-            else 
-                fail_counter = (fail_counter == 100 ? 0 : fail_counter++);
+            else
+                fail_counter = (fail_counter == 100 ? 0 : fail_counter + 1);
             return;
-            
+
         }
         retransmit = 0;
         so->so_timeout = timeout;
         so->so_timeout_arg = req;
         req->nbyte = byte;
         memcpy(req->byte, buf, byte); /* copying original request */
-    } 
-    else 
+    }
+    else
     {
         retransmit = 1;
     }
@@ -374,7 +382,7 @@ dnsproxy_query(PNATState pData, struct socket *so, struct mbuf *m, int iphlen)
 /* do_answer -- Process a packet coming from our authoritative or recursive
  * server. Find the corresponding query and send answer back to querying
  * host.
- * 
+ *
  * Slirp: we call this from the routine from socrecvfrom routine handling UDP responses.
  * So at the moment of call response already has been readed and packed into the mbuf
  */
@@ -421,7 +429,7 @@ dnsproxy_answer(PNATState pData, struct socket *so, struct mbuf *m)
 #ifdef VBOX
     if ((query = hash_find_request(pData, *((unsigned short *)buf))) == NULL) {
         ++late_answers;
-        /* Probably, this request wasn't serviced by 
+        /* Probably, this request wasn't serviced by
          * dnsproxy so we won't care about it here*/
         so->so_expire = curtime + SO_EXPIREFAST;
         Log2(("NAT: query wasn't found\n"));
@@ -623,7 +631,7 @@ main(int argc, char *argv[])
 
 }
 #else
-int 
+int
 dnsproxy_init(PNATState pData)
 {
     /* globals initialization */
