@@ -1,4 +1,23 @@
+/* $Id: ip_icmp.c 28800 2010-04-27 08:22:32Z vboxsync $ */
+/** @file
+ * NAT - IP/ICMP handling.
+ */
+
 /*
+ * Copyright (C) 2006-2010 Oracle Corporation
+ *
+ * This file is part of VirtualBox Open Source Edition (OSE), as
+ * available from http://www.virtualbox.org. This file is free software;
+ * you can redistribute it and/or modify it under the terms of the GNU
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ */
+
+/*
+ * This code is based on:
+ *
  * Copyright (c) 1982, 1986, 1988, 1993
  *      The Regents of the University of California.  All rights reserved.
  *
@@ -89,7 +108,7 @@ icmp_init(PNATState pData)
     if (pData->icmp_socket.s == -1)
     {
         int rc = RTErrConvertFromErrno(errno);
-        LogRel(("NAT: ICMP/ping not available (could open ICMP socket, error %Rrc)\n", rc));
+        LogRel(("NAT: ICMP/ping not available (could not open ICMP socket, error %Rrc)\n", rc));
         return 1;
     }
     fd_nonblock(pData->icmp_socket.s);
@@ -104,7 +123,7 @@ icmp_init(PNATState pData)
                                     GetProcAddress(pData->hmIcmpLibrary, "IcmpCloseHandle");
         pData->pfGetAdaptersAddresses = (ULONG (WINAPI *)(ULONG, ULONG, PVOID, PIP_ADAPTER_ADDRESSES, PULONG))
                                     GetProcAddress(pData->hmIcmpLibrary, "GetAdaptersAddresses");
-        if (pData->pfGetAdaptersAddresses == NULL) 
+        if (pData->pfGetAdaptersAddresses == NULL)
         {
             LogRel(("NAT: Can't find GetAdapterAddresses in Iphlpapi.dll"));
         }
@@ -112,7 +131,7 @@ icmp_init(PNATState pData)
 
     if (pData->pfIcmpParseReplies == NULL)
     {
-        if(pData->pfGetAdaptersAddresses == NULL) 
+        if(pData->pfGetAdaptersAddresses == NULL)
             FreeLibrary(pData->hmIcmpLibrary);
         pData->hmIcmpLibrary = LoadLibrary("Icmp.dll");
         if (pData->hmIcmpLibrary == NULL)
@@ -181,8 +200,24 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
             {
                 m0 = icm->im_m;
                 ip0 = mtod(m0, struct ip *);
-                AssertRelease(ip0->ip_p == IPPROTO_ICMP);
+                if (ip0->ip_p != IPPROTO_ICMP)
+                {
+                    /* try next item */
+                    continue;
+                }
                 icp0 = (struct icmp *)((char *)ip0 + (ip0->ip_hl << 2));
+                /*
+                 * IP could pointer to ICMP_REPLY datagram (1)
+                 * or pointer IP header in ICMP payload in case of
+                 * ICMP_TIMXCEED or ICMP_UNREACH (2)
+                 *
+                 * if (1) and then ICMP (type should be ICMP_ECHOREPLY) and we need check that
+                 * IP.IP_SRC == IP0.IP_DST received datagramm comes from destination.
+                 *
+                 * if (2) then check that payload ICMP has got type ICMP_ECHO and
+                 * IP.IP_DST == IP0.IP_DST destination of returned datagram is the same as
+                 * one was sent.
+                 */
                 if (  (   (icp->icmp_type != ICMP_ECHO && ip->ip_src.s_addr == ip0->ip_dst.s_addr)
                        || (icp->icmp_type == ICMP_ECHO && ip->ip_dst.s_addr == ip0->ip_dst.s_addr))
                     && icp->icmp_id == icp0->icmp_id
@@ -250,7 +285,7 @@ icmp_find_original_mbuf(PNATState pData, struct ip *ip)
     sofound:
     if (found == 1 && icm == NULL)
     {
-        if (so->so_state == SS_NOFDREF) 
+        if (so->so_state == SS_NOFDREF)
         {
             /* socket is shutdowning we've already sent ICMP on it.*/
             LogRel(("NAT: Received icmp on shutdowning socket (probably corresponding ICMP socket has been already sent)\n"));
@@ -325,7 +360,20 @@ freeit:
 
     m->m_len -= hlen;
     m->m_data += hlen;
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
     icp = mtod(m, struct icmp *);
+#else
+    if (m->m_next != NULL)
+    {
+        char *buf = RTMemAlloc(icmplen);
+        m_copydata(m, 0, icmplen, buf);
+        icp = (struct icmp *)buf;
+    }
+    else
+    {
+        icp = mtod(m, struct icmp *);
+    }
+#endif
     if (cksum(m, icmplen))
     {
         icmpstat.icps_checksum++;
@@ -358,10 +406,10 @@ freeit:
                 int error;
 #endif
                 addr.sin_family = AF_INET;
-                if ((ip->ip_dst.s_addr & htonl(pData->netmask)) == special_addr.s_addr)
+                if ((ip->ip_dst.s_addr & RT_H2N_U32(pData->netmask)) == pData->special_addr.s_addr)
                 {
                     /* It's an alias */
-                    switch (ntohl(ip->ip_dst.s_addr) & ~pData->netmask)
+                    switch (RT_N2H_U32(ip->ip_dst.s_addr) & ~pData->netmask)
                     {
                         case CTL_DNS:
                         case CTL_ALIAS:
@@ -375,6 +423,7 @@ freeit:
 #ifndef RT_OS_WINDOWS
                 if (pData->icmp_socket.s != -1)
                 {
+                    ssize_t rc;
                     m->m_so = &pData->icmp_socket;
                     icmp_attach(pData, m);
                     ttl = ip->ip_ttl;
@@ -384,10 +433,11 @@ freeit:
                     if (status < 0)
                         LogRel(("NAT: Error (%s) occurred while setting TTL attribute of IP packet\n",
                                 strerror(errno)));
-                    if (sendto(pData->icmp_socket.s, icp, icmplen, 0,
-                              (struct sockaddr *)&addr, sizeof(addr)) == -1)
+                    rc = sendto(pData->icmp_socket.s, icp, icmplen, 0,
+                              (struct sockaddr *)&addr, sizeof(addr));
+                    if (rc < 0)
                     {
-                        Log((dfd,"icmp_input udp sendto tx errno = %d-%s\n",
+                        LogRel((dfd,"icmp_input udp sendto tx errno = %d-%s\n",
                                     errno, strerror(errno)));
                         icmp_error(pData, m, ICMP_UNREACH, ICMP_UNREACH_NET, 0, strerror(errno));
                     }
@@ -454,6 +504,10 @@ freeit:
             icmpstat.icps_badtype++;
             m_freem(pData, m);
     } /* switch */
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+    if (m->m_next != NULL && icp != NULL)
+        RTMemFree(icp);
+#endif
 
 end_error:
     /* m is m_free()'d xor put in a socket xor or given to ip_send */
@@ -486,12 +540,18 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     register struct ip *ip;
     register struct icmp *icp;
     register struct mbuf *m;
+    int new_m_size = 0;
+    int size = 0;
 
     DEBUG_CALL("icmp_error");
     DEBUG_ARG("msrc = %lx", (long )msrc);
     DEBUG_ARG("msrc_len = %d", msrc ? msrc->m_len : 0);
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+    if (msrc != NULL)
+        M_ASSERTPKTHDR(msrc);
+#endif
 
-    if (type!=ICMP_UNREACH && type!=ICMP_TIMXCEED)
+    if (type!=ICMP_UNREACH && type!=ICMP_TIMXCEED && type != ICMP_SOURCEQUENCH)
         goto end_error;
 
     /* check msrc */
@@ -507,7 +567,7 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
         DEBUG_MISC((dfd, " %.16s to %.16s\n", bufa, bufb));
     }
 #endif
-    if (ip->ip_off & IP_OFFMASK)
+    if (ip->ip_off & IP_OFFMASK && type != ICMP_SOURCEQUENCH)
         goto end_error;    /* Only reply to fragment 0 */
 
     shlen = ip->ip_hl << 2;
@@ -523,16 +583,48 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
             goto end_error;
     }
 
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
     /* make a copy */
-    if (!(m = m_get(pData)))
-        goto end_error;                    /* get mbuf */
+    m = m_get(pData);
+#else
+    new_m_size = sizeof(struct ip) + ICMP_MINLEN + msrc->m_len + ICMP_MAXDATALEN;
+    if (new_m_size < MSIZE)
     {
-        int new_m_size;
-        m->m_data += if_maxlinkhdr;
-        new_m_size = sizeof(struct ip) + ICMP_MINLEN + msrc->m_len + ICMP_MAXDATALEN;
-        if (new_m_size > m->m_size)
-            m_inc(m, new_m_size);
+        size = MCLBYTES;
     }
+    else if (new_m_size < MCLBYTES)
+    {
+        size = MCLBYTES;
+    }
+    else if(new_m_size < MJUM9BYTES)
+    {
+        size = MJUM9BYTES;
+    }
+    else if (new_m_size < MJUM16BYTES)
+    {
+        size = MJUM16BYTES;
+    }
+    else
+    {
+        AssertMsgFailed(("Unsupported size"));
+    }
+    m = m_getjcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR, size);
+#endif
+    if (m == NULL)
+        goto end_error;                    /* get mbuf */
+
+    m->m_data += if_maxlinkhdr;
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+    m->m_pkthdr.header = mtod(m, void *);
+#else
+    new_m_size = if_maxlinkhdr + sizeof(struct ip) + ICMP_MINLEN + msrc->m_len + ICMP_MAXDATALEN;
+    if (new_m_size > m->m_size)
+    {
+        m_inc(m, new_m_size);
+        if (new_m_size > m->m_size)
+            goto end_error_free_m;
+    }
+#endif
 
     memcpy(m->m_data, msrc->m_data, msrc->m_len);
     m->m_len = msrc->m_len;                /* copy msrc to m */
@@ -562,6 +654,7 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     icp->icmp_seq = 0;
 
     memcpy(&icp->icmp_ip, msrc->m_data, s_ip_len);   /* report the ip packet */
+
     HTONS(icp->icmp_ip.ip_len);
     HTONS(icp->icmp_ip.ip_id);
     HTONS(icp->icmp_ip.ip_off);
@@ -576,16 +669,22 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
         if (message_len > ICMP_MAXDATALEN)
             message_len = ICMP_MAXDATALEN;
         cpnt = (char *)m->m_data+m->m_len;
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
         memcpy(cpnt, message, message_len);
         m->m_len += message_len;
+#else
+        m_append(pData, m, message_len, message);
+#endif
     }
 #endif
 
     icp->icmp_cksum = 0;
     icp->icmp_cksum = cksum(m, m->m_len);
 
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
     m->m_data -= hlen;
     m->m_len += hlen;
+#endif
 
     /* fill in ip */
     ip->ip_hl = hlen >> 2;
@@ -598,12 +697,14 @@ void icmp_error(PNATState pData, struct mbuf *msrc, u_char type, u_char code, in
     ip->ip_dst = ip->ip_src;    /* ip adresses */
     ip->ip_src = alias_addr;
 
-    (void ) ip_output(pData, (struct socket *)NULL, m);
+    (void ) ip_output0(pData, (struct socket *)NULL, m, 1);
 
     icmpstat.icps_reflect++;
 
     return;
 
+end_error_free_m:
+    m_freem(pData, m);
 end_error:
     LogRel(("NAT: error occurred while sending ICMP error message \n"));
 }
