@@ -1,9 +1,9 @@
 /** @file
- * NAT state/configuration.
+ * NAT - slirp state/configuration.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -12,21 +12,20 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___slirp_state_h
 #define ___slirp_state_h
 
 #include <iprt/req.h>
+#include <iprt/critsect.h>
+
+#define COUNTERS_INIT
+#include "counters.h"
+
 #include "ip_icmp.h"
 #include "dnsproxy/dnsproxy.h"
 
-/** Number of DHCP clients supported by NAT. */
-#define NB_ADDR     16
 
 /** Where to start DHCP IP number allocation. */
 #define START_ADDR  15
@@ -34,17 +33,6 @@
 /** DHCP Lease time. */
 #define LEASE_TIME (24 * 3600)
 
-/** Entry in the table of known DHCP clients. */
-typedef struct
-{
-    bool allocated;
-    uint8_t macaddr[6];
-#ifdef VBOX_WITHOUT_SLIRP_CLIENT_ETHER
-    struct in_addr addr;
-#endif
-} BOOTPClient;
-
-
 /*
  * ARP cache this is naive implementaion of ARP
  * cache of mapping 4 byte IPv4 address to 6 byte
@@ -83,6 +71,34 @@ struct dns_entry
     TAILQ_ENTRY(dns_entry) de_list;
 };
 TAILQ_HEAD(dns_list_head, dns_entry);
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+TAILQ_HEAD(if_queue, mbuf);
+#endif
+
+struct port_forward_rule
+{
+    uint16_t proto;
+    uint16_t host_port;
+    uint16_t guest_port;
+#ifndef VBOX_WITH_NAT_SERVICE
+    struct in_addr guest_addr;
+#endif
+    struct in_addr bind_ip;
+    uint8_t mac_address[6]; /*need ETH_ALEN here */
+    int activated;
+    LIST_ENTRY(port_forward_rule) list;
+};
+LIST_HEAD(port_forward_rule_list, port_forward_rule);
+
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
+struct mbuf_zone
+{
+    LIST_ENTRY(mbuf_zone) list;
+    uint8_t *mbuf_zone_base_addr;
+};
+LIST_HEAD(mbuf_zone_list, mbuf_zone);
+#endif
+
 
 /* forward declaration */
 struct proto_handler;
@@ -90,8 +106,11 @@ struct proto_handler;
 /** Main state/configuration structure for slirp NAT. */
 typedef struct NATState
 {
+#define PROFILE_COUNTER(name, dsc)     STAMPROFILE Stat ## name
+#define COUNTING_COUNTER(name, dsc)    STAMCOUNTER Stat ## name
+#include "counters.h"
     /* Stuff from boot.c */
-    BOOTPClient bootp_clients[NB_ADDR];
+    void *pbootp_clients;
     const char *bootp_filename;
     /* Stuff from if.c */
     int if_mtu, if_mru;
@@ -99,8 +118,13 @@ typedef struct NATState
     int if_maxlinkhdr;
     int if_queued;
     int if_thresh;
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
     struct mbuf if_fastq;
     struct mbuf if_batchq;
+#else
+    struct if_queue if_fastq;
+    struct if_queue if_batchq;
+#endif
     struct mbuf *next_m;
     /* Stuff from icmp.c */
     struct icmpstat_t icmpstat;
@@ -115,6 +139,15 @@ typedef struct NATState
     int mbuf_alloced, mbuf_max;
     int msize;
     struct mbuf m_freelist, m_usedlist;
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
+    struct mbuf_zone_list mbuf_zone_head;
+    RTCRITSECT cs_mbuf_zone;
+    int fmbuf_water_line;
+    int mbuf_water_line_limit;
+    int mbuf_zone_count;
+    int fmbuf_water_warn_sent;
+    uint32_t tsmbuf_water_warn_sent;
+#endif
     /* Stuff from slirp.c */
     void *pvUser;
     uint32_t curtime;
@@ -137,17 +170,15 @@ typedef struct NATState
 #ifdef RT_OS_WINDOWS
     ULONG (WINAPI * pfGetAdaptersAddresses)(ULONG, ULONG, PVOID, PIP_ADAPTER_ADDRESSES, PULONG);
 #endif
-    struct dns_list_head dns_list_head;
-    struct dns_domain_list_head dns_domain_list_head;
+    struct dns_list_head pDnsList;
+    struct dns_domain_list_head pDomainList;
     struct in_addr tftp_server;
     struct in_addr loopback_addr;
     uint32_t netmask;
-#ifndef VBOX_WITHOUT_SLIRP_CLIENT_ETHER
+#ifndef VBOX_WITH_NAT_SERVICE
     uint8_t client_ethaddr[6];
-#else
-    const uint8_t *slirp_ethaddr;
 #endif
-    struct ex_list *exec_list;
+    const uint8_t *slirp_ethaddr;
     char slirp_hostname[33];
     bool fPassDomain;
     struct in_addr bindIP;
@@ -209,7 +240,42 @@ typedef struct NATState
 # define VBOX_SOCKET_EVENT (pData->phEvents[VBOX_SOCKET_EVENT_INDEX])
     HANDLE phEvents[VBOX_EVENT_COUNT];
 #endif
-
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+# ifdef zone_mbuf
+#  undef zone_mbuf
+# endif
+    uma_zone_t zone_mbuf;
+# ifdef zone_clust
+#  undef zone_clust
+# endif
+    uma_zone_t zone_clust;
+# ifdef zone_pack
+#  undef zone_pack
+# endif
+    uma_zone_t zone_pack;
+# ifdef zone_jumbop
+#  undef zone_jumbop
+# endif
+    uma_zone_t zone_jumbop;
+# ifdef zone_jumbo9
+#  undef zone_jumbo9
+# endif
+    uma_zone_t zone_jumbo9;
+# ifdef zone_jumbo16
+#  undef zone_jumbo16
+# endif
+    uma_zone_t zone_jumbo16;
+# ifdef zone_ext_refcnt
+#  undef zone_ext_refcnt
+    int nmbclusters;                    /* limits number of mbuf clusters */
+    int nmbjumbop;                      /* limits number of page size jumbo clusters */
+    int nmbjumbo9;                      /* limits number of 9k jumbo clusters */
+    int nmbjumbo16;                     /* limits number of 16k jumbo clusters */
+    struct mbstat mbstat;
+# endif
+    uma_zone_t zone_ext_refcnt;
+#endif
+    bool fUseHostResolver;
     /* from dnsproxy/dnsproxy.h*/
     unsigned int authoritative_port;
     unsigned int authoritative_timeout;
@@ -239,25 +305,21 @@ typedef struct NATState
 #define HASH(id) (id & ((1 << HASHSIZE) - 1))
     struct request *request_hash[1 << HASHSIZE];
     /* this field control behaviour of DHCP server */
-    bool use_dns_proxy;
+    bool fUseDnsProxy;
 
-    struct arp_cache_head arp_cache;
-#ifdef VBOX_WITH_SLIRP_ALIAS
     LIST_HEAD(RT_NOTHING, libalias) instancehead;
+    int    i32AliasMode;
     struct libalias *proxy_alias;
+    struct libalias *dns_alias;
     LIST_HEAD(handler_chain, proto_handler) handler_chain;
+    struct port_forward_rule_list port_forward_rule_head;
+    int cRedirectionsActive;
+    int cRedirectionsStored;
+    struct arp_cache_head arp_cache;
     /*libalis modules' handlers*/
     struct proto_handler *ftp_module;
     struct proto_handler *nbt_module;
-#endif
-
-#define PROFILE_COUNTER(name, dsc)     STAMPROFILE Stat ## name
-#define COUNTING_COUNTER(name, dsc)    STAMCOUNTER Stat ## name
-
-#include "counters.h"
-
-#undef PROFILE_COUNTER
-#undef COUNTING_COUNTER
+    struct proto_handler *dns_module;
 
 } NATState;
 
@@ -301,7 +363,6 @@ typedef struct NATState
 
 
 #define bootp_filename pData->bootp_filename
-#define bootp_clients pData->bootp_clients
 
 #define if_mtu pData->if_mtu
 #define if_mru pData->if_mru
@@ -338,11 +399,9 @@ typedef struct NATState
 #else
 # define handler_chain pData->handler_chain
 #endif
-#define special_addr pData->special_addr
 #define dns_addr pData->dns_addr
 #define loopback_addr pData->loopback_addr
 #define client_ethaddr pData->client_ethaddr
-#define exec_list pData->exec_list
 #define slirp_hostname pData->slirp_hostname
 
 #define tcb pData->tcb
@@ -372,8 +431,6 @@ typedef struct NATState
 #define queue_udp_label udb
 #define VBOX_X2(x) x
 #define VBOX_X(x) VBOX_X2(x)
-#define VBOX_STR2(x) #x
-#define VBOX_STR(x) VBOX_STR2(x)
 
 #ifdef VBOX_WITH_SLIRP_MT
 
@@ -382,24 +439,24 @@ typedef struct NATState
         int rc;                                                       \
         /* Assert(strcmp(RTThreadSelfName(), "EMT") != 0); */         \
         rc = RTCritSectEnter(&VBOX_X(queue) ## _mutex);               \
-        AssertReleaseRC(rc);                                          \
+        AssertRC(rc);                                                 \
     } while (0)
 # define QSOCKET_UNLOCK(queue)                                        \
     do {                                                              \
         int rc;                                                       \
         rc = RTCritSectLeave(&VBOX_X(queue) ## _mutex);               \
-        AssertReleaseRC(rc);                                          \
+        AssertRC(rc);                                                 \
     } while (0)
 # define QSOCKET_LOCK_CREATE(queue)                                   \
     do {                                                              \
         int rc;                                                       \
         rc = RTCritSectInit(&pData->queue ## _mutex);                 \
-        AssertReleaseRC(rc);                                          \
+        AssertRC(rc);                                                 \
     } while (0)
 # define QSOCKET_LOCK_DESTROY(queue)                                  \
     do {                                                              \
         int rc = RTCritSectDelete(&pData->queue ## _mutex);           \
-        AssertReleaseRC(rc);                                          \
+        AssertRC(rc);                                                 \
     } while (0)
 
 # define QSOCKET_FOREACH(so, sonext, label)                           \
@@ -441,16 +498,10 @@ typedef struct NATState
 
 # define DO_TCP_OUTPUT(data, sotcb)                                     \
     do {                                                                \
-        PRTREQ pReq = NULL;                                             \
+        PRTREQ pReq;                                                    \
         int rc;                                                         \
-        rc = RTReqAlloc((data)->pReqQueue, &pReq, RTREQTYPE_INTERNAL);  \
-        AssertReleaseRC(rc);                                            \
-        pReq->u.Internal.pfn      = (PFNRT)tcp_output;                  \
-        pReq->u.Internal.cArgs    = 2;                                  \
-        pReq->u.Internal.aArgs[0] = (uintptr_t)(data);                  \
-        pReq->u.Internal.aArgs[1] = (uintptr_t)(sotcb);                 \
-        pReq->fFlags              = RTREQFLAGS_VOID;                    \
-        rc = RTReqQueue(pReq, 0);                                       \
+        rc = RTReqCallVoid((data)->pReqQueue, &pReq, 0 /*cMillies*/,    \
+                           (PFNRT)tcp_output 2, data, sotcb);           \
         if (RT_LIKELY(rc) == VERR_TIMEOUT)                              \
         {                                                               \
             SOCKET_UNLOCK(so);                                          \
@@ -465,33 +516,19 @@ typedef struct NATState
 
 # define DO_TCP_INPUT(data, mbuf, size, so)                             \
     do {                                                                \
-        PRTREQ pReq = NULL;                                             \
         int rc;                                                         \
-        rc = RTReqAlloc((data)->pReqQueue, &pReq, RTREQTYPE_INTERNAL);  \
-        AssertReleaseRC(rc);                                            \
-        pReq->u.Internal.pfn      = (PFNRT)tcp_input;                   \
-        pReq->u.Internal.cArgs    = 4;                                  \
-        pReq->u.Internal.aArgs[0] = (uintptr_t)(data);                  \
-        pReq->u.Internal.aArgs[1] = (uintptr_t)(mbuf);                  \
-        pReq->u.Internal.aArgs[2] = (uintptr_t)(size);                  \
-        pReq->u.Internal.aArgs[3] = (uintptr_t)(so);                    \
-        pReq->fFlags              = RTREQFLAGS_VOID|RTREQFLAGS_NO_WAIT; \
-        rc = RTReqQueue(pReq, 0);                                       \
+        rc = RTReqCallEx((data)->pReqQueue, NULL, 0 /*cMillies*/,       \
+                         RTREQFLAGS_VOID | RTREQFLAGS_NO_WAIT,          \
+                         (PFNRT)tcp_input, 4, data, mbuf, size, so);    \
         AssertReleaseRC(rc);                                            \
     } while(0)
 
 # define DO_TCP_CONNECT(data, so)                                       \
     do {                                                                \
-        PRTREQ pReq = NULL;                                             \
+        PRTREQ pReq;                                                    \
         int rc;                                                         \
-        rc = RTReqAlloc((data)->pReqQueue, &pReq, RTREQTYPE_INTERNAL);  \
-        AssertReleaseRC(rc);                                            \
-        pReq->u.Internal.pfn      = (PFNRT)tcp_connect;                 \
-        pReq->u.Internal.cArgs    = 2;                                  \
-        pReq->u.Internal.aArgs[0] = (uintptr_t)(data);                  \
-        pReq->u.Internal.aArgs[1] = (uintptr_t)(so);                    \
-        pReq->fFlags              = RTREQFLAGS_VOID;                    \
-        rc = RTReqQueue(pReq, 0); /* don't wait, we have to release lock before*/ \
+        rc = RTReqCallVoid((data)->pReqQueue, &pReq, 0 /*cMillies*/,    \
+                           (PFNRT)tcp_connect, 2, data, so);            \
         if (RT_LIKELY(rc) == VERR_TIMEOUT)                              \
         {                                                               \
             SOCKET_UNLOCK(so);                                          \
@@ -506,18 +543,11 @@ typedef struct NATState
 
 # define DO_SOREAD(ret, data, so, ifclose)                              \
     do {                                                                \
-        PRTREQ pReq = NULL;                                             \
+        PRTREQ pReq;                                                    \
         int rc;                                                         \
-        rc = RTReqAlloc((data)->pReqQueue, &pReq, RTREQTYPE_INTERNAL);  \
-        AssertReleaseRC(rc);                                            \
-        pReq->u.Internal.pfn      = (PFNRT)soread_queue;                \
-        pReq->u.Internal.cArgs    = 4;                                  \
-        pReq->u.Internal.aArgs[0] = (uintptr_t)(data);                  \
-        pReq->u.Internal.aArgs[1] = (uintptr_t)(so);                    \
-        pReq->u.Internal.aArgs[2] = (uintptr_t)(ifclose);               \
-        pReq->u.Internal.aArgs[3] = (uintptr_t)&(ret);                  \
-        pReq->fFlags              = RTREQFLAGS_VOID;                    \
-        rc = RTReqQueue(pReq, 0); /* don't wait, we have to release lock before*/ \
+        rc = RTReqCallVoid((data)->pReqQueue, &pReq, 0 /*cMillies*/,    \
+                           (PFNRT)soread_queue, 4,                      \
+                           data, so, ifclose, &(ret));                  \
         if (RT_LIKELY(rc) == VERR_TIMEOUT)                              \
         {                                                               \
             SOCKET_UNLOCK(so);                                          \
@@ -532,16 +562,10 @@ typedef struct NATState
 
 # define DO_SOWRITE(ret, data, so)                                      \
     do {                                                                \
-        PRTREQ pReq = NULL;                                             \
+        PRTREQ pReq;                                                    \
         int rc;                                                         \
-        rc = RTReqAlloc((data)->pReqQueue, &pReq, RTREQTYPE_INTERNAL);  \
-        AssertReleaseRC(rc);                                            \
-        pReq->u.Internal.pfn      = (PFNRT)sowrite;                     \
-        pReq->u.Internal.cArgs    = 2;                                  \
-        pReq->u.Internal.aArgs[0] = (uintptr_t)(data);                  \
-        pReq->u.Internal.aArgs[1] = (uintptr_t)(so);                    \
-        pReq->fFlags              = RTREQFLAGS_RETURN_MASK;             \
-        rc = RTReqQueue(pReq, 0); /* don't wait, we have to release lock before*/ \
+        rc = RTReqCall((data)->pReqQueue, &pReq, 0 /*cMillies*/,        \
+                       (PFNRT)sowrite, 2, data, so);                    \
         if (RT_LIKELY(rc) == VERR_TIMEOUT)                              \
         {                                                               \
             SOCKET_UNLOCK(so);                                          \
@@ -556,16 +580,10 @@ typedef struct NATState
 
 # define DO_SORECFROM(data, so)                                         \
     do {                                                                \
-        PRTREQ pReq = NULL;                                             \
+        PRTREQ pReq;                                                    \
         int rc;                                                         \
-        rc = RTReqAlloc((data)->pReqQueue, &pReq, RTREQTYPE_INTERNAL);  \
-        AssertReleaseRC(rc);                                            \
-        pReq->u.Internal.pfn      = (PFNRT)sorecvfrom;                  \
-        pReq->u.Internal.cArgs    = 2;                                  \
-        pReq->u.Internal.aArgs[0] = (uintptr_t)(data);                  \
-        pReq->u.Internal.aArgs[1] = (uintptr_t)(so);                    \
-        pReq->fFlags              = RTREQFLAGS_VOID;                    \
-        rc = RTReqQueue(pReq, 0);                                       \
+        rc = RTReqCallVoid((data)->pReqQueue, &pReq, 0 /*cMillies */,   \
+                           (PFNRT)sorecvfrom, 2, data, so);             \
         if (RT_LIKELY(rc) == VERR_TIMEOUT)                              \
         {                                                               \
             SOCKET_UNLOCK(so);                                          \
@@ -580,16 +598,10 @@ typedef struct NATState
 
 # define DO_UDP_DETACH(data, so, so_next)                               \
     do {                                                                \
-        PRTREQ pReq = NULL;                                             \
+        PRTREQ pReq;                                                    \
         int rc;                                                         \
-        rc = RTReqAlloc((data)->pReqQueue, &pReq, RTREQTYPE_INTERNAL);  \
-        AssertReleaseRC(rc);                                            \
-        pReq->u.Internal.pfn      = (PFNRT)udp_detach;                  \
-        pReq->u.Internal.cArgs    = 2;                                  \
-        pReq->u.Internal.aArgs[0] = (uintptr_t)(data);                  \
-        pReq->u.Internal.aArgs[1] = (uintptr_t)(so);                    \
-        pReq->fFlags              = RTREQFLAGS_VOID;                    \
-        rc = RTReqQueue(pReq, 0); /* don't wait, we have to release lock before*/ \
+        rc = RTReqCallVoid((data)->pReqQueue, &pReq, 0 /* cMillies*/,   \
+                           (PFNRT)udp_detach, 2, data, so);             \
         if (RT_LIKELY(rc) == VERR_TIMEOUT)                              \
         {                                                               \
             SOCKET_UNLOCK(so);                                          \
@@ -629,11 +641,12 @@ typedef struct NATState
 # define QSOCKET_LOCK_CREATE(queue) do {} while (0)
 # define QSOCKET_LOCK_DESTROY(queue) do {} while (0)
 # define QSOCKET_FOREACH(so, sonext, label)                              \
-    for ((so)  = VBOX_X2(queue_ ## label ## _label).so_next;                 \
-         (so) != &(VBOX_X2(queue_ ## label ## _label));                      \
+    for ((so)  = VBOX_X2(queue_ ## label ## _label).so_next;             \
+         (so) != &(VBOX_X2(queue_ ## label ## _label));                  \
          (so) = (sonext))                                                \
     {                                                                    \
-        (sonext) = (so)->so_next;
+        (sonext) = (so)->so_next;                                        \
+         Log2(("%s:%d Processing so:%R[natsock]\n", __FUNCTION__, __LINE__, (so)));
 # define CONTINUE(label) continue
 # define CONTINUE_NO_UNLOCK(label) continue
 # define LOOP_LABEL(label, so, sonext) /* empty*/
@@ -692,8 +705,53 @@ typedef struct NATState
 #define sock_query pData->sock_query
 #define sock_answer pData->sock_answer
 
-#ifdef VBOX_WITH_SLIRP_ALIAS
-# define instancehead pData->instancehead
+#define instancehead pData->instancehead
+
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+# define nmbclusters    pData->nmbclusters
+# define nmbjumbop  pData->nmbjumbop
+# define nmbjumbo9  pData->nmbjumbo9
+# define nmbjumbo16 pData->nmbjumbo16
+# define mbstat pData->mbstat
+# include "ext.h"
+# undef zone_mbuf
+# undef zone_clust
+# undef zone_pack
+# undef zone_jumbop
+# undef zone_jumbo9
+# undef zone_jumbo16
+# undef zone_ext_refcnt
+static inline uma_zone_t slirp_zone_pack(PNATState pData)
+{
+    return pData->zone_pack;
+}
+static inline uma_zone_t slirp_zone_jumbop(PNATState pData)
+{
+    return pData->zone_jumbop;
+}
+static inline uma_zone_t slirp_zone_jumbo9(PNATState pData)
+{
+    return pData->zone_jumbo9;
+}
+static inline uma_zone_t slirp_zone_jumbo16(PNATState pData)
+{
+    return pData->zone_jumbo16;
+}
+static inline uma_zone_t slirp_zone_ext_refcnt(PNATState pData)
+{
+    return pData->zone_ext_refcnt;
+}
+static inline uma_zone_t slirp_zone_mbuf(PNATState pData)
+{
+    return pData->zone_mbuf;
+}
+static inline uma_zone_t slirp_zone_clust(PNATState pData)
+{
+    return pData->zone_clust;
+}
+#ifndef VBOX_SLIRP_BSD
+# define m_adj(m, len) m_adj(pData, (m), (len))
+#endif
 #endif
 
 #endif /* !___slirp_state_h */
