@@ -1,4 +1,23 @@
+/* $Id: socket.c 28800 2010-04-27 08:22:32Z vboxsync $ */
+/** @file
+ * NAT - socket handling.
+ */
+
 /*
+ * Copyright (C) 2006-2010 Oracle Corporation
+ *
+ * This file is part of VirtualBox Open Source Edition (OSE), as
+ * available from http://www.virtualbox.org. This file is free software;
+ * you can redistribute it and/or modify it under the terms of the GNU
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ */
+
+/*
+ * This code is based on:
+ *
  * Copyright (c) 1995 Danny Gasparovski.
  *
  * Please read the file COPYRIGHT for the
@@ -31,7 +50,6 @@ so_init()
 {
 }
 
-
 struct socket *
 solookup(struct socket *head, struct in_addr laddr,
          u_int lport, struct in_addr faddr, u_int fport)
@@ -88,7 +106,7 @@ sofree(PNATState pData, struct socket *so)
 
     /* check if mbuf haven't been already freed  */
     if (so->so_m != NULL)
-        m_free(pData, so->so_m);
+        m_freem(pData, so->so_m);
 #ifndef VBOX_WITH_SLIRP_MT
     if (so->so_next && so->so_prev)
     {
@@ -133,7 +151,7 @@ soread(PNATState pData, struct socket *so)
     QSOCKET_UNLOCK(tcb);
 
     DEBUG_CALL("soread");
-    DEBUG_ARG("so = %lx", (long )so);
+    DEBUG_ARG("so = %lx", (long)so);
 
     /*
      * No need to check if there's enough room to read.
@@ -199,11 +217,10 @@ soread(PNATState pData, struct socket *so)
     nn = readv(so->s, (struct iovec *)iov, n);
     DEBUG_MISC((dfd, " ... read nn = %d bytes\n", nn));
 #else
-    nn = recv(so->s, iov[0].iov_base, iov[0].iov_len, 0);
+    nn = recv(so->s, iov[0].iov_base, iov[0].iov_len, (so->so_tcpcb->t_force? MSG_OOB:0));
 #endif
     if (nn <= 0)
     {
-#if defined(RT_OS_WINDOWS)
         /*
          * Special case for WSAEnumNetworkEvents: If we receive 0 bytes that
          * _could_ mean that the connection is closed. But we will receive an
@@ -211,18 +228,17 @@ soread(PNATState pData, struct socket *so)
          * www.youtube.com I see this very often. Closing the socket too early
          * would be dangerous.
          */
-        int status, ignored;
+        int status;
         unsigned long pending = 0;
-        status = WSAIoctl(so->s, FIONREAD, NULL, 0, &pending, sizeof(unsigned long), &ignored, NULL, NULL);
+        status = ioctlsocket(so->s, FIONREAD, &pending);
         if (status < 0)
-            LogRel(("NAT:error in WSAIoctl: %d\n", WSAGetLastError()));
+            LogRel(("NAT:error in WSAIoctl: %d\n", errno));
         if (nn == 0 && (pending != 0))
         {
             SOCKET_UNLOCK(so);
             STAM_PROFILE_STOP(&pData->StatIOread, a);
             return 0;
         }
-#endif
         if (   nn < 0
             && (   errno == EINTR
                 || errno == EAGAIN
@@ -306,6 +322,7 @@ void
 sorecvoob(PNATState pData, struct socket *so)
 {
     struct tcpcb *tp = sototcpcb(so);
+    ssize_t ret;
 
     DEBUG_CALL("sorecvoob");
     DEBUG_ARG("so = %lx", (long)so);
@@ -318,7 +335,7 @@ sorecvoob(PNATState pData, struct socket *so)
      * urgent data, or the read() doesn't return all the
      * urgent data.
      */
-    soread(pData, so);
+    ret = soread(pData, so);
     tp->snd_up = tp->snd_una + so->so_snd.sb_cc;
     tp->t_force = 1;
     tcp_output(pData, tp);
@@ -568,14 +585,19 @@ sorecvfrom(PNATState pData, struct socket *so)
     {
         /* A "normal" UDP packet */
         struct mbuf *m;
-        struct ethhdr *eh;
-        size_t len;
+        ssize_t len;
         u_long n = 0;
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+        int size;
+#endif
+        int rc = 0;
+        static int signalled = 0;
 
         QSOCKET_LOCK(udb);
         SOCKET_LOCK(so);
         QSOCKET_UNLOCK(udb);
 
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
         if (!(m = m_get(pData)))
         {
             SOCKET_UNLOCK(so);
@@ -589,41 +611,83 @@ sorecvfrom(PNATState pData, struct socket *so)
          * but I don't know the max packet size for DNS lookups
          */
         len = M_FREEROOM(m);
-        /* if (so->so_fport != htons(53)) */
+        /* if (so->so_fport != RT_H2N_U16_C(53)) */
+        rc = ioctlsocket(so->s, FIONREAD, &n);
+        if (   rc == -1
+            && (  errno == EAGAIN
+               || errno == EWOULDBLOCK
+               || errno == EINPROGRESS
+               || errno == ENOTCONN))
         {
-            int rc = 0;
-            static int signaled = 0;
-            rc = ioctlsocket(so->s, FIONREAD, &n);
-            if (   rc == -1
-                && (   errno == EAGAIN
-                    || errno == EWOULDBLOCK
-                    || errno == EINPROGRESS
-                    || errno == ENOTCONN))
-            {
-                m_free(pData, m);
-                return;
-            }
-
-            if (rc == -1 && signaled == 0)
-            {
-                LogRel(("NAT: can't fetch amount of bytes on socket %R[natsock], so message will be truncated.\n", so));
-                signaled = 1;
-                m_free(pData, m);
-                return;
-            }
-
-            if (rc != -1 && n > len)
-            {
-                n = (m->m_data - m->m_dat) + m->m_len + n + 1;
-                m_inc(m, n);
-                len = M_FREEROOM(m);
-            }
+            m_freem(pData, m);
+            return;
         }
 
+        Log2(("NAT: %R[natsock] ioctlsocket before read "
+            "(rc:%d errno:%d, n:%d)\n", so, rc, errno, n));
+
+        if (rc == -1 && signalled == 0)
+        {
+            LogRel(("NAT: can't fetch amount of bytes on socket %R[natsock], so message will be truncated.\n", so));
+            signalled = 1;
+            m_freem(pData, m);
+            return;
+        }
+
+        if (rc != -1 && n > len)
+        {
+            n = (m->m_data - m->m_dat) + m->m_len + n + 1;
+            m_inc(m, n);
+            len = M_FREEROOM(m);
+        }
         ret = recvfrom(so->s, m->m_data, len, 0,
                             (struct sockaddr *)&addr, &addrlen);
-        Log2((" did recvfrom %d, errno = %d-%s\n",
-                    m->m_len, errno, strerror(errno)));
+        Log2(("NAT: %R[natsock] ioctlsocket after read "
+            "(rc:%d errno:%d, n:%d) ret:%d, len:%d\n", so,
+             rc, errno, n, ret, len));
+#else
+        /*How many data has been received ?*/
+        /*
+        * 1. calculate how much we can read
+        * 2. read as much as possible
+        * 3. attach buffer to allocated header mbuf
+        */
+        rc = ioctlsocket(so->s, FIONREAD, &n);
+        if (rc == -1 && signalled == 0)
+        {
+            LogRel(("NAT: can't fetch amount of bytes on socket %R[natsock], so message will be truncated.\n", so));
+            signalled = 1;
+        }
+
+        len = sizeof(struct udpiphdr) + ETH_HLEN;
+        if (n > (if_mtu - len))
+        {
+            n = if_mtu - len; /* can't read than we can put in the mbuf*/
+        }
+        len += n;
+
+        size = MCLBYTES;
+        if (len < MSIZE)
+            size = MCLBYTES;
+        else if (len < MCLBYTES)
+            size = MCLBYTES;
+        else if (len < MJUM9BYTES)
+            size = MJUM9BYTES;
+        else if (len < MJUM16BYTES)
+            size = MJUM16BYTES;
+        else
+            AssertMsgFailed(("Unsupported size"));
+
+        m = m_getjcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR, size);
+        if (m == NULL)
+            return;
+        m->m_data += ETH_HLEN;
+        m->m_pkthdr.header = mtod(m, void *);
+        m->m_data += sizeof(struct udpiphdr);
+        ret = recvfrom(so->s, mtod(m, char *), n, 0,
+                            (struct sockaddr *)&addr, &addrlen);
+        /* @todo (vvl) check which flags and type should be passed */
+#endif
         m->m_len = ret;
         if (ret < 0)
         {
@@ -634,7 +698,7 @@ sorecvfrom(PNATState pData, struct socket *so)
             else if (errno == ENETUNREACH)
                 code = ICMP_UNREACH_NET;
 
-            m_free(pData, m);
+            m_freem(pData, m);
             if (   errno == EAGAIN
                 || errno == EWOULDBLOCK
                 || errno == EINPROGRESS
@@ -657,15 +721,17 @@ sorecvfrom(PNATState pData, struct socket *so)
              */
             if (so->so_expire)
             {
-                if (so->so_fport != htons(53))
+                if (so->so_fport != RT_H2N_U16_C(53))
                     so->so_expire = curtime + SO_EXPIRE;
             }
             /*
              *  last argument should be changed if Slirp will inject IP attributes
              *  Note: Here we can't check if dnsproxy's sent initial request
              */
-            if (so->so_fport == htons(53))
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
+            if (so->so_fport == RT_H2N_U16_C(53))
                 dnsproxy_answer(pData, so, m);
+#endif
 
 #if 0
             if (m->m_len == len)
@@ -692,20 +758,30 @@ int
 sosendto(PNATState pData, struct socket *so, struct mbuf *m)
 {
     int ret;
-    struct sockaddr_in addr;
+    struct sockaddr_in *paddr;
+    struct sockaddr addr;
 #if 0
     struct sockaddr_in host_addr;
 #endif
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+    caddr_t buf;
+    int mlen;
+#endif
 
     DEBUG_CALL("sosendto");
     DEBUG_ARG("so = %lx", (long)so);
     DEBUG_ARG("m = %lx", (long)m);
 
-    addr.sin_family = AF_INET;
-    if ((so->so_faddr.s_addr & htonl(pData->netmask)) == special_addr.s_addr)
+    memset(&addr, 0, sizeof(struct sockaddr));
+#ifdef RT_OS_DARWIN
+    addr.sa_len = sizeof(struct sockaddr_in);
+#endif
+    paddr = (struct sockaddr_in *)&addr;
+    paddr->sin_family = AF_INET;
+    if ((so->so_faddr.s_addr & RT_H2N_U32(pData->netmask)) == pData->special_addr.s_addr)
     {
         /* It's an alias */
-        uint32_t last_byte = ntohl(so->so_faddr.s_addr) & ~pData->netmask;
+        uint32_t last_byte = RT_N2H_U32(so->so_faddr.s_addr) & ~pData->netmask;
         switch(last_byte)
         {
 #if 0
@@ -728,22 +804,33 @@ sosendto(PNATState pData, struct socket *so, struct mbuf *m)
             case CTL_ALIAS:
             default:
                 if (last_byte == ~pData->netmask)
-                    addr.sin_addr.s_addr = INADDR_BROADCAST;
+                    paddr->sin_addr.s_addr = INADDR_BROADCAST;
                 else
-                    addr.sin_addr = loopback_addr;
+                    paddr->sin_addr = loopback_addr;
                 break;
         }
     }
     else
-        addr.sin_addr = so->so_faddr;
-    addr.sin_port = so->so_fport;
+        paddr->sin_addr = so->so_faddr;
+    paddr->sin_port = so->so_fport;
 
     DEBUG_MISC((dfd, " sendto()ing, addr.sin_port=%d, addr.sin_addr.s_addr=%.16s\n",
-                ntohs(addr.sin_port), inet_ntoa(addr.sin_addr)));
+                RT_N2H_U16(paddr->sin_port), inet_ntoa(paddr->sin_addr)));
 
     /* Don't care what port we get */
-    ret = sendto(so->s, m->m_data, m->m_len, 0,
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
+    ret = sendto(so->s, m->m_data, m->m_len, 0, &addr, sizeof (struct sockaddr_in));
+#else
+    mlen = m_length(m, NULL);
+    buf = RTMemAlloc(mlen);
+    if (buf == NULL)
+    {
+        return -1;
+    }
+    m_copydata(m, 0, mlen, buf);
+    ret = sendto(so->s, buf, mlen, 0,
                  (struct sockaddr *)&addr, sizeof (struct sockaddr));
+#endif
     if (ret < 0)
     {
         Log2(("UDP: sendto fails (%s)\n", strerror(errno)));
@@ -983,9 +1070,26 @@ send_icmp_to_guest(PNATState pData, char *buff, size_t len, struct socket *so, c
     struct icmp_msg *icm;
     uint8_t proto;
     int type = 0;
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
+    int m_room;
+#endif
 
     ip = (struct ip *)buff;
+    /* Fix ip->ip_len to  contain the total packet length including the header
+     * in _host_ byte order for all OSes. On Darwin, that value already is in
+     * host byte order. Solaris and Darwin report only the payload. */
+#ifndef RT_OS_DARWIN
+    ip->ip_len = RT_N2H_U16(ip->ip_len);
+#endif
     hlen = (ip->ip_hl << 2);
+#if defined(RT_OS_SOLARIS) || defined(RT_OS_DARWIN)
+    ip->ip_len += hlen;
+#endif
+    if (ip->ip_len < hlen + ICMP_MINLEN)
+    {
+       Log(("send_icmp_to_guest: ICMP header is too small to understand which type/subtype of the datagram\n"));
+       return;
+    }
     icp = (struct icmp *)((char *)ip + hlen);
 
     Log(("ICMP:received msg(t:%d, c:%d)\n", icp->icmp_type, icp->icmp_code));
@@ -996,15 +1100,34 @@ send_icmp_to_guest(PNATState pData, char *buff, size_t len, struct socket *so, c
         return;
     }
 
+    /*
+     * ICMP_ECHOREPLY, ICMP_TIMXCEED, ICMP_UNREACH minimal header size is
+     * ICMP_ECHOREPLY assuming data 0
+     * icmp_{type(8), code(8), cksum(16),identifier(16),seqnum(16)}
+     */
+    if (ip->ip_len < hlen + 8)
+    {
+        Log(("send_icmp_to_guest: NAT accept ICMP_{ECHOREPLY, TIMXCEED, UNREACH} the minimum size is 64 (see rfc792)\n"));
+        return;
+    }
+
     type = icp->icmp_type;
     if (   type == ICMP_TIMXCEED
         || type == ICMP_UNREACH)
     {
+        /*
+         * ICMP_TIMXCEED, ICMP_UNREACH minimal header size is
+         * icmp_{type(8), code(8), cksum(16),unused(32)} + IP header + 64 bit of original datagram
+         */
+        if (ip->ip_len < hlen + 2*8 + sizeof(struct ip))
+        {
+            Log(("send_icmp_to_guest: NAT accept ICMP_{TIMXCEED, UNREACH} the minimum size of ipheader + 64 bit of data (see rfc792)\n"));
+            return;
+        }
         ip = &icp->icmp_ip;
     }
 
     icm = icmp_find_original_mbuf(pData, ip);
-
     if (icm == NULL)
     {
         Log(("NAT: Can't find the corresponding packet for the received ICMP\n"));
@@ -1015,7 +1138,31 @@ send_icmp_to_guest(PNATState pData, char *buff, size_t len, struct socket *so, c
     Assert(m != NULL);
 
     src = addr->sin_addr.s_addr;
+    if (type == ICMP_ECHOREPLY)
+    {
+        struct ip *ip0 = mtod(m, struct ip *);
+        struct icmp *icp0 = (struct icmp *)((char *)ip0 + (ip0->ip_hl << 2));
+        if (icp0->icmp_type != ICMP_ECHO)
+        {
+            Log(("NAT: we haven't found echo for this reply\n"));
+            return;
+        }
+        /*
+         * while combining buffer to send (see ip_icmp.c) we control ICMP header only,
+         * IP header combined by OS network stack, our local copy of IP header contians values
+         * in host byte order so no byte order conversion is required. IP headers fields are converting
+         * in ip_output0 routine only.
+         */
+        if (   (ip->ip_len - hlen)
+            != (ip0->ip_len - (ip0->ip_hl << 2)))
+        {
+            Log(("NAT: ECHO(%d) lenght doesn't match ECHOREPLY(%d)\n",
+                (ip->ip_len - hlen), (ip0->ip_len - (ip0->ip_hl << 2))));
+            return;
+        }
+    }
 
+    /* ip points on origianal ip header */
     ip = mtod(m, struct ip *);
     proto = ip->ip_p;
     /* Now ip is pointing on header we've sent from guest */
@@ -1035,9 +1182,26 @@ send_icmp_to_guest(PNATState pData, char *buff, size_t len, struct socket *so, c
     ip = mtod(m, struct ip *); /* ip is from mbuf we've overrided */
     original_hlen = ip->ip_hl << 2;
     /* saves original ip header and options */
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+    m_copyback(pData, m, original_hlen, len - hlen, buff + hlen);
+    ip->ip_len = m_length(m, NULL);
+#else
+    /* m_room space in the saved m buffer */
+    m_room = M_ROOM(m);
+    if (m_room < len - hlen + original_hlen)
+    {
+        /* we need involve ether header length into new buffer buffer calculation */
+        m_inc(m, if_maxlinkhdr + len - hlen + original_hlen);
+        if (m->m_size < if_maxlinkhdr + len - hlen + original_hlen)
+        {
+            Log(("send_icmp_to_guest: extending buffer was failed (packet is dropped)\n"));
+            return;
+        }
+    }
     memcpy(m->m_data + original_hlen, buff + hlen, len - hlen);
     m->m_len = len - hlen + original_hlen;
     ip->ip_len = m->m_len;
+#endif
     ip->ip_p = IPPROTO_ICMP; /* the original package could be whatever, but we're response via ICMP*/
 
     icp = (struct icmp *)((char *)ip + (ip->ip_hl << 2));
@@ -1094,6 +1258,8 @@ sorecvfrom_icmp_win(PNATState pData, struct socket *so)
     int data_len = 0;
     int nbytes = 0;
     u_char code = ~0;
+    int out_len;
+    int size;
 
     len = pData->pfIcmpParseReplies(pData->pvIcmpBuffer, pData->szIcmpBuffer);
     if (len < 0)
@@ -1122,7 +1288,28 @@ sorecvfrom_icmp_win(PNATState pData, struct socket *so)
                 so->so_m = NULL;
                 break;
             case IP_SUCCESS: /* echo replied */
+# ifndef VBOX_WITH_SLIRP_BSD_MBUF
                 m = m_get(pData);
+# else
+                out_len = ETH_HLEN + sizeof(struct ip) +  8;
+                size;
+                size = MCLBYTES;
+                if (out_len < MSIZE)
+                    size = MCLBYTES;
+                else if (out_len < MCLBYTES)
+                    size = MCLBYTES;
+                else if (out_len < MJUM9BYTES)
+                    size = MJUM9BYTES;
+                else if (out_len < MJUM16BYTES)
+                    size = MJUM16BYTES;
+                else
+                    AssertMsgFailed(("Unsupported size"));
+
+                m = m_getjcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR, size);
+                if (m == NULL)
+                    return;
+# endif
+                m->m_len = 0;
                 m->m_data += if_maxlinkhdr;
                 ip = mtod(m, struct ip *);
                 ip->ip_src.s_addr = icr[i].Address;
@@ -1140,8 +1327,16 @@ sorecvfrom_icmp_win(PNATState pData, struct socket *so)
 
                 data_len += ICMP_MINLEN;
 
+# ifndef VBOX_WITH_SLIRP_BSD_MBUF
                 nbytes = (data_len + icr[i].DataSize > m->m_size? m->m_size - data_len: icr[i].DataSize);
                 memcpy(icp->icmp_data, icr[i].Data, nbytes);
+# else
+                hlen = (ip->ip_hl << 2);
+                m->m_pkthdr.header = mtod(m, void *);
+                m->m_len = data_len;
+
+                m_copyback(pData, m, hlen + 8, icr[i].DataSize, icr[i].Data);
+# endif
 
                 data_len += icr[i].DataSize;
 
@@ -1170,8 +1365,14 @@ sorecvfrom_icmp_win(PNATState pData, struct socket *so)
                 ip_broken->ip_src.s_addr = src; /*it packet sent from host not from guest*/
                 data_len = (ip_broken->ip_hl << 2) + 64;
 
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
                 nbytes =(hlen + ICMP_MINLEN + data_len > m->m_size? m->m_size - (hlen + ICMP_MINLEN): data_len);
                 memcpy(icp->icmp_data, ip_broken,  nbytes);
+#else
+                m->m_len = data_len;
+                m->m_pkthdr.header = mtod(m, void *);
+                m_copyback(pData, m, ip->ip_hl >> 2, icr[i].DataSize, icr[i].Data);
+#endif
                 icmp_reflect(pData, m);
                 break;
             default:
@@ -1180,48 +1381,33 @@ sorecvfrom_icmp_win(PNATState pData, struct socket *so)
         }
     }
 }
-#else /* RT_OS_WINDOWS */
+#else /* !RT_OS_WINDOWS */
 static void sorecvfrom_icmp_unix(PNATState pData, struct socket *so)
 {
     struct sockaddr_in addr;
     socklen_t addrlen = sizeof(struct sockaddr_in);
+    struct ip ip;
     char *buff;
     int len = 0;
-    int rc = 0;
-    static int signalled = 0;
-    rc = ioctlsocket(so->s, FIONREAD, &len);
-    if (   rc == -1
-        && (errno == EAGAIN
-        || errno == EWOULDBLOCK
-        || errno == EINPROGRESS
-        || errno == ENOTCONN))
-    {
-        return;
-    }
-    if (rc == -1 && signalled == 0)
-    {
-        signalled = 1;
-        LogRel(("NAT: fetching number of bits has been failed for ICMP socket (%d: %s)\n",
-            errno, strerror(errno)));
-        return;
-    }
-    len = (len != 0 && rc != -1 ? len : 1500);
-    buff = RTMemAlloc(len);
-    len = recvfrom(so->s, buff, len, 0,
-                   (struct sockaddr *)&addr, &addrlen);
-    /* XXX Check if reply is "correct"? */
 
-    if (len == -1 || len == 0)
+    /* 1- step: read the ip header */
+    len = recvfrom(so->s, &ip, sizeof(struct ip), MSG_PEEK,
+                   (struct sockaddr *)&addr, &addrlen);
+    if (   len < 0
+        && (   errno == EAGAIN
+            || errno == EWOULDBLOCK
+            || errno == EINPROGRESS
+            || errno == ENOTCONN))
+    {
+        Log(("sorecvfrom_icmp_unix: 1 - step can't read IP datagramm (would block)\n"));
+        return;
+    }
+
+    if (   len < sizeof(struct ip)
+        || len < 0
+        || len == 0)
     {
         u_char code;
-        if (   len == -1
-            && (   errno == EAGAIN
-                || errno == EWOULDBLOCK
-                || errno == EINPROGRESS
-                || errno == ENOTCONN))
-        {
-            return;
-        }
         code = ICMP_UNREACH_PORT;
 
         if (errno == EHOSTUNREACH)
@@ -1233,11 +1419,61 @@ static void sorecvfrom_icmp_unix(PNATState pData, struct socket *so)
                     errno, strerror(errno)));
         icmp_error(pData, so->so_m, ICMP_UNREACH, code, 0, strerror(errno));
         so->so_m = NULL;
+        Log(("sorecvfrom_icmp_unix: 1 - step can't read IP datagramm \n"));
+        return;
     }
-    else
+    /* basic check of IP header */
+    if (   ip.ip_v != IPVERSION
+# ifndef RT_OS_DARWIN
+        || ip.ip_p != IPPROTO_ICMP
+# endif
+        )
     {
-        send_icmp_to_guest(pData, buff, len, so, &addr);
+        Log(("sorecvfrom_icmp_unix: 1 - step IP isn't IPv4 \n"));
+        return;
     }
+# ifndef RT_OS_DARWIN
+    /* Darwin reports the IP length already in host byte order. */
+    ip.ip_len = RT_N2H_U16(ip.ip_len);
+# endif
+# if defined(RT_OS_SOLARIS) || defined(RT_OS_DARWIN)
+    /* Solaris and Darwin report the payload only */
+    ip.ip_len += (ip.ip_hl << 2);
+# endif
+    /* Note: ip->ip_len in host byte order (all OS) */
+    len = ip.ip_len;
+    buff = RTMemAlloc(len);
+    if (buff == NULL)
+    {
+        Log(("sorecvfrom_icmp_unix: 1 - step can't allocate enought room for datagram\n"));
+        return;
+    }
+    /* 2 - step: we're reading rest of the datagramm to the buffer */
+    addrlen = sizeof(struct sockaddr_in);
+    memset(&addr, 0, addrlen);
+    len = recvfrom(so->s, buff, len, 0,
+                   (struct sockaddr *)&addr, &addrlen);
+    if (   len < 0
+        && (   errno == EAGAIN
+            || errno == EWOULDBLOCK
+            || errno == EINPROGRESS
+            || errno == ENOTCONN))
+    {
+        Log(("sorecvfrom_icmp_unix: 2 - step can't read IP body (would block expected:%d)\n",
+            ip.ip_len));
+        RTMemFree(buff);
+        return;
+    }
+    if (   len < 0
+        || len == 0)
+    {
+        Log(("sorecvfrom_icmp_unix: 2 - step read of the rest of datagramm is fallen (errno:%d, len:%d expected: %d)\n",
+             errno, len, (ip.ip_len - sizeof(struct ip))));
+        RTMemFree(buff);
+        return;
+    }
+    /* len is modified in 2nd read, when the rest of the datagramm was read */
+    send_icmp_to_guest(pData, buff, len, so, &addr);
     RTMemFree(buff);
 }
 #endif /* !RT_OS_WINDOWS */
