@@ -1,4 +1,23 @@
+/* $Id: tcp_output.c 28800 2010-04-27 08:22:32Z vboxsync $ */
+/** @file
+ * NAT - TCP output.
+ */
+
 /*
+ * Copyright (C) 2006-2010 Oracle Corporation
+ *
+ * This file is part of VirtualBox Open Source Edition (OSE), as
+ * available from http://www.virtualbox.org. This file is free software;
+ * you can redistribute it and/or modify it under the terms of the GNU
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
+ */
+
+/*
+ * This code is based on:
+ *
  * Copyright (c) 1982, 1986, 1988, 1990, 1993
  *      The Regents of the University of California.  All rights reserved.
  *
@@ -75,11 +94,14 @@ tcp_output(PNATState pData, register struct tcpcb *tp)
     register struct socket *so = tp->t_socket;
     register long len, win;
     int off, flags, error;
-    register struct mbuf *m;
+    register struct mbuf *m = NULL;
     register struct tcpiphdr *ti;
     u_char opt[MAX_TCPOPTLEN];
     unsigned optlen, hdrlen;
     int idle, sendalot;
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+    int size;
+#endif
 
     DEBUG_CALL("tcp_output");
     DEBUG_ARG("tp = %lx", (long )tp);
@@ -214,8 +236,8 @@ again:
          * taking into account that we are limited by
          * TCP_MAXWIN << tp->rcv_scale.
          */
-        long adv = min(win, 
-                       (long)TCP_MAXWIN << tp->rcv_scale) - 
+        long adv = min(win,
+                       (long)TCP_MAXWIN << tp->rcv_scale) -
                        (tp->rcv_adv - tp->rcv_nxt);
 
         if (adv >= (long) (2 * tp->t_maxseg))
@@ -299,7 +321,7 @@ send:
 
             opt[0] = TCPOPT_MAXSEG;
             opt[1] = 4;
-            mss = htons((u_int16_t) tcp_mss(pData, tp, 0));
+            mss = RT_H2N_U16((u_int16_t) tcp_mss(pData, tp, 0));
             memcpy((caddr_t)(opt + 2), (caddr_t)&mss, sizeof(mss));
             optlen = 4;
 
@@ -308,10 +330,10 @@ send:
                 && (   (flags & TH_ACK) == 0
                     || (tp->t_flags & TF_RCVD_SCALE)))
             {
-                *((u_int32_t *) (opt + optlen)) = htonl(  TCPOPT_NOP << 24
-                                                        | TCPOPT_WINDOW << 16
-                                                        | TCPOLEN_WINDOW << 8
-                                                        | tp->request_r_scale);
+                *((u_int32_t *) (opt + optlen)) = RT_H2N_U32(  TCPOPT_NOP << 24
+                                                             | TCPOPT_WINDOW << 16
+                                                             | TCPOLEN_WINDOW << 8
+                                                             | tp->request_r_scale);
                 optlen += 4;
             }
 #endif
@@ -332,9 +354,9 @@ send:
         u_int32_t *lp = (u_int32_t *)(opt + optlen);
 
         /* Form timestamp option as shown in appendix A of RFC 1323. */
-        *lp++ = htonl(TCPOPT_TSTAMP_HDR);
-        *lp++ = htonl(tcp_now);
-        *lp   = htonl(tp->ts_recent);
+        *lp++ = RT_H2N_U32_C(TCPOPT_TSTAMP_HDR);
+        *lp++ = RT_H2N_U32(tcp_now);
+        *lp   = RT_H2N_U32(tp->ts_recent);
         optlen += TCPOLEN_TSTAMP_APPA;
     }
 #endif
@@ -370,7 +392,22 @@ send:
             tcpstat.tcps_sndbyte += len;
         }
 
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
         m = m_get(pData);
+#else
+        size = MCLBYTES;
+        if ((len + hdrlen + ETH_HLEN) < MSIZE)
+            size = MCLBYTES;
+        else if ((len + hdrlen + ETH_HLEN) < MCLBYTES)
+            size = MCLBYTES;
+        else if((len + hdrlen + ETH_HLEN) < MJUM9BYTES)
+            size = MJUM9BYTES;
+        else if ((len + hdrlen + ETH_HLEN) < MJUM16BYTES)
+            size = MJUM16BYTES;
+        else
+            AssertMsgFailed(("Unsupported size"));
+        m = m_getjcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR, size);
+#endif
         if (m == NULL)
         {
 /*          error = ENOBUFS; */
@@ -378,6 +415,9 @@ send:
             goto out;
         }
         m->m_data += if_maxlinkhdr;
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+        m->m_pkthdr.header = mtod(m, void *);
+#endif
         m->m_len = hdrlen;
 
         /*
@@ -419,14 +459,47 @@ send:
         else
             tcpstat.tcps_sndwinup++;
 
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
         m = m_get(pData);
         if (m == NULL)
         {
+/*          error = ENOBUFS; */
+            error = 1;
+            goto out;
+        }
+#else
+        if ((hdrlen + ETH_HLEN) < MSIZE)
+        {
+            size = MCLBYTES;
+        }
+        else if ((hdrlen + ETH_HLEN) < MCLBYTES)
+        {
+            size = MCLBYTES;
+        }
+        else if((hdrlen + ETH_HLEN) < MJUM9BYTES)
+        {
+            size = MJUM9BYTES;
+        }
+        else if ((hdrlen + ETH_HLEN) < MJUM16BYTES)
+        {
+            size = MJUM16BYTES;
+        }
+        else
+        {
+            AssertMsgFailed(("Unsupported size"));
+        }
+        m = m_getjcl(pData, M_NOWAIT, MT_HEADER, M_PKTHDR, size);
+#endif
+        if (m == NULL)
+        {
 /*          error = ENOBUFS; */
             error = 1;
             goto out;
         }
         m->m_data += if_maxlinkhdr;
+#ifdef VBOX_WITH_SLIRP_BSD_MBUF
+        m->m_pkthdr.header = mtod(m, void *);
+#endif
         m->m_len = hdrlen;
     }
 
@@ -457,10 +530,10 @@ send:
      * (retransmit and persist are mutually exclusive...)
      */
     if (len || (flags & (TH_SYN|TH_FIN)) || tp->t_timer[TCPT_PERSIST])
-        ti->ti_seq = htonl(tp->snd_nxt);
+        ti->ti_seq = RT_H2N_U32(tp->snd_nxt);
     else
-        ti->ti_seq = htonl(tp->snd_max);
-    ti->ti_ack = htonl(tp->rcv_nxt);
+        ti->ti_seq = RT_H2N_U32(tp->snd_max);
+    ti->ti_ack = RT_H2N_U32(tp->rcv_nxt);
     if (optlen)
     {
         memcpy((caddr_t)(ti + 1), (caddr_t)opt, optlen);
@@ -477,16 +550,16 @@ send:
         win = (long)TCP_MAXWIN << tp->rcv_scale;
     if (win < (long)(tp->rcv_adv - tp->rcv_nxt))
         win = (long)(tp->rcv_adv - tp->rcv_nxt);
-    ti->ti_win = htons((u_int16_t) (win>>tp->rcv_scale));
+    ti->ti_win = RT_H2N_U16((u_int16_t) (win>>tp->rcv_scale));
 
 #if 0
     if (SEQ_GT(tp->snd_up, tp->snd_nxt))
     {
-        ti->ti_urp = htons((u_int16_t)(tp->snd_up - tp->snd_nxt));
+        ti->ti_urp = RT_H2N_U16((u_int16_t)(tp->snd_up - tp->snd_nxt));
 #else
     if (SEQ_GT(tp->snd_up, tp->snd_una))
     {
-        ti->ti_urp = htons((u_int16_t)(tp->snd_up - ntohl(ti->ti_seq)));
+        ti->ti_urp = RT_H2N_U16((u_int16_t)(tp->snd_up - RT_N2H_U32(ti->ti_seq)));
 #endif
         ti->ti_flags |= TH_URG;
     }
@@ -504,15 +577,15 @@ send:
      * checksum extended header and data.
      */
     if (len + optlen)
-        ti->ti_len = htons((u_int16_t)(sizeof (struct tcphdr)
-                                       + optlen + len));
+        ti->ti_len = RT_H2N_U16((u_int16_t)(sizeof (struct tcphdr)
+                                            + optlen + len));
     ti->ti_sum = cksum(m, (int)(hdrlen + len));
 
     /*
      * In transmit state, time the transmission and arrange for
      * the retransmit.  In persist state, just set snd_max.
      */
-    if (tp->t_force == 0 || tp->t_timer[TCPT_PERSIST] == 0) 
+    if (tp->t_force == 0 || tp->t_timer[TCPT_PERSIST] == 0)
     {
         tcp_seq startseq = tp->snd_nxt;
 
@@ -574,13 +647,18 @@ send:
      * to handle ttl and tos; we could keep them in
      * the template, but need a way to checksum without them.
      */
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
     Assert(m->m_len == (hdrlen + len));
+#else
+    M_ASSERTPKTHDR(m);
+    m->m_pkthdr.header = mtod(m, void *);
+#endif
     m->m_len = hdrlen + len; /* XXX Needed? m_len should be correct */
 
     {
         ((struct ip *)ti)->ip_len = m->m_len;
         ((struct ip *)ti)->ip_ttl = ip_defttl;
-        ((struct ip *)ti)->ip_tos = so->so_iptos;
+       ((struct ip *)ti)->ip_tos = so->so_iptos;
 
         /* #if BSD >= 43 */
         /* Don't do IP options... */
@@ -588,19 +666,7 @@ send:
         error = ip_output(m, tp->t_inpcb->inp_options, &tp->t_inpcb->inp_route,
                          so->so_options & SO_DONTROUTE, 0);
 #endif
-#ifdef VBOX_WITH_NAT_SERVICE
-        {
-            struct ethhdr *eh0, *eh;
-            eh = (struct ethhdr *)m->m_dat;
-
-            if (so->so_m != NULL)
-            {
-                eh0 = (struct ethhdr *)so->so_m->m_dat;
-                memcpy(eh->h_source, eh0->h_source, ETH_ALEN);
-            }
-        }
-#endif
-#ifdef VBOX_WITH_SLIRP_ALIAS
+#ifndef VBOX_WITH_SLIRP_BSD_MBUF
         if(so->so_la != NULL)
             m->m_la = so->so_la;
 #endif
@@ -631,6 +697,8 @@ out:
             return (0);
         }
 #endif
+        if (m != NULL)
+            m_freem(pData, m);
         return (error);
     }
     tcpstat.tcps_sndtotal++;
