@@ -1,10 +1,10 @@
-/* $Id: DevDMA.cpp 12978 2008-10-03 23:28:44Z vboxsync $ */
+/* $Id: DevDMA.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DevDMA - DMA Controller Device.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -55,7 +51,6 @@
 #define LOG_GROUP LOG_GROUP_DEFAULT ///@todo LOG_GROUP_DEV_DMA
 #include <VBox/log.h>
 #include <iprt/assert.h>
-#include <iprt/uuid.h>
 #include <iprt/string.h>
 
 #include <stdio.h>
@@ -845,14 +840,14 @@ static uint8_t get_mode_wrapper (PPDMDEVINS pDevIns, unsigned nchan)
     return DMA_get_channel_mode (s, nchan);
 }
 
-static void DMAReset (PPDMDEVINS pDevIns)
+static void dmaReset (PPDMDEVINS pDevIns)
 {
     DMAState *s = PDMINS_2_DATA (pDevIns, DMAState *);
     dma_reset (&s->dma_controllers[0]);
     dma_reset (&s->dma_controllers[1]);
 }
 
-static DECLCALLBACK(int) SaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) dmaSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
 {
     DMAState *s = PDMINS_2_DATA (pDevIns, DMAState *);
     dma_save (pSSMHandle, &s->dma_controllers[0]);
@@ -860,37 +855,26 @@ static DECLCALLBACK(int) SaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
     return VINF_SUCCESS;
 }
 
-static DECLCALLBACK(int) LoadExec (PPDMDEVINS pDevIns,
-                                   PSSMHANDLE pSSMHandle,
-                                   uint32_t u32Version)
+static DECLCALLBACK(int) dmaLoadExec (PPDMDEVINS pDevIns,
+                                      PSSMHANDLE pSSMHandle,
+                                      uint32_t uVersion,
+                                      uint32_t uPass)
 {
     DMAState *s = PDMINS_2_DATA (pDevIns, DMAState *);
 
-    if (u32Version != 1) {
-        AssertFailed ();
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
-    }
+    AssertMsgReturn (uVersion == 1, ("%d\n", uVersion), VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION);
+    Assert (uPass == SSM_PASS_FINAL); NOREF(uPass);
 
-    dma_load (pSSMHandle, &s->dma_controllers[0], u32Version);
-    return dma_load (pSSMHandle, &s->dma_controllers[1], u32Version);
+    dma_load (pSSMHandle, &s->dma_controllers[0], uVersion);
+    return dma_load (pSSMHandle, &s->dma_controllers[1], uVersion);
 }
 
 /**
- * Construct a device instance for a VM.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- *                      If the registration structure is needed, pDevIns->pDevReg points to it.
- * @param   iInstance   Instance number. Use this to figure out which registers and such to use.
- *                      The device number is also found in pDevIns->iInstance, but since it's
- *                      likely to be freqently used PDM passes it as parameter.
- * @param   pCfgHandle  Configuration node handle for the device. Use this to obtain the configuration
- *                      of the device instance. It's also found in pDevIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @interface_method_impl{PDMDEVREG,pfnConstruct}
  */
-static DECLCALLBACK(int) DMAConstruct(PPDMDEVINS pDevIns,
+static DECLCALLBACK(int) dmaConstruct(PPDMDEVINS pDevIns,
                                       int iInstance,
-                                      PCFGMNODE pCfgHandle)
+                                      PCFGMNODE pCfg)
 {
     DMAState *s = PDMINS_2_DATA (pDevIns, DMAState *);
     bool high_page_enable = 0;
@@ -902,11 +886,11 @@ static DECLCALLBACK(int) DMAConstruct(PPDMDEVINS pDevIns,
     /*
      * Validate configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "\0")) /* "HighPageEnable\0")) */
+    if (!CFGMR3AreValuesValid(pCfg, "\0")) /* "HighPageEnable\0")) */
         return VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES;
 
 #if 0
-    rc = CFGMR3QueryBool (pCfgHandle, "HighPageEnable", &high_page_enable);
+    rc = CFGMR3QueryBool (pCfg, "HighPageEnable", &high_page_enable);
     if (RT_FAILURE (rc)) {
         return rc;
     }
@@ -925,14 +909,12 @@ static DECLCALLBACK(int) DMAConstruct(PPDMDEVINS pDevIns,
     reg.pfnSetDREQ        = set_DREQ_wrapper;
     reg.pfnGetChannelMode = get_mode_wrapper;
 
-    Assert(pDevIns->pDevHlpR3->pfnDMARegister);
-    rc = pDevIns->pDevHlpR3->pfnDMACRegister (pDevIns, &reg, &s->pHlp);
+    rc = PDMDevHlpDMACRegister (pDevIns, &reg, &s->pHlp);
     if (RT_FAILURE (rc)) {
         return rc;
     }
 
-    rc = PDMDevHlpSSMRegister (pDevIns, pDevIns->pDevReg->szDeviceName, iInstance, 1, sizeof (*s),
-                               NULL, SaveExec, NULL, NULL, LoadExec, NULL);
+    rc = PDMDevHlpSSMRegister (pDevIns, 1 /*uVersion*/, sizeof (*s), dmaSaveExec, dmaLoadExec);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -946,7 +928,7 @@ const PDMDEVREG g_DeviceDMA =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "8237A",
     /* szRCMod */
     "",
@@ -963,7 +945,7 @@ const PDMDEVREG g_DeviceDMA =
     /* cbInstance */
     sizeof(DMAState),
     /* pfnConstruct */
-    DMAConstruct,
+    dmaConstruct,
     /* pfnDestruct */
     NULL,
     /* pfnRelocate */
@@ -973,7 +955,7 @@ const PDMDEVREG g_DeviceDMA =
     /* pfnPowerOn */
     NULL,
     /* pfnReset */
-    DMAReset,
+    dmaReset,
     /* pfnSuspend */
     NULL,
     /* pfnResume */
