@@ -1,10 +1,10 @@
-/* $Id: DevPit-i8254.cpp $ */
+/* $Id: DevPit-i8254.cpp 29250 2010-05-09 17:53:58Z vboxsync $ */
 /** @file
  * DevPIT-i8254 - Intel 8254 Programmable Interval Timer (PIT) And Dummy Speaker Device.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -52,7 +48,13 @@
 #include <VBox/log.h>
 #include <VBox/stam.h>
 #include <iprt/assert.h>
-#include <iprt/asm.h>
+#include <iprt/asm-math.h>
+
+#ifdef IN_RING3
+# include <iprt/alloc.h>
+# include <iprt/string.h>
+# include <iprt/uuid.h>
+#endif /* IN_RING3 */
 
 #include "../Builtins.h"
 
@@ -68,8 +70,14 @@
 #define RW_STATE_WORD0 3
 #define RW_STATE_WORD1 4
 
-/** The version of the saved state. */
-#define PIT_SAVED_STATE_VERSION 2
+/** The current saved state version. */
+#define PIT_SAVED_STATE_VERSION             4
+/** The saved state version used by VirtualBox 3.1 and earlier.
+ * This did not include disable by HPET flag. */
+#define PIT_SAVED_STATE_VERSION_VBOX_31     3
+/** The saved state version used by VirtualBox 3.0 and earlier.
+ * This did not include the config part. */
+#define PIT_SAVED_STATE_VERSION_VBOX_30     2
 
 /** @def FAKE_REFRESH_CLOCK
  * Define this to flip the 15usec refresh bit on every read.
@@ -139,11 +147,16 @@ typedef struct PITState
 #else
     uint32_t                Alignment1;
 #endif
+    /** Config: I/O port base. */
+    RTIOPORT                IOPortBaseCfg;
+    /** Config: Speaker enabled. */
+    bool                    fSpeakerCfg;
+    bool                    fDisabledByHpet;
+    bool                    afAlignment0[HC_ARCH_BITS == 32 ? 4 : 4];
+    /** PIT port interface. */
+    PDMIHPETLEGACYNOTIFY    IHpetLegacyNotify;
     /** Pointer to the device instance. */
     PPDMDEVINSR3            pDevIns;
-#if HC_ARCH_BITS == 32
-    uint32_t                Alignment0;
-#endif
     /** Number of IRQs that's been raised. */
     STAMCOUNTER             StatPITIrq;
     /** Profiling the timer callback handler. */
@@ -155,15 +168,15 @@ typedef struct PITState
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 PDMBOTHCBDECL(int) pitIOPortRead(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Port, uint32_t *pu32, unsigned cb);
 PDMBOTHCBDECL(int) pitIOPortWrite(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Port, uint32_t u32, unsigned cb);
 PDMBOTHCBDECL(int) pitIOPortSpeakerRead(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Port, uint32_t *pu32, unsigned cb);
 #ifdef IN_RING3
 PDMBOTHCBDECL(int) pitIOPortSpeakerWrite(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Port, uint32_t u32, unsigned cb);
-static void pit_irq_timer_update(PITChannelState *s, uint64_t current_time);
+static void pit_irq_timer_update(PITChannelState *s, uint64_t current_time, uint64_t now);
 #endif
-__END_DECLS
+RT_C_DECLS_END
 
 
 
@@ -181,11 +194,14 @@ static int pit_get_count(PITChannelState *s)
             d = ASMMultU64ByU32DivByU32(TMTimerGet(pTimer) - s->count_load_time, PIT_FREQ, TMTimerGetFreq(pTimer));
             return s->count - (d % s->count); /** @todo check this value. */
         }
+        uint64_t Interval = s->u64NextTS - s->u64ReloadTS;
+        if (!Interval)
+            return s->count - 1; /** @todo This is WRONG! But I'm too tired to fix it properly and just want to shut up a DIV/0 trap now. */
         d = TMTimerGet(pTimer);
-        d = ASMMultU64ByU32DivByU32(d - s->u64ReloadTS, s->count, s->u64NextTS - s->u64ReloadTS);
+        d = ASMMultU64ByU32DivByU32(d - s->u64ReloadTS, s->count, Interval);
         if (d >= s->count)
             return 1;
-        return  s->count - d;
+        return s->count - d;
     }
     d = ASMMultU64ByU32DivByU32(TMTimerGet(pTimer) - s->count_load_time, PIT_FREQ, TMTimerGetFreq(pTimer));
     switch(s->mode) {
@@ -286,16 +302,18 @@ static void pit_set_gate(PITState *pit, int channel, int val)
     case 5:
         if (s->gate < val) {
             /* restart counting on rising edge */
+            Log(("pit_set_gate: restarting mode %d\n", s->mode));
             s->count_load_time = TMTimerGet(pTimer);
-            pit_irq_timer_update(s, s->count_load_time);
+            pit_irq_timer_update(s, s->count_load_time, s->count_load_time);
         }
         break;
     case 2:
     case 3:
         if (s->gate < val) {
             /* restart counting on rising edge */
+            Log(("pit_set_gate: restarting mode %d\n", s->mode));
             s->count_load_time = s->u64ReloadTS = TMTimerGet(pTimer);
-            pit_irq_timer_update(s, s->count_load_time);
+            pit_irq_timer_update(s, s->count_load_time, s->count_load_time);
         }
         /* XXX: disable/enable counting */
         break;
@@ -310,13 +328,16 @@ DECLINLINE(void) pit_load_count(PITChannelState *s, int val)
         val = 0x10000;
     s->count_load_time = s->u64ReloadTS = TMTimerGet(pTimer);
     s->count = val;
-    pit_irq_timer_update(s, s->count_load_time);
+    pit_irq_timer_update(s, s->count_load_time, s->count_load_time);
 
     /* log the new rate (ch 0 only). */
     if (    s->pTimerR3 /* ch 0 */
         &&  s->cRelLogEntries++ < 32)
         LogRel(("PIT: mode=%d count=%#x (%u) - %d.%02d Hz (ch=0)\n",
                 s->mode, s->count, s->count, PIT_FREQ / s->count, (PIT_FREQ * 100 / s->count) % 100));
+    else
+        Log(("PIT: mode=%d count=%#x (%u) - %d.%02d Hz (ch=0)\n",
+             s->mode, s->count, s->count, PIT_FREQ / s->count, (PIT_FREQ * 100 / s->count) % 100));
 }
 
 /* return -1 if no transition will occur.  */
@@ -376,7 +397,7 @@ static int64_t pit_get_next_transition_time(PITChannelState *s,
         break;
     }
     /* convert to timer units */
-    LogFlow(("PIT: next_time=%14RI64 %20RI64 mode=%#x count=%#06x\n", next_time,
+    LogFlow(("PIT: next_time=%'14RU64 %'20RU64 mode=%#x count=%#06x\n", next_time,
              ASMMultU64ByU32DivByU32(next_time, TMTimerGetFreq(pTimer), PIT_FREQ), s->mode, s->count));
     next_time = s->count_load_time + ASMMultU64ByU32DivByU32(next_time, TMTimerGetFreq(pTimer), PIT_FREQ);
     /* fix potential rounding problems */
@@ -386,9 +407,8 @@ static int64_t pit_get_next_transition_time(PITChannelState *s,
     return next_time;
 }
 
-static void pit_irq_timer_update(PITChannelState *s, uint64_t current_time)
+static void pit_irq_timer_update(PITChannelState *s, uint64_t current_time, uint64_t now)
 {
-    uint64_t now;
     int64_t expire_time;
     int irq_level;
     PPDMDEVINS pDevIns;
@@ -401,11 +421,18 @@ static void pit_irq_timer_update(PITChannelState *s, uint64_t current_time)
 
     /* We just flip-flop the irq level to save that extra timer call, which isn't generally required (we haven't served it for months). */
     pDevIns = s->CTX_SUFF(pPit)->pDevIns;
-    PDMDevHlpISASetIrq(pDevIns, s->irq, irq_level);
-    if (irq_level)
-        PDMDevHlpISASetIrq(pDevIns, s->irq, 0);
-    now = TMTimerGet(pTimer);
-    Log3(("pit_irq_timer_update: %lldns late\n", now - s->u64NextTS));
+
+    /* If PIT disabled by HPET - just disconnect ticks from interrupt controllers, and not modify
+     * other moments of device functioning.
+     * @todo: is it correct?
+     */
+    if (!s->pPitR3->fDisabledByHpet)
+    {
+        PDMDevHlpISASetIrq(pDevIns, s->irq, irq_level);
+        if (irq_level)
+            PDMDevHlpISASetIrq(pDevIns, s->irq, 0);
+    }
+
     if (irq_level)
     {
         s->u64ReloadTS = now;
@@ -414,6 +441,7 @@ static void pit_irq_timer_update(PITChannelState *s, uint64_t current_time)
 
     if (expire_time != -1)
     {
+        Log3(("pit_irq_timer_update: next=%'RU64 now=%'RU64\n", expire_time, now));
         s->u64NextTS = expire_time;
         TMTimerSet(s->CTX_SUFF(pTimer), s->u64NextTS);
     }
@@ -703,100 +731,142 @@ PDMBOTHCBDECL(int) pitIOPortSpeakerWrite(PPDMDEVINS pDevIns, void *pvUser, RTIOP
 
 
 /**
- * Saves a state of the programmable interval timer device.
- *
- * @returns VBox status code.
- * @param   pDevIns     The device instance.
- * @param   pSSMHandle  The handle to save the state to.
+ * @copydoc FNSSMDEVLIVEEXEC
  */
-static DECLCALLBACK(int) pitSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) pitLiveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t uPass)
 {
     PITState *pThis = PDMINS_2_DATA(pDevIns, PITState *);
-    unsigned i;
-
-    for (i = 0; i < RT_ELEMENTS(pThis->channels); i++)
-    {
-        PITChannelState *s = &pThis->channels[i];
-        SSMR3PutU32(pSSMHandle, s->count);
-        SSMR3PutU16(pSSMHandle, s->latched_count);
-        SSMR3PutU8(pSSMHandle, s->count_latched);
-        SSMR3PutU8(pSSMHandle, s->status_latched);
-        SSMR3PutU8(pSSMHandle, s->status);
-        SSMR3PutU8(pSSMHandle, s->read_state);
-        SSMR3PutU8(pSSMHandle, s->write_state);
-        SSMR3PutU8(pSSMHandle, s->write_latch);
-        SSMR3PutU8(pSSMHandle, s->rw_mode);
-        SSMR3PutU8(pSSMHandle, s->mode);
-        SSMR3PutU8(pSSMHandle, s->bcd);
-        SSMR3PutU8(pSSMHandle, s->gate);
-        SSMR3PutU64(pSSMHandle, s->count_load_time);
-        SSMR3PutU64(pSSMHandle, s->u64NextTS);
-        SSMR3PutU64(pSSMHandle, s->u64ReloadTS);
-        SSMR3PutS64(pSSMHandle, s->next_transition_time);
-        if (s->CTX_SUFF(pTimer))
-            TMR3TimerSave(s->CTX_SUFF(pTimer), pSSMHandle);
-    }
-
-    SSMR3PutS32(pSSMHandle, pThis->speaker_data_on);
-#ifdef FAKE_REFRESH_CLOCK
-    return SSMR3PutS32(pSSMHandle, pThis->dummy_refresh_clock);
-#else
-    return SSMR3PutS32(pSSMHandle, 0);
-#endif
+    SSMR3PutIOPort(pSSM, pThis->IOPortBaseCfg);
+    SSMR3PutU8(    pSSM, pThis->channels[0].irq);
+    SSMR3PutBool(  pSSM, pThis->fSpeakerCfg);
+    return VINF_SSM_DONT_CALL_AGAIN;
 }
 
 
 /**
- * Loads a saved programmable interval timer device state.
- *
- * @returns VBox status code.
- * @param   pDevIns     The device instance.
- * @param   pSSMHandle  The handle to the saved state.
- * @param   u32Version  The data unit version number.
+ * @copydoc FNSSMDEVSAVEEXEC
  */
-static DECLCALLBACK(int) pitLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle, uint32_t u32Version)
+static DECLCALLBACK(int) pitSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSM)
 {
     PITState *pThis = PDMINS_2_DATA(pDevIns, PITState *);
     unsigned i;
 
-    if (u32Version != PIT_SAVED_STATE_VERSION)
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
+    /* The config. */
+    pitLiveExec(pDevIns, pSSM, SSM_PASS_FINAL);
 
+    /* The state. */
     for (i = 0; i < RT_ELEMENTS(pThis->channels); i++)
     {
         PITChannelState *s = &pThis->channels[i];
-        SSMR3GetU32(pSSMHandle, &s->count);
-        SSMR3GetU16(pSSMHandle, &s->latched_count);
-        SSMR3GetU8(pSSMHandle, &s->count_latched);
-        SSMR3GetU8(pSSMHandle, &s->status_latched);
-        SSMR3GetU8(pSSMHandle, &s->status);
-        SSMR3GetU8(pSSMHandle, &s->read_state);
-        SSMR3GetU8(pSSMHandle, &s->write_state);
-        SSMR3GetU8(pSSMHandle, &s->write_latch);
-        SSMR3GetU8(pSSMHandle, &s->rw_mode);
-        SSMR3GetU8(pSSMHandle, &s->mode);
-        SSMR3GetU8(pSSMHandle, &s->bcd);
-        SSMR3GetU8(pSSMHandle, &s->gate);
-        SSMR3GetU64(pSSMHandle, &s->count_load_time);
-        SSMR3GetU64(pSSMHandle, &s->u64NextTS);
-        SSMR3GetU64(pSSMHandle, &s->u64ReloadTS);
-        SSMR3GetS64(pSSMHandle, &s->next_transition_time);
+        SSMR3PutU32(pSSM, s->count);
+        SSMR3PutU16(pSSM, s->latched_count);
+        SSMR3PutU8(pSSM, s->count_latched);
+        SSMR3PutU8(pSSM, s->status_latched);
+        SSMR3PutU8(pSSM, s->status);
+        SSMR3PutU8(pSSM, s->read_state);
+        SSMR3PutU8(pSSM, s->write_state);
+        SSMR3PutU8(pSSM, s->write_latch);
+        SSMR3PutU8(pSSM, s->rw_mode);
+        SSMR3PutU8(pSSM, s->mode);
+        SSMR3PutU8(pSSM, s->bcd);
+        SSMR3PutU8(pSSM, s->gate);
+        SSMR3PutU64(pSSM, s->count_load_time);
+        SSMR3PutU64(pSSM, s->u64NextTS);
+        SSMR3PutU64(pSSM, s->u64ReloadTS);
+        SSMR3PutS64(pSSM, s->next_transition_time);
+        if (s->CTX_SUFF(pTimer))
+            TMR3TimerSave(s->CTX_SUFF(pTimer), pSSM);
+    }
+
+    SSMR3PutS32(pSSM, pThis->speaker_data_on);
+#ifdef FAKE_REFRESH_CLOCK
+    SSMR3PutS32(pSSM, pThis->dummy_refresh_clock);
+#else
+    SSMR3PutS32(pSSM, 0);
+#endif
+
+    return SSMR3PutBool(pSSM, pThis->fDisabledByHpet);
+}
+
+
+/**
+ * @copydoc FNSSMDEVLOADEXEC
+ */
+static DECLCALLBACK(int) pitLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
+{
+    PITState   *pThis = PDMINS_2_DATA(pDevIns, PITState *);
+    int         rc;
+
+    if (    uVersion != PIT_SAVED_STATE_VERSION
+        &&  uVersion != PIT_SAVED_STATE_VERSION_VBOX_30
+        &&  uVersion != PIT_SAVED_STATE_VERSION_VBOX_31)
+        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
+
+    /* The config. */
+    if (uVersion > PIT_SAVED_STATE_VERSION_VBOX_30)
+    {
+        RTIOPORT IOPortBaseCfg;
+        rc = SSMR3GetIOPort(pSSM, &IOPortBaseCfg); AssertRCReturn(rc, rc);
+        if (IOPortBaseCfg != pThis->IOPortBaseCfg)
+            return SSMR3SetCfgError(pSSM, RT_SRC_POS, N_("Config mismatch - IOPortBaseCfg: saved=%RTiop config=%RTiop"),
+                                    IOPortBaseCfg, pThis->IOPortBaseCfg);
+
+        uint8_t u8Irq;
+        rc = SSMR3GetU8(pSSM, &u8Irq); AssertRCReturn(rc, rc);
+        if (u8Irq != pThis->channels[0].irq)
+            return SSMR3SetCfgError(pSSM, RT_SRC_POS, N_("Config mismatch - u8Irq: saved=%#x config=%#x"),
+                                    u8Irq, pThis->channels[0].irq);
+
+        bool fSpeakerCfg;
+        rc = SSMR3GetBool(pSSM, &fSpeakerCfg); AssertRCReturn(rc, rc);
+        if (fSpeakerCfg != pThis->fSpeakerCfg)
+            return SSMR3SetCfgError(pSSM, RT_SRC_POS, N_("Config mismatch - fSpeakerCfg: saved=%RTbool config=%RTbool"),
+                                    fSpeakerCfg, pThis->fSpeakerCfg);
+    }
+
+    if (uPass != SSM_PASS_FINAL)
+        return VINF_SUCCESS;
+
+    /* The state. */
+    for (unsigned i = 0; i < RT_ELEMENTS(pThis->channels); i++)
+    {
+        PITChannelState *s = &pThis->channels[i];
+        SSMR3GetU32(pSSM, &s->count);
+        SSMR3GetU16(pSSM, &s->latched_count);
+        SSMR3GetU8(pSSM, &s->count_latched);
+        SSMR3GetU8(pSSM, &s->status_latched);
+        SSMR3GetU8(pSSM, &s->status);
+        SSMR3GetU8(pSSM, &s->read_state);
+        SSMR3GetU8(pSSM, &s->write_state);
+        SSMR3GetU8(pSSM, &s->write_latch);
+        SSMR3GetU8(pSSM, &s->rw_mode);
+        SSMR3GetU8(pSSM, &s->mode);
+        SSMR3GetU8(pSSM, &s->bcd);
+        SSMR3GetU8(pSSM, &s->gate);
+        SSMR3GetU64(pSSM, &s->count_load_time);
+        SSMR3GetU64(pSSM, &s->u64NextTS);
+        SSMR3GetU64(pSSM, &s->u64ReloadTS);
+        SSMR3GetS64(pSSM, &s->next_transition_time);
         if (s->CTX_SUFF(pTimer))
         {
-            TMR3TimerLoad(s->CTX_SUFF(pTimer), pSSMHandle);
+            TMR3TimerLoad(s->CTX_SUFF(pTimer), pSSM);
             LogRel(("PIT: mode=%d count=%#x (%u) - %d.%02d Hz (ch=%d) (restore)\n",
                     s->mode, s->count, s->count, PIT_FREQ / s->count, (PIT_FREQ * 100 / s->count) % 100, i));
         }
         pThis->channels[0].cRelLogEntries = 0;
     }
 
-    SSMR3GetS32(pSSMHandle, &pThis->speaker_data_on);
+    SSMR3GetS32(pSSM, &pThis->speaker_data_on);
 #ifdef FAKE_REFRESH_CLOCK
-    return SSMR3GetS32(pSSMHandle, &pThis->dummy_refresh_clock);
+    SSMR3GetS32(pSSM, &pThis->dummy_refresh_clock);
 #else
     int32_t u32Dummy;
-    return SSMR3GetS32(pSSMHandle, &u32Dummy);
+    SSMR3GetS32(pSSM, &u32Dummy);
 #endif
+    if (uVersion > PIT_SAVED_STATE_VERSION_VBOX_31)
+        SSMR3GetBool(pSSM, &pThis->fDisabledByHpet);
+
+    return VINF_SUCCESS;
 }
 
 
@@ -805,78 +875,18 @@ static DECLCALLBACK(int) pitLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle,
  *
  * @param   pDevIns         Device instance of the device which registered the timer.
  * @param   pTimer          The timer handle.
+ * @param   pvUser          Pointer to the PIT channel state.
  */
-static DECLCALLBACK(void) pitTimer(PPDMDEVINS pDevIns, PTMTIMER pTimer)
+static DECLCALLBACK(void) pitTimer(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
 {
-    PITState *pThis = PDMINS_2_DATA(pDevIns, PITState *);
-    PITChannelState *s = &pThis->channels[0];
+    PITChannelState *s = (PITChannelState *)pvUser;
     STAM_PROFILE_ADV_START(&s->CTX_SUFF(pPit)->StatPITHandler, a);
-    pit_irq_timer_update(s, s->next_transition_time);
+    Log(("pitTimer\n"));
+    pit_irq_timer_update(s, s->next_transition_time, TMTimerGet(pTimer));
     STAM_PROFILE_ADV_STOP(&s->CTX_SUFF(pPit)->StatPITHandler, a);
 }
 
 
-/**
- * Relocation notification.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- * @param   offDelta    The delta relative to the old address.
- */
-static DECLCALLBACK(void) pitRelocate(PPDMDEVINS pDevIns, RTGCINTPTR offDelta)
-{
-    PITState *pThis = PDMINS_2_DATA(pDevIns, PITState *);
-    unsigned i;
-    LogFlow(("pitRelocate: \n"));
-
-    for (i = 0; i < RT_ELEMENTS(pThis->channels); i++)
-    {
-        PITChannelState *pCh = &pThis->channels[i];
-        if (pCh->pTimerR3)
-            pCh->pTimerRC = TMTimerRCPtr(pCh->pTimerR3);
-        pThis->channels[i].pPitRC = PDMINS_2_DATA_RCPTR(pDevIns);
-    }
-}
-
-/** @todo remove this! */
-static DECLCALLBACK(void) pitInfo(PPDMDEVINS pDevIns, PCDBGFINFOHLP pHlp, const char *pszArgs);
-
-/**
- * Reset notification.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- */
-static DECLCALLBACK(void) pitReset(PPDMDEVINS pDevIns)
-{
-    PITState *pThis = PDMINS_2_DATA(pDevIns, PITState *);
-    unsigned i;
-    LogFlow(("pitReset: \n"));
-
-    for (i = 0; i < RT_ELEMENTS(pThis->channels); i++)
-    {
-        PITChannelState *s = &pThis->channels[i];
-
-#if 1 /* Set everything back to virgin state. (might not be strictly correct) */
-        s->latched_count = 0;
-        s->count_latched = 0;
-        s->status_latched = 0;
-        s->status = 0;
-        s->read_state = 0;
-        s->write_state = 0;
-        s->write_latch = 0;
-        s->rw_mode = 0;
-        s->bcd = 0;
-#endif
-        s->u64NextTS = UINT64_MAX;
-        s->cRelLogEntries = 0;
-        s->mode = 3;
-        s->gate = (i != 2);
-        pit_load_count(s, 0);
-    }
-}
-
-
 /**
  * Info handler, device version.
  *
@@ -915,23 +925,99 @@ static DECLCALLBACK(void) pitInfo(PPDMDEVINS pDevIns, PCDBGFINFOHLP pHlp, const
 #else
     pHlp->pfnPrintf(pHlp, "speaker_data_on=%#x\n", pThis->speaker_data_on);
 #endif
+    if (pThis->fDisabledByHpet)
+        pHlp->pfnPrintf(pHlp, "Disabled by HPET\n");
 }
 
 
 /**
- * Construct a device instance for a VM.
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
+ */
+static DECLCALLBACK(void *) pitQueryInterface(PPDMIBASE pInterface, const char *pszIID)
+{
+    PPDMDEVINS  pDevIns = RT_FROM_MEMBER(pInterface, PDMDEVINS, IBase);
+    PITState   *pThis   = PDMINS_2_DATA(pDevIns, PITState *);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE,    &pDevIns->IBase);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIHPETLEGACYNOTIFY, &pThis->IHpetLegacyNotify);
+    return NULL;
+}
+
+
+/**
+ * @interface_method_impl{PDMIHPETLEGACYNOTIFY,pfnModeChanged}
+ */
+static DECLCALLBACK(void) pitNotifyHpetLegacyNotify_ModeChanged(PPDMIHPETLEGACYNOTIFY pInterface, bool fActivated)
+{
+    PITState *pThis = RT_FROM_MEMBER(pInterface, PITState, IHpetLegacyNotify);
+    pThis->fDisabledByHpet = fActivated;
+}
+
+
+/**
+ * Relocation notification.
  *
  * @returns VBox status.
  * @param   pDevIns     The device instance data.
- *                      If the registration structure is needed, pDevIns->pDevReg points to it.
- * @param   iInstance   Instance number. Use this to figure out which registers and such to use.
- *                      The device number is also found in pDevIns->iInstance, but since it's
- *                      likely to be freqently used PDM passes it as parameter.
- * @param   pCfgHandle  Configuration node handle for the device. Use this to obtain the configuration
- *                      of the device instance. It's also found in pDevIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @param   offDelta    The delta relative to the old address.
  */
-static DECLCALLBACK(int)  pitConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfgHandle)
+static DECLCALLBACK(void) pitRelocate(PPDMDEVINS pDevIns, RTGCINTPTR offDelta)
+{
+    PITState *pThis = PDMINS_2_DATA(pDevIns, PITState *);
+    unsigned i;
+    LogFlow(("pitRelocate: \n"));
+
+    for (i = 0; i < RT_ELEMENTS(pThis->channels); i++)
+    {
+        PITChannelState *pCh = &pThis->channels[i];
+        if (pCh->pTimerR3)
+            pCh->pTimerRC = TMTimerRCPtr(pCh->pTimerR3);
+        pThis->channels[i].pPitRC = PDMINS_2_DATA_RCPTR(pDevIns);
+    }
+}
+
+
+/**
+ * Reset notification.
+ *
+ * @returns VBox status.
+ * @param   pDevIns     The device instance data.
+ */
+static DECLCALLBACK(void) pitReset(PPDMDEVINS pDevIns)
+{
+    PITState *pThis = PDMINS_2_DATA(pDevIns, PITState *);
+    unsigned i;
+    LogFlow(("pitReset: \n"));
+
+    pThis->fDisabledByHpet = false;
+
+    for (i = 0; i < RT_ELEMENTS(pThis->channels); i++)
+    {
+        PITChannelState *s = &pThis->channels[i];
+
+#if 1 /* Set everything back to virgin state. (might not be strictly correct) */
+        s->latched_count = 0;
+        s->count_latched = 0;
+        s->status_latched = 0;
+        s->status = 0;
+        s->read_state = 0;
+        s->write_state = 0;
+        s->write_latch = 0;
+        s->rw_mode = 0;
+        s->bcd = 0;
+#endif
+        s->u64NextTS = UINT64_MAX;
+        s->cRelLogEntries = 0;
+        s->mode = 3;
+        s->gate = (i != 2);
+        pit_load_count(s, 0);
+    }
+}
+
+
+/**
+ * @interface_method_impl{PDMDEVREG,pfnConstruct}
+ */
+static DECLCALLBACK(int)  pitConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfg)
 {
     PITState   *pThis = PDMINS_2_DATA(pDevIns, PITState *);
     int         rc;
@@ -946,38 +1032,40 @@ static DECLCALLBACK(int)  pitConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
     /*
      * Validate configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "Irq\0" "Base\0" "Speaker\0" "GCEnabled\0" "R0Enabled\0"))
+    if (!CFGMR3AreValuesValid(pCfg, "Irq\0" "Base\0" "SpeakerEnabled\0" "GCEnabled\0" "R0Enabled\0"))
         return VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES;
 
     /*
      * Init the data.
      */
-    rc = CFGMR3QueryU8Def(pCfgHandle, "Irq", &u8Irq, 0);
+    rc = CFGMR3QueryU8Def(pCfg, "Irq", &u8Irq, 0);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Querying \"Irq\" as a uint8_t failed"));
 
-    rc = CFGMR3QueryU16Def(pCfgHandle, "Base", &u16Base, 0x40);
+    rc = CFGMR3QueryU16Def(pCfg, "Base", &u16Base, 0x40);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Querying \"Base\" as a uint16_t failed"));
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "SpeakerEnabled", &fSpeaker, true);
+    rc = CFGMR3QueryBoolDef(pCfg, "SpeakerEnabled", &fSpeaker, true);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Querying \"SpeakerEnabled\" as a bool failed"));
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "GCEnabled", &fGCEnabled, true);
+    rc = CFGMR3QueryBoolDef(pCfg, "GCEnabled", &fGCEnabled, true);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Querying \"GCEnabled\" as a bool failed"));
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "R0Enabled", &fR0Enabled, true);
+    rc = CFGMR3QueryBoolDef(pCfg, "R0Enabled", &fR0Enabled, true);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: failed to read R0Enabled as boolean"));
 
-    pThis->pDevIns = pDevIns;
+    pThis->pDevIns         = pDevIns;
+    pThis->IOPortBaseCfg   = u16Base;
+    pThis->fSpeakerCfg     = fSpeaker;
     pThis->channels[0].irq = u8Irq;
     for (i = 0; i < RT_ELEMENTS(pThis->channels); i++)
     {
@@ -986,10 +1074,19 @@ static DECLCALLBACK(int)  pitConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
         pThis->channels[i].pPitRC = PDMINS_2_DATA_RCPTR(pDevIns);
     }
 
+    /*
+     * Interfaces
+     */
+    /* IBase */
+    pDevIns->IBase.pfnQueryInterface        = pitQueryInterface;
+    /* IHpetLegacyNotify */
+    pThis->IHpetLegacyNotify.pfnModeChanged = pitNotifyHpetLegacyNotify_ModeChanged;
+
     /*
      * Create timer, register I/O Ports and save state.
      */
-    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, pitTimer, "i8254 Programmable Interval Timer",
+    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, pitTimer, &pThis->channels[0],
+                                TMTIMER_FLAGS_DEFAULT_CRIT_SECT, "i8254 Programmable Interval Timer",
                                 &pThis->channels[0].pTimerR3);
     if (RT_FAILURE(rc))
         return rc;
@@ -1001,7 +1098,7 @@ static DECLCALLBACK(int)  pitConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
         return rc;
     if (fGCEnabled)
     {
-        rc = PDMDevHlpIOPortRegisterGC(pDevIns, u16Base, 4, 0, "pitIOPortWrite", "pitIOPortRead", NULL, NULL, "i8254 Programmable Interval Timer");
+        rc = PDMDevHlpIOPortRegisterRC(pDevIns, u16Base, 4, 0, "pitIOPortWrite", "pitIOPortRead", NULL, NULL, "i8254 Programmable Interval Timer");
         if (RT_FAILURE(rc))
             return rc;
     }
@@ -1019,15 +1116,13 @@ static DECLCALLBACK(int)  pitConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
             return rc;
         if (fGCEnabled)
         {
-            rc = PDMDevHlpIOPortRegisterGC(pDevIns, 0x61, 1, 0, NULL, "pitIOPortSpeakerRead", NULL, NULL, "PC Speaker");
+            rc = PDMDevHlpIOPortRegisterRC(pDevIns, 0x61, 1, 0, NULL, "pitIOPortSpeakerRead", NULL, NULL, "PC Speaker");
             if (RT_FAILURE(rc))
                 return rc;
         }
     }
 
-    rc = PDMDevHlpSSMRegister(pDevIns, pDevIns->pDevReg->szDeviceName, iInstance, PIT_SAVED_STATE_VERSION, sizeof(*pThis),
-                                          NULL, pitSaveExec, NULL,
-                                          NULL, pitLoadExec, NULL);
+    rc = PDMDevHlpSSMRegister3(pDevIns, PIT_SAVED_STATE_VERSION, sizeof(*pThis), pitLiveExec, pitSaveExec, pitLoadExec);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -1055,7 +1150,7 @@ const PDMDEVREG g_DeviceI8254 =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "i8254",
     /* szRCMod */
     "VBoxDDGC.gc",
@@ -1091,7 +1186,7 @@ const PDMDEVREG g_DeviceI8254 =
     NULL,
     /* pfnDetach */
     NULL,
-    /* pfnQueryInterface. */
+    /* pfnQueryInterface */
     NULL,
     /* pfnInitComplete */
     NULL,
