@@ -1,10 +1,10 @@
-/* $Id: DevRTC.cpp 13840 2008-11-05 03:31:46Z vboxsync $ */
+/* $Id: DevRTC.cpp 29250 2010-05-09 17:53:58Z vboxsync $ */
 /** @file
- * Motorola MC146818 RTC/CMOS Device.
+ * Motorola MC146818 RTC/CMOS Device with PIIX4 extensions.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -50,10 +46,15 @@
 #define LOG_GROUP LOG_GROUP_DEV_RTC
 #include <VBox/pdmdev.h>
 #include <VBox/log.h>
-#include <iprt/asm.h>
+#include <iprt/asm-math.h>
 #include <iprt/assert.h>
 #include <iprt/string.h>
 
+#ifdef IN_RING3
+# include <iprt/alloc.h>
+# include <iprt/uuid.h>
+#endif /* IN_RING3 */
+
 #include "../Builtins.h"
 
 struct RTCState;
@@ -69,13 +70,13 @@ typedef struct RTCState RTCState;
 *   Internal Functions                                                         *
 *******************************************************************************/
 #ifndef VBOX_DEVICE_STRUCT_TESTCASE
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 PDMBOTHCBDECL(int) rtcIOPortRead(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Port, uint32_t *pu32, unsigned cb);
 PDMBOTHCBDECL(int) rtcIOPortWrite(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Port, uint32_t u32, unsigned cb);
-PDMBOTHCBDECL(void) rtcTimerPeriodic(PPDMDEVINS pDevIns, PTMTIMER pTimer);
-PDMBOTHCBDECL(void) rtcTimerSecond(PPDMDEVINS pDevIns, PTMTIMER pTimer);
-PDMBOTHCBDECL(void) rtcTimerSecond2(PPDMDEVINS pDevIns, PTMTIMER pTimer);
-__END_DECLS
+PDMBOTHCBDECL(void) rtcTimerPeriodic(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser);
+PDMBOTHCBDECL(void) rtcTimerSecond(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser);
+PDMBOTHCBDECL(void) rtcTimerSecond2(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser);
+RT_C_DECLS_END
 #endif /* !VBOX_DEVICE_STRUCT_TESTCASE */
 
 
@@ -110,6 +111,19 @@ __END_DECLS
 #define REG_B_UIE 0x10
 
 
+/** The saved state version. */
+#define RTC_SAVED_STATE_VERSION             4
+/** The saved state version used by VirtualBox pre-3.2.
+ * This does not include the second 128-byte bank. */
+#define RTC_SAVED_STATE_VERSION_VBOX_32PRE  3
+/** The saved state version used by VirtualBox 3.1 and earlier.
+ * This does not include disabled by HPET state. */
+#define RTC_SAVED_STATE_VERSION_VBOX_31     2
+/** The saved state version used by VirtualBox 3.0 and earlier.
+ * This does not include the configuration.  */
+#define RTC_SAVED_STATE_VERSION_VBOX_30     1
+
+
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
 *******************************************************************************/
@@ -128,13 +142,18 @@ struct my_tm
 
 
 struct RTCState {
-    uint8_t cmos_data[128];
-    uint8_t cmos_index;
-    uint8_t Alignment0[7];
+    uint8_t cmos_data[256];
+    uint8_t cmos_index[2];
+    uint8_t Alignment0[6];
     struct my_tm current_tm;
+    /** The configured IRQ. */
     int32_t irq;
+    /** The configured I/O port base. */
+    RTIOPORT IOPortBase;
     /** Use UTC or local time initially. */
     bool fUTC;
+    /** Disabled by HPET legacy mode. */
+    bool fDisabledByHpet;
     /* periodic timer */
     int64_t next_periodic_time;
     /* second update */
@@ -175,6 +194,9 @@ struct RTCState {
     uint32_t cRelLogEntries;
     /** The current/previous timer period. Used to prevent flooding changes. */
     int32_t CurPeriod;
+
+    /** HPET legacy mode notification interface. */
+    PDMIHPETLEGACYNOTIFY  IHpetLegacyNotify;
 };
 
 #ifndef VBOX_DEVICE_STRUCT_TESTCASE
@@ -215,28 +237,38 @@ static void rtc_timer_update(RTCState *s, int64_t current_time)
     }
 }
 
+static void rtc_raise_irq(RTCState* pThis, uint32_t iLevel)
+{
+    if (!pThis->fDisabledByHpet)
+        PDMDevHlpISASetIrq(pThis->CTX_SUFF(pDevIns), pThis->irq, iLevel);
+}
+
 static void rtc_periodic_timer(void *opaque)
 {
     RTCState *s = (RTCState*)opaque;
 
     rtc_timer_update(s, s->next_periodic_time);
     s->cmos_data[RTC_REG_C] |= 0xc0;
-    PDMDevHlpISASetIrq(s->CTX_SUFF(pDevIns), s->irq, 1);
+
+    rtc_raise_irq(s, 1);
 }
 
 static void cmos_ioport_write(void *opaque, uint32_t addr, uint32_t data)
 {
     RTCState *s = (RTCState*)opaque;
+    uint32_t bank;
 
+    bank = (addr >> 1) & 1;
     if ((addr & 1) == 0) {
-        s->cmos_index = data & 0x7f;
+        s->cmos_index[bank] = (data & 0x7f) + (bank * 128);
     } else {
-        Log(("CMOS: Write idx %#04x: %#04x (old %#04x)\n", s->cmos_index, data, s->cmos_data[s->cmos_index]));
-        switch(s->cmos_index) {
+        Log(("CMOS: Write bank %d idx %#04x: %#04x (old %#04x)\n", bank,
+             s->cmos_index[bank], data, s->cmos_data[s->cmos_index[bank]]));
+        switch(s->cmos_index[bank]) {
         case RTC_SECONDS_ALARM:
         case RTC_MINUTES_ALARM:
         case RTC_HOURS_ALARM:
-            s->cmos_data[s->cmos_index] = data;
+            s->cmos_data[s->cmos_index[0]] = data;
             break;
         case RTC_SECONDS:
         case RTC_MINUTES:
@@ -245,7 +277,7 @@ static void cmos_ioport_write(void *opaque, uint32_t addr, uint32_t data)
         case RTC_DAY_OF_MONTH:
         case RTC_MONTH:
         case RTC_YEAR:
-            s->cmos_data[s->cmos_index] = data;
+            s->cmos_data[s->cmos_index[0]] = data;
             /* if in set mode, do not update the time */
             if (!(s->cmos_data[RTC_REG_B] & REG_B_SET)) {
                 rtc_set_time(s);
@@ -278,7 +310,7 @@ static void cmos_ioport_write(void *opaque, uint32_t addr, uint32_t data)
             /* cannot write to them */
             break;
         default:
-            s->cmos_data[s->cmos_index] = data;
+            s->cmos_data[s->cmos_index[bank]] = data;
             break;
         }
     }
@@ -407,6 +439,7 @@ static void rtc_update_second(void *opaque)
 
         if (!(s->cmos_data[RTC_REG_B] & REG_B_SET)) {
             /* update in progress bit */
+            Log2(("RTC: UIP %x -> 1\n", !!(s->cmos_data[RTC_REG_A] & REG_A_UIP)));
             s->cmos_data[RTC_REG_A] |= REG_A_UIP;
         }
 
@@ -434,17 +467,18 @@ static void rtc_update_second2(void *opaque)
              from_bcd(s, s->cmos_data[RTC_HOURS_ALARM]) == s->current_tm.tm_hour)) {
 
             s->cmos_data[RTC_REG_C] |= 0xa0;
-            PDMDevHlpISASetIrq(s->CTX_SUFF(pDevIns), s->irq, 1);
+            rtc_raise_irq(s, 1);
         }
     }
 
     /* update ended interrupt */
     if (s->cmos_data[RTC_REG_B] & REG_B_UIE) {
         s->cmos_data[RTC_REG_C] |= 0x90;
-        PDMDevHlpISASetIrq(s->CTX_SUFF(pDevIns), s->irq, 1);
+        rtc_raise_irq(s, 1);
     }
 
     /* clear update in progress bit */
+    Log2(("RTC: UIP %x -> 0\n", !!(s->cmos_data[RTC_REG_A] & REG_A_UIP)));
     s->cmos_data[RTC_REG_A] &= ~REG_A_UIP;
 
     s->next_second_time += TMTimerGetFreq(s->CTX_SUFF(pSecondTimer));
@@ -455,10 +489,13 @@ static uint32_t cmos_ioport_read(void *opaque, uint32_t addr)
 {
     RTCState *s = (RTCState*)opaque;
     int ret;
+    unsigned bank;
+
+    bank = (addr >> 1) & 1;
     if ((addr & 1) == 0) {
         return 0xff;
     } else {
-        switch(s->cmos_index) {
+        switch(s->cmos_index[bank]) {
         case RTC_SECONDS:
         case RTC_MINUTES:
         case RTC_HOURS:
@@ -466,21 +503,21 @@ static uint32_t cmos_ioport_read(void *opaque, uint32_t addr)
         case RTC_DAY_OF_MONTH:
         case RTC_MONTH:
         case RTC_YEAR:
-            ret = s->cmos_data[s->cmos_index];
+            ret = s->cmos_data[s->cmos_index[0]];
             break;
         case RTC_REG_A:
-            ret = s->cmos_data[s->cmos_index];
+            ret = s->cmos_data[s->cmos_index[0]];
             break;
         case RTC_REG_C:
-            ret = s->cmos_data[s->cmos_index];
-            PDMDevHlpISASetIrq(s->CTX_SUFF(pDevIns), s->irq, 0);
+            ret = s->cmos_data[s->cmos_index[0]];
+            rtc_raise_irq(s, 0);
             s->cmos_data[RTC_REG_C] = 0x00;
             break;
         default:
-            ret = s->cmos_data[s->cmos_index];
+            ret = s->cmos_data[s->cmos_index[bank]];
             break;
         }
-        Log(("CMOS: Read idx %#04x: %#04x\n", s->cmos_index, ret));
+        Log(("CMOS: Read bank %d idx %#04x: %#04x\n", bank, s->cmos_index[bank], ret));
         return ret;
     }
 }
@@ -550,10 +587,11 @@ PDMBOTHCBDECL(int) rtcIOPortWrite(PPDMDEVINS pDevIns, void *pvUser, RTIOPORT Por
  *
  * @param   pDevIns         Device instance of the device which registered the timer.
  * @param   pTimer          The timer handle.
+ * @param   pvUser          Pointer to the RTC state.
  */
-PDMBOTHCBDECL(void) rtcTimerPeriodic(PPDMDEVINS pDevIns, PTMTIMER pTimer)
+PDMBOTHCBDECL(void) rtcTimerPeriodic(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
 {
-    rtc_periodic_timer(PDMINS_2_DATA(pDevIns, RTCState *));
+    rtc_periodic_timer((RTCState *)pvUser);
 }
 
 
@@ -562,10 +600,11 @@ PDMBOTHCBDECL(void) rtcTimerPeriodic(PPDMDEVINS pDevIns, PTMTIMER pTimer)
  *
  * @param   pDevIns         Device instance of the device which registered the timer.
  * @param   pTimer          The timer handle.
+ * @param   pvUser          Pointer to the RTC state.
  */
-PDMBOTHCBDECL(void) rtcTimerSecond(PPDMDEVINS pDevIns, PTMTIMER pTimer)
+PDMBOTHCBDECL(void) rtcTimerSecond(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
 {
-    rtc_update_second(PDMINS_2_DATA(pDevIns, RTCState *));
+    rtc_update_second((RTCState *)pvUser);
 }
 
 
@@ -574,81 +613,132 @@ PDMBOTHCBDECL(void) rtcTimerSecond(PPDMDEVINS pDevIns, PTMTIMER pTimer)
  *
  * @param   pDevIns         Device instance of the device which registered the timer.
  * @param   pTimer          The timer handle.
+ * @param   pvUser          Pointer to the RTC state.
  */
-PDMBOTHCBDECL(void) rtcTimerSecond2(PPDMDEVINS pDevIns, PTMTIMER pTimer)
+PDMBOTHCBDECL(void) rtcTimerSecond2(PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
 {
-    rtc_update_second2(PDMINS_2_DATA(pDevIns, RTCState *));
+    rtc_update_second2((RTCState *)pvUser);
 }
 
-
 #ifdef IN_RING3
+
 /**
- * Saves a state of the programmable interval timer device.
- *
- * @returns VBox status code.
- * @param   pDevIns     The device instance.
- * @param   pSSMHandle  The handle to save the state to.
+ * @copydoc FNSSMDEVLIVEEXEC
  */
-static DECLCALLBACK(int) rtcSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) rtcLiveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t uPass)
 {
     RTCState *pThis = PDMINS_2_DATA(pDevIns, RTCState *);
 
-    SSMR3PutMem(pSSMHandle, pThis->cmos_data, 128);
-    SSMR3PutU8(pSSMHandle, pThis->cmos_index);
+    SSMR3PutU8(    pSSM, pThis->irq);
+    SSMR3PutIOPort(pSSM, pThis->IOPortBase);
+    SSMR3PutBool(  pSSM, pThis->fUTC);
 
-    SSMR3PutS32(pSSMHandle, pThis->current_tm.tm_sec);
-    SSMR3PutS32(pSSMHandle, pThis->current_tm.tm_min);
-    SSMR3PutS32(pSSMHandle, pThis->current_tm.tm_hour);
-    SSMR3PutS32(pSSMHandle, pThis->current_tm.tm_wday);
-    SSMR3PutS32(pSSMHandle, pThis->current_tm.tm_mday);
-    SSMR3PutS32(pSSMHandle, pThis->current_tm.tm_mon);
-    SSMR3PutS32(pSSMHandle, pThis->current_tm.tm_year);
-
-    TMR3TimerSave(pThis->CTX_SUFF(pPeriodicTimer), pSSMHandle);
-
-    SSMR3PutS64(pSSMHandle, pThis->next_periodic_time);
-
-    SSMR3PutS64(pSSMHandle, pThis->next_second_time);
-    TMR3TimerSave(pThis->CTX_SUFF(pSecondTimer), pSSMHandle);
-    TMR3TimerSave(pThis->CTX_SUFF(pSecondTimer2), pSSMHandle);
-
-    return VINF_SUCCESS;
+    return VINF_SSM_DONT_CALL_AGAIN;
 }
 
 
 /**
- * Loads a saved programmable interval timer device state.
- *
- * @returns VBox status code.
- * @param   pDevIns     The device instance.
- * @param   pSSMHandle  The handle to the saved state.
- * @param   u32Version  The data unit version number.
+ * @copydoc FNSSMDEVSAVEEXEC
  */
-static DECLCALLBACK(int) rtcLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle, uint32_t u32Version)
+static DECLCALLBACK(int) rtcSaveExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSM)
 {
     RTCState *pThis = PDMINS_2_DATA(pDevIns, RTCState *);
 
-    if (u32Version != 1)
+    /* The config. */
+    rtcLiveExec(pDevIns, pSSM, SSM_PASS_FINAL);
+
+    /* The state. */
+    SSMR3PutMem(pSSM, pThis->cmos_data, 128);
+    SSMR3PutU8(pSSM, pThis->cmos_index[0]);
+
+    SSMR3PutS32(pSSM, pThis->current_tm.tm_sec);
+    SSMR3PutS32(pSSM, pThis->current_tm.tm_min);
+    SSMR3PutS32(pSSM, pThis->current_tm.tm_hour);
+    SSMR3PutS32(pSSM, pThis->current_tm.tm_wday);
+    SSMR3PutS32(pSSM, pThis->current_tm.tm_mday);
+    SSMR3PutS32(pSSM, pThis->current_tm.tm_mon);
+    SSMR3PutS32(pSSM, pThis->current_tm.tm_year);
+
+    TMR3TimerSave(pThis->CTX_SUFF(pPeriodicTimer), pSSM);
+
+    SSMR3PutS64(pSSM, pThis->next_periodic_time);
+
+    SSMR3PutS64(pSSM, pThis->next_second_time);
+    TMR3TimerSave(pThis->CTX_SUFF(pSecondTimer), pSSM);
+    TMR3TimerSave(pThis->CTX_SUFF(pSecondTimer2), pSSM);
+
+    SSMR3PutBool(pSSM, pThis->fDisabledByHpet);
+
+    SSMR3PutMem(pSSM, &pThis->cmos_data[128], 128);
+    return SSMR3PutU8(pSSM, pThis->cmos_index[1]);
+}
+
+
+/**
+ * @copydoc FNSSMDEVLOADEXEC
+ */
+static DECLCALLBACK(int) rtcLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
+{
+    RTCState   *pThis = PDMINS_2_DATA(pDevIns, RTCState *);
+    int         rc;
+
+    if (    uVersion != RTC_SAVED_STATE_VERSION
+        &&  uVersion != RTC_SAVED_STATE_VERSION_VBOX_32PRE
+        &&  uVersion != RTC_SAVED_STATE_VERSION_VBOX_31
+        &&  uVersion != RTC_SAVED_STATE_VERSION_VBOX_30)
         return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
 
-    SSMR3GetMem(pSSMHandle, pThis->cmos_data, 128);
-    SSMR3GetU8(pSSMHandle, &pThis->cmos_index);
+    /* The config. */
+    if (uVersion > RTC_SAVED_STATE_VERSION_VBOX_30)
+    {
+        uint8_t u8Irq;
+        rc = SSMR3GetU8(pSSM, &u8Irq);          AssertRCReturn(rc, rc);
+        if (u8Irq != pThis->irq)
+            return SSMR3SetCfgError(pSSM, RT_SRC_POS, N_("Config mismatch - u8Irq: saved=%#x config=%#x"), u8Irq, pThis->irq);
 
-    SSMR3GetS32(pSSMHandle, &pThis->current_tm.tm_sec);
-    SSMR3GetS32(pSSMHandle, &pThis->current_tm.tm_min);
-    SSMR3GetS32(pSSMHandle, &pThis->current_tm.tm_hour);
-    SSMR3GetS32(pSSMHandle, &pThis->current_tm.tm_wday);
-    SSMR3GetS32(pSSMHandle, &pThis->current_tm.tm_mday);
-    SSMR3GetS32(pSSMHandle, &pThis->current_tm.tm_mon);
-    SSMR3GetS32(pSSMHandle, &pThis->current_tm.tm_year);
+        RTIOPORT IOPortBase;
+        rc = SSMR3GetIOPort(pSSM, &IOPortBase); AssertRCReturn(rc, rc);
+        if (IOPortBase != pThis->IOPortBase)
+            return SSMR3SetCfgError(pSSM, RT_SRC_POS, N_("Config mismatch - IOPortBase: saved=%RTiop config=%RTiop"), IOPortBase, pThis->IOPortBase);
 
-    TMR3TimerLoad(pThis->CTX_SUFF(pPeriodicTimer), pSSMHandle);
+        bool fUTC;
+        rc = SSMR3GetBool(pSSM, &fUTC);         AssertRCReturn(rc, rc);
+        if (fUTC != pThis->fUTC)
+            LogRel(("RTC: Config mismatch - fUTC: saved=%RTbool config=%RTbool\n", fUTC, pThis->fUTC));
+    }
 
-    SSMR3GetS64(pSSMHandle, &pThis->next_periodic_time);
+    if (uPass != SSM_PASS_FINAL)
+        return VINF_SUCCESS;
 
-    SSMR3GetS64(pSSMHandle, &pThis->next_second_time);
-    TMR3TimerLoad(pThis->CTX_SUFF(pSecondTimer), pSSMHandle);
-    TMR3TimerLoad(pThis->CTX_SUFF(pSecondTimer2), pSSMHandle);
+    /* The state. */
+    SSMR3GetMem(pSSM, pThis->cmos_data, 128);
+    SSMR3GetU8(pSSM, &pThis->cmos_index[0]);
+
+    SSMR3GetS32(pSSM, &pThis->current_tm.tm_sec);
+    SSMR3GetS32(pSSM, &pThis->current_tm.tm_min);
+    SSMR3GetS32(pSSM, &pThis->current_tm.tm_hour);
+    SSMR3GetS32(pSSM, &pThis->current_tm.tm_wday);
+    SSMR3GetS32(pSSM, &pThis->current_tm.tm_mday);
+    SSMR3GetS32(pSSM, &pThis->current_tm.tm_mon);
+    SSMR3GetS32(pSSM, &pThis->current_tm.tm_year);
+
+    TMR3TimerLoad(pThis->CTX_SUFF(pPeriodicTimer), pSSM);
+
+    SSMR3GetS64(pSSM, &pThis->next_periodic_time);
+
+    SSMR3GetS64(pSSM, &pThis->next_second_time);
+    TMR3TimerLoad(pThis->CTX_SUFF(pSecondTimer), pSSM);
+    TMR3TimerLoad(pThis->CTX_SUFF(pSecondTimer2), pSSM);
+
+    if (uVersion > RTC_SAVED_STATE_VERSION_VBOX_31)
+         SSMR3GetBool(pSSM, &pThis->fDisabledByHpet);
+
+    if (uVersion > RTC_SAVED_STATE_VERSION_VBOX_32PRE)
+    {
+        /* Second CMOS bank. */
+        SSMR3GetMem(pSSM, &pThis->cmos_data[128], 128);
+        SSMR3GetU8(pSSM, &pThis->cmos_index[1]);
+    }
 
     int period_code = pThis->cmos_data[RTC_REG_A] & 0x0f;
     if (    period_code != 0
@@ -663,6 +753,7 @@ static DECLCALLBACK(int) rtcLoadExec(PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle,
         pThis->CurPeriod = 0;
     }
     pThis->cRelLogEntries = 0;
+
     return VINF_SUCCESS;
 }
 
@@ -691,7 +782,7 @@ static void rtcCalcCRC(RTCState *pThis)
  *
  * @returns VBox status code.
  * @param   pDevIns     Device instance of the RTC.
- * @param   iReg        The CMOS register index.
+ * @param   iReg        The CMOS register index; bit 8 determines bank.
  * @param   u8Value     The CMOS register value.
  */
 static DECLCALLBACK(int) rtcCMOSWrite(PPDMDEVINS pDevIns, unsigned iReg, uint8_t u8Value)
@@ -718,7 +809,7 @@ static DECLCALLBACK(int) rtcCMOSWrite(PPDMDEVINS pDevIns, unsigned iReg, uint8_t
  *
  * @returns VBox status code.
  * @param   pDevIns     Device instance of the RTC.
- * @param   iReg        The CMOS register index.
+ * @param   iReg        The CMOS register index; bit 8 determines bank.
  * @param   pu8Value    Where to store the CMOS register value.
  */
 static DECLCALLBACK(int) rtcCMOSRead(PPDMDEVINS pDevIns, unsigned iReg, uint8_t *pu8Value)
@@ -746,7 +837,7 @@ static DECLCALLBACK(int)  rtcInitComplete(PPDMDEVINS pDevIns)
      * Set the CMOS date/time.
      */
     RTTIMESPEC  Now;
-    PDMDevHlpUTCNow(pDevIns, &Now);
+    PDMDevHlpTMUtcNow(pDevIns, &Now);
     RTTIME Time;
     if (pThis->fUTC)
         RTTimeExplode(&Time, &Now);
@@ -758,7 +849,7 @@ static DECLCALLBACK(int)  rtcInitComplete(PPDMDEVINS pDevIns)
     Tm.tm_year = Time.i32Year - 1900;
     Tm.tm_mon  = Time.u8Month - 1;
     Tm.tm_mday = Time.u8MonthDay;
-    Tm.tm_wday = (Time.u8WeekDay - 1 + 7) % 7; /* 0 = monday -> sunday */
+    Tm.tm_wday = (Time.u8WeekDay + 1 + 7) % 7; /* 0 = monday -> sunday */
     Tm.tm_yday = Time.u16YearDay - 1;
     Tm.tm_hour = Time.u8Hour;
     Tm.tm_min  = Time.u8Minute;
@@ -775,13 +866,37 @@ static DECLCALLBACK(int)  rtcInitComplete(PPDMDEVINS pDevIns)
      */
     rtcCalcCRC(pThis);
 
-    Log(("CMOS: \n%16.128Rhxd\n", pThis->cmos_data));
+    Log(("CMOS bank 0: \n%16.128Rhxd\n", &pThis->cmos_data[0]));
+    Log(("CMOS bank 1: \n%16.128Rhxd\n", &pThis->cmos_data[128]));
     return VINF_SUCCESS;
 }
 
 
 /* -=-=-=-=-=- real code -=-=-=-=-=- */
 
+/**
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
+ */
+static DECLCALLBACK(void *) rtcQueryInterface(PPDMIBASE pInterface, const char *pszIID)
+{
+    PPDMDEVINS  pDevIns = RT_FROM_MEMBER(pInterface, PDMDEVINS, IBase);
+    RTCState   *pThis   = PDMINS_2_DATA(pDevIns, RTCState *);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE,             &pDevIns->IBase);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIHPETLEGACYNOTIFY, &pThis->IHpetLegacyNotify);
+    return NULL;
+}
+
+
+/**
+ * @interface_method_impl{PDMIHPETLEGACYNOTIFY,pfnModeChanged}
+ */
+static DECLCALLBACK(void) rtcHpetLegacyNotify_ModeChanged(PPDMIHPETLEGACYNOTIFY pInterface, bool fActivated)
+{
+    RTCState *pThis = RT_FROM_MEMBER(pInterface, RTCState, IHpetLegacyNotify);
+    pThis->fDisabledByHpet = fActivated;
+}
+
+
 /**
  * @copydoc
  */
@@ -797,64 +912,64 @@ static DECLCALLBACK(void) rtcRelocate(PPDMDEVINS pDevIns, RTGCINTPTR offDelta)
 
 
 /**
- * Construct a device instance for a VM.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- *                      If the registration structure is needed, pDevIns->pDevReg points to it.
- * @param   iInstance   Instance number. Use this to figure out which registers and such to use.
- *                      The device number is also found in pDevIns->iInstance, but since it's
- *                      likely to be freqently used PDM passes it as parameter.
- * @param   pCfgHandle  Configuration node handle for the device. Use this to obtain the configuration
- *                      of the device instance. It's also found in pDevIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @interface_method_impl{PDMDEVREG,pfnConstruct}
  */
-static DECLCALLBACK(int)  rtcConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfgHandle)
+static DECLCALLBACK(int)  rtcConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMNODE pCfg)
 {
     RTCState   *pThis = PDMINS_2_DATA(pDevIns, RTCState *);
     int         rc;
-    uint8_t     u8Irq;
-    uint16_t    u16Base;
-    bool        fGCEnabled;
-    bool        fR0Enabled;
     Assert(iInstance == 0);
 
     /*
      * Validate configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "Irq\0" "Base\0" "GCEnabled\0" "R0Enabled\0"))
+    if (!CFGMR3AreValuesValid(pCfg,
+                              "Irq\0"
+                              "Base\0"
+                              "UseUTC\0"
+                              "GCEnabled\0"
+                              "R0Enabled\0"))
         return VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES;
 
     /*
      * Init the data.
      */
-    rc = CFGMR3QueryU8Def(pCfgHandle, "Irq", &u8Irq, 8);
+    uint8_t u8Irq;
+    rc = CFGMR3QueryU8Def(pCfg, "Irq", &u8Irq, 8);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Querying \"Irq\" as a uint8_t failed"));
+    pThis->irq = u8Irq;
 
-    rc = CFGMR3QueryU16Def(pCfgHandle, "Base", &u16Base, 0x70);
+    rc = CFGMR3QueryPortDef(pCfg, "Base", &pThis->IOPortBase, 0x70);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
-                                N_("Configuration error: Querying \"Base\" as a uint16_t failed"));
+                                N_("Configuration error: Querying \"Base\" as a RTIOPORT failed"));
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "GCEnabled", &fGCEnabled, true);
+    rc = CFGMR3QueryBoolDef(pCfg, "UseUTC", &pThis->fUTC, false);
+    if (RT_FAILURE(rc))
+        return PDMDEV_SET_ERROR(pDevIns, rc,
+                                N_("Configuration error: Querying \"UseUTC\" as a bool failed"));
+
+    bool fGCEnabled;
+    rc = CFGMR3QueryBoolDef(pCfg, "GCEnabled", &fGCEnabled, true);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: failed to read GCEnabled as boolean"));
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "R0Enabled", &fR0Enabled, true);
+    bool fR0Enabled;
+    rc = CFGMR3QueryBoolDef(pCfg, "R0Enabled", &fR0Enabled, true);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: failed to read R0Enabled as boolean"));
 
-    Log(("RTC: Irq=%#x Base=%#x fGCEnabled=%RTbool fR0Enabled=%RTbool\n", u8Irq, u16Base, fGCEnabled, fR0Enabled));
+    Log(("RTC: Irq=%#x Base=%#x fGCEnabled=%RTbool fR0Enabled=%RTbool\n",
+         u8Irq, pThis->IOPortBase, fGCEnabled, fR0Enabled));
 
 
     pThis->pDevInsR3            = pDevIns;
     pThis->pDevInsR0            = PDMDEVINS_2_R0PTR(pDevIns);
     pThis->pDevInsRC            = PDMDEVINS_2_RCPTR(pDevIns);
-    pThis->irq                  = u8Irq;
     pThis->cmos_data[RTC_REG_A] = 0x26;
     pThis->cmos_data[RTC_REG_B] = 0x02;
     pThis->cmos_data[RTC_REG_C] = 0x00;
@@ -862,58 +977,72 @@ static DECLCALLBACK(int)  rtcConstruct(PPDMDEVINS pDevIns, int iInstance, PCFGMN
     pThis->RtcReg.u32Version    = PDM_RTCREG_VERSION;
     pThis->RtcReg.pfnRead       = rtcCMOSRead;
     pThis->RtcReg.pfnWrite      = rtcCMOSWrite;
+    pThis->fDisabledByHpet      = false;
+
+    /* IBase */
+    pDevIns->IBase.pfnQueryInterface        = rtcQueryInterface;
+    /* IHpetLegacyNotify */
+    pThis->IHpetLegacyNotify.pfnModeChanged = rtcHpetLegacyNotify_ModeChanged;
 
     /*
      * Create timers, arm them, register I/O Ports and save state.
      */
-    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, rtcTimerPeriodic, "MC146818 RTC/CMOS - Periodic", &pThis->pPeriodicTimerR3);
+    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, rtcTimerPeriodic, pThis,
+                                TMTIMER_FLAGS_DEFAULT_CRIT_SECT, "MC146818 RTC/CMOS - Periodic",
+                                &pThis->pPeriodicTimerR3);
     if (RT_FAILURE(rc))
         return rc;
     pThis->pPeriodicTimerR0 = TMTimerR0Ptr(pThis->pPeriodicTimerR3);
     pThis->pPeriodicTimerRC = TMTimerRCPtr(pThis->pPeriodicTimerR3);
 
-    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, rtcTimerSecond,   "MC146818 RTC/CMOS - Second", &pThis->pSecondTimerR3);
+    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, rtcTimerSecond, pThis,
+                                TMTIMER_FLAGS_DEFAULT_CRIT_SECT, "MC146818 RTC/CMOS - Second",
+                                &pThis->pSecondTimerR3);
     if (RT_FAILURE(rc))
         return rc;
     pThis->pSecondTimerR0 = TMTimerR0Ptr(pThis->pSecondTimerR3);
     pThis->pSecondTimerRC = TMTimerRCPtr(pThis->pSecondTimerR3);
 
-    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, rtcTimerSecond2,  "MC146818 RTC/CMOS - Second2", &pThis->pSecondTimer2R3);
+    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL_SYNC, rtcTimerSecond2, pThis,
+                                TMTIMER_FLAGS_DEFAULT_CRIT_SECT, "MC146818 RTC/CMOS - Second2",
+                                &pThis->pSecondTimer2R3);
     if (RT_FAILURE(rc))
         return rc;
-    pThis->pSecondTimer2R0 = TMTimerR0Ptr(pThis->pSecondTimer2R3);
-    pThis->pSecondTimer2RC = TMTimerRCPtr(pThis->pSecondTimer2R3);
-    pThis->next_second_time = TMTimerGet(pThis->CTX_SUFF(pSecondTimer2)) + (TMTimerGetFreq(pThis->CTX_SUFF(pSecondTimer2)) * 99) / 100;
+    pThis->pSecondTimer2R0  = TMTimerR0Ptr(pThis->pSecondTimer2R3);
+    pThis->pSecondTimer2RC  = TMTimerRCPtr(pThis->pSecondTimer2R3);
+    pThis->next_second_time = TMTimerGet(pThis->CTX_SUFF(pSecondTimer2))
+                            + (TMTimerGetFreq(pThis->CTX_SUFF(pSecondTimer2)) * 99) / 100;
     rc = TMTimerSet(pThis->CTX_SUFF(pSecondTimer2), pThis->next_second_time);
     if (RT_FAILURE(rc))
         return rc;
 
-    rc = PDMDevHlpIOPortRegister(pDevIns, u16Base, 2, NULL, rtcIOPortWrite, rtcIOPortRead, NULL, NULL, "MC146818 RTC/CMOS");
+    rc = PDMDevHlpIOPortRegister(pDevIns, pThis->IOPortBase, 4, NULL,
+                                 rtcIOPortWrite, rtcIOPortRead, NULL, NULL, "MC146818 RTC/CMOS");
     if (RT_FAILURE(rc))
         return rc;
     if (fGCEnabled)
     {
-        rc = PDMDevHlpIOPortRegisterGC(pDevIns, u16Base, 2, 0, "rtcIOPortWrite", "rtcIOPortRead", NULL, NULL, "MC146818 RTC/CMOS");
+        rc = PDMDevHlpIOPortRegisterRC(pDevIns, pThis->IOPortBase, 4, 0,
+                                       "rtcIOPortWrite", "rtcIOPortRead", NULL, NULL, "MC146818 RTC/CMOS");
         if (RT_FAILURE(rc))
             return rc;
     }
     if (fR0Enabled)
     {
-        rc = PDMDevHlpIOPortRegisterR0(pDevIns, u16Base, 2, 0, "rtcIOPortWrite", "rtcIOPortRead", NULL, NULL, "MC146818 RTC/CMOS");
+        rc = PDMDevHlpIOPortRegisterR0(pDevIns, pThis->IOPortBase, 4, 0,
+                                       "rtcIOPortWrite", "rtcIOPortRead", NULL, NULL, "MC146818 RTC/CMOS");
         if (RT_FAILURE(rc))
             return rc;
     }
 
-    rc = PDMDevHlpSSMRegister(pDevIns, pDevIns->pDevReg->szDeviceName, iInstance, 1 /* version */, sizeof(*pThis),
-                              NULL, rtcSaveExec, NULL,
-                              NULL, rtcLoadExec, NULL);
+    rc = PDMDevHlpSSMRegister3(pDevIns, RTC_SAVED_STATE_VERSION, sizeof(*pThis), rtcLiveExec, rtcSaveExec, rtcLoadExec);
     if (RT_FAILURE(rc))
         return rc;
 
     /*
      * Register ourselves as the RTC/CMOS with PDM.
      */
-    rc = pDevIns->pDevHlpR3->pfnRTCRegister(pDevIns, &pThis->RtcReg, &pThis->pRtcHlpR3);
+    rc = PDMDevHlpRTCRegister(pDevIns, &pThis->RtcReg, &pThis->pRtcHlpR3);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -928,7 +1057,7 @@ const PDMDEVREG g_DeviceMC146818 =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "mc146818",
     /* szRCMod */
     "VBoxDDGC.gc",
@@ -975,6 +1104,7 @@ const PDMDEVREG g_DeviceMC146818 =
     /* u32VersionEnd */
     PDM_DEVREG_VERSION
 };
+
 #endif /* IN_RING3 */
 #endif /* !VBOX_DEVICE_STRUCT_TESTCASE */
 
