@@ -1,10 +1,10 @@
-/* $Id: DevSerial.cpp $ */
+/* $Id: DevSerial.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DevSerial - 16450 UART emulation.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*
@@ -71,55 +67,61 @@
 *******************************************************************************/
 #define SERIAL_SAVED_STATE_VERSION  3
 
-#define UART_LCR_DLAB	            0x80	/* Divisor latch access bit */
+#define UART_LCR_DLAB               0x80        /* Divisor latch access bit */
 
-#define UART_IER_MSI	            0x08	/* Enable Modem status interrupt */
-#define UART_IER_RLSI	            0x04	/* Enable receiver line status interrupt */
-#define UART_IER_THRI	            0x02	/* Enable Transmitter holding register int. */
-#define UART_IER_RDI	            0x01	/* Enable receiver data interrupt */
+#define UART_IER_MSI                0x08        /* Enable Modem status interrupt */
+#define UART_IER_RLSI               0x04        /* Enable receiver line status interrupt */
+#define UART_IER_THRI               0x02        /* Enable Transmitter holding register int. */
+#define UART_IER_RDI                0x01        /* Enable receiver data interrupt */
 
-#define UART_IIR_NO_INT	            0x01	/* No interrupts pending */
-#define UART_IIR_ID	                0x06	/* Mask for the interrupt ID */
+#define UART_IIR_NO_INT             0x01        /* No interrupts pending */
+#define UART_IIR_ID                     0x06    /* Mask for the interrupt ID */
 
-#define UART_IIR_MSI	            0x00	/* Modem status interrupt */
-#define UART_IIR_THRI	            0x02	/* Transmitter holding register empty */
-#define UART_IIR_RDI	            0x04	/* Receiver data interrupt */
-#define UART_IIR_RLSI	            0x06	/* Receiver line status interrupt */
+#define UART_IIR_MSI                0x00        /* Modem status interrupt */
+#define UART_IIR_THRI               0x02        /* Transmitter holding register empty */
+#define UART_IIR_RDI                0x04        /* Receiver data interrupt */
+#define UART_IIR_RLSI               0x06        /* Receiver line status interrupt */
 
 /*
  * These are the definitions for the Modem Control Register
  */
-#define UART_MCR_LOOP	            0x10	/* Enable loopback test mode */
-#define UART_MCR_OUT2	            0x08	/* Out2 complement */
-#define UART_MCR_OUT1	            0x04	/* Out1 complement */
-#define UART_MCR_RTS	            0x02	/* RTS complement */
-#define UART_MCR_DTR	            0x01	/* DTR complement */
+#define UART_MCR_LOOP               0x10        /* Enable loopback test mode */
+#define UART_MCR_OUT2               0x08        /* Out2 complement */
+#define UART_MCR_OUT1               0x04        /* Out1 complement */
+#define UART_MCR_RTS                0x02        /* RTS complement */
+#define UART_MCR_DTR                0x01        /* DTR complement */
 
 /*
  * These are the definitions for the Modem Status Register
  */
-#define UART_MSR_DCD	            0x80	/* Data Carrier Detect */
-#define UART_MSR_RI	                0x40	/* Ring Indicator */
-#define UART_MSR_DSR	            0x20	/* Data Set Ready */
-#define UART_MSR_CTS	            0x10	/* Clear to Send */
-#define UART_MSR_DDCD	            0x08	/* Delta DCD */
-#define UART_MSR_TERI	            0x04	/* Trailing edge ring indicator */
-#define UART_MSR_DDSR	            0x02	/* Delta DSR */
-#define UART_MSR_DCTS	            0x01	/* Delta CTS */
-#define UART_MSR_ANY_DELTA          0x0F	/* Any of the delta bits! */
+#define UART_MSR_DCD                0x80        /* Data Carrier Detect */
+#define UART_MSR_RI                     0x40    /* Ring Indicator */
+#define UART_MSR_DSR                0x20        /* Data Set Ready */
+#define UART_MSR_CTS                0x10        /* Clear to Send */
+#define UART_MSR_DDCD               0x08        /* Delta DCD */
+#define UART_MSR_TERI               0x04        /* Trailing edge ring indicator */
+#define UART_MSR_DDSR               0x02        /* Delta DSR */
+#define UART_MSR_DCTS               0x01        /* Delta CTS */
+#define UART_MSR_ANY_DELTA          0x0F        /* Any of the delta bits! */
 
-#define UART_LSR_TEMT	            0x40	/* Transmitter empty */
-#define UART_LSR_THRE	            0x20	/* Transmit-hold-register empty */
-#define UART_LSR_BI	                0x10	/* Break interrupt indicator */
-#define UART_LSR_FE	                0x08	/* Frame error indicator */
-#define UART_LSR_PE	                0x04	/* Parity error indicator */
-#define UART_LSR_OE	                0x02	/* Overrun error indicator */
-#define UART_LSR_DR	                0x01	/* Receiver data ready */
+#define UART_LSR_TEMT               0x40        /* Transmitter empty */
+#define UART_LSR_THRE               0x20        /* Transmit-hold-register empty */
+#define UART_LSR_BI                     0x10    /* Break interrupt indicator */
+#define UART_LSR_FE                     0x08    /* Frame error indicator */
+#define UART_LSR_PE                     0x04    /* Parity error indicator */
+#define UART_LSR_OE                     0x02    /* Overrun error indicator */
+#define UART_LSR_DR                     0x01    /* Receiver data ready */
 
 
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
 *******************************************************************************/
+/**
+ * Serial device.
+ *
+ * @implements  PDMIBASE
+ * @implements  PDMICHARPORT
+ */
 struct SerialState
 {
     /** Access critical section. */
@@ -132,14 +134,14 @@ struct SerialState
     /** Pointer to the device instance - RC Ptr. */
     PPDMDEVINSRC                    pDevInsRC;
     RTRCPTR                         Alignment0; /**< Alignment. */
-    /** The base interface. */
+    /** LUN\#0: The base interface. */
     PDMIBASE                        IBase;
-    /** The character port interface. */
+    /** LUN\#0: The character port interface. */
     PDMICHARPORT                    ICharPort;
     /** Pointer to the attached base driver. */
     R3PTRTYPE(PPDMIBASE)            pDrvBase;
     /** Pointer to the attached character driver. */
-    R3PTRTYPE(PPDMICHAR)            pDrvChar;
+    R3PTRTYPE(PPDMICHARCONNECTOR)            pDrvChar;
 
     uint16_t                        divider;
     uint16_t                        auAlignment[3];
@@ -199,6 +201,8 @@ static void serial_update_irq(SerialState *s)
         s->iir = UART_IIR_THRI;
     } else if (s->msr_changed && (s->ier & UART_IER_RLSI)) {
         s->iir = UART_IIR_RLSI;
+    } else if (s->lsr & UART_LSR_BI) {
+        s->iir = 0; /* No special status bit */
     } else {
         s->iir = UART_IIR_NO_INT;
     }
@@ -265,6 +269,21 @@ static int serial_ioport_write(void *opaque, uint32_t addr, uint32_t val)
         if (s->lcr & UART_LCR_DLAB) {
             s->divider = (s->divider & 0xff00) | val;
             serial_update_parameters(s);
+#if 0 /* disabled because this causes regressions */
+        } else if (s->lsr & UART_LSR_THRE) {
+            s->thr_ipending = 0;
+            ch = val;
+            if (RT_LIKELY(s->pDrvChar))
+            {
+                Log(("serial_ioport_write: write 0x%X\n", ch));
+                int rc = s->pDrvChar->pfnWrite(s->pDrvChar, &ch, 1);
+                AssertRC(rc);
+            }
+            s->thr_ipending = 1;
+            serial_update_irq(s);
+        } else
+            Log(("serial: THR not EMPTY!\n"));
+#else
         } else {
             s->thr_ipending = 0;
             s->lsr &= ~UART_LSR_THRE;
@@ -272,7 +291,7 @@ static int serial_ioport_write(void *opaque, uint32_t addr, uint32_t val)
             ch = val;
             if (RT_LIKELY(s->pDrvChar))
             {
-                Log(("serial_io_port_write: write 0x%X\n", ch));
+                Log(("serial_ioport_write: write 0x%X\n", ch));
                 int rc = s->pDrvChar->pfnWrite(s->pDrvChar, &ch, 1);
                 AssertRC(rc);
             }
@@ -281,6 +300,7 @@ static int serial_ioport_write(void *opaque, uint32_t addr, uint32_t val)
             s->lsr |= UART_LSR_TEMT;
             serial_update_irq(s);
         }
+#endif
         break;
     case 1:
         if (s->lcr & UART_LCR_DLAB) {
@@ -307,6 +327,12 @@ static int serial_ioport_write(void *opaque, uint32_t addr, uint32_t val)
             break_enable = (val >> 6) & 1;
             if (break_enable != s->last_break_enable) {
                 s->last_break_enable = break_enable;
+                if (RT_LIKELY(s->pDrvChar))
+                {
+                    Log(("serial_ioport_write: Set break %d\n", break_enable));
+                    int rc = s->pDrvChar->pfnSetBreak(s->pDrvChar, !!break_enable);
+                    AssertRC(rc);
+                }
             }
         }
         break;
@@ -465,13 +491,13 @@ static DECLCALLBACK(int) serialNotifyStatusLinesChanged(PPDMICHARPORT pInterface
     PDMCritSectEnter(&pThis->CritSect, VERR_PERMISSION_DENIED);
 
     /* Set new states. */
-    if (newStatusLines & PDM_ICHAR_STATUS_LINES_DCD)
+    if (newStatusLines & PDMICHARPORT_STATUS_LINES_DCD)
         newMsr |= UART_MSR_DCD;
-    if (newStatusLines & PDM_ICHAR_STATUS_LINES_RI)
+    if (newStatusLines & PDMICHARPORT_STATUS_LINES_RI)
         newMsr |= UART_MSR_RI;
-    if (newStatusLines & PDM_ICHAR_STATUS_LINES_DSR)
+    if (newStatusLines & PDMICHARPORT_STATUS_LINES_DSR)
         newMsr |= UART_MSR_DSR;
-    if (newStatusLines & PDM_ICHAR_STATUS_LINES_CTS)
+    if (newStatusLines & PDMICHARPORT_STATUS_LINES_CTS)
         newMsr |= UART_MSR_CTS;
 
     /* Compare the old and the new states and set the delta bits accordingly. */
@@ -493,6 +519,43 @@ static DECLCALLBACK(int) serialNotifyStatusLinesChanged(PPDMICHARPORT pInterface
     return VINF_SUCCESS;
 }
 
+static DECLCALLBACK(int) serialNotifyBufferFull(PPDMICHARPORT pInterface, bool fFull)
+{
+#if 0
+    SerialState *pThis = PDMICHARPORT_2_SERIALSTATE(pInterface);
+    PDMCritSectEnter(&pThis->CritSect, VERR_PERMISSION_DENIED);
+    if (fFull)
+    {
+        pThis->lsr &= ~UART_LSR_THRE;
+    }
+    else
+    {
+        pThis->thr_ipending = 1;
+        pThis->lsr |= UART_LSR_THRE;
+        pThis->lsr |= UART_LSR_TEMT;
+    }
+    serial_update_irq(pThis);
+    PDMCritSectLeave(&pThis->CritSect);
+#endif
+    return VINF_SUCCESS;
+}
+
+static DECLCALLBACK(int) serialNotifyBreak(PPDMICHARPORT pInterface)
+{
+    SerialState *pThis = PDMICHARPORT_2_SERIALSTATE(pInterface);
+
+    Log(("%s: pInterface=%p\n", __FUNCTION__, pInterface));
+
+    PDMCritSectEnter(&pThis->CritSect, VERR_PERMISSION_DENIED);
+
+    pThis->lsr |= UART_LSR_BI;
+    serial_update_irq(pThis);
+
+    PDMCritSectLeave(&pThis->CritSect);
+
+    return VINF_SUCCESS;
+}
+
 #endif /* IN_RING3 */
 
 /**
@@ -564,91 +627,104 @@ PDMBOTHCBDECL(int) serialIOPortRead(PPDMDEVINS pDevIns, void *pvUser,
 #ifdef IN_RING3
 
 /**
- * Saves a state of the serial port device.
- *
- * @returns VBox status code.
- * @param   pDevIns     The device instance.
- * @param   pSSMHandle  The handle to save the state to.
+ * @copydoc FNSSMDEVLIVEEXEC
  */
-static DECLCALLBACK(int) serialSaveExec(PPDMDEVINS pDevIns,
-                                        PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) serialLiveExec(PPDMDEVINS pDevIns,
+                                        PSSMHANDLE pSSM,
+                                        uint32_t uPass)
 {
     SerialState *pThis = PDMINS_2_DATA(pDevIns, SerialState *);
-
-    SSMR3PutU16(pSSMHandle, pThis->divider);
-    SSMR3PutU8(pSSMHandle, pThis->rbr);
-    SSMR3PutU8(pSSMHandle, pThis->ier);
-    SSMR3PutU8(pSSMHandle, pThis->lcr);
-    SSMR3PutU8(pSSMHandle, pThis->mcr);
-    SSMR3PutU8(pSSMHandle, pThis->lsr);
-    SSMR3PutU8(pSSMHandle, pThis->msr);
-    SSMR3PutU8(pSSMHandle, pThis->scr);
-    SSMR3PutS32(pSSMHandle, pThis->thr_ipending);
-    SSMR3PutS32(pSSMHandle, pThis->irq);
-    SSMR3PutS32(pSSMHandle, pThis->last_break_enable);
-    SSMR3PutU32(pSSMHandle, pThis->base);
-    SSMR3PutBool(pSSMHandle, pThis->msr_changed);
-    return SSMR3PutU32(pSSMHandle, ~0); /* sanity/terminator */
+    SSMR3PutS32(pSSM, pThis->irq);
+    SSMR3PutU32(pSSM, pThis->base);
+    return VINF_SSM_DONT_CALL_AGAIN;
 }
 
 /**
- * Loads a saved serial port device state.
- *
- * @returns VBox status code.
- * @param   pDevIns     The device instance.
- * @param   pSSMHandle  The handle to the saved state.
- * @param   u32Version  The data unit version number.
+ * @copydoc FNSSMDEVSAVEEXEC
  */
-static DECLCALLBACK(int) serialLoadExec(PPDMDEVINS pDevIns,
-                                        PSSMHANDLE pSSMHandle,
-                                        uint32_t u32Version)
+static DECLCALLBACK(int) serialSaveExec(PPDMDEVINS pDevIns,
+                                        PSSMHANDLE pSSM)
 {
-    int          rc;
-    uint32_t     u32;
     SerialState *pThis = PDMINS_2_DATA(pDevIns, SerialState *);
 
-    if (u32Version != SERIAL_SAVED_STATE_VERSION)
+    SSMR3PutU16(pSSM, pThis->divider);
+    SSMR3PutU8(pSSM, pThis->rbr);
+    SSMR3PutU8(pSSM, pThis->ier);
+    SSMR3PutU8(pSSM, pThis->lcr);
+    SSMR3PutU8(pSSM, pThis->mcr);
+    SSMR3PutU8(pSSM, pThis->lsr);
+    SSMR3PutU8(pSSM, pThis->msr);
+    SSMR3PutU8(pSSM, pThis->scr);
+    SSMR3PutS32(pSSM, pThis->thr_ipending);
+    SSMR3PutS32(pSSM, pThis->irq);
+    SSMR3PutS32(pSSM, pThis->last_break_enable);
+    SSMR3PutU32(pSSM, pThis->base);
+    SSMR3PutBool(pSSM, pThis->msr_changed);
+    return SSMR3PutU32(pSSM, ~0); /* sanity/terminator */
+}
+
+/**
+ * @copydoc FNSSMDEVLOADEXEC
+ */
+static DECLCALLBACK(int) serialLoadExec(PPDMDEVINS pDevIns,
+                                        PSSMHANDLE pSSM,
+                                        uint32_t uVersion,
+                                        uint32_t uPass)
+{
+    SerialState *pThis = PDMINS_2_DATA(pDevIns, SerialState *);
+
+    AssertMsgReturn(uVersion == SERIAL_SAVED_STATE_VERSION, ("%d\n", uVersion), VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION);
+
+    if (uPass == SSM_PASS_FINAL)
     {
-        AssertMsgFailed(("u32Version=%d\n", u32Version));
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
+        SSMR3GetU16(pSSM, &pThis->divider);
+        SSMR3GetU8(pSSM, &pThis->rbr);
+        SSMR3GetU8(pSSM, &pThis->ier);
+        SSMR3GetU8(pSSM, &pThis->lcr);
+        SSMR3GetU8(pSSM, &pThis->mcr);
+        SSMR3GetU8(pSSM, &pThis->lsr);
+        SSMR3GetU8(pSSM, &pThis->msr);
+        SSMR3GetU8(pSSM, &pThis->scr);
+        SSMR3GetS32(pSSM, &pThis->thr_ipending);
     }
 
-    SSMR3GetU16(pSSMHandle, &pThis->divider);
-    SSMR3GetU8(pSSMHandle, &pThis->rbr);
-    SSMR3GetU8(pSSMHandle, &pThis->ier);
-    SSMR3GetU8(pSSMHandle, &pThis->lcr);
-    SSMR3GetU8(pSSMHandle, &pThis->mcr);
-    SSMR3GetU8(pSSMHandle, &pThis->lsr);
-    SSMR3GetU8(pSSMHandle, &pThis->msr);
-    SSMR3GetU8(pSSMHandle, &pThis->scr);
-    SSMR3GetS32(pSSMHandle, &pThis->thr_ipending);
-    SSMR3GetS32(pSSMHandle, &pThis->irq);
-    SSMR3GetS32(pSSMHandle, &pThis->last_break_enable);
-    SSMR3GetU32(pSSMHandle, &pThis->base);
-    SSMR3GetBool(pSSMHandle, &pThis->msr_changed);
+    int32_t  iIrq;
+    SSMR3GetS32(pSSM, &iIrq);
 
-    rc = SSMR3GetU32(pSSMHandle, &u32);
-    if (RT_FAILURE(rc))
-        return rc;
+    if (uPass == SSM_PASS_FINAL)
+        SSMR3GetS32(pSSM, &pThis->last_break_enable);
 
-    if (u32 != ~0U)
+    uint32_t IOBase;
+    int rc = SSMR3GetU32(pSSM, &IOBase);
+    AssertRCReturn(rc, rc);
+
+    if (    pThis->irq  != iIrq
+        ||  pThis->base != IOBase)
+        return SSMR3SetCfgError(pSSM, RT_SRC_POS, N_("Config mismatch - saved irq=%#x iobase=%#x; configured irq=%#x iobase=%#x"),
+                                iIrq, IOBase, pThis->irq, pThis->base);
+
+    if (uPass == SSM_PASS_FINAL)
     {
-        AssertLogRelMsgFailed(("u32=%#x expected ~0\n", u32));
-        return VERR_SSM_DATA_UNIT_FORMAT_CHANGED;
-    }
-    /* Be careful with pointers in the structure; they are not preserved
-     * in the saved state. */
+        SSMR3GetBool(pSSM, &pThis->msr_changed);
 
-    if (pThis->lsr & UART_LSR_DR)
-    {
-        int rc = RTSemEventSignal(pThis->ReceiveSem);
-        AssertRC(rc);
+        uint32_t u32;
+        rc = SSMR3GetU32(pSSM, &u32);
+        if (RT_FAILURE(rc))
+            return rc;
+        AssertMsgReturn(u32 == ~0U, ("%#x\n", u32), VERR_SSM_DATA_UNIT_FORMAT_CHANGED);
+
+        if (pThis->lsr & UART_LSR_DR)
+        {
+            rc = RTSemEventSignal(pThis->ReceiveSem);
+            AssertRC(rc);
+        }
+
+        /* this isn't strictly necessary but cannot hurt... */
+        pThis->pDevInsR3 = pDevIns;
+        pThis->pDevInsR0 = PDMDEVINS_2_R0PTR(pDevIns);
+        pThis->pDevInsRC = PDMDEVINS_2_RCPTR(pDevIns);
     }
 
-    /* this isn't strictly necessary but cannot hurt... */
-    pThis->pDevInsR3 = pDevIns;
-    pThis->pDevInsR0 = PDMDEVINS_2_R0PTR(pDevIns);
-    pThis->pDevInsRC = PDMDEVINS_2_RCPTR(pDevIns);
     return VINF_SUCCESS;
 }
 
@@ -688,20 +764,15 @@ static DECLCALLBACK(int) serialIOPortRegionMap(PPCIDEVICE pPciDev, /* unsigned *
 
 #endif /* VBOX_SERIAL_PCI */
 
-
-/** @copyfrom PIBASE::pfnqueryInterface */
-static DECLCALLBACK(void *) serialQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface)
+/**
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
+ */
+static DECLCALLBACK(void *) serialQueryInterface(PPDMIBASE pInterface, const char *pszIID)
 {
     SerialState *pThis = PDMIBASE_2_SERIALSTATE(pInterface);
-    switch (enmInterface)
-    {
-        case PDMINTERFACE_BASE:
-            return &pThis->IBase;
-        case PDMINTERFACE_CHAR_PORT:
-            return &pThis->ICharPort;
-        default:
-            return NULL;
-    }
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pThis->IBase);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMICHARPORT, &pThis->ICharPort);
+    return NULL;
 }
 
 /**
@@ -716,6 +787,7 @@ static DECLCALLBACK(void *) serialQueryInterface(PPDMIBASE pInterface, PDMINTERF
 static DECLCALLBACK(int) serialDestruct(PPDMDEVINS pDevIns)
 {
     SerialState *pThis = PDMINS_2_DATA(pDevIns, SerialState *);
+    PDMDEV_CHECK_VERSIONS_RETURN_QUIET(pDevIns);
 
     RTSemEventDestroy(pThis->ReceiveSem);
     pThis->ReceiveSem = NIL_RTSEMEVENT;
@@ -726,21 +798,11 @@ static DECLCALLBACK(int) serialDestruct(PPDMDEVINS pDevIns)
 
 
 /**
- * Construct a device instance for a VM.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- *                      If the registration structure is needed, pDevIns->pDevReg points to it.
- * @param   iInstance   Instance number. Use this to figure out which registers and such to use.
- *                      The device number is also found in pDevIns->iInstance, but since it's
- *                      likely to be freqently used PDM passes it as parameter.
- * @param   pCfgHandle  Configuration node handle for the device. Use this to obtain the configuration
- *                      of the device instance. It's also found in pDevIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @interface_method_impl{PDMDEVREG,pfnConstruct}
  */
 static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
                                          int iInstance,
-                                         PCFGMNODE pCfgHandle)
+                                         PCFGMNODE pCfg)
 {
     int            rc;
     SerialState   *pThis = PDMINS_2_DATA(pDevIns, SerialState*);
@@ -748,6 +810,7 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
     uint8_t        irq_lvl;
 
     Assert(iInstance < 4);
+    PDMDEV_CHECK_VERSIONS_RETURN(pDevIns);
 
     /*
      * Initialize the instance data.
@@ -765,8 +828,10 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
     pThis->IBase.pfnQueryInterface = serialQueryInterface;
 
     /* ICharPort */
-    pThis->ICharPort.pfnNotifyRead = serialNotifyRead;
+    pThis->ICharPort.pfnNotifyRead               = serialNotifyRead;
     pThis->ICharPort.pfnNotifyStatusLinesChanged = serialNotifyStatusLinesChanged;
+    pThis->ICharPort.pfnNotifyBufferFull         = serialNotifyBufferFull;
+    pThis->ICharPort.pfnNotifyBreak              = serialNotifyBreak;
 
 #ifdef VBOX_SERIAL_PCI
     /* the PCI device */
@@ -786,28 +851,28 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
     /*
      * Validate and read the configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "IRQ\0" "IOBase\0" "GCEnabled\0" "R0Enabled\0" "YieldOnLSRRead\0"))
+    if (!CFGMR3AreValuesValid(pCfg, "IRQ\0" "IOBase\0" "GCEnabled\0" "R0Enabled\0" "YieldOnLSRRead\0"))
     {
         AssertMsgFailed(("serialConstruct Invalid configuration values\n"));
         return VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES;
     }
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "GCEnabled", &pThis->fGCEnabled, true);
+    rc = CFGMR3QueryBoolDef(pCfg, "GCEnabled", &pThis->fGCEnabled, true);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"GCEnabled\" value"));
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "R0Enabled", &pThis->fR0Enabled, true);
+    rc = CFGMR3QueryBoolDef(pCfg, "R0Enabled", &pThis->fR0Enabled, true);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"R0Enabled\" value"));
 
-    rc = CFGMR3QueryBoolDef(pCfgHandle, "YieldOnLSRRead", &pThis->fYieldOnLSRRead, false);
+    rc = CFGMR3QueryBoolDef(pCfg, "YieldOnLSRRead", &pThis->fYieldOnLSRRead, false);
     if (RT_FAILURE(rc))
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"YieldOnLSRRead\" value"));
 
-    rc = CFGMR3QueryU8(pCfgHandle, "IRQ", &irq_lvl);
+    rc = CFGMR3QueryU8(pCfg, "IRQ", &irq_lvl);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
     {
         /* Provide sensible defaults. */
@@ -822,7 +887,7 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
         return PDMDEV_SET_ERROR(pDevIns, rc,
                                 N_("Configuration error: Failed to get the \"IRQ\" value"));
 
-    rc = CFGMR3QueryU16(pCfgHandle, "IOBase", &io_base);
+    rc = CFGMR3QueryU16(pCfg, "IOBase", &io_base);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND)
     {
         if (iInstance == 0)
@@ -849,9 +914,7 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
      * Initialize critical section and the semaphore.
      * This must of course be done before attaching drivers or anything else which can call us back..
      */
-    char szName[24];
-    RTStrPrintf(szName, sizeof(szName), "Serial#%d", iInstance);
-    rc = PDMDevHlpCritSectInit(pDevIns, &pThis->CritSect, szName);
+    rc = PDMDevHlpCritSectInit(pDevIns, &pThis->CritSect, RT_SRC_POS, "Serial#%d", iInstance);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -881,30 +944,28 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
         return rc;
 
     if (pThis->fGCEnabled)
-        rc = PDMDevHlpIOPortRegisterGC(pDevIns, io_base, 8, 0, "serialIOPortWrite",
+    {
+        rc = PDMDevHlpIOPortRegisterRC(pDevIns, io_base, 8, 0, "serialIOPortWrite",
                                       "serialIOPortRead", NULL, NULL, "Serial");
+        if (RT_FAILURE(rc))
+            return rc;
+    }
+
 
     if (pThis->fR0Enabled)
+    {
         rc = PDMDevHlpIOPortRegisterR0(pDevIns, io_base, 8, 0, "serialIOPortWrite",
                                       "serialIOPortRead", NULL, NULL, "Serial");
+        if (RT_FAILURE(rc))
+            return rc;
+    }
 #endif /* !VBOX_SERIAL_PCI */
 
     /*
      * Saved state.
      */
-    rc = PDMDevHlpSSMRegister(
-        pDevIns,                /* pDevIns */
-        pDevIns->pDevReg->szDeviceName, /* pszName */
-        iInstance,              /* u32Instance */
-        SERIAL_SAVED_STATE_VERSION, /* u32Version */
-        sizeof (*pThis),        /* cbGuess */
-        NULL,                   /* pfnSavePrep */
-        serialSaveExec,         /* pfnSaveExec */
-        NULL,                   /* pfnSaveDone */
-        NULL,                   /* pfnLoadPrep */
-        serialLoadExec,         /* pfnLoadExec */
-        NULL                    /* pfnLoadDone */
-        );
+    rc = PDMDevHlpSSMRegister3(pDevIns, SERIAL_SAVED_STATE_VERSION, sizeof (*pThis),
+                               serialLiveExec, serialSaveExec, serialLoadExec);
     if (RT_FAILURE(rc))
         return rc;
 
@@ -915,7 +976,7 @@ static DECLCALLBACK(int) serialConstruct(PPDMDEVINS pDevIns,
     rc = PDMDevHlpDriverAttach(pDevIns, 0, &pThis->IBase, &pThis->pDrvBase, "Serial Char");
     if (RT_SUCCESS(rc))
     {
-        pThis->pDrvChar = (PDMICHAR *)pThis->pDrvBase->pfnQueryInterface(pThis->pDrvBase, PDMINTERFACE_CHAR);
+        pThis->pDrvChar = PDMIBASE_QUERY_INTERFACE(pThis->pDrvBase, PDMICHARCONNECTOR);
         if (!pThis->pDrvChar)
         {
             AssertLogRelMsgFailed(("Configuration error: instance %d has no char interface!\n", iInstance));
@@ -947,7 +1008,7 @@ const PDMDEVREG g_DeviceSerialPort =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "serial",
     /* szRCMod */
     "VBoxDDGC.gc",
@@ -960,7 +1021,7 @@ const PDMDEVREG g_DeviceSerialPort =
     /* fClass */
     PDM_DEVREG_CLASS_SERIAL,
     /* cMaxInstances */
-    1,
+    UINT32_MAX,
     /* cbInstance */
     sizeof(SerialState),
     /* pfnConstruct */
