@@ -1,11 +1,10 @@
+/* $Id: DrvRawFile.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
- *
- * VBox stream devices:
- * Raw file output
+ * VBox stream drivers - Raw file output.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,24 +13,21 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#define LOG_GROUP LOG_GROUP_DRV_NAMEDPIPE
+#define LOG_GROUP LOG_GROUP_DEFAULT
 #include <VBox/pdmdrv.h>
 #include <iprt/assert.h>
 #include <iprt/file.h>
-#include <iprt/stream.h>
-#include <iprt/alloc.h>
-#include <iprt/string.h>
+#include <iprt/mem.h>
 #include <iprt/semaphore.h>
+#include <iprt/stream.h>
+#include <iprt/string.h>
+#include <iprt/uuid.h>
 
 #include "Builtins.h"
 
@@ -39,18 +35,20 @@
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
-
 /** Converts a pointer to DRVRAWFILE::IMedia to a PDRVRAWFILE. */
 #define PDMISTREAM_2_DRVRAWFILE(pInterface) ( (PDRVRAWFILE)((uintptr_t)pInterface - RT_OFFSETOF(DRVRAWFILE, IStream)) )
 
 /** Converts a pointer to PDMDRVINS::IBase to a PPDMDRVINS. */
 #define PDMIBASE_2_DRVINS(pInterface)   ( (PPDMDRVINS)((uintptr_t)pInterface - RT_OFFSETOF(PDMDRVINS, IBase)) )
 
+
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
 *******************************************************************************/
 /**
  * Raw file output driver instance data.
+ *
+ * @implements  PDMISTREAM
  */
 typedef struct DRVRAWFILE
 {
@@ -59,16 +57,14 @@ typedef struct DRVRAWFILE
     /** Pointer to the driver instance. */
     PPDMDRVINS          pDrvIns;
     /** Pointer to the file name. (Freed by MM) */
-    char                *pszLocation;
+    char               *pszLocation;
     /** Flag whether VirtualBox represents the server or client side. */
     RTFILE              OutputFile;
 } DRVRAWFILE, *PDRVRAWFILE;
 
 
-/*******************************************************************************
-*   Internal Functions                                                         *
-*******************************************************************************/
 
+/* -=-=-=-=- PDMISTREAM -=-=-=-=- */
 
 /** @copydoc PDMISTREAM::pfnWrite */
 static DECLCALLBACK(int) drvRawFileWrite(PPDMISTREAM pInterface, const void *pvBuf, size_t *pcbWrite)
@@ -94,113 +90,22 @@ static DECLCALLBACK(int) drvRawFileWrite(PPDMISTREAM pInterface, const void *pvB
     return rc;
 }
 
+/* -=-=-=-=- PDMIBASE -=-=-=-=- */
 
 /**
- * Queries an interface to the driver.
- *
- * @returns Pointer to interface.
- * @returns NULL if the interface was not supported by the driver.
- * @param   pInterface          Pointer to this interface structure.
- * @param   enmInterface        The requested interface identification.
- * @thread  Any thread.
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
  */
-static DECLCALLBACK(void *) drvRawFileQueryInterface(PPDMIBASE pInterface, PDMINTERFACE enmInterface)
+static DECLCALLBACK(void *) drvRawFileQueryInterface(PPDMIBASE pInterface, const char *pszIID)
 {
-    PPDMDRVINS pDrvIns = PDMIBASE_2_DRVINS(pInterface);
-    PDRVRAWFILE pDrv = PDMINS_2_DATA(pDrvIns, PDRVRAWFILE);
-    switch (enmInterface)
-    {
-        case PDMINTERFACE_BASE:
-            return &pDrvIns->IBase;
-        case PDMINTERFACE_STREAM:
-            return &pDrv->IStream;
-        default:
-            return NULL;
-    }
+    PPDMDRVINS  pDrvIns = PDMIBASE_2_DRVINS(pInterface);
+    PDRVRAWFILE pThis   = PDMINS_2_DATA(pDrvIns, PDRVRAWFILE);
+
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pDrvIns->IBase);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMISTREAM, &pThis->IStream);
+    return NULL;
 }
 
-
-/**
- * Construct a raw output stream driver instance.
- *
- * @returns VBox status.
- * @param   pDrvIns     The driver instance data.
- *                      If the registration structure is needed, pDrvIns->pDrvReg points to it.
- * @param   pCfgHandle  Configuration node handle for the driver. Use this to obtain the configuration
- *                      of the driver instance. It's also found in pDrvIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
- */
-static DECLCALLBACK(int) drvRawFileConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfgHandle)
-{
-    int rc;
-    char *pszLocation = NULL;
-    PDRVRAWFILE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWFILE);
-
-    /*
-     * Init the static parts.
-     */
-    pThis->pDrvIns                      = pDrvIns;
-    pThis->pszLocation                  = NULL;
-    pThis->OutputFile                   = NIL_RTFILE;
-    /* IBase */
-    pDrvIns->IBase.pfnQueryInterface    = drvRawFileQueryInterface;
-    /* IStream */
-    pThis->IStream.pfnWrite             = drvRawFileWrite;
-
-    /*
-     * Read the configuration.
-     */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "Location\0"))
-    {
-        rc = VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES;
-        goto out;
-    }
-
-    rc = CFGMR3QueryStringAlloc(pCfgHandle, "Location", &pszLocation);
-    if (RT_FAILURE(rc))
-    {
-        AssertMsgFailed(("Configuration error: query \"Location\" resulted in %Rrc.\n", rc));
-        goto out;
-    }
-    pThis->pszLocation = pszLocation;
-
-    rc = RTFileOpen(&pThis->OutputFile, pThis->pszLocation, RTFILE_O_WRITE | RTFILE_O_CREATE_REPLACE);
-    if (RT_FAILURE(rc))
-    {
-        LogRel(("RawFile%d: CreateFile failed rc=%Rrc\n", pThis->pDrvIns->iInstance));
-        return PDMDrvHlpVMSetError(pDrvIns, rc, RT_SRC_POS, N_("RawFile#%d failed to create the raw output file %s"), pDrvIns->iInstance, pszLocation);
-    }
-
-out:
-    if (RT_FAILURE(rc))
-    {
-        if (pszLocation)
-            MMR3HeapFree(pszLocation);
-        return PDMDrvHlpVMSetError(pDrvIns, rc, RT_SRC_POS, N_("RawFile#%d failed to initialize"), pDrvIns->iInstance);
-    }
-
-    LogFlow(("drvRawFileConstruct: location %s\n", pszLocation));
-    LogRel(("RawFile: location %s\n", pszLocation));
-    return VINF_SUCCESS;
-}
-
-
-/**
- * Destruct a raw output stream driver instance.
- *
- * Most VM resources are freed by the VM. This callback is provided so that
- * any non-VM resources can be freed correctly.
- *
- * @param   pDrvIns     The driver instance data.
- */
-static DECLCALLBACK(void) drvRawFileDestruct(PPDMDRVINS pDrvIns)
-{
-    PDRVRAWFILE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWFILE);
-    LogFlow(("%s: %s\n", __FUNCTION__, pThis->pszLocation));
-
-    if (pThis->pszLocation)
-        MMR3HeapFree(pThis->pszLocation);
-}
+/* -=-=-=-=- PDMDRVREG -=-=-=-=- */
 
 
 /**
@@ -216,7 +121,82 @@ static DECLCALLBACK(void) drvRawFilePowerOff(PPDMDRVINS pDrvIns)
     LogFlow(("%s: %s\n", __FUNCTION__, pThis->pszLocation));
 
     if (pThis->OutputFile != NIL_RTFILE)
+    {
         RTFileClose(pThis->OutputFile);
+        pThis->OutputFile = NIL_RTFILE;
+    }
+}
+
+
+/**
+ * Destruct a raw output stream driver instance.
+ *
+ * Most VM resources are freed by the VM. This callback is provided so that
+ * any non-VM resources can be freed correctly.
+ *
+ * @param   pDrvIns     The driver instance data.
+ */
+static DECLCALLBACK(void) drvRawFileDestruct(PPDMDRVINS pDrvIns)
+{
+    PDRVRAWFILE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWFILE);
+    LogFlow(("%s: %s\n", __FUNCTION__, pThis->pszLocation));
+    PDMDRV_CHECK_VERSIONS_RETURN_VOID(pDrvIns);
+
+    if (pThis->pszLocation)
+        MMR3HeapFree(pThis->pszLocation);
+
+    if (pThis->OutputFile != NIL_RTFILE)
+    {
+        RTFileClose(pThis->OutputFile);
+        pThis->OutputFile = NIL_RTFILE;
+    }
+}
+
+
+/**
+ * Construct a raw output stream driver instance.
+ *
+ * @copydoc FNPDMDRVCONSTRUCT
+ */
+static DECLCALLBACK(int) drvRawFileConstruct(PPDMDRVINS pDrvIns, PCFGMNODE pCfg, uint32_t fFlags)
+{
+    PDRVRAWFILE pThis = PDMINS_2_DATA(pDrvIns, PDRVRAWFILE);
+    PDMDRV_CHECK_VERSIONS_RETURN(pDrvIns);
+
+    /*
+     * Init the static parts.
+     */
+    pThis->pDrvIns                      = pDrvIns;
+    pThis->pszLocation                  = NULL;
+    pThis->OutputFile                   = NIL_RTFILE;
+    /* IBase */
+    pDrvIns->IBase.pfnQueryInterface    = drvRawFileQueryInterface;
+    /* IStream */
+    pThis->IStream.pfnWrite             = drvRawFileWrite;
+
+    /*
+     * Read the configuration.
+     */
+    if (!CFGMR3AreValuesValid(pCfg, "Location\0"))
+        AssertFailedReturn(VERR_PDM_DRVINS_UNKNOWN_CFG_VALUES);
+
+    int rc = CFGMR3QueryStringAlloc(pCfg, "Location", &pThis->pszLocation);
+    if (RT_FAILURE(rc))
+        AssertMsgFailedReturn(("Configuration error: query \"Location\" resulted in %Rrc.\n", rc), rc);
+
+    /*
+     * Open the raw file.
+     */
+    rc = RTFileOpen(&pThis->OutputFile, pThis->pszLocation, RTFILE_O_WRITE | RTFILE_O_CREATE_REPLACE | RTFILE_O_DENY_NONE);
+    if (RT_FAILURE(rc))
+    {
+        LogRel(("RawFile%d: CreateFile failed rc=%Rrc\n", pDrvIns->iInstance));
+        return PDMDrvHlpVMSetError(pDrvIns, rc, RT_SRC_POS, N_("RawFile#%d failed to create the raw output file %s"), pDrvIns->iInstance, pThis->pszLocation);
+    }
+
+    LogFlow(("drvRawFileConstruct: location %s\n", pThis->pszLocation));
+    LogRel(("RawFile#%u: location %s\n", pDrvIns->iInstance, pThis->pszLocation));
+    return VINF_SUCCESS;
 }
 
 
@@ -227,8 +207,12 @@ const PDMDRVREG g_DrvRawFile =
 {
     /* u32Version */
     PDM_DRVREG_VERSION,
-    /* szDriverName */
+    /* szName */
     "RawFile",
+    /* szRCMod */
+    "",
+    /* szR0Mod */
+    "",
     /* pszDescription */
     "RawFile stream driver.",
     /* fFlags */
@@ -243,6 +227,8 @@ const PDMDRVREG g_DrvRawFile =
     drvRawFileConstruct,
     /* pfnDestruct */
     drvRawFileDestruct,
+    /* pfnRelocate */
+    NULL,
     /* pfnIOCtl */
     NULL,
     /* pfnPowerOn */
@@ -253,8 +239,15 @@ const PDMDRVREG g_DrvRawFile =
     NULL,
     /* pfnResume */
     NULL,
+    /* pfnAttach */
+    NULL,
     /* pfnDetach */
     NULL,
     /* pfnPowerOff */
     drvRawFilePowerOff,
+    /* pfnSoftReset */
+    NULL,
+    /* u32EndVersion */
+    PDM_DRVREG_VERSION
 };
+
