@@ -1,10 +1,10 @@
-/* $Id: ATAController.h 15252 2008-12-10 15:37:23Z vboxsync $ */
+/* $Id: ATAController.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DevATA, DevAHCI - Shared ATA/ATAPI controller types.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___Storage_ATAController_h
@@ -72,6 +68,7 @@
 #define ATA_EVENT_STATUS_MEDIA_NEW              1    /**< new medium inserted */
 #define ATA_EVENT_STATUS_MEDIA_REMOVED          2    /**< medium removed */
 #define ATA_EVENT_STATUS_MEDIA_CHANGED          3    /**< medium was removed + new medium was inserted */
+#define ATA_EVENT_STATUS_MEDIA_EJECT_REQUESTED  4    /**< medium eject requested (eject button pressed) */
 
 
 /*******************************************************************************
@@ -178,7 +175,7 @@ typedef struct AHCIATADevState {
     R0PTRTYPE(uint8_t *) pbIOBufferR0;
     /** Pointer to the I/O buffer. */
     RCPTRTYPE(uint8_t *) pbIOBufferRC;
-    
+
     RTRCPTR Aligmnent1; /**< Align the statistics at an 8-byte boundrary. */
 
     /*
@@ -365,7 +362,10 @@ typedef struct AHCIATACONTROLLER
     uint8_t             AsyncIOReqHead;
     /** The position at which to get a new request for the AIO thread. */
     uint8_t             AsyncIOReqTail;
-    uint8_t             Alignment3[2]; /**< Explicit padding of the 2 byte gap. */
+    /** Whether to call RTThreadUserSignal and PDMDevHlpAsyncNotificationCompleted
+     * when idle.  Before setting this, call RTThreadUserReset. */
+    bool volatile       fSignalIdle;
+    uint8_t             Alignment3[1]; /**< Explicit padding of the 1 byte gap. */
     /** Magic delay before triggering interrupts in DMA mode. */
     uint32_t            DelayIRQMillies;
     /** The mutex protecting the request queue. */
@@ -396,7 +396,7 @@ typedef struct AHCIATACONTROLLER
 /*******************************************************************************
  *  Internal Functions                                                         *
  ******************************************************************************/
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 int ataControllerIOPortWrite1(PAHCIATACONTROLLER pCtl, RTIOPORT Port, uint32_t u32, unsigned cb);
 int ataControllerIOPortRead1(PAHCIATACONTROLLER pCtl, RTIOPORT Port, uint32_t *u32, unsigned cb);
 int ataControllerIOPortWriteStr1(PAHCIATACONTROLLER pCtl, RTIOPORT Port, RTGCPTR *pGCPtrSrc, PRTGCUINTREG pcTransfer, unsigned cb);
@@ -405,7 +405,7 @@ int ataControllerIOPortWrite2(PAHCIATACONTROLLER pCtl, RTIOPORT Port, uint32_t u
 int ataControllerIOPortRead2(PAHCIATACONTROLLER pCtl, RTIOPORT Port, uint32_t *u32, unsigned cb);
 int ataControllerBMDMAIOPortRead(PAHCIATACONTROLLER pCtl, RTIOPORT Port, uint32_t *pu32, unsigned cb);
 int ataControllerBMDMAIOPortWrite(PAHCIATACONTROLLER pCtl, RTIOPORT Port, uint32_t u32, unsigned cb);
-__END_DECLS
+RT_C_DECLS_END
 
 #ifdef IN_RING3
 /**
@@ -431,12 +431,12 @@ int ataControllerInit(PPDMDEVINS pDevIns, PAHCIATACONTROLLER pCtl, PPDMIBASE pDr
 int ataControllerDestroy(PAHCIATACONTROLLER pCtl);
 
 /**
- * Power off a controller.
+ * Tests if the controller is idle, leaving the PDM notifications on if busy.
  *
- * @returns nothing.
+ * @returns true if idle, false if idle.
  * @param   pCtl the controller instance.
  */
-void ataControllerPowerOff(PAHCIATACONTROLLER pCtl);
+bool ataControllerIsIdle(PAHCIATACONTROLLER pCtl);
 
 /**
  * Reset a controller instance to an initial state.
@@ -446,14 +446,6 @@ void ataControllerPowerOff(PAHCIATACONTROLLER pCtl);
  */
 void ataControllerReset(PAHCIATACONTROLLER pCtl);
 
-/**
- * Suspend operation of an controller.
- *
- * @returns nothing
- * @param   pCtl The controller instance.
- */
-void ataControllerSuspend(PAHCIATACONTROLLER pCtl);
-
 /**
  * Resume operation of an controller.
  *
@@ -481,15 +473,6 @@ void ataControllerRelocate(PAHCIATACONTROLLER pCtl, RTGCINTPTR offDelta);
  */
 int ataControllerSaveExec(PAHCIATACONTROLLER pCtl, PSSMHANDLE pSSM);
 
-/**
- * Prepare state save operation.
- *
- * @returns VBox status code.
- * @param   pCtl    The controller instance.
- * @param   pSSM    SSM operation handle.
- */
-int ataControllerSavePrep(PAHCIATACONTROLLER pCtl, PSSMHANDLE pSSM);
-
 /**
  * Excute state load operation.
  *
@@ -499,15 +482,6 @@ int ataControllerSavePrep(PAHCIATACONTROLLER pCtl, PSSMHANDLE pSSM);
  */
 int ataControllerLoadExec(PAHCIATACONTROLLER pCtl, PSSMHANDLE pSSM);
 
-/**
- * Prepare state load operation.
- *
- * @returns VBox status code.
- * @param   pCtl    The controller instance.
- * @param   pSSM    SSM operation handle.
- */
-int ataControllerLoadPrep(PAHCIATACONTROLLER pCtl, PSSMHANDLE pSSM);
-
 #endif /* IN_RING3 */
 
 #endif /* !VBOX_DEVICE_STRUCT_TESTCASE */
