@@ -1,10 +1,10 @@
-/* $Id: RawHDDCore.cpp 18066 2009-03-18 14:16:06Z vboxsync $ */
+/* $Id: RawHDDCore.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * RawHDDCore - Raw Disk image, Core Code.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,17 +13,13 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_VD_RAW
-#include "VBoxHDD-Internal.h"
+#include <VBox/VBoxHDD-Plugin.h>
 #include <VBox/err.h>
 
 #include <VBox/log.h>
@@ -42,12 +38,23 @@
 typedef struct RAWIMAGE
 {
     /** Base image name. */
-    const char      *pszFilename;
+    const char       *pszFilename;
+#ifndef VBOX_WITH_NEW_IO_CODE
     /** File descriptor. */
-    RTFILE          File;
+    RTFILE            File;
+#else
+    /** Storage handle. */
+    PVDIOSTORAGE      pStorage;
+    /** I/O interface. */
+    PVDINTERFACE      pInterfaceIO;
+    /** Async I/O interface callbacks. */
+    PVDINTERFACEIO    pInterfaceIOCallbacks;
+#endif
 
     /** Pointer to the per-disk VD interface list. */
     PVDINTERFACE      pVDIfsDisk;
+    /** Pointer to the per-image VD interface list. */
+    PVDINTERFACE      pVDIfsImage;
 
     /** Error callback. */
     PVDINTERFACE      pInterfaceError;
@@ -55,15 +62,15 @@ typedef struct RAWIMAGE
     PVDINTERFACEERROR pInterfaceErrorCallbacks;
 
     /** Open flags passed by VBoxHD layer. */
-    unsigned        uOpenFlags;
+    unsigned          uOpenFlags;
     /** Image flags defined during creation or determined during open. */
-    unsigned        uImageFlags;
+    unsigned          uImageFlags;
     /** Total size of the image. */
-    uint64_t        cbSize;
+    uint64_t          cbSize;
     /** Physical geometry of this image. */
-    PDMMEDIAGEOMETRY PCHSGeometry;
+    PDMMEDIAGEOMETRY  PCHSGeometry;
     /** Logical geometry of this image. */
-    PDMMEDIAGEOMETRY LCHSGeometry;
+    PDMMEDIAGEOMETRY  LCHSGeometry;
 
 } RAWIMAGE, *PRAWIMAGE;
 
@@ -103,13 +110,150 @@ DECLINLINE(int) rawError(PRAWIMAGE pImage, int rc, RT_SRC_POS_DECL,
     return rc;
 }
 
+static int rawFileOpen(PRAWIMAGE pImage, bool fReadonly, bool fCreate)
+{
+    int rc = VINF_SUCCESS;
+
+    AssertMsg(!(fReadonly && fCreate), ("Image can't be opened readonly while being created\n"));
+
+#ifndef VBOX_WITH_NEW_IO_CODE
+    uint32_t fOpen = fReadonly ? RTFILE_O_READ      | RTFILE_O_DENY_NONE
+                               : RTFILE_O_READWRITE | RTFILE_O_DENY_WRITE;
+
+    if (fCreate)
+        fOpen |= RTFILE_O_CREATE;
+    else
+        fOpen |= RTFILE_O_OPEN;
+
+    rc = RTFileOpen(&pImage->File, pImage->pszFilename, fOpen);
+#else
+    unsigned uOpenFlags = fReadonly ? VD_INTERFACEASYNCIO_OPEN_FLAGS_READONLY : 0;
+
+    if (fCreate)
+        uOpenFlags |= VD_INTERFACEASYNCIO_OPEN_FLAGS_CREATE;
+
+    rc = pImage->pInterfaceIOCallbacks->pfnOpen(pImage->pInterfaceIO->pvUser,
+                                                pImage->pszFilename,
+                                                uOpenFlags,
+                                                &pImage->pStorage);
+#endif
+
+    return rc;
+}
+
+static int rawFileClose(PRAWIMAGE pImage)
+{
+    int rc = VINF_SUCCESS;
+
+#ifndef VBOX_WITH_NEW_IO_CODE
+    if (pImage->File != NIL_RTFILE)
+        rc = RTFileClose(pImage->File);
+
+    pImage->File = NIL_RTFILE;
+#else
+    if (pImage->pStorage)
+        rc = pImage->pInterfaceIOCallbacks->pfnClose(pImage->pInterfaceIO->pvUser,
+                                                     pImage->pStorage);
+
+    pImage->pStorage = NULL;
+#endif
+
+    return rc;
+}
+
+static int rawFileFlushSync(PRAWIMAGE pImage)
+{
+    int rc = VINF_SUCCESS;
+
+#ifndef VBOX_WITH_NEW_IO_CODE
+    rc = RTFileFlush(pImage->File);
+#else
+    rc = pImage->pInterfaceIOCallbacks->pfnFlushSync(pImage->pInterfaceIO->pvUser,
+                                                     pImage->pStorage);
+#endif
+
+    return rc;
+}
+
+static int rawFileGetSize(PRAWIMAGE pImage, uint64_t *pcbSize)
+{
+    int rc = VINF_SUCCESS;
+
+#ifndef VBOX_WITH_NEW_IO_CODE
+    rc = RTFileGetSize(pImage->File, pcbSize);
+#else
+    rc = pImage->pInterfaceIOCallbacks->pfnGetSize(pImage->pInterfaceIO->pvUser,
+                                                   pImage->pStorage,
+                                                   pcbSize);
+#endif
+
+    return rc;
+
+}
+
+static int rawFileSetSize(PRAWIMAGE pImage, uint64_t cbSize)
+{
+    int rc = VINF_SUCCESS;
+
+#ifndef VBOX_WITH_NEW_IO_CODE
+    rc = RTFileSetSize(pImage->File, cbSize);
+#else
+    rc = pImage->pInterfaceIOCallbacks->pfnSetSize(pImage->pInterfaceIO->pvUser,
+                                                   pImage->pStorage,
+                                                   cbSize);
+#endif
+
+    return rc;
+}
+
+
+static int rawFileWriteSync(PRAWIMAGE pImage, uint64_t off, const void *pcvBuf, size_t cbWrite, size_t *pcbWritten)
+{
+    int rc = VINF_SUCCESS;
+
+#ifndef VBOX_WITH_NEW_IO_CODE
+    rc = RTFileWriteAt(pImage->File, off, pcvBuf, cbWrite, pcbWritten);
+#else
+    rc = pImage->pInterfaceIOCallbacks->pfnWriteSync(pImage->pInterfaceIO->pvUser,
+                                                     pImage->pStorage,
+                                                     off, cbWrite, pcvBuf,
+                                                     pcbWritten);
+#endif
+
+    return rc;
+}
+
+static int rawFileReadSync(PRAWIMAGE pImage, uint64_t off, void *pvBuf, size_t cbRead, size_t *pcbRead)
+{
+    int rc = VINF_SUCCESS;
+
+#ifndef VBOX_WITH_NEW_IO_CODE
+    rc = RTFileReadAt(pImage->File, off, pvBuf, cbRead, pcbRead);
+#else
+    rc = pImage->pInterfaceIOCallbacks->pfnReadSync(pImage->pInterfaceIO->pvUser,
+                                                    pImage->pStorage,
+                                                    off, cbRead, pvBuf,
+                                                    pcbRead);
+#endif
+
+    return rc;
+}
+
+static bool rawFileOpened(PRAWIMAGE pImage)
+{
+#ifndef VBOX_WITH_NEW_IO_CODE
+    return pImage->File != NIL_RTFILE;
+#else
+    return pImage->pStorage != NULL;
+#endif
+}
+
 /**
  * Internal: Open an image, constructing all necessary data structures.
  */
 static int rawOpenImage(PRAWIMAGE pImage, unsigned uOpenFlags)
 {
     int rc;
-    RTFILE File;
 
     if (uOpenFlags & VD_OPEN_FLAGS_ASYNC_IO)
         return VERR_NOT_SUPPORTED;
@@ -120,22 +264,26 @@ static int rawOpenImage(PRAWIMAGE pImage, unsigned uOpenFlags)
     if (pImage->pInterfaceError)
         pImage->pInterfaceErrorCallbacks = VDGetInterfaceError(pImage->pInterfaceError);
 
+#ifdef VBOX_WITH_NEW_IO_CODE
+    /* Try to get I/O interface. */
+    pImage->pInterfaceIO = VDInterfaceGet(pImage->pVDIfsImage, VDINTERFACETYPE_IO);
+    AssertPtr(pImage->pInterfaceIO);
+    pImage->pInterfaceIOCallbacks = VDGetInterfaceIO(pImage->pInterfaceIO);
+    AssertPtr(pImage->pInterfaceIOCallbacks);
+#endif
+
     /*
      * Open the image.
      */
-    rc = RTFileOpen(&File, pImage->pszFilename,
-                    uOpenFlags & VD_OPEN_FLAGS_READONLY
-                     ? RTFILE_O_READ      | RTFILE_O_OPEN | RTFILE_O_DENY_NONE
-                     : RTFILE_O_READWRITE | RTFILE_O_OPEN | RTFILE_O_DENY_WRITE);
+    rc = rawFileOpen(pImage, !!(uOpenFlags & VD_OPEN_FLAGS_READONLY), false);
     if (RT_FAILURE(rc))
     {
         /* Do NOT signal an appropriate error here, as the VD layer has the
          * choice of retrying the open if it failed. */
         goto out;
     }
-    pImage->File = File;
 
-    rc = RTFileGetSize(pImage->File, &pImage->cbSize);
+    rc = rawFileGetSize(pImage, &pImage->cbSize);
     if (RT_FAILURE(rc))
         goto out;
     if (pImage->cbSize % 512)
@@ -158,11 +306,10 @@ static int rawCreateImage(PRAWIMAGE pImage, uint64_t cbSize,
                           unsigned uImageFlags, const char *pszComment,
                           PCPDMMEDIAGEOMETRY pPCHSGeometry,
                           PCPDMMEDIAGEOMETRY pLCHSGeometry,
-                          PFNVMPROGRESS pfnProgress, void *pvUser,
+                          PFNVDPROGRESS pfnProgress, void *pvUser,
                           unsigned uPercentStart, unsigned uPercentSpan)
 {
     int rc;
-    RTFILE File;
     RTFOFF cbFree = 0;
     uint64_t uOff;
     size_t cbBuf = 128 * _1K;
@@ -183,15 +330,21 @@ static int rawCreateImage(PRAWIMAGE pImage, uint64_t cbSize,
     if (pImage->pInterfaceError)
         pImage->pInterfaceErrorCallbacks = VDGetInterfaceError(pImage->pInterfaceError);
 
+#ifdef VBOX_WITH_NEW_IO_CODE
+    /* Try to get async I/O interface. */
+    pImage->pInterfaceIO = VDInterfaceGet(pImage->pVDIfsImage, VDINTERFACETYPE_IO);
+    AssertPtr(pImage->pInterfaceIO);
+    pImage->pInterfaceIOCallbacks = VDGetInterfaceIO(pImage->pInterfaceIO);
+    AssertPtr(pImage->pInterfaceIOCallbacks);
+#endif
+
     /* Create image file. */
-    rc = RTFileOpen(&File, pImage->pszFilename,
-                    RTFILE_O_READWRITE | RTFILE_O_CREATE | RTFILE_O_DENY_ALL);
+    rc = rawFileOpen(pImage, false, true);
     if (RT_FAILURE(rc))
     {
         rc = rawError(pImage, rc, RT_SRC_POS, N_("Raw: cannot create image '%s'"), pImage->pszFilename);
         goto out;
     }
-    pImage->File = File;
 
     /* Check the free space on the disk and leave early if there is not
      * sufficient space available. */
@@ -204,7 +357,7 @@ static int rawCreateImage(PRAWIMAGE pImage, uint64_t cbSize,
 
     /* Allocate & commit whole file if fixed image, it must be more
      * effective than expanding file by write operations. */
-    rc = RTFileSetSize(File, cbSize);
+    rc = rawFileSetSize(pImage, cbSize);
     if (RT_FAILURE(rc))
     {
         rc = rawError(pImage, rc, RT_SRC_POS, N_("Raw: setting image size failed for '%s'"), pImage->pszFilename);
@@ -228,7 +381,7 @@ static int rawCreateImage(PRAWIMAGE pImage, uint64_t cbSize,
     {
         unsigned cbChunk = (unsigned)RT_MIN(cbSize, cbBuf);
 
-        rc = RTFileWriteAt(File, uOff, pvBuf, cbChunk, NULL);
+        rc = rawFileWriteSync(pImage, uOff, pvBuf, cbChunk, NULL);
         if (RT_FAILURE(rc))
         {
             rc = rawError(pImage, rc, RT_SRC_POS, N_("Raw: writing block failed for '%s'"), pImage->pszFilename);
@@ -239,9 +392,8 @@ static int rawCreateImage(PRAWIMAGE pImage, uint64_t cbSize,
 
         if (pfnProgress)
         {
-            rc = pfnProgress(NULL /* WARNING! pVM=NULL  */,
-                             uPercentStart + uOff * uPercentSpan * 98 / (cbSize * 100),
-                             pvUser);
+            rc = pfnProgress(pvUser,
+                             uPercentStart + uOff * uPercentSpan * 98 / (cbSize * 100));
             if (RT_FAILURE(rc))
                 goto out;
         }
@@ -249,8 +401,7 @@ static int rawCreateImage(PRAWIMAGE pImage, uint64_t cbSize,
     RTMemTmpFree(pvBuf);
 
     if (RT_SUCCESS(rc) && pfnProgress)
-        pfnProgress(NULL /* WARNING! pVM=NULL  */,
-                    uPercentStart + uPercentSpan * 98 / 100, pvUser);
+        pfnProgress(pvUser, uPercentStart + uPercentSpan * 98 / 100);
 
     pImage->cbSize = cbSize;
 
@@ -258,8 +409,7 @@ static int rawCreateImage(PRAWIMAGE pImage, uint64_t cbSize,
 
 out:
     if (RT_SUCCESS(rc) && pfnProgress)
-        pfnProgress(NULL /* WARNING! pVM=NULL  */,
-                    uPercentStart + uPercentSpan, pvUser);
+        pfnProgress(pvUser, uPercentStart + uPercentSpan);
 
     if (RT_FAILURE(rc))
         rawFreeImage(pImage, rc != VERR_ALREADY_EXISTS);
@@ -274,12 +424,11 @@ static void rawFreeImage(PRAWIMAGE pImage, bool fDelete)
 {
     Assert(pImage);
 
-    if (pImage->File != NIL_RTFILE)
+    if (rawFileOpened(pImage))
     {
         if (!(pImage->uOpenFlags & VD_OPEN_FLAGS_READONLY))
             rawFlushImage(pImage);
-        RTFileClose(pImage->File);
-        pImage->File = NIL_RTFILE;
+        rawFileClose(pImage);
     }
     if (fDelete && pImage->pszFilename)
         RTFileDelete(pImage->pszFilename);
@@ -292,16 +441,16 @@ static int rawFlushImage(PRAWIMAGE pImage)
 {
     int rc = VINF_SUCCESS;
 
-    if (   pImage->File != NIL_RTFILE
+    if (   rawFileOpened(pImage)
         && !(pImage->uOpenFlags & VD_OPEN_FLAGS_READONLY))
-        rc = RTFileFlush(pImage->File);
+        rc = rawFileFlushSync(pImage);
 
     return rc;
 }
 
 
 /** @copydoc VBOXHDDBACKEND::pfnCheckIfValid */
-static int rawCheckIfValid(const char *pszFilename)
+static int rawCheckIfValid(const char *pszFilename, PVDINTERFACE pVDIfsDisk)
 {
     LogFlowFunc(("pszFilename=\"%s\"\n", pszFilename));
     int rc = VINF_SUCCESS;
@@ -353,12 +502,19 @@ static int rawOpen(const char *pszFilename, unsigned uOpenFlags,
         goto out;
     }
     pImage->pszFilename = pszFilename;
+#ifndef VBOX_WITH_NEW_IO_CODE
     pImage->File = NIL_RTFILE;
+#else
+    pImage->pStorage = NULL;
+#endif
     pImage->pVDIfsDisk = pVDIfsDisk;
+    pImage->pVDIfsImage = pVDIfsImage;
 
     rc = rawOpenImage(pImage, uOpenFlags);
     if (RT_SUCCESS(rc))
         *ppBackendData = pImage;
+    else
+        RTMemFree(pImage);
 
 out:
     LogFlowFunc(("returns %Rrc (pBackendData=%#p)\n", rc, *ppBackendData));
@@ -379,7 +535,7 @@ static int rawCreate(const char *pszFilename, uint64_t cbSize,
     int rc;
     PRAWIMAGE pImage;
 
-    PFNVMPROGRESS pfnProgress = NULL;
+    PFNVDPROGRESS pfnProgress = NULL;
     void *pvUser = NULL;
     PVDINTERFACE pIfProgress = VDInterfaceGet(pVDIfsOperation,
                                               VDINTERFACETYPE_PROGRESS);
@@ -416,8 +572,13 @@ static int rawCreate(const char *pszFilename, uint64_t cbSize,
         goto out;
     }
     pImage->pszFilename = pszFilename;
+#ifndef VBOX_WITH_NEW_IO_CODE
     pImage->File = NIL_RTFILE;
+#else
+    pImage->pStorage = NULL;
+#endif
     pImage->pVDIfsDisk = pVDIfsDisk;
+    pImage->pVDIfsImage = pVDIfsImage;
 
     rc = rawCreateImage(pImage, cbSize, uImageFlags, pszComment,
                         pPCHSGeometry, pLCHSGeometry,
@@ -431,10 +592,15 @@ static int rawCreate(const char *pszFilename, uint64_t cbSize,
             rawFreeImage(pImage, false);
             rc = rawOpenImage(pImage, uOpenFlags);
             if (RT_FAILURE(rc))
+            {
+                RTMemFree(pImage);
                 goto out;
+            }
         }
         *ppBackendData = pImage;
     }
+    else
+        RTMemFree(pImage);
 
 out:
     LogFlowFunc(("returns %Rrc (pBackendData=%#p)\n", rc, *ppBackendData));
@@ -461,7 +627,10 @@ static int rawClose(void *pBackendData, bool fDelete)
     /* Freeing a never allocated image (e.g. because the open failed) is
      * not signalled as an error. After all nothing bad happens. */
     if (pImage)
+    {
         rawFreeImage(pImage, fDelete);
+        RTMemFree(pImage);
+    }
 
     LogFlowFunc(("returns %Rrc\n", rc));
     return rc;
@@ -486,7 +655,7 @@ static int rawRead(void *pBackendData, uint64_t uOffset, void *pvBuf,
         goto out;
     }
 
-    rc = RTFileReadAt(pImage->File, uOffset, pvBuf, cbToRead, NULL);
+    rc = rawFileReadSync(pImage, uOffset, pvBuf, cbToRead, NULL);
     *pcbActuallyRead = cbToRead;
 
 out:
@@ -520,7 +689,7 @@ static int rawWrite(void *pBackendData, uint64_t uOffset, const void *pvBuf,
         goto out;
     }
 
-    rc = RTFileWriteAt(pImage->File, uOffset, pvBuf, cbToWrite, NULL);
+    rc = rawFileWriteSync(pImage, uOffset, pvBuf, cbToWrite, NULL);
     if (pcbWriteProcess)
         *pcbWriteProcess = cbToWrite;
 
@@ -581,9 +750,9 @@ static uint64_t rawGetFileSize(void *pBackendData)
     if (pImage)
     {
         uint64_t cbFile;
-        if (pImage->File != NIL_RTFILE)
+        if (rawFileOpened(pImage))
         {
-            int rc = RTFileGetSize(pImage->File, &cbFile);
+            int rc = rawFileGetSize(pImage, &cbFile);
             if (RT_SUCCESS(rc))
                 cb += cbFile;
         }
@@ -986,7 +1155,7 @@ static void rawDump(void *pBackendData)
     Assert(pImage);
     if (pImage)
     {
-        RTLogPrintf("Header: Geometry PCHS=%u/%u/%u LCHS=%u/%u/%u cbSector=%llu\n",
+        pImage->pInterfaceErrorCallbacks->pfnMessage(pImage->pInterfaceError->pvUser, "Header: Geometry PCHS=%u/%u/%u LCHS=%u/%u/%u cbSector=%llu\n",
                     pImage->PCHSGeometry.cCylinders, pImage->PCHSGeometry.cHeads, pImage->PCHSGeometry.cSectors,
                     pImage->LCHSGeometry.cCylinders, pImage->LCHSGeometry.cHeads, pImage->LCHSGeometry.cSectors,
                     pImage->cbSize / 512);
@@ -1030,19 +1199,47 @@ static int rawSetParentFilename(void *pvBackendData, const char *pszParentFilena
 
 static bool rawIsAsyncIOSupported(void *pvBackendData)
 {
-    return false;
+    return true;
 }
 
 static int rawAsyncRead(void *pvBackendData, uint64_t uOffset, size_t cbRead,
-                        PPDMDATASEG paSeg, unsigned cSeg, void *pvUser)
+                        PVDIOCTX pIoCtx, size_t *pcbActuallyRead)
 {
-    int rc = VERR_NOT_IMPLEMENTED;
-    LogFlowFunc(("returns %Rrc\n", rc));
+    int rc = VINF_SUCCESS;
+    PRAWIMAGE pImage = (PRAWIMAGE)pvBackendData;
+
+    rc = pImage->pInterfaceIOCallbacks->pfnReadUserAsync(pImage->pInterfaceIO->pvUser,
+                                                         pImage->pStorage,
+                                                         uOffset, pIoCtx, cbRead);
+    if (RT_SUCCESS(rc))
+        *pcbActuallyRead = cbRead;
+
     return rc;
 }
 
 static int rawAsyncWrite(void *pvBackendData, uint64_t uOffset, size_t cbWrite,
-                         PPDMDATASEG paSeg, unsigned cSeg, void *pvUser)
+                         PVDIOCTX pIoCtx,
+                         size_t *pcbWriteProcess, size_t *pcbPreRead,
+                         size_t *pcbPostRead, unsigned fWrite)
+{
+    int rc = VINF_SUCCESS;
+    PRAWIMAGE pImage = (PRAWIMAGE)pvBackendData;
+
+    rc = pImage->pInterfaceIOCallbacks->pfnWriteUserAsync(pImage->pInterfaceIO->pvUser,
+                                                         pImage->pStorage,
+                                                         uOffset, pIoCtx, cbWrite);
+
+    if (RT_SUCCESS(rc))
+    {
+        *pcbWriteProcess = cbWrite;
+        *pcbPostRead = 0;
+        *pcbPreRead  = 0;
+    }
+
+    return rc;
+}
+
+static int rawAsyncFlush(void *pvBackendData, PVDIOCTX pIoCtx)
 {
     int rc = VERR_NOT_IMPLEMENTED;
     LogFlowFunc(("returns %Rrc\n", rc));
@@ -1056,7 +1253,7 @@ VBOXHDDBACKEND g_RawBackend =
     /* cbSize */
     sizeof(VBOXHDDBACKEND),
     /* uBackendCaps */
-    VD_CAP_CREATE_FIXED | VD_CAP_FILE,
+    VD_CAP_CREATE_FIXED | VD_CAP_FILE | VD_CAP_ASYNC,
     /* papszFileExtensions */
     s_apszRawFileExtensions,
     /* paConfigInfo */
@@ -1137,9 +1334,13 @@ VBOXHDDBACKEND g_RawBackend =
     rawAsyncRead,
     /* pfnAsyncWrite */
     rawAsyncWrite,
+    /* pfnAsyncFlush */
+    rawAsyncFlush,
     /* pfnComposeLocation */
     genericFileComposeLocation,
     /* pfnComposeName */
-    genericFileComposeName
+    genericFileComposeName,
+    /* pfnCompact */
+    NULL
 };
 
