@@ -1,20 +1,19 @@
 #ifdef VBOX
+/* $Id: fdc.c 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
- *
- * VBox storage devices:
- * Floppy disk controller
+ * VBox storage devices: Floppy disk controller
  */
 
 /*
- * Copyright (C) 2006-2007 innotek GmbH
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
  * you can redistribute it and/or modify it under the terms of the GNU
- * General Public License as published by the Free Software Foundation,
- * in version 2 as it comes in the "COPYING" file of the VirtualBox OSE
- * distribution. VirtualBox OSE is distributed in the hope that it will
- * be useful, but WITHOUT ANY WARRANTY of any kind.
+ * General Public License (GPL) as published by the Free Software
+ * Foundation, in version 2 as it comes in the "COPYING" file of the
+ * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
+ * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  * --------------------------------------------------------------------
  *
  * This code is based on:
@@ -50,8 +49,8 @@
 #define LOG_GROUP LOG_GROUP_DEV_FDC
 #include <VBox/pdmdev.h>
 #include <iprt/assert.h>
-#include <iprt/uuid.h>
 #include <iprt/string.h>
+#include <iprt/uuid.h>
 
 #include "Builtins.h"
 #include "../vl_vbox.h"
@@ -127,30 +126,48 @@ typedef enum fdrive_type_t {
     FDRIVE_DRV_288  = 0x01,   /* 2.88 MB 3"5 drive      */
     FDRIVE_DRV_120  = 0x02,   /* 1.2  MB 5"25 drive     */
     FDRIVE_DRV_NONE = 0x03    /* No drive connected     */
+#ifdef VBOX
+    , FDRIVE_DRV_DUMMY = INT32_MAX
+#endif
 } fdrive_type_t;
 
 typedef enum fdrive_flags_t {
     FDRIVE_MOTOR_ON   = 0x01, /* motor on/off           */
     FDRIVE_REVALIDATE = 0x02  /* Revalidated            */
+#ifdef VBOX
+    , FDRIVE_DUMMY = INT32_MAX
+#endif
 } fdrive_flags_t;
 
 typedef enum fdisk_flags_t {
     FDISK_DBL_SIDES  = 0x01
+#ifdef VBOX
+    , FDDISK_DUMMY = INT32_MAX
+#endif
 } fdisk_flags_t;
 
+#ifdef VBOX
+/**
+ * The status for one drive.
+ *
+ * @implements  PDMIBASE
+ * @implements  PDMIBLOCKPORT
+ * @implements  PDMIMOUNTNOTIFY
+ */
+#endif
 typedef struct fdrive_t {
 #ifndef VBOX
     BlockDriverState *bs;
 #else /* VBOX */
     /** Pointer to the attached driver's base interface. */
-    HCPTRTYPE(PPDMIBASE)            pDrvBase;
+    R3PTRTYPE(PPDMIBASE)            pDrvBase;
     /** Pointer to the attached driver's block interface. */
-    HCPTRTYPE(PPDMIBLOCK)           pDrvBlock;
+    R3PTRTYPE(PPDMIBLOCK)           pDrvBlock;
     /** Pointer to the attached driver's block bios interface. */
-    HCPTRTYPE(PPDMIBLOCKBIOS)       pDrvBlockBios;
+    R3PTRTYPE(PPDMIBLOCKBIOS)       pDrvBlockBios;
     /** Pointer to the attached driver's mount interface.
      * This is NULL if the driver isn't a removable unit. */
-    HCPTRTYPE(PPDMIMOUNT)           pDrvMount;
+    R3PTRTYPE(PPDMIMOUNT)           pDrvMount;
     /** The base interface. */
     PDMIBASE                        IBase;
     /** The block port interface. */
@@ -197,14 +214,14 @@ static void fd_init (fdrive_t *drv, BlockDriverState *bs)
 }
 #endif
 
-static int _fd_sector (uint8_t head, uint8_t track,
+static unsigned _fd_sector (uint8_t head, uint8_t track,
                         uint8_t sect, uint8_t last_sect)
 {
-    return (((track * 2) + head) * last_sect) + sect - 1;
+    return (((track * 2) + head) * last_sect) + sect - 1; /* sect >= 1 */
 }
 
 /* Returns current position, in sectors, for given drive */
-static int fd_sector (fdrive_t *drv)
+static unsigned fd_sector (fdrive_t *drv)
 {
     return _fd_sector(drv->head, drv->track, drv->sect, drv->last_sect);
 }
@@ -351,8 +368,8 @@ static void fd_revalidate (fdrive_t *drv)
 #else /* VBOX */
             /* @todo */ /** @todo r=bird: todo what exactly?!?!? */
             {
-                uint64_t size =  drv->pDrvBlock->pfnGetSize (drv->pDrvBlock);
-                nb_sectors = size / 512;
+                uint64_t size2 = drv->pDrvBlock->pfnGetSize (drv->pDrvBlock);
+                nb_sectors = size2 / 512;
             }
 #endif /* VBOX */
             match = -1;
@@ -490,6 +507,13 @@ do { (state) = ((state) & ~FD_STATE_STATE) | (new_state); } while (0)
 #define FD_DID_SEEK(state) ((state) & FD_STATE_SEEK)
 #define FD_FORMAT_CMD(state) ((state) & FD_STATE_FORMAT)
 
+#ifdef VBOX
+/**
+ * Floppy controller state.
+ *
+ * @implements  PDMILEDPORTS
+ */
+#endif
 struct fdctrl_t {
 #ifndef VBOX
     fdctrl_t *fdctrl;
@@ -535,11 +559,11 @@ struct fdctrl_t {
     /** Pointer to device instance. */
     PPDMDEVINS pDevIns;
 
-    /** Status Port - The base interface. */
+    /** Status LUN: The base interface. */
     PDMIBASE IBaseStatus;
-    /** Status Port - The Leds interface. */
+    /** Status LUN: The Leds interface. */
     PDMILEDPORTS ILeds;
-    /** Status Port - The Partner of ILeds. */
+    /** Status LUN: The Partner of ILeds. */
     PPDMILEDCONNECTORS pLedsConnector;
 #endif
 };
@@ -708,8 +732,7 @@ static void fdctrl_reset_irq (fdctrl_t *fdctrl)
 {
     FLOPPY_DPRINTF("Reset interrupt\n");
 #ifdef VBOX
-    fdctrl->pDevIns->pDevHlp->pfnISASetIrq (fdctrl->pDevIns,
-                                            fdctrl->irq_lvl, 0);
+    PDMDevHlpISASetIrq (fdctrl->pDevIns, fdctrl->irq_lvl, 0);
 #else
     pic_set_irq(fdctrl->irq_lvl, 0);
 #endif
@@ -720,8 +743,7 @@ static void fdctrl_raise_irq (fdctrl_t *fdctrl, uint8_t status)
 {
     if (~(fdctrl->state & FD_CTRL_INTR)) {
 #ifdef VBOX
-        fdctrl->pDevIns->pDevHlp->pfnISASetIrq (fdctrl->pDevIns,
-                                                fdctrl->irq_lvl, 1);
+        PDMDevHlpISASetIrq (fdctrl->pDevIns, fdctrl->irq_lvl, 1);
 #else
         pic_set_irq(fdctrl->irq_lvl, 1);
 #endif
@@ -996,9 +1018,7 @@ static void fdctrl_stop_transfer (fdctrl_t *fdctrl, uint8_t status0,
     fdctrl->data_dir = FD_DIR_READ;
     if (fdctrl->state & FD_CTRL_BUSY) {
 #ifdef VBOX
-        fdctrl->pDevIns->pDevHlp->pfnDMASetDREQ (fdctrl->pDevIns,
-                                                 fdctrl->dma_chann,
-                                                 0);
+        PDMDevHlpDMASetDREQ (fdctrl->pDevIns, fdctrl->dma_chann, 0);
 #else
         DMA_release_DREQ(fdctrl->dma_chann);
 #endif
@@ -1080,10 +1100,7 @@ static void fdctrl_start_transfer (fdctrl_t *fdctrl, int direction)
 #ifndef VBOX
         dma_mode = DMA_get_channel_mode(fdctrl->dma_chann);
 #else
-        dma_mode = fdctrl->pDevIns->pDevHlp->pfnDMAGetChannelMode (
-            fdctrl->pDevIns,
-            fdctrl->dma_chann
-            );
+        dma_mode = PDMDevHlpDMAGetChannelMode (fdctrl->pDevIns, fdctrl->dma_chann);
 #endif
         dma_mode = (dma_mode >> 2) & 3;
         FLOPPY_DPRINTF("dma_mode=%d direction=%d (%d - %d)\n",
@@ -1103,10 +1120,8 @@ static void fdctrl_start_transfer (fdctrl_t *fdctrl, int direction)
             DMA_hold_DREQ(fdctrl->dma_chann);
             DMA_schedule(fdctrl->dma_chann);
 #else
-            fdctrl->pDevIns->pDevHlp->pfnDMASetDREQ (fdctrl->pDevIns,
-                                                     fdctrl->dma_chann,
-                                                     1);
-            fdctrl->pDevIns->pDevHlp->pfnDMASchedule (fdctrl->pDevIns);
+            PDMDevHlpDMASetDREQ (fdctrl->pDevIns, fdctrl->dma_chann, 1);
+            PDMDevHlpDMASchedule (fdctrl->pDevIns);
 #endif
             return;
         } else {
@@ -1233,9 +1248,9 @@ static int fdctrl_transfer_handler (void *opaque, int nchan,
 
             cur_drv->Led.Actual.s.fReading = 0;
 
-            if (VBOX_FAILURE (rc)) {
+            if (RT_FAILURE (rc)) {
                 AssertMsgFailed (
-                    ("Floppy: error getting sector %d, rc = %Vrc",
+                    ("Floppy: error getting sector %d, rc = %Rrc",
                      fd_sector (cur_drv), rc));
                 memset(fdctrl->fifo, 0, FD_SECTOR_LEN);
             }
@@ -1247,16 +1262,12 @@ static int fdctrl_transfer_handler (void *opaque, int nchan,
 #ifdef VBOX
             {
                 uint32_t read;
-                int rc = fdctrl->pDevIns->pDevHlp->pfnDMAWriteMemory(
-                    fdctrl->pDevIns,
-                    nchan,
-                    fdctrl->fifo + rel_pos,
-                    fdctrl->data_pos,
-                    len,
-                    &read);
+                int rc2 = PDMDevHlpDMAWriteMemory(fdctrl->pDevIns, nchan,
+                                                  fdctrl->fifo + rel_pos,
+                                                  fdctrl->data_pos,
+                                                  len, &read);
                 dump (fdctrl->fifo + rel_pos, len);
-                AssertMsgRC (rc,
-                             ("DMAWriteMemory -> %Vrc\n", rc));
+                AssertMsgRC (rc2, ("DMAWriteMemory -> %Rrc\n", rc2));
             }
 #else
             DMA_write_memory (nchan, fdctrl->fifo + rel_pos,
@@ -1270,15 +1281,11 @@ static int fdctrl_transfer_handler (void *opaque, int nchan,
 #ifdef VBOX
             {
                 uint32_t written;
-                int rc = fdctrl->pDevIns->pDevHlp->pfnDMAReadMemory(
-                    fdctrl->pDevIns,
-                    nchan,
-                    fdctrl->fifo + rel_pos,
-                    fdctrl->data_pos,
-                    len,
-                    &written);
-                AssertMsgRC (rc,
-                             ("DMAReadMemory -> %Vrc\n", rc));
+                int rc2 = PDMDevHlpDMAReadMemory(fdctrl->pDevIns, nchan,
+                                                 fdctrl->fifo + rel_pos,
+                                                 fdctrl->data_pos,
+                                                 len, &written);
+                AssertMsgRC (rc2, ("DMAReadMemory -> %Rrc\n", rc2));
             }
 #else
             DMA_read_memory (nchan, fdctrl->fifo + rel_pos,
@@ -1306,8 +1313,8 @@ static int fdctrl_transfer_handler (void *opaque, int nchan,
 
             cur_drv->Led.Actual.s.fWriting = 0;
 
-            if (VBOX_FAILURE (rc)) {
-                AssertMsgFailed (("Floppy: error writing sector %d. rc=%Vrc",
+            if (RT_FAILURE (rc)) {
+                AssertMsgFailed (("Floppy: error writing sector %d. rc=%Rrc",
                                   fd_sector (cur_drv), rc));
                 fdctrl_stop_transfer (fdctrl, 0x60, 0x00, 0x00);
                 goto transfer_error;
@@ -1320,18 +1327,10 @@ static int fdctrl_transfer_handler (void *opaque, int nchan,
                 uint8_t tmpbuf[FD_SECTOR_LEN];
                 int ret;
 #ifdef VBOX
-                int rc;
                 uint32_t read;
-
-                rc = fdctrl->pDevIns->pDevHlp->pfnDMAReadMemory (
-                    fdctrl->pDevIns,
-                    nchan,
-                    tmpbuf,
-                    fdctrl->data_pos,
-                    len,
-                    &read);
-                AssertMsg (VBOX_SUCCESS (rc),
-                           ("DMAReadMemory -> %Vrc\n", rc));
+                int rc2 = PDMDevHlpDMAReadMemory (fdctrl->pDevIns, nchan, tmpbuf,
+                                                  fdctrl->data_pos, len, &read);
+                AssertMsg (RT_SUCCESS (rc2), ("DMAReadMemory -> %Rrc2\n", rc2));
 #else
                 DMA_read_memory (nchan, tmpbuf, fdctrl->data_pos, len);
 #endif
@@ -1469,9 +1468,9 @@ transfer_error:
 
             cur_drv->Led.Actual.s.fReading = 0;
 
-            if (VBOX_FAILURE (rc)) {
+            if (RT_FAILURE (rc)) {
                 AssertMsgFailed (
-                    ("Floppy: error getting sector %d. rc=%Vrc\n",
+                    ("Floppy: error getting sector %d. rc=%Rrc\n",
                      fd_sector(cur_drv), rc));
                 /* Sure, image size is too small... */
                 memset(fdctrl->fifo, 0, FD_SECTOR_LEN);
@@ -1483,11 +1482,8 @@ transfer_error:
         case FD_DIR_READ:
             /* READ commands */
 #ifdef VBOX
-            fdctrl->pDevIns->pDevHlp->pfnPhysWrite (
-                fdctrl->pDevIns,
-                addr + fdctrl->data_pos,
-                fdctrl->fifo + rel_pos,
-                len);
+            PDMDevHlpPhysWrite (fdctrl->pDevIns, addr + fdctrl->data_pos,
+                                fdctrl->fifo + rel_pos, len);
 #else
             cpu_physical_memory_write(addr + fdctrl->data_pos,
                                       fdctrl->fifo + rel_pos, len);
@@ -1499,11 +1495,8 @@ transfer_error:
             {
                 int rc;
 
-                fdctrl->pDevIns->pDevHlp->pfnPhysRead (
-                    fdctrl->pDevIns,
-                    addr + fdctrl->data_pos,
-                    fdctrl->fifo + rel_pos,
-                    len);
+                PDMDevHlpPhysRead (fdctrl->pDevIns, addr + fdctrl->data_pos,
+                                   fdctrl->fifo + rel_pos, len);
 
                 cur_drv->Led.Asserted.s.fWriting
                     = cur_drv->Led.Actual.s.fWriting = 1;
@@ -1517,9 +1510,9 @@ transfer_error:
 
                 cur_drv->Led.Actual.s.fWriting = 0;
 
-                if (VBOX_FAILURE (rc)) {
+                if (RT_FAILURE (rc)) {
                     AssertMsgFailed (
-                        ("Floppy: error writting sector %d. rc=%Vrc\n",
+                        ("Floppy: error writting sector %d. rc=%Rrc\n",
                          fd_sector(cur_drv), rc));
                     fdctrl_stop_transfer(fdctrl, 0x60, 0x00, 0x00);
                     goto transfer_error;
@@ -1543,11 +1536,8 @@ transfer_error:
                 uint8_t tmpbuf[FD_SECTOR_LEN];
                 int ret;
 #ifdef VBOX
-                fdctrl->pDevIns->pDevHlp->pfnPhysRead (
-                    fdctrl->pDevIns,
-                    addr + fdctrl->data_pos,
-                    tmpbuf,
-                    len);
+                PDMDevHlpPhysRead (fdctrl->pDevIns, addr + fdctrl->data_pos,
+                                   tmpbuf, len);
 #else
                 cpu_physical_memory_read(addr + fdctrl->data_pos,
                                          tmpbuf, len);
@@ -1624,7 +1614,7 @@ static uint32_t fdctrl_read_data (fdctrl_t *fdctrl)
 {
     fdrive_t *cur_drv;
     uint32_t retval = 0;
-    int pos, len;
+    unsigned pos, len;
 #ifdef VBOX
     int rc;
 #endif
@@ -1646,18 +1636,19 @@ static uint32_t fdctrl_read_data (fdctrl_t *fdctrl)
             cur_drv->Led.Asserted.s.fReading
                 = cur_drv->Led.Actual.s.fReading = 1;
 
+            Assert(len <= sizeof(fdctrl->fifo));
             rc = cur_drv->pDrvBlock->pfnRead (
                 cur_drv->pDrvBlock,
                 fd_sector (cur_drv) * 512,
                 fdctrl->fifo,
-                len * 512
+                len
                 );
 
             cur_drv->Led.Actual.s.fReading = 0;
 
-            if (VBOX_FAILURE (rc)) {
+            if (RT_FAILURE (rc)) {
                 AssertMsgFailed (
-                    ("Floppy: Failure to read sector %d. rc=%Vrc",
+                    ("Floppy: Failure to read sector %d. rc=%Rrc",
                      fd_sector (cur_drv), rc));
             }
 #else
@@ -1746,8 +1737,8 @@ static void fdctrl_format_sector (fdctrl_t *fdctrl)
 
         cur_drv->Led.Actual.s.fWriting = 0;
 
-        if (VBOX_FAILURE (rc)) {
-            AssertMsgFailed (("Floppy: error formating sector %d. rc=%Vrc\n",
+        if (RT_FAILURE (rc)) {
+            AssertMsgFailed (("Floppy: error formating sector %d. rc=%Rrc\n",
                               fd_sector (cur_drv), rc));
             fdctrl_stop_transfer(fdctrl, 0x60, 0x00, 0x00);
         }
@@ -1809,14 +1800,14 @@ static void fdctrl_write_data (fdctrl_t *fdctrl, uint32_t value)
                 cur_drv->pDrvBlock,
                 fd_sector (cur_drv) * 512,
                 fdctrl->fifo,
-                FD_SECTOR_LEN * 512
+                FD_SECTOR_LEN
                 );
 
             cur_drv->Led.Actual.s.fWriting = 0;
 
-            if (VBOX_FAILURE (rc)) {
+            if (RT_FAILURE (rc)) {
                 AssertMsgFailed (
-                    ("Floppy: failed to write sector %d. rc=%Vrc\n",
+                    ("Floppy: failed to write sector %d. rc=%Rrc\n",
                      fd_sector (cur_drv), rc));
             }
 #else
@@ -2370,9 +2361,9 @@ static void fdctrl_result_timer(void *opaque)
 
 
 #ifdef VBOX
-static DECLCALLBACK(void) fdc_timer (PPDMDEVINS pDevIns, PTMTIMER pTimer)
+static DECLCALLBACK(void) fdc_timer (PPDMDEVINS pDevIns, PTMTIMER pTimer, void *pvUser)
 {
-    fdctrl_t *fdctrl = PDMINS2DATA (pDevIns, fdctrl_t *);
+    fdctrl_t *fdctrl = (fdctrl_t *)pvUser;
     fdctrl_result_timer (fdctrl);
 }
 
@@ -2406,9 +2397,9 @@ static DECLCALLBACK(int) fdc_io_read (PPDMDEVINS pDevIns,
     }
 }
 
-static DECLCALLBACK(int) SaveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
+static DECLCALLBACK(int) fdcSaveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
 {
-    fdctrl_t *s = PDMINS2DATA (pDevIns, fdctrl_t *);
+    fdctrl_t *s = PDMINS_2_DATA (pDevIns, fdctrl_t *);
     QEMUFile *f = pSSMHandle;
     unsigned int i;
 
@@ -2455,18 +2446,17 @@ static DECLCALLBACK(int) SaveExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSMHandle)
     return TMR3TimerSave (s->result_timer, pSSMHandle);
 }
 
-static DECLCALLBACK(int) LoadExec (PPDMDEVINS pDevIns,
-                                   PSSMHANDLE pSSMHandle,
-                                   uint32_t u32Version)
+static DECLCALLBACK(int) fdcLoadExec (PPDMDEVINS pDevIns,
+                                      PSSMHANDLE pSSMHandle,
+                                      uint32_t uVersion,
+                                      uint32_t uPass)
 {
-    fdctrl_t *s = PDMINS2DATA (pDevIns, fdctrl_t *);
+    fdctrl_t *s = PDMINS_2_DATA (pDevIns, fdctrl_t *);
     QEMUFile *f = pSSMHandle;
     unsigned int i;
 
-    if (u32Version != 1) {
-        AssertMsgFailed(("u32Version=%d\n", u32Version));
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
-    }
+    AssertMsgReturn(uVersion == 1, ("%d\n", uVersion), VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION);
+    Assert(uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     qemu_get_8s (f, &s->version);
     qemu_get_8s (f, &s->irq_lvl);
@@ -2512,27 +2502,16 @@ static DECLCALLBACK(int) LoadExec (PPDMDEVINS pDevIns,
 }
 
 /**
- * Queries an interface to the driver.
- *
- * @returns Pointer to interface.
- * @returns NULL if the interface was not supported by the device.
- * @param   pInterface          Pointer to IDEState::IBase.
- * @param   enmInterface        The requested interface identification.
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
  */
-static DECLCALLBACK(void *) fdQueryInterface (PPDMIBASE pInterface,
-                                              PDMINTERFACE enmInterface)
+static DECLCALLBACK(void *) fdQueryInterface (PPDMIBASE pInterface, const char *pszIID)
 {
-    fdrive_t *drv = PDMIBASE_2_FDRIVE(pInterface);
-    switch (enmInterface) {
-    case PDMINTERFACE_BASE:
-        return &drv->IBase;
-    case PDMINTERFACE_BLOCK_PORT:
-        return &drv->IPort;
-    case PDMINTERFACE_MOUNT_NOTIFY:
-        return &drv->IMountNotify;
-    default:
-        return NULL;
-    }
+    fdrive_t *pDrive = PDMIBASE_2_FDRIVE(pInterface);
+
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pDrive->IBase);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBLOCKPORT, &pDrive->IPort);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIMOUNTNOTIFY, &pDrive->IMountNotify);
+    return NULL;
 }
 
 /**
@@ -2549,7 +2528,7 @@ static DECLCALLBACK(int) fdcStatusQueryStatusLed (PPDMILEDPORTS pInterface,
 {
     fdctrl_t *fdctrl = (fdctrl_t *)
         ((uintptr_t )pInterface - RT_OFFSETOF (fdctrl_t, ILeds));
-    if (iLUN < ELEMENTS(fdctrl->drives)) {
+    if (iLUN < RT_ELEMENTS(fdctrl->drives)) {
         *ppLed = &fdctrl->drives[iLUN].Led;
         Assert ((*ppLed)->u32Magic == PDMLED_MAGIC);
         return VINF_SUCCESS;
@@ -2559,26 +2538,15 @@ static DECLCALLBACK(int) fdcStatusQueryStatusLed (PPDMILEDPORTS pInterface,
 
 
 /**
- * Queries an interface to the status LUN.
- *
- * @returns Pointer to interface.
- * @returns NULL if the interface was not supported by the device.
- * @param   pInterface          Pointer to IDEState::IBase.
- * @param   enmInterface        The requested interface identification.
+ * @interface_method_impl{PDMIBASE,pfnQueryInterface}
  */
-static DECLCALLBACK(void *) fdcStatusQueryInterface (PPDMIBASE pInterface,
-                                                     PDMINTERFACE enmInterface)
+static DECLCALLBACK(void *) fdcStatusQueryInterface(PPDMIBASE pInterface, const char *pszIID)
 {
-    fdctrl_t *fdctrl = (fdctrl_t *)
-        ((uintptr_t)pInterface - RT_OFFSETOF (fdctrl_t, IBaseStatus));
-    switch (enmInterface) {
-    case PDMINTERFACE_BASE:
-        return &fdctrl->IBaseStatus;
-    case PDMINTERFACE_LED_PORTS:
-        return &fdctrl->ILeds;
-    default:
-        return NULL;
-    }
+    fdctrl_t *pThis = RT_FROM_MEMBER (pInterface, fdctrl_t, IBaseStatus);
+
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMIBASE, &pThis->IBaseStatus);
+    PDMIBASE_RETURN_INTERFACE(pszIID, PDMILEDPORTS, &pThis->ILeds);
+    return NULL;
 }
 
 
@@ -2597,7 +2565,7 @@ static int fdConfig (fdrive_t *drv, PPDMDEVINS pDevIns)
     /*
      * Reset the LED just to be on the safe side.
      */
-    Assert (ELEMENTS(descs) > drv->iLUN);
+    Assert (RT_ELEMENTS(descs) > drv->iLUN);
     Assert (drv->Led.u32Magic == PDMLED_MAGIC);
     drv->Led.Actual.u32 = 0;
     drv->Led.Asserted.u32 = 0;
@@ -2606,22 +2574,12 @@ static int fdConfig (fdrive_t *drv, PPDMDEVINS pDevIns)
      * Try attach the block device and get the interfaces.
      */
     rc = PDMDevHlpDriverAttach (pDevIns, drv->iLUN, &drv->IBase, &drv->pDrvBase, descs[drv->iLUN]);
-    if (VBOX_SUCCESS (rc))
-    {
-        drv->pDrvBlock = drv->pDrvBase->pfnQueryInterface (
-            drv->pDrvBase,
-            PDMINTERFACE_BLOCK
-            );
+    if (RT_SUCCESS (rc)) {
+        drv->pDrvBlock = PDMIBASE_QUERY_INTERFACE(drv->pDrvBase, PDMIBLOCK);
         if (drv->pDrvBlock) {
-            drv->pDrvBlockBios = drv->pDrvBase->pfnQueryInterface (
-                drv->pDrvBase,
-                PDMINTERFACE_BLOCK_BIOS
-                );
+            drv->pDrvBlockBios = PDMIBASE_QUERY_INTERFACE(drv->pDrvBase, PDMIBLOCKBIOS);
             if (drv->pDrvBlockBios) {
-                drv->pDrvMount = drv->pDrvBase->pfnQueryInterface (
-                    drv->pDrvBase,
-                    PDMINTERFACE_MOUNT
-                    );
+                drv->pDrvMount = PDMIBASE_QUERY_INTERFACE(drv->pDrvBase, PDMIMOUNT);
                 if (drv->pDrvMount) {
                     /*
                      * Init the state.
@@ -2633,36 +2591,44 @@ static int fdConfig (fdrive_t *drv, PPDMDEVINS pDevIns)
                     drv->last_sect = 0;
                     drv->max_track = 0;
                     drv->fMediaPresent = false;
-                }
-                else {
+                } else {
                     AssertMsgFailed (("Configuration error: LUN#%d without mountable interface!\n", drv->iLUN));
                     rc = VERR_PDM_MISSING_INTERFACE;
                 }
 
-            }
-            else {
+            } else {
                 AssertMsgFailed (("Configuration error: LUN#%d hasn't a block BIOS interface!\n", drv->iLUN));
                 rc = VERR_PDM_MISSING_INTERFACE;
             }
 
-        }
-        else {
+        } else {
             AssertMsgFailed (("Configuration error: LUN#%d hasn't a block interface!\n", drv->iLUN));
             rc = VERR_PDM_MISSING_INTERFACE;
         }
-    }
-    else {
+    } else {
         AssertMsg (rc == VERR_PDM_NO_ATTACHED_DRIVER,
-                   ("Failed to attach LUN#%d. rc=%Vrc\n", drv->iLUN, rc));
+                   ("Failed to attach LUN#%d. rc=%Rrc\n", drv->iLUN, rc));
+        switch (rc) {
+        case VERR_ACCESS_DENIED:
+            /* Error already catched by DrvHostBase */
+            break;
+        case VERR_PDM_NO_ATTACHED_DRIVER:
+            /* Legal on architectures without a floppy controller */
+            break;
+        default:
+            rc = PDMDevHlpVMSetError (pDevIns, rc, RT_SRC_POS,
+                                      N_ ("The floppy controller cannot attach to the floppy drive"));
+            break;
+        }
     }
 
-    if (VBOX_FAILURE (rc)) {
+    if (RT_FAILURE (rc)) {
         drv->pDrvBase = NULL;
         drv->pDrvBlock = NULL;
         drv->pDrvBlockBios = NULL;
         drv->pDrvMount = NULL;
     }
-    LogFlow (("fdConfig: returns %Vrc\n", rc));
+    LogFlow (("fdConfig: returns %Rrc\n", rc));
     return rc;
 }
 
@@ -2677,13 +2643,17 @@ static int fdConfig (fdrive_t *drv, PPDMDEVINS pDevIns)
  * @param   iLUN        The logical unit which is being detached.
  */
 static DECLCALLBACK(int)  fdcAttach (PPDMDEVINS pDevIns,
-                                     unsigned iLUN)
+                                     unsigned iLUN, uint32_t fFlags)
 {
-    fdctrl_t *fdctrl = PDMINS2DATA (pDevIns, fdctrl_t *);
+    fdctrl_t *fdctrl = PDMINS_2_DATA (pDevIns, fdctrl_t *);
     fdrive_t *drv;
     int rc;
     LogFlow (("ideDetach: iLUN=%u\n", iLUN));
 
+    AssertMsgReturn(fFlags & PDM_TACH_FLAGS_NOT_HOT_PLUG,
+                    ("The FDC device does not support hotplugging\n"),
+                    VERR_INVALID_PARAMETER);
+
     /*
      * Validate.
      */
@@ -2706,12 +2676,12 @@ static DECLCALLBACK(int)  fdcAttach (PPDMDEVINS pDevIns,
 
     rc = fdConfig (drv, pDevIns);
     AssertMsg (rc != VERR_PDM_NO_ATTACHED_DRIVER,
-               ("Configuration error: failed to configure drive %d, rc=%Vrc\n", rc));
-    if (VBOX_SUCCESS(rc)) {
+               ("Configuration error: failed to configure drive %d, rc=%Rrc\n", rc));
+    if (RT_SUCCESS(rc)) {
         fd_revalidate (drv);
     }
 
-    LogFlow (("floppyAttach: returns %Vrc\n", rc));
+    LogFlow (("floppyAttach: returns %Rrc\n", rc));
     return rc;
 }
 
@@ -2725,9 +2695,9 @@ static DECLCALLBACK(int)  fdcAttach (PPDMDEVINS pDevIns,
  * @param   iLUN        The logical unit which is being detached.
  */
 static DECLCALLBACK(void) fdcDetach (PPDMDEVINS pDevIns,
-                                     unsigned iLUN)
+                                     unsigned iLUN, uint32_t fFlags)
 {
-    fdctrl_t *fdctrl = PDMINS2DATA (pDevIns, fdctrl_t *);
+    fdctrl_t *fdctrl = PDMINS_2_DATA (pDevIns, fdctrl_t *);
     LogFlow (("ideDetach: iLUN=%u\n", iLUN));
 
     switch (iLUN) {
@@ -2759,89 +2729,75 @@ static DECLCALLBACK(void) fdcDetach (PPDMDEVINS pDevIns,
  */
 static DECLCALLBACK(void) fdcReset (PPDMDEVINS pDevIns)
 {
-    fdctrl_t *fdctrl = PDMINS2DATA (pDevIns, fdctrl_t *);
-    int i;
+    fdctrl_t *fdctrl = PDMINS_2_DATA (pDevIns, fdctrl_t *);
+    unsigned i;
     LogFlow (("fdcReset:\n"));
 
     fdctrl_reset(fdctrl, 0);
     fdctrl->state = FD_CTRL_ACTIVE;
 
-    for (i = 0; i < ELEMENTS(fdctrl->drives); i++) {
+    for (i = 0; i < RT_ELEMENTS(fdctrl->drives); i++) {
         fd_revalidate(&fdctrl->drives[i]);
     }
 }
 
 
 /**
- * Construct a device instance for a VM.
- *
- * @returns VBox status.
- * @param   pDevIns     The device instance data.
- *                      If the registration structure is needed, pDevIns->pDevReg points to it.
- * @param   iInstance   Instance number. Use this to figure out which registers and such to use.
- *                      The device number is also found in pDevIns->iInstance, but since it's
- *                      likely to be freqently used PDM passes it as parameter.
- * @param   pCfgHandle  Configuration node handle for the device. Use this to obtain the configuration
- *                      of the device instance. It's also found in pDevIns->pCfgHandle, but like
- *                      iInstance it's expected to be used a bit in this function.
+ * @interface_method_impl{PDMDEVREG,pfnConstruct}
  */
 static DECLCALLBACK(int) fdcConstruct (PPDMDEVINS pDevIns,
                                        int iInstance,
-                                       PCFGMNODE pCfgHandle)
+                                       PCFGMNODE pCfg)
 {
     int            rc;
-    fdctrl_t       *fdctrl = PDMINS2DATA(pDevIns, fdctrl_t*);
-    int            i;
+    fdctrl_t       *fdctrl = PDMINS_2_DATA(pDevIns, fdctrl_t*);
+    unsigned       i;
     bool           mem_mapped;
     uint16_t       io_base;
     uint8_t        irq_lvl, dma_chann;
     PPDMIBASE      pBase;
 
     Assert(iInstance == 0);
+    PDMDEV_CHECK_VERSIONS_RETURN(pDevIns);
 
     /*
      * Validate configuration.
      */
-    if (!CFGMR3AreValuesValid(pCfgHandle, "IRQ\0DMA\0MemMapped\0IOBase\0")) {
+    if (!CFGMR3AreValuesValid(pCfg, "IRQ\0DMA\0MemMapped\0IOBase\0"))
         return VERR_PDM_DEVINS_UNKNOWN_CFG_VALUES;
-    }
 
     /*
      * Read the configuration.
      */
-    rc = CFGMR3QueryU8 (pCfgHandle, "IRQ", &irq_lvl);
-    if (rc == VERR_CFGM_VALUE_NOT_FOUND) {
+    rc = CFGMR3QueryU8 (pCfg, "IRQ", &irq_lvl);
+    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         irq_lvl = 6;
-    }
-    else if (VBOX_FAILURE (rc)) {
-        AssertMsgFailed (("Configuration error: Failed to read U8 IRQ, rc=%Vrc\n", rc));
+    else if (RT_FAILURE (rc)) {
+        AssertMsgFailed (("Configuration error: Failed to read U8 IRQ, rc=%Rrc\n", rc));
         return rc;
     }
 
-    rc = CFGMR3QueryU8 (pCfgHandle, "DMA", &dma_chann);
-    if (rc == VERR_CFGM_VALUE_NOT_FOUND) {
+    rc = CFGMR3QueryU8 (pCfg, "DMA", &dma_chann);
+    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         dma_chann = 2;
-    }
-    else if (VBOX_FAILURE (rc)) {
-        AssertMsgFailed (("Configuration error: Failed to read U8 DMA, rc=%Vrc\n", rc));
+    else if (RT_FAILURE (rc)) {
+        AssertMsgFailed (("Configuration error: Failed to read U8 DMA, rc=%Rrc\n", rc));
         return rc;
     }
 
-    rc = CFGMR3QueryU16 (pCfgHandle, "IOBase", &io_base);
-    if (rc == VERR_CFGM_VALUE_NOT_FOUND) {
+    rc = CFGMR3QueryU16 (pCfg, "IOBase", &io_base);
+    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         io_base = 0x3f0;
-    }
-    else if (VBOX_FAILURE (rc)) {
-        AssertMsgFailed (("Configuration error: Failed to read U16 IOBase, rc=%Vrc\n", rc));
+    else if (RT_FAILURE (rc)) {
+        AssertMsgFailed (("Configuration error: Failed to read U16 IOBase, rc=%Rrc\n", rc));
         return rc;
     }
 
-    rc = CFGMR3QueryBool (pCfgHandle, "MemMapped", &mem_mapped);
-    if (rc == VERR_CFGM_VALUE_NOT_FOUND) {
+    rc = CFGMR3QueryBool (pCfg, "MemMapped", &mem_mapped);
+    if (rc == VERR_CFGM_VALUE_NOT_FOUND)
         mem_mapped = false;
-    }
-    else if (VBOX_FAILURE (rc)) {
-        AssertMsgFailed (("Configuration error: Failed to read bool value MemMapped rc=%Vrc\n", rc));
+    else if (RT_FAILURE (rc)) {
+        AssertMsgFailed (("Configuration error: Failed to read bool value MemMapped rc=%Rrc\n", rc));
         return rc;
     }
 
@@ -2859,7 +2815,7 @@ static DECLCALLBACK(int) fdcConstruct (PPDMDEVINS pDevIns,
     fdctrl->IBaseStatus.pfnQueryInterface = fdcStatusQueryInterface;
     fdctrl->ILeds.pfnQueryStatusLed = fdcStatusQueryStatusLed;
 
-    for (i = 0; i < ELEMENTS(fdctrl->drives); ++i) {
+    for (i = 0; i < RT_ELEMENTS(fdctrl->drives); ++i) {
         fdrive_t *drv = &fdctrl->drives[i];
 
         drv->drive = FDRIVE_DRV_NONE;
@@ -2874,27 +2830,21 @@ static DECLCALLBACK(int) fdcConstruct (PPDMDEVINS pDevIns,
     /*
      * Create the FDC timer.
      */
-    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL, fdc_timer, "FDC Timer", &fdctrl->result_timer);
-    if (VBOX_FAILURE (rc)) {
+    rc = PDMDevHlpTMTimerCreate(pDevIns, TMCLOCK_VIRTUAL, fdc_timer, fdctrl,
+                                TMTIMER_FLAGS_DEFAULT_CRIT_SECT, "FDC Timer", &fdctrl->result_timer);
+    if (RT_FAILURE (rc))
         return rc;
-    }
 
     /*
      * Register DMA channel.
      */
     if (fdctrl->dma_chann != 0xff) {
         fdctrl->dma_en = 1;
-        rc = pDevIns->pDevHlp->pfnDMARegister (
-            pDevIns,
-            dma_chann,
-            &fdctrl_transfer_handler,
-            fdctrl);
-        if (VBOX_FAILURE (rc)) {
+        rc = PDMDevHlpDMARegister (pDevIns, dma_chann, &fdctrl_transfer_handler, fdctrl);
+        if (RT_FAILURE (rc))
             return rc;
-        }
-    } else {
+    } else
         fdctrl->dma_en = 0;
-    }
 
     /*
      * IO / MMIO.
@@ -2908,64 +2858,32 @@ static DECLCALLBACK(int) fdcConstruct (PPDMDEVINS pDevIns,
         cpu_register_physical_memory(base, 0x08, io_mem);
 #endif
     } else {
-        rc = pDevIns->pDevHlp->pfnIOPortRegister (
-            pDevIns,
-            io_base + 0x1,
-            5,
-            fdctrl,
-            fdc_io_write,
-            fdc_io_read,
-            NULL, NULL,
-            "FDC#1"
-            );
-        if (VBOX_FAILURE (rc)) {
+        rc = PDMDevHlpIOPortRegister (pDevIns, io_base + 0x1, 5, fdctrl,
+                                      fdc_io_write, fdc_io_read, NULL, NULL, "FDC#1");
+        if (RT_FAILURE (rc))
             return rc;
-        }
 
-        rc = pDevIns->pDevHlp->pfnIOPortRegister (
-            pDevIns,
-            io_base + 0x7,
-            1,
-            fdctrl,
-            fdc_io_write,
-            fdc_io_read,
-            NULL, NULL,
-            "FDC#2"
-            );
-        if (VBOX_FAILURE (rc)) {
+        rc = PDMDevHlpIOPortRegister (pDevIns, io_base + 0x7, 1, fdctrl,
+                                      fdc_io_write, fdc_io_read, NULL, NULL, "FDC#2");
+        if (RT_FAILURE (rc))
             return rc;
-        }
     }
 
     /*
      * Register the saved state data unit.
      */
-    rc = pDevIns->pDevHlp->pfnSSMRegister (
-        pDevIns,                /* pDevIns */
-        pDevIns->pDevReg->szDeviceName, /* pszName */
-        iInstance,              /* u32Instance */
-        1                       /* u32Version */,
-        sizeof (*fdctrl),       /* cbGuess */
-        NULL,                   /* pfnSavePrep */
-        SaveExec,               /* pfnSaveExec */
-        NULL,                   /* pfnSaveDone */
-        NULL,                   /* pfnLoadPrep */
-        LoadExec,               /* pfnLoadExec */
-        NULL                    /* pfnLoadDone */
-        );
-    if (VBOX_FAILURE(rc))
+    rc = PDMDevHlpSSMRegister (pDevIns, 1 /*uVersion*/, sizeof(*fdctrl), fdcSaveExec, fdcLoadExec);
+    if (RT_FAILURE(rc))
         return rc;
 
     /*
      * Attach the status port (optional).
      */
     rc = PDMDevHlpDriverAttach (pDevIns, PDM_STATUS_LUN, &fdctrl->IBaseStatus, &pBase, "Status Port");
-    if (VBOX_SUCCESS (rc)) {
-        fdctrl->pLedsConnector =
-            pBase->pfnQueryInterface (pBase, PDMINTERFACE_LED_CONNECTORS);
-    }
-    else if (rc != VERR_PDM_NO_ATTACHED_DRIVER) {
-        AssertMsgFailed (("Failed to attach to status driver. rc=%Vrc\n",
+    if (RT_SUCCESS (rc)) {
+        fdctrl->pLedsConnector = PDMIBASE_QUERY_INTERFACE(pBase, PDMILEDCONNECTORS);
+    } else if (rc != VERR_PDM_NO_ATTACHED_DRIVER) {
+        AssertMsgFailed (("Failed to attach to status driver. rc=%Rrc\n",
                           rc));
         return rc;
     }
@@ -2973,12 +2891,12 @@ static DECLCALLBACK(int) fdcConstruct (PPDMDEVINS pDevIns,
     /*
      * Initialize drives.
      */
-    for (i = 0; i < ELEMENTS(fdctrl->drives); i++) {
+    for (i = 0; i < RT_ELEMENTS(fdctrl->drives); i++) {
         fdrive_t *drv = &fdctrl->drives[i];
         rc = fdConfig (drv, pDevIns);
-        if (    VBOX_FAILURE (rc)
+        if (    RT_FAILURE (rc)
             &&  rc != VERR_PDM_NO_ATTACHED_DRIVER) {
-            AssertMsgFailed (("Configuration error: failed to configure drive %d, rc=%Vrc\n", rc));
+            AssertMsgFailed (("Configuration error: failed to configure drive %d, rc=%Rrc\n", rc));
             return rc;
         }
     }
@@ -2986,9 +2904,8 @@ static DECLCALLBACK(int) fdcConstruct (PPDMDEVINS pDevIns,
     fdctrl_reset(fdctrl, 0);
     fdctrl->state = FD_CTRL_ACTIVE;
 
-    for (i = 0; i < ELEMENTS(fdctrl->drives); i++) {
+    for (i = 0; i < RT_ELEMENTS(fdctrl->drives); i++)
         fd_revalidate(&fdctrl->drives[i]);
-    }
 
     return VINF_SUCCESS;
 }
@@ -3000,16 +2917,16 @@ const PDMDEVREG g_DeviceFloppyController =
 {
     /* u32Version */
     PDM_DEVREG_VERSION,
-    /* szDeviceName */
+    /* szName */
     "i82078",
-    /* szGCMod */
+    /* szRCMod */
     "",
     /* szR0Mod */
     "",
     /* pszDescription */
     "Floppy drive controller (Intel 82078)",
     /* fFlags */
-    PDM_DEVREG_FLAGS_HOST_BITS_DEFAULT | PDM_DEVREG_FLAGS_GUEST_BITS_DEFAULT,
+    PDM_DEVREG_FLAGS_DEFAULT_BITS,
     /* fClass */
     PDM_DEVREG_CLASS_STORAGE,
     /* cMaxInstances */
@@ -3037,7 +2954,15 @@ const PDMDEVREG g_DeviceFloppyController =
     /* pfnDetach */
     fdcDetach,
     /* pfnQueryInterface. */
-    NULL
+    NULL,
+    /* pfnInitComplete */
+    NULL,
+    /* pfnPowerOff */
+    NULL,
+    /* pfnSoftReset */
+    NULL,
+    /* u32VersionEnd */
+    PDM_DEVREG_VERSION
 };
 
 #endif /* VBOX */
