@@ -1,4 +1,4 @@
-/* $Id: USBProxyServiceLinux.cpp 13837 2008-11-05 02:54:02Z vboxsync $ */
+/* $Id: USBProxyServiceLinux.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBox frontends: Basic Frontend (BFE):
  * Implementation of USBProxyServiceLinux class
@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,10 +16,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -32,8 +28,9 @@
 #include <VBox/usb.h>
 #include <VBox/err.h>
 
-#include <iprt/string.h>
 #include <iprt/alloc.h>
+#include <iprt/ctype.h>
+#include <iprt/string.h>
 #include <iprt/assert.h>
 #include <iprt/file.h>
 #include <iprt/err.h>
@@ -41,12 +38,11 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <errno.h>
 #include <sys/statfs.h>
 #include <sys/poll.h>
 #include <unistd.h>
-#ifndef VBOX_WITHOUT_LINUX_COMPILER_H
+#ifdef VBOX_WITH_LINUX_COMPILER_H
 # include <linux/compiler.h>
 #endif
 #include <linux/usbdevice_fs.h>
@@ -281,7 +277,7 @@ int USBProxyServiceLinux::interruptWait (void)
 static int usbReadSkipSuffix (char **ppszNext)
 {
     char *pszNext = *ppszNext;
-    if (!isspace (*pszNext) && *pszNext)
+    if (!RT_C_IS_SPACE (*pszNext) && *pszNext)
     {
         /* skip unit */
         if (pszNext[0] == 'm' && pszNext[1] == 's')
@@ -301,7 +297,7 @@ static int usbReadSkipSuffix (char **ppszNext)
         }
 
         /* blank or end of the line. */
-        if (!isspace (*pszNext) && *pszNext)
+        if (!RT_C_IS_SPACE (*pszNext) && *pszNext)
         {
             AssertMsgFailed (("pszNext=%s\n", pszNext));
             return VERR_PARSE_ERROR;
@@ -358,12 +354,12 @@ static int usbReadNum (const char *pszValue, unsigned uBase, uint32_t u32Mask, P
          */
         if (paSuffs)
         {
-            if (!isspace (*pszNext) && *pszNext)
+            if (!RT_C_IS_SPACE (*pszNext) && *pszNext)
             {
                 for (PCUSBSUFF pSuff = paSuffs; pSuff->szSuff[0]; pSuff++)
                 {
                     if (    !strncmp (pSuff->szSuff, pszNext, pSuff->cchSuff)
-                        &&  (!pszNext[pSuff->cchSuff] || isspace (pszNext[pSuff->cchSuff])))
+                        &&  (!pszNext[pSuff->cchSuff] || RT_C_IS_SPACE (pszNext[pSuff->cchSuff])))
                     {
                         if (pSuff->uDiv)
                             u32 /= pSuff->uDiv;
@@ -506,7 +502,7 @@ static char * usbReadSkip (const char *pszValue)
         psz = strchr (psz + 1, '=');
     if (!psz)
         return strchr (pszValue,  '\0');
-    while (psz > pszValue && !isspace (psz[-1]))
+    while (psz > pszValue && !RT_C_IS_SPACE (psz[-1]))
         psz--;
     Assert (psz > pszValue);
     return psz;
