@@ -1,4 +1,4 @@
-/* $Id: VBoxBFE.cpp $ */
+/* $Id: VBoxBFE.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * Basic Frontend (BFE): VBoxBFE main routines.
  *
@@ -10,7 +10,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -19,10 +19,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -78,7 +74,7 @@ using namespace com;
 #include "DisplayImpl.h"
 #include "MouseImpl.h"
 #include "KeyboardImpl.h"
-#include "VMMDevInterface.h"
+#include "VMMDev.h"
 #include "StatusImpl.h"
 #include "Framebuffer.h"
 #include "MachineDebuggerImpl.h"
@@ -124,9 +120,9 @@ static DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser);
 *   Global Variables                                                           *
 *******************************************************************************/
 
-PVM                pVM              = NULL;
+PVM                gpVM             = NULL;
 Mouse             *gMouse           = NULL;
-VMDisplay         *gDisplay         = NULL;
+Display         *gDisplay         = NULL;
 Keyboard          *gKeyboard        = NULL;
 VMMDev            *gVMMDev          = NULL;
 Framebuffer       *gFramebuffer     = NULL;
@@ -332,10 +328,11 @@ void startProgressInfo(const char *pszStr)
 /**
  * Update progress display.
  */
-void callProgressInfo(PVM pVM, unsigned uPercent, void *pvUser)
+int callProgressInfo(PVM pVM, unsigned uPercent, void *pvUser)
 {
     if (gConsole)
         gConsole->progressInfo(pVM, uPercent, pvUser);
+    return VINF_SUCCESS;
 }
 
 /**
@@ -660,10 +657,10 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char **envp)
                 return SyntaxError("The network MAC address has an invalid length: %s (%s)\n", pszMac, pszArg);
             for (unsigned j = 0; j < RT_ELEMENTS(g_aNetDevs[i].Mac.au8); j++)
             {
-                char c1 = toupper(*pszMac++) - '0';
+                char c1 = RT_C_TO_UPPER(*pszMac++) - '0';
                 if (c1 > 9)
                     c1 -= 7;
-                char c2 = toupper(*pszMac++) - '0';
+                char c2 = RT_C_TO_UPPER(*pszMac++) - '0';
                 if (c2 > 9)
                     c2 -= 7;
                 if (c2 > 16 || c1 > 16)
@@ -788,8 +785,10 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char **envp)
     gStatus = new VMStatus();
     gKeyboard = new Keyboard();
     gMouse = new Mouse();
+    if (FAILED(gMouse->FinalConstruct()))
+        goto leave;
     gVMMDev = new VMMDev();
-    gDisplay = new VMDisplay();
+    gDisplay = new Display();
 #if defined(USE_SDL)
     /* First console, then framebuffer!! */
     gConsole = new SDLConsole();
@@ -802,7 +801,7 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char **envp)
 #endif
     if (!gConsole->initialized())
         goto leave;
-    gDisplay->RegisterExternalFramebuffer(gFramebuffer);
+    gDisplay->SetFramebuffer(0, gFramebuffer);
 
     /* start with something in the titlebar */
     gConsole->updateTitlebar();
@@ -954,7 +953,7 @@ extern "C" DECLEXPORT(int) TrustedMain (int argc, char **argv, char **envp)
 leave:
     LogFlow(("Returning from main()!\n"));
 
-    if (pVM)
+    if (gpVM)
     {
         /*
          * If get here because the guest terminated using ACPI off we don't have to
@@ -964,12 +963,12 @@ leave:
         if (machineState != VMSTATE_OFF)
         {
             /* Power off VM */
-            PVMREQ pReq;
-            rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)VMR3PowerOff, 1, pVM);
+            rc = VMR3PowerOff(gpVM);
+            AssertRC(rc);
         }
 
         /* And destroy it */
-        rc = VMR3Destroy(pVM);
+        rc = VMR3Destroy(gpVM);
         AssertRC(rc);
     }
 
@@ -977,6 +976,7 @@ leave:
     delete gConsole;
     delete gDisplay;
     delete gKeyboard;
+    gMouse->FinalRelease();
     delete gMouse;
     delete gStatus;
     delete gMachineDebugger;
@@ -1138,11 +1138,11 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
     if (g_fReleaseLog)
     {
         static const char * const s_apszGroups[] = VBOX_LOGGROUP_NAMES;
-        static char szError[RTPATH_MAX + 128] = "";
+        static char s_szError[RTPATH_MAX + 128] = "";
         PRTLOGGER pLogger;
         rc2 = RTLogCreateEx(&pLogger, RTLOGFLAGS_PREFIX_TIME_PROG, "all",
                             "VBOX_RELEASE_LOG", RT_ELEMENTS(s_apszGroups), s_apszGroups,
-                            RTLOGDEST_FILE, szError, sizeof(szError), "./VBoxBFE.log");
+                            RTLOGDEST_FILE, s_szError, sizeof(s_szError), "./VBoxBFE.log");
         if (RT_SUCCESS(rc2))
         {
             /* some introductory information */
@@ -1159,7 +1159,7 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
             RTLogRelSetDefaultInstance(pLogger);
         }
         else
-            RTPrintf("Could not open release log (%s)\n", szError);
+            RTPrintf("Could not open release log (%s)\n", s_szError);
     }
 
     /*
@@ -1170,7 +1170,7 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
     /*
      * Create empty VM.
      */
-    rc = VMR3Create(1, setVMErrorCallback, NULL, vboxbfeConfigConstructor, NULL, &pVM);
+    rc = VMR3Create(1, setVMErrorCallback, NULL, vboxbfeConfigConstructor, NULL, &gpVM);
     if (RT_FAILURE(rc))
     {
         RTPrintf("Error: VM creation failed with %Rrc.\n", rc);
@@ -1181,7 +1181,7 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
     /*
      * Register VM state change handler
      */
-    rc = VMR3AtStateRegister(pVM, vmstateChangeCallback, NULL);
+    rc = VMR3AtStateRegister(gpVM, vmstateChangeCallback, NULL);
     if (RT_FAILURE(rc))
     {
         RTPrintf("Error: VMR3AtStateRegister failed with %Rrc.\n", rc);
@@ -1200,12 +1200,11 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
             SHFLSTRING      *pFolderName, *pMapName;
             int              cbString;
             PRTUTF16         aHostPath, aMapName;
-            int              rc;
 
-            rc = RTStrToUtf16(g_pszShareDir[i], &aHostPath);
-            AssertRC(rc);
-            rc = RTStrToUtf16(g_pszShareName[i], &aMapName);
-            AssertRC(rc);
+            rc2 = RTStrToUtf16(g_pszShareDir[i], &aHostPath);
+            AssertRC(rc2);
+            rc2 = RTStrToUtf16(g_pszShareName[i], &aMapName);
+            AssertRC(rc2);
 
             cbString = (RTUtf16Len (aHostPath) + 1) * sizeof (RTUTF16);
             pFolderName = (SHFLSTRING *) RTMemAllocZ (sizeof (SHFLSTRING) + cbString);
@@ -1234,9 +1233,9 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
             parms[2].type = VBOX_HGCM_SVC_PARM_32BIT;
             parms[2].u.uint32 = !g_fShareReadOnly[i];
 
-            rc = gVMMDev->hgcmHostCall ("VBoxSharedFolders",
-                                        SHFL_FN_ADD_MAPPING, SHFL_CPARMS_ADD_MAPPING, &parms[0]);
-            AssertRC(rc);
+            rc2 = gVMMDev->hgcmHostCall ("VBoxSharedFolders",
+                                         SHFL_FN_ADD_MAPPING, SHFL_CPARMS_ADD_MAPPING, &parms[0]);
+            AssertRC(rc2);
             LogRel(("Added share %s: (%s)\n", g_pszShareName[i], g_pszShareDir[i]));
             RTMemFree (pFolderName);
             RTMemFree (pMapName);
@@ -1253,58 +1252,36 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
     if (g_fUSB)
     {
         gHostUSB = new HostUSB();
-        gHostUSB->init(pVM);
+        gHostUSB->init(gpVM);
     }
 #endif /* VBOXBFE_WITH_USB */
 
-#ifdef RT_OS_L4
-    /* L4 console cannot draw a host cursor */
-    gMouse->setHostCursor(false);
-#else
-    gMouse->setHostCursor(true);
-#endif
-
     /*
      * Power on the VM (i.e. start executing).
      */
     if (RT_SUCCESS(rc))
     {
-        PVMREQ pReq;
-
         if (   g_fRestoreState
             && g_pszStateFile
             && *g_pszStateFile
             && RTPathExists(g_pszStateFile))
         {
             startProgressInfo("Restoring");
-            rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT,
-                             (PFNRT)VMR3Load, 4, pVM, g_pszStateFile, &callProgressInfo, (uintptr_t)NULL);
+            rc = VMR3LoadFromFile(gpVM, g_pszStateFile, callProgressInfo, (uintptr_t)NULL);
             endProgressInfo();
             if (RT_SUCCESS(rc))
             {
-                VMR3ReqFree(pReq);
-                rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT,
-                                 (PFNRT)VMR3Resume, 1, pVM);
-                if (RT_SUCCESS(rc))
-                {
-                    rc = pReq->iStatus;
-                    VMR3ReqFree(pReq);
-                }
+                rc = VMR3Resume(gpVM);
+                AssertRC(rc);
                 gDisplay->setRunning();
             }
             else
-                AssertMsgFailed(("VMR3Load failed, rc=%Rrc\n", rc));
+                AssertMsgFailed(("VMR3LoadFromFile failed, rc=%Rrc\n", rc));
         }
         else
         {
-            rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)VMR3PowerOn, 1, pVM);
-            if (RT_SUCCESS(rc))
-            {
-                rc = pReq->iStatus;
-                AssertRC(rc);
-                VMR3ReqFree(pReq);
-            }
-            else
+            rc = VMR3PowerOn(gpVM);
+            if (RT_FAILURE(rc))
                 AssertMsgFailed(("VMR3PowerOn failed, rc=%Rrc\n", rc));
         }
     }
@@ -1318,11 +1295,11 @@ DECLCALLBACK(int) VMPowerUpThread(RTTHREAD Thread, void *pvUser)
     return 0;
 
 failure:
-    if (pVM)
+    if (gpVM)
     {
-        rc2 = VMR3Destroy(pVM);
+        rc2 = VMR3Destroy(gpVM);
         AssertRC(rc2);
-        pVM = NULL;
+        gpVM = NULL;
     }
     machineState = VMSTATE_TERMINATED;
 
@@ -1352,7 +1329,7 @@ DECLCALLBACK(int) VBoxDriversRegister(PCPDMDRVREGCB pCallbacks, uint32_t u32Vers
     if (RT_FAILURE(rc))
         return rc;
 
-    rc = pCallbacks->pfnRegister(pCallbacks, &VMDisplay::DrvReg);
+    rc = pCallbacks->pfnRegister(pCallbacks, &Display::DrvReg);
     AssertRC(rc);
     if (RT_FAILURE(rc))
         return rc;
@@ -1923,7 +1900,7 @@ static DECLCALLBACK(int) vboxbfeConfigConstructor(PVM pVM, void *pvUser)
 
     /* the VMM device's Main driver */
     rc = CFGMR3InsertNode(pInst,    "LUN#0",          &pLunL0);                     UPDATE_RC();
-    rc = CFGMR3InsertString(pLunL0, "Driver",        "MainVMMDev");                 UPDATE_RC();
+    rc = CFGMR3InsertString(pLunL0, "Driver",        "HGCM");                 UPDATE_RC();
     rc = CFGMR3InsertNode(pLunL0,   "Config",        &pCfg);                        UPDATE_RC();
     rc = CFGMR3InsertInteger(pCfg,  "Object",        (uintptr_t)gVMMDev);           UPDATE_RC();
 
@@ -1949,6 +1926,12 @@ static DECLCALLBACK(int) vboxbfeConfigConstructor(PVM pVM, void *pvUser)
         rc = CFGMR3InsertString(pCfg, "AudioDriver",      "coreaudio");             UPDATE_RC();
 #elif defined(RT_OS_LINUX)
         rc = CFGMR3InsertString(pCfg, "AudioDriver",      "oss");                   UPDATE_RC();
+#elif defined(RT_OS_SOLARIS)
+# ifdef VBOX_WITH_SOLARIS_OSS
+        rc = CFGMR3InsertString(pCfg, "AudioDriver",      "oss");                   UPDATE_RC();
+# else
+        rc = CFGMR3InsertString(pCfg, "AudioDriver",      "solaudio");              UPDATE_RC();
+# endif
 #elif defined(RT_OS_L4)
         rc = CFGMR3InsertString(pCfg, "AudioDriver",      "oss");                   UPDATE_RC();
 #else /* portme */
