@@ -1,10 +1,10 @@
-/* $Id: VBoxManage.h $ */
+/* $Id: VBoxManage.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxManage - VirtualBox command-line interface, internal header file.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___H_VBOXMANAGE
@@ -26,16 +22,11 @@
 #include <VBox/com/com.h>
 #include <VBox/com/ptr.h>
 #include <VBox/com/VirtualBox.h>
-#include <VBox/com/EventQueue.h>
 #include <VBox/com/string.h>
 #endif /* !VBOX_ONLY_DOCS */
 
 #include <iprt/types.h>
 
-#if defined(VBOX_WITH_XPCOM) && !defined(RT_OS_DARWIN) && !defined(RT_OS_OS2)
-# define USE_XPCOM_QUEUE
-#endif
-
 ////////////////////////////////////////////////////////////////////////////////
 //
 // definitions
@@ -95,6 +86,12 @@
 #define USAGE_EXPORTAPPLIANCE       RT_BIT_64(45)
 #define USAGE_HOSTONLYIFS           RT_BIT_64(46)
 #define USAGE_DHCPSERVER            RT_BIT_64(47)
+#define USAGE_DUMPHDINFO            RT_BIT_64(48)
+#define USAGE_STORAGEATTACH         RT_BIT_64(49)
+#define USAGE_STORAGECONTROLLER     RT_BIT_64(50)
+#ifdef VBOX_WITH_GUEST_CONTROL
+#define USAGE_GUESTCONTROL          RT_BIT_64(51)
+#endif  /* VBOX_WITH_GUEST_CONTROL defined */
 #define USAGE_ALL                   (~(uint64_t)0)
 /** @} */
 
@@ -106,9 +103,6 @@ struct HandlerArg
     int argc;
     char **argv;
 
-#ifdef USE_XPCOM_QUEUE
-    nsCOMPtr<nsIEventQueue> eventQ;
-#endif
 #ifndef VBOX_ONLY_DOCS
     ComPtr<IVirtualBox> virtualBox;
     ComPtr<ISession> session;
@@ -123,10 +117,9 @@ typedef enum
 {
     VMINFO_NONE             = 0,
     VMINFO_STANDARD         = 1,    /**< standard details */
-    VMINFO_STATISTICS       = 2,    /**< guest statistics */
-    VMINFO_FULL             = 3,    /**< both */
-    VMINFO_MACHINEREADABLE  = 4,    /**< both, and make it machine readable */
-    VMINFO_COMPACT          = 5
+    VMINFO_FULL             = 2,    /**< both */
+    VMINFO_MACHINEREADABLE  = 3,    /**< both, and make it machine readable */
+    VMINFO_COMPACT          = 4
 } VMINFO_DETAILS;
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -146,12 +139,13 @@ extern bool g_fDetailedProgress;        // in VBoxManage.cpp
 /* VBoxManageHelp.cpp */
 void printUsage(USAGECATEGORY u64Cmd);
 int errorSyntax(USAGECATEGORY u64Cmd, const char *pszFormat, ...);
+int errorGetOpt(USAGECATEGORY u64Cmd, int rc, union RTGETOPTUNION const *pValueUnion);
 int errorArgument(const char *pszFormat, ...);
 
 void printUsageInternal(USAGECATEGORY u64Cmd);
 
 #ifndef VBOX_ONLY_DOCS
-void showProgress(ComPtr<IProgress> progress);
+HRESULT showProgress(ComPtr<IProgress> progress);
 #endif
 
 /* VBoxManage.cpp */
@@ -159,10 +153,11 @@ void showLogo(void);
 
 #ifndef VBOX_ONLY_DOCS
 int handleInternalCommands(HandlerArg *a);
-
-unsigned parseNum(const char *psz, unsigned cMaxNum, const char *name);
 #endif /* !VBOX_ONLY_DOCS */
 
+/* VBoxManageControlVM.cpp */
+int handleControlVM(HandlerArg *a);
+
 /* VBoxManageModifyVM.cpp */
 int handleModifyVM(HandlerArg *a);
 
@@ -171,8 +166,16 @@ extern void usageGuestProperty(void);
 #ifndef VBOX_ONLY_DOCS
 extern int handleGuestProperty(HandlerArg *a);
 
+/* VBoxManageGuestCtrl.cpp */
+extern int handleGuestControl(HandlerArg *a);
+extern void usageGuestControl(void);
+
 /* VBoxManageVMInfo.cpp */
-void showSnapshots(ComPtr<ISnapshot> rootSnapshot, VMINFO_DETAILS details, const com::Bstr &prefix = "", int level = 0);
+void showSnapshots(ComPtr<ISnapshot> &rootSnapshot,
+                   ComPtr<ISnapshot> &currentSnapshot,
+                   VMINFO_DETAILS details,
+                   const com::Bstr &prefix = "",
+                   int level = 0);
 int handleShowVMInfo(HandlerArg *a);
 HRESULT showVMInfo(ComPtr<IVirtualBox> virtualBox,
                    ComPtr<IMachine> machine,
@@ -185,6 +188,19 @@ int handleList(HandlerArg *a);
 /* VBoxManageMetrics.cpp */
 int handleMetrics(HandlerArg *a);
 
+/* VBoxManageMisc.cpp */
+int handleRegisterVM(HandlerArg *a);
+int handleUnregisterVM(HandlerArg *a);
+int handleCreateVM(HandlerArg *a);
+int handleStartVM(HandlerArg *a);
+int handleDiscardState(HandlerArg *a);
+int handleAdoptState(HandlerArg *a);
+int handleGetExtraData(HandlerArg *a);
+int handleSetExtraData(HandlerArg *a);
+int handleSetProperty(HandlerArg *a);
+int handleSharedFolder(HandlerArg *a);
+int handleVMStatistics(HandlerArg *a);
+
 /* VBoxManageDisk.cpp */
 int handleCreateHardDisk(HandlerArg *a);
 int handleModifyHardDisk(HandlerArg *a);
@@ -195,6 +211,10 @@ int handleShowHardDiskInfo(HandlerArg *a);
 int handleOpenMedium(HandlerArg *a);
 int handleCloseMedium(HandlerArg *a);
 
+/* VBoxManageStorageController.cpp */
+int handleStorageAttach(HandlerArg *a);
+int handleStorageController(HandlerArg *a);
+
 // VBoxManageImport.cpp
 int handleImportAppliance(HandlerArg *a);
 int handleExportAppliance(HandlerArg *a);
@@ -212,7 +232,6 @@ int handleHostonlyIf(HandlerArg *a);
 int handleDHCPServer(HandlerArg *a);
 
 #endif /* !VBOX_ONLY_DOCS */
-unsigned long VBoxSVNRev();
 
 #endif /* !___H_VBOXMANAGE */
 
