@@ -1,10 +1,10 @@
-/* $Id: VBoxManageDisk.cpp $ */
+/* $Id: VBoxManageDisk.cpp 28888 2010-04-29 11:50:05Z vboxsync $ */
 /** @file
  * VBoxManage - The disk delated commands.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef VBOX_ONLY_DOCS
@@ -58,7 +54,7 @@ static DECLCALLBACK(void) handleVDError(void *pvUser, int rc, RT_SRC_POS_DECL, c
 }
 
 
-static int parseDiskVariant(const char *psz, HardDiskVariant_T *pDiskVariant)
+static int parseDiskVariant(const char *psz, MediumVariant_T *pDiskVariant)
 {
     int rc = VINF_SUCCESS;
     unsigned DiskVariant = (unsigned)(*pDiskVariant);
@@ -75,19 +71,19 @@ static int parseDiskVariant(const char *psz, HardDiskVariant_T *pDiskVariant)
             // Parsing is intentionally inconsistent: "standard" resets the
             // variant, whereas the other flags are cumulative.
             if (!RTStrNICmp(psz, "standard", len))
-                DiskVariant = HardDiskVariant_Standard;
+                DiskVariant = MediumVariant_Standard;
             else if (   !RTStrNICmp(psz, "fixed", len)
                      || !RTStrNICmp(psz, "static", len))
-                DiskVariant |= HardDiskVariant_Fixed;
+                DiskVariant |= MediumVariant_Fixed;
             else if (!RTStrNICmp(psz, "Diff", len))
-                DiskVariant |= HardDiskVariant_Diff;
+                DiskVariant |= MediumVariant_Diff;
             else if (!RTStrNICmp(psz, "split2g", len))
-                DiskVariant |= HardDiskVariant_VmdkSplit2G;
+                DiskVariant |= MediumVariant_VmdkSplit2G;
             else if (   !RTStrNICmp(psz, "stream", len)
                      || !RTStrNICmp(psz, "streamoptimized", len))
-                DiskVariant |= HardDiskVariant_VmdkStreamOptimized;
+                DiskVariant |= MediumVariant_VmdkStreamOptimized;
             else if (!RTStrNICmp(psz, "esx", len))
-                DiskVariant |= HardDiskVariant_VmdkESX;
+                DiskVariant |= MediumVariant_VmdkESX;
             else
                 rc = VERR_PARSE_ERROR;
         }
@@ -98,20 +94,22 @@ static int parseDiskVariant(const char *psz, HardDiskVariant_T *pDiskVariant)
     }
 
     if (RT_SUCCESS(rc))
-        *pDiskVariant = (HardDiskVariant_T)DiskVariant;
+        *pDiskVariant = (MediumVariant_T)DiskVariant;
     return rc;
 }
 
-static int parseDiskType(const char *psz, HardDiskType_T *pDiskType)
+static int parseDiskType(const char *psz, MediumType_T *pDiskType)
 {
     int rc = VINF_SUCCESS;
-    HardDiskType_T DiskType = HardDiskType_Normal;
+    MediumType_T DiskType = MediumType_Normal;
     if (!RTStrICmp(psz, "normal"))
-        DiskType = HardDiskType_Normal;
+        DiskType = MediumType_Normal;
     else if (!RTStrICmp(psz, "immutable"))
-        DiskType = HardDiskType_Immutable;
+        DiskType = MediumType_Immutable;
     else if (!RTStrICmp(psz, "writethrough"))
-        DiskType = HardDiskType_Writethrough;
+        DiskType = MediumType_Writethrough;
+    else if (!RTStrICmp(psz, "shareable"))
+        DiskType = MediumType_Shareable;
     else
         rc = VERR_PARSE_ERROR;
 
@@ -177,16 +175,17 @@ int handleCreateHardDisk(HandlerArg *a)
     Bstr filename;
     uint64_t sizeMB = 0;
     Bstr format = "VDI";
-    HardDiskVariant_T DiskVariant = HardDiskVariant_Standard;
+    MediumVariant_T DiskVariant = MediumVariant_Standard;
     Bstr comment;
     bool fRemember = false;
-    HardDiskType_T DiskType = HardDiskType_Normal;
+    MediumType_T DiskType = MediumType_Normal;
 
     int c;
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aCreateHardDiskOptions, RT_ELEMENTS(g_aCreateHardDiskOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, a->argc, a->argv, g_aCreateHardDiskOptions, RT_ELEMENTS(g_aCreateHardDiskOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -206,8 +205,8 @@ int handleCreateHardDisk(HandlerArg *a)
             case 'F':   // --static ("fixed"/"flat")
             {
                 unsigned uDiskVariant = (unsigned)DiskVariant;
-                uDiskVariant |= HardDiskVariant_Fixed;
-                DiskVariant = (HardDiskVariant_T)uDiskVariant;
+                uDiskVariant |= MediumVariant_Fixed;
+                DiskVariant = (MediumVariant_T)uDiskVariant;
                 break;
             }
 
@@ -228,7 +227,7 @@ int handleCreateHardDisk(HandlerArg *a)
             case 't':   // --type
                 vrc = parseDiskType(ValueUnion.psz, &DiskType);
                 if (    RT_FAILURE(vrc)
-                    ||  (DiskType != HardDiskType_Normal && DiskType != HardDiskType_Writethrough))
+                    ||  (DiskType != MediumType_Normal && DiskType != MediumType_Writethrough))
                     return errorArgument("Invalid hard disk type '%s'", ValueUnion.psz);
                 break;
 
@@ -253,10 +252,25 @@ int handleCreateHardDisk(HandlerArg *a)
     }
 
     /* check the outcome */
-    if (!filename || (sizeMB == 0))
+    if (   !filename
+        || sizeMB == 0)
         return errorSyntax(USAGE_CREATEHD, "Parameters --filename and --size are required");
 
-    ComPtr<IHardDisk> hardDisk;
+    /* check for filename extension */
+    Utf8Str strName(filename);
+    if (!RTPathHaveExt(strName.c_str()))
+    {
+        Utf8Str strFormat(format);
+        if (strFormat.compare("vmdk", iprt::MiniString::CaseInsensitive) == 0)
+            strName.append(".vmdk");
+        else if (strFormat.compare("vhd", iprt::MiniString::CaseInsensitive) == 0)
+            strName.append(".vhd");
+        else
+            strName.append(".vdi");
+        filename = Bstr(strName);
+    }
+
+    ComPtr<IMedium> hardDisk;
     CHECK_ERROR(a->virtualBox, CreateHardDisk(format, filename, hardDisk.asOutParam()));
     if (SUCCEEDED(rc) && hardDisk)
     {
@@ -264,42 +278,37 @@ int handleCreateHardDisk(HandlerArg *a)
          * created unless fRemember is set */
         bool doClose = false;
 
-        if (!comment.isNull())
+        if (!comment.isEmpty())
         {
             CHECK_ERROR(hardDisk,COMSETTER(Description)(comment));
         }
+
         ComPtr<IProgress> progress;
         CHECK_ERROR(hardDisk, CreateBaseStorage(sizeMB, DiskVariant, progress.asOutParam()));
         if (SUCCEEDED(rc) && progress)
         {
-            showProgress(progress);
-            if (SUCCEEDED(rc))
+            rc = showProgress(progress);
+            if (FAILED(rc))
             {
-                LONG iRc;
-                progress->COMGETTER(ResultCode)(&iRc);
-                rc = iRc;
-                if (FAILED(rc))
-                {
-                    com::ProgressErrorInfo info(progress);
-                    if (info.isBasicAvailable())
-                        RTPrintf("Error: failed to create hard disk. Error message: %lS\n", info.getText().raw());
-                    else
-                        RTPrintf("Error: failed to create hard disk. No error message available!\n");
-                }
+                com::ProgressErrorInfo info(progress);
+                if (info.isBasicAvailable())
+                    RTPrintf("Error: failed to create hard disk. Error message: %lS\n", info.getText().raw());
                 else
+                    RTPrintf("Error: failed to create hard disk. No error message available!\n");
+            }
+            else
+            {
+                doClose = !fRemember;
+
+                Bstr uuid;
+                CHECK_ERROR(hardDisk, COMGETTER(Id)(uuid.asOutParam()));
+
+                if (DiskType == MediumType_Writethrough)
                 {
-                    doClose = !fRemember;
-
-                    Bstr uuid;
-                    CHECK_ERROR(hardDisk, COMGETTER(Id)(uuid.asOutParam()));
-
-                    if (DiskType == HardDiskType_Writethrough)
-                    {
-                        CHECK_ERROR(hardDisk, COMSETTER(Type)(HardDiskType_Writethrough));
-                    }
-
-                    RTPrintf("Disk image created. UUID: %s\n", Utf8Str(uuid).raw());
+                    CHECK_ERROR(hardDisk, COMSETTER(Type)(MediumType_Writethrough));
                 }
+
+                RTPrintf("Disk image created. UUID: %s\n", Utf8Str(uuid).raw());
             }
         }
         if (doClose)
@@ -345,8 +354,8 @@ int handleModifyHardDisk(HandlerArg *a)
 {
     HRESULT rc;
     int vrc;
-    ComPtr<IHardDisk> hardDisk;
-    HardDiskType_T DiskType;
+    ComPtr<IMedium> hardDisk;
+    MediumType_T DiskType;
     bool AutoReset = false;
     bool fModifyDiskType = false, fModifyAutoReset = false, fModifyCompact = false;;
     const char *FilenameOrUuid = NULL;
@@ -355,7 +364,8 @@ int handleModifyHardDisk(HandlerArg *a)
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aModifyHardDiskOptions, RT_ELEMENTS(g_aModifyHardDiskOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, a->argc, a->argv, g_aModifyHardDiskOptions, RT_ELEMENTS(g_aModifyHardDiskOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -424,7 +434,7 @@ int handleModifyHardDisk(HandlerArg *a)
         /* hard disk must be registered */
         if (SUCCEEDED(rc) && hardDisk)
         {
-            HardDiskType_T hddType;
+            MediumType_T hddType;
             CHECK_ERROR(hardDisk, COMGETTER(Type)(&hddType));
 
             if (hddType != DiskType)
@@ -450,8 +460,8 @@ int handleModifyHardDisk(HandlerArg *a)
             if (rc == VBOX_E_FILE_ERROR)
             {
                 char szFilenameAbs[RTPATH_MAX] = "";
-                int vrc = RTPathAbs(FilenameOrUuid, szFilenameAbs, sizeof(szFilenameAbs));
-                if (RT_FAILURE(vrc))
+                int irc = RTPathAbs(FilenameOrUuid, szFilenameAbs, sizeof(szFilenameAbs));
+                if (RT_FAILURE(irc))
                 {
                     RTPrintf("Cannot convert filename \"%s\" to absolute path\n", FilenameOrUuid);
                     return 1;
@@ -466,12 +476,7 @@ int handleModifyHardDisk(HandlerArg *a)
             ComPtr<IProgress> progress;
             CHECK_ERROR(hardDisk, Compact(progress.asOutParam()));
             if (SUCCEEDED(rc))
-            {
-                showProgress(progress);
-                LONG iRc;
-                progress->COMGETTER(ResultCode)(&iRc);
-                rc = iRc;
-            }
+                rc = showProgress(progress);
             if (FAILED(rc))
             {
                 if (rc == E_NOTIMPL)
@@ -517,17 +522,18 @@ int handleCloneHardDisk(HandlerArg *a)
     int vrc;
     Bstr src, dst;
     Bstr format;
-    HardDiskVariant_T DiskVariant = HardDiskVariant_Standard;
+    MediumVariant_T DiskVariant = MediumVariant_Standard;
     bool fExisting = false;
     bool fRemember = false;
     bool fSetDiskType = false;
-    HardDiskType_T DiskType = HardDiskType_Normal;
+    MediumType_T DiskType = MediumType_Normal;
 
     int c;
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aCloneHardDiskOptions, RT_ELEMENTS(g_aCloneHardDiskOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, a->argc, a->argv, g_aCloneHardDiskOptions, RT_ELEMENTS(g_aCloneHardDiskOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -539,8 +545,8 @@ int handleCloneHardDisk(HandlerArg *a)
             case 'F':   // --static
             {
                 unsigned uDiskVariant = (unsigned)DiskVariant;
-                uDiskVariant |= HardDiskVariant_Fixed;
-                DiskVariant = (HardDiskVariant_T)uDiskVariant;
+                uDiskVariant |= MediumVariant_Fixed;
+                DiskVariant = (MediumVariant_T)uDiskVariant;
                 break;
             }
 
@@ -595,11 +601,11 @@ int handleCloneHardDisk(HandlerArg *a)
         return errorSyntax(USAGE_CLONEHD, "Mandatory UUID or input file parameter missing");
     if (dst.isEmpty())
         return errorSyntax(USAGE_CLONEHD, "Mandatory output file parameter missing");
-    if (fExisting && (!format.isEmpty() || DiskVariant != HardDiskType_Normal))
+    if (fExisting && (!format.isEmpty() || DiskVariant != MediumType_Normal))
         return errorSyntax(USAGE_CLONEHD, "Specified options which cannot be used with --existing");
 
-    ComPtr<IHardDisk> srcDisk;
-    ComPtr<IHardDisk> dstDisk;
+    ComPtr<IMedium> srcDisk;
+    ComPtr<IMedium> dstDisk;
     bool fSrcUnknown = false;
     bool fDstUnknown = false;
 
@@ -615,8 +621,8 @@ int handleCloneHardDisk(HandlerArg *a)
         if (rc == VBOX_E_FILE_ERROR)
         {
             char szFilenameAbs[RTPATH_MAX] = "";
-            int vrc = RTPathAbs(Utf8Str(src), szFilenameAbs, sizeof(szFilenameAbs));
-            if (RT_FAILURE(vrc))
+            int irc = RTPathAbs(Utf8Str(src).c_str(), szFilenameAbs, sizeof(szFilenameAbs));
+            if (RT_FAILURE(irc))
             {
                 RTPrintf("Cannot convert filename \"%s\" to absolute path\n", Utf8Str(src).raw());
                 return 1;
@@ -649,8 +655,8 @@ int handleCloneHardDisk(HandlerArg *a)
                 if (rc == VBOX_E_FILE_ERROR)
                 {
                     char szFilenameAbs[RTPATH_MAX] = "";
-                    int vrc = RTPathAbs(Utf8Str(dst), szFilenameAbs, sizeof(szFilenameAbs));
-                    if (RT_FAILURE(vrc))
+                    int irc = RTPathAbs(Utf8Str(dst).c_str(), szFilenameAbs, sizeof(szFilenameAbs));
+                    if (RT_FAILURE(irc))
                     {
                         RTPrintf("Cannot convert filename \"%s\" to absolute path\n", Utf8Str(dst).raw());
                         return 1;
@@ -667,8 +673,8 @@ int handleCloneHardDisk(HandlerArg *a)
             if (SUCCEEDED(rc))
             {
                 /* Perform accessibility check now. */
-                MediaState_T state;
-                CHECK_ERROR_BREAK(dstDisk, COMGETTER(State)(&state));
+                MediumState_T state;
+                CHECK_ERROR_BREAK(dstDisk, RefreshState(&state));
             }
             CHECK_ERROR_BREAK(dstDisk, COMGETTER(Format) (format.asOutParam()));
         }
@@ -683,10 +689,7 @@ int handleCloneHardDisk(HandlerArg *a)
         ComPtr<IProgress> progress;
         CHECK_ERROR_BREAK(srcDisk, CloneTo(dstDisk, DiskVariant, NULL, progress.asOutParam()));
 
-        showProgress(progress);
-        LONG iRc;
-        progress->COMGETTER(ResultCode)(&iRc);
-        rc = iRc;
+        rc = showProgress(progress);
         if (FAILED(rc))
         {
             com::ProgressErrorInfo info(progress);
@@ -749,7 +752,8 @@ int handleConvertFromRaw(int argc, char *argv[])
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, argc, argv, g_aConvertFromRawHardDiskOptions, RT_ELEMENTS(g_aConvertFromRawHardDiskOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, argc, argv, g_aConvertFromRawHardDiskOptions, RT_ELEMENTS(g_aConvertFromRawHardDiskOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -759,7 +763,7 @@ int handleConvertFromRaw(int argc, char *argv[])
                 break;
 
             case 'm':   // --variant
-                HardDiskVariant_T DiskVariant;
+                MediumVariant_T DiskVariant;
                 rc = parseDiskVariant(ValueUnion.psz, &DiskVariant);
                 if (RT_FAILURE(rc))
                     return errorArgument("Invalid hard disk variant '%s'", ValueUnion.psz);
@@ -784,19 +788,7 @@ int handleConvertFromRaw(int argc, char *argv[])
                 break;
 
             default:
-                if (c > 0)
-                {
-                    if (RT_C_IS_PRINT(c))
-                        return errorSyntax(USAGE_CONVERTFROMRAW, "Invalid option -%c", c);
-                    else
-                        return errorSyntax(USAGE_CONVERTFROMRAW, "Invalid option case %i", c);
-                }
-                else if (c == VERR_GETOPT_UNKNOWN_OPTION)
-                    return errorSyntax(USAGE_CREATEHD, "unknown option: %s\n", ValueUnion.psz);
-                else if (ValueUnion.pDef)
-                    return errorSyntax(USAGE_CONVERTFROMRAW, "%s: %Rrs", ValueUnion.pDef->pszLong, c);
-                else
-                    return errorSyntax(USAGE_CONVERTFROMRAW, "error: %Rrs", c);
+                return errorGetOpt(USAGE_CONVERTFROMRAW, c, &ValueUnion);
         }
     }
 
@@ -813,6 +805,7 @@ int handleConvertFromRaw(int argc, char *argv[])
     vdInterfaceErrorCallbacks.cbSize       = sizeof(VDINTERFACEERROR);
     vdInterfaceErrorCallbacks.enmInterface = VDINTERFACETYPE_ERROR;
     vdInterfaceErrorCallbacks.pfnError     = handleVDError;
+    vdInterfaceErrorCallbacks.pfnMessage   = NULL;
 
     rc = VDInterfaceAdd(&vdInterfaceError, "VBoxManage_IError", VDINTERFACETYPE_ERROR,
                         &vdInterfaceErrorCallbacks, NULL, &pVDIfs);
@@ -823,7 +816,7 @@ int handleConvertFromRaw(int argc, char *argv[])
     if (fReadFromStdIn)
         File = 0;
     else
-        rc = RTFileOpen(&File, srcfilename, RTFILE_O_OPEN | RTFILE_O_READ | RTFILE_O_DENY_WRITE);
+        rc = RTFileOpen(&File, srcfilename, RTFILE_O_READ | RTFILE_O_OPEN | RTFILE_O_DENY_WRITE);
     if (RT_FAILURE(rc))
     {
         RTPrintf("File=\"%s\" open error: %Rrf\n", srcfilename, rc);
@@ -930,8 +923,6 @@ static const RTGETOPTDEF g_aAddiSCSIDiskOptions[] =
     { "-password",      'P', RTGETOPT_REQ_STRING },     // deprecated
     { "--type",         't', RTGETOPT_REQ_STRING },
     { "-type",          't', RTGETOPT_REQ_STRING },     // deprecated
-    { "--comment",      'c', RTGETOPT_REQ_STRING },
-    { "-comment",       'c', RTGETOPT_REQ_STRING },     // deprecated
     { "--intnet",       'I', RTGETOPT_REQ_NOTHING },
     { "-intnet",        'I', RTGETOPT_REQ_NOTHING },    // deprecated
 };
@@ -948,13 +939,14 @@ int handleAddiSCSIDisk(HandlerArg *a)
     Bstr password;
     Bstr comment;
     bool fIntNet = false;
-    HardDiskType_T DiskType = HardDiskType_Normal;
+    MediumType_T DiskType = MediumType_Normal;
 
     int c;
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aAddiSCSIDiskOptions, RT_ELEMENTS(g_aAddiSCSIDiskOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, a->argc, a->argv, g_aAddiSCSIDiskOptions, RT_ELEMENTS(g_aAddiSCSIDiskOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -993,10 +985,6 @@ int handleAddiSCSIDisk(HandlerArg *a)
                     return errorArgument("Invalid hard disk type '%s'", ValueUnion.psz);
                 break;
 
-            case 'c':   // --comment
-                comment = ValueUnion.psz;
-                break;
-
             case 'I':   // --intnet
                 fIntNet = true;
                 break;
@@ -1027,7 +1015,7 @@ int handleAddiSCSIDisk(HandlerArg *a)
 
     do
     {
-        ComPtr<IHardDisk> hardDisk;
+        ComPtr<IMedium> hardDisk;
         /** @todo move the location stuff to Main, which can use pfnComposeName
          * from the disk backends to construct the location properly. Also do
          * not use slashes to separate the parts, as otherwise only the last
@@ -1046,12 +1034,9 @@ int handleAddiSCSIDisk(HandlerArg *a)
                                BstrFmt ("%ls|%ls|%ls", server.raw(), target.raw(), lun.raw()),
                                hardDisk.asOutParam()));
         }
-        CheckComRCBreakRC (rc);
+        if (FAILED(rc)) break;
 
-        if (!comment.isNull())
-            CHECK_ERROR(hardDisk, COMSETTER(Description)(comment));
-
-        if (!port.isNull())
+        if (!port.isEmpty())
             server = BstrFmt ("%ls:%ls", server.raw(), port.raw());
 
         com::SafeArray <BSTR> names;
@@ -1062,25 +1047,28 @@ int handleAddiSCSIDisk(HandlerArg *a)
         Bstr ("TargetName").detachTo (names.appendedRaw());
         target.detachTo (values.appendedRaw());
 
-        if (!lun.isNull())
+        if (!lun.isEmpty())
         {
             Bstr ("LUN").detachTo (names.appendedRaw());
             lun.detachTo (values.appendedRaw());
         }
-        if (!username.isNull())
+        if (!username.isEmpty())
         {
             Bstr ("InitiatorUsername").detachTo (names.appendedRaw());
             username.detachTo (values.appendedRaw());
         }
-        if (!password.isNull())
+        if (!password.isEmpty())
         {
             Bstr ("InitiatorSecret").detachTo (names.appendedRaw());
             password.detachTo (values.appendedRaw());
         }
 
-        /// @todo add --initiator option
-        Bstr ("InitiatorName").detachTo (names.appendedRaw());
-        Bstr ("iqn.2008-04.com.sun.virtualbox.initiator").detachTo (values.appendedRaw());
+        /// @todo add --initiator option - until that happens rely on the
+        // defaults of the iSCSI initiator code. Setting it to a constant
+        // value does more harm than good, as the initiator name is supposed
+        // to identify a particular initiator uniquely.
+//        Bstr ("InitiatorName").detachTo (names.appendedRaw());
+//        Bstr ("iqn.2008-04.com.sun.virtualbox.initiator").detachTo (values.appendedRaw());
 
         /// @todo add --targetName and --targetPassword options
 
@@ -1094,7 +1082,7 @@ int handleAddiSCSIDisk(HandlerArg *a)
             SetProperties (ComSafeArrayAsInParam (names),
                            ComSafeArrayAsInParam (values)));
 
-        if (DiskType != HardDiskType_Normal)
+        if (DiskType != MediumType_Normal)
         {
             CHECK_ERROR(hardDisk, COMSETTER(Type)(DiskType));
         }
@@ -1122,7 +1110,8 @@ int handleShowHardDiskInfo(HandlerArg *a)
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aShowHardDiskInfoOptions, RT_ELEMENTS(g_aShowHardDiskInfoOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, a->argc, a->argv, g_aShowHardDiskInfoOptions, RT_ELEMENTS(g_aShowHardDiskInfoOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -1155,7 +1144,7 @@ int handleShowHardDiskInfo(HandlerArg *a)
     if (!FilenameOrUuid)
         return errorSyntax(USAGE_SHOWHDINFO, "Disk name or UUID required");
 
-    ComPtr<IHardDisk> hardDisk;
+    ComPtr<IMedium> hardDisk;
     bool unknown = false;
     /* first guess is that it's a UUID */
     Bstr uuid(FilenameOrUuid);
@@ -1198,11 +1187,11 @@ int handleShowHardDiskInfo(HandlerArg *a)
         /* check for accessibility */
         /// @todo NEWMEDIA check accessibility of all parents
         /// @todo NEWMEDIA print the full state value
-        MediaState_T state;
-        CHECK_ERROR_BREAK (hardDisk, COMGETTER(State)(&state));
-        RTPrintf("Accessible:           %s\n", state != MediaState_Inaccessible ? "yes" : "no");
+        MediumState_T state;
+        CHECK_ERROR_BREAK (hardDisk, RefreshState(&state));
+        RTPrintf("Accessible:           %s\n", state != MediumState_Inaccessible ? "yes" : "no");
 
-        if (state == MediaState_Inaccessible)
+        if (state == MediumState_Inaccessible)
         {
             Bstr err;
             CHECK_ERROR_BREAK (hardDisk, COMGETTER(LastAccessError)(err.asOutParam()));
@@ -1223,26 +1212,29 @@ int handleShowHardDiskInfo(HandlerArg *a)
         hardDisk->COMGETTER(Size)(&actualSize);
         RTPrintf("Current size on disk: %llu MBytes\n", actualSize >> 20);
 
-        ComPtr <IHardDisk> parent;
+        ComPtr <IMedium> parent;
         hardDisk->COMGETTER(Parent) (parent.asOutParam());
 
-        HardDiskType_T type;
+        MediumType_T type;
         hardDisk->COMGETTER(Type)(&type);
         const char *typeStr = "unknown";
         switch (type)
         {
-            case HardDiskType_Normal:
+            case MediumType_Normal:
                 if (!parent.isNull())
                     typeStr = "normal (differencing)";
                 else
                     typeStr = "normal (base)";
                 break;
-            case HardDiskType_Immutable:
+            case MediumType_Immutable:
                 typeStr = "immutable";
                 break;
-            case HardDiskType_Writethrough:
+            case MediumType_Writethrough:
                 typeStr = "writethrough";
                 break;
+            case MediumType_Shareable:
+                typeStr = "shareable";
+                break;
         }
         RTPrintf("Type:                 %s\n", typeStr);
 
@@ -1315,7 +1307,7 @@ int handleOpenMedium(HandlerArg *a)
         CMD_FLOPPY
     } cmd = CMD_NONE;
     const char *Filename = NULL;
-    HardDiskType_T DiskType = HardDiskType_Normal;
+    MediumType_T DiskType = MediumType_Normal;
     bool fDiskType = false;
     bool fSetImageId = false;
     bool fSetParentId = false;
@@ -1328,7 +1320,8 @@ int handleOpenMedium(HandlerArg *a)
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aOpenMediumOptions, RT_ELEMENTS(g_aOpenMediumOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, a->argc, a->argv, g_aOpenMediumOptions, RT_ELEMENTS(g_aOpenMediumOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -1407,15 +1400,15 @@ int handleOpenMedium(HandlerArg *a)
      * chance to clean up the API semantics. */
     if (cmd == CMD_DISK)
     {
-        ComPtr<IHardDisk> hardDisk;
+        ComPtr<IMedium> hardDisk;
         Bstr ImageIdStr = BstrFmt("%RTuuid", &ImageId);
         Bstr ParentIdStr = BstrFmt("%RTuuid", &ParentId);
         rc = a->virtualBox->OpenHardDisk(Bstr(Filename), AccessMode_ReadWrite, fSetImageId, ImageIdStr, fSetParentId, ParentIdStr, hardDisk.asOutParam());
         if (rc == VBOX_E_FILE_ERROR)
         {
             char szFilenameAbs[RTPATH_MAX] = "";
-            int vrc = RTPathAbs(Filename, szFilenameAbs, sizeof(szFilenameAbs));
-            if (RT_FAILURE(vrc))
+            int irc = RTPathAbs(Filename, szFilenameAbs, sizeof(szFilenameAbs));
+            if (RT_FAILURE(irc))
             {
                 RTPrintf("Cannot convert filename \"%s\" to absolute path\n", Filename);
                 return 1;
@@ -1427,7 +1420,7 @@ int handleOpenMedium(HandlerArg *a)
         if (SUCCEEDED(rc) && hardDisk)
         {
             /* change the type if requested */
-            if (DiskType != HardDiskType_Normal)
+            if (DiskType != MediumType_Normal)
             {
                 CHECK_ERROR(hardDisk, COMSETTER(Type)(DiskType));
             }
@@ -1435,43 +1428,45 @@ int handleOpenMedium(HandlerArg *a)
     }
     else if (cmd == CMD_DVD)
     {
-        if (fDiskType || fSetImageId || fSetParentId)
+        if (fDiskType || fSetParentId)
             return errorSyntax(USAGE_OPENMEDIUM, "Invalid option for DVD images");
-        ComPtr<IDVDImage> dvdImage;
-        rc = a->virtualBox->OpenDVDImage(Bstr(Filename), Bstr(), dvdImage.asOutParam());
+        Bstr ImageIdStr = BstrFmt("%RTuuid", &ImageId);
+        ComPtr<IMedium> dvdImage;
+        rc = a->virtualBox->OpenDVDImage(Bstr(Filename), ImageIdStr, dvdImage.asOutParam());
         if (rc == VBOX_E_FILE_ERROR)
         {
             char szFilenameAbs[RTPATH_MAX] = "";
-            int vrc = RTPathAbs(Filename, szFilenameAbs, sizeof(szFilenameAbs));
-            if (RT_FAILURE(vrc))
+            int irc = RTPathAbs(Filename, szFilenameAbs, sizeof(szFilenameAbs));
+            if (RT_FAILURE(irc))
             {
                 RTPrintf("Cannot convert filename \"%s\" to absolute path\n", Filename);
                 return 1;
             }
-            CHECK_ERROR(a->virtualBox, OpenDVDImage(Bstr(szFilenameAbs), Bstr(), dvdImage.asOutParam()));
+            CHECK_ERROR(a->virtualBox, OpenDVDImage(Bstr(szFilenameAbs), ImageIdStr, dvdImage.asOutParam()));
         }
         else if (FAILED(rc))
-            CHECK_ERROR(a->virtualBox, OpenDVDImage(Bstr(Filename), Bstr(), dvdImage.asOutParam()));
+            CHECK_ERROR(a->virtualBox, OpenDVDImage(Bstr(Filename), ImageIdStr, dvdImage.asOutParam()));
     }
     else if (cmd == CMD_FLOPPY)
     {
         if (fDiskType || fSetImageId || fSetParentId)
             return errorSyntax(USAGE_OPENMEDIUM, "Invalid option for floppy images");
-        ComPtr<IFloppyImage> floppyImage;
-         rc = a->virtualBox->OpenFloppyImage(Bstr(Filename), Bstr(), floppyImage.asOutParam());
+        Bstr ImageIdStr = BstrFmt("%RTuuid", &ImageId);
+        ComPtr<IMedium> floppyImage;
+        rc = a->virtualBox->OpenFloppyImage(Bstr(Filename), ImageIdStr, floppyImage.asOutParam());
         if (rc == VBOX_E_FILE_ERROR)
         {
             char szFilenameAbs[RTPATH_MAX] = "";
-            int vrc = RTPathAbs(Filename, szFilenameAbs, sizeof(szFilenameAbs));
-            if (RT_FAILURE(vrc))
+            int irc = RTPathAbs(Filename, szFilenameAbs, sizeof(szFilenameAbs));
+            if (RT_FAILURE(irc))
             {
                 RTPrintf("Cannot convert filename \"%s\" to absolute path\n", Filename);
                 return 1;
             }
-            CHECK_ERROR(a->virtualBox, OpenFloppyImage(Bstr(szFilenameAbs), Bstr(), floppyImage.asOutParam()));
+            CHECK_ERROR(a->virtualBox, OpenFloppyImage(Bstr(szFilenameAbs), ImageIdStr, floppyImage.asOutParam()));
         }
         else if (FAILED(rc))
-            CHECK_ERROR(a->virtualBox, OpenFloppyImage(Bstr(Filename), Bstr(), floppyImage.asOutParam()));
+            CHECK_ERROR(a->virtualBox, OpenFloppyImage(Bstr(Filename), ImageIdStr, floppyImage.asOutParam()));
     }
 
     return SUCCEEDED(rc) ? 0 : 1;
@@ -1482,6 +1477,7 @@ static const RTGETOPTDEF g_aCloseMediumOptions[] =
     { "disk",           'd', RTGETOPT_REQ_NOTHING },
     { "dvd",            'D', RTGETOPT_REQ_NOTHING },
     { "floppy",         'f', RTGETOPT_REQ_NOTHING },
+    { "--delete",       'r', RTGETOPT_REQ_NOTHING },
 };
 
 int handleCloseMedium(HandlerArg *a)
@@ -1494,12 +1490,14 @@ int handleCloseMedium(HandlerArg *a)
         CMD_FLOPPY
     } cmd = CMD_NONE;
     const char *FilenameOrUuid = NULL;
+    bool fDelete = false;
 
     int c;
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aCloseMediumOptions, RT_ELEMENTS(g_aCloseMediumOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, a->argc, a->argv, g_aCloseMediumOptions, RT_ELEMENTS(g_aCloseMediumOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -1522,6 +1520,10 @@ int handleCloseMedium(HandlerArg *a)
                 cmd = CMD_FLOPPY;
                 break;
 
+            case 'r':   // --delete
+                fDelete = true;
+                break;
+
             case VINF_GETOPT_NOT_OPTION:
                 if (!FilenameOrUuid)
                     FilenameOrUuid = ValueUnion.psz;
@@ -1552,52 +1554,63 @@ int handleCloseMedium(HandlerArg *a)
     if (!FilenameOrUuid)
         return errorSyntax(USAGE_CLOSEMEDIUM, "Disk name or UUID required");
 
+    ComPtr<IMedium> medium;
+
     /* first guess is that it's a UUID */
     Bstr uuid(FilenameOrUuid);
 
     if (cmd == CMD_DISK)
     {
-        ComPtr<IHardDisk> hardDisk;
-        rc = a->virtualBox->GetHardDisk(uuid, hardDisk.asOutParam());
+        rc = a->virtualBox->GetHardDisk(uuid, medium.asOutParam());
         /* not a UUID or not registered? Then it must be a filename */
-        if (!hardDisk)
+        if (!medium)
         {
-            CHECK_ERROR(a->virtualBox, FindHardDisk(Bstr(FilenameOrUuid), hardDisk.asOutParam()));
-        }
-        if (SUCCEEDED(rc) && hardDisk)
-        {
-            CHECK_ERROR(hardDisk, Close());
+            CHECK_ERROR(a->virtualBox, FindHardDisk(Bstr(FilenameOrUuid), medium.asOutParam()));
         }
     }
     else
     if (cmd == CMD_DVD)
     {
-        ComPtr<IDVDImage> dvdImage;
-        rc = a->virtualBox->GetDVDImage(uuid, dvdImage.asOutParam());
+        rc = a->virtualBox->GetDVDImage(uuid, medium.asOutParam());
         /* not a UUID or not registered? Then it must be a filename */
-        if (!dvdImage)
+        if (!medium)
         {
-            CHECK_ERROR(a->virtualBox, FindDVDImage(Bstr(FilenameOrUuid), dvdImage.asOutParam()));
-        }
-        if (SUCCEEDED(rc) && dvdImage)
-        {
-            CHECK_ERROR(dvdImage, Close());
+            CHECK_ERROR(a->virtualBox, FindDVDImage(Bstr(FilenameOrUuid), medium.asOutParam()));
         }
     }
     else
     if (cmd == CMD_FLOPPY)
     {
-        ComPtr<IFloppyImage> floppyImage;
-        rc = a->virtualBox->GetFloppyImage(uuid, floppyImage.asOutParam());
+        rc = a->virtualBox->GetFloppyImage(uuid, medium.asOutParam());
         /* not a UUID or not registered? Then it must be a filename */
-        if (!floppyImage)
+        if (!medium)
         {
-            CHECK_ERROR(a->virtualBox, FindFloppyImage(Bstr(FilenameOrUuid), floppyImage.asOutParam()));
+            CHECK_ERROR(a->virtualBox, FindFloppyImage(Bstr(FilenameOrUuid), medium.asOutParam()));
         }
-        if (SUCCEEDED(rc) && floppyImage)
+    }
+
+    if (SUCCEEDED(rc) && medium)
+    {
+        if (fDelete)
         {
-            CHECK_ERROR(floppyImage, Close());
+            ComPtr<IProgress> progress;
+            CHECK_ERROR(medium, DeleteStorage(progress.asOutParam()));
+            if (SUCCEEDED(rc))
+            {
+                rc = showProgress(progress);
+                if (FAILED(rc))
+                {
+                    com::ProgressErrorInfo info(progress);
+                    if (info.isBasicAvailable())
+                        RTPrintf("Error: failed to delete medium. Error message: %lS\n", info.getText().raw());
+                    else
+                        RTPrintf("Error: failed to delete medium. No error message available!\n");
+                }
+            }
+            else
+                RTPrintf("Error: failed to delete medium. Error code %Rrc\n", rc);
         }
+        CHECK_ERROR(medium, Close());
     }
 
     return SUCCEEDED(rc) ? 0 : 1;
