@@ -1,10 +1,10 @@
-/* $Id: VBoxManageHelp.cpp $ */
+/* $Id: VBoxManageHelp.cpp 29634 2010-05-18 13:41:15Z vboxsync $ */
 /** @file
  * VBoxManage - help and other message output.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,19 +13,41 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
-#include <iprt/stream.h>
+
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
+#include <VBox/version.h>
+
+#include <iprt/ctype.h>
+#include <iprt/err.h>
 #include <iprt/getopt.h>
+#include <iprt/stream.h>
 
 #include "VBoxManage.h"
 
+
+
+void showLogo(void)
+{
+    static bool s_fShown; /* show only once */
+
+    if (!s_fShown)
+    {
+        RTPrintf(VBOX_PRODUCT" Command Line Management Interface Version "
+                 VBOX_VERSION_STRING  "\n"
+                 "(C) 2005-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
+                 "All rights reserved.\n"
+                 "\n");
+        s_fShown = true;
+    }
+}
+
 void printUsage(USAGECATEGORY u64Cmd)
 {
+    bool fDumpOpts = false;
 #ifdef RT_OS_LINUX
     bool fLinux = true;
 #else
@@ -41,6 +63,11 @@ void printUsage(USAGECATEGORY u64Cmd)
 #else
     bool fSolaris = false;
 #endif
+#ifdef RT_OS_FREEBSD
+    bool fFreeBSD = true;
+#else
+    bool fFreeBSD = false;
+#endif
 #ifdef RT_OS_DARWIN
     bool fDarwin = true;
 #else
@@ -59,9 +86,11 @@ void printUsage(USAGECATEGORY u64Cmd)
 
     if (u64Cmd == USAGE_DUMPOPTS)
     {
+        fDumpOpts = true;
         fLinux = true;
         fWin = true;
         fSolaris = true;
+        fFreeBSD = true;
         fDarwin = true;
         fVRDP = true;
         fVBoxSDL = true;
@@ -84,9 +113,9 @@ void printUsage(USAGECATEGORY u64Cmd)
 #if defined(VBOX_WITH_NETFLT)
                  "                            bridgedifs|hostonlyifs|dhcpservers|hostinfo|\n"
 #else
-                 "                            bridgedifs|hostinfo|dhcpservers|\n"
+                 "                            bridgedifs|dhcpservers|hostinfo|\n"
 #endif
-                 "                            hddbackends|hdds|dvds|floppies|\n"
+                 "                            hostcpuids|hddbackends|hdds|dvds|floppies|\n"
                  "                            usbhost|usbfilters|systemproperties\n"
                  "\n");
     }
@@ -95,6 +124,7 @@ void printUsage(USAGECATEGORY u64Cmd)
     {
         RTPrintf("VBoxManage showvminfo       <uuid>|<name> [--details] [--statistics]\n"
                  "                            [--machinereadable]\n"
+                 "VBoxManage showvminfo       <uuid>|<name> --log <idx>\n"
                  "\n");
     }
 
@@ -126,16 +156,29 @@ void printUsage(USAGECATEGORY u64Cmd)
                  "                            [--name <name>]\n"
                  "                            [--ostype <ostype>]\n"
                  "                            [--memory <memorysize in MB>]\n"
+                 "                            [--pagefusion on|off]\n"
                  "                            [--vram <vramsize in MB>]\n"
                  "                            [--acpi on|off]\n"
                  "                            [--ioapic on|off]\n"
                  "                            [--pae on|off]\n"
+                 "                            [--hpet on|off]\n"
                  "                            [--hwvirtex on|off]\n"
                  "                            [--nestedpaging on|off]\n"
+                 "                            [--largepages on|off]\n"
                  "                            [--vtxvpid on|off]\n"
+                 "                            [--synthcpu on|off]\n"
+                 "                            [--cpuidset <leaf> <eax> <ebx> <ecx> <edx>]\n"
+                 "                            [--cpuidremove <leaf>]\n"
+                 "                            [--cpuidremoveall]\n"
+                 "                            [--hardwareuuid <uuid>]\n"
                  "                            [--cpus <number>]\n"
+                 "                            [--rtcuseutc]\n"
                  "                            [--monitorcount <number>]\n"
-                 "                            [--accelerate3d <on|off>]\n"
+                 "                            [--accelerate3d on|off]\n"
+#ifdef VBOX_WITH_VIDEOHWACCEL
+                 "                            [--accelerate2dvideo on|off]\n"
+#endif
+                 "                            [--firmware bios|efi|efi32|efi64]\n"
                  "                            [--bioslogofadein on|off]\n"
                  "                            [--bioslogofadeout on|off]\n"
                  "                            [--bioslogodisplaytime <msec>]\n"
@@ -144,53 +187,61 @@ void printUsage(USAGECATEGORY u64Cmd)
                  "                            [--biossystemtimeoffset <msec>]\n"
                  "                            [--biospxedebug on|off]\n"
                  "                            [--boot<1-4> none|floppy|dvd|disk|net>]\n"
-                 "                            [--hd<a|b|d> none|<uuid>|<filename>]\n"
-                 "                            [--idecontroller PIIX3|PIIX4]\n"
-#ifdef VBOX_WITH_AHCI
-                 "                            [--sata on|off]\n"
-                 "                            [--sataportcount <1-30>]\n"
-                 "                            [--sataport<1-30> none|<uuid>|<filename>]\n"
-                 "                            [--sataideemulation<1-4> <1-30>]\n"
-#endif
-#ifdef VBOX_WITH_SCSI
-                 "                            [--scsi on|off]\n"
-                 "                            [--scsiport<1-16> none|<uuid>|<filename>]\n"
-                 "                            [--scsitype LsiLogic|BusLogic]\n"
-#endif
-                 "                            [--dvd none|<uuid>|<filename>|host:<drive>]\n"
-                 "                            [--dvdpassthrough on|off]\n"
-                 "                            [--floppy disabled|empty|<uuid>|\n"
-                 "                                      <filename>|host:<drive>]\n"
+                 "                            [--nic<1-N> none|null|nat|bridged|intnet"
 #if defined(VBOX_WITH_NETFLT)
-                 "                            [--nic<1-N> none|null|nat|bridged|intnet|hostonly]\n"
-#else /* !RT_OS_LINUX && !RT_OS_DARWIN */
-                 "                            [--nic<1-N> none|null|nat|bridged|intnet]\n"
-#endif /* !RT_OS_LINUX && !RT_OS_DARWIN  */
+                 "|hostonly"
+#endif
+#ifdef VBOX_WITH_VDE
+                 "|\n"
+                 "                                        vde"
+#endif
+                 "]\n"
                  "                            [--nictype<1-N> Am79C970A|Am79C973"
 #ifdef VBOX_WITH_E1000
               "|\n                                            82540EM|82543GC|82545EM"
 #endif
+#ifdef VBOX_WITH_VIRTIO
+              "|\n                                            virtio"
+#endif /* VBOX_WITH_VIRTIO */
                  "]\n"
                  "                            [--cableconnected<1-N> on|off]\n"
                  "                            [--nictrace<1-N> on|off]\n"
                  "                            [--nictracefile<1-N> <filename>]\n"
                  "                            [--nicspeed<1-N> <kbps>]\n"
+                 "                            [--nicbootprio<1-N> <priority>]\n"
                  "                            [--bridgeadapter<1-N> none|<devicename>]\n"
 #if defined(VBOX_WITH_NETFLT)
                  "                            [--hostonlyadapter<1-N> none|<devicename>]\n"
 #endif
                  "                            [--intnet<1-N> <network name>]\n"
                  "                            [--natnet<1-N> <network>|default]\n"
+#ifdef VBOX_WITH_VDE
+                 "                            [--vdenet<1-N> <network>|default]\n"
+#endif
+                 "                            [--natsettings<1-N> [<mtu>],[<socksnd>],\n"
+                 "                                                [<sockrcv>],[<tcpsnd>],\n"
+                 "                                                [<tcprcv>]]\n"
+                 "                            [--natpf<1-N> [<rulename>],tcp|udp,[<hostip>],\n"
+                 "                                          <hostport>,[<guestip>],<guestport>]\n"
+                 "                            [--natpf<1-N> delete <rulename>]\n"
+                 "                            [--nattftpprefix<1-N> <prefix>]\n"
+                 "                            [--nattftpfile<1-N> <file>]\n"
+                 "                            [--nattftpserver<1-N> <ip>]\n"
+                 "                            [--natdnspassdomain<1-N> on|off]\n"
+                 "                            [--natdnsproxy<1-N> on|off]\n"
+                 "                            [--natdnshostresolver<1-N> on|off]\n"
+                 "                            [--nataliasmode<1-N> default|[log],[proxyonly],\n"
+                 "                                                         [sameports]]\n"
                  "                            [--macaddress<1-N> auto|<mac>]\n"
+                 "                            [--mouse ps2|usb|usbtablet\n"
+                 "                            [--keyboard ps2|usb\n"
                  "                            [--uart<1-N> off|<I/O base> <IRQ>]\n"
                  "                            [--uartmode<1-N> disconnected|\n"
                  "                                             server <pipe>|\n"
                  "                                             client <pipe>|\n"
                  "                                             file <file>|\n"
                  "                                             <devicename>]\n"
-#ifdef VBOX_WITH_MEM_BALLOONING
                  "                            [--guestmemoryballoon <balloonsize in MB>]\n"
-#endif
                  "                            [--gueststatisticsinterval <seconds>]\n"
                  );
         RTPrintf("                            [--audio none|null");
@@ -221,6 +272,20 @@ void printUsage(USAGECATEGORY u64Cmd)
 #endif
                                              );
         }
+        if (fFreeBSD)
+        {
+            /* Get the line break sorted when dumping all option variants. */
+            if (fDumpOpts)
+            {
+                RTPrintf(                    "|\n"
+                 "                                     oss");
+            }
+            else
+                RTPrintf(                    "|oss");
+#ifdef VBOX_WITH_PULSE
+            RTPrintf(                        "|pulse");
+#endif
+        }
         if (fDarwin)
         {
             RTPrintf(                        "|coreaudio");
@@ -232,22 +297,35 @@ void printUsage(USAGECATEGORY u64Cmd)
         if (fVRDP)
         {
             RTPrintf("                            [--vrdp on|off]\n"
-                     "                            [--vrdpport default|<port>]\n"
+                     "                            [--vrdpport default|<ports>]\n"
                      "                            [--vrdpaddress <host>]\n"
                      "                            [--vrdpauthtype null|external|guest]\n"
                      "                            [--vrdpmulticon on|off]\n"
-                     "                            [--vrdpreusecon on|off]\n");
+                     "                            [--vrdpreusecon on|off]\n"
+                     "                            [--vrdpvideochannel on|off]\n"
+                     "                            [--vrdpvideochannelquality <percent>]\n");
         }
         RTPrintf("                            [--usb on|off]\n"
                  "                            [--usbehci on|off]\n"
-                 "                            [--snapshotfolder default|<path>]\n");
+                 "                            [--snapshotfolder default|<path>]\n"
+                 "                            [--teleporter on|off]\n"
+                 "                            [--teleporterport <port>]\n"
+                 "                            [--teleporteraddress <address|empty>\n"
+                 "                            [--teleporterpassword <password>]\n"
+#if 0
+                 "                            [--iocache on|off]\n"
+                 "                            [--iocachesize <I/O cache size in MB>]\n"
+                 "                            [--iobandwidthmax <Maximum I/O bandwidth in MB/s>]\n"
+#endif
+                );
         RTPrintf("\n");
     }
 
     if (u64Cmd & USAGE_IMPORTAPPLIANCE)
     {
         RTPrintf("VBoxManage import           <ovf> [--dry-run|-n] [more options]\n"
-                 "    (run with -n to have options displayed for a particular OVF)\n\n");
+                 "                            (run with -n to have options displayed\n"
+                 "                             for a particular OVF)\n\n");
     }
 
     if (u64Cmd & USAGE_EXPORTAPPLIANCE)
@@ -287,24 +365,30 @@ void printUsage(USAGECATEGORY u64Cmd)
                  "                            setlinkstate<1-N> on|off |\n"
 #ifdef VBOX_DYNAMIC_NET_ATTACH
 #if defined(VBOX_WITH_NETFLT)
-                 "                            nic<1-N> none|null|nat|bridged|intnet|hostonly\n"
+                 "                            nic<1-N> null|nat|bridged|intnet|hostonly\n"
                  "                                     [<devicename>] |\n"
 #else /* !RT_OS_LINUX && !RT_OS_DARWIN */
-                 "                            nic<1-N> none|null|nat|bridged|intnet\n"
+                 "                            nic<1-N> null|nat|bridged|intnet\n"
                  "                                     [<devicename>] |\n"
 #endif /* !RT_OS_LINUX && !RT_OS_DARWIN  */
+                 "                            nictrace<1-N> on|off\n"
+                 "                            nictracefile<1-N> <filename>\n"
 #endif /* VBOX_DYNAMIC_NET_ATTACH */
+                 "                            guestmemoryballoon <balloonsize in MB>]\n"
+                 "                            gueststatisticsinterval <seconds>]\n"
                  "                            usbattach <uuid>|<address> |\n"
-                 "                            usbdetach <uuid>|<address> |\n"
-                 "                            dvdattach none|<uuid>|<filename>|host:<drive> |\n"
-                 "                            floppyattach none|<uuid>|<filename>|host:<drive> |\n");
+                 "                            usbdetach <uuid>|<address> |\n");
         if (fVRDP)
         {
-            RTPrintf("                            vrdp on|off] |\n");
+            RTPrintf("                            vrdp on|off |\n");
+            RTPrintf("                            vrdpport default|<ports> |\n"
+                     "                            vrdpvideochannelquality <percent>\n");
         }
-        RTPrintf("                            setvideomodehint <xres> <yres> <bpp> [display]|\n"
+        RTPrintf("                            setvideomodehint <xres> <yres> <bpp> [display] |\n"
                  "                            setcredentials <username> <password> <domain>\n"
-                 "                                           [--allowlocallogon <yes|no>]\n"
+                 "                                           [--allowlocallogon <yes|no>] |\n"
+                 "                            teleport --host <name> --port <port>\n"
+                 "                                   [--maxdowntime <msec>] [--password password]\n"
                  "\n");
     }
 
@@ -323,9 +407,10 @@ void printUsage(USAGECATEGORY u64Cmd)
     if (u64Cmd & USAGE_SNAPSHOT)
     {
         RTPrintf("VBoxManage snapshot         <uuid>|<name>\n"
-                 "                            take <name> [--description <desc>] |\n"
-                 "                            discard <uuid>|<name> |\n"
-                 "                            discardcurrent --state|--all |\n"
+                 "                            take <name> [--description <desc>] [--pause] |\n"
+                 "                            delete <uuid>|<name> |\n"
+                 "                            restore <uuid>|<name> |\n"
+                 "                            restorecurrent |\n"
                  "                            edit <uuid>|<name>|--current\n"
                  "                                 [--name <name>]\n"
                  "                                 [--description <desc>] |\n"
@@ -337,12 +422,43 @@ void printUsage(USAGECATEGORY u64Cmd)
     {
         RTPrintf("VBoxManage openmedium       disk|dvd|floppy <filename>\n"
                  "                            [--type normal|immutable|writethrough] (disk only)\n"
+                 "                            [--uuid <uuid>]\n"
+                 "                            [--parentuuid <uuid>] (disk only)\n"
                  "\n");
     }
 
     if (u64Cmd & USAGE_CLOSEMEDIUM)
     {
         RTPrintf("VBoxManage closemedium      disk|dvd|floppy <uuid>|<filename>\n"
+                 "                            [--delete]\n"
+                 "\n");
+    }
+
+    if (u64Cmd & USAGE_STORAGEATTACH)
+    {
+        RTPrintf("VBoxManage storageattach    <uuid|vmname>\n"
+                 "                            --storagectl <name>\n"
+                 "                            --port <number>\n"
+                 "                            --device <number>\n"
+                 "                            [--type dvddrive|hdd|fdd]\n"
+                 "                            [--medium none|emptydrive|\n"
+                 "                                      <uuid>|<filename>|host:<drive>]\n"
+                 "                            [--passthrough on|off]\n"
+                 "                            [--forceunmount]\n"
+                 "\n");
+    }
+
+    if (u64Cmd & USAGE_STORAGECONTROLLER)
+    {
+        RTPrintf("VBoxManage storagectl       <uuid|vmname>\n"
+                 "                            --name <name>\n"
+                 "                            [--add ide|sata|scsi|floppy|sas]\n"
+                 "                            [--controller LSILogic|LSILogicSAS|BusLogic|\n"
+                 "                                          IntelAHCI|PIIX3|PIIX4|ICH6|I82078]\n"
+                 "                            [--sataideemulation<1-4> <1-30>]\n"
+                 "                            [--sataportcount <1-30>]\n"
+                 "                            [--hostiocache on|off]\n"
+                 "                            [--remove]\n"
                  "\n");
     }
 
@@ -404,7 +520,6 @@ void printUsage(USAGECATEGORY u64Cmd)
                  "                            [--username <username>]\n"
                  "                            [--password <password>]\n"
                  "                            [--type normal|writethrough|immutable]\n"
-                 "                            [--comment <comment>]\n"
                  "                            [--intnet]\n"
                  "\n");
     }
@@ -504,19 +619,30 @@ void printUsage(USAGECATEGORY u64Cmd)
         usageGuestProperty();
 #endif /* VBOX_WITH_GUEST_PROPS defined */
 
+#ifdef VBOX_WITH_GUEST_CONTROL
+    if (u64Cmd & USAGE_GUESTCONTROL)
+        usageGuestControl();
+#endif /* VBOX_WITH_GUEST_CONTROL defined */
+
     if (u64Cmd & USAGE_METRICS)
     {
         RTPrintf("VBoxManage metrics          list [*|host|<vmname> [<metric_list>]]\n"
                  "                                                 (comma-separated)\n\n"
                  "VBoxManage metrics          setup\n"
-                 "                            [--period <seconds>]\n"
-                 "                            [--samples <count>]\n"
+                 "                            [--period <seconds>] (default: 1)\n"
+                 "                            [--samples <count>] (default: 1)\n"
                  "                            [--list]\n"
                  "                            [*|host|<vmname> [<metric_list>]]\n\n"
                  "VBoxManage metrics          query [*|host|<vmname> [<metric_list>]]\n\n"
+                 "VBoxManage metrics          enable\n"
+                 "                            [--list]\n"
+                 "                            [*|host|<vmname> [<metric_list>]]\n\n"
+                 "VBoxManage metrics          disable\n"
+                 "                            [--list]\n"
+                 "                            [*|host|<vmname> [<metric_list>]]\n\n"
                  "VBoxManage metrics          collect\n"
-                 "                            [--period <seconds>]\n"
-                 "                            [--samples <count>]\n"
+                 "                            [--period <seconds>] (default: 1)\n"
+                 "                            [--samples <count>] (default: 1)\n"
                  "                            [--list]\n"
                  "                            [--detach]\n"
                  "                            [*|host|<vmname> [<metric_list>]]\n"
@@ -547,7 +673,7 @@ void printUsage(USAGECATEGORY u64Cmd)
                  "                                 --netmask <network_mask>\n"
                  "                                 --lowerip <lower_ip>\n"
                  "                                 --upperip <upper_ip>]\n"
-                 "                                [--enable | --disable]\n"
+                 "                                [--enable | --disable]\n\n"
                  "VBoxManage dhcpserver       remove --netname <network_name> |\n"
 #if defined(VBOX_WITH_NETFLT)
                  "                                   --ifname <hostonly_if_name>\n"
@@ -576,6 +702,40 @@ int errorSyntax(USAGECATEGORY u64Cmd, const char *pszFormat, ...)
     return 1;
 }
 
+/**
+ * errorSyntax for RTGetOpt users.
+ *
+ * @returns 1.
+ *
+ * @param   fUsageCategory  The usage category of the command.
+ * @param   rc              The RTGetOpt return code.
+ * @param   pValueUnion     The value union.
+ */
+int errorGetOpt(USAGECATEGORY fUsageCategory, int rc, union RTGETOPTUNION const *pValueUnion)
+{
+    showLogo(); // show logo even if suppressed
+#ifndef VBOX_ONLY_DOCS
+    if (g_fInternalMode)
+        printUsageInternal(fUsageCategory);
+    else
+        printUsage(fUsageCategory);
+#endif /* !VBOX_ONLY_DOCS */
+
+    if (rc == VINF_GETOPT_NOT_OPTION)
+        return RTPrintf("error: Invalid parameter '%s'\n", pValueUnion->psz);
+    if (rc > 0)
+    {
+        if (RT_C_IS_PRINT(rc))
+            return RTPrintf("error: Invalid option -%c\n", rc);
+        return RTPrintf("error: Invalid option case %i\n", rc);
+    }
+    if (rc == VERR_GETOPT_UNKNOWN_OPTION)
+        return RTPrintf("error: unknown option: %s\n", pValueUnion->psz);
+    if (pValueUnion->pDef)
+        return RTPrintf("error: %s: %Rrs\n", pValueUnion->pDef->pszLong, rc);
+    return RTPrintf("error: %Rrs\n", rc);
+}
+
 /**
  * Print an error message without the syntax stuff.
  */
@@ -587,4 +747,3 @@ int errorArgument(const char *pszFormat, ...)
     va_end(args);
     return 1;
 }
-
