@@ -1,10 +1,10 @@
-/* $Id: VBoxManageHostonly.cpp $ */
+/* $Id: VBoxManageHostonly.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxManage - Implementation of hostonlyif command.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -26,7 +22,7 @@
 #include <VBox/com/com.h>
 #include <VBox/com/array.h>
 #include <VBox/com/ErrorInfo.h>
-#include <VBox/com/errorprint2.h>
+#include <VBox/com/errorprint.h>
 #include <VBox/com/EventQueue.h>
 
 #include <VBox/com/VirtualBox.h>
@@ -70,20 +66,15 @@ static int handleCreate(HandlerArg *a, int iStart, int *pcProcessed)
 
     CHECK_ERROR(host, CreateHostOnlyNetworkInterface (hif.asOutParam(), progress.asOutParam()));
 
-    showProgress(progress);
-
-    HRESULT hr;
-    CHECK_ERROR(progress, COMGETTER(ResultCode) (&hr));
-
+    rc = showProgress(progress);
     *pcProcessed = index - iStart;
-
-    if(FAILED(hr))
+    if (FAILED(rc))
     {
         com::ProgressErrorInfo info(progress);
         if (info.isBasicAvailable())
             RTPrintf("Error: failed to create the host-only adapter. Error message: %lS\n", info.getText().raw());
         else
-            RTPrintf("Error: failed to create the host-only adapter. No error message available, HRESULT code: 0x%x\n", hr);
+            RTPrintf("Error: failed to create the host-only adapter. No error message available, HRESULT code: 0x%x\n", rc);
 
         return 1;
     }
@@ -113,26 +104,21 @@ static int handleRemove(HandlerArg *a, int iStart, int *pcProcessed)
     ComPtr<IHostNetworkInterface> hif;
     CHECK_ERROR(host, FindHostNetworkInterfaceByName(name, hif.asOutParam()));
 
-    GUID guid;
-    CHECK_ERROR(hif, COMGETTER(Id)(&guid));
+    Bstr guid;
+    CHECK_ERROR(hif, COMGETTER(Id)(guid.asOutParam()));
 
     ComPtr<IProgress> progress;
-    CHECK_ERROR(host, RemoveHostOnlyNetworkInterface (guid, hif.asOutParam(),progress.asOutParam()));
-
-    showProgress(progress);
-
-    HRESULT hr;
-    CHECK_ERROR(progress, COMGETTER(ResultCode) (&hr));
+    CHECK_ERROR(host, RemoveHostOnlyNetworkInterface (guid, progress.asOutParam()));
 
+    rc = showProgress(progress);
     *pcProcessed = index - iStart;
-
-    if(FAILED(hr))
+    if (FAILED(rc))
     {
         com::ProgressErrorInfo info(progress);
         if (info.isBasicAvailable())
             RTPrintf("Error: failed to remove the host-only adapter. Error message: %lS\n", info.getText().raw());
         else
-            RTPrintf("Error: failed to remove the host-only adapter. No error message available, HRESULT code: 0x%x\n", hr);
+            RTPrintf("Error: failed to remove the host-only adapter. No error message available, HRESULT code: 0x%x\n", rc);
 
         return 1;
     }
@@ -182,7 +168,7 @@ static int handleIpconfig(HandlerArg *a, int iStart, int *pcProcessed)
                  g_aHostOnlyIPOptions,
                  RT_ELEMENTS(g_aHostOnlyIPOptions),
                  index,
-                 0 /* fFlags */);
+                 RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
