@@ -1,10 +1,10 @@
-/* $Id: VBoxManageImport.cpp $ */
+/* $Id: VBoxManageImport.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxManage - The appliance-related commands.
  */
 
 /*
- * Copyright (C) 2009 Sun Microsystems, Inc.
+ * Copyright (C) 2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef VBOX_ONLY_DOCS
@@ -30,7 +26,7 @@
 #include <VBox/com/Guid.h>
 #include <VBox/com/array.h>
 #include <VBox/com/ErrorInfo.h>
-#include <VBox/com/errorprint2.h>
+#include <VBox/com/errorprint.h>
 #include <VBox/com/EventQueue.h>
 
 #include <VBox/com/VirtualBox.h>
@@ -46,6 +42,7 @@
 #include <iprt/file.h>
 
 #include <VBox/log.h>
+#include <VBox/param.h>
 
 #include "VBoxManage.h"
 using namespace com;
@@ -93,6 +90,9 @@ static const RTGETOPTDEF g_aImportApplianceOptions[] =
     { "-ostype",                'o', RTGETOPT_REQ_STRING },     // deprecated
     { "--vmname",               'V', RTGETOPT_REQ_STRING },
     { "-vmname",                'V', RTGETOPT_REQ_STRING },     // deprecated
+    { "--memory",               'm', RTGETOPT_REQ_STRING },
+    { "-memory",                'm', RTGETOPT_REQ_STRING },     // deprecated
+    { "--cpus",                 'c', RTGETOPT_REQ_STRING },
     { "--description",          'd', RTGETOPT_REQ_STRING },
     { "--eula",                 'L', RTGETOPT_REQ_STRING },
     { "-eula",                  'L', RTGETOPT_REQ_STRING },     // deprecated
@@ -106,7 +106,7 @@ static const RTGETOPTDEF g_aImportApplianceOptions[] =
     { "-type",                  'T', RTGETOPT_REQ_UINT32 },     // deprecated
 };
 
-int handleImportAppliance(HandlerArg *a)
+int handleImportAppliance(HandlerArg *arg)
 {
     HRESULT rc = S_OK;
 
@@ -124,7 +124,8 @@ int handleImportAppliance(HandlerArg *a)
     RTGETOPTUNION ValueUnion;
     RTGETOPTSTATE GetState;
     // start at 0 because main() has hacked both the argc and argv given to us
-    RTGetOptInit(&GetState, a->argc, a->argv, g_aImportApplianceOptions, RT_ELEMENTS(g_aImportApplianceOptions), 0, 0 /* fFlags */);
+    RTGetOptInit(&GetState, arg->argc, arg->argv, g_aImportApplianceOptions, RT_ELEMENTS(g_aImportApplianceOptions),
+                 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
     while ((c = RTGetOpt(&GetState, &ValueUnion)))
     {
         switch (c)
@@ -172,6 +173,12 @@ int handleImportAppliance(HandlerArg *a)
                 mapArgsMapsPerVsys[ulCurVsys]["memory"] = ValueUnion.psz;
                 break;
 
+            case 'c':   // --cpus
+                if (ulCurVsys == (uint32_t)-1)
+                    return errorSyntax(USAGE_IMPORTAPPLIANCE, "Option \"%s\" requires preceding --vsys argument.", GetState.pDef->pszLong);
+                mapArgsMapsPerVsys[ulCurVsys]["cpus"] = ValueUnion.psz;
+                break;
+
             case 'u':   // --unit
                 ulCurUnit = ValueUnion.u32;
                 break;
@@ -201,7 +208,7 @@ int handleImportAppliance(HandlerArg *a)
                 break;
 
             case VINF_GETOPT_NOT_OPTION:
-                if (!strOvfFilename)
+                if (strOvfFilename.isEmpty())
                     strOvfFilename = ValueUnion.psz;
                 else
                     return errorSyntax(USAGE_IMPORTAPPLIANCE, "Invalid parameter '%s'", ValueUnion.psz);
@@ -224,21 +231,40 @@ int handleImportAppliance(HandlerArg *a)
         }
     }
 
-    if (!strOvfFilename)
+    if (strOvfFilename.isEmpty())
         return errorSyntax(USAGE_IMPORTAPPLIANCE, "Not enough arguments for \"import\" command.");
 
     do
     {
         ComPtr<IAppliance> pAppliance;
-        CHECK_ERROR_BREAK(a->virtualBox, CreateAppliance(pAppliance.asOutParam()));
+        CHECK_ERROR_BREAK(arg->virtualBox, CreateAppliance(pAppliance.asOutParam()));
 
-        char *pszAbsFilePath = RTPathAbsDup(strOvfFilename.c_str());
-        CHECK_ERROR_BREAK(pAppliance, Read(Bstr(pszAbsFilePath)));
+        char *pszAbsFilePath;
+        if (strOvfFilename.startsWith("S3://", iprt::MiniString::CaseInsensitive) ||
+            strOvfFilename.startsWith("SunCloud://", iprt::MiniString::CaseInsensitive) ||
+            strOvfFilename.startsWith("webdav://", iprt::MiniString::CaseInsensitive))
+            pszAbsFilePath = RTStrDup(strOvfFilename.c_str());
+        else
+            pszAbsFilePath = RTPathAbsDup(strOvfFilename.c_str());
+        ComPtr<IProgress> progressRead;
+        CHECK_ERROR_BREAK(pAppliance, Read(Bstr(pszAbsFilePath), progressRead.asOutParam()));
         RTStrFree(pszAbsFilePath);
 
+        rc = showProgress(progressRead);
+
+        if (FAILED(rc))
+        {
+            com::ProgressErrorInfo info(progressRead);
+            com::GluePrintErrorInfo(info);
+            com::GluePrintErrorContext("ImportAppliance", __FILE__, __LINE__);
+            return 1;
+        }
+
+        Bstr path; /* fetch the path, there is stuff like username/password removed if any */
+        CHECK_ERROR_BREAK(pAppliance, COMGETTER(Path)(path.asOutParam()));
         // call interpret(); this can yield both warnings and errors, so we need
         // to tinker with the error info a bit
-        RTPrintf("Interpreting %s...\n", strOvfFilename.c_str());
+        RTPrintf("Interpreting %ls...\n", path.raw());
         rc = pAppliance->Interpret();
         com::ErrorInfo info0(pAppliance);
 
@@ -433,8 +459,26 @@ int handleImportAppliance(HandlerArg *a)
                         break;
 
                         case VirtualSystemDescriptionType_CPU:
-                            RTPrintf("%2u: Number of CPUs (ignored): %ls\n",
-                                     a, aVboxValues[a]);
+                            if (findArgValue(strOverride, pmapArgs, "cpus"))
+                            {
+                                uint32_t cCPUs;
+                                if (    strOverride.toInt(cCPUs) == VINF_SUCCESS
+                                     && cCPUs >= VMM_MIN_CPU_COUNT
+                                     && cCPUs <= VMM_MAX_CPU_COUNT
+                                   )
+                                {
+                                    bstrFinalValue = strOverride;
+                                    RTPrintf("%2u: No. of CPUs specified with --cpus: %ls\n",
+                                             a, bstrFinalValue.raw());
+                                }
+                                else
+                                    return errorSyntax(USAGE_IMPORTAPPLIANCE,
+                                                       "Argument to --cpus option must be a number greater than %d and less than %d.",
+                                                       VMM_MIN_CPU_COUNT - 1, VMM_MAX_CPU_COUNT + 1);
+                            }
+                            else
+                                RTPrintf("%2u: Number of CPUs: %ls\n    (change with \"--vsys %u --cpus <n>\")\n",
+                                         a, bstrFinalValue.raw(), i);
                         break;
 
                         case VirtualSystemDescriptionType_Memory:
@@ -643,16 +687,14 @@ int handleImportAppliance(HandlerArg *a)
                 CHECK_ERROR_BREAK(pAppliance,
                                   ImportMachines(progress.asOutParam()));
 
-                showProgress(progress);
-
-                if (SUCCEEDED(rc))
-                    progress->COMGETTER(ResultCode)(&rc);
+                rc = showProgress(progress);
 
                 if (FAILED(rc))
                 {
                     com::ProgressErrorInfo info(progress);
                     com::GluePrintErrorInfo(info);
                     com::GluePrintErrorContext("ImportAppliance", __FILE__, __LINE__);
+                    return 1;
                 }
                 else
                     RTPrintf("Successfully imported the appliance.\n");
@@ -696,7 +738,7 @@ int handleExportAppliance(HandlerArg *a)
         RTGETOPTSTATE GetState;
         // start at 0 because main() has hacked both the argc and argv given to us
         RTGetOptInit(&GetState, a->argc, a->argv, g_aExportOptions,
-                     RT_ELEMENTS(g_aExportOptions), 0, 0 /* fFlags */);
+                     RT_ELEMENTS(g_aExportOptions), 0, RTGETOPTINIT_FLAGS_NO_STD_OPTS);
 
         Utf8Str strProductUrl;
         while ((c = RTGetOpt(&GetState, &ValueUnion)))
@@ -766,7 +808,7 @@ int handleExportAppliance(HandlerArg *a)
                     // must be machine: try UUID or name
                     ComPtr<IMachine> machine;
                     /* assume it's a UUID */
-                    rc = a->virtualBox->GetMachine(Guid(strMachine), machine.asOutParam());
+                    rc = a->virtualBox->GetMachine(Bstr(strMachine), machine.asOutParam());
                     if (FAILED(rc) || !machine)
                     {
                         /* must be a name */
@@ -861,11 +903,11 @@ int handleExportAppliance(HandlerArg *a)
                         Utf8Str strContent;
                         void *pvFile;
                         size_t cbFile;
-                        int rc = RTFileReadAll(itD->second.c_str(), &pvFile, &cbFile);
-                        if (RT_SUCCESS(rc))
+                        int irc = RTFileReadAll(itD->second.c_str(), &pvFile, &cbFile);
+                        if (RT_SUCCESS(irc))
                         {
-                            Bstr strContent((char*)pvFile);
-                            pVSD->AddDescription (VirtualSystemDescriptionType_License, strContent, strContent);
+                            Bstr bstrContent((char*)pvFile);
+                            pVSD->AddDescription(VirtualSystemDescriptionType_License, bstrContent, bstrContent);
                             RTFileReadAllFree(pvFile, cbFile);
                         }
                         else
@@ -884,20 +926,24 @@ int handleExportAppliance(HandlerArg *a)
             break;
 
         ComPtr<IProgress> progress;
-        char *pszAbsFilePath = RTPathAbsDup(strOutputFile.c_str());
+        char *pszAbsFilePath;
+        if (strOutputFile.startsWith("S3://", iprt::MiniString::CaseInsensitive) ||
+            strOutputFile.startsWith("SunCloud://", iprt::MiniString::CaseInsensitive) ||
+            strOutputFile.startsWith("webdav://", iprt::MiniString::CaseInsensitive))
+            pszAbsFilePath = RTStrDup(strOutputFile.c_str());
+        else
+            pszAbsFilePath = RTPathAbsDup(strOutputFile.c_str());
         CHECK_ERROR_BREAK(pAppliance, Write(Bstr(strOvfFormat), Bstr(pszAbsFilePath), progress.asOutParam()));
         RTStrFree(pszAbsFilePath);
 
-        showProgress(progress);
-
-        if (SUCCEEDED(rc))
-            progress->COMGETTER(ResultCode)(&rc);
+        rc = showProgress(progress);
 
         if (FAILED(rc))
         {
             com::ProgressErrorInfo info(progress);
             com::GluePrintErrorInfo(info);
             com::GluePrintErrorContext("Write", __FILE__, __LINE__);
+            return 1;
         }
         else
             RTPrintf("Successfully exported %d machine(s).\n", llMachines.size());
