@@ -1,10 +1,10 @@
-/* $Id: VBoxManageMetrics.cpp $ */
+/* $Id: VBoxManageMetrics.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxManage - The 'metrics' command.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef VBOX_ONLY_DOCS
@@ -27,7 +23,7 @@
 #include <VBox/com/com.h>
 #include <VBox/com/array.h>
 #include <VBox/com/ErrorInfo.h>
-#include <VBox/com/errorprint2.h>
+#include <VBox/com/errorprint.h>
 #include <VBox/com/VirtualBox.h>
 
 #include <iprt/asm.h>
@@ -504,19 +500,19 @@ static int handleMetricsCollect(int argc, char *argv[],
                                                  ComSafeArrayAsOutParam(retIndices),
                                                  ComSafeArrayAsOutParam(retLengths),
                                                  ComSafeArrayAsOutParam(retData)) );
-        for (unsigned i = 0; i < retNames.size(); i++)
+        for (unsigned j = 0; j < retNames.size(); j++)
         {
-            Bstr metricUnit(retUnits[i]);
-            Bstr metricName(retNames[i]);
-            RTPrintf("%-12s %-10ls %-20ls ", ts, getObjectName(aVirtualBox, retObjects[i]).raw(), metricName.raw());
+            Bstr metricUnit(retUnits[j]);
+            Bstr metricName(retNames[j]);
+            RTPrintf("%-12s %-10ls %-20ls ", ts, getObjectName(aVirtualBox, retObjects[j]).raw(), metricName.raw());
             const char *separator = "";
-            for (unsigned j = 0; j < retLengths[i]; j++)
+            for (unsigned k = 0; k < retLengths[j]; k++)
             {
-                if (retScales[i] == 1)
-                    RTPrintf("%s%d %ls", separator, retData[retIndices[i] + j], metricUnit.raw());
+                if (retScales[j] == 1)
+                    RTPrintf("%s%d %ls", separator, retData[retIndices[j] + k], metricUnit.raw());
                 else
-                    RTPrintf("%s%d.%02d%ls", separator, retData[retIndices[i] + j] / retScales[i],
-                             (retData[retIndices[i] + j] * 100 / retScales[i]) % 100, metricUnit.raw());
+                    RTPrintf("%s%d.%02d%ls", separator, retData[retIndices[j] + k] / retScales[j],
+                             (retData[retIndices[j] + k] * 100 / retScales[j]) % 100, metricUnit.raw());
                 separator = ", ";
             }
             RTPrintf("\n");
@@ -531,6 +527,91 @@ static int handleMetricsCollect(int argc, char *argv[],
     return 0;
 }
 
+/**
+ * Enable metrics
+ */
+static int handleMetricsEnable(int argc, char *argv[],
+                               ComPtr<IVirtualBox> aVirtualBox,
+                               ComPtr<IPerformanceCollector> performanceCollector)
+{
+    HRESULT rc;
+    com::SafeArray<BSTR>          metrics;
+    com::SafeArray<BSTR>          baseMetrics;
+    com::SafeIfaceArray<IUnknown> objects;
+    bool listMatches = false;
+    int i;
+
+    for (i = 1; i < argc; i++)
+    {
+        if (   !strcmp(argv[i], "--list")
+            || !strcmp(argv[i], "-list"))
+            listMatches = true;
+        else
+            break; /* The rest of params should define the filter */
+    }
+
+    rc = parseFilterParameters(argc - i, &argv[i], aVirtualBox,
+                               ComSafeArrayAsOutParam(metrics),
+                               ComSafeArrayAsOutParam(baseMetrics),
+                               ComSafeArrayAsOutParam(objects));
+    if (FAILED(rc))
+        return 1;
+
+    com::SafeIfaceArray<IPerformanceMetric> affectedMetrics;
+    CHECK_ERROR(performanceCollector,
+        EnableMetrics(ComSafeArrayAsInParam(metrics),
+                      ComSafeArrayAsInParam(objects),
+                      ComSafeArrayAsOutParam(affectedMetrics)));
+    if (listMatches)
+        listAffectedMetrics(aVirtualBox,
+                            ComSafeArrayAsInParam(affectedMetrics));
+
+    return 0;
+}
+
+/**
+ * Disable metrics
+ */
+static int handleMetricsDisable(int argc, char *argv[],
+                                ComPtr<IVirtualBox> aVirtualBox,
+                                ComPtr<IPerformanceCollector> performanceCollector)
+{
+    HRESULT rc;
+    com::SafeArray<BSTR>          metrics;
+    com::SafeArray<BSTR>          baseMetrics;
+    com::SafeIfaceArray<IUnknown> objects;
+    bool listMatches = false;
+    int i;
+
+    for (i = 1; i < argc; i++)
+    {
+        if (   !strcmp(argv[i], "--list")
+            || !strcmp(argv[i], "-list"))
+            listMatches = true;
+        else
+            break; /* The rest of params should define the filter */
+    }
+
+    rc = parseFilterParameters(argc - i, &argv[i], aVirtualBox,
+                               ComSafeArrayAsOutParam(metrics),
+                               ComSafeArrayAsOutParam(baseMetrics),
+                               ComSafeArrayAsOutParam(objects));
+    if (FAILED(rc))
+        return 1;
+
+    com::SafeIfaceArray<IPerformanceMetric> affectedMetrics;
+    CHECK_ERROR(performanceCollector,
+        DisableMetrics(ComSafeArrayAsInParam(metrics),
+                       ComSafeArrayAsInParam(objects),
+                       ComSafeArrayAsOutParam(affectedMetrics)));
+    if (listMatches)
+        listAffectedMetrics(aVirtualBox,
+                            ComSafeArrayAsInParam(affectedMetrics));
+
+    return 0;
+}
+
+
 int handleMetrics(HandlerArg *a)
 {
     int rc;
@@ -550,6 +631,10 @@ int handleMetrics(HandlerArg *a)
         rc = handleMetricsQuery(a->argc, a->argv, a->virtualBox, performanceCollector);
     else if (!strcmp(a->argv[0], "collect"))
         rc = handleMetricsCollect(a->argc, a->argv, a->virtualBox, performanceCollector);
+    else if (!strcmp(a->argv[0], "enable"))
+        rc = handleMetricsEnable(a->argc, a->argv, a->virtualBox, performanceCollector);
+    else if (!strcmp(a->argv[0], "disable"))
+        rc = handleMetricsDisable(a->argc, a->argv, a->virtualBox, performanceCollector);
     else
         return errorSyntax(USAGE_METRICS, "Invalid subcommand '%s'", a->argv[0]);
 
