@@ -1,10 +1,10 @@
-/* $Id: VBoxManageUSB.cpp $ */
+/* $Id: VBoxManageUSB.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxManage - VirtualBox's command-line interface.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <VBox/com/com.h>
@@ -24,7 +20,7 @@
 #include <VBox/com/Guid.h>
 #include <VBox/com/array.h>
 #include <VBox/com/ErrorInfo.h>
-#include <VBox/com/errorprint2.h>
+#include <VBox/com/errorprint.h>
 #include <VBox/com/EventQueue.h>
 
 #include <VBox/com/VirtualBox.h>
@@ -78,6 +74,10 @@ public:
         Guid guid(iid);
         if (guid == Guid(NS_GET_IID(IUnknown)))
             *ppvObject = (IUnknown *)this;
+#ifdef RT_OS_WINDOWS
+        else if (guid == Guid(NS_GET_IID(IDispatch)))
+            *ppvObject = (IDispatch *)this;
+#endif
         else if (guid == Guid(NS_GET_IID(IUSBDevice)))
             *ppvObject = (IUSBDevice *)this;
         else
@@ -233,7 +233,7 @@ int handleUSBFilter (HandlerArg *a)
                     else
                     {
                         /* assume it's a UUID of a machine */
-                        rc = a->virtualBox->GetMachine(Guid(a->argv[i]), cmd.mMachine.asOutParam());
+                        rc = a->virtualBox->GetMachine(Bstr(a->argv[i]), cmd.mMachine.asOutParam());
                         if (FAILED(rc) || !cmd.mMachine)
                         {
                             /* must be a name */
@@ -325,9 +325,9 @@ int handleUSBFilter (HandlerArg *a)
                         return errorArgument("Missing argument to '%s'", a->argv[i]);
                     i++;
                     uint32_t u32;
-                    rc = RTStrToUInt32Full(a->argv[i], 0, &u32);
-                    if (RT_FAILURE(rc))
-                        return errorArgument("Failed to convert the --maskedinterfaces value '%s' to a number, rc=%Rrc", a->argv[i], rc);
+                    int vrc = RTStrToUInt32Full(a->argv[i], 0, &u32);
+                    if (RT_FAILURE(vrc))
+                        return errorArgument("Failed to convert the --maskedinterfaces value '%s' to a number, vrc=%Rrc", a->argv[i], vrc);
                     cmd.mFilter.mMaskedInterfaces = u32;
                 }
                 else if (   !strcmp(a->argv[i], "--action")
@@ -387,7 +387,7 @@ int handleUSBFilter (HandlerArg *a)
                     else
                     {
                         /* assume it's a UUID of a machine */
-                        rc = a->virtualBox->GetMachine(Guid(a->argv[i]), cmd.mMachine.asOutParam());
+                        rc = a->virtualBox->GetMachine(Bstr(a->argv[i]), cmd.mMachine.asOutParam());
                         if (FAILED(rc) || !cmd.mMachine)
                         {
                             /* must be a name */
@@ -415,7 +415,7 @@ int handleUSBFilter (HandlerArg *a)
         CHECK_ERROR_RET (a->virtualBox, COMGETTER(Host) (host.asOutParam()), 1);
     else
     {
-        Guid uuid;
+        Bstr uuid;
         cmd.mMachine->COMGETTER(Id)(uuid.asOutParam());
         /* open a session for the VM */
         CHECK_ERROR_RET (a->virtualBox, OpenSession(a->session, uuid), 1);
@@ -436,16 +436,16 @@ int handleUSBFilter (HandlerArg *a)
 
                 if (!f.mActive.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(Active) (f.mActive));
-                if (!f.mVendorId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId.setNullIfEmpty()));
-                if (!f.mProductId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId.setNullIfEmpty()));
-                if (!f.mRevision.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision.setNullIfEmpty()));
-                if (!f.mManufacturer.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer.setNullIfEmpty()));
-                if (!f.mSerialNumber.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber.setNullIfEmpty()));
+                if (!f.mVendorId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId));
+                if (!f.mProductId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId));
+                if (!f.mRevision.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision));
+                if (!f.mManufacturer.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer));
+                if (!f.mSerialNumber.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber));
                 if (!f.mMaskedInterfaces.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(MaskedInterfaces) (f.mMaskedInterfaces));
 
@@ -461,18 +461,18 @@ int handleUSBFilter (HandlerArg *a)
 
                 if (!f.mActive.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(Active) (f.mActive));
-                if (!f.mVendorId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId.setNullIfEmpty()));
-                if (!f.mProductId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId.setNullIfEmpty()));
-                if (!f.mRevision.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision.setNullIfEmpty()));
-                if (!f.mManufacturer.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer.setNullIfEmpty()));
-                if (!f.mRemote.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Remote) (f.mRemote.setNullIfEmpty()));
-                if (!f.mSerialNumber.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber.setNullIfEmpty()));
+                if (!f.mVendorId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId));
+                if (!f.mProductId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId));
+                if (!f.mRevision.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision));
+                if (!f.mManufacturer.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer));
+                if (!f.mRemote.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Remote) (f.mRemote));
+                if (!f.mSerialNumber.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber));
                 if (!f.mMaskedInterfaces.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(MaskedInterfaces) (f.mMaskedInterfaces));
 
@@ -489,20 +489,20 @@ int handleUSBFilter (HandlerArg *a)
 
                 ComPtr <IHostUSBDeviceFilter> flt = coll[cmd.mIndex];
 
-                if (!f.mName.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Name) (f.mName.setNullIfEmpty()));
+                if (!f.mName.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Name) (f.mName));
                 if (!f.mActive.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(Active) (f.mActive));
-                if (!f.mVendorId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId.setNullIfEmpty()));
-                if (!f.mProductId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId.setNullIfEmpty()));
-                if (!f.mRevision.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision.setNullIfEmpty()));
-                if (!f.mManufacturer.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer.setNullIfEmpty()));
-                if (!f.mSerialNumber.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber.setNullIfEmpty()));
+                if (!f.mVendorId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId));
+                if (!f.mProductId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId));
+                if (!f.mRevision.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision));
+                if (!f.mManufacturer.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer));
+                if (!f.mSerialNumber.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber));
                 if (!f.mMaskedInterfaces.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(MaskedInterfaces) (f.mMaskedInterfaces));
 
@@ -516,22 +516,22 @@ int handleUSBFilter (HandlerArg *a)
 
                 ComPtr <IUSBDeviceFilter> flt = coll[cmd.mIndex];
 
-                if (!f.mName.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Name) (f.mName.setNullIfEmpty()));
+                if (!f.mName.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Name) (f.mName));
                 if (!f.mActive.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(Active) (f.mActive));
-                if (!f.mVendorId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId.setNullIfEmpty()));
-                if (!f.mProductId.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId.setNullIfEmpty()));
-                if (!f.mRevision.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision.setNullIfEmpty()));
-                if (!f.mManufacturer.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer.setNullIfEmpty()));
-                if (!f.mRemote.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(Remote) (f.mRemote.setNullIfEmpty()));
-                if (!f.mSerialNumber.isNull())
-                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber.setNullIfEmpty()));
+                if (!f.mVendorId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(VendorId) (f.mVendorId));
+                if (!f.mProductId.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(ProductId) (f.mProductId));
+                if (!f.mRevision.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Revision) (f.mRevision));
+                if (!f.mManufacturer.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Manufacturer) (f.mManufacturer));
+                if (!f.mRemote.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(Remote) (f.mRemote));
+                if (!f.mSerialNumber.isEmpty())
+                    CHECK_ERROR_BREAK (flt, COMSETTER(SerialNumber) (f.mSerialNumber));
                 if (!f.mMaskedInterfaces.isNull())
                     CHECK_ERROR_BREAK (flt, COMSETTER(MaskedInterfaces) (f.mMaskedInterfaces));
             }
@@ -542,7 +542,7 @@ int handleUSBFilter (HandlerArg *a)
             if (cmd.mGlobal)
             {
                 ComPtr <IHostUSBDeviceFilter> flt;
-                CHECK_ERROR_BREAK (host, RemoveUSBDeviceFilter (cmd.mIndex, flt.asOutParam()));
+                CHECK_ERROR_BREAK (host, RemoveUSBDeviceFilter (cmd.mIndex));
             }
             else
             {
@@ -557,8 +557,12 @@ int handleUSBFilter (HandlerArg *a)
 
     if (cmd.mMachine)
     {
-        /* commit and close the session */
-        CHECK_ERROR(cmd.mMachine, SaveSettings());
+        if (SUCCEEDED (rc))
+        {
+            /* commit the session */
+            CHECK_ERROR(cmd.mMachine, SaveSettings());
+        }
+        /* close the session */
         a->session->Close();
     }
 
