@@ -1,3 +1,4 @@
+/* $Id: VBoxFrameBuffer.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  *
  * VBox frontends: Qt GUI ("VirtualBox"):
@@ -5,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,12 +15,11 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include "precomp.h"
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 #include "VBoxFrameBuffer.h"
 
 #include "VBoxConsoleView.h"
@@ -28,6 +28,7 @@
 
 /* Qt includes */
 #include <QPainter>
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
 
 //
 // VBoxFrameBuffer class
@@ -38,13 +39,15 @@
  *  Base class for all frame buffer implementations.
  */
 
-#if !defined (Q_OS_WIN32)
+#if defined (Q_OS_WIN32)
+static CComModule _Module;
+#else
 NS_DECL_CLASSINFO (VBoxFrameBuffer)
 NS_IMPL_THREADSAFE_ISUPPORTS1_CI (VBoxFrameBuffer, IFramebuffer)
 #endif
 
 VBoxFrameBuffer::VBoxFrameBuffer (VBoxConsoleView *aView)
-    : mView (aView), mMutex (new QMutex (QMutex::Recursive))
+    : mView (aView)
     , mWdt (0), mHgt (0)
 #if defined (Q_OS_WIN32)
     , refcnt (0)
@@ -52,11 +55,13 @@ VBoxFrameBuffer::VBoxFrameBuffer (VBoxConsoleView *aView)
 {
     AssertMsg (mView, ("VBoxConsoleView must not be null\n"));
     mWinId = (mView && mView->viewport()) ? (ULONG64) mView->viewport()->winId() : 0;
+    int rc = RTCritSectInit(&mCritSect);
+    AssertRC(rc);
 }
 
 VBoxFrameBuffer::~VBoxFrameBuffer()
 {
-    delete mMutex;
+    RTCritSectDelete(&mCritSect);
 }
 
 // IFramebuffer implementation methods.
@@ -184,17 +189,6 @@ STDMETHODIMP VBoxFrameBuffer::RequestResize (ULONG aScreenId, ULONG aPixelFormat
     return S_OK;
 }
 
-STDMETHODIMP
-VBoxFrameBuffer::OperationSupported (FramebufferAccelerationOperation_T aOperation,
-                                     BOOL *aSupported)
-{
-    NOREF(aOperation);
-    if (!aSupported)
-        return E_POINTER;
-    *aSupported = FALSE;
-    return S_OK;
-}
-
 /**
  * Returns whether we like the given video mode.
  *
@@ -223,39 +217,8 @@ STDMETHODIMP VBoxFrameBuffer::VideoModeSupported (ULONG aWidth, ULONG aHeight,
         && (aHeight > (ULONG) screen.height())
        )
         *aSupported = FALSE;
-    LogFlowThisFunc(("returning aSupported=%d\n", *aSupported));
-    return S_OK;
-}
-
-STDMETHODIMP VBoxFrameBuffer::SolidFill (ULONG aX, ULONG aY,
-                                         ULONG aWidth, ULONG aHeight,
-                                         ULONG aColor, BOOL *aHandled)
-{
-    NOREF(aX);
-    NOREF(aY);
-    NOREF(aWidth);
-    NOREF(aHeight);
-    NOREF(aColor);
-    if (!aHandled)
-        return E_POINTER;
-    *aHandled = FALSE;
-    return S_OK;
-}
-
-STDMETHODIMP VBoxFrameBuffer::CopyScreenBits (ULONG aXDst, ULONG aYDst,
-                                              ULONG aXSrc, ULONG aYSrc,
-                                              ULONG aWidth, ULONG aHeight,
-                                              BOOL *aHandled)
-{
-    NOREF(aXDst);
-    NOREF(aYDst);
-    NOREF(aXSrc);
-    NOREF(aYSrc);
-    NOREF(aWidth);
-    NOREF(aHeight);
-    if (!aHandled)
-        return E_POINTER;
-    *aHandled = FALSE;
+    LogFlowThisFunc(("screenW=%lu, screenH=%lu -> aSupported=%s\n",
+                    screen.width(), screen.height(), *aSupported ? "TRUE" : "FALSE"));
     return S_OK;
 }
 
@@ -299,6 +262,21 @@ STDMETHODIMP VBoxFrameBuffer::SetVisibleRegion (BYTE *aRectangles, ULONG aCount)
     return S_OK;
 }
 
+STDMETHODIMP VBoxFrameBuffer::ProcessVHWACommand(BYTE *pCommand)
+{
+    Q_UNUSED (pCommand);
+    return E_NOTIMPL;
+}
+
+#ifdef VBOX_WITH_VIDEOHWACCEL
+void VBoxFrameBuffer::doProcessVHWACommand(QEvent * pEvent)
+{
+    Q_UNUSED(pEvent);
+    /* should never be here */
+    AssertBreakpoint();
+}
+#endif
+
 //
 // VBoxQImageFrameBuffer class
 /////////////////////////////////////////////////////////////////////////////
@@ -315,14 +293,14 @@ STDMETHODIMP VBoxFrameBuffer::SetVisibleRegion (BYTE *aRectangles, ULONG aCount)
 VBoxQImageFrameBuffer::VBoxQImageFrameBuffer (VBoxConsoleView *aView) :
     VBoxFrameBuffer (aView)
 {
+    /* Initialize the framebuffer the first time */
     resizeEvent (new VBoxResizeEvent (FramebufferPixelFormat_Opaque,
                                       NULL, 0, 0, 640, 480));
 }
 
 /** @note This method is called on EMT from under this object's lock */
 STDMETHODIMP VBoxQImageFrameBuffer::NotifyUpdate (ULONG aX, ULONG aY,
-                                                  ULONG aW, ULONG aH,
-                                                  BOOL *aFinished)
+                                                  ULONG aW, ULONG aH)
 {
     /* We're not on the GUI thread and update() isn't thread safe in
      * Qt 4.3.x on the Win, Qt 3.3.x on the Mac (4.2.x is),
@@ -331,14 +309,12 @@ STDMETHODIMP VBoxQImageFrameBuffer::NotifyUpdate (ULONG aX, ULONG aY,
     QApplication::postEvent (mView,
                              new VBoxRepaintEvent (aX, aY, aW, aH));
 
-    /* The update has been finished, return TRUE */
-    *aFinished = TRUE;
     return S_OK;
 }
 
 void VBoxQImageFrameBuffer::paintEvent (QPaintEvent *pe)
 {
-    const QRect &r = pe->rect().intersected (mView->viewport()->rect());
+    const QRect &r = pe->rect().intersected (QRect (0, 0, mWdt, mHgt));
 
     /* Some outdated rectangle during processing VBoxResizeEvent */
     if (r.isEmpty())
@@ -393,7 +369,7 @@ void VBoxQImageFrameBuffer::resizeEvent (VBoxResizeEvent *re)
         QImage::Format format;
         switch (re->bitsPerPixel())
         {
-            /* 32-, 8- and 1-bpp are the only depths suported by QImage */
+            /* 32-, 8- and 1-bpp are the only depths supported by QImage */
             case 32:
                 format = QImage::Format_RGB32;
                 break;
@@ -406,6 +382,7 @@ void VBoxQImageFrameBuffer::resizeEvent (VBoxResizeEvent *re)
                 remind = true;
                 break;
             default:
+                format = QImage::Format_Invalid; /* set it to something so gcc keeps quiet. */
                 remind = true;
                 fallback = true;
                 break;
@@ -464,6 +441,16 @@ void VBoxQImageFrameBuffer::resizeEvent (VBoxResizeEvent *re)
 
 #endif
 
+//
+// VBoxQGLFrameBuffer class
+/////////////////////////////////////////////////////////////////////////////
+
+#if defined (VBOX_GUI_USE_QGLFB)
+
+/* The class is defined in VBoxFBQGL.cpp */
+
+#endif
+
 //
 // VBoxSDLFrameBuffer class
 /////////////////////////////////////////////////////////////////////////////
@@ -504,8 +491,7 @@ VBoxSDLFrameBuffer::~VBoxSDLFrameBuffer()
 
 /** @note This method is called on EMT from under this object's lock */
 STDMETHODIMP VBoxSDLFrameBuffer::NotifyUpdate (ULONG aX, ULONG aY,
-                                               ULONG aW, ULONG aH,
-                                               BOOL *aFinished)
+                                               ULONG aW, ULONG aH)
 {
 #if !defined (Q_WS_WIN) && !defined (Q_WS_PM)
     /* we're not on the GUI thread and update() isn't thread safe in Qt 3.3.x
@@ -519,8 +505,6 @@ STDMETHODIMP VBoxSDLFrameBuffer::NotifyUpdate (ULONG aX, ULONG aY,
                                aY - mView->contentsY(),
                                aW, aH);
 #endif
-    /* the update has been finished, return TRUE */
-    *aFinished = TRUE;
     return S_OK;
 }
 
@@ -689,3 +673,4 @@ void VBoxSDLFrameBuffer::resizeEvent (VBoxResizeEvent *re)
 /* The class is defined in VBoxQuartz2D.cpp */
 
 #endif
+
