@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,12 +14,11 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+#ifdef VBOX_WITH_PRECOMPILED_HEADERS
+# include "precomp.h"
+#else  /* !VBOX_WITH_PRECOMPILED_HEADERS */
 #include "VBoxProblemReporter.h"
 #include "VBoxSelectorWnd.h"
 #include "VBoxVMListView.h"
@@ -27,15 +26,17 @@
 #include "VBoxToolBar.h"
 
 #include "VBoxSnapshotsWgt.h"
-#include "VBoxNewVMWzd.h"
+#include "UINewVMWzd.h"
 #include "VBoxMediaManagerDlg.h"
-#include "VBoxImportApplianceWzd.h"
-#include "VBoxExportApplianceWzd.h"
+#include "UIImportApplianceWzd.h"
+#include "UIExportApplianceWzd.h"
 #include "VBoxSettingsDialogSpecific.h"
 #include "VBoxVMLogViewer.h"
 #include "VBoxGlobal.h"
 #include "VBoxUtils.h"
 
+#include "UIDownloaderUserManual.h"
+
 #ifdef Q_WS_X11
 #include <iprt/env.h>
 #endif
@@ -49,6 +50,10 @@
 #include <QDesktopWidget>
 #include <QToolButton>
 
+#include <iprt/buildconfig.h>
+#include <VBox/version.h>
+#endif /* !VBOX_WITH_PRECOMPILED_HEADERS */
+
 // VBoxVMDetailsView class
 ////////////////////////////////////////////////////////////////////////////////
 
@@ -474,18 +479,18 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent,
     centralLayout->addLayout (rightVLayout, 2);
 
     /* VM list toolbar */
-    VBoxToolBar *vmTools = new VBoxToolBar (this);
+    mVMToolBar = new VBoxToolBar (this);
 #if MAC_LEOPARD_STYLE
     /* Enable unified toolbars on Mac OS X. Available on Qt >= 4.3 */
-    addToolBar (vmTools);
-    vmTools->setMacToolbar();
+    addToolBar (mVMToolBar);
+    mVMToolBar->setMacToolbar();
     /* No spacing/margin on the mac */
     VBoxGlobal::setLayoutMargin (centralLayout, 0);
     leftVLayout->setSpacing (0);
     rightVLayout->setSpacing (0);
     rightVLayout->insertSpacing (0, 10);
 #else /* MAC_LEOPARD_STYLE */
-    leftVLayout->addWidget (vmTools);
+    leftVLayout->addWidget (mVMToolBar);
     centralLayout->setSpacing (9);
     VBoxGlobal::setLayoutMargin (centralLayout, 5);
     leftVLayout->setSpacing (5);
@@ -532,17 +537,17 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent,
 
     /* add actions to the toolbar */
 
-    vmTools->setIconSize (QSize (32, 32));
-    vmTools->setToolButtonStyle (Qt::ToolButtonTextUnderIcon);
-    vmTools->setSizePolicy (QSizePolicy::Fixed, QSizePolicy::Preferred);
+    mVMToolBar->setIconSize (QSize (32, 32));
+    mVMToolBar->setToolButtonStyle (Qt::ToolButtonTextUnderIcon);
+    mVMToolBar->setSizePolicy (QSizePolicy::Fixed, QSizePolicy::Preferred);
 
-    vmTools->addAction (mVmNewAction);
-    vmTools->addAction (mVmConfigAction);
+    mVMToolBar->addAction (mVmNewAction);
+    mVMToolBar->addAction (mVmConfigAction);
 #if 0 /* delete action is really rare */
-    vmTools->addAction (mVmDeleteAction);
+    mVMToolBar->addAction (mVmDeleteAction);
 #endif
-    vmTools->addAction (mVmStartAction);
-    vmTools->addAction (mVmDiscardAction);
+    mVMToolBar->addAction (mVmStartAction);
+    mVMToolBar->addAction (mVmDiscardAction);
 
     /* add actions to menubar */
 
@@ -582,6 +587,11 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent,
     mVMCtxtMenu->addAction (mVmRefreshAction);
     mVMCtxtMenu->addAction (mVmShowLogsAction);
 
+    /* Make sure every status bar hint from the context menu is cleared when
+     * the menu is closed. */
+    connect (mVMCtxtMenu, SIGNAL(aboutToHide()),
+             statusBar(), SLOT(clearMessage()));
+
     mHelpMenu = menuBar()->addMenu (QString::null);
     mHelpActions.addTo (mHelpMenu);
 
@@ -706,6 +716,9 @@ VBoxSelectorWnd (VBoxSelectorWnd **aSelf, QWidget* aParent,
              this, SLOT (trayIconChanged (const VBoxChangeTrayIconEvent &)));
 #endif
 
+    /* Listen to potential downloaders signals: */
+    connect(&vboxProblem(), SIGNAL(sigDownloaderUserManualCreated()), this, SLOT(sltDownloaderUserManualEmbed()));
+
     /* bring the VM list to the focus */
     mVMListView->setFocus();
 }
@@ -718,16 +731,20 @@ VBoxSelectorWnd::~VBoxSelectorWnd()
     {
         int y = mNormalGeo.y();
 #if defined (Q_WS_MAC) && !defined (QT_MAC_USE_COCOA)
-        /* The toolbar counts to the content not to the frame. Unfortunaly the
-         * toolbar isn't fully initialized when this window will be moved to
-         * the last position after VBox starting. As a workaround just do
+        /* The toolbar counts to the content not to the frame. Unfortunately
+         * the toolbar isn't fully initialized when this window will be moved
+         * to the last position after VBox starting. As a workaround just do
          * remove the toolbar height part when save the last position. */
         y -= ::darwinWindowToolBarHeight (this);
 #endif /* Q_WS_MAC && !QT_MAC_USE_COCOA */
         QString winPos = QString ("%1,%2,%3,%4")
             .arg (mNormalGeo.x()).arg (y)
             .arg (mNormalGeo.width()).arg (mNormalGeo.height());
+#ifdef Q_WS_MAC
+        if (::darwinIsWindowMaximized(this))
+#else /* Q_WS_MAC */
         if (isMaximized())
+#endif /* !Q_WS_MAC */
             winPos += QString (",%1").arg (VBoxDefs::GUI_LastWindowPosition_Max);
 
         vbox.SetExtraData (VBoxDefs::GUI_LastWindowPosition, winPos);
@@ -763,7 +780,7 @@ void VBoxSelectorWnd::fileMediaMgr()
 
 void VBoxSelectorWnd::fileImportAppliance()
 {
-    VBoxImportApplianceWzd wzd (this);
+    UIImportApplianceWzd wzd (this);
 
     wzd.exec();
 }
@@ -776,7 +793,7 @@ void VBoxSelectorWnd::fileExportAppliance()
     if (item)
         name = item->name();
 
-    VBoxExportApplianceWzd wzd (this, name);
+    UIExportApplianceWzd wzd (this, name);
 
     wzd.exec();
 }
@@ -813,7 +830,7 @@ void VBoxSelectorWnd::fileExit()
 
 void VBoxSelectorWnd::vmNew()
 {
-    VBoxNewVMWzd wzd (this);
+    UINewVMWzd wzd (this);
     if (wzd.exec() == QDialog::Accepted)
     {
         CMachine m = wzd.machine();
@@ -904,18 +921,18 @@ void VBoxSelectorWnd::vmDelete (const QString &aUuid /*= QUuid_null*/)
                 return;
             CMachine machine = session.GetMachine();
             /* Detach all attached Hard Disks */
-            CHardDiskAttachmentVector vec = machine.GetHardDiskAttachments();
+            CMediumAttachmentVector vec = machine.GetMediumAttachments();
             for (int i = 0; i < vec.size(); ++ i)
             {
-                CHardDiskAttachment hda = vec [i];
+                CMediumAttachment hda = vec [i];
                 const QString ctlName = hda.GetController();
 
-                machine.DetachHardDisk(ctlName, hda.GetPort(), hda.GetDevice());
+                machine.DetachDevice(ctlName, hda.GetPort(), hda.GetDevice());
                 if (!machine.isOk())
                 {
                     CStorageController ctl = machine.GetStorageControllerByName(ctlName);
-                    vboxProblem().cannotDetachHardDisk (this, machine,
-                        vboxGlobal().getMedium (CMedium (hda.GetHardDisk())).location(),
+                    vboxProblem().cannotDetachDevice (this, machine, VBoxDefs::MediumType_HardDisk,
+                        vboxGlobal().getMedium (CMedium (hda.GetMedium())).location(),
                         ctl.GetBus(), hda.GetPort(), hda.GetDevice());
                 }
             }
@@ -979,8 +996,11 @@ void VBoxSelectorWnd::vmStart (const QString &aUuid /*= QUuid_null*/)
         return;
     }
 
-    AssertMsg (item->state() < KMachineState_Running,
-               ("Machine must be PoweredOff/Saved/Aborted"));
+    AssertMsg (   item->state() == KMachineState_PoweredOff
+               || item->state() == KMachineState_Saved
+               || item->state() == KMachineState_Teleported
+               || item->state() == KMachineState_Aborted
+               , ("Machine must be PoweredOff/Saved/Aborted (%d)", item->state()));
 
     QString id = item->id();
     CVirtualBox vbox = vboxGlobal().virtualBox();
@@ -1016,11 +1036,12 @@ void VBoxSelectorWnd::vmStart (const QString &aUuid /*= QUuid_null*/)
         return;
     }
 
-    /* show the "VM spawning" progress dialog */
-    vboxProblem().showModalProgressDialog (progress, item->name(), this);
-
+    /* Hide the "VM spawning" progress dialog */
+    /* I hope 1 minute will be enough to spawn any running VM silently, isn't it? */
+    int iSpawningDuration = 60000;
+    vboxProblem().showModalProgressDialog(progress, item->name(), this, iSpawningDuration);
     if (progress.GetResultCode() != 0)
-        vboxProblem().cannotOpenSession (vbox, item->machine(), progress);
+        vboxProblem().cannotOpenSession(vbox, item->machine(), progress);
 
     session.Close();
 
@@ -1116,7 +1137,7 @@ void VBoxSelectorWnd::vmRefresh (const QString &aUuid /*= QUuid_null*/)
                    true /* aDescription */);
 
     if (!oldAccessible && item->accessible())
-        vboxGlobal().checkForAutoConvertedSettingsAfterRefresh();
+        fileExit();
 }
 
 void VBoxSelectorWnd::vmShowLogs (const QString &aUuid /*= QUuid_null*/)
@@ -1232,7 +1253,13 @@ bool VBoxSelectorWnd::event (QEvent *e)
                 mNormalGeo.moveTo (geometry().x(), geometry().y());
             break;
         }
-
+        case QEvent::WindowDeactivate:
+        {
+            /* Make sure every status bar hint is cleared when the window lost
+             * focus. */
+            statusBar()->clearMessage();
+            break;
+        }
         default:
             break;
     }
@@ -1286,11 +1313,21 @@ bool VBoxSelectorWnd::eventFilter (QObject *aObject, QEvent *aEvent)
 void VBoxSelectorWnd::retranslateUi()
 {
 #ifdef VBOX_OSE
-    setWindowTitle (tr ("VirtualBox OSE"));
+    QString title (tr ("VirtualBox OSE"));
 #else
-    setWindowTitle (tr ("Sun VirtualBox"));
+    QString title (VBOX_PRODUCT);
 #endif
 
+#ifdef VBOX_BLEEDING_EDGE
+    title += QString(" EXPERIMENTAL build ")
+          +  QString(RTBldCfgVersion())
+          +  QString(" r")
+          +  QString(RTBldCfgRevisionStr())
+          +  QString(" - "VBOX_BLEEDING_EDGE);
+#endif
+
+    setWindowTitle (title);
+
     mVmTabWidget->setTabText (mVmTabWidget->indexOf (mVmDetailsView), tr ("&Details"));
     /* note: Snapshots and Details tabs are changed dynamically by
      * vmListViewCurrentChanged() */
@@ -1308,7 +1345,7 @@ void VBoxSelectorWnd::retranslateUi()
 
     mFileApplianceExportAction->setText (tr ("&Export Appliance..."));
     mFileApplianceExportAction->setShortcut (QKeySequence ("Ctrl+E"));
-    mFileApplianceExportAction->setStatusTip (tr ("Export an appliance out of VM's from VirtualBox"));
+    mFileApplianceExportAction->setStatusTip (tr ("Export one or more VirtualBox virtual machines as an appliance"));
 
 #ifdef Q_WS_MAC
     /*
@@ -1386,6 +1423,13 @@ void VBoxSelectorWnd::retranslateUi()
         mTrayIcon->refresh();
     }
 #endif
+
+#ifdef QT_MAC_USE_COCOA
+    /* There is a bug in Qt Cocoa which result in showing a "more
+     * arrow" when the necessary size of the toolbar is increased. So
+     * manually adjust the size after changing the text. */
+    mVMToolBar->adjustSize();
+#endif /* QT_MAC_USE_COCOA */
 }
 
 
@@ -1444,29 +1488,51 @@ void VBoxSelectorWnd::vmListViewCurrentChanged (bool aRefreshDetails,
         mVmConfigAction->setEnabled (modifyEnabled);
         mVmDeleteAction->setEnabled (modifyEnabled);
         mVmDiscardAction->setEnabled (state == KMachineState_Saved && !running);
-        mVmPauseAction->setEnabled (state == KMachineState_Running ||
-                                   state == KMachineState_Paused);
+        mVmPauseAction->setEnabled (   state == KMachineState_Running
+                                    || state == KMachineState_Teleporting
+                                    || state == KMachineState_LiveSnapshotting
+                                    || state == KMachineState_Paused
+                                    || state == KMachineState_TeleportingPausedVM /** @todo Live Migration: does this make sense? */
+                                   );
 
         /* change the Start button text accordingly */
-        if (state >= KMachineState_Running)
-        {
-            mVmStartAction->setText (tr ("S&how"));
-            mVmStartAction->setStatusTip (
-                tr ("Switch to the window of the selected virtual machine"));
-
-            mVmStartAction->setEnabled (item->canSwitchTo());
-        }
-        else
+        if (   state == KMachineState_PoweredOff
+            || state == KMachineState_Saved
+            || state == KMachineState_Teleported
+            || state == KMachineState_Aborted
+           )
         {
             mVmStartAction->setText (tr ("S&tart"));
+#ifdef QT_MAC_USE_COCOA
+            /* There is a bug in Qt Cocoa which result in showing a "more
+             * arrow" when the necessary size of the toolbar is increased. So
+             * manually adjust the size after changing the text. */
+            mVMToolBar->adjustSize();
+#endif /* QT_MAC_USE_COCOA */
             mVmStartAction->setStatusTip (
                 tr ("Start the selected virtual machine"));
 
             mVmStartAction->setEnabled (!running);
         }
+        else
+        {
+            mVmStartAction->setText (tr ("S&how"));
+#ifdef QT_MAC_USE_COCOA
+            /* There is a bug in Qt Cocoa which result in showing a "more
+             * arrow" when the necessary size of the toolbar is increased. So
+             * manually adjust the size after changing the text. */
+            mVMToolBar->adjustSize();
+#endif /* QT_MAC_USE_COCOA */
+            mVmStartAction->setStatusTip (
+                tr ("Switch to the window of the selected virtual machine"));
+
+            mVmStartAction->setEnabled (item->canSwitchTo());
+        }
 
         /* change the Pause/Resume button text accordingly */
-        if (state == KMachineState_Paused)
+        if (   state == KMachineState_Paused
+            || state == KMachineState_TeleportingPausedVM /*?*/
+           )
         {
             mVmPauseAction->setText (tr ("R&esume"));
             mVmPauseAction->setShortcut (QKeySequence ("Ctrl+P"));
@@ -1513,7 +1579,7 @@ void VBoxSelectorWnd::vmListViewCurrentChanged (bool aRefreshDetails,
             mVmDetailsView->setDetailsText
                 (tr ("<h3>"
                      "Welcome to VirtualBox!</h3>"
-                     "<p>The left part of this window is intended to display "
+                     "<p>The left part of this window is  "
                      "a list of all virtual machines on your computer. "
                      "The list is empty now because you haven't created any virtual "
                      "machines yet."
@@ -1592,7 +1658,7 @@ void VBoxSelectorWnd::mediumEnumFinished (const VBoxMediaList &list)
         /* look for at least one inaccessible media */
         VBoxMediaList::const_iterator it;
         for (it = list.begin(); it != list.end(); ++ it)
-            if ((*it).state() == KMediaState_Inaccessible)
+            if ((*it).state() == KMediumState_Inaccessible)
                 break;
 
         if (it != list.end() && vboxProblem().remindAboutInaccessibleMedia())
@@ -1854,26 +1920,36 @@ void VBoxTrayIcon::showSubMenu ()
         mVmDiscardAction->setEnabled (s == KMachineState_Saved && !running);
 
         /* Change the Start button text accordingly */
-        if (s >= KMachineState_Running)
-        {
-            mVmStartAction->setText (VBoxVMListView::tr ("S&how"));
-            mVmStartAction->setStatusTip (
-                  VBoxVMListView::tr ("Switch to the window of the selected virtual machine"));
-            mVmStartAction->setEnabled (pItem->canSwitchTo());
-        }
-        else
+        if (   s == KMachineState_PoweredOff
+            || s == KMachineState_Saved
+            || s == KMachineState_Teleported
+            || s == KMachineState_Aborted
+           )
         {
             mVmStartAction->setText (VBoxVMListView::tr ("S&tart"));
             mVmStartAction->setStatusTip (
                   VBoxVMListView::tr ("Start the selected virtual machine"));
             mVmStartAction->setEnabled (!running);
         }
+        else
+        {
+            mVmStartAction->setText (VBoxVMListView::tr ("S&how"));
+            mVmStartAction->setStatusTip (
+                  VBoxVMListView::tr ("Switch to the window of the selected virtual machine"));
+            mVmStartAction->setEnabled (pItem->canSwitchTo());
+        }
 
         /* Change the Pause/Resume button text accordingly */
-        mVmPauseAction->setEnabled (s == KMachineState_Running ||
-                                    s == KMachineState_Paused);
+        mVmPauseAction->setEnabled (   s == KMachineState_Running
+                                    || s == KMachineState_Teleporting
+                                    || s == KMachineState_LiveSnapshotting
+                                    || s == KMachineState_Paused
+                                    || s == KMachineState_TeleportingPausedVM
+                                   );
 
-        if (s == KMachineState_Paused)
+        if (   s == KMachineState_Paused
+            || s == KMachineState_TeleportingPausedVM /*?*/
+           )
         {
             mVmPauseAction->setText (VBoxVMListView::tr ("R&esume"));
             mVmPauseAction->setStatusTip (
@@ -2087,4 +2163,11 @@ void VBoxTrayIcon::vmShowLogs()
 
 #endif // VBOX_GUI_WITH_SYSTRAY
 
+void VBoxSelectorWnd::sltDownloaderUserManualEmbed()
+{
+    /* If there is User Manual downloader created => show the process bar: */
+    if (UIDownloaderUserManual *pDl = UIDownloaderUserManual::current())
+        statusBar()->addWidget(pDl->processWidget(this), 0);
+}
+
 #include "VBoxSelectorWnd.moc"
