@@ -1,10 +1,10 @@
-/* $Id: VBoxCocoaApplication.h $ */
+/* $Id: VBoxCocoaApplication.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBoxCocoaApplication - NSApplication subclass for handling -sendEvent.
  */
 
 /*
- * Copyright (C) 2009 Sun Microsystems, Inc.
+ * Copyright (C) 2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___darwin_VBoxCocoaApplication_h
@@ -28,7 +24,7 @@
 #endif
 #include <Carbon/Carbon.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 /** Event handler callback.
  * @returns true if handled, false if not.
@@ -90,10 +86,11 @@ unsigned long VBoxCocoaApplication_getEventModifierFlags(const void *pvEvent);
 uint32_t VBoxCocoaApplication_getEventModifierFlagsXlated(const void *pvEvent);
 void VBoxCocoaApplication_setMouseCoalescingEnabled(bool fEnabled);
 const char *VBoxCocoaApplication_eventTypeName(unsigned long eEvtType);
+bool VBoxCocoaApplication_isApplicationCommand(const void *pvEvent);
 void VBoxCocoaApplication_printEvent(const char *pszPrefix, const void *pvEvent);
 /** @} */
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
