@@ -1,10 +1,10 @@
-/* $Id: VBoxUtils-darwin.cpp $ */
+/* $Id: VBoxUtils-darwin.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * Qt GUI - Utility Classes and Functions specific to Darwin.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,15 +13,11 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "VBoxUtils-darwin.h"
 
-#include <iprt/assert.h>
+#include <iprt/mem.h>
 
 #include <QApplication>
 #include <QWidget>
@@ -106,6 +102,17 @@ void darwinSetShowsResizeIndicator (QWidget *aWidget, bool aEnabled)
     ::darwinSetShowsResizeIndicatorImpl (::darwinToNativeWindow (aWidget), aEnabled);
 }
 
+bool darwinIsWindowMaximized(QWidget *aWidget)
+{
+#ifdef QT_MAC_USE_COCOA
+    /* Currently only necessary in the Cocoa version */
+    return ::darwinIsWindowMaximized(::darwinToNativeWindow(aWidget));
+#else /* QT_MAC_USE_COCOA */
+    NOREF (aWidget);
+    return false;
+#endif /* !QT_MAC_USE_COCOA */
+}
+
 QString darwinSystemLanguage (void)
 {
     /* Get the locales supported by our bundle */
@@ -139,6 +146,31 @@ void darwinDisableIconsInMenus (void)
 #endif /* QT_VERSION >= 0x040400 */
 }
 
+int darwinWindowToolBarHeight (QWidget *aWidget)
+{
+#ifndef QT_MAC_USE_COCOA
+    return ::darwinWindowToolBarHeight (::darwinToNativeWindow (aWidget));
+#else /* QT_MAC_USE_COCOA */
+    NOREF (aWidget);
+    return 0;
+#endif /* QT_MAC_USE_COCOA */
+}
+
+bool darwinSetFrontMostProcess()
+{
+    ProcessSerialNumber psn = { 0, kCurrentProcess };
+    return ::SetFrontProcess(&psn) == 0;
+}
+
+uint64_t darwinGetCurrentProcessId()
+{
+    uint64_t processId = 0;
+    ProcessSerialNumber psn = { 0, kCurrentProcess };
+    if (::GetCurrentProcess(&psn) == 0)
+        processId = RT_MAKE_U64(psn.lowLongOfPSN, psn.highLongOfPSN);
+    return processId;
+}
+
 CGContextRef darwinToCGContextRef (QWidget *aWidget)
 {
     return static_cast<CGContext *> (aWidget->macCGHandle());
@@ -211,7 +243,36 @@ CGImageRef darwinToCGImageRef (const QImage *aImage)
  */
 CGImageRef darwinToCGImageRef (const QPixmap *aPixmap)
 {
-    return aPixmap->toMacCGImageRef();
+    /* It seems Qt releases the memory to an returned CGImageRef when the
+     * associated QPixmap is destroyed. This shouldn't happen as long a
+     * CGImageRef has a retrain count. As a workaround we make a real copy. */
+    int bitmapBytesPerRow = aPixmap->width() * 4;
+    int bitmapByteCount = (bitmapBytesPerRow * aPixmap->height());
+    /* Create a memory block for the temporary image. It is initialized by zero
+     * which means black & zero alpha. */
+    void *pBitmapData = RTMemAllocZ (bitmapByteCount);
+    CGColorSpaceRef cs = CGColorSpaceCreateDeviceRGB();
+    /* Create a context to paint on */
+    CGContextRef ctx = CGBitmapContextCreate (pBitmapData,
+                                              aPixmap->width(),
+                                              aPixmap->height(),
+                                              8,
+                                              bitmapBytesPerRow,
+                                              cs,
+                                              kCGImageAlphaPremultipliedFirst);
+    /* Get the CGImageRef from Qt */
+    CGImageRef qtPixmap = aPixmap->toMacCGImageRef();
+    /* Draw the image from Qt & convert the context back to a new CGImageRef. */
+    CGContextDrawImage (ctx, CGRectMake (0, 0, aPixmap->width(), aPixmap->height()), qtPixmap);
+    CGImageRef newImage = CGBitmapContextCreateImage (ctx);
+    /* Now release all used resources */
+    CGImageRelease (qtPixmap);
+    CGContextRelease (ctx);
+    CGColorSpaceRelease (cs);
+    RTMemFree (pBitmapData);
+
+    /* Return the new CGImageRef */
+    return newImage;
 }
 
 /**
@@ -399,7 +460,7 @@ void darwinDebugPrintEvent (const char *psz, EventRef evtRef)
       UInt32 keyCode = 0;
       ::GetEventParameter (evtRef, kEventParamKeyCode, typeUInt32, NULL,
                            sizeof (keyCode), NULL, &keyCode);
-      printf(" keyCode=%d (%#x) ", keyCode, keyCode);
+      printf(" keyCode=%d (%#x) ", (int)keyCode, (unsigned)keyCode);
 
       char macCharCodes[8] = {0,0,0,0, 0,0,0,0};
       ::GetEventParameter (evtRef, kEventParamKeyCode, typeChar, NULL,
@@ -412,7 +473,7 @@ void darwinDebugPrintEvent (const char *psz, EventRef evtRef)
       UInt32 modifierMask = 0;
       ::GetEventParameter (evtRef, kEventParamKeyModifiers, typeUInt32, NULL,
                            sizeof (modifierMask), NULL, &modifierMask);
-      printf(" modifierMask=%08x", modifierMask);
+      printf(" modifierMask=%08x", (unsigned)modifierMask);
 
       UniChar keyUnicodes[8] = {0,0,0,0, 0,0,0,0};
       ::GetEventParameter (evtRef, kEventParamKeyUnicodes, typeUnicodeText, NULL,
@@ -425,12 +486,12 @@ void darwinDebugPrintEvent (const char *psz, EventRef evtRef)
       UInt32 keyboardType = 0;
       ::GetEventParameter (evtRef, kEventParamKeyboardType, typeUInt32, NULL,
                            sizeof (keyboardType), NULL, &keyboardType);
-      printf(" keyboardType=%08x", keyboardType);
+      printf(" keyboardType=%08x", (unsigned)keyboardType);
 
       EventHotKeyID evtHotKeyId = {0,0};
       ::GetEventParameter (evtRef, typeEventHotKeyID, typeEventHotKeyID, NULL,
                            sizeof (evtHotKeyId), NULL, &evtHotKeyId);
-      printf(" evtHotKeyId={signature=%08x, .id=%08x}", evtHotKeyId.signature, evtHotKeyId.id);
+      printf(" evtHotKeyId={signature=%08x, .id=%08x}", (unsigned)evtHotKeyId.signature, (unsigned)evtHotKeyId.id);
       printf("\n");
   }
   else
