@@ -1,11 +1,11 @@
-/* $Id: vboxhgcm.c $ */
+/* $Id: vboxhgcm.c 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  * VBox HGCM connection
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,10 +14,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifdef RT_OS_WINDOWS
@@ -40,7 +36,11 @@
 #include "cr_threads.h"
 #include "net_internals.h"
 
-#include <VBox/VBoxGuest.h>
+#if 1 /** @todo Try use the Vbgl interface instead of talking directly to the driver? */
+# include <VBox/VBoxGuest.h>
+#else
+# include <VBox/VBoxGuestLib.h>
+#endif
 #include <VBox/HostServices/VBoxCrOpenGLSvc.h>
 
 typedef struct {
@@ -146,12 +146,12 @@ static bool _crVBoxHGCMWriteBytes(CRConnection *conn, const void *buf, uint32_t
  * @param   pvData      Data pointer
  * @param   cbData      Data size
  */
-/*@todo use vbglR3DoIOCtl here instead */
+/** @todo use vbglR3DoIOCtl here instead */
 static int crVBoxHGCMCall(void *pvData, unsigned cbData)
 {
 #ifdef IN_GUEST
 
-#ifdef RT_OS_WINDOWS
+# ifdef RT_OS_WINDOWS
     DWORD cbReturned;
 
     if (DeviceIoControl (g_crvboxhgcm.hGuestDrv,
@@ -165,25 +165,36 @@ static int crVBoxHGCMCall(void *pvData, unsigned cbData)
     }
     crDebug("vboxCall failed with %x\n", GetLastError());
     return VERR_NOT_SUPPORTED;
-#else
-# ifdef RT_OS_SOLARIS
+# else
+    int rc;
+#  if defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
     VBGLBIGREQ Hdr;
     Hdr.u32Magic = VBGLBIGREQ_MAGIC;
     Hdr.cbData = cbData;
     Hdr.pvDataR3 = pvData;
-#  if HC_ARCH_BITS == 32
+#   if HC_ARCH_BITS == 32
     Hdr.u32Padding = 0;
+#   endif
+    rc = ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CALL(cbData), &Hdr);
+#  else
+    rc = ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CALL(cbData), pvData);
+#  endif
+#  ifdef RT_OS_LINUX
+    if (rc == 0)
+#  else
+    if (rc >= 0)
 #  endif
-    if (ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CALL(cbData), &Hdr) >= 0)
-# else
-    if (ioctl(g_crvboxhgcm.iGuestDrv, VBOXGUEST_IOCTL_HGCM_CALL(cbData), pvData) >= 0)
-# endif
     {
         return VINF_SUCCESS;
     }
-    crWarning("vboxCall failed with %x\n", errno);
+#  ifdef RT_OS_LINUX
+    if (rc >= 0) /* positive values are negated VBox error status codes. */
+        crWarning("vboxCall failed with VBox status code %d\n", -rc);
+    else
+#  endif
+        crWarning("vboxCall failed with %x\n", errno);
     return VERR_NOT_SUPPORTED;
-#endif /*#ifdef RT_OS_WINDOWS*/
+# endif /*#ifdef RT_OS_WINDOWS*/
 
 #else /*#ifdef IN_GUEST*/
     crError("crVBoxHGCMCall called on host side!");
@@ -322,14 +333,14 @@ static void crVBoxHGCMWriteExact(CRConnection *conn, const void *buf, unsigned i
     CRVBOXHGCMWRITE parms;
     int rc;
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_WRITE;
     parms.hdr.cParms      = SHCRGL_CPARMS_WRITE;
 
     parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_In;
     parms.pBuffer.u.Pointer.size         = len;
-    parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) buf;
+    parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) buf;
 
     rc = crVBoxHGCMCall(&parms, sizeof(parms));
 
@@ -344,7 +355,7 @@ static void crVBoxHGCMReadExact( CRConnection *conn, const void *buf, unsigned i
     CRVBOXHGCMREAD parms;
     int rc;
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_READ;
     parms.hdr.cParms      = SHCRGL_CPARMS_READ;
@@ -352,7 +363,7 @@ static void crVBoxHGCMReadExact( CRConnection *conn, const void *buf, unsigned i
     CRASSERT(!conn->pBuffer); //make sure there's no data to process
     parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_Out;
     parms.pBuffer.u.Pointer.size         = conn->cbHostBufferAllocated;
-    parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) conn->pHostBuffer;
+    parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) conn->pHostBuffer;
 
     parms.cbBuffer.type      = VMMDevHGCMParmType_32bit;
     parms.cbBuffer.u.value32 = 0;
@@ -387,7 +398,7 @@ crVBoxHGCMWriteReadExact(CRConnection *conn, const void *buf, unsigned int len,
     CRVBOXHGCMWRITEREAD parms;
     int rc;
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_WRITE_READ;
     parms.hdr.cParms      = SHCRGL_CPARMS_WRITE_READ;
@@ -396,19 +407,19 @@ crVBoxHGCMWriteReadExact(CRConnection *conn, const void *buf, unsigned int len,
     {
         parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_In;
         parms.pBuffer.u.Pointer.size         = len;
-        parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) buf;
+        parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) buf;
     }
-    /*else //@todo it fails badly, have to check why
+    /*else ///@todo it fails badly, have to check why. bird: This fails because buf isn't a physical address?
     {
         parms.pBuffer.type                 = VMMDevHGCMParmType_PhysAddr;
         parms.pBuffer.u.Pointer.size       = len;
-        parms.pBuffer.u.Pointer.u.physAddr = (VMMDEVHYPPHYS32) buf;
+        parms.pBuffer.u.Pointer.u.physAddr = (uintptr_t) buf;
     }*/
 
     CRASSERT(!conn->pBuffer); //make sure there's no data to process
     parms.pWriteback.type                   = VMMDevHGCMParmType_LinAddr_Out;
     parms.pWriteback.u.Pointer.size         = conn->cbHostBufferAllocated;
-    parms.pWriteback.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) conn->pHostBuffer;
+    parms.pWriteback.u.Pointer.u.linearAddr = (uintptr_t) conn->pHostBuffer;
 
     parms.cbWriteback.type      = VMMDevHGCMParmType_32bit;
     parms.cbWriteback.u.value32 = 0;
@@ -506,14 +517,14 @@ static void crVBoxHGCMPollHost(CRConnection *conn)
 
     CRASSERT(!conn->pBuffer);
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_READ;
     parms.hdr.cParms      = SHCRGL_CPARMS_READ;
 
     parms.pBuffer.type                   = VMMDevHGCMParmType_LinAddr_Out;
     parms.pBuffer.u.Pointer.size         = conn->cbHostBufferAllocated;
-    parms.pBuffer.u.Pointer.u.linearAddr = (VMMDEVHYPPTR) conn->pHostBuffer;
+    parms.pBuffer.u.Pointer.u.linearAddr = (uintptr_t) conn->pHostBuffer;
 
     parms.cbBuffer.type      = VMMDevHGCMParmType_32bit;
     parms.cbBuffer.u.value32 = 0;
@@ -671,7 +682,7 @@ static int crVBoxHGCMSetVersion(CRConnection *conn, unsigned int vMajor, unsigne
     CRVBOXHGCMSETVERSION parms;
     int rc;
 
-    parms.hdr.result      = VINF_SUCCESS;
+    parms.hdr.result      = VERR_WRONG_ORDER;
     parms.hdr.u32ClientID = conn->u32ClientID;
     parms.hdr.u32Function = SHCRGL_GUEST_FN_SET_VERSION;
     parms.hdr.cParms      = SHCRGL_CPARMS_SET_VERSION;
@@ -751,7 +762,7 @@ static int crVBoxHGCMDoConnect( CRConnection *conn )
                         &info, sizeof (info),
                         &cbReturned,
                         NULL))
-#elif defined(RT_OS_SOLARIS)
+#elif defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
     VBGLBIGREQ Hdr;
     Hdr.u32Magic = VBGLBIGREQ_MAGIC;
     Hdr.cbData = sizeof(info);
@@ -851,7 +862,7 @@ static void crVBoxHGCMDoDisconnect( CRConnection *conn )
         {
             crDebug("Disconnect failed with %x\n", GetLastError());
         }
-# elif defined(RT_OS_SOLARIS)
+# elif defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD)
         VBGLBIGREQ Hdr;
         Hdr.u32Magic = VBGLBIGREQ_MAGIC;
         Hdr.cbData = sizeof(info);
