@@ -1,10 +1,10 @@
-/* $Id: clipboard-helper.cpp $ */
+/* $Id: clipboard-helper.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * Shared Clipboard: Some helper function for converting between the various eol.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <iprt/assert.h>
@@ -52,9 +48,11 @@ int vboxClipboardUtf16GetWinSize(PRTUTF16 pwszSrc, size_t cwSrc, size_t *pcwDest
         /* Check for a single line feed */
         if (pwszSrc[i] == LINEFEED)
             ++cwDest;
+#ifdef RT_OS_DARWIN
         /* Check for a single carriage return (MacOS) */
         if (pwszSrc[i] == CARRIAGERETURN)
             ++cwDest;
+#endif
         if (pwszSrc[i] == 0)
         {
             /* Don't count this, as we do so below. */
@@ -116,6 +114,7 @@ int vboxClipboardUtf16LinToWin(PRTUTF16 pwszSrc, size_t cwSrc, PRTUTF16 pu16Dest
                 return VERR_BUFFER_OVERFLOW;
             }
         }
+#ifdef RT_OS_DARWIN
         /* Check for a single carriage return (MacOS) */
         else if (pwszSrc[i] == CARRIAGERETURN)
         {
@@ -131,6 +130,7 @@ int vboxClipboardUtf16LinToWin(PRTUTF16 pwszSrc, size_t cwSrc, PRTUTF16 pu16Dest
             pu16Dest[j] = LINEFEED;
             continue;
         }
+#endif
         pu16Dest[j] = pwszSrc[i];
     }
     /* Add the trailing null. */
