@@ -1,10 +1,10 @@
-/* $Revision: 49004 $ */
+/* $Revision: 28800 $ */
 /** @file
  * VirtualBox Support Driver - IOCtl definitions.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___SUPDrvIOC_h___
@@ -34,7 +30,7 @@
 /*
  * Basic types.
  */
-#include <iprt/stdint.h>
+#include <iprt/types.h>
 
 /*
  * IOCtl numbers.
@@ -42,9 +38,9 @@
  * The SUP_IOCTL_FLAG macro is used to separate requests from 32-bit
  * and 64-bit processes.
  */
-#ifdef RT_ARCH_AMD64
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_SPARC64)
 # define SUP_IOCTL_FLAG     128
-#elif defined(RT_ARCH_X86)
+#elif defined(RT_ARCH_X86) || defined(RT_ARCH_SPARC)
 # define SUP_IOCTL_FLAG     0
 #else
 # error "dunno which arch this is!"
@@ -183,17 +179,20 @@ typedef SUPREQHDR *PSUPREQHDR;
  * When incompatible changes are made, the upper major number has to be changed.
  *
  * Update rules:
- *  -# Only update the major number when incompatible changes has been made.
+ *  -# Only update the major number when incompatible changes have been made to
+ *     the IOC interface or the ABI provided via the functions returned by
+ *     SUPQUERYFUNCS.
  *  -# When adding new features (new IOC number, new flags, new exports, ++)
  *     only update the minor number and change SUPLib.cpp to require the
  *     new IOC version.
  *  -# When incrementing the major number, clear the minor part and reset
  *     any IOC version requirements in SUPLib.cpp.
+ *  -# When increment the major number, execute all pending work.
  *
  * @todo Pending work on next major version change:
  *          - Nothing.
  */
-#define SUPDRV_IOC_VERSION                              0x000e0000
+#define SUPDRV_IOC_VERSION                              0x00140001
 
 /** SUP_IOCTL_COOKIE. */
 typedef struct SUPCOOKIE
@@ -285,12 +284,17 @@ typedef struct SUPLDROPEN
     {
         struct
         {
-            /** Size of the image we'll be loading. */
-            uint32_t        cbImage;
+            /** Size of the image we'll be loading (includeing tables). */
+            uint32_t        cbImageWithTabs;
+            /** The size of the image bits. (Less or equal to cbImageWithTabs.) */
+            uint32_t        cbImageBits;
             /** Image name.
              * This is the NAME of the image, not the file name. It is used
              * to share code with other processes. (Max len is 32 chars!)  */
             char            szName[32];
+            /** Image file name.
+             * This can be used to load the image using a native loader. */
+            char            szFilename[196];
         } In;
         struct
         {
@@ -298,6 +302,8 @@ typedef struct SUPLDROPEN
             RTR0PTR         pvImageBase;
             /** Indicate whether or not the image requires loading. */
             bool            fNeedsLoading;
+            /** Indicates that we're using the native ring-0 loader. */
+            bool            fNativeLoader;
         } Out;
     } u;
 } SUPLDROPEN, *PSUPLDROPEN;
@@ -309,8 +315,8 @@ typedef struct SUPLDROPEN
  * @{
  */
 #define SUP_IOCTL_LDR_LOAD                              SUP_CTL_CODE_BIG(4)
-#define SUP_IOCTL_LDR_LOAD_SIZE(cbImage)                RT_UOFFSETOF(SUPLDRLOAD, u.In.achImage[cbImage])
-#define SUP_IOCTL_LDR_LOAD_SIZE_IN(cbImage)             RT_UOFFSETOF(SUPLDRLOAD, u.In.achImage[cbImage])
+#define SUP_IOCTL_LDR_LOAD_SIZE(cbImage)                RT_UOFFSETOF(SUPLDRLOAD, u.In.abImage[cbImage])
+#define SUP_IOCTL_LDR_LOAD_SIZE_IN(cbImage)             RT_UOFFSETOF(SUPLDRLOAD, u.In.abImage[cbImage])
 #define SUP_IOCTL_LDR_LOAD_SIZE_OUT                     sizeof(SUPREQHDR)
 
 /**
@@ -399,6 +405,9 @@ typedef struct SUPLDRLOAD
             RTR0PTR         pvImageBase;
             /** Entry point type. */
             SUPLDRLOADEP    eEPType;
+            /** The size of the image bits (starting at offset 0 and
+             * approaching offSymbols). */
+            uint32_t        cbImageBits;
             /** The offset of the symbol table. */
             uint32_t        offSymbols;
             /** The number of entries in the symbol table. */
@@ -407,10 +416,10 @@ typedef struct SUPLDRLOAD
             uint32_t        offStrTab;
             /** Size of the string table. */
             uint32_t        cbStrTab;
-            /** Size of image (including string and symbol tables). */
-            uint32_t        cbImage;
+            /** Size of image data in achImage. */
+            uint32_t        cbImageWithTabs;
             /** The image data. */
-            char            achImage[1];
+            uint8_t         abImage[1];
         } In;
     } u;
 } SUPLDRLOAD, *PSUPLDRLOAD;
@@ -1076,6 +1085,28 @@ typedef struct SUPSEMOP
 
 /** @} */
 
+/** @name SUP_IOCTL_VT_CAPS Input.
+ * @{
+ */
+/** Free contious memory. */
+#define SUP_IOCTL_VT_CAPS                               SUP_CTL_CODE_SIZE(26, SUP_IOCTL_VT_CAPS_SIZE)
+#define SUP_IOCTL_VT_CAPS_SIZE                          sizeof(SUPVTCAPS)
+#define SUP_IOCTL_VT_CAPS_SIZE_IN                       sizeof(SUPREQHDR)
+#define SUP_IOCTL_VT_CAPS_SIZE_OUT                      sizeof(SUPVTCAPS)
+typedef struct SUPVTCAPS
+{
+    /** The header. */
+    SUPREQHDR               Hdr;
+    union
+    {
+        struct
+        {
+            /** The VT capability dword. */
+            uint32_t        Caps;
+        } Out;
+    } u;
+} SUPVTCAPS, *PSUPVTCAPS;
+/** @} */
 
 #pragma pack()                          /* paranoia */
 
