@@ -1,10 +1,10 @@
-/* $Id: SUPLib.cpp $ */
+/* $Id: SUPLib.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VirtualBox Support Library - Common code.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /** @page   pg_sup          SUP - The Support Library
@@ -271,8 +267,8 @@ SUPR3DECL(int) SUPR3Init(PSUPDRVSESSION *ppSession)
         CookieReq.Hdr.rc = VERR_INTERNAL_ERROR;
         strcpy(CookieReq.u.In.szMagic, SUPCOOKIE_MAGIC);
         CookieReq.u.In.u32ReqVersion = SUPDRV_IOC_VERSION;
-        const uint32_t uMinVersion = (SUPDRV_IOC_VERSION & 0xffff0000) == 0x00110000
-                                   ?  0x00110000
+        const uint32_t uMinVersion = (SUPDRV_IOC_VERSION & 0xffff0000) == 0x00140000
+                                   ?  0x00140001
                                    :  SUPDRV_IOC_VERSION & 0xffff0000;
         CookieReq.u.In.u32MinVersion = uMinVersion;
         rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_COOKIE, &CookieReq, SUP_IOCTL_COOKIE_SIZE);
@@ -412,62 +408,61 @@ static int supInitFake(PSUPDRVSESSION *ppSession)
         { "SUPR0MemFree",                           0xefef000a },
         { "SUPR0Printf",                            0xefef000b },
         { "SUPR0GetPagingMode",                     0xefef000c },
-        { "SUPR0EnableVTx",                         0xefef000c },
-        { "RTMemAlloc",                             0xefef000d },
-        { "RTMemAllocZ",                            0xefef000e },
-        { "RTMemFree",                              0xefef000f },
-        { "RTR0MemObjAddress",                      0xefef0010 },
-        { "RTR0MemObjAddressR3",                    0xefef0011 },
-        { "RTR0MemObjAllocPage",                    0xefef0012 },
-        { "RTR0MemObjAllocPhysNC",                  0xefef0013 },
-        { "RTR0MemObjAllocLow",                     0xefef0014 },
-        { "RTR0MemObjEnterPhys",                    0xefef0014 },
-        { "RTR0MemObjFree",                         0xefef0015 },
-        { "RTR0MemObjGetPagePhysAddr",              0xefef0016 },
-        { "RTR0MemObjMapUser",                      0xefef0017 },
-        { "RTR0MemObjMapKernel",                    0xefef0017 },
-        { "RTR0MemObjMapKernelEx",                  0xefef0017 },
-        { "RTProcSelf",                             0xefef0038 },
-        { "RTR0ProcHandleSelf",                     0xefef0039 },
-        { "RTSemEventCreate",                       0xefef0018 },
-        { "RTSemEventSignal",                       0xefef0019 },
-        { "RTSemEventWait",                         0xefef001a },
-        { "RTSemEventWaitNoResume",                 0xefef001b },
-        { "RTSemEventDestroy",                      0xefef001c },
-        { "RTSemEventMultiCreate",                  0xefef001d },
-        { "RTSemEventMultiSignal",                  0xefef001e },
-        { "RTSemEventMultiReset",                   0xefef001f },
-        { "RTSemEventMultiWait",                    0xefef0020 },
-        { "RTSemEventMultiWaitNoResume",            0xefef0021 },
-        { "RTSemEventMultiDestroy",                 0xefef0022 },
-        { "RTSemFastMutexCreate",                   0xefef0023 },
-        { "RTSemFastMutexDestroy",                  0xefef0024 },
-        { "RTSemFastMutexRequest",                  0xefef0025 },
-        { "RTSemFastMutexRelease",                  0xefef0026 },
-        { "RTSpinlockCreate",                       0xefef0027 },
-        { "RTSpinlockDestroy",                      0xefef0028 },
-        { "RTSpinlockAcquire",                      0xefef0029 },
-        { "RTSpinlockRelease",                      0xefef002a },
-        { "RTSpinlockAcquireNoInts",                0xefef002b },
-        { "RTSpinlockReleaseNoInts",                0xefef002c },
-        { "RTTimeNanoTS",                           0xefef002d },
-        { "RTTimeMillieTS",                         0xefef002e },
-        { "RTTimeSystemNanoTS",                     0xefef002f },
-        { "RTTimeSystemMillieTS",                   0xefef0030 },
-        { "RTThreadNativeSelf",                     0xefef0031 },
-        { "RTThreadSleep",                          0xefef0032 },
-        { "RTThreadYield",                          0xefef0033 },
-        { "RTLogDefaultInstance",                   0xefef0034 },
-        { "RTLogRelDefaultInstance",                0xefef0035 },
-        { "RTLogSetDefaultInstanceThread",          0xefef0036 },
-        { "RTLogLogger",                            0xefef0037 },
-        { "RTLogLoggerEx",                          0xefef0038 },
-        { "RTLogLoggerExV",                         0xefef0039 },
-        { "AssertMsg1",                             0xefef003a },
-        { "AssertMsg2",                             0xefef003b },
-        { "RTAssertMsg1",                           0xefef003c },
-        { "RTAssertMsg2",                           0xefef003d },
-        { "RTAssertMsg2V",                          0xefef003e },
+        { "SUPR0EnableVTx",                         0xefef000e },
+        { "RTMemAlloc",                             0xefef000f },
+        { "RTMemAllocZ",                            0xefef0010 },
+        { "RTMemFree",                              0xefef0011 },
+        { "RTR0MemObjAddress",                      0xefef0012 },
+        { "RTR0MemObjAddressR3",                    0xefef0013 },
+        { "RTR0MemObjAllocPage",                    0xefef0014 },
+        { "RTR0MemObjAllocPhysNC",                  0xefef0015 },
+        { "RTR0MemObjAllocLow",                     0xefef0016 },
+        { "RTR0MemObjEnterPhys",                    0xefef0017 },
+        { "RTR0MemObjFree",                         0xefef0018 },
+        { "RTR0MemObjGetPagePhysAddr",              0xefef0019 },
+        { "RTR0MemObjMapUser",                      0xefef001a },
+        { "RTR0MemObjMapKernel",                    0xefef001b },
+        { "RTR0MemObjMapKernelEx",                  0xefef001c },
+        { "RTProcSelf",                             0xefef001d },
+        { "RTR0ProcHandleSelf",                     0xefef001e },
+        { "RTSemEventCreate",                       0xefef001f },
+        { "RTSemEventSignal",                       0xefef0020 },
+        { "RTSemEventWait",                         0xefef0021 },
+        { "RTSemEventWaitNoResume",                 0xefef0022 },
+        { "RTSemEventDestroy",                      0xefef0023 },
+        { "RTSemEventMultiCreate",                  0xefef0024 },
+        { "RTSemEventMultiSignal",                  0xefef0025 },
+        { "RTSemEventMultiReset",                   0xefef0026 },
+        { "RTSemEventMultiWait",                    0xefef0027 },
+        { "RTSemEventMultiWaitNoResume",            0xefef0028 },
+        { "RTSemEventMultiDestroy",                 0xefef0029 },
+        { "RTSemFastMutexCreate",                   0xefef002a },
+        { "RTSemFastMutexDestroy",                  0xefef002b },
+        { "RTSemFastMutexRequest",                  0xefef002c },
+        { "RTSemFastMutexRelease",                  0xefef002d },
+        { "RTSpinlockCreate",                       0xefef002e },
+        { "RTSpinlockDestroy",                      0xefef002f },
+        { "RTSpinlockAcquire",                      0xefef0030 },
+        { "RTSpinlockRelease",                      0xefef0031 },
+        { "RTSpinlockAcquireNoInts",                0xefef0032 },
+        { "RTSpinlockReleaseNoInts",                0xefef0033 },
+        { "RTTimeNanoTS",                           0xefef0034 },
+        { "RTTimeMillieTS",                         0xefef0035 },
+        { "RTTimeSystemNanoTS",                     0xefef0036 },
+        { "RTTimeSystemMillieTS",                   0xefef0037 },
+        { "RTThreadNativeSelf",                     0xefef0038 },
+        { "RTThreadSleep",                          0xefef0039 },
+        { "RTThreadYield",                          0xefef003a },
+        { "RTLogDefaultInstance",                   0xefef003b },
+        { "RTLogRelDefaultInstance",                0xefef003c },
+        { "RTLogSetDefaultInstanceThread",          0xefef003d },
+        { "RTLogLogger",                            0xefef003e },
+        { "RTLogLoggerEx",                          0xefef003f },
+        { "RTLogLoggerExV",                         0xefef0040 },
+        { "RTAssertMsg1",                           0xefef0041 },
+        { "RTAssertMsg2",                           0xefef0042 },
+        { "RTAssertMsg2V",                          0xefef0043 },
+        { "SUPR0QueryVTCaps",                       0xefef0044 },
     };
 
     /* fake r0 functions. */
@@ -1184,12 +1179,12 @@ SUPR3DECL(int) SUPR3PageFreeEx(void *pvPages, size_t cPages)
     /* fake */
     if (RT_UNLIKELY(g_u32FakeMode))
     {
-        RTMemPageFree(pvPages);
+        RTMemPageFree(pvPages, cPages * PAGE_SIZE);
         return VINF_SUCCESS;
     }
 
     /*
-     * Try normal free first, then if it fails check if we're using the fallback                                            .
+     * Try normal free first, then if it fails check if we're using the fallback
      * for the allocations without kernel mappings and attempt unlocking it.
      */
     NOREF(cPages);
@@ -1282,7 +1277,7 @@ SUPR3DECL(int) SUPR3ContFree(void *pv, size_t cPages)
     /* fake */
     if (RT_UNLIKELY(g_u32FakeMode))
     {
-        RTMemPageFree(pv);
+        RTMemPageFree(pv, cPages * PAGE_SIZE);
         return VINF_SUCCESS;
     }
 
@@ -1385,7 +1380,7 @@ SUPR3DECL(int) SUPR3LowFree(void *pv, size_t cPages)
     /* fake */
     if (RT_UNLIKELY(g_u32FakeMode))
     {
-        RTMemPageFree(pv);
+        RTMemPageFree(pv, cPages * PAGE_SIZE);
         return VINF_SUCCESS;
     }
 
@@ -1490,7 +1485,7 @@ static DECLCALLBACK(int) supLoadModuleResolveImport(RTLDRMOD hLdrMod, const char
      */
     if (    pszModule
         &&  *pszModule
-        &&  strcmp(pszModule, "SUPR0.dll")
+        &&  strcmp(pszModule, "VBoxDrv.sys")
         &&  strcmp(pszModule, "VMMR0.r0"))
     {
         AssertMsgFailed(("%s is importing from %s! (expected 'SUPR0.dll' or 'VMMR0.r0', case-sensitiv)\n", pvUser, pszModule));
@@ -1509,6 +1504,7 @@ static DECLCALLBACK(int) supLoadModuleResolveImport(RTLDRMOD hLdrMod, const char
     /*
      * Lookup symbol.
      */
+    /** @todo is this actually used??? */
     /* skip the 64-bit ELF import prefix first. */
     if (!strncmp(pszSymbol, "SUPR0$", sizeof("SUPR0$") - 1))
         pszSymbol += sizeof("SUPR0$") - 1;
@@ -1544,11 +1540,11 @@ static DECLCALLBACK(int) supLoadModuleResolveImport(RTLDRMOD hLdrMod, const char
     /*
      * The GIP.
      */
-    /** @todo R0 mapping? */
     if (    pszSymbol
         &&  g_pSUPGlobalInfoPage
         &&  g_pSUPGlobalInfoPageR0
-        &&  !strcmp(pszSymbol, "g_SUPGlobalInfoPage"))
+        &&  !strcmp(pszSymbol, "g_SUPGlobalInfoPage")
+       )
     {
         *pValue = (uintptr_t)g_pSUPGlobalInfoPageR0;
         return VINF_SUCCESS;
@@ -1561,11 +1557,11 @@ static DECLCALLBACK(int) supLoadModuleResolveImport(RTLDRMOD hLdrMod, const char
     pFunc = &g_pFunctions->u.Out.aFunctions[0];
     while (c-- > 0)
     {
-        AssertMsg2("%d: %s\n", g_pFunctions->u.Out.cFunctions - c, pFunc->szName);
+        RTAssertMsg2Weak("%d: %s\n", g_pFunctions->u.Out.cFunctions - c, pFunc->szName);
         pFunc++;
     }
 
-    AssertMsg2("%s is importing %s which we couldn't find\n", pvUser, pszSymbol);
+    RTAssertMsg2Weak("%s is importing %s which we couldn't find\n", pvUser, pszSymbol);
     AssertMsgFailed(("%s is importing %s which we couldn't find\n", pvUser, pszSymbol));
     if (g_u32FakeMode)
     {
@@ -1642,6 +1638,8 @@ static DECLCALLBACK(int) supLoadModuleCreateTabsCB(RTLDRMOD hLdrMod, const char
  */
 static int supLoadModule(const char *pszFilename, const char *pszModule, const char *pszSrvReqHandler, void **ppvImageBase)
 {
+    int rc;
+
     /*
      * Validate input.
      */
@@ -1649,6 +1647,11 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
     AssertPtrReturn(pszModule, VERR_INVALID_PARAMETER);
     AssertPtrReturn(ppvImageBase, VERR_INVALID_PARAMETER);
     AssertReturn(strlen(pszModule) < RT_SIZEOFMEMB(SUPLDROPEN, u.In.szName), VERR_FILENAME_TOO_LONG);
+    char szAbsFilename[RT_SIZEOFMEMB(SUPLDROPEN, u.In.szFilename)];
+    rc = RTPathAbs(pszFilename, szAbsFilename, sizeof(szAbsFilename));
+    if (RT_FAILURE(rc))
+        return rc;
+    pszFilename = szAbsFilename;
 
     const bool fIsVMMR0 = !strcmp(pszModule, "VMMR0.r0");
     AssertReturn(!pszSrvReqHandler || !fIsVMMR0, VERR_INTERNAL_ERROR);
@@ -1658,7 +1661,7 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
      * Open image file and figure its size.
      */
     RTLDRMOD hLdrMod;
-    int rc = RTLdrOpen(pszFilename, 0, RTLDRARCH_HOST, &hLdrMod);
+    rc = RTLdrOpen(pszFilename, 0, RTLDRARCH_HOST, &hLdrMod);
     if (!RT_SUCCESS(rc))
         return rc;
 
@@ -1671,7 +1674,7 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
     {
         const uint32_t  offSymTab = RT_ALIGN_32(CalcArgs.cbImage, 8);
         const uint32_t  offStrTab = offSymTab + CalcArgs.cSymbols * sizeof(SUPLDRSYM);
-        const uint32_t  cbImage   = RT_ALIGN_32(offStrTab + CalcArgs.cbStrings, 8);
+        const uint32_t  cbImageWithTabs = RT_ALIGN_32(offStrTab + CalcArgs.cbStrings, 8);
 
         /*
          * Open the R0 image.
@@ -1683,8 +1686,10 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
         OpenReq.Hdr.cbOut = SUP_IOCTL_LDR_OPEN_SIZE_OUT;
         OpenReq.Hdr.fFlags = SUPREQHDR_FLAGS_DEFAULT;
         OpenReq.Hdr.rc = VERR_INTERNAL_ERROR;
-        OpenReq.u.In.cbImage = cbImage;
+        OpenReq.u.In.cbImageWithTabs = cbImageWithTabs;
+        OpenReq.u.In.cbImageBits = (uint32_t)CalcArgs.cbImage;
         strcpy(OpenReq.u.In.szName, pszModule);
+        strcpy(OpenReq.u.In.szFilename, pszFilename);
         if (!g_u32FakeMode)
         {
             rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_LDR_OPEN, &OpenReq, SUP_IOCTL_LDR_OPEN_SIZE);
@@ -1704,13 +1709,13 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
              * We need to load it.
              * Allocate memory for the image bits.
              */
-            PSUPLDRLOAD pLoadReq = (PSUPLDRLOAD)RTMemTmpAlloc(SUP_IOCTL_LDR_LOAD_SIZE(cbImage));
+            PSUPLDRLOAD pLoadReq = (PSUPLDRLOAD)RTMemTmpAlloc(SUP_IOCTL_LDR_LOAD_SIZE(cbImageWithTabs));
             if (pLoadReq)
             {
                 /*
                  * Get the image bits.
                  */
-                rc = RTLdrGetBits(hLdrMod, &pLoadReq->u.In.achImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase,
+                rc = RTLdrGetBits(hLdrMod, &pLoadReq->u.In.abImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase,
                                   supLoadModuleResolveImport, (void *)pszModule);
 
                 if (RT_SUCCESS(rc))
@@ -1726,21 +1731,21 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
                     RTUINTPTR ModuleTerm = 0;
                     if (fIsVMMR0)
                     {
-                        rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.achImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "VMMR0EntryInt", &VMMR0EntryInt);
+                        rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.abImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "VMMR0EntryInt", &VMMR0EntryInt);
                         if (RT_SUCCESS(rc))
-                            rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.achImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "VMMR0EntryFast", &VMMR0EntryFast);
+                            rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.abImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "VMMR0EntryFast", &VMMR0EntryFast);
                         if (RT_SUCCESS(rc))
-                            rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.achImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "VMMR0EntryEx", &VMMR0EntryEx);
+                            rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.abImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "VMMR0EntryEx", &VMMR0EntryEx);
                     }
                     else if (pszSrvReqHandler)
-                        rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.achImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, pszSrvReqHandler, &SrvReqHandler);
+                        rc = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.abImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, pszSrvReqHandler, &SrvReqHandler);
                     if (RT_SUCCESS(rc))
                     {
-                        int rc2 = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.achImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "ModuleInit", &ModuleInit);
+                        int rc2 = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.abImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "ModuleInit", &ModuleInit);
                         if (RT_FAILURE(rc2))
                             ModuleInit = 0;
 
-                        rc2 = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.achImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "ModuleTerm", &ModuleTerm);
+                        rc2 = RTLdrGetSymbolEx(hLdrMod, &pLoadReq->u.In.abImage[0], (uintptr_t)OpenReq.u.Out.pvImageBase, "ModuleTerm", &ModuleTerm);
                         if (RT_FAILURE(rc2))
                             ModuleTerm = 0;
                     }
@@ -1751,21 +1756,21 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
                          */
                         SUPLDRCREATETABSARGS CreateArgs;
                         CreateArgs.cbImage = CalcArgs.cbImage;
-                        CreateArgs.pSym    = (PSUPLDRSYM)&pLoadReq->u.In.achImage[offSymTab];
-                        CreateArgs.pszBase =     (char *)&pLoadReq->u.In.achImage[offStrTab];
+                        CreateArgs.pSym    = (PSUPLDRSYM)&pLoadReq->u.In.abImage[offSymTab];
+                        CreateArgs.pszBase =     (char *)&pLoadReq->u.In.abImage[offStrTab];
                         CreateArgs.psz     = CreateArgs.pszBase;
                         rc = RTLdrEnumSymbols(hLdrMod, 0, NULL, 0, supLoadModuleCreateTabsCB, &CreateArgs);
                         if (RT_SUCCESS(rc))
                         {
                             AssertRelease((size_t)(CreateArgs.psz - CreateArgs.pszBase) <= CalcArgs.cbStrings);
-                            AssertRelease((size_t)(CreateArgs.pSym - (PSUPLDRSYM)&pLoadReq->u.In.achImage[offSymTab]) <= CalcArgs.cSymbols);
+                            AssertRelease((size_t)(CreateArgs.pSym - (PSUPLDRSYM)&pLoadReq->u.In.abImage[offSymTab]) <= CalcArgs.cSymbols);
 
                             /*
                              * Upload the image.
                              */
                             pLoadReq->Hdr.u32Cookie = g_u32Cookie;
                             pLoadReq->Hdr.u32SessionCookie = g_u32SessionCookie;
-                            pLoadReq->Hdr.cbIn = SUP_IOCTL_LDR_LOAD_SIZE_IN(cbImage);
+                            pLoadReq->Hdr.cbIn = SUP_IOCTL_LDR_LOAD_SIZE_IN(cbImageWithTabs);
                             pLoadReq->Hdr.cbOut = SUP_IOCTL_LDR_LOAD_SIZE_OUT;
                             pLoadReq->Hdr.fFlags = SUPREQHDR_FLAGS_MAGIC | SUPREQHDR_FLAGS_EXTRA_IN;
                             pLoadReq->Hdr.rc = VERR_INTERNAL_ERROR;
@@ -1793,13 +1798,14 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
                             pLoadReq->u.In.offStrTab                  = offStrTab;
                             pLoadReq->u.In.cbStrTab                   = (uint32_t)CalcArgs.cbStrings;
                             AssertRelease(pLoadReq->u.In.cbStrTab == CalcArgs.cbStrings);
+                            pLoadReq->u.In.cbImageBits                = (uint32_t)CalcArgs.cbImage;
                             pLoadReq->u.In.offSymbols                 = offSymTab;
                             pLoadReq->u.In.cSymbols                   = CalcArgs.cSymbols;
-                            pLoadReq->u.In.cbImage                    = cbImage;
+                            pLoadReq->u.In.cbImageWithTabs            = cbImageWithTabs;
                             pLoadReq->u.In.pvImageBase                = OpenReq.u.Out.pvImageBase;
                             if (!g_u32FakeMode)
                             {
-                                rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_LDR_LOAD, pLoadReq, SUP_IOCTL_LDR_LOAD_SIZE(cbImage));
+                                rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_LDR_LOAD, pLoadReq, SUP_IOCTL_LDR_LOAD_SIZE(cbImageWithTabs));
                                 if (RT_SUCCESS(rc))
                                     rc = pLoadReq->Hdr.rc;
                             }
@@ -1809,8 +1815,9 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
                                 ||  rc == VERR_ALREADY_LOADED /* A competing process. */
                                )
                             {
-                                LogRel(("SUP: Loaded %s (%s) at %#p - ModuleInit at %RTptr and ModuleTerm at %RTptr\n", pszModule, pszFilename,
-                                        OpenReq.u.Out.pvImageBase, ModuleInit, ModuleTerm));
+                                LogRel(("SUP: Loaded %s (%s) at %#p - ModuleInit at %RTptr and ModuleTerm at %RTptr%s\n",
+                                        pszModule, pszFilename, OpenReq.u.Out.pvImageBase, ModuleInit, ModuleTerm,
+                                        OpenReq.u.Out.fNativeLoader ? " using the native ring-0 loader" : ""));
                                 if (fIsVMMR0)
                                 {
                                     g_pvVMMR0 = OpenReq.u.Out.pvImageBase;
@@ -1832,7 +1839,7 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
             }
             else
             {
-                AssertMsgFailed(("failed to allocated %d bytes for SUPLDRLOAD_IN structure!\n", SUP_IOCTL_LDR_LOAD_SIZE(cbImage)));
+                AssertMsgFailed(("failed to allocated %u bytes for SUPLDRLOAD_IN structure!\n", SUP_IOCTL_LDR_LOAD_SIZE(cbImageWithTabs)));
                 rc = VERR_NO_TMP_MEMORY;
             }
         }
@@ -1840,7 +1847,8 @@ static int supLoadModule(const char *pszFilename, const char *pszModule, const c
         {
             if (fIsVMMR0)
                 g_pvVMMR0 = OpenReq.u.Out.pvImageBase;
-            LogRel(("SUP: Opened %s (%s) at %#p.\n", pszModule, pszFilename, OpenReq.u.Out.pvImageBase));
+            LogRel(("SUP: Opened %s (%s) at %#p.\n", pszModule, pszFilename, OpenReq.u.Out.pvImageBase,
+                    OpenReq.u.Out.fNativeLoader ? " loaded by the native ring-0 loader" : ""));
 #ifdef RT_OS_WINDOWS
             LogRel(("SUP: windbg> .reload /f %s=%#p\n", pszFilename, OpenReq.u.Out.pvImageBase));
 #endif
@@ -2064,3 +2072,36 @@ SUPR3DECL(int) SUPR3QueryVTxSupported(void)
     return VINF_SUCCESS;
 #endif
 }
+
+
+SUPR3DECL(int) SUPR3QueryVTCaps(uint32_t *pfCaps)
+{
+    AssertPtrReturn(pfCaps, VERR_INVALID_POINTER);
+
+    *pfCaps = 0;
+
+    /* fake */
+    if (RT_UNLIKELY(g_u32FakeMode))
+        return VINF_SUCCESS;
+
+    /*
+     * Issue IOCtl to the SUPDRV kernel module.
+     */
+    SUPVTCAPS Req;
+    Req.Hdr.u32Cookie = g_u32Cookie;
+    Req.Hdr.u32SessionCookie = g_u32SessionCookie;
+    Req.Hdr.cbIn = SUP_IOCTL_VT_CAPS_SIZE_IN;
+    Req.Hdr.cbOut = SUP_IOCTL_VT_CAPS_SIZE_OUT;
+    Req.Hdr.fFlags = SUPREQHDR_FLAGS_DEFAULT;
+    Req.Hdr.rc = VERR_INTERNAL_ERROR;
+    Req.u.Out.Caps = 0;
+    int rc = suplibOsIOCtl(&g_supLibData, SUP_IOCTL_VT_CAPS, &Req, SUP_IOCTL_VT_CAPS_SIZE);
+    if (RT_SUCCESS(rc))
+    {
+        rc = Req.Hdr.rc;
+        if (RT_SUCCESS(rc))
+            *pfCaps = Req.u.Out.Caps;
+    }
+    return rc;
+}
+
