@@ -1,10 +1,10 @@
-/* $Id: SUPLibInternal.h $ */
+/* $Id: SUPLibInternal.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VirtualBox Support Library - Internal header.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___SUPLibInternal_h___
@@ -51,6 +47,12 @@
 # define SUPLIB_DLL_SUFF    ".so"
 #endif
 
+#ifdef RT_OS_SOLARIS
+/** Number of dummy files to open (2:ip4, 1:ip6, 1:extra) see
+ *  @bugref{4650}. */
+# define SUPLIB_FLT_DUMMYFILES 4
+#endif
+
 /** @def SUPLIB_EXE_SUFF
  * The (typical) executable suffix. */
 #if defined(RT_OS_OS2) || defined(RT_OS_WINDOWS)
@@ -78,7 +80,7 @@
 # define supR3HardenedPathAppPrivateArch   supR3HardenedStaticPathAppPrivateArch
 # define supR3HardenedPathSharedLibs       supR3HardenedStaticPathSharedLibs
 # define supR3HardenedPathAppDocs          supR3HardenedStaticPathAppDocs
-# define supR3HardenedPathProgram          supR3HardenedStaticPathProgram
+# define supR3HardenedPathExecDir          supR3HardenedStaticPathExecDir
 # define supR3HardenedPathFilename         supR3HardenedStaticPathFilename
 # define supR3HardenedFatalV               supR3HardenedStaticFatalV
 # define supR3HardenedFatal                supR3HardenedStaticFatal
@@ -95,17 +97,6 @@
 #endif /* IN_SUP_HARDENED_R3 */
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
-/** The negotiated interrupt number. */
-extern DECLHIDDEN(uint8_t)        g_uchInterruptNo;
-/** The negotiated cookie. */
-extern DECLHIDDEN(uint32_t)       g_u32Cookie;
-/** The negotiated cookie. */
-extern DECLHIDDEN(uint32_t)       g_u32CookieSession;
-
-
 /*******************************************************************************
 *   Structures and Typedefs                                                    *
 *******************************************************************************/
@@ -198,6 +189,10 @@ typedef struct SUPLIBDATA
 #elif defined(RT_OS_LINUX)
     /** Indicates whether madvise(,,MADV_DONTFORK) works. */
     bool                fSysMadviseWorks;
+#elif defined(RT_OS_SOLARIS)
+    /** Extra dummy file descriptors to prevent growing file-descriptor table on
+     *  clean up (see @bugref{4650}). */
+    int                 ahDummy[SUPLIB_FLT_DUMMYFILES];
 #elif defined(RT_OS_WINDOWS)
 #endif
 } SUPLIBDATA;
@@ -241,10 +236,18 @@ typedef DECLCALLBACK(int) FNSUPR3PREINIT(PSUPPREINITDATA pPreInitData, uint32_t
 typedef FNSUPR3PREINIT *PFNSUPR3PREINIT;
 
 
+/*******************************************************************************
+*   Global Variables                                                           *
+*******************************************************************************/
+extern DECLHIDDEN(uint32_t)     g_u32Cookie;
+extern DECLHIDDEN(uint32_t)     g_u32SessionCookie;
+extern DECLHIDDEN(SUPLIBDATA)   g_supLibData;
+
+
 /*******************************************************************************
 *   OS Specific Function                                                       *
 *******************************************************************************/
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 int     suplibOsInstall(void);
 int     suplibOsUninstall(void);
 int     suplibOsInit(PSUPLIBDATA pThis, bool fPreInited);
@@ -253,6 +256,7 @@ int     suplibOsIOCtl(PSUPLIBDATA pThis, uintptr_t uFunction, void *pvReq, size_
 int     suplibOsIOCtlFast(PSUPLIBDATA pThis, uintptr_t uFunction, uintptr_t idCpu);
 int     suplibOsPageAlloc(PSUPLIBDATA pThis, size_t cPages, void **ppvPages);
 int     suplibOsPageFree(PSUPLIBDATA pThis, void *pvPages, size_t cPages);
+int     suplibOsQueryVTxSupported(void);
 
 
 /**
@@ -276,8 +280,8 @@ DECLHIDDEN(int)    supR3HardenedPathAppPrivateArch(char *pszPath, size_t cchPath
 DECLHIDDEN(int)    supR3HardenedPathSharedLibs(char *pszPath, size_t cchPath);
 /** @copydoc RTPathAppDocs */
 DECLHIDDEN(int)    supR3HardenedPathAppDocs(char *pszPath, size_t cchPath);
-/** @copydoc RTPathProgram */
-DECLHIDDEN(int)    supR3HardenedPathProgram(char *pszPath, size_t cchPath);
+/** @copydoc RTPathExecDir */
+DECLHIDDEN(int)    supR3HardenedPathExecDir(char *pszPath, size_t cchPath);
 /** @copydoc RTPathFilename */
 DECLHIDDEN(char *) supR3HardenedPathFilename(const char *pszPath);
 
@@ -317,7 +321,10 @@ DECLHIDDEN(void)   supR3HardenedGetPreInitData(PSUPPREINITDATA pPreInitData);
 DECLHIDDEN(int)    supR3HardenedRecvPreInitData(PCSUPPREINITDATA pPreInitData);
 
 
-__END_DECLS
+SUPR3DECL(int)      supR3PageLock(void *pvStart, size_t cPages, PSUPPAGE paPages);
+SUPR3DECL(int)      supR3PageUnlock(void *pvStart);
+
+RT_C_DECLS_END
 
 
 #endif
