@@ -1,10 +1,10 @@
-/* $Id: SUPR3HardenedMain.cpp $ */
+/* $Id: SUPR3HardenedMain.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VirtualBox Support Library - Hardened main().
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -66,6 +62,9 @@
 #  ifndef CAP_TO_MASK
 #   define CAP_TO_MASK(cap) RT_BIT(cap)
 #  endif
+# elif defined(RT_OS_FREEBSD)
+#  include <sys/param.h>
+#  include <sys/sysctl.h>
 # elif defined(RT_OS_SOLARIS)
 #  include <priv.h>
 # endif
@@ -137,6 +136,7 @@ static uint32_t g_uCaps;
 # endif
 #endif
 
+
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
@@ -234,7 +234,7 @@ DECLHIDDEN(int) supR3HardenedPathAppPrivateNoArch(char *pszPath, size_t cchPath)
     return VINF_SUCCESS;
 
 #else
-    return supR3HardenedPathProgram(pszPath, cchPath);
+    return supR3HardenedPathExecDir(pszPath, cchPath);
 #endif
 }
 
@@ -254,7 +254,7 @@ DECLHIDDEN(int) supR3HardenedPathAppPrivateArch(char *pszPath, size_t cchPath)
     return VINF_SUCCESS;
 
 #else
-    return supR3HardenedPathProgram(pszPath, cchPath);
+    return supR3HardenedPathExecDir(pszPath, cchPath);
 #endif
 }
 
@@ -274,7 +274,7 @@ DECLHIDDEN(int) supR3HardenedPathSharedLibs(char *pszPath, size_t cchPath)
     return VINF_SUCCESS;
 
 #else
-    return supR3HardenedPathProgram(pszPath, cchPath);
+    return supR3HardenedPathExecDir(pszPath, cchPath);
 #endif
 }
 
@@ -294,7 +294,7 @@ DECLHIDDEN(int) supR3HardenedPathAppDocs(char *pszPath, size_t cchPath)
     return VINF_SUCCESS;
 
 #else
-    return supR3HardenedPathProgram(pszPath, cchPath);
+    return supR3HardenedPathExecDir(pszPath, cchPath);
 #endif
 }
 
@@ -320,15 +320,28 @@ static void supR3HardenedGetFullExePath(void)
 #if defined(RT_OS_LINUX) || defined(RT_OS_FREEBSD) || defined(RT_OS_SOLARIS)
 # ifdef RT_OS_LINUX
     int cchLink = readlink("/proc/self/exe", &g_szSupLibHardenedExePath[0], sizeof(g_szSupLibHardenedExePath) - 1);
+
 # elif defined(RT_OS_SOLARIS)
     char szFileBuf[PATH_MAX + 1];
     sprintf(szFileBuf, "/proc/%ld/path/a.out", (long)getpid());
     int cchLink = readlink(szFileBuf, &g_szSupLibHardenedExePath[0], sizeof(g_szSupLibHardenedExePath) - 1);
-# else /* RT_OS_FREEBSD: */
-    int cchLink = readlink("/proc/curproc/file", &g_szSupLibHardenedExePath[0], sizeof(g_szSupLibHardenedExePath) - 1);
+
+# else /* RT_OS_FREEBSD */
+    int aiName[4];
+    aiName[0] = CTL_KERN;
+    aiName[1] = KERN_PROC;
+    aiName[2] = KERN_PROC_PATHNAME;
+    aiName[3] = getpid();
+
+    size_t cbPath = sizeof(g_szSupLibHardenedExePath);
+    if (sysctl(aiName, RT_ELEMENTS(aiName), g_szSupLibHardenedExePath, &cbPath, NULL, 0) < 0)
+        supR3HardenedFatal("supR3HardenedExecDir: sysctl failed\n");
+    g_szSupLibHardenedExePath[sizeof(g_szSupLibHardenedExePath) - 1] = '\0';
+    int cchLink = strlen(g_szSupLibHardenedExePath); /* paranoid? can't we use cbPath? */
+
 # endif
     if (cchLink < 0 || cchLink == sizeof(g_szSupLibHardenedExePath) - 1)
-        supR3HardenedFatal("supR3HardenedPathProgram: couldn't read \"%s\", errno=%d cchLink=%d\n",
+        supR3HardenedFatal("supR3HardenedExecDir: couldn't read \"%s\", errno=%d cchLink=%d\n",
                             g_szSupLibHardenedExePath, errno, cchLink);
     g_szSupLibHardenedExePath[cchLink] = '\0';
 
@@ -338,16 +351,16 @@ static void supR3HardenedGetFullExePath(void)
 #elif defined(RT_OS_DARWIN)
     const char *pszImageName = _dyld_get_image_name(0);
     if (!pszImageName)
-        supR3HardenedFatal("supR3HardenedPathProgram: _dyld_get_image_name(0) failed\n");
+        supR3HardenedFatal("supR3HardenedExecDir: _dyld_get_image_name(0) failed\n");
     size_t cchImageName = strlen(pszImageName);
     if (!cchImageName || cchImageName >= sizeof(g_szSupLibHardenedExePath))
-        supR3HardenedFatal("supR3HardenedPathProgram: _dyld_get_image_name(0) failed, cchImageName=%d\n", cchImageName);
+        supR3HardenedFatal("supR3HardenedExecDir: _dyld_get_image_name(0) failed, cchImageName=%d\n", cchImageName);
     memcpy(g_szSupLibHardenedExePath, pszImageName, cchImageName + 1);
 
 #elif defined(RT_OS_WINDOWS)
     HMODULE hExe = GetModuleHandle(NULL);
     if (!GetModuleFileName(hExe, &g_szSupLibHardenedExePath[0], sizeof(g_szSupLibHardenedExePath)))
-        supR3HardenedFatal("supR3HardenedPathProgram: GetModuleFileName failed, rc=%d\n", GetLastError());
+        supR3HardenedFatal("supR3HardenedExecDir: GetModuleFileName failed, rc=%d\n", GetLastError());
 #else
 # error needs porting.
 #endif
@@ -380,9 +393,9 @@ static bool supR3HardenedMainIsProcSelfExeAccssible(void)
 
 
 /**
- * @copydoc RTPathProgram
+ * @copydoc RTPathExecDir
  */
-DECLHIDDEN(int) supR3HardenedPathProgram(char *pszPath, size_t cchPath)
+DECLHIDDEN(int) supR3HardenedPathExecDir(char *pszPath, size_t cchPath)
 {
     /*
      * Lazy init (probably not required).
@@ -400,7 +413,7 @@ DECLHIDDEN(int) supR3HardenedPathProgram(char *pszPath, size_t cchPath)
         return VINF_SUCCESS;
     }
 
-    supR3HardenedFatal("supR3HardenedPathProgram: Buffer too small (%u < %u)\n", cchPath, cch);
+    supR3HardenedFatal("supR3HardenedPathExecDir: Buffer too small (%u < %u)\n", cchPath, cch);
     return VERR_BUFFER_OVERFLOW;
 }
 
@@ -617,7 +630,8 @@ static void supR3HardenedMainGrabCapabilites(void)
 #  ifdef USE_LIB_PCAP
         /* XXX cap_net_bind_service */
         if (!cap_set_proc(cap_from_text("all-eip cap_net_raw+ep")))
-            prctl(PR_SET_KEEPCAPS, /*keep=*/1, 0, 0, 0);
+            prctl(PR_SET_KEEPCAPS, 1 /*keep=*/, 0, 0, 0);
+        prctl(PR_SET_DUMPABLE, 1 /*dump*/, 0, 0, 0);
 #  else
         cap_user_header_t hdr = (cap_user_header_t)alloca(sizeof(*hdr));
         cap_user_data_t   cap = (cap_user_data_t)alloca(sizeof(*cap));
@@ -627,7 +641,8 @@ static void supR3HardenedMainGrabCapabilites(void)
         cap->effective = g_uCaps;
         cap->permitted = g_uCaps;
         if (!capset(hdr, cap))
-            prctl(PR_SET_KEEPCAPS, /*keep=*/1, 0, 0, 0);
+            prctl(PR_SET_KEEPCAPS, 1 /*keep*/, 0, 0, 0);
+        prctl(PR_SET_DUMPABLE, 1 /*dump*/, 0, 0, 0);
 #  endif /* !USE_LIB_PCAP */
     }
 
@@ -1068,4 +1083,3 @@ DECLHIDDEN(int) SUPR3HardenedMain(const char *pszProgName, uint32_t fFlags, int
     return pfnTrustedMain(argc, argv, envp);
 }
 
-
