@@ -1,10 +1,11 @@
 #!/bin/bash
+# $Id: load.sh 28800 2010-04-27 08:22:32Z vboxsync $
 ## @file
 # For development.
 #
 
 #
-# Copyright (C) 2006-2007 Sun Microsystems, Inc.
+# Copyright (C) 2006-2007 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -23,10 +24,6 @@
 # You may elect to license modified versions of this file under the
 # terms and conditions of either the GPL or the CDDL or both.
 #
-# Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
-# Clara, CA 95054 USA or visit http://www.sun.com if you need
-# additional information or have any questions.
-#
 
 XNU_VERSION=`LC_ALL=C uname -r | LC_ALL=C cut -d . -f 1`
 if [ "$XNU_VERSION" -ge "9" ]; then
@@ -114,6 +111,7 @@ if test -n "$VBOX_DARWIN_SYMS"; then
 fi
 
 trap "sudo chown -R `whoami` $DIR; exit 1" INT
+trap "sudo chown -R `whoami` $DIR; exit 1" ERR
 # On smbfs, this might succeed just fine but make no actual changes,
 # so we might have to temporarily copy the driver to a local directory.
 if sudo chown -R root:wheel "$DIR"; then
@@ -153,4 +151,8 @@ sync
 sudo chown -R `whoami` "$DIR"
 #sudo chmod 666 /dev/vboxdrv
 kextstat | grep org.virtualbox.kext
+if [ -n "${VBOX_DARWIN_SYMS}"  -a   "$XNU_VERSION" -ge "10" ]; then
+    dsymutil -o "${VBOX_DARWIN_SYMS}/${DRVNAME}.dSYM" "${DIR}/Contents/MacOS/`basename -s .kext ${DRVNAME}`"
+    sync
+fi
 
