@@ -1,10 +1,10 @@
-/* $Id: VBoxNetFltCommon-win.h $ */
+/* $Id: VBoxNetFltCommon-win.h 29643 2010-05-18 15:26:50Z vboxsync $ */
 /** @file
  * VBoxNetFltCommon.h - Network Filter Driver (Host), Windows Specific Code. Common headeer with commonly used defines and decls
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 /*
  * Based in part on Microsoft DDK sample code for Ndis Intermediate Miniport passthru driver sample.
@@ -26,15 +22,7 @@
 #ifndef ___VBoxNetFltCommon_win_h___
 #define ___VBoxNetFltCommon_win_h___
 
-#ifndef VBOX_NETFLT_ONDEMAND_BIND
-# define NDIS_MINIPORT_DRIVER
-# define NDIS50_MINIPORT 1
-#endif
-
-#define BINARY_COMPATIBLE 0
-#define NDIS_WDM 0
-#define NDIS50 1
-#define NTSTRSAFE_LIB
+//#define NTSTRSAFE_LIB
 
 #ifdef DEBUG
 //# define DEBUG_NETFLT_PACKETS
@@ -152,6 +140,10 @@ typedef struct VBOXNETFLTINS *PVBOXNETFLTINS;
 
 #define VBOX_NETFLT_PACKET_HEADER_MATCH_SIZE 24
 
+#if defined(DEBUG_NETFLT_PACKETS) || !defined(VBOX_LOOPBACK_USEFLAGS)
+# define VBOXNETFLT_PACKETMATCH_LENGTH (ETH_HEADER_SIZE + 2)
+#endif
+
 #ifdef VBOXNETADP
 #define     VBOXNETADP_HEADER_SIZE             14
 #define     VBOXNETADP_MAX_DATA_SIZE           1500
@@ -183,6 +175,7 @@ typedef struct VBOXNETFLTINS *PVBOXNETFLTINS;
  * if clear the packet comes from the wire (underlying miniport) */
 #define PACKET_SRC_HOST             0x00000002
 
+#ifndef VBOXNETFLT_NO_PACKET_QUEUE
 /** flag specifying the packet was originated by our driver
  * i.e. we could use it on our needs and should not return it
  * we are enqueueing "our" packets on ProtocolReceive call-back when
@@ -193,6 +186,7 @@ typedef struct VBOXNETFLTINS *PVBOXNETFLTINS;
 /** flag passed to vboxNetFltWinQuEnqueuePacket specifying that the packet should be copied
  * this is supported for Ndis Packet only */
 #define PACKET_COPY                 0x00000008
+#endif
 
 /** packet queue element containing the packet info */
 typedef struct _PACKET_INFO
@@ -315,15 +309,6 @@ typedef struct _ADAPT
     NDIS_HANDLE                    hMiniportHandle;
     /** miniport device state */
     ADAPT_DEVICE             MPState;
-#ifdef DEBUG_NETFLT_LOOPBACK
-# error "implement (see comments in the sources below this #error:)"
-        /* @todo FIXME no need for the PPACKET_INFO mechanism here;
-        instead the the NDIS_PACKET.ProtocolReserved + INTERLOCKED_SINGLE_LIST mechanism \
-        similar to that used in TrasferData handling should be used;
-        */
-    /** Pending receive packet queue (i.e. packets that were indicated to the upperlying protocols, but not completed yet)*/
-//    INTERLOCKED_PACKET_QUEUE                   RecvPacketQueue;
-#endif
     /** ndis packet pool used for receives */
     NDIS_HANDLE                    hRecvPacketPoolHandle;
     /** ndis buffer pool used for receives */
@@ -388,28 +373,18 @@ typedef struct _ADAPT
 #endif /* !VBOX_NETFLT_ONDEMAND_BIND */
 
 #ifndef VBOXNETADP
+#if defined(DEBUG_NETFLT_LOOPBACK) || !defined(VBOX_LOOPBACK_USEFLAGS)
+    /** used for maintaining the pending send packets for handling packet loopback */
+    INTERLOCKED_SINGLE_LIST SendPacketQueue;
+#endif
     /** used for serializing calls to the NdisRequest in the vboxNetFltWinSynchNdisRequest */
     RTSEMFASTMUTEX                 hSynchRequestMutex;
-
     /** event used to synchronize with the Ndis Request completion in the vboxNetFltWinSynchNdisRequest */
     KEVENT                         hSynchCompletionEvent;
     /** status of the Ndis Request initiated by the vboxNetFltWinSynchNdisRequest */
     NDIS_STATUS   volatile         fSynchCompletionStatus;
     /** pointer to the Ndis Request being executed by the vboxNetFltWinSynchNdisRequest */
     PNDIS_REQUEST volatile         pSynchRequest;
-#endif
-#ifdef DEBUG_NETFLT_LOOPBACK
-# error "implement (see comments in the sources below this #error:)"
-        /* @todo FIXME no need for the PPACKET_INFO mechanism here;
-        instead the the NDIS_PACKET.ProtocolReserved + INTERLOCKED_SINGLE_LIST mechanism \
-        similar to that used in TrasferData handling should be used;
-        */
-    /** Pending send packet queue (i.e. packets that were sent to the underlying miniport, but not completed yet)*/
-
-    INTERLOCKED_PACKET_QUEUE                   SendPacketQueue;
-    /** Packet info pool, i.e. the pool for the packet queue elements */
-#endif
-#ifndef VBOXNETADP
     /** ndis packet pool used for sends */
     NDIS_HANDLE                    hSendPacketPoolHandle;
     /** ndis buffer pool used for sends */
@@ -453,10 +428,14 @@ typedef struct _SEND_RSVD
     /** original packet receiver from the upperlying protocol
      * can be null if the packet was originated by intnet */
     PNDIS_PACKET    pOriginalPkt;
-
     /** pointer to the buffer to be freed on send completion
      * can be null if no buffer is to be freed */
     PVOID           pBufToFree;
+#if !defined(VBOX_LOOPBACK_USEFLAGS) || defined(DEBUG_NETFLT_PACKETS)
+    SINGLE_LIST_ENTRY ListEntry;
+    /* true if the packet is from IntNet */
+    bool bFromIntNet;
+#endif
 } SEND_RSVD, *PSEND_RSVD;
 
 /** represents the data stored in the protocol recerved field of ndis packet on NdisTransferData processing*/
@@ -481,19 +460,10 @@ typedef struct _RECV_RSVD
     PVOID           pBufToFree;
 } RECV_RSVD, *PRECV_RSVD;
 
-typedef struct _PT_RSVD
-{
-    union
-    {
-        RECV_RSVD RecvRsvd;
-        TRANSFERDATA_RSVD TransferDataRsvd;
-    } u;
-} PT_RSVD, *PPT_RSVD;
-
-
 #ifndef VBOX_NETFLT_ONDEMAND_BIND
 
 C_ASSERT(sizeof(RECV_RSVD) <= sizeof(((PNDIS_PACKET)0)->MiniportReserved));
+C_ASSERT(sizeof(TRANSFERDATA_RSVD) <= PROTOCOL_RESERVED_SIZE_IN_PACKET);
 #endif
 
 C_ASSERT(sizeof(NDIS_DEVICE_POWER_STATE) == sizeof(uint32_t));
