@@ -1,11 +1,10 @@
+/* $Id: service.cpp 29394 2010-05-12 01:27:04Z vboxsync $ */
 /** @file
- *
- * Guest Property Service:
- * Host service entry points.
+ * Guest Property Service: Host service entry points.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,13 +13,10 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
-/**
+/** @page pg_svc_guest_properties   Guest Property HGCM Service
+ *
  * This HGCM service allows the guest to set and query values in a property
  * store on the host.  The service proxies the guest requests to the service
  * owner on the host using a request callback provided by the owner, and is
@@ -38,23 +34,23 @@
  * is changed or when the request times out.
  */
 
-#define LOG_GROUP LOG_GROUP_HGCM
-
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
+#define LOG_GROUP LOG_GROUP_HGCM
 #include <VBox/HostServices/GuestPropertySvc.h>
 
 #include <VBox/log.h>
-#include <iprt/err.h>
+#include <iprt/asm.h>
 #include <iprt/assert.h>
-#include <iprt/string.h>
+#include <iprt/cpp/autores.h>
+#include <iprt/cpp/utils.h>
+#include <iprt/err.h>
 #include <iprt/mem.h>
-#include <iprt/autores.h>
-#include <iprt/time.h>
-#include <iprt/cpputils.h>
 #include <iprt/req.h>
+#include <iprt/string.h>
 #include <iprt/thread.h>
+#include <iprt/time.h>
 
 #include <memory>  /* for auto_ptr */
 #include <string>
@@ -100,13 +96,17 @@ struct Property
     }
 
     /** Are two properties equal? */
-    bool operator== (const Property &prop)
+    bool operator==(const Property &prop)
     {
-        return (   mName == prop.mName
-                && mValue == prop.mValue
-                && mTimestamp == prop.mTimestamp
-                && mFlags == prop.mFlags
-               );
+        if (mTimestamp != prop.mTimestamp)
+            return false;
+        if (mFlags != prop.mFlags)
+            return false;
+        if (mName != prop.mName)
+            return false;
+        if (mValue != prop.mValue)
+            return false;
+        return true;
     }
 
     /* Is the property nil? */
@@ -153,6 +153,8 @@ private:
     typedef Service SELF;
     /** HGCM helper functions. */
     PVBOXHGCMSVCHELPERS mpHelpers;
+    /** Global flags for the service */
+    ePropFlags meGlobalFlags;
     /** The property list */
     PropertyList mProperties;
     /** The list of property changes for guest notifications */
@@ -160,17 +162,19 @@ private:
     /** The list of outstanding guest notification calls */
     CallList mGuestWaiters;
     /** @todo we should have classes for thread and request handler thread */
-    /** Queue of outstanding property change notifications */
-    RTREQQUEUE *mReqQueue;
-    /** Thread for processing the request queue */
-    RTTHREAD mReqThread;
-    /** Tell the thread that it should exit */
-    bool mfExitThread;
     /** Callback function supplied by the host for notification of updates
      * to properties */
     PFNHGCMSVCEXT mpfnHostCallback;
     /** User data pointer to be supplied to the host callback function */
     void *mpvHostData;
+    /** The previous timestamp.
+     * This is used by getCurrentTimestamp() to decrease the chance of
+     * generating duplicate timestamps.  */
+    uint64_t mPrevTimestamp;
+    /** The number of consecutive timestamp adjustments that we've made.
+     * Together with mPrevTimestamp, this defines a set of obsolete timestamp
+     * values: {(mPrevTimestamp - mcTimestampAdjustments), ..., mPrevTimestamp} */
+    uint64_t mcTimestampAdjustments;
 
     /**
      * Get the next property change notification from the queue of saved
@@ -200,9 +204,12 @@ private:
          *  - Appears later than u64Timestamp
          *  - Matches the pszPatterns
          */
+        /** @todo r=bird: This incorrectly ASSUMES that mTimestamp is unique.
+         *  The timestamp resolution can be very coarse on windows for instance. */
         PropertyList::const_iterator it = mGuestNotifications.begin();
         for (;    it != mGuestNotifications.end()
-               && it->mTimestamp != u64Timestamp; ++it) {}
+               && it->mTimestamp != u64Timestamp; ++it)
+            {}
         if (it == mGuestNotifications.end())  /* Not found */
             it = mGuestNotifications.begin();
         else
@@ -219,21 +226,36 @@ private:
         return rc;
     }
 
+    /**
+     * Check whether we have permission to change a property.
+     *
+     * @returns Strict VBox status code.
+     * @retval  VINF_SUCCESS if we do.
+     * @retval  VERR_PERMISSION_DENIED if the value is read-only for the requesting
+     *          side.
+     * @retval  VINF_PERMISSION_DENIED if the side is globally marked read-only.
+     *
+     * @param   eFlags   the flags on the property in question
+     * @param   isGuest  is the guest or the host trying to make the change?
+     */
+    int checkPermission(ePropFlags eFlags, bool isGuest)
+    {
+        if (eFlags & (isGuest ? RDONLYGUEST : RDONLYHOST))
+            return VERR_PERMISSION_DENIED;
+        if (isGuest && (meGlobalFlags & RDONLYGUEST))
+            return VINF_PERMISSION_DENIED;
+        return VINF_SUCCESS;
+    }
+
 public:
     explicit Service(PVBOXHGCMSVCHELPERS pHelpers)
-        : mpHelpers(pHelpers), mfExitThread(false), mpfnHostCallback(NULL),
-          mpvHostData(NULL)
-    {
-        int rc = RTReqCreateQueue(&mReqQueue);
-#ifndef VBOX_GUEST_PROP_TEST_NOTHREAD
-        if (RT_SUCCESS(rc))
-            rc = RTThreadCreate(&mReqThread, reqThreadFn, this, 0,
-                                RTTHREADTYPE_MSG_PUMP, RTTHREADFLAGS_WAITABLE,
-                                "GuestPropReq");
-#endif
-        if (RT_FAILURE(rc))
-            throw rc;
-    }
+        : mpHelpers(pHelpers)
+        , meGlobalFlags(NILFLAG)
+        , mpfnHostCallback(NULL)
+        , mpvHostData(NULL)
+        , mPrevTimestamp(0)
+        , mcTimestampAdjustments(0)
+    { }
 
     /**
      * @copydoc VBOXHGCMSVCHELPERS::pfnUnload
@@ -313,6 +335,7 @@ public:
     }
 private:
     static DECLCALLBACK(int) reqThreadFn(RTTHREAD ThreadSelf, void *pvUser);
+    uint64_t getCurrentTimestamp(void);
     int validateName(const char *pszName, uint32_t cbName);
     int validateValue(const char *pszValue, uint32_t cbValue);
     int setPropertyBlock(uint32_t cParms, VBOXHGCMSVCPARM paParms[]);
@@ -326,16 +349,8 @@ private:
                                    uint64_t u64Timestamp, Property *pProp);
     int getNotificationWriteOut(VBOXHGCMSVCPARM paParms[], Property prop);
     void doNotifications(const char *pszProperty, uint64_t u64Timestamp);
-    static DECLCALLBACK(int) reqNotify(PFNHGCMSVCEXT pfnCallback,
-                                       void *pvData, char *pszName,
-                                       char *pszValue, uint32_t u32TimeHigh,
-                                       uint32_t u32TimeLow, char *pszFlags);
-    /**
-     * Empty request function for terminating the request thread.
-     * @returns VINF_EOF to cause the request processing function to return
-     * @todo    return something more appropriate
-     */
-    static DECLCALLBACK(int) reqVoid() { return VINF_EOF; }
+    int notifyHost(const char *pszName, const char *pszValue,
+                   uint64_t u64Timestamp, const char *pszFlags);
 
     void call (VBOXHGCMCALLHANDLE callHandle, uint32_t u32ClientID,
                void *pvClient, uint32_t eFunction, uint32_t cParms,
@@ -346,42 +361,45 @@ private:
 
 
 /**
- * Thread function for processing the request queue
- * @copydoc FNRTTHREAD
+ * Gets the current timestamp.
+ *
+ * Since the RTTimeNow resolution can be very coarse, this method takes some
+ * simple steps to try avoid returning the same timestamp for two consecutive
+ * calls.  Code like getOldNotification() more or less assumes unique
+ * timestamps.
+ *
+ * @returns Nanosecond timestamp.
  */
-DECLCALLBACK(int) Service::reqThreadFn(RTTHREAD ThreadSelf, void *pvUser)
+uint64_t Service::getCurrentTimestamp(void)
 {
-    SELF *pSelf = reinterpret_cast<SELF *>(pvUser);
-    while (!pSelf->mfExitThread)
-        RTReqProcess(pSelf->mReqQueue, RT_INDEFINITE_WAIT);
-    return VINF_SUCCESS;
+    RTTIMESPEC time;
+    uint64_t u64NanoTS = RTTimeSpecGetNano(RTTimeNow(&time));
+    if (mPrevTimestamp - u64NanoTS > mcTimestampAdjustments)
+        mcTimestampAdjustments = 0;
+    else
+    {
+        mcTimestampAdjustments++;
+        u64NanoTS = mPrevTimestamp + 1;
+    }
+    this->mPrevTimestamp = u64NanoTS;
+    return u64NanoTS;
 }
 
-
 /**
- * Checking that the name passed by the guest fits our criteria for a
- * property name.
+ * Check that a string fits our criteria for a property name.
  *
  * @returns IPRT status code
- * @param   pszName   the name passed by the guest
- * @param   cbName    the number of bytes pszName points to, including the
+ * @param   pszName   the string to check, must be valid Utf8
+ * @param   cbName    the number of bytes @a pszName points to, including the
  *                    terminating '\0'
  * @thread  HGCM
  */
 int Service::validateName(const char *pszName, uint32_t cbName)
 {
     LogFlowFunc(("cbName=%d\n", cbName));
-
-    /*
-     * Validate the name, checking that it's proper UTF-8 and has
-     * a string terminator.
-     */
     int rc = VINF_SUCCESS;
     if (RT_SUCCESS(rc) && (cbName < 2))
         rc = VERR_INVALID_PARAMETER;
-    if (RT_SUCCESS(rc))
-        rc = RTStrValidateEncodingEx(pszName, RT_MIN(cbName, (uint32_t) MAX_NAME_LEN),
-                                     RTSTR_VALIDATE_ENCODING_ZERO_TERMINATED);
     for (unsigned i = 0; RT_SUCCESS(rc) && i < cbName; ++i)
         if (pszName[i] == '*' || pszName[i] == '?' || pszName[i] == '|')
             rc = VERR_INVALID_PARAMETER;
@@ -391,28 +409,21 @@ int Service::validateName(const char *pszName, uint32_t cbName)
 
 
 /**
- * Check that the data passed by the guest fits our criteria for the value of
- * a guest property.
+ * Check a string fits our criteria for the value of a guest property.
  *
  * @returns IPRT status code
- * @param   pszValue  the value to store in the property
- * @param   cbValue   the number of bytes in the buffer pszValue points to
+ * @param   pszValue  the string to check, must be valid Utf8
+ * @param   cbValue   the length in bytes of @a pszValue, including the
+ *                    terminator
  * @thread  HGCM
  */
 int Service::validateValue(const char *pszValue, uint32_t cbValue)
 {
     LogFlowFunc(("cbValue=%d\n", cbValue));
 
-    /*
-     * Validate the value, checking that it's proper UTF-8 and has
-     * a string terminator.
-     */
     int rc = VINF_SUCCESS;
     if (RT_SUCCESS(rc) && cbValue == 0)
         rc = VERR_INVALID_PARAMETER;
-    if (RT_SUCCESS(rc))
-        rc = RTStrValidateEncodingEx(pszValue, RT_MIN(cbValue, (uint32_t) MAX_VALUE_LEN),
-                                     RTSTR_VALIDATE_ENCODING_ZERO_TERMINATED);
     if (RT_SUCCESS(rc))
         LogFlow(("    pszValue=%s\n", cbValue > 0 ? pszValue : NULL));
     LogFlowFunc(("returning %Rrc\n", rc));
@@ -484,16 +495,12 @@ int Service::setPropertyBlock(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
     {
         ++itEnd;
         for (unsigned i = 0; ppNames[i] != NULL; ++i)
-        {
-            bool found = false;
-            for (PropertyList::iterator it = mProperties.begin();
-                !found && it != itEnd; ++it)
+            for (PropertyList::iterator it = mProperties.begin(); it != itEnd; ++it)
                 if (it->mName.compare(ppNames[i]) == 0)
                 {
-                    found = true;
                     mProperties.erase(it);
+                    break;
                 }
-        }
     }
 
     /*
@@ -524,7 +531,7 @@ int Service::setPropertyBlock(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
 int Service::getProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
 {
     int rc = VINF_SUCCESS;
-    const char *pcszName;
+    const char *pcszName = NULL;        /* shut up gcc */
     char *pchBuf;
     uint32_t cchName, cchBuf;
     char szFlags[MAX_FLAGS_LEN];
@@ -534,8 +541,8 @@ int Service::getProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
      */
     LogFlowThisFunc(("\n"));
     if (   cParms != 4  /* Hardcoded value as the next lines depend on it. */
-        || RT_FAILURE (paParms[0].getPointer ((const void **) &pcszName, &cchName))    /* name */
-        || RT_FAILURE (paParms[1].getPointer ((void **) &pchBuf, &cchBuf))    /* buffer */
+        || RT_FAILURE (paParms[0].getString(&pcszName, &cchName))  /* name */
+        || RT_FAILURE (paParms[1].getBuffer((void **) &pchBuf, &cchBuf))  /* buffer */
        )
         rc = VERR_INVALID_PARAMETER;
     else
@@ -600,11 +607,14 @@ int Service::getProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
 int Service::setProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGuest)
 {
     int rc = VINF_SUCCESS;
-    const char *pcszName, *pcszValue, *pcszFlags = NULL;
-    uint32_t cchName, cchValue, cchFlags = 0;
+    const char *pcszName = NULL;        /* shut up gcc */
+    const char *pcszValue = NULL;       /* ditto */
+    const char *pcszFlags = NULL;
+    uint32_t cchName = 0;               /* ditto */
+    uint32_t cchValue = 0;              /* ditto */
+    uint32_t cchFlags = 0;
     uint32_t fFlags = NILFLAG;
-    RTTIMESPEC time;
-    uint64_t u64TimeNano = RTTimeSpecGetNano(RTTimeNow(&time));
+    uint64_t u64TimeNano = getCurrentTimestamp();
 
     LogFlowThisFunc(("\n"));
     /*
@@ -618,13 +628,10 @@ int Service::setProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGues
      */
     if (   RT_SUCCESS(rc)
         && (   (cParms < 2) || (cParms > 3)  /* Hardcoded value as the next lines depend on it. */
-            || RT_FAILURE(paParms[0].getPointer ((const void **) &pcszName,
-                                                 &cchName)) /* name */
-            || RT_FAILURE(paParms[1].getPointer ((const void **) &pcszValue,
-                                                 &cchValue)) /* value */
+            || RT_FAILURE(paParms[0].getString(&pcszName, &cchName))  /* name */
+            || RT_FAILURE(paParms[1].getString(&pcszValue, &cchValue))  /* value */
             || (   (3 == cParms)
-                && RT_FAILURE(paParms[2].getPointer ((const void **) &pcszFlags,
-                                                     &cchFlags)) /* flags */
+                && RT_FAILURE(paParms[2].getString(&pcszFlags, &cchFlags)) /* flags */
                )
            )
        )
@@ -642,51 +649,47 @@ int Service::setProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGues
                                      RTSTR_VALIDATE_ENCODING_ZERO_TERMINATED);
     if ((3 == cParms) && RT_SUCCESS(rc))
         rc = validateFlags(pcszFlags, &fFlags);
-
-    /*
-     * If the property already exists, check its flags to see if we are allowed
-     * to change it.
-     */
-    PropertyList::iterator it;
-    bool found = false;
     if (RT_SUCCESS(rc))
+    {
+        /*
+         * If the property already exists, check its flags to see if we are allowed
+         * to change it.
+         */
+        PropertyList::iterator it;
+        bool found = false;
         for (it = mProperties.begin(); it != mProperties.end(); ++it)
             if (it->mName.compare(pcszName) == 0)
             {
                 found = true;
                 break;
             }
-    if (RT_SUCCESS(rc) && found)
-        if (   (isGuest && (it->mFlags & RDONLYGUEST))
-            || (!isGuest && (it->mFlags & RDONLYHOST))
-           )
-            rc = VERR_PERMISSION_DENIED;
 
-    /*
-     * Set the actual value
-     */
-    if (RT_SUCCESS(rc))
-    {
-        if (found)
+        rc = checkPermission(found ? (ePropFlags)it->mFlags : NILFLAG,
+                             isGuest);
+        if (rc == VINF_SUCCESS)
         {
-            it->mValue = pcszValue;
-            it->mTimestamp = u64TimeNano;
-            it->mFlags = fFlags;
+            /*
+             * Set the actual value
+             */
+            if (found)
+            {
+                it->mValue = pcszValue;
+                it->mTimestamp = u64TimeNano;
+                it->mFlags = fFlags;
+            }
+            else  /* This can throw.  No problem as we have nothing to roll back. */
+                mProperties.push_back(Property(pcszName, pcszValue, u64TimeNano, fFlags));
+
+            /*
+             * Send a notification to the host and return.
+             */
+            // if (isGuest)  /* Notify the host even for properties that the host
+            //                * changed.  Less efficient, but ensures consistency. */
+                doNotifications(pcszName, u64TimeNano);
+            Log2(("Set string %s, rc=%Rrc, value=%s\n", pcszName, rc, pcszValue));
         }
-        else  /* This can throw.  No problem as we have nothing to roll back. */
-            mProperties.push_back(Property(pcszName, pcszValue, u64TimeNano, fFlags));
     }
 
-    /*
-     * Send a notification to the host and return.
-     */
-    if (RT_SUCCESS(rc))
-    {
-        // if (isGuest)  /* Notify the host even for properties that the host
-        //                * changed.  Less efficient, but ensures consistency. */
-            doNotifications(pcszName, u64TimeNano);
-        Log2(("Set string %s, rc=%Rrc, value=%s\n", pcszName, rc, pcszValue));
-    }
     LogFlowThisFunc(("rc = %Rrc\n", rc));
     return rc;
 }
@@ -705,7 +708,7 @@ int Service::setProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGues
 int Service::delProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGuest)
 {
     int rc = VINF_SUCCESS;
-    const char *pcszName;
+    const char *pcszName = NULL;        /* shut up gcc */
     uint32_t cbName;
 
     LogFlowThisFunc(("\n"));
@@ -713,45 +716,41 @@ int Service::delProperty(uint32_t cParms, VBOXHGCMSVCPARM paParms[], bool isGues
     /*
      * Check the user-supplied parameters.
      */
-    if (   (cParms != 1)  /* Hardcoded value as the next lines depend on it. */
-        || RT_FAILURE(paParms[0].getPointer ((const void **) &pcszName,
-                                             &cbName))  /* name */
+    if (   (cParms == 1)  /* Hardcoded value as the next lines depend on it. */
+        && RT_SUCCESS(paParms[0].getString(&pcszName, &cbName))  /* name */
        )
+        rc = validateName(pcszName, cbName);
+    else
         rc = VERR_INVALID_PARAMETER;
     if (RT_SUCCESS(rc))
-        rc = validateName(pcszName, cbName);
-
-    /*
-     * If the property exists, check its flags to see if we are allowed
-     * to change it.
-     */
-    PropertyList::iterator it;
-    bool found = false;
-    if (RT_SUCCESS(rc))
+    {
+        /*
+         * If the property exists, check its flags to see if we are allowed
+         * to change it.
+         */
+        PropertyList::iterator it;
+        bool found = false;
         for (it = mProperties.begin(); it != mProperties.end(); ++it)
             if (it->mName.compare(pcszName) == 0)
             {
                 found = true;
+                rc = checkPermission((ePropFlags)it->mFlags, isGuest);
                 break;
             }
-    if (RT_SUCCESS(rc) && found)
-        if (   (isGuest && (it->mFlags & RDONLYGUEST))
-            || (!isGuest && (it->mFlags & RDONLYHOST))
-           )
-            rc = VERR_PERMISSION_DENIED;
 
-    /*
-     * And delete the property if all is well.
-     */
-    if (RT_SUCCESS(rc) && found)
-    {
-        RTTIMESPEC time;
-        uint64_t u64Timestamp = RTTimeSpecGetNano(RTTimeNow(&time));
-        mProperties.erase(it);
-        // if (isGuest)  /* Notify the host even for properties that the host
-        //                * changed.  Less efficient, but ensures consistency. */
-            doNotifications(pcszName, u64Timestamp);
+        /*
+         * And delete the property if all is well.
+         */
+        if (rc == VINF_SUCCESS && found)
+        {
+            uint64_t u64Timestamp = getCurrentTimestamp();
+            mProperties.erase(it);
+            // if (isGuest)  /* Notify the host even for properties that the host
+            //                * changed.  Less efficient, but ensures consistency. */
+                doNotifications(pcszName, u64Timestamp);
+        }
     }
+
     LogFlowThisFunc(("rc = %Rrc\n", rc));
     return rc;
 }
@@ -776,9 +775,8 @@ int Service::enumProps(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
     uint32_t cchPatterns = 0, cchBuf = 0;
     LogFlowThisFunc(("\n"));
     if (   (cParms != 3)  /* Hardcoded value as the next lines depend on it. */
-        || RT_FAILURE(paParms[0].getPointer ((const void **) &pcchPatterns,
-                                             &cchPatterns))  /* patterns */
-        || RT_FAILURE(paParms[1].getPointer ((void **) &pchBuf, &cchBuf))  /* return buffer */
+        || RT_FAILURE(paParms[0].getString(&pcchPatterns, &cchPatterns))  /* patterns */
+        || RT_FAILURE(paParms[1].getBuffer((void **) &pchBuf, &cchBuf))  /* return buffer */
        )
         rc = VERR_INVALID_PARAMETER;
     if (RT_SUCCESS(rc) && cchPatterns > MAX_PATTERN_LEN)
@@ -788,9 +786,7 @@ int Service::enumProps(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
      * First repack the patterns into the format expected by RTStrSimplePatternMatch()
      */
     char pszPatterns[MAX_PATTERN_LEN];
-    if (NULL == pcchPatterns)
-        pszPatterns[0] = '\0';
-    else
+    if (RT_SUCCESS(rc))
     {
         for (unsigned i = 0; i < cchPatterns - 1; ++i)
             if (pcchPatterns[i] != '\0')
@@ -827,7 +823,8 @@ int Service::enumProps(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
             buffer += '\0';
         }
     }
-    buffer.append(4, '\0');  /* The final terminators */
+    if (RT_SUCCESS(rc))
+        buffer.append(4, '\0');  /* The final terminators */
 
     /*
      * Finally write out the temporary buffer to the real one if it is not too
@@ -845,36 +842,37 @@ int Service::enumProps(uint32_t cParms, VBOXHGCMSVCPARM paParms[])
     return rc;
 }
 
+
 /** Helper query used by getOldNotification */
 int Service::getOldNotificationInternal(const char *pszPatterns,
                                         uint64_t u64Timestamp,
                                         Property *pProp)
 {
-    int rc = VINF_SUCCESS;
-    bool warn = false;
-
     /* We count backwards, as the guest should normally be querying the
      * most recent events. */
+    int rc = VWRN_NOT_FOUND;
     PropertyList::reverse_iterator it = mGuestNotifications.rbegin();
-    for (; it->mTimestamp != u64Timestamp && it != mGuestNotifications.rend();
-         ++it) {}
-    /* Warn if the timestamp was not found. */
-    if (it->mTimestamp != u64Timestamp)
-        warn = true;
+    for (; it != mGuestNotifications.rend(); ++it)
+        if (it->mTimestamp == u64Timestamp)
+        {
+            rc = VINF_SUCCESS;
+            break;
+        }
+
     /* Now look for an event matching the patterns supplied.  The base()
      * member conveniently points to the following element. */
     PropertyList::iterator base = it.base();
-    for (; !base->Matches(pszPatterns) && base != mGuestNotifications.end();
-         ++base) {}
-    if (RT_SUCCESS(rc) && base != mGuestNotifications.end())
-        *pProp = *base;
-    else if (RT_SUCCESS(rc))
-        *pProp = Property();
-    if (warn)
-        rc = VWRN_NOT_FOUND;
+    for (; base != mGuestNotifications.end(); ++base)
+        if (base->Matches(pszPatterns))
+        {
+            *pProp = *base;
+            return rc;
+        }
+    *pProp = Property();
     return rc;
 }
 
+
 /** Helper query used by getNotification */
 int Service::getNotificationWriteOut(VBOXHGCMSVCPARM paParms[], Property prop)
 {
@@ -884,7 +882,7 @@ int Service::getNotificationWriteOut(VBOXHGCMSVCPARM paParms[], Property prop)
     uint64_t u64Timestamp;
     char *pchBuf;
     uint32_t cchBuf;
-    rc = paParms[2].getPointer((void **) &pchBuf, &cchBuf);
+    rc = paParms[2].getBuffer((void **) &pchBuf, &cchBuf);
     if (RT_SUCCESS(rc))
     {
         char szFlags[MAX_FLAGS_LEN];
@@ -913,6 +911,7 @@ int Service::getNotificationWriteOut(VBOXHGCMSVCPARM paParms[], Property prop)
     return rc;
 }
 
+
 /**
  * Get the next guest notification.
  *
@@ -926,8 +925,10 @@ int Service::getNotification(VBOXHGCMCALLHANDLE callHandle, uint32_t cParms,
                              VBOXHGCMSVCPARM paParms[])
 {
     int rc = VINF_SUCCESS;
-    char *pszPatterns, *pchBuf;
-    uint32_t cchPatterns = 0, cchBuf = 0;
+    char *pszPatterns = NULL;           /* shut up gcc */
+    char *pchBuf;
+    uint32_t cchPatterns = 0;
+    uint32_t cchBuf = 0;
     uint64_t u64Timestamp;
 
     /*
@@ -935,13 +936,14 @@ int Service::getNotification(VBOXHGCMCALLHANDLE callHandle, uint32_t cParms,
      */
     LogFlowThisFunc(("\n"));
     if (   (cParms != 4)  /* Hardcoded value as the next lines depend on it. */
-        || RT_FAILURE(paParms[0].getPointer ((void **) &pszPatterns, &cchPatterns))  /* patterns */
-        || pszPatterns[cchPatterns - 1] != '\0'  /* The patterns string must be zero-terminated */
-        || RT_FAILURE(paParms[1].getUInt64 (&u64Timestamp))  /* timestamp */
-        || RT_FAILURE(paParms[2].getPointer ((void **) &pchBuf, &cchBuf))  /* return buffer */
-        || cchBuf < 1
+        || RT_FAILURE(paParms[0].getString(&pszPatterns, &cchPatterns))  /* patterns */
+        || RT_FAILURE(paParms[1].getUInt64(&u64Timestamp))  /* timestamp */
+        || RT_FAILURE(paParms[2].getBuffer((void **) &pchBuf, &cchBuf))  /* return buffer */
        )
         rc = VERR_INVALID_PARAMETER;
+    if (RT_SUCCESS(rc))
+        LogFlow(("    pszPatterns=%s, u64Timestamp=%llu\n", pszPatterns,
+                 u64Timestamp));
 
     /*
      * If no timestamp was supplied or no notification was found in the queue
@@ -968,13 +970,12 @@ int Service::getNotification(VBOXHGCMCALLHANDLE callHandle, uint32_t cParms,
     return rc;
 }
 
+
 /**
  * Notify the service owner and the guest that a property has been
  * added/deleted/changed
  * @param pszProperty  the name of the property which has changed
  * @param u64Timestamp the time at which the change took place
- * @note this call allocates memory which the reqNotify request is expected to
- *       free again, using RTStrFree().
  *
  * @thread  HGCM service
  */
@@ -1021,14 +1022,14 @@ void Service::doNotifications(const char *pszProperty, uint64_t u64Timestamp)
             {
                 const char *pszPatterns;
                 uint32_t cchPatterns;
-                it->mParms[0].getPointer((void **) &pszPatterns, &cchPatterns);
+                it->mParms[0].getString(&pszPatterns, &cchPatterns);
                 if (prop.Matches(pszPatterns))
                 {
-                    GuestCall call = *it;
-                    int rc2 = getNotificationWriteOut(call.mParms, prop);
+                    GuestCall curCall = *it;
+                    int rc2 = getNotificationWriteOut(curCall.mParms, prop);
                     if (RT_SUCCESS(rc2))
-                        rc2 = call.mRc;
-                    mpHelpers->pfnCallComplete (call.mHandle, rc2);
+                        rc2 = curCall.mRc;
+                    mpHelpers->pfnCallComplete(curCall.mHandle, rc2);
                     it = mGuestWaiters.erase(it);
                 }
                 else
@@ -1044,35 +1045,19 @@ void Service::doNotifications(const char *pszProperty, uint64_t u64Timestamp)
     if (mGuestNotifications.size() > MAX_GUEST_NOTIFICATIONS)
         mGuestNotifications.pop_front();
 
-#ifndef VBOX_GUEST_PROP_TEST_NOTHREAD
     /*
      * Host notifications - first case: if the property exists then send its
      * current value
      */
-    char *pszName = NULL, *pszValue = NULL, *pszFlags = NULL;
-
     if (found && mpfnHostCallback != NULL)
     {
         char szFlags[MAX_FLAGS_LEN];
         /* Send out a host notification */
-        rc = writeFlags(prop.mFlags, szFlags);
+        const char *pszValue = prop.mValue.c_str();
         if (RT_SUCCESS(rc))
-            rc = RTStrDupEx(&pszName, pszProperty);
+            rc = writeFlags(prop.mFlags, szFlags);
         if (RT_SUCCESS(rc))
-            rc = RTStrDupEx(&pszValue, prop.mValue.c_str());
-        if (RT_SUCCESS(rc))
-            rc = RTStrDupEx(&pszFlags, szFlags);
-        if (RT_SUCCESS(rc))
-        {
-            LogFlowThisFunc (("pszName=%p (%s)\n", pszName, pszName));
-            LogFlowThisFunc (("pszValue=%p (%s)\n", pszValue, pszValue));
-            LogFlowThisFunc (("pszFlags=%p (%s)\n", pszFlags, pszFlags));
-            rc = RTReqCallEx(mReqQueue, NULL, 0, RTREQFLAGS_NO_WAIT,
-                             (PFNRT)Service::reqNotify, 7, mpfnHostCallback,
-                             mpvHostData, pszName, pszValue,
-                             (uint32_t) RT_HIDWORD(u64Timestamp),
-                             (uint32_t) RT_LODWORD(u64Timestamp), pszFlags);
-        }
+            rc = notifyHost(pszProperty, pszValue, u64Timestamp, szFlags);
     }
 
     /*
@@ -1082,62 +1067,36 @@ void Service::doNotifications(const char *pszProperty, uint64_t u64Timestamp)
     if (!found && mpfnHostCallback != NULL)
     {
         /* Send out a host notification */
-        rc = RTStrDupEx(&pszName, pszProperty);
         if (RT_SUCCESS(rc))
-        {
-            LogFlowThisFunc (("pszName=%p (%s)\n", pszName, pszName));
-            rc = RTReqCallEx(mReqQueue, NULL, 0, RTREQFLAGS_NO_WAIT,
-                             (PFNRT)Service::reqNotify, 7, mpfnHostCallback,
-                             mpvHostData, pszName, (uintptr_t) NULL,
-                             (uint32_t) RT_HIDWORD(u64Timestamp),
-                             (uint32_t) RT_LODWORD(u64Timestamp),
-                             (uintptr_t) NULL);
-       }
-    }
-    if (RT_FAILURE(rc)) /* clean up if we failed somewhere */
-    {
-        LogThisFunc (("Failed, freeing allocated strings.\n"));
-        RTStrFree(pszName);
-        RTStrFree(pszValue);
-        RTStrFree(pszFlags);
+            rc = notifyHost(pszProperty, NULL, u64Timestamp, NULL);
     }
     LogFlowThisFunc (("returning\n"));
-#endif /* VBOX_GUEST_PROP_TEST_NOTHREAD not defined */
 }
 
 /**
  * Notify the service owner that a property has been added/deleted/changed.
- * asynchronous part.
- * @param pszProperty the name of the property which has changed
- * @note this call allocates memory which the reqNotify request is expected to
- *       free again, using RTStrFree().
- *
- * @thread  request thread
+ * @returns  IPRT status value
+ * @param    pszName       the property name
+ * @param    pszValue      the new value, or NULL if the property was deleted
+ * @param    u64Timestamp  the time of the change
+ * @param    pszFlags      the new flags string
  */
-/* static */
-int Service::reqNotify(PFNHGCMSVCEXT pfnCallback, void *pvData,
-                       char *pszName, char *pszValue, uint32_t u32TimeHigh,
-                       uint32_t u32TimeLow, char *pszFlags)
+int Service::notifyHost(const char *pszName, const char *pszValue,
+                        uint64_t u64Timestamp, const char *pszFlags)
 {
-    LogFlowFunc (("pfnCallback=%p, pvData=%p, pszName=%p, pszValue=%p, u32TimeHigh=%u, u32TimeLow=%u, pszFlags=%p\n", pfnCallback, pvData, pszName, pszValue, u32TimeHigh, u32TimeLow, pszFlags));
-    LogFlowFunc (("pszName=%s\n", pszName));
-    LogFlowFunc (("pszValue=%s\n", pszValue));
-    LogFlowFunc (("pszFlags=%s\n", pszFlags));
-    /* LogFlowFunc (("pfnCallback=%p, pvData=%p, pszName=%s, pszValue=%s, u32TimeHigh=%u, u32TimeLow=%u, pszFlags=%s\n", pfnCallback, pvData, pszName, pszValue, u32TimeHigh, u32TimeLow, pszFlags)); */
+    LogFlowFunc (("pszName=%s, pszValue=%s, u64Timestamp=%llu, pszFlags=%s\n",
+                  pszName, pszValue, u64Timestamp, pszFlags));
     HOSTCALLBACKDATA HostCallbackData;
     HostCallbackData.u32Magic     = HOSTCALLBACKMAGIC;
     HostCallbackData.pcszName     = pszName;
     HostCallbackData.pcszValue    = pszValue;
-    HostCallbackData.u64Timestamp = RT_MAKE_U64(u32TimeLow, u32TimeHigh);
+    HostCallbackData.u64Timestamp = u64Timestamp;
     HostCallbackData.pcszFlags    = pszFlags;
-    AssertRC(pfnCallback(pvData, 0, reinterpret_cast<void *>(&HostCallbackData),
-                         sizeof(HostCallbackData)));
-    LogFlowFunc (("Freeing strings\n"));
-    RTStrFree(pszName);
-    RTStrFree(pszValue);
-    RTStrFree(pszFlags);
-    LogFlowFunc (("returning success\n"));
-    return VINF_SUCCESS;
+    int rc = mpfnHostCallback (mpvHostData, 0 /*u32Function*/,
+                           (void *)(&HostCallbackData),
+                           sizeof(HostCallbackData));
+    LogFlowFunc (("returning %Rrc\n", rc));
+    return rc;
 }
 
 
@@ -1266,6 +1225,20 @@ int Service::hostCall (uint32_t eFunction, uint32_t cParms, VBOXHGCMSVCPARM paPa
                 rc = enumProps(cParms, paParms);
                 break;
 
+            /* The host wishes to set global flags for the service */
+            case SET_GLOBAL_FLAGS_HOST:
+                LogFlowFunc(("SET_GLOBAL_FLAGS_HOST\n"));
+                if (cParms == 1)
+                {
+                    uint32_t eFlags;
+                    rc = paParms[0].getUInt32(&eFlags);
+                    if (RT_SUCCESS(rc))
+                        meGlobalFlags = (ePropFlags)eFlags;
+                }
+                else
+                    rc = VERR_INVALID_PARAMETER;
+                break;
+
             default:
                 rc = VERR_NOT_SUPPORTED;
                 break;
@@ -1282,23 +1255,7 @@ int Service::hostCall (uint32_t eFunction, uint32_t cParms, VBOXHGCMSVCPARM paPa
 
 int Service::uninit()
 {
-    int rc = VINF_SUCCESS;
-    unsigned count = 0;
-
-    mfExitThread = true;
-#ifndef VBOX_GUEST_PROP_TEST_NOTHREAD
-    rc = RTReqCallEx(mReqQueue, NULL, 0, RTREQFLAGS_NO_WAIT, (PFNRT)reqVoid, 0);
-    if (RT_SUCCESS(rc))
-        do
-        {
-            rc = RTThreadWait(mReqThread, 1000, NULL);
-            ++count;
-            Assert(RT_SUCCESS(rc) || ((VERR_TIMEOUT == rc) && (count != 5)));
-        } while ((VERR_TIMEOUT == rc) && (count < 300));
-#endif /* VBOX_GUEST_PROP_TEST_NOTHREAD not defined */
-    if (RT_SUCCESS(rc))
-        RTReqDestroyQueue(mReqQueue);
-    return rc;
+    return VINF_SUCCESS;
 }
 
 } /* namespace guestProp */
