@@ -1,11 +1,11 @@
-/* $Id: crservice.cpp $ */
+/* $Id: crservice.cpp 29019 2010-05-04 13:57:05Z vboxsync $ */
 
 /** @file
  * VBox crOpenGL: Host service entry points.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,12 +14,9 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+#define __STDC_CONSTANT_MACROS  /* needed for a definition in iprt/string.h */
 
 #ifdef RT_OS_WINDOWS
 #include <iprt/alloc.h>
@@ -52,42 +49,22 @@
 #include "cr_server.h"
 #define LOG_GROUP LOG_GROUP_SHARED_CROPENGL
 #include <VBox/log.h>
-#endif
+#include <VBox/com/ErrorInfo.h>
+#endif /* RT_OS_WINDOWS */
 
-#include "render/renderspu.h"
+#include <VBox/com/errorprint.h>
 
 PVBOXHGCMSVCHELPERS g_pHelpers;
-static IFramebuffer* g_pFrameBuffer;
-static ULONG64 g_winId = 0;
+static IConsole* g_pConsole = NULL;
+static PVM g_pVM = NULL;
 
 #ifndef RT_OS_WINDOWS
 #define DWORD int
 #define WINAPI
 #endif
 
-#define CR_USE_HGCM
-
 static const char* gszVBoxOGLSSMMagic = "***OpenGL state data***";
-#define SHCROGL_SSM_VERSION 3
-
-typedef struct
-{
-    DWORD dwThreadID;
-
-} VBOXOGLCTX, *PVBOXOGLCTX;
-
-/*@todo remove this workaround for crstate "unshareable" data*/
-static int crIsThreadWorking=0;
-
-static DWORD WINAPI crServerProc(void* pv)
-{
-    uint64_t winId = *((uint64_t*)pv);
-    renderspuSetWindowId((uint32_t)winId);
-    CRServerMain(0, NULL);
-    crIsThreadWorking = 0;
-    return 0;
-}
-
+#define SHCROGL_SSM_VERSION 19
 
 static DECLCALLBACK(int) svcUnload (void *)
 {
@@ -95,7 +72,6 @@ static DECLCALLBACK(int) svcUnload (void *)
 
     Log(("SHARED_CROPENGL svcUnload\n"));
 
-    //vboxglGlobalUnload();
     crVBoxServerTearDown();
 
     return rc;
@@ -105,33 +81,11 @@ static DECLCALLBACK(int) svcConnect (void *, uint32_t u32ClientID, void *pvClien
 {
     int rc = VINF_SUCCESS;
 
-    NOREF(u32ClientID);
-    VBOXOGLCTX *pClient = (VBOXOGLCTX *)pvClient;
-    Assert(pClient);
+    NOREF(pvClient);
 
     Log(("SHARED_CROPENGL svcConnect: u32ClientID = %d\n", u32ClientID));
 
-#ifndef CR_USE_HGCM
-    if (!crIsThreadWorking)
-    {
-        HANDLE h;
-        Assert(g_pFrameBuffer);
-
-        g_pFrameBuffer->COMGETTER(WinId)(&g_winId);
-        //CHECK_ERROR_RET(g_piConsole, COMGETTER(Display)(display.asOutParam()), rc);
-
-        //vboxglConnect((PVBOXOGLCTX)pvClient);
-        crIsThreadWorking=1;
-        h = CreateThread(NULL, 0, crServerProc, (void*)&g_winId, 0, &pClient->dwThreadID);
-        if (!h) rc = VERR_MAX_THRDS_REACHED;
-    }
-    else
-        rc = VERR_MAX_THRDS_REACHED;
-#else
-    g_pFrameBuffer->COMGETTER(WinId)(&g_winId);
-    renderspuSetWindowId((uint32_t)g_winId);
     rc = crVBoxServerAddClient(u32ClientID);
-#endif
 
     return rc;
 }
@@ -139,31 +93,24 @@ static DECLCALLBACK(int) svcConnect (void *, uint32_t u32ClientID, void *pvClien
 static DECLCALLBACK(int) svcDisconnect (void *, uint32_t u32ClientID, void *pvClient)
 {
     int rc = VINF_SUCCESS;
-    VBOXOGLCTX *pClient = (VBOXOGLCTX *)pvClient;
-    Assert(pClient);
+
+    NOREF(pvClient);
 
     Log(("SHARED_CROPENGL svcDisconnect: u32ClientID = %d\n", u32ClientID));
 
-#ifndef CR_USE_HGCM
-    if (crIsThreadWorking && pClient->dwThreadID)
-        PostThreadMessage(pClient->dwThreadID, WM_QUIT, 0, 0);
-#else
     crVBoxServerRemoveClient(u32ClientID);
-#endif
-    //vboxglDisconnect((PVBOXOGLCTX)pvClient);
+
     return rc;
 }
 
 static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
 {
-    VBOXOGLCTX *pClient = (VBOXOGLCTX *)pvClient;
+    int rc = VINF_SUCCESS;
 
-    NOREF(pClient);
+    NOREF(pvClient);
 
     Log(("SHARED_CROPENGL svcSaveState: u32ClientID = %d\n", u32ClientID));
 
-    int rc;
-
     /* Start*/
     rc = SSMR3PutStrZ(pSSM, gszVBoxOGLSSMMagic);
     AssertRCReturn(rc, rc);
@@ -185,15 +132,13 @@ static DECLCALLBACK(int) svcSaveState(void *, uint32_t u32ClientID, void *pvClie
 
 static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClient, PSSMHANDLE pSSM)
 {
-    VBOXOGLCTX *pClient = (VBOXOGLCTX *)pvClient;
+    int rc = VINF_SUCCESS;
 
-    NOREF(pClient);
-    NOREF(pSSM);
+    NOREF(pvClient);
 
     Log(("SHARED_CROPENGL svcLoadState: u32ClientID = %d\n", u32ClientID));
 
     char psz[2000];
-    int rc;
     uint32_t ui32;
 
     /* Start of data */
@@ -205,11 +150,19 @@ static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClie
     /* Version */
     rc = SSMR3GetU32(pSSM, &ui32);
     AssertRCReturn(rc, rc);
-    if (SHCROGL_SSM_VERSION != ui32)
-        return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
+    if ((SHCROGL_SSM_VERSION != ui32)
+        && ((SHCROGL_SSM_VERSION!=4) || (3!=ui32)))
+    {
+        /*@todo: add some warning here*/
+        /*@todo: in many cases saved states would be made without any opengl guest app running.
+         *       that means we could safely restore the default context.
+         */
+        rc = SSMR3SkipToEndOfUnit(pSSM);
+        return rc;
+    }
 
     /* The state itself */
-    rc = crVBoxServerLoadState(pSSM);
+    rc = crVBoxServerLoadState(pSSM, ui32);
     AssertRCReturn(rc, rc);
 
     /* End of data */
@@ -224,16 +177,59 @@ static DECLCALLBACK(int) svcLoadState(void *, uint32_t u32ClientID, void *pvClie
 static void svcClientVersionUnsupported(uint32_t minor, uint32_t major)
 {
     LogRel(("SHARED_CROPENGL: unsupported client version %d.%d\n", minor, major));
-    /*todo add warning window*/
+
+    /*MS's opengl32 tryes to load our ICD around 30 times on failure...this is to prevent unnecessary spam*/
+    static int shown = 0;
+
+    if (g_pVM && !shown)
+    {
+        VMSetRuntimeError(g_pVM, VMSETRTERR_FLAGS_NO_WAIT, "3DSupportIncompatibleAdditions",
+        "An attempt by the virtual machine to use hardware 3D acceleration failed. "
+        "The version of the Guest Additions installed in the virtual machine does not match the "
+        "version of VirtualBox on the host. Please install appropriate Guest Additions to fix this issue");
+        shown = 1;
+    }
+}
+
+static DECLCALLBACK(void) svcPresentFBO(void *data, int32_t screenId, int32_t x, int32_t y, uint32_t w, uint32_t h)
+{
+#if 0
+    ComPtr<IDisplay> pDisplay;
+    BYTE *data;
+    int i,j;
+    CHECK_ERROR(g_pConsole, COMGETTER(Display)(pDisplay.asOutParam()));
+
+    data = (BYTE*) RTMemTmpAllocZ(100*100*4);
+
+    for (i=0; i<100; i+=2)
+    {
+        for (j=0; j<100; ++j)
+        {
+            *(data+i*100*4+j*4+0) = 0xFF;
+            *(data+i*100*4+j*4+1) = 0xFF;
+            *(data+i*100*4+j*4+2) = 0xFF;
+            *(data+i*100*4+j*4+3) = 0xFF;
+        }
+    }
+
+    CHECK_ERROR(pDisplay, DrawToScreen(screenId, data, 0, 0, 100, 100));
+
+    RTMemTmpFree(data);
+#endif
+    HRESULT rc;
+    ComPtr<IDisplay> pDisplay;
+
+    CHECK_ERROR(g_pConsole, COMGETTER(Display)(pDisplay.asOutParam()));
+    CHECK_ERROR(pDisplay, DrawToScreen(screenId, (BYTE*)data, x, y, w, h));
 }
 
 static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32_t u32ClientID, void *pvClient, uint32_t u32Function, uint32_t cParms, VBOXHGCMSVCPARM paParms[])
 {
     int rc = VINF_SUCCESS;
 
-    Log(("SHARED_CROPENGL svcCall: u32ClientID = %d, fn = %d, cParms = %d, pparms = %d\n", u32ClientID, u32Function, cParms, paParms));
+    NOREF(pvClient);
 
-    VBOXOGLCTX *pClient = (VBOXOGLCTX *)pvClient;
+    Log(("SHARED_CROPENGL svcCall: u32ClientID = %d, fn = %d, cParms = %d, pparms = %d\n", u32ClientID, u32Function, cParms, paParms));
 
 #ifdef DEBUG
     uint32_t i;
@@ -363,6 +359,7 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
                 /* Return the required buffer size always */
                 paParms[2].u.uint32 = cbWriteback;
             }
+
             break;
         }
 
@@ -393,7 +390,6 @@ static DECLCALLBACK(void) svcCall (void *, VBOXHGCMCALLHANDLE callHandle, uint32
 
                 if (!RT_SUCCESS(rc))
                 {
-                    /*@todo, add warning window*/
                     svcClientVersionUnsupported(vMajor, vMinor);
                 }
             }
@@ -434,12 +430,12 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
 
     switch (u32Function)
     {
-        case SHCRGL_HOST_FN_SET_FRAMEBUFFER:
+        case SHCRGL_HOST_FN_SET_CONSOLE:
         {
-            Log(("svcCall: SHCRGL_HOST_FN_SET_FRAMEBUFFER\n"));
+            Log(("svcCall: SHCRGL_HOST_FN_SET_DISPLAY\n"));
 
             /* Verify parameter count and types. */
-            if (cParms != SHCRGL_CPARMS_SET_FRAMEBUFFER)
+            if (cParms != SHCRGL_CPARMS_SET_CONSOLE)
             {
                 rc = VERR_INVALID_PARAMETER;
             }
@@ -450,18 +446,88 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
             else
             {
                 /* Fetch parameters. */
-                IFramebuffer* pFrameBuffer = (IFramebuffer*)paParms[0].u.pointer.addr;
+                IConsole* pConsole = (IConsole*)paParms[0].u.pointer.addr;
                 uint32_t  cbData = paParms[0].u.pointer.size;
 
                 /* Verify parameters values. */
-                if (cbData != sizeof (IFramebuffer*))
+                if (cbData != sizeof (IConsole*))
+                {
+                    rc = VERR_INVALID_PARAMETER;
+                }
+                else if (!pConsole)
+                {
+                    rc = VERR_INVALID_PARAMETER;
+                }
+                else /* Execute the function. */
+                {
+                    ComPtr<IMachine> pMachine;
+                    ComPtr<IDisplay> pDisplay;
+                    ComPtr<IFramebuffer> pFramebuffer;
+                    LONG xo, yo;
+                    ULONG64 winId = 0;
+                    ULONG monitorCount, i, w, h;
+
+                    CHECK_ERROR_BREAK(pConsole, COMGETTER(Machine)(pMachine.asOutParam()));
+                    CHECK_ERROR_BREAK(pMachine, COMGETTER(MonitorCount)(&monitorCount));
+                    CHECK_ERROR_BREAK(pConsole, COMGETTER(Display)(pDisplay.asOutParam()));
+
+                    rc = crVBoxServerSetScreenCount(monitorCount);
+                    AssertRCReturn(rc, rc);
+
+                    for (i=0; i<monitorCount; ++i)
+                    {
+                        CHECK_ERROR_RET(pDisplay, GetFramebuffer(i, pFramebuffer.asOutParam(), &xo, &yo), rc);
+
+                        if (!pDisplay)
+                        {
+                            rc = crVBoxServerUnmapScreen(i);
+                            AssertRCReturn(rc, rc);
+                        }
+                        else
+                        {
+                            CHECK_ERROR_RET(pFramebuffer, COMGETTER(WinId)(&winId), rc);
+                            CHECK_ERROR_RET(pFramebuffer, COMGETTER(Width)(&w), rc);
+                            CHECK_ERROR_RET(pFramebuffer, COMGETTER(Height)(&h), rc);
+
+                            rc = crVBoxServerMapScreen(i, xo, yo, w, h, winId);
+                            AssertRCReturn(rc, rc);
+                        }
+                    }
+
+                    g_pConsole = pConsole;
+                    rc = VINF_SUCCESS;
+                }
+            }
+            break;
+        }
+        case SHCRGL_HOST_FN_SET_VM:
+        {
+            Log(("svcCall: SHCRGL_HOST_FN_SET_VM\n"));
+
+            /* Verify parameter count and types. */
+            if (cParms != SHCRGL_CPARMS_SET_VM)
+            {
+                rc = VERR_INVALID_PARAMETER;
+            }
+            else if (paParms[0].type != VBOX_HGCM_SVC_PARM_PTR)
+            {
+                rc = VERR_INVALID_PARAMETER;
+            }
+            else
+            {
+                /* Fetch parameters. */
+                PVM pVM = (PVM)paParms[0].u.pointer.addr;
+                uint32_t  cbData = paParms[0].u.pointer.size;
+
+                /* Verify parameters values. */
+                if (cbData != sizeof (PVM))
                 {
                     rc = VERR_INVALID_PARAMETER;
                 }
                 else
                 {
                     /* Execute the function. */
-                    g_pFrameBuffer = pFrameBuffer;
+                    g_pVM = pVM;
                     rc = VINF_SUCCESS;
                 }
             }
@@ -487,7 +553,55 @@ static DECLCALLBACK(int) svcHostCall (void *, uint32_t u32Function, uint32_t cPa
 
             Assert(sizeof(RTRECT)==4*sizeof(GLint));
 
-            renderspuSetRootVisibleRegion(paParms[1].u.uint32, (GLint*)paParms[0].u.pointer.addr);
+            rc = crVBoxServerSetRootVisibleRegion(paParms[1].u.uint32, (GLint*)paParms[0].u.pointer.addr);
+            break;
+        }
+        case SHCRGL_HOST_FN_SCREEN_CHANGED:
+        {
+            Log(("svcCall: SHCRGL_HOST_FN_SCREEN_CHANGED\n"));
+
+            /* Verify parameter count and types. */
+            if (cParms != SHCRGL_CPARMS_SCREEN_CHANGED)
+            {
+                rc = VERR_INVALID_PARAMETER;
+            }
+            else if (paParms[0].type != VBOX_HGCM_SVC_PARM_32BIT)
+            {
+                rc = VERR_INVALID_PARAMETER;
+            }
+            else
+            {
+                /* Fetch parameters. */
+                uint32_t screenId = paParms[0].u.uint32;
+
+                /* Execute the function. */
+                ComPtr<IDisplay> pDisplay;
+                ComPtr<IFramebuffer> pFramebuffer;
+                LONG xo, yo;
+                ULONG64 winId = 0;
+                ULONG w, h;
+
+                Assert(g_pConsole);
+                CHECK_ERROR_RET(g_pConsole, COMGETTER(Display)(pDisplay.asOutParam()), rc);
+                CHECK_ERROR_RET(pDisplay, GetFramebuffer(screenId, pFramebuffer.asOutParam(), &xo, &yo), rc);
+
+                if (!pFramebuffer)
+                {
+                    rc = crVBoxServerUnmapScreen(screenId);
+                    AssertRCReturn(rc, rc);
+                }
+                else
+                {
+                    CHECK_ERROR_RET(pFramebuffer, COMGETTER(WinId)(&winId), rc);
+                    CHECK_ERROR_RET(pFramebuffer, COMGETTER(Width)(&w), rc);
+                    CHECK_ERROR_RET(pFramebuffer, COMGETTER(Height)(&h), rc);
+
+                    rc = crVBoxServerMapScreen(screenId, xo, yo, w, h, winId);
+                    AssertRCReturn(rc, rc);
+                }
+
+                rc = VINF_SUCCESS;
+            }
             break;
         }
         default:
@@ -522,7 +636,7 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
         {
             g_pHelpers = ptable->pHelpers;
 
-            ptable->cbClient = sizeof (VBOXOGLCTX);
+            ptable->cbClient = sizeof (void*);
 
             ptable->pfnUnload     = svcUnload;
             ptable->pfnConnect    = svcConnect;
@@ -532,12 +646,14 @@ extern "C" DECLCALLBACK(DECLEXPORT(int)) VBoxHGCMSvcLoad (VBOXHGCMSVCFNTABLE *pt
             ptable->pfnSaveState  = svcSaveState;
             ptable->pfnLoadState  = svcLoadState;
             ptable->pvService     = NULL;
-#ifdef CR_USE_HGCM
+
             if (!crVBoxServerInit())
                 return VERR_NOT_SUPPORTED;
-#endif
+
+            crVBoxServerSetPresentFBOCB(svcPresentFBO);
         }
     }
 
     return rc;
 }
+
