@@ -5,7 +5,7 @@
 #
 
 #
-# Copyright (C) 2007-2009 Sun Microsystems, Inc.
+# Copyright (C) 2007-2009 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
 # VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 # hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 #
-# Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
-# Clara, CA 95054 USA or visit http://www.sun.com if you need
-# additional information or have any questions.
-#
 
 #
 # Usage:
@@ -98,12 +94,16 @@ symlink_fixup()
 # Prepare file list
 cd "$PKG_BASE_DIR"
 echo 'i pkginfo=./vbox.pkginfo' > prototype
+echo 'i checkinstall=./checkinstall.sh' >> prototype
 echo 'i postinstall=./postinstall.sh' >> prototype
 echo 'i preremove=./preremove.sh' >> prototype
 echo 'i space=./vbox.space' >> prototype
 if test -f "./vbox.copyright"; then
     echo 'i copyright=./vbox.copyright' >> prototype
 fi
+if test -f "./vbox.depend"; then
+    echo 'i depend=./vbox.depend' >> prototype
+fi
 
 # Create relative hardlinks
 cd "$VBOX_INSTALLED_DIR"
@@ -112,6 +112,10 @@ ln -f ./VBoxISAExec $VBOX_INSTALLED_DIR/VBoxSDL
 ln -f ./VBoxISAExec $VBOX_INSTALLED_DIR/vboxwebsrv
 ln -f ./VBoxISAExec $VBOX_INSTALLED_DIR/webtest
 ln -f ./VBoxISAExec $VBOX_INSTALLED_DIR/VBoxZoneAccess
+if test -f $VBOX_INSTALLED_DIR/amd64/VBoxTestOGL || test -f $VBOX_INSTALLED_DIR/i386/VBoxTestOGL; then
+    ln -f ./VBoxISAExec $VBOX_INSTALLED_DIR/VBoxTestOGL
+fi
+
 if test -f $VBOX_INSTALLED_DIR/amd64/VirtualBox || test -f $VBOX_INSTALLED_DIR/i386/VirtualBox; then
     ln -f ./VBoxISAExec $VBOX_INSTALLED_DIR/VirtualBox
 fi
@@ -125,7 +129,10 @@ fi
 
 # Exclude directories to not cause install-time conflicts with existing system directories
 cd "$PKG_BASE_DIR"
-find . ! -type d | $VBOX_GGREP -v -E 'prototype|makepackage.sh|vbox.pkginfo|postinstall.sh|preremove.sh|ReadMe.txt|vbox.space|vbox.copyright|VirtualBoxKern' | pkgproto >> prototype
+find . ! -type d | $VBOX_GGREP -v -E 'prototype|makepackage.sh|vbox.pkginfo|postinstall.sh|checkinstall.sh|preremove.sh|ReadMe.txt|vbox.space|vbox.depend|vbox.copyright|VirtualBoxKern' | pkgproto >> prototype
+
+# Include only opt/VirtualBox and subdirectories as we want uninstall to clean up directory structure as well
+find . -type d | $VBOX_GGREP -E 'opt/VirtualBox' | pkgproto >> prototype
 
 # fix up file permissions (owner/group)
 # don't grok for class-specific files (like sed, if any)
@@ -152,6 +159,10 @@ filelist_fixup prototype '$3 == "platform/i86pc/kernel/drv/amd64/vboxnet"'
 filelist_fixup prototype '$3 == "platform/i86pc/kernel/drv/vboxusbmon"'                                '$6 = "sys"'
 filelist_fixup prototype '$3 == "platform/i86pc/kernel/drv/amd64/vboxusbmon"'                          '$6 = "sys"'
 
+# USB Client vboxusb
+filelist_fixup prototype '$3 == "platform/i86pc/kernel/drv/vboxusb"'                                   '$6 = "sys"'
+filelist_fixup prototype '$3 == "platform/i86pc/kernel/drv/amd64/vboxusb"'                             '$6 = "sys"'
+
 # hardening requires some executables to be marked setuid.
 if test -n "$HARDENED"; then
     $VBOX_AWK 'NF == 6 \
