@@ -1,7 +1,7 @@
 #!/sbin/sh
-# $Id: smf-vboxwebsrv.sh $
+# $Id: smf-vboxwebsrv.sh 28800 2010-04-27 08:22:32Z vboxsync $
 
-# Copyright (C) 2008 Sun Microsystems, Inc.
+# Copyright (C) 2008 Oracle Corporation
 #
 # This file is part of VirtualBox Open Source Edition (OSE), as
 # available from http://www.virtualbox.org. This file is free software;
@@ -11,10 +11,6 @@
 # VirtualBox OSE distribution. VirtualBox OSE is distributed in the
 # hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
 #
-# Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
-# Clara, CA 95054 USA or visit http://www.sun.com if you need
-# additional information or have any questions.
-#
 
 #
 # smf-vboxwebsrv method
@@ -56,42 +52,13 @@ case $VW_OPT in
         [ -z "$VW_PORT" -o "$VW_PORT" -eq 0 ] && VW_PORT=18083
         [ -z "$VW_TIMEOUT" ] && VW_TIMEOUT=20
         [ -z "$VW_CHECK_INTERVAL" ] && VW_CHECK_INTERVAL=5
-        su - "$VW_USER" -c "/opt/VirtualBox/vboxwebsrv --background --host \"$VW_HOST\" --port \"$VW_PORT\" --timeout \"$VW_TIMEOUT\" --check-interval \"$VW_CHECK_INTERVAL\""
+        exec su - "$VW_USER" -c "/opt/VirtualBox/vboxwebsrv --host \"$VW_HOST\" --port \"$VW_PORT\" --timeout \"$VW_TIMEOUT\" --check-interval \"$VW_CHECK_INTERVAL\""
 
         VW_EXIT=$?
         if [ $VW_EXIT != 0 ]; then
             echo "vboxwebsrv failed with $VW_EXIT."
             VW_EXIT=1
         fi
-
-        # Bump per-process semaphore limit of VBoxSVC
-        PRCTLBIN=`which prctl`
-        if test ! -f "$PRTCLBIN"; then
-            # Wait for VBoxSVC to spawn
-            TRIES=0
-            while test $TRIES -le 3; do
-                VBOXSVC_PID=`ps -eo pid,fname | grep VBoxSVC | grep -v grep | cut -f 1 -d " "`
-                if test $VBOXSVC_PID -ge 0; then
-                    $PRCTLBIN -r -n project.max-sem-ids -v 1024 $VBOXSVC_PID
-                    if test $? -eq 0; then
-                        echo "Successfully bumped VBoxSVC (pid $VBOXSVC_PID) semaphore id limit to 1024."
-                    else
-                        echo "Failed to bump VBoxSVC (pid $VBOXSVC_PID) semaphore id limit."
-                    fi
-                    break
-                else
-                    sleep 1
-                fi
-                TRIES=`expr $TRIES + 1`
-            done
-            if test $TRIES -eq 3; then
-                echo "Stopped waiting for VBoxSVC to spawn..."
-                echo "Failed to bump VBoxSVC process' semaphore id limit."
-            fi
-        else
-            echo "Failed to find prctl to bump VBoxSVC semaphore id limit."
-            echo "As a result, not more than 100 VMs can be started."
-        fi
     ;;
     stop)
         # Kill service contract
