@@ -1,4 +1,4 @@
-/* $Id: DHCPServerImpl.cpp $ */
+/* $Id: DHCPServerImpl.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,22 +15,28 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "DHCPServerRunner.h"
 #include "DHCPServerImpl.h"
+#include "AutoCaller.h"
 #include "Logging.h"
 
 #include <VBox/settings.h>
 
+#include "VirtualBoxImpl.h"
+
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
-DEFINE_EMPTY_CTOR_DTOR (DHCPServer)
+DHCPServer::DHCPServer()
+    : mVirtualBox(NULL)
+{
+}
+
+DHCPServer::~DHCPServer()
+{
+}
 
 HRESULT DHCPServer::FinalConstruct()
 {
@@ -45,28 +51,22 @@ void DHCPServer::FinalRelease()
 void DHCPServer::uninit()
 {
     /* Enclose the state transition Ready->InUninit->NotReady */
-    AutoUninitSpan autoUninitSpan (this);
+    AutoUninitSpan autoUninitSpan(this);
     if (autoUninitSpan.uninitDone())
         return;
 
-//    /* we uninit children and reset mParent
-//     * and VirtualBox::removeDependentChild() needs a write lock */
-//    AutoMultiWriteLock2 alock (mVirtualBox->lockHandle(), this->treeLock());
-
-    mVirtualBox->removeDependentChild (this);
-
-    unconst (mVirtualBox).setNull();
+    unconst(mVirtualBox) = NULL;
 }
 
 HRESULT DHCPServer::init(VirtualBox *aVirtualBox, IN_BSTR aName)
 {
-    AssertReturn (aName != NULL, E_INVALIDARG);
+    AssertReturn(aName != NULL, E_INVALIDARG);
 
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
     /* share VirtualBox weakly (parent remains NULL so far) */
-    unconst (mVirtualBox) = aVirtualBox;
+    unconst(mVirtualBox) = aVirtualBox;
 
     unconst(mName) = aName;
     m.IPAddress = "0.0.0.0";
@@ -75,60 +75,47 @@ HRESULT DHCPServer::init(VirtualBox *aVirtualBox, IN_BSTR aName)
     m.lowerIP = "0.0.0.0";
     m.upperIP = "0.0.0.0";
 
-    /* register with VirtualBox early, since uninit() will
-     * unconditionally unregister on failure */
-    aVirtualBox->addDependentChild (this);
-
     /* Confirm a successful initialization */
     autoInitSpan.setSucceeded();
 
     return S_OK;
 }
 
-HRESULT DHCPServer::init(VirtualBox *aVirtualBox, const settings::Key &aNode)
+HRESULT DHCPServer::init(VirtualBox *aVirtualBox,
+                         const settings::DHCPServer &data)
 {
-    using namespace settings;
-
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
     /* share VirtualBox weakly (parent remains NULL so far) */
-    unconst (mVirtualBox) = aVirtualBox;
+    unconst(mVirtualBox) = aVirtualBox;
 
-    aVirtualBox->addDependentChild (this);
-
-    unconst(mName) = aNode.stringValue ("networkName");
-    m.IPAddress = aNode.stringValue ("IPAddress");
-    m.networkMask = aNode.stringValue ("networkMask");
-    m.enabled = aNode.value <BOOL> ("enabled");
-    m.lowerIP = aNode.stringValue ("lowerIP");
-    m.upperIP = aNode.stringValue ("upperIP");
+    unconst(mName) = data.strNetworkName;
+    m.IPAddress = data.strIPAddress;
+    m.networkMask = data.strIPNetworkMask;
+    m.enabled = data.fEnabled;
+    m.lowerIP = data.strIPLower;
+    m.upperIP = data.strIPUpper;
 
     autoInitSpan.setSucceeded();
 
     return S_OK;
 }
 
-HRESULT DHCPServer::saveSettings (settings::Key &aParentNode)
+HRESULT DHCPServer::saveSettings(settings::DHCPServer &data)
 {
-    using namespace settings;
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AssertReturn (!aParentNode.isNull(), E_FAIL);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-
-    AutoReadLock alock (this);
-
-    Key aNode = aParentNode.appendKey ("DHCPServer");
-    /* required */
-    aNode.setValue <Bstr> ("networkName", mName);
-    aNode.setValue <Bstr> ("IPAddress", m.IPAddress);
-    aNode.setValue <Bstr> ("networkMask", m.networkMask);
-    aNode.setValue <Bstr> ("lowerIP", m.lowerIP);
-    aNode.setValue <Bstr> ("upperIP", m.upperIP);
-    aNode.setValue <BOOL> ("enabled", m.enabled);
+    data.strNetworkName = mName;
+    data.strIPAddress = m.IPAddress;
+    data.strIPNetworkMask = m.networkMask;
+    data.fEnabled = !!m.enabled;
+    data.strIPLower = m.lowerIP;
+    data.strIPUpper = m.upperIP;
 
     return S_OK;
 }
@@ -137,8 +124,8 @@ STDMETHODIMP DHCPServer::COMGETTER(NetworkName) (BSTR *aName)
 {
     CheckComArgOutPointerValid(aName);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     mName.cloneTo(aName);
 
@@ -149,8 +136,8 @@ STDMETHODIMP DHCPServer::COMGETTER(Enabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aEnabled = m.enabled;
 
@@ -159,11 +146,11 @@ STDMETHODIMP DHCPServer::COMGETTER(Enabled) (BOOL *aEnabled)
 
 STDMETHODIMP DHCPServer::COMSETTER(Enabled) (BOOL aEnabled)
 {
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* VirtualBox::saveSettings() needs a write lock */
-    AutoMultiWriteLock2 alock (mVirtualBox, this);
+    AutoMultiWriteLock2 alock(mVirtualBox, this COMMA_LOCKVAL_SRC_POS);
 
     m.enabled = aEnabled;
 
@@ -176,8 +163,8 @@ STDMETHODIMP DHCPServer::COMGETTER(IPAddress) (BSTR *aIPAddress)
 {
     CheckComArgOutPointerValid(aIPAddress);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     m.IPAddress.cloneTo(aIPAddress);
 
@@ -188,8 +175,8 @@ STDMETHODIMP DHCPServer::COMGETTER(NetworkMask) (BSTR *aNetworkMask)
 {
     CheckComArgOutPointerValid(aNetworkMask);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     m.networkMask.cloneTo(aNetworkMask);
 
@@ -200,8 +187,8 @@ STDMETHODIMP DHCPServer::COMGETTER(LowerIP) (BSTR *aIPAddress)
 {
     CheckComArgOutPointerValid(aIPAddress);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     m.lowerIP.cloneTo(aIPAddress);
 
@@ -212,8 +199,8 @@ STDMETHODIMP DHCPServer::COMGETTER(UpperIP) (BSTR *aIPAddress)
 {
     CheckComArgOutPointerValid(aIPAddress);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     m.upperIP.cloneTo(aIPAddress);
 
@@ -222,16 +209,16 @@ STDMETHODIMP DHCPServer::COMGETTER(UpperIP) (BSTR *aIPAddress)
 
 STDMETHODIMP DHCPServer::SetConfiguration (IN_BSTR aIPAddress, IN_BSTR aNetworkMask, IN_BSTR aLowerIP, IN_BSTR aUpperIP)
 {
-    AssertReturn (aIPAddress != NULL, E_INVALIDARG);
-    AssertReturn (aNetworkMask != NULL, E_INVALIDARG);
-    AssertReturn (aLowerIP != NULL, E_INVALIDARG);
-    AssertReturn (aUpperIP != NULL, E_INVALIDARG);
+    AssertReturn(aIPAddress != NULL, E_INVALIDARG);
+    AssertReturn(aNetworkMask != NULL, E_INVALIDARG);
+    AssertReturn(aLowerIP != NULL, E_INVALIDARG);
+    AssertReturn(aUpperIP != NULL, E_INVALIDARG);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /* VirtualBox::saveSettings() needs a write lock */
-    AutoMultiWriteLock2 alock (mVirtualBox, this);
+    AutoMultiWriteLock2 alock(mVirtualBox, this COMMA_LOCKVAL_SRC_POS);
 
     m.IPAddress = aIPAddress;
     m.networkMask = aNetworkMask;
@@ -241,37 +228,37 @@ STDMETHODIMP DHCPServer::SetConfiguration (IN_BSTR aIPAddress, IN_BSTR aNetworkM
     return mVirtualBox->saveSettings();
 }
 
-STDMETHODIMP DHCPServer::Start (IN_BSTR aNetworkName, IN_BSTR aTrunkName, IN_BSTR aTrunkType)
+STDMETHODIMP DHCPServer::Start(IN_BSTR aNetworkName, IN_BSTR aTrunkName, IN_BSTR aTrunkType)
 {
     /* Silently ignore attepmts to run disabled servers. */
     if (!m.enabled)
         return S_OK;
 
-    m.dhcp.setOption(DHCPCFG_NETNAME, Utf8Str(aNetworkName));
+    m.dhcp.setOption(DHCPCFG_NETNAME, Utf8Str(aNetworkName), true);
     Bstr tmp(aTrunkName);
     if (!tmp.isEmpty())
-        m.dhcp.setOption(DHCPCFG_TRUNKNAME, Utf8Str(tmp));
-    m.dhcp.setOption(DHCPCFG_TRUNKTYPE, Utf8Str(aTrunkType));
+        m.dhcp.setOption(DHCPCFG_TRUNKNAME, Utf8Str(tmp), true);
+    m.dhcp.setOption(DHCPCFG_TRUNKTYPE, Utf8Str(aTrunkType), true);
     //temporary hack for testing
     //    DHCPCFG_NAME
-    char strMAC[13];
+    char strMAC[32];
     Guid guid;
     guid.create();
-    RTStrPrintf (strMAC, sizeof(strMAC), "080027%02X%02X%02X",
+    RTStrPrintf (strMAC, sizeof(strMAC), "08:00:27:%02X:%02X:%02X",
                  guid.ptr()->au8[0], guid.ptr()->au8[1], guid.ptr()->au8[2]);
-    m.dhcp.setOption(DHCPCFG_MACADDRESS, strMAC);
-    m.dhcp.setOption(DHCPCFG_IPADDRESS,  Utf8Str(m.IPAddress));
+    m.dhcp.setOption(DHCPCFG_MACADDRESS, strMAC, true);
+    m.dhcp.setOption(DHCPCFG_IPADDRESS,  Utf8Str(m.IPAddress), true);
     //        DHCPCFG_LEASEDB,
     //        DHCPCFG_VERBOSE,
     //        DHCPCFG_GATEWAY,
-    m.dhcp.setOption(DHCPCFG_LOWERIP,  Utf8Str(m.lowerIP));
-    m.dhcp.setOption(DHCPCFG_UPPERIP,  Utf8Str(m.upperIP));
-    m.dhcp.setOption(DHCPCFG_NETMASK,  Utf8Str(m.networkMask));
+    m.dhcp.setOption(DHCPCFG_LOWERIP,  Utf8Str(m.lowerIP), true);
+    m.dhcp.setOption(DHCPCFG_UPPERIP,  Utf8Str(m.upperIP), true);
+    m.dhcp.setOption(DHCPCFG_NETMASK,  Utf8Str(m.networkMask), true);
 
     //        DHCPCFG_HELP,
     //        DHCPCFG_VERSION,
     //        DHCPCFG_NOTOPT_MAXVAL
-    m.dhcp.setOption(DHCPCFG_BEGINCONFIG,  "");
+    m.dhcp.setOption(DHCPCFG_BEGINCONFIG,  "", true);
 
     return RT_FAILURE(m.dhcp.start()) ? E_FAIL : S_OK;
     //m.dhcp.detachFromServer(); /* need to do this to avoid server shutdown on runner destruction */
