@@ -1,4 +1,4 @@
-/* $Id: HostNetworkInterfaceImpl.cpp $ */
+/* $Id: HostNetworkInterfaceImpl.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,24 +15,28 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "HostNetworkInterfaceImpl.h"
+#include "AutoCaller.h"
 #include "Logging.h"
 #include "netif.h"
 
-#ifndef RT_OS_WINDOWS
-#include <arpa/inet.h>
-#endif /* RT_OS_WINDOWS */
+#ifdef RT_OS_FREEBSD
+# include <netinet/in.h> /* INADDR_NONE */
+#endif /* RT_OS_FREEBSD */
 
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
-DEFINE_EMPTY_CTOR_DTOR (HostNetworkInterface)
+HostNetworkInterface::HostNetworkInterface()
+    : mVBox(NULL)
+{
+}
+
+HostNetworkInterface::~HostNetworkInterface()
+{
+}
 
 HRESULT HostNetworkInterface::FinalConstruct()
 {
@@ -56,18 +60,18 @@ void HostNetworkInterface::FinalRelease()
  */
 HRESULT HostNetworkInterface::init (Bstr aInterfaceName, Guid aGuid, HostNetworkInterfaceType_T ifType)
 {
-    LogFlowThisFunc (("aInterfaceName={%ls}, aGuid={%s}\n",
+    LogFlowThisFunc(("aInterfaceName={%ls}, aGuid={%s}\n",
                       aInterfaceName.raw(), aGuid.toString().raw()));
 
-    ComAssertRet (aInterfaceName, E_INVALIDARG);
-    ComAssertRet (!aGuid.isEmpty(), E_INVALIDARG);
+    ComAssertRet(aInterfaceName, E_INVALIDARG);
+    ComAssertRet(!aGuid.isEmpty(), E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst (mInterfaceName) = aInterfaceName;
-    unconst (mGuid) = aGuid;
+    unconst(mInterfaceName) = aInterfaceName;
+    unconst(mGuid) = aGuid;
     mIfType = ifType;
 
 
@@ -83,7 +87,7 @@ HRESULT HostNetworkInterface::updateConfig ()
 {
     NETIFINFO info;
     int rc = NetIfGetConfig(this, &info);
-    if(RT_SUCCESS(rc))
+    if (RT_SUCCESS(rc))
     {
         m.realIPAddress = m.IPAddress = info.IPAddress.u;
         m.realNetworkMask = m.networkMask = info.IPNetMask.u;
@@ -113,19 +117,19 @@ HRESULT HostNetworkInterface::updateConfig ()
  */
 HRESULT HostNetworkInterface::init (Bstr aInterfaceName, HostNetworkInterfaceType_T ifType, PNETIFINFO pIf)
 {
-//    LogFlowThisFunc (("aInterfaceName={%ls}, aGuid={%s}\n",
+//    LogFlowThisFunc(("aInterfaceName={%ls}, aGuid={%s}\n",
 //                      aInterfaceName.raw(), aGuid.toString().raw()));
 
-//    ComAssertRet (aInterfaceName, E_INVALIDARG);
-//    ComAssertRet (!aGuid.isEmpty(), E_INVALIDARG);
-    ComAssertRet (pIf, E_INVALIDARG);
+//    ComAssertRet(aInterfaceName, E_INVALIDARG);
+//    ComAssertRet(!aGuid.isEmpty(), E_INVALIDARG);
+    ComAssertRet(pIf, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst (mInterfaceName) = aInterfaceName;
-    unconst (mGuid) = pIf->Uuid;
+    unconst(mInterfaceName) = aInterfaceName;
+    unconst(mGuid) = pIf->Uuid;
     mIfType = ifType;
 
     m.realIPAddress = m.IPAddress = pIf->IPAddress.u;
@@ -162,10 +166,10 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(Name) (BSTR *aInterfaceName)
 {
     CheckComArgOutPointerValid(aInterfaceName);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    mInterfaceName.cloneTo (aInterfaceName);
+    mInterfaceName.cloneTo(aInterfaceName);
 
     return S_OK;
 }
@@ -176,14 +180,14 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(Name) (BSTR *aInterfaceName)
  * @returns COM status code
  * @param   aGuid address of result pointer
  */
-STDMETHODIMP HostNetworkInterface::COMGETTER(Id) (OUT_GUID aGuid)
+STDMETHODIMP HostNetworkInterface::COMGETTER(Id) (BSTR *aGuid)
 {
     CheckComArgOutPointerValid(aGuid);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    mGuid.cloneTo (aGuid);
+    mGuid.toUtf16().cloneTo(aGuid);
 
     return S_OK;
 }
@@ -192,8 +196,8 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(DhcpEnabled) (BOOL *aDhcpEnabled)
 {
     CheckComArgOutPointerValid(aDhcpEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aDhcpEnabled = m.dhcpEnabled;
 
@@ -211,12 +215,12 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPAddress) (BSTR *aIPAddress)
 {
     CheckComArgOutPointerValid(aIPAddress);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     if (m.IPAddress == 0)
     {
-        Bstr(VBOXNET_IPV4ADDR_DEFAULT).detachTo(aIPAddress);
+        getDefaultIPv4Address(mInterfaceName).detachTo(aIPAddress);
         return S_OK;
     }
 
@@ -227,7 +231,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPAddress) (BSTR *aIPAddress)
     tmp.s_addr = m.IPAddress;
 #endif
     char *addr = inet_ntoa(tmp);
-    if(addr)
+    if (addr)
     {
         Bstr(addr).detachTo(aIPAddress);
         return S_OK;
@@ -246,8 +250,8 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(NetworkMask) (BSTR *aNetworkMask)
 {
     CheckComArgOutPointerValid(aNetworkMask);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     if (m.networkMask == 0)
     {
@@ -262,7 +266,7 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(NetworkMask) (BSTR *aNetworkMask)
     tmp.s_addr = m.networkMask;
 #endif
     char *addr = inet_ntoa(tmp);
-    if(addr)
+    if (addr)
     {
         Bstr(addr).detachTo(aNetworkMask);
         return S_OK;
@@ -293,10 +297,10 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPV6Address) (BSTR *aIPV6Address)
 {
     CheckComArgOutPointerValid(aIPV6Address);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    m.IPV6Address.cloneTo (aIPV6Address);
+    m.IPV6Address.cloneTo(aIPV6Address);
 
     return S_OK;
 }
@@ -311,8 +315,8 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(IPV6NetworkMaskPrefixLength) (ULONG
 {
     CheckComArgOutPointerValid(aIPV6NetworkMaskPrefixLength);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aIPV6NetworkMaskPrefixLength = m.IPV6NetworkMaskPrefixLength;
 
@@ -329,10 +333,10 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(HardwareAddress) (BSTR *aHardwareAd
 {
     CheckComArgOutPointerValid(aHardwareAddress);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    m.hardwareAddress.cloneTo (aHardwareAddress);
+    m.hardwareAddress.cloneTo(aHardwareAddress);
 
     return S_OK;
 }
@@ -347,8 +351,8 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(MediumType) (HostNetworkInterfaceMe
 {
     CheckComArgOutPointerValid(aType);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aType = m.mediumType;
 
@@ -365,8 +369,8 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(Status) (HostNetworkInterfaceStatus
 {
     CheckComArgOutPointerValid(aStatus);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aStatus = m.status;
 
@@ -383,8 +387,8 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(InterfaceType) (HostNetworkInterfac
 {
     CheckComArgOutPointerValid(aType);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     *aType = mIfType;
 
@@ -394,13 +398,13 @@ STDMETHODIMP HostNetworkInterface::COMGETTER(InterfaceType) (HostNetworkInterfac
 
 STDMETHODIMP HostNetworkInterface::COMGETTER(NetworkName) (BSTR *aNetworkName)
 {
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     Utf8Str utf8Name("HostInterfaceNetworking-");
     utf8Name.append(Utf8Str(mInterfaceName)) ;
     Bstr netName(utf8Name);
-    netName.detachTo (aNetworkName);
+    netName.detachTo(aNetworkName);
 
     return S_OK;
 }
@@ -410,8 +414,8 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
 #ifndef VBOX_WITH_HOSTNETIF_API
     return E_NOTIMPL;
 #else
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     if (Bstr(aIPAddress).isEmpty())
     {
@@ -421,9 +425,9 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
             if (RT_SUCCESS(rc))
             {
                 m.realIPAddress = 0;
-                if (FAILED(mVBox->SetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw())), Bstr(""))))
+                if (FAILED(mVBox->SetExtraData(BstrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw()), NULL)))
                     return E_FAIL;
-                if (FAILED(mVBox->SetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPNetMask", mInterfaceName.raw())), Bstr(""))))
+                if (FAILED(mVBox->SetExtraData(BstrFmt("HostOnly/%ls/IPNetMask", mInterfaceName.raw()), NULL)))
                     return E_FAIL;
                 return S_OK;
             }
@@ -434,13 +438,13 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
 
     ULONG ip, mask;
     ip = inet_addr(Utf8Str(aIPAddress).raw());
-    if(ip != INADDR_NONE)
+    if (ip != INADDR_NONE)
     {
         if (Bstr(aNetMask).isEmpty())
             mask = 0xFFFFFF;
         else
             mask = inet_addr(Utf8Str(aNetMask).raw());
-        if(mask != INADDR_NONE)
+        if (mask != INADDR_NONE)
         {
             if (m.realIPAddress == ip && m.realNetworkMask == mask)
                 return S_OK;
@@ -457,7 +461,7 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfig (IN_BSTR aIPAddress, IN_
             }
             else
             {
-                LogRel(("Failed to EnableStaticIpConfig with rc=%Vrc\n", rc));
+                LogRel(("Failed to EnableStaticIpConfig with rc=%Rrc\n", rc));
                 return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
             }
 
@@ -477,8 +481,8 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfigV6 (IN_BSTR aIPV6Address,
     if (aIPV6MaskPrefixLength > 128)
         return E_INVALIDARG;
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     int rc = S_OK;
     if (m.realIPV6Address != aIPV6Address || m.realIPV6PrefixLength != aIPV6MaskPrefixLength)
@@ -488,7 +492,7 @@ STDMETHODIMP HostNetworkInterface::EnableStaticIpConfigV6 (IN_BSTR aIPV6Address,
         rc = NetIfEnableStaticIpConfigV6(mVBox, this, m.IPV6Address, aIPV6Address, aIPV6MaskPrefixLength);
         if (RT_FAILURE(rc))
         {
-            LogRel(("Failed to EnableStaticIpConfigV6 with rc=%Vrc\n", rc));
+            LogRel(("Failed to EnableStaticIpConfigV6 with rc=%Rrc\n", rc));
             return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
         }
         else
@@ -512,13 +516,13 @@ STDMETHODIMP HostNetworkInterface::EnableDynamicIpConfig ()
 #ifndef VBOX_WITH_HOSTNETIF_API
     return E_NOTIMPL;
 #else
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     int rc = NetIfEnableDynamicIpConfig(mVBox, this);
     if (RT_FAILURE(rc))
     {
-        LogRel(("Failed to EnableDynamicIpConfig with rc=%Vrc\n", rc));
+        LogRel(("Failed to EnableDynamicIpConfig with rc=%Rrc\n", rc));
         return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
     }
     return S_OK;
@@ -530,13 +534,13 @@ STDMETHODIMP HostNetworkInterface::DhcpRediscover ()
 #ifndef VBOX_WITH_HOSTNETIF_API
     return E_NOTIMPL;
 #else
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     int rc = NetIfDhcpRediscover(mVBox, this);
     if (RT_FAILURE(rc))
     {
-        LogRel(("Failed to DhcpRediscover with rc=%Vrc\n", rc));
+        LogRel(("Failed to DhcpRediscover with rc=%Rrc\n", rc));
         return rc == VERR_NOT_IMPLEMENTED ? E_NOTIMPL : E_FAIL;
     }
     return S_OK;
@@ -546,9 +550,9 @@ STDMETHODIMP HostNetworkInterface::DhcpRediscover ()
 HRESULT HostNetworkInterface::setVirtualBox(VirtualBox *pVBox)
 {
     HRESULT hrc;
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
-    mVBox = pVBox;
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
+    unconst(mVBox) = pVBox;
 
     /* If IPv4 address hasn't been initialized */
     if (m.IPAddress == 0)
@@ -557,7 +561,7 @@ HRESULT HostNetworkInterface::setVirtualBox(VirtualBox *pVBox)
         hrc = mVBox->GetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPAddress", mInterfaceName.raw())), tmpAddr.asOutParam());
         hrc = mVBox->GetExtraData(Bstr(Utf8StrFmt("HostOnly/%ls/IPNetMask", mInterfaceName.raw())), tmpMask.asOutParam());
         if (tmpAddr.isEmpty())
-            tmpAddr = Bstr(VBOXNET_IPV4ADDR_DEFAULT);
+            tmpAddr = getDefaultIPv4Address(mInterfaceName);
         if (tmpMask.isEmpty())
             tmpMask = Bstr(VBOXNET_IPV4MASK_DEFAULT);
         m.IPAddress = inet_addr(Utf8Str(tmpAddr).raw());
