@@ -1,4 +1,4 @@
-/* $Id: MachineDebuggerImpl.cpp 15827 2009-01-07 10:22:15Z vboxsync $ */
+/* $Id: MachineDebuggerImpl.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,16 +15,14 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "MachineDebuggerImpl.h"
 
 #include "Global.h"
 #include "ConsoleImpl.h"
+
+#include "AutoCaller.h"
 #include "Logging.h"
 
 #include <VBox/em.h>
@@ -46,11 +44,18 @@
 // constructor / destructor
 /////////////////////////////////////////////////////////////////////////////
 
-DEFINE_EMPTY_CTOR_DTOR (MachineDebugger)
+MachineDebugger::MachineDebugger()
+    : mParent(NULL)
+{
+}
+
+MachineDebugger::~MachineDebugger()
+{
+}
 
 HRESULT MachineDebugger::FinalConstruct()
 {
-    unconst (mParent) = NULL;
+    unconst(mParent) = NULL;
     return S_OK;
 }
 
@@ -70,15 +75,15 @@ void MachineDebugger::FinalRelease()
  */
 HRESULT MachineDebugger::init (Console *aParent)
 {
-    LogFlowThisFunc (("aParent=%p\n", aParent));
+    LogFlowThisFunc(("aParent=%p\n", aParent));
 
-    ComAssertRet (aParent, E_INVALIDARG);
+    ComAssertRet(aParent, E_INVALIDARG);
 
     /* Enclose the state transition NotReady->InInit->Ready */
-    AutoInitSpan autoInitSpan (this);
-    AssertReturn (autoInitSpan.isOk(), E_FAIL);
+    AutoInitSpan autoInitSpan(this);
+    AssertReturn(autoInitSpan.isOk(), E_FAIL);
 
-    unconst (mParent) = aParent;
+    unconst(mParent) = aParent;
 
     mSinglestepQueued = ~0;
     mRecompileUserQueued = ~0;
@@ -101,14 +106,14 @@ HRESULT MachineDebugger::init (Console *aParent)
  */
 void MachineDebugger::uninit()
 {
-    LogFlowThisFunc (("\n"));
+    LogFlowThisFunc(("\n"));
 
     /* Enclose the state transition Ready->InUninit->NotReady */
-    AutoUninitSpan autoUninitSpan (this);
+    AutoUninitSpan autoUninitSpan(this);
     if (autoUninitSpan.uninitDone())
         return;
 
-    unconst (mParent).setNull();
+    unconst(mParent) = NULL;
     mFlushMode = false;
 }
 
@@ -125,8 +130,8 @@ STDMETHODIMP MachineDebugger::COMGETTER(Singlestep) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /** @todo */
     ReturnComNotImplemented();
@@ -140,8 +145,8 @@ STDMETHODIMP MachineDebugger::COMGETTER(Singlestep) (BOOL *aEnabled)
  */
 STDMETHODIMP MachineDebugger::COMSETTER(Singlestep) (BOOL aEnable)
 {
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
     /** @todo */
     ReturnComNotImplemented();
@@ -157,10 +162,10 @@ STDMETHODIMP MachineDebugger::COMGETTER(RecompileUser) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
@@ -180,12 +185,12 @@ STDMETHODIMP MachineDebugger::COMGETTER(RecompileUser) (BOOL *aEnabled)
  */
 STDMETHODIMP MachineDebugger::COMSETTER(RecompileUser) (BOOL aEnable)
 {
-    LogFlowThisFunc (("enable=%d\n", aEnable));
+    LogFlowThisFunc(("enable=%d\n", aEnable));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (queueSettings())
     {
@@ -195,19 +200,11 @@ STDMETHODIMP MachineDebugger::COMSETTER(RecompileUser) (BOOL aEnable)
     }
 
     Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    if (FAILED(pVM.rc())) return pVM.rc();
 
-    PVMREQ pReq;
     EMRAWMODE rawModeFlag = aEnable ? EMRAW_RING3_DISABLE : EMRAW_RING3_ENABLE;
-    int rcVBox = VMR3ReqCall (pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT,
-                              (PFNRT)EMR3RawSetMode, 2, pVM.raw(), rawModeFlag);
-    if (RT_SUCCESS (rcVBox))
-    {
-        rcVBox = pReq->iStatus;
-        VMR3ReqFree (pReq);
-    }
-
-    if (RT_SUCCESS (rcVBox))
+    int rcVBox = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)EMR3RawSetMode, 2, pVM.raw(), rawModeFlag);
+    if (RT_SUCCESS(rcVBox))
         return S_OK;
 
     AssertMsgFailed (("Could not set raw mode flags to %d, rcVBox = %Rrc\n",
@@ -225,10 +222,10 @@ STDMETHODIMP MachineDebugger::COMGETTER(RecompileSupervisor) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
@@ -248,12 +245,12 @@ STDMETHODIMP MachineDebugger::COMGETTER(RecompileSupervisor) (BOOL *aEnabled)
  */
 STDMETHODIMP MachineDebugger::COMSETTER(RecompileSupervisor) (BOOL aEnable)
 {
-    LogFlowThisFunc (("enable=%d\n", aEnable));
+    LogFlowThisFunc(("enable=%d\n", aEnable));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (queueSettings())
     {
@@ -263,19 +260,11 @@ STDMETHODIMP MachineDebugger::COMSETTER(RecompileSupervisor) (BOOL aEnable)
     }
 
     Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    if (FAILED(pVM.rc())) return pVM.rc();
 
-    PVMREQ pReq;
     EMRAWMODE rawModeFlag = aEnable ? EMRAW_RING0_DISABLE : EMRAW_RING0_ENABLE;
-    int rcVBox = VMR3ReqCall (pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT,
-                              (PFNRT)EMR3RawSetMode, 2, pVM.raw(), rawModeFlag);
-    if (RT_SUCCESS (rcVBox))
-    {
-        rcVBox = pReq->iStatus;
-        VMR3ReqFree (pReq);
-    }
-
-    if (RT_SUCCESS (rcVBox))
+    int rcVBox = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)EMR3RawSetMode, 2, pVM.raw(), rawModeFlag);
+    if (RT_SUCCESS(rcVBox))
         return S_OK;
 
     AssertMsgFailed (("Could not set raw mode flags to %d, rcVBox = %Rrc\n",
@@ -293,10 +282,10 @@ STDMETHODIMP MachineDebugger::COMGETTER(PATMEnabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
@@ -316,12 +305,12 @@ STDMETHODIMP MachineDebugger::COMGETTER(PATMEnabled) (BOOL *aEnabled)
  */
 STDMETHODIMP MachineDebugger::COMSETTER(PATMEnabled) (BOOL aEnable)
 {
-    LogFlowThisFunc (("enable=%d\n", aEnable));
+    LogFlowThisFunc(("enable=%d\n", aEnable));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (queueSettings())
     {
@@ -330,8 +319,8 @@ STDMETHODIMP MachineDebugger::COMSETTER(PATMEnabled) (BOOL aEnable)
         return S_OK;
     }
 
-    Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    Console::SafeVMPtr pVM(mParent);
+    if (FAILED(pVM.rc())) return pVM.rc();
 
     PATMR3AllowPatching (pVM, aEnable);
 
@@ -348,10 +337,10 @@ STDMETHODIMP MachineDebugger::COMGETTER(CSAMEnabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
@@ -371,12 +360,12 @@ STDMETHODIMP MachineDebugger::COMGETTER(CSAMEnabled) (BOOL *aEnabled)
  */
 STDMETHODIMP MachineDebugger::COMSETTER(CSAMEnabled) (BOOL aEnable)
 {
-    LogFlowThisFunc (("enable=%d\n", aEnable));
+    LogFlowThisFunc(("enable=%d\n", aEnable));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (queueSettings())
     {
@@ -385,8 +374,8 @@ STDMETHODIMP MachineDebugger::COMSETTER(CSAMEnabled) (BOOL aEnable)
         return S_OK;
     }
 
-    Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    Console::SafeVMPtr pVM(mParent);
+    if (FAILED(pVM.rc())) return pVM.rc();
 
     int vrc;
     if (aEnable)
@@ -394,7 +383,7 @@ STDMETHODIMP MachineDebugger::COMSETTER(CSAMEnabled) (BOOL aEnable)
     else
         vrc = CSAMDisableScanning (pVM);
 
-    if (RT_FAILURE (vrc))
+    if (RT_FAILURE(vrc))
     {
         /** @todo handle error case */
     }
@@ -412,11 +401,11 @@ STDMETHODIMP MachineDebugger::COMGETTER(LogEnabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
 #ifdef LOG_ENABLED
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     const PRTLOGGER pLogInstance = RTLogDefaultInstance();
     *aEnabled = pLogInstance && !(pLogInstance->fFlags & RTLOGFLAGS_DISABLED);
@@ -435,12 +424,12 @@ STDMETHODIMP MachineDebugger::COMGETTER(LogEnabled) (BOOL *aEnabled)
  */
 STDMETHODIMP MachineDebugger::COMSETTER(LogEnabled) (BOOL aEnabled)
 {
-    LogFlowThisFunc (("aEnabled=%d\n", aEnabled));
+    LogFlowThisFunc(("aEnabled=%d\n", aEnabled));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (queueSettings())
     {
@@ -449,12 +438,12 @@ STDMETHODIMP MachineDebugger::COMSETTER(LogEnabled) (BOOL aEnabled)
         return S_OK;
     }
 
-    Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    Console::SafeVMPtr pVM(mParent);
+    if (FAILED(pVM.rc())) return pVM.rc();
 
 #ifdef LOG_ENABLED
     int vrc = DBGFR3LogModifyFlags (pVM, aEnabled ? "enabled" : "disabled");
-    if (RT_FAILURE (vrc))
+    if (RT_FAILURE(vrc))
     {
         /** @todo handle error code. */
     }
@@ -473,10 +462,10 @@ STDMETHODIMP MachineDebugger::COMGETTER(HWVirtExEnabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
@@ -498,10 +487,10 @@ STDMETHODIMP MachineDebugger::COMGETTER(HWVirtExNestedPagingEnabled) (BOOL *aEna
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
@@ -523,10 +512,10 @@ STDMETHODIMP MachineDebugger::COMGETTER(HWVirtExVPIDEnabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
@@ -548,16 +537,16 @@ STDMETHODIMP MachineDebugger::COMGETTER(PAEEnabled) (BOOL *aEnabled)
 {
     CheckComArgOutPointerValid(aEnabled);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
     if (pVM.isOk())
     {
-        uint64_t cr4 = CPUMGetGuestCR4 (pVM.raw());
+        uint64_t cr4 = CPUMGetGuestCR4 (VMMGetCpu0(pVM.raw()));
         *aEnabled = !!(cr4 & X86_CR4_PAE);
     }
     else
@@ -576,15 +565,15 @@ STDMETHODIMP MachineDebugger::COMGETTER(VirtualTimeRate) (ULONG *aPct)
 {
     CheckComArgOutPointerValid(aPct);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     Console::SafeVMPtrQuiet pVM (mParent);
 
     if (pVM.isOk())
-        *aPct = TMVirtualGetWarpDrive (pVM);
+        *aPct = TMGetWarpDrive (pVM);
     else
         *aPct = 100;
 
@@ -602,10 +591,10 @@ STDMETHODIMP MachineDebugger::COMSETTER(VirtualTimeRate) (ULONG aPct)
     if (aPct < 2 || aPct > 20000)
         return E_INVALIDARG;
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
     if (queueSettings())
     {
@@ -614,11 +603,11 @@ STDMETHODIMP MachineDebugger::COMSETTER(VirtualTimeRate) (ULONG aPct)
         return S_OK;
     }
 
-    Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    Console::SafeVMPtr pVM(mParent);
+    if (FAILED(pVM.rc())) return pVM.rc();
 
-    int vrc = TMVirtualSetWarpDrive (pVM, aPct);
-    if (RT_FAILURE (vrc))
+    int vrc = TMR3SetWarpDrive (pVM, aPct);
+    if (RT_FAILURE(vrc))
     {
         /** @todo handle error code. */
     }
@@ -639,13 +628,13 @@ STDMETHODIMP MachineDebugger::COMGETTER(VM) (ULONG64 *aVm)
 {
     CheckComArgOutPointerValid(aVm);
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoReadLock alock (this);
+    AutoReadLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    Console::SafeVMPtr pVM(mParent);
+    if (FAILED(pVM.rc())) return pVM.rc();
 
     *aVm = (uintptr_t)pVM.raw();
 
@@ -671,7 +660,7 @@ STDMETHODIMP MachineDebugger::ResetStats (IN_BSTR aPattern)
     Console::SafeVMPtrQuiet pVM (mParent);
 
     if (!pVM.isOk())
-        return E_FAIL;
+        return setError(VBOX_E_INVALID_VM_STATE, "Machine is not running");
 
     STAMR3Reset (pVM, Utf8Str (aPattern).raw());
 
@@ -689,7 +678,7 @@ STDMETHODIMP MachineDebugger::DumpStats (IN_BSTR aPattern)
     Console::SafeVMPtrQuiet pVM (mParent);
 
     if (!pVM.isOk())
-        return E_FAIL;
+        return setError(VBOX_E_INVALID_VM_STATE, "Machine is not running");
 
     STAMR3Dump (pVM, Utf8Str (aPattern).raw());
 
@@ -709,19 +698,19 @@ STDMETHODIMP MachineDebugger::GetStats (IN_BSTR aPattern, BOOL aWithDescriptions
     Console::SafeVMPtrQuiet pVM (mParent);
 
     if (!pVM.isOk())
-        return E_FAIL;
+        return setError(VBOX_E_INVALID_VM_STATE, "Machine is not running");
 
     char *pszSnapshot;
     int vrc = STAMR3Snapshot (pVM, Utf8Str (aPattern).raw(), &pszSnapshot, NULL,
                               !!aWithDescriptions);
-    if (RT_FAILURE (vrc))
+    if (RT_FAILURE(vrc))
         return vrc == VERR_NO_MEMORY ? E_OUTOFMEMORY : E_FAIL;
 
     /** @todo this is horribly inefficient! And it's kinda difficult to tell whether it failed...
      * Must use UTF-8 or ASCII here and completely avoid these two extra copy operations.
      * Until that's done, this method is kind of useless for debugger statistics GUI because
      * of the amount statistics in a debug build. */
-    Bstr (pszSnapshot).cloneTo (aStats);
+    Bstr (pszSnapshot).cloneTo(aStats);
 
     return S_OK;
 }
@@ -734,15 +723,15 @@ STDMETHODIMP MachineDebugger::GetStats (IN_BSTR aPattern, BOOL aWithDescriptions
  */
 STDMETHODIMP MachineDebugger::InjectNMI()
 {
-    LogFlowThisFunc ((""));
+    LogFlowThisFunc(("\n"));
 
-    AutoCaller autoCaller (this);
-    CheckComRCReturnRC (autoCaller.rc());
+    AutoCaller autoCaller(this);
+    if (FAILED(autoCaller.rc())) return autoCaller.rc();
 
-    AutoWriteLock alock (this);
+    AutoWriteLock alock(this COMMA_LOCKVAL_SRC_POS);
 
-    Console::SafeVMPtr pVM (mParent);
-    CheckComRCReturnRC (pVM.rc());
+    Console::SafeVMPtr pVM(mParent);
+    if (FAILED(pVM.rc())) return pVM.rc();
 
     HWACCMR3InjectNMI(pVM);
 
@@ -804,9 +793,19 @@ bool MachineDebugger::queueSettings() const
         // check if the machine is running
         MachineState_T machineState;
         mParent->COMGETTER(State) (&machineState);
-        if (!Global::IsActive (machineState))
+        switch (machineState)
+        {
             // queue the request
-            return true;
+            default:
+                return true;
+
+            case MachineState_Running:
+            case MachineState_Paused:
+            case MachineState_Stuck:
+            case MachineState_LiveSnapshotting:
+            case MachineState_Teleporting:
+                break;
+        }
     }
     return false;
 }
