@@ -1,4 +1,4 @@
-/* $Id: Performance.cpp 17911 2009-03-16 10:30:55Z vboxsync $ */
+/* $Id: Performance.cpp 29631 2010-05-18 12:59:09Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*
@@ -27,6 +23,10 @@
  * 1) Detection of erroneous metric names
  */
 
+#ifndef VBOX_COLLECTOR_TEST_CASE
+#include "VirtualBoxImpl.h"
+#include "MachineImpl.h"
+#endif
 #include "Performance.h"
 
 #include <VBox/com/array.h>
@@ -65,6 +65,26 @@ int CollectorHAL::getRawProcessCpuLoad(RTPROCESS  /* process */, uint64_t * /* u
     return E_NOTIMPL;
 }
 
+int CollectorHAL::getHostMemoryUsage(ULONG * /* total */, ULONG * /* used */, ULONG * /* available */)
+{
+    return E_NOTIMPL;
+}
+
+int CollectorHAL::getProcessMemoryUsage(RTPROCESS /* process */, ULONG * /* used */)
+{
+    return E_NOTIMPL;
+}
+
+int CollectorHAL::enable()
+{
+    return E_NOTIMPL;
+}
+
+int  CollectorHAL::disable()
+{
+    return E_NOTIMPL;
+}
+
 /* Generic implementations */
 
 int CollectorHAL::getHostCpuMHz(ULONG *mhz)
@@ -92,14 +112,73 @@ int CollectorHAL::getHostCpuMHz(ULONG *mhz)
         }
     }
 
-    // @todo Replace 'if' with 'AssertReturn' when done debugging
-    //AssertReturn(cCpus, VERR_NOT_IMPLEMENTED);
-    if (cCpus == 0) return VERR_NOT_IMPLEMENTED;
+    AssertReturn(cCpus, VERR_NOT_IMPLEMENTED);
     *mhz = (ULONG)(u64TotalMHz / cCpus);
 
     return VINF_SUCCESS;
 }
 
+#ifndef VBOX_COLLECTOR_TEST_CASE
+CollectorGuestHAL::~CollectorGuestHAL()
+{
+    Assert(!cEnabled);
+}
+
+int CollectorGuestHAL::enable()
+{
+    HRESULT ret = S_OK;
+
+    if (ASMAtomicIncU32(&cEnabled) == 1)
+    {
+        ComPtr<IInternalSessionControl> directControl;
+
+        ret = mMachine->getDirectControl(&directControl);
+        if (ret != S_OK)
+            return ret;
+
+        /* get the associated console; this is a remote call (!) */
+        ret = directControl->GetRemoteConsole(mConsole.asOutParam());
+        if (ret != S_OK)
+            return ret;
+
+        ret = mConsole->COMGETTER(Guest)(mGuest.asOutParam());
+        if (ret == S_OK)
+            mGuest->COMSETTER(StatisticsUpdateInterval)(1 /* 1 sec */);
+    }
+    return ret;
+}
+
+int CollectorGuestHAL::disable()
+{
+    if (ASMAtomicDecU32(&cEnabled) == 0)
+    {
+        Assert(mGuest && mConsole);
+        mGuest->COMSETTER(StatisticsUpdateInterval)(0 /* off */);
+    }
+    return S_OK;
+}
+
+int CollectorGuestHAL::preCollect(const CollectorHints& /* hints */, uint64_t iTick)
+{
+    if (    mGuest
+        &&  iTick != mLastTick)
+    {
+        ULONG ulMemAllocTotal, ulMemFreeTotal, ulMemBalloonTotal, ulMemSharedTotal;
+
+        mGuest->InternalGetStatistics(&mCpuUser, &mCpuKernel, &mCpuIdle,
+                                      &mMemTotal, &mMemFree, &mMemBalloon, &mMemShared, &mMemCache,
+                                      &mPageTotal, &ulMemAllocTotal, &ulMemFreeTotal, &ulMemBalloonTotal, &ulMemSharedTotal);
+
+        if (mHostHAL)
+            mHostHAL->setMemHypervisorStats(ulMemAllocTotal, ulMemFreeTotal, ulMemBalloonTotal, ulMemSharedTotal);
+
+        mLastTick = iTick;
+    }
+    return S_OK;
+}
+
+#endif /* !VBOX_COLLECTOR_TEST_CASE */
+
 bool BaseMetric::collectorBeat(uint64_t nowAt)
 {
     if (isEnabled())
@@ -117,7 +196,7 @@ bool BaseMetric::collectorBeat(uint64_t nowAt)
 
 /*bool BaseMetric::associatedWith(ComPtr<IUnknown> object)
 {
-    LogFlowThisFunc (("mObject(%p) == object(%p) is %s.\n", mObject, object, mObject == object ? "true" : "false"));
+    LogFlowThisFunc(("mObject(%p) == object(%p) is %s.\n", mObject, object, mObject == object ? "true" : "false"));
     return mObject == object;
 }*/
 
@@ -142,7 +221,7 @@ void HostCpuLoad::collect()
     }
 }
 
-void HostCpuLoadRaw::preCollect(CollectorHints& hints)
+void HostCpuLoadRaw::preCollect(CollectorHints& hints, uint64_t /* iTick */)
 {
     hints.collectHostCpuLoad();
 }
@@ -163,7 +242,7 @@ void HostCpuLoadRaw::collect()
         if (totalDiff == 0)
         {
             /* This is only possible if none of counters has changed! */
-            LogFlowThisFunc (("Impossible! User, kernel and idle raw "
+            LogFlowThisFunc(("Impossible! User, kernel and idle raw "
                 "counters has not changed since last sample.\n" ));
             mUser->put(0);
             mKernel->put(0);
@@ -204,9 +283,13 @@ void HostRamUsage::init(ULONG period, ULONG length)
     mTotal->init(mLength);
     mUsed->init(mLength);
     mAvailable->init(mLength);
+    mAllocVMM->init(mLength);
+    mFreeVMM->init(mLength);
+    mBalloonVMM->init(mLength);
+    mSharedVMM->init(mLength);
 }
 
-void HostRamUsage::preCollect(CollectorHints& hints)
+void HostRamUsage::preCollect(CollectorHints& hints, uint64_t /* iTick */)
 {
     hints.collectHostRamUsage();
 }
@@ -220,7 +303,15 @@ void HostRamUsage::collect()
         mTotal->put(total);
         mUsed->put(used);
         mAvailable->put(available);
+
     }
+    ULONG allocVMM, freeVMM, balloonVMM, sharedVMM;
+
+    mHAL->getMemHypervisorStats(&allocVMM, &freeVMM, &balloonVMM, &sharedVMM);
+    mAllocVMM->put(allocVMM);
+    mFreeVMM->put(freeVMM);
+    mBalloonVMM->put(balloonVMM);
+    mSharedVMM->put(sharedVMM);
 }
 
 
@@ -244,7 +335,7 @@ void MachineCpuLoad::collect()
     }
 }
 
-void MachineCpuLoadRaw::preCollect(CollectorHints& hints)
+void MachineCpuLoadRaw::preCollect(CollectorHints& hints, uint64_t /* iTick */)
 {
     hints.collectProcessCpuLoad(mProcess);
 }
@@ -281,7 +372,7 @@ void MachineRamUsage::init(ULONG period, ULONG length)
     mUsed->init(mLength);
 }
 
-void MachineRamUsage::preCollect(CollectorHints& hints)
+void MachineRamUsage::preCollect(CollectorHints& hints, uint64_t /* iTick */)
 {
     hints.collectProcessRamUsage(mProcess);
 }
@@ -294,13 +385,70 @@ void MachineRamUsage::collect()
         mUsed->put(used);
 }
 
-void CircularBuffer::init(ULONG length)
+
+void GuestCpuLoad::init(ULONG period, ULONG length)
+{
+    mPeriod = period;
+    mLength = length;
+
+    mUser->init(mLength);
+    mKernel->init(mLength);
+    mIdle->init(mLength);
+}
+
+void GuestCpuLoad::preCollect(CollectorHints& hints, uint64_t iTick)
+{
+    mHAL->preCollect(hints, iTick);
+}
+
+void GuestCpuLoad::collect()
+{
+    ULONG CpuUser = 0, CpuKernel = 0, CpuIdle = 0;
+
+    mGuestHAL->getGuestCpuLoad(&CpuUser, &CpuKernel, &CpuIdle);
+    mUser->put((ULONG)(PM_CPU_LOAD_MULTIPLIER * CpuUser) / 100);
+    mKernel->put((ULONG)(PM_CPU_LOAD_MULTIPLIER * CpuKernel) / 100);
+    mIdle->put((ULONG)(PM_CPU_LOAD_MULTIPLIER * CpuIdle) / 100);
+}
+
+void GuestRamUsage::init(ULONG period, ULONG length)
+{
+    mPeriod = period;
+    mLength = length;
+
+    mTotal->init(mLength);
+    mFree->init(mLength);
+    mBallooned->init(mLength);
+    mShared->init(mLength);
+    mCache->init(mLength);
+    mPagedTotal->init(mLength);
+}
+
+void GuestRamUsage::preCollect(CollectorHints& hints,  uint64_t iTick)
+{
+    mHAL->preCollect(hints, iTick);
+}
+
+void GuestRamUsage::collect()
+{
+    ULONG ulMemTotal = 0, ulMemFree = 0, ulMemBalloon = 0, ulMemShared = 0, ulMemCache = 0, ulPageTotal = 0;
+
+    mGuestHAL->getGuestMemLoad(&ulMemTotal, &ulMemFree, &ulMemBalloon, &ulMemShared, &ulMemCache, &ulPageTotal);
+    mTotal->put(ulMemTotal);
+    mFree->put(ulMemFree);
+    mBallooned->put(ulMemBalloon);
+    mShared->put(ulMemShared);
+    mCache->put(ulMemCache);
+    mPagedTotal->put(ulPageTotal);
+}
+
+void CircularBuffer::init(ULONG ulLength)
 {
     if (mData)
         RTMemFree(mData);
-    mLength = length;
+    mLength = ulLength;
     if (mLength)
-        mData = (ULONG *)RTMemAllocZ(length * sizeof(ULONG));
+        mData = (ULONG*)RTMemAllocZ(ulLength * sizeof(ULONG));
     else
         mData = NULL;
     mWrapped = false;
@@ -431,34 +579,34 @@ Filter::Filter(ComSafeArrayIn(IN_BSTR, metricNames),
      */
     if (ComSafeArrayInIsNull(metricNames))
     {
-        com::SafeArray <BSTR> nameArray;
+        com::SafeArray<BSTR> nameArray;
         if (ComSafeArrayInIsNull(objects))
         {
-            com::SafeIfaceArray <IUnknown> objectArray;
+            com::SafeIfaceArray<IUnknown> objectArray;
             objectArray.reset(0);
             init(ComSafeArrayAsInParam(nameArray),
                  ComSafeArrayAsInParam(objectArray));
         }
         else
         {
-            com::SafeIfaceArray <IUnknown> objectArray(ComSafeArrayInArg(objects));
+            com::SafeIfaceArray<IUnknown> objectArray(ComSafeArrayInArg(objects));
             init(ComSafeArrayAsInParam(nameArray),
                  ComSafeArrayAsInParam(objectArray));
         }
     }
     else
     {
-        com::SafeArray <IN_BSTR> nameArray(ComSafeArrayInArg(metricNames));
+        com::SafeArray<IN_BSTR> nameArray(ComSafeArrayInArg(metricNames));
         if (ComSafeArrayInIsNull(objects))
         {
-            com::SafeIfaceArray <IUnknown> objectArray;
+            com::SafeIfaceArray<IUnknown> objectArray;
             objectArray.reset(0);
             init(ComSafeArrayAsInParam(nameArray),
                  ComSafeArrayAsInParam(objectArray));
         }
         else
         {
-            com::SafeIfaceArray <IUnknown> objectArray(ComSafeArrayInArg(objects));
+            com::SafeIfaceArray<IUnknown> objectArray(ComSafeArrayInArg(objects));
             init(ComSafeArrayAsInParam(nameArray),
                  ComSafeArrayAsInParam(objectArray));
         }
@@ -468,18 +616,18 @@ Filter::Filter(ComSafeArrayIn(IN_BSTR, metricNames),
 void Filter::init(ComSafeArrayIn(IN_BSTR, metricNames),
                   ComSafeArrayIn(IUnknown *, objects))
 {
-    com::SafeArray <IN_BSTR> nameArray(ComSafeArrayInArg(metricNames));
-    com::SafeIfaceArray <IUnknown> objectArray(ComSafeArrayInArg(objects));
+    com::SafeArray<IN_BSTR> nameArray(ComSafeArrayInArg(metricNames));
+    com::SafeIfaceArray<IUnknown> objectArray(ComSafeArrayInArg(objects));
 
     if (!objectArray.size())
     {
         if (nameArray.size())
         {
             for (size_t i = 0; i < nameArray.size(); ++i)
-                processMetricList(std::string(com::Utf8Str(nameArray[i])), ComPtr<IUnknown>());
+                processMetricList(com::Utf8Str(nameArray[i]), ComPtr<IUnknown>());
         }
         else
-            processMetricList(std::string("*"), ComPtr<IUnknown>());
+            processMetricList("*", ComPtr<IUnknown>());
     }
     else
     {
@@ -487,30 +635,30 @@ void Filter::init(ComSafeArrayIn(IN_BSTR, metricNames),
             switch (nameArray.size())
             {
                 case 0:
-                    processMetricList(std::string("*"), objectArray[i]);
+                    processMetricList("*", objectArray[i]);
                     break;
                 case 1:
-                    processMetricList(std::string(com::Utf8Str(nameArray[0])), objectArray[i]);
+                    processMetricList(com::Utf8Str(nameArray[0]), objectArray[i]);
                     break;
                 default:
-                    processMetricList(std::string(com::Utf8Str(nameArray[i])), objectArray[i]);
+                    processMetricList(com::Utf8Str(nameArray[i]), objectArray[i]);
                     break;
             }
     }
 }
 
-void Filter::processMetricList(const std::string &name, const ComPtr<IUnknown> object)
+void Filter::processMetricList(const com::Utf8Str &name, const ComPtr<IUnknown> object)
 {
-    std::string::size_type startPos = 0;
+    size_t startPos = 0;
 
-    for (std::string::size_type pos = name.find(",");
-         pos != std::string::npos;
+    for (size_t pos = name.find(",");
+         pos != com::Utf8Str::npos;
          pos = name.find(",", startPos))
     {
-        mElements.push_back(std::make_pair(object, name.substr(startPos, pos - startPos)));
+        mElements.push_back(std::make_pair(object, iprt::MiniString(name.substr(startPos, pos - startPos).c_str())));
         startPos = pos + 1;
     }
-    mElements.push_back(std::make_pair(object, name.substr(startPos)));
+    mElements.push_back(std::make_pair(object, iprt::MiniString(name.substr(startPos).c_str())));
 }
 
 /**
@@ -586,7 +734,7 @@ bool Filter::patternMatch(const char *pszPat, const char *pszName,
     return true;
 }
 
-bool Filter::match(const ComPtr<IUnknown> object, const std::string &name) const
+bool Filter::match(const ComPtr<IUnknown> object, const iprt::MiniString &name) const
 {
     ElementList::const_iterator it;
 
