@@ -1,4 +1,4 @@
-/* $Id: iokit.cpp $ */
+/* $Id: iokit.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * Main - Darwin IOKit Routines.
  *
@@ -8,7 +8,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -17,10 +17,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -1476,7 +1472,7 @@ PDARWINETHERNIC DarwinGetEthernetControllers(void)
             if (krc == KERN_SUCCESS)
             {
                 CFMutableDictionaryRef IfPropsRef = 0;
-                kern_return_t krc = IORegistryEntryCreateCFProperties(EtherIfService, &IfPropsRef, kCFAllocatorDefault, kNilOptions);
+                krc = IORegistryEntryCreateCFProperties(EtherIfService, &IfPropsRef, kCFAllocatorDefault, kNilOptions);
                 if (krc == KERN_SUCCESS)
                 {
                     /*
@@ -1537,7 +1533,7 @@ PDARWINETHERNIC DarwinGetEthernetControllers(void)
                         CFIndex i;
 
                         /* look it up among the current services */
-                        for (CFIndex i = 0; i < cServices; i++)
+                        for (i = 0; i < cServices; i++)
                         {
                             SCNetworkServiceRef ServiceRef = (SCNetworkServiceRef)CFArrayGetValueAtIndex(ServicesRef, i);
                             SCNetworkInterfaceRef IfRef = SCNetworkServiceGetInterface(ServiceRef);
@@ -1780,7 +1776,7 @@ int main(int argc, char **argv)
         for (PDARWINETHERNIC pCur = pEtherNICs; pCur; pCur = pCur->pNext)
         {
             RTPrintf("%s\n", pCur->szName);
-            RTPrintf("    szBSDName=%d\n", pCur->szBSDName);
+            RTPrintf("    szBSDName=%s\n", pCur->szBSDName);
             RTPrintf("         UUID=%RTuuid\n", &pCur->Uuid);
             RTPrintf("          Mac=%.6Rhxs\n", &pCur->Mac);
             RTPrintf("    fWireless=%RTbool\n", pCur->fWireless);
@@ -1795,5 +1791,3 @@ int main(int argc, char **argv)
     return 0;
 }
 #endif
-
-
