@@ -1,4 +1,4 @@
-/* $Id: ErrorInfo.cpp $ */
+/* $Id: ErrorInfo.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #if !defined (VBOX_WITH_XPCOM)
@@ -53,16 +49,16 @@ void ErrorInfo::init (bool aKeepObj /* = false */)
 
 #if !defined (VBOX_WITH_XPCOM)
 
-    ComPtr <IErrorInfo> err;
+    ComPtr<IErrorInfo> err;
     rc = ::GetErrorInfo (0, err.asOutParam());
     if (rc == S_OK && err)
     {
         if (aKeepObj)
             mErrorInfo = err;
 
-        ComPtr <IVirtualBoxErrorInfo> info;
-        rc = err.queryInterfaceTo (info.asOutParam());
-        if (SUCCEEDED (rc) && info)
+        ComPtr<IVirtualBoxErrorInfo> info;
+        rc = err.queryInterfaceTo(info.asOutParam());
+        if (SUCCEEDED(rc) && info)
             init (info);
 
         if (!mIsFullAvailable)
@@ -70,15 +66,15 @@ void ErrorInfo::init (bool aKeepObj /* = false */)
             bool gotSomething = false;
 
             rc = err->GetGUID (mInterfaceID.asOutParam());
-            gotSomething |= SUCCEEDED (rc);
-            if (SUCCEEDED (rc))
+            gotSomething |= SUCCEEDED(rc);
+            if (SUCCEEDED(rc))
                 GetInterfaceNameByIID (mInterfaceID, mInterfaceName.asOutParam());
 
             rc = err->GetSource (mComponent.asOutParam());
-            gotSomething |= SUCCEEDED (rc);
+            gotSomething |= SUCCEEDED(rc);
 
             rc = err->GetDescription (mText.asOutParam());
-            gotSomething |= SUCCEEDED (rc);
+            gotSomething |= SUCCEEDED(rc);
 
             if (gotSomething)
                 mIsBasicAvailable = true;
@@ -87,40 +83,43 @@ void ErrorInfo::init (bool aKeepObj /* = false */)
         }
     }
 
-#else // !defined (VBOX_WITH_XPCOM)
+#else // defined (VBOX_WITH_XPCOM)
 
-    nsCOMPtr <nsIExceptionService> es;
-    es = do_GetService (NS_EXCEPTIONSERVICE_CONTRACTID, &rc);
-    if (NS_SUCCEEDED (rc))
+    nsCOMPtr<nsIExceptionService> es;
+    es = do_GetService(NS_EXCEPTIONSERVICE_CONTRACTID, &rc);
+    if (NS_SUCCEEDED(rc))
     {
-        nsCOMPtr <nsIExceptionManager> em;
-        rc = es->GetCurrentExceptionManager (getter_AddRefs (em));
-        if (NS_SUCCEEDED (rc))
+        nsCOMPtr<nsIExceptionManager> em;
+        rc = es->GetCurrentExceptionManager(getter_AddRefs (em));
+        if (NS_SUCCEEDED(rc))
         {
-            ComPtr <nsIException> ex;
-            rc = em->GetCurrentException (ex.asOutParam());
-            if (NS_SUCCEEDED (rc) && ex)
+            ComPtr<nsIException> ex;
+            rc = em->GetCurrentException(ex.asOutParam());
+            if (NS_SUCCEEDED(rc) && ex)
             {
                 if (aKeepObj)
                     mErrorInfo = ex;
 
-                ComPtr <IVirtualBoxErrorInfo> info;
-                rc = ex.queryInterfaceTo (info.asOutParam());
-                if (NS_SUCCEEDED (rc) && info)
+                ComPtr<IVirtualBoxErrorInfo> info;
+                rc = ex.queryInterfaceTo(info.asOutParam());
+                if (NS_SUCCEEDED(rc) && info)
                     init (info);
 
                 if (!mIsFullAvailable)
                 {
                     bool gotSomething = false;
 
-                    rc = ex->GetResult (&mResultCode);
-                    gotSomething |= NS_SUCCEEDED (rc);
+                    rc = ex->GetResult(&mResultCode);
+                    gotSomething |= NS_SUCCEEDED(rc);
 
-                    Utf8Str message;
-                    rc = ex->GetMessage (message.asOutParam());
-                    gotSomething |= NS_SUCCEEDED (rc);
-                    if (NS_SUCCEEDED (rc))
-                        mText = message;
+                    char *pszMsg;
+                    rc = ex->GetMessage(&pszMsg);
+                    gotSomething |= NS_SUCCEEDED(rc);
+                    if (NS_SUCCEEDED(rc))
+                    {
+                        mText = Bstr(pszMsg);
+                        nsMemory::Free(mText);
+                    }
 
                     if (gotSomething)
                         mIsBasicAvailable = true;
@@ -138,24 +137,24 @@ void ErrorInfo::init (bool aKeepObj /* = false */)
 
     AssertComRC (rc);
 
-#endif // !defined (VBOX_WITH_XPCOM)
+#endif // defined (VBOX_WITH_XPCOM)
 }
 
 void ErrorInfo::init (IUnknown *aI, const GUID &aIID, bool aKeepObj /* = false */)
 {
-    Assert (aI);
+    Assert(aI);
     if (!aI)
         return;
 
 #if !defined (VBOX_WITH_XPCOM)
 
-    ComPtr <IUnknown> iface = aI;
-    ComPtr <ISupportErrorInfo> serr;
-    HRESULT rc = iface.queryInterfaceTo (serr.asOutParam());
-    if (SUCCEEDED (rc))
+    ComPtr<IUnknown> iface = aI;
+    ComPtr<ISupportErrorInfo> serr;
+    HRESULT rc = iface.queryInterfaceTo(serr.asOutParam());
+    if (SUCCEEDED(rc))
     {
         rc = serr->InterfaceSupportsErrorInfo (aIID);
-        if (SUCCEEDED (rc))
+        if (SUCCEEDED(rc))
             init (aKeepObj);
     }
 
@@ -179,38 +178,43 @@ void ErrorInfo::init (IVirtualBoxErrorInfo *info)
     HRESULT rc = E_FAIL;
     bool gotSomething = false;
     bool gotAll = true;
+    LONG lrc;
 
-    rc = info->COMGETTER(ResultCode) (&mResultCode);
-    gotSomething |= SUCCEEDED (rc);
-    gotAll &= SUCCEEDED (rc);
+    rc = info->COMGETTER(ResultCode) (&lrc); mResultCode = lrc;
+    gotSomething |= SUCCEEDED(rc);
+    gotAll &= SUCCEEDED(rc);
 
-    rc = info->COMGETTER(InterfaceID) (mInterfaceID.asOutParam());
-    gotSomething |= SUCCEEDED (rc);
-    gotAll &= SUCCEEDED (rc);
-    if (SUCCEEDED (rc))
+    Bstr iid;
+    rc = info->COMGETTER(InterfaceID) (iid.asOutParam());
+    gotSomething |= SUCCEEDED(rc);
+    gotAll &= SUCCEEDED(rc);
+    if (SUCCEEDED(rc))
+    {
+        mInterfaceID = iid;
         GetInterfaceNameByIID (mInterfaceID, mInterfaceName.asOutParam());
+    }
 
     rc = info->COMGETTER(Component) (mComponent.asOutParam());
-    gotSomething |= SUCCEEDED (rc);
-    gotAll &= SUCCEEDED (rc);
+    gotSomething |= SUCCEEDED(rc);
+    gotAll &= SUCCEEDED(rc);
 
     rc = info->COMGETTER(Text) (mText.asOutParam());
-    gotSomething |= SUCCEEDED (rc);
-    gotAll &= SUCCEEDED (rc);
+    gotSomething |= SUCCEEDED(rc);
+    gotAll &= SUCCEEDED(rc);
 
-    ComPtr <IVirtualBoxErrorInfo> next;
+    ComPtr<IVirtualBoxErrorInfo> next;
     rc = info->COMGETTER(Next) (next.asOutParam());
-    if (SUCCEEDED (rc) && !next.isNull())
+    if (SUCCEEDED(rc) && !next.isNull())
     {
         mNext.reset (new ErrorInfo (next));
-        Assert (mNext.get());
+        Assert(mNext.get());
         if (!mNext.get())
             rc = E_OUTOFMEMORY;
     }
     else
         mNext.reset();
-    gotSomething |= SUCCEEDED (rc);
-    gotAll &= SUCCEEDED (rc);
+    gotSomething |= SUCCEEDED(rc);
+    gotAll &= SUCCEEDED(rc);
 
     mIsBasicAvailable = gotSomething;
     mIsFullAvailable = gotAll;
@@ -228,13 +232,13 @@ ErrorInfo::~ErrorInfo()
 ProgressErrorInfo::ProgressErrorInfo (IProgress *progress) :
     ErrorInfo (false /* aDummy */)
 {
-    Assert (progress);
+    Assert(progress);
     if (!progress)
         return;
 
-    ComPtr <IVirtualBoxErrorInfo> info;
+    ComPtr<IVirtualBoxErrorInfo> info;
     HRESULT rc = progress->COMGETTER(ErrorInfo) (info.asOutParam());
-    if (SUCCEEDED (rc) && info)
+    if (SUCCEEDED(rc) && info)
         init (info);
 }
 
@@ -250,10 +254,10 @@ HRESULT ErrorInfoKeeper::restore()
 
 #if !defined (VBOX_WITH_XPCOM)
 
-    ComPtr <IErrorInfo> err;
+    ComPtr<IErrorInfo> err;
     if (!mErrorInfo.isNull())
     {
-        rc = mErrorInfo.queryInterfaceTo (err.asOutParam());
+        rc = mErrorInfo.queryInterfaceTo(err.asOutParam());
         AssertComRC (rc);
     }
     rc = ::SetErrorInfo (0, err);
@@ -262,16 +266,16 @@ HRESULT ErrorInfoKeeper::restore()
 
     nsCOMPtr <nsIExceptionService> es;
     es = do_GetService (NS_EXCEPTIONSERVICE_CONTRACTID, &rc);
-    if (NS_SUCCEEDED (rc))
+    if (NS_SUCCEEDED(rc))
     {
         nsCOMPtr <nsIExceptionManager> em;
         rc = es->GetCurrentExceptionManager (getter_AddRefs (em));
-        if (NS_SUCCEEDED (rc))
+        if (NS_SUCCEEDED(rc))
         {
-            ComPtr <nsIException> ex;
+            ComPtr<nsIException> ex;
             if (!mErrorInfo.isNull())
             {
-                rc = mErrorInfo.queryInterfaceTo (ex.asOutParam());
+                rc = mErrorInfo.queryInterfaceTo(ex.asOutParam());
                 AssertComRC (rc);
             }
             rc = em->SetCurrentException (ex);
@@ -280,7 +284,7 @@ HRESULT ErrorInfoKeeper::restore()
 
 #endif // !defined (VBOX_WITH_XPCOM)
 
-    if (SUCCEEDED (rc))
+    if (SUCCEEDED(rc))
     {
         mErrorInfo.setNull();
         mForgot = true;
