@@ -1,4 +1,4 @@
-/* $Id: SupportErrorInfo.cpp $ */
+/* $Id: SupportErrorInfo.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  * MS COM / XPCOM Abstraction Layer:
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "VBox/com/SupportErrorInfo.h"
@@ -49,21 +45,21 @@ void MultiResult::incCounter()
     if (sCounter == NIL_RTTLS)
     {
         sCounter = RTTlsAlloc();
-        AssertReturnVoid (sCounter != NIL_RTTLS);
+        AssertReturnVoid(sCounter != NIL_RTTLS);
     }
 
-    uintptr_t counter = (uintptr_t) RTTlsGet (sCounter);
-    ++ counter;
-    RTTlsSet (sCounter, (void *) counter);
+    uintptr_t counter = (uintptr_t)RTTlsGet(sCounter);
+    ++counter;
+    RTTlsSet(sCounter, (void*)counter);
 }
 
 /*static*/
 void MultiResult::decCounter()
 {
-    uintptr_t counter = (uintptr_t) RTTlsGet (sCounter);
-    AssertReturnVoid (counter != 0);
-    -- counter;
-    RTTlsSet (sCounter, (void *) counter);
+    uintptr_t counter = (uintptr_t)RTTlsGet(sCounter);
+    AssertReturnVoid(counter != 0);
+    --counter;
+    RTTlsSet(sCounter, (void*)counter);
 }
 
 // SupportErrorInfoBase methods
@@ -97,27 +93,32 @@ void MultiResult::decCounter()
  * code is returned.
  */
 /* static */
-HRESULT SupportErrorInfoBase::setErrorInternal (
-    HRESULT aResultCode, const GUID *aIID,
-    const char *aComponent, const char *aText,
-    bool aWarning, IVirtualBoxErrorInfo *aInfo /*= NULL*/)
+HRESULT SupportErrorInfoBase::setErrorInternal(HRESULT aResultCode,
+                                               const GUID *aIID,
+                                               const char *aComponent,
+                                               const Utf8Str &strText,
+                                               bool aWarning,
+                                               IVirtualBoxErrorInfo *aInfo /*= NULL*/)
 {
     /* whether multi-error mode is turned on */
-    bool preserve = ((uintptr_t) RTTlsGet (MultiResult::sCounter)) > 0;
+    bool preserve = ((uintptr_t)RTTlsGet(MultiResult::sCounter)) > 0;
 
-    LogRel (("ERROR [COM]: aRC=%#08x aIID={%Vuuid} aComponent={%s} aText={%s} "
-             "aWarning=%RTbool, aInfo=%p, preserve=%RTbool\n",
-             aResultCode, aIID, aComponent, aText, aWarning, aInfo,
-             preserve));
+    LogRel(("ERROR [COM]: aRC=%#08x aIID={%RTuuid} aComponent={%s} aText={%s} aWarning=%RTbool, aInfo=%p, preserve=%RTbool\n",
+            aResultCode,
+            aIID,
+            aComponent,
+            strText.c_str(),
+            aWarning,
+            aInfo,
+            preserve));
 
     if (aInfo == NULL)
     {
         /* these are mandatory, others -- not */
-        AssertReturn ((!aWarning && FAILED (aResultCode)) ||
+        AssertReturn((!aWarning && FAILED(aResultCode)) ||
                       (aWarning && aResultCode != S_OK),
                       E_FAIL);
-        AssertReturn (aText != NULL, E_FAIL);
-        AssertReturn (*aText != '\0', E_FAIL);
+        AssertReturn(!strText.isEmpty(), E_FAIL);
 
         /* reset the error severity bit if it's a warning */
         if (aWarning)
@@ -128,34 +129,34 @@ HRESULT SupportErrorInfoBase::setErrorInternal (
 
     do
     {
-        ComPtr <IVirtualBoxErrorInfo> info;
+        ComPtr<IVirtualBoxErrorInfo> info;
 
 #if !defined (VBOX_WITH_XPCOM)
 
-        ComPtr <IVirtualBoxErrorInfo> curInfo;
+        ComPtr<IVirtualBoxErrorInfo> curInfo;
         if (preserve)
         {
             /* get the current error info if any */
-            ComPtr <IErrorInfo> err;
-            rc = ::GetErrorInfo (0, err.asOutParam());
-            CheckComRCBreakRC (rc);
-            rc = err.queryInterfaceTo (curInfo.asOutParam());
-            if (FAILED (rc))
+            ComPtr<IErrorInfo> err;
+            rc = ::GetErrorInfo(0, err.asOutParam());
+            if (FAILED(rc)) break;
+            rc = err.queryInterfaceTo(curInfo.asOutParam());
+            if (FAILED(rc))
             {
                 /* create a IVirtualBoxErrorInfo wrapper for the native
                  * IErrorInfo object */
-                ComObjPtr <VirtualBoxErrorInfo> wrapper;
+                ComObjPtr<VirtualBoxErrorInfo> wrapper;
                 rc = wrapper.createObject();
-                if (SUCCEEDED (rc))
+                if (SUCCEEDED(rc))
                 {
-                    rc = wrapper->init (err);
-                    if (SUCCEEDED (rc))
+                    rc = wrapper->init(err);
+                    if (SUCCEEDED(rc))
                         curInfo = wrapper;
                 }
             }
         }
         /* On failure, curInfo will stay null */
-        Assert (SUCCEEDED (rc) || curInfo.isNull());
+        Assert(SUCCEEDED(rc) || curInfo.isNull());
 
         /* set the current error info and preserve the previous one if any */
         if (aInfo != NULL)
@@ -166,71 +167,71 @@ HRESULT SupportErrorInfoBase::setErrorInternal (
             }
             else
             {
-                ComObjPtr <VirtualBoxErrorInfoGlue> infoObj;
+                ComObjPtr<VirtualBoxErrorInfoGlue> infoObj;
                 rc = infoObj.createObject();
-                CheckComRCBreakRC (rc);
+                if (FAILED(rc)) break;
 
-                rc = infoObj->init (aInfo, curInfo);
-                CheckComRCBreakRC (rc);
+                rc = infoObj->init(aInfo, curInfo);
+                if (FAILED(rc)) break;
 
                 info = infoObj;
             }
 
             /* we want to return the head's result code */
-            rc = info->COMGETTER(ResultCode) (&aResultCode);
-            CheckComRCBreakRC (rc);
+            rc = info->COMGETTER(ResultCode)(&aResultCode);
+            if (FAILED(rc)) break;
         }
         else
         {
-            ComObjPtr <VirtualBoxErrorInfo> infoObj;
+            ComObjPtr<VirtualBoxErrorInfo> infoObj;
             rc = infoObj.createObject();
-            CheckComRCBreakRC (rc);
+            if (FAILED(rc)) break;
 
-            rc = infoObj->init (aResultCode, aIID, aComponent, aText, curInfo);
-            CheckComRCBreakRC (rc);
+            rc = infoObj->init(aResultCode, aIID, aComponent, strText.c_str(), curInfo);
+            if (FAILED(rc)) break;
 
             info = infoObj;
         }
 
-        ComPtr <IErrorInfo> err;
-        rc = info.queryInterfaceTo (err.asOutParam());
-        if (SUCCEEDED (rc))
-            rc = ::SetErrorInfo (0, err);
+        ComPtr<IErrorInfo> err;
+        rc = info.queryInterfaceTo(err.asOutParam());
+        if (SUCCEEDED(rc))
+            rc = ::SetErrorInfo(0, err);
 
 #else // !defined (VBOX_WITH_XPCOM)
 
         nsCOMPtr <nsIExceptionService> es;
-        es = do_GetService (NS_EXCEPTIONSERVICE_CONTRACTID, &rc);
-        if (NS_SUCCEEDED (rc))
+        es = do_GetService(NS_EXCEPTIONSERVICE_CONTRACTID, &rc);
+        if (NS_SUCCEEDED(rc))
         {
             nsCOMPtr <nsIExceptionManager> em;
-            rc = es->GetCurrentExceptionManager (getter_AddRefs (em));
-            CheckComRCBreakRC (rc);
+            rc = es->GetCurrentExceptionManager(getter_AddRefs(em));
+            if (FAILED(rc)) break;
 
-            ComPtr <IVirtualBoxErrorInfo> curInfo;
+            ComPtr<IVirtualBoxErrorInfo> curInfo;
             if (preserve)
             {
                 /* get the current error info if any */
-                ComPtr <nsIException> ex;
-                rc = em->GetCurrentException (ex.asOutParam());
-                CheckComRCBreakRC (rc);
-                rc = ex.queryInterfaceTo (curInfo.asOutParam());
-                if (FAILED (rc))
+                ComPtr<nsIException> ex;
+                rc = em->GetCurrentException(ex.asOutParam());
+                if (FAILED(rc)) break;
+                rc = ex.queryInterfaceTo(curInfo.asOutParam());
+                if (FAILED(rc))
                 {
                     /* create a IVirtualBoxErrorInfo wrapper for the native
                      * nsIException object */
-                    ComObjPtr <VirtualBoxErrorInfo> wrapper;
+                    ComObjPtr<VirtualBoxErrorInfo> wrapper;
                     rc = wrapper.createObject();
-                    if (SUCCEEDED (rc))
+                    if (SUCCEEDED(rc))
                     {
-                        rc = wrapper->init (ex);
-                        if (SUCCEEDED (rc))
+                        rc = wrapper->init(ex);
+                        if (SUCCEEDED(rc))
                             curInfo = wrapper;
                     }
                 }
             }
             /* On failure, curInfo will stay null */
-            Assert (SUCCEEDED (rc) || curInfo.isNull());
+            Assert(SUCCEEDED(rc) || curInfo.isNull());
 
             /* set the current error info and preserve the previous one if any */
             if (aInfo != NULL)
@@ -241,36 +242,37 @@ HRESULT SupportErrorInfoBase::setErrorInternal (
                 }
                 else
                 {
-                    ComObjPtr <VirtualBoxErrorInfoGlue> infoObj;
+                    ComObjPtr<VirtualBoxErrorInfoGlue> infoObj;
                     rc = infoObj.createObject();
-                    CheckComRCBreakRC (rc);
+                    if (FAILED(rc)) break;
 
-                    rc = infoObj->init (aInfo, curInfo);
-                    CheckComRCBreakRC (rc);
+                    rc = infoObj->init(aInfo, curInfo);
+                    if (FAILED(rc)) break;
 
                     info = infoObj;
                 }
 
                 /* we want to return the head's result code */
-                rc = info->COMGETTER(ResultCode) (&aResultCode);
-                CheckComRCBreakRC (rc);
+                PRInt32 lrc;
+                rc = info->COMGETTER(ResultCode)(&lrc); aResultCode = lrc;
+                if (FAILED(rc)) break;
             }
             else
             {
-                ComObjPtr <VirtualBoxErrorInfo> infoObj;
+                ComObjPtr<VirtualBoxErrorInfo> infoObj;
                 rc = infoObj.createObject();
-                CheckComRCBreakRC (rc);
+                if (FAILED(rc)) break;
 
-                rc = infoObj->init (aResultCode, aIID, aComponent, aText, curInfo);
-                CheckComRCBreakRC (rc);
+                rc = infoObj->init(aResultCode, aIID, aComponent, strText, curInfo);
+                if (FAILED(rc)) break;
 
                 info = infoObj;
             }
 
-            ComPtr <nsIException> ex;
-            rc = info.queryInterfaceTo (ex.asOutParam());
-            if (SUCCEEDED (rc))
-                rc = em->SetCurrentException (ex);
+            ComPtr<nsIException> ex;
+            rc = info.queryInterfaceTo(ex.asOutParam());
+            if (SUCCEEDED(rc))
+                rc = em->SetCurrentException(ex);
         }
         else if (rc == NS_ERROR_UNEXPECTED)
         {
@@ -294,72 +296,72 @@ HRESULT SupportErrorInfoBase::setErrorInternal (
     }
     while (0);
 
-    AssertComRC (rc);
+    AssertComRC(rc);
 
-    return SUCCEEDED (rc) ? aResultCode : rc;
+    return SUCCEEDED(rc) ? aResultCode : rc;
 }
 
 /* static */
-HRESULT SupportErrorInfoBase::setError (HRESULT aResultCode, const GUID &aIID,
-                                        const char *aComponent, const char *aText,
-                                        ...)
+HRESULT SupportErrorInfoBase::setError(HRESULT aResultCode, const GUID &aIID,
+                                       const char *aComponent, const char *aText,
+                                       ...)
 {
     va_list args;
-    va_start (args, aText);
-    HRESULT rc = setErrorV (aResultCode, aIID, aComponent, aText, args);
-    va_end (args);
+    va_start(args, aText);
+    HRESULT rc = setErrorV(aResultCode, aIID, aComponent, aText, args);
+    va_end(args);
     return rc;
 }
 
 /* static */
-HRESULT SupportErrorInfoBase::setWarning (HRESULT aResultCode, const GUID &aIID,
-                                          const char *aComponent, const char *aText,
-                                          ...)
+HRESULT SupportErrorInfoBase::setWarning(HRESULT aResultCode, const GUID &aIID,
+                                         const char *aComponent, const char *aText,
+                                         ...)
 {
     va_list args;
-    va_start (args, aText);
-    HRESULT rc = setWarningV (aResultCode, aIID, aComponent, aText, args);
-    va_end (args);
+    va_start(args, aText);
+    HRESULT rc = setWarningV(aResultCode, aIID, aComponent, aText, args);
+    va_end(args);
     return rc;
 }
 
-HRESULT SupportErrorInfoBase::setError (HRESULT aResultCode, const char *aText, ...)
+HRESULT SupportErrorInfoBase::setError(HRESULT aResultCode, const char *aText, ...)
 {
     va_list args;
-    va_start (args, aText);
-    HRESULT rc = setErrorV (aResultCode, mainInterfaceID(), componentName(),
-                            aText, args);
-    va_end (args);
+    va_start(args, aText);
+    HRESULT rc = setErrorV(aResultCode, mainInterfaceID(), componentName(),
+                           aText, args);
+    va_end(args);
     return rc;
 }
 
-HRESULT SupportErrorInfoBase::setWarning (HRESULT aResultCode, const char *aText, ...)
+HRESULT SupportErrorInfoBase::setWarning(HRESULT aResultCode, const char *aText, ...)
 {
     va_list args;
-    va_start (args, aText);
-    HRESULT rc = setWarningV (aResultCode, mainInterfaceID(), componentName(),
-                              aText, args);
-    va_end (args);
+    va_start(args, aText);
+    HRESULT rc = setWarningV(aResultCode, mainInterfaceID(), componentName(),
+                             aText, args);
+    va_end(args);
     return rc;
 }
 
-HRESULT SupportErrorInfoBase::setError (HRESULT aResultCode, const GUID &aIID,
-                                        const char *aText, ...)
+HRESULT SupportErrorInfoBase::setError(HRESULT aResultCode, const GUID &aIID,
+                                       const char *aText, ...)
 {
     va_list args;
-    va_start (args, aText);
-    HRESULT rc = setErrorV (aResultCode, aIID, componentName(), aText, args);
-    va_end (args);
+    va_start(args, aText);
+    HRESULT rc = setErrorV(aResultCode, aIID, componentName(), aText, args);
+    va_end(args);
     return rc;
 }
 
-HRESULT SupportErrorInfoBase::setWarning (HRESULT aResultCode, const GUID &aIID,
-                                          const char *aText, ...)
+HRESULT SupportErrorInfoBase::setWarning(HRESULT aResultCode, const GUID &aIID,
+                                         const char *aText, ...)
 {
     va_list args;
-    va_start (args, aText);
-    HRESULT rc = setWarningV (aResultCode, aIID, componentName(), aText, args);
-    va_end (args);
+    va_start(args, aText);
+    HRESULT rc = setWarningV(aResultCode, aIID, componentName(), aText, args);
+    va_end(args);
     return rc;
 }
 
