@@ -1,4 +1,4 @@
-/* $Id: VirtualBoxErrorInfo.cpp $ */
+/* $Id: VirtualBoxErrorInfo.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  * MS COM / XPCOM Abstraction Layer:
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include "VBox/com/VirtualBoxErrorInfo.h"
@@ -40,9 +36,11 @@ namespace com
 /**
  * Initializes the error info object with the given error details.
  */
-HRESULT VirtualBoxErrorInfo::init (HRESULT aResultCode, const GUID *aIID,
-                                   const char *aComponent, const char *aText,
-                                   IVirtualBoxErrorInfo *aNext)
+HRESULT VirtualBoxErrorInfo::init(HRESULT aResultCode,
+                                  const GUID *aIID,
+                                  const char *aComponent,
+                                  const Utf8Str &strText,
+                                  IVirtualBoxErrorInfo *aNext)
 {
     mResultCode = aResultCode;
 
@@ -50,7 +48,7 @@ HRESULT VirtualBoxErrorInfo::init (HRESULT aResultCode, const GUID *aIID,
         mIID = *aIID;
 
     mComponent = aComponent;
-    mText = aText;
+    mText = strText;
     mNext = aNext;
 
     return S_OK;
@@ -68,12 +66,12 @@ STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(ResultCode) (LONG *aResultCode)
     return S_OK;
 }
 
-STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(InterfaceID) (OUT_GUID aIID)
+STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(InterfaceID) (BSTR *aIID)
 {
     if (!aIID)
         return E_POINTER;
 
-    mIID.cloneTo (aIID);
+    mIID.toUtf16().cloneTo(aIID);
     return S_OK;
 }
 
@@ -82,7 +80,7 @@ STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(Component) (BSTR *aComponent)
     if (!aComponent)
         return E_POINTER;
 
-    mComponent.cloneTo (aComponent);
+    mComponent.cloneTo(aComponent);
     return S_OK;
 }
 
@@ -91,7 +89,7 @@ STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(Text) (BSTR *aText)
     if (!aText)
         return E_POINTER;
 
-    mText.cloneTo (aText);
+    mText.cloneTo(aText);
     return S_OK;
 }
 
@@ -101,10 +99,10 @@ STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(Next) (IVirtualBoxErrorInfo **aNext)
         return E_POINTER;
 
     /* this will set aNext to NULL if mNext is null */
-    return mNext.queryInterfaceTo (aNext);
+    return mNext.queryInterfaceTo(aNext);
 }
 
-#if !defined (VBOX_WITH_XPCOM)
+#if !defined(VBOX_WITH_XPCOM)
 
 /**
  *  Initializes itself by fetching error information from the given error info
@@ -112,7 +110,7 @@ STDMETHODIMP VirtualBoxErrorInfo::COMGETTER(Next) (IVirtualBoxErrorInfo **aNext)
  */
 HRESULT VirtualBoxErrorInfo::init (IErrorInfo *aInfo)
 {
-    AssertReturn (aInfo, E_FAIL);
+    AssertReturn(aInfo, E_FAIL);
 
     HRESULT rc = S_OK;
 
@@ -141,7 +139,11 @@ STDMETHODIMP VirtualBoxErrorInfo::GetDescription (BSTR *description)
 
 STDMETHODIMP VirtualBoxErrorInfo::GetGUID (GUID *guid)
 {
-    return COMGETTER(InterfaceID) (guid);
+    Bstr iid;
+    HRESULT rc = COMGETTER(InterfaceID) (iid.asOutParam());
+    if (SUCCEEDED(rc))
+        *guid = Guid(iid);
+    return rc;
 }
 
 STDMETHODIMP VirtualBoxErrorInfo::GetHelpContext (DWORD *pdwHelpContext)
@@ -159,15 +161,15 @@ STDMETHODIMP VirtualBoxErrorInfo::GetSource (BSTR *source)
     return COMGETTER(Component) (source);
 }
 
-#else // !defined (VBOX_WITH_XPCOM)
+#else // !defined(VBOX_WITH_XPCOM)
 
 /**
  *  Initializes itself by fetching error information from the given error info
  *  object.
  */
-HRESULT VirtualBoxErrorInfo::init (nsIException *aInfo)
+HRESULT VirtualBoxErrorInfo::init(nsIException *aInfo)
 {
-    AssertReturn (aInfo, E_FAIL);
+    AssertReturn(aInfo, E_FAIL);
 
     HRESULT rc = S_OK;
 
@@ -175,12 +177,19 @@ HRESULT VirtualBoxErrorInfo::init (nsIException *aInfo)
      * protect ourselves from bad nsIException implementations (the
      * corresponding fields will simply remain null in this case). */
 
-    rc = aInfo->GetResult (&mResultCode);
-    AssertComRC (rc);
-    Utf8Str message;
-    rc = aInfo->GetMessage (message.asOutParam());
-    AssertComRC (rc);
-    mText = message;
+    rc = aInfo->GetResult(&mResultCode);
+    AssertComRC(rc);
+
+    char *pszMsg;
+    rc = aInfo->GetMessage(&pszMsg);
+    AssertComRC(rc);
+    if (NS_SUCCEEDED(rc))
+    {
+        mText = Bstr(pszMsg);
+        nsMemory::Free(pszMsg);
+    }
+    else
+        mText.setNull();
 
     return S_OK;
 }
@@ -194,7 +203,7 @@ NS_IMETHODIMP VirtualBoxErrorInfo::GetMessage (char **aMessage)
     if (!aMessage)
         return NS_ERROR_INVALID_POINTER;
 
-    Utf8Str (mText).cloneTo (aMessage);
+    Utf8Str(mText).cloneTo(aMessage);
     return S_OK;
 }
 
@@ -244,10 +253,10 @@ NS_IMETHODIMP VirtualBoxErrorInfo::GetLocation (nsIStackFrame **aLocation)
 /* readonly attribute nsIException inner; */
 NS_IMETHODIMP VirtualBoxErrorInfo::GetInner (nsIException **aInner)
 {
-    ComPtr <IVirtualBoxErrorInfo> info;
+    ComPtr<IVirtualBoxErrorInfo> info;
     nsresult rv = COMGETTER(Next) (info.asOutParam());
-    CheckComRCReturnRC (rv);
-    return info.queryInterfaceTo (aInner);
+    if (FAILED(rv)) return rv;
+    return info.queryInterfaceTo(aInner);
 }
 
 /* readonly attribute nsISupports data; */
@@ -265,7 +274,7 @@ NS_IMETHODIMP VirtualBoxErrorInfo::ToString (char **_retval)
 NS_IMPL_THREADSAFE_ISUPPORTS2 (VirtualBoxErrorInfo,
                                nsIException, IVirtualBoxErrorInfo)
 
-#endif /* !defined (VBOX_WITH_XPCOM) */
+#endif /* !defined(VBOX_WITH_XPCOM) */
 
 ////////////////////////////////////////////////////////////////////////////////
 // VirtualBoxErrorInfoGlue class
@@ -283,21 +292,21 @@ NS_IMPL_THREADSAFE_ISUPPORTS2 (VirtualBoxErrorInfo,
 HRESULT VirtualBoxErrorInfoGlue::init (IVirtualBoxErrorInfo *aHead,
                                        IVirtualBoxErrorInfo *aTail)
 {
-    AssertReturn (aHead != NULL, E_INVALIDARG);
-    AssertReturn (aTail != NULL, E_INVALIDARG);
+    AssertReturn(aHead != NULL, E_INVALIDARG);
+    AssertReturn(aTail != NULL, E_INVALIDARG);
 
     HRESULT rc = S_OK;
 
-    typedef std::list <ComPtr <IVirtualBoxErrorInfo> > List;
+    typedef std::list <ComPtr<IVirtualBoxErrorInfo> > List;
     List list;
 
-    ComPtr <IVirtualBoxErrorInfo> cur = aHead;
+    ComPtr<IVirtualBoxErrorInfo> cur = aHead;
 
     do
     {
-        ComPtr <IVirtualBoxErrorInfo> next;
+        ComPtr<IVirtualBoxErrorInfo> next;
         rc = cur->COMGETTER(Next) (next.asOutParam());
-        CheckComRCReturnRC (rc);
+        if (FAILED(rc)) return rc;
 
         if (next.isNull())
             break;
@@ -317,9 +326,9 @@ HRESULT VirtualBoxErrorInfoGlue::init (IVirtualBoxErrorInfo *aHead,
 
     for (List::iterator it = list.end(), prev = it; it != list.begin(); -- it)
     {
-        ComObjPtr <VirtualBoxErrorInfoGlue> wrapper;
+        ComObjPtr<VirtualBoxErrorInfoGlue> wrapper;
         rc = wrapper.createObject();
-        CheckComRCBreakRC (rc);
+        if (FAILED(rc)) break;
 
         -- prev;
 
@@ -330,7 +339,7 @@ HRESULT VirtualBoxErrorInfoGlue::init (IVirtualBoxErrorInfo *aHead,
 
         *prev = wrapper;
 
-        CheckComRCBreakRC (rc);
+        if (FAILED(rc)) break;
     }
 
     mReal = aHead;
@@ -348,7 +357,7 @@ HRESULT VirtualBoxErrorInfoGlue::init (IVirtualBoxErrorInfo *aHead,
 HRESULT VirtualBoxErrorInfoGlue::protectedInit (IVirtualBoxErrorInfo *aReal,
                                                 IVirtualBoxErrorInfo *aNext)
 {
-    AssertReturn (aReal != NULL, E_INVALIDARG);
+    AssertReturn(aReal != NULL, E_INVALIDARG);
 
     mReal = aReal;
     mNext = aNext;
@@ -365,7 +374,7 @@ STDMETHODIMP VirtualBoxErrorInfoGlue::COMGETTER(Next) (IVirtualBoxErrorInfo **aN
         return E_POINTER;
 
     /* this will set aNext to NULL if mNext is null */
-    return mNext.queryInterfaceTo (aNext);
+    return mNext.queryInterfaceTo(aNext);
 }
 
 #if defined (VBOX_WITH_XPCOM)
