@@ -1,4 +1,4 @@
-/* $Id: ApplianceImpl.h $ */
+/* $Id: ApplianceImpl.h 29422 2010-05-12 14:08:52Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,31 +15,42 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_APPLIANCEIMPL
 #define ____H_APPLIANCEIMPL
 
+/* VBox includes */
 #include "VirtualBoxBase.h"
 
+/* VBox forward declarations */
+class Progress;
+class VirtualSystemDescription;
+struct VirtualSystemDescriptionEntry;
+
+namespace ovf
+{
+    struct HardDiskController;
+    struct VirtualSystem;
+    class OVFReader;
+    struct DiskImage;
+}
+
 namespace xml
 {
-    class Node;
     class ElementNode;
 }
 
-class VirtualBox;
-class Progress;
+namespace settings
+{
+    class MachineConfigFile;
+}
 
 class ATL_NO_VTABLE Appliance :
-    public VirtualBoxBaseWithChildrenNEXT,
-    public VirtualBoxSupportErrorInfoImpl <Appliance, IAppliance>,
-    public VirtualBoxSupportTranslation <Appliance>,
-    public IAppliance
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<Appliance, IAppliance>,
+    public VirtualBoxSupportTranslation<Appliance>,
+    VBOX_SCRIPTABLE_IMPL(IAppliance)
 {
 public:
     VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT (Appliance)
@@ -51,12 +62,18 @@ public:
     BEGIN_COM_MAP(Appliance)
         COM_INTERFACE_ENTRY(ISupportErrorInfo)
         COM_INTERFACE_ENTRY(IAppliance)
+        COM_INTERFACE_ENTRY(IDispatch)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (Appliance)
 
+    enum OVFFormat
+    {
+        unspecified,
+        OVF_0_9,
+        OVF_1_0
+    };
+
     // public initializer/uninitializer for internal purposes only
     HRESULT FinalConstruct() { return S_OK; }
     void FinalRelease() { uninit(); }
@@ -73,10 +90,14 @@ public:
     STDMETHOD(COMGETTER(VirtualSystemDescriptions))(ComSafeArrayOut(IVirtualSystemDescription*, aVirtualSystemDescriptions));
 
     /* IAppliance methods */
-    STDMETHOD(Read)(IN_BSTR path);
+    /* Import methods */
+    STDMETHOD(Read)(IN_BSTR path, IProgress **aProgress);
     STDMETHOD(Interpret)(void);
     STDMETHOD(ImportMachines)(IProgress **aProgress);
+    /* Export methods */
+    STDMETHOD(CreateVFSExplorer)(IN_BSTR aURI, IVFSExplorer **aExplorer);
     STDMETHOD(Write)(IN_BSTR format, IN_BSTR path, IProgress **aProgress);
+
     STDMETHOD(GetWarnings)(ComSafeArrayOut(BSTR, aWarnings));
 
     /* public methods only for internal purposes */
@@ -84,27 +105,74 @@ public:
     /* private instance data */
 private:
     /** weak VirtualBox parent */
-    const ComObjPtr <VirtualBox, ComWeakRef> mVirtualBox;
+    VirtualBox* const   mVirtualBox;
 
-    struct Data;            // obscure, defined in AppliannceImpl.cpp
+    struct Data;            // opaque, defined in ApplianceImpl.cpp
     Data *m;
 
-    HRESULT LoopThruSections(const char *pcszPath, const xml::ElementNode *pReferencesElem, const xml::ElementNode *pCurElem);
-    HRESULT HandleDiskSection(const char *pcszPath, const xml::ElementNode *pReferencesElem, const xml::ElementNode *pSectionElem);
-    HRESULT HandleNetworkSection(const char *pcszPath, const xml::ElementNode *pSectionElem);
-    HRESULT HandleVirtualSystemContent(const char *pcszPath, const xml::ElementNode *pContentElem);
+    bool isApplianceIdle() const;
 
     HRESULT searchUniqueVMName(Utf8Str& aName) const;
     HRESULT searchUniqueDiskImageFilePath(Utf8Str& aName) const;
-    HRESULT setUpProgress(ComObjPtr<Progress> &pProgress, const Bstr &bstrDescription);
+    HRESULT getDefaultHardDiskFolder(Utf8Str &str) const;
     void waitForAsyncProgress(ComObjPtr<Progress> &pProgressThis, ComPtr<IProgress> &pProgressAsync);
     void addWarning(const char* aWarning, ...);
 
-    struct TaskImportMachines;  /* Worker thread for import */
-    static DECLCALLBACK(int) taskThreadImportMachines(RTTHREAD thread, void *pvUser);
+    void disksWeight();
+    enum SetUpProgressMode { Regular, ImportS3, WriteS3 };
+    HRESULT setUpProgress(ComObjPtr<Progress> &pProgress,
+                          const Bstr &bstrDescription,
+                          SetUpProgressMode mode);
 
-    struct TaskWriteOVF;       /* Worker thread for export */
-    static DECLCALLBACK(int) taskThreadWriteOVF(RTTHREAD thread, void *pvUser);
+    struct LocationInfo;
+    void parseURI(Utf8Str strUri, LocationInfo &locInfo) const;
+    void parseBucket(Utf8Str &aPath, Utf8Str &aBucket) const;
+    Utf8Str manifestFileName(Utf8Str aPath) const;
+
+    HRESULT readImpl(const LocationInfo &aLocInfo, ComObjPtr<Progress> &aProgress);
+
+    struct TaskOVF;
+    static DECLCALLBACK(int) taskThreadImportOrExport(RTTHREAD aThread, void *pvUser);
+
+    HRESULT readFS(const LocationInfo &locInfo);
+    HRESULT readS3(TaskOVF *pTask);
+
+    void convertDiskAttachmentValues(const ovf::HardDiskController &hdc,
+                                     uint32_t ulAddressOnParent,
+                                     Bstr &controllerType,
+                                     int32_t &lControllerPort,
+                                     int32_t &lDevice);
+
+    HRESULT importImpl(const LocationInfo &aLocInfo, ComObjPtr<Progress> &aProgress);
+    HRESULT manifestVerify(const LocationInfo &locInfo, const ovf::OVFReader &reader);
+
+    HRESULT importFS(const LocationInfo &locInfo, ComObjPtr<Progress> &aProgress);
+
+    struct ImportStack;
+    void importOneDiskImage(const ovf::DiskImage &di,
+                            const Utf8Str &strTargetPath,
+                            ComPtr<IMedium> &pTargetHD,
+                            ImportStack &stack);
+    void importMachineGeneric(const ovf::VirtualSystem &vsysThis,
+                              ComObjPtr<VirtualSystemDescription> &vsdescThis,
+                              ComPtr<IMachine> &pNewMachine,
+                              ImportStack &stack);
+    void importVBoxMachine(ComObjPtr<VirtualSystemDescription> &vsdescThis,
+                           ComPtr<IMachine> &pNewMachine,
+                           ImportStack &stack);
+
+    HRESULT importS3(TaskOVF *pTask);
+
+    HRESULT writeImpl(OVFFormat aFormat, const LocationInfo &aLocInfo, ComObjPtr<Progress> &aProgress);
+
+    struct XMLStack;
+    void buildXMLForOneVirtualSystem(xml::ElementNode &elmToAddVirtualSystemsTo,
+                                     ComObjPtr<VirtualSystemDescription> &vsdescThis,
+                                     OVFFormat enFormat,
+                                     XMLStack &stack);
+
+    HRESULT writeFS(const LocationInfo &locInfo, const OVFFormat enFormat, ComObjPtr<Progress> &pProgress);
+    HRESULT writeS3(TaskOVF *pTask);
 
     friend class Machine;
 };
@@ -118,14 +186,14 @@ struct VirtualSystemDescriptionEntry
     Utf8Str strVbox;                        // configuration value (type-dependent)
     Utf8Str strExtraConfig;                 // extra configuration key=value strings (type-dependent)
 
-    uint32_t ulSizeMB;                      // hard disk images only: size of the uncompressed image in MB
+    uint32_t ulSizeMB;                      // hard disk images only: a copy of ovf::DiskImage::ulSuggestedSizeMB
 };
 
 class ATL_NO_VTABLE VirtualSystemDescription :
-    public VirtualBoxBaseWithChildrenNEXT,
-    public VirtualBoxSupportErrorInfoImpl <VirtualSystemDescription, IVirtualSystemDescription>,
-    public VirtualBoxSupportTranslation <VirtualSystemDescription>,
-    public IVirtualSystemDescription
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<VirtualSystemDescription, IVirtualSystemDescription>,
+    public VirtualBoxSupportTranslation<VirtualSystemDescription>,
+    VBOX_SCRIPTABLE_IMPL(IVirtualSystemDescription)
 {
     friend class Appliance;
 
@@ -139,10 +207,9 @@ public:
     BEGIN_COM_MAP(VirtualSystemDescription)
         COM_INTERFACE_ENTRY(ISupportErrorInfo)
         COM_INTERFACE_ENTRY(IVirtualSystemDescription)
+        COM_INTERFACE_ENTRY(IDispatch)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (VirtualSystemDescription)
 
     // public initializer/uninitializer for internal purposes only
@@ -188,14 +255,17 @@ public:
 
     void addEntry(VirtualSystemDescriptionType_T aType,
                   const Utf8Str &strRef,
-                  const Utf8Str &aOrigValue,
-                  const Utf8Str &aAutoValue,
+                  const Utf8Str &aOvfValue,
+                  const Utf8Str &aVboxValue,
                   uint32_t ulSizeMB = 0,
                   const Utf8Str &strExtraConfig = "");
 
     std::list<VirtualSystemDescriptionEntry*> findByType(VirtualSystemDescriptionType_T aType);
     const VirtualSystemDescriptionEntry* findControllerFromID(uint32_t id);
 
+    void importVboxMachineXML(const xml::ElementNode &elmMachine);
+    const settings::MachineConfigFile* getMachineConfig() const;
+
     /* private instance data */
 private:
     struct Data;
