@@ -1,4 +1,4 @@
-/* $Id: ConsoleVRDPServer.h $ */
+/* $Id: ConsoleVRDPServer.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_CONSOLEVRDPSERVER
@@ -125,7 +121,7 @@ public:
 
     void notifyRemoteUSBThreadRunning (RTTHREAD thread);
     bool isRemoteUSBThreadRunning (void);
-    void waitRemoteUSBThreadEvent (unsigned cMillies);
+    void waitRemoteUSBThreadEvent (RTMSINTERVAL cMillies);
 
     void ClipboardCreate (uint32_t u32ClientId);
     void ClipboardDelete (uint32_t u32ClientId);
@@ -184,6 +180,8 @@ private:
     IConsoleCallback *mConsoleCallback;
 
     VRDPInputSynch m_InputSynch;
+
+    int32_t mVRDPBindPort;
 #endif /* VBOX_WITH_VRDP */
 
     RTCRITSECT mCritSect;
@@ -232,9 +230,9 @@ private:
 class Console;
 
 class ATL_NO_VTABLE RemoteDisplayInfo :
-    public VirtualBoxBaseNEXT,
-    public VirtualBoxSupportErrorInfoImpl <RemoteDisplayInfo, IRemoteDisplayInfo>,
-    public VirtualBoxSupportTranslation <RemoteDisplayInfo>,
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<RemoteDisplayInfo, IRemoteDisplayInfo>,
+    public VirtualBoxSupportTranslation<RemoteDisplayInfo>,
     VBOX_SCRIPTABLE_IMPL(IRemoteDisplayInfo)
 {
 public:
@@ -251,8 +249,6 @@ public:
         COM_INTERFACE_ENTRY(IDispatch)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (RemoteDisplayInfo)
 
     HRESULT FinalConstruct();
@@ -265,6 +261,7 @@ public:
     /* IRemoteDisplayInfo properties */
     #define DECL_GETTER(_aType, _aName) STDMETHOD(COMGETTER(_aName)) (_aType *a##_aName)
         DECL_GETTER (BOOL,    Active);
+        DECL_GETTER (LONG,    Port);
         DECL_GETTER (ULONG,   NumberOfClients);
         DECL_GETTER (LONG64,  BeginTime);
         DECL_GETTER (LONG64,  EndTime);
@@ -285,7 +282,7 @@ public:
 
 private:
 
-    const ComObjPtr <Console, ComWeakRef> mParent;
+    Console * const         mParent;
 };
 
 #endif // ____H_CONSOLEVRDPSERVER
