@@ -1,4 +1,4 @@
-/* $Id: DHCPServerImpl.h $ */
+/* $Id: DHCPServerImpl.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,29 +15,27 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_H_DHCPSERVERIMPL
 #define ____H_H_DHCPSERVERIMPL
 
 #include "VirtualBoxBase.h"
-#include "VirtualBoxImpl.h"
 
 #ifdef VBOX_WITH_HOSTNETIF_API
-/* class DHCPServer; */
-/* #include "netif.h" */
 struct NETIFINFO;
 #endif
 
+namespace settings
+{
+    struct DHCPServer;
+}
+
 class ATL_NO_VTABLE DHCPServer :
-    public VirtualBoxBaseNEXT,
-    public VirtualBoxSupportErrorInfoImpl <DHCPServer, IDHCPServer>,
-    public VirtualBoxSupportTranslation <DHCPServer>,
-    public IDHCPServer
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<DHCPServer, IDHCPServer>,
+    public VirtualBoxSupportTranslation<DHCPServer>,
+    VBOX_SCRIPTABLE_IMPL(IDHCPServer)
 {
 public:
 
@@ -50,41 +48,42 @@ public:
     BEGIN_COM_MAP (DHCPServer)
         COM_INTERFACE_ENTRY (ISupportErrorInfo)
         COM_INTERFACE_ENTRY (IDHCPServer)
+        COM_INTERFACE_ENTRY (IDispatch)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (DHCPServer)
 
     HRESULT FinalConstruct();
     void FinalRelease();
 
-    HRESULT init(VirtualBox *aVirtualBox, IN_BSTR aName);
-    HRESULT init(VirtualBox *aVirtualBox, const settings::Key &aNode);
-    HRESULT saveSettings (settings::Key &aParentNode);
+    HRESULT init(VirtualBox *aVirtualBox,
+                 IN_BSTR aName);
+    HRESULT init(VirtualBox *aVirtualBox,
+                 const settings::DHCPServer &data);
+    HRESULT saveSettings(settings::DHCPServer &data);
 
     void uninit();
 
     // IDHCPServer properties
-    STDMETHOD(COMGETTER(NetworkName)) (BSTR *aName);
-    STDMETHOD(COMGETTER(Enabled)) (BOOL *aEnabled);
-    STDMETHOD(COMSETTER(Enabled)) (BOOL aEnabled);
-    STDMETHOD(COMGETTER(IPAddress)) (BSTR *aIPAddress);
-    STDMETHOD(COMGETTER(NetworkMask)) (BSTR *aNetworkMask);
-    STDMETHOD(COMGETTER(LowerIP)) (BSTR *aIPAddress);
-    STDMETHOD(COMGETTER(UpperIP)) (BSTR *aIPAddress);
+    STDMETHOD(COMGETTER(NetworkName))(BSTR *aName);
+    STDMETHOD(COMGETTER(Enabled))(BOOL *aEnabled);
+    STDMETHOD(COMSETTER(Enabled))(BOOL aEnabled);
+    STDMETHOD(COMGETTER(IPAddress))(BSTR *aIPAddress);
+    STDMETHOD(COMGETTER(NetworkMask))(BSTR *aNetworkMask);
+    STDMETHOD(COMGETTER(LowerIP))(BSTR *aIPAddress);
+    STDMETHOD(COMGETTER(UpperIP))(BSTR *aIPAddress);
 
-    STDMETHOD(SetConfiguration) (IN_BSTR aIPAddress, IN_BSTR aNetworkMask, IN_BSTR aFromIPAddress, IN_BSTR aToIPAddress);
+    STDMETHOD(SetConfiguration)(IN_BSTR aIPAddress, IN_BSTR aNetworkMask, IN_BSTR aFromIPAddress, IN_BSTR aToIPAddress);
 
-    STDMETHOD(Start) (IN_BSTR aNetworkName, IN_BSTR aTrunkName, IN_BSTR aTrunkType);
-    STDMETHOD(Stop) ();
+    STDMETHOD(Start)(IN_BSTR aNetworkName, IN_BSTR aTrunkName, IN_BSTR aTrunkType);
+    STDMETHOD(Stop)();
 
     // for VirtualBoxSupportErrorInfoImpl
     static const wchar_t *getComponentName() { return L"DHCPServer"; }
 
 private:
     /** weak VirtualBox parent */
-    const ComObjPtr <VirtualBox, ComWeakRef> mVirtualBox;
+    VirtualBox * const      mVirtualBox;
 
     const Bstr mName;
 
@@ -104,4 +103,3 @@ private:
 };
 
 #endif // ____H_H_DHCPSERVERIMPL
-/* vi: set tabstop=4 shiftwidth=4 expandtab: */
