@@ -1,4 +1,4 @@
-/* $Id: HostNetworkInterfaceImpl.h $ */
+/* $Id: HostNetworkInterfaceImpl.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_HOSTNETWORKINTERFACEIMPL
@@ -34,10 +30,10 @@ struct NETIFINFO;
 #endif
 
 class ATL_NO_VTABLE HostNetworkInterface :
-    public VirtualBoxBaseNEXT,
-    public VirtualBoxSupportErrorInfoImpl <HostNetworkInterface, IHostNetworkInterface>,
-    public VirtualBoxSupportTranslation <HostNetworkInterface>,
-    public IHostNetworkInterface
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<HostNetworkInterface, IHostNetworkInterface>,
+    public VirtualBoxSupportTranslation<HostNetworkInterface>,
+    VBOX_SCRIPTABLE_IMPL(IHostNetworkInterface)
 {
 public:
 
@@ -50,10 +46,9 @@ public:
     BEGIN_COM_MAP (HostNetworkInterface)
         COM_INTERFACE_ENTRY (ISupportErrorInfo)
         COM_INTERFACE_ENTRY (IHostNetworkInterface)
+        COM_INTERFACE_ENTRY (IDispatch)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (HostNetworkInterface)
 
     HRESULT FinalConstruct();
@@ -68,7 +63,7 @@ public:
 
     // IHostNetworkInterface properties
     STDMETHOD(COMGETTER(Name)) (BSTR *aInterfaceName);
-    STDMETHOD(COMGETTER(Id)) (OUT_GUID aGuid);
+    STDMETHOD(COMGETTER(Id)) (BSTR *aGuid);
     STDMETHOD(COMGETTER(DhcpEnabled)) (BOOL *aDhcpEnabled);
     STDMETHOD(COMGETTER(IPAddress)) (BSTR *aIPAddress);
     STDMETHOD(COMGETTER(NetworkMask)) (BSTR *aNetworkMask);
@@ -95,7 +90,7 @@ private:
     const Guid mGuid;
     HostNetworkInterfaceType_T mIfType;
 
-    ComObjPtr <VirtualBox, ComWeakRef> mVBox;
+    VirtualBox * const  mVBox;
 
     struct Data
     {
