@@ -1,4 +1,4 @@
-/* $Id: NetworkAdapterImpl.h $ */
+/* $Id: NetworkAdapterImpl.h 28867 2010-04-28 13:28:20Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,52 +15,45 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_NETWORKADAPTER
 #define ____H_NETWORKADAPTER
 
 #include "VirtualBoxBase.h"
+#include "NATEngineImpl.h"
 
-class Machine;
 class GuestOSType;
 
+namespace settings
+{
+    struct NetworkAdapter;
+}
+
 class ATL_NO_VTABLE NetworkAdapter :
-    public VirtualBoxBaseNEXT,
-    public VirtualBoxSupportErrorInfoImpl <NetworkAdapter, INetworkAdapter>,
-    public VirtualBoxSupportTranslation <NetworkAdapter>,
-    public INetworkAdapter
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<NetworkAdapter, INetworkAdapter>,
+    public VirtualBoxSupportTranslation<NetworkAdapter>,
+    VBOX_SCRIPTABLE_IMPL(INetworkAdapter)
 {
 public:
 
     struct Data
     {
-        Data()
-            : mSlot (0), mEnabled (FALSE)
-            , mAttachmentType (NetworkAttachmentType_Null)
-            ,  mCableConnected (TRUE), mLineSpeed (0), mTraceEnabled (FALSE)
-            , mHostInterface ("") /* cannot be null */
+        Data() : mSlot(0),
+                 mEnabled(FALSE),
+                 mAttachmentType(NetworkAttachmentType_Null),
+                 mCableConnected(TRUE),
+                 mLineSpeed(0),
+                 mTraceEnabled(FALSE),
+                 mHostInterface("") /* cannot be null */,
+#ifdef VBOX_WITH_VDE
+                 mVDENetwork("") /* can be null */,
+#endif
+                 mNATNetwork("") /* cannot be null */,
+                 mBootPriority(0)
         {}
 
-        bool operator== (const Data &that) const
-        {
-            return this == &that ||
-                   (mSlot == that.mSlot &&
-                    mEnabled == that.mEnabled &&
-                    mMACAddress == that.mMACAddress &&
-                    mAttachmentType == that.mAttachmentType &&
-                    mCableConnected == that.mCableConnected &&
-                    mLineSpeed == that.mLineSpeed &&
-                    mTraceEnabled == that.mTraceEnabled &&
-                    mHostInterface == that.mHostInterface &&
-                    mInternalNetwork == that.mInternalNetwork &&
-                    mNATNetwork == that.mNATNetwork);
-        }
-
         NetworkAdapterType_T mAdapterType;
         ULONG mSlot;
         BOOL mEnabled;
@@ -72,7 +65,11 @@ public:
         Bstr mTraceFile;
         Bstr mHostInterface;
         Bstr mInternalNetwork;
+#ifdef VBOX_WITH_VDE
+        Bstr mVDENetwork;
+#endif
         Bstr mNATNetwork;
+        ULONG mBootPriority;
     };
 
     VIRTUALBOXBASE_ADD_ERRORINFO_SUPPORT (NetworkAdapter)
@@ -82,12 +79,11 @@ public:
     DECLARE_PROTECT_FINAL_CONSTRUCT()
 
     BEGIN_COM_MAP(NetworkAdapter)
-        COM_INTERFACE_ENTRY(ISupportErrorInfo)
-        COM_INTERFACE_ENTRY(INetworkAdapter)
+        COM_INTERFACE_ENTRY  (ISupportErrorInfo)
+        COM_INTERFACE_ENTRY  (INetworkAdapter)
+        COM_INTERFACE_ENTRY2 (IDispatch, INetworkAdapter)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (NetworkAdapter)
 
     HRESULT FinalConstruct();
@@ -114,6 +110,8 @@ public:
     STDMETHOD(COMSETTER(InternalNetwork)) (IN_BSTR aInternalNetwork);
     STDMETHOD(COMGETTER(NATNetwork)) (BSTR *aNATNetwork);
     STDMETHOD(COMSETTER(NATNetwork)) (IN_BSTR aNATNetwork);
+    STDMETHOD(COMGETTER(VDENetwork)) (BSTR *aVDENetwork);
+    STDMETHOD(COMSETTER(VDENetwork)) (IN_BSTR aVDENetwork);
     STDMETHOD(COMGETTER(CableConnected)) (BOOL *aConnected);
     STDMETHOD(COMSETTER(CableConnected)) (BOOL aConnected);
     STDMETHOD(COMGETTER(TraceEnabled)) (BOOL *aEnabled);
@@ -122,31 +120,29 @@ public:
     STDMETHOD(COMSETTER(LineSpeed)) (ULONG aSpeed);
     STDMETHOD(COMGETTER(TraceFile)) (BSTR *aTraceFile);
     STDMETHOD(COMSETTER(TraceFile)) (IN_BSTR aTraceFile);
+    STDMETHOD(COMGETTER(NatDriver)) (INATEngine **aNatDriver);
+    STDMETHOD(COMGETTER(BootPriority)) (ULONG *aBootPriority);
+    STDMETHOD(COMSETTER(BootPriority)) (ULONG aBootPriority);
 
     // INetworkAdapter methods
     STDMETHOD(AttachToNAT)();
     STDMETHOD(AttachToBridgedInterface)();
     STDMETHOD(AttachToInternalNetwork)();
     STDMETHOD(AttachToHostOnlyInterface)();
+    STDMETHOD(AttachToVDE)();
     STDMETHOD(Detach)();
 
     // public methods only for internal purposes
 
-    HRESULT loadSettings (const settings::Key &aAdapterNode);
-    HRESULT saveSettings (settings::Key &aAdapterNode);
+    HRESULT loadSettings(const settings::NetworkAdapter &data);
+    HRESULT saveSettings(settings::NetworkAdapter &data);
 
-    bool isModified() { AutoWriteLock alock (this); return mData.isBackedUp(); }
-    bool isReallyModified() { AutoWriteLock alock (this); return mData.hasActualChanges(); }
-    bool rollback();
+    bool isModified();
+    void rollback();
     void commit();
     void copyFrom (NetworkAdapter *aThat);
     void applyDefaults (GuestOSType *aOsType);
 
-    // public methods for internal purposes only
-    // (ensure there is a caller and a read lock before calling them!)
-
-    const Backupable <Data> &data() const { return mData; }
-
     // for VirtualBoxSupportErrorInfoImpl
     static const wchar_t *getComponentName() { return L"NetworkAdapter"; }
 
@@ -155,10 +151,12 @@ private:
     void detach();
     void generateMACAddress();
 
-    const ComObjPtr <Machine, ComWeakRef> mParent;
-    const ComObjPtr <NetworkAdapter> mPeer;
+    Machine * const     mParent;
+    const ComObjPtr<NetworkAdapter> mPeer;
+    const ComObjPtr<NATEngine> mNATEngine;
 
-    Backupable <Data> mData;
+    bool                m_fModified;
+    Backupable<Data>    mData;
 };
 
 #endif // ____H_NETWORKADAPTER
