@@ -1,4 +1,4 @@
-/* $Id: Performance.h $ */
+/* $Id: Performance.h 29632 2010-05-18 13:22:37Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,24 +15,26 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
-
+#ifndef ___performance_h
+#define ___performance_h
 
 #include <VBox/com/defs.h>
 #include <VBox/com/ptr.h>
+#include <VBox/com/string.h>
+#include <VBox/com/VirtualBox.h>
 
 #include <iprt/types.h>
 #include <iprt/err.h>
 
 #include <algorithm>
+#include <functional> /* For std::fun_ptr in testcase */
 #include <list>
-#include <string>
 #include <vector>
 
+/* Forward decl. */
+class Machine;
+
 namespace pm
 {
     /* CPU load is measured in 1/1000 of per cent. */
@@ -72,18 +74,13 @@ namespace pm
 
     /* Collector Hardware Abstraction Layer *********************************/
     enum {
-        COLLECT_NONE      = 0x0,
-        COLLECT_CPU_LOAD  = 0x1,
-        COLLECT_RAM_USAGE = 0x2
+        COLLECT_NONE        = 0x0,
+        COLLECT_CPU_LOAD    = 0x1,
+        COLLECT_RAM_USAGE   = 0x2
     };
     typedef int HintFlags;
     typedef std::pair<RTPROCESS, HintFlags> ProcessFlagsPair;
 
-    inline bool processEqual(ProcessFlagsPair pair, RTPROCESS process)
-    {
-        return pair.first == process;
-    }
-
     class CollectorHints
     {
     public:
@@ -124,13 +121,9 @@ namespace pm
         ProcessFlagsPair& findProcess(RTPROCESS process)
         {
             ProcessList::iterator it;
-
-            it = std::find_if(mProcesses.begin(),
-                              mProcesses.end(),
-                              std::bind2nd(std::ptr_fun(processEqual), process));
-
-            if (it != mProcesses.end())
-                return *it;
+            for (it = mProcesses.begin(); it != mProcesses.end(); it++)
+                if (it->first == process)
+                    return *it;
 
             /* Not found -- add new */
             mProcesses.push_back(ProcessFlagsPair(process, COLLECT_NONE));
@@ -141,16 +134,107 @@ namespace pm
     class CollectorHAL
     {
     public:
+                 CollectorHAL() : mMemAllocVMM(0), mMemFreeVMM(0), mMemBalloonedVMM(0), mMemSharedVMM(0) {};
         virtual ~CollectorHAL() { };
-        virtual int preCollect(const CollectorHints& /* hints */) { return VINF_SUCCESS; }
+        virtual int preCollect(const CollectorHints& /* hints */, uint64_t /* iTick */) { return VINF_SUCCESS; }
+        /** Returns averaged CPU usage in 1/1000th per cent across all host's CPUs. */
         virtual int getHostCpuLoad(ULONG *user, ULONG *kernel, ULONG *idle);
+        /** Returns the average frequency in MHz across all host's CPUs. */
         virtual int getHostCpuMHz(ULONG *mhz);
-        virtual int getHostMemoryUsage(ULONG *total, ULONG *used, ULONG *available) = 0;
+        /** Returns the amount of physical memory in kilobytes. */
+        virtual int getHostMemoryUsage(ULONG *total, ULONG *used, ULONG *available);
+        /** Returns CPU usage in 1/1000th per cent by a particular process. */
         virtual int getProcessCpuLoad(RTPROCESS process, ULONG *user, ULONG *kernel);
-        virtual int getProcessMemoryUsage(RTPROCESS process, ULONG *used) = 0;
+        /** Returns the amount of memory used by a process in kilobytes. */
+        virtual int getProcessMemoryUsage(RTPROCESS process, ULONG *used);
 
+        /** Returns CPU usage counters in platform-specific units. */
         virtual int getRawHostCpuLoad(uint64_t *user, uint64_t *kernel, uint64_t *idle);
+        /** Returns process' CPU usage counter in platform-specific units. */
         virtual int getRawProcessCpuLoad(RTPROCESS process, uint64_t *user, uint64_t *kernel, uint64_t *total);
+
+        /** Enable metrics collecting (if applicable) */
+        virtual int enable();
+        /** Disable metrics collecting (if applicable) */
+        virtual int disable();
+
+        virtual int setMemHypervisorStats(ULONG memAlloc, ULONG memFree, ULONG memBallooned, ULONG memShared)
+        {
+            mMemAllocVMM     = memAlloc;
+            mMemFreeVMM      = memFree;
+            mMemBalloonedVMM = memBallooned;
+            mMemSharedVMM    = memShared;
+            return S_OK;
+        }
+
+        virtual void getMemHypervisorStats(ULONG *pMemAlloc, ULONG *pMemFree, ULONG *pMemBallooned, ULONG *pMemShared)
+        {
+            *pMemAlloc     = mMemAllocVMM;
+            *pMemFree      = mMemFreeVMM;
+            *pMemBallooned = mMemBalloonedVMM;
+            *pMemShared    = mMemSharedVMM;
+        }
+
+    private:
+        ULONG       mMemAllocVMM;
+        ULONG       mMemFreeVMM;
+        ULONG       mMemBalloonedVMM;
+        ULONG       mMemSharedVMM;
+    };
+
+    class CollectorGuestHAL : public CollectorHAL
+    {
+    public:
+        CollectorGuestHAL(Machine *machine, CollectorHAL *hostHAL) : CollectorHAL(), cEnabled(0), mMachine(machine), mConsole(NULL), mGuest(NULL),
+                                              mLastTick(0), mHostHAL(hostHAL), mCpuUser(0), mCpuKernel(0), mCpuIdle(0), mMemTotal(0), mMemFree(0),
+                                              mMemBalloon(0), mMemShared(0), mMemCache(0), mPageTotal(0) {};
+        ~CollectorGuestHAL();
+
+        virtual int preCollect(const CollectorHints& hints, uint64_t iTick);
+
+        /** Enable metrics collecting (if applicable) */
+        virtual int enable();
+        /** Disable metrics collecting (if applicable) */
+        virtual int disable();
+
+        /** Return guest cpu absolute load values (0-100). */
+        void getGuestCpuLoad(ULONG *pulCpuUser, ULONG *pulCpuKernel, ULONG *pulCpuIdle)
+        {
+            *pulCpuUser   = mCpuUser;
+            *pulCpuKernel = mCpuKernel;
+            *pulCpuIdle   = mCpuIdle;
+        }
+
+        /** Return guest memory information in KB. */
+        void getGuestMemLoad(ULONG *pulMemTotal, ULONG *pulMemFree, ULONG *pulMemBalloon, ULONG *pulMemShared, ULONG *pulMemCache, ULONG *pulPageTotal)
+        {
+            *pulMemTotal        = mMemTotal;
+            *pulMemFree         = mMemFree;
+            *pulMemBalloon      = mMemBalloon;
+            *pulMemShared       = mMemShared;
+            *pulMemCache        = mMemCache;
+            *pulPageTotal       = mPageTotal;
+        }
+
+
+    protected:
+        uint32_t             cEnabled;
+        Machine             *mMachine;
+        ComPtr<IConsole>     mConsole;
+        ComPtr<IGuest>       mGuest;
+        uint64_t             mLastTick;
+
+        CollectorHAL        *mHostHAL;
+
+        ULONG                mCpuUser;
+        ULONG                mCpuKernel;
+        ULONG                mCpuIdle;
+        ULONG                mMemTotal;
+        ULONG                mMemFree;
+        ULONG                mMemBalloon;
+        ULONG                mMemShared;
+        ULONG                mMemCache;
+        ULONG                mPageTotal;
     };
 
     extern CollectorHAL *createHAL();
@@ -160,11 +244,11 @@ namespace pm
     {
     public:
         BaseMetric(CollectorHAL *hal, const char *name, ComPtr<IUnknown> object)
-            : mHAL(hal), mPeriod(0), mLength(0), mName(name), mObject(object), mLastSampleTaken(0), mEnabled(false) {};
+            : mPeriod(0), mLength(0), mHAL(hal), mName(name), mObject(object), mLastSampleTaken(0), mEnabled(false) {};
         virtual ~BaseMetric() {};
 
         virtual void init(ULONG period, ULONG length) = 0;
-        virtual void preCollect(CollectorHints& hints) = 0;
+        virtual void preCollect(CollectorHints& hints, uint64_t iTick) = 0;
         virtual void collect() = 0;
         virtual const char *getUnit() = 0;
         virtual ULONG getMinValue() = 0;
@@ -173,8 +257,16 @@ namespace pm
 
         bool collectorBeat(uint64_t nowAt);
 
-        void enable() { mEnabled = true; };
-        void disable() { mEnabled = false; };
+        void enable()
+        {
+            mEnabled = true;
+            mHAL->enable();
+        };
+        void disable()
+        {
+            mHAL->disable();
+            mEnabled = false;
+        };
 
         bool isEnabled() { return mEnabled; };
         ULONG getPeriod() { return mPeriod; };
@@ -184,9 +276,9 @@ namespace pm
         bool associatedWith(ComPtr<IUnknown> object) { return mObject == object; };
 
     protected:
+        ULONG           mPeriod;
+        ULONG           mLength;
         CollectorHAL    *mHAL;
-        ULONG    mPeriod;
-        ULONG    mLength;
         const char      *mName;
         ComPtr<IUnknown> mObject;
         uint64_t         mLastSampleTaken;
@@ -220,7 +312,7 @@ namespace pm
         HostCpuLoadRaw(CollectorHAL *hal, ComPtr<IUnknown> object, SubMetric *user, SubMetric *kernel, SubMetric *idle)
         : HostCpuLoad(hal, object, user, kernel, idle), mUserPrev(0), mKernelPrev(0), mIdlePrev(0) {};
 
-        void preCollect(CollectorHints& hints);
+        void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
     private:
         uint64_t mUserPrev;
@@ -236,7 +328,7 @@ namespace pm
         ~HostCpuMhz() { delete mMHz; };
 
         void init(ULONG period, ULONG length);
-        void preCollect(CollectorHints& /* hints */) {}
+        void preCollect(CollectorHints& /* hints */, uint64_t /* iTick */) {}
         void collect();
         const char *getUnit() { return "MHz"; };
         ULONG getMinValue() { return 0; };
@@ -249,12 +341,12 @@ namespace pm
     class HostRamUsage : public BaseMetric
     {
     public:
-        HostRamUsage(CollectorHAL *hal, ComPtr<IUnknown> object, SubMetric *total, SubMetric *used, SubMetric *available)
-        : BaseMetric(hal, "RAM/Usage", object), mTotal(total), mUsed(used), mAvailable(available) {};
-        ~HostRamUsage() { delete mTotal; delete mUsed; delete mAvailable; };
+        HostRamUsage(CollectorHAL *hal, ComPtr<IUnknown> object, SubMetric *total, SubMetric *used, SubMetric *available, SubMetric *allocVMM, SubMetric *freeVMM, SubMetric *balloonVMM, SubMetric *sharedVMM)
+        : BaseMetric(hal, "RAM/Usage", object), mTotal(total), mUsed(used), mAvailable(available), mAllocVMM(allocVMM), mFreeVMM(freeVMM), mBalloonVMM(balloonVMM), mSharedVMM(sharedVMM) {};
+        ~HostRamUsage() { delete mTotal; delete mUsed; delete mAvailable; delete mAllocVMM; delete mFreeVMM; delete mBalloonVMM; delete mSharedVMM; };
 
         void init(ULONG period, ULONG length);
-        void preCollect(CollectorHints& hints);
+        void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
         const char *getUnit() { return "kB"; };
         ULONG getMinValue() { return 0; };
@@ -264,6 +356,10 @@ namespace pm
         SubMetric *mTotal;
         SubMetric *mUsed;
         SubMetric *mAvailable;
+        SubMetric *mAllocVMM;
+        SubMetric *mFreeVMM;
+        SubMetric *mBalloonVMM;
+        SubMetric *mSharedVMM;
     };
 
     class MachineCpuLoad : public BaseMetric
@@ -291,7 +387,7 @@ namespace pm
         MachineCpuLoadRaw(CollectorHAL *hal, ComPtr<IUnknown> object, RTPROCESS process, SubMetric *user, SubMetric *kernel)
         : MachineCpuLoad(hal, object, process, user, kernel), mHostTotalPrev(0), mProcessUserPrev(0), mProcessKernelPrev(0) {};
 
-        void preCollect(CollectorHints& hints);
+        void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
     private:
         uint64_t mHostTotalPrev;
@@ -307,7 +403,7 @@ namespace pm
         ~MachineRamUsage() { delete mUsed; };
 
         void init(ULONG period, ULONG length);
-        void preCollect(CollectorHints& hints);
+        void preCollect(CollectorHints& hints, uint64_t iTick);
         void collect();
         const char *getUnit() { return "kB"; };
         ULONG getMinValue() { return 0; };
@@ -318,6 +414,47 @@ namespace pm
         SubMetric *mUsed;
     };
 
+
+    class GuestCpuLoad : public BaseMetric
+    {
+    public:
+        GuestCpuLoad(CollectorGuestHAL *hal, ComPtr<IUnknown> object, SubMetric *user, SubMetric *kernel, SubMetric *idle)
+        : BaseMetric(hal, "CPU/Load", object), mUser(user), mKernel(kernel), mIdle(idle), mGuestHAL(hal) {};
+        ~GuestCpuLoad() { delete mUser; delete mKernel; delete mIdle; };
+
+        void init(ULONG period, ULONG length);
+        void preCollect(CollectorHints& hints, uint64_t iTick);
+        void collect();
+        const char *getUnit() { return "%"; };
+        ULONG getMinValue() { return 0; };
+        ULONG getMaxValue() { return PM_CPU_LOAD_MULTIPLIER; };
+        ULONG getScale() { return PM_CPU_LOAD_MULTIPLIER / 100; }
+    protected:
+        SubMetric *mUser;
+        SubMetric *mKernel;
+        SubMetric *mIdle;
+        CollectorGuestHAL *mGuestHAL;
+    };
+
+    class GuestRamUsage : public BaseMetric
+    {
+    public:
+        GuestRamUsage(CollectorGuestHAL *hal, ComPtr<IUnknown> object, SubMetric *total, SubMetric *free, SubMetric *balloon, SubMetric *shared, SubMetric *cache, SubMetric *pagedtotal)
+        : BaseMetric(hal, "RAM/Usage", object), mTotal(total), mFree(free), mBallooned(balloon), mCache(cache), mPagedTotal(pagedtotal), mShared(shared), mGuestHAL(hal) {};
+        ~GuestRamUsage() { delete mTotal; delete mFree; delete mBallooned; delete mShared; delete mCache; delete mPagedTotal; };
+
+        void init(ULONG period, ULONG length);
+        void preCollect(CollectorHints& hints, uint64_t iTick);
+        void collect();
+        const char *getUnit() { return "kB"; };
+        ULONG getMinValue() { return 0; };
+        ULONG getMaxValue() { return INT32_MAX; };
+        ULONG getScale() { return 1; }
+    private:
+        SubMetric *mTotal, *mFree, *mBallooned, *mCache, *mPagedTotal, *mShared;
+        CollectorGuestHAL *mGuestHAL;
+    };
+
     /* Aggregate Functions **************************************************/
     class Aggregate
     {
@@ -356,8 +493,8 @@ namespace pm
         {
             if (mAggregate)
             {
-                mName += ":";
-                mName += mAggregate->getName();
+                mName.append(":");
+                mName.append(mAggregate->getName());
             }
         }
 
@@ -381,7 +518,7 @@ namespace pm
         void query(ULONG **data, ULONG *count, ULONG *sequenceNumber);
 
     private:
-        std::string mName;
+        iprt::MiniString mName;
         BaseMetric *mBaseMetric;
         SubMetric  *mSubMetric;
         Aggregate  *mAggregate;
@@ -396,18 +533,18 @@ namespace pm
                ComSafeArrayIn(IUnknown * , objects));
         static bool patternMatch(const char *pszPat, const char *pszName,
                                  bool fSeenColon = false);
-        bool match(const ComPtr<IUnknown> object, const std::string &name) const;
+        bool match(const ComPtr<IUnknown> object, const iprt::MiniString &name) const;
     private:
         void init(ComSafeArrayIn(IN_BSTR, metricNames),
                   ComSafeArrayIn(IUnknown * , objects));
 
-        typedef std::pair<const ComPtr<IUnknown>, const std::string> FilterElement;
+        typedef std::pair<const ComPtr<IUnknown>, const iprt::MiniString> FilterElement;
         typedef std::list<FilterElement> ElementList;
 
         ElementList mElements;
 
-        void processMetricList(const std::string &name, const ComPtr<IUnknown> object);
+        void processMetricList(const com::Utf8Str &name, const ComPtr<IUnknown> object);
     };
 }
-
+#endif /* ___performance_h */
 /* vi: set tabstop=4 shiftwidth=4 expandtab: */
