@@ -1,4 +1,4 @@
-/* $Id: RemoteUSBDeviceImpl.h $ */
+/* $Id: RemoteUSBDeviceImpl.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -7,7 +7,7 @@
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -16,10 +16,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_REMOTEUSBDEVICEIMPL
@@ -31,10 +27,10 @@ struct _VRDPUSBDEVICEDESC;
 typedef _VRDPUSBDEVICEDESC VRDPUSBDEVICEDESC;
 
 class ATL_NO_VTABLE RemoteUSBDevice :
-    public VirtualBoxBaseNEXT,
-    public VirtualBoxSupportErrorInfoImpl <RemoteUSBDevice, IHostUSBDevice>,
-    public VirtualBoxSupportTranslation <RemoteUSBDevice>,
-    public IHostUSBDevice
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<RemoteUSBDevice, IHostUSBDevice>,
+    public VirtualBoxSupportTranslation<RemoteUSBDevice>,
+    VBOX_SCRIPTABLE_IMPL(IHostUSBDevice)
 {
 public:
 
@@ -45,13 +41,12 @@ public:
     DECLARE_PROTECT_FINAL_CONSTRUCT()
 
     BEGIN_COM_MAP (RemoteUSBDevice)
-        COM_INTERFACE_ENTRY (ISupportErrorInfo)
-        COM_INTERFACE_ENTRY (IHostUSBDevice)
-        COM_INTERFACE_ENTRY (IUSBDevice)
+        COM_INTERFACE_ENTRY  (ISupportErrorInfo)
+        COM_INTERFACE_ENTRY  (IHostUSBDevice)
+        COM_INTERFACE_ENTRY  (IUSBDevice)
+        COM_INTERFACE_ENTRY2 (IDispatch, IUSBDevice)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (RemoteUSBDevice)
 
     HRESULT FinalConstruct();
@@ -62,7 +57,7 @@ public:
     void uninit();
 
     // IUSBDevice properties
-    STDMETHOD(COMGETTER(Id)) (OUT_GUID aId);
+    STDMETHOD(COMGETTER(Id)) (BSTR *aId);
     STDMETHOD(COMGETTER(VendorId)) (USHORT *aVendorId);
     STDMETHOD(COMGETTER(ProductId)) (USHORT *aProductId);
     STDMETHOD(COMGETTER(Revision)) (USHORT *aRevision);
