@@ -1,4 +1,4 @@
-/* $Id: SystemPropertiesImpl.h $ */
+/* $Id: SystemPropertiesImpl.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,29 +15,28 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_SYSTEMPROPERTIESIMPL
 #define ____H_SYSTEMPROPERTIESIMPL
 
 #include "VirtualBoxBase.h"
-#include "HardDiskFormatImpl.h"
+#include "MediumFormatImpl.h"
 
 #include <VBox/com/array.h>
 
 #include <list>
 
-class VirtualBox;
+namespace settings
+{
+    struct SystemProperties;
+}
 
 class ATL_NO_VTABLE SystemProperties :
-    public VirtualBoxBaseNEXT,
-    public VirtualBoxSupportErrorInfoImpl <SystemProperties, ISystemProperties>,
-    public VirtualBoxSupportTranslation <SystemProperties>,
-    public ISystemProperties
+    public VirtualBoxBase,
+    public VirtualBoxSupportErrorInfoImpl<SystemProperties, ISystemProperties>,
+    public VirtualBoxSupportTranslation<SystemProperties>,
+    VBOX_SCRIPTABLE_IMPL(ISystemProperties)
 {
 public:
 
@@ -48,12 +47,11 @@ public:
     DECLARE_PROTECT_FINAL_CONSTRUCT()
 
     BEGIN_COM_MAP(SystemProperties)
-        COM_INTERFACE_ENTRY(ISupportErrorInfo)
-        COM_INTERFACE_ENTRY(ISystemProperties)
+        COM_INTERFACE_ENTRY  (ISupportErrorInfo)
+        COM_INTERFACE_ENTRY  (ISystemProperties)
+        COM_INTERFACE_ENTRY2 (IDispatch, ISystemProperties)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (SystemProperties)
 
     HRESULT FinalConstruct();
@@ -80,68 +78,71 @@ public:
     STDMETHOD(COMSETTER(DefaultMachineFolder)) (IN_BSTR aDefaultMachineFolder);
     STDMETHOD(COMGETTER(DefaultHardDiskFolder)) (BSTR *aDefaultHardDiskFolder);
     STDMETHOD(COMSETTER(DefaultHardDiskFolder)) (IN_BSTR aDefaultHardDiskFolder);
-    STDMETHOD(COMGETTER(HardDiskFormats)) (ComSafeArrayOut (IHardDiskFormat *, aHardDiskFormats));
-    STDMETHOD(COMGETTER(DefaultHardDiskFormat)) (BSTR *aDefaultHardDiskFolder);
-    STDMETHOD(COMSETTER(DefaultHardDiskFormat)) (IN_BSTR aDefaultHardDiskFolder);
+    STDMETHOD(COMGETTER(MediumFormats)) (ComSafeArrayOut (IMediumFormat *, aMediumFormats));
+    STDMETHOD(COMGETTER(DefaultHardDiskFormat)) (BSTR *aDefaultHardDiskFormat);
+    STDMETHOD(COMSETTER(DefaultHardDiskFormat)) (IN_BSTR aDefaultHardDiskFormat);
+    STDMETHOD(COMGETTER(FreeDiskSpaceWarning))(ULONG64 *aFreeDiskSpace);
+    STDMETHOD(COMSETTER(FreeDiskSpaceWarning))(ULONG64 aFreeDiskSpace);
+    STDMETHOD(COMGETTER(FreeDiskSpacePercentWarning))(ULONG *aFreeDiskSpacePercent);
+    STDMETHOD(COMSETTER(FreeDiskSpacePercentWarning))(ULONG aFreeDiskSpacePercent);
+    STDMETHOD(COMGETTER(FreeDiskSpaceError))(ULONG64 *aFreeDiskSpace);
+    STDMETHOD(COMSETTER(FreeDiskSpaceError))(ULONG64 aFreeDiskSpace);
+    STDMETHOD(COMGETTER(FreeDiskSpacePercentError))(ULONG *aFreeDiskSpacePercent);
+    STDMETHOD(COMSETTER(FreeDiskSpacePercentError))(ULONG aFreeDiskSpacePercent);
     STDMETHOD(COMGETTER(RemoteDisplayAuthLibrary)) (BSTR *aRemoteDisplayAuthLibrary);
     STDMETHOD(COMSETTER(RemoteDisplayAuthLibrary)) (IN_BSTR aRemoteDisplayAuthLibrary);
     STDMETHOD(COMGETTER(WebServiceAuthLibrary)) (BSTR *aWebServiceAuthLibrary);
     STDMETHOD(COMSETTER(WebServiceAuthLibrary)) (IN_BSTR aWebServiceAuthLibrary);
-    STDMETHOD(COMGETTER(HWVirtExEnabled)) (BOOL *enabled);
-    STDMETHOD(COMSETTER(HWVirtExEnabled)) (BOOL enabled);
     STDMETHOD(COMGETTER(LogHistoryCount)) (ULONG *count);
     STDMETHOD(COMSETTER(LogHistoryCount)) (ULONG count);
+    STDMETHOD(COMGETTER(DefaultAudioDriver)) (AudioDriverType_T *aAudioDriver);
+
+    STDMETHOD(GetMaxDevicesPerPortForStorageBus) (StorageBus_T aBus, ULONG *aMaxDevicesPerPort);
+    STDMETHOD(GetMinPortCountForStorageBus) (StorageBus_T aBus, ULONG *aMinPortCount);
+    STDMETHOD(GetMaxPortCountForStorageBus) (StorageBus_T aBus, ULONG *aMaxPortCount);
+    STDMETHOD(GetMaxInstancesOfStorageBus)(StorageBus_T aBus, ULONG *aMaxInstances);
+    STDMETHOD(GetDeviceTypesForStorageBus)(StorageBus_T aBus, ComSafeArrayOut(DeviceType_T, aDeviceTypes));
 
     // public methods only for internal purposes
 
-    HRESULT loadSettings (const settings::Key &aGlobal);
-    HRESULT saveSettings (settings::Key &aGlobal);
+    HRESULT loadSettings(const settings::SystemProperties &data);
+    HRESULT saveSettings(settings::SystemProperties &data);
 
-    ComObjPtr <HardDiskFormat> hardDiskFormat (CBSTR aFormat);
+    ComObjPtr<MediumFormat> mediumFormat(CBSTR aFormat);
 
     // public methods for internal purposes only
     // (ensure there is a caller and a read lock before calling them!)
 
-    /** Default Machine path (as is, not full). Not thread safe (use object lock). */
-    const Bstr &defaultMachineFolder() const { return mDefaultMachineFolder; }
-    /** Default Machine path (full). Not thread safe (use object lock). */
-    const Bstr &defaultMachineFolderFull() const { return mDefaultMachineFolderFull; }
-    /** Default hard disk path (as is, not full). Not thread safe (use object lock). */
-    const Bstr &defaultHardDiskFolder() const { return mDefaultHardDiskFolder; }
-    /** Default hard disk path (full). Not thread safe (use object lock). */
-    const Bstr &defaultHardDiskFolderFull() const { return mDefaultHardDiskFolderFull; }
-
-    /** Default hard disk format. Not thread safe (use object lock). */
-    const Bstr &defaultHardDiskFormat() const { return mDefaultHardDiskFormat; }
-
     // for VirtualBoxSupportErrorInfoImpl
     static const wchar_t *getComponentName() { return L"SystemProperties"; }
 
 private:
 
-    typedef std::list <ComObjPtr <HardDiskFormat> > HardDiskFormatList;
+    typedef std::list<ComObjPtr<MediumFormat> > MediumFormatList;
 
-    HRESULT setDefaultMachineFolder (CBSTR aPath);
-    HRESULT setDefaultHardDiskFolder (CBSTR aPath);
-    HRESULT setDefaultHardDiskFormat (CBSTR aFormat);
+    HRESULT setDefaultMachineFolder(const Utf8Str &aPath);
+    HRESULT setDefaultHardDiskFolder(const Utf8Str &aPath);
+    HRESULT setDefaultHardDiskFormat(const Utf8Str &aFormat);
 
-    HRESULT setRemoteDisplayAuthLibrary (CBSTR aPath);
-    HRESULT setWebServiceAuthLibrary (CBSTR aPath);
+    HRESULT setRemoteDisplayAuthLibrary(const Utf8Str &aPath);
+    HRESULT setWebServiceAuthLibrary(const Utf8Str &aPath);
 
-    const ComObjPtr <VirtualBox, ComWeakRef> mParent;
+    VirtualBox * const  mParent;
 
-    Bstr mDefaultMachineFolder;
-    Bstr mDefaultMachineFolderFull;
-    Bstr mDefaultHardDiskFolder;
-    Bstr mDefaultHardDiskFolderFull;
-    Bstr mDefaultHardDiskFormat;
+    Utf8Str             m_strDefaultMachineFolder;
+    Utf8Str             m_strDefaultMachineFolderFull;
+    Utf8Str             m_strDefaultHardDiskFolder;
+    Utf8Str             m_strDefaultHardDiskFolderFull;
+    Utf8Str             m_strDefaultHardDiskFormat;
 
-    HardDiskFormatList mHardDiskFormats;
+    MediumFormatList    mMediumFormats;
 
-    Bstr mRemoteDisplayAuthLibrary;
-    Bstr mWebServiceAuthLibrary;
-    BOOL mHWVirtExEnabled;
-    ULONG mLogHistoryCount;
+    Utf8Str             m_strRemoteDisplayAuthLibrary;
+    Utf8Str             m_strWebServiceAuthLibrary;
+    ULONG               mLogHistoryCount;
+    AudioDriverType_T   mDefaultAudioDriver;
+
+    friend class VirtualBox;
 };
 
 #endif // ____H_SYSTEMPROPERTIESIMPL
