@@ -1,11 +1,11 @@
-/* $Id: USBDeviceImpl.h $ */
+/* $Id: USBDeviceImpl.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 
 /** @file
  * Header file for the OUSBDevice (IUSBDevice) class, VBoxC.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,28 +14,22 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ____H_USBDEVICEIMPL
 #define ____H_USBDEVICEIMPL
 
 #include "VirtualBoxBase.h"
-#include "Logging.h"
-
 
 /**
  * Object class used for maintaining devices attached to a USB controller.
  * Generally this contains much less information.
  */
 class ATL_NO_VTABLE OUSBDevice :
-    public VirtualBoxBaseNEXT,
+    public VirtualBoxBase,
     public VirtualBoxSupportErrorInfoImpl<OUSBDevice, IUSBDevice>,
     public VirtualBoxSupportTranslation<OUSBDevice>,
-    public IUSBDevice
+    VBOX_SCRIPTABLE_IMPL(IUSBDevice)
 {
 public:
 
@@ -46,12 +40,11 @@ public:
     DECLARE_PROTECT_FINAL_CONSTRUCT()
 
     BEGIN_COM_MAP(OUSBDevice)
-        COM_INTERFACE_ENTRY (ISupportErrorInfo)
-        COM_INTERFACE_ENTRY (IUSBDevice)
+        COM_INTERFACE_ENTRY  (ISupportErrorInfo)
+        COM_INTERFACE_ENTRY  (IUSBDevice)
+        COM_INTERFACE_ENTRY2 (IDispatch, IUSBDevice)
     END_COM_MAP()
 
-    NS_DECL_ISUPPORTS
-
     DECLARE_EMPTY_CTOR_DTOR (OUSBDevice)
 
     HRESULT FinalConstruct();
@@ -62,7 +55,7 @@ public:
     void uninit();
 
     // IUSBDevice properties
-    STDMETHOD(COMGETTER(Id))(OUT_GUID aId);
+    STDMETHOD(COMGETTER(Id))(BSTR *aId);
     STDMETHOD(COMGETTER(VendorId))(USHORT *aVendorId);
     STDMETHOD(COMGETTER(ProductId))(USHORT *aProductId);
     STDMETHOD(COMGETTER(Revision))(USHORT *aRevision);
