@@ -1,10 +1,10 @@
-/* $Id: NetIf-win.cpp $ */
+/* $Id: NetIf-win.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * Main - NetIfList, Windows implementation.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -201,8 +197,8 @@ struct NetworkInterfaceHelperClientData
     SVCHlpMsg::Code msgCode;
     /* for SVCHlpMsg::CreateHostOnlyNetworkInterface */
     Bstr name;
-    ComObjPtr <HostNetworkInterface> iface;
-    ComObjPtr <VirtualBox> vBox;
+    ComObjPtr<HostNetworkInterface> iface;
+    ComObjPtr<VirtualBox> vBox;
     /* for SVCHlpMsg::RemoveHostOnlyNetworkInterface */
     Guid guid;
 
@@ -223,10 +219,10 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
     LogFlowFunc (("aClient={%p}, aProgress={%p}, aUser={%p}\n",
                   aClient, aProgress, aUser));
 
-    AssertReturn ((aClient == NULL && aProgress == NULL && aVrc == NULL) ||
+    AssertReturn((aClient == NULL && aProgress == NULL && aVrc == NULL) ||
                   (aClient != NULL && aProgress != NULL && aVrc != NULL),
                   E_POINTER);
-    AssertReturn (aUser, E_POINTER);
+    AssertReturn(aUser, E_POINTER);
 
     std::auto_ptr <NetworkInterfaceHelperClientData>
         d (static_cast <NetworkInterfaceHelperClientData *> (aUser));
@@ -249,9 +245,9 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
 
             /* write message and parameters */
             vrc = aClient->write (d->msgCode);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 //            vrc = aClient->write (Utf8Str (d->name));
-//            if (RT_FAILURE (vrc)) break;
+//            if (RT_FAILURE(vrc)) break;
 
             /* wait for a reply */
             bool endLoop = false;
@@ -260,7 +256,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                 SVCHlpMsg::Code reply = SVCHlpMsg::Null;
 
                 vrc = aClient->read (reply);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
 
                 switch (reply)
                 {
@@ -270,9 +266,9 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                         Guid guid;
                         Utf8Str name;
                         vrc = aClient->read (name);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
                         vrc = aClient->read (guid);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
 
                         LogFlowFunc (("Network connection GUID = {%RTuuid}\n", guid.raw()));
 
@@ -295,7 +291,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                         /* read the error message */
                         Utf8Str errMsg;
                         vrc = aClient->read (errMsg);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
 
                         rc = E_FAIL;//TODO: setError (E_FAIL, errMsg);
                         endLoop = true;
@@ -321,9 +317,9 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
 
             /* write message and parameters */
             vrc = aClient->write (d->msgCode);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             /* wait for a reply */
             bool endLoop = false;
@@ -332,7 +328,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                 SVCHlpMsg::Code reply = SVCHlpMsg::Null;
 
                 vrc = aClient->read (reply);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
 
                 switch (reply)
                 {
@@ -348,7 +344,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                         /* read the error message */
                         Utf8Str errMsg;
                         vrc = aClient->read (errMsg);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
 
                         rc = E_FAIL; // TODO: setError (E_FAIL, errMsg);
                         endLoop = true;
@@ -374,9 +370,9 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
 
             /* write message and parameters */
             vrc = aClient->write (d->msgCode);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             /* wait for a reply */
             bool endLoop = false;
@@ -385,7 +381,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                 SVCHlpMsg::Code reply = SVCHlpMsg::Null;
 
                 vrc = aClient->read (reply);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
 
                 switch (reply)
                 {
@@ -401,7 +397,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                         /* read the error message */
                         Utf8Str errMsg;
                         vrc = aClient->read (errMsg);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
 
                         rc = E_FAIL; // TODO: setError (E_FAIL, errMsg);
                         endLoop = true;
@@ -427,13 +423,13 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
 
             /* write message and parameters */
             vrc = aClient->write (d->msgCode);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->u.StaticIP.IPAddress);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->u.StaticIP.IPNetMask);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             /* wait for a reply */
             bool endLoop = false;
@@ -442,7 +438,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                 SVCHlpMsg::Code reply = SVCHlpMsg::Null;
 
                 vrc = aClient->read (reply);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
 
                 switch (reply)
                 {
@@ -458,7 +454,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                         /* read the error message */
                         Utf8Str errMsg;
                         vrc = aClient->read (errMsg);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
 
                         rc = E_FAIL; // TODO: setError (E_FAIL, errMsg);
                         endLoop = true;
@@ -484,13 +480,13 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
 
             /* write message and parameters */
             vrc = aClient->write (d->msgCode);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (Utf8Str(d->u.StaticIPV6.IPV6Address));
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->u.StaticIPV6.IPV6NetMaskLength);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             /* wait for a reply */
             bool endLoop = false;
@@ -499,7 +495,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                 SVCHlpMsg::Code reply = SVCHlpMsg::Null;
 
                 vrc = aClient->read (reply);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
 
                 switch (reply)
                 {
@@ -515,7 +511,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                         /* read the error message */
                         Utf8Str errMsg;
                         vrc = aClient->read (errMsg);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
 
                         rc = E_FAIL; // TODO: setError (E_FAIL, errMsg);
                         endLoop = true;
@@ -541,9 +537,9 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
 
             /* write message and parameters */
             vrc = aClient->write (d->msgCode);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->write (d->guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             /* wait for a reply */
             bool endLoop = false;
@@ -552,7 +548,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                 SVCHlpMsg::Code reply = SVCHlpMsg::Null;
 
                 vrc = aClient->read (reply);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
 
                 switch (reply)
                 {
@@ -568,7 +564,7 @@ static HRESULT netIfNetworkInterfaceHelperClient (SVCHlpClient *aClient,
                         /* read the error message */
                         Utf8Str errMsg;
                         vrc = aClient->read (errMsg);
-                        if (RT_FAILURE (vrc)) break;
+                        if (RT_FAILURE(vrc)) break;
 
                         rc = E_FAIL; // TODO: setError (E_FAIL, errMsg);
                         endLoop = true;
@@ -609,7 +605,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
     LogFlowFuncEnter();
     LogFlowFunc (("aClient={%p}, aMsgCode=%d\n", aClient, aMsgCode));
 
-    AssertReturn (aClient, VERR_INVALID_POINTER);
+    AssertReturn(aClient, VERR_INVALID_POINTER);
 
     int vrc = VINF_SUCCESS;
     HRESULT hrc;
@@ -622,14 +618,14 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
 
 //            Utf8Str name;
 //            vrc = aClient->read (name);
-//            if (RT_FAILURE (vrc)) break;
+//            if (RT_FAILURE(vrc)) break;
 
             Guid guid;
             Utf8Str errMsg;
             Bstr name;
             Bstr bstrErr;
 
-            hrc = VBoxNetCfgWinCreateHostOnlyNetworkInterface (guid.asOutParam(), name.asOutParam(), bstrErr.asOutParam());
+            hrc = VBoxNetCfgWinCreateHostOnlyNetworkInterface (NULL, false, guid.asOutParam(), name.asOutParam(), bstrErr.asOutParam());
 
             if (hrc == S_OK)
             {
@@ -645,11 +641,11 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
 
                 /* write success followed by GUID */
                 vrc = aClient->write (SVCHlpMsg::CreateHostOnlyNetworkInterface_OK);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (Utf8Str (name));
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (guid);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
             else
             {
@@ -659,9 +655,9 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
                 if (errMsg.isEmpty())
                     errMsg = Utf8StrFmt ("Unspecified error (%Rrc)", vrc);
                 vrc = aClient->write (SVCHlpMsg::Error);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (errMsg);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
 
             break;
@@ -674,7 +670,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
             Bstr bstrErr;
 
             vrc = aClient->read (guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             Utf8Str errMsg;
             hrc = VBoxNetCfgWinRemoveHostOnlyNetworkInterface ((const GUID*)guid.raw(), bstrErr.asOutParam());
@@ -683,7 +679,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
             {
                 /* write parameter-less success */
                 vrc = aClient->write (SVCHlpMsg::OK);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
             else
             {
@@ -693,9 +689,9 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
                 if (errMsg.isEmpty())
                     errMsg = Utf8StrFmt ("Unspecified error (%Rrc)", vrc);
                 vrc = aClient->write (SVCHlpMsg::Error);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (errMsg);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
 
             break;
@@ -708,20 +704,20 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
             Utf8Str ipV6;
             ULONG maskLengthV6;
             vrc = aClient->read (guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->read (ipV6);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->read (maskLengthV6);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             Utf8Str errMsg;
             vrc = VERR_NOT_IMPLEMENTED;
 
-            if (RT_SUCCESS (vrc))
+            if (RT_SUCCESS(vrc))
             {
                 /* write success followed by GUID */
                 vrc = aClient->write (SVCHlpMsg::OK);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
             else
             {
@@ -729,9 +725,9 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
                 if (errMsg.isEmpty())
                     errMsg = Utf8StrFmt ("Unspecified error (%Rrc)", vrc);
                 vrc = aClient->write (SVCHlpMsg::Error);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (errMsg);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
 
             break;
@@ -743,11 +739,11 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
             Guid guid;
             ULONG ip, mask;
             vrc = aClient->read (guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->read (ip);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
             vrc = aClient->read (mask);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             Utf8Str errMsg;
             hrc = VBoxNetCfgWinEnableStaticIpConfig ((const GUID *)guid.raw(), ip, mask);
@@ -756,7 +752,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
             {
                 /* write success followed by GUID */
                 vrc = aClient->write (SVCHlpMsg::OK);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
             else
             {
@@ -765,9 +761,9 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
                 if (errMsg.isEmpty())
                     errMsg = Utf8StrFmt ("Unspecified error (%Rrc)", vrc);
                 vrc = aClient->write (SVCHlpMsg::Error);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (errMsg);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
 
             break;
@@ -778,7 +774,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
 
             Guid guid;
             vrc = aClient->read (guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             Utf8Str errMsg;
             hrc = VBoxNetCfgWinEnableDynamicIpConfig ((const GUID *)guid.raw());
@@ -787,7 +783,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
             {
                 /* write success followed by GUID */
                 vrc = aClient->write (SVCHlpMsg::OK);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
             else
             {
@@ -796,9 +792,9 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
                 if (errMsg.isEmpty())
                     errMsg = Utf8StrFmt ("Unspecified error (%Rrc)", vrc);
                 vrc = aClient->write (SVCHlpMsg::Error);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (errMsg);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
 
             break;
@@ -809,7 +805,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
 
             Guid guid;
             vrc = aClient->read (guid);
-            if (RT_FAILURE (vrc)) break;
+            if (RT_FAILURE(vrc)) break;
 
             Utf8Str errMsg;
             hrc = VBoxNetCfgWinDhcpRediscover ((const GUID *)guid.raw());
@@ -818,7 +814,7 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
             {
                 /* write success followed by GUID */
                 vrc = aClient->write (SVCHlpMsg::OK);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
             else
             {
@@ -827,9 +823,9 @@ int netIfNetworkInterfaceHelperServer (SVCHlpClient *aClient,
                 if (errMsg.isEmpty())
                     errMsg = Utf8StrFmt ("Unspecified error (%Rrc)", vrc);
                 vrc = aClient->write (SVCHlpMsg::Error);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
                 vrc = aClient->write (errMsg);
-                if (RT_FAILURE (vrc)) break;
+                if (RT_FAILURE(vrc)) break;
             }
 
             break;
@@ -858,7 +854,7 @@ static BOOL IsUACEnabled()
     ZeroMemory (&info, sizeof (OSVERSIONINFOEX));
     info.dwOSVersionInfoSize = sizeof (OSVERSIONINFOEX);
     rc = GetVersionEx ((OSVERSIONINFO *) &info);
-    AssertReturn (rc != 0, FALSE);
+    AssertReturn(rc != 0, FALSE);
 
     LogFlowFunc (("dwMajorVersion=%d, dwMinorVersion=%d\n",
                   info.dwMajorVersion, info.dwMinorVersion));
@@ -876,7 +872,7 @@ static BOOL IsUACEnabled()
                         "Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\System",
                         0, KEY_QUERY_VALUE, &hKey);
 
-    Assert (rc == ERROR_SUCCESS || rc == ERROR_PATH_NOT_FOUND);
+    Assert(rc == ERROR_SUCCESS || rc == ERROR_PATH_NOT_FOUND);
     if (rc == ERROR_SUCCESS)
     {
 
@@ -886,7 +882,7 @@ static BOOL IsUACEnabled()
 
         RegCloseKey (hKey);
 
-        Assert (rc == ERROR_SUCCESS || rc == ERROR_FILE_NOT_FOUND);
+        Assert(rc == ERROR_SUCCESS || rc == ERROR_FILE_NOT_FOUND);
     }
 
     LogFlowFunc (("rc=%d, dwEnableLUA=%d\n", rc, dwEnableLUA));
@@ -896,7 +892,7 @@ static BOOL IsUACEnabled()
 
 /* end */
 
-static int vboxNetWinAddComponent(std::list <ComObjPtr <HostNetworkInterface> > * pPist, INetCfgComponent * pncc, HostNetworkInterfaceType enmType)
+static int vboxNetWinAddComponent(std::list <ComObjPtr<HostNetworkInterface> > * pPist, INetCfgComponent * pncc, HostNetworkInterfaceType enmType)
 {
     LPWSTR              lpszName;
     GUID                IfGuid;
@@ -907,10 +903,7 @@ static int vboxNetWinAddComponent(std::list <ComObjPtr <HostNetworkInterface> >
     Assert(hr == S_OK);
     if(hr == S_OK)
     {
-        size_t cUnicodeName = wcslen(lpszName) + 1;
-        size_t uniLen = (cUnicodeName * 2 + sizeof (OLECHAR) - 1) / sizeof (OLECHAR);
-        Bstr name (uniLen + 1 /* extra zero */);
-        wcscpy((wchar_t *) name.mutableRaw(), lpszName);
+        Bstr name(lpszName);
 
         hr = pncc->GetInstanceGuid(&IfGuid);
         Assert(hr == S_OK);
@@ -922,13 +915,13 @@ static int vboxNetWinAddComponent(std::list <ComObjPtr <HostNetworkInterface> >
             rc = collectNetIfInfo(name, Guid(IfGuid), &Info);
             if (RT_FAILURE(rc))
             {
-                Log(("vboxNetWinAddComponent: collectNetIfInfo() -> %Vrc\n", rc));
+                Log(("vboxNetWinAddComponent: collectNetIfInfo() -> %Rrc\n", rc));
             }
             /* create a new object and add it to the list */
-            ComObjPtr <HostNetworkInterface> iface;
+            ComObjPtr<HostNetworkInterface> iface;
             iface.createObject();
             /* remove the curly bracket at the end */
-            if (SUCCEEDED (iface->init (name, enmType, &Info)))
+            if (SUCCEEDED(iface->init (name, enmType, &Info)))
             {
                 pPist->push_back (iface);
                 rc = VINF_SUCCESS;
@@ -947,7 +940,7 @@ static int vboxNetWinAddComponent(std::list <ComObjPtr <HostNetworkInterface> >
 #endif /* VBOX_WITH_NETFLT */
 
 
-static int netIfListHostAdapters(std::list <ComObjPtr <HostNetworkInterface> > &list)
+static int netIfListHostAdapters(std::list <ComObjPtr<HostNetworkInterface> > &list)
 {
 #ifndef VBOX_WITH_NETFLT
     /* VBoxNetAdp is available only when VBOX_WITH_NETFLT is enabled */
@@ -974,9 +967,6 @@ static int netIfListHostAdapters(std::list <ComObjPtr <HostNetworkInterface> > &
             {
                 ULONG uComponentStatus;
                 hr = pMpNcc->GetDeviceStatus(&uComponentStatus);
-//#ifndef DEBUG_bird
-//                Assert(hr == S_OK);
-//#endif
                 if(hr == S_OK)
                 {
                     if(uComponentStatus == 0)
@@ -1024,8 +1014,8 @@ int NetIfGetConfig(HostNetworkInterface * pIf, NETIFINFO *pInfo)
     HRESULT hr = pIf->COMGETTER(Name)(name.asOutParam());
     if(hr == S_OK)
     {
-        GUID                IfGuid;
-        hr = pIf->COMGETTER(Id)(&IfGuid);
+        Bstr                IfGuid;
+        hr = pIf->COMGETTER(Id)(IfGuid.asOutParam());
         Assert(hr == S_OK);
         if (hr == S_OK)
         {
@@ -1040,6 +1030,11 @@ int NetIfGetConfig(HostNetworkInterface * pIf, NETIFINFO *pInfo)
 #endif
 }
 
+int NetIfGetConfigByName(PNETIFINFO)
+{
+    return VERR_NOT_IMPLEMENTED;
+}
+
 int NetIfCreateHostOnlyNetworkInterface (VirtualBox *pVBox,
                                   IHostNetworkInterface **aHostNetworkInterface,
                                   IProgress **aProgress)
@@ -1048,7 +1043,7 @@ int NetIfCreateHostOnlyNetworkInterface (VirtualBox *pVBox,
     return VERR_NOT_IMPLEMENTED;
 #else
     /* create a progress object */
-    ComObjPtr <Progress> progress;
+    ComObjPtr<Progress> progress;
     progress.createObject();
 
     ComPtr<IHost> host;
@@ -1060,18 +1055,18 @@ int NetIfCreateHostOnlyNetworkInterface (VirtualBox *pVBox,
                              FALSE /* aCancelable */);
         if(SUCCEEDED(rc))
         {
-            CheckComRCReturnRC (rc);
-            progress.queryInterfaceTo (aProgress);
+            if (FAILED(rc)) return rc;
+            progress.queryInterfaceTo(aProgress);
 
             /* create a new uninitialized host interface object */
-            ComObjPtr <HostNetworkInterface> iface;
+            ComObjPtr<HostNetworkInterface> iface;
             iface.createObject();
-            iface.queryInterfaceTo (aHostNetworkInterface);
+            iface.queryInterfaceTo(aHostNetworkInterface);
 
             /* create the networkInterfaceHelperClient() argument */
             std::auto_ptr <NetworkInterfaceHelperClientData>
                 d (new NetworkInterfaceHelperClientData());
-            AssertReturn (d.get(), E_OUTOFMEMORY);
+            AssertReturn(d.get(), E_OUTOFMEMORY);
 
             d->msgCode = SVCHlpMsg::CreateHostOnlyNetworkInterface;
 //            d->name = aName;
@@ -1084,7 +1079,7 @@ int NetIfCreateHostOnlyNetworkInterface (VirtualBox *pVBox,
                 static_cast <void *> (d.get()),
                 progress);
 
-            if (SUCCEEDED (rc))
+            if (SUCCEEDED(rc))
             {
                 /* d is now owned by netIfNetworkInterfaceHelperClient(), so release it */
                 d.release();
@@ -1097,14 +1092,13 @@ int NetIfCreateHostOnlyNetworkInterface (VirtualBox *pVBox,
 }
 
 int NetIfRemoveHostOnlyNetworkInterface (VirtualBox *pVBox, IN_GUID aId,
-                                  IHostNetworkInterface **aHostNetworkInterface,
-                                  IProgress **aProgress)
+                                         IProgress **aProgress)
 {
 #ifndef VBOX_WITH_NETFLT
     return VERR_NOT_IMPLEMENTED;
 #else
     /* create a progress object */
-    ComObjPtr <Progress> progress;
+    ComObjPtr<Progress> progress;
     progress.createObject();
     ComPtr<IHost> host;
     HRESULT rc = pVBox->COMGETTER(Host)(host.asOutParam());
@@ -1115,13 +1109,13 @@ int NetIfRemoveHostOnlyNetworkInterface (VirtualBox *pVBox, IN_GUID aId,
                             FALSE /* aCancelable */);
         if(SUCCEEDED(rc))
         {
-            CheckComRCReturnRC (rc);
-            progress.queryInterfaceTo (aProgress);
+            if (FAILED(rc)) return rc;
+            progress.queryInterfaceTo(aProgress);
 
             /* create the networkInterfaceHelperClient() argument */
             std::auto_ptr <NetworkInterfaceHelperClientData>
                 d (new NetworkInterfaceHelperClientData());
-            AssertReturn (d.get(), E_OUTOFMEMORY);
+            AssertReturn(d.get(), E_OUTOFMEMORY);
 
             d->msgCode = SVCHlpMsg::RemoveHostOnlyNetworkInterface;
             d->guid = aId;
@@ -1132,7 +1126,7 @@ int NetIfRemoveHostOnlyNetworkInterface (VirtualBox *pVBox, IN_GUID aId,
                 static_cast <void *> (d.get()),
                 progress);
 
-            if (SUCCEEDED (rc))
+            if (SUCCEEDED(rc))
             {
                 /* d is now owned by netIfNetworkInterfaceHelperClient(), so release it */
                 d.release();
@@ -1150,8 +1144,8 @@ int NetIfEnableStaticIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf, ULON
     return VERR_NOT_IMPLEMENTED;
 #else
     HRESULT rc;
-    GUID guid;
-    rc = pIf->COMGETTER(Id) (&guid);
+    Bstr guid;
+    rc = pIf->COMGETTER(Id) (guid.asOutParam());
     if(SUCCEEDED(rc))
     {
 //        ComPtr<VirtualBox> vBox;
@@ -1159,7 +1153,7 @@ int NetIfEnableStaticIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf, ULON
 //        if(SUCCEEDED(rc))
         {
             /* create a progress object */
-            ComObjPtr <Progress> progress;
+            ComObjPtr<Progress> progress;
             progress.createObject();
 //            ComPtr<IHost> host;
 //            HRESULT rc = vBox->COMGETTER(Host)(host.asOutParam());
@@ -1170,16 +1164,16 @@ int NetIfEnableStaticIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf, ULON
                                     FALSE /* aCancelable */);
                 if(SUCCEEDED(rc))
                 {
-                    CheckComRCReturnRC (rc);
-//                    progress.queryInterfaceTo (aProgress);
+                    if (FAILED(rc)) return rc;
+//                    progress.queryInterfaceTo(aProgress);
 
                     /* create the networkInterfaceHelperClient() argument */
                     std::auto_ptr <NetworkInterfaceHelperClientData>
                         d (new NetworkInterfaceHelperClientData());
-                    AssertReturn (d.get(), E_OUTOFMEMORY);
+                    AssertReturn(d.get(), E_OUTOFMEMORY);
 
                     d->msgCode = SVCHlpMsg::EnableStaticIpConfig;
-                    d->guid = guid;
+                    d->guid = Guid(guid);
                     d->iface = pIf;
                     d->u.StaticIP.IPAddress = ip;
                     d->u.StaticIP.IPNetMask = mask;
@@ -1190,7 +1184,7 @@ int NetIfEnableStaticIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf, ULON
                         static_cast <void *> (d.get()),
                         progress);
 
-                    if (SUCCEEDED (rc))
+                    if (SUCCEEDED(rc))
                     {
                         /* d is now owned by netIfNetworkInterfaceHelperClient(), so release it */
                         d.release();
@@ -1212,8 +1206,8 @@ int NetIfEnableStaticIpConfigV6(VirtualBox *vBox, HostNetworkInterface * pIf, IN
     return VERR_NOT_IMPLEMENTED;
 #else
     HRESULT rc;
-    GUID guid;
-    rc = pIf->COMGETTER(Id) (&guid);
+    Bstr guid;
+    rc = pIf->COMGETTER(Id) (guid.asOutParam());
     if(SUCCEEDED(rc))
     {
 //        ComPtr<VirtualBox> vBox;
@@ -1221,7 +1215,7 @@ int NetIfEnableStaticIpConfigV6(VirtualBox *vBox, HostNetworkInterface * pIf, IN
 //        if(SUCCEEDED(rc))
         {
             /* create a progress object */
-            ComObjPtr <Progress> progress;
+            ComObjPtr<Progress> progress;
             progress.createObject();
 //            ComPtr<IHost> host;
 //            HRESULT rc = vBox->COMGETTER(Host)(host.asOutParam());
@@ -1232,13 +1226,13 @@ int NetIfEnableStaticIpConfigV6(VirtualBox *vBox, HostNetworkInterface * pIf, IN
                                     FALSE /* aCancelable */);
                 if(SUCCEEDED(rc))
                 {
-                    CheckComRCReturnRC (rc);
-//                    progress.queryInterfaceTo (aProgress);
+                    if (FAILED(rc)) return rc;
+//                    progress.queryInterfaceTo(aProgress);
 
                     /* create the networkInterfaceHelperClient() argument */
                     std::auto_ptr <NetworkInterfaceHelperClientData>
                         d (new NetworkInterfaceHelperClientData());
-                    AssertReturn (d.get(), E_OUTOFMEMORY);
+                    AssertReturn(d.get(), E_OUTOFMEMORY);
 
                     d->msgCode = SVCHlpMsg::EnableStaticIpConfigV6;
                     d->guid = guid;
@@ -1252,7 +1246,7 @@ int NetIfEnableStaticIpConfigV6(VirtualBox *vBox, HostNetworkInterface * pIf, IN
                         static_cast <void *> (d.get()),
                         progress);
 
-                    if (SUCCEEDED (rc))
+                    if (SUCCEEDED(rc))
                     {
                         /* d is now owned by netIfNetworkInterfaceHelperClient(), so release it */
                         d.release();
@@ -1274,8 +1268,8 @@ int NetIfEnableDynamicIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf)
     return VERR_NOT_IMPLEMENTED;
 #else
     HRESULT rc;
-    GUID guid;
-    rc = pIf->COMGETTER(Id) (&guid);
+    Bstr guid;
+    rc = pIf->COMGETTER(Id) (guid.asOutParam());
     if(SUCCEEDED(rc))
     {
 //        ComPtr<VirtualBox> vBox;
@@ -1283,7 +1277,7 @@ int NetIfEnableDynamicIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf)
 //        if(SUCCEEDED(rc))
         {
             /* create a progress object */
-            ComObjPtr <Progress> progress;
+            ComObjPtr<Progress> progress;
             progress.createObject();
 //            ComPtr<IHost> host;
 //            HRESULT rc = vBox->COMGETTER(Host)(host.asOutParam());
@@ -1294,13 +1288,13 @@ int NetIfEnableDynamicIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf)
                                     FALSE /* aCancelable */);
                 if(SUCCEEDED(rc))
                 {
-                    CheckComRCReturnRC (rc);
-//                    progress.queryInterfaceTo (aProgress);
+                    if (FAILED(rc)) return rc;
+//                    progress.queryInterfaceTo(aProgress);
 
                     /* create the networkInterfaceHelperClient() argument */
                     std::auto_ptr <NetworkInterfaceHelperClientData>
                         d (new NetworkInterfaceHelperClientData());
-                    AssertReturn (d.get(), E_OUTOFMEMORY);
+                    AssertReturn(d.get(), E_OUTOFMEMORY);
 
                     d->msgCode = SVCHlpMsg::EnableDynamicIpConfig;
                     d->guid = guid;
@@ -1312,7 +1306,7 @@ int NetIfEnableDynamicIpConfig(VirtualBox *vBox, HostNetworkInterface * pIf)
                         static_cast <void *> (d.get()),
                         progress);
 
-                    if (SUCCEEDED (rc))
+                    if (SUCCEEDED(rc))
                     {
                         /* d is now owned by netIfNetworkInterfaceHelperClient(), so release it */
                         d.release();
@@ -1334,8 +1328,8 @@ int NetIfDhcpRediscover(VirtualBox *vBox, HostNetworkInterface * pIf)
     return VERR_NOT_IMPLEMENTED;
 #else
     HRESULT rc;
-    GUID guid;
-    rc = pIf->COMGETTER(Id) (&guid);
+    Bstr guid;
+    rc = pIf->COMGETTER(Id) (guid.asOutParam());
     if(SUCCEEDED(rc))
     {
 //        ComPtr<VirtualBox> vBox;
@@ -1343,7 +1337,7 @@ int NetIfDhcpRediscover(VirtualBox *vBox, HostNetworkInterface * pIf)
 //        if(SUCCEEDED(rc))
         {
             /* create a progress object */
-            ComObjPtr <Progress> progress;
+            ComObjPtr<Progress> progress;
             progress.createObject();
 //            ComPtr<IHost> host;
 //            HRESULT rc = vBox->COMGETTER(Host)(host.asOutParam());
@@ -1354,13 +1348,13 @@ int NetIfDhcpRediscover(VirtualBox *vBox, HostNetworkInterface * pIf)
                                     FALSE /* aCancelable */);
                 if(SUCCEEDED(rc))
                 {
-                    CheckComRCReturnRC (rc);
-//                    progress.queryInterfaceTo (aProgress);
+                    if (FAILED(rc)) return rc;
+//                    progress.queryInterfaceTo(aProgress);
 
                     /* create the networkInterfaceHelperClient() argument */
                     std::auto_ptr <NetworkInterfaceHelperClientData>
                         d (new NetworkInterfaceHelperClientData());
-                    AssertReturn (d.get(), E_OUTOFMEMORY);
+                    AssertReturn(d.get(), E_OUTOFMEMORY);
 
                     d->msgCode = SVCHlpMsg::DhcpRediscover;
                     d->guid = guid;
@@ -1372,7 +1366,7 @@ int NetIfDhcpRediscover(VirtualBox *vBox, HostNetworkInterface * pIf)
                         static_cast <void *> (d.get()),
                         progress);
 
-                    if (SUCCEEDED (rc))
+                    if (SUCCEEDED(rc))
                     {
                         /* d is now owned by netIfNetworkInterfaceHelperClient(), so release it */
                         d.release();
@@ -1388,7 +1382,7 @@ int NetIfDhcpRediscover(VirtualBox *vBox, HostNetworkInterface * pIf)
 #endif
 }
 
-int NetIfList(std::list <ComObjPtr <HostNetworkInterface> > &list)
+int NetIfList(std::list <ComObjPtr<HostNetworkInterface> > &list)
 {
 #ifndef VBOX_WITH_NETFLT
     return VERR_NOT_IMPLEMENTED;
@@ -1453,9 +1447,6 @@ int NetIfList(std::list <ComObjPtr <HostNetworkInterface> > &list)
                                 {
                                     ULONG uComponentStatus;
                                     hr = pMpNcc->GetDeviceStatus(&uComponentStatus);
-//#ifndef DEBUG_bird
-//                                    Assert(hr == S_OK);
-//#endif
                                     if(hr == S_OK)
                                     {
                                         if(uComponentStatus == 0)
