@@ -1,10 +1,10 @@
-/* $Id: VBoxNetDHCP.cpp $ */
+/* $Id: VBoxNetDHCP.cpp 28845 2010-04-27 16:41:56Z vboxsync $ */
 /** @file
  * VBoxNetDHCP - DHCP Service for connecting to IntNet.
  */
 
 /*
- * Copyright (C) 2009 Sun Microsystems, Inc.
+ * Copyright (C) 2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /** @page pg_net_dhcp       VBoxNetDHCP
@@ -28,19 +24,21 @@
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include <iprt/net.h>
-#include <iprt/initterm.h>
 #include <iprt/alloca.h>
+#include <iprt/buildconfig.h>
 #include <iprt/err.h>
-#include <iprt/time.h>
-#include <iprt/stream.h>
-#include <iprt/path.h>
-#include <iprt/param.h>
+#include <iprt/net.h>                   /* must come before getopt */
 #include <iprt/getopt.h>
+#include <iprt/initterm.h>
+#include <iprt/param.h>
+#include <iprt/path.h>
+#include <iprt/stream.h>
+#include <iprt/time.h>
 #include <iprt/string.h>
 
 #include <VBox/sup.h>
 #include <VBox/intnet.h>
+#include <VBox/intnetinline.h>
 #include <VBox/vmm.h>
 #include <VBox/version.h>
 
@@ -576,13 +574,13 @@ VBoxNetDhcp::~VBoxNetDhcp()
         CloseReq.pSession = m_pSession;
         CloseReq.hIf = m_hIf;
         m_hIf = INTNET_HANDLE_INVALID;
-        int rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_CLOSE, 0, &CloseReq.Hdr);
+        int rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_CLOSE, 0, &CloseReq.Hdr);
         AssertRC(rc);
     }
 
     if (m_pSession)
     {
-        SUPTerm(false /* not forced */);
+        SUPR3Term(false /*fForced*/);
         m_pSession = NIL_RTR0PTR;
     }
 }
@@ -651,18 +649,18 @@ void VBoxNetDhcp::explodeConfig(void)
         VBoxNetDhcpCfg *pCfg = m_Cfgs[iCfg];
 
         /* Expand the IP lease range. */
-        uint32_t const uEnd = RT_N2H_U32(pCfg->m_UpperAddr.u);
-        for (uint32_t i = RT_N2H_U32(pCfg->m_LowerAddr.u); i < uEnd; i++)
+        uint32_t const uLast = RT_N2H_U32(pCfg->m_UpperAddr.u);
+        for (uint32_t i = RT_N2H_U32(pCfg->m_LowerAddr.u); i <= uLast; i++)
         {
             RTNETADDRIPV4 IPv4Addr;
             IPv4Addr.u = RT_H2N_U32(i);
 
             /* Check if it exists and is configured. */
             VBoxNetDhcpLease *pLease = NULL;
-            for (size_t i = 0; i < m_Leases.size(); i++)
-                if (m_Leases[i].m_IPv4Address.u == IPv4Addr.u)
+            for (size_t j = 0; j < m_Leases.size(); j++)
+                if (m_Leases[j].m_IPv4Address.u == IPv4Addr.u)
                 {
-                    pLease = &m_Leases[i];
+                    pLease = &m_Leases[j];
                     break;
                 }
             if (pLease)
@@ -709,13 +707,10 @@ int VBoxNetDhcp::parseArgs(int argc, char **argv)
         { "--lower-ip",       'l',   RTGETOPT_REQ_IPV4ADDR },
         { "--upper-ip",       'u',   RTGETOPT_REQ_IPV4ADDR },
         { "--netmask",        'm',   RTGETOPT_REQ_IPV4ADDR },
-
-        { "--help",           'h',   RTGETOPT_REQ_NOTHING },
-        { "--version ",       'V',   RTGETOPT_REQ_NOTHING },
     };
 
     RTGETOPTSTATE State;
-    int rc = RTGetOptInit(&State, argc, argv, &s_aOptionDefs[0], RT_ELEMENTS(s_aOptionDefs), 0, 0);
+    int rc = RTGetOptInit(&State, argc, argv, &s_aOptionDefs[0], RT_ELEMENTS(s_aOptionDefs), 0, 0 /*fFlags*/);
     AssertRCReturn(rc, 49);
 
     VBoxNetDhcpCfg *pCurCfg = NULL;
@@ -816,29 +811,26 @@ int VBoxNetDhcp::parseArgs(int argc, char **argv)
                 break;
 
             case 'V':
-                RTPrintf("%sr%d\n", VBOX_VERSION_STRING, VBOX_SVN_REV);
-                return 0;
+                RTPrintf("%sr%u\n", RTBldCfgVersion(), RTBldCfgRevision());
+                return 1;
 
             case 'h':
                 RTPrintf("VBoxNetDHCP Version %s\n"
-                         "(C) 2009 Sun Microsystems, Inc.\n"
-                         "All rights reserved\n"
+                         "(C) 2009-" VBOX_C_YEAR " " VBOX_VENDOR "\n"
+                         "All rights reserved.\n"
                          "\n"
                          "Usage: VBoxNetDHCP <options>\n"
                          "\n"
                          "Options:\n",
-                         VBOX_VERSION_STRING);
+                         RTBldCfgVersion());
                 for (size_t i = 0; i < RT_ELEMENTS(s_aOptionDefs); i++)
                     RTPrintf("    -%c, %s\n", s_aOptionDefs[i].iShort, s_aOptionDefs[i].pszLong);
                 return 1;
 
-            case VERR_GETOPT_UNKNOWN_OPTION:
-            case VINF_GETOPT_NOT_OPTION:
-                RTPrintf("Unknown option '%s'. Use --help for more information.\n", Val.psz);
-                return 1;
-
             default:
-                break;
+                rc = RTGetOptPrintError(rc, &Val);
+                RTPrintf("Use --help for more information.\n");
+                return rc;
         }
     }
 
@@ -871,17 +863,17 @@ int VBoxNetDhcp::tryGoOnline(void)
     }
 
     char szPath[RTPATH_MAX];
-    rc = RTPathProgram(szPath, sizeof(szPath) - sizeof("/VMMR0.r0"));
+    rc = RTPathExecDir(szPath, sizeof(szPath) - sizeof("/VMMR0.r0"));
     if (RT_FAILURE(rc))
     {
         RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: RTPathProgram -> %Rrc", rc);
         return 1;
     }
 
-    rc = SUPLoadVMM(strcat(szPath, "/VMMR0.r0"));
+    rc = SUPR3LoadVMM(strcat(szPath, "/VMMR0.r0"));
     if (RT_FAILURE(rc))
     {
-        RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPLoadVMM(\"%s\") -> %Rrc", szPath, rc);
+        RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPR3LoadVMM(\"%s\") -> %Rrc", szPath, rc);
         return 1;
     }
 
@@ -906,7 +898,7 @@ int VBoxNetDhcp::tryGoOnline(void)
      * Issue the request.
      */
     debugPrint(2, false, "attempting to open/create network \"%s\"...", OpenReq.szNetwork);
-    rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_OPEN, 0, &OpenReq.Hdr);
+    rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_OPEN, 0, &OpenReq.Hdr);
     if (RT_SUCCESS(rc))
     {
         m_hIf = OpenReq.hIf;
@@ -915,16 +907,17 @@ int VBoxNetDhcp::tryGoOnline(void)
         /*
          * Get the ring-3 address of the shared interface buffer.
          */
-        INTNETIFGETRING3BUFFERREQ GetRing3BufferReq;
-        GetRing3BufferReq.Hdr.u32Magic = SUPVMMR0REQHDR_MAGIC;
-        GetRing3BufferReq.Hdr.cbReq = sizeof(GetRing3BufferReq);
-        GetRing3BufferReq.pSession = m_pSession;
-        GetRing3BufferReq.hIf = m_hIf;
-        GetRing3BufferReq.pRing3Buf = NULL;
-        rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_GET_RING3_BUFFER, 0, &GetRing3BufferReq.Hdr);
+        INTNETIFGETBUFFERPTRSREQ GetBufferPtrsReq;
+        GetBufferPtrsReq.Hdr.u32Magic = SUPVMMR0REQHDR_MAGIC;
+        GetBufferPtrsReq.Hdr.cbReq = sizeof(GetBufferPtrsReq);
+        GetBufferPtrsReq.pSession = m_pSession;
+        GetBufferPtrsReq.hIf = m_hIf;
+        GetBufferPtrsReq.pRing3Buf = NULL;
+        GetBufferPtrsReq.pRing0Buf = NIL_RTR0PTR;
+        rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_GET_BUFFER_PTRS, 0, &GetBufferPtrsReq.Hdr);
         if (RT_SUCCESS(rc))
         {
-            PINTNETBUF pBuf = GetRing3BufferReq.pRing3Buf;
+            PINTNETBUF pBuf = GetBufferPtrsReq.pRing3Buf;
             debugPrint(1, false, "pBuf=%p cbBuf=%d cbSend=%d cbRecv=%d",
                        pBuf, pBuf->cbBuf, pBuf->cbSend, pBuf->cbRecv);
             m_pIfBuf = pBuf;
@@ -938,18 +931,18 @@ int VBoxNetDhcp::tryGoOnline(void)
             ActiveReq.pSession = m_pSession;
             ActiveReq.hIf = m_hIf;
             ActiveReq.fActive = true;
-            rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_SET_ACTIVE, 0, &ActiveReq.Hdr);
+            rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_SET_ACTIVE, 0, &ActiveReq.Hdr);
             if (RT_SUCCESS(rc))
                 return 0;
 
             /* bail out */
-            RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE,) failed, rc=%Rrc\n", rc);
+            RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_IF_SET_PROMISCUOUS_MODE,) failed, rc=%Rrc\n", rc);
         }
         else
-            RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_IF_GET_RING3_BUFFER,) failed, rc=%Rrc\n", rc);
+            RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_IF_GET_BUFFER_PTRS,) failed, rc=%Rrc\n", rc);
     }
     else
-        RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPCallVMMR0Ex(,VMMR0_DO_INTNET_OPEN,) failed, rc=%Rrc\n", rc);
+        RTStrmPrintf(g_pStdErr, "VBoxNetDHCP: SUPR3CallVMMR0Ex(,VMMR0_DO_INTNET_OPEN,) failed, rc=%Rrc\n", rc);
 
     return RT_SUCCESS(rc) ? 0 : 1;
 }
@@ -978,7 +971,7 @@ int VBoxNetDhcp::run(void)
         WaitReq.pSession = m_pSession;
         WaitReq.hIf = m_hIf;
         WaitReq.cMillies = 2000; /* 2 secs - the sleep is for some reason uninterruptible... */  /** @todo fix interruptability in SrvIntNet! */
-        int rc = SUPCallVMMR0Ex(NIL_RTR0PTR, VMMR0_DO_INTNET_IF_WAIT, 0, &WaitReq.Hdr);
+        int rc = SUPR3CallVMMR0Ex(NIL_RTR0PTR, NIL_VMCPUID, VMMR0_DO_INTNET_IF_WAIT, 0, &WaitReq.Hdr);
         if (RT_FAILURE(rc))
         {
             if (rc == VERR_TIMEOUT)
@@ -990,7 +983,7 @@ int VBoxNetDhcp::run(void)
         /*
          * Process the receive buffer.
          */
-        while (INTNETRingGetReadable(pRingBuf) > 0)
+        while (IntNetRingHasMoreToRead(pRingBuf))
         {
             size_t  cb;
             void   *pv = VBoxNetUDPMatch(m_pIfBuf, RTNETIPV4_PORT_BOOTPS, &m_MacAddress,
@@ -1022,7 +1015,7 @@ int VBoxNetDhcp::run(void)
             }
 
             /* Advance to the next frame. */
-            INTNETRingSkipFrame(m_pIfBuf, pRingBuf);
+            IntNetRingSkipFrame(pRingBuf);
         }
     }
 
@@ -1166,14 +1159,13 @@ bool VBoxNetDhcp::handleDhcpReqRequest(PCRTNETBOOTP pDhcpMsg, size_t cb)
     {
         if (pLease->isBeingOffered())
         {
-            fAckIt = true;
             if (pLease->m_xid == pDhcpMsg->bp_xid)
                 debugPrint(2, true, "REQUEST for offered lease.");
             else
                 debugPrint(2, true, "REQUEST for offered lease, xid mismatch. Expected %#x, got %#x.",
                            pLease->m_xid, pDhcpMsg->bp_xid);
-            pLease->m_xid = pDhcpMsg->bp_xid; /* update xid */
-            pLease->activate();
+            pLease->activate(pDhcpMsg->bp_xid);
+            fAckIt = true;
         }
         else if (!pLease->isInCurrentConfig())
             debugPrint(1, true, "REQUEST for obsolete lease -> NAK");
