@@ -1,10 +1,10 @@
-/* $Id: log-vbox.cpp $ */
+/* $Id: log-vbox.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
- * Virtual Box Runtime - Logging configuration.
+ * VirtualBox Runtime - Logging configuration.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /** @page pg_rtlog      Runtime - Logging
@@ -135,6 +131,12 @@
 #  include <Windows.h>
 # elif defined(RT_OS_LINUX)
 #  include <unistd.h>
+# elif defined(RT_OS_FREEBSD)
+#  include <sys/param.h>
+#  include <sys/sysctl.h>
+#  include <sys/user.h>
+#  include <stdlib.h>
+#  include <unistd.h>
 # elif defined(RT_OS_SOLARIS)
 #  define _STRUCTURED_PROC 1
 #  undef _FILE_OFFSET_BITS /* procfs doesn't like this */
@@ -157,6 +159,7 @@
 # include <iprt/path.h>
 # include <iprt/process.h>
 # include <iprt/string.h>
+# include <iprt/mem.h>
 # include <stdio.h>
 #endif
 
@@ -259,6 +262,8 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
     ASSERT_LOG_GROUP(MM_HYPER_HEAP);
     ASSERT_LOG_GROUP(MM_PHYS);
     ASSERT_LOG_GROUP(MM_POOL);
+    ASSERT_LOG_GROUP(NAT_SERVICE);
+    ASSERT_LOG_GROUP(NET_SERVICE);
     ASSERT_LOG_GROUP(PATM);
     ASSERT_LOG_GROUP(PDM);
     ASSERT_LOG_GROUP(PDM_DEVICE);
@@ -291,7 +296,7 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
      * Create the default logging instance.
      */
 #ifdef IN_RING3
-# ifndef IN_GUEST_R3
+# ifndef IN_GUEST
     char szExecName[RTPATH_MAX];
     if (!RTProcGetExecutableName(szExecName, sizeof(szExecName)))
         strcpy(szExecName, "VBox");
@@ -337,12 +342,8 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
             fclose(pFile);
         }
 
-#  elif defined(RT_OS_LINUX) || defined(RT_OS_FREEBSD)
-#   ifdef RT_OS_LINUX
+#  elif defined(RT_OS_LINUX)
         FILE *pFile = fopen("/proc/self/cmdline", "r");
-#   else /* RT_OS_FREEBSD: */
-        FILE *pFile = fopen("/proc/curproc/cmdline", "r");
-#   endif
         if (pFile)
         {
             /* braindead */
@@ -369,6 +370,40 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
             fclose(pFile);
         }
 
+#  elif defined(RT_OS_FREEBSD)
+        /* Retrieve the required length first */
+        int aiName[4];
+        aiName[0] = CTL_KERN;
+        aiName[1] = KERN_PROC;
+        aiName[2] = KERN_PROC_ARGS;     /* Introduced in FreeBSD 4.0 */
+        aiName[3] = getpid();
+        size_t cchArgs = 0;
+        int rcBSD = sysctl(aiName, RT_ELEMENTS(aiName), NULL, &cchArgs, NULL, 0);
+        if (cchArgs > 0)
+        {
+            char *pszArgFileBuf = (char *)RTMemAllocZ(cchArgs + 1 /* Safety */);
+            if (pszArgFileBuf)
+            {
+                /* Retrieve the argument list */
+                rcBSD = sysctl(aiName, RT_ELEMENTS(aiName), pszArgFileBuf, &cchArgs, NULL, 0);
+                if (!rcBSD)
+                {
+                    unsigned    iArg = 0;
+                    size_t      off = 0;
+                    while (off < cchArgs)
+                    {
+                        size_t cchArg = strlen(&pszArgFileBuf[off]);
+                        RTLogLoggerEx(pLogger, 0, ~0U, "Arg[%u]: %s\n", iArg, &pszArgFileBuf[off]);
+
+                        /* advance */
+                        off += cchArg + 1;
+                        iArg++;
+                    }
+                }
+                RTMemFree(pszArgFileBuf);
+            }
+        }
+
 #  elif defined(RT_OS_L4) || defined(RT_OS_OS2) || defined(RT_OS_DARWIN)
         /* commandline? */
 #  else
@@ -376,21 +411,19 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
 #  endif
     }
 
-# else  /* IN_GUEST_R3  */
+# else  /* IN_GUEST */
     /* The user destination is backdoor logging. */
     rc = RTLogCreate(&pLogger, 0, NULL, "VBOX_LOG",
                      RT_ELEMENTS(g_apszGroups), &g_apszGroups[0],
                      RTLOGDEST_USER, "VBox.log");
-# endif /* IN_GUEST_R3 */
+# endif /* IN_GUEST */
 
 #else /* IN_RING0 */
-    rc = RTLogCreate(&pLogger, 0, NULL, "VBOX_LOG", RT_ELEMENTS(g_apszGroups), &g_apszGroups[0],
-# ifdef LOG_TO_BACKDOOR  /** @todo look at guest ring 0 logging */
-                         RTLOGDEST_USER,
-# else
-                         RTLOGDEST_FILE,
-# endif
-                         "VBox-ring0.log");
+# ifndef IN_GUEST
+    rc = RTLogCreate(&pLogger, 0, NULL, "VBOX_LOG", RT_ELEMENTS(g_apszGroups), &g_apszGroups[0], RTLOGDEST_FILE, "VBox-ring0.log");
+# else  /* IN_GUEST */
+    rc = RTLogCreate(&pLogger, 0, NULL, "VBOX_LOG", RT_ELEMENTS(g_apszGroups), &g_apszGroups[0], RTLOGDEST_USER, "VBox-ring0.log");
+# endif /* IN_GUEST */
     if (RT_SUCCESS(rc))
     {
         /*
@@ -401,11 +434,12 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
          * destination is the one doing all the work. On platforms
          * that do differ (like Darwin), STDOUT is the kernel log.
          */
-# if 0 /*defined(DEBUG_bird) && !defined(IN_GUEST)*/
-        //RTLogGroupSettings(pLogger, "all=~0 -default.l6.l5.l4.l3");
-        RTLogGroupSettings(pLogger, "all=0 pgm*=~0 gmm=~0");
+# if defined(DEBUG_bird)
+        /*RTLogGroupSettings(pLogger, "all=~0 -default.l6.l5.l4.l3");*/
         RTLogFlags(pLogger, "enabled unbuffered pid tid");
+#  ifndef IN_GUEST
         pLogger->fDestFlags |= RTLOGDEST_DEBUGGER | RTLOGDEST_STDOUT;
+#  endif
 # endif
 # if defined(DEBUG_sandervl) && !defined(IN_GUEST)
         RTLogGroupSettings(pLogger, "+all");
@@ -418,10 +452,15 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
         pLogger->fDestFlags |= RTLOGDEST_DEBUGGER;
 # endif
 # if defined(DEBUG_aleksey)  /* Guest ring-0 as well */
-        RTLogGroupSettings(pLogger, "+net_tap_drv.e.l.f+net_flt_drv.e.l.f+srv_intnet.e.l.f");
+        RTLogGroupSettings(pLogger, "+net_adp_drv.e.l.f+net_flt_drv.e.l.l2.l3.l4.f");
         RTLogFlags(pLogger, "enabled unbuffered");
         pLogger->fDestFlags |= RTLOGDEST_DEBUGGER | RTLOGDEST_STDOUT;
 # endif
+# if defined(DEBUG_andy)  /* Guest ring-0 as well */
+        RTLogGroupSettings(pLogger, "+all.e.l.f");
+        RTLogFlags(pLogger, "enabled unbuffered pid tid");
+        pLogger->fDestFlags |= RTLOGDEST_DEBUGGER | RTLOGDEST_STDOUT;
+# endif
 # if 0 /* defined(DEBUG_misha) */ /* Guest ring-0 as well */
         RTLogGroupSettings(pLogger, "+net_flt_drv.e.l.f.l2+srv_intnet.e.l.f");
         RTLogFlags(pLogger, "enabled unbuffered");
