@@ -1,10 +1,10 @@
-/* $Id: alloc.cpp $ */
+/* $Id: alloc.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Memory Allocation.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,20 +22,22 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include <iprt/alloc.h>
+#include <iprt/mem.h>
+#include "internal/iprt.h"
+
 #include <iprt/assert.h>
 #include <iprt/string.h>
 
+#undef RTMemDup
+#undef RTMemDupEx
+
+
 
 /**
  * Duplicates a chunk of memory into a new heap block.
@@ -52,6 +54,7 @@ RTDECL(void *) RTMemDup(const void *pvSrc, size_t cb) RT_NO_THROW
         memcpy(pvDst, pvSrc, cb);
     return pvDst;
 }
+RT_EXPORT_SYMBOL(RTMemDup);
 
 
 /**
@@ -74,5 +77,5 @@ RTDECL(void *) RTMemDupEx(const void *pvSrc, size_t cbSrc, size_t cbExtra) RT_NO
     }
     return pvDst;
 }
-
+RT_EXPORT_SYMBOL(RTMemDupEx);
 
