@@ -1,10 +1,10 @@
-/* $Id: ipv4.cpp $ */
+/* $Id: ipv4.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - IPv4 Checksum calculation and validation.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,16 +22,15 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/net.h>
+#include "internal/iprt.h"
+
 #include <iprt/asm.h>
 #include <iprt/assert.h>
 
@@ -82,6 +81,7 @@ RTDECL(uint16_t) RTNetIPv4HdrChecksum(PCRTNETIPV4 pIpHdr)
     u32Sum += u32Sum >> 16;                     /* carry */
     return (uint16_t)~u32Sum;
 }
+RT_EXPORT_SYMBOL(RTNetIPv4HdrChecksum);
 
 
 /**
@@ -94,8 +94,9 @@ RTDECL(uint16_t) RTNetIPv4HdrChecksum(PCRTNETIPV4 pIpHdr)
  *                      to if you like. Note that an IPv4 header can be up to 60 bytes.
  * @param   cbPktMax    The max IP packet size, IP header and payload. This doesn't have
  *                      to be mapped following pIpHdr.
+ * @param   fChecksum   Whether to validate the checksum (GSO).
  */
-RTDECL(bool) RTNetIPv4IsHdrValid(PCRTNETIPV4 pIpHdr, size_t cbHdrMax, size_t cbPktMax)
+RTDECL(bool) RTNetIPv4IsHdrValid(PCRTNETIPV4 pIpHdr, size_t cbHdrMax, size_t cbPktMax, bool fChecksum)
 {
     /*
      * The header fields.
@@ -116,13 +117,17 @@ RTDECL(bool) RTNetIPv4IsHdrValid(PCRTNETIPV4 pIpHdr, size_t cbHdrMax, size_t cbP
         return false;
 
     /*
-     * The header checksum.
+     * The header checksum if requested.
      */
-    uint16_t u16Sum = RTNetIPv4HdrChecksum(pIpHdr);
-    if (RT_UNLIKELY(pIpHdr->ip_sum != u16Sum))
-        return false;
+    if (fChecksum)
+    {
+        uint16_t u16Sum = RTNetIPv4HdrChecksum(pIpHdr);
+        if (RT_UNLIKELY(pIpHdr->ip_sum != u16Sum))
+            return false;
+    }
     return true;
 }
+RT_EXPORT_SYMBOL(RTNetIPv4IsHdrValid);
 
 
 /**
@@ -158,6 +163,7 @@ RTDECL(uint32_t) RTNetIPv4PseudoChecksum(PCRTNETIPV4 pIpHdr)
 {
     return rtNetIPv4PseudoChecksum(pIpHdr);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4PseudoChecksum);
 
 
 /**
@@ -183,6 +189,7 @@ RTDECL(uint32_t) RTNetIPv4PseudoChecksumBits(RTNETADDRIPV4 SrcAddr, RTNETADDRIPV
                     + RT_H2BE_U16(cbPkt);
     return u32Sum;
 }
+RT_EXPORT_SYMBOL(RTNetIPv4PseudoChecksumBits);
 
 
 /**
@@ -213,15 +220,16 @@ RTDECL(uint32_t) RTNetIPv4AddUDPChecksum(PCRTNETUDP pUdpHdr, uint32_t u32Sum)
 {
     return rtNetIPv4AddUDPChecksum(pUdpHdr, u32Sum);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4AddUDPChecksum);
 
 
 /**
  * Adds the checksum of the TCP header to the intermediate checksum value [inlined].
  *
  * @returns 32-bit intermediary checksum value.
- * @param   pUdpHdr         Pointer to the TCP header to checksum, network endian (big).
- *                          Assums the caller has already validate it and made sure the
- *                          entire header is present.
+ * @param   pTcpHdr         Pointer to the TCP header to checksum, network
+ *                          endian (big). Assums the caller has already validate
+ *                          it and made sure the entire header is present.
  * @param   u32Sum          The 32-bit intermediate checksum value.
  */
 DECLINLINE(uint32_t) rtNetIPv4AddTCPChecksum(PCRTNETTCP pTcpHdr, uint32_t u32Sum)
@@ -265,22 +273,24 @@ DECLINLINE(uint32_t) rtNetIPv4AddTCPChecksum(PCRTNETTCP pTcpHdr, uint32_t u32Sum
  * Adds the checksum of the TCP header to the intermediate checksum value.
  *
  * @returns 32-bit intermediary checksum value.
- * @param   pUdpHdr         Pointer to the TCP header to checksum, network endian (big).
- *                          Assums the caller has already validate it and made sure the
- *                          entire header is present.
+ * @param   pTcpHdr         Pointer to the TCP header to checksum, network
+ *                          endian (big). Assums the caller has already validate
+ *                          it and made sure the entire header is present.
  * @param   u32Sum          The 32-bit intermediate checksum value.
  */
 RTDECL(uint32_t) RTNetIPv4AddTCPChecksum(PCRTNETTCP pTcpHdr, uint32_t u32Sum)
 {
     return rtNetIPv4AddTCPChecksum(pTcpHdr, u32Sum);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4AddTCPChecksum);
 
 
 /**
  * Adds the checksum of the specified data segment to the intermediate checksum value [inlined].
  *
  * @returns 32-bit intermediary checksum value.
- * @param   pUdpHdr         Pointer to the UDP header to checksum, network endian (big).
+ * @param   pvData          Pointer to the data that should be checksummed.
+ * @param   cbData          The number of bytes to checksum.
  * @param   u32Sum          The 32-bit intermediate checksum value.
  * @param   pfOdd           This is used to keep track of odd bits, initialize to false
  *                          when starting to checksum the data (aka text) after a TCP
@@ -332,7 +342,8 @@ DECLINLINE(uint32_t) rtNetIPv4AddDataChecksum(void const *pvData, size_t cbData,
  * Adds the checksum of the specified data segment to the intermediate checksum value.
  *
  * @returns 32-bit intermediary checksum value.
- * @param   pUdpHdr         Pointer to the UDP header to checksum, network endian (big).
+ * @param   pvData          The data bits to checksum.
+ * @param   cbData          The number of bytes to checksum.
  * @param   u32Sum          The 32-bit intermediate checksum value.
  * @param   pfOdd           This is used to keep track of odd bits, initialize to false
  *                          when starting to checksum the data (aka text) after a TCP
@@ -342,6 +353,7 @@ RTDECL(uint32_t) RTNetIPv4AddDataChecksum(void const *pvData, size_t cbData, uin
 {
     return rtNetIPv4AddDataChecksum(pvData, cbData, u32Sum, pfOdd);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4AddDataChecksum);
 
 
 /**
@@ -369,6 +381,28 @@ RTDECL(uint16_t) RTNetIPv4FinalizeChecksum(uint32_t u32Sum)
 {
     return rtNetIPv4FinalizeChecksum(u32Sum);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4FinalizeChecksum);
+
+
+/**
+ * Calculates the checksum for the UDP header given the UDP header w/ payload
+ * and the checksum of the pseudo header.
+ *
+ * @returns The checksum (network endian).
+ * @param   u32Sum          The checksum of the pseudo header. See
+ *                          RTNetIPv4PseudoChecksum and RTNetIPv6PseudoChecksum.
+ * @param   pUdpHdr         Pointer to the UDP header and the payload, in
+ *                          network endian (big).  We use the uh_ulen field to
+ *                          figure out how much to checksum.
+ */
+RTDECL(uint16_t) RTNetUDPChecksum(uint32_t u32Sum, PCRTNETUDP pUdpHdr)
+{
+    u32Sum = rtNetIPv4AddUDPChecksum(pUdpHdr, u32Sum);
+    bool fOdd = false;
+    u32Sum = rtNetIPv4AddDataChecksum(pUdpHdr + 1, RT_BE2H_U16(pUdpHdr->uh_ulen) - sizeof(*pUdpHdr), u32Sum, &fOdd);
+    return rtNetIPv4FinalizeChecksum(u32Sum);
+}
+RT_EXPORT_SYMBOL(RTNetUDPChecksum);
 
 
 /**
@@ -390,6 +424,7 @@ RTDECL(uint16_t) RTNetIPv4UDPChecksum(PCRTNETIPV4 pIpHdr, PCRTNETUDP pUdpHdr, vo
     u32Sum = rtNetIPv4AddDataChecksum(pvData, RT_BE2H_U16(pUdpHdr->uh_ulen) - sizeof(*pUdpHdr), u32Sum, &fOdd);
     return rtNetIPv4FinalizeChecksum(u32Sum);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4UDPChecksum);
 
 
 /**
@@ -430,6 +465,7 @@ RTDECL(bool) RTNetIPv4IsUDPSizeValid(PCRTNETIPV4 pIpHdr, PCRTNETUDP pUdpHdr, siz
 {
     return rtNetIPv4IsUDPSizeValid(pIpHdr, pUdpHdr, cbPktMax);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4IsUDPSizeValid);
 
 
 /**
@@ -442,12 +478,13 @@ RTDECL(bool) RTNetIPv4IsUDPSizeValid(PCRTNETIPV4 pIpHdr, PCRTNETUDP pUdpHdr, siz
  * @param   pvData          Pointer to the data, assuming it's one single segment
  *                          and that cbPktMax - sizeof(RTNETUDP) is mapped here.
  * @param   cbPktMax        The max UDP packet size, UDP header and payload (data).
+ * @param   fChecksum       Whether to validate the checksum (GSO).
  */
-RTDECL(bool) RTNetIPv4IsUDPValid(PCRTNETIPV4 pIpHdr, PCRTNETUDP pUdpHdr, void const *pvData, size_t cbPktMax)
+RTDECL(bool) RTNetIPv4IsUDPValid(PCRTNETIPV4 pIpHdr, PCRTNETUDP pUdpHdr, void const *pvData, size_t cbPktMax, bool fChecksum)
 {
     if (RT_UNLIKELY(!rtNetIPv4IsUDPSizeValid(pIpHdr, pUdpHdr, cbPktMax)))
         return false;
-    if (pUdpHdr->uh_sum)
+    if (fChecksum && pUdpHdr->uh_sum)
     {
         uint16_t u16Sum = RTNetIPv4UDPChecksum(pIpHdr, pUdpHdr, pvData);
         if (RT_UNLIKELY(pUdpHdr->uh_sum != u16Sum))
@@ -455,6 +492,7 @@ RTDECL(bool) RTNetIPv4IsUDPValid(PCRTNETIPV4 pIpHdr, PCRTNETUDP pUdpHdr, void co
     }
     return true;
 }
+RT_EXPORT_SYMBOL(RTNetIPv4IsUDPValid);
 
 
 /**
@@ -464,11 +502,10 @@ RTDECL(bool) RTNetIPv4IsUDPValid(PCRTNETIPV4 pIpHdr, PCRTNETUDP pUdpHdr, void co
  * @returns The checksum (network endian).
  * @param   pIpHdr          Pointer to the IPv4 header, in network endian (big).
  * @param   pTcpHdr         Pointer to the TCP header, in network endian (big).
- * @param   pvData          Pointer to the TCP payload. The size is taken from the
- *                          TCP header and the caller is supposed to have validated
- *                          this before calling.
- *                          If NULL then we assume the data follows immediately after
- *                          the TCP header.
+ * @param   pvData          Pointer to the TCP payload. The size is derived from
+ *                          the two headers and the caller is supposed to have
+ *                          validated this before calling.  If NULL, we assume
+ *                          the data follows immediately after the TCP header.
  */
 RTDECL(uint16_t) RTNetIPv4TCPChecksum(PCRTNETIPV4 pIpHdr, PCRTNETTCP pTcpHdr, void const *pvData)
 {
@@ -480,6 +517,30 @@ RTDECL(uint16_t) RTNetIPv4TCPChecksum(PCRTNETIPV4 pIpHdr, PCRTNETTCP pTcpHdr, vo
                                       cbData, u32Sum, &fOdd);
     return rtNetIPv4FinalizeChecksum(u32Sum);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4TCPChecksum);
+
+
+/**
+ * Calculates the checksum for the TCP header given the TCP header, payload and
+ * the checksum of the pseudo header.
+ *
+ * This is not specific to IPv4.
+ *
+ * @returns The checksum (network endian).
+ * @param   u32Sum          The checksum of the pseudo header. See
+ *                          RTNetIPv4PseudoChecksum and RTNetIPv6PseudoChecksum.
+ * @param   pTcpHdr         Pointer to the TCP header, in network endian (big).
+ * @param   pvData          Pointer to the TCP payload.
+ * @param   cbData          The size of the TCP payload.
+ */
+RTDECL(uint16_t) RTNetTCPChecksum(uint32_t u32Sum, PCRTNETTCP pTcpHdr, void const *pvData, size_t cbData)
+{
+    u32Sum = rtNetIPv4AddTCPChecksum(pTcpHdr, u32Sum);
+    bool fOdd = false;
+    u32Sum = rtNetIPv4AddDataChecksum(pvData, cbData, u32Sum, &fOdd);
+    return rtNetIPv4FinalizeChecksum(u32Sum);
+}
+RT_EXPORT_SYMBOL(RTNetTCPChecksum);
 
 
 /**
@@ -525,6 +586,7 @@ RTDECL(bool) RTNetIPv4IsTCPSizeValid(PCRTNETIPV4 pIpHdr, PCRTNETTCP pTcpHdr, siz
 {
     return rtNetIPv4IsTCPSizeValid(pIpHdr, pTcpHdr, cbHdrMax, cbPktMax);
 }
+RT_EXPORT_SYMBOL(RTNetIPv4IsTCPSizeValid);
 
 
 /**
@@ -540,16 +602,22 @@ RTDECL(bool) RTNetIPv4IsTCPSizeValid(PCRTNETIPV4 pIpHdr, PCRTNETTCP pTcpHdr, siz
  *                          If NULL then we assume the data follows immediately after
  *                          the TCP header.
  * @param   cbPktMax        The max TCP packet size, TCP header and payload (data).
+ * @param   fChecksum       Whether to validate the checksum (GSO).
  */
-RTDECL(bool) RTNetIPv4IsTCPValid(PCRTNETIPV4 pIpHdr, PCRTNETTCP pTcpHdr, size_t cbHdrMax, void const *pvData, size_t cbPktMax)
+RTDECL(bool) RTNetIPv4IsTCPValid(PCRTNETIPV4 pIpHdr, PCRTNETTCP pTcpHdr, size_t cbHdrMax, void const *pvData, size_t cbPktMax,
+                                 bool fChecksum)
 {
     if (RT_UNLIKELY(!rtNetIPv4IsTCPSizeValid(pIpHdr, pTcpHdr, cbHdrMax, cbPktMax)))
         return false;
-    uint16_t u16Sum = RTNetIPv4TCPChecksum(pIpHdr, pTcpHdr, pvData);
-    if (RT_UNLIKELY(pTcpHdr->th_sum != u16Sum))
-        return false;
+    if (fChecksum)
+    {
+        uint16_t u16Sum = RTNetIPv4TCPChecksum(pIpHdr, pTcpHdr, pvData);
+        if (RT_UNLIKELY(pTcpHdr->th_sum != u16Sum))
+            return false;
+    }
     return true;
 }
+RT_EXPORT_SYMBOL(RTNetIPv4IsTCPValid);
 
 
 /**
@@ -678,4 +746,5 @@ RTDECL(bool) RTNetIPv4IsDHCPValid(PCRTNETUDP pUdpHdr, PCRTNETBOOTP pDhcp, size_t
 
     return true;
 }
+RT_EXPORT_SYMBOL(RTNetIPv4IsDHCPValid);
 
