@@ -1,10 +1,10 @@
-/* $Id: dbgas.cpp $ */
+/* $Id: dbgas.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Debug Address Space.
  */
 
 /*
- * Copyright (C) 2009 Sun Microsystems, Inc.
+ * Copyright (C) 2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,16 +22,15 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/dbg.h>
+#include "internal/iprt.h"
+
 #include <iprt/asm.h>
 #include <iprt/avl.h>
 #include <iprt/assert.h>
@@ -64,6 +63,8 @@ typedef struct RTDBGASMOD
     PRTDBGASMAP         pMapHead;
     /** Pointer to the next module with an identical name. */
     PRTDBGASMOD         pNextName;
+    /** The index into RTDBGASINT::papModules. */
+    uint32_t            iOrdinal;
 } RTDBGASMOD;
 
 /**
@@ -108,7 +109,7 @@ typedef struct RTDBGASINT
     uint32_t            cModules;
     /** Pointer to the module table.
      * The valid array length is given by cModules. */
-    PRTDBGASMOD         paModules;
+    PRTDBGASMOD        *papModules;
     /** AVL tree translating module handles to module entries. */
     AVLPVTREE           ModTree;
     /** AVL tree mapping addresses to modules. */
@@ -205,7 +206,7 @@ RTDECL(int) RTDbgAsCreate(PRTDBGAS phDbgAs, RTUINTPTR FirstAddr, RTUINTPTR LastA
     pDbgAs->cRefs       = 1;
     pDbgAs->hLock       = NIL_RTSEMRW;
     pDbgAs->cModules    = 0;
-    pDbgAs->paModules   = NULL;
+    pDbgAs->papModules  = NULL;
     pDbgAs->ModTree     = NULL;
     pDbgAs->MapTree     = NULL;
     pDbgAs->NameSpace   = NULL;
@@ -223,6 +224,7 @@ RTDECL(int) RTDbgAsCreate(PRTDBGAS phDbgAs, RTUINTPTR FirstAddr, RTUINTPTR LastA
     RTMemFree(pDbgAs);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsCreate);
 
 
 /**
@@ -250,6 +252,7 @@ RTDECL(int) RTDbgAsCreateV(PRTDBGAS phDbgAs, RTUINTPTR FirstAddr, RTUINTPTR Last
     RTStrFree(pszName);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsCreateV);
 
 
 /**
@@ -271,6 +274,7 @@ RTDECL(int) RTDbgAsCreateF(PRTDBGAS phDbgAs, RTUINTPTR FirstAddr, RTUINTPTR Last
     va_end(va);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsCreateF);
 
 
 /**
@@ -325,11 +329,20 @@ static void rtDbgAsDestroy(PRTDBGASINT pDbgAs)
     uint32_t i = pDbgAs->cModules;
     while (i-- > 0)
     {
-        RTDbgModRelease((RTDBGMOD)pDbgAs->paModules[i].Core.Key);
-        pDbgAs->paModules[i].Core.Key = NIL_RTDBGMOD;
+        PRTDBGASMOD pMod = pDbgAs->papModules[i];
+        AssertPtr(pMod);
+        if (VALID_PTR(pMod))
+        {
+            Assert(pMod->iOrdinal == i);
+            RTDbgModRelease((RTDBGMOD)pMod->Core.Key);
+            pMod->Core.Key = NIL_RTDBGMOD;
+            pMod->iOrdinal = UINT32_MAX;
+            RTMemFree(pMod);
+        }
+        pDbgAs->papModules[i] = NULL;
     }
-    RTMemFree(pDbgAs->paModules);
-    pDbgAs->paModules = NULL;
+    RTMemFree(pDbgAs->papModules);
+    pDbgAs->papModules = NULL;
 
     RTMemFree(pDbgAs);
 }
@@ -350,6 +363,7 @@ RTDECL(uint32_t) RTDbgAsRetain(RTDBGAS hDbgAs)
     RTDBGAS_VALID_RETURN_RC(pDbgAs, UINT32_MAX);
     return ASMAtomicIncU32(&pDbgAs->cRefs);
 }
+RT_EXPORT_SYMBOL(RTDbgAsRetain);
 
 
 /**
@@ -379,6 +393,7 @@ RTDECL(uint32_t) RTDbgAsRelease(RTDBGAS hDbgAs)
         rtDbgAsDestroy(pDbgAs);
     return cRefs;
 }
+RT_EXPORT_SYMBOL(RTDbgAsRelease);
 
 
 /**
@@ -397,6 +412,7 @@ RTDECL(const char *) RTDbgAsName(RTDBGAS hDbgAs)
     RTDBGAS_VALID_RETURN_RC(pDbgAs, NULL);
     return pDbgAs->szName;
 }
+RT_EXPORT_SYMBOL(RTDbgAsName);
 
 
 /**
@@ -415,6 +431,7 @@ RTDECL(RTUINTPTR) RTDbgAsFirstAddr(RTDBGAS hDbgAs)
     RTDBGAS_VALID_RETURN_RC(pDbgAs, 0);
     return pDbgAs->FirstAddr;
 }
+RT_EXPORT_SYMBOL(RTDbgAsFirstAddr);
 
 
 /**
@@ -433,6 +450,7 @@ RTDECL(RTUINTPTR) RTDbgAsLastAddr(RTDBGAS hDbgAs)
     RTDBGAS_VALID_RETURN_RC(pDbgAs, 0);
     return pDbgAs->LastAddr;
 }
+RT_EXPORT_SYMBOL(RTDbgAsLastAddr);
 
 /**
  * Gets the number of modules in the address space.
@@ -452,6 +470,7 @@ RTDECL(uint32_t) RTDbgAsModuleCount(RTDBGAS hDbgAs)
     RTDBGAS_VALID_RETURN_RC(pDbgAs, 0);
     return pDbgAs->cModules;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleCount);
 
 
 /**
@@ -487,7 +506,7 @@ int rtDbgAsModuleLinkCommon(PRTDBGASINT pDbgAs, RTDBGMOD hDbgMod, RTDBGSEGIDX iS
         }
         pAdjMod = (PRTDBGASMAP)RTAvlrUIntPtrGetBestFit(&pDbgAs->MapTree, Addr, true /* fAbove */);
         if (     pAdjMod
-            &&   pAdjMod->Core.Key >= Addr + cb - 1)
+            &&   pAdjMod->Core.Key <= Addr + cb - 1)
         {
             if (!(fFlags & RTDBGASLINK_FLAGS_REPLACE))
                 return VERR_ADDRESS_CONFLICT;
@@ -508,23 +527,27 @@ int rtDbgAsModuleLinkCommon(PRTDBGASINT pDbgAs, RTDBGMOD hDbgMod, RTDBGSEGIDX iS
          */
         if (!(pDbgAs->cModules % 32))
         {
-            void *pvNew = RTMemRealloc(pDbgAs->paModules, sizeof(pDbgAs->paModules[0]) * (pDbgAs->cModules + 32));
+            void *pvNew = RTMemRealloc(pDbgAs->papModules, sizeof(pDbgAs->papModules[0]) * (pDbgAs->cModules + 32));
             if (!pvNew)
                 return VERR_NO_MEMORY;
-            pDbgAs->paModules = (PRTDBGASMOD)pvNew;
+            pDbgAs->papModules = (PRTDBGASMOD *)pvNew;
         }
-        pMod = &pDbgAs->paModules[pDbgAs->cModules];
-        pDbgAs->cModules++;
-
+        pMod = (PRTDBGASMOD)RTMemAlloc(sizeof(*pMod));
+        if (!pMod)
+            return VERR_NO_MEMORY;
         pMod->Core.Key  = hDbgMod;
         pMod->pMapHead  = NULL;
         pMod->pNextName = NULL;
-        if (!RTAvlPVInsert(&pDbgAs->ModTree, &pMod->Core))
+        if (RT_UNLIKELY(!RTAvlPVInsert(&pDbgAs->ModTree, &pMod->Core)))
         {
             AssertFailed();
             pDbgAs->cModules--;
+            RTMemFree(pMod);
             return VERR_INTERNAL_ERROR;
         }
+        pMod->iOrdinal = pDbgAs->cModules;
+        pDbgAs->papModules[pDbgAs->cModules] = pMod;
+        pDbgAs->cModules++;
         RTDbgModRetain(hDbgMod);
 
         /*
@@ -537,13 +560,17 @@ int rtDbgAsModuleLinkCommon(PRTDBGASINT pDbgAs, RTDBGMOD hDbgMod, RTDBGSEGIDX iS
             pName = (PRTDBGASNAME)RTMemAlloc(sizeof(*pName) + cchName + 1);
             if (!pName)
             {
-                pDbgAs->cModules--;
                 RTDbgModRelease(hDbgMod);
+                pDbgAs->cModules--;
+                RTAvlPVRemove(&pDbgAs->ModTree, hDbgMod);
+                RTMemFree(pMod);
                 return VERR_NO_MEMORY;
             }
             pName->StrCore.cchString = cchName;
             pName->StrCore.pszString = (char *)memcpy(pName + 1, pszName, cchName + 1);
             pName->pHead = pMod;
+            if (!RTStrSpaceInsert(&pDbgAs->NameSpace, &pName->StrCore))
+                AssertFailed();
         }
         else
         {
@@ -634,6 +661,7 @@ RTDECL(int) RTDbgAsModuleLink(RTDBGAS hDbgAs, RTDBGMOD hDbgMod, RTUINTPTR ImageA
     RTDBGAS_UNLOCK_WRITE(pDbgAs);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleLink);
 
 
 /**
@@ -682,6 +710,7 @@ RTDECL(int) RTDbgAsModuleLinkSeg(RTDBGAS hDbgAs, RTDBGMOD hDbgMod, RTDBGSEGIDX i
     RTDBGAS_UNLOCK_WRITE(pDbgAs);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleLinkSeg);
 
 
 /**
@@ -699,7 +728,8 @@ static void rtDbgAsModuleUnlinkMod(PRTDBGASINT pDbgAs, PRTDBGASMOD pMod)
     /*
      * Unlink it from the name.
      */
-    PRTDBGASNAME pName = (PRTDBGASNAME)RTStrSpaceGet(&pDbgAs->NameSpace, RTDbgModName((RTDBGMOD)pMod->Core.Key));
+    const char  *pszName = RTDbgModName((RTDBGMOD)pMod->Core.Key);
+    PRTDBGASNAME pName   = (PRTDBGASNAME)RTStrSpaceGet(&pDbgAs->NameSpace, pszName);
     AssertReturnVoid(pName);
 
     if (pName->pHead == pMod)
@@ -732,11 +762,21 @@ static void rtDbgAsModuleUnlinkMod(PRTDBGASINT pDbgAs, PRTDBGASMOD pMod)
     /*
      * Remove it from the module table by replacing it by the last entry.
      */
-    uint32_t iMod = pMod - &pDbgAs->paModules[0];
-    Assert(iMod < pDbgAs->cModules);
     pDbgAs->cModules--;
-    if (iMod <= pDbgAs->cModules)
-        pDbgAs->paModules[iMod] = pDbgAs->paModules[pDbgAs->cModules];
+    uint32_t iMod = pMod->iOrdinal;
+    Assert(iMod <= pDbgAs->cModules);
+    if (iMod != pDbgAs->cModules)
+    {
+        PRTDBGASMOD pTailMod = pDbgAs->papModules[pDbgAs->cModules];
+        pTailMod->iOrdinal = iMod;
+        pDbgAs->papModules[iMod] = pTailMod;
+    }
+    pMod->iOrdinal = UINT32_MAX;
+
+    /*
+     * Free it.
+     */
+    RTMemFree(pMod);
 }
 
 
@@ -752,15 +792,15 @@ static void rtDbgAsModuleUnlinkMap(PRTDBGASINT pDbgAs, PRTDBGASMAP pMap)
 {
     /* remove from the tree */
     PAVLRUINTPTRNODECORE pNode = RTAvlrUIntPtrRemove(&pDbgAs->MapTree, pMap->Core.Key);
-    Assert(pNode);
+    Assert(pNode == &pMap->Core);
 
     /* unlink */
     PRTDBGASMOD pMod = pMap->pMod;
-    if (pMod->pMapHead)
+    if (pMod->pMapHead == pMap)
         pMod->pMapHead = pMap->pNext;
     else
     {
-        bool        fFound = false;
+        bool fFound = false;
         for (PRTDBGASMAP pCur = pMod->pMapHead; pCur; pCur = pCur->pNext)
             if (pCur->pNext == pMap)
             {
@@ -838,6 +878,7 @@ RTDECL(int) RTDbgAsModuleUnlink(RTDBGAS hDbgAs, RTDBGMOD hDbgMod)
     RTDBGAS_UNLOCK_WRITE(pDbgAs);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleUnlink);
 
 
 /**
@@ -873,6 +914,7 @@ RTDECL(int) RTDbgAsModuleUnlinkByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr)
     RTDBGAS_UNLOCK_WRITE(pDbgAs);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleUnlinkByAddr);
 
 
 /**
@@ -907,12 +949,13 @@ RTDECL(RTDBGMOD) RTDbgAsModuleByIndex(RTDBGAS hDbgAs, uint32_t iModule)
     /*
      * Get, retain and return it.
      */
-    RTDBGMOD hMod = (RTDBGMOD)pDbgAs->paModules[iModule].Core.Key;
+    RTDBGMOD hMod = (RTDBGMOD)pDbgAs->papModules[iModule]->Core.Key;
     RTDbgModRetain(hMod);
 
     RTDBGAS_UNLOCK_READ(pDbgAs);
     return hMod;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleByIndex);
 
 
 /**
@@ -964,6 +1007,7 @@ RTDECL(int) RTDbgAsModuleByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTDBGMOD phMod,
     RTDBGAS_UNLOCK_READ(pDbgAs);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleByAddr);
 
 
 /**
@@ -1018,6 +1062,70 @@ RTDECL(int) RTDbgAsModuleByName(RTDBGAS hDbgAs, const char *pszName, uint32_t iN
     RTDBGAS_UNLOCK_READ(pDbgAs);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTDbgAsModuleByName);
+
+
+/**
+ * Queries mapping information for a module given by index.
+ *
+ * @returns IRPT status code.
+ * @retval  VERR_INVALID_HANDLE if hDbgAs is invalid.
+ * @retval  VERR_OUT_OF_RANGE if the name index was out of range.
+ * @retval  VINF_BUFFER_OVERFLOW if the array is too small and the returned
+ *          information is incomplete.
+ *
+ * @param   hDbgAs          The address space handle.
+ * @param   iModule         The index of the module to get.
+ * @param   paMappings      Where to return the mapping information.  The buffer
+ *                          size is given by *pcMappings.
+ * @param   pcMappings      IN: Size of the paMappings array. OUT: The number of
+ *                          entries returned.
+ * @param   fFlags          Flags for reserved for future use. MBZ.
+ *
+ * @remarks See remarks for RTDbgAsModuleByIndex regarding the volatility of the
+ *          iModule parameter.
+ */
+RTDECL(int) RTDbgAsModuleQueryMapByIndex(RTDBGAS hDbgAs, uint32_t iModule, PRTDBGASMAPINFO paMappings, uint32_t *pcMappings, uint32_t fFlags)
+{
+    /*
+     * Validate input.
+     */
+    uint32_t const  cMappings = *pcMappings;
+    PRTDBGASINT     pDbgAs = hDbgAs;
+    RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
+    AssertReturn(!fFlags, VERR_INVALID_PARAMETER);
+
+    RTDBGAS_LOCK_READ(pDbgAs);
+    if (iModule >= pDbgAs->cModules)
+    {
+        RTDBGAS_UNLOCK_READ(pDbgAs);
+        return VERR_OUT_OF_RANGE;
+    }
+
+    /*
+     * Copy the mapping information about the module.
+     */
+    int         rc    = VINF_SUCCESS;
+    PRTDBGASMAP pMap  = pDbgAs->papModules[iModule]->pMapHead;
+    uint32_t    cMaps = 0;
+    while (pMap)
+    {
+        if (cMaps >= cMappings)
+        {
+            rc = VINF_BUFFER_OVERFLOW;
+            break;
+        }
+        paMappings[cMaps].Address = pMap->Core.Key;
+        paMappings[cMaps].iSeg    = pMap->iSeg;
+        cMaps++;
+        pMap = pMap->pNext;
+    }
+
+    RTDBGAS_UNLOCK_READ(pDbgAs);
+    *pcMappings = cMaps;
+    return rc;
+}
+RT_EXPORT_SYMBOL(RTDbgAsModuleQueryMapByIndex);
 
 
 /**
@@ -1054,8 +1162,8 @@ DECLINLINE(RTDBGMOD) rtDbgAsModuleByAddr(PRTDBGASINT pDbgAs, RTUINTPTR Addr, PRT
 /**
  * Adjusts the address to correspond to the mapping of the module/segment.
  *
- * @param   pSymbol         The returned symbol info.
- * @param   pMap            The mapping record.
+ * @param   pAddr           The address to adjust (in/out).
+ * @param   iSeg            The related segment.
  * @param   hDbgMod         The module handle.
  * @param   MapAddr         The mapping address.
  * @param   iMapSeg         The segment that's mapped, NIL_RTDBGSEGIDX or
@@ -1146,6 +1254,9 @@ DECLINLINE(void) rtDbgAsAdjustLineAddress(PRTDBGLINE pLine, RTDBGMOD hDbgMod, RT
  * @param   Addr            The address of the symbol.
  * @param   cb              The size of the symbol.
  * @param   fFlags          Symbol flags.
+ * @param   piOrdinal       Where to return the symbol ordinal on success. If
+ *                          the interpreter doesn't do ordinals, this will be set to
+ *                          UINT32_MAX. Optional
  */
 RTDECL(int) RTDbgAsSymbolAdd(RTDBGAS hDbgAs, const char *pszSymbol, RTUINTPTR Addr, RTUINTPTR cb, uint32_t fFlags, uint32_t *piOrdinal)
 {
@@ -1155,9 +1266,9 @@ RTDECL(int) RTDbgAsSymbolAdd(RTDBGAS hDbgAs, const char *pszSymbol, RTUINTPTR Ad
     PRTDBGASINT pDbgAs = hDbgAs;
     RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
 
-    RTDBGSEGIDX iSeg;
-    RTUINTPTR offSeg;
-    RTDBGMOD hMod = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, NULL);
+    RTDBGSEGIDX iSeg   = NIL_RTDBGSEGIDX; /* shut up gcc */
+    RTUINTPTR   offSeg = 0;
+    RTDBGMOD    hMod   = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, NULL);
     if (hMod == NIL_RTDBGMOD)
         return VERR_NOT_FOUND;
 
@@ -1168,6 +1279,7 @@ RTDECL(int) RTDbgAsSymbolAdd(RTDBGAS hDbgAs, const char *pszSymbol, RTUINTPTR Ad
     RTDbgModRelease(hMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsSymbolAdd);
 
 
 /**
@@ -1182,8 +1294,9 @@ RTDECL(int) RTDbgAsSymbolAdd(RTDBGAS hDbgAs, const char *pszSymbol, RTUINTPTR Ad
  * @param   poffDisp        Where to return the distance between the symbol
  *                          and address. Optional.
  * @param   pSymbol         Where to return the symbol info.
+ * @param   phMod           Where to return the module handle. Optional.
  */
-RTDECL(int) RTDbgAsSymbolByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDisp, PRTDBGSYMBOL pSymbol)
+RTDECL(int) RTDbgAsSymbolByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDisp, PRTDBGSYMBOL pSymbol, PRTDBGMOD phMod)
 {
     /*
      * Validate input and resolve the address.
@@ -1191,12 +1304,16 @@ RTDECL(int) RTDbgAsSymbolByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDi
     PRTDBGASINT pDbgAs = hDbgAs;
     RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
 
-    RTDBGSEGIDX iSeg;
-    RTUINTPTR   offSeg;
-    RTUINTPTR   MapAddr;
-    RTDBGMOD    hMod = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
+    RTDBGSEGIDX iSeg    = NIL_RTDBGSEGIDX; /* shut up gcc */
+    RTUINTPTR   offSeg  = 0;
+    RTUINTPTR   MapAddr = 0;
+    RTDBGMOD    hMod    = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
     if (hMod == NIL_RTDBGMOD)
+    {
+        if (phMod)
+            *phMod = NIL_RTDBGMOD;
         return VERR_NOT_FOUND;
+    }
 
     /*
      * Forward the call.
@@ -1204,9 +1321,13 @@ RTDECL(int) RTDbgAsSymbolByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDi
     int rc = RTDbgModSymbolByAddr(hMod, iSeg, offSeg, poffDisp, pSymbol);
     if (RT_SUCCESS(rc))
         rtDbgAsAdjustSymbolValue(pSymbol, hMod, MapAddr, iSeg);
-    RTDbgModRelease(hMod);
+    if (phMod)
+        *phMod = hMod;
+    else
+        RTDbgModRelease(hMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsSymbolByAddr);
 
 
 /**
@@ -1220,10 +1341,11 @@ RTDECL(int) RTDbgAsSymbolByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDi
  * @param   Addr            The address which closest symbol is requested.
  * @param   poffDisp        Where to return the distance between the symbol
  *                          and address. Optional.
- * @param   ppSymbol        Where to return the pointer to the allocated
- *                          symbol info. Always set. Free with RTDbgSymbolFree.
+ * @param   ppSymInfo       Where to return the pointer to the allocated symbol
+ *                          info. Always set. Free with RTDbgSymbolFree.
+ * @param   phMod           Where to return the module handle. Optional.
  */
-RTDECL(int) RTDbgAsSymbolByAddrA(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDisp, PRTDBGSYMBOL *ppSymbol)
+RTDECL(int) RTDbgAsSymbolByAddrA(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDisp, PRTDBGSYMBOL *ppSymInfo, PRTDBGMOD phMod)
 {
     /*
      * Validate input and resolve the address.
@@ -1231,22 +1353,30 @@ RTDECL(int) RTDbgAsSymbolByAddrA(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffD
     PRTDBGASINT pDbgAs = hDbgAs;
     RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
 
-    RTDBGSEGIDX iSeg;
-    RTUINTPTR   offSeg;
-    RTUINTPTR   MapAddr;
-    RTDBGMOD    hMod = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
+    RTDBGSEGIDX iSeg    = NIL_RTDBGSEGIDX;
+    RTUINTPTR   offSeg  = 0;
+    RTUINTPTR   MapAddr = 0;
+    RTDBGMOD    hMod    = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
     if (hMod == NIL_RTDBGMOD)
+    {
+        if (phMod)
+            *phMod = NIL_RTDBGMOD;
         return VERR_NOT_FOUND;
+    }
 
     /*
      * Forward the call.
      */
-    int rc = RTDbgModSymbolByAddrA(hMod, iSeg, offSeg, poffDisp, ppSymbol);
+    int rc = RTDbgModSymbolByAddrA(hMod, iSeg, offSeg, poffDisp, ppSymInfo);
     if (RT_SUCCESS(rc))
-        rtDbgAsAdjustSymbolValue(*ppSymbol, hMod, MapAddr, iSeg);
-    RTDbgModRelease(hMod);
+        rtDbgAsAdjustSymbolValue(*ppSymInfo, hMod, MapAddr, iSeg);
+    if (phMod)
+        *phMod = hMod;
+    else
+        RTDbgModRelease(hMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsSymbolByAddrA);
 
 
 /**
@@ -1264,19 +1394,19 @@ DECLINLINE(PRTDBGMOD) rtDbgAsSnapshotModuleTable(PRTDBGASINT pDbgAs, uint32_t *p
     RTDBGAS_LOCK_READ(pDbgAs);
 
     uint32_t iMod = *pcModules = pDbgAs->cModules;
-    PRTDBGMOD paModules = (PRTDBGMOD)RTMemTmpAlloc(sizeof(paModules[0]) * RT_MAX(iMod, 1));
-    if (paModules)
+    PRTDBGMOD pahModules = (PRTDBGMOD)RTMemTmpAlloc(sizeof(pahModules[0]) * RT_MAX(iMod, 1));
+    if (pahModules)
     {
         while (iMod-- > 0)
         {
-            RTDBGMOD hMod = paModules[iMod];
-            paModules[iMod] = hMod;
+            RTDBGMOD hMod = (RTDBGMOD)pDbgAs->papModules[iMod]->Core.Key;
+            pahModules[iMod] = hMod;
             RTDbgModRetain(hMod);
         }
     }
 
     RTDBGAS_UNLOCK_READ(pDbgAs);
-    return paModules;
+    return pahModules;
 }
 
 
@@ -1295,7 +1425,7 @@ static bool rtDbgAsFindMappingAndAdjustSymbolValue(PRTDBGASINT pDbgAs, RTDBGMOD
      * Absolute segments needs no fixing.
      */
     RTDBGSEGIDX const iSeg = pSymbol->iSeg;
-    if (iSeg)
+    if (iSeg == RTDBGSEGIDX_ABS)
         return true;
 
     RTDBGAS_LOCK_READ(pDbgAs);
@@ -1354,10 +1484,14 @@ static bool rtDbgAsFindMappingAndAdjustSymbolValue(PRTDBGASINT pDbgAs, RTDBGMOD
  * @retval  VERR_SYMBOL_NOT_FOUND if not found.
  *
  * @param   hDbgAs          The address space handle.
- * @param   pszSymbol       The symbol name.
+ * @param   pszSymbol       The symbol name. It is possible to limit the scope
+ *                          of the search by prefixing the symbol with a module
+ *                          name pattern followed by a bang (!) character.
+ *                          RTStrSimplePatternNMatch is used for the matching.
  * @param   pSymbol         Where to return the symbol info.
+ * @param   phMod           Where to return the module handle. Optional.
  */
-RTDECL(int) RTDbgAsSymbolByName(RTDBGAS hDbgAs, const char *pszSymbol, PRTDBGSYMBOL pSymbol)
+RTDECL(int) RTDbgAsSymbolByName(RTDBGAS hDbgAs, const char *pszSymbol, PRTDBGSYMBOL pSymbol, PRTDBGMOD phMod)
 {
     /*
      * Validate input.
@@ -1367,47 +1501,71 @@ RTDECL(int) RTDbgAsSymbolByName(RTDBGAS hDbgAs, const char *pszSymbol, PRTDBGSYM
     AssertPtrReturn(pszSymbol, VERR_INVALID_POINTER);
     AssertPtrReturn(pSymbol, VERR_INVALID_POINTER);
 
+    /*
+     * Look for module pattern.
+     */
+    const char *pachModPat = NULL;
+    size_t      cchModPat  = 0;
+    const char *pszBang    = strchr(pszSymbol, '!');
+    if (pszBang)
+    {
+        pachModPat = pszSymbol;
+        cchModPat = pszBang - pszSymbol;
+        pszSymbol = pszBang + 1;
+        if (!*pszSymbol)
+            return VERR_DBG_SYMBOL_NAME_OUT_OF_RANGE;
+        /* Note! Zero length module -> no pattern -> escape for symbol with '!'. */
+    }
+
     /*
      * Iterate the modules, looking for the symbol.
      */
     uint32_t cModules;
-    PRTDBGMOD paModules = rtDbgAsSnapshotModuleTable(pDbgAs, &cModules);
-    if (!paModules)
+    PRTDBGMOD pahModules = rtDbgAsSnapshotModuleTable(pDbgAs, &cModules);
+    if (!pahModules)
         return VERR_NO_TMP_MEMORY;
 
     for (uint32_t i = 0; i < cModules; i++)
     {
-        int rc = RTDbgModSymbolByName(paModules[i], pszSymbol, pSymbol);
-        if (RT_SUCCESS(rc))
+        if (    cchModPat == 0
+            ||  RTStrSimplePatternNMatch(pachModPat, cchModPat, RTDbgModName(pahModules[i]), RTSTR_MAX))
         {
-            if (rtDbgAsFindMappingAndAdjustSymbolValue(pDbgAs, paModules[i], pSymbol))
+            int rc = RTDbgModSymbolByName(pahModules[i], pszSymbol, pSymbol);
+            if (RT_SUCCESS(rc))
             {
-                for (; i < cModules; i++)
-                    RTDbgModRelease(paModules[i]);
-                RTMemTmpFree(paModules);
-                return rc;
+                if (rtDbgAsFindMappingAndAdjustSymbolValue(pDbgAs, pahModules[i], pSymbol))
+                {
+                    if (phMod)
+                        RTDbgModRetain(*phMod = pahModules[i]);
+                    for (; i < cModules; i++)
+                        RTDbgModRelease(pahModules[i]);
+                    RTMemTmpFree(pahModules);
+                    return rc;
+                }
             }
         }
-        RTDbgModRelease(paModules[i]);
+        RTDbgModRelease(pahModules[i]);
     }
 
-    RTMemTmpFree(paModules);
+    RTMemTmpFree(pahModules);
     return VERR_SYMBOL_NOT_FOUND;
 }
+RT_EXPORT_SYMBOL(RTDbgAsSymbolByName);
 
 
 /**
- * Query a symbol by name.
+ * Query a symbol by name, allocating the returned symbol structure.
  *
  * @returns IPRT status code.
  * @retval  VERR_SYMBOL_NOT_FOUND if not found.
  *
  * @param   hDbgAs          The address space handle.
- * @param   pszSymbol       The symbol name.
+ * @param   pszSymbol       The symbol name. See RTDbgAsSymbolByName for more.
  * @param   ppSymbol        Where to return the pointer to the allocated
  *                          symbol info. Always set. Free with RTDbgSymbolFree.
+ * @param   phMod           Where to return the module handle. Optional.
  */
-RTDECL(int) RTDbgAsSymbolByNameA(RTDBGAS hDbgAs, const char *pszSymbol, PRTDBGSYMBOL *ppSymbol)
+RTDECL(int) RTDbgAsSymbolByNameA(RTDBGAS hDbgAs, const char *pszSymbol, PRTDBGSYMBOL *ppSymbol, PRTDBGMOD phMod)
 {
     /*
      * Validate input.
@@ -1418,36 +1576,56 @@ RTDECL(int) RTDbgAsSymbolByNameA(RTDBGAS hDbgAs, const char *pszSymbol, PRTDBGSY
     RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
     AssertPtrReturn(pszSymbol, VERR_INVALID_POINTER);
 
+    /*
+     * Look for module pattern.
+     */
+    const char *pachModPat = NULL;
+    size_t      cchModPat  = 0;
+    const char *pszBang    = strchr(pszSymbol, '!');
+    if (pszBang)
+    {
+        pachModPat = pszSymbol;
+        cchModPat = pszBang - pszSymbol;
+        pszSymbol = pszBang + 1;
+        if (!*pszSymbol)
+            return VERR_DBG_SYMBOL_NAME_OUT_OF_RANGE;
+        /* Note! Zero length module -> no pattern -> escape for symbol with '!'. */
+    }
+
     /*
      * Iterate the modules, looking for the symbol.
      */
     uint32_t cModules;
-    PRTDBGMOD paModules = rtDbgAsSnapshotModuleTable(pDbgAs, &cModules);
-    if (!paModules)
+    PRTDBGMOD pahModules = rtDbgAsSnapshotModuleTable(pDbgAs, &cModules);
+    if (!pahModules)
         return VERR_NO_TMP_MEMORY;
 
     for (uint32_t i = 0; i < cModules; i++)
     {
-        int rc = RTDbgModSymbolByNameA(paModules[i], pszSymbol, ppSymbol);
-        if (RT_SUCCESS(rc))
+        if (    cchModPat == 0
+            ||  RTStrSimplePatternNMatch(pachModPat, cchModPat, RTDbgModName(pahModules[i]), RTSTR_MAX))
         {
-            if (rtDbgAsFindMappingAndAdjustSymbolValue(pDbgAs, paModules[i], *ppSymbol))
+            int rc = RTDbgModSymbolByNameA(pahModules[i], pszSymbol, ppSymbol);
+            if (RT_SUCCESS(rc))
             {
-                for (; i < cModules; i++)
-                    RTDbgModRelease(paModules[i]);
-                RTMemTmpFree(paModules);
-                return rc;
+                if (rtDbgAsFindMappingAndAdjustSymbolValue(pDbgAs, pahModules[i], *ppSymbol))
+                {
+                    if (phMod)
+                        RTDbgModRetain(*phMod = pahModules[i]);
+                    for (; i < cModules; i++)
+                        RTDbgModRelease(pahModules[i]);
+                    RTMemTmpFree(pahModules);
+                    return rc;
+                }
             }
-
-            RTDbgSymbolFree(*ppSymbol);
-            *ppSymbol = NULL;
         }
-        RTDbgModRelease(paModules[i]);
+        RTDbgModRelease(pahModules[i]);
     }
 
-    RTMemTmpFree(paModules);
+    RTMemTmpFree(pahModules);
     return VERR_SYMBOL_NOT_FOUND;
 }
+RT_EXPORT_SYMBOL(RTDbgAsSymbolByNameA);
 
 
 /**
@@ -1475,9 +1653,9 @@ RTDECL(int) RTDbgAsLineAdd(RTDBGAS hDbgAs, const char *pszFile, uint32_t uLineNo
     PRTDBGASINT pDbgAs = hDbgAs;
     RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
 
-    RTDBGSEGIDX iSeg;
-    RTUINTPTR offSeg;
-    RTDBGMOD hMod = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, NULL);
+    RTDBGSEGIDX iSeg   = NIL_RTDBGSEGIDX; /* shut up gcc */
+    RTUINTPTR   offSeg = 0;               /* ditto */
+    RTDBGMOD    hMod   = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, NULL);
     if (hMod == NIL_RTDBGMOD)
         return VERR_NOT_FOUND;
 
@@ -1488,6 +1666,7 @@ RTDECL(int) RTDbgAsLineAdd(RTDBGAS hDbgAs, const char *pszFile, uint32_t uLineNo
     RTDbgModRelease(hMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsLineAdd);
 
 
 /**
@@ -1511,10 +1690,10 @@ RTDECL(int) RTDbgAsLineByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDisp
     PRTDBGASINT pDbgAs = hDbgAs;
     RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
 
-    RTDBGSEGIDX iSeg;
-    RTUINTPTR   offSeg;
-    RTUINTPTR   MapAddr;
-    RTDBGMOD    hMod = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
+    RTDBGSEGIDX iSeg    = NIL_RTDBGSEGIDX; /* shut up gcc */
+    RTUINTPTR   offSeg  = 0;
+    RTUINTPTR   MapAddr = 0;
+    RTDBGMOD    hMod    = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
     if (hMod == NIL_RTDBGMOD)
         return VERR_NOT_FOUND;
 
@@ -1527,6 +1706,7 @@ RTDECL(int) RTDbgAsLineByAddr(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDisp
     RTDbgModRelease(hMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsLineByAddr);
 
 
 /**
@@ -1551,10 +1731,10 @@ RTDECL(int) RTDbgAsLineByAddrA(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDis
     PRTDBGASINT pDbgAs = hDbgAs;
     RTDBGAS_VALID_RETURN_RC(pDbgAs, VERR_INVALID_HANDLE);
 
-    RTDBGSEGIDX iSeg;
-    RTUINTPTR   offSeg;
-    RTUINTPTR   MapAddr;
-    RTDBGMOD    hMod = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
+    RTDBGSEGIDX iSeg    = NIL_RTDBGSEGIDX; /* shut up gcc */
+    RTUINTPTR   offSeg  = 0;
+    RTUINTPTR   MapAddr = 0;
+    RTDBGMOD    hMod    = rtDbgAsModuleByAddr(pDbgAs, Addr, &iSeg, &offSeg, &MapAddr);
     if (hMod == NIL_RTDBGMOD)
         return VERR_NOT_FOUND;
 
@@ -1567,4 +1747,5 @@ RTDECL(int) RTDbgAsLineByAddrA(RTDBGAS hDbgAs, RTUINTPTR Addr, PRTINTPTR poffDis
     RTDbgModRelease(hMod);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTDbgAsLineByAddrA);
 
