@@ -1,10 +1,10 @@
-/* $Id: ldrELFRelocatable.cpp.h $ */
+/* $Id: ldrELFRelocatable.cpp.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Binary Image Loader, Template for ELF Relocatable Images.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,12 +22,9 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
@@ -74,11 +71,8 @@
 #define Elf_Sword           RTLDRELF_MID(Elf,_Sword)
 #define Elf_Word            RTLDRELF_MID(Elf,_Word)
 
-/* gcc-4.4 warning: ''empty macro arguments are undefined in ISO C90 and ISO C++98'' */
-#define NO_SUFFIX
-#define RTLDRMODELF         RTLDRELF_MID(RTLDRMODELF,NO_SUFFIX)
-#define PRTLDRMODELF        RTLDRELF_MID(PRTLDRMODELF,NO_SUFFIX)
-#undef NO_SUFFIX
+#define RTLDRMODELF         RTLDRELF_MID(RTLDRMODELF,RT_NOTHING)
+#define PRTLDRMODELF        RTLDRELF_MID(PRTLDRMODELF,RT_NOTHING)
 
 #define ELF_R_SYM(info)     RTLDRELF_MID(ELF,_R_SYM)(info)
 #define ELF_R_TYPE(info)    RTLDRELF_MID(ELF,_R_TYPE)(info)
@@ -490,7 +484,7 @@ static DECLCALLBACK(int) RTLDRELF_NAME(EnumSymbols)(PRTLDRMODINTERNAL pMod, unsi
                  * Call back.
                  */
                 AssertMsgReturn(Value == (RTUINTPTR)Value, (FMT_ELF_ADDR "\n", Value), VERR_SYMBOL_VALUE_TOO_BIG);
-                int rc = pfnCallback(pMod, pszName, ~0, (RTUINTPTR)Value, pvUser);
+                rc = pfnCallback(pMod, pszName, ~0, (RTUINTPTR)Value, pvUser);
                 if (rc)
                     return rc;
             }
@@ -651,10 +645,10 @@ static DECLCALLBACK(int) RTLDRELF_NAME(GetSymbolEx)(PRTLDRMODINTERNAL pMod, cons
     for (unsigned iSym = 1; iSym < cSyms; iSym++)
     {
         /* Undefined symbols are not exports, they are imports. */
-	if (    paSyms[iSym].st_shndx != SHN_UNDEF
+        if (    paSyms[iSym].st_shndx != SHN_UNDEF
             &&  (   ELF_ST_BIND(paSyms[iSym].st_info) == STB_GLOBAL
                  || ELF_ST_BIND(paSyms[iSym].st_info) == STB_WEAK))
-	{
+        {
             /* Validate the name string and try match with it. */
             if (paSyms[iSym].st_name < pModElf->cbStr)
             {
@@ -683,7 +677,7 @@ static DECLCALLBACK(int) RTLDRELF_NAME(GetSymbolEx)(PRTLDRMODINTERNAL pMod, cons
                 AssertMsgFailed(("String outside string table! iSym=%d paSyms[iSym].st_name=%#x\n", iSym, paSyms[iSym].st_name));
                 return VERR_LDRELF_INVALID_SYMBOL_NAME_OFFSET;
             }
-	}
+        }
     }
 
     return VERR_SYMBOL_NOT_FOUND;
@@ -874,7 +868,7 @@ const char *RTLDRELF_NAME(GetSHdrName)(PRTLDRMODELF pModElf, Elf_Word offName, c
         /* read by for byte. */
         for (unsigned i = 0; i < cbName; i++, off++)
         {
-            int rc = pModElf->pReader->pfnRead(pModElf->pReader, pszName + i, 1, off);
+            rc = pModElf->pReader->pfnRead(pModElf->pReader, pszName + i, 1, off);
             if (RT_FAILURE(rc))
             {
                 pszName[i] = '\0';
@@ -1027,7 +1021,7 @@ static int RTLDRELF_NAME(Open)(PRTLDRREADER pReader, uint32_t fFlags, RTLDRARCH
     int rc = pReader->pfnRead(pReader, &pModElf->Ehdr, sizeof(pModElf->Ehdr), 0);
     if (RT_SUCCESS(rc))
     {
-        RTLDRARCH enmArchImage;
+        RTLDRARCH enmArchImage = RTLDRARCH_INVALID; /* shut up gcc */
         rc = RTLDRELF_NAME(ValidateElfHeader)(&pModElf->Ehdr, pszLogName, cbRawImage, &enmArchImage);
         if (RT_SUCCESS(rc))
         {
