@@ -1,10 +1,10 @@
-/* $Id: ldrEx.cpp $ */
+/* $Id: ldrEx.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Binary Image Loader, Extended Features.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -34,6 +30,8 @@
 *******************************************************************************/
 #define LOG_GROUP RTLOGGROUP_LDR
 #include <iprt/ldr.h>
+#include "internal/iprt.h"
+
 #include <iprt/alloc.h>
 #include <iprt/assert.h>
 #include <iprt/log.h>
@@ -194,6 +192,7 @@ RTDECL(size_t) RTLdrSize(RTLDRMOD hLdrMod)
     LogFlow(("RTLdrSize: returns %zu\n", cb));
     return cb;
 }
+RT_EXPORT_SYMBOL(RTLdrSize);
 
 
 /**
@@ -230,6 +229,7 @@ RTDECL(int) RTLdrGetBits(RTLDRMOD hLdrMod, void *pvBits, RTUINTPTR BaseAddress,
     LogFlow(("RTLdrGetBits: returns %Rrc\n",rc));
     return rc;
 }
+RT_EXPORT_SYMBOL(RTLdrGetBits);
 
 
 /**
@@ -268,6 +268,7 @@ RTDECL(int) RTLdrRelocate(RTLDRMOD hLdrMod, void *pvBits, RTUINTPTR NewBaseAddre
     LogFlow(("RTLdrRelocate: returns %Rrc\n", rc));
     return rc;
 }
+RT_EXPORT_SYMBOL(RTLdrRelocate);
 
 
 /**
@@ -320,6 +321,7 @@ RTDECL(int) RTLdrGetSymbolEx(RTLDRMOD hLdrMod, const void *pvBits, RTUINTPTR Bas
     LogFlow(("RTLdrGetSymbolEx: returns %Rrc *pValue=%p\n", rc, *pValue));
     return rc;
 }
+RT_EXPORT_SYMBOL(RTLdrGetSymbolEx);
 
 
 /**
@@ -356,4 +358,5 @@ RTDECL(int) RTLdrEnumSymbols(RTLDRMOD hLdrMod, unsigned fFlags, const void *pvBi
     LogFlow(("RTLdrEnumSymbols: returns %Rrc\n", rc));
     return rc;
 }
+RT_EXPORT_SYMBOL(RTLdrEnumSymbols);
 
