@@ -1,10 +1,10 @@
-/* $Id: cidr.cpp $ */
+/* $Id: cidr.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - IPv4 address parsing.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,16 +22,15 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/cidr.h>
+#include "internal/iprt.h"
+
 #include <iprt/ctype.h>
 #include <iprt/string.h>
 #include <iprt/stream.h>
@@ -72,7 +71,7 @@ static int scanIPv4Digit(int iDigit, const char *psz, char **ppszNext, uint8_t *
 }
 
 
-int RTCidrStrToIPv4(const char *pszAddress, PRTIPV4ADDR pNetwork, PRTIPV4ADDR pNetmask)
+RTDECL(int) RTCidrStrToIPv4(const char *pszAddress, PRTIPV4ADDR pNetwork, PRTIPV4ADDR pNetmask)
 {
     uint8_t cBits;
     uint8_t a;
@@ -134,3 +133,5 @@ int RTCidrStrToIPv4(const char *pszAddress, PRTIPV4ADDR pNetwork, PRTIPV4ADDR pN
     *pNetmask = ~(((uint32_t)1 << (32 - cBits)) - 1);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTCidrStrToIPv4);
+
