@@ -1,10 +1,10 @@
-/* $Id: once.cpp $ */
+/* $Id: once.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Execute Once.
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,16 +22,15 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/once.h>
+#include "internal/iprt.h"
+
 #include <iprt/semaphore.h>
 #include <iprt/thread.h>
 #include <iprt/err.h>
@@ -116,7 +115,7 @@ RTDECL(int) RTOnce(PRTONCE pOnce, PFNRTONCE pfnOnce, void *pvUser1, void *pvUser
                  * the init code, if it isn't valid just do the yield/sleep thing.
                  */
                 RTSEMEVENTMULTI hEventMulti;
-                ASMAtomicUoReadSize(&pOnce->hEventMulti, &hEventMulti);
+                ASMAtomicUoReadHandle(&pOnce->hEventMulti, &hEventMulti);
                 if (hEventMulti != NIL_RTSEMEVENTMULTI)
                 {
                     fYieldSleep = false;
@@ -158,6 +157,7 @@ RTDECL(int) RTOnce(PRTONCE pOnce, PFNRTONCE pfnOnce, void *pvUser1, void *pvUser
      */
     return ASMAtomicUoReadS32(&pOnce->rc);
 }
+RT_EXPORT_SYMBOL(RTOnce);
 
 
 RTDECL(void) RTOnceReset(PRTONCE pOnce)
@@ -171,4 +171,5 @@ RTDECL(void) RTOnceReset(PRTONCE pOnce)
     ASMAtomicWriteS32(&pOnce->rc, VERR_INTERNAL_ERROR);
     ASMAtomicWriteS32(&pOnce->iState, -1);
 }
+RT_EXPORT_SYMBOL(RTOnceReset);
 
