@@ -1,10 +1,10 @@
-/* $Id: VMReq.cpp 17451 2007-01-15 14:08:28Z bird $ */
+/* $Id: req.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Request packets
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,16 +22,14 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
+#include <iprt/req.h>
+#include "internal/iprt.h"
 
 #include <iprt/assert.h>
 #include <iprt/asm.h>
@@ -39,7 +37,6 @@
 #include <iprt/time.h>
 #include <iprt/semaphore.h>
 #include <iprt/thread.h>
-#include <iprt/req.h>
 #include <iprt/log.h>
 #include <iprt/mem.h>
 
@@ -59,16 +56,20 @@ static int  rtReqProcessOne(PRTREQ pReq);
  */
 RTDECL(int) RTReqCreateQueue(PRTREQQUEUE *ppQueue)
 {
-    *ppQueue = (PRTREQQUEUE)RTMemAllocZ(sizeof(RTREQQUEUE));
-    if (!ppQueue)
+    PRTREQQUEUE pQueue = (PRTREQQUEUE)RTMemAllocZ(sizeof(RTREQQUEUE));
+    if (!pQueue)
         return VERR_NO_MEMORY;
+    int rc = RTSemEventCreate(&pQueue->EventSem);
+    if (RT_SUCCESS(rc))
+    {
+        *ppQueue = pQueue;
+        return VINF_SUCCESS;
+    }
 
-    int rc = RTSemEventCreate(&(*ppQueue)->EventSem);
-    if (rc != VINF_SUCCESS)
-        RTMemFree(*ppQueue);
-
+    RTMemFree(pQueue);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqCreateQueue);
 
 
 /**
@@ -83,14 +84,15 @@ RTDECL(int) RTReqDestroyQueue(PRTREQQUEUE pQueue)
      * Check input.
      */
     if (!pQueue)
-    {
-        AssertFailed();
-        return VERR_INVALID_PARAMETER;
-    }
+        return VINF_SUCCESS;
+    AssertPtr(pQueue);
     RTSemEventDestroy(pQueue->EventSem);
+    pQueue->EventSem = NIL_RTSEMEVENT;
     RTMemFree(pQueue);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTReqDestroyQueue);
+
 
 /**
  * Process one or more request packets
@@ -102,7 +104,7 @@ RTDECL(int) RTReqDestroyQueue(PRTREQQUEUE pQueue)
  * @param   cMillies        Number of milliseconds to wait for a pending request.
  *                          Use RT_INDEFINITE_WAIT to only wait till one is added.
  */
-RTDECL(int) RTReqProcess(PRTREQQUEUE pQueue, unsigned cMillies)
+RTDECL(int) RTReqProcess(PRTREQQUEUE pQueue, RTMSINTERVAL cMillies)
 {
     LogFlow(("RTReqProcess %x\n", pQueue));
     /*
@@ -129,12 +131,15 @@ RTDECL(int) RTReqProcess(PRTREQQUEUE pQueue, unsigned cMillies)
         PRTREQ pReqs = (PRTREQ)ASMAtomicXchgPtr((void * volatile *)&pQueue->pReqs, NULL);
         if (!pReqs)
         {
-            /** @note We currently don't care if the entire time wasted here is larger than cMillies */
+            ASMAtomicWriteBool(&pQueue->fBusy, false); /* this aint 100% perfect, but it's good enough for now... */
+            /** @todo We currently don't care if the entire time wasted here is larger than
+             *        cMillies */
             rc = RTSemEventWait(pQueue->EventSem, cMillies);
             if (rc != VINF_SUCCESS)
                 break;
             continue;
         }
+        ASMAtomicWriteBool(&pQueue->fBusy, true);
 
         /*
          * Reverse the list to process it in FIFO order.
@@ -175,6 +180,7 @@ RTDECL(int) RTReqProcess(PRTREQQUEUE pQueue, unsigned cMillies)
     LogFlow(("RTReqProcess: returns %Rrc\n", rc));
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqProcess);
 
 /**
  * Allocate and queue a call request.
@@ -196,10 +202,11 @@ RTDECL(int) RTReqProcess(PRTREQQUEUE pQueue, unsigned cMillies)
  *                          wait till it's completed.
  * @param   pfnFunction     Pointer to the function to call.
  * @param   cArgs           Number of arguments following in the ellipsis.
- *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
+ *
+ * @remarks See remarks on RTReqCallV.
  */
-RTDECL(int) RTReqCall(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
+RTDECL(int) RTReqCall(PRTREQQUEUE pQueue, PRTREQ *ppReq, RTMSINTERVAL cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
@@ -207,6 +214,7 @@ RTDECL(int) RTReqCall(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, PFNR
     va_end(va);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqCall);
 
 
 /**
@@ -229,10 +237,11 @@ RTDECL(int) RTReqCall(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, PFNR
  *                          wait till it's completed.
  * @param   pfnFunction     Pointer to the function to call.
  * @param   cArgs           Number of arguments following in the ellipsis.
- *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
+ *
+ * @remarks See remarks on RTReqCallV.
  */
-RTDECL(int) RTReqCallVoid(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
+RTDECL(int) RTReqCallVoid(PRTREQQUEUE pQueue, PRTREQ *ppReq, RTMSINTERVAL cMillies, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
@@ -240,6 +249,7 @@ RTDECL(int) RTReqCallVoid(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies,
     va_end(va);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqCallVoid);
 
 
 /**
@@ -264,10 +274,11 @@ RTDECL(int) RTReqCallVoid(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies,
  * @param   fFlags          A combination of the RTREQFLAGS values.
  * @param   pfnFunction     Pointer to the function to call.
  * @param   cArgs           Number of arguments following in the ellipsis.
- *                          Not possible to pass 64-bit arguments!
  * @param   ...             Function arguments.
+ *
+ * @remarks See remarks on RTReqCallV.
  */
-RTDECL(int) RTReqCallEx(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...)
+RTDECL(int) RTReqCallEx(PRTREQQUEUE pQueue, PRTREQ *ppReq, RTMSINTERVAL cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, ...)
 {
     va_list va;
     va_start(va, cArgs);
@@ -275,6 +286,7 @@ RTDECL(int) RTReqCallEx(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, un
     va_end(va);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqCallEx);
 
 
 /**
@@ -299,10 +311,18 @@ RTDECL(int) RTReqCallEx(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, un
  * @param   fFlags          A combination of the RTREQFLAGS values.
  * @param   pfnFunction     Pointer to the function to call.
  * @param   cArgs           Number of arguments following in the ellipsis.
- *                          Not possible to pass 64-bit arguments!
  * @param   Args            Variable argument vector.
+ *
+ * @remarks Caveats:
+ *              - Do not pass anything which is larger than an uintptr_t.
+ *              - 64-bit integers are larger than uintptr_t on 32-bit hosts.
+ *                Pass integers > 32-bit by reference (pointers).
+ *              - Don't use NULL since it should be the integer 0 in C++ and may
+ *                therefore end up with garbage in the bits 63:32 on 64-bit
+ *                hosts because 'int' is 32-bit.
+ *                Use (void *)NULL or (uintptr_t)0 instead of NULL.
  */
-RTDECL(int) RTReqCallV(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, va_list Args)
+RTDECL(int) RTReqCallV(PRTREQQUEUE pQueue, PRTREQ *ppReq, RTMSINTERVAL cMillies, unsigned fFlags, PFNRT pfnFunction, unsigned cArgs, va_list Args)
 {
     LogFlow(("RTReqCallV: cMillies=%d fFlags=%#x pfnFunction=%p cArgs=%d\n", cMillies, fFlags, pfnFunction, cArgs));
 
@@ -362,6 +382,7 @@ RTDECL(int) RTReqCallV(PRTREQQUEUE pQueue, PRTREQ *ppReq, unsigned cMillies, uns
     Assert(rc != VERR_INTERRUPTED);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqCallV);
 
 
 /**
@@ -431,6 +452,7 @@ static void vmr3ReqJoinFree(PRTREQQUEUE pQueue, PRTREQ pList)
  */
 RTDECL(int) RTReqAlloc(PRTREQQUEUE pQueue, PRTREQ *ppReq, RTREQTYPE enmType)
 {
+    RT_EXPORT_SYMBOL(RTReqAlloc);
     /*
      * Validate input.
      */
@@ -608,6 +630,7 @@ RTDECL(int) RTReqFree(PRTREQ pReq)
     }
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTReqFree);
 
 
 /**
@@ -628,7 +651,7 @@ RTDECL(int) RTReqFree(PRTREQ pReq)
  *                          be completed. Use RT_INDEFINITE_WAIT to only
  *                          wait till it's completed.
  */
-RTDECL(int) RTReqQueue(PRTREQ pReq, unsigned cMillies)
+RTDECL(int) RTReqQueue(PRTREQ pReq, RTMSINTERVAL cMillies)
 {
     LogFlow(("RTReqQueue: pReq=%p cMillies=%d\n", pReq, cMillies));
     /*
@@ -666,6 +689,7 @@ RTDECL(int) RTReqQueue(PRTREQ pReq, unsigned cMillies)
     {
         pNext = pQueue->pReqs;
         pReq->pNext = pNext;
+        ASMAtomicWriteBool(&pQueue->fBusy, true);
     } while (!ASMAtomicCmpXchgPtr((void * volatile *)&pQueue->pReqs, (void *)pReq, (void *)pNext));
 
     /*
@@ -681,6 +705,7 @@ RTDECL(int) RTReqQueue(PRTREQ pReq, unsigned cMillies)
     LogFlow(("RTReqQueue: returns %Rrc\n", rc));
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqQueue);
 
 
 /**
@@ -694,7 +719,7 @@ RTDECL(int) RTReqQueue(PRTREQ pReq, unsigned cMillies)
  * @param   cMillies        Number of milliseconds to wait.
  *                          Use RT_INDEFINITE_WAIT to only wait till it's completed.
  */
-RTDECL(int) RTReqWait(PRTREQ pReq, unsigned cMillies)
+RTDECL(int) RTReqWait(PRTREQ pReq, RTMSINTERVAL cMillies)
 {
     LogFlow(("RTReqWait: pReq=%p cMillies=%d\n", pReq, cMillies));
 
@@ -744,6 +769,7 @@ RTDECL(int) RTReqWait(PRTREQ pReq, unsigned cMillies)
     Assert(rc != VERR_INTERRUPTED);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTReqWait);
 
 
 /**
@@ -790,7 +816,7 @@ static int  rtReqProcessOne(PRTREQ pReq)
                 DECLCALLBACKMEMBER(int, pfn12)(uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t, uintptr_t);
             } u;
             u.pfn = pReq->u.Internal.pfn;
-#ifdef RT_ARCH_AMD64
+#ifndef RT_ARCH_X86
             switch (pReq->u.Internal.cArgs)
             {
                 case 0:  rcRet = u.pfn00(); break;
@@ -811,7 +837,7 @@ static int  rtReqProcessOne(PRTREQ pReq)
                     rcRet = rcReq = VERR_INTERNAL_ERROR;
                     break;
             }
-#else /* x86: */
+#else /* RT_ARCH_X86 */
             size_t cbArgs = pReq->u.Internal.cArgs * sizeof(uintptr_t);
 # ifdef __GNUC__
             __asm__ __volatile__("movl  %%esp, %%edx\n\t"
@@ -848,7 +874,7 @@ static int  rtReqProcessOne(PRTREQ pReq)
                 mov     rcRet, eax
             }
 # endif
-#endif /* x86 */
+#endif /* RT_ARCH_X86 */
             if ((pReq->fFlags & (RTREQFLAGS_RETURN_MASK)) == RTREQFLAGS_VOID)
                 rcRet = VINF_SUCCESS;
             rcReq = rcRet;
@@ -890,5 +916,24 @@ static int  rtReqProcessOne(PRTREQ pReq)
 }
 
 
+/**
+ * Checks if the queue is busy or not.
+ *
+ * The caller is responsible for dealing with any concurrent submitts.
+ *
+ * @returns true if busy, false if idle.
+ * @param   pQueue              The queue.
+ */
+RTDECL(bool) RTReqIsBusy(PRTREQQUEUE pQueue)
+{
+    AssertPtrReturn(pQueue, false);
 
-
+    if (ASMAtomicReadBool(&pQueue->fBusy))
+        return true;
+    if (ASMAtomicReadPtr((void * volatile *)&pQueue->pReqs) != NULL)
+        return true;
+    if (ASMAtomicReadBool(&pQueue->fBusy))
+        return true;
+    return false;
+}
+RT_EXPORT_SYMBOL(RTReqIsBusy);
