@@ -1,10 +1,10 @@
-/* $Id: sanity.h $ */
+/* $Id: sanity.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Setup Sanity Checks, C and C++.
  */
 
 /*
- * Copyright (C) 2007 Sun Microsystems, Inc.
+ * Copyright (C) 2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <iprt/cdefs.h>
@@ -204,3 +200,12 @@ TEST_CONST_MACRO(INT32_C(-1), int32_t);
 TEST_CONST_MACRO(INT64_C(1),  int64_t);
 TEST_CONST_MACRO(INT64_C(-1), int64_t);
 
+
+/*
+ * Our union types.
+ */
+AssertCompileSize(RTUINT16U,  2);
+AssertCompileSize(RTUINT32U,  4);
+AssertCompileSize(RTUINT64U,  8);
+AssertCompileSize(RTUINT128U, 16);
+
