@@ -1,10 +1,10 @@
-/* $Id: thread.cpp $ */
+/* $Id: thread.cpp 28903 2010-04-29 14:58:12Z vboxsync $ */
 /** @file
  * IPRT - Threads, common routines.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,23 +22,21 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
-
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP RTLOGGROUP_THREAD
 #include <iprt/thread.h>
+#include "internal/iprt.h"
+
 #include <iprt/log.h>
 #include <iprt/avl.h>
 #include <iprt/alloc.h>
 #include <iprt/assert.h>
+#include <iprt/lockvalidator.h>
 #include <iprt/semaphore.h>
 #ifdef IN_RING0
 # include <iprt/spinlock.h>
@@ -46,9 +44,13 @@
 #include <iprt/asm.h>
 #include <iprt/err.h>
 #include <iprt/string.h>
+#include "internal/magics.h"
 #include "internal/thread.h"
 #include "internal/sched.h"
 #include "internal/process.h"
+#ifdef RT_WITH_ICONV_CACHE
+# include "internal/string.h"
+#endif
 
 
 /*******************************************************************************
@@ -87,9 +89,9 @@ static RTSPINLOCK       g_ThreadSpinlock = NIL_RTSPINLOCK;
 *   Internal Functions                                                         *
 *******************************************************************************/
 static void rtThreadDestroy(PRTTHREADINT pThread);
-static int rtThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *pszName);
+static int rtThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, uint32_t fIntFlags, const char *pszName);
 static void rtThreadRemoveLocked(PRTTHREADINT pThread);
-static PRTTHREADINT rtThreadAlloc(RTTHREADTYPE enmType, unsigned fFlags, unsigned fIntFlags, const char *pszName);
+static PRTTHREADINT rtThreadAlloc(RTTHREADTYPE enmType, unsigned fFlags, uint32_t fIntFlags, const char *pszName);
 
 
 /** @page pg_rt_thread  IPRT Thread Internals
@@ -134,12 +136,12 @@ int rtThreadInit(void)
          * We assume the caller is the 1st thread, which we'll call 'main'.
          * But first, we'll create the semaphore.
          */
-        int rc = RTSemRWCreate(&g_ThreadRWSem);
+        rc = RTSemRWCreateEx(&g_ThreadRWSem, RTSEMRW_FLAGS_NO_LOCK_VAL, NIL_RTLOCKVALCLASS, RTLOCKVAL_SUB_CLASS_NONE, NULL);
         if (RT_SUCCESS(rc))
         {
             rc = rtThreadNativeInit();
             if (RT_SUCCESS(rc))
-                rc = rtThreadAdopt(RTTHREADTYPE_DEFAULT, 0, "main");
+                rc = rtThreadAdopt(RTTHREADTYPE_DEFAULT, 0, RTTHREADINT_FLAGS_MAIN, "main");
             if (RT_SUCCESS(rc))
                 rc = rtSchedNativeCalcDefaultPriority(RTTHREADTYPE_DEFAULT);
             if (RT_SUCCESS(rc))
@@ -188,12 +190,11 @@ void rtThreadTerm(void)
     RTSpinlockDestroy(g_ThreadSpinlock);
     g_ThreadSpinlock = NIL_RTSPINLOCK;
     if (g_ThreadTree != NULL)
-        AssertMsg2("WARNING: g_ThreadTree=%p\n", g_ThreadTree);
+        RTAssertMsg2Weak("WARNING: g_ThreadTree=%p\n", g_ThreadTree);
 #endif
 }
 
 
-
 #ifdef IN_RING3
 
 DECLINLINE(void) rtThreadLockRW(void)
@@ -234,7 +235,7 @@ DECLINLINE(void) rtThreadUnLockRD(void)
  * Adopts the calling thread.
  * No locks are taken or released by this function.
  */
-static int rtThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *pszName)
+static int rtThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, uint32_t fIntFlags, const char *pszName)
 {
     Assert(!(fFlags & RTTHREADFLAGS_WAITABLE));
     fFlags &= ~RTTHREADFLAGS_WAITABLE;
@@ -245,7 +246,7 @@ static int rtThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *pszN
      * we try inserting the thread because of locking.)
      */
     int rc = VERR_NO_MEMORY;
-    PRTTHREADINT pThread = rtThreadAlloc(enmType, fFlags, RTTHREADINT_FLAGS_ALIEN, pszName);
+    PRTTHREADINT pThread = rtThreadAlloc(enmType, fFlags, RTTHREADINT_FLAGS_ALIEN | fIntFlags, pszName);
     if (pThread)
     {
         RTNATIVETHREAD NativeThread = RTThreadNativeSelf();
@@ -253,7 +254,7 @@ static int rtThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *pszN
         if (RT_SUCCESS(rc))
         {
             rtThreadInsert(pThread, NativeThread);
-            ASMAtomicWriteSize(&pThread->enmState, RTTHREADSTATE_RUNNING);
+            rtThreadSetState(pThread, RTTHREADSTATE_RUNNING);
             rtThreadRelease(pThread);
         }
     }
@@ -291,7 +292,7 @@ RTDECL(int) RTThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *psz
         }
 
         /* try adopt it */
-        rc = rtThreadAdopt(enmType, fFlags, pszName);
+        rc = rtThreadAdopt(enmType, fFlags, 0, pszName);
         Thread = RTThreadSelf();
         Log(("RTThreadAdopt: %RTthrd %RTnthrd '%s' enmType=%d fFlags=%#x rc=%Rrc\n",
              Thread, RTThreadNativeSelf(), pszName, enmType, fFlags, rc));
@@ -304,6 +305,23 @@ RTDECL(int) RTThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *psz
         *pThread = Thread;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadAdopt);
+
+
+/**
+ * Get the thread handle of the current thread, automatically adopting alien
+ * threads.
+ *
+ * @returns Thread handle.
+ */
+RTDECL(RTTHREAD) RTThreadSelfAutoAdopt(void)
+{
+    RTTHREAD hSelf = RTThreadSelf();
+    if (RT_UNLIKELY(hSelf == NIL_RTTHREAD))
+        RTThreadAdopt(RTTHREADTYPE_DEFAULT, 0, NULL, &hSelf);
+    return hSelf;
+}
+RT_EXPORT_SYMBOL(RTThreadSelfAutoAdopt);
 
 
 /**
@@ -317,7 +335,7 @@ RTDECL(int) RTThreadAdopt(RTTHREADTYPE enmType, unsigned fFlags, const char *psz
  * @param   fIntFlags   The internal thread flags.
  * @param   pszName     Pointer to the thread name.
  */
-PRTTHREADINT rtThreadAlloc(RTTHREADTYPE enmType, unsigned fFlags, unsigned fIntFlags, const char *pszName)
+PRTTHREADINT rtThreadAlloc(RTTHREADTYPE enmType, unsigned fFlags, uint32_t fIntFlags, const char *pszName)
 {
     PRTTHREADINT pThread = (PRTTHREADINT)RTMemAllocZ(sizeof(RTTHREADINT));
     if (pThread)
@@ -329,12 +347,19 @@ PRTTHREADINT rtThreadAlloc(RTTHREADTYPE enmType, unsigned fFlags, unsigned fIntF
             cchName = RTTHREAD_NAME_LEN - 1;
         memcpy(pThread->szName, pszName, cchName);
         pThread->szName[cchName] = '\0';
-        pThread->cRefs      = 2 + !!(fFlags & RTTHREADFLAGS_WAITABLE); /* And extra reference if waitable. */
-        pThread->rc         = VERR_PROCESS_RUNNING; /** @todo get a better error code! */
-        pThread->enmType    = enmType;
-        pThread->fFlags     = fFlags;
-        pThread->fIntFlags  = fIntFlags;
-        pThread->enmState   = RTTHREADSTATE_INITIALIZING;
+        pThread->cRefs           = 2 + !!(fFlags & RTTHREADFLAGS_WAITABLE); /* And extra reference if waitable. */
+        pThread->rc              = VERR_PROCESS_RUNNING; /** @todo get a better error code! */
+        pThread->enmType         = enmType;
+        pThread->fFlags          = fFlags;
+        pThread->fIntFlags       = fIntFlags;
+        pThread->enmState        = RTTHREADSTATE_INITIALIZING;
+        pThread->fReallySleeping = false;
+#ifdef IN_RING3
+        rtLockValidatorInitPerThread(&pThread->LockValidator);
+#endif
+#ifdef RT_WITH_ICONV_CACHE
+        rtStrIconvCacheInit(pThread);
+#endif
         int rc = RTSemEventMultiCreate(&pThread->EventUser);
         if (RT_SUCCESS(rc))
         {
@@ -373,7 +398,7 @@ void rtThreadInsert(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread)
      * gets this far. Since the OS may quickly reuse the native thread ID
      * it should not be reinserted at this point.
      */
-    if (pThread->enmState != RTTHREADSTATE_TERMINATED)
+    if (rtThreadGetState(pThread) != RTTHREADSTATE_TERMINATED)
     {
         /*
          * Before inserting we must check if there is a thread with this id
@@ -500,7 +525,6 @@ PRTTHREADINT rtThreadGet(RTTHREAD Thread)
     return NULL;
 }
 
-
 /**
  * Release a per thread data structure.
  *
@@ -518,7 +542,10 @@ uint32_t rtThreadRelease(PRTTHREADINT pThread)
             rtThreadDestroy(pThread);
     }
     else
+    {
         cRefs = 0;
+        AssertFailed();
+    }
     return cRefs;
 }
 
@@ -545,20 +572,37 @@ static void rtThreadDestroy(PRTTHREADINT pThread)
         rtThreadRemove(pThread);
         ASMAtomicDecU32(&pThread->cRefs);
     }
-    ASMAtomicXchgU32(&pThread->u32Magic, RTTHREADINT_MAGIC_DEAD);
 
     /*
-     * Free resources.
+     * Invalidate the thread structure.
      */
+#ifdef IN_RING3
+    rtLockValidatorSerializeDestructEnter();
+
+    rtLockValidatorDeletePerThread(&pThread->LockValidator);
+#endif
+#ifdef RT_WITH_ICONV_CACHE
+    rtStrIconvCacheDestroy(pThread);
+#endif
+    ASMAtomicXchgU32(&pThread->u32Magic, RTTHREADINT_MAGIC_DEAD);
     ASMAtomicWritePtr(&pThread->Core.Key, (void *)NIL_RTTHREAD);
-    pThread->enmType    = RTTHREADTYPE_INVALID;
-    RTSemEventMultiDestroy(pThread->EventUser);
-    pThread->EventUser  = NIL_RTSEMEVENTMULTI;
-    if (pThread->EventTerminated != NIL_RTSEMEVENTMULTI)
-    {
-        RTSemEventMultiDestroy(pThread->EventTerminated);
-        pThread->EventTerminated = NIL_RTSEMEVENTMULTI;
-    }
+    pThread->enmType         = RTTHREADTYPE_INVALID;
+    RTSEMEVENTMULTI hEvt1    = pThread->EventUser;
+    pThread->EventUser       = NIL_RTSEMEVENTMULTI;
+    RTSEMEVENTMULTI hEvt2    = pThread->EventTerminated;
+    pThread->EventTerminated = NIL_RTSEMEVENTMULTI;
+
+#ifdef IN_RING3
+    rtLockValidatorSerializeDestructLeave();
+#endif
+
+    /*
+     * Destroy semaphore resources and free the bugger.
+     */
+    RTSemEventMultiDestroy(hEvt1);
+    if (hEvt2 != NIL_RTSEMEVENTMULTI)
+        RTSemEventMultiDestroy(hEvt2);
+
     RTMemFree(pThread);
 }
 
@@ -585,7 +629,7 @@ void rtThreadTerminate(PRTTHREADINT pThread, int rc)
      * Set the rc, mark it terminated and signal anyone waiting.
      */
     pThread->rc = rc;
-    ASMAtomicWriteSize(&pThread->enmState, RTTHREADSTATE_TERMINATED);
+    rtThreadSetState(pThread, RTTHREADSTATE_TERMINATED);
     ASMAtomicOrU32(&pThread->fIntFlags, RTTHREADINT_FLAGS_TERMINATED);
     if (pThread->EventTerminated != NIL_RTSEMEVENTMULTI)
         RTSemEventMultiSignal(pThread->EventTerminated);
@@ -631,9 +675,19 @@ int rtThreadMain(PRTTHREADINT pThread, RTNATIVETHREAD NativeThread, const char *
     /*
      * Call thread function and terminate when it returns.
      */
-    ASMAtomicWriteSize(&pThread->enmState, RTTHREADSTATE_RUNNING);
+    rtThreadSetState(pThread, RTTHREADSTATE_RUNNING);
     rc = pThread->pfnThread(pThread, pThread->pvUser);
 
+    /*
+     * Paranoia checks for leftover resources.
+     */
+#ifdef RTSEMRW_STRICT
+    int32_t cWrite = ASMAtomicReadS32(&pThread->cWriteLocks);
+    Assert(!cWrite);
+    int32_t cRead = ASMAtomicReadS32(&pThread->cReadLocks);
+    Assert(!cRead);
+#endif
+
     Log(("rtThreadMain: Terminating: rc=%d pThread=%p NativeThread=%RTnthrd Name=%s pfnThread=%p pvUser=%p\n",
          rc, pThread, NativeThread, pThread->szName, pThread->pfnThread, pThread->pvUser));
     rtThreadTerminate(pThread, rc);
@@ -717,6 +771,7 @@ RTDECL(int) RTThreadCreate(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUse
     AssertReleaseRC(rc);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadCreate);
 
 
 /**
@@ -731,7 +786,7 @@ RTDECL(int) RTThreadCreate(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUse
  * @param   cbStack     See RTThreadCreate.
  * @param   enmType     See RTThreadCreate.
  * @param   fFlags      See RTThreadCreate.
- * @param   pszName     Thread name format.
+ * @param   pszNameFmt  Thread name format.
  * @param   va          Format arguments.
  */
 RTDECL(int) RTThreadCreateV(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUser, size_t cbStack,
@@ -741,6 +796,7 @@ RTDECL(int) RTThreadCreateV(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUs
     RTStrPrintfV(szName, sizeof(szName), pszNameFmt, va);
     return RTThreadCreate(pThread, pfnThread, pvUser, cbStack, enmType, fFlags, szName);
 }
+RT_EXPORT_SYMBOL(RTThreadCreateV);
 
 
 /**
@@ -755,7 +811,7 @@ RTDECL(int) RTThreadCreateV(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUs
  * @param   cbStack     See RTThreadCreate.
  * @param   enmType     See RTThreadCreate.
  * @param   fFlags      See RTThreadCreate.
- * @param   pszName     Thread name format.
+ * @param   pszNameFmt  Thread name format.
  * @param   ...         Format arguments.
  */
 RTDECL(int) RTThreadCreateF(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUser, size_t cbStack,
@@ -767,6 +823,7 @@ RTDECL(int) RTThreadCreateF(PRTTHREAD pThread, PFNRTTHREAD pfnThread, void *pvUs
     va_end(va);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadCreateF);
 
 
 /**
@@ -786,6 +843,7 @@ RTDECL(RTNATIVETHREAD) RTThreadGetNative(RTTHREAD Thread)
     }
     return NIL_RTNATIVETHREAD;
 }
+RT_EXPORT_SYMBOL(RTThreadGetNative);
 
 
 /**
@@ -802,6 +860,7 @@ RTDECL(RTTHREAD) RTThreadFromNative(RTNATIVETHREAD NativeThread)
         return pThread;
     return NIL_RTTHREAD;
 }
+RT_EXPORT_SYMBOL(RTThreadFromNative);
 
 
 /**
@@ -825,6 +884,7 @@ RTDECL(const char *) RTThreadSelfName(void)
     }
     return NULL;
 }
+RT_EXPORT_SYMBOL(RTThreadSelfName);
 
 
 /**
@@ -847,6 +907,7 @@ RTDECL(const char *) RTThreadGetName(RTTHREAD Thread)
     }
     return NULL;
 }
+RT_EXPORT_SYMBOL(RTThreadGetName);
 
 
 /**
@@ -879,6 +940,33 @@ RTDECL(int) RTThreadSetName(RTTHREAD Thread, const char *pszName)
     rtThreadRelease(pThread);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTThreadSetName);
+
+
+/**
+ * Checks if the specified thread is the main thread.
+ *
+ * @returns true if it is, false if it isn't.
+ *
+ * @param   hThread     The thread handle.
+ *
+ * @remarks This function may not return the correct value when RTR3Init was
+ *          called on a thread of the than the main one.  This could for
+ *          instance happen when the DLL/DYLIB/SO containing IPRT is dynamically
+ *          loaded at run time by a different thread.
+ */
+RTDECL(bool) RTThreadIsMain(RTTHREAD hThread)
+{
+    PRTTHREADINT pThread = rtThreadGet(hThread);
+    if (pThread)
+    {
+        bool fRc = !!(pThread->fIntFlags & RTTHREADINT_FLAGS_MAIN);
+        rtThreadRelease(pThread);
+        return fRc;
+    }
+    return false;
+}
+RT_EXPORT_SYMBOL(RTThreadIsMain);
 
 
 /**
@@ -899,6 +987,7 @@ RTDECL(int) RTThreadUserSignal(RTTHREAD Thread)
         rc = VERR_INVALID_HANDLE;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadUserSignal);
 
 
 /**
@@ -909,7 +998,7 @@ RTDECL(int) RTThreadUserSignal(RTTHREAD Thread)
  * @param       cMillies        The number of milliseconds to wait. Use RT_INDEFINITE_WAIT for
  *                              an indefinite wait.
  */
-RTDECL(int) RTThreadUserWait(RTTHREAD Thread, unsigned cMillies)
+RTDECL(int) RTThreadUserWait(RTTHREAD Thread, RTMSINTERVAL cMillies)
 {
     int             rc;
     PRTTHREADINT    pThread = rtThreadGet(Thread);
@@ -922,6 +1011,7 @@ RTDECL(int) RTThreadUserWait(RTTHREAD Thread, unsigned cMillies)
         rc = VERR_INVALID_HANDLE;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadUserWait);
 
 
 /**
@@ -932,7 +1022,7 @@ RTDECL(int) RTThreadUserWait(RTTHREAD Thread, unsigned cMillies)
  * @param       cMillies        The number of milliseconds to wait. Use RT_INDEFINITE_WAIT for
  *                              an indefinite wait.
  */
-RTDECL(int) RTThreadUserWaitNoResume(RTTHREAD Thread, unsigned cMillies)
+RTDECL(int) RTThreadUserWaitNoResume(RTTHREAD Thread, RTMSINTERVAL cMillies)
 {
     int             rc;
     PRTTHREADINT    pThread = rtThreadGet(Thread);
@@ -945,6 +1035,7 @@ RTDECL(int) RTThreadUserWaitNoResume(RTTHREAD Thread, unsigned cMillies)
         rc = VERR_INVALID_HANDLE;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadUserWaitNoResume);
 
 
 /**
@@ -966,6 +1057,7 @@ RTDECL(int) RTThreadUserReset(RTTHREAD Thread)
         rc = VERR_INVALID_HANDLE;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadUserReset);
 
 
 /**
@@ -978,7 +1070,7 @@ RTDECL(int) RTThreadUserReset(RTTHREAD Thread)
  * @param       prc             Where to store the return code of the thread. Optional.
  * @param       fAutoResume     Whether or not to resume the wait on VERR_INTERRUPTED.
  */
-static int rtThreadWait(RTTHREAD Thread, unsigned cMillies, int *prc, bool fAutoResume)
+static int rtThreadWait(RTTHREAD Thread, RTMSINTERVAL cMillies, int *prc, bool fAutoResume)
 {
     int rc = VERR_INVALID_HANDLE;
     if (Thread != NIL_RTTHREAD)
@@ -1028,12 +1120,13 @@ static int rtThreadWait(RTTHREAD Thread, unsigned cMillies, int *prc, bool fAuto
  *                              an indefinite wait.
  * @param       prc             Where to store the return code of the thread. Optional.
  */
-RTDECL(int) RTThreadWait(RTTHREAD Thread, unsigned cMillies, int *prc)
+RTDECL(int) RTThreadWait(RTTHREAD Thread, RTMSINTERVAL cMillies, int *prc)
 {
     int rc = rtThreadWait(Thread, cMillies, prc, true);
     Assert(rc != VERR_INTERRUPTED);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadWait);
 
 
 /**
@@ -1045,10 +1138,11 @@ RTDECL(int) RTThreadWait(RTTHREAD Thread, unsigned cMillies, int *prc)
  *                              an indefinite wait.
  * @param       prc             Where to store the return code of the thread. Optional.
  */
-RTDECL(int) RTThreadWaitNoResume(RTTHREAD Thread, unsigned cMillies, int *prc)
+RTDECL(int) RTThreadWaitNoResume(RTTHREAD Thread, RTMSINTERVAL cMillies, int *prc)
 {
     return rtThreadWait(Thread, cMillies, prc, false);
 }
+RT_EXPORT_SYMBOL(RTThreadWaitNoResume);
 
 
 /**
@@ -1098,6 +1192,7 @@ RTDECL(int) RTThreadSetType(RTTHREAD Thread, RTTHREADTYPE enmType)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTThreadSetType);
 
 
 /**
@@ -1118,120 +1213,10 @@ RTDECL(RTTHREADTYPE) RTThreadGetType(RTTHREAD Thread)
     }
     return enmType;
 }
-
+RT_EXPORT_SYMBOL(RTThreadGetType);
 
 #ifdef IN_RING3
 
-/**
- * Gets the number of write locks and critical sections the specified
- * thread owns.
- *
- * This number does not include any nested lock/critect entries.
- *
- * Note that it probably will return 0 for non-strict builds since
- * release builds doesn't do unnecessary diagnostic counting like this.
- *
- * @returns Number of locks on success (0+) and VERR_INVALID_HANDLER on failure
- * @param   Thread          The thread we're inquiring about.
- */
-RTDECL(int32_t) RTThreadGetWriteLockCount(RTTHREAD Thread)
-{
-    if (Thread == NIL_RTTHREAD)
-        return 0;
-
-    PRTTHREADINT pThread = rtThreadGet(Thread);
-    if (!pThread)
-        return VERR_INVALID_HANDLE;
-    int32_t cWriteLocks = ASMAtomicReadS32(&pThread->cWriteLocks);
-    rtThreadRelease(pThread);
-    return cWriteLocks;
-}
-
-
-/**
- * Works the THREADINT::cWriteLocks member, mostly internal.
- *
- * @param   Thread      The current thread.
- */
-RTDECL(void) RTThreadWriteLockInc(RTTHREAD Thread)
-{
-    PRTTHREADINT pThread = rtThreadGet(Thread);
-    Assert(pThread);
-    ASMAtomicIncS32(&pThread->cWriteLocks);
-    rtThreadRelease(pThread);
-}
-
-
-/**
- * Works the THREADINT::cWriteLocks member, mostly internal.
- *
- * @param   Thread      The current thread.
- */
-RTDECL(void) RTThreadWriteLockDec(RTTHREAD Thread)
-{
-    PRTTHREADINT pThread = rtThreadGet(Thread);
-    Assert(pThread);
-    ASMAtomicDecS32(&pThread->cWriteLocks);
-    rtThreadRelease(pThread);
-}
-
-
-/**
- * Gets the number of read locks the specified thread owns.
- *
- * Note that nesting read lock entry will be included in the
- * total sum. And that it probably will return 0 for non-strict
- * builds since release builds doesn't do unnecessary diagnostic
- * counting like this.
- *
- * @returns Number of read locks on success (0+) and VERR_INVALID_HANDLER on failure
- * @param   Thread          The thread we're inquiring about.
- */
-RTDECL(int32_t) RTThreadGetReadLockCount(RTTHREAD Thread)
-{
-    if (Thread == NIL_RTTHREAD)
-        return 0;
-
-    PRTTHREADINT pThread = rtThreadGet(Thread);
-    if (!pThread)
-        return VERR_INVALID_HANDLE;
-    int32_t cReadLocks = ASMAtomicReadS32(&pThread->cReadLocks);
-    rtThreadRelease(pThread);
-    return cReadLocks;
-}
-
-
-/**
- * Works the THREADINT::cReadLocks member.
- *
- * @param   Thread      The current thread.
- */
-RTDECL(void) RTThreadReadLockInc(RTTHREAD Thread)
-{
-    PRTTHREADINT pThread = rtThreadGet(Thread);
-    Assert(pThread);
-    ASMAtomicIncS32(&pThread->cReadLocks);
-    rtThreadRelease(pThread);
-}
-
-
-/**
- * Works the THREADINT::cReadLocks member.
- *
- * @param   Thread      The current thread.
- */
-RTDECL(void) RTThreadReadLockDec(RTTHREAD Thread)
-{
-    PRTTHREADINT pThread = rtThreadGet(Thread);
-    Assert(pThread);
-    ASMAtomicDecS32(&pThread->cReadLocks);
-    rtThreadRelease(pThread);
-}
-
-
-
-
-
 /**
  * Recalculates scheduling attributes for the default process
  * priority using the specified priority type for the calling thread.
@@ -1319,181 +1304,25 @@ int rtThreadDoSetProcPriority(RTPROCPRIORITY enmPriority)
 
 
 /**
- * Bitch about a deadlock.
+ * Change the thread state to blocking.
  *
- * @param   pThread     This thread.
- * @param   pCur        The thread we're deadlocking with.
- * @param   enmState    The sleep state.
- * @param   u64Block    The block data. A pointer or handle.
- * @param   pszFile     Where we are gonna block.
- * @param   uLine       Where we are gonna block.
- * @param   uId         Where we are gonna block.
+ * @param   hThread         The current thread.
+ * @param   enmState        The sleep state.
+ * @param   fReallySleeping Really going to sleep now.
  */
-static void rtThreadDeadlock(PRTTHREADINT pThread, PRTTHREADINT pCur, RTTHREADSTATE enmState, uint64_t u64Block,
-                             const char *pszFile, unsigned uLine, RTUINTPTR uId)
-{
-    AssertMsg1(pCur == pThread ? "!!Deadlock detected!!" : "!!Deadlock exists!!", uLine, pszFile, "");
-
-    /*
-     * Print the threads and locks involved.
-     */
-    PRTTHREADINT    apSeenThreads[8] = {0,0,0,0,0,0,0,0};
-    unsigned        iSeenThread = 0;
-    pCur = pThread;
-    for (unsigned iEntry = 0; pCur && iEntry < 256; iEntry++)
-    {
-        /*
-         * Print info on pCur. Determin next while doing so.
-         */
-        AssertMsg2(" #%d: %RTthrd/%RTnthrd %s: %s(%u) %RTptr\n",
-                   iEntry, pCur, pCur->Core.Key, pCur->szName,
-                   pCur->pszBlockFile, pCur->uBlockLine, pCur->uBlockId);
-        PRTTHREADINT pNext = NULL;
-        switch (pCur->enmState)
-        {
-            case RTTHREADSTATE_CRITSECT:
-            {
-                PRTCRITSECT pCritSect = pCur->Block.pCritSect;
-                if (pCur->enmState != RTTHREADSTATE_CRITSECT)
-                {
-                    AssertMsg2("Impossible!!!\n");
-                    break;
-                }
-                if (VALID_PTR(pCritSect) && RTCritSectIsInitialized(pCritSect))
-                {
-                    AssertMsg2("     Waiting on CRITSECT %p: Entered %s(%u) %RTptr\n",
-                               pCritSect, pCritSect->Strict.pszEnterFile,
-                               pCritSect->Strict.u32EnterLine, pCritSect->Strict.uEnterId);
-                    pNext = pCritSect->Strict.ThreadOwner;
-                }
-                else
-                    AssertMsg2("     Waiting on CRITSECT %p: invalid pointer or uninitialized critsect\n", pCritSect);
-                break;
-            }
-
-            default:
-                AssertMsg2(" Impossible!!! enmState=%d\n", pCur->enmState);
-                break;
-        }
-
-        /*
-         * Check for cycle.
-         */
-        if (iEntry && pCur == pThread)
-            break;
-        for (unsigned i = 0; i < RT_ELEMENTS(apSeenThreads); i++)
-            if (apSeenThreads[i] == pCur)
-            {
-                AssertMsg2(" Cycle!\n");
-                pNext = NULL;
-                break;
-            }
-
-        /*
-         * Advance to the next thread.
-         */
-        iSeenThread = (iSeenThread + 1) % RT_ELEMENTS(apSeenThreads);
-        apSeenThreads[iSeenThread] = pCur;
-        pCur = pNext;
-    }
-    AssertBreakpoint();
-}
-
-
-/**
- * Change the thread state to blocking and do deadlock detection.
- *
- * This is a RT_STRICT method for debugging locks and detecting deadlocks.
- *
- * @param   pThread     This thread.
- * @param   enmState    The sleep state.
- * @param   u64Block    The block data. A pointer or handle.
- * @param   pszFile     Where we are blocking.
- * @param   uLine       Where we are blocking.
- * @param   uId         Where we are blocking.
- */
-void rtThreadBlocking(PRTTHREADINT pThread, RTTHREADSTATE enmState, uint64_t u64Block,
-                     const char *pszFile, unsigned uLine, RTUINTPTR uId)
+RTDECL(void) RTThreadBlocking(RTTHREAD hThread, RTTHREADSTATE enmState, bool fReallySleeping)
 {
     Assert(RTTHREAD_IS_SLEEPING(enmState));
-    if (pThread && pThread->enmState == RTTHREADSTATE_RUNNING)
+    PRTTHREADINT pThread = hThread;
+    if (pThread != NIL_RTTHREAD)
     {
-        /** @todo This has to be serialized! The deadlock detection isn't 100% safe!!! */
-        pThread->Block.u64      = u64Block;
-        pThread->pszBlockFile   = pszFile;
-        pThread->uBlockLine     = uLine;
-        pThread->uBlockId       = uId;
-        ASMAtomicWriteSize(&pThread->enmState, enmState);
-
-        /*
-         * Do deadlock detection.
-         *
-         * Since we're missing proper serialization, we don't declare it a
-         * deadlock until we've got three runs with the same list length.
-         * While this isn't perfect, it should avoid out the most obvious
-         * races on SMP boxes.
-         */
-        PRTTHREADINT    pCur;
-        unsigned        cPrevLength = ~0U;
-        unsigned        cEqualRuns = 0;
-        unsigned        iParanoia = 256;
-        do
-        {
-            unsigned cLength = 0;
-            pCur = pThread;
-            for (;;)
-            {
-                /*
-                 * Get the next thread.
-                 */
-                for (;;)
-                {
-                    switch (pCur->enmState)
-                    {
-                        case RTTHREADSTATE_CRITSECT:
-                        {
-                            PRTCRITSECT pCritSect = pCur->Block.pCritSect;
-                            if (pCur->enmState != RTTHREADSTATE_CRITSECT)
-                                continue;
-                            pCur = pCritSect->Strict.ThreadOwner;
-                            break;
-                        }
-
-                        default:
-                            pCur = NULL;
-                            break;
-                    }
-                    break;
-                }
-                if (!pCur)
-                    return;
-
-                /*
-                 * If we've got back to the blocking thread id we've got a deadlock.
-                 * If we've got a chain of more than 256 items, there is some kind of cycle
-                 * in the list, which means that there is already a deadlock somewhere.
-                 */
-                if (pCur == pThread || cLength >= 256)
-                    break;
-                cLength++;
-            }
-
-            /* compare with previous list run. */
-            if (cLength != cPrevLength)
-            {
-                cPrevLength = cLength;
-                cEqualRuns = 0;
-            }
-            else
-                cEqualRuns++;
-        } while (cEqualRuns < 3 && --iParanoia > 0);
-
-        /*
-         * Ok, if we ever get here, it's most likely a genuine deadlock.
-         */
-        rtThreadDeadlock(pThread, pCur, enmState, u64Block, pszFile, uLine, uId);
+        Assert(pThread == RTThreadSelf());
+        if (rtThreadGetState(pThread) == RTTHREADSTATE_RUNNING)
+            rtThreadSetState(pThread, enmState);
+        ASMAtomicWriteBool(&pThread->fReallySleeping, fReallySleeping);
     }
 }
+RT_EXPORT_SYMBOL(RTThreadBlocking);
 
 
 /**
@@ -1501,19 +1330,103 @@ void rtThreadBlocking(PRTTHREADINT pThread, RTTHREADSTATE enmState, uint64_t u64
  *
  * This function is paired with rtThreadBlocking.
  *
- * @param   pThread     The current thread.
+ * @param   hThread     The current thread.
  * @param   enmCurState The current state, used to check for nested blocking.
  *                      The new state will be running.
  */
-void rtThreadUnblocked(PRTTHREADINT pThread, RTTHREADSTATE enmCurState)
+RTDECL(void) RTThreadUnblocked(RTTHREAD hThread, RTTHREADSTATE enmCurState)
 {
-    if (pThread && pThread->enmState == enmCurState)
-        ASMAtomicWriteSize(&pThread->enmState, RTTHREADSTATE_RUNNING);
+    PRTTHREADINT pThread = hThread;
+    if (pThread != NIL_RTTHREAD)
+    {
+        Assert(pThread == RTThreadSelf());
+        ASMAtomicWriteBool(&pThread->fReallySleeping, false);
+
+        RTTHREADSTATE enmActualState = rtThreadGetState(pThread);
+        if (enmActualState == enmCurState)
+        {
+            rtThreadSetState(pThread, RTTHREADSTATE_RUNNING);
+            if (   pThread->LockValidator.pRec
+                && pThread->LockValidator.enmRecState == enmCurState)
+                ASMAtomicWritePtr((void * volatile *)&pThread->LockValidator.pRec, NULL);
+        }
+        /* This is a bit ugly... :-/ */
+        else if (   (   enmActualState == RTTHREADSTATE_TERMINATED
+                     || enmActualState == RTTHREADSTATE_INITIALIZING)
+                 && pThread->LockValidator.pRec)
+            ASMAtomicWritePtr((void * volatile *)&pThread->LockValidator.pRec, NULL);
+        Assert(   pThread->LockValidator.pRec == NULL
+               || RTTHREAD_IS_SLEEPING(enmActualState));
+    }
 }
+RT_EXPORT_SYMBOL(RTThreadUnblocked);
+
+
+/**
+ * Get the current thread state.
+ *
+ * @returns The thread state.
+ * @param   hThread         The thread.
+ */
+RTDECL(RTTHREADSTATE) RTThreadGetState(RTTHREAD hThread)
+{
+    RTTHREADSTATE   enmState = RTTHREADSTATE_INVALID;
+    PRTTHREADINT    pThread  = rtThreadGet(hThread);
+    if (pThread)
+    {
+        enmState = rtThreadGetState(pThread);
+        rtThreadRelease(pThread);
+    }
+    return enmState;
+}
+RT_EXPORT_SYMBOL(RTThreadGetState);
+
+
+RTDECL(RTTHREADSTATE) RTThreadGetReallySleeping(RTTHREAD hThread)
+{
+    RTTHREADSTATE   enmState = RTTHREADSTATE_INVALID;
+    PRTTHREADINT    pThread  = rtThreadGet(hThread);
+    if (pThread)
+    {
+        enmState = rtThreadGetState(pThread);
+        if (!ASMAtomicUoReadBool(&pThread->fReallySleeping))
+            enmState = RTTHREADSTATE_RUNNING;
+        rtThreadRelease(pThread);
+    }
+    return enmState;
+}
+RT_EXPORT_SYMBOL(RTThreadGetReallySleeping);
+
+
+/**
+ * Translate a thread state into a string.
+ *
+ * @returns Pointer to a read-only string containing the state name.
+ * @param   enmState            The state.
+ */
+RTDECL(const char *) RTThreadStateName(RTTHREADSTATE enmState)
+{
+    switch (enmState)
+    {
+        case RTTHREADSTATE_INVALID:         return "INVALID";
+        case RTTHREADSTATE_INITIALIZING:    return "INITIALIZING";
+        case RTTHREADSTATE_TERMINATED:      return "TERMINATED";
+        case RTTHREADSTATE_RUNNING:         return "RUNNING";
+        case RTTHREADSTATE_CRITSECT:        return "CRITSECT";
+        case RTTHREADSTATE_EVENT:           return "EVENT";
+        case RTTHREADSTATE_EVENT_MULTI:     return "EVENT_MULTI";
+        case RTTHREADSTATE_FAST_MUTEX:      return "FAST_MUTEX";
+        case RTTHREADSTATE_MUTEX:           return "MUTEX";
+        case RTTHREADSTATE_RW_READ:         return "RW_READ";
+        case RTTHREADSTATE_RW_WRITE:        return "RW_WRITE";
+        case RTTHREADSTATE_SLEEP:           return "SLEEP";
+        case RTTHREADSTATE_SPIN_MUTEX:      return "SPIN_MUTEX";
+        default:                            return "UnknownThreadState";
+    }
+}
+RT_EXPORT_SYMBOL(RTThreadStateName);
 
 #endif /* IN_RING3 */
-
-
 #ifdef IPRT_WITH_GENERIC_TLS
 
 /**
