@@ -1,10 +1,10 @@
-/* $Id: randadv.cpp $ */
+/* $Id: randadv.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Random Numbers, Generic Glue.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -33,6 +29,8 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/rand.h>
+#include "internal/iprt.h"
+
 #include <iprt/mem.h>
 #include <iprt/err.h>
 #include <iprt/assert.h>
@@ -52,6 +50,7 @@ RTDECL(int) RTRandAdvDestroy(RTRAND hRand) RT_NO_THROW
     /* forward the call */
     return pThis->pfnDestroy(pThis);
 }
+RT_EXPORT_SYMBOL(RTRandAdvDestroy);
 
 
 RTDECL(int) RTRandAdvSeed(RTRAND hRand, uint64_t u64Seed) RT_NO_THROW
@@ -64,6 +63,7 @@ RTDECL(int) RTRandAdvSeed(RTRAND hRand, uint64_t u64Seed) RT_NO_THROW
     /* forward the call */
     return pThis->pfnSeed(pThis, u64Seed);
 }
+RT_EXPORT_SYMBOL(RTRandAdvSeed);
 
 
 RTDECL(int) RTRandAdvSaveState(RTRAND hRand, char *pszState, size_t *pcbState) RT_NO_THROW
@@ -78,6 +78,7 @@ RTDECL(int) RTRandAdvSaveState(RTRAND hRand, char *pszState, size_t *pcbState) R
     /* forward the call */
     return pThis->pfnSaveState(pThis, pszState, pcbState);
 }
+RT_EXPORT_SYMBOL(RTRandAdvSaveState);
 
 
 RTDECL(int) RTRandAdvRestoreState(RTRAND hRand, char const *pszState) RT_NO_THROW
@@ -91,6 +92,7 @@ RTDECL(int) RTRandAdvRestoreState(RTRAND hRand, char const *pszState) RT_NO_THRO
     /* forward the call */
     return pThis->pfnRestoreState(pThis, pszState);
 }
+RT_EXPORT_SYMBOL(RTRandAdvRestoreState);
 
 
 RTDECL(void) RTRandAdvBytes(RTRAND hRand, void *pv, size_t cb) RT_NO_THROW
@@ -104,6 +106,7 @@ RTDECL(void) RTRandAdvBytes(RTRAND hRand, void *pv, size_t cb) RT_NO_THROW
     /* forward the call */
     return pThis->pfnGetBytes(pThis, (uint8_t *)pv, cb);
 }
+RT_EXPORT_SYMBOL(RTRandAdvBytes);
 
 
 RTDECL(int32_t) RTRandAdvS32Ex(RTRAND hRand, int32_t i32First, int32_t i32Last) RT_NO_THROW
@@ -116,6 +119,7 @@ RTDECL(int32_t) RTRandAdvS32Ex(RTRAND hRand, int32_t i32First, int32_t i32Last)
     /* wrap the call */
     return pThis->pfnGetU32(pThis, 0, i32Last - i32First) + i32First;
 }
+RT_EXPORT_SYMBOL(RTRandAdvS32Ex);
 
 
 RTDECL(int32_t) RTRandAdvS32(RTRAND hRand) RT_NO_THROW
@@ -128,6 +132,7 @@ RTDECL(int32_t) RTRandAdvS32(RTRAND hRand) RT_NO_THROW
     /* wrap the call */
     return pThis->pfnGetU32(pThis, 0, UINT32_MAX) + INT32_MAX;
 }
+RT_EXPORT_SYMBOL(RTRandAdvS32);
 
 
 RTDECL(uint32_t) RTRandAdvU32Ex(RTRAND hRand, uint32_t u32First, uint32_t u32Last) RT_NO_THROW
@@ -140,6 +145,7 @@ RTDECL(uint32_t) RTRandAdvU32Ex(RTRAND hRand, uint32_t u32First, uint32_t u32Las
     /* forward the call */
     return pThis->pfnGetU32(pThis, u32First, u32Last);
 }
+RT_EXPORT_SYMBOL(RTRandAdvU32Ex);
 
 
 RTDECL(uint32_t) RTRandAdvU32(RTRAND hRand) RT_NO_THROW
@@ -152,6 +158,7 @@ RTDECL(uint32_t) RTRandAdvU32(RTRAND hRand) RT_NO_THROW
     /* forward the call */
     return pThis->pfnGetU32(pThis, 0, UINT32_MAX);
 }
+RT_EXPORT_SYMBOL(RTRandAdvU32);
 
 
 RTDECL(int64_t) RTRandAdvS64Ex(RTRAND hRand, int64_t i64First, int64_t i64Last) RT_NO_THROW
@@ -164,6 +171,7 @@ RTDECL(int64_t) RTRandAdvS64Ex(RTRAND hRand, int64_t i64First, int64_t i64Last)
     /* wrap the call */
     return pThis->pfnGetU64(pThis, 0, i64Last - i64First) + i64First;
 }
+RT_EXPORT_SYMBOL(RTRandAdvS64Ex);
 
 
 RTDECL(int64_t) RTRandAdvS64(RTRAND hRand) RT_NO_THROW
@@ -176,6 +184,7 @@ RTDECL(int64_t) RTRandAdvS64(RTRAND hRand) RT_NO_THROW
     /* wrap the call */
     return pThis->pfnGetU64(pThis, 0, UINT64_MAX) + INT64_MAX;
 }
+RT_EXPORT_SYMBOL(RTRandAdvS64);
 
 
 RTDECL(uint64_t) RTRandAdvU64Ex(RTRAND hRand, uint64_t u64First, uint64_t u64Last) RT_NO_THROW
@@ -188,6 +197,7 @@ RTDECL(uint64_t) RTRandAdvU64Ex(RTRAND hRand, uint64_t u64First, uint64_t u64Las
     /* forward the call */
     return pThis->pfnGetU64(pThis, u64First, u64Last);
 }
+RT_EXPORT_SYMBOL(RTRandAdvU64Ex);
 
 
 RTDECL(uint64_t) RTRandAdvU64(RTRAND hRand) RT_NO_THROW
@@ -200,6 +210,7 @@ RTDECL(uint64_t) RTRandAdvU64(RTRAND hRand) RT_NO_THROW
     /* forward the call */
     return pThis->pfnGetU64(pThis, 0, UINT64_MAX);
 }
+RT_EXPORT_SYMBOL(RTRandAdvU64);
 
 
 DECLCALLBACK(void)  rtRandAdvSynthesizeBytesFromU32(PRTRANDINT pThis, uint8_t *pb, size_t cb)
@@ -314,7 +325,7 @@ DECLCALLBACK(uint32_t)  rtRandAdvSynthesizeU32FromBytes(PRTRANDINT pThis, uint32
 
 DECLCALLBACK(uint32_t)  rtRandAdvSynthesizeU32FromU64(PRTRANDINT pThis, uint32_t u32First, uint32_t u32Last)
 {
-    return pThis->pfnGetU64(pThis, u32First, u32Last);
+    return (uint32_t)pThis->pfnGetU64(pThis, u32First, u32Last);
 }
 
 
@@ -357,9 +368,9 @@ DECLCALLBACK(uint64_t)  rtRandAdvSynthesizeU64FromU32(PRTRANDINT pThis, uint64_t
 {
     uint64_t off = u64Last - u64First;
     if (off <= UINT32_MAX)
-        return pThis->pfnGetU32(pThis, 0, off) + u64First;
+        return (uint64_t)pThis->pfnGetU32(pThis, 0, off) + u64First;
 
-    return (    pThis->pfnGetU32(pThis, 0, UINT32_MAX)
+    return (    (uint64_t)pThis->pfnGetU32(pThis, 0, UINT32_MAX)
             |  ((uint64_t)pThis->pfnGetU32(pThis, 0, off >> 32) << 32))
          + u64First;
 }
@@ -401,4 +412,3 @@ DECLCALLBACK(int) rtRandAdvDefaultDestroy(PRTRANDINT pThis)
     return VINF_SUCCESS;
 }
 
-
