@@ -1,10 +1,10 @@
-/* $Id: base64.cpp 16774 2009-02-15 04:23:47Z vboxsync $ */
+/* $Id: base64.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Base64, MIME content transfer encoding.
  */
 
 /*
- * Copyright (C) 2009 Sun Microsystems, Inc.
+ * Copyright (C) 2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,16 +22,15 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/base64.h>
+#include "internal/iprt.h"
+
 #include <iprt/assert.h>
 #include <iprt/err.h>
 #include <iprt/ctype.h>
@@ -215,6 +214,7 @@ RTDECL(ssize_t) RTBase64DecodedSize(const char *pszString, char **ppszEnd)
         *ppszEnd = (char *)pszString;
     return cb;
 }
+RT_EXPORT_SYMBOL(RTBase64DecodedSize);
 
 
 /**
@@ -247,7 +247,7 @@ RTDECL(int) RTBase64Decode(const char *pszString, void *pvData, size_t cbData, s
     /*
      * Process input in groups of 4 input / 3 output chars.
      */
-    uint8_t     u8Trio[3];
+    uint8_t     u8Trio[3] = { 0, 0, 0 }; /* shuts up gcc */
     uint8_t    *pbData    = (uint8_t *)pvData;
     uint8_t     u8        = BASE64_INVALID;
     unsigned    c6Bits    = 0;
@@ -398,6 +398,7 @@ RTDECL(int) RTBase64Decode(const char *pszString, void *pvData, size_t cbData, s
         *pcbActual = pbData - (uint8_t *)pvData;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTBase64Decode);
 
 
 /**
@@ -435,6 +436,7 @@ RTDECL(size_t) RTBase64EncodedLength(size_t cbData)
     cch -= (cch % RTBASE64_LINE_LEN) == 0;
     return cch;
 }
+RT_EXPORT_SYMBOL(RTBase64EncodedLength);
 
 
 /**
@@ -533,4 +535,5 @@ RTDECL(int) RTBase64Encode(const void *pvData, size_t cbData, char *pszBuf, size
         *pcchActual = pchDst - pszBuf;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTBase64Encode);
 
