@@ -1,10 +1,10 @@
-/* $Id: strformat.cpp 14066 2008-11-10 23:33:15Z vboxsync $ */
+/* $Id: strformat.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - String Formatter.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -42,6 +38,8 @@
 *******************************************************************************/
 #define LOG_GROUP RTLOGGROUP_STRING
 #include <iprt/string.h>
+#include "internal/iprt.h"
+
 #include <iprt/assert.h>
 #ifdef IN_RING3
 # include <iprt/alloc.h>
@@ -153,6 +151,7 @@ RTDECL(int) RTStrFormatNumber(char *psz, uint64_t u64Value, unsigned int uiBase,
 {
     return rtStrFormatNumber(psz, *(KSIZE64 *)(void *)&u64Value, uiBase, cchWidth, cchPrecision, fFlags);
 }
+RT_EXPORT_SYMBOL(RTStrFormatNumber);
 
 
 
@@ -169,29 +168,25 @@ RTDECL(int) RTStrFormatNumber(char *psz, uint64_t u64Value, unsigned int uiBase,
  */
 static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, signed int cchWidth, signed int cchPrecision, unsigned int fFlags)
 {
-    const char *    pachDigits = "0123456789abcdef";
-    char *          pszStart = psz;
+    const char     *pachDigits = "0123456789abcdef";
+    char           *pszStart = psz;
     int             cchValue;
     unsigned long   ul;
-#if 0
-    unsigned long   ullow;
-#endif
     int             i;
     int             j;
 
-/** @todo Formatting of 64 bit numbers is broken, fix it! */
-
     /*
-     * Validate and addjust input...
+     * Validate and adjust input...
      */
-/** @todo r=bird: Dmitry, who is calling this code with uiBase == 0? */
-    if (uiBase == 0)
-        uiBase = 10;
-    kASSERT((uiBase >= 2 || uiBase <= 16));
+    Assert(uiBase >= 2 || uiBase <= 16);
     if (fFlags & RTSTR_F_CAPITAL)
         pachDigits = "0123456789ABCDEF";
     if (fFlags & RTSTR_F_LEFT)
         fFlags &= ~RTSTR_F_ZEROPAD;
+    if (    (fFlags & RTSTR_F_THOUSAND_SEP)
+        &&  (   uiBase != 10
+             || (fFlags & RTSTR_F_ZEROPAD))) /** @todo implement RTSTR_F_ZEROPAD + RTSTR_F_THOUSAND_SEP. */
+        fFlags &= ~RTSTR_F_THOUSAND_SEP;
 
     /*
      * Determin value length
@@ -217,6 +212,13 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
             ul /= uiBase;
         } while (ul);
     }
+    if (fFlags & RTSTR_F_THOUSAND_SEP)
+    {
+        if (cchValue <= 3)
+            fFlags &= ~RTSTR_F_THOUSAND_SEP;
+        else
+            cchValue += cchValue / 3 - (cchValue % 3 == 0);
+    }
 
     /*
      * Sign (+/-).
@@ -280,23 +282,48 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
     if (ullValue.ulHi || (fFlags & RTSTR_F_64BIT))
     {
         uint64_t    u64 = *(uint64_t *)(void *)&ullValue;
-        do
+        if (fFlags & RTSTR_F_THOUSAND_SEP)
         {
-            psz[i--] = pachDigits[u64 % uiBase];
-            u64 /= uiBase;
-        } while (u64);
+            do
+            {
+                if ((-i - 1) % 4 == 3)
+                    psz[i--] = ' ';
+                psz[i--] = pachDigits[u64 % uiBase];
+                u64 /= uiBase;
+            } while (u64);
+        }
+        else
+        {
+            do
+            {
+                psz[i--] = pachDigits[u64 % uiBase];
+                u64 /= uiBase;
+            } while (u64);
+        }
     }
     else
     {
         ul = (fFlags & RTSTR_F_VALSIGNED) && (ullValue.ulLo & 0x80000000) ? -(int32_t)ullValue.ulLo : ullValue.ulLo;
-        do
+        if (fFlags & RTSTR_F_THOUSAND_SEP)
         {
-            psz[i--] = pachDigits[ul % uiBase];
-            ul /= uiBase;
-        } while (ul);
+            do
+            {
+                if ((-i - 1) % 4 == 3)
+                    psz[i--] = ' ';
+                psz[i--] = pachDigits[ul % uiBase];
+                ul /= uiBase;
+            } while (ul);
+        }
+        else
+        {
+            do
+            {
+                psz[i--] = pachDigits[ul % uiBase];
+                ul /= uiBase;
+            } while (ul);
+        }
     }
 
-
     /*
      * width if RTSTR_F_LEFT
      */
@@ -357,11 +384,12 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                 {
                     switch (*pszFormat++)
                     {
-                        case '#':   fFlags |= RTSTR_F_SPECIAL;   continue;
-                        case '-':   fFlags |= RTSTR_F_LEFT;      continue;
-                        case '+':   fFlags |= RTSTR_F_PLUS;      continue;
-                        case ' ':   fFlags |= RTSTR_F_BLANK;     continue;
-                        case '0':   fFlags |= RTSTR_F_ZEROPAD;   continue;
+                        case '#':   fFlags |= RTSTR_F_SPECIAL;      continue;
+                        case '-':   fFlags |= RTSTR_F_LEFT;         continue;
+                        case '+':   fFlags |= RTSTR_F_PLUS;         continue;
+                        case ' ':   fFlags |= RTSTR_F_BLANK;        continue;
+                        case '0':   fFlags |= RTSTR_F_ZEROPAD;      continue;
+                        case '\'':  fFlags |= RTSTR_F_THOUSAND_SEP; continue;
                     }
                     pszFormat--;
                     break;
@@ -792,6 +820,7 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                         va_copy(ArgsNested, *pArgsNested);
                         Assert(pszFormatNested);
                         cch += RTStrFormatV(pfnOutput, pvArgOutput, pfnFormat, pvArgFormat, pszFormatNested, ArgsNested);
+                        va_end(ArgsNested);
                         break;
                     }
 
@@ -813,18 +842,6 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                         break;
                     }
 
-#ifdef RT_WITH_VBOX
-                    /*
-                     * VBox extensions.
-                     */
-                    case 'V':
-                    {
-                        pszFormat--;
-                        cch += rtstrFormatVBox(pfnOutput, pvArgOutput, &pszFormat, &args, cchPrecision, cchWidth, fFlags, chArgSize);
-                        break;
-                    }
-#endif
-
                     /*
                      * Custom format.
                      */
@@ -854,6 +871,7 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
 
     return cch;
 }
+RT_EXPORT_SYMBOL(RTStrFormatV);
 
 
 /**
@@ -880,4 +898,5 @@ RTDECL(size_t) RTStrFormat(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRFO
     va_end(args);
     return cch;
 }
+RT_EXPORT_SYMBOL(RTStrFormat);
 
