@@ -1,10 +1,10 @@
-/* $Id: strformatrt.cpp 13837 2008-11-05 02:54:02Z vboxsync $ */
+/* $Id: strformatrt.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - IPRT String Formatter Extensions.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /** @page pg_rt_str_format_rt   The IPRT String Format Extensions
@@ -50,6 +46,11 @@
  *      - \%RTint           - Takes a #RTINT value.
  *      - \%RTiop           - Takes a #RTIOPORT value.
  *      - \%RTldrm          - Takes a #RTLDRMOD value.
+ *      - \%RTmac           - Takes a #PCRTMAC pointer.
+ *      - \%RTnaddr         - Takes a #PCRTNETADDR value.
+ *      - \%RTnaipv4        - Takes a #RTNETADDRIPV4 value.
+ *      - \%RTnaipv6        - Takes a #PCRTNETADDRIPV6 value.
+ *      - \%RTnthrd         - Takes a #RTNATIVETHREAD value.
  *      - \%RTnthrd         - Takes a #RTNATIVETHREAD value.
  *      - \%RTproc          - Takes a #RTPROCESS value.
  *      - \%RTptr           - Takes a #RTINTPTR or #RTUINTPTR value (but not void *).
@@ -77,7 +78,7 @@
  *      - \%RHv             - Takes a #RTHCPTR, #RTHCINTPTR or #RTHCUINTPTR value.
  *      - \%RHx             - Takes a #RTHCUINT or #RTHCINT value, formatting it as hex.
  *      - \%RRv             - Takes a #RTRCPTR, #RTRCINTPTR or #RTRCUINTPTR value.
- *      - \%RCi             - Takes a #RTCCINT value.
+ *      - \%RCi             - Takes a #RTINT value.
  *      - \%RCp             - Takes a #RTCCPHYS value.
  *      - \%RCr             - Takes a #RTCCUINTREG value.
  *      - \%RCu             - Takes a #RTUINT value.
@@ -108,8 +109,6 @@
  *                            full description of the specified status code.
  *      - \%Rra             - Takes an integer iprt status code as argument. Will insert the
  *                            status code define + full description.
- *      - \%Rt              - Current thread (RTThreadSelf()), no arguments.
- *
  *      - \%Rwc             - Takes a long Windows error code as argument. Will insert the status
  *                            code define corresponding to the Windows error code.
  *      - \%Rwf             - Takes a long Windows error code as argument. Will insert the
@@ -126,6 +125,8 @@
  *
  *      - \%Rfn             - Pretty printing of a function or method. It drops the
  *                            return code and parameter list.
+ *      - \%Rbn             - Prints the base name.  For dropping the path in
+ *                            order to save space when printing a path name.
  *
  * On other platforms, \%Rw? simply prints the argument in a form of 0xXXXXXXXX.
  *
@@ -135,14 +136,25 @@
  *      - \%RDtimespec      - Takes a PCRTTIMESPEC.
  *
  *
+ * Group 5, XML / HTML escapers.
+ *      - \%RMas            - Takes a string pointer (const char *) and outputs
+ *                            it as an attribute value with the proper escaping.
+ *                            This typically ends up in double quotes.
+ *
+ *      - \%RMes            - Takes a string pointer (const char *) and outputs
+ *                            it as an element with the necessary escaping.
+ *
+ *
  */
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP RTLOGGROUP_STRING
-#include <iprt/log.h>
 #include <iprt/string.h>
+#include "internal/iprt.h"
+
+#include <iprt/log.h>
 #include <iprt/assert.h>
 #include <iprt/string.h>
 #include <iprt/stdarg.h>
@@ -150,8 +162,10 @@
 # include <iprt/thread.h>
 # include <iprt/err.h>
 #endif
-#include <iprt/time.h>
 #include <iprt/ctype.h>
+#include <iprt/time.h>
+#include <iprt/net.h>
+#include <iprt/path.h>
 #include "internal/string.h"
 
 
@@ -195,7 +209,20 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                 /*
                  * Interpret the type.
                  */
-                typedef enum { RTSF_INT, RTSF_INTW, RTSF_FP16, RTSF_FP32, RTSF_FP64, RTSF_UUID, RTSF_BOOL } RTSF;
+                typedef enum
+                {
+                    RTSF_INT,
+                    RTSF_INTW,
+                    RTSF_BOOL,
+                    RTSF_FP16,
+                    RTSF_FP32,
+                    RTSF_FP64,
+                    RTSF_IPV4,
+                    RTSF_IPV6,
+                    RTSF_MAC,
+                    RTSF_NETADDR,
+                    RTSF_UUID
+                } RTSF;
                 static const struct
                 {
                     uint8_t     cch;        /**< the length of the string. */
@@ -210,7 +237,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                 {
 #define STRMEM(str) sizeof(str) - 1, str
                     { STRMEM("Ci"),      sizeof(RTINT),          10, RTSF_INT,   RTSTR_F_VALSIGNED },
-                    { STRMEM("Cp"),      sizeof(RTGCPHYS),       16, RTSF_INTW,  0 },
+                    { STRMEM("Cp"),      sizeof(RTCCPHYS),       16, RTSF_INTW,  0 },
                     { STRMEM("Cr"),      sizeof(RTCCUINTREG),    16, RTSF_INTW,  0 },
                     { STRMEM("Cu"),      sizeof(RTUINT),         10, RTSF_INT,   0 },
                     { STRMEM("Cv"),      sizeof(void *),         16, RTSF_INTW,  0 },
@@ -244,6 +271,10 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                     { STRMEM("Tint"),    sizeof(RTINT),          10, RTSF_INT,   RTSTR_F_VALSIGNED },
                     { STRMEM("Tiop"),    sizeof(RTIOPORT),       16, RTSF_INTW,  0 },
                     { STRMEM("Tldrm"),   sizeof(RTLDRMOD),       16, RTSF_INTW,  0 },
+                    { STRMEM("Tmac"),    sizeof(PCRTMAC),        16, RTSF_MAC,   0 },
+                    { STRMEM("Tnaddr"),  sizeof(PCRTNETADDR),    10, RTSF_NETADDR,0 },
+                    { STRMEM("Tnaipv4"), sizeof(RTNETADDRIPV4),  10, RTSF_IPV4,  0 },
+                    { STRMEM("Tnaipv6"), sizeof(PCRTNETADDRIPV6),16, RTSF_IPV6,  0 },
                     { STRMEM("Tnthrd"),  sizeof(RTNATIVETHREAD), 16, RTSF_INTW,  0 },
                     { STRMEM("Tproc"),   sizeof(RTPROCESS),      16, RTSF_INTW,  0 },
                     { STRMEM("Tptr"),    sizeof(RTUINTPTR),      16, RTSF_INTW,  0 },
@@ -268,6 +299,8 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                     { STRMEM("X8"),      sizeof(uint8_t),        16, RTSF_INT,   0 },
 #undef STRMEM
                 };
+                static const char s_szNull[] = "<NULL>";
+
                 const char *pszType = *ppszFormat - 1;
                 int         iStart  = 0;
                 int         iEnd    = RT_ELEMENTS(s_aTypes) - 1;
@@ -275,19 +308,23 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
 
                 union
                 {
-                    uint8_t     u8;
-                    uint16_t    u16;
-                    uint32_t    u32;
-                    uint64_t    u64;
-                    int8_t      i8;
-                    int16_t     i16;
-                    int32_t     i32;
-                    int64_t     i64;
-                    RTFAR16     fp16;
-                    RTFAR32     fp32;
-                    RTFAR64     fp64;
-                    bool        fBool;
-                    PCRTUUID    pUuid;
+                    uint8_t             u8;
+                    uint16_t            u16;
+                    uint32_t            u32;
+                    uint64_t            u64;
+                    int8_t              i8;
+                    int16_t             i16;
+                    int32_t             i32;
+                    int64_t             i64;
+                    RTFAR16             fp16;
+                    RTFAR32             fp32;
+                    RTFAR64             fp64;
+                    bool                fBool;
+                    PCRTMAC             pMac;
+                    RTNETADDRIPV4       Ipv4Addr;
+                    PCRTNETADDRIPV6     pIpv6Addr;
+                    PCRTNETADDR         pNetAddr;
+                    PCRTUUID            pUuid;
                 } u;
                 char        szBuf[80];
                 unsigned    cch;
@@ -329,7 +366,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                  * Fetch the argument.
                  * It's important that a signed value gets sign-extended up to 64-bit.
                  */
-                u.u64 = 0;
+                RT_ZERO(u);
                 if (fFlags & RTSTR_F_VALSIGNED)
                 {
                     switch (s_aTypes[i].cb)
@@ -411,9 +448,21 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                         break;
                     }
 
+                    case RTSF_BOOL:
+                    {
+                        static const char s_szTrue[]  = "true ";
+                        static const char s_szFalse[] = "false";
+                        if (u.u64 == 1)
+                            return pfnOutput(pvArgOutput, s_szTrue,  sizeof(s_szTrue) - 1);
+                        if (u.u64 == 0)
+                            return pfnOutput(pvArgOutput, s_szFalse, sizeof(s_szFalse) - 1);
+                        /* invalid boolean value */
+                        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "!%lld!", u.u64);
+                    }
+
                     case RTSF_FP16:
                     {
-                        fFlags &= ~(RTSTR_F_VALSIGNED | RTSTR_F_BIT_MASK | RTSTR_F_WIDTH | RTSTR_F_PRECISION);
+                        fFlags &= ~(RTSTR_F_VALSIGNED | RTSTR_F_BIT_MASK | RTSTR_F_WIDTH | RTSTR_F_PRECISION | RTSTR_F_THOUSAND_SEP);
                         cch = RTStrFormatNumber(&szBuf[0], u.fp16.sel, 16, 4, -1, fFlags | RTSTR_F_16BIT);
                         Assert(cch == 4);
                         szBuf[4] = ':';
@@ -424,7 +473,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                     }
                     case RTSF_FP32:
                     {
-                        fFlags &= ~(RTSTR_F_VALSIGNED | RTSTR_F_BIT_MASK | RTSTR_F_WIDTH | RTSTR_F_PRECISION);
+                        fFlags &= ~(RTSTR_F_VALSIGNED | RTSTR_F_BIT_MASK | RTSTR_F_WIDTH | RTSTR_F_PRECISION | RTSTR_F_THOUSAND_SEP);
                         cch = RTStrFormatNumber(&szBuf[0], u.fp32.sel, 16, 4, -1, fFlags | RTSTR_F_16BIT);
                         Assert(cch == 4);
                         szBuf[4] = ':';
@@ -435,7 +484,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                     }
                     case RTSF_FP64:
                     {
-                        fFlags &= ~(RTSTR_F_VALSIGNED | RTSTR_F_BIT_MASK | RTSTR_F_WIDTH | RTSTR_F_PRECISION);
+                        fFlags &= ~(RTSTR_F_VALSIGNED | RTSTR_F_BIT_MASK | RTSTR_F_WIDTH | RTSTR_F_PRECISION | RTSTR_F_THOUSAND_SEP);
                         cch = RTStrFormatNumber(&szBuf[0], u.fp64.sel, 16, 4, -1, fFlags | RTSTR_F_16BIT);
                         Assert(cch == 4);
                         szBuf[4] = ':';
@@ -445,10 +494,135 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                         break;
                     }
 
+                    case RTSF_IPV4:
+                        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                           "%u.%u.%u.%u",
+                                           u.Ipv4Addr.au8[0],
+                                           u.Ipv4Addr.au8[1],
+                                           u.Ipv4Addr.au8[2],
+                                           u.Ipv4Addr.au8[3]);
+
+                    case RTSF_IPV6:
+                    {
+                        if (VALID_PTR(u.pIpv6Addr))
+                            return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                               "%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x",
+                                               u.pIpv6Addr->au8[0],
+                                               u.pIpv6Addr->au8[1],
+                                               u.pIpv6Addr->au8[2],
+                                               u.pIpv6Addr->au8[3],
+                                               u.pIpv6Addr->au8[4],
+                                               u.pIpv6Addr->au8[5],
+                                               u.pIpv6Addr->au8[6],
+                                               u.pIpv6Addr->au8[7],
+                                               u.pIpv6Addr->au8[8],
+                                               u.pIpv6Addr->au8[9],
+                                               u.pIpv6Addr->au8[10],
+                                               u.pIpv6Addr->au8[11],
+                                               u.pIpv6Addr->au8[12],
+                                               u.pIpv6Addr->au8[13],
+                                               u.pIpv6Addr->au8[14],
+                                               u.pIpv6Addr->au8[15]);
+                        return pfnOutput(pvArgOutput, s_szNull, sizeof(s_szNull) - 1);
+                    }
+
+                    case RTSF_MAC:
+                    {
+                        if (VALID_PTR(u.pMac))
+                            return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                               "%02x:%02x:%02x:%02x:%02x:%02x",
+                                               u.pMac->au8[0],
+                                               u.pMac->au8[1],
+                                               u.pMac->au8[2],
+                                               u.pMac->au8[3],
+                                               u.pMac->au8[4],
+                                               u.pMac->au8[5]);
+                        return pfnOutput(pvArgOutput, s_szNull, sizeof(s_szNull) - 1);
+                    }
+
+                    case RTSF_NETADDR:
+                    {
+                        if (VALID_PTR(u.pNetAddr))
+                        {
+                            switch (u.pNetAddr->enmType)
+                            {
+                                case RTNETADDRTYPE_IPV4:
+                                    if (u.pNetAddr->uPort == RTNETADDR_PORT_NA)
+                                        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                                           "%u.%u.%u.%u",
+                                                           u.pNetAddr->uAddr.IPv4.au8[0],
+                                                           u.pNetAddr->uAddr.IPv4.au8[1],
+                                                           u.pNetAddr->uAddr.IPv4.au8[2],
+                                                           u.pNetAddr->uAddr.IPv4.au8[3]);
+                                    return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                                       "%u.%u.%u.%u:%u",
+                                                       u.pNetAddr->uAddr.IPv4.au8[0],
+                                                       u.pNetAddr->uAddr.IPv4.au8[1],
+                                                       u.pNetAddr->uAddr.IPv4.au8[2],
+                                                       u.pNetAddr->uAddr.IPv4.au8[3],
+                                                       u.pNetAddr->uPort);
+
+                                case RTNETADDRTYPE_IPV6:
+                                    if (u.pNetAddr->uPort == RTNETADDR_PORT_NA)
+                                        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                                           "%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x",
+                                                           u.pNetAddr->uAddr.IPv6.au8[0],
+                                                           u.pNetAddr->uAddr.IPv6.au8[1],
+                                                           u.pNetAddr->uAddr.IPv6.au8[2],
+                                                           u.pNetAddr->uAddr.IPv6.au8[3],
+                                                           u.pNetAddr->uAddr.IPv6.au8[4],
+                                                           u.pNetAddr->uAddr.IPv6.au8[5],
+                                                           u.pNetAddr->uAddr.IPv6.au8[6],
+                                                           u.pNetAddr->uAddr.IPv6.au8[7],
+                                                           u.pNetAddr->uAddr.IPv6.au8[8],
+                                                           u.pNetAddr->uAddr.IPv6.au8[9],
+                                                           u.pNetAddr->uAddr.IPv6.au8[10],
+                                                           u.pNetAddr->uAddr.IPv6.au8[11],
+                                                           u.pNetAddr->uAddr.IPv6.au8[12],
+                                                           u.pNetAddr->uAddr.IPv6.au8[13],
+                                                           u.pNetAddr->uAddr.IPv6.au8[14],
+                                                           u.pNetAddr->uAddr.IPv6.au8[15]);
+                                    return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                                       "%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x:%02x%02x %u",
+                                                       u.pNetAddr->uAddr.IPv6.au8[0],
+                                                       u.pNetAddr->uAddr.IPv6.au8[1],
+                                                       u.pNetAddr->uAddr.IPv6.au8[2],
+                                                       u.pNetAddr->uAddr.IPv6.au8[3],
+                                                       u.pNetAddr->uAddr.IPv6.au8[4],
+                                                       u.pNetAddr->uAddr.IPv6.au8[5],
+                                                       u.pNetAddr->uAddr.IPv6.au8[6],
+                                                       u.pNetAddr->uAddr.IPv6.au8[7],
+                                                       u.pNetAddr->uAddr.IPv6.au8[8],
+                                                       u.pNetAddr->uAddr.IPv6.au8[9],
+                                                       u.pNetAddr->uAddr.IPv6.au8[10],
+                                                       u.pNetAddr->uAddr.IPv6.au8[11],
+                                                       u.pNetAddr->uAddr.IPv6.au8[12],
+                                                       u.pNetAddr->uAddr.IPv6.au8[13],
+                                                       u.pNetAddr->uAddr.IPv6.au8[14],
+                                                       u.pNetAddr->uAddr.IPv6.au8[15],
+                                                       u.pNetAddr->uPort);
+
+                                case RTNETADDRTYPE_MAC:
+                                    return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                                       "%02x:%02x:%02x:%02x:%02x:%02x",
+                                                       u.pNetAddr->uAddr.Mac.au8[0],
+                                                       u.pNetAddr->uAddr.Mac.au8[1],
+                                                       u.pNetAddr->uAddr.Mac.au8[2],
+                                                       u.pNetAddr->uAddr.Mac.au8[3],
+                                                       u.pNetAddr->uAddr.Mac.au8[4],
+                                                       u.pNetAddr->uAddr.Mac.au8[5]);
+
+                                default:
+                                    return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0,
+                                                       "unsupported-netaddr-type=%u", u.pNetAddr->enmType);
+
+                            }
+                        }
+                        return pfnOutput(pvArgOutput, s_szNull, sizeof(s_szNull) - 1);
+                    }
+
                     case RTSF_UUID:
                     {
-                        static const char szNull[] = "<NULL>";
-
                         if (VALID_PTR(u.pUuid))
                         {
                             /* cannot call RTUuidToStr because of GC/R0. */
@@ -466,19 +640,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                                                u.pUuid->Gen.au8Node[4],
                                                u.pUuid->Gen.au8Node[5]);
                         }
-                        return pfnOutput(pvArgOutput, szNull, sizeof(szNull) - 1);
-                    }
-
-                    case RTSF_BOOL:
-                    {
-                        static const char szTrue[]  = "true ";
-                        static const char szFalse[] = "false";
-                        if (u.u64 == 1)
-                            return pfnOutput(pvArgOutput, szTrue, sizeof(szTrue) - 1);
-                        if (u.u64 == 0)
-                            return pfnOutput(pvArgOutput, szFalse, sizeof(szFalse) - 1);
-                        /* invalid boolean value */
-                        return RTStrFormat(pfnOutput, pvArgOutput, NULL, 0, "!%lld!", u.u64);
+                        return pfnOutput(pvArgOutput, s_szNull, sizeof(s_szNull) - 1);
                     }
 
                     default:
@@ -495,13 +657,51 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
 
             /* Group 3 */
 
+            /*
+             * Base name printing.
+             */
+            case 'b':
+            {
+                switch (*(*ppszFormat)++)
+                {
+                    case 'n':
+                    {
+                        const char *pszLastSep;
+                        const char *psz = pszLastSep = va_arg(*pArgs, const char *);
+                        if (!VALID_PTR(psz))
+                            return pfnOutput(pvArgOutput, "<null>", sizeof("<null>") - 1);
+
+                        while ((ch = *psz) != '\0')
+                        {
+                            if (RTPATH_IS_SEP(ch))
+                            {
+                                do
+                                    psz++;
+                                while ((ch = *psz) != '\0' && RTPATH_IS_SEP(ch));
+                                if (!ch)
+                                    break;
+                                pszLastSep = psz;
+                            }
+                            psz++;
+                        }
+
+                        return pfnOutput(pvArgOutput, pszLastSep, psz - pszLastSep);
+                    }
+
+                    default:
+                        AssertMsgFailed(("Invalid status code format type '%.10s'!\n", pszFormatOrg));
+                        break;
+                }
+                break;
+            }
+
+
             /*
              * Pretty function / method name printing.
              */
             case 'f':
             {
-                char ch = *(*ppszFormat)++;
-                switch (ch)
+                switch (*(*ppszFormat)++)
                 {
                     /*
                      * Pretty function / method name printing.
@@ -535,7 +735,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                     }
 
                     default:
-                        AssertMsgFailed(("Invalid status code format type '%.10s'!\n", ch, pszFormatOrg));
+                        AssertMsgFailed(("Invalid status code format type '%.10s'!\n", pszFormatOrg));
                         break;
                 }
                 break;
@@ -547,8 +747,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
              */
             case 'h':
             {
-                char ch = *(*ppszFormat)++;
-                switch (ch)
+                switch (*(*ppszFormat)++)
                 {
                     /*
                      * Hex stuff.
@@ -560,8 +759,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                             cchWidth = 16;
                         if (pu8)
                         {
-                            ch = *(*ppszFormat)++;
-                            switch (ch)
+                            switch (*(*ppszFormat)++)
                             {
                                 /*
                                  * Regular hex dump.
@@ -615,7 +813,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                                 }
 
                                 default:
-                                    AssertMsgFailed(("Invalid status code format type '%.10s'!\n", ch, pszFormatOrg));
+                                    AssertMsgFailed(("Invalid status code format type '%.10s'!\n", pszFormatOrg));
                                     break;
                             }
                         }
@@ -632,11 +830,9 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                      */
                     case 'r':
                     {
-
-                        char ch = *(*ppszFormat)++;
                         uint32_t hrc = va_arg(*pArgs, uint32_t);
                         PCRTCOMERRMSG pMsg = RTErrCOMGet(hrc);
-                        switch (ch)
+                        switch (*(*ppszFormat)++)
                         {
                             case 'c':
                                 return pfnOutput(pvArgOutput, pMsg->pszDefine, strlen(pMsg->pszDefine));
@@ -653,7 +849,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
 #endif /* IN_RING3 */
 
                     default:
-                        AssertMsgFailed(("Invalid status code format type '%.10s'!\n", ch, pszFormatOrg));
+                        AssertMsgFailed(("Invalid status code format type '%.10s'!\n", pszFormatOrg));
                         return 0;
 
                 }
@@ -666,10 +862,9 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
             case 'r':
             {
                 int rc = va_arg(*pArgs, int);
-                char ch = *(*ppszFormat)++;
 #ifdef IN_RING3                         /* we don't want this anywhere else yet. */
                 PCRTSTATUSMSG pMsg = RTErrGet(rc);
-                switch (ch)
+                switch (*(*ppszFormat)++)
                 {
                     case 'c':
                         return pfnOutput(pvArgOutput, pMsg->pszDefine,    strlen(pMsg->pszDefine));
@@ -684,7 +879,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                         return 0;
                 }
 #else /* !IN_RING3 */
-                switch (ch)
+                switch (*(*ppszFormat)++)
                 {
                     case 'c':
                     case 's':
@@ -706,11 +901,10 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
             case 'w':
             {
                 long rc = va_arg(*pArgs, long);
-                char ch = *(*ppszFormat)++;
 # if defined(RT_OS_WINDOWS)
                 PCRTWINERRMSG pMsg = RTErrWinGet(rc);
 # endif
-                switch (ch)
+                switch (*(*ppszFormat)++)
                 {
 # if defined(RT_OS_WINDOWS)
                     case 'c':
@@ -828,7 +1022,7 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                 switch (s_aTypes[i].enmType)
                 {
                     case RTST_TIMESPEC:
-                        return RTStrFormat(pfnOutput, pvArgOutput, NULL, NULL, "%lld ns", RTTimeSpecGetNano(u.pTimeSpec));
+                        return RTStrFormat(pfnOutput, pvArgOutput, NULL, NULL, "%'lld ns", RTTimeSpecGetNano(u.pTimeSpec));
 
                     default:
                         AssertMsgFailed(("Invalid/unhandled enmType=%d\n", s_aTypes[i].enmType));
@@ -837,19 +1031,93 @@ size_t rtstrFormatRt(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, const char **p
                 break;
             }
 
+#ifdef IN_RING3
+            /*
+             * Group 5, XML / HTML escapers.
+             */
+            case 'M':
+            {
+                char chWhat = (*ppszFormat)[0];
+                bool fAttr  = chWhat == 'a';
+                char chType = (*ppszFormat)[1];
+                AssertMsgBreak(chWhat == 'a' || chWhat == 'e', ("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
+                *ppszFormat += 2;
+                switch (chType)
+                {
+                    case 's':
+                    {
+                        static const char   s_szElemEscape[] = "<>&\"'";
+                        static const char   s_szAttrEscape[] = "<>&\"\n\r"; /* more? */
+                        const char * const  pszEscape =  fAttr ?             s_szAttrEscape  :             s_szElemEscape;
+                        size_t       const  cchEscape = (fAttr ? RT_ELEMENTS(s_szAttrEscape) : RT_ELEMENTS(s_szElemEscape)) - 1;
+                        size_t      cchOutput = 0;
+                        const char *pszStr    = va_arg(*pArgs, char *);
+                        ssize_t     cchStr;
+                        ssize_t     offCur;
+                        ssize_t     offLast;
+
+                        if (!VALID_PTR(pszStr))
+                            pszStr = "<NULL>";
+                        cchStr = RTStrNLen(pszStr, (unsigned)cchPrecision);
+
+                        if (fAttr)
+                            cchOutput += pfnOutput(pvArgOutput, "\"", 1);
+                        if (!(fFlags & RTSTR_F_LEFT))
+                            while (--cchWidth >= cchStr)
+                                cchOutput += pfnOutput(pvArgOutput, " ", 1);
+
+                        offLast = offCur = 0;
+                        while (offCur < cchStr)
+                        {
+                            if (memchr(pszEscape, pszStr[offCur], cchEscape))
+                            {
+                                if (offLast < offCur)
+                                    cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
+                                switch (pszStr[offCur])
+                                {
+                                    case '<':   cchOutput += pfnOutput(pvArgOutput, "&lt;", 4); break;
+                                    case '>':   cchOutput += pfnOutput(pvArgOutput, "&gt;", 4); break;
+                                    case '&':   cchOutput += pfnOutput(pvArgOutput, "&amp;", 5); break;
+                                    case '\'':  cchOutput += pfnOutput(pvArgOutput, "&apos;", 6); break;
+                                    case '"':   cchOutput += pfnOutput(pvArgOutput, "&qout;", 6); break;
+                                    case '\n':  cchOutput += pfnOutput(pvArgOutput, "&#xA;", 5); break;
+                                    case '\r':  cchOutput += pfnOutput(pvArgOutput, "&#xD;", 5); break;
+                                    default:
+                                        AssertFailed();
+                                }
+                                offLast = offCur + 1;
+                            }
+                            offCur++;
+                        }
+                        if (offLast < offCur)
+                            cchOutput += pfnOutput(pvArgOutput, &pszStr[offLast], offCur - offLast);
+
+                        while (--cchWidth >= cchStr)
+                            cchOutput += pfnOutput(pvArgOutput, " ", 1);
+                        if (fAttr)
+                            cchOutput += pfnOutput(pvArgOutput, "\"", 1);
+                        return cchOutput;
+                    }
+
+                    default:
+                        AssertMsgFailed(("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
+                }
+                break;
+            }
+#endif /* IN_RING3 */
+
             /*
              * Invalid/Unknown. Bitch about it.
              */
             default:
-                AssertMsgFailed(("Invalid VBox format type '%.10s'!\n", pszFormatOrg));
+                AssertMsgFailed(("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
                 break;
         }
     }
     else
-        AssertMsgFailed(("Invalid VBox format type '%.10s'!\n", pszFormatOrg));
+        AssertMsgFailed(("Invalid IPRT format type '%.10s'!\n", pszFormatOrg));
 
     NOREF(pszFormatOrg);
     return 0;
 }
 
-
