@@ -1,10 +1,10 @@
-/* $Id: utf-8.cpp $ */
+/* $Id: utf-8.cpp 28903 2010-04-29 14:58:12Z vboxsync $ */
 /** @file
  * IPRT - UTF-8 Decoding.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -33,6 +29,8 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/string.h>
+#include "internal/iprt.h"
+
 #include <iprt/uni.h>
 #include <iprt/alloc.h>
 #include <iprt/assert.h>
@@ -53,7 +51,7 @@
  * @param   pcchActual      Where to store the actual size of the UTF-8 string
  *                          on success (cch = cb again). Optional.
  */
-static int rtUtf8Length(const char *psz, size_t cch, size_t *pcuc, size_t *pcchActual)
+int rtUtf8Length(const char *psz, size_t cch, size_t *pcuc, size_t *pcchActual)
 {
     const unsigned char *puch = (const unsigned char *)psz;
     size_t cCodePoints = 0;
@@ -186,15 +184,12 @@ static int rtUtf8Length(const char *psz, size_t cch, size_t *pcuc, size_t *pcchA
  *                  The recoding will stop when cch or '\\0' is reached. Pass RTSTR_MAX to process up to '\\0'.
  * @param   paCps   Where to store the code points array.
  * @param   cCps    The number of RTUNICP items the paCps buffer can hold, excluding the terminator ('\\0').
- * @param   pcCps   Where to store the actual number of decoded code points. This excludes the terminator.
  */
-static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps, size_t *pcCps)
+static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps)
 {
-    int                     rc = VINF_SUCCESS;
+    int                     rc   = VINF_SUCCESS;
     const unsigned char    *puch = (const unsigned char *)psz;
-    const PRTUNICP          pCpEnd = paCps + cCps;
-    PRTUNICP                pCp = paCps;
-    Assert(pCpEnd >= pCp);
+    PRTUNICP                pCp  = paCps;
     while (cch > 0)
     {
         /* read the next char and check for terminator. */
@@ -203,11 +198,12 @@ static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps
             break;
 
         /* check for output overflow */
-        if (pCp >= pCpEnd)
+        if (RT_UNLIKELY(cCps < 1))
         {
             rc = VERR_BUFFER_OVERFLOW;
             break;
         }
+        cCps--;
 
         /* decode and recode the code point */
         if (!(uch & RT_BIT(7)))
@@ -270,7 +266,6 @@ static int rtUtf8Decode(const char *psz, size_t cch, PRTUNICP paCps, size_t cCps
 
     /* done */
     *pCp = 0;
-    *pcCps = pCp - paCps;
     return rc;
 }
 
@@ -281,6 +276,7 @@ RTDECL(size_t) RTStrUniLen(const char *psz)
     int rc = rtUtf8Length(psz, RTSTR_MAX, &cCodePoints, NULL);
     return RT_SUCCESS(rc) ? cCodePoints : 0;
 }
+RT_EXPORT_SYMBOL(RTStrUniLen);
 
 
 RTDECL(int) RTStrUniLenEx(const char *psz, size_t cch, size_t *pcCps)
@@ -291,12 +287,14 @@ RTDECL(int) RTStrUniLenEx(const char *psz, size_t cch, size_t *pcCps)
         *pcCps = RT_SUCCESS(rc) ? cCodePoints : 0;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrUniLenEx);
 
 
 RTDECL(int) RTStrValidateEncoding(const char *psz)
 {
     return RTStrValidateEncodingEx(psz, RTSTR_MAX, 0);
 }
+RT_EXPORT_SYMBOL(RTStrValidateEncoding);
 
 
 RTDECL(int) RTStrValidateEncodingEx(const char *psz, size_t cch, uint32_t fFlags)
@@ -317,10 +315,8 @@ RTDECL(int) RTStrValidateEncodingEx(const char *psz, size_t cch, uint32_t fFlags
             rc = VERR_BUFFER_OVERFLOW;
     }
     return rc;
-
-
-    return RTStrUniLenEx(psz, cch, &cCpsIgnored);
 }
+RT_EXPORT_SYMBOL(RTStrValidateEncodingEx);
 
 
 RTDECL(bool) RTStrIsValidEncoding(const char *psz)
@@ -328,6 +324,7 @@ RTDECL(bool) RTStrIsValidEncoding(const char *psz)
     int rc = RTStrValidateEncodingEx(psz, RTSTR_MAX, 0);
     return RT_SUCCESS(rc);
 }
+RT_EXPORT_SYMBOL(RTStrIsValidEncoding);
 
 
 RTDECL(int) RTStrToUni(const char *pszString, PRTUNICP *ppaCps)
@@ -355,7 +352,7 @@ RTDECL(int) RTStrToUni(const char *pszString, PRTUNICP *ppaCps)
             /*
              * Decode the string.
              */
-            rc = rtUtf8Decode(pszString, RTSTR_MAX, paCps, cCps, &cCps);
+            rc = rtUtf8Decode(pszString, RTSTR_MAX, paCps, cCps);
             if (RT_SUCCESS(rc))
             {
                 *ppaCps = paCps;
@@ -368,6 +365,7 @@ RTDECL(int) RTStrToUni(const char *pszString, PRTUNICP *ppaCps)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUni);
 
 
 RTDECL(int)  RTStrToUniEx(const char *pszString, size_t cchString, PRTUNICP *ppaCps, size_t cCps, size_t *pcCps)
@@ -413,7 +411,7 @@ RTDECL(int)  RTStrToUniEx(const char *pszString, size_t cchString, PRTUNICP *ppa
             /*
              * Encode the UTF-16 string.
              */
-            rc = rtUtf8Decode(pszString, cchString, paCpsResult, cCps - 1, &cCpsResult);
+            rc = rtUtf8Decode(pszString, cchString, paCpsResult, cCps - 1);
             if (RT_SUCCESS(rc))
             {
                 *ppaCps = paCpsResult;
@@ -427,6 +425,7 @@ RTDECL(int)  RTStrToUniEx(const char *pszString, size_t cchString, PRTUNICP *ppa
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUniEx);
 
 
 /**
@@ -435,7 +434,7 @@ RTDECL(int)  RTStrToUniEx(const char *pszString, size_t cchString, PRTUNICP *ppa
  * @returns IPRT status code.
  * @param   psz     Pointer to the UTF-8 string.
  * @param   cch     The max length of the string. (btw cch = cb)
- *                  Use RTSTR_MAX if all of the string is to be examined.s
+ *                  Use RTSTR_MAX if all of the string is to be examined.
  * @param   pcwc    Where to store the length of the UTF-16 string as a number of RTUTF16 characters.
  */
 static int rtUtf8CalcUtf16Length(const char *psz, size_t cch, size_t *pcwc)
@@ -575,15 +574,12 @@ static int rtUtf8CalcUtf16Length(const char *psz, size_t cch, size_t *pcwc)
  *                  The recoding will stop when cch or '\\0' is reached. Pass RTSTR_MAX to process up to '\\0'.
  * @param   pwsz    Where to store the UTF-16 string.
  * @param   cwc     The number of RTUTF16 items the pwsz buffer can hold, excluding the terminator ('\\0').
- * @param   pcwc    Where to store the actual number of RTUTF16 items encoded into the UTF-16. This excludes the terminator.
  */
-static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_t cwc, size_t *pcwc)
+static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_t cwc)
 {
-    int                     rc = VINF_SUCCESS;
+    int                     rc   = VINF_SUCCESS;
     const unsigned char    *puch = (const unsigned char *)psz;
-    const PRTUTF16          pwszEnd = pwsz + cwc;
-    PRTUTF16                pwc = pwsz;
-    Assert(pwszEnd >= pwc);
+    PRTUTF16                pwc  = pwsz;
     while (cch > 0)
     {
         /* read the next char and check for terminator. */
@@ -592,11 +588,12 @@ static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_
             break;
 
         /* check for output overflow */
-        if (pwc >= pwszEnd)
+        if (RT_UNLIKELY(cwc < 1))
         {
             rc = VERR_BUFFER_OVERFLOW;
             break;
         }
+        cwc--;
 
         /* decode and recode the code point */
         if (!(uch & RT_BIT(7)))
@@ -630,11 +627,13 @@ static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_
                        | ((RTUNICP)(puch[2] & 0x3f) << 6)
                        | ((RTUNICP)(puch[1] & 0x3f) << 12)
                        | ((RTUNICP)(uch     & 0x07) << 18);
-            if (pwc + 1 >= pwszEnd)
+            if (RT_UNLIKELY(cwc < 1))
             {
                 rc = VERR_BUFFER_OVERFLOW;
                 break;
             }
+            cwc--;
+
             uc -= 0x10000;
             *pwc++ = 0xd800 | (uc >> 10);
             *pwc++ = 0xdc00 | (uc & 0x3ff);
@@ -645,7 +644,6 @@ static int rtUtf8RecodeAsUtf16(const char *psz, size_t cch, PRTUTF16 pwsz, size_
 
     /* done */
     *pwc = '\0';
-    *pcwc = pwc - pwsz;
     return rc;
 }
 
@@ -675,7 +673,7 @@ RTDECL(int) RTStrToUtf16(const char *pszString, PRTUTF16 *ppwszString)
             /*
              * Encode the UTF-16 string.
              */
-            rc = rtUtf8RecodeAsUtf16(pszString, RTSTR_MAX, pwsz, cwc, &cwc);
+            rc = rtUtf8RecodeAsUtf16(pszString, RTSTR_MAX, pwsz, cwc);
             if (RT_SUCCESS(rc))
             {
                 *ppwszString = pwsz;
@@ -688,6 +686,7 @@ RTDECL(int) RTStrToUtf16(const char *pszString, PRTUTF16 *ppwszString)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUtf16);
 
 
 RTDECL(int)  RTStrToUtf16Ex(const char *pszString, size_t cchString, PRTUTF16 *ppwsz, size_t cwc, size_t *pcwc)
@@ -733,7 +732,7 @@ RTDECL(int)  RTStrToUtf16Ex(const char *pszString, size_t cchString, PRTUTF16 *p
             /*
              * Encode the UTF-16 string.
              */
-            rc = rtUtf8RecodeAsUtf16(pszString, cchString, pwszResult, cwc - 1, &cwcResult);
+            rc = rtUtf8RecodeAsUtf16(pszString, cchString, pwszResult, cwc - 1);
             if (RT_SUCCESS(rc))
             {
                 *ppwsz = pwszResult;
@@ -747,6 +746,7 @@ RTDECL(int)  RTStrToUtf16Ex(const char *pszString, size_t cchString, PRTUTF16 *p
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrToUtf16Ex);
 
 
 RTDECL(size_t) RTStrCalcUtf16Len(const char *psz)
@@ -755,6 +755,7 @@ RTDECL(size_t) RTStrCalcUtf16Len(const char *psz)
     int rc = rtUtf8CalcUtf16Length(psz, RTSTR_MAX, &cwc);
     return RT_SUCCESS(rc) ? cwc : 0;
 }
+RT_EXPORT_SYMBOL(RTStrCalcUtf16Len);
 
 
 RTDECL(int) RTStrCalcUtf16LenEx(const char *psz, size_t cch, size_t *pcwc)
@@ -765,6 +766,7 @@ RTDECL(int) RTStrCalcUtf16LenEx(const char *psz, size_t cch, size_t *pcwc)
         *pcwc = RT_SUCCESS(rc) ? cwc : ~(size_t)0;
     return rc;
 }
+RT_EXPORT_SYMBOL(RTStrCalcUtf16LenEx);
 
 
 /**
@@ -791,6 +793,7 @@ RTDECL(RTUNICP) RTStrGetCpInternal(const char *psz)
     RTStrGetCpExInternal(&psz, &Cp);
     return Cp;
 }
+RT_EXPORT_SYMBOL(RTStrGetCpInternal);
 
 
 RTDECL(int) RTStrGetCpExInternal(const char **ppsz, PRTUNICP pCp)
@@ -913,6 +916,7 @@ RTDECL(int) RTStrGetCpExInternal(const char **ppsz, PRTUNICP pCp)
     *ppsz = (const char *)puch;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTStrGetCpExInternal);
 
 
 /**
@@ -1068,6 +1072,7 @@ RTDECL(int) RTStrGetCpNExInternal(const char **ppsz, size_t *pcch, PRTUNICP pCp)
     (*pcch) = cch;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTStrGetCpNExInternal);
 
 
 RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
@@ -1105,7 +1110,7 @@ RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
     }
     else if (uc < 0x04000000)
     {
-        *puch++ = 0xf1 | (uc >> 24);
+        *puch++ = 0xf8 | (uc >> 24);
         *puch++ = 0x80 | ((uc >> 18) & 0x3f);
         *puch++ = 0x80 | ((uc >> 12) & 0x3f);
         *puch++ = 0x80 | ((uc >> 6) & 0x3f);
@@ -1113,7 +1118,7 @@ RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
     }
     else if (uc <= 0x7fffffff)
     {
-        *puch++ = 0xf3 | (uc >> 30);
+        *puch++ = 0xfc | (uc >> 30);
         *puch++ = 0x80 | ((uc >> 24) & 0x3f);
         *puch++ = 0x80 | ((uc >> 18) & 0x3f);
         *puch++ = 0x80 | ((uc >> 12) & 0x3f);
@@ -1128,6 +1133,7 @@ RTDECL(char *) RTStrPutCpInternal(char *psz, RTUNICP uc)
 
     return (char *)puch;
 }
+RT_EXPORT_SYMBOL(RTStrPutCpInternal);
 
 
 RTDECL(char *) RTStrPrevCp(const char *pszStart, const char *psz)
@@ -1146,7 +1152,7 @@ RTDECL(char *) RTStrPrevCp(const char *pszStart, const char *psz)
         while (     (const unsigned char *)pszStart < puch
                &&   !(uMask & 1))
         {
-            unsigned uch = *--puch;
+            uch = *--puch;
             if ((uch & 0xc0) != 0x80)
             {
                 RTStrAssertMsgReturn((uch & (uMask >> 1)) == (uMask & 0xff),
@@ -1160,342 +1166,5 @@ RTDECL(char *) RTStrPrevCp(const char *pszStart, const char *psz)
     }
     return (char *)pszStart;
 }
-
-
-/**
- * Performs a case sensitive string compare between two UTF-8 strings.
- *
- * Encoding errors are ignored by the current implementation. So, the only
- * difference between this and the CRT strcmp function is the handling of
- * NULL arguments.
- *
- * @returns < 0 if the first string less than the second string.
- * @returns 0 if the first string identical to the second string.
- * @returns > 0 if the first string greater than the second string.
- * @param   psz1        First UTF-8 string. Null is allowed.
- * @param   psz2        Second UTF-8 string. Null is allowed.
- */
-RTDECL(int) RTStrCmp(const char *psz1, const char *psz2)
-{
-    if (psz1 == psz2)
-        return 0;
-    if (!psz1)
-        return -1;
-    if (!psz2)
-        return 1;
-
-    return strcmp(psz1, psz2);
-}
-
-
-/**
- * Performs a case sensitive string compare between two UTF-8 strings, given
- * a maximum string length.
- *
- * Encoding errors are ignored by the current implementation. So, the only
- * difference between this and the CRT strncmp function is the handling of
- * NULL arguments.
- *
- * @returns < 0 if the first string less than the second string.
- * @returns 0 if the first string identical to the second string.
- * @returns > 0 if the first string greater than the second string.
- * @param   psz1        First UTF-8 string. Null is allowed.
- * @param   psz2        Second UTF-8 string. Null is allowed.
- * @param   cchMax      The maximum string length
- */
-RTDECL(int) RTStrNCmp(const char *psz1, const char *psz2, size_t cchMax)
-{
-    if (psz1 == psz2)
-        return 0;
-    if (!psz1)
-        return -1;
-    if (!psz2)
-        return 1;
-
-    return strncmp(psz1, psz2, cchMax);
-}
-
-
-/**
- * Performs a case insensitive string compare between two UTF-8 strings.
- *
- * This is a simplified compare, as only the simplified lower/upper case folding
- * specified by the unicode specs are used. It does not consider character pairs
- * as they are used in some languages, just simple upper & lower case compares.
- *
- * The result is the difference between the mismatching codepoints after they
- * both have been lower cased.
- *
- * If the string encoding is invalid the function will assert (strict builds)
- * and use RTStrCmp for the remainder of the string.
- *
- * @returns < 0 if the first string less than the second string.
- * @returns 0 if the first string identical to the second string.
- * @returns > 0 if the first string greater than the second string.
- * @param   psz1        First UTF-8 string. Null is allowed.
- * @param   psz2        Second UTF-8 string. Null is allowed.
- */
-RTDECL(int) RTStrICmp(const char *psz1, const char *psz2)
-{
-    if (psz1 == psz2)
-        return 0;
-    if (!psz1)
-        return -1;
-    if (!psz2)
-        return 1;
-
-    const char *pszStart1 = psz1;
-    for (;;)
-    {
-        /* Get the codepoints */
-        RTUNICP cp1;
-        int rc = RTStrGetCpEx(&psz1, &cp1);
-        if (RT_FAILURE(rc))
-        {
-            AssertRC(rc);
-            psz1--;
-            break;
-        }
-
-        RTUNICP cp2;
-        rc = RTStrGetCpEx(&psz2, &cp2);
-        if (RT_FAILURE(rc))
-        {
-            AssertRC(rc);
-            psz2--;
-            psz1 = RTStrPrevCp(pszStart1, psz1);
-            break;
-        }
-
-        /* compare */
-        int iDiff = cp1 - cp2;
-        if (iDiff)
-        {
-            iDiff = RTUniCpToUpper(cp1) != RTUniCpToUpper(cp2);
-            if (iDiff)
-            {
-                iDiff = RTUniCpToLower(cp1) - RTUniCpToLower(cp2); /* lower case diff last! */
-                if (iDiff)
-                    return iDiff;
-            }
-        }
-
-        /* hit the terminator? */
-        if (!cp1)
-            return 0;
-    }
-
-    /* Hit some bad encoding, continue in case insensitive mode. */
-    return RTStrCmp(psz1, psz2);
-}
-
-
-/**
- * Performs a case insensitive string compare between two UTF-8 strings, given a
- * maximum string length.
- *
- * This is a simplified compare, as only the simplified lower/upper case folding
- * specified by the unicode specs are used. It does not consider character pairs
- * as they are used in some languages, just simple upper & lower case compares.
- *
- * The result is the difference between the mismatching codepoints after they
- * both have been lower cased.
- *
- * If the string encoding is invalid the function will assert (strict builds)
- * and use RTStrCmp for the remainder of the string.
- *
- * @returns < 0 if the first string less than the second string.
- * @returns 0 if the first string identical to the second string.
- * @returns > 0 if the first string greater than the second string.
- * @param   psz1        First UTF-8 string. Null is allowed.
- * @param   psz2        Second UTF-8 string. Null is allowed.
- * @param   cchMax      Maximum string length
- */
-RTDECL(int) RTStrNICmp(const char *psz1, const char *psz2, size_t cchMax)
-{
-    if (cchMax == 0)
-        return 0;
-    if (psz1 == psz2)
-        return 0;
-    if (!psz1)
-        return -1;
-    if (!psz2)
-        return 1;
-
-    for (;;)
-    {
-        /* Get the codepoints */
-        RTUNICP cp1;
-        size_t cchMax2 = cchMax;
-        int rc = RTStrGetCpNEx(&psz1, &cchMax, &cp1);
-        if (RT_FAILURE(rc))
-        {
-            AssertRC(rc);
-            psz1--;
-            cchMax++;
-            break;
-        }
-
-        RTUNICP cp2;
-        rc = RTStrGetCpNEx(&psz2, &cchMax2, &cp2);
-        if (RT_FAILURE(rc))
-        {
-            AssertRC(rc);
-            psz2--;
-            psz1 -= (cchMax - cchMax2 + 1);  /* This can't overflow, can it? */
-            cchMax = cchMax2 + 1;
-            break;
-        }
-
-        /* compare */
-        int iDiff = cp1 - cp2;
-        if (iDiff)
-        {
-            iDiff = RTUniCpToUpper(cp1) != RTUniCpToUpper(cp2);
-            if (iDiff)
-            {
-                iDiff = RTUniCpToLower(cp1) - RTUniCpToLower(cp2); /* lower case diff last! */
-                if (iDiff)
-                    return iDiff;
-            }
-        }
-
-        /* hit the terminator? */
-        if (!cp1 || cchMax == 0)
-            return 0;
-    }
-
-    /* Hit some bad encoding, continue in case insensitive mode. */
-    return RTStrNCmp(psz1, psz2, cchMax);
-}
-
-
-RTDECL(char *) RTStrStr(const char *pszHaystack, const char *pszNeedle)
-{
-    /* Any NULL strings means NULL return. (In the RTStrCmp tradition.) */
-    if (!pszHaystack)
-        return NULL;
-    if (!pszNeedle)
-        return NULL;
-
-    /* The rest is CRT. */
-    return (char *)strstr(pszHaystack, pszNeedle);
-}
-
-
-RTDECL(char *) RTStrIStr(const char *pszHaystack, const char *pszNeedle)
-{
-    /* Any NULL strings means NULL return. (In the RTStrCmp tradition.) */
-    if (!pszHaystack)
-        return NULL;
-    if (!pszNeedle)
-        return NULL;
-
-    /* The empty string matches everything. */
-    if (!*pszNeedle)
-        return (char *)pszHaystack;
-
-    /*
-     * The search strategy is to pick out the first char of the needle, fold it,
-     * and match it against the haystack code point by code point. When encountering
-     * a matching code point we use RTStrNICmp for the remainder (if any) of the needle.
-     */
-    const char * const pszNeedleStart = pszNeedle;
-    RTUNICP Cp0;
-    RTStrGetCpEx(&pszNeedle, &Cp0);     /* pszNeedle is advanced one code point. */
-    size_t const    cchNeedle   = strlen(pszNeedle);
-    size_t const    cchNeedleCp0= pszNeedle - pszNeedleStart;
-    RTUNICP const   Cp0Lower    = RTUniCpToLower(Cp0);
-    RTUNICP const   Cp0Upper    = RTUniCpToUpper(Cp0);
-    if (    Cp0Lower == Cp0Upper
-        &&  Cp0Lower == Cp0)
-    {
-        /* Cp0 is not a case sensitive char. */
-        for (;;)
-        {
-            RTUNICP Cp;
-            RTStrGetCpEx(&pszHaystack, &Cp);
-            if (!Cp)
-                break;
-            if (    Cp == Cp0
-                &&  !RTStrNICmp(pszHaystack, pszNeedle, cchNeedle))
-                return (char *)pszHaystack - cchNeedleCp0;
-        }
-    }
-    else if (   Cp0Lower == Cp0
-             || Cp0Upper != Cp0)
-    {
-        /* Cp0 is case sensitive */
-        for (;;)
-        {
-            RTUNICP Cp;
-            RTStrGetCpEx(&pszHaystack, &Cp);
-            if (!Cp)
-                break;
-            if (    (   Cp == Cp0Upper
-                     || Cp == Cp0Lower)
-                &&  !RTStrNICmp(pszHaystack, pszNeedle, cchNeedle))
-                return (char *)pszHaystack - cchNeedleCp0;
-        }
-    }
-    else
-    {
-        /* Cp0 is case sensitive and folds to two difference chars. (paranoia) */
-        for (;;)
-        {
-            RTUNICP Cp;
-            RTStrGetCpEx(&pszHaystack, &Cp);
-            if (!Cp)
-                break;
-            if (    (   Cp == Cp0
-                     || Cp == Cp0Upper
-                     || Cp == Cp0Lower)
-                &&  !RTStrNICmp(pszHaystack, pszNeedle, cchNeedle))
-                return (char *)pszHaystack - cchNeedleCp0;
-        }
-    }
-
-
-    return NULL;
-}
-
-
-RTDECL(char *) RTStrToLower(char *psz)
-{
-    /*
-     * Loop the code points in the string, converting them one by one.
-     * ASSUMES that the code points for upper and lower case are encoded
-     *         with the exact same length.
-     */
-    /** @todo Handled bad encodings correctly+quietly, remove assumption,
-     *        optimize. */
-    char *pszCur = psz;
-    while (*pszCur)
-    {
-        RTUNICP cp = RTStrGetCp(pszCur);
-        cp = RTUniCpToLower(cp);
-        pszCur = RTStrPutCp(pszCur, cp);
-    }
-    return psz;
-}
-
-
-RTDECL(char *) RTStrToUpper(char *psz)
-{
-    /*
-     * Loop the code points in the string, converting them one by one.
-     * ASSUMES that the code points for upper and lower case are encoded
-     *         with the exact same length.
-     */
-    /** @todo Handled bad encodings correctly+quietly, remove assumption,
-     *        optimize. */
-    char *pszCur = psz;
-    while(*pszCur)
-    {
-        RTUNICP cp = RTStrGetCp(pszCur);
-        cp = RTUniCpToUpper(cp);
-        pszCur = RTStrPutCp(pszCur, cp);
-    }
-    return psz;
-}
+RT_EXPORT_SYMBOL(RTStrPrevCp);
 
