@@ -1,10 +1,10 @@
-/* $Id: time.cpp 14318 2008-11-18 16:56:53Z vboxsync $ */
+/* $Id: time.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Time.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -34,6 +30,8 @@
 *******************************************************************************/
 #define LOG_GROUP RTLOGGROUP_TIME
 #include <iprt/time.h>
+#include "internal/iprt.h"
+
 #include <iprt/string.h>
 #include <iprt/assert.h>
 #include "internal/time.h"
@@ -234,6 +232,7 @@ DECLINLINE(bool) rtTimeIsLeapYear(int32_t i32Year)
             ||  i32Year % 400 == 0);
 }
 
+
 /**
  * Checks if a year is a leap year or not.
  *
@@ -245,6 +244,7 @@ RTDECL(bool) RTTimeIsLeapYear(int32_t i32Year)
 {
     return rtTimeIsLeapYear(i32Year);
 }
+RT_EXPORT_SYMBOL(RTTimeIsLeapYear);
 
 
 /**
@@ -350,6 +350,7 @@ RTDECL(PRTTIME) RTTimeExplode(PRTTIME pTime, PCRTTIMESPEC pTimeSpec)
 
     return pTime;
 }
+RT_EXPORT_SYMBOL(RTTimeExplode);
 
 
 /**
@@ -401,6 +402,7 @@ RTDECL(PRTTIMESPEC) RTTimeImplode(PRTTIMESPEC pTimeSpec, PCRTTIME pTime)
     pTimeSpec->i64NanosecondsRelativeToUnixEpoch = i64Nanos;
     return pTimeSpec;
 }
+RT_EXPORT_SYMBOL(RTTimeImplode);
 
 
 /**
@@ -651,6 +653,7 @@ RTDECL(PRTTIME) RTTimeNormalize(PRTTIME pTime)
         pTime->fFlags |= RTTIME_FLAGS_TYPE_UTC;
     return pTime;
 }
+RT_EXPORT_SYMBOL(RTTimeNormalize);
 
 
 /**
@@ -700,6 +703,7 @@ RTDECL(char *) RTTimeToString(PCRTTIME pTime, char *psz, size_t cb)
     }
     return psz;
 }
+RT_EXPORT_SYMBOL(RTTimeToString);
 
 
 /**
@@ -716,4 +720,5 @@ RTDECL(char *) RTTimeSpecToString(PCRTTIMESPEC pTime, char *psz, size_t cb)
     RTTIME Time;
     return RTTimeToString(RTTimeExplode(&Time, pTime), psz, cb);
 }
+RT_EXPORT_SYMBOL(RTTimeSpecToString);
 
