@@ -1,10 +1,10 @@
-/* $Id: RTLogDefaultInit-generic.cpp 8245 2008-04-21 17:24:28Z vboxsync $ */
+/* $Id: RTLogDefaultInit-generic.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Default Log Init, Generic Dummy.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,17 +22,18 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
 #include <iprt/log.h>
 
+
 RTDECL(PRTLOGGER) RTLogDefaultInit(void)
 {
     return NULL;
 }
-
+RT_EXPORT_SYMBOL(RTLogDefaultInit);
 
