@@ -1,10 +1,10 @@
-/* $Id: env-generic.cpp 8245 2008-04-21 17:24:28Z vboxsync $ */
+/* $Id: env-generic.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Environment, Generic.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -33,10 +29,13 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/env.h>
+#include "internal/iprt.h"
+
 #include <iprt/assert.h>
 #include <iprt/alloc.h>
 #include <iprt/alloca.h>
 #include <iprt/string.h>
+#include <iprt/sort.h>
 #include <iprt/err.h>
 #include "internal/magics.h"
 
@@ -48,11 +47,12 @@
 # include <crt_externs.h>
 #endif
 #if defined(RT_OS_SOLARIS) || defined(RT_OS_FREEBSD) || defined(RT_OS_NETBSD) || defined(RT_OS_OPENBSD)
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 extern char **environ;
-__END_DECLS
+RT_C_DECLS_END
 #endif
 
+
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
@@ -72,16 +72,16 @@ typedef struct RTENVINTERNAL
 {
     /** Magic value . */
     uint32_t    u32Magic;
-    /** Number of variables in the array. 
+    /** Number of variables in the array.
      * This does not include the terminating NULL entry. */
     size_t      cVars;
-    /** Capacity (allocated size) of the array. 
-     * This includes space for the terminating NULL element (for compatibility 
+    /** Capacity (allocated size) of the array.
+     * This includes space for the terminating NULL element (for compatibility
      * with the C library), so that c <= cCapacity - 1. */
     size_t      cAllocated;
     /** Array of environment variables. */
     char      **papszEnv;
-    /** Array of environment variables in the process CP. 
+    /** Array of environment variables in the process CP.
      * This get (re-)constructed when RTEnvGetExecEnvP method is called. */
     char      **papszEnvOtherCP;
 } RTENVINTERNAL, *PRTENVINTERNAL;
@@ -91,9 +91,9 @@ typedef struct RTENVINTERNAL
 
 
 /**
- * Internal worker that resolves the pointer to the default 
+ * Internal worker that resolves the pointer to the default
  * process environment. (environ)
- * 
+ *
  * @returns Pointer to the default environment.
  *          This may be NULL.
  */
@@ -101,18 +101,18 @@ static const char * const *rtEnvDefault(void)
 {
 #ifdef RT_OS_DARWIN
     return *(_NSGetEnviron());
-#elif defined(RT_OS_L4)  
+#elif defined(RT_OS_L4)
     /* So far, our L4 libraries do not include environment support. */
     return NULL;
 #else
     return environ;
-#endif 
+#endif
 }
 
 
 /**
  * Internal worker that creates an environment handle with a specified capacity.
- * 
+ *
  * @returns IPRT status code.
  * @param   ppIntEnv    Where to store the result.
  * @param   cAllocated  The initial array size.
@@ -140,7 +140,7 @@ static int rtEnvCreate(PRTENVINTERNAL *ppIntEnv, size_t cAllocated)
         }
 
         RTMemFree(pIntEnv);
-    }    
+    }
 
     return VERR_NO_MEMORY;
 }
@@ -151,11 +151,12 @@ RTDECL(int) RTEnvCreate(PRTENV pEnv)
     AssertPtrReturn(pEnv, VERR_INVALID_POINTER);
     return rtEnvCreate(pEnv, RTENV_GROW_SIZE);
 }
+RT_EXPORT_SYMBOL(RTEnvCreate);
 
 
 RTDECL(int) RTEnvDestroy(RTENV Env)
 {
-    /* 
+    /*
      * Ignore NIL_RTENV and validate input.
      */
     if (    Env == NIL_RTENV
@@ -166,7 +167,7 @@ RTDECL(int) RTEnvDestroy(RTENV Env)
     AssertPtrReturn(pIntEnv, VERR_INVALID_HANDLE);
     AssertReturn(pIntEnv->u32Magic == RTENV_MAGIC, VERR_INVALID_HANDLE);
 
-    /* 
+    /*
      * Do the cleanup.
      */
     RTENV_LOCK(pIntEnv);
@@ -191,9 +192,10 @@ RTDECL(int) RTEnvDestroy(RTENV Env)
     RTENV_UNLOCK(pIntEnv);
     /*RTCritSectDelete(&pIntEnv->CritSect) */
     RTMemFree(pIntEnv);
-    
+
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTEnvDestroy);
 
 
 RTDECL(int) RTEnvClone(PRTENV pEnv, RTENV EnvToClone)
@@ -239,12 +241,12 @@ RTDECL(int) RTEnvClone(PRTENV pEnv, RTENV EnvToClone)
             /* ASSUMES the default environment is in the current codepage. */
             for (size_t iVar = 0; iVar < cVars; iVar++)
             {
-                int rc = RTStrCurrentCPToUtf8(&pIntEnv->papszEnv[iVar], papszEnv[iVar]);
-                if (RT_FAILURE(rc))
+                int rc2 = RTStrCurrentCPToUtf8(&pIntEnv->papszEnv[iVar], papszEnv[iVar]);
+                if (RT_FAILURE(rc2))
                 {
                     pIntEnv->cVars = iVar;
                     RTEnvDestroy(pIntEnv);
-                    return rc;
+                    return rc2;
                 }
             }
         }
@@ -259,7 +261,7 @@ RTDECL(int) RTEnvClone(PRTENV pEnv, RTENV EnvToClone)
 
                     pIntEnv->cVars = iVar;
                     RTEnvDestroy(pIntEnv);
-                    return rc;
+                    return VERR_NO_STR_MEMORY;
                 }
                 pIntEnv->papszEnv[iVar] = pszVar;
             }
@@ -273,6 +275,7 @@ RTDECL(int) RTEnvClone(PRTENV pEnv, RTENV EnvToClone)
         RTENV_UNLOCK(pIntEnvToClone);
     return rc;
 }
+RT_EXPORT_SYMBOL(RTEnvClone);
 
 
 RTDECL(int) RTEnvPutEx(RTENV Env, const char *pszVarEqualValue)
@@ -284,7 +287,7 @@ RTDECL(int) RTEnvPutEx(RTENV Env, const char *pszVarEqualValue)
         rc = RTEnvUnsetEx(Env, pszVarEqualValue);
     else
     {
-        /* 
+        /*
          * Make a copy of the variable name so we can terminate it
          * properly and then pass the request on to RTEnvSetEx.
          */
@@ -300,6 +303,7 @@ RTDECL(int) RTEnvPutEx(RTENV Env, const char *pszVarEqualValue)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTEnvPutEx);
 
 
 RTDECL(int) RTEnvSetEx(RTENV Env, const char *pszVar, const char *pszValue)
@@ -311,9 +315,9 @@ RTDECL(int) RTEnvSetEx(RTENV Env, const char *pszVar, const char *pszValue)
     int rc;
     if (Env == RTENV_DEFAULT)
     {
-        /* 
+        /*
          * Since RTEnvPut isn't UTF-8 clean and actually expects the strings
-         * to be in the current code page (codeset), we'll do the necessary 
+         * to be in the current code page (codeset), we'll do the necessary
          * conversions here.
          */
         char *pszVarOtherCP;
@@ -361,7 +365,7 @@ RTDECL(int) RTEnvSetEx(RTENV Env, const char *pszVar, const char *pszValue)
                     break;
             if (iVar < pIntEnv->cVars)
             {
-                /* 
+                /*
                  * Replace the current entry. Simple.
                  */
                 RTMemFree(pIntEnv->papszEnv[iVar]);
@@ -370,7 +374,7 @@ RTDECL(int) RTEnvSetEx(RTENV Env, const char *pszVar, const char *pszValue)
             else
             {
                 /*
-                 * Adding a new variable. Resize the array if required 
+                 * Adding a new variable. Resize the array if required
                  * and then insert the new value at the end.
                  */
                 if (pIntEnv->cVars + 2 > pIntEnv->cAllocated)
@@ -394,7 +398,7 @@ RTDECL(int) RTEnvSetEx(RTENV Env, const char *pszVar, const char *pszValue)
                     Assert(pIntEnv->cVars == iVar + 1);
                 }
             }
-    
+
             RTENV_UNLOCK(pIntEnv);
 
             if (RT_FAILURE(rc))
@@ -405,6 +409,7 @@ RTDECL(int) RTEnvSetEx(RTENV Env, const char *pszVar, const char *pszValue)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTEnvSetEx);
 
 
 RTDECL(int) RTEnvUnsetEx(RTENV Env, const char *pszVar)
@@ -415,9 +420,9 @@ RTDECL(int) RTEnvUnsetEx(RTENV Env, const char *pszVar)
     int rc;
     if (Env == RTENV_DEFAULT)
     {
-        /* 
+        /*
          * Since RTEnvUnset isn't UTF-8 clean and actually expects the strings
-         * to be in the current code page (codeset), we'll do the necessary 
+         * to be in the current code page (codeset), we'll do the necessary
          * conversions here.
          */
         char *pszVarOtherCP;
@@ -458,8 +463,9 @@ RTDECL(int) RTEnvUnsetEx(RTENV Env, const char *pszVar)
         RTENV_UNLOCK(pIntEnv);
     }
     return rc;
-    
+
 }
+RT_EXPORT_SYMBOL(RTEnvUnsetEx);
 
 
 RTDECL(int) RTEnvGetEx(RTENV Env, const char *pszVar, char *pszValue, size_t cbValue, size_t *pcchActual)
@@ -474,9 +480,9 @@ RTDECL(int) RTEnvGetEx(RTENV Env, const char *pszVar, char *pszValue, size_t cbV
     int rc;
     if (Env == RTENV_DEFAULT)
     {
-        /* 
+        /*
          * Since RTEnvGet isn't UTF-8 clean and actually expects the strings
-         * to be in the current code page (codeset), we'll do the necessary 
+         * to be in the current code page (codeset), we'll do the necessary
          * conversions here.
          */
         char *pszVarOtherCP;
@@ -544,8 +550,9 @@ RTDECL(int) RTEnvGetEx(RTENV Env, const char *pszVar, char *pszValue, size_t cbV
         RTENV_UNLOCK(pIntEnv);
     }
     return rc;
-    
+
 }
+RT_EXPORT_SYMBOL(RTEnvGetEx);
 
 
 RTDECL(bool) RTEnvExistEx(RTENV Env, const char *pszVar)
@@ -555,9 +562,9 @@ RTDECL(bool) RTEnvExistEx(RTENV Env, const char *pszVar)
     bool fExist = false;
     if (Env == RTENV_DEFAULT)
     {
-        /* 
+        /*
          * Since RTEnvExist isn't UTF-8 clean and actually expects the strings
-         * to be in the current code page (codeset), we'll do the necessary 
+         * to be in the current code page (codeset), we'll do the necessary
          * conversions here.
          */
         char *pszVarOtherCP;
@@ -592,6 +599,7 @@ RTDECL(bool) RTEnvExistEx(RTENV Env, const char *pszVar)
     }
     return fExist;
 }
+RT_EXPORT_SYMBOL(RTEnvExistEx);
 
 
 RTDECL(char const * const *) RTEnvGetExecEnvP(RTENV Env)
@@ -614,8 +622,8 @@ RTDECL(char const * const *) RTEnvGetExecEnvP(RTENV Env)
 
         RTENV_LOCK(pIntEnv);
 
-        /* 
-         * Free any old envp. 
+        /*
+         * Free any old envp.
          */
         if (pIntEnv->papszEnvOtherCP)
         {
@@ -654,3 +662,121 @@ RTDECL(char const * const *) RTEnvGetExecEnvP(RTENV Env)
     }
     return papszRet;
 }
+RT_EXPORT_SYMBOL(RTEnvGetExecEnvP);
+
+
+/**
+ * RTSort callback for comparing two environment varibles.
+ *
+ * @returns -1, 0, 1. See PFNRTSORTCMP.
+ * @param   pvElement1          Variable 1.
+ * @param   pvElement2          Variable 2.
+ * @param   pvUser              Ignored.
+ */
+DECLCALLBACK(int) rtEnvSortCompare(const void *pvElement1, const void *pvElement2, void *pvUser)
+{
+    NOREF(pvUser);
+    int iDiff = strcmp((const char *)pvElement1, (const char *)pvElement2);
+    if (iDiff < 0)
+        iDiff = -1;
+    else if (iDiff > 0)
+        iDiff = 1;
+    return iDiff;
+}
+
+
+RTDECL(int) RTEnvQueryUtf16Block(RTENV hEnv, PRTUTF16 *ppwszzBlock)
+{
+    RTENV           hClone  = NIL_RTENV;
+    PRTENVINTERNAL  pIntEnv;
+    int             rc;
+
+    /*
+     * Validate / simplify input.
+     */
+    if (hEnv == RTENV_DEFAULT)
+    {
+        rc = RTEnvClone(&hClone, RTENV_DEFAULT);
+        if (RT_FAILURE(rc))
+            return rc;
+        pIntEnv = hClone;
+    }
+    else
+    {
+        pIntEnv = hEnv;
+        AssertPtrReturn(pIntEnv, VERR_INVALID_HANDLE);
+        AssertReturn(pIntEnv->u32Magic == RTENV_MAGIC, VERR_INVALID_HANDLE);
+        rc = VINF_SUCCESS;
+    }
+
+    RTENV_LOCK(pIntEnv);
+
+    /*
+     * Sort it first.
+     */
+    RTSortApvShell((void **)pIntEnv->papszEnv, pIntEnv->cVars, rtEnvSortCompare, pIntEnv);
+
+    /*
+     * Calculate the size.
+     */
+    size_t cwc;
+    size_t cwcTotal = 2;
+    for (size_t iVar = 0; iVar < pIntEnv->cVars; iVar++)
+    {
+        rc = RTStrCalcUtf16LenEx(pIntEnv->papszEnv[iVar], RTSTR_MAX, &cwc);
+        AssertRCBreak(rc);
+        cwcTotal += cwc + 1;
+    }
+
+    PRTUTF16 pwszzBlock = NULL;
+    if (RT_SUCCESS(rc))
+    {
+        /*
+         * Perform the conversion.
+         */
+        PRTUTF16 pwszz = pwszzBlock = (PRTUTF16)RTMemAlloc(cwcTotal * sizeof(RTUTF16));
+        if (pwszz)
+        {
+            size_t cwcLeft = cwcTotal;
+            for (size_t iVar = 0; iVar < pIntEnv->cVars; iVar++)
+            {
+                rc = RTStrToUtf16Ex(pIntEnv->papszEnv[iVar], RTSTR_MAX,
+                                    &pwszz, cwcTotal - (pwszz - pwszzBlock), &cwc);
+                AssertRCBreak(rc);
+                pwszz   += cwc + 1;
+                cwcLeft -= cwc + 1;
+                AssertBreakStmt(cwcLeft >= 2, rc = VERR_INTERNAL_ERROR_3);
+            }
+            AssertStmt(cwcLeft == 2 || RT_FAILURE(rc), rc = VERR_INTERNAL_ERROR_2);
+            if (RT_SUCCESS(rc))
+            {
+                pwszz[0] = '\0';
+                pwszz[1] = '\0';
+            }
+            else
+            {
+                RTMemFree(pwszzBlock);
+                pwszzBlock = NULL;
+            }
+        }
+        else
+            rc = VERR_NO_MEMORY;
+    }
+
+    RTENV_UNLOCK(pIntEnv);
+
+    if (hClone != NIL_RTENV)
+        RTEnvDestroy(hClone);
+    if (RT_SUCCESS(rc))
+        *ppwszzBlock = pwszzBlock;
+    return rc;
+}
+RT_EXPORT_SYMBOL(RTEnvGetExecEnvP);
+
+
+RTDECL(void) RTEnvFreeUtf16Block(PRTUTF16 pwszzBlock)
+{
+    RTMemFree(pwszzBlock);
+}
+RT_EXPORT_SYMBOL(RTEnvFreeUtf16Block);
+
