@@ -1,10 +1,10 @@
-/* $Id: spinlock-generic.cpp 8245 2008-04-21 17:24:28Z vboxsync $ */
+/* $Id: spinlock-generic.cpp 29271 2010-05-09 21:25:16Z vboxsync $ */
 /** @file
  * IPRT - Spinlock, generic implementation.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -42,8 +38,13 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/spinlock.h>
+#include "internal/iprt.h"
+
 #include <iprt/alloc.h>
 #include <iprt/asm.h>
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
+# include <iprt/asm-amd64-x86.h>
+#endif
 #include <iprt/err.h>
 #include <iprt/assert.h>
 #if RT_CFG_SPINLOCK_GENERIC_DO_SLEEP
@@ -87,6 +88,7 @@ RTDECL(int)  RTSpinlockCreate(PRTSPINLOCK pSpinlock)
     *pSpinlock = pSpinlockInt;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTSpinlockCreate);
 
 
 RTDECL(int)  RTSpinlockDestroy(RTSPINLOCK Spinlock)
@@ -107,6 +109,7 @@ RTDECL(int)  RTSpinlockDestroy(RTSPINLOCK Spinlock)
     RTMemFree(pSpinlockInt);
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTSpinlockDestroy);
 
 
 RTDECL(void) RTSpinlockAcquireNoInts(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
@@ -115,22 +118,31 @@ RTDECL(void) RTSpinlockAcquireNoInts(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
     AssertMsg(pSpinlockInt && pSpinlockInt->u32Magic == RTSPINLOCK_MAGIC,
               ("pSpinlockInt=%p u32Magic=%08x\n", pSpinlockInt, pSpinlockInt ? (int)pSpinlockInt->u32Magic : 0));
 
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
     pTmp->uFlags = ASMGetFlags();
+#else
+    pTmp->uFlags = 0;
+#endif
 #if RT_CFG_SPINLOCK_GENERIC_DO_SLEEP
     for (;;)
     {
+# if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
         ASMIntDisable();
+# endif
         for (int c = RT_CFG_SPINLOCK_GENERIC_DO_SLEEP; c > 0; c--)
             if (ASMAtomicCmpXchgU32(&pSpinlockInt->fLocked, 1, 0))
                 return;
         RTThreadYield();
     }
 #else
+# if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
     ASMIntDisable();
+# endif
     while (!ASMAtomicCmpXchgU32(&pSpinlockInt->fLocked, 1, 0))
         /*nothing */;
 #endif
 }
+RT_EXPORT_SYMBOL(RTSpinlockAcquireNoInts);
 
 
 RTDECL(void) RTSpinlockReleaseNoInts(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
@@ -142,8 +154,11 @@ RTDECL(void) RTSpinlockReleaseNoInts(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
 
     if (!ASMAtomicCmpXchgU32(&pSpinlockInt->fLocked, 0, 1))
         AssertMsgFailed(("Spinlock %p was not locked!\n", pSpinlockInt));
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
     ASMSetFlags(pTmp->uFlags);
+#endif
 }
+RT_EXPORT_SYMBOL(RTSpinlockReleaseNoInts);
 
 
 RTDECL(void) RTSpinlockAcquire(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
@@ -166,6 +181,7 @@ RTDECL(void) RTSpinlockAcquire(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
         /*nothing */;
 #endif
 }
+RT_EXPORT_SYMBOL(RTSpinlockAcquire);
 
 
 RTDECL(void) RTSpinlockRelease(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
@@ -178,4 +194,5 @@ RTDECL(void) RTSpinlockRelease(RTSPINLOCK Spinlock, PRTSPINLOCKTMP pTmp)
     if (!ASMAtomicCmpXchgU32(&pSpinlockInt->fLocked, 0, 1))
         AssertMsgFailed(("Spinlock %p was not locked!\n", pSpinlockInt));
 }
+RT_EXPORT_SYMBOL(RTSpinlockRelease);
 
