@@ -1,10 +1,10 @@
-/* $Id: uuid-generic.cpp $ */
+/* $Id: uuid-generic.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - UUID, Generic.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -33,6 +29,8 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/uuid.h>
+#include "internal/iprt.h"
+
 #include <iprt/assert.h>
 #include <iprt/err.h>
 
@@ -78,6 +76,7 @@ RTDECL(int)  RTUuidClear(PRTUUID pUuid)
     pUuid->au64[1] = 0;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTUuidClear);
 
 
 RTDECL(bool)  RTUuidIsNull(PCRTUUID pUuid)
@@ -86,6 +85,7 @@ RTDECL(bool)  RTUuidIsNull(PCRTUUID pUuid)
     return !pUuid->au64[0]
         && !pUuid->au64[1];
 }
+RT_EXPORT_SYMBOL(RTUuidIsNull);
 
 
 RTDECL(int)  RTUuidCompare(PCRTUUID pUuid1, PCRTUUID pUuid2)
@@ -129,25 +129,51 @@ RTDECL(int)  RTUuidCompare(PCRTUUID pUuid1, PCRTUUID pUuid2)
         return pUuid1->Gen.au8Node[5] < pUuid2->Gen.au8Node[5] ? -1 : 1;
     return 0;
 }
+RT_EXPORT_SYMBOL(RTUuidCompare);
 
 
-RTDECL(int)  RTUuidCompareStr(PCRTUUID pUuid1, const char *pszString)
+RTDECL(int)  RTUuidCompareStr(PCRTUUID pUuid1, const char *pszString2)
 {
     RTUUID Uuid2;
     int rc;
 
     /* check params */
     AssertPtrReturn(pUuid1, -1);
-    AssertPtrReturn(pszString, 1);
+    AssertPtrReturn(pszString2, 1);
 
     /*
      * Try convert the string to a UUID and then compare the two.
      */
-    rc = RTUuidFromStr(&Uuid2, pszString);
+    rc = RTUuidFromStr(&Uuid2, pszString2);
     AssertRCReturn(rc, 1);
 
     return RTUuidCompare(pUuid1, &Uuid2);
 }
+RT_EXPORT_SYMBOL(RTUuidCompareStr);
+
+
+RTDECL(int)  RTUuidCompare2Strs(const char *pszString1, const char *pszString2)
+{
+    RTUUID Uuid1;
+    RTUUID Uuid2;
+    int rc;
+
+    /* check params */
+    AssertPtrReturn(pszString1, -1);
+    AssertPtrReturn(pszString2, 1);
+
+    /*
+     * Try convert the strings to UUIDs and then compare them.
+     */
+    rc = RTUuidFromStr(&Uuid1, pszString1);
+    AssertRCReturn(rc, -1);
+
+    rc = RTUuidFromStr(&Uuid2, pszString2);
+    AssertRCReturn(rc, 1);
+
+    return RTUuidCompare(&Uuid1, &Uuid2);
+}
+RT_EXPORT_SYMBOL(RTUuidCompare2Strs);
 
 
 RTDECL(int)  RTUuidToStr(PCRTUUID pUuid, char *pszString, size_t cchString)
@@ -217,6 +243,7 @@ RTDECL(int)  RTUuidToStr(PCRTUUID pUuid, char *pszString, size_t cchString)
 
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTUuidToStr);
 
 
 RTDECL(int)  RTUuidFromStr(PRTUUID pUuid, const char *pszString)
@@ -311,11 +338,11 @@ RTDECL(int)  RTUuidFromStr(PRTUUID pUuid, const char *pszString)
 #undef MY_TONUM
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTUuidFromStr);
 
 
 RTDECL(int)  RTUuidToUtf16(PCRTUUID pUuid, PRTUTF16 pwszString, size_t cwcString)
 {
-    static const char g_achDigits[17] = "0123456789abcdef";
     uint32_t u32TimeLow;
     unsigned u;
 
@@ -381,6 +408,7 @@ RTDECL(int)  RTUuidToUtf16(PCRTUUID pUuid, PRTUTF16 pwszString, size_t cwcString
 
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTUuidToUtf16);
 
 
 RTDECL(int)  RTUuidFromUtf16(PRTUUID pUuid, PCRTUTF16 pwszString)
@@ -476,4 +504,5 @@ RTDECL(int)  RTUuidFromUtf16(PRTUUID pUuid, PCRTUTF16 pwszString)
 #undef MY_TONUM
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTUuidFromUtf16);
 
