@@ -1,10 +1,10 @@
-/* $Id: initterm.h 8245 2008-04-21 17:24:28Z vboxsync $ */
+/* $Id: initterm.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Initialization & Termination.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___internal_initterm_h
@@ -33,7 +29,7 @@
 
 #include <iprt/cdefs.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 #ifdef IN_RING0
 
@@ -51,7 +47,7 @@ void rtR0TermNative(void);
 
 #endif /* IN_RING0 */
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
