@@ -1,10 +1,10 @@
-/* $Id: process.h $ */
+/* $Id: process.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Internal RTProc header.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___internal_process_h
@@ -34,7 +30,7 @@
 #include <iprt/process.h>
 #include <iprt/param.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 extern RTPROCESS        g_ProcessSelf;
 extern RTPROCPRIORITY   g_enmProcessPriority;
@@ -66,7 +62,7 @@ int rtProcNativeSetPriority(RTPROCPRIORITY enmPriority);
  */
 DECLHIDDEN(int) rtProcInitExePath(char *pszPath, size_t cchPath);
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
