@@ -1,10 +1,10 @@
-/* $Id: time.h $ */
+/* $Id: time.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Internal RTTime header
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___internal_time_h
@@ -33,7 +29,7 @@
 
 #include <iprt/types.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 #if defined(IN_RING3) || defined(IN_RC)
 
@@ -43,6 +39,6 @@ extern uint64_t g_u64ProgramStartMilliTS;
 
 #endif
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
