@@ -1,10 +1,10 @@
-/* $Id: mp-r0drv-darwin.cpp $ */
+/* $Id: mp-r0drv-darwin.cpp 29255 2010-05-09 18:11:24Z vboxsync $ */
 /** @file
  * IPRT - Multiprocessor, Ring-0 Driver, Darwin.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -33,13 +29,20 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include "the-darwin-kernel.h"
-
+#include "internal/iprt.h"
 #include <iprt/mp.h>
+
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
+# include <iprt/asm-amd64-x86.h>
+#endif
 #include <iprt/cpuset.h>
 #include <iprt/err.h>
-#include <iprt/asm.h>
 #include "r0drv/mp-r0drv.h"
 
+
+/*******************************************************************************
+*   Defined Constants And Macros                                               *
+*******************************************************************************/
 #define MY_DARWIN_MAX_CPUS      (0xf + 1) /* see MAX_CPUS */
 
 
@@ -201,6 +204,8 @@ static void rtmpOnAllDarwinWrapper(void *pvArg)
 
 RTDECL(int) RTMpOnAll(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
+    RT_ASSERT_INTS_ON();
+
     RTMPARGS Args;
     Args.pfnWorker = pfnWorker;
     Args.pvUser1 = pvUser1;
@@ -229,12 +234,14 @@ static void rtmpOnOthersDarwinWrapper(void *pvArg)
 
 RTDECL(int) RTMpOnOthers(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
+    RT_ASSERT_INTS_ON();
+
     int rc;
     RTMPARGS Args;
     Args.pfnWorker = pfnWorker;
     Args.pvUser1 = pvUser1;
     Args.pvUser2 = pvUser2;
-    Args.idCpu = NIL_RTCPUID;
+    Args.idCpu = RTMpCpuId();
     Args.cHits = 0;
     mp_rendezvous_no_intrs(rtmpOnOthersDarwinWrapper, &Args);
     return VINF_SUCCESS;
@@ -261,6 +268,8 @@ static void rtmpOnSpecificDarwinWrapper(void *pvArg)
 
 RTDECL(int) RTMpOnSpecific(RTCPUID idCpu, PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
+    RT_ASSERT_INTS_ON();
+
     int rc;
     RTMPARGS Args;
     Args.pfnWorker = pfnWorker;
@@ -274,3 +283,12 @@ RTDECL(int) RTMpOnSpecific(RTCPUID idCpu, PFNRTMPWORKER pfnWorker, void *pvUser1
          : VERR_CPU_NOT_FOUND;
 }
 
+
+RTDECL(int) RTMpPokeCpu(RTCPUID idCpu)
+{
+    RT_ASSERT_INTS_ON();
+
+    /* no unicast IPI */
+    return VERR_NOT_SUPPORTED;
+}
+
