@@ -1,10 +1,10 @@
-/* $Id: the-darwin-kernel.h $ */
+/* $Id: the-darwin-kernel.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Include all necessary headers for the Darwing kernel.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___the_darwin_kernel_h
@@ -78,7 +74,16 @@
 #include <IOKit/IOMapper.h>
 
 
-__BEGIN_DECLS
+/* See osfmk/kern/ast.h. */
+#ifndef AST_PREEMPT
+# define AST_PREEMPT    UINT32_C(1)
+# define AST_QUANTUM    UINT32_C(2)
+# define AST_URGENT     UINT32_C(4)
+#endif
+
+
+RT_C_DECLS_BEGIN
+
 /* mach/vm_types.h */
 typedef struct pmap *pmap_t;
 
@@ -108,19 +113,24 @@ extern void mp_rendezvous_no_intrs(void (*)(void *), void *);
 /* osfmk/i386/cpu_number.h */
 extern int cpu_number(void);
 
+/* osfmk/vm/vm_user.c */
+extern kern_return_t vm_protect(vm_map_t, vm_offset_t, vm_size_t, boolean_t, vm_prot_t);
+
 /* i386/machine_routines.h */
 extern int ml_get_max_cpus(void);
 
-__END_DECLS
+RT_C_DECLS_END
 
 
 /*
  * Internals of the Darwin Ring-0 IPRT.
  */
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 extern lck_grp_t *g_pDarwinLockGroup;
-__END_DECLS
+int  rtThreadPreemptDarwinInit(void);
+void rtThreadPreemptDarwinTerm(void);
+RT_C_DECLS_END
 
 
 /**
