@@ -1,10 +1,10 @@
-/* $Id: initterm-r0drv.cpp $ */
+/* $Id: initterm-r0drv.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Initialization & Termination, R0 Driver, Common.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -33,9 +29,13 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/initterm.h>
+#include "internal/iprt.h"
+
 #include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/err.h>
+#include <iprt/mp.h>
+#include <iprt/thread.h>
 #ifndef IN_GUEST /* play safe for now */
 # include "r0drv/mp-r0drv.h"
 # include "r0drv/power-r0drv.h"
@@ -65,6 +65,7 @@ RTR0DECL(int) RTR0Init(unsigned fReserved)
 {
     int rc;
     Assert(fReserved == 0);
+    RT_ASSERT_PREEMPTIBLE();
 
     /*
      * The first user initializes it.
@@ -103,6 +104,7 @@ RTR0DECL(int) RTR0Init(unsigned fReserved)
     }
     return rc;
 }
+RT_EXPORT_SYMBOL(RTR0Init);
 
 
 static void rtR0Term(void)
@@ -123,16 +125,21 @@ static void rtR0Term(void)
  */
 RTR0DECL(void) RTR0Term(void)
 {
-    int32_t cNewUsers = ASMAtomicDecS32(&g_crtR0Users);
+    int32_t cNewUsers;
+    RT_ASSERT_PREEMPTIBLE();
+
+    cNewUsers = ASMAtomicDecS32(&g_crtR0Users);
     Assert(cNewUsers >= 0);
     if (cNewUsers == 0)
         rtR0Term();
 }
+RT_EXPORT_SYMBOL(RTR0Term);
 
 
 /* Note! Should *not* be exported since it's only for static linking. */
 RTR0DECL(void) RTR0TermForced(void)
 {
+    RT_ASSERT_PREEMPTIBLE();
     AssertMsg(g_crtR0Users == 1, ("%d\n", g_crtR0Users));
 
     rtR0Term();
