@@ -1,10 +1,10 @@
-/* $Id: timer-r0drv-linux.c $ */
+/* $Id: timer-r0drv-linux.c 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Timers, Ring-0 Driver, Linux.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,16 +22,14 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include "the-linux-kernel.h"
+#include "internal/iprt.h"
 
 #include <iprt/timer.h>
 #include <iprt/time.h>
@@ -45,16 +43,22 @@
 
 #include "internal/magics.h"
 
+/* We use the API of Linux 2.6.28+ (hrtimer_add_expires_ns()) */
 #if !defined(RT_USE_LINUX_HRTIMER) \
-    && LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23) \
-    && 0 /* disabled because it somehow sucks. */
+    && LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 28) \
+    && 0 /* currently disabled */
 # define RT_USE_LINUX_HRTIMER
 #endif
 
 /* This check must match the ktime usage in rtTimeGetSystemNanoTS() / time-r0drv-linux.c. */
 #if defined(RT_USE_LINUX_HRTIMER) \
- && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
-# error "RT_USE_LINUX_HRTIMER requires 2.6.16 or later, sorry."
+ && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 28)
+# error "RT_USE_LINUX_HRTIMER requires 2.6.28 or later, sorry."
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
+# define mod_timer_pinned               mod_timer
+# define HRTIMER_MODE_ABS_PINNED        HRTIMER_MODE_ABS
 #endif
 
 
@@ -96,13 +100,13 @@ typedef struct RTTIMERLNXSUBTIMER
     struct hrtimer          LnxTimer;
 #else
     struct timer_list       LnxTimer;
-#endif
     /** The start of the current run (ns).
      * This is used to calculate when the timer ought to fire the next time. */
     uint64_t                u64StartTS;
     /** The start of the current run (ns).
      * This is used to calculate when the timer ought to fire the next time. */
     uint64_t                u64NextTS;
+#endif
     /** The current tick number (since u64StartTS). */
     uint64_t                iTick;
     /** Pointer to the parent timer. */
@@ -247,13 +251,13 @@ DECLINLINE(unsigned long) rtTimerLnxNanoToJiffies(uint64_t cNanoSecs)
     /* this can be made even better... */
     if (cNanoSecs > (uint64_t)TICK_NSEC * MAX_JIFFY_OFFSET)
         return MAX_JIFFY_OFFSET;
-#if ARCH_BITS == 32
+# if ARCH_BITS == 32
     if (RT_LIKELY(cNanoSecs <= UINT32_MAX))
         return ((uint32_t)cNanoSecs + (TICK_NSEC-1)) / TICK_NSEC;
-#endif
+# endif
     return (cNanoSecs + (TICK_NSEC-1)) / TICK_NSEC;
 }
-#endif
+#endif /* ! RT_USE_LINUX_HRTIMER */
 
 
 /**
@@ -262,24 +266,35 @@ DECLINLINE(unsigned long) rtTimerLnxNanoToJiffies(uint64_t cNanoSecs)
  * @param   pSubTimer   The sub-timer to start.
  * @param   u64Now      The current timestamp (RTTimeNanoTS()).
  * @param   u64First    The interval from u64Now to the first time the timer should fire.
+ * @param   fPinned     true = timer pinned to a specific CPU,
+ *                      false = timer can migrate between CPUs
  */
-static void rtTimerLnxStartSubTimer(PRTTIMERLNXSUBTIMER pSubTimer, uint64_t u64Now, uint64_t u64First)
+static void rtTimerLnxStartSubTimer(PRTTIMERLNXSUBTIMER pSubTimer, uint64_t u64Now, uint64_t u64First, bool fPinned)
 {
     /*
      * Calc when it should start firing.
      */
     uint64_t u64NextTS = u64Now + u64First;
+#ifndef RT_USE_LINUX_HRTIMER
     pSubTimer->u64StartTS = u64NextTS;
     pSubTimer->u64NextTS = u64NextTS;
+#endif
+
     pSubTimer->iTick = 0;
 
 #ifdef RT_USE_LINUX_HRTIMER
-    hrtimer_start(&pSubTimer->LnxTimer, rtTimerLnxNanoToKt(u64NextTS), HRTIMER_MODE_ABS);
+    hrtimer_start(&pSubTimer->LnxTimer, rtTimerLnxNanoToKt(u64NextTS),
+                  fPinned ? HRTIMER_MODE_ABS_PINNED : HRTIMER_MODE_ABS);
 #else
     {
         unsigned long cJiffies = !u64First ? 0 : rtTimerLnxNanoToJiffies(u64First);
         pSubTimer->ulNextJiffies = jiffies + cJiffies;
-        mod_timer(&pSubTimer->LnxTimer, pSubTimer->ulNextJiffies);
+# ifdef CONFIG_SMP
+        if (fPinned)
+            mod_timer_pinned(&pSubTimer->LnxTimer, pSubTimer->ulNextJiffies);
+        else
+# endif
+            mod_timer(&pSubTimer->LnxTimer, pSubTimer->ulNextJiffies);
     }
 #endif
 
@@ -365,6 +380,14 @@ static void rtTimerLinuxCallback(unsigned long ulUser)
     }
     else
     {
+        const uint64_t iTick = ++pSubTimer->iTick;
+
+#ifdef RT_USE_LINUX_HRTIMER
+        hrtimer_add_expires_ns(&pSubTimer->LnxTimer, pTimer->u64NanoInterval);
+        rc = HRTIMER_RESTART;
+#else
+        const uint64_t u64NanoTS = RTTimeNanoTS();
+
         /*
          * Interval timer, calculate the next timeout and re-arm it.
          *
@@ -372,28 +395,13 @@ static void rtTimerLinuxCallback(unsigned long ulUser)
          * try prevent some jittering if we were started at a bad time.
          * This may of course backfire with highres timers...
          */
-        const uint64_t u64NanoTS = RTTimeNanoTS();
-        const uint64_t iTick = ++pSubTimer->iTick;
-
         if (RT_UNLIKELY(iTick == 1))
         {
-#ifdef RT_USE_LINUX_HRTIMER
-            pSubTimer->u64StartTS = pSubTimer->u64NextTS = u64NanoTS;//rtTimerLnxKtToNano(pSubTimer->LnxTimer.base->softirq_time);
-#else
             pSubTimer->u64StartTS = pSubTimer->u64NextTS = u64NanoTS;
             pSubTimer->ulNextJiffies = jiffies;
-#endif
         }
 
         pSubTimer->u64NextTS += pTimer->u64NanoInterval;
-
-#ifdef RT_USE_LINUX_HRTIMER
-        while (pSubTimer->u64NextTS < u64NanoTS)
-            pSubTimer->u64NextTS += pTimer->u64NanoInterval;
-
-        pSubTimer->LnxTimer.expires = rtTimerLnxNanoToKt(pSubTimer->u64NextTS);
-        rc = HRTIMER_RESTART;
-#else
         if (pTimer->cJiffies)
         {
             pSubTimer->ulNextJiffies += pTimer->cJiffies;
@@ -412,7 +420,12 @@ static void rtTimerLinuxCallback(unsigned long ulUser)
             pSubTimer->ulNextJiffies = jiffies + rtTimerLnxNanoToJiffies(pSubTimer->u64NextTS - u64NanoTS);
         }
 
-        mod_timer(&pSubTimer->LnxTimer, pSubTimer->ulNextJiffies);
+# ifdef CONFIG_SMP
+        if (pTimer->fSpecificCpu || pTimer->fAllCpus)
+            mod_timer_pinned(&pSubTimer->LnxTimer, pSubTimer->ulNextJiffies);
+        else
+# endif
+            mod_timer(&pSubTimer->LnxTimer, pSubTimer->ulNextJiffies);
 #endif
 
         /*
@@ -441,7 +454,7 @@ static DECLCALLBACK(void) rtTimerLnxStartAllOnCpu(RTCPUID idCpu, void *pvUser1,
     PRTTIMERLINUXSTARTONCPUARGS pArgs = (PRTTIMERLINUXSTARTONCPUARGS)pvUser2;
     PRTTIMER pTimer = (PRTTIMER)pvUser1;
     Assert(idCpu < pTimer->cCpus);
-    rtTimerLnxStartSubTimer(&pTimer->aSubTimers[idCpu], pArgs->u64Now, pArgs->u64First);
+    rtTimerLnxStartSubTimer(&pTimer->aSubTimers[idCpu], pArgs->u64Now, pArgs->u64First, true /*fPinned*/);
 }
 
 
@@ -454,7 +467,7 @@ static DECLCALLBACK(void) rtTimerLnxStartAllOnCpu(RTCPUID idCpu, void *pvUser1,
  */
 static int rtTimerLnxStartAll(PRTTIMER pTimer, PRTTIMERLINUXSTARTONCPUARGS pArgs)
 {
-    RTSPINLOCKTMP   Tmp;
+    RTSPINLOCKTMP   Tmp = RTSPINLOCKTMP_INITIALIZER;
     RTCPUID         iCpu;
     RTCPUSET        OnlineSet;
     RTCPUSET        OnlineSet2;
@@ -522,8 +535,8 @@ static int rtTimerLnxStartAll(PRTTIMER pTimer, PRTTIMERLINUXSTARTONCPUARGS pArgs
  */
 static int rtTimerLnxStopAll(PRTTIMER pTimer)
 {
+    RTSPINLOCKTMP   Tmp = RTSPINLOCKTMP_INITIALIZER;
     RTCPUID         iCpu;
-    RTSPINLOCKTMP   Tmp;
 
 
     /*
@@ -584,7 +597,7 @@ static DECLCALLBACK(void) rtTimerLinuxMpStartOnCpu(RTCPUID idCpu, void *pvUser1,
     if (    hSpinlock != NIL_RTSPINLOCK
         &&  pTimer->u32Magic == RTTIMER_MAGIC)
     {
-        RTSPINLOCKTMP Tmp;
+        RTSPINLOCKTMP Tmp = RTSPINLOCKTMP_INITIALIZER;
         RTSpinlockAcquire(hSpinlock, &Tmp);
 
         if (    !ASMAtomicUoReadBool(&pTimer->fSuspended)
@@ -593,7 +606,7 @@ static DECLCALLBACK(void) rtTimerLinuxMpStartOnCpu(RTCPUID idCpu, void *pvUser1,
             /* We're sane and the timer is not suspended yet. */
             PRTTIMERLNXSUBTIMER pSubTimer = &pTimer->aSubTimers[idCpu];
             if (rtTimerLnxCmpXchgState(&pSubTimer->enmState, RTTIMERLNXSTATE_MP_STARTING, RTTIMERLNXSTATE_STOPPED))
-                rtTimerLnxStartSubTimer(pSubTimer, pArgs->u64Now, pArgs->u64First);
+                rtTimerLnxStartSubTimer(pSubTimer, pArgs->u64Now, pArgs->u64First, true /*fPinned*/);
         }
 
         RTSpinlockRelease(hSpinlock, &Tmp);
@@ -610,10 +623,10 @@ static DECLCALLBACK(void) rtTimerLinuxMpStartOnCpu(RTCPUID idCpu, void *pvUser1,
  */
 static DECLCALLBACK(void) rtTimerLinuxMpEvent(RTMPEVENT enmEvent, RTCPUID idCpu, void *pvUser)
 {
-    PRTTIMER pTimer = (PRTTIMER)pvUser;
+    PRTTIMER            pTimer = (PRTTIMER)pvUser;
     PRTTIMERLNXSUBTIMER pSubTimer = &pTimer->aSubTimers[idCpu];
-    RTSPINLOCK hSpinlock;
-    RTSPINLOCKTMP Tmp;
+    RTSPINLOCK          hSpinlock;
+    RTSPINLOCKTMP       Tmp = RTSPINLOCKTMP_INITIALIZER;
 
     Assert(idCpu < pTimer->cCpus);
 
@@ -646,7 +659,7 @@ static DECLCALLBACK(void) rtTimerLinuxMpEvent(RTMPEVENT enmEvent, RTCPUID idCpu,
                     Args.u64First = 0;
 
                     if (RTMpCpuId() == idCpu)
-                        rtTimerLnxStartSubTimer(pSubTimer, Args.u64Now, Args.u64First);
+                        rtTimerLnxStartSubTimer(pSubTimer, Args.u64Now, Args.u64First, true /*fPinned*/);
                     else
                     {
                         rtTimerLnxSetState(&pSubTimer->enmState, RTTIMERLNXSTATE_STOPPED); /* we'll recheck it. */
@@ -694,7 +707,7 @@ static DECLCALLBACK(void) rtTimerLnxStartOnSpecificCpu(RTCPUID idCpu, void *pvUs
 {
     PRTTIMERLINUXSTARTONCPUARGS pArgs = (PRTTIMERLINUXSTARTONCPUARGS)pvUser2;
     PRTTIMER pTimer = (PRTTIMER)pvUser1;
-    rtTimerLnxStartSubTimer(&pTimer->aSubTimers[0], pArgs->u64Now, pArgs->u64First);
+    rtTimerLnxStartSubTimer(&pTimer->aSubTimers[0], pArgs->u64Now, pArgs->u64First, true /*fPinned*/);
 }
 
 
@@ -728,7 +741,7 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
     rtTimerLnxSetState(&pTimer->aSubTimers[0].enmState, RTTIMERLNXSTATE_STARTING);
     ASMAtomicWriteBool(&pTimer->fSuspended, false);
     if (!pTimer->fSpecificCpu)
-        rtTimerLnxStartSubTimer(&pTimer->aSubTimers[0], Args.u64Now, Args.u64First);
+        rtTimerLnxStartSubTimer(&pTimer->aSubTimers[0], Args.u64Now, Args.u64First, false /*fPinned*/);
     else
     {
         rc2 = RTMpOnSpecific(pTimer->idCpu, rtTimerLnxStartOnSpecificCpu, pTimer, &Args);
@@ -743,6 +756,7 @@ RTDECL(int) RTTimerStart(PRTTIMER pTimer, uint64_t u64First)
 
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTTimerStart);
 
 
 RTDECL(int) RTTimerStop(PRTTIMER pTimer)
@@ -774,6 +788,7 @@ RTDECL(int) RTTimerStop(PRTTIMER pTimer)
 
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTTimerStop);
 
 
 RTDECL(int) RTTimerDestroy(PRTTIMER pTimer)
@@ -817,6 +832,7 @@ RTDECL(int) RTTimerDestroy(PRTTIMER pTimer)
 
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTTimerDestroy);
 
 
 RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, unsigned fFlags, PFNRTTIMER pfnTimer, void *pvUser)
@@ -890,9 +906,9 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, unsigne
         pTimer->aSubTimers[iCpu].LnxTimer.data     = (unsigned long)&pTimer->aSubTimers[iCpu];
         pTimer->aSubTimers[iCpu].LnxTimer.function = rtTimerLinuxCallback;
         pTimer->aSubTimers[iCpu].LnxTimer.expires  = jiffies;
-#endif
         pTimer->aSubTimers[iCpu].u64StartTS = 0;
         pTimer->aSubTimers[iCpu].u64NextTS = 0;
+#endif
         pTimer->aSubTimers[iCpu].iTick = 0;
         pTimer->aSubTimers[iCpu].pParent = pTimer;
         pTimer->aSubTimers[iCpu].enmState = RTTIMERLNXSTATE_STOPPED;
@@ -923,6 +939,7 @@ RTDECL(int) RTTimerCreateEx(PRTTIMER *ppTimer, uint64_t u64NanoInterval, unsigne
     *ppTimer = pTimer;
     return VINF_SUCCESS;
 }
+RT_EXPORT_SYMBOL(RTTimerCreateEx);
 
 
 RTDECL(uint32_t) RTTimerGetSystemGranularity(void)
@@ -938,16 +955,19 @@ RTDECL(uint32_t) RTTimerGetSystemGranularity(void)
 #endif
     return 1000000000 / HZ; /* ns */
 }
+RT_EXPORT_SYMBOL(RTTimerGetSystemGranularity);
 
 
 RTDECL(int) RTTimerRequestSystemGranularity(uint32_t u32Request, uint32_t *pu32Granted)
 {
     return VERR_NOT_SUPPORTED;
 }
+RT_EXPORT_SYMBOL(RTTimerRequestSystemGranularity);
 
 
 RTDECL(int) RTTimerReleaseSystemGranularity(uint32_t u32Granted)
 {
     return VERR_NOT_SUPPORTED;
 }
+RT_EXPORT_SYMBOL(RTTimerReleaseSystemGranularity);
 
