@@ -1,10 +1,10 @@
-/* $Id: mp-r0drv.h $ */
+/* $Id: mp-r0drv.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Multiprocessor, Ring-0 Driver, Internal Header.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___r0drv_mp_r0drv_h
@@ -33,7 +29,7 @@
 
 #include <iprt/mp.h>
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 /**
  * MP callback
@@ -70,7 +66,7 @@ int rtR0MpNotificationNativeInit(void);
 void rtR0MpNotificationNativeTerm(void);
 void rtMpNotificationDoCallbacks(RTMPEVENT enmEvent, RTCPUID idCpu);
 
-__END_DECLS
+RT_C_DECLS_END
 
 #endif
 
