@@ -1,10 +1,10 @@
-/* $Id: mp-r0drv-solaris.c $ */
+/* $Id: mp-r0drv-solaris.c 29284 2010-05-10 00:22:16Z vboxsync $ */
 /** @file
  * IPRT - Multiprocessor, Ring-0 Driver, Solaris.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,25 +22,26 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include "the-solaris-kernel.h"
-
+#include "../the-solaris-kernel.h"
+#include "internal/iprt.h"
 #include <iprt/mp.h>
-#include <iprt/err.h>
+#include <iprt/cpuset.h>
+
 #include <iprt/asm.h>
-#include "internal-r0drv-solaris.h"
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
+# include <iprt/asm-amd64-x86.h>
+#endif
+#include <iprt/err.h>
 #include "r0drv/mp-r0drv.h"
 
 
+
 RTDECL(bool) RTMpIsCpuWorkPending(void)
 {
     return false;
@@ -183,6 +184,7 @@ static int rtmpOnAllSolarisWrapper(void *uArg, void *uIgnored1, void *uIgnored2)
 RTDECL(int) RTMpOnAll(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
     RTMPARGS Args;
+    RT_ASSERT_INTS_ON();
 
     Args.pfnWorker = pfnWorker;
     Args.pvUser1 = pvUser1;
@@ -224,8 +226,8 @@ static int rtmpOnOthersSolarisWrapper(void *uArg, void *uIgnored1, void *uIgnore
 
 RTDECL(int) RTMpOnOthers(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
-    int rc;
     RTMPARGS Args;
+    RT_ASSERT_INTS_ON();
 
     /* The caller is supposed to have disabled preemption, but take no chances. */
     vbi_preempt_disable();
@@ -234,7 +236,7 @@ RTDECL(int) RTMpOnOthers(PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
     Args.pvUser1 = pvUser1;
     Args.pvUser2 = pvUser2;
     Args.idCpu = RTMpCpuId();
-    Args.cHits = 0; 
+    Args.cHits = 0;
 
     vbi_execute_on_others(rtmpOnOthersSolarisWrapper, &Args);
 
@@ -270,8 +272,8 @@ static int rtmpOnSpecificSolarisWrapper(void *uArg, void *uIgnored1, void *uIgno
 
 RTDECL(int) RTMpOnSpecific(RTCPUID idCpu, PFNRTMPWORKER pfnWorker, void *pvUser1, void *pvUser2)
 {
-    int rc;
     RTMPARGS Args;
+    RT_ASSERT_INTS_ON();
 
     if (idCpu >= vbi_cpu_count())
         return VERR_CPU_NOT_FOUND;
