@@ -1,10 +1,10 @@
-/* $Id: thread2-r0drv-solaris.c $ */
+/* $Id: thread2-r0drv-solaris.c 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Threads (Part 2), Ring-0 Driver, Solaris.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,23 +22,22 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
+
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
-#include "the-solaris-kernel.h"
+#include "../the-solaris-kernel.h"
+#include "internal/iprt.h"
+#include <iprt/thread.h>
 
 #include <iprt/assert.h>
 #include <iprt/err.h>
-#include <iprt/thread.h>
-
 #include "internal/thread.h"
 
+
+
 int rtThreadNativeInit(void)
 {
     return VINF_SUCCESS;
@@ -89,24 +88,24 @@ int rtThreadNativeAdopt(PRTTHREADINT pThread)
 static void rtThreadNativeMain(void *pvThreadInt)
 {
     PRTTHREADINT pThreadInt = (PRTTHREADINT)pvThreadInt;
-    int rc;
 
-    rc = rtThreadMain(pThreadInt, (RTNATIVETHREAD)vbi_curthread(), &pThreadInt->szName[0]);
+    rtThreadMain(pThreadInt, (RTNATIVETHREAD)vbi_curthread(), &pThreadInt->szName[0]);
     vbi_thread_exit();
 }
 
 
 int rtThreadNativeCreate(PRTTHREADINT pThreadInt, PRTNATIVETHREAD pNativeThread)
 {
-    int rc;
-    void *pKernThread = vbi_thread_create(rtThreadNativeMain, pThreadInt,
-                                           sizeof (pThreadInt), minclsyspri);
-    if (pKernThread)
+    void   *pvKernThread;
+    RT_ASSERT_PREEMPTIBLE();
+
+    pvKernThread = vbi_thread_create(rtThreadNativeMain, pThreadInt, sizeof(pThreadInt), minclsyspri);
+    if (pvKernThread)
     {
-        *pNativeThread = (RTNATIVETHREAD)pKernThread;
+        *pNativeThread = (RTNATIVETHREAD)pvKernThread;
         return VINF_SUCCESS;
     }
 
-    return RTErrConvertFromErrno(rc);
+    return VERR_OUT_OF_RESOURCES;
 }
 
