@@ -1,10 +1,10 @@
-/* $Id: fs.cpp $ */
+/* $Id: fs.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - File System.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -36,6 +32,10 @@
 # define RTTIME_INCL_TIMESPEC
 # include <sys/time.h>
 # include <sys/param.h>
+# ifndef DEV_BSIZE
+#  include <sys/stat.h>
+#  define DEV_BSIZE S_BLKSIZE /** @todo bird: add DEV_BSIZE to sys/param.h on OS/2. */
+# endif
 #endif
 
 #include <iprt/fs.h>
@@ -74,9 +74,9 @@ RTFMODE rtFsModeFromDos(RTFMODE fMode, const char *pszName, size_t cbName)
             /* check for executable extension. */
             const char *pszExt = &pszName[cbName - 3];
             char szExt[4];
-            szExt[0] = tolower(pszExt[0]);
-            szExt[1] = tolower(pszExt[1]);
-            szExt[2] = tolower(pszExt[2]);
+            szExt[0] = RT_C_TO_LOWER(pszExt[0]);
+            szExt[1] = RT_C_TO_LOWER(pszExt[1]);
+            szExt[2] = RT_C_TO_LOWER(pszExt[2]);
             szExt[3] = '\0';
             if (    !memcmp(szExt, "exe", 4)
                 ||  !memcmp(szExt, "bat", 4)
