@@ -1,10 +1,10 @@
-/* $Id: init.cpp $ */
+/* $Id: init.cpp 28903 2010-04-29 14:58:12Z vboxsync $ */
 /** @file
  * IPRT - Init Ring-3.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -33,12 +29,17 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP RTLOGGROUP_DEFAULT
+#include <iprt/types.h>                 /* darwin: UINT32_C and others. */
+
 #ifdef RT_OS_WINDOWS
 # include <process.h>
 #else
 # include <unistd.h>
 # ifndef RT_OS_OS2
 #  include <pthread.h>
+#  include <signal.h>
+#  include <errno.h>
+#  define IPRT_USE_SIG_CHILD_DUMMY
 # endif
 #endif
 #ifdef RT_OS_OS2
@@ -58,9 +59,10 @@
 #if !defined(IN_GUEST) && !defined(RT_NO_GIP)
 # include <iprt/file.h>
 # include <VBox/sup.h>
-# include <stdlib.h>
 #endif
+#include <stdlib.h>
 
+#include "internal/alignmentchecks.h"
 #include "internal/path.h"
 #include "internal/process.h"
 #include "internal/thread.h"
@@ -77,7 +79,7 @@ static int32_t volatile g_cUsers = 0;
 static bool volatile    g_fInitializing = false;
 
 /** The process path.
- * This is used by RTPathProgram and RTProcGetExecutableName and set by rtProcInitName. */
+ * This is used by RTPathExecDir and RTProcGetExecutableName and set by rtProcInitName. */
 char        g_szrtProcExePath[RTPATH_MAX];
 /** The length of g_szrtProcExePath. */
 size_t      g_cchrtProcExePath;
@@ -111,6 +113,35 @@ RTPROCESS   g_ProcessSelf = NIL_RTPROCESS;
  */
 RTPROCPRIORITY g_enmProcessPriority = RTPROCPRIORITY_DEFAULT;
 
+#ifdef IPRT_WITH_ALIGNMENT_CHECKS
+/**
+ * Whether alignment checks are enabled.
+ * This is set if the environment variable IPRT_ALIGNMENT_CHECKS is 1.
+ */
+RTDATADECL(bool) g_fRTAlignmentChecks = false;
+#endif
+
+
+/**
+ * atexit callback.
+ *
+ * This makes sure any loggers are flushed and will later also work the
+ * termination callback chain.
+ */
+static void rtR3ExitCallback(void)
+{
+    if (g_cUsers > 0)
+    {
+        PRTLOGGER pLogger = RTLogGetDefaultInstance();
+        if (pLogger)
+            RTLogFlush(pLogger);
+
+        pLogger = RTLogRelDefaultInstance();
+        if (pLogger)
+            RTLogFlush(pLogger);
+    }
+}
+
 
 #ifndef RT_OS_WINDOWS
 /**
@@ -123,11 +154,21 @@ static void rtR3ForkChildCallback(void)
 #endif /* RT_OS_WINDOWS */
 
 #ifdef RT_OS_OS2
+/** Fork completion callback for OS/2.  Only called in the child. */
+static void rtR3ForkOs2ChildCompletionCallback(void *pvArg, int rc, __LIBC_FORKCTX enmCtx)
+{
+    Assert(enmCtx == __LIBC_FORK_CTX_CHILD); NOREF(enmCtx);
+    NOREF(pvArg);
+
+    if (!rc)
+        rtR3ForkChildCallback();
+}
+
 /** Low-level fork callback for OS/2.  */
 int rtR3ForkOs2Child(__LIBC_PFORKHANDLE pForkHandle, __LIBC_FORKOP enmOperation)
 {
-    if (enmOperation == __LIBC_FORK_STAGE_COMPLETION_CHILD)
-        rtR3ForkChildCallback();
+    if (enmOperation == __LIBC_FORK_OP_EXEC_CHILD)
+        return pForkHandle->pfnCompletionCallback(pForkHandle, rtR3ForkOs2ChildCompletionCallback, NULL, __LIBC_FORK_CTX_CHILD);
     return 0;
 }
 
@@ -171,11 +212,32 @@ static int rtR3InitProgramPath(const char *pszProgramPath)
     return VINF_SUCCESS;
 }
 
+
+#ifdef IPRT_USE_SIG_CHILD_DUMMY
+/**
+ * Dummy SIGCHILD handler.
+ *
+ * Assigned on rtR3Init only when SIGCHILD handler is set SIGIGN or SIGDEF to
+ * ensure waitpid works properly for the terminated processes.
+ */
+static void rtR3SigChildHandler(int iSignal)
+{
+    NOREF(iSignal);
+}
+#endif /* IPRT_USE_SIG_CHILD_DUMMY */
+
+
 /**
  * rtR3Init worker.
  */
 static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
 {
+    /*
+     * Init C runtime locale before we do anything that may end up converting
+     * paths or we'll end up using the "C" locale for path conversion.
+     */
+    setlocale(LC_CTYPE, "");
+
     /*
      * The Process ID.
      */
@@ -194,7 +256,7 @@ static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
      */
     const char *pszDisableHostCache = getenv("VBOX_DISABLE_HOST_DISK_CACHE");
     if (    pszDisableHostCache != NULL
-        &&  strlen(pszDisableHostCache) > 0
+        &&  *pszDisableHostCache
         &&  strcmp(pszDisableHostCache, "0") != 0)
     {
         RTFileSetForceFlags(RTFILE_O_WRITE, RTFILE_O_WRITE_THROUGH, 0);
@@ -253,14 +315,59 @@ static int rtR3InitBody(bool fInitSUPLib, const char *pszProgramPath)
      * The remainder cannot easily be undone, so it has to go last.
      */
 
-    /* Init C runtime locale. */
-    setlocale(LC_CTYPE, "");
-
-    /* Fork callbacks. */
+    /* Fork and exit callbacks. */
 #if !defined(RT_OS_WINDOWS) && !defined(RT_OS_OS2)
     rc = pthread_atfork(NULL, NULL, rtR3ForkChildCallback);
     AssertMsg(rc == 0, ("%d\n", rc));
 #endif
+    atexit(rtR3ExitCallback);
+
+#ifdef IPRT_USE_SIG_CHILD_DUMMY
+    /*
+     * SIGCHLD must not be ignored (that's default), otherwise posix compliant waitpid
+     * implementations won't work right.
+     */
+    for (;;)
+    {
+        struct sigaction saOld;
+        rc = sigaction(SIGCHLD, 0, &saOld);         AssertMsg(rc == 0, ("%d/%d\n", rc, errno));
+        if (    rc != 0
+            ||  (saOld.sa_flags & SA_SIGINFO)
+            || (   saOld.sa_handler != SIG_IGN
+                && saOld.sa_handler != SIG_DFL)
+           )
+            break;
+
+        /* Try install dummy handler. */
+        struct sigaction saNew = saOld;
+        saNew.sa_flags   = SA_NOCLDSTOP | SA_RESTART;
+        saNew.sa_handler = rtR3SigChildHandler;
+        rc = sigemptyset(&saNew.sa_mask);           AssertMsg(rc == 0, ("%d/%d\n", rc, errno));
+        struct sigaction saOld2;
+        rc = sigaction(SIGCHLD, &saNew, &saOld2);   AssertMsg(rc == 0, ("%d/%d\n", rc, errno));
+        if (    rc != 0
+            ||  (   saOld2.sa_handler == saOld.sa_handler
+                 && !(saOld2.sa_flags & SA_SIGINFO))
+           )
+            break;
+
+        /* Race during dynamic load, restore and try again... */
+        sigaction(SIGCHLD, &saOld2, NULL);
+        RTThreadYield();
+    }
+#endif /* IPRT_USE_SIG_CHILD_DUMMY */
+
+#ifdef IPRT_WITH_ALIGNMENT_CHECKS
+    /*
+     * Enable alignment checks.
+     */
+    const char *pszAlignmentChecks = getenv("IPRT_ALIGNMENT_CHECKS");
+    g_fRTAlignmentChecks = pszAlignmentChecks != NULL
+                        && pszAlignmentChecks[0] == '1'
+                        && pszAlignmentChecks[1] == '\0';
+    if (g_fRTAlignmentChecks)
+        IPRT_ALIGNMENT_CHECKS_ENABLE();
+#endif
 
     return VINF_SUCCESS;
 }
