@@ -1,10 +1,10 @@
-/* $Id: env-posix.cpp $ */
+/* $Id: env-posix.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Environment, Posix.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -40,20 +36,28 @@
 #include <iprt/string.h>
 #include <iprt/alloca.h>
 #include <iprt/assert.h>
+#if defined(DEBUG) && defined(RT_OS_LINUX)
+# include <iprt/asm.h>
+#endif
 
 #include <stdlib.h>
 #include <errno.h>
 
+#include "internal/alignmentchecks.h"
+
 
 RTDECL(bool) RTEnvExist(const char *pszVar)
 {
-    return getenv(pszVar) != NULL;
+    return RTEnvGet(pszVar) != NULL;
 }
 
 
 RTDECL(const char *) RTEnvGet(const char *pszVar)
 {
-    return getenv(pszVar);
+    IPRT_ALIGNMENT_CHECKS_DISABLE(); /* glibc causes trouble */
+    const char *pszValue = getenv(pszVar);
+    IPRT_ALIGNMENT_CHECKS_ENABLE();
+    return pszValue;
 }
 
 
@@ -85,12 +89,12 @@ RTDECL(int) RTEnvSet(const char *pszVar, const char *pszValue)
     if (!putenv(pszTmp))
         return 0;
     return RTErrConvertFromErrno(errno);
-    
+
 #else
     if (!setenv(pszVar, pszValue, 1))
         return VINF_SUCCESS;
     return RTErrConvertFromErrno(errno);
-#endif 
+#endif
 }
 
 
