@@ -1,10 +1,10 @@
-/* $Id: fileio-posix.cpp $ */
+/* $Id: fileio-posix.cpp 29263 2010-05-09 19:52:03Z vboxsync $ */
 /** @file
  * IPRT - File I/O, POSIX.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -47,6 +43,9 @@
 # include <unistd.h>
 # include <sys/time.h>
 #endif
+#ifdef RT_OS_LINUX
+# include <sys/file.h>
+#endif
 #if defined(RT_OS_OS2) && (!defined(__INNOTEK_LIBC__) || __INNOTEK_LIBC__ < 0x006)
 # include <io.h>
 #endif
@@ -75,12 +74,6 @@ extern int futimes(int __fd, __const struct timeval __tvp[2]) __THROW;
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
-/** @def RT_DONT_CONVERT_FILENAMES
- * Define this to pass UTF-8 unconverted to the kernel. */
-#ifdef __DOXYGEN__
-#define RT_DONT_CONVERT_FILENAMES 1
-#endif
-
 /** Default file permissions for newly created files. */
 #if defined(S_IRUSR) && defined(S_IWUSR)
 # define RT_FILE_PERMISSION  (S_IRUSR | S_IWUSR)
@@ -92,15 +85,15 @@ extern int futimes(int __fd, __const struct timeval __tvp[2]) __THROW;
 RTDECL(bool) RTFileExists(const char *pszPath)
 {
     bool fRc = false;
-    char *pszNativePath;
-    int rc = rtPathToNative(&pszNativePath, pszPath);
+    char const *pszNativePath;
+    int rc = rtPathToNative(&pszNativePath, pszPath, NULL);
     if (RT_SUCCESS(rc))
     {
         struct stat s;
         fRc = !stat(pszNativePath, &s)
             && S_ISREG(s.st_mode);
 
-        rtPathFreeNative(pszNativePath);
+        rtPathFreeNative(pszNativePath, pszPath);
     }
 
     LogFlow(("RTFileExists(%p={%s}): returns %RTbool\n", pszPath, pszPath, fRc));
@@ -108,22 +101,14 @@ RTDECL(bool) RTFileExists(const char *pszPath)
 }
 
 
-RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen)
+RTR3DECL(int) RTFileOpen(PRTFILE pFile, const char *pszFilename, uint32_t fOpen)
 {
     /*
      * Validate input.
      */
-    if (!VALID_PTR(pFile))
-    {
-        AssertMsgFailed(("Invalid pFile %p\n", pFile));
-        return VERR_INVALID_PARAMETER;
-    }
+    AssertPtrReturn(pFile, VERR_INVALID_POINTER);
     *pFile = NIL_RTFILE;
-    if (!VALID_PTR(pszFilename))
-    {
-        AssertMsgFailed(("Invalid pszFilename %p\n", pszFilename));
-        return VERR_INVALID_PARAMETER;
-    }
+    AssertPtrReturn(pszFilename, VERR_INVALID_POINTER);
 
     /*
      * Merge forced open flags and validate them.
@@ -166,6 +151,11 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
     if (fOpen & RTFILE_O_ASYNC_IO)
         fOpenMode |= O_DIRECT;
 #endif
+#if defined(O_DIRECT) && (defined(RT_OS_LINUX) || defined(RT_OS_FREEBSD))
+    /* Disable the kernel cache. */
+    if (fOpen & RTFILE_O_NO_CACHE)
+        fOpenMode |= O_DIRECT;
+#endif
 
     /* create/truncate file */
     switch (fOpen & RTFILE_O_ACTION_MASK)
@@ -180,9 +170,15 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
 
     switch (fOpen & RTFILE_O_ACCESS_MASK)
     {
-        case RTFILE_O_READ:             fOpenMode |= O_RDONLY; break;
-        case RTFILE_O_WRITE:            fOpenMode |= O_WRONLY; break;
-        case RTFILE_O_READWRITE:        fOpenMode |= O_RDWR; break;
+        case RTFILE_O_READ:
+            fOpenMode |= O_RDONLY; /* RTFILE_O_APPEND is ignored. */
+            break;
+        case RTFILE_O_WRITE:
+            fOpenMode |= fOpen & RTFILE_O_APPEND ? O_APPEND | O_WRONLY : O_WRONLY;
+            break;
+        case RTFILE_O_READWRITE:
+            fOpenMode |= fOpen & RTFILE_O_APPEND ? O_APPEND | O_RDWR   : O_RDWR;
+            break;
         default:
             AssertMsgFailed(("RTFileOpen received an invalid RW value, fOpen=%#x\n", fOpen));
             return VERR_INVALID_PARAMETER;
@@ -198,29 +194,117 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
     /*
      * Open/create the file.
      */
-#ifdef RT_DONT_CONVERT_FILENAMES
-    int fh = open(pszFilename, fOpenMode, fMode);
-    int iErr = errno;
-#else
-    char *pszNativeFilename;
-    rc = rtPathToNative(&pszNativeFilename, pszFilename);
+    char const *pszNativeFilename;
+    rc = rtPathToNative(&pszNativeFilename, pszFilename, NULL);
     if (RT_FAILURE(rc))
         return (rc);
 
     int fh = open(pszNativeFilename, fOpenMode, fMode);
     int iErr = errno;
-    rtPathFreeNative(pszNativeFilename);
-#endif
+    rtPathFreeNative(pszNativeFilename, pszFilename);
     if (fh >= 0)
     {
+        iErr = 0;
+
         /*
          * Mark the file handle close on exec, unless inherit is specified.
          */
         if (    !(fOpen & RTFILE_O_INHERIT)
 #ifdef O_NOINHERIT
-            ||  (fOpenMode & O_NOINHERIT) /* careful since it could be a dummy. */
+            &&  !(fOpenMode & O_NOINHERIT)  /* Take care since it might be a zero value dummy. */
 #endif
-            ||  fcntl(fh, F_SETFD, FD_CLOEXEC) >= 0)
+            )
+            iErr = fcntl(fh, F_SETFD, FD_CLOEXEC) >= 0 ? 0 : errno;
+
+        /*
+         * Switch direct I/O on now if requested and required.
+         */
+#if defined(RT_OS_DARWIN) \
+ || (defined(RT_OS_SOLARIS) && !defined(IN_GUEST))
+        if (iErr == 0 && (fOpen & RTFILE_O_NO_CACHE))
+        {
+# if defined(RT_OS_DARWIN)
+            iErr = fcntl(fh, F_NOCACHE, 1)        >= 0 ? 0 : errno;
+# else
+            iErr = directio(fh, DIRECTIO_ON)      >= 0 ? 0 : errno;
+# endif
+        }
+#endif
+
+        /*
+         * Implement / emulate file sharing.
+         *
+         * We need another mode which allows skipping this stuff completely
+         * and do things the UNIX way. So for the present this is just a debug
+         * aid that can be enabled by developers too lazy to test on Windows.
+         */
+#if 0 && defined(RT_OS_LINUX)
+        if (iErr == 0)
+        {
+            /* This approach doesn't work because only knfsd checks for these
+               buggers. :-( */
+            int iLockOp;
+            switch (fOpen & RTFILE_O_DENY_MASK)
+            {
+                default:
+                AssertFailed();
+                case RTFILE_O_DENY_NONE:
+                case RTFILE_O_DENY_NOT_DELETE:
+                    iLockOp = LOCK_MAND | LOCK_READ | LOCK_WRITE;
+                    break;
+                case RTFILE_O_DENY_READ:
+                case RTFILE_O_DENY_READ | RTFILE_O_DENY_NOT_DELETE:
+                    iLockOp = LOCK_MAND | LOCK_WRITE;
+                    break;
+                case RTFILE_O_DENY_WRITE:
+                case RTFILE_O_DENY_WRITE | RTFILE_O_DENY_NOT_DELETE:
+                    iLockOp = LOCK_MAND | LOCK_READ;
+                    break;
+                case RTFILE_O_DENY_WRITE | RTFILE_O_DENY_READ:
+                case RTFILE_O_DENY_WRITE | RTFILE_O_DENY_READ | RTFILE_O_DENY_NOT_DELETE:
+                    iLockOp = LOCK_MAND;
+                    break;
+            }
+            iErr = flock(fh, iLockOp | LOCK_NB);
+            if (iErr != 0)
+                iErr = errno == EAGAIN ? ETXTBSY : 0;
+        }
+#endif /* 0 && RT_OS_LINUX */
+#if defined(DEBUG_bird) && !defined(RT_OS_SOLARIS)
+        if (iErr == 0)
+        {
+            /* This emulation is incomplete but useful. */
+            switch (fOpen & RTFILE_O_DENY_MASK)
+            {
+                default:
+                AssertFailed();
+                case RTFILE_O_DENY_NONE:
+                case RTFILE_O_DENY_NOT_DELETE:
+                case RTFILE_O_DENY_READ:
+                case RTFILE_O_DENY_READ | RTFILE_O_DENY_NOT_DELETE:
+                    break;
+                case RTFILE_O_DENY_WRITE:
+                case RTFILE_O_DENY_WRITE | RTFILE_O_DENY_NOT_DELETE:
+                case RTFILE_O_DENY_WRITE | RTFILE_O_DENY_READ:
+                case RTFILE_O_DENY_WRITE | RTFILE_O_DENY_READ | RTFILE_O_DENY_NOT_DELETE:
+                    if (fOpen & RTFILE_O_WRITE)
+                    {
+                        iErr = flock(fh, LOCK_EX | LOCK_NB);
+                        if (iErr != 0)
+                            iErr = errno == EAGAIN ? ETXTBSY : 0;
+                    }
+                    break;
+            }
+        }
+#endif
+#ifdef RT_OS_SOLARIS
+        /** @todo Use fshare_t and associates, it's a perfect match. see sys/fcntl.h */
+#endif
+
+        /*
+         * We're done.
+         */
+        if (iErr == 0)
         {
             *pFile = (RTFILE)fh;
             Assert((int)*pFile == fh);
@@ -228,13 +312,23 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
                      pFile, *pFile, pszFilename, pszFilename, fOpen, rc));
             return VINF_SUCCESS;
         }
-        iErr = errno;
+
         close(fh);
     }
     return RTErrConvertFromErrno(iErr);
 }
 
 
+RTR3DECL(int)  RTFileOpenBitBucket(PRTFILE phFile, uint32_t fAccess)
+{
+    AssertReturn(   fAccess == RTFILE_O_READ
+                 || fAccess == RTFILE_O_WRITE
+                 || fAccess == RTFILE_O_READWRITE,
+                 VERR_INVALID_PARAMETER);
+    return RTFileOpen(phFile, "/dev/null", fAccess | RTFILE_O_DENY_NONE | RTFILE_O_OPEN);
+}
+
+
 RTR3DECL(int)  RTFileClose(RTFILE File)
 {
     if (close((int)File) == 0)
@@ -266,13 +360,13 @@ RTR3DECL(RTHCINTPTR) RTFileToNative(RTFILE File)
 
 RTR3DECL(int)  RTFileDelete(const char *pszFilename)
 {
-    char *pszNativeFilename;
-    int rc = rtPathToNative(&pszNativeFilename, pszFilename);
+    char const *pszNativeFilename;
+    int rc = rtPathToNative(&pszNativeFilename, pszFilename, NULL);
     if (RT_SUCCESS(rc))
     {
         if (unlink(pszNativeFilename) != 0)
             rc = RTErrConvertFromErrno(errno);
-        rtPathFreeNative(pszNativeFilename);
+        rtPathFreeNative(pszNativeFilename, pszFilename);
     }
     return rc;
 }
@@ -487,9 +581,9 @@ RTR3DECL(int)  RTFileFlush(RTFILE File)
 }
 
 
-RTR3DECL(int) RTFileIoCtl(RTFILE File, int iRequest, void *pvData, unsigned cbData, int *piRet)
+RTR3DECL(int) RTFileIoCtl(RTFILE File, unsigned long ulRequest, void *pvData, unsigned cbData, int *piRet)
 {
-    int rc = ioctl((int)File, iRequest, pvData);
+    int rc = ioctl((int)File, ulRequest, pvData);
     if (piRet)
         *piRet = rc;
     return rc >= 0 ? VINF_SUCCESS : RTErrConvertFromErrno(errno);
@@ -639,4 +733,3 @@ RTR3DECL(int) RTFileRename(const char *pszSrc, const char *pszDst, unsigned fRen
     return rc;
 }
 
-
