@@ -1,10 +1,10 @@
-/* $Id: path-posix.cpp $ */
+/* $Id: path-posix.cpp 28915 2010-04-29 18:12:35Z vboxsync $ */
 /** @file
  * IPRT - Path Manipulation, POSIX.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -65,8 +61,8 @@ RTDECL(int) RTPathReal(const char *pszPath, char *pszRealPath, size_t cchRealPat
     /*
      * Convert input.
      */
-    char *pszNativePath;
-    int rc = rtPathToNative(&pszNativePath, pszPath);
+    char const *pszNativePath;
+    int rc = rtPathToNative(&pszNativePath, pszPath, NULL);
     if (RT_SUCCESS(rc))
     {
         /*
@@ -76,25 +72,10 @@ RTDECL(int) RTPathReal(const char *pszPath, char *pszRealPath, size_t cchRealPat
         char szTmpPath[PATH_MAX + 1];
         const char *psz = realpath(pszNativePath, szTmpPath);
         if (psz)
-        {
-            /*
-             * Convert result and copy it to the return buffer.
-             */
-            char *pszUtf8RealPath;
-            rc = rtPathFromNative(&pszUtf8RealPath, szTmpPath);
-            if (RT_SUCCESS(rc))
-            {
-                size_t cch = strlen(pszUtf8RealPath) + 1;
-                if (cch <= cchRealPath)
-                    memcpy(pszRealPath, pszUtf8RealPath, cch);
-                else
-                    rc = VERR_BUFFER_OVERFLOW;
-                RTStrFree(pszUtf8RealPath);
-            }
-        }
+            rc = rtPathFromNativeCopy(pszRealPath, cchRealPath, szTmpPath, NULL);
         else
             rc = RTErrConvertFromErrno(errno);
-        RTStrFree(pszNativePath);
+        rtPathFreeNative(pszNativePath, pszPath);
     }
 
     LogFlow(("RTPathReal(%p:{%s}, %p:{%s}, %u): returns %Rrc\n", pszPath, pszPath,
@@ -363,18 +344,7 @@ static int rtPathUserHomeByPasswd(char *pszPath, size_t cchPath, uid_t uid)
     /*
      * Convert it to UTF-8 and copy it to the return buffer.
      */
-    char *pszUtf8Path;
-    rc = rtPathFromNative(&pszUtf8Path, pPasswd->pw_dir);
-    if (RT_SUCCESS(rc))
-    {
-        size_t cchHome = strlen(pszUtf8Path);
-        if (cchHome < cchPath)
-            memcpy(pszPath, pszUtf8Path, cchHome + 1);
-        else
-            rc = VERR_BUFFER_OVERFLOW;
-        RTStrFree(pszUtf8Path);
-    }
-    return rc;
+    return rtPathFromNativeCopy(pszPath, cchPath, pPasswd->pw_dir, NULL);
 }
 #endif
 
@@ -392,30 +362,15 @@ static int rtPathUserHomeByEnv(char *pszPath, size_t cchPath)
     /*
      * Get HOME env. var it and validate it's existance.
      */
-    int rc = VERR_PATH_NOT_FOUND;
-    const char *pszHome = RTEnvGet("HOME");
+    int         rc      = VERR_PATH_NOT_FOUND;
+    const char *pszHome = RTEnvGet("HOME"); /** @todo Codeset confusion in RTEnv. */
     if (pszHome)
 
     {
         struct stat st;
         if (    !stat(pszHome, &st)
             &&  S_ISDIR(st.st_mode))
-        {
-            /*
-             * Convert it to UTF-8 and copy it to the return buffer.
-             */
-            char *pszUtf8Path;
-            rc = rtPathFromNative(&pszUtf8Path, pszHome);
-            if (RT_SUCCESS(rc))
-            {
-                size_t cchHome = strlen(pszUtf8Path);
-                if (cchHome < cchPath)
-                    memcpy(pszPath, pszUtf8Path, cchHome + 1);
-                else
-                    rc = VERR_BUFFER_OVERFLOW;
-                RTStrFree(pszUtf8Path);
-            }
-        }
+            rc = rtPathFromNativeCopy(pszPath, cchPath, pszHome, NULL);
     }
     return rc;
 }
@@ -461,27 +416,38 @@ RTDECL(int) RTPathUserHome(char *pszPath, size_t cchPath)
 
 
 RTR3DECL(int) RTPathQueryInfo(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAdditionalAttribs)
+{
+    return RTPathQueryInfoEx(pszPath, pObjInfo, enmAdditionalAttribs, RTPATH_F_ON_LINK);
+}
+
+
+RTR3DECL(int) RTPathQueryInfoEx(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOBJATTRADD enmAdditionalAttribs, uint32_t fFlags)
 {
     /*
      * Validate input.
      */
-    AssertMsgReturn(VALID_PTR(pszPath), ("%p\n", pszPath), VERR_INVALID_POINTER);
+    AssertPtrReturn(pszPath, VERR_INVALID_POINTER);
     AssertReturn(*pszPath, VERR_INVALID_PARAMETER);
-    AssertMsgReturn(VALID_PTR(pObjInfo), ("%p\n", pszPath), VERR_INVALID_POINTER);
+    AssertPtrReturn(pObjInfo, VERR_INVALID_POINTER);
     AssertMsgReturn(    enmAdditionalAttribs >= RTFSOBJATTRADD_NOTHING
                     &&  enmAdditionalAttribs <= RTFSOBJATTRADD_LAST,
                     ("Invalid enmAdditionalAttribs=%p\n", enmAdditionalAttribs),
                     VERR_INVALID_PARAMETER);
+    AssertMsgReturn(RTPATH_F_IS_VALID(fFlags, 0), ("%#x\n", fFlags), VERR_INVALID_PARAMETER);
 
     /*
      * Convert the filename.
      */
-    char *pszNativePath;
-    int rc = rtPathToNative(&pszNativePath, pszPath);
+    char const *pszNativePath;
+    int rc = rtPathToNative(&pszNativePath, pszPath, NULL);
     if (RT_SUCCESS(rc))
     {
         struct stat Stat;
-        if (!stat(pszNativePath, &Stat))
+        if (fFlags & RTPATH_F_FOLLOW_LINK)
+            rc = stat(pszNativePath, &Stat);
+        else
+            rc = lstat(pszNativePath, &Stat); /** @todo how doesn't have lstat again? */
+        if (!rc)
         {
             rtFsConvertStatToObjInfo(pObjInfo, &Stat, pszPath, 0);
             switch (enmAdditionalAttribs)
@@ -504,7 +470,7 @@ RTR3DECL(int) RTPathQueryInfo(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOB
         }
         else
             rc = RTErrConvertFromErrno(errno);
-        rtPathFreeNative(pszNativePath);
+        rtPathFreeNative(pszNativePath, pszPath);
     }
 
     LogFlow(("RTPathQueryInfo(%p:{%s}, pObjInfo=%p, %d): returns %Rrc\n",
@@ -515,38 +481,39 @@ RTR3DECL(int) RTPathQueryInfo(const char *pszPath, PRTFSOBJINFO pObjInfo, RTFSOB
 
 RTR3DECL(int) RTPathSetTimes(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
                              PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime)
+{
+    return RTPathSetTimesEx(pszPath, pAccessTime, pModificationTime, pChangeTime, pBirthTime, RTPATH_F_ON_LINK);
+}
+
+
+RTR3DECL(int) RTPathSetTimesEx(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRTTIMESPEC pModificationTime,
+                               PCRTTIMESPEC pChangeTime, PCRTTIMESPEC pBirthTime, uint32_t fFlags)
 {
     /*
      * Validate input.
      */
-    AssertMsgReturn(VALID_PTR(pszPath), ("%p\n", pszPath), VERR_INVALID_POINTER);
-    AssertMsgReturn(*pszPath, ("%p\n", pszPath), VERR_INVALID_PARAMETER);
-    AssertMsgReturn(!pAccessTime || VALID_PTR(pAccessTime), ("%p\n", pAccessTime), VERR_INVALID_POINTER);
-    AssertMsgReturn(!pModificationTime || VALID_PTR(pModificationTime), ("%p\n", pModificationTime), VERR_INVALID_POINTER);
-    AssertMsgReturn(!pChangeTime || VALID_PTR(pChangeTime), ("%p\n", pChangeTime), VERR_INVALID_POINTER);
-    AssertMsgReturn(!pBirthTime || VALID_PTR(pBirthTime), ("%p\n", pBirthTime), VERR_INVALID_POINTER);
+    AssertPtrReturn(pszPath, VERR_INVALID_POINTER);
+    AssertReturn(*pszPath, VERR_INVALID_PARAMETER);
+    AssertPtrNullReturn(pAccessTime, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pModificationTime, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pChangeTime, VERR_INVALID_POINTER);
+    AssertPtrNullReturn(pBirthTime, VERR_INVALID_POINTER);
+    AssertMsgReturn(RTPATH_F_IS_VALID(fFlags, 0), ("%#x\n", fFlags), VERR_INVALID_PARAMETER);
 
     /*
      * Convert the paths.
      */
-    char *pszNativePath;
-    int rc = rtPathToNative(&pszNativePath, pszPath);
+    char const *pszNativePath;
+    int rc = rtPathToNative(&pszNativePath, pszPath, NULL);
     if (RT_SUCCESS(rc))
     {
+        RTFSOBJINFO ObjInfo;
+
         /*
          * If it's a no-op, we'll only verify the existance of the file.
          */
         if (!pAccessTime && !pModificationTime)
-        {
-            struct stat Stat;
-            if (!stat(pszNativePath, &Stat))
-                rc = VINF_SUCCESS;
-            else
-            {
-                rc = RTErrConvertFromErrno(errno);
-                Log(("RTPathSetTimes('%s',,,,): failed with %Rrc and errno=%d\n", pszPath, rc, errno));
-            }
-        }
+            rc = RTPathQueryInfoEx(pszPath, &ObjInfo, RTFSOBJATTRADD_NOTHING, fFlags);
         else
         {
             /*
@@ -561,8 +528,7 @@ RTR3DECL(int) RTPathSetTimes(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRT
             }
             else
             {
-                RTFSOBJINFO ObjInfo;
-                int rc = RTPathQueryInfo(pszPath, &ObjInfo, RTFSOBJATTRADD_UNIX);
+                rc = RTPathQueryInfoEx(pszPath, &ObjInfo, RTFSOBJATTRADD_UNIX, fFlags);
                 if (RT_SUCCESS(rc))
                 {
                     RTTimeSpecGetTimeval(pAccessTime        ? pAccessTime       : &ObjInfo.AccessTime,       &aTimevals[0]);
@@ -574,15 +540,40 @@ RTR3DECL(int) RTPathSetTimes(const char *pszPath, PCRTTIMESPEC pAccessTime, PCRT
             }
             if (RT_SUCCESS(rc))
             {
-                if (utimes(pszNativePath, aTimevals))
+                if (fFlags & RTPATH_F_FOLLOW_LINK)
                 {
-                    rc = RTErrConvertFromErrno(errno);
+                    if (utimes(pszNativePath, aTimevals))
+                        rc = RTErrConvertFromErrno(errno);
+                }
+#if (defined(RT_OS_DARWIN) && MAC_OS_X_VERSION_MIN_REQUIRED >= 1050) \
+ || defined(RT_OS_FREEBSD) \
+ || defined(RT_OS_LINUX) \
+ || defined(RT_OS_OS2) /** @todo who really has lutimes? */
+                else
+                {
+                    if (lutimes(pszNativePath, aTimevals))
+                        rc = RTErrConvertFromErrno(errno);
+                }
+#else
+                else
+                {
+                    if (pAccessTime && pModificationTime)
+                        rc = RTPathQueryInfoEx(pszPath, &ObjInfo, RTFSOBJATTRADD_UNIX, fFlags);
+                    if (RT_SUCCESS(rc) && RTFS_IS_SYMLINK(ObjInfo.Attr.fMode))
+                        rc = VERR_NS_SYMLINK_SET_TIME;
+                    else if (RT_SUCCESS(rc))
+                    {
+                        if (utimes(pszNativePath, aTimevals))
+                            rc = RTErrConvertFromErrno(errno);
+                    }
+                }
+#endif
+                if (RT_FAILURE(rc))
                     Log(("RTPathSetTimes('%s',%p,%p,,): failed with %Rrc and errno=%d\n",
                          pszPath, pAccessTime, pModificationTime, rc, errno));
-                }
             }
         }
-        rtPathFreeNative(pszNativePath);
+        rtPathFreeNative(pszNativePath, pszPath);
     }
 
     LogFlow(("RTPathSetTimes(%p:{%s}, %p:{%RDtimespec}, %p:{%RDtimespec}, %p:{%RDtimespec}, %p:{%RDtimespec}): return %Rrc\n",
@@ -625,17 +616,17 @@ static bool rtPathSame(const char *pszNativeSrc, const char *pszNativeDst)
  *                      source is a directory. If Its RTFS_TYPE_FILE we'll check that it's
  *                      not a directory (we are NOT checking whether it's a file).
  */
-int rtPathPosixRename(const char *pszSrc, const char *pszDst, unsigned fRename, RTFMODE fFileType)
+DECLHIDDEN(int) rtPathPosixRename(const char *pszSrc, const char *pszDst, unsigned fRename, RTFMODE fFileType)
 {
     /*
      * Convert the paths.
      */
-    char *pszNativeSrc;
-    int rc = rtPathToNative(&pszNativeSrc, pszSrc);
+    char const *pszNativeSrc;
+    int rc = rtPathToNative(&pszNativeSrc, pszSrc, NULL);
     if (RT_SUCCESS(rc))
     {
-        char *pszNativeDst;
-        rc = rtPathToNative(&pszNativeDst, pszDst);
+        char const *pszNativeDst;
+        rc = rtPathToNative(&pszNativeDst, pszDst, NULL);
         if (RT_SUCCESS(rc))
         {
             /*
@@ -673,7 +664,7 @@ int rtPathPosixRename(const char *pszSrc, const char *pszDst, unsigned fRename,
                     Assert(SrcStat.st_ino && DstStat.st_ino);
                     if (    SrcStat.st_dev == DstStat.st_dev
                         &&  SrcStat.st_ino == DstStat.st_ino
-                        &&  (SrcStat.st_mode & S_IFMT) == (SrcStat.st_mode & S_IFMT))
+                        &&  (SrcStat.st_mode & S_IFMT) == (DstStat.st_mode & S_IFMT))
                     {
                         /*
                          * It's likely that we're talking about the same file here.
@@ -756,9 +747,9 @@ int rtPathPosixRename(const char *pszSrc, const char *pszDst, unsigned fRename,
                 Log(("rtPathRename('%s', '%s', %#x ,%RTfmode): source type check failed rc=%Rrc errno=%d\n",
                      pszSrc, pszDst, fRename, fFileType, rc, errno));
 
-            rtPathFreeNative(pszNativeDst);
+            rtPathFreeNative(pszNativeDst, pszDst);
         }
-        rtPathFreeNative(pszNativeSrc);
+        rtPathFreeNative(pszNativeSrc, pszSrc);
     }
     return rc;
 }
@@ -786,26 +777,37 @@ RTR3DECL(int) RTPathRename(const char *pszSrc, const char *pszDst, unsigned fRen
 
 
 RTDECL(bool) RTPathExists(const char *pszPath)
+{
+    return RTPathExistsEx(pszPath, RTPATH_F_FOLLOW_LINK);
+}
+
+
+RTDECL(bool) RTPathExistsEx(const char *pszPath, uint32_t fFlags)
 {
     /*
      * Validate input.
      */
     AssertPtrReturn(pszPath, false);
     AssertReturn(*pszPath, false);
+    Assert(RTPATH_F_IS_VALID(fFlags, 0));
 
     /*
      * Convert the path and check if it exists using stat().
      */
-    char *pszNativePath;
-    int rc = rtPathToNative(&pszNativePath, pszPath);
+    char const *pszNativePath;
+    int rc = rtPathToNative(&pszNativePath, pszPath, NULL);
     if (RT_SUCCESS(rc))
     {
         struct stat Stat;
-        if (!stat(pszNativePath, &Stat))
+        if (fFlags & RTPATH_F_FOLLOW_LINK)
+            rc = stat(pszNativePath, &Stat);
+        else
+            rc = lstat(pszNativePath, &Stat);
+        if (!rc)
             rc = VINF_SUCCESS;
         else
             rc = VERR_GENERAL_FAILURE;
-        RTStrFree(pszNativePath);
+        rtPathFreeNative(pszNativePath, pszPath);
     }
     return RT_SUCCESS(rc);
 }
@@ -816,23 +818,7 @@ RTDECL(int)  RTPathGetCurrent(char *pszPath, size_t cchPath)
     int rc;
     char szNativeCurDir[RTPATH_MAX];
     if (getcwd(szNativeCurDir, sizeof(szNativeCurDir)) != NULL)
-    {
-        char *pszCurDir;
-        rc = rtPathFromNative(&pszCurDir, szNativeCurDir);
-        if (RT_SUCCESS(rc))
-        {
-            size_t cchCurDir = strlen(pszCurDir);
-            if (cchCurDir < cchPath)
-            {
-                memcpy(pszPath, pszCurDir, cchCurDir + 1);
-                RTStrFree(pszCurDir);
-                return VINF_SUCCESS;
-            }
-
-            rc = VERR_BUFFER_OVERFLOW;
-            RTStrFree(pszCurDir);
-        }
-    }
+        rc = rtPathFromNativeCopy(pszPath, cchPath, szNativeCurDir, NULL);
     else
         rc = RTErrConvertFromErrno(errno);
     return rc;
@@ -850,13 +836,13 @@ RTDECL(int) RTPathSetCurrent(const char *pszPath)
     /*
      * Change the directory.
      */
-    char *pszNativePath;
-    int rc = rtPathToNative(&pszNativePath, pszPath);
+    char const *pszNativePath;
+    int rc = rtPathToNative(&pszNativePath, pszPath, NULL);
     if (RT_SUCCESS(rc))
     {
         if (chdir(pszNativePath))
             rc = RTErrConvertFromErrno(errno);
-        RTStrFree(pszNativePath);
+        rtPathFreeNative(pszNativePath, pszPath);
     }
     return rc;
 }
