@@ -1,10 +1,10 @@
-/* $Id: alloc-solaris.cpp 11020 2008-07-30 22:48:35Z vboxsync $ */
+/* $Id: alloc-solaris.cpp 29277 2010-05-09 23:25:51Z vboxsync $ */
 /** @file
  * IPRT - Memory Allocation, POSIX.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -44,7 +40,13 @@
 #include <strings.h>
 
 
-#ifdef IN_RING3
+/*******************************************************************************
+*   Defined Constants And Macros                                               *
+*******************************************************************************/
+#if 0
+# define RT_USE_MMAP_PAGE
+#endif
+
 
 /**
  * Allocates memory which may contain code.
@@ -107,7 +109,15 @@ RTDECL(void)    RTMemExecFree(void *pv) RT_NO_THROW
  */
 RTDECL(void *) RTMemPageAlloc(size_t cb) RT_NO_THROW
 {
+#ifdef RT_USE_MMAP_PAGE
+    size_t  cbAligned = RT_ALIGN_Z(cb, PAGE_SIZE);
+    void   *pv = mmap(NULL, cbAligned, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+    AssertMsgReturn(pv != MAP_FAILED, ("errno=%d cb=%#zx\n", errno, cb), NULL);
+    return pv;
+
+#else
     return valloc(RT_ALIGN_Z(cb, PAGE_SIZE));
+#endif
 }
 
 
@@ -120,11 +130,19 @@ RTDECL(void *) RTMemPageAlloc(size_t cb) RT_NO_THROW
  */
 RTDECL(void *) RTMemPageAllocZ(size_t cb) RT_NO_THROW
 {
+#ifdef RT_USE_MMAP_PAGE
+    size_t  cbAligned = RT_ALIGN_Z(cb, PAGE_SIZE);
+    void   *pv = mmap(NULL, cbAligned, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANONYMOUS, -1, 0);
+    AssertMsgReturn(pv != MAP_FAILED, ("errno=%d cb=%#zx\n", errno, cb), NULL);
+    return pv;
+
+#else
     cb = RT_ALIGN_Z(cb, PAGE_SIZE);
     void *pv = valloc(cb);
     if (pv)
         bzero(pv, RT_ALIGN_Z(cb, PAGE_SIZE));
     return pv;
+#endif
 }
 
 
@@ -134,10 +152,18 @@ RTDECL(void *) RTMemPageAllocZ(size_t cb) RT_NO_THROW
  * @param   pv      Pointer to the block as it was returned by the allocation function.
  *                  NULL will be ignored.
  */
-RTDECL(void) RTMemPageFree(void *pv) RT_NO_THROW
+RTDECL(void) RTMemPageFree(void *pv, size_t cb) RT_NO_THROW
 {
     if (pv)
+    {
+#ifdef RT_USE_MMAP_PAGE
+        size_t cbAligned = RT_ALIGN_Z(cb, PAGE_SIZE);
+        int rc = munmap(pv, cbAligned);
+        AssertMsg(!rc, ("munmap(%p, %#zx) -> %d errno=%d\n", pv, cbAligned, rc, errno)); NOREF(rc);
+#else
         free(pv);
+#endif
+    }
 }
 
 
@@ -204,5 +230,3 @@ RTDECL(int) RTMemProtect(void *pv, size_t cb, unsigned fProtect) RT_NO_THROW
         return rc;
     return RTErrConvertFromErrno(errno);
 }
-
-#endif
