@@ -1,10 +1,10 @@
-/* $Id: alloc-win.cpp $ */
+/* $Id: alloc-win.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Memory Allocation, Win32.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -110,7 +106,7 @@ RTDECL(void *) RTMemPageAlloc(size_t cb) RT_NO_THROW
 #ifdef USE_VIRTUAL_ALLOC
     void *pv = VirtualAlloc(NULL, RT_ALIGN_Z(cb, PAGE_SIZE), MEM_COMMIT, PAGE_READWRITE);
 #else
-    void *pv = _aligned_malloc(cb, PAGE_SIZE);
+    void *pv = _aligned_malloc(RT_ALIGN_Z(cb, PAGE_SIZE), PAGE_SIZE);
 #endif
     AssertMsg(pv, ("cb=%d lasterr=%d\n", cb, GetLastError()));
     return pv;
@@ -129,7 +125,7 @@ RTDECL(void *) RTMemPageAllocZ(size_t cb) RT_NO_THROW
 #ifdef USE_VIRTUAL_ALLOC
     void *pv = VirtualAlloc(NULL, RT_ALIGN_Z(cb, PAGE_SIZE), MEM_COMMIT, PAGE_READWRITE);
 #else
-    void *pv = _aligned_malloc(cb, PAGE_SIZE);
+    void *pv = _aligned_malloc(RT_ALIGN_Z(cb, PAGE_SIZE), PAGE_SIZE);
 #endif
     if (pv)
     {
@@ -147,7 +143,7 @@ RTDECL(void *) RTMemPageAllocZ(size_t cb) RT_NO_THROW
  * @param   pv      Pointer to the block as it was returned by the allocation function.
  *                  NULL will be ignored.
  */
-RTDECL(void) RTMemPageFree(void *pv) RT_NO_THROW
+RTDECL(void) RTMemPageFree(void *pv, size_t cb) RT_NO_THROW
 {
     if (pv)
     {
