@@ -1,10 +1,10 @@
-/* $Id: fileio-win.cpp $ */
+/* $Id: fileio-win.cpp 28918 2010-04-29 18:30:09Z vboxsync $ */
 /** @file
  * IPRT - File I/O, native implementation for the Windows host platform.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -49,11 +45,6 @@
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
 *******************************************************************************/
-/** @def RT_DONT_CONVERT_FILENAMES
- * Define this to pass UTF-8 unconverted to the kernel. */
-#ifdef __DOXYGEN__
-# define RT_DONT_CONVERT_FILENAMES 1
-#endif
 
 
 /**
@@ -68,7 +59,7 @@
  * @param   poffNew     Where to store the new file offset. NULL allowed.
  * @param   uMethod     Seek method. (The windows one!)
  */
-inline bool MySetFilePointer(RTFILE File, uint64_t offSeek, uint64_t *poffNew, unsigned uMethod)
+DECLINLINE(bool) MySetFilePointer(RTFILE File, uint64_t offSeek, uint64_t *poffNew, unsigned uMethod)
 {
     bool            fRc;
     LARGE_INTEGER   off;
@@ -150,7 +141,7 @@ RTR3DECL(RTHCINTPTR) RTFileToNative(RTFILE File)
 }
 
 
-RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen)
+RTR3DECL(int) RTFileOpen(PRTFILE pFile, const char *pszFilename, uint32_t fOpen)
 {
     /*
      * Validate input.
@@ -201,27 +192,40 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
     DWORD dwDesiredAccess;
     switch (fOpen & RTFILE_O_ACCESS_MASK)
     {
-        case RTFILE_O_READ:        dwDesiredAccess = GENERIC_READ; break;
-        case RTFILE_O_WRITE:       dwDesiredAccess = GENERIC_WRITE; break;
-        case RTFILE_O_READWRITE:   dwDesiredAccess = GENERIC_READ | GENERIC_WRITE; break;
+        case RTFILE_O_READ:
+            dwDesiredAccess = FILE_GENERIC_READ; /* RTFILE_O_APPEND is ignored. */
+            break;
+        case RTFILE_O_WRITE:
+            dwDesiredAccess = fOpen & RTFILE_O_APPEND
+                            ? FILE_GENERIC_WRITE & ~FILE_WRITE_DATA
+                            : FILE_GENERIC_WRITE;
+            break;
+        case RTFILE_O_READWRITE:
+            dwDesiredAccess = fOpen & RTFILE_O_APPEND
+                            ? FILE_GENERIC_READ | (FILE_GENERIC_WRITE & ~FILE_WRITE_DATA)
+                            : FILE_GENERIC_READ | FILE_GENERIC_WRITE;
+            break;
         default:
             AssertMsgFailed(("Impossible fOpen=%#x\n", fOpen));
             return VERR_INVALID_PARAMETER;
     }
+    if (dwCreationDisposition == TRUNCATE_EXISTING)
+        /* Required for truncating the file (see MSDN), it is *NOT* part of FILE_GENERIC_WRITE. */
+        dwDesiredAccess |= GENERIC_WRITE;
 
     /* RTFileSetMode needs following rights as well. */
     switch (fOpen & RTFILE_O_ACCESS_ATTR_MASK)
     {
-        case RTFILE_O_ACCESS_ATTR_READ:      dwDesiredAccess |= FILE_READ_ATTRIBUTES | SYNCHRONIZE; break;
+        case RTFILE_O_ACCESS_ATTR_READ:      dwDesiredAccess |= FILE_READ_ATTRIBUTES  | SYNCHRONIZE; break;
         case RTFILE_O_ACCESS_ATTR_WRITE:     dwDesiredAccess |= FILE_WRITE_ATTRIBUTES | SYNCHRONIZE; break;
         case RTFILE_O_ACCESS_ATTR_READWRITE: dwDesiredAccess |= FILE_READ_ATTRIBUTES | FILE_WRITE_ATTRIBUTES | SYNCHRONIZE; break;
         default:
             /* Attributes access is the same as the file access. */
             switch (fOpen & RTFILE_O_ACCESS_MASK)
             {
-                case RTFILE_O_READ:      dwDesiredAccess |= FILE_READ_ATTRIBUTES | SYNCHRONIZE; break;
-                case RTFILE_O_WRITE:     dwDesiredAccess |= FILE_WRITE_ATTRIBUTES | SYNCHRONIZE; break;
-                case RTFILE_O_READWRITE: dwDesiredAccess |= FILE_READ_ATTRIBUTES | FILE_WRITE_ATTRIBUTES | SYNCHRONIZE; break;
+                case RTFILE_O_READ:          dwDesiredAccess |= FILE_READ_ATTRIBUTES  | SYNCHRONIZE; break;
+                case RTFILE_O_WRITE:         dwDesiredAccess |= FILE_WRITE_ATTRIBUTES | SYNCHRONIZE; break;
+                case RTFILE_O_READWRITE:     dwDesiredAccess |= FILE_READ_ATTRIBUTES | FILE_WRITE_ATTRIBUTES | SYNCHRONIZE; break;
                 default:
                     AssertMsgFailed(("Impossible fOpen=%#x\n", fOpen));
                     return VERR_INVALID_PARAMETER;
@@ -229,7 +233,6 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
     }
 
     DWORD dwShareMode;
-    Assert(RTFILE_O_DENY_READWRITE == RTFILE_O_DENY_ALL && !RTFILE_O_DENY_NONE);
     switch (fOpen & RTFILE_O_DENY_MASK)
     {
         case RTFILE_O_DENY_NONE:                                dwShareMode = FILE_SHARE_READ | FILE_SHARE_WRITE; break;
@@ -260,19 +263,17 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
     dwFlagsAndAttributes = FILE_ATTRIBUTE_NORMAL;
     if (fOpen & RTFILE_O_WRITE_THROUGH)
         dwFlagsAndAttributes |= FILE_FLAG_WRITE_THROUGH;
+    if (fOpen & RTFILE_O_ASYNC_IO)
+        dwFlagsAndAttributes |= FILE_FLAG_OVERLAPPED;
+    if (fOpen & RTFILE_O_NO_CACHE)
+    {
+        dwFlagsAndAttributes |= FILE_FLAG_NO_BUFFERING;
+        dwDesiredAccess &= ~FILE_APPEND_DATA;
+    }
 
     /*
      * Open/Create the file.
      */
-#ifdef RT_DONT_CONVERT_FILENAMES
-    HANDLE hFile = CreateFile(pszFilename,
-                              dwDesiredAccess,
-                              dwShareMode,
-                              pSecurityAttributes,
-                              dwCreationDisposition,
-                              dwFlagsAndAttributes,
-                              NULL);
-#else
     PRTUTF16 pwszFilename;
     rc = RTStrToUtf16(pszFilename, &pwszFilename);
     if (RT_FAILURE(rc))
@@ -285,7 +286,6 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
                                dwCreationDisposition,
                                dwFlagsAndAttributes,
                                NULL);
-#endif
     if (hFile != INVALID_HANDLE_VALUE)
     {
         bool fCreated = dwCreationDisposition == CREATE_ALWAYS
@@ -298,11 +298,7 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
         if (    fCreated
             &&  (fOpen & RTFILE_O_NOT_CONTENT_INDEXED))
         {
-#ifdef RT_DONT_CONVERT_FILENAMES
-            if (!SetFileAttributes(pszFilename, FILE_ATTRIBUTE_NOT_CONTENT_INDEXED))
-#else
             if (!SetFileAttributesW(pwszFilename, FILE_ATTRIBUTE_NOT_CONTENT_INDEXED))
-#endif
                 rc = RTErrConvertFromWin32(GetLastError());
         }
         /*
@@ -319,9 +315,7 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
         {
             *pFile = (RTFILE)hFile;
             Assert((HANDLE)*pFile == hFile);
-#ifndef RT_DONT_CONVERT_FILENAMES
             RTUtf16Free(pwszFilename);
-#endif
             return VINF_SUCCESS;
         }
 
@@ -329,13 +323,21 @@ RTR3DECL(int)  RTFileOpen(PRTFILE pFile, const char *pszFilename, unsigned fOpen
     }
     else
         rc = RTErrConvertFromWin32(GetLastError());
-#ifndef RT_DONT_CONVERT_FILENAMES
     RTUtf16Free(pwszFilename);
-#endif
     return rc;
 }
 
 
+RTR3DECL(int)  RTFileOpenBitBucket(PRTFILE phFile, uint32_t fAccess)
+{
+    AssertReturn(   fAccess == RTFILE_O_READ
+                 || fAccess == RTFILE_O_WRITE
+                 || fAccess == RTFILE_O_READWRITE,
+                 VERR_INVALID_PARAMETER);
+    return RTFileOpen(phFile, "NUL", fAccess | RTFILE_O_DENY_NONE | RTFILE_O_OPEN);
+}
+
+
 RTR3DECL(int)  RTFileClose(RTFILE File)
 {
     if (CloseHandle((HANDLE)File))
@@ -448,11 +450,9 @@ RTR3DECL(int)  RTFileWrite(RTFILE File, const void *pvBuf, size_t cbToWrite, siz
 
 RTR3DECL(int)  RTFileFlush(RTFILE File)
 {
-    int rc;
-
-    if (FlushFileBuffers((HANDLE)File) == FALSE)
+    if (!FlushFileBuffers((HANDLE)File))
     {
-        rc = GetLastError();
+        int rc = GetLastError();
         Log(("FlushFileBuffers failed with %d\n", rc));
         return RTErrConvertFromWin32(rc);
     }
@@ -770,14 +770,17 @@ RTR3DECL(int) RTFileSetMode(RTFILE File, RTFMODE fMode)
 }
 
 
+RTR3DECL(int) RTFileQueryFsSizes(RTFILE hFile, PRTFOFF pcbTotal, RTFOFF *pcbFree,
+                                 uint32_t *pcbBlock, uint32_t *pcbSector)
+{
+    /** @todo implement this using NtQueryVolumeInformationFile(hFile,,,,
+     *        FileFsSizeInformation). */
+    return VERR_NOT_SUPPORTED;
+}
+
+
 RTR3DECL(int)  RTFileDelete(const char *pszFilename)
 {
-#ifdef RT_DONT_CONVERT_FILENAMES
-    if (DeleteFile(pszFilename))
-        return VINF_SUCCESS;
-    return RTErrConvertFromWin32(GetLastError());
-
-#else
     PRTUTF16 pwszFilename;
     int rc = RTStrToUtf16(pszFilename, &pwszFilename);
     if (RT_SUCCESS(rc))
@@ -788,7 +791,6 @@ RTR3DECL(int)  RTFileDelete(const char *pszFilename)
     }
 
     return rc;
-#endif
 }
 
 
