@@ -1,10 +1,10 @@
-/* $Id: localipc-win.cpp 11725 2008-08-27 22:21:47Z vboxsync $ */
+/* $Id: localipc-win.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT - Local IPC, Windows Implementation Using Named Pipes.
  */
 
 /*
- * Copyright (C) 2008 Sun Microsystems, Inc.
+ * Copyright (C) 2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -192,10 +188,17 @@ static DWORD rtLocalIpcServerWinCreatePipeInstance(PHANDLE phNmPipe, const char
      */
     DWORD err;
     PSECURITY_DESCRIPTOR pSecDesc = NULL;
+#if 0 /** @todo dynamically resolve this as it is the only thing that prevents
+       * loading IPRT on NT4. */
     if (ConvertStringSecurityDescriptorToSecurityDescriptor(RTLOCALIPC_WIN_SDDL,
                                                             SDDL_REVISION_1,
                                                             &pSecDesc,
                                                             NULL))
+#else
+    AssertFatalFailed();
+    SetLastError(-1);
+    if (0)
+#endif
     {
         SECURITY_ATTRIBUTES SecAttrs;
         SecAttrs.nLength = sizeof(SecAttrs);
@@ -636,4 +639,3 @@ RTDECL(int) RTLocalIpcSessionQueryGroupId(RTLOCALIPCSESSION hSession, PRTUID pUi
     return VERR_NOT_SUPPORTED;
 }
 
-
