@@ -1,10 +1,10 @@
-/* $Id: tstInlineAsm.cpp 14831 2008-11-30 10:31:16Z vboxsync $ */
+/* $Id: tstInlineAsm.cpp 29279 2010-05-09 23:29:11Z vboxsync $ */
 /** @file
  * IPRT Testcase - inline assembly.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,29 +22,26 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
 *   Header Files                                                               *
 *******************************************************************************/
 #include <iprt/asm.h>
+#include <iprt/asm-math.h>
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
+# include <iprt/asm-amd64-x86.h>
+#else
+# include <iprt/time.h>
+#endif
 #include <iprt/stream.h>
 #include <iprt/string.h>
 #include <iprt/initterm.h>
 #include <iprt/param.h>
 #include <iprt/thread.h>
+#include <iprt/test.h>
 
 
-/*******************************************************************************
-*   Global Variables                                                           *
-*******************************************************************************/
-/** Global error count. */
-static unsigned g_cErrors;
-
 
 /*******************************************************************************
 *   Defined Constants And Macros                                               *
@@ -54,7 +51,7 @@ static unsigned g_cErrors;
     { \
         if ((val) != (expect)) \
         { \
-            g_cErrors++; \
+            RTTestIErrorInc(); \
             RTPrintf("%s, %d: " #val ": expected " fmt " got " fmt "\n", __FUNCTION__, __LINE__, (expect), (val)); \
         } \
     } while (0)
@@ -65,13 +62,14 @@ static unsigned g_cErrors;
         type val = op; \
         if (val != (type)(expect)) \
         { \
-            g_cErrors++; \
+            RTTestIErrorInc(); \
             RTPrintf("%s, %d: " #op ": expected " fmt " got " fmt "\n", __FUNCTION__, __LINE__, (type)(expect), val); \
         } \
     } while (0)
 
 
-#if !defined(PIC) || !defined(RT_ARCH_X86)
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
+
 const char *getCacheAss(unsigned u)
 {
     if (u == 0)
@@ -438,8 +436,8 @@ void tstASMCpuId(void)
                   s.uEBX, s.uEBX);
      }
 }
-#endif /* !PIC || !X86 */
 
+#endif /* AMD64 || X86 */
 
 static void tstASMAtomicXchgU8(void)
 {
@@ -553,65 +551,6 @@ static void tstASMAtomicXchgU64(void)
 }
 
 
-#ifdef RT_ARCH_AMD64
-static void tstASMAtomicXchgU128(void)
-{
-    struct
-    {
-        RTUINT128U  u128Dummy0;
-        RTUINT128U  u128;
-        RTUINT128U  u128Dummy1;
-    } s;
-    RTUINT128U u128Ret;
-    RTUINT128U u128Arg;
-
-
-    s.u128Dummy0.s.Lo = s.u128Dummy0.s.Hi = 0x1122334455667788;
-    s.u128.s.Lo = 0;
-    s.u128.s.Hi = 0;
-    s.u128Dummy1 = s.u128Dummy0;
-
-    u128Arg.s.Lo = 1;
-    u128Arg.s.Hi = 0;
-    u128Ret.u = ASMAtomicXchgU128(&s.u128.u, u128Arg.u);
-    CHECKVAL(u128Ret.s.Lo, 0ULL, "%#llx");
-    CHECKVAL(u128Ret.s.Hi, 0ULL, "%#llx");
-    CHECKVAL(s.u128.s.Lo, 1ULL, "%#llx");
-    CHECKVAL(s.u128.s.Hi, 0ULL, "%#llx");
-
-    u128Arg.s.Lo = 0;
-    u128Arg.s.Hi = 0;
-    u128Ret.u = ASMAtomicXchgU128(&s.u128.u, u128Arg.u);
-    CHECKVAL(u128Ret.s.Lo, 1ULL, "%#llx");
-    CHECKVAL(u128Ret.s.Hi, 0ULL, "%#llx");
-    CHECKVAL(s.u128.s.Lo, 0ULL, "%#llx");
-    CHECKVAL(s.u128.s.Hi, 0ULL, "%#llx");
-
-    u128Arg.s.Lo = ~0ULL;
-    u128Arg.s.Hi = ~0ULL;
-    u128Ret.u = ASMAtomicXchgU128(&s.u128.u, u128Arg.u);
-    CHECKVAL(u128Ret.s.Lo, 0ULL, "%#llx");
-    CHECKVAL(u128Ret.s.Hi, 0ULL, "%#llx");
-    CHECKVAL(s.u128.s.Lo, ~0ULL, "%#llx");
-    CHECKVAL(s.u128.s.Hi, ~0ULL, "%#llx");
-
-
-    u128Arg.s.Lo = 0xfedcba0987654321ULL;
-    u128Arg.s.Hi = 0x8897a6b5c4d3e2f1ULL;
-    u128Ret.u = ASMAtomicXchgU128(&s.u128.u, u128Arg.u);
-    CHECKVAL(u128Ret.s.Lo, ~0ULL, "%#llx");
-    CHECKVAL(u128Ret.s.Hi, ~0ULL, "%#llx");
-    CHECKVAL(s.u128.s.Lo, 0xfedcba0987654321ULL, "%#llx");
-    CHECKVAL(s.u128.s.Hi, 0x8897a6b5c4d3e2f1ULL, "%#llx");
-
-    CHECKVAL(s.u128Dummy0.s.Lo, 0x1122334455667788, "%#llx");
-    CHECKVAL(s.u128Dummy0.s.Hi, 0x1122334455667788, "%#llx");
-    CHECKVAL(s.u128Dummy1.s.Lo, 0x1122334455667788, "%#llx");
-    CHECKVAL(s.u128Dummy1.s.Hi, 0x1122334455667788, "%#llx");
-}
-#endif
-
-
 static void tstASMAtomicXchgPtr(void)
 {
     void *pv = NULL;
@@ -627,6 +566,29 @@ static void tstASMAtomicXchgPtr(void)
 }
 
 
+static void tstASMAtomicCmpXchgU8(void)
+{
+    struct
+    {
+        uint8_t u8Before;
+        uint8_t u8;
+        uint8_t u8After;
+    } u = { 0xcc, 0xff, 0xaa };
+
+    CHECKOP(ASMAtomicCmpXchgU8(&u.u8, 0, 0), false, "%d", bool);
+    CHECKVAL(u.u8, 0xff, "%x"); CHECKVAL(u.u8Before, 0xcc, "%x"); CHECKVAL(u.u8After, 0xaa, "%x");
+
+    CHECKOP(ASMAtomicCmpXchgU8(&u.u8, 0, 0xff), true, "%d", bool);
+    CHECKVAL(u.u8, 0, "%x");    CHECKVAL(u.u8Before, 0xcc, "%x"); CHECKVAL(u.u8After, 0xaa, "%x");
+
+    CHECKOP(ASMAtomicCmpXchgU8(&u.u8, 0x79, 0xff), false, "%d", bool);
+    CHECKVAL(u.u8, 0, "%x");    CHECKVAL(u.u8Before, 0xcc, "%x"); CHECKVAL(u.u8After, 0xaa, "%x");
+
+    CHECKOP(ASMAtomicCmpXchgU8(&u.u8, 0x97, 0), true, "%d", bool);
+    CHECKVAL(u.u8, 0x97, "%x"); CHECKVAL(u.u8Before, 0xcc, "%x"); CHECKVAL(u.u8After, 0xaa, "%x");
+}
+
+
 static void tstASMAtomicCmpXchgU32(void)
 {
     uint32_t u32 = 0xffffffff;
@@ -741,6 +703,23 @@ static void tstASMAtomicReadU64(void)
 }
 
 
+static void tstASMAtomicUoReadU64(void)
+{
+    uint64_t u64 = 0;
+
+    CHECKOP(ASMAtomicUoReadU64(&u64), 0ULL, "%#llx", uint64_t);
+    CHECKVAL(u64, 0ULL, "%#llx");
+
+    u64 = ~0ULL;
+    CHECKOP(ASMAtomicUoReadU64(&u64), ~0ULL, "%#llx", uint64_t);
+    CHECKVAL(u64, ~0ULL, "%#llx");
+
+    u64 = 0xfedcba0987654321ULL;
+    CHECKOP(ASMAtomicUoReadU64(&u64), 0xfedcba0987654321ULL, "%#llx", uint64_t);
+    CHECKVAL(u64, 0xfedcba0987654321ULL, "%#llx");
+}
+
+
 static void tstASMAtomicAddS32(void)
 {
     int32_t i32Rc;
@@ -751,12 +730,12 @@ static void tstASMAtomicAddS32(void)
         if (i32Rc != (rc)) \
         { \
             RTPrintf("%s, %d: FAILURE: %s -> %d expected %d\n", __FUNCTION__, __LINE__, #op, i32Rc, rc); \
-            g_cErrors++; \
+            RTTestIErrorInc(); \
         } \
         if (i32 != (val)) \
         { \
             RTPrintf("%s, %d: FAILURE: %s => i32=%d expected %d\n", __FUNCTION__, __LINE__, #op, i32, val); \
-            g_cErrors++; \
+            RTTestIErrorInc(); \
         } \
     } while (0)
     MYCHECK(ASMAtomicAddS32(&i32, 1),               10,             11);
@@ -780,12 +759,12 @@ static void tstASMAtomicDecIncS32(void)
         if (i32Rc != (rc)) \
         { \
             RTPrintf("%s, %d: FAILURE: %s -> %d expected %d\n", __FUNCTION__, __LINE__, #op, i32Rc, rc); \
-            g_cErrors++; \
+            RTTestIErrorInc(); \
         } \
         if (i32 != (rc)) \
         { \
             RTPrintf("%s, %d: FAILURE: %s => i32=%d expected %d\n", __FUNCTION__, __LINE__, #op, i32, rc); \
-            g_cErrors++; \
+            RTTestIErrorInc(); \
         } \
     } while (0)
     MYCHECK(ASMAtomicDecS32(&i32), 9);
@@ -875,29 +854,64 @@ void tstASMMemZeroPage(void)
         ||  Buf3.u64Magic2 != UINT64_C(0xffffffffffffffff))
     {
         RTPrintf("tstInlineAsm: ASMMemZeroPage violated one/both magic(s)!\n");
-        g_cErrors++;
+        RTTestIErrorInc();
     }
     for (unsigned i = 0; i < sizeof(Buf1.abPage); i++)
         if (Buf1.abPage[i])
         {
             RTPrintf("tstInlineAsm: ASMMemZeroPage didn't clear byte at offset %#x!\n", i);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
     for (unsigned i = 0; i < sizeof(Buf2.abPage); i++)
         if (Buf2.abPage[i])
         {
             RTPrintf("tstInlineAsm: ASMMemZeroPage didn't clear byte at offset %#x!\n", i);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
     for (unsigned i = 0; i < sizeof(Buf3.abPage); i++)
         if (Buf3.abPage[i])
         {
             RTPrintf("tstInlineAsm: ASMMemZeroPage didn't clear byte at offset %#x!\n", i);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
 }
 
 
+void tstASMMemIsZeroPage(RTTEST hTest)
+{
+    RTTestSub(hTest, "ASMMemIsZeroPage");
+
+    void *pvPage1 = RTTestGuardedAllocHead(hTest, PAGE_SIZE);
+    void *pvPage2 = RTTestGuardedAllocTail(hTest, PAGE_SIZE);
+    RTTESTI_CHECK_RETV(pvPage1 && pvPage2);
+
+    memset(pvPage1, 0, PAGE_SIZE);
+    memset(pvPage2, 0, PAGE_SIZE);
+    RTTESTI_CHECK(ASMMemIsZeroPage(pvPage1));
+    RTTESTI_CHECK(ASMMemIsZeroPage(pvPage2));
+
+    memset(pvPage1, 0xff, PAGE_SIZE);
+    memset(pvPage2, 0xff, PAGE_SIZE);
+    RTTESTI_CHECK(!ASMMemIsZeroPage(pvPage1));
+    RTTESTI_CHECK(!ASMMemIsZeroPage(pvPage2));
+
+    memset(pvPage1, 0, PAGE_SIZE);
+    memset(pvPage2, 0, PAGE_SIZE);
+    for (unsigned off = 0; off < PAGE_SIZE; off++)
+    {
+        ((uint8_t *)pvPage1)[off] = 1;
+        RTTESTI_CHECK(!ASMMemIsZeroPage(pvPage1));
+        ((uint8_t *)pvPage1)[off] = 0;
+
+        ((uint8_t *)pvPage2)[off] = 0x80;
+        RTTESTI_CHECK(!ASMMemIsZeroPage(pvPage2));
+        ((uint8_t *)pvPage2)[off] = 0;
+    }
+
+    RTTestSubDone(hTest);
+}
+
+
 void tstASMMemZero32(void)
 {
     struct
@@ -927,25 +941,25 @@ void tstASMMemZero32(void)
         ||  Buf3.u64Magic2 != UINT64_C(0xffffffffffffffff))
     {
         RTPrintf("tstInlineAsm: ASMMemZero32 violated one/both magic(s)!\n");
-        g_cErrors++;
+        RTTestIErrorInc();
     }
     for (unsigned i = 0; i < RT_ELEMENTS(Buf1.abPage); i++)
         if (Buf1.abPage[i])
         {
             RTPrintf("tstInlineAsm: ASMMemZero32 didn't clear byte at offset %#x!\n", i);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
     for (unsigned i = 0; i < RT_ELEMENTS(Buf2.abPage); i++)
         if (Buf2.abPage[i])
         {
             RTPrintf("tstInlineAsm: ASMMemZero32 didn't clear byte at offset %#x!\n", i);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
     for (unsigned i = 0; i < RT_ELEMENTS(Buf3.abPage); i++)
         if (Buf3.abPage[i])
         {
             RTPrintf("tstInlineAsm: ASMMemZero32 didn't clear byte at offset %#x!\n", i);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
 }
 
@@ -991,25 +1005,25 @@ void tstASMMemFill32(void)
         ||  Buf3.u64Magic2 != UINT64_C(0xffffffffffffffff))
     {
         RTPrintf("tstInlineAsm: ASMMemFill32 violated one/both magic(s)!\n");
-        g_cErrors++;
+        RTTestIErrorInc();
     }
     for (unsigned i = 0; i < RT_ELEMENTS(Buf1.au32Page); i++)
         if (Buf1.au32Page[i] != 0xdeadbeef)
         {
             RTPrintf("tstInlineAsm: ASMMemFill32 %#x: %#x exepcted %#x\n", i, Buf1.au32Page[i], 0xdeadbeef);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
     for (unsigned i = 0; i < RT_ELEMENTS(Buf2.au32Page); i++)
         if (Buf2.au32Page[i] != 0xcafeff01)
         {
             RTPrintf("tstInlineAsm: ASMMemFill32 %#x: %#x exepcted %#x\n", i, Buf2.au32Page[i], 0xcafeff01);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
     for (unsigned i = 0; i < RT_ELEMENTS(Buf3.au32Page); i++)
         if (Buf3.au32Page[i] != 0xf00dd00f)
         {
             RTPrintf("tstInlineAsm: ASMMemFill32 %#x: %#x exepcted %#x\n", i, Buf3.au32Page[i], 0xf00dd00f);
-            g_cErrors++;
+            RTTestIErrorInc();
         }
 }
 
@@ -1023,6 +1037,7 @@ void tstASMMath(void)
     uint32_t u32 = ASMDivU64ByU32RetU32(UINT64_C(0x0800000000000000), UINT32_C(0x10000000));
     CHECKVAL(u32, UINT32_C(0x80000000), "%#010RX32");
 
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
     u64 = ASMMultU64ByU32DivByU32(UINT64_C(0x0000000000000001), UINT32_C(0x00000001), UINT32_C(0x00000001));
     CHECKVAL(u64, UINT64_C(0x0000000000000001), "%#018RX64");
     u64 = ASMMultU64ByU32DivByU32(UINT64_C(0x0000000100000000), UINT32_C(0x80000000), UINT32_C(0x00000002));
@@ -1038,7 +1053,7 @@ void tstASMMath(void)
     u64 = ASMMultU64ByU32DivByU32(UINT64_C(0x3415934810359583), UINT32_C(0xf8694045), UINT32_C(0x58734981));
     CHECKVAL(u64, UINT64_C(0x924719355cd35a27), "%#018RX64");
 
-#if 0 /* bird: question is whether this should trap or not:
+# if 0 /* bird: question is whether this should trap or not:
        *
        * frank: Of course it must trap:
        *
@@ -1068,7 +1083,8 @@ void tstASMMath(void)
        */
     u64 = ASMMultU64ByU32DivByU32(UINT64_C(0xfffffff8c65d6731), UINT32_C(0x77d7daf8), UINT32_C(0x3b9aca00));
     CHECKVAL(u64, UINT64_C(0x02b8f9a2aa74e3dc), "%#018RX64");
-#endif
+# endif
+#endif /* AMD64 || X86 */
 
     u32 = ASMModU64ByU32RetU32(UINT64_C(0x0ffffff8c65d6731), UINT32_C(0x77d7daf8));
     CHECKVAL(u32, UINT32_C(0x3B642451), "%#010RX32");
@@ -1165,20 +1181,50 @@ void tstASMBench(void)
     static uint64_t volatile s_u64;
     static int64_t  volatile s_i64;
     register unsigned i;
-    const unsigned cRounds = 1000000;
+    const unsigned cRounds = 2000000;
     register uint64_t u64Elapsed;
 
     RTPrintf("tstInlineASM: Benchmarking:\n");
 
-#define BENCH(op, str)  \
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
+# define BENCH(op, str) \
+    do { \
         RTThreadYield(); \
         u64Elapsed = ASMReadTSC(); \
         for (i = cRounds; i > 0; i--) \
             op; \
         u64Elapsed = ASMReadTSC() - u64Elapsed; \
-        RTPrintf(" %-30s %3llu cycles\n", str, u64Elapsed / cRounds);
+        RTPrintf(" %-30s %3llu cycles\n", str, u64Elapsed / cRounds); \
+    } while (0)
+#else
+# define BENCH(op, str) \
+    do { \
+        RTThreadYield(); \
+        u64Elapsed = RTTimeNanoTS(); \
+        for (i = cRounds; i > 0; i--) \
+            op; \
+        u64Elapsed = RTTimeNanoTS() - u64Elapsed; \
+        RTPrintf(" %-30s %3llu ns\n", str, u64Elapsed / cRounds); \
+    } while (0)
+#endif
 
     BENCH(s_u32 = 0,                            "s_u32 = 0:");
+    BENCH(ASMAtomicUoReadU8(&s_u8),             "ASMAtomicUoReadU8:");
+    BENCH(ASMAtomicUoReadS8(&s_i8),             "ASMAtomicUoReadS8:");
+    BENCH(ASMAtomicUoReadU16(&s_u16),           "ASMAtomicUoReadU16:");
+    BENCH(ASMAtomicUoReadS16(&s_i16),           "ASMAtomicUoReadS16:");
+    BENCH(ASMAtomicUoReadU32(&s_u32),           "ASMAtomicUoReadU32:");
+    BENCH(ASMAtomicUoReadS32(&s_i32),           "ASMAtomicUoReadS32:");
+    BENCH(ASMAtomicUoReadU64(&s_u64),           "ASMAtomicUoReadU64:");
+    BENCH(ASMAtomicUoReadS64(&s_i64),           "ASMAtomicUoReadS64:");
+    BENCH(ASMAtomicReadU8(&s_u8),               "ASMAtomicReadU8:");
+    BENCH(ASMAtomicReadS8(&s_i8),               "ASMAtomicReadS8:");
+    BENCH(ASMAtomicReadU16(&s_u16),             "ASMAtomicReadU16:");
+    BENCH(ASMAtomicReadS16(&s_i16),             "ASMAtomicReadS16:");
+    BENCH(ASMAtomicReadU32(&s_u32),             "ASMAtomicReadU32:");
+    BENCH(ASMAtomicReadS32(&s_i32),             "ASMAtomicReadS32:");
+    BENCH(ASMAtomicReadU64(&s_u64),             "ASMAtomicReadU64:");
+    BENCH(ASMAtomicReadS64(&s_i64),             "ASMAtomicReadS64:");
     BENCH(ASMAtomicUoWriteU8(&s_u8, 0),         "ASMAtomicUoWriteU8:");
     BENCH(ASMAtomicUoWriteS8(&s_i8, 0),         "ASMAtomicUoWriteS8:");
     BENCH(ASMAtomicUoWriteU16(&s_u16, 0),       "ASMAtomicUoWriteU16:");
@@ -1226,46 +1272,44 @@ void tstASMBench(void)
 
 int main(int argc, char *argv[])
 {
-    RTR3Init();
-    RTPrintf("tstInlineAsm: TESTING\n");
+    RTTEST hTest;
+    int rc = RTTestInitAndCreate("tstInlineAsm", &hTest);
+    if (rc)
+        return rc;
+    RTTestBanner(hTest);
 
     /*
      * Execute the tests.
      */
-#if !defined(PIC) || !defined(RT_ARCH_X86)
+#if defined(RT_ARCH_AMD64) || defined(RT_ARCH_X86)
     tstASMCpuId();
 #endif
     tstASMAtomicXchgU8();
     tstASMAtomicXchgU16();
     tstASMAtomicXchgU32();
     tstASMAtomicXchgU64();
-#ifdef RT_ARCH_AMD64
-    tstASMAtomicXchgU128();
-#endif
     tstASMAtomicXchgPtr();
+    tstASMAtomicCmpXchgU8();
     tstASMAtomicCmpXchgU32();
     tstASMAtomicCmpXchgU64();
     tstASMAtomicCmpXchgExU32();
     tstASMAtomicCmpXchgExU64();
     tstASMAtomicReadU64();
+    tstASMAtomicUoReadU64();
     tstASMAtomicAddS32();
     tstASMAtomicDecIncS32();
     tstASMAtomicAndOrU32();
     tstASMMemZeroPage();
+    tstASMMemIsZeroPage(hTest);
     tstASMMemZero32();
     tstASMMemFill32();
     tstASMMath();
     tstASMByteSwap();
-
     tstASMBench();
 
     /*
      * Show the result.
      */
-    if (!g_cErrors)
-        RTPrintf("tstInlineAsm: SUCCESS\n", g_cErrors);
-    else
-        RTPrintf("tstInlineAsm: FAILURE - %d errors\n", g_cErrors);
-    return !!g_cErrors;
+    return RTTestSummaryAndDestroy(hTest);
 }
 
