@@ -1,10 +1,10 @@
-/* $Id: tstRTAvl.cpp $ */
+/* $Id: tstRTAvl.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IPRT Testcase - AVL trees.
  */
 
 /*
- * Copyright (C) 2006-2009 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2009 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -840,7 +836,10 @@ int avlrogcphys(void)
     {
         AVLROGCPHYSTREE     Tree;
         AVLROGCPHYSNODECORE aNode[4];
-    } s1 = {0}, s2 = {0}, s3 = {0};
+    }   s1, s2, s3;
+    RT_ZERO(s1);
+    RT_ZERO(s2);
+    RT_ZERO(s3);
 
     s1.aNode[0].Key        = 0x00030000;
     s1.aNode[0].KeyLast    = 0x00030fff;
