@@ -1,10 +1,10 @@
-/* $Id: CFGM.cpp $ */
+/* $Id: CFGM.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * CFGM - Configuration Manager.
  */
 
 /*
- * Copyright (C) 2006-2008 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2008 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /** @page pg_cfgm       CFGM - The Configuration Manager
@@ -29,7 +25,7 @@
  * The configuration is normally created via a callback passed to VMR3Create()
  * via the pfnCFGMConstructor parameter. To make testcase writing a bit simpler,
  * we allow the callback to be NULL, in which case a simple default
- * configuration will be created by cfgmR3CreateDefaultTree(). The
+ * configuration will be created by CFGMR3ConstructDefaultTree(). The
  * Console::configConstructor() method in Main/ConsoleImpl2.cpp creates the
  * configuration from the XML.
  *
@@ -75,7 +71,6 @@
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-static int  cfgmR3CreateDefaultTree(PVM pVM);
 static void cfgmR3DumpPath(PCFGMNODE pNode, PCDBGFINFOHLP pHlp);
 static void cfgmR3Dump(PCFGMNODE pRoot, unsigned iLevel, PCDBGFINFOHLP pHlp);
 static DECLCALLBACK(void) cfgmR3Info(PVM pVM, PCDBGFINFOHLP pHlp, const char *pszArgs);
@@ -113,34 +108,29 @@ VMMR3DECL(int) CFGMR3Init(PVM pVM, PFNCFGMCONSTRUCTOR pfnCFGMConstructor, void *
     AssertRCReturn(rc,rc);
 
     /*
-     * Create the configuration tree.
+     * Root Node.
      */
-    if (pfnCFGMConstructor)
-    {
-        /*
-         * Root Node.
-         */
-        PCFGMNODE pRoot = (PCFGMNODE)MMR3HeapAllocZ(pVM, MM_TAG_CFGM, sizeof(*pRoot));
-        if (!pRoot)
-            return VERR_NO_MEMORY;
-        pRoot->pVM           = pVM;
-        pRoot->cchName       = 0;
-        pVM->cfgm.s.pRoot   = pRoot;
+    PCFGMNODE pRoot = (PCFGMNODE)MMR3HeapAllocZ(pVM, MM_TAG_CFGM, sizeof(*pRoot));
+    if (!pRoot)
+        return VERR_NO_MEMORY;
+    pRoot->pVM        = pVM;
+    pRoot->cchName    = 0;
+    pVM->cfgm.s.pRoot = pRoot;
 
         /*
-         * Call the constructor.
+         * Call the constructor if specified, if not use the default one.
          */
+    if (pfnCFGMConstructor)
         rc = pfnCFGMConstructor(pVM, pvUser);
-    }
     else
-        rc = cfgmR3CreateDefaultTree(pVM);
+        rc = CFGMR3ConstructDefaultTree(pVM);
     if (RT_SUCCESS(rc))
     {
         Log(("CFGMR3Init: Successfully constructed the configuration\n"));
         CFGMR3Dump(CFGMR3GetRoot(pVM));
     }
     else
-        NOT_DMIK(AssertMsgFailed(("Constructor failed with rc=%Rrc pfnCFGMConstructor=%p\n", rc, pfnCFGMConstructor)));
+        AssertMsgFailed(("Constructor failed with rc=%Rrc pfnCFGMConstructor=%p\n", rc, pfnCFGMConstructor));
 
     return rc;
 }
@@ -263,9 +253,9 @@ VMMR3DECL(PCFGMNODE) CFGMR3GetChildFV(PCFGMNODE pNode, const char *pszPathFormat
     {
         PCFGMNODE pChild;
         int rc = cfgmR3ResolveNode(pNode, pszPath, &pChild);
+        RTStrFree(pszPath);
         if (RT_SUCCESS(rc))
             return pChild;
-        RTStrFree(pszPath);
     }
     return NULL;
 }
@@ -352,6 +342,8 @@ VMMR3DECL(size_t) CFGMR3GetNameLen(PCFGMNODE pCur)
  * @param   pNode           The node which children should be examined.
  * @param   pszzValid       List of valid names separated by '\\0' and ending with
  *                          a double '\\0'.
+ *
+ * @deprecated Use CFGMR3ValidateConfig.
  */
 VMMR3DECL(bool) CFGMR3AreChildrenValid(PCFGMNODE pNode, const char *pszzValid)
 {
@@ -476,6 +468,7 @@ VMMR3DECL(CFGMVALUETYPE) CFGMR3GetValueType(PCFGMLEAF pCur)
  * @param   pNode           The node which values should be examined.
  * @param   pszzValid       List of valid names separated by '\\0' and ending with
  *                          a double '\\0'.
+ * @deprecated Use CFGMR3ValidateConfig.
  */
 VMMR3DECL(bool) CFGMR3AreValuesValid(PCFGMNODE pNode, const char *pszzValid)
 {
@@ -554,7 +547,7 @@ VMMR3DECL(int) CFGMR3QuerySize(PCFGMNODE pNode, const char *pszName, size_t *pcb
                 break;
 
             case CFGMVALUETYPE_STRING:
-                *pcb = pLeaf->Value.String.cch;
+                *pcb = pLeaf->Value.String.cb;
                 break;
 
             case CFGMVALUETYPE_BYTES:
@@ -643,10 +636,11 @@ VMMR3DECL(int) CFGMR3QueryString(PCFGMNODE pNode, const char *pszName, char *psz
     {
         if (pLeaf->enmType == CFGMVALUETYPE_STRING)
         {
-            if (cchString >= pLeaf->Value.String.cch)
+            size_t cbSrc = pLeaf->Value.String.cb;
+            if (cchString >= cbSrc)
             {
-                memcpy(pszString, pLeaf->Value.String.psz, pLeaf->Value.String.cch);
-                memset(pszString + pLeaf->Value.String.cch, 0, cchString - pLeaf->Value.String.cch);
+                memcpy(pszString, pLeaf->Value.String.psz, cbSrc);
+                memset(pszString + cbSrc, 0, cchString - cbSrc);
             }
             else
                 rc = VERR_CFGM_NOT_ENOUGH_SPACE;
@@ -676,10 +670,11 @@ VMMR3DECL(int) CFGMR3QueryStringDef(PCFGMNODE pNode, const char *pszName, char *
     {
         if (pLeaf->enmType == CFGMVALUETYPE_STRING)
         {
-            if (cchString >= pLeaf->Value.String.cch)
+            size_t cbSrc = pLeaf->Value.String.cb;
+            if (cchString >= cbSrc)
             {
-                memcpy(pszString, pLeaf->Value.String.psz, pLeaf->Value.String.cch);
-                memset(pszString + pLeaf->Value.String.cch, 0, cchString - pLeaf->Value.String.cch);
+                memcpy(pszString, pLeaf->Value.String.psz, cbSrc);
+                memset(pszString + cbSrc, 0, cchString - cbSrc);
             }
             else
                 rc = VERR_CFGM_NOT_ENOUGH_SPACE;
@@ -739,29 +734,101 @@ VMMR3DECL(int) CFGMR3QueryBytes(PCFGMNODE pNode, const char *pszName, void *pvDa
 
 
 /**
- * Creates the default configuration.
- * This assumes an empty tree.
+ * Validate one level of a configuration node.
+ *
+ * This replaces the CFGMR3AreChildrenValid and CFGMR3AreValuesValid APIs.
+ *
+ * @returns VBox status code.
+ *
+ *          When an error is returned, both VMSetError and AssertLogRelMsgFailed
+ *          have been called.  So, all the caller needs to do is to propagate
+ *          the error status up to PDM.
+ *
+ * @param   pNode               The node to validate.
+ * @param   pszNode             The node path, always ends with a slash.  Use
+ *                              "/" for the root config node.
+ * @param   pszValidValues      Patterns describing the valid value names.  See
+ *                              RTStrSimplePatternMultiMatch for details on the
+ *                              pattern syntax.
+ * @param   pszValidNodes       Patterns describing the valid node (key) names.
+ *                              See RTStrSimplePatternMultiMatch for details on
+ *                              the pattern syntax.
+ * @param   pszWho              Who is calling.
+ * @param   uInstance           The instance number of the caller.
+ */
+VMMR3DECL(int) CFGMR3ValidateConfig(PCFGMNODE pNode, const char *pszNode,
+                                    const char *pszValidValues, const char *pszValidNodes,
+                                    const char *pszWho, uint32_t uInstance)
+{
+    /* Input validation. */
+    AssertPtrNullReturn(pNode, VERR_INVALID_POINTER);
+    AssertPtrReturn(pszNode, VERR_INVALID_POINTER);
+    Assert(*pszNode && pszNode[strlen(pszNode) - 1] == '/');
+    AssertPtrReturn(pszValidValues, VERR_INVALID_POINTER);
+    AssertPtrReturn(pszValidNodes, VERR_INVALID_POINTER);
+    AssertPtrReturn(pszWho, VERR_INVALID_POINTER);
+
+    if (pNode)
+    {
+        /*
+         * Enumerate the leafs and check them against pszValidValues.
+         */
+        for (PCFGMLEAF pLeaf = pNode->pFirstLeaf; pLeaf; pLeaf = pLeaf->pNext)
+        {
+            if (!RTStrSimplePatternMultiMatch(pszValidValues, RTSTR_MAX,
+                                              pLeaf->szName, pLeaf->cchName,
+                                              NULL))
+            {
+                AssertLogRelMsgFailed(("%s/%u: Value '%s/%s' didn't match '%s'\n",
+                                       pszWho, uInstance, pszNode, pLeaf->szName, pszValidValues));
+                return VMSetError(pNode->pVM, VERR_CFGM_CONFIG_UNKNOWN_VALUE, RT_SRC_POS,
+                                  N_("Unknown configuration value '%s/%s' found in the configuration of %s instance #%u"),
+                                  pszNode, pLeaf->szName, pszWho, uInstance);
+            }
+
+        }
+
+        /*
+         * Enumerate the child nodes and check them against pszValidNodes.
+         */
+        for (PCFGMNODE pChild = pNode->pFirstChild; pChild; pChild = pChild->pNext)
+        {
+            if (!RTStrSimplePatternMultiMatch(pszValidNodes, RTSTR_MAX,
+                                              pChild->szName, pChild->cchName,
+                                              NULL))
+            {
+                AssertLogRelMsgFailed(("%s/%u: Node '%s/%s' didn't match '%s'\n",
+                                       pszWho, uInstance, pszNode, pChild->szName, pszValidNodes));
+                return VMSetError(pNode->pVM, VERR_CFGM_CONFIG_UNKNOWN_NODE, RT_SRC_POS,
+                                  N_("Unknown configuration node '%s/%s' found in the configuration of %s instance #%u"),
+                                  pszNode, pChild->szName, pszWho, uInstance);
+            }
+        }
+    }
+
+    /* All is well. */
+    return VINF_SUCCESS;
+}
+
+
+
+/**
+ * Populates the CFGM tree with the default configuration.
+ *
+ * This assumes an empty tree and is intended for testcases and such that only
+ * need to do very small adjustments to the config.
  *
  * @returns VBox status code.
  * @param   pVM     VM handle.
  */
-static int cfgmR3CreateDefaultTree(PVM pVM)
+VMMR3DECL(int) CFGMR3ConstructDefaultTree(PVM pVM)
 {
     int rc;
     int rcAll = VINF_SUCCESS;
 #define UPDATERC() do { if (RT_FAILURE(rc) && RT_SUCCESS(rcAll)) rcAll = rc; } while (0)
 
-    /*
-     * Root level.
-     */
-    PCFGMNODE pRoot = (PCFGMNODE)MMR3HeapAllocZ(pVM, MM_TAG_CFGM, sizeof(*pRoot));
-    if (!pRoot)
-        return VERR_NO_MEMORY;
-    pRoot->pVM           = pVM;
-    pRoot->cchName       = 0;
-
-    Assert(!pVM->cfgm.s.pRoot);
-    pVM->cfgm.s.pRoot   = pRoot;
+    PCFGMNODE pRoot = CFGMR3GetRoot(pVM);
+    AssertReturn(pRoot, VERR_WRONG_ORDER);
 
     /*
      * Create VM default values.
@@ -1017,50 +1084,52 @@ static int cfgmR3CreateDefaultTree(PVM pVM)
  */
 static int cfgmR3ResolveNode(PCFGMNODE pNode, const char *pszPath, PCFGMNODE *ppChild)
 {
-    if (pNode)
+    if (!pNode)
+        return VERR_CFGM_NO_PARENT;
+    PCFGMNODE pChild = NULL;
+    for (;;)
     {
-        PCFGMNODE pChild = NULL;
-        for (;;)
+        /* skip leading slashes. */
+        while (*pszPath == '/')
+            pszPath++;
+
+        /* End of path? */
+        if (!*pszPath)
         {
-            /* skip leading slashes. */
-            while (*pszPath == '/')
-                pszPath++;
-
-            /* End of path? */
-            if (!*pszPath)
-            {
-                if (!pChild)
-                    return VERR_CFGM_INVALID_CHILD_PATH;
-                *ppChild = pChild;
-                return VINF_SUCCESS;
-            }
-
-            /* find end of component. */
-            const char *pszNext = strchr(pszPath, '/');
-            if (!pszNext)
-                pszNext = strchr(pszPath,  '\0');
-            RTUINT cchName = pszNext - pszPath;
-
-            /* search child list. */
-            pChild = pNode->pFirstChild;
-            for ( ; pChild; pChild = pChild->pNext)
-                if (    pChild->cchName == cchName
-                    &&  !memcmp(pszPath, pChild->szName, cchName) )
-                    break;
-
-            /* if not found, we're done. */
             if (!pChild)
-                return VERR_CFGM_CHILD_NOT_FOUND;
-
-            /* next iteration */
-            pNode = pChild;
-            pszPath = pszNext;
+                return VERR_CFGM_INVALID_CHILD_PATH;
+            *ppChild = pChild;
+            return VINF_SUCCESS;
         }
 
-        /* won't get here */
+        /* find end of component. */
+        const char *pszNext = strchr(pszPath, '/');
+        if (!pszNext)
+            pszNext = strchr(pszPath,  '\0');
+        RTUINT cchName = pszNext - pszPath;
+
+        /* search child list. */
+        pChild = pNode->pFirstChild;
+        for ( ; pChild; pChild = pChild->pNext)
+            if (pChild->cchName == cchName)
+            {
+                int iDiff = memcmp(pszPath, pChild->szName, cchName);
+                if (iDiff <= 0)
+                {
+                    if (iDiff != 0)
+                        pChild = NULL;
+                    break;
+                }
+            }
+        if (!pChild)
+            return VERR_CFGM_CHILD_NOT_FOUND;
+
+        /* next iteration */
+        pNode = pChild;
+        pszPath = pszNext;
     }
-    else
-        return VERR_CFGM_NO_PARENT;
+
+    /* won't get here */
 }
 
 
@@ -1074,28 +1143,29 @@ static int cfgmR3ResolveNode(PCFGMNODE pNode, const char *pszPath, PCFGMNODE *pp
  */
 static int cfgmR3ResolveLeaf(PCFGMNODE pNode, const char *pszName, PCFGMLEAF *ppLeaf)
 {
-    int rc;
-    if (pNode)
+    if (!pNode)
+        return VERR_CFGM_NO_PARENT;
+
+    size_t      cchName = strlen(pszName);
+    PCFGMLEAF   pLeaf   = pNode->pFirstLeaf;
+    while (pLeaf)
     {
-        size_t      cchName = strlen(pszName);
-        PCFGMLEAF   pLeaf = pNode->pFirstLeaf;
-        while (pLeaf)
+        if (cchName == pLeaf->cchName)
         {
-            if (    cchName == pLeaf->cchName
-                && !memcmp(pszName, pLeaf->szName, cchName) )
+            int iDiff = memcmp(pszName, pLeaf->szName, cchName);
+            if (iDiff <= 0)
             {
+                if (iDiff != 0)
+                    break;
                 *ppLeaf = pLeaf;
                 return VINF_SUCCESS;
             }
-
-            /* next */
-            pLeaf = pLeaf->pNext;
         }
-        rc = VERR_CFGM_VALUE_NOT_FOUND;
+
+        /* next */
+        pLeaf = pLeaf->pNext;
     }
-    else
-        rc = VERR_CFGM_NO_PARENT;
-    return rc;
+    return VERR_CFGM_VALUE_NOT_FOUND;
 }
 
 
@@ -1184,6 +1254,34 @@ VMMR3DECL(int) CFGMR3InsertSubTree(PCFGMNODE pNode, const char *pszName, PCFGMNO
 }
 
 
+/**
+ * Compares two names.
+ *
+ * @returns Similar to memcpy.
+ * @param   pszName1            The first name.
+ * @param   cchName1            The length of the first name.
+ * @param   pszName2            The second name.
+ * @param   cchName2            The length of the second name.
+ */
+DECLINLINE(int) cfgmR3CompareNames(const char *pszName1, size_t cchName1, const char *pszName2, size_t cchName2)
+{
+    int iDiff;
+    if (cchName1 <= cchName2)
+    {
+        iDiff = memcmp(pszName1, pszName2, cchName1);
+        if (!iDiff && cchName1 < cchName2)
+            iDiff = -1;
+    }
+    else
+    {
+        iDiff = memcmp(pszName1, pszName2, cchName2);
+        if (!iDiff)
+            iDiff = 1;
+    }
+    return iDiff;
+}
+
+
 /**
  * Insert a node.
  *
@@ -1262,17 +1360,20 @@ VMMR3DECL(int) CFGMR3InsertNode(PCFGMNODE pNode, const char *pszName, PCFGMNODE
             /*
              * Check if already exists and find last node in chain.
              */
-            size_t cchName = strlen(pszName);
-            PCFGMNODE pPrev = pNode->pFirstChild;
-            if (pPrev)
+            size_t    cchName = strlen(pszName);
+            PCFGMNODE pPrev   = NULL;
+            PCFGMNODE pNext   = pNode->pFirstChild;
+            if (pNext)
             {
-                for (;; pPrev = pPrev->pNext)
+                for ( ; pNext; pPrev = pNext, pNext = pNext->pNext)
                 {
-                    if (    cchName == pPrev->cchName
-                        &&  !memcmp(pszName, pPrev->szName, cchName))
-                        return VERR_CFGM_NODE_EXISTS;
-                    if (!pPrev->pNext)
+                    int iDiff = cfgmR3CompareNames(pszName, cchName, pNext->szName, pNext->cchName);
+                    if (iDiff <= 0)
+                    {
+                        if (!iDiff)
+                            return VERR_CFGM_NODE_EXISTS;
                         break;
+                    }
                 }
             }
 
@@ -1293,12 +1394,15 @@ VMMR3DECL(int) CFGMR3InsertNode(PCFGMNODE pNode, const char *pszName, PCFGMNODE
                 /*
                  * Insert into child list.
                  */
-                pNew->pNext         = NULL;
                 pNew->pPrev         = pPrev;
                 if (pPrev)
                     pPrev->pNext    = pNew;
                 else
                     pNode->pFirstChild = pNew;
+                pNew->pNext         = pNext;
+                if (pNext)
+                    pNext->pPrev    = pNew;
+
                 if (ppChild)
                     *ppChild = pNew;
                 rc = VINF_SUCCESS;
@@ -1398,17 +1502,20 @@ static int cfgmR3InsertLeaf(PCFGMNODE pNode, const char *pszName, PCFGMLEAF *ppL
             /*
              * Check if already exists and find last node in chain.
              */
-            size_t cchName = strlen(pszName);
-            PCFGMLEAF pPrev = pNode->pFirstLeaf;
-            if (pPrev)
+            size_t    cchName  = strlen(pszName);
+            PCFGMLEAF pPrev    = NULL;
+            PCFGMLEAF pNext    = pNode->pFirstLeaf;
+            if (pNext)
             {
-                for (;; pPrev = pPrev->pNext)
+                for ( ; pNext; pPrev = pNext, pNext = pNext->pNext)
                 {
-                    if (    cchName == pPrev->cchName
-                        &&  !memcmp(pszName, pPrev->szName, cchName))
-                        return VERR_CFGM_LEAF_EXISTS;
-                    if (!pPrev->pNext)
+                    int iDiff = cfgmR3CompareNames(pszName, cchName, pNext->szName, pNext->cchName);
+                    if (iDiff <= 0)
+                    {
+                        if (!iDiff)
+                            return VERR_CFGM_LEAF_EXISTS;
                         break;
+                    }
                 }
             }
 
@@ -1424,12 +1531,15 @@ static int cfgmR3InsertLeaf(PCFGMNODE pNode, const char *pszName, PCFGMLEAF *ppL
                 /*
                  * Insert into child list.
                  */
-                pNew->pNext         = NULL;
                 pNew->pPrev         = pPrev;
                 if (pPrev)
                     pPrev->pNext    = pNew;
                 else
                     pNode->pFirstLeaf = pNew;
+                pNew->pNext         = pNext;
+                if (pNext)
+                    pNext->pPrev    = pNew;
+
                 *ppLeaf = pNew;
                 rc = VINF_SUCCESS;
             }
@@ -1547,7 +1657,7 @@ static void cfgmR3FreeValue(PCFGMLEAF pLeaf)
             case CFGMVALUETYPE_STRING:
                 MMR3HeapFree(pLeaf->Value.String.psz);
                 pLeaf->Value.String.psz = NULL;
-                pLeaf->Value.String.cch = 0;
+                pLeaf->Value.String.cb = 0;
                 break;
 
             case CFGMVALUETYPE_INTEGER:
@@ -1595,11 +1705,11 @@ VMMR3DECL(int) CFGMR3InsertString(PCFGMNODE pNode, const char *pszName, const ch
         /*
          * Allocate string object first.
          */
-        size_t cchString = strlen(pszString) + 1;
-        char *pszStringCopy = (char *)MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_STRING, RT_ALIGN_Z(cchString, 16));
+        size_t cbString = strlen(pszString) + 1;
+        char *pszStringCopy = (char *)MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_STRING, cbString);
         if (pszStringCopy)
         {
-            memcpy(pszStringCopy, pszString, cchString);
+            memcpy(pszStringCopy, pszString, cbString);
 
             /*
              * Create value leaf and set it to string type.
@@ -1610,8 +1720,10 @@ VMMR3DECL(int) CFGMR3InsertString(PCFGMNODE pNode, const char *pszName, const ch
             {
                 pLeaf->enmType = CFGMVALUETYPE_STRING;
                 pLeaf->Value.String.psz = pszStringCopy;
-                pLeaf->Value.String.cch = cchString;
+                pLeaf->Value.String.cb  = cbString;
             }
+            else
+                MMR3HeapFree(pszStringCopy);
         }
         else
             rc = VERR_NO_MEMORY;
@@ -1623,6 +1735,91 @@ VMMR3DECL(int) CFGMR3InsertString(PCFGMNODE pNode, const char *pszName, const ch
 }
 
 
+/**
+ * Same as CFGMR3InsertString except the string value given in RTStrPrintfV
+ * fashion.
+ *
+ * @returns VBox status code.
+ * @param   pNode           Parent node.
+ * @param   pszName         Value name.
+ * @param   pszFormat       The value given as a format string.
+ * @param   va              Argument to pszFormat.
+ */
+VMMR3DECL(int) CFGMR3InsertStringFV(PCFGMNODE pNode, const char *pszName, const char *pszFormat, va_list va)
+{
+    int rc;
+    if (pNode)
+    {
+        /*
+         * Allocate string object first.
+         */
+        char *pszString = MMR3HeapAPrintfVU(pNode->pVM->pUVM, MM_TAG_CFGM_STRING, pszFormat, va);
+        if (pszString)
+        {
+            /*
+             * Create value leaf and set it to string type.
+             */
+            PCFGMLEAF pLeaf;
+            rc = cfgmR3InsertLeaf(pNode, pszName, &pLeaf);
+            if (RT_SUCCESS(rc))
+            {
+                pLeaf->enmType = CFGMVALUETYPE_STRING;
+                pLeaf->Value.String.psz = pszString;
+                pLeaf->Value.String.cb  = strlen(pszString) + 1;
+            }
+            else
+                MMR3HeapFree(pszString);
+        }
+        else
+            rc = VERR_NO_MEMORY;
+    }
+    else
+        rc = VERR_CFGM_NO_PARENT;
+
+    return rc;
+}
+
+
+/**
+ * Same as CFGMR3InsertString except the string value given in RTStrPrintf
+ * fashion.
+ *
+ * @returns VBox status code.
+ * @param   pNode           Parent node.
+ * @param   pszName         Value name.
+ * @param   pszFormat       The value given as a format string.
+ * @param   ...             Argument to pszFormat.
+ */
+VMMR3DECL(int) CFGMR3InsertStringF(PCFGMNODE pNode, const char *pszName, const char *pszFormat, ...)
+{
+    va_list va;
+    va_start(va, pszFormat);
+    int rc = CFGMR3InsertStringFV(pNode, pszName, pszFormat, va);
+    va_end(va);
+    return rc;
+}
+
+
+/**
+ * Same as CFGMR3InsertString except the string value given as a UTF-16 string.
+ *
+ * @returns VBox status code.
+ * @param   pNode           Parent node.
+ * @param   pszName         Value name.
+ * @param   pwszValue       The string value (UTF-16).
+ */
+VMMR3DECL(int) CFGMR3InsertStringW(PCFGMNODE pNode, const char *pszName, PCRTUTF16 pwszValue)
+{
+    char *pszValue;
+    int rc = RTUtf16ToUtf8(pwszValue, &pszValue);
+    if (RT_SUCCESS(rc))
+    {
+        rc = CFGMR3InsertString(pNode, pszName, pszValue);
+        RTStrFree(pszValue);
+    }
+    return rc;
+}
+
 
 /**
  * Inserts a new integer value.
@@ -1643,7 +1840,7 @@ VMMR3DECL(int) CFGMR3InsertBytes(PCFGMNODE pNode, const char *pszName, const voi
             /*
              * Allocate string object first.
              */
-            void *pvCopy = MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_STRING, RT_ALIGN_Z(cbBytes, 16));
+            void *pvCopy = MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_STRING, cbBytes);
             if (pvCopy || !cbBytes)
             {
                 memcpy(pvCopy, pvBytes, cbBytes);
@@ -2390,14 +2587,14 @@ VMMR3DECL(int) CFGMR3QueryGCPtrSDef(PCFGMNODE pNode, const char *pszName, PRTGCI
  */
 VMMR3DECL(int) CFGMR3QueryStringAlloc(PCFGMNODE pNode, const char *pszName, char **ppszString)
 {
-    size_t cch;
-    int rc = CFGMR3QuerySize(pNode, pszName, &cch);
+    size_t cbString;
+    int rc = CFGMR3QuerySize(pNode, pszName, &cbString);
     if (RT_SUCCESS(rc))
     {
-        char *pszString = (char *)MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_USER, cch);
+        char *pszString = (char *)MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_USER, cbString);
         if (pszString)
         {
-            rc = CFGMR3QueryString(pNode, pszName, pszString, cch);
+            rc = CFGMR3QueryString(pNode, pszName, pszString, cbString);
             if (RT_SUCCESS(rc))
                 *ppszString = pszString;
             else
@@ -2422,19 +2619,19 @@ VMMR3DECL(int) CFGMR3QueryStringAlloc(PCFGMNODE pNode, const char *pszName, char
  */
 VMMR3DECL(int) CFGMR3QueryStringAllocDef(PCFGMNODE pNode, const char *pszName, char **ppszString, const char *pszDef)
 {
-    size_t cch;
-    int rc = CFGMR3QuerySize(pNode, pszName, &cch);
+    size_t cbString;
+    int rc = CFGMR3QuerySize(pNode, pszName, &cbString);
     if (rc == VERR_CFGM_VALUE_NOT_FOUND || rc == VERR_CFGM_NO_PARENT)
     {
-        cch = strlen(pszDef) + 1;
+        cbString = strlen(pszDef) + 1;
         rc = VINF_SUCCESS;
     }
     if (RT_SUCCESS(rc))
     {
-        char *pszString = (char *)MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_USER, cch);
+        char *pszString = (char *)MMR3HeapAlloc(pNode->pVM, MM_TAG_CFGM_USER, cbString);
         if (pszString)
         {
-            rc = CFGMR3QueryStringDef(pNode, pszName, pszString, cch, pszDef);
+            rc = CFGMR3QueryStringDef(pNode, pszName, pszString, cbString, pszDef);
             if (RT_SUCCESS(rc))
                 *ppszString = pszString;
             else
@@ -2455,7 +2652,7 @@ VMMR3DECL(int) CFGMR3QueryStringAllocDef(PCFGMNODE pNode, const char *pszName, c
 VMMR3DECL(void) CFGMR3Dump(PCFGMNODE pRoot)
 {
     LogRel(("************************* CFGM dump *************************\n"));
-    cfgmR3Info(pRoot->pVM, DBGFR3InfoLogRelHlp(), NULL);
+    cfgmR3Dump(pRoot, 0, DBGFR3InfoLogRelHlp());
     LogRel(("********************* End of CFGM dump **********************\n"));
 }
 
@@ -2532,11 +2729,11 @@ static void cfgmR3Dump(PCFGMNODE pRoot, unsigned iLevel, PCDBGFINFOHLP pHlp)
                 break;
 
             case CFGMVALUETYPE_STRING:
-                pHlp->pfnPrintf(pHlp, "  %-*s <string>  = \"%s\" (cch=%d)\n", (int)cchMax, pLeaf->szName, pLeaf->Value.String.psz, pLeaf->Value.String.cch);
+                pHlp->pfnPrintf(pHlp, "  %-*s <string>  = \"%s\" (cb=%zu)\n", (int)cchMax, pLeaf->szName, pLeaf->Value.String.psz, pLeaf->Value.String.cb);
                 break;
 
             case CFGMVALUETYPE_BYTES:
-                pHlp->pfnPrintf(pHlp, "  %-*s <bytes>   = \"%.*Rhxs\" (cb=%d)\n", (int)cchMax, pLeaf->szName, pLeaf->Value.Bytes.cb, pLeaf->Value.Bytes.pau8, pLeaf->Value.Bytes.cb);
+                pHlp->pfnPrintf(pHlp, "  %-*s <bytes>   = \"%.*Rhxs\" (cb=%zu)\n", (int)cchMax, pLeaf->szName, pLeaf->Value.Bytes.cb, pLeaf->Value.Bytes.pau8, pLeaf->Value.Bytes.cb);
                 break;
 
             default:
