@@ -1,10 +1,10 @@
-/* $Id: DBGF.cpp 14660 2008-11-26 18:20:22Z vboxsync $ */
+/* $Id: DBGF.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DBGF - Debugger Facility.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -122,7 +118,7 @@ DECLINLINE(DBGFCMD) dbgfR3SetCmd(PVM pVM, DBGFCMD enmCmd)
         AssertMsg(pVM->dbgf.s.enmVMMCmd == DBGFCMD_NO_COMMAND, ("enmCmd=%d enmVMMCmd=%d\n", enmCmd, pVM->dbgf.s.enmVMMCmd));
         rc = (DBGFCMD)ASMAtomicXchgU32((uint32_t volatile *)(void *)&pVM->dbgf.s.enmVMMCmd, enmCmd);
         VM_FF_SET(pVM, VM_FF_DBGF);
-        VMR3NotifyFF(pVM, false /* didn't notify REM */);
+        VMR3NotifyGlobalFFU(pVM->pUVM, 0 /* didn't notify REM */);
     }
     return rc;
 }
@@ -137,6 +133,8 @@ DECLINLINE(DBGFCMD) dbgfR3SetCmd(PVM pVM, DBGFCMD enmCmd)
 VMMR3DECL(int) DBGFR3Init(PVM pVM)
 {
     int rc = dbgfR3InfoInit(pVM);
+    if (RT_SUCCESS(rc))
+        rc = dbgfR3AsInit(pVM);
     if (RT_SUCCESS(rc))
         rc = dbgfR3SymInit(pVM);
     if (RT_SUCCESS(rc))
@@ -207,6 +205,7 @@ VMMR3DECL(int) DBGFR3Term(PVM pVM)
      * Terminate the other bits.
      */
     dbgfR3OSTerm(pVM);
+    dbgfR3AsTerm(pVM);
     dbgfR3InfoTerm(pVM);
     return VINF_SUCCESS;
 }
@@ -222,6 +221,7 @@ VMMR3DECL(int) DBGFR3Term(PVM pVM)
  */
 VMMR3DECL(void) DBGFR3Relocate(PVM pVM, RTGCINTPTR offDelta)
 {
+    dbgfR3AsRelocate(pVM, offDelta);
 }
 
 
@@ -290,30 +290,30 @@ bool dbgfR3WaitForAttach(PVM pVM, DBGFEVENTTYPE enmEvent)
  */
 VMMR3DECL(int) DBGFR3VMMForcedAction(PVM pVM)
 {
-    /*
-     * Clear the FF DBGF request flag.
-     */
-    Assert(pVM->fForcedActions & VM_FF_DBGF);
-    VM_FF_CLEAR(pVM, VM_FF_DBGF);
-
-    /*
-     * Commands?
-     */
     int rc = VINF_SUCCESS;
-    if (pVM->dbgf.s.enmVMMCmd != DBGFCMD_NO_COMMAND)
+
+    if (VM_FF_TESTANDCLEAR(pVM, VM_FF_DBGF))
     {
-        /** @todo stupid GDT/LDT sync hack. go away! */
-        SELMR3UpdateFromCPUM(pVM);
+        PVMCPU pVCpu = VMMGetCpu(pVM);
 
         /*
-         * Process the command.
+         * Commands?
          */
-        bool            fResumeExecution;
-        DBGFCMDDATA     CmdData = pVM->dbgf.s.VMMCmdData;
-        DBGFCMD         enmCmd = dbgfR3SetCmd(pVM, DBGFCMD_NO_COMMAND);
-        rc = dbgfR3VMMCmd(pVM, enmCmd, &CmdData, &fResumeExecution);
-        if (!fResumeExecution)
-            rc = dbgfR3VMMWait(pVM);
+        if (pVM->dbgf.s.enmVMMCmd != DBGFCMD_NO_COMMAND)
+        {
+            /** @todo stupid GDT/LDT sync hack. go away! */
+            SELMR3UpdateFromCPUM(pVM, pVCpu);
+
+            /*
+             * Process the command.
+             */
+            bool            fResumeExecution;
+            DBGFCMDDATA     CmdData = pVM->dbgf.s.VMMCmdData;
+            DBGFCMD         enmCmd = dbgfR3SetCmd(pVM, DBGFCMD_NO_COMMAND);
+            rc = dbgfR3VMMCmd(pVM, enmCmd, &CmdData, &fResumeExecution);
+            if (!fResumeExecution)
+                rc = dbgfR3VMMWait(pVM);
+        }
     }
     return rc;
 }
@@ -343,14 +343,17 @@ static void dbgfR3EventSetStoppedInHyperFlag(PVM pVM, DBGFEVENTTYPE enmEvent)
 
 
 /**
- * Try determin the event context.
+ * Try to determine the event context.
  *
  * @returns debug event context.
  * @param   pVM         The VM handle.
  */
 static DBGFEVENTCTX dbgfR3FigureEventCtx(PVM pVM)
 {
-    switch (EMGetState(pVM))
+    /** @todo SMP support! */
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
+    switch (EMGetState(pVCpu))
     {
         case EMSTATE_RAW:
         case EMSTATE_DEBUG_GUEST_RAW:
@@ -380,6 +383,9 @@ static DBGFEVENTCTX dbgfR3FigureEventCtx(PVM pVM)
  */
 static int dbgfR3EventPrologue(PVM pVM, DBGFEVENTTYPE enmEvent)
 {
+    /** @todo SMP */
+    PVMCPU pVCpu = VMMGetCpu(pVM);
+
     /*
      * Check if a debugger is attached.
      */
@@ -395,7 +401,7 @@ static int dbgfR3EventPrologue(PVM pVM, DBGFEVENTTYPE enmEvent)
      */
     dbgfR3EventSetStoppedInHyperFlag(pVM, enmEvent);
     if (!pVM->dbgf.s.fStoppedInHyper)
-        REMR3StateUpdate(pVM);
+        REMR3StateUpdate(pVM, pVCpu);
 
     /*
      * Look thru pending commands and finish those which make sense now.
@@ -553,9 +559,12 @@ VMMR3DECL(int) DBGFR3EventBreakpoint(PVM pVM, DBGFEVENTTYPE enmEvent)
     /*
      * Send the event and process the reply communication.
      */
+    /** @todo SMP */
+    PVMCPU pVCpu = VMMGetCpu0(pVM);
+
     pVM->dbgf.s.DbgEvent.enmType = enmEvent;
-    RTUINT iBp = pVM->dbgf.s.DbgEvent.u.Bp.iBp = pVM->dbgf.s.iActiveBp;
-    pVM->dbgf.s.iActiveBp = ~0U;
+    RTUINT iBp = pVM->dbgf.s.DbgEvent.u.Bp.iBp = pVCpu->dbgf.s.iActiveBp;
+    pVCpu->dbgf.s.iActiveBp = ~0U;
     if (iBp != ~0U)
         pVM->dbgf.s.DbgEvent.enmCtx = DBGFEVENTCTX_RAW;
     else
@@ -564,8 +573,8 @@ VMMR3DECL(int) DBGFR3EventBreakpoint(PVM pVM, DBGFEVENTTYPE enmEvent)
 #if 0   /** @todo get flat PC api! */
         uint32_t eip = CPUMGetGuestEIP(pVM);
 #else
-        /* @todo SMP */
-        PCPUMCTX pCtx = CPUMQueryGuestCtxPtrEx(pVM, VMMGetCpuEx(pVM, 0));
+        /* @todo SMP support!! */
+        PCPUMCTX pCtx = CPUMQueryGuestCtxPtr(VMMGetCpu(pVM));
         RTGCPTR  eip = pCtx->rip + pCtx->csHid.u64Base;
 #endif
         for (iBp = 0; iBp < RT_ELEMENTS(pVM->dbgf.s.aBreakpoints); iBp++)
@@ -590,10 +599,12 @@ VMMR3DECL(int) DBGFR3EventBreakpoint(PVM pVM, DBGFEVENTTYPE enmEvent)
  */
 static int dbgfR3VMMWait(PVM pVM)
 {
+    PVMCPU pVCpu = VMMGetCpu(pVM);
+
     LogFlow(("dbgfR3VMMWait:\n"));
 
     /** @todo stupid GDT/LDT sync hack. go away! */
-    SELMR3UpdateFromCPUM(pVM);
+    SELMR3UpdateFromCPUM(pVM, pVCpu);
     int rcRet = VINF_SUCCESS;
 
     /*
@@ -604,60 +615,82 @@ static int dbgfR3VMMWait(PVM pVM)
         /*
          * Wait.
          */
+        uint32_t cPollHack = 1; /** @todo this interface is horrible now that we're using lots of VMR3ReqCall stuff all over DBGF. */
         for (;;)
         {
-            int rc = RTSemPingWait(&pVM->dbgf.s.PingPong, 250);
-            if (RT_SUCCESS(rc))
-                break;
-            if (rc != VERR_TIMEOUT)
+            int rc;
+            if (    !VM_FF_ISPENDING(pVM, VM_FF_EMT_RENDEZVOUS | VM_FF_REQUEST)
+                &&  !VMCPU_FF_ISPENDING(pVCpu, VMCPU_FF_REQUEST))
             {
-                LogFlow(("dbgfR3VMMWait: returns %Rrc\n", rc));
-                return rc;
-            }
-
-            if (VM_FF_ISSET(pVM, VM_FF_REQUEST))
-            {
-                LogFlow(("dbgfR3VMMWait: Processes requests...\n"));
-                rc = VMR3ReqProcessU(pVM->pUVM, VMREQDEST_ANY);
-                LogFlow(("dbgfR3VMMWait: VMR3ReqProcess -> %Rrc rcRet=%Rrc\n", rc, rcRet));
-                if (rc >= VINF_EM_FIRST && rc <= VINF_EM_LAST)
-                {
-                    switch (rc)
-                    {
-                        case VINF_EM_DBG_BREAKPOINT:
-                        case VINF_EM_DBG_STEPPED:
-                        case VINF_EM_DBG_STEP:
-                        case VINF_EM_DBG_STOP:
-                            AssertMsgFailed(("rc=%Rrc\n", rc));
-                            break;
-
-                        /* return straight away */
-                        case VINF_EM_TERMINATE:
-                        case VINF_EM_OFF:
-                            LogFlow(("dbgfR3VMMWait: returns %Rrc\n", rc));
-                            return rc;
-
-                        /* remember return code. */
-                        default:
-                            AssertReleaseMsgFailed(("rc=%Rrc is not in the switch!\n", rc));
-                        case VINF_EM_RESET:
-                        case VINF_EM_SUSPEND:
-                        case VINF_EM_HALT:
-                        case VINF_EM_RESUME:
-                        case VINF_EM_RESCHEDULE:
-                        case VINF_EM_RESCHEDULE_REM:
-                        case VINF_EM_RESCHEDULE_RAW:
-                            if (rc < rcRet || rcRet == VINF_SUCCESS)
-                                rcRet = rc;
-                            break;
-                    }
-                }
-                else if (RT_FAILURE(rc))
+                rc = RTSemPingWait(&pVM->dbgf.s.PingPong, cPollHack);
+                if (RT_SUCCESS(rc))
+                    break;
+                if (rc != VERR_TIMEOUT)
                 {
                     LogFlow(("dbgfR3VMMWait: returns %Rrc\n", rc));
                     return rc;
                 }
             }
+
+            if (VM_FF_ISPENDING(pVM, VM_FF_EMT_RENDEZVOUS))
+            {
+                rc = VMMR3EmtRendezvousFF(pVM, pVCpu);
+                cPollHack = 1;
+            }
+            else if (   VM_FF_ISPENDING(pVM, VM_FF_REQUEST)
+                     || VMCPU_FF_ISPENDING(pVCpu, VMCPU_FF_REQUEST))
+            {
+                LogFlow(("dbgfR3VMMWait: Processes requests...\n"));
+                rc = VMR3ReqProcessU(pVM->pUVM, VMCPUID_ANY);
+                if (rc == VINF_SUCCESS)
+                    rc = VMR3ReqProcessU(pVM->pUVM, pVCpu->idCpu);
+                LogFlow(("dbgfR3VMMWait: VMR3ReqProcess -> %Rrc rcRet=%Rrc\n", rc, rcRet));
+                cPollHack = 1;
+            }
+            else
+            {
+                rc = VINF_SUCCESS;
+                if (cPollHack < 120)
+                    cPollHack++;
+            }
+
+            if (rc >= VINF_EM_FIRST && rc <= VINF_EM_LAST)
+            {
+                switch (rc)
+                {
+                    case VINF_EM_DBG_BREAKPOINT:
+                    case VINF_EM_DBG_STEPPED:
+                    case VINF_EM_DBG_STEP:
+                    case VINF_EM_DBG_STOP:
+                        AssertMsgFailed(("rc=%Rrc\n", rc));
+                        break;
+
+                    /* return straight away */
+                    case VINF_EM_TERMINATE:
+                    case VINF_EM_OFF:
+                        LogFlow(("dbgfR3VMMWait: returns %Rrc\n", rc));
+                        return rc;
+
+                    /* remember return code. */
+                    default:
+                        AssertReleaseMsgFailed(("rc=%Rrc is not in the switch!\n", rc));
+                    case VINF_EM_RESET:
+                    case VINF_EM_SUSPEND:
+                    case VINF_EM_HALT:
+                    case VINF_EM_RESUME:
+                    case VINF_EM_RESCHEDULE:
+                    case VINF_EM_RESCHEDULE_REM:
+                    case VINF_EM_RESCHEDULE_RAW:
+                        if (rc < rcRet || rcRet == VINF_SUCCESS)
+                            rcRet = rc;
+                        break;
+                }
+            }
+            else if (RT_FAILURE(rc))
+            {
+                LogFlow(("dbgfR3VMMWait: returns %Rrc\n", rc));
+                return rc;
+            }
         }
 
         /*
@@ -758,7 +791,9 @@ static int dbgfR3VMMCmd(PVM pVM, DBGFCMD enmCmd, PDBGFCMDDATA pCmdData, bool *pf
         {
             Log2(("Single step\n"));
             rc = VINF_EM_DBG_STEP;
-            pVM->dbgf.s.fSingleSteppingRaw = true;
+            /** @todo SMP */
+            PVMCPU pVCpu = VMMGetCpu0(pVM);
+            pVCpu->dbgf.s.fSingleSteppingRaw = true;
             fSendEvent = false;
             fResume = true;
             break;
@@ -829,13 +864,8 @@ VMMR3DECL(int) DBGFR3Attach(PVM pVM)
     /*
      * Call the VM, use EMT for serialization.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3Attach, 1, pVM);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
-
-    return rc;
+    /** @todo SMP */
+    return VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3Attach, 1, pVM);
 }
 
 
@@ -923,7 +953,7 @@ VMMR3DECL(int) DBGFR3Detach(PVM pVM)
  * @param   cMillies    Number of millies to wait.
  * @param   ppEvent     Where to store the event pointer.
  */
-VMMR3DECL(int) DBGFR3EventWait(PVM pVM, unsigned cMillies, PCDBGFEVENT *ppEvent)
+VMMR3DECL(int) DBGFR3EventWait(PVM pVM, RTMSINTERVAL cMillies, PCDBGFEVENT *ppEvent)
 {
     /*
      * Check state.
@@ -1042,18 +1072,21 @@ VMMR3DECL(int) DBGFR3Resume(PVM pVM)
  *
  * @returns VBox status.
  * @param   pVM     VM handle.
+ * @param   idCpu   The ID of the CPU to single step on.
  */
-VMMR3DECL(int) DBGFR3Step(PVM pVM)
+VMMR3DECL(int) DBGFR3Step(PVM pVM, VMCPUID idCpu)
 {
     /*
      * Check state.
      */
     AssertReturn(pVM->dbgf.s.fAttached, VERR_DBGF_NOT_ATTACHED);
     AssertReturn(RTSemPongIsSpeaker(&pVM->dbgf.s.PingPong), VERR_SEM_OUT_OF_TURN);
+    AssertReturn(idCpu < pVM->cCpus, VERR_INVALID_PARAMETER);
 
     /*
      * Send the ping back to the emulation thread telling it to run.
      */
+/** @todo SMP (idCpu) */
     dbgfR3SetCmd(pVM, DBGFCMD_SINGLE_STEP);
     int rc = RTSemPong(&pVM->dbgf.s.PingPong);
     AssertRC(rc);
@@ -1062,20 +1095,23 @@ VMMR3DECL(int) DBGFR3Step(PVM pVM)
 
 
 /**
- * Call this to single step rawmode or recompiled mode.
+ * Call this to single step programatically.
  *
  * You must pass down the return code to the EM loop! That's
  * where the actual single stepping take place (at least in the
  * current implementation).
  *
  * @returns VINF_EM_DBG_STEP
- * @thread  EMT
+ *
+ * @param   pVCpu       The virtual CPU handle.
+ *
+ * @thread  VCpu EMT
  */
-VMMR3DECL(int) DBGFR3PrgStep(PVM pVM)
+VMMR3DECL(int) DBGFR3PrgStep(PVMCPU pVCpu)
 {
-    VM_ASSERT_EMT(pVM);
+    VMCPU_ASSERT_EMT(pVCpu);
 
-    pVM->dbgf.s.fSingleSteppingRaw = true;
+    pVCpu->dbgf.s.fSingleSteppingRaw = true;
     return VINF_EM_DBG_STEP;
 }
 
