@@ -1,10 +1,10 @@
-/* $Id: DBGFBp.cpp 13818 2008-11-04 22:59:47Z vboxsync $ */
+/* $Id: DBGFBp.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DBGF - Debugger Facility, Breakpoint Management.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -39,7 +35,7 @@
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 static DECLCALLBACK(int) dbgfR3BpSetReg(PVM pVM, PCDBGFADDRESS pAddress, uint64_t *piHitTrigger, uint64_t *piHitDisable,
                                         uint8_t u8Type, uint8_t cb, PRTUINT piBp);
 static DECLCALLBACK(int) dbgfR3BpSetInt3(PVM pVM, PCDBGFADDRESS pAddress, uint64_t *piHitTrigger, uint64_t *piHitDisable, PRTUINT piBp);
@@ -52,7 +48,7 @@ static int dbgfR3BpRegArm(PVM pVM, PDBGFBP pBp);
 static int dbgfR3BpRegDisarm(PVM pVM, PDBGFBP pBp);
 static int dbgfR3BpInt3Arm(PVM pVM, PDBGFBP pBp);
 static int dbgfR3BpInt3Disarm(PVM pVM, PDBGFBP pBp);
-__END_DECLS
+RT_C_DECLS_END
 
 
 
@@ -286,11 +282,8 @@ VMMR3DECL(int) DBGFR3BpSet(PVM pVM, PCDBGFADDRESS pAddress, uint64_t iHitTrigger
     /*
      * This must be done in EMT.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3BpSetInt3, 5, pVM, pAddress, &iHitTrigger, &iHitDisable, piBp);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    /** @todo SMP? */
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3BpSetInt3, 5, pVM, pAddress, &iHitTrigger, &iHitDisable, piBp);
     LogFlow(("DBGFR3BpSet: returns %Rrc\n", rc));
     return rc;
 }
@@ -378,14 +371,20 @@ static DECLCALLBACK(int) dbgfR3BpSetInt3(PVM pVM, PCDBGFADDRESS pAddress, uint64
 static int dbgfR3BpInt3Arm(PVM pVM, PDBGFBP pBp)
 {
     /** @todo should actually use physical address here! */
+
+    /* @todo SMP support! */
+    VMCPUID idCpu = 0;
+
     /*
      * Save current byte and write int3 instruction.
      */
-    int rc = MMR3ReadGCVirt(pVM, &pBp->u.Int3.bOrg, pBp->GCPtr, 1);
+    DBGFADDRESS Addr;
+    DBGFR3AddrFromFlat(pVM, &Addr, pBp->GCPtr);
+    int rc = DBGFR3MemRead(pVM, idCpu, &Addr, &pBp->u.Int3.bOrg, 1);
     if (RT_SUCCESS(rc))
     {
         static const uint8_t s_bInt3 = 0xcc;
-        rc = MMR3WriteGCVirt(pVM, pBp->GCPtr, &s_bInt3, 1);
+        rc = DBGFR3MemWrite(pVM, idCpu, &Addr, &s_bInt3, 1);
     }
     return rc;
 }
@@ -401,14 +400,19 @@ static int dbgfR3BpInt3Arm(PVM pVM, PDBGFBP pBp)
  */
 static int dbgfR3BpInt3Disarm(PVM pVM, PDBGFBP pBp)
 {
+    /* @todo SMP support! */
+    VMCPUID idCpu = 0;
+
     /*
      * Check that the current byte is the int3 instruction, and restore the original one.
      * We currently ignore invalid bytes.
      */
-    uint8_t bCurrent;
-    int rc = MMR3ReadGCVirt(pVM, &bCurrent, pBp->GCPtr, 1);
+    DBGFADDRESS     Addr;
+    DBGFR3AddrFromFlat(pVM, &Addr, pBp->GCPtr);
+    uint8_t         bCurrent;
+    int rc = DBGFR3MemRead(pVM, idCpu, &Addr, &bCurrent, 1);
     if (bCurrent == 0xcc)
-        rc = MMR3WriteGCVirt(pVM, pBp->GCPtr, &pBp->u.Int3.bOrg, 1);
+        rc = DBGFR3MemWrite(pVM, idCpu, &Addr, &pBp->u.Int3.bOrg, 1);
     return rc;
 }
 
@@ -430,16 +434,13 @@ static int dbgfR3BpInt3Disarm(PVM pVM, PDBGFBP pBp)
  * @thread  Any thread.
  */
 VMMR3DECL(int) DBGFR3BpSetReg(PVM pVM, PCDBGFADDRESS pAddress, uint64_t iHitTrigger, uint64_t iHitDisable,
-                               uint8_t fType, uint8_t cb, PRTUINT piBp)
+                              uint8_t fType, uint8_t cb, PRTUINT piBp)
 {
+    /** @todo SMP - broadcast, VT-x/AMD-V. */
     /*
      * This must be done in EMT.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3BpSetReg, 7, pVM, pAddress, &iHitTrigger, &iHitDisable, fType, cb, piBp);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3BpSetReg, 7, pVM, pAddress, &iHitTrigger, &iHitDisable, fType, cb, piBp);
     LogFlow(("DBGFR3BpSetReg: returns %Rrc\n", rc));
     return rc;
 
@@ -562,8 +563,11 @@ static DECLCALLBACK(int) dbgfR3BpSetReg(PVM pVM, PCDBGFADDRESS pAddress, uint64_
  */
 static int dbgfR3BpRegArm(PVM pVM, PDBGFBP pBp)
 {
+    /* @todo SMP support! */
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     Assert(pBp->fEnabled);
-    return CPUMRecalcHyperDRx(pVM);
+    return CPUMRecalcHyperDRx(pVCpu);
 }
 
 
@@ -577,8 +581,11 @@ static int dbgfR3BpRegArm(PVM pVM, PDBGFBP pBp)
  */
 static int dbgfR3BpRegDisarm(PVM pVM, PDBGFBP pBp)
 {
+    /** @todo SMP support! */
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     Assert(!pBp->fEnabled);
-    return CPUMRecalcHyperDRx(pVM);
+    return CPUMRecalcHyperDRx(pVCpu);
 }
 
 
@@ -600,11 +607,7 @@ VMMR3DECL(int) DBGFR3BpSetREM(PVM pVM, PCDBGFADDRESS pAddress, uint64_t iHitTrig
     /*
      * This must be done in EMT.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3BpSetREM, 5, pVM, pAddress, &iHitTrigger, &iHitDisable, piBp);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3BpSetREM, 5, pVM, pAddress, &iHitTrigger, &iHitDisable, piBp);
     LogFlow(("DBGFR3BpSetREM: returns %Rrc\n", rc));
     return rc;
 }
@@ -696,11 +699,7 @@ VMMR3DECL(int) DBGFR3BpClear(PVM pVM, RTUINT iBp)
     /*
      * This must be done in EMT.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3BpClear, 2, pVM, iBp);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3BpClear, 2, pVM, iBp);
     LogFlow(("DBGFR3BpClear: returns %Rrc\n", rc));
     return rc;
 }
@@ -773,11 +772,7 @@ VMMR3DECL(int) DBGFR3BpEnable(PVM pVM, RTUINT iBp)
     /*
      * This must be done in EMT.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3BpEnable, 2, pVM, iBp);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3BpEnable, 2, pVM, iBp);
     LogFlow(("DBGFR3BpEnable: returns %Rrc\n", rc));
     return rc;
 }
@@ -850,11 +845,7 @@ VMMR3DECL(int) DBGFR3BpDisable(PVM pVM, RTUINT iBp)
     /*
      * This must be done in EMT.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3BpDisable, 2, pVM, iBp);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3BpDisable, 2, pVM, iBp);
     LogFlow(("DBGFR3BpDisable: returns %Rrc\n", rc));
     return rc;
 }
@@ -926,11 +917,7 @@ VMMR3DECL(int) DBGFR3BpEnum(PVM pVM, PFNDBGFBPENUM pfnCallback, void *pvUser)
     /*
      * This must be done in EMT.
      */
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)dbgfR3BpEnum, 3, pVM, pfnCallback, pvUser);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)dbgfR3BpEnum, 3, pVM, pfnCallback, pvUser);
     LogFlow(("DBGFR3BpClear: returns %Rrc\n", rc));
     return rc;
 }
