@@ -1,10 +1,10 @@
-/* $Id: DBGFSym.cpp 16080 2009-01-20 11:15:58Z vboxsync $ */
+/* $Id: DBGFSym.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * DBGF - Debugger Facility, Symbol Management.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -24,7 +20,7 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_DBGF
-#if defined(RT_OS_WINDOWS) && 0 //defined(DEBUG_bird) // enabled this is you want to debug win32 guests or the hypervisor.
+#if defined(RT_OS_WINDOWS) && 0 //defined(DEBUG_bird) // enabled this is you want to debug win32 guests, the hypervisor of EFI.
 # include <Windows.h>
 # define _IMAGEHLP64
 # include <DbgHelp.h>
@@ -32,18 +28,21 @@
 #endif
 /** @todo Only use DBGHELP for reading modules since it doesn't do all we want (relocations), or is way to slow in some cases (add symbol)! */
 #include <VBox/dbgf.h>
+#include <VBox/mm.h>
+#include <VBox/pdmapi.h>
 #include "DBGFInternal.h"
 #include <VBox/vm.h>
-#include <VBox/mm.h>
-#include <VBox/pdm.h>
 #include <VBox/err.h>
 #include <VBox/log.h>
-#include <iprt/assert.h>
 
+#include <iprt/assert.h>
+#include <iprt/path.h>
+#include <iprt/ctype.h>
+#include <iprt/env.h>
+#include <iprt/param.h>
 #ifndef HAVE_DBGHELP
 # include <iprt/avl.h>
 # include <iprt/string.h>
-# include <iprt/ctype.h>
 #endif
 
 #include <stdio.h> /* for fopen(). */ /** @todo use iprt/stream.h! */
@@ -54,7 +53,8 @@
 *   Internal Functions                                                         *
 *******************************************************************************/
 #ifdef HAVE_DBGHELP
-static DECLCALLBACK(int) dbgfR3EnumModules(PVM pVM, const char *pszFilename, const char *pszName, RTUINTPTR ImageBase, size_t cbImage, bool fGC);
+static DECLCALLBACK(int) dbgfR3EnumModules(PVM pVM, const char *pszFilename, const char *pszName,
+                                           RTUINTPTR ImageBase, size_t cbImage, bool fRC, void *pvArg);
 static int win32Error(PVM pVM);
 #endif
 
@@ -161,7 +161,7 @@ static int dbgfR3SymbolInsert(PVM pVM, const char *pszName, RTGCPTR Address, siz
                 if (!pName)
                 {
                     /* make new symbol space node. */
-                    PDBGFSYMSPACE pName = (PDBGFSYMSPACE)MMR3HeapAlloc(pVM, MM_TAG_DBGF_SYMBOL, sizeof(*pName) + cchName);
+                    pName = (PDBGFSYMSPACE)MMR3HeapAlloc(pVM, MM_TAG_DBGF_SYMBOL, sizeof(*pName) + cchName);
                     if (pName)
                     {
                         pName->Core.pszString = (char *)memcpy(pName + 1, pszName, cchName);
@@ -225,10 +225,10 @@ static PDBGFSYM dbgfR3SymbolGetName(PVM pVM, const char *pszSymbol)
  */
 static char *dbgfR3Strip(char *psz)
 {
-    while (*psz && isspace(*psz))
+    while (*psz && RT_C_IS_SPACE(*psz))
         psz++;
     char *psz2 = strchr(psz, '\0') - 1;
-    while (psz2 >= psz && isspace(*psz2))
+    while (psz2 >= psz && RT_C_IS_SPACE(*psz2))
         *psz2-- = '\0';
     return psz;
 }
@@ -352,7 +352,7 @@ int dbgfR3SymLazyInit(PVM pVM)
         /*
          * Enumerate all modules loaded by PDM and add them to the symbol database.
          */
-        PDMR3EnumModules(pVM, dbgfR3EnumModules, NULL);
+        PDMR3LdrEnumModules(pVM, dbgfR3EnumModules, NULL);
         return VINF_SUCCESS;
     }
     return win32Error(pVM);
@@ -374,19 +374,19 @@ int dbgfR3SymLazyInit(PVM pVM)
  * @param   pszName         Module name. (short and unique)
  * @param   ImageBase       Address where to executable image is loaded.
  * @param   cbImage         Size of the executable image.
- * @param   fGC             Set if guest context, clear if host context.
+ * @param   fRC             Set if guest context, clear if host context.
  * @param   pvArg           User argument.
  */
-static DECLCALLBACK(int) dbgfR3EnumModules(PVM pVM, const char *pszFilename, const char *pszName, RTUINTPTR ImageBase, size_t cbImage, bool fGC)
+static DECLCALLBACK(int) dbgfR3EnumModules(PVM pVM, const char *pszFilename, const char *pszName,
+                                           RTUINTPTR ImageBase, size_t cbImage, bool fRC, void *pvArg)
 {
-    if (fGC)
-    {
-        DWORD64 LoadedImageBase = SymLoadModule64(pVM, NULL, (char *)(void *)pszFilename, (char *)(void *)pszName, ImageBase, cbImage);
-        if (!LoadedImageBase)
-            Log(("SymLoadModule64(,,%s,,) -> lasterr=%d\n", pszFilename, GetLastError()));
-        else
-            Log(("Loaded debuginfo for %s - %s %llx\n", pszName, pszFilename, LoadedImageBase));
-    }
+    DWORD64 LoadedImageBase = SymLoadModule64(pVM, NULL, (char *)(void *)pszFilename,
+                                                (char *)(void *)pszName, ImageBase, (DWORD)cbImage);
+    if (!LoadedImageBase)
+        Log(("SymLoadModule64(,,%s,,) -> lasterr=%d\n", pszFilename, GetLastError()));
+    else
+        Log(("Loaded debuginfo for %s - %s %llx\n", pszName, pszFilename, LoadedImageBase));
+
     return VINF_SUCCESS;
 }
 #endif
@@ -450,41 +450,41 @@ SYMFILETYPE dbgfR3ModuleProbe(FILE *pFile)
             ||  strstr(szHead, "Linker script and memory map"))
             return SYMFILETYPE_LD_MAP;
 
-        if (   isxdigit(szHead[0])
-            && isxdigit(szHead[1])
-            && isxdigit(szHead[2])
-            && isxdigit(szHead[3])
-            && isxdigit(szHead[4])
-            && isxdigit(szHead[5])
-            && isxdigit(szHead[6])
-            && isxdigit(szHead[7])
+        if (   RT_C_IS_XDIGIT(szHead[0])
+            && RT_C_IS_XDIGIT(szHead[1])
+            && RT_C_IS_XDIGIT(szHead[2])
+            && RT_C_IS_XDIGIT(szHead[3])
+            && RT_C_IS_XDIGIT(szHead[4])
+            && RT_C_IS_XDIGIT(szHead[5])
+            && RT_C_IS_XDIGIT(szHead[6])
+            && RT_C_IS_XDIGIT(szHead[7])
             && szHead[8] == ' '
-            && isalpha(szHead[9])
+            && RT_C_IS_ALPHA(szHead[9])
             && szHead[10] == ' '
-            && (isalpha(szHead[11]) || szHead[11] == '_' || szHead[11] == '$')
+            && (RT_C_IS_ALPHA(szHead[11]) || szHead[11] == '_' || szHead[11] == '$')
             )
             return SYMFILETYPE_LINUX_SYSTEM_MAP;
 
-        if (   isxdigit(szHead[0])
-            && isxdigit(szHead[1])
-            && isxdigit(szHead[2])
-            && isxdigit(szHead[3])
-            && isxdigit(szHead[4])
-            && isxdigit(szHead[5])
-            && isxdigit(szHead[6])
-            && isxdigit(szHead[7])
-            && isxdigit(szHead[8])
-            && isxdigit(szHead[9])
-            && isxdigit(szHead[10])
-            && isxdigit(szHead[11])
-            && isxdigit(szHead[12])
-            && isxdigit(szHead[13])
-            && isxdigit(szHead[14])
-            && isxdigit(szHead[15])
+        if (   RT_C_IS_XDIGIT(szHead[0])
+            && RT_C_IS_XDIGIT(szHead[1])
+            && RT_C_IS_XDIGIT(szHead[2])
+            && RT_C_IS_XDIGIT(szHead[3])
+            && RT_C_IS_XDIGIT(szHead[4])
+            && RT_C_IS_XDIGIT(szHead[5])
+            && RT_C_IS_XDIGIT(szHead[6])
+            && RT_C_IS_XDIGIT(szHead[7])
+            && RT_C_IS_XDIGIT(szHead[8])
+            && RT_C_IS_XDIGIT(szHead[9])
+            && RT_C_IS_XDIGIT(szHead[10])
+            && RT_C_IS_XDIGIT(szHead[11])
+            && RT_C_IS_XDIGIT(szHead[12])
+            && RT_C_IS_XDIGIT(szHead[13])
+            && RT_C_IS_XDIGIT(szHead[14])
+            && RT_C_IS_XDIGIT(szHead[15])
             && szHead[16] == ' '
-            && isalpha(szHead[17])
+            && RT_C_IS_ALPHA(szHead[17])
             && szHead[18] == ' '
-            && (isalpha(szHead[19]) || szHead[19] == '_' || szHead[19] == '$')
+            && (RT_C_IS_ALPHA(szHead[19]) || szHead[19] == '_' || szHead[19] == '$')
             )
             return SYMFILETYPE_LINUX_SYSTEM_MAP;
 
@@ -528,7 +528,7 @@ static int dbgfR3LoadLinuxSystemMap(PVM pVM, FILE *pFile, RTGCUINTPTR ModuleAddr
             &&  u64Address != (RTGCUINTPTR)~0)
         {
             pszEnd++;
-            if (    isalpha(*pszEnd)
+            if (    RT_C_IS_ALPHA(*pszEnd)
                 &&  (pszEnd[1] == ' ' || pszEnd[1] == '\t'))
             {
                 psz = dbgfR3Strip(pszEnd + 2);
@@ -544,6 +544,86 @@ static int dbgfR3LoadLinuxSystemMap(PVM pVM, FILE *pFile, RTGCUINTPTR ModuleAddr
     return VINF_SUCCESS;
 }
 
+/**
+ * Tries to open the file using the image search paths.
+ *
+ * This is currently a quick hack and the only way to specifying the path is by setting
+ * VBOXDBG_IMAGE_PATH in the environment. It uses semicolon as separator everywhere.
+ *
+ * @returns VBox status code.
+ * @param   pVM             The VM handle.
+ * @param   pszFilename     The name of the file to locate and open.
+ * @param   pszFound        Where to return the actual filename.
+ * @param   cchFound        The buffer size.
+ * @param   ppFile          Where to return the opened file.
+ */
+int dbgfR3ModuleLocateAndOpen(PVM pVM, const char *pszFilename, char *pszFound, size_t cchFound, FILE **ppFile)
+{
+    /* Check the filename length. */
+    size_t const    cchFilename = strlen(pszFilename);
+    if (cchFilename >= cchFound)
+        return VERR_FILENAME_TOO_LONG;
+    const char     *pszName = RTPathFilename(pszFilename);
+    if (!pszName)
+        return VERR_IS_A_DIRECTORY;
+    size_t const    cchName = strlen(pszName);
+
+    /*
+     * Try default location first.
+     */
+    memcpy(pszFound, pszFilename, cchFilename + 1);
+    FILE *pFile = *ppFile = fopen(pszFound, "rb");
+    if (pFile)
+        return VINF_SUCCESS;
+
+    /*
+     * Walk the search path.
+     */
+    char *pszFreeMe = RTEnvDupEx(RTENV_DEFAULT, "VBOXDBG_IMAGE_PATH");
+    const char *psz = pszFreeMe ? pszFreeMe : ".";
+    while (*psz)
+    {
+        /* Skip leading blanks - no directories with leading spaces, thank you. */
+        while (RT_C_IS_BLANK(*psz))
+            psz++;
+
+        /* Fine the end of this element. */
+        const char *pszNext;
+        const char *pszEnd = strchr(psz, ';');
+        if (!pszEnd)
+            pszEnd = pszNext = strchr(psz, '\0');
+        else
+            pszNext = pszEnd + 1;
+        if (pszEnd != psz)
+        {
+            size_t const cch = pszEnd - psz;
+            if (cch + 1 + cchName < cchFound)
+            {
+                /** @todo RTPathCompose, RTPathComposeN(). This code isn't right
+                 * for 'E:' on DOS systems. It may also create unwanted double slashes. */
+                memcpy(pszFound, psz, cch);
+                pszFound[cch] = '/';
+                memcpy(pszFound + cch + 1, pszName, cchName + 1);
+                *ppFile = pFile = fopen(pszFound, "rb");
+                if (pFile)
+                {
+                    RTStrFree(pszFreeMe);
+                    return VINF_SUCCESS;
+                }
+            }
+
+            /** @todo do a depth search using the specified path. */
+        }
+
+        /* advance */
+        psz = pszNext;
+    }
+
+    /* not found */
+    RTStrFree(pszFreeMe);
+    return VERR_OPEN_FAILED;
+}
+
 
 /**
  * Load debug info, optionally related to a specific module.
@@ -574,8 +654,9 @@ VMMR3DECL(int) DBGFR3ModuleLoad(PVM pVM, const char *pszFilename, RTGCUINTPTR Ad
     /*
      * Open the load file.
      */
-    int rc = VINF_SUCCESS;
-    FILE *pFile = fopen(pszFilename, "rb");
+    FILE *pFile = NULL;
+    char szFoundFile[RTPATH_MAX];
+    int rc = dbgfR3ModuleLocateAndOpen(pVM, pszFilename, szFoundFile, sizeof(szFoundFile), &pFile);
     if (pFile)
     {
         /*
@@ -591,7 +672,7 @@ VMMR3DECL(int) DBGFR3ModuleLoad(PVM pVM, const char *pszFilename, RTGCUINTPTR Ad
             {
                 #ifdef HAVE_DBGHELP
                 /** @todo arg! checkout the inserting of modules and then loading them again.... Or just the module representation.... */
-                DWORD64 ImageBase = SymLoadModule64(pVM, NULL, (char *)(void *)pszFilename, (char *)(void *)pszName, ModuleAddress, cbImage);
+                DWORD64 ImageBase = SymLoadModule64(pVM, NULL, (char *)(void *)szFoundFile, (char *)(void *)pszName, ModuleAddress, cbImage);
                 if (!ImageBase)
                     ImageBase = SymLoadModule64(pVM, NULL, (char *)(void *)pszName, (char *)(void *)pszName, ModuleAddress, cbImage);
                 if (ImageBase)
@@ -648,8 +729,6 @@ VMMR3DECL(int) DBGFR3ModuleLoad(PVM pVM, const char *pszFilename, RTGCUINTPTR Ad
         /** @todo check for read errors */
         fclose(pFile);
     }
-    else
-        rc = VERR_OPEN_FAILED;
     return rc;
 }
 
@@ -873,72 +952,6 @@ VMMR3DECL(int) DBGFR3SymbolByName(PVM pVM, const char *pszSymbol, PDBGFSYMBOL pS
 }
 
 
-/**
- * Duplicates a symbol.
- *
- * @returns Pointer to the duplicated symbol.
- * @param   pVM             The VM handle.
- * @param   pSymbol         The symbol to duplicate.
- */
-static PDBGFSYMBOL dbgfR3SymbolDup(PVM pVM, PCDBGFSYMBOL pSymbol)
-{
-    size_t cb = strlen(pSymbol->szName) + RT_OFFSETOF(DBGFSYMBOL, szName[1]);
-    PDBGFSYMBOL pDup = (PDBGFSYMBOL)MMR3HeapAlloc(pVM, MM_TAG_DBGF_SYMBOL_DUP, cb);
-    if (pDup)
-        memcpy(pDup, pSymbol, cb);
-    return pDup;
-}
-
-
-/**
- * Find symbol by address (nearest), allocate return buffer.
- *
- * @returns Pointer to the symbol. Must be freed using DBGFR3SymbolFree().
- * @returns NULL if the symbol was not found or if we're out of memory.
- * @param   pVM                 VM handle.
- * @param   Address             Address.
- * @param   poffDisplacement    Where to store the symbol displacement from Address.
- */
-VMMR3DECL(PDBGFSYMBOL) DBGFR3SymbolByAddrAlloc(PVM pVM, RTGCUINTPTR Address, PRTGCINTPTR poffDisplacement)
-{
-    DBGFSYMBOL Symbol;
-    int rc = DBGFR3SymbolByAddr(pVM, Address, poffDisplacement, &Symbol);
-    if (RT_FAILURE(rc))
-        return NULL;
-    return dbgfR3SymbolDup(pVM, &Symbol);
-}
-
-
-/**
- * Find symbol by name (first), allocate return buffer.
- *
- * @returns Pointer to the symbol. Must be freed using DBGFR3SymbolFree().
- * @returns NULL if the symbol was not found or if we're out of memory.
- * @param   pVM                 VM handle.
- * @param   pszSymbol           Symbol name.
- */
-VMMR3DECL(PDBGFSYMBOL) DBGFR3SymbolByNameAlloc(PVM pVM, const char *pszSymbol)
-{
-    DBGFSYMBOL Symbol;
-    int rc = DBGFR3SymbolByName(pVM, pszSymbol, &Symbol);
-    if (RT_FAILURE(rc))
-        return NULL;
-    return dbgfR3SymbolDup(pVM, &Symbol);
-}
-
-
-/**
- * Frees a symbol returned by DBGFR3SymbolbyNameAlloc() or DBGFR3SymbolByAddressAlloc().
- *
- * @param   pSymbol         Pointer to the symbol.
- */
-VMMR3DECL(void) DBGFR3SymbolFree(PDBGFSYMBOL pSymbol)
-{
-    if (pSymbol)
-        MMR3HeapFree(pSymbol);
-}
-
-
 /**
  * Find line by address (nearest).
  *
