@@ -1,10 +1,10 @@
-/* $Id: HWACCMInternal.h $ */
+/* $Id: HWACCMInternal.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * HWACCM - Internal header file.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___HWACCMInternal_h
@@ -121,10 +117,10 @@ RT_C_DECLS_BEGIN
  *  Currently #NM and #PF only
  */
 #ifdef VBOX_STRICT
-#define HWACCM_VMX_TRAP_MASK                RT_BIT(X86_XCPT_DE) | RT_BIT(X86_XCPT_NM) | RT_BIT(X86_XCPT_PF) | RT_BIT(X86_XCPT_UD) | RT_BIT(X86_XCPT_NP) | RT_BIT(X86_XCPT_SS) | RT_BIT(X86_XCPT_GP) | RT_BIT(X86_XCPT_MF)
+#define HWACCM_VMX_TRAP_MASK                RT_BIT(X86_XCPT_BP) | RT_BIT(X86_XCPT_DB) | RT_BIT(X86_XCPT_DE) | RT_BIT(X86_XCPT_NM) | RT_BIT(X86_XCPT_PF) | RT_BIT(X86_XCPT_UD) | RT_BIT(X86_XCPT_NP) | RT_BIT(X86_XCPT_SS) | RT_BIT(X86_XCPT_GP) | RT_BIT(X86_XCPT_MF)
 #define HWACCM_SVM_TRAP_MASK                HWACCM_VMX_TRAP_MASK
 #else
-#define HWACCM_VMX_TRAP_MASK                RT_BIT(X86_XCPT_NM) | RT_BIT(X86_XCPT_PF)
+#define HWACCM_VMX_TRAP_MASK                RT_BIT(X86_XCPT_DB) | RT_BIT(X86_XCPT_NM) | RT_BIT(X86_XCPT_PF)
 #define HWACCM_SVM_TRAP_MASK                RT_BIT(X86_XCPT_NM) | RT_BIT(X86_XCPT_PF)
 #endif
 /* All exceptions have to be intercept in emulated real-mode (minues NM & PF as they are always intercepted. */
@@ -173,22 +169,14 @@ typedef struct
     /** Configured for VT-x or AMD-V. */
     bool                fConfigured;
 
+    /** Set if the VBOX_HWVIRTEX_IGNORE_SVM_IN_USE hack is active. */
+    bool                fIgnoreAMDVInUseError;
+
     /** In use by our code. (for power suspend) */
     volatile bool       fInUse;
 } HWACCM_CPUINFO;
 typedef HWACCM_CPUINFO *PHWACCM_CPUINFO;
 
-/* VT-x capability qword. */
-typedef union
-{
-    struct
-    {
-        uint32_t        disallowed0;
-        uint32_t        allowed1;
-    } n;
-    uint64_t            u;
-} VMX_CAPABILITY;
-
 typedef enum
 {
     HWACCMPENDINGIO_INVALID = 0,
@@ -267,6 +255,9 @@ typedef struct HWACCM
     /** Set if nested paging is allowed. */
     bool                        fAllowNestedPaging;
 
+    /** Set if large pages are enabled (requires nested paging). */
+    bool                        fLargePages;
+
     /** Set if we can support 64-bit guests or not. */
     bool                        fAllow64BitGuests;
 
@@ -281,7 +272,7 @@ typedef struct HWACCM
 
     /** Set when TPR patching is active. */
     bool                        fTPRPatchingActive;
-    bool                        u8Alignment[7];
+    bool                        u8Alignment[6];
 
     /** And mask for copying register contents. */
     uint64_t                    u64RegisterMask;
@@ -339,6 +330,10 @@ typedef struct HWACCM
         /** Set if VT-x VPID is allowed. */
         bool                        fAllowVPID;
 
+        /** Set if unrestricted guest execution is allowed (real and protected mode without paging). */
+        bool                        fUnrestrictedGuest;
+        bool                        uAlignment[3];
+
         /** Virtual address of the TSS page used for real mode emulation. */
         R3PTRTYPE(PVBOXTSS)         pRealModeTSS;
 
@@ -381,9 +376,15 @@ typedef struct HWACCM
         /** Ring 0 handlers for VT-x. */
         DECLR0CALLBACKMEMBER(void, pfnSetupTaggedTLB, (PVM pVM, PVMCPU pVCpu));
 
+#if HC_ARCH_BITS == 32 && defined(VBOX_ENABLE_64_BITS_GUESTS)
+        uint32_t                    u32Alignment;
+#endif
         /** Host CR4 value (set by ring-0 VMX init) */
         uint64_t                    hostCR4;
 
+        /** Host EFER value (set by ring-0 VMX init) */
+        uint64_t                    hostEFER;
+
         /** VMX MSR values */
         struct
         {
@@ -416,7 +417,8 @@ typedef struct HWACCM
         bool                        fEnabled;
         /** Set if erratum 170 affects the AMD cpu. */
         bool                        fAlwaysFlushTLB;
-        bool                        u8Alignment;
+        /** Set when the hack to ignore VERR_SVM_IN_USE is active. */
+        bool                        fIgnoreInUseError;
 
         /** R0 memory object for the IO bitmap (12kb). */
         RTR0MEMOBJ                  pMemObjIOBitmap;
@@ -425,6 +427,9 @@ typedef struct HWACCM
         /** Virtual address of the IO bitmap. */
         R0PTRTYPE(void *)           pIOBitmap;
 
+        /* HWCR msr (for diagnostics) */
+        uint64_t                    msrHWCR;
+
         /** SVM revision. */
         uint32_t                    u32Rev;
 
@@ -565,7 +570,7 @@ typedef struct HWACCMCPU
     /** Current ASID in use by the VM */
     RTUINT                      uCurrentASID;
 
-    /** World switch exit counter. */ 
+    /** World switch exit counter. */
     volatile uint32_t           cWorldSwitchExit;
     uint32_t                    u32Alignment;
 
@@ -808,6 +813,7 @@ typedef struct HWACCMCPU
     STAMCOUNTER             StatExitCLTS;
     STAMCOUNTER             StatExitHlt;
     STAMCOUNTER             StatExitMwait;
+    STAMCOUNTER             StatExitMonitor;
     STAMCOUNTER             StatExitLMSW;
     STAMCOUNTER             StatExitIOWrite;
     STAMCOUNTER             StatExitIORead;
@@ -819,8 +825,10 @@ typedef struct HWACCMCPU
     STAMCOUNTER             StatIntReinject;
     STAMCOUNTER             StatPendingHostIrq;
 
+    STAMCOUNTER             StatFlushPage;
     STAMCOUNTER             StatFlushPageManual;
     STAMCOUNTER             StatFlushPhysPageManual;
+    STAMCOUNTER             StatFlushTLB;
     STAMCOUNTER             StatFlushTLBManual;
     STAMCOUNTER             StatFlushPageInvlpg;
     STAMCOUNTER             StatFlushTLBWorldSwitch;
