@@ -1,10 +1,10 @@
-/* $Id: MMInternal.h $ */
+/* $Id: MMInternal.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * MM - Internal header file.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___MMInternal_h
@@ -26,6 +22,8 @@
 #include <VBox/types.h>
 #include <VBox/sup.h>
 #include <VBox/stam.h>
+#include <VBox/pdmcritsect.h>
+#include <iprt/assert.h>
 #include <iprt/avl.h>
 #include <iprt/critsect.h>
 
@@ -65,6 +63,10 @@ typedef struct MMHEAPSTAT
     /** Pointer to the heap the memory belongs to. */
     struct MMHEAP          *pHeap;
 #ifdef MMR3HEAP_WITH_STATISTICS
+# if HC_ARCH_BITS == 32
+    /** Aligning the statistics on an 8 byte boundrary (for uint64_t and STAM). */
+    void                   *pvAlignment;
+# endif
     /** Number of allocation. */
     uint64_t                cAllocations;
     /** Number of reallocations. */
@@ -81,11 +83,15 @@ typedef struct MMHEAPSTAT
     size_t                  cbCurAllocated;
 #endif
 } MMHEAPSTAT;
+#if defined(MMR3HEAP_WITH_STATISTICS) && defined(IN_RING3)
+AssertCompileMemberAlignment(MMHEAPSTAT, cAllocations, 8);
+#endif
 /** Pointer to heap statistics record. */
 typedef MMHEAPSTAT *PMMHEAPSTAT;
 
 
 
+
 /**
  * Additional heap block header for relating allocations to the VM.
  */
@@ -167,6 +173,9 @@ typedef struct MMUKHEAPSTAT
     /** Number of bytes currently allocated. */
     size_t                  cbCurAllocated;
 } MMUKHEAPSTAT;
+#ifdef IN_RING3
+AssertCompileMemberAlignment(MMUKHEAPSTAT, cAllocations, 8);
+#endif
 /** Pointer to heap statistics record. */
 typedef MMUKHEAPSTAT *PMMUKHEAPSTAT;
 
@@ -201,9 +210,16 @@ typedef struct MMUKHEAP
     PAVLULNODECORE          pStatTree;
     /** The VM handle. */
     PUVM                    pUVM;
+#if HC_ARCH_BITS == 32
+    /** Aligning the statistics on an 8 byte boundrary (for uint64_t and STAM). */
+    void                   *pvAlignment;
+#endif
     /** Heap global statistics. */
     MMUKHEAPSTAT            Stat;
 } MMUKHEAP;
+#ifdef IN_RING3
+AssertCompileMemberAlignment(MMUKHEAP, Stat, 8);
+#endif
 /** Pointer to MM Heap structure. */
 typedef MMUKHEAP *PMMUKHEAP;
 
@@ -275,6 +291,7 @@ typedef struct MMHYPERSTAT
     /** Max number of bytes allocated. */
     uint32_t                cbMaxAllocated;
 } MMHYPERSTAT;
+AssertCompileMemberAlignment(MMHYPERSTAT, cAllocations, 8);
 /** Pointer to hypervisor heap statistics record. */
 typedef MMHYPERSTAT *PMMHYPERSTAT;
 
@@ -324,6 +341,8 @@ typedef struct MMHYPERHEAP
     uint32_t                u32Magic;
     /** The heap size. (This structure is not included!) */
     uint32_t                cbHeap;
+    /** Lock protecting the heap. */
+    PDMCRITSECT             Lock;
     /** The HC ring-3 address of the heap. */
     R3PTRTYPE(uint8_t *)    pbHeapR3;
     /** The HC ring-3 address of the shared VM strcture. */
@@ -522,8 +541,8 @@ typedef struct MMPAGEPOOL
     R3R0PTRTYPE(PVM)                    pVM;
 #endif
     /** Flag indicating the allocation method.
-     * Set: SUPLowAlloc().
-     * Clear: SUPPageAlloc() + SUPPageLock(). */
+     * Set: SUPR3LowAlloc().
+     * Clear: SUPR3PageAllocEx(). */
     bool                                fLow;
     /** Number of subpools. */
     uint32_t                            cSubPools;
@@ -532,6 +551,10 @@ typedef struct MMPAGEPOOL
 #ifdef VBOX_WITH_STATISTICS
     /** Number of free pages in pool. */
     uint32_t                            cFreePages;
+# if HC_ARCH_BITS == 32
+    /** Aligning the statistics on an 8 byte boundrary. */
+    uint32_t                            u32Alignment;
+# endif
     /** Number of alloc calls. */
     STAMCOUNTER                         cAllocCalls;
     /** Number of free calls. */
@@ -544,6 +567,12 @@ typedef struct MMPAGEPOOL
     STAMCOUNTER                         cErrors;
 #endif
 } MMPAGEPOOL;
+#ifndef IN_RC
+AssertCompileMemberAlignment(MMPAGEPOOL, cSubPools, 4);
+# ifdef VBOX_WITH_STATISTICS
+AssertCompileMemberAlignment(MMPAGEPOOL, cAllocCalls, 8);
+# endif
+#endif
 /** Pointer to page pool. */
 typedef MMPAGEPOOL *PMMPAGEPOOL;
 
@@ -766,7 +795,7 @@ typedef struct MMUSERPERVM
 typedef MMUSERPERVM *PMMUSERPERVM;
 
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 
 int  mmR3UpdateReservation(PVM pVM);
@@ -782,9 +811,10 @@ int  mmR3UkHeapCreateU(PUVM pUVM, PMMUKHEAP *ppHeap);
 
 
 int  mmR3HyperInit(PVM pVM);
+int  mmR3HyperTerm(PVM pVM);
 int  mmR3HyperInitPaging(PVM pVM);
 
-const char *mmR3GetTagName(MMTAG enmTag);
+const char *mmGetTagName(MMTAG enmTag);
 
 /**
  * Converts a pool address to a physical address.
@@ -810,7 +840,7 @@ RTHCPHYS mmPagePoolPtr2Phys(PMMPAGEPOOL pPool, void *pv);
  */
 void *mmPagePoolPhys2Ptr(PMMPAGEPOOL pPool, RTHCPHYS HCPhys);
 
-__END_DECLS
+RT_C_DECLS_END
 
 /** @} */
 
