@@ -1,10 +1,10 @@
-/* $Id: PATMA.h $ */
+/* $Id: PATMA.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * PATM macros & definitions (identical to PATMA.mac!!)
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___PATMA_H
@@ -109,7 +105,7 @@ typedef struct
 {
     uint8_t *pFunction;
     uint32_t offJump;
-    uint32_t offRelJump;	//used only by loop/loopz/loopnz
+    uint32_t offRelJump;        //used only by loop/loopz/loopnz
     uint32_t offSizeOverride;   //size override byte position
     uint32_t size;
     uint32_t nrRelocs;
