@@ -1,4 +1,4 @@
-/* $Id: PATMSSM.cpp $ */
+/* $Id: PATMSSM.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * PATMSSM - Dynamic Guest OS Patching Manager; Save and load state
  *
@@ -6,7 +6,7 @@
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -15,10 +15,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -26,16 +22,9 @@
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_PATM
 #include <VBox/patm.h>
-#include <VBox/hwaccm.h>
-#include <VBox/stam.h>
-#include <VBox/pgm.h>
 #include <VBox/cpum.h>
-#include <VBox/iom.h>
-#include <VBox/sup.h>
 #include <VBox/mm.h>
 #include <VBox/ssm.h>
-#include <VBox/pdm.h>
-#include <VBox/trpm.h>
 #include <VBox/param.h>
 #include <iprt/avl.h>
 #include "PATMInternal.h"
@@ -53,11 +42,235 @@
 #include <VBox/dis.h>
 #include <VBox/disopcode.h>
 
+/*******************************************************************************
+*   Defined Constants And Macros                                               *
+*******************************************************************************/
 #define PATM_SUBTRACT_PTR(a, b) *(uintptr_t *)&(a) = (uintptr_t)(a) - (uintptr_t)(b)
 #define PATM_ADD_PTR(a, b)      *(uintptr_t *)&(a) = (uintptr_t)(a) + (uintptr_t)(b)
 
+/*******************************************************************************
+*   Internal Functions                                                         *
+*******************************************************************************/
 static void patmCorrectFixup(PVM pVM, unsigned ulSSMVersion, PATM &patmInfo, PPATCHINFO pPatch, PRELOCREC pRec, int32_t offset, RTRCPTR *pFixup);
 
+/*******************************************************************************
+*   Global Variables                                                           *
+*******************************************************************************/
+/**
+ * SSM descriptor table for the PATM structure.
+ */
+static SSMFIELD const g_aPatmFields[] =
+{
+    /** @todo there are a bunch more fields here which can be marked as ignored. */
+    SSMFIELD_ENTRY_IGNORE(          PATM, offVM),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pPatchMemGC),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATM, pPatchMemHC),
+    SSMFIELD_ENTRY(                 PATM, cbPatchMem),
+    SSMFIELD_ENTRY(                 PATM, offPatchMem),
+    SSMFIELD_ENTRY(                 PATM, fOutOfMemory),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     3, 3),
+    SSMFIELD_ENTRY(                 PATM, deltaReloc),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATM, pGCStateHC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pGCStateGC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pGCStackGC),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATM, pGCStackHC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pCPUMCtxGC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pStatsGC),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATM, pStatsHC),
+    SSMFIELD_ENTRY(                 PATM, uCurrentPatchIdx),
+    SSMFIELD_ENTRY(                 PATM, ulCallDepth),
+    SSMFIELD_ENTRY(                 PATM, cPageRecords),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pPatchedInstrGCLowest),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pPatchedInstrGCHighest),
+    SSMFIELD_ENTRY_RCPTR(           PATM, PatchLookupTreeGC),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATM, PatchLookupTreeHC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pfnHelperCallGC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pfnHelperRetGC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pfnHelperJumpGC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pfnHelperIretGC),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATM, pGlobalPatchRec),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pfnSysEnterGC),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pfnSysEnterPatchGC),
+    SSMFIELD_ENTRY(                 PATM, uSysEnterPatchIdx),
+    SSMFIELD_ENTRY_RCPTR(           PATM, pvFaultMonitor),
+    SSMFIELD_ENTRY_GCPHYS(          PATM, mmio.GCPhys),
+    SSMFIELD_ENTRY_RCPTR(           PATM, mmio.pCachedData),
+    SSMFIELD_ENTRY_IGN_RCPTR(       PATM, mmio.Alignment0),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATM, savedstate.pSSM),
+    SSMFIELD_ENTRY(                 PATM, savedstate.cPatches),
+    SSMFIELD_ENTRY_PAD_HC64(        PATM, savedstate.Alignment0, sizeof(uint32_t)),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatNrOpcodeRead),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatDisabled),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatUnusable),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatEnabled),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInstalled),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInstalledFunctionPatches),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInstalledTrampoline),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInstalledJump),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInt3Callable),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInt3BlockRun),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatOverwritten),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatFixedConflicts),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatFlushed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPageBoundaryCrossed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatMonitored),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatHandleTrap),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatSwitchBack),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatSwitchBackFail),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPATMMemoryUsed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatDuplicateREQSuccess),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatDuplicateREQFailed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatDuplicateUseExisting),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatFunctionFound),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatFunctionNotFound),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchWrite),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchWriteDetect),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatDirty),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPushTrap),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchWriteInterpreted),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchWriteInterpretedFailed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatSysEnter),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatSysExit),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatEmulIret),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatEmulIretFailed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInstrDirty),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInstrDirtyGood),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatInstrDirtyBad),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchPageInserted),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchPageRemoved),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchRefreshSuccess),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatPatchRefreshFailed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatGenRet),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatGenRetReused),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatGenJump),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatGenCall),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatGenPopf),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatCheckPendingIRQ),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatFunctionLookupReplace),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatFunctionLookupInsert),
+    SSMFIELD_ENTRY_IGNORE(          PATM, StatU32FunctionMaxSlotsUsed),
+    SSMFIELD_ENTRY_IGNORE(          PATM, Alignment0),
+    SSMFIELD_ENTRY_TERM()
+};
+
+/**
+ * SSM descriptor table for the PATMGCSTATE structure.
+ */
+static SSMFIELD const g_aPatmGCStateFields[] =
+{
+    SSMFIELD_ENTRY(                 PATMGCSTATE, uVMFlags),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, uPendingAction),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, uPatchCalls),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, uScratch),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, uIretEFlags),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, uIretCS),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, uIretEIP),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, Psp),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, fPIF),
+    SSMFIELD_ENTRY_RCPTR(           PATMGCSTATE, GCPtrInhibitInterrupts),
+    SSMFIELD_ENTRY_RCPTR(           PATMGCSTATE, GCCallPatchTargetAddr),
+    SSMFIELD_ENTRY_RCPTR(           PATMGCSTATE, GCCallReturnAddr),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, Restore.uEAX),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, Restore.uECX),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, Restore.uEDI),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, Restore.eFlags),
+    SSMFIELD_ENTRY(                 PATMGCSTATE, Restore.uFlags),
+    SSMFIELD_ENTRY_TERM()
+};
+
+/**
+ * SSM descriptor table for the PATMPATCHREC structure.
+ */
+static SSMFIELD const g_aPatmPatchRecFields[] =
+{
+    SSMFIELD_ENTRY(                 PATMPATCHREC, Core.Key),
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, Core.pLeft),
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, Core.pRight),
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, Core.uchHeight),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     3, 3),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, CoreOffset.Key),
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, CoreOffset.pLeft),
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, CoreOffset.pRight),
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, CoreOffset.uchHeight),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     3, 3),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.uState),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.uOldState),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.uOpMode),
+    SSMFIELD_ENTRY_RCPTR(           PATMPATCHREC, patch.pPrivInstrGC),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.pPrivInstrHC),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.aPrivInstr),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.cbPrivInstr),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.opcode),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.cbPatchJump),
+    SSMFIELD_ENTRY_RCPTR(           PATMPATCHREC, patch.pPatchJumpDestGC),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.pPatchBlockOffset),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.cbPatchBlockSize),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.uCurPatchOffset),
+    SSMFIELD_ENTRY_PAD_HC64(        PATMPATCHREC, patch.Alignment0, sizeof(uint32_t)),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.flags),
+    SSMFIELD_ENTRY_RCPTR(           PATMPATCHREC, patch.pInstrGCLowest),
+    SSMFIELD_ENTRY_RCPTR(           PATMPATCHREC, patch.pInstrGCHighest),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.FixupTree),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.nrFixups),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.nrJumpRecs), // should be zero?
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.JumpTree),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.Patch2GuestAddrTree),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.Guest2PatchAddrTree),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.nrPatch2GuestRecs),
+    SSMFIELD_ENTRY_PAD_HC64(        PATMPATCHREC, patch.Alignment1, sizeof(uint32_t)),
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.cacheRec.pPatchLocStartHC), // saved as zero
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.cacheRec.pPatchLocEndHC),   // ditto
+    SSMFIELD_ENTRY_IGN_RCPTR(       PATMPATCHREC, patch.cacheRec.pGuestLoc),        // ditto
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, patch.cacheRec.opsize),           // ditto
+    SSMFIELD_ENTRY_IGN_HCPTR(       PATMPATCHREC, patch.pTempInfo),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.cCodeWrites),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.cTraps),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.cInvalidWrites),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.uPatchIdx),
+    SSMFIELD_ENTRY(                 PATMPATCHREC, patch.bDirtyOpcode),
+    SSMFIELD_ENTRY_IGNORE(          PATMPATCHREC, patch.Alignment2),
+    SSMFIELD_ENTRY_TERM()
+};
+
+/**
+ * SSM descriptor table for the RELOCREC structure.
+ */
+static SSMFIELD const g_aPatmRelocRec[] =
+{
+    SSMFIELD_ENTRY_HCPTR_HACK_U32(  RELOCREC, Core.Key),        // Used to store the relocation type
+    SSMFIELD_ENTRY_IGN_HCPTR(       RELOCREC, Core.pLeft),
+    SSMFIELD_ENTRY_IGN_HCPTR(       RELOCREC, Core.pRight),
+    SSMFIELD_ENTRY_IGNORE(          RELOCREC, Core.uchHeight),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     3, 7),
+    SSMFIELD_ENTRY(                 RELOCREC, uType),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     0, 4),
+    SSMFIELD_ENTRY_HCPTR_HACK_U32(  RELOCREC, pRelocPos),       // converted to a patch member offset.
+    SSMFIELD_ENTRY_RCPTR(           RELOCREC, pSource),
+    SSMFIELD_ENTRY_RCPTR(           RELOCREC, pDest),
+    SSMFIELD_ENTRY_TERM()
+};
+
+/**
+ * SSM descriptor table for the RECPATCHTOGUEST structure.
+ */
+static SSMFIELD const g_aPatmRecPatchToGuest[] =
+{
+    SSMFIELD_ENTRY(                 RECPATCHTOGUEST, Core.Key),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     0, 4),
+    SSMFIELD_ENTRY_IGN_HCPTR(       RECPATCHTOGUEST, Core.pLeft),
+    SSMFIELD_ENTRY_IGN_HCPTR(       RECPATCHTOGUEST, Core.pRight),
+    SSMFIELD_ENTRY_IGNORE(          RECPATCHTOGUEST, Core.uchHeight),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     3, 7),
+    SSMFIELD_ENTRY_RCPTR(           RECPATCHTOGUEST, pOrgInstrGC),
+    SSMFIELD_ENTRY(                 RECPATCHTOGUEST, enmType),
+    SSMFIELD_ENTRY(                 RECPATCHTOGUEST, fDirty),
+    SSMFIELD_ENTRY(                 RECPATCHTOGUEST, fJumpTarget),
+    SSMFIELD_ENTRY(                 RECPATCHTOGUEST, u8DirtyOpcode),
+    SSMFIELD_ENTRY_PAD_HC_AUTO(     1, 5),
+    SSMFIELD_ENTRY_TERM()
+};
+
+
 #ifdef VBOX_STRICT
 /**
  * Callback function for RTAvlPVDoWithAll
@@ -152,16 +365,16 @@ static DECLCALLBACK(int) patmSaveFixupRecords(PAVLPVNODECORE pNode, void *pVM1)
         /* Core.Key abused to store the fixup type. */
         if (*pFixup == pVM->pVMRC + RT_OFFSETOF(VM, aCpus[0].fLocalForcedActions))
             rec.Core.Key = (AVLPVKEY)PATM_FIXUP_CPU_FF_ACTION;
-        else 
+        else
         if (*pFixup == CPUMR3GetGuestCpuIdDefRCPtr(pVM))
             rec.Core.Key = (AVLPVKEY)PATM_FIXUP_CPUID_DEFAULT;
-        else 
+        else
         if (*pFixup == CPUMR3GetGuestCpuIdStdRCPtr(pVM))
             rec.Core.Key = (AVLPVKEY)PATM_FIXUP_CPUID_STANDARD;
-        else 
+        else
         if (*pFixup == CPUMR3GetGuestCpuIdExtRCPtr(pVM))
             rec.Core.Key = (AVLPVKEY)PATM_FIXUP_CPUID_EXTENDED;
-        else 
+        else
         if (*pFixup == CPUMR3GetGuestCpuIdCentaurRCPtr(pVM))
             rec.Core.Key = (AVLPVKEY)PATM_FIXUP_CPUID_CENTAUR;
     }
@@ -235,7 +448,7 @@ static DECLCALLBACK(int) patmSavePatchState(PAVLOU32NODECORE pNode, void *pVM1)
  * @param   pVM             VM Handle.
  * @param   pSSM            SSM operation handle.
  */
-DECLCALLBACK(int) patmr3Save(PVM pVM, PSSMHANDLE pSSM)
+DECLCALLBACK(int) patmR3Save(PVM pVM, PSSMHANDLE pSSM)
 {
     PATM patmInfo = pVM->patm.s;
     int  rc;
@@ -298,25 +511,27 @@ DECLCALLBACK(int) patmr3Save(PVM pVM, PSSMHANDLE pSSM)
  * @returns VBox status code.
  * @param   pVM             VM Handle.
  * @param   pSSM            SSM operation handle.
- * @param   u32Version      Data layout version.
+ * @param   uVersion        Data layout version.
+ * @param   uPass           The data pass.
  */
-DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
+DECLCALLBACK(int) patmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
 {
     PATM patmInfo;
     int  rc;
 
-    if (    u32Version != PATM_SSM_VERSION
-        &&  u32Version != PATM_SSM_VERSION_FIXUP_HACK
-        &&  u32Version != PATM_SSM_VERSION_VER16
+    if (    uVersion != PATM_SSM_VERSION
+        &&  uVersion != PATM_SSM_VERSION_FIXUP_HACK
+        &&  uVersion != PATM_SSM_VERSION_VER16
 #ifdef PATM_WITH_NEW_SSM
-        &&  u32Version != PATM_SSM_VERSION_GETPUTMEM)
+        &&  uVersion != PATM_SSM_VERSION_GETPUTMEM)
 #else
        )
 #endif
     {
-        AssertMsgFailed(("patmR3Load: Invalid version u32Version=%d!\n", u32Version));
+        AssertMsgFailed(("patmR3Load: Invalid version uVersion=%d!\n", uVersion));
         return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
     }
+    Assert(uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     pVM->patm.s.savedstate.pSSM = pSSM;
 
@@ -324,11 +539,17 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
      * Restore PATM structure
      */
 #ifdef PATM_WITH_NEW_SSM
-    if (u32Version == PATM_SSM_VERSION_GETPUTMEM)
+    if (uVersion == PATM_SSM_VERSION_GETPUTMEM)
     {
 #endif
+#if 0
         rc = SSMR3GetMem(pSSM, &patmInfo, sizeof(patmInfo));
+#else
+        RT_ZERO(patmInfo);
+        rc = SSMR3GetStructEx(pSSM, &patmInfo, sizeof(patmInfo), SSMSTRUCT_FLAGS_MEM_BAND_AID, &g_aPatmFields[0], NULL);
+#endif
         AssertRCReturn(rc, rc);
+
 #ifdef PATM_WITH_NEW_SSM
     }
     else
@@ -477,10 +698,15 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
      * Restore GC state memory
      */
 #ifdef PATM_WITH_NEW_SSM
-    if (u32Version == PATM_SSM_VERSION_GETPUTMEM)
+    if (uVersion == PATM_SSM_VERSION_GETPUTMEM)
     {
 #endif
+#if 0
         rc = SSMR3GetMem(pSSM, pVM->patm.s.pGCStateHC, sizeof(PATMGCSTATE));
+#else
+        RT_BZERO(pVM->patm.s.pGCStateHC, sizeof(PATMGCSTATE));
+        rc = SSMR3GetStructEx(pSSM, pVM->patm.s.pGCStateHC, sizeof(PATMGCSTATE), SSMSTRUCT_FLAGS_MEM_BAND_AID, &g_aPatmGCStateFields[0], NULL);
+#endif
         AssertRCReturn(rc, rc);
 #ifdef PATM_WITH_NEW_SSM
     }
@@ -569,7 +795,12 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
     {
         PATMPATCHREC patch, *pPatchRec;
 
+#if 0
         rc = SSMR3GetMem(pSSM, &patch, sizeof(patch));
+#else
+        RT_ZERO(patch);
+        rc = SSMR3GetStructEx(pSSM, &patch, sizeof(patch), SSMSTRUCT_FLAGS_MEM_BAND_AID, &g_aPatmPatchRecFields[0], NULL);
+#endif
         AssertRCReturn(rc, rc);
 
         Assert(!(patch.patch.flags & PATMFL_GLOBAL_FUNCTIONS));
@@ -612,17 +843,22 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
          */
         pPatchRec->patch.FixupTree = 0;
         pPatchRec->patch.nrFixups  = 0;    /* increased by patmPatchAddReloc32 */
-        for (int i=0;i<patch.patch.nrFixups;i++)
+        for (int j=0;j<patch.patch.nrFixups;j++)
         {
             RELOCREC rec;
             int32_t offset;
             RTRCPTR *pFixup;
 
+#if 0
             rc = SSMR3GetMem(pSSM, &rec, sizeof(rec));
+#else
+            RT_ZERO(rec);
+            rc = SSMR3GetStructEx(pSSM, &rec, sizeof(rec), SSMSTRUCT_FLAGS_MEM_BAND_AID, &g_aPatmRelocRec[0], NULL);
+#endif
             AssertRCReturn(rc, rc);
 
             /* rec.pRelocPos now contains the relative position inside the hypervisor area. */
-            offset = (int32_t)(int64_t)rec.pRelocPos;
+            offset = (int32_t)(intptr_t)rec.pRelocPos;
             /* Convert to HC pointer again. */
             PATM_ADD_PTR(rec.pRelocPos, pVM->patm.s.pPatchMemHC);
             pFixup = (RTRCPTR *)rec.pRelocPos;
@@ -633,17 +869,17 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
                     &&  (pPatchRec->patch.flags & PATMFL_PATCHED_GUEST_CODE))
                 {
                     Assert(pPatchRec->patch.cbPatchJump == SIZEOF_NEARJUMP32 || pPatchRec->patch.cbPatchJump == SIZEOF_NEAR_COND_JUMP32);
-                    unsigned offset = (pPatchRec->patch.cbPatchJump == SIZEOF_NEARJUMP32) ? 1 : 2;
+                    unsigned offset2 = (pPatchRec->patch.cbPatchJump == SIZEOF_NEARJUMP32) ? 1 : 2;
 
                     /** @todo This will fail & crash in patmCorrectFixup if the page isn't present
                      *        when we restore. Happens with my XP image here
                      *        (pPrivInstrGC=0x8069e051). */
                     AssertLogRelMsg(pPatchRec->patch.pPrivInstrHC, ("%RRv rc=%Rrc uState=%u\n", pPatchRec->patch.pPrivInstrGC, rc2, pPatchRec->patch.uState));
-                    rec.pRelocPos = pPatchRec->patch.pPrivInstrHC + offset;
+                    rec.pRelocPos = pPatchRec->patch.pPrivInstrHC + offset2;
                     pFixup        = (RTRCPTR *)rec.pRelocPos;
                 }
 
-                patmCorrectFixup(pVM, u32Version, patmInfo, &pPatchRec->patch, &rec, offset, pFixup);
+                patmCorrectFixup(pVM, uVersion, patmInfo, &pPatchRec->patch, &rec, offset, pFixup);
             }
 
             rc = patmPatchAddReloc32(pVM, &pPatchRec->patch, rec.pRelocPos, rec.uType, rec.pSource, rec.pDest);
@@ -661,9 +897,15 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
             uint32_t        nrPatch2GuestRecs = pPatchRec->patch.nrPatch2GuestRecs;
 
             pPatchRec->patch.nrPatch2GuestRecs = 0;    /* incremented by patmr3AddP2GLookupRecord */
-            for (uint32_t i=0;i<nrPatch2GuestRecs;i++)
+            for (uint32_t j=0;j<nrPatch2GuestRecs;j++)
             {
+#if 0
                 rc = SSMR3GetMem(pSSM, &rec, sizeof(rec));
+#else
+                RT_ZERO(rec);
+                rc = SSMR3GetStructEx(pSSM, &rec, sizeof(rec), SSMSTRUCT_FLAGS_MEM_BAND_AID, &g_aPatmRecPatchToGuest[0], NULL);
+#endif
+
                 AssertRCReturn(rc, rc);
 
                 patmr3AddP2GLookupRecord(pVM, &pPatchRec->patch, (uintptr_t)rec.Core.Key + pVM->patm.s.pPatchMemHC, rec.pOrgInstrGC, rec.enmType, rec.fDirty);
@@ -718,7 +960,7 @@ DECLCALLBACK(int) patmr3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
         pFixup = (RTRCPTR *)pRec->pRelocPos;
 
         /* Correct fixups that refer to PATM structures in the hypervisor region (their addresses might have changed). */
-        patmCorrectFixup(pVM, u32Version, patmInfo, &pVM->patm.s.pGlobalPatchRec->patch, pRec, offset, pFixup);
+        patmCorrectFixup(pVM, uVersion, patmInfo, &pVM->patm.s.pGlobalPatchRec->patch, pRec, offset, pFixup);
     }
 
 #ifdef VBOX_WITH_STATISTICS
@@ -754,7 +996,7 @@ static void patmCorrectFixup(PVM pVM, unsigned ulSSMVersion, PATM &patmInfo, PPA
     {
     case FIXUP_ABSOLUTE:
     {
-        if (pRec->pSource && !PATMIsPatchGCAddr(pVM, pRec->pSource))
+        if (pRec->pSource && !PATMIsPatchGCAddr(pVM, (RTRCUINTPTR)pRec->pSource))
             break;
 
         if (    *pFixup >= patmInfo.pGCStateGC
@@ -895,17 +1137,21 @@ static void patmCorrectFixup(PVM pVM, unsigned ulSSMVersion, PATM &patmInfo, PPA
             *pFixup = (*pFixup - patmInfo.pPatchMemGC) + pVM->patm.s.pPatchMemGC;
         }
         else
+        /* Boldly ASSUMES:
+         * 1. That pCPUMCtxGC is in the VM structure and that its location is
+         *    at the first page of the same 4 MB chunk.
+         * 2. That the forced actions were in the first 32 bytes of the VM
+         *    structure.
+         * 3. That the CPUM leafs are less than 8KB into the structure. */
         if (    ulSSMVersion <= PATM_SSM_VERSION_FIXUP_HACK
-            &&  *pFixup >= pVM->pVMRC
-            &&  *pFixup < pVM->pVMRC + 32)
+            &&  *pFixup - (patmInfo.pCPUMCtxGC & UINT32_C(0xffc00000)) < UINT32_C(32))
         {
-            LogFlow(("Changing fLocalForcedActions fixup from %x to %x\n", *pFixup, pVM->pVMRC + RT_OFFSETOF(VM, aCpus[0].fLocalForcedActions)));
+            LogFlow(("Changing fLocalForcedActions fixup from %RRv to %RRv\n", *pFixup, pVM->pVMRC + RT_OFFSETOF(VM, aCpus[0].fLocalForcedActions)));
             *pFixup = pVM->pVMRC + RT_OFFSETOF(VM, aCpus[0].fLocalForcedActions);
         }
         else
         if (    ulSSMVersion <= PATM_SSM_VERSION_FIXUP_HACK
-            &&  *pFixup >= pVM->pVMRC
-            &&  *pFixup < pVM->pVMRC + 8192)
+            &&  *pFixup - (patmInfo.pCPUMCtxGC & UINT32_C(0xffc00000)) < UINT32_C(8192))
         {
             static int cCpuidFixup = 0;
 #ifdef LOG_ENABLED
@@ -927,7 +1173,7 @@ static void patmCorrectFixup(PVM pVM, unsigned ulSSMVersion, PATM &patmInfo, PPA
                 *pFixup = CPUMR3GetGuestCpuIdCentaurRCPtr(pVM);
                 break;
             }
-            LogFlow(("Changing cpuid fixup %d from %x to %x\n", cCpuidFixup, oldFixup, *pFixup));
+            LogFlow(("Changing cpuid fixup %d from %RRv to %RRv\n", cCpuidFixup, oldFixup, *pFixup));
             cCpuidFixup++;
         }
         else
