@@ -1,10 +1,10 @@
-/* $Id: PDMQueue.cpp $ */
+/* $Id: PDMQueue.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * PDM Queue - Transport data and tasks to EMT and R3.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -29,6 +25,7 @@
 #include <VBox/mm.h>
 #include <VBox/rem.h>
 #include <VBox/vm.h>
+#include <VBox/uvm.h>
 #include <VBox/err.h>
 
 #include <VBox/log.h>
@@ -56,10 +53,14 @@ static DECLCALLBACK(void)   pdmR3QueueTimer(PVM pVM, PTMTIMER pTimer, void *pvUs
  * @param   cMilliesInterval    Number of milliseconds between polling the queue.
  *                              If 0 then the emulation thread will be notified whenever an item arrives.
  * @param   fRZEnabled          Set if the queue will be used from RC/R0 and need to be allocated from the hyper heap.
+ * @param   pszName             The queue name. Unique. Not copied.
  * @param   ppQueue             Where to store the queue handle.
  */
-static int pdmR3QueueCreate(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMilliesInterval, bool fRZEnabled, PPDMQUEUE *ppQueue)
+static int pdmR3QueueCreate(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMilliesInterval, bool fRZEnabled,
+                            const char *pszName, PPDMQUEUE *ppQueue)
 {
+    PUVM pUVM = pVM->pUVM;
+
     /*
      * Validate input.
      */
@@ -94,6 +95,7 @@ static int pdmR3QueueCreate(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMil
     pQueue->pVMR3 = pVM;
     pQueue->pVMR0 = fRZEnabled ? pVM->pVMR0 : NIL_RTR0PTR;
     pQueue->pVMRC = fRZEnabled ? pVM->pVMRC : NIL_RTRCPTR;
+    pQueue->pszName = pszName;
     pQueue->cMilliesInterval = cMilliesInterval;
     //pQueue->pTimer = NULL;
     pQueue->cbItem = cbItem;
@@ -119,14 +121,14 @@ static int pdmR3QueueCreate(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMil
      */
     if (cMilliesInterval)
     {
-        int rc = TMR3TimerCreateInternal(pVM, TMCLOCK_REAL, pdmR3QueueTimer, pQueue, "Queue timer", &pQueue->pTimer);
+        rc = TMR3TimerCreateInternal(pVM, TMCLOCK_REAL, pdmR3QueueTimer, pQueue, "Queue timer", &pQueue->pTimer);
         if (RT_SUCCESS(rc))
         {
             rc = TMTimerSetMillies(pQueue->pTimer, cMilliesInterval);
             if (RT_FAILURE(rc))
             {
                 AssertMsgFailed(("TMTimerSetMillies failed rc=%Rrc\n", rc));
-                int rc2 = TMTimerDestroy(pQueue->pTimer); AssertRC(rc2);
+                int rc2 = TMR3TimerDestroy(pQueue->pTimer); AssertRC(rc2);
             }
         }
         else
@@ -143,8 +145,10 @@ static int pdmR3QueueCreate(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMil
         /*
          * Insert into the queue list for timer driven queues.
          */
-        pQueue->pNext = pVM->pdm.s.pQueuesTimer;
-        pVM->pdm.s.pQueuesTimer = pQueue;
+        pdmLock(pVM);
+        pQueue->pNext = pUVM->pdm.s.pQueuesTimer;
+        pUVM->pdm.s.pQueuesTimer = pQueue;
+        pdmUnlock(pVM);
     }
     else
     {
@@ -152,23 +156,39 @@ static int pdmR3QueueCreate(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMil
          * Insert into the queue list for forced action driven queues.
          * This is a FIFO, so insert at the end.
          */
-        /** @todo we should add a priority priority to the queues so we don't have to rely on
+        /** @todo we should add a priority to the queues so we don't have to rely on
          * the initialization order to deal with problems like #1605 (pgm/pcnet deadlock
          * caused by the critsect queue to be last in the chain).
          * - Update, the critical sections are no longer using queues, so this isn't a real
          *   problem any longer. The priority might be a nice feature for later though.
          */
-        if (!pVM->pdm.s.pQueuesForced)
-            pVM->pdm.s.pQueuesForced = pQueue;
+        pdmLock(pVM);
+        if (!pUVM->pdm.s.pQueuesForced)
+            pUVM->pdm.s.pQueuesForced = pQueue;
         else
         {
-            PPDMQUEUE pPrev = pVM->pdm.s.pQueuesForced;
+            PPDMQUEUE pPrev = pUVM->pdm.s.pQueuesForced;
             while (pPrev->pNext)
                 pPrev = pPrev->pNext;
             pPrev->pNext = pQueue;
         }
+        pdmUnlock(pVM);
     }
 
+    /*
+     * Register the statistics.
+     */
+    STAMR3RegisterF(pVM, &pQueue->cbItem,               STAMTYPE_U32,     STAMVISIBILITY_ALWAYS, STAMUNIT_BYTES,        "Item size.",                       "/PDM/Queue/%s/cbItem",         pQueue->pszName);
+    STAMR3RegisterF(pVM, &pQueue->cItems,               STAMTYPE_U32,     STAMVISIBILITY_ALWAYS, STAMUNIT_COUNT,        "Queue size.",                      "/PDM/Queue/%s/cItems",         pQueue->pszName);
+    STAMR3RegisterF(pVM, &pQueue->StatAllocFailures,    STAMTYPE_COUNTER, STAMVISIBILITY_ALWAYS, STAMUNIT_OCCURENCES,   "PDMQueueAlloc failures.",          "/PDM/Queue/%s/AllocFailures",  pQueue->pszName);
+    STAMR3RegisterF(pVM, &pQueue->StatInsert,           STAMTYPE_COUNTER, STAMVISIBILITY_ALWAYS, STAMUNIT_CALLS,        "Calls to PDMQueueInsert.",         "/PDM/Queue/%s/Insert",         pQueue->pszName);
+    STAMR3RegisterF(pVM, &pQueue->StatFlush,            STAMTYPE_COUNTER, STAMVISIBILITY_ALWAYS, STAMUNIT_CALLS,        "Calls to pdmR3QueueFlush.",        "/PDM/Queue/%s/Flush",          pQueue->pszName);
+    STAMR3RegisterF(pVM, &pQueue->StatFlushLeftovers,   STAMTYPE_COUNTER, STAMVISIBILITY_ALWAYS, STAMUNIT_OCCURENCES,   "Left over items after flush.",     "/PDM/Queue/%s/FlushLeftovers", pQueue->pszName);
+#ifdef VBOX_WITH_STATISTICS
+    STAMR3RegisterF(pVM, &pQueue->StatFlushPrf,         STAMTYPE_PROFILE, STAMVISIBILITY_ALWAYS, STAMUNIT_CALLS,        "Profiling pdmR3QueueFlush.",       "/PDM/Queue/%s/FlushPrf",       pQueue->pszName);
+    STAMR3RegisterF(pVM, (void *)&pQueue->cStatPending, STAMTYPE_U32,     STAMVISIBILITY_ALWAYS, STAMUNIT_COUNT,        "Pending items.",                   "/PDM/Queue/%s/Pending",        pQueue->pszName);
+#endif
+
     *ppQueue = pQueue;
     return VINF_SUCCESS;
 }
@@ -186,19 +206,20 @@ static int pdmR3QueueCreate(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMil
  *                              If 0 then the emulation thread will be notified whenever an item arrives.
  * @param   pfnCallback         The consumer function.
  * @param   fRZEnabled          Set if the queue must be usable from RC/R0.
+ * @param   pszName             The queue name. Unique. Not copied.
  * @param   ppQueue             Where to store the queue handle on success.
  * @thread  Emulation thread only.
  */
 VMMR3DECL(int) PDMR3QueueCreateDevice(PVM pVM, PPDMDEVINS pDevIns, RTUINT cbItem, RTUINT cItems, uint32_t cMilliesInterval,
-                                      PFNPDMQUEUEDEV pfnCallback, bool fRZEnabled, PPDMQUEUE *ppQueue)
+                                      PFNPDMQUEUEDEV pfnCallback, bool fRZEnabled, const char *pszName, PPDMQUEUE *ppQueue)
 {
-    LogFlow(("PDMR3QueueCreateDevice: pDevIns=%p cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p fRZEnabled=%RTbool\n",
-             pDevIns, cbItem, cItems, cMilliesInterval, pfnCallback, fRZEnabled));
+    LogFlow(("PDMR3QueueCreateDevice: pDevIns=%p cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p fRZEnabled=%RTbool pszName=%s\n",
+             pDevIns, cbItem, cItems, cMilliesInterval, pfnCallback, fRZEnabled, pszName));
 
     /*
      * Validate input.
      */
-    VM_ASSERT_EMT(pVM);
+    VMCPU_ASSERT_EMT(&pVM->aCpus[0]);
     if (!pfnCallback)
     {
         AssertMsgFailed(("No consumer callback!\n"));
@@ -209,7 +230,7 @@ VMMR3DECL(int) PDMR3QueueCreateDevice(PVM pVM, PPDMDEVINS pDevIns, RTUINT cbItem
      * Create the queue.
      */
     PPDMQUEUE pQueue;
-    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, fRZEnabled, &pQueue);
+    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, fRZEnabled, pszName, &pQueue);
     if (RT_SUCCESS(rc))
     {
         pQueue->enmType = PDMQUEUETYPE_DEV;
@@ -235,19 +256,20 @@ VMMR3DECL(int) PDMR3QueueCreateDevice(PVM pVM, PPDMDEVINS pDevIns, RTUINT cbItem
  * @param   cMilliesInterval    Number of milliseconds between polling the queue.
  *                              If 0 then the emulation thread will be notified whenever an item arrives.
  * @param   pfnCallback         The consumer function.
+ * @param   pszName             The queue name. Unique. Not copied.
  * @param   ppQueue             Where to store the queue handle on success.
  * @thread  Emulation thread only.
  */
 VMMR3DECL(int) PDMR3QueueCreateDriver(PVM pVM, PPDMDRVINS pDrvIns, RTUINT cbItem, RTUINT cItems, uint32_t cMilliesInterval,
-                                      PFNPDMQUEUEDRV pfnCallback, PPDMQUEUE *ppQueue)
+                                      PFNPDMQUEUEDRV pfnCallback, const char *pszName, PPDMQUEUE *ppQueue)
 {
-    LogFlow(("PDMR3QueueCreateDriver: pDrvIns=%p cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p\n",
-             pDrvIns, cbItem, cItems, cMilliesInterval, pfnCallback));
+    LogFlow(("PDMR3QueueCreateDriver: pDrvIns=%p cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p pszName=%s\n",
+             pDrvIns, cbItem, cItems, cMilliesInterval, pfnCallback, pszName));
 
     /*
      * Validate input.
      */
-    VM_ASSERT_EMT(pVM);
+    VMCPU_ASSERT_EMT(&pVM->aCpus[0]);
     if (!pfnCallback)
     {
         AssertMsgFailed(("No consumer callback!\n"));
@@ -258,7 +280,7 @@ VMMR3DECL(int) PDMR3QueueCreateDriver(PVM pVM, PPDMDRVINS pDrvIns, RTUINT cbItem
      * Create the queue.
      */
     PPDMQUEUE pQueue;
-    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, false, &pQueue);
+    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, false, pszName, &pQueue);
     if (RT_SUCCESS(rc))
     {
         pQueue->enmType = PDMQUEUETYPE_DRV;
@@ -284,19 +306,20 @@ VMMR3DECL(int) PDMR3QueueCreateDriver(PVM pVM, PPDMDRVINS pDrvIns, RTUINT cbItem
  *                              If 0 then the emulation thread will be notified whenever an item arrives.
  * @param   pfnCallback         The consumer function.
  * @param   fRZEnabled          Set if the queue must be usable from RC/R0.
+ * @param   pszName             The queue name. Unique. Not copied.
  * @param   ppQueue             Where to store the queue handle on success.
  * @thread  Emulation thread only.
  */
 VMMR3DECL(int) PDMR3QueueCreateInternal(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMilliesInterval,
-                                        PFNPDMQUEUEINT pfnCallback, bool fRZEnabled, PPDMQUEUE *ppQueue)
+                                        PFNPDMQUEUEINT pfnCallback, bool fRZEnabled, const char *pszName, PPDMQUEUE *ppQueue)
 {
-    LogFlow(("PDMR3QueueCreateInternal: cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p fRZEnabled=%RTbool\n",
-             cbItem, cItems, cMilliesInterval, pfnCallback, fRZEnabled));
+    LogFlow(("PDMR3QueueCreateInternal: cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p fRZEnabled=%RTbool pszName=%s\n",
+             cbItem, cItems, cMilliesInterval, pfnCallback, fRZEnabled, pszName));
 
     /*
      * Validate input.
      */
-    VM_ASSERT_EMT(pVM);
+    VMCPU_ASSERT_EMT(&pVM->aCpus[0]);
     if (!pfnCallback)
     {
         AssertMsgFailed(("No consumer callback!\n"));
@@ -307,7 +330,7 @@ VMMR3DECL(int) PDMR3QueueCreateInternal(PVM pVM, RTUINT cbItem, RTUINT cItems, u
      * Create the queue.
      */
     PPDMQUEUE pQueue;
-    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, fRZEnabled, &pQueue);
+    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, fRZEnabled, pszName, &pQueue);
     if (RT_SUCCESS(rc))
     {
         pQueue->enmType = PDMQUEUETYPE_INTERNAL;
@@ -332,17 +355,18 @@ VMMR3DECL(int) PDMR3QueueCreateInternal(PVM pVM, RTUINT cbItem, RTUINT cItems, u
  *                              If 0 then the emulation thread will be notified whenever an item arrives.
  * @param   pfnCallback         The consumer function.
  * @param   pvUser              The user argument to the consumer function.
+ * @param   pszName             The queue name. Unique. Not copied.
  * @param   ppQueue             Where to store the queue handle on success.
  * @thread  Emulation thread only.
  */
-VMMR3DECL(int) PDMR3QueueCreateExternal(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMilliesInterval, PFNPDMQUEUEEXT pfnCallback, void *pvUser, PPDMQUEUE *ppQueue)
+VMMR3DECL(int) PDMR3QueueCreateExternal(PVM pVM, RTUINT cbItem, RTUINT cItems, uint32_t cMilliesInterval, PFNPDMQUEUEEXT pfnCallback, void *pvUser, const char *pszName, PPDMQUEUE *ppQueue)
 {
-    LogFlow(("PDMR3QueueCreateExternal: cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p\n", cbItem, cItems, cMilliesInterval, pfnCallback));
+    LogFlow(("PDMR3QueueCreateExternal: cbItem=%d cItems=%d cMilliesInterval=%d pfnCallback=%p pszName=%s\n", cbItem, cItems, cMilliesInterval, pfnCallback, pszName));
 
     /*
      * Validate input.
      */
-    VM_ASSERT_EMT(pVM);
+    VMCPU_ASSERT_EMT(&pVM->aCpus[0]);
     if (!pfnCallback)
     {
         AssertMsgFailed(("No consumer callback!\n"));
@@ -353,7 +377,7 @@ VMMR3DECL(int) PDMR3QueueCreateExternal(PVM pVM, RTUINT cbItem, RTUINT cItems, u
      * Create the queue.
      */
     PPDMQUEUE pQueue;
-    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, false, &pQueue);
+    int rc = pdmR3QueueCreate(pVM, cbItem, cItems, cMilliesInterval, false, pszName, &pQueue);
     if (RT_SUCCESS(rc))
     {
         pQueue->enmType = PDMQUEUETYPE_EXTERNAL;
@@ -385,17 +409,19 @@ VMMR3DECL(int) PDMR3QueueDestroy(PPDMQUEUE pQueue)
     if (!pQueue)
         return VERR_INVALID_PARAMETER;
     Assert(pQueue && pQueue->pVMR3);
-    PVM pVM = pQueue->pVMR3;
-    VM_ASSERT_EMT(pVM);
+    PVM     pVM  = pQueue->pVMR3;
+    PUVM    pUVM = pVM->pUVM;
+
+    pdmLock(pVM);
 
     /*
      * Unlink it.
      */
     if (pQueue->pTimer)
     {
-        if (pVM->pdm.s.pQueuesTimer != pQueue)
+        if (pUVM->pdm.s.pQueuesTimer != pQueue)
         {
-            PPDMQUEUE pCur = pVM->pdm.s.pQueuesTimer;
+            PPDMQUEUE pCur = pUVM->pdm.s.pQueuesTimer;
             while (pCur)
             {
                 if (pCur->pNext == pQueue)
@@ -408,13 +434,13 @@ VMMR3DECL(int) PDMR3QueueDestroy(PPDMQUEUE pQueue)
             AssertMsg(pCur, ("Didn't find the queue!\n"));
         }
         else
-            pVM->pdm.s.pQueuesTimer = pQueue->pNext;
+            pUVM->pdm.s.pQueuesTimer = pQueue->pNext;
     }
     else
     {
-        if (pVM->pdm.s.pQueuesForced != pQueue)
+        if (pUVM->pdm.s.pQueuesForced != pQueue)
         {
-            PPDMQUEUE pCur = pVM->pdm.s.pQueuesForced;
+            PPDMQUEUE pCur = pUVM->pdm.s.pQueuesForced;
             while (pCur)
             {
                 if (pCur->pNext == pQueue)
@@ -427,17 +453,32 @@ VMMR3DECL(int) PDMR3QueueDestroy(PPDMQUEUE pQueue)
             AssertMsg(pCur, ("Didn't find the queue!\n"));
         }
         else
-            pVM->pdm.s.pQueuesForced = pQueue->pNext;
+            pUVM->pdm.s.pQueuesForced = pQueue->pNext;
     }
     pQueue->pNext = NULL;
     pQueue->pVMR3 = NULL;
+    pdmUnlock(pVM);
+
+    /*
+     * Deregister statistics.
+     */
+    STAMR3Deregister(pVM, &pQueue->cbItem);
+    STAMR3Deregister(pVM, &pQueue->cbItem);
+    STAMR3Deregister(pVM, &pQueue->StatAllocFailures);
+    STAMR3Deregister(pVM, &pQueue->StatInsert);
+    STAMR3Deregister(pVM, &pQueue->StatFlush);
+    STAMR3Deregister(pVM, &pQueue->StatFlushLeftovers);
+#ifdef VBOX_WITH_STATISTICS
+    STAMR3Deregister(pVM, &pQueue->StatFlushPrf);
+    STAMR3Deregister(pVM, (void *)&pQueue->cStatPending);
+#endif
 
     /*
      * Destroy the timer and free it.
      */
     if (pQueue->pTimer)
     {
-        TMTimerDestroy(pQueue->pTimer);
+        TMR3TimerDestroy(pQueue->pTimer);
         pQueue->pTimer = NULL;
     }
     if (pQueue->pVMRC)
@@ -470,13 +511,15 @@ VMMR3DECL(int) PDMR3QueueDestroyDevice(PVM pVM, PPDMDEVINS pDevIns)
      */
     if (!pDevIns)
         return VERR_INVALID_PARAMETER;
-    VM_ASSERT_EMT(pVM);
+
+    PUVM pUVM = pVM->pUVM;
+    pdmLock(pVM);
 
     /*
      * Unlink it.
      */
-    PPDMQUEUE pQueueNext = pVM->pdm.s.pQueuesTimer;
-    PPDMQUEUE pQueue = pVM->pdm.s.pQueuesForced;
+    PPDMQUEUE pQueueNext = pUVM->pdm.s.pQueuesTimer;
+    PPDMQUEUE pQueue     = pUVM->pdm.s.pQueuesForced;
     do
     {
         while (pQueue)
@@ -498,6 +541,7 @@ VMMR3DECL(int) PDMR3QueueDestroyDevice(PVM pVM, PPDMDEVINS pDevIns)
         pQueueNext = NULL;
     } while (pQueue);
 
+    pdmUnlock(pVM);
     return VINF_SUCCESS;
 }
 
@@ -519,13 +563,15 @@ VMMR3DECL(int) PDMR3QueueDestroyDriver(PVM pVM, PPDMDRVINS pDrvIns)
      */
     if (!pDrvIns)
         return VERR_INVALID_PARAMETER;
-    VM_ASSERT_EMT(pVM);
+
+    PUVM pUVM = pVM->pUVM;
+    pdmLock(pVM);
 
     /*
      * Unlink it.
      */
-    PPDMQUEUE pQueueNext = pVM->pdm.s.pQueuesTimer;
-    PPDMQUEUE pQueue = pVM->pdm.s.pQueuesForced;
+    PPDMQUEUE pQueueNext = pUVM->pdm.s.pQueuesTimer;
+    PPDMQUEUE pQueue     = pUVM->pdm.s.pQueuesForced;
     do
     {
         while (pQueue)
@@ -547,6 +593,7 @@ VMMR3DECL(int) PDMR3QueueDestroyDriver(PVM pVM, PPDMDRVINS pDrvIns)
         pQueueNext = NULL;
     } while (pQueue);
 
+    pdmUnlock(pVM);
     return VINF_SUCCESS;
 }
 
@@ -562,8 +609,9 @@ void pdmR3QueueRelocate(PVM pVM, RTGCINTPTR offDelta)
     /*
      * Process the queues.
      */
-    PPDMQUEUE pQueueNext = pVM->pdm.s.pQueuesTimer;
-    PPDMQUEUE pQueue = pVM->pdm.s.pQueuesForced;
+    PUVM      pUVM       = pVM->pUVM;
+    PPDMQUEUE pQueueNext = pUVM->pdm.s.pQueuesTimer;
+    PPDMQUEUE pQueue     = pUVM->pdm.s.pQueuesForced;
     do
     {
         while (pQueue)
@@ -616,18 +664,30 @@ VMMR3DECL(void) PDMR3QueueFlushAll(PVM pVM)
     VM_ASSERT_EMT(pVM);
     LogFlow(("PDMR3QueuesFlush:\n"));
 
+    /*
+     * Only let one EMT flushing queues at any one time to queue preserve order
+     * and to avoid wasting time. The FF is always cleared here, because it's
+     * only used to get someones attention. Queue inserts occuring during the
+     * flush are caught using the pending bit.
+     *
+     * Note. The order in which the FF and pending bit are set and cleared is important.
+     */
     VM_FF_CLEAR(pVM, VM_FF_PDM_QUEUES);
-    for (PPDMQUEUE pCur = pVM->pdm.s.pQueuesForced; pCur; pCur = pCur->pNext)
+    if (!ASMAtomicBitTestAndSet(&pVM->pdm.s.fQueueFlushing, PDM_QUEUE_FLUSH_FLAG_ACTIVE_BIT))
     {
-        if (    pCur->pPendingR3
-            ||  pCur->pPendingR0
-            ||  pCur->pPendingRC)
+        ASMAtomicBitClear(&pVM->pdm.s.fQueueFlushing, PDM_QUEUE_FLUSH_FLAG_PENDING_BIT);
+        do
         {
-            if (    pdmR3QueueFlush(pCur)
-                &&  pCur->pPendingR3)
-                /* new items arrived while flushing. */
-                pdmR3QueueFlush(pCur);
-        }
+            VM_FF_CLEAR(pVM, VM_FF_PDM_QUEUES);
+            for (PPDMQUEUE pCur = pVM->pUVM->pdm.s.pQueuesForced; pCur; pCur = pCur->pNext)
+                if (    pCur->pPendingR3
+                    ||  pCur->pPendingR0
+                    ||  pCur->pPendingRC)
+                    pdmR3QueueFlush(pCur);
+        } while (   ASMAtomicBitTestAndClear(&pVM->pdm.s.fQueueFlushing, PDM_QUEUE_FLUSH_FLAG_PENDING_BIT)
+                 || VM_FF_ISPENDING(pVM, VM_FF_PDM_QUEUES));
+
+        ASMAtomicBitClear(&pVM->pdm.s.fQueueFlushing, PDM_QUEUE_FLUSH_FLAG_ACTIVE_BIT);
     }
 }
 
@@ -641,6 +701,8 @@ VMMR3DECL(void) PDMR3QueueFlushAll(PVM pVM)
  */
 static bool pdmR3QueueFlush(PPDMQUEUE pQueue)
 {
+    STAM_PROFILE_START(&pQueue->StatFlushPrf,p);
+
     /*
      * Get the lists.
      */
@@ -648,8 +710,11 @@ static bool pdmR3QueueFlush(PPDMQUEUE pQueue)
     RTRCPTR           pItemsRC = ASMAtomicXchgRCPtr(&pQueue->pPendingRC, NIL_RTRCPTR);
     RTR0PTR           pItemsR0 = ASMAtomicXchgR0Ptr(&pQueue->pPendingR0, NIL_RTR0PTR);
 
-    AssertMsg(pItems || pItemsRC || pItemsR0, ("ERROR: can't all be NULL now!\n"));
-
+    if (    !pItems
+        &&  !pItemsRC
+        &&  !pItemsR0)
+        /* Somebody may be racing us ... never mind. */
+        return true;
 
     /*
      * Reverse the list (it's inserted in LIFO order to avoid semaphores, remember).
@@ -697,10 +762,10 @@ static bool pdmR3QueueFlush(PPDMQUEUE pQueue)
         case PDMQUEUETYPE_DEV:
             while (pItems)
             {
+                if (!pQueue->u.Dev.pfnCallback(pQueue->u.Dev.pDevIns, pItems))
+                    break;
                 pCur = pItems;
                 pItems = pItems->pNextR3;
-                if (!pQueue->u.Dev.pfnCallback(pQueue->u.Dev.pDevIns, pCur))
-                    break;
                 pdmR3QueueFree(pQueue, pCur);
             }
             break;
@@ -708,10 +773,10 @@ static bool pdmR3QueueFlush(PPDMQUEUE pQueue)
         case PDMQUEUETYPE_DRV:
             while (pItems)
             {
+                if (!pQueue->u.Drv.pfnCallback(pQueue->u.Drv.pDrvIns, pItems))
+                    break;
                 pCur = pItems;
                 pItems = pItems->pNextR3;
-                if (!pQueue->u.Drv.pfnCallback(pQueue->u.Drv.pDrvIns, pCur))
-                    break;
                 pdmR3QueueFree(pQueue, pCur);
             }
             break;
@@ -719,10 +784,10 @@ static bool pdmR3QueueFlush(PPDMQUEUE pQueue)
         case PDMQUEUETYPE_INTERNAL:
             while (pItems)
             {
+                if (!pQueue->u.Int.pfnCallback(pQueue->pVMR3, pItems))
+                    break;
                 pCur = pItems;
                 pItems = pItems->pNextR3;
-                if (!pQueue->u.Int.pfnCallback(pQueue->pVMR3, pCur))
-                    break;
                 pdmR3QueueFree(pQueue, pCur);
             }
             break;
@@ -730,10 +795,10 @@ static bool pdmR3QueueFlush(PPDMQUEUE pQueue)
         case PDMQUEUETYPE_EXTERNAL:
             while (pItems)
             {
+                if (!pQueue->u.Ext.pfnCallback(pQueue->u.Ext.pvUser, pItems))
+                    break;
                 pCur = pItems;
                 pItems = pItems->pNextR3;
-                if (!pQueue->u.Ext.pfnCallback(pQueue->u.Ext.pvUser, pCur))
-                    break;
                 pdmR3QueueFree(pQueue, pCur);
             }
             break;
@@ -749,34 +814,42 @@ static bool pdmR3QueueFlush(PPDMQUEUE pQueue)
     if (pItems)
     {
         /*
-         * Shit, no!
-         *      1. Insert pCur.
-         *      2. Reverse the list.
-         *      3. Insert the LIFO at the tail of the pending list.
+         * Reverse the list.
          */
-        pCur->pNextR3 = pItems;
-        pItems = pCur;
-
-        //pCur = pItems;
+        pCur = pItems;
         pItems = NULL;
         while (pCur)
         {
             PPDMQUEUEITEMCORE pInsert = pCur;
-            pCur = pCur->pNextR3;
+            pCur = pInsert->pNextR3;
             pInsert->pNextR3 = pItems;
             pItems = pInsert;
         }
 
-        if (!ASMAtomicCmpXchgPtr((void * volatile *)&pQueue->pPendingR3, pItems, NULL))
+        /*
+         * Insert the list at the tail of the pending list.
+         */
+        for (;;)
         {
-            pCur = pQueue->pPendingR3;
-            while (pCur->pNextR3)
-                pCur = pCur->pNextR3;
-            pCur->pNextR3 = pItems;
+            if (ASMAtomicCmpXchgPtr((void * volatile *)&pQueue->pPendingR3, pItems, NULL))
+                break;
+            PPDMQUEUEITEMCORE pPending = (PPDMQUEUEITEMCORE)ASMAtomicXchgPtr((void * volatile *)&pQueue->pPendingR3, NULL);
+            if (pPending)
+            {
+                pCur = pPending;
+                while (pCur->pNextR3)
+                    pCur = pCur->pNextR3;
+                pCur->pNextR3 = pItems;
+                pItems = pPending;
+            }
         }
+
+        STAM_REL_COUNTER_INC(&pQueue->StatFlushLeftovers);
+        STAM_PROFILE_STOP(&pQueue->StatFlushPrf,p);
         return false;
     }
 
+    STAM_PROFILE_STOP(&pQueue->StatFlushPrf,p);
     return true;
 }
 
@@ -797,6 +870,9 @@ VMMR3DECL(void) PDMR3QueueFlushWorker(PVM pVM, PPDMQUEUE pQueue)
     Assert(pVM->pdm.s.pQueueFlushR0 || pVM->pdm.s.pQueueFlushRC || pQueue);
     VM_ASSERT_EMT(pVM);
 
+    /** @todo This will clash with PDMR3QueueFlushAll (guest SMP)! */
+    Assert(!(pVM->pdm.s.fQueueFlushing & PDM_QUEUE_FLUSH_FLAG_ACTIVE));
+
     /*
      * Flush the queue.
      */
@@ -819,7 +895,7 @@ VMMR3DECL(void) PDMR3QueueFlushWorker(PVM pVM, PPDMQUEUE pQueue)
          * Recalc the FF (for the queues using force action).
          */
         VM_FF_CLEAR(pVM, VM_FF_PDM_QUEUES);
-        for (pQueue = pVM->pdm.s.pQueuesForced; pQueue; pQueue = pQueue->pNext)
+        for (pQueue = pVM->pUVM->pdm.s.pQueuesForced; pQueue; pQueue = pQueue->pNext)
             if (    pQueue->pPendingRC
                 ||  pQueue->pPendingR0
                 ||  pQueue->pPendingR3)
@@ -853,6 +929,7 @@ DECLINLINE(void) pdmR3QueueFree(PPDMQUEUE pQueue, PPDMQUEUEITEMCORE pItem)
 
     if (!ASMAtomicCmpXchgU32(&pQueue->iFreeHead, iNext, i))
         AssertMsgFailed(("huh? i=%d iNext=%d iFreeHead=%d iFreeTail=%d\n", i, iNext, pQueue->iFreeHead, pQueue->iFreeTail));
+    STAM_STATS({ ASMAtomicDecU32(&pQueue->cStatPending); });
 }
 
 
