@@ -1,10 +1,10 @@
-/* $Id: PGMMap.cpp $ */
+/* $Id: PGMMap.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * PGM - Page Manager, Guest Context Mappings.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -28,6 +24,7 @@
 #include <VBox/pgm.h>
 #include "PGMInternal.h"
 #include <VBox/vm.h>
+#include "PGMInline.h"
 
 #include <VBox/log.h>
 #include <VBox/err.h>
@@ -73,21 +70,12 @@ VMMR3DECL(int) PGMR3MapPT(PVM pVM, RTGCPTR GCPtr, uint32_t cb, uint32_t fFlags,
     }
     cb = RT_ALIGN_32(cb, _4M);
     RTGCPTR GCPtrLast = GCPtr + cb - 1;
-    if (GCPtrLast < GCPtr)
-    {
-        AssertMsgFailed(("Range wraps! GCPtr=%x GCPtrLast=%x\n", GCPtr, GCPtrLast));
-        return VERR_INVALID_PARAMETER;
-    }
-    if (pVM->pgm.s.fMappingsFixed)
-    {
-        AssertMsgFailed(("Mappings are fixed! It's not possible to add new mappings at this time!\n"));
-        return VERR_PGM_MAPPINGS_FIXED;
-    }
-    if (!pfnRelocate)
-    {
-        AssertMsgFailed(("Callback is required\n"));
-        return VERR_INVALID_PARAMETER;
-    }
+
+    AssertMsgReturn(GCPtrLast >= GCPtr, ("Range wraps! GCPtr=%x GCPtrLast=%x\n", GCPtr, GCPtrLast),
+                    VERR_INVALID_PARAMETER);
+    AssertMsgReturn(!pVM->pgm.s.fMappingsFixed, ("Mappings are fixed! It's not possible to add new mappings at this time!\n"),
+                    VERR_PGM_MAPPINGS_FIXED);
+    AssertPtrReturn(pfnRelocate, VERR_INVALID_PARAMETER);
 
     /*
      * Find list location.
@@ -141,9 +129,9 @@ VMMR3DECL(int) PGMR3MapPT(PVM pVM, RTGCPTR GCPtr, uint32_t cb, uint32_t fFlags,
     pNew->GCPtr         = GCPtr;
     pNew->GCPtrLast     = GCPtrLast;
     pNew->cb            = cb;
-    pNew->pszDesc       = pszDesc;
     pNew->pfnRelocate   = pfnRelocate;
     pNew->pvUser        = pvUser;
+    pNew->pszDesc       = pszDesc;
     pNew->cPTs          = cPTs;
 
     /*
@@ -213,7 +201,11 @@ VMMR3DECL(int) PGMR3MapPT(PVM pVM, RTGCPTR GCPtr, uint32_t cb, uint32_t fFlags,
         pVM->pgm.s.pMappingsR0 = MMHyperR3ToR0(pVM, pNew);
     }
 
-    VM_FF_SET(pVM, VM_FF_PGM_SYNC_CR3);
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
+    {
+        PVMCPU pVCpu = &pVM->aCpus[i];
+        VMCPU_FF_SET(pVCpu, VMCPU_FF_PGM_SYNC_CR3);
+    }
     return VINF_SUCCESS;
 }
 
@@ -263,10 +255,15 @@ VMMR3DECL(int)  PGMR3UnmapPT(PVM pVM, RTGCPTR GCPtr)
              * and free the page tables and node memory.
              */
             MMHyperFree(pVM, pCur->aPTs[0].pPTR3);
-            pgmR3MapClearPDEs(pVM, pCur, pCur->GCPtr >> X86_PD_SHIFT);
+            if (pCur->GCPtr != NIL_RTGCPTR)
+                pgmR3MapClearPDEs(pVM, pCur, pCur->GCPtr >> X86_PD_SHIFT);
             MMHyperFree(pVM, pCur);
 
-            VM_FF_SET(pVM, VM_FF_PGM_SYNC_CR3);
+            for (VMCPUID i = 0; i < pVM->cCpus; i++)
+            {
+                PVMCPU pVCpu = &pVM->aCpus[i];
+                VMCPU_FF_SET(pVCpu, VMCPU_FF_PGM_SYNC_CR3);
+            }
             return VINF_SUCCESS;
         }
 
@@ -405,12 +402,14 @@ VMMR3DECL(int) PGMR3FinalizeMappings(PVM pVM)
     /*
      * Loop until all mappings have been finalized.
      */
-    /*unsigned    iPDNext = UINT32_C(0xc0000000) >> X86_PD_SHIFT;*/ /* makes CSAM/PATM freak out booting linux. :-/ */
 #if 0
+    unsigned    iPDNext = UINT32_C(0xc0000000) >> X86_PD_SHIFT; /* makes CSAM/PATM freak out booting linux. :-/ */
+#elif 0
     unsigned    iPDNext = MM_HYPER_AREA_ADDRESS >> X86_PD_SHIFT;
 #else
     unsigned    iPDNext = 1 << X86_PD_SHIFT; /* no hint, map them from the top. */
 #endif
+
     PPGMMAPPING pCur;
     do
     {
@@ -495,7 +494,7 @@ VMMR3DECL(int) PGMR3MappingsSize(PVM pVM, uint32_t *pcb)
 
 
 /**
- * Fixes the guest context mappings in a range reserved from the Guest OS.
+ * Fixates the guest context mappings in a range reserved from the Guest OS.
  *
  * @returns VBox status code.
  * @param   pVM         The VM.
@@ -504,32 +503,55 @@ VMMR3DECL(int) PGMR3MappingsSize(PVM pVM, uint32_t *pcb)
  */
 VMMR3DECL(int) PGMR3MappingsFix(PVM pVM, RTGCPTR GCPtrBase, uint32_t cb)
 {
-    Log(("PGMR3MappingsFix: GCPtrBase=%#x cb=%#x\n", GCPtrBase, cb));
-
-    /* Ignore the additions mapping fix call in VT-x/AMD-V. */
-    if (    pVM->pgm.s.fMappingsFixed
-        &&  HWACCMR3IsActive(pVM))
-        return VINF_SUCCESS;
+    Log(("PGMR3MappingsFix: GCPtrBase=%RGv cb=%#x (fMappingsFixed=%RTbool fMappingsDisabled=%RTbool)\n",
+         GCPtrBase, cb, pVM->pgm.s.fMappingsFixed, pVM->pgm.s.fMappingsDisabled));
 
     /*
-     * This is all or nothing at all. So, a tiny bit of paranoia first.
+     * Ignore the additions mapping fix call if disabled.
      */
-    if (GCPtrBase & X86_PAGE_4M_OFFSET_MASK)
+    if (!pgmMapAreMappingsEnabled(&pVM->pgm.s))
     {
-        AssertMsgFailed(("GCPtrBase (%#x) has to be aligned on a 4MB address!\n", GCPtrBase));
-        return VERR_INVALID_PARAMETER;
-    }
-    if (!cb || (cb & X86_PAGE_4M_OFFSET_MASK))
-    {
-        AssertMsgFailed(("cb (%#x) is 0 or not aligned on a 4MB address!\n", cb));
-        return VERR_INVALID_PARAMETER;
+        Assert(HWACCMIsEnabled(pVM));
+        return VINF_SUCCESS;
     }
 
+    /*
+     * Only applies to VCPU 0 as we don't support SMP guests with raw mode.
+     */
+    Assert(pVM->cCpus == 1);
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     /*
      * Before we do anything we'll do a forced PD sync to try make sure any
      * pending relocations because of these mappings have been resolved.
      */
-    PGMSyncCR3(pVM, CPUMGetGuestCR0(pVM), CPUMGetGuestCR3(pVM), CPUMGetGuestCR4(pVM), true);
+    PGMSyncCR3(pVCpu, CPUMGetGuestCR0(pVCpu), CPUMGetGuestCR3(pVCpu), CPUMGetGuestCR4(pVCpu), true);
+
+    return pgmR3MappingsFixInternal(pVM, GCPtrBase, cb);
+}
+
+
+/**
+ * Internal worker for PGMR3MappingsFix and pgmR3Load.
+ *
+ * (This does not perform a SyncCR3 before the fixation like PGMR3MappingsFix.)
+ *
+ * @returns VBox status code.
+ * @param   pVM         The VM.
+ * @param   GCPtrBase   The address of the reserved range of guest memory.
+ * @param   cb          The size of the range starting at GCPtrBase.
+ */
+int pgmR3MappingsFixInternal(PVM pVM, RTGCPTR GCPtrBase, uint32_t cb)
+{
+    /*
+     * Check input arguments and pre-conditions.
+     */
+    AssertMsgReturn(!(GCPtrBase & X86_PAGE_4M_OFFSET_MASK), ("GCPtrBase (%#x) has to be aligned on a 4MB address!\n", GCPtrBase),
+                    VERR_INVALID_PARAMETER);
+    AssertMsgReturn(cb && !(cb & X86_PAGE_4M_OFFSET_MASK), ("cb (%#x) is 0 or not aligned on a 4MB address!\n", cb),
+                    VERR_INVALID_PARAMETER);
+    AssertReturn(pgmMapAreMappingsEnabled(&pVM->pgm.s), VERR_INTERNAL_ERROR_3);
+    AssertReturn(pVM->cCpus == 1, VERR_INTERNAL_ERROR_4);
 
     /*
      * Check that it's not conflicting with a core code mapping in the intermediate page table.
@@ -560,10 +582,11 @@ VMMR3DECL(int) PGMR3MappingsFix(PVM pVM, RTGCPTR GCPtrBase, uint32_t cb)
     /*
      * In PAE / PAE mode, make sure we don't cross page directories.
      */
-    if (    (   pVM->pgm.s.enmGuestMode  == PGMMODE_PAE
-             || pVM->pgm.s.enmGuestMode  == PGMMODE_PAE_NX)
-        &&  (   pVM->pgm.s.enmShadowMode == PGMMODE_PAE
-             || pVM->pgm.s.enmShadowMode == PGMMODE_PAE_NX))
+    PVMCPU pVCpu = &pVM->aCpus[0];
+    if (    (   pVCpu->pgm.s.enmGuestMode  == PGMMODE_PAE
+             || pVCpu->pgm.s.enmGuestMode  == PGMMODE_PAE_NX)
+        &&  (   pVCpu->pgm.s.enmShadowMode == PGMMODE_PAE
+             || pVCpu->pgm.s.enmShadowMode == PGMMODE_PAE_NX))
     {
         unsigned iPdptBase = GCPtrBase >> X86_PDPT_SHIFT;
         unsigned iPdptLast = (GCPtrBase + cb - 1) >> X86_PDPT_SHIFT;
@@ -605,14 +628,14 @@ VMMR3DECL(int) PGMR3MappingsFix(PVM pVM, RTGCPTR GCPtrBase, uint32_t cb)
     pCur = pVM->pgm.s.pMappingsR3;
     while (pCur)
     {
-        unsigned iPDOld = pCur->GCPtr >> X86_PD_SHIFT;
-        iPDNew = GCPtrCur >> X86_PD_SHIFT;
+        RTGCPTR const GCPtrOld = pCur->GCPtr;
 
         /*
          * Relocate the page table(s).
          */
-        pgmR3MapClearPDEs(pVM, pCur, iPDOld);
-        pgmR3MapSetPDEs(pVM, pCur, iPDNew);
+        if (pCur->GCPtr != NIL_RTGCPTR)
+            pgmR3MapClearPDEs(pVM, pCur, GCPtrOld >> X86_PD_SHIFT);
+        pgmR3MapSetPDEs(pVM, pCur, GCPtrCur >> X86_PD_SHIFT);
 
         /*
          * Update the entry.
@@ -623,7 +646,7 @@ VMMR3DECL(int) PGMR3MappingsFix(PVM pVM, RTGCPTR GCPtrBase, uint32_t cb)
         /*
          * Callback to execute the relocation.
          */
-        pCur->pfnRelocate(pVM, iPDOld << X86_PD_SHIFT, iPDNew << X86_PD_SHIFT, PGMRELOCATECALL_RELOCATE, pCur->pvUser);
+        pCur->pfnRelocate(pVM, GCPtrOld, GCPtrCur, PGMRELOCATECALL_RELOCATE, pCur->pvUser);
 
         /*
          * Advance.
@@ -633,71 +656,113 @@ VMMR3DECL(int) PGMR3MappingsFix(PVM pVM, RTGCPTR GCPtrBase, uint32_t cb)
     }
 
     /*
-     * Mark the mappings as fixed and return.
+     * Mark the mappings as fixed at this new location and return.
      */
-    pVM->pgm.s.fMappingsFixed    = true;
-    pVM->pgm.s.GCPtrMappingFixed = GCPtrBase;
-    pVM->pgm.s.cbMappingFixed    = cb;
-    pVM->pgm.s.fSyncFlags       &= ~PGM_SYNC_MONITOR_CR3;
-    VM_FF_SET(pVM, VM_FF_PGM_SYNC_CR3);
+    pVM->pgm.s.fMappingsFixed           = true;
+    pVM->pgm.s.fMappingsFixedRestored   = false;
+    pVM->pgm.s.GCPtrMappingFixed        = GCPtrBase;
+    pVM->pgm.s.cbMappingFixed           = cb;
+
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
+    {
+        pVM->aCpus[idCpu].pgm.s.fSyncFlags &= ~PGM_SYNC_MONITOR_CR3;
+        VMCPU_FF_SET(&pVM->aCpus[idCpu], VMCPU_FF_PGM_SYNC_CR3);
+    }
     return VINF_SUCCESS;
 }
 
+
 /**
- * Disable the hypervisor mappings in the shadow page tables (doesn't touch the intermediate table!)
+ * Interface for disabling the guest mappings when switching to HWACCM mode
+ * during VM creation and VM reset.
+ *
+ * (This doesn't touch the intermediate table!)
  *
  * @returns VBox status code.
  * @param   pVM         The VM.
  */
 VMMR3DECL(int) PGMR3MappingsDisable(PVM pVM)
 {
-    uint32_t cb;
-    int rc = PGMR3MappingsSize(pVM, &cb);
-    AssertRCReturn(rc, rc);
-
-    rc = pgmMapDeactivateCR3(pVM, pVM->pgm.s.pShwPageCR3R3);
-    AssertRCReturn(rc, rc);
+    AssertReturn(!pVM->pgm.s.fMappingsFixed,            VERR_INTERNAL_ERROR_4);
+    AssertReturn(!pVM->pgm.s.fMappingsFixedRestored,    VERR_INTERNAL_ERROR_4);
+    if (pVM->pgm.s.fMappingsDisabled)
+        return VINF_SUCCESS;
 
     /*
-     * Mark the mappings as fixed (using fake values) and disabled.
+     * Deactivate (only applies to Virtual CPU #0).
      */
-    pVM->pgm.s.fDisableMappings  = true;
-    pVM->pgm.s.fMappingsFixed    = true;
-    pVM->pgm.s.GCPtrMappingFixed = MM_HYPER_AREA_ADDRESS;
-    pVM->pgm.s.cbMappingFixed    = cb;
-    pVM->pgm.s.fSyncFlags       &= ~PGM_SYNC_MONITOR_CR3;
-    VM_FF_SET(pVM, VM_FF_PGM_SYNC_CR3);
+    if (pVM->aCpus[0].pgm.s.pShwPageCR3R3)
+    {
+        pgmLock(pVM);                           /* to avoid assertions */
+        int rc = pgmMapDeactivateCR3(pVM, pVM->aCpus[0].pgm.s.pShwPageCR3R3);
+        pgmUnlock(pVM);
+        AssertRCReturn(rc, rc);
+    }
+
+    /*
+     * Mark the mappings as disabled and trigger a CR3 re-sync.
+     */
+    pVM->pgm.s.fMappingsDisabled = true;
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
+    {
+        pVM->aCpus[idCpu].pgm.s.fSyncFlags &= ~PGM_SYNC_MONITOR_CR3;
+        VMCPU_FF_SET(&pVM->aCpus[idCpu], VMCPU_FF_PGM_SYNC_CR3);
+    }
     return VINF_SUCCESS;
 }
 
 
 /**
  * Unfixes the mappings.
- * After calling this function mapping conflict detection will be enabled.
+ *
+ * Unless PGMR3MappingsDisable is in effect, mapping conflict detection will be
+ * enabled after this call.  If the mappings are fixed, a full CR3 resync will
+ * take place afterwards.
  *
  * @returns VBox status code.
  * @param   pVM         The VM.
  */
 VMMR3DECL(int) PGMR3MappingsUnfix(PVM pVM)
 {
-    Log(("PGMR3MappingsUnfix: fMappingsFixed=%d\n", pVM->pgm.s.fMappingsFixed));
+    Log(("PGMR3MappingsUnfix: fMappingsFixed=%RTbool fMappingsDisabled=%RTbool\n", pVM->pgm.s.fMappingsFixed, pVM->pgm.s.fMappingsDisabled));
+    if (   pgmMapAreMappingsEnabled(&pVM->pgm.s)
+        && (    pVM->pgm.s.fMappingsFixed
+            ||  pVM->pgm.s.fMappingsFixedRestored)
+       )
+    {
+        bool const fResyncCR3 = pVM->pgm.s.fMappingsFixed;
 
-    /* Ignore in VT-x/AMD-V mode. */
-    if (HWACCMR3IsActive(pVM))
-        return VINF_SUCCESS;
+        pVM->pgm.s.fMappingsFixed           = false;
+        pVM->pgm.s.fMappingsFixedRestored   = false;
+        pVM->pgm.s.GCPtrMappingFixed        = 0;
+        pVM->pgm.s.cbMappingFixed           = 0;
 
-    pVM->pgm.s.fMappingsFixed    = false;
-    pVM->pgm.s.GCPtrMappingFixed = 0;
-    pVM->pgm.s.cbMappingFixed    = 0;
-    VM_FF_SET(pVM, VM_FF_PGM_SYNC_CR3);
+        if (fResyncCR3)
+            for (VMCPUID i = 0; i < pVM->cCpus; i++)
+                VMCPU_FF_SET(&pVM->aCpus[i], VMCPU_FF_PGM_SYNC_CR3);
+    }
     return VINF_SUCCESS;
 }
 
 
+/**
+ * Checks if the mappings needs re-fixing after a restore.
+ *
+ * @returns true if they need, false if not.
+ * @param   pVM                 The VM handle.
+ */
+VMMR3DECL(bool) PGMR3MappingsNeedReFixing(PVM pVM)
+{
+    VM_ASSERT_VALID_EXT_RETURN(pVM, false);
+    return pVM->pgm.s.fMappingsFixedRestored;
+}
+
+
 /**
  * Map pages into the intermediate context (switcher code).
- * These pages are mapped at both the give virtual address and at
- * the physical address (for identity mapping).
+ *
+ * These pages are mapped at both the give virtual address and at the physical
+ * address (for identity mapping).
  *
  * @returns VBox status code.
  * @param   pVM         The virtual machine.
@@ -910,9 +975,11 @@ static void pgmR3MapIntermediateDoOne(PVM pVM, uintptr_t uAddress, RTHCPHYS HCPh
  */
 static void pgmR3MapClearPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iOldPDE)
 {
-    unsigned i = pMap->cPTs;
+    unsigned i     = pMap->cPTs;
+    PVMCPU   pVCpu = VMMGetCpu(pVM);
+    pgmLock(pVM);                           /* to avoid assertions */
 
-    pgmMapClearShadowPDEs(pVM, pVM->pgm.s.CTX_SUFF(pShwPageCR3), pMap, iOldPDE, false /*fDeactivateCR3*/);
+    pgmMapClearShadowPDEs(pVM, pVCpu->pgm.s.CTX_SUFF(pShwPageCR3), pMap, iOldPDE, false /*fDeactivateCR3*/);
 
     iOldPDE += i;
     while (i-- > 0)
@@ -923,6 +990,7 @@ static void pgmR3MapClearPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iOldPDE)
          * 32-bit.
          */
         pVM->pgm.s.pInterPD->a[iOldPDE].u        = 0;
+
         /*
          * PAE.
          */
@@ -933,8 +1001,11 @@ static void pgmR3MapClearPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iOldPDE)
         AssertFatal(iPDE < 512);
         pVM->pgm.s.apInterPaePDs[iPD]->a[iPDE].u = 0;
     }
+
+    pgmUnlock(pVM);
 }
 
+
 /**
  * Sets all PDEs involved with the mapping in the shadow and intermediate page tables.
  *
@@ -944,9 +1015,11 @@ static void pgmR3MapClearPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iOldPDE)
  */
 static void pgmR3MapSetPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iNewPDE)
 {
-    PPGM pPGM = &pVM->pgm.s;
+    PPGM   pPGM  = &pVM->pgm.s;
+    PVMCPU pVCpu = VMMGetCpu(pVM);
+    pgmLock(pVM);                           /* to avoid assertions */
 
-    Assert(!pgmMapAreMappingsEnabled(&pVM->pgm.s) || PGMGetGuestMode(pVM) <= PGMMODE_PAE_NX);
+    Assert(!pgmMapAreMappingsEnabled(&pVM->pgm.s) || PGMGetGuestMode(pVCpu) <= PGMMODE_PAE_NX);
 
     pgmMapSetShadowPDEs(pVM, pMap, iNewPDE);
 
@@ -980,29 +1053,33 @@ static void pgmR3MapSetPDEs(PVM pVM, PPGMMAPPING pMap, unsigned iNewPDE)
         PdePae1.u = PGM_PDFLAGS_MAPPING | X86_PDE_P | X86_PDE_A | X86_PDE_RW | X86_PDE_US | pMap->aPTs[i].HCPhysPaePT1;
         pPGM->apInterPaePDs[iPD]->a[iPDE] = PdePae1;
     }
+
+    pgmUnlock(pVM);
 }
 
+
 /**
  * Relocates a mapping to a new address.
  *
  * @param   pVM                 VM handle.
  * @param   pMapping            The mapping to relocate.
  * @param   GCPtrOldMapping     The address of the start of the old mapping.
+ *                              NIL_RTGCPTR if not currently mapped.
  * @param   GCPtrNewMapping     The address of the start of the new mapping.
  */
-void pgmR3MapRelocate(PVM pVM, PPGMMAPPING pMapping, RTGCPTR GCPtrOldMapping, RTGCPTR GCPtrNewMapping)
+static void pgmR3MapRelocate(PVM pVM, PPGMMAPPING pMapping, RTGCPTR GCPtrOldMapping, RTGCPTR GCPtrNewMapping)
 {
-    unsigned iPDOld = GCPtrOldMapping >> X86_PD_SHIFT;
-    unsigned iPDNew = GCPtrNewMapping >> X86_PD_SHIFT;
-
     Log(("PGM: Relocating %s from %RGv to %RGv\n", pMapping->pszDesc, GCPtrOldMapping, GCPtrNewMapping));
-    AssertMsg(((unsigned)iPDOld << X86_PD_SHIFT) == pMapping->GCPtr, ("%RGv vs %RGv\n", (RTGCPTR)((unsigned)iPDOld << X86_PD_SHIFT), pMapping->GCPtr));
+    AssertMsg(GCPtrOldMapping == pMapping->GCPtr, ("%RGv vs %RGv\n", GCPtrOldMapping, pMapping->GCPtr));
+    AssertMsg((GCPtrOldMapping >> X86_PD_SHIFT) < X86_PG_ENTRIES, ("%RGv\n", GCPtrOldMapping));
+    AssertMsg((GCPtrNewMapping >> X86_PD_SHIFT) < X86_PG_ENTRIES, ("%RGv\n", GCPtrOldMapping));
 
     /*
      * Relocate the page table(s).
      */
-    pgmR3MapClearPDEs(pVM, pMapping, iPDOld);
-    pgmR3MapSetPDEs(pVM, pMapping, iPDNew);
+    if (GCPtrOldMapping != NIL_RTGCPTR)
+        pgmR3MapClearPDEs(pVM, pMapping, GCPtrOldMapping >> X86_PD_SHIFT);
+    pgmR3MapSetPDEs(pVM, pMapping, GCPtrNewMapping >> X86_PD_SHIFT);
 
     /*
      * Update and resort the mapping list.
@@ -1020,7 +1097,7 @@ void pgmR3MapRelocate(PVM pVM, PPGMMAPPING pMapping, RTGCPTR GCPtrOldMapping, RT
     Assert(pCur);
 
     /* Find mapping which >= than pMapping. */
-    RTGCPTR     GCPtrNew = iPDNew << X86_PD_SHIFT;
+    RTGCPTR     GCPtrNew = GCPtrNewMapping;
     PPGMMAPPING pPrev = NULL;
     pCur = pVM->pgm.s.pMappingsR3;
     while (pCur && pCur->GCPtr < GCPtrNew)
@@ -1079,7 +1156,7 @@ void pgmR3MapRelocate(PVM pVM, PPGMMAPPING pMapping, RTGCPTR GCPtrOldMapping, RT
     /*
      * Callback to execute the relocation.
      */
-    pMapping->pfnRelocate(pVM, iPDOld << X86_PD_SHIFT, iPDNew << X86_PD_SHIFT, PGMRELOCATECALL_RELOCATE, pMapping->pvUser);
+    pMapping->pfnRelocate(pVM, GCPtrOldMapping, GCPtrNewMapping, PGMRELOCATECALL_RELOCATE, pMapping->pvUser);
 }
 
 
@@ -1116,6 +1193,9 @@ int pgmR3SyncPTResolveConflict(PVM pVM, PPGMMAPPING pMapping, PX86PD pPDSrc, RTG
     STAM_REL_COUNTER_INC(&pVM->pgm.s.cRelocations);
     STAM_PROFILE_START(&pVM->pgm.s.StatR3ResolveConflict, a);
 
+    /* Raw mode only which implies one VCPU. */
+    Assert(pVM->cCpus == 1);
+
     pMapping->aGCPtrConflicts[pMapping->cConflicts & (PGMMAPPING_CONFLICT_MAX-1)] = GCPtrOldMapping;
     pMapping->cConflicts++;
 
@@ -1159,7 +1239,7 @@ int pgmR3SyncPTResolveConflict(PVM pVM, PPGMMAPPING pMapping, PX86PD pPDSrc, RTG
         /*
          * Ask for the mapping.
          */
-        RTGCPTR GCPtrNewMapping = iPDNew << X86_PD_SHIFT;
+        RTGCPTR GCPtrNewMapping = (RTGCPTR32)iPDNew << X86_PD_SHIFT;
 
         if (pMapping->pfnRelocate(pVM, GCPtrOldMapping, GCPtrNewMapping, PGMRELOCATECALL_SUGGEST, pMapping->pvUser))
         {
@@ -1189,13 +1269,17 @@ int pgmR3SyncPTResolveConflictPAE(PVM pVM, PPGMMAPPING pMapping, RTGCPTR GCPtrOl
     STAM_REL_COUNTER_INC(&pVM->pgm.s.cRelocations);
     STAM_PROFILE_START(&pVM->pgm.s.StatR3ResolveConflict, a);
 
+    /* Raw mode only which implies one VCPU. */
+    Assert(pVM->cCpus == 1);
+    PVMCPU pVCpu = VMMGetCpu(pVM);
+
     pMapping->aGCPtrConflicts[pMapping->cConflicts & (PGMMAPPING_CONFLICT_MAX-1)] = GCPtrOldMapping;
     pMapping->cConflicts++;
 
     for (int iPDPTE = X86_PG_PAE_PDPE_ENTRIES - 1; iPDPTE >= 0; iPDPTE--)
     {
         unsigned  iPDSrc;
-        PX86PDPAE pPDSrc = pgmGstGetPaePDPtr(&pVM->pgm.s, (RTGCPTR32)iPDPTE << X86_PDPT_SHIFT, &iPDSrc, NULL);
+        PX86PDPAE pPDSrc = pgmGstGetPaePDPtr(&pVCpu->pgm.s, (RTGCPTR32)iPDPTE << X86_PDPT_SHIFT, &iPDSrc, NULL);
 
         /*
          * Scan for free page directory entries.
@@ -1243,7 +1327,7 @@ int pgmR3SyncPTResolveConflictPAE(PVM pVM, PPGMMAPPING pMapping, RTGCPTR GCPtrOl
             /*
              * Ask for the mapping.
              */
-            RTGCPTR GCPtrNewMapping = ((RTGCPTR32)iPDPTE << X86_PDPT_SHIFT) + (iPDNew << X86_PD_PAE_SHIFT);
+            RTGCPTR GCPtrNewMapping = ((RTGCPTR32)iPDPTE << X86_PDPT_SHIFT) + ((RTGCPTR32)iPDNew << X86_PD_PAE_SHIFT);
 
             if (pMapping->pfnRelocate(pVM, GCPtrOldMapping, GCPtrNewMapping, PGMRELOCATECALL_SUGGEST, pMapping->pvUser))
             {
@@ -1353,11 +1437,32 @@ VMMR3DECL(int) PGMR3MapRead(PVM pVM, void *pvDst, RTGCPTR GCPtrSrc, size_t cb)
  */
 DECLCALLBACK(void) pgmR3MapInfo(PVM pVM, PCDBGFINFOHLP pHlp, const char *pszArgs)
 {
-    pHlp->pfnPrintf(pHlp, pVM->pgm.s.fMappingsFixed
-                    ? "\nThe mappings are FIXED.\n"
-                    : "\nThe mappings are FLOATING.\n");
+    if (pVM->pgm.s.fMappingsDisabled)
+        pHlp->pfnPrintf(pHlp, "\nThe mappings are DISABLED.\n");
+    else if (pVM->pgm.s.fMappingsFixed)
+        pHlp->pfnPrintf(pHlp, "\nThe mappings are FIXED: %RGv-%RGv\n",
+                        pVM->pgm.s.GCPtrMappingFixed, pVM->pgm.s.GCPtrMappingFixed + pVM->pgm.s.cbMappingFixed - 1);
+    else if (pVM->pgm.s.fMappingsFixedRestored)
+        pHlp->pfnPrintf(pHlp, "\nThe mappings are FLOATING-RESTORED-FIXED: %RGv-%RGv\n",
+                        pVM->pgm.s.GCPtrMappingFixed, pVM->pgm.s.GCPtrMappingFixed + pVM->pgm.s.cbMappingFixed - 1);
+    else
+        pHlp->pfnPrintf(pHlp, "\nThe mappings are FLOATING.\n");
+
     PPGMMAPPING pCur;
     for (pCur = pVM->pgm.s.pMappingsR3; pCur; pCur = pCur->pNextR3)
+    {
         pHlp->pfnPrintf(pHlp, "%RGv - %RGv  %s\n", pCur->GCPtr, pCur->GCPtrLast, pCur->pszDesc);
+        if (pCur->cConflicts > 0)
+        {
+            pHlp->pfnPrintf(pHlp, "  %u conflict%s: ", pCur->cConflicts, pCur->cConflicts == 1 ? "" : "s");
+            uint32_t cLeft = RT_MIN(pCur->cConflicts, RT_ELEMENTS(pCur->aGCPtrConflicts));
+            uint32_t i     = pCur->cConflicts;
+            while (cLeft-- > 0)
+            {
+                i = (i - 1) & (PGMMAPPING_CONFLICT_MAX - 1);
+                pHlp->pfnPrintf(pHlp, cLeft ? "%RGv, " : "%RGv\n", pCur->aGCPtrConflicts[i]);
+            }
+        }
+    }
 }
 
