@@ -1,10 +1,10 @@
-/* $Id: TRPM.cpp $ */
+/* $Id: TRPM.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * TRPM - The Trap Monitor.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /** @page pg_trpm   TRPM - The Trap Monitor
@@ -36,7 +32,7 @@
  * When executing in the raw-mode context, TRPM will be managing the IDT and
  * processing all traps and interrupts.  It will also monitor the guest IDT
  * because CSAM wishes to know about changes to it (trap/interrupt/syscall
- * handler patching) and TRPM needs to keep the #\BP gate in sync (ring-3
+ * handler patching) and TRPM needs to keep the \#BP gate in sync (ring-3
  * considerations).  See TRPMR3SyncIDT and CSAMR3CheckGates.
  *
  * External interrupts will be forwarded to the host context by the quickest
@@ -430,14 +426,15 @@ static VBOXIDTE_GENERIC     g_aIdt[256] =
 #define TRPM_TRACK_SHADOW_IDT_CHANGES
 
 /** TRPM saved state version. */
-#define TRPM_SAVED_STATE_VERSION    8
+#define TRPM_SAVED_STATE_VERSION        9
+#define TRPM_SAVED_STATE_VERSION_UNI    8   /* SMP support bumped the version */
 
 
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
 static DECLCALLBACK(int) trpmR3Save(PVM pVM, PSSMHANDLE pSSM);
-static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version);
+static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass);
 static DECLCALLBACK(int) trpmR3GuestIDTWriteHandler(PVM pVM, RTGCPTR GCPtr, void *pvPtr, void *pvBuf, size_t cbBuf, PGMACCESSTYPE enmAccessType, void *pvUser);
 
 
@@ -463,7 +460,17 @@ VMMR3DECL(int) TRPMR3Init(PVM pVM)
      * Initialize members.
      */
     pVM->trpm.s.offVM              = RT_OFFSETOF(VM, trpm);
-    pVM->trpm.s.uActiveVector      = ~0;
+    pVM->trpm.s.offTRPMCPU         = RT_OFFSETOF(VM, aCpus[0].trpm) - RT_OFFSETOF(VM, trpm);
+
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
+    {
+        PVMCPU pVCpu = &pVM->aCpus[i];
+
+        pVCpu->trpm.s.offVM         = RT_OFFSETOF(VM, aCpus[i].trpm);
+        pVCpu->trpm.s.offVMCpu      = RT_OFFSETOF(VMCPU, trpm);
+        pVCpu->trpm.s.uActiveVector = ~0;
+    }
+
     pVM->trpm.s.GuestIdtr.pIdt     = RTRCPTR_MAX;
     pVM->trpm.s.pvMonShwIdtRC            = RTRCPTR_MAX;
     pVM->trpm.s.fDisableMonitoring = false;
@@ -496,6 +503,7 @@ VMMR3DECL(int) TRPMR3Init(PVM pVM)
      * Register the saved state data unit.
      */
     int rc = SSMR3RegisterInternal(pVM, "trpm", 1, TRPM_SAVED_STATE_VERSION, sizeof(TRPM),
+                                   NULL, NULL, NULL,
                                    NULL, trpmR3Save, NULL,
                                    NULL, trpmR3Load, NULL);
     if (RT_FAILURE(rc))
@@ -553,7 +561,8 @@ VMMR3DECL(int) TRPMR3Init(PVM pVM)
     /*
      * Default action when entering raw mode for the first time
      */
-    VM_FF_SET(pVM, VM_FF_TRPM_SYNC_IDT);
+    PVMCPU pVCpu = &pVM->aCpus[0];  /* raw mode implies on VCPU */
+    VMCPU_FF_SET(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
     return 0;
 }
 
@@ -569,6 +578,9 @@ VMMR3DECL(int) TRPMR3Init(PVM pVM)
  */
 VMMR3DECL(void) TRPMR3Relocate(PVM pVM, RTGCINTPTR offDelta)
 {
+    /* Only applies to raw mode which supports only 1 VCPU. */
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     LogFlow(("TRPMR3Relocate\n"));
     /*
      * Get the trap handler addresses.
@@ -577,9 +589,9 @@ VMMR3DECL(void) TRPMR3Relocate(PVM pVM, RTGCINTPTR offDelta)
      * would make init order impossible if we should assert the presence of these
      * exports in TRPMR3Init().
      */
-    RTRCPTR aRCPtrs[TRPM_HANDLER_MAX] = {0};
-    int rc;
-    rc = PDMR3LdrGetSymbolRC(pVM, VMMGC_MAIN_MODULE_NAME, "TRPMGCHandlerInterupt", &aRCPtrs[TRPM_HANDLER_INT]);
+    RTRCPTR aRCPtrs[TRPM_HANDLER_MAX];
+    RT_ZERO(aRCPtrs);
+    int rc = PDMR3LdrGetSymbolRC(pVM, VMMGC_MAIN_MODULE_NAME, "TRPMGCHandlerInterupt", &aRCPtrs[TRPM_HANDLER_INT]);
     AssertReleaseMsgRC(rc, ("Couldn't find TRPMGCHandlerInterupt in VMMGC.gc!\n"));
 
     rc = PDMR3LdrGetSymbolRC(pVM, VMMGC_MAIN_MODULE_NAME, "TRPMGCHandlerGeneric",  &aRCPtrs[TRPM_HANDLER_TRAP]);
@@ -591,7 +603,7 @@ VMMR3DECL(void) TRPMR3Relocate(PVM pVM, RTGCINTPTR offDelta)
     rc = PDMR3LdrGetSymbolRC(pVM, VMMGC_MAIN_MODULE_NAME, "TRPMGCHandlerTrap12",   &aRCPtrs[TRPM_HANDLER_TRAP_12]);
     AssertReleaseMsgRC(rc, ("Couldn't find TRPMGCHandlerTrap12 in VMMGC.gc!\n"));
 
-    RTSEL SelCS = CPUMGetHyperCS(pVM);
+    RTSEL SelCS = CPUMGetHyperCS(pVCpu);
 
     /*
      * Iterate the idt and set the addresses.
@@ -638,7 +650,7 @@ VMMR3DECL(void) TRPMR3Relocate(PVM pVM, RTGCINTPTR offDelta)
     /*
      * Update IDTR (limit is including!).
      */
-    CPUMSetHyperIDTR(pVM, VM_RC_ADDR(pVM, &pVM->trpm.s.aIdt[0]), sizeof(pVM->trpm.s.aIdt)-1);
+    CPUMSetHyperIDTR(pVCpu, VM_RC_ADDR(pVM, &pVM->trpm.s.aIdt[0]), sizeof(pVM->trpm.s.aIdt)-1);
 
     if (!pVM->trpm.s.fDisableMonitoring)
     {
@@ -666,15 +678,14 @@ VMMR3DECL(void) TRPMR3Relocate(PVM pVM, RTGCINTPTR offDelta)
 
         if (ASMBitTest(&pVM->trpm.s.au32IdtPatched[0], iTrap))
         {
-            PVBOXIDTE   pIdte = &pVM->trpm.s.aIdt[iTrap];
-            RTGCPTR     pHandler = VBOXIDTE_OFFSET(*pIdte);
+            PVBOXIDTE   pIdteCur = &pVM->trpm.s.aIdt[iTrap];
+            RTGCPTR     pHandler = VBOXIDTE_OFFSET(*pIdteCur);
 
             Log(("TRPMR3Relocate: *iGate=%2X Handler %RGv -> %RGv\n", iTrap, pHandler, pHandler + offDelta));
             pHandler += offDelta;
 
-            pIdte->Gen.u16OffsetHigh = pHandler >> 16;
-            pIdte->Gen.u16OffsetLow  = pHandler & 0xFFFF;
-
+            pIdteCur->Gen.u16OffsetHigh = pHandler >> 16;
+            pIdteCur->Gen.u16OffsetLow  = pHandler & 0xFFFF;
         }
     }
 
@@ -698,6 +709,19 @@ VMMR3DECL(int) TRPMR3Term(PVM pVM)
 }
 
 
+/**
+ * Resets a virtual CPU.
+ *
+ * Used by TRPMR3Reset and CPU hot plugging.
+ *
+ * @param   pVCpu               The virtual CPU handle.
+ */
+VMMR3DECL(void) TRPMR3ResetCpu(PVMCPU pVCpu)
+{
+    pVCpu->trpm.s.uActiveVector = ~0;
+}
+
+
 /**
  * The VM is being reset.
  *
@@ -727,7 +751,8 @@ VMMR3DECL(void) TRPMR3Reset(PVM pVM)
     /*
      * Reinitialize other members calling the relocator to get things right.
      */
-    pVM->trpm.s.uActiveVector  = ~0;
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
+        TRPMR3ResetCpu(&pVM->aCpus[i]);
     memcpy(&pVM->trpm.s.aIdt[0], &g_aIdt[0], sizeof(pVM->trpm.s.aIdt));
     memset(pVM->trpm.s.aGuestTrapHandler, 0, sizeof(pVM->trpm.s.aGuestTrapHandler));
     TRPMR3Relocate(pVM, 0);
@@ -735,7 +760,8 @@ VMMR3DECL(void) TRPMR3Reset(PVM pVM)
     /*
      * Default action when entering raw mode for the first time
      */
-    VM_FF_SET(pVM, VM_FF_TRPM_SYNC_IDT);
+    PVMCPU pVCpu = &pVM->aCpus[0];  /* raw mode implies on VCPU */
+    VMCPU_FF_SET(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
 }
 
 
@@ -748,27 +774,28 @@ VMMR3DECL(void) TRPMR3Reset(PVM pVM)
  */
 static DECLCALLBACK(int) trpmR3Save(PVM pVM, PSSMHANDLE pSSM)
 {
+    PTRPM pTrpm = &pVM->trpm.s;
     LogFlow(("trpmR3Save:\n"));
 
     /*
      * Active and saved traps.
      */
-    PTRPM pTrpm = &pVM->trpm.s;
-    SSMR3PutUInt(pSSM,      pTrpm->uActiveVector);
-    SSMR3PutUInt(pSSM,      pTrpm->enmActiveType);
-    SSMR3PutGCUInt(pSSM,    pTrpm->uActiveErrorCode);
-    SSMR3PutGCUIntPtr(pSSM, pTrpm->uActiveCR2);
-    SSMR3PutGCUInt(pSSM,    pTrpm->uSavedVector);
-    SSMR3PutUInt(pSSM,      pTrpm->enmSavedType);
-    SSMR3PutGCUInt(pSSM,    pTrpm->uSavedErrorCode);
-    SSMR3PutGCUIntPtr(pSSM, pTrpm->uSavedCR2);
-    SSMR3PutGCUInt(pSSM,    pTrpm->uPrevVector);
-#if 0  /** @todo Enable this (+ load change) on the next version change. */
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
+    {
+        PTRPMCPU pTrpmCpu = &pVM->aCpus[i].trpm.s;
+        SSMR3PutUInt(pSSM,      pTrpmCpu->uActiveVector);
+        SSMR3PutUInt(pSSM,      pTrpmCpu->enmActiveType);
+        SSMR3PutGCUInt(pSSM,    pTrpmCpu->uActiveErrorCode);
+        SSMR3PutGCUIntPtr(pSSM, pTrpmCpu->uActiveCR2);
+        SSMR3PutGCUInt(pSSM,    pTrpmCpu->uSavedVector);
+        SSMR3PutUInt(pSSM,      pTrpmCpu->enmSavedType);
+        SSMR3PutGCUInt(pSSM,    pTrpmCpu->uSavedErrorCode);
+        SSMR3PutGCUIntPtr(pSSM, pTrpmCpu->uSavedCR2);
+        SSMR3PutGCUInt(pSSM,    pTrpmCpu->uPrevVector);
+    }
     SSMR3PutBool(pSSM,      pTrpm->fDisableMonitoring);
-#else
-    SSMR3PutGCUInt(pSSM,    pTrpm->fDisableMonitoring);
-#endif
-    SSMR3PutUInt(pSSM,      VM_FF_ISSET(pVM, VM_FF_TRPM_SYNC_IDT));
+    PVMCPU pVCpu = &pVM->aCpus[0];  /* raw mode implies 1 VCPU */
+    SSMR3PutUInt(pSSM,      VMCPU_FF_ISSET(pVCpu, VMCPU_FF_TRPM_SYNC_IDT));
     SSMR3PutMem(pSSM,       &pTrpm->au32IdtPatched[0], sizeof(pTrpm->au32IdtPatched));
     SSMR3PutU32(pSSM, ~0);              /* separator. */
 
@@ -795,18 +822,21 @@ static DECLCALLBACK(int) trpmR3Save(PVM pVM, PSSMHANDLE pSSM)
  * @returns VBox status code.
  * @param   pVM             VM Handle.
  * @param   pSSM            SSM operation handle.
- * @param   u32Version      Data layout version.
+ * @param   uVersion        Data layout version.
+ * @param   uPass           The data pass.
  */
-static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Version)
+static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t uVersion, uint32_t uPass)
 {
     LogFlow(("trpmR3Load:\n"));
+    Assert(uPass == SSM_PASS_FINAL); NOREF(uPass);
 
     /*
      * Validate version.
      */
-    if (u32Version != TRPM_SAVED_STATE_VERSION)
+    if (    uVersion != TRPM_SAVED_STATE_VERSION
+        &&  uVersion != TRPM_SAVED_STATE_VERSION_UNI)
     {
-        AssertMsgFailed(("trpmR3Load: Invalid version u32Version=%d!\n", u32Version));
+        AssertMsgFailed(("trpmR3Load: Invalid version uVersion=%d!\n", uVersion));
         return VERR_SSM_UNSUPPORTED_DATA_UNIT_VERSION;
     }
 
@@ -819,22 +849,42 @@ static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Versio
      * Active and saved traps.
      */
     PTRPM pTrpm = &pVM->trpm.s;
-    SSMR3GetUInt(pSSM,      &pTrpm->uActiveVector);
-    SSMR3GetUInt(pSSM,      (uint32_t *)&pTrpm->enmActiveType);
-    SSMR3GetGCUInt(pSSM,    &pTrpm->uActiveErrorCode);
-    SSMR3GetGCUIntPtr(pSSM, &pTrpm->uActiveCR2);
-    SSMR3GetGCUInt(pSSM,    &pTrpm->uSavedVector);
-    SSMR3GetUInt(pSSM,      (uint32_t *)&pTrpm->enmSavedType);
-    SSMR3GetGCUInt(pSSM,    &pTrpm->uSavedErrorCode);
-    SSMR3GetGCUIntPtr(pSSM, &pTrpm->uSavedCR2);
-    SSMR3GetGCUInt(pSSM,    &pTrpm->uPrevVector);
-#if 0 /** @todo Enable this + the corresponding save code on the next version change. */
-    SSMR3GetBool(pSSM,      &pTrpm->fDisableMonitoring);
-#else
-    RTGCUINT fDisableMonitoring;
-    SSMR3GetGCUInt(pSSM,    &fDisableMonitoring);
-    pTrpm->fDisableMonitoring = !!fDisableMonitoring;
-#endif
+
+    if (uVersion == TRPM_SAVED_STATE_VERSION)
+    {
+        for (VMCPUID i = 0; i < pVM->cCpus; i++)
+        {
+            PTRPMCPU pTrpmCpu = &pVM->aCpus[i].trpm.s;
+            SSMR3GetUInt(pSSM,      &pTrpmCpu->uActiveVector);
+            SSMR3GetUInt(pSSM,      (uint32_t *)&pTrpmCpu->enmActiveType);
+            SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uActiveErrorCode);
+            SSMR3GetGCUIntPtr(pSSM, &pTrpmCpu->uActiveCR2);
+            SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uSavedVector);
+            SSMR3GetUInt(pSSM,      (uint32_t *)&pTrpmCpu->enmSavedType);
+            SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uSavedErrorCode);
+            SSMR3GetGCUIntPtr(pSSM, &pTrpmCpu->uSavedCR2);
+            SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uPrevVector);
+        }
+
+        SSMR3GetBool(pSSM,      &pVM->trpm.s.fDisableMonitoring);
+    }
+    else
+    {
+        PTRPMCPU pTrpmCpu = &pVM->aCpus[0].trpm.s;
+        SSMR3GetUInt(pSSM,      &pTrpmCpu->uActiveVector);
+        SSMR3GetUInt(pSSM,      (uint32_t *)&pTrpmCpu->enmActiveType);
+        SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uActiveErrorCode);
+        SSMR3GetGCUIntPtr(pSSM, &pTrpmCpu->uActiveCR2);
+        SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uSavedVector);
+        SSMR3GetUInt(pSSM,      (uint32_t *)&pTrpmCpu->enmSavedType);
+        SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uSavedErrorCode);
+        SSMR3GetGCUIntPtr(pSSM, &pTrpmCpu->uSavedCR2);
+        SSMR3GetGCUInt(pSSM,    &pTrpmCpu->uPrevVector);
+
+        RTGCUINT fDisableMonitoring;
+        SSMR3GetGCUInt(pSSM,    &fDisableMonitoring);
+        pTrpm->fDisableMonitoring = !!fDisableMonitoring;
+    }
 
     RTUINT fSyncIDT;
     int rc = SSMR3GetUInt(pSSM, &fSyncIDT);
@@ -846,7 +896,10 @@ static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Versio
         return VERR_SSM_DATA_UNIT_FORMAT_CHANGED;
     }
     if (fSyncIDT)
-        VM_FF_SET(pVM, VM_FF_TRPM_SYNC_IDT);
+    {
+        PVMCPU pVCpu = &pVM->aCpus[0];  /* raw mode implies 1 VCPU */
+        VMCPU_FF_SET(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
+    }
     /* else: cleared by reset call above. */
 
     SSMR3GetMem(pSSM, &pTrpm->au32IdtPatched[0], sizeof(pTrpm->au32IdtPatched));
@@ -898,12 +951,13 @@ static DECLCALLBACK(int) trpmR3Load(PVM pVM, PSSMHANDLE pSSM, uint32_t u32Versio
 
 /**
  * Check if gate handlers were updated
- * (callback for the VM_FF_TRPM_SYNC_IDT forced action).
+ * (callback for the VMCPU_FF_TRPM_SYNC_IDT forced action).
  *
  * @returns VBox status code.
  * @param   pVM         The VM handle.
+ * @param   pVCpu       The VMCPU handle.
  */
-VMMR3DECL(int) TRPMR3SyncIDT(PVM pVM)
+VMMR3DECL(int) TRPMR3SyncIDT(PVM pVM, PVMCPU pVCpu)
 {
     STAM_PROFILE_START(&pVM->trpm.s.StatSyncIDT, a);
     const bool  fRawRing0 = EMIsRawRing0Enabled(pVM);
@@ -911,7 +965,7 @@ VMMR3DECL(int) TRPMR3SyncIDT(PVM pVM)
 
     if (pVM->trpm.s.fDisableMonitoring)
     {
-        VM_FF_CLEAR(pVM, VM_FF_TRPM_SYNC_IDT);
+        VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
         return VINF_SUCCESS;    /* Nothing to do */
     }
 
@@ -931,7 +985,7 @@ VMMR3DECL(int) TRPMR3SyncIDT(PVM pVM)
      * Get the IDTR.
      */
     VBOXIDTR IDTR;
-    IDTR.pIdt = CPUMGetGuestIDTR(pVM, &IDTR.cbIdt);
+    IDTR.pIdt = CPUMGetGuestIDTR(pVCpu, &IDTR.cbIdt);
     if (!IDTR.cbIdt)
     {
         Log(("No IDT entries...\n"));
@@ -984,7 +1038,7 @@ VMMR3DECL(int) TRPMR3SyncIDT(PVM pVM)
      * Should probably check/sync the others too, but for now we'll handle that in #GP.
      */
     X86DESC  Idte3;
-    rc = PGMPhysSimpleReadGCPtr(pVM, &Idte3, IDTR.pIdt + sizeof(Idte3) * 3,  sizeof(Idte3));
+    rc = PGMPhysSimpleReadGCPtr(pVCpu, &Idte3, IDTR.pIdt + sizeof(Idte3) * 3,  sizeof(Idte3));
     if (RT_FAILURE(rc))
     {
         AssertMsgRC(rc, ("Failed to read IDT[3]! rc=%Rrc\n", rc));
@@ -999,7 +1053,7 @@ VMMR3DECL(int) TRPMR3SyncIDT(PVM pVM)
     /*
      * Clear the FF and we're done.
      */
-    VM_FF_CLEAR(pVM, VM_FF_TRPM_SYNC_IDT);
+    VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
     STAM_PROFILE_STOP(&pVM->trpm.s.StatSyncIDT, a);
     return VINF_SUCCESS;
 }
@@ -1037,7 +1091,8 @@ VMMR3DECL(void) TRPMR3DisableMonitoring(PVM pVM)
     }
 #endif
 
-    VM_FF_CLEAR(pVM, VM_FF_TRPM_SYNC_IDT);
+    PVMCPU pVCpu = &pVM->aCpus[0];  /* raw mode implies on VCPU */
+    VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
 
     pVM->trpm.s.fDisableMonitoring = true;
 }
@@ -1063,7 +1118,7 @@ static DECLCALLBACK(int) trpmR3GuestIDTWriteHandler(PVM pVM, RTGCPTR GCPtr, void
 {
     Assert(enmAccessType == PGMACCESSTYPE_WRITE);
     Log(("trpmR3GuestIDTWriteHandler: write to %RGv size %d\n", GCPtr, cbBuf));
-    VM_FF_SET(pVM, VM_FF_TRPM_SYNC_IDT);
+    VMCPU_FF_SET(VMMGetCpu(pVM), VMCPU_FF_TRPM_SYNC_IDT);
     return VINF_PGM_HANDLER_DO_DEFAULT;
 }
 
@@ -1077,6 +1132,9 @@ static DECLCALLBACK(int) trpmR3GuestIDTWriteHandler(PVM pVM, RTGCPTR GCPtr, void
  */
 VMMR3DECL(int) trpmR3ClearPassThroughHandler(PVM pVM, unsigned iTrap)
 {
+    /* Only applies to raw mode which supports only 1 VCPU. */
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     /** @todo cleanup trpmR3ClearPassThroughHandler()! */
     RTRCPTR aGCPtrs[TRPM_HANDLER_MAX];
     int rc;
@@ -1097,7 +1155,7 @@ VMMR3DECL(int) trpmR3ClearPassThroughHandler(PVM pVM, unsigned iTrap)
     /* Unmark it for relocation purposes. */
     ASMBitClear(&pVM->trpm.s.au32IdtPatched[0], iTrap);
 
-    RTSEL               SelCS         = CPUMGetHyperCS(pVM);
+    RTSEL               SelCS         = CPUMGetHyperCS(pVCpu);
     PVBOXIDTE           pIdte         = &pVM->trpm.s.aIdt[iTrap];
     PVBOXIDTE_GENERIC   pIdteTemplate = &g_aIdt[iTrap];
     if (pIdte->Gen.u1Present)
@@ -1181,6 +1239,10 @@ VMMR3DECL(RTRCPTR) TRPMR3GetGuestTrapHandler(PVM pVM, unsigned iTrap)
  */
 VMMR3DECL(int) TRPMR3SetGuestTrapHandler(PVM pVM, unsigned iTrap, RTRCPTR pHandler)
 {
+    /* Only valid in raw mode which implies 1 VCPU */
+    Assert(PATMIsEnabled(pVM) && pVM->cCpus == 1);
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     /*
      * Validate.
      */
@@ -1193,7 +1255,7 @@ VMMR3DECL(int) TRPMR3SetGuestTrapHandler(PVM pVM, unsigned iTrap, RTRCPTR pHandl
     AssertReturn(pHandler == TRPM_INVALID_HANDLER || PATMIsPatchGCAddr(pVM, pHandler), VERR_INVALID_PARAMETER);
 
     uint16_t    cbIDT;
-    RTGCPTR     GCPtrIDT = CPUMGetGuestIDTR(pVM, &cbIDT);
+    RTGCPTR     GCPtrIDT = CPUMGetGuestIDTR(pVCpu, &cbIDT);
     if (iTrap * sizeof(VBOXIDTE) >= cbIDT)
         return VERR_INVALID_PARAMETER;  /* Silently ignore out of range requests. */
 
@@ -1208,7 +1270,7 @@ VMMR3DECL(int) TRPMR3SetGuestTrapHandler(PVM pVM, unsigned iTrap, RTRCPTR pHandl
      * Read the guest IDT entry.
      */
     VBOXIDTE GuestIdte;
-    int rc = PGMPhysSimpleReadGCPtr(pVM, &GuestIdte, GCPtrIDT + iTrap * sizeof(GuestIdte),  sizeof(GuestIdte));
+    int rc = PGMPhysSimpleReadGCPtr(pVCpu, &GuestIdte, GCPtrIDT + iTrap * sizeof(GuestIdte),  sizeof(GuestIdte));
     if (RT_FAILURE(rc))
     {
         AssertMsgRC(rc, ("Failed to read IDTE! rc=%Rrc\n", rc));
@@ -1300,11 +1362,15 @@ VMMR3DECL(int) TRPMR3SetGuestTrapHandler(PVM pVM, unsigned iTrap, RTRCPTR pHandl
  */
 VMMR3DECL(bool) TRPMR3IsGateHandler(PVM pVM, RTRCPTR GCPtr)
 {
+    /* Only valid in raw mode which implies 1 VCPU */
+    Assert(PATMIsEnabled(pVM) && pVM->cCpus == 1);
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     /*
      * Read IDTR and calc last entry.
      */
     uint16_t    cbIDT;
-    RTGCPTR     GCPtrIDTE = CPUMGetGuestIDTR(pVM, &cbIDT);
+    RTGCPTR     GCPtrIDTE = CPUMGetGuestIDTR(pVCpu, &cbIDT);
     unsigned    cEntries = (cbIDT + 1) / sizeof(VBOXIDTE);
     if (!cEntries)
         return false;
@@ -1321,7 +1387,7 @@ VMMR3DECL(bool) TRPMR3IsGateHandler(PVM pVM, RTRCPTR GCPtr)
          */
         PCVBOXIDTE      pIDTE;
         PGMPAGEMAPLOCK  Lock;
-        int rc = PGMPhysGCPtr2CCPtrReadOnly(pVM, GCPtrIDTE, (const void **)&pIDTE, &Lock);
+        int rc = PGMPhysGCPtr2CCPtrReadOnly(pVCpu, GCPtrIDTE, (const void **)&pIDTE, &Lock);
         if (RT_SUCCESS(rc))
         {
             /*
@@ -1370,40 +1436,41 @@ VMMR3DECL(bool) TRPMR3IsGateHandler(PVM pVM, RTRCPTR GCPtr)
  *
  * @returns VBox status code.
  * @param   pVM         The VM to operate on.
+ * @param   pVCpu       The VMCPU to operate on.
  * @param   enmEvent    Trpm event type
  */
-VMMR3DECL(int) TRPMR3InjectEvent(PVM pVM, TRPMEVENT enmEvent)
+VMMR3DECL(int) TRPMR3InjectEvent(PVM pVM, PVMCPU pVCpu, TRPMEVENT enmEvent)
 {
     PCPUMCTX pCtx;
     int      rc;
 
-    pCtx = CPUMQueryGuestCtxPtr(pVM);
-    Assert(!PATMIsPatchGCAddr(pVM, (RTGCPTR)pCtx->eip));
-    Assert(!VM_FF_ISSET(pVM, VM_FF_INHIBIT_INTERRUPTS));
+    pCtx = CPUMQueryGuestCtxPtr(pVCpu);
+    Assert(!PATMIsPatchGCAddr(pVM, pCtx->eip));
+    Assert(!VMCPU_FF_ISSET(pVCpu, VMCPU_FF_INHIBIT_INTERRUPTS));
 
     /* Currently only useful for external hardware interrupts. */
     Assert(enmEvent == TRPM_HARDWARE_INT);
 
-    if (REMR3QueryPendingInterrupt(pVM) == REM_NO_PENDING_IRQ)
+    if (REMR3QueryPendingInterrupt(pVM, pVCpu) == REM_NO_PENDING_IRQ)
     {
 #ifdef TRPM_FORWARD_TRAPS_IN_GC
 
 # ifdef LOG_ENABLED
         DBGFR3InfoLog(pVM, "cpumguest", "TRPMInject");
-        DBGFR3DisasInstrCurrentLog(pVM, "TRPMInject");
+        DBGFR3DisasInstrCurrentLog(pVCpu, "TRPMInject");
 # endif
 
         uint8_t u8Interrupt;
-        rc = PDMGetInterrupt(pVM, &u8Interrupt);
-        Log(("TRPMR3InjectEvent: u8Interrupt=%d (%#x) rc=%Rrc\n", u8Interrupt, u8Interrupt, rc));
+        rc = PDMGetInterrupt(pVCpu, &u8Interrupt);
+        Log(("TRPMR3InjectEvent: CPU%d u8Interrupt=%d (%#x) rc=%Rrc\n", pVCpu->idCpu, u8Interrupt, u8Interrupt, rc));
         if (RT_SUCCESS(rc))
         {
-            if (HWACCMR3IsActive(pVM))
+            if (HWACCMIsEnabled(pVM))
             {
-                rc = TRPMAssertTrap(pVM, u8Interrupt, enmEvent);
+                rc = TRPMAssertTrap(pVCpu, u8Interrupt, enmEvent);
                 AssertRC(rc);
                 STAM_COUNTER_INC(&pVM->trpm.s.paStatForwardedIRQR3[u8Interrupt]);
-                return VINF_EM_RESCHEDULE_HWACC;
+                return HWACCMR3IsActive(pVCpu) ? VINF_EM_RESCHEDULE_HWACC : VINF_EM_RESCHEDULE_REM;
             }
             /* If the guest gate is not patched, then we will check (again) if we can patch it. */
             if (pVM->trpm.s.aGuestTrapHandler[u8Interrupt] == TRPM_INVALID_HANDLER)
@@ -1415,14 +1482,14 @@ VMMR3DECL(int) TRPMR3InjectEvent(PVM pVM, TRPMEVENT enmEvent)
             if (pVM->trpm.s.aGuestTrapHandler[u8Interrupt] != TRPM_INVALID_HANDLER)
             {
                 /* Must check pending forced actions as our IDT or GDT might be out of sync */
-                rc = EMR3CheckRawForcedActions(pVM);
+                rc = EMR3CheckRawForcedActions(pVM, pVCpu);
                 if (rc == VINF_SUCCESS)
                 {
                     /* There's a handler -> let's execute it in raw mode */
-                    rc = TRPMForwardTrap(pVM, CPUMCTX2CORE(pCtx), u8Interrupt, 0, TRPM_TRAP_NO_ERRORCODE, enmEvent, -1);
+                    rc = TRPMForwardTrap(pVCpu, CPUMCTX2CORE(pCtx), u8Interrupt, 0, TRPM_TRAP_NO_ERRORCODE, enmEvent, -1);
                     if (rc == VINF_SUCCESS /* Don't use RT_SUCCESS */)
                     {
-                        Assert(!VM_FF_ISPENDING(pVM, VM_FF_SELM_SYNC_GDT | VM_FF_SELM_SYNC_LDT | VM_FF_TRPM_SYNC_IDT | VM_FF_SELM_SYNC_TSS));
+                        Assert(!VMCPU_FF_ISPENDING(pVCpu, VMCPU_FF_SELM_SYNC_GDT | VMCPU_FF_SELM_SYNC_LDT | VMCPU_FF_TRPM_SYNC_IDT | VMCPU_FF_SELM_SYNC_TSS));
 
                         STAM_COUNTER_INC(&pVM->trpm.s.paStatForwardedIRQR3[u8Interrupt]);
                         return VINF_EM_RESCHEDULE_RAW;
@@ -1431,15 +1498,18 @@ VMMR3DECL(int) TRPMR3InjectEvent(PVM pVM, TRPMEVENT enmEvent)
             }
             else
                 STAM_COUNTER_INC(&pVM->trpm.s.StatForwardFailNoHandler);
-            REMR3NotifyPendingInterrupt(pVM, u8Interrupt);
+            REMR3NotifyPendingInterrupt(pVM, pVCpu, u8Interrupt);
         }
         else
+        {
             AssertRC(rc);
+            return HWACCMR3IsActive(pVCpu) ? VINF_EM_RESCHEDULE_HWACC : VINF_EM_RESCHEDULE_REM; /* (Heed the halted state if this is changed!) */
+        }
 #else
         if (HWACCMR3IsActive(pVM))
         {
             uint8_t u8Interrupt;
-            rc = PDMGetInterrupt(pVM, &u8Interrupt);
+            rc = PDMGetInterrupt(pVCpu, &u8Interrupt);
             Log(("TRPMR3InjectEvent: u8Interrupt=%d (%#x) rc=%Rrc\n", u8Interrupt, u8Interrupt, rc));
             if (RT_SUCCESS(rc))
             {
