@@ -1,10 +1,10 @@
-/* $Id: VMInternal.h $ */
+/* $Id: VMInternal.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VM - Internal header file.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VMInternal_h
@@ -24,6 +20,8 @@
 
 #include <VBox/cdefs.h>
 #include <VBox/vmapi.h>
+#include <iprt/assert.h>
+#include <iprt/critsect.h>
 #include <setjmp.h>
 
 
@@ -35,65 +33,6 @@
  */
 
 
-/**
- * At-reset callback type.
- */
-typedef enum VMATRESETTYPE
-{
-    /** Device callback. */
-    VMATRESETTYPE_DEV = 1,
-    /** Internal callback . */
-    VMATRESETTYPE_INTERNAL,
-    /** External callback. */
-    VMATRESETTYPE_EXTERNAL
-} VMATRESETTYPE;
-
-
-/** Pointer to at-reset callback. */
-typedef struct VMATRESET *PVMATRESET;
-
-/**
- * At reset callback.
- */
-typedef struct VMATRESET
-{
-    /** Pointer to the next one in the list. */
-    PVMATRESET                      pNext;
-    /** Callback type. */
-    VMATRESETTYPE                   enmType;
-    /** User argument for the callback. */
-    void                           *pvUser;
-    /** Description. */
-    const char                     *pszDesc;
-    /** Type specific data. */
-    union
-    {
-        /** VMATRESETTYPE_DEV. */
-        struct
-        {
-            /** Callback. */
-            PFNVMATRESET            pfnCallback;
-            /** Device instance. */
-            PPDMDEVINS              pDevIns;
-        } Dev;
-
-        /** VMATRESETTYPE_INTERNAL. */
-        struct
-        {
-            /** Callback. */
-            PFNVMATRESETINT         pfnCallback;
-        } Internal;
-
-        /** VMATRESETTYPE_EXTERNAL. */
-        struct
-        {
-            /** Callback. */
-            PFNVMATRESETEXT         pfnCallback;
-        } External;
-    } u;
-} VMATRESET;
-
-
 /**
  * VM state change callback.
  */
@@ -222,9 +161,8 @@ typedef struct VMINT
     R3PTRTYPE(PVMERROR)             pErrorR3;
     /** VM Runtime Error Message. */
     R3PTRTYPE(PVMRUNTIMEERROR)      pRuntimeErrorR3;
-    /** Set by VMR3SuspendNoSave; cleared by VMR3Resume; signals the VM is in an
-     * inconsistent state and saving is not allowed. */
-    bool                            fPreventSaveState;
+    /** The VM was/is-being teleported and has not yet been fully resumed. */
+    bool                            fTeleportedAndNotFullyResumedYet;
 } VMINT;
 /** Pointer to the VM Internal Data (part of the VM structure). */
 typedef VMINT *PVMINT;
@@ -255,22 +193,56 @@ typedef struct VMINTUSERPERVM
     STAMCOUNTER                     StatReqFree;
     /** Number of times the request was actually freed. */
     STAMCOUNTER                     StatReqFreeOverflow;
+    /** Number of requests served. */
+    STAMCOUNTER                     StatReqProcessed;
+    /** Number of times there are more than one request and the others needed to be
+     * pushed back onto the list. */
+    STAMCOUNTER                     StatReqMoreThan1;
+    /** Number of times we've raced someone when pushing the other requests back
+     * onto the list. */
+    STAMCOUNTER                     StatReqPushBackRaces;
 #endif
 
     /** Pointer to the support library session.
-     * Mainly for creation and destruction.. */
+     * Mainly for creation and destruction. */
     PSUPDRVSESSION                  pSession;
 
-    /** Wait event semaphore. */
-    RTSEMEVENT                      EventSemWait;
-    /** Wait/Idle indicator. */
-    bool volatile                   fWait;
     /** Force EMT to terminate. */
     bool volatile                   fTerminateEMT;
-    /** If set the EMT does the final VM cleanup when it exits.
+    /** If set the EMT(0) does the final VM cleanup when it exits.
      * If clear the VMR3Destroy() caller does so. */
     bool                            fEMTDoesTheCleanup;
 
+    /** Critical section for pAtState and enmPrevVMState. */
+    RTCRITSECT                      AtStateCritSect;
+    /** List of registered state change callbacks. */
+    PVMATSTATE                      pAtState;
+    /** List of registered state change callbacks. */
+    PVMATSTATE                     *ppAtStateNext;
+    /** The previous VM state.
+     * This is mainly used for the 'Resetting' state, but may come in handy later
+     * and when debugging. */
+    VMSTATE                         enmPrevVMState;
+
+    /** Critical section for pAtError and pAtRuntimeError. */
+    RTCRITSECT                      AtErrorCritSect;
+
+    /** List of registered error callbacks. */
+    PVMATERROR                      pAtError;
+    /** List of registered error callbacks. */
+    PVMATERROR                     *ppAtErrorNext;
+    /** The error message count.
+     * This is incremented every time an error is raised.  */
+    uint32_t volatile               cErrors;
+
+    /** The runtime error message count.
+     * This is incremented every time a runtime error is raised.  */
+    uint32_t volatile               cRuntimeErrors;
+    /** List of registered error callbacks. */
+    PVMATRUNTIMEERROR               pAtRuntimeError;
+    /** List of registered error callbacks. */
+    PVMATRUNTIMEERROR              *ppAtRuntimeErrorNext;
+
     /** @name Generic Halt data
      * @{
      */
@@ -279,18 +251,10 @@ typedef struct VMINTUSERPERVM
     VMHALTMETHOD                    enmHaltMethod;
     /** The index into g_aHaltMethods of the current halt method. */
     uint32_t volatile               iHaltMethod;
-    /** The average time (ns) between two halts in the last second. (updated once per second) */
-    uint32_t                        HaltInterval;
-    /** The average halt frequency for the last second. (updated once per second) */
-    uint32_t                        HaltFrequency;
-    /** The number of halts in the current period. */
-    uint32_t                        cHalts;
-    uint32_t                        padding; /**< alignment padding. */
-    /** When we started counting halts in cHalts (RTTimeNanoTS). */
-    uint64_t                        u64HaltsStartTS;
     /** @} */
 
-    /** Union containing data and config for the different halt algorithms. */
+    /** @todo Do NOT add new members here or resue the current, we need to store the config for
+     *  each halt method seperately because we're racing on SMP guest rigs. */
     union
     {
        /**
@@ -305,21 +269,6 @@ typedef struct VMINTUSERPERVM
         */
         struct
         {
-            /** How many times we've blocked while cBlockedNS and cBlockedTooLongNS has been accumulating. */
-            uint32_t                cBlocks;
-            /** Avg. time spend oversleeping when blocking. (Re-calculated every so often.) */
-            uint64_t                cNSBlockedTooLongAvg;
-            /** Total time spend oversleeping when blocking. */
-            uint64_t                cNSBlockedTooLong;
-            /** Total time spent blocking. */
-            uint64_t                cNSBlocked;
-            /** The timestamp (RTTimeNanoTS) of the last block. */
-            uint64_t                u64LastBlockTS;
-
-            /** When we started spinning relentlessly in order to catch up some of the oversleeping.
-             * This is 0 when we're not spinning. */
-            uint64_t                u64StartSpinTS;
-
             /** The max interval without blocking (when spinning). */
             uint32_t                u32MinBlockIntervalCfg;
             /** The minimum interval between blocking (when spinning). */
@@ -331,71 +280,8 @@ typedef struct VMINTUSERPERVM
             /** When to stop spinning (lag / nano secs). */
             uint32_t                u32StopSpinningCfg;
         }                           Method12;
-
-#if 0
-       /**
-        * Method 3 & 4 - Same as method 1 & 2 respectivly, except that we
-        * sprinkle it with yields.
-        */
-       struct
-       {
-           /** How many times we've blocked while cBlockedNS and cBlockedTooLongNS has been accumulating. */
-           uint32_t                 cBlocks;
-           /** Avg. time spend oversleeping when blocking. (Re-calculated every so often.) */
-           uint64_t                 cBlockedTooLongNSAvg;
-           /** Total time spend oversleeping when blocking. */
-           uint64_t                 cBlockedTooLongNS;
-           /** Total time spent blocking. */
-           uint64_t                 cBlockedNS;
-           /** The timestamp (RTTimeNanoTS) of the last block. */
-           uint64_t                 u64LastBlockTS;
-
-           /** How many times we've yielded while cBlockedNS and cBlockedTooLongNS has been accumulating. */
-           uint32_t                 cYields;
-           /** Avg. time spend oversleeping when yielding. */
-           uint32_t                 cYieldTooLongNSAvg;
-           /** Total time spend oversleeping when yielding. */
-           uint64_t                 cYieldTooLongNS;
-           /** Total time spent yielding. */
-           uint64_t                 cYieldedNS;
-           /** The timestamp (RTTimeNanoTS) of the last block. */
-           uint64_t                 u64LastYieldTS;
-
-           /** When we started spinning relentlessly in order to catch up some of the oversleeping. */
-           uint64_t                 u64StartSpinTS;
-       }                            Method34;
-#endif
     }                               Halt;
 
-    /** Profiling the halted state; yielding vs blocking.
-     * @{ */
-    STAMPROFILE                     StatHaltYield;
-    STAMPROFILE                     StatHaltBlock;
-    STAMPROFILE                     StatHaltTimers;
-    STAMPROFILE                     StatHaltPoll;
-    /** @} */
-
-
-    /** List of registered reset callbacks. */
-    PVMATRESET                      pAtReset;
-    /** List of registered reset callbacks. */
-    PVMATRESET                     *ppAtResetNext;
-
-    /** List of registered state change callbacks. */
-    PVMATSTATE                      pAtState;
-    /** List of registered state change callbacks. */
-    PVMATSTATE                     *ppAtStateNext;
-
-    /** List of registered error callbacks. */
-    PVMATERROR                      pAtError;
-    /** List of registered error callbacks. */
-    PVMATERROR                     *ppAtErrorNext;
-
-    /** List of registered error callbacks. */
-    PVMATRUNTIMEERROR               pAtRuntimeError;
-    /** List of registered error callbacks. */
-    PVMATRUNTIMEERROR              *ppAtRuntimeErrorNext;
-
     /** Pointer to the DBGC instance data. */
     void                           *pvDBGC;
 
@@ -430,15 +316,12 @@ typedef struct VMINTUSERPERVMCPU
     /** If set the EMT does the final VM cleanup when it exits.
      * If clear the VMR3Destroy() caller does so. */
     bool                            fEMTDoesTheCleanup;
+    /** Align the next bit. */
+    bool                            afAlignment[5];
 
     /** @name Generic Halt data
      * @{
      */
-    /** The current halt method.
-     * Can be selected by CFGM option 'VM/HaltMethod'. */
-    VMHALTMETHOD                    enmHaltMethod;
-    /** The index into g_aHaltMethods of the current halt method. */
-    uint32_t volatile               iHaltMethod;
     /** The average time (ns) between two halts in the last second. (updated once per second) */
     uint32_t                        HaltInterval;
     /** The average halt frequency for the last second. (updated once per second) */
@@ -467,6 +350,8 @@ typedef struct VMINTUSERPERVMCPU
         {
             /** How many times we've blocked while cBlockedNS and cBlockedTooLongNS has been accumulating. */
             uint32_t                cBlocks;
+            /** Align the next member. */
+            uint32_t                u32Alignment;
             /** Avg. time spend oversleeping when blocking. (Re-calculated every so often.) */
             uint64_t                cNSBlockedTooLongAvg;
             /** Total time spend oversleeping when blocking. */
@@ -479,17 +364,6 @@ typedef struct VMINTUSERPERVMCPU
             /** When we started spinning relentlessly in order to catch up some of the oversleeping.
              * This is 0 when we're not spinning. */
             uint64_t                u64StartSpinTS;
-
-            /** The max interval without blocking (when spinning). */
-            uint32_t                u32MinBlockIntervalCfg;
-            /** The minimum interval between blocking (when spinning). */
-            uint32_t                u32MaxBlockIntervalCfg;
-            /** The value to divide the current lag by to get the raw blocking interval (when spinning). */
-            uint32_t                u32LagBlockIntervalDivisorCfg;
-            /** When to start spinning (lag / nano secs). */
-            uint32_t                u32StartSpinningCfg;
-            /** When to stop spinning (lag / nano secs). */
-            uint32_t                u32StopSpinningCfg;
         }                           Method12;
 
 #if 0
@@ -534,30 +408,30 @@ typedef struct VMINTUSERPERVMCPU
     STAMPROFILE                     StatHaltTimers;
     STAMPROFILE                     StatHaltPoll;
     /** @} */
-
-    /** Pointer to the DBGC instance data. */
-    void                           *pvDBGC;
-
-    /** vmR3EmulationThread longjmp buffer. Must be last in the structure. */
-    jmp_buf                         emtJumpEnv;
 } VMINTUSERPERVMCPU;
+#ifdef IN_RING3
+AssertCompileMemberAlignment(VMINTUSERPERVMCPU, u64HaltsStartTS, 8);
+AssertCompileMemberAlignment(VMINTUSERPERVMCPU, Halt.Method12.cNSBlockedTooLongAvg, 8);
+AssertCompileMemberAlignment(VMINTUSERPERVMCPU, StatHaltYield, 8);
+#endif
 
 /** Pointer to the VM internal data kept in the UVM. */
 typedef VMINTUSERPERVMCPU *PVMINTUSERPERVMCPU;
 
-__BEGIN_DECLS
+RT_C_DECLS_BEGIN
 
 DECLCALLBACK(int)   vmR3EmulationThread(RTTHREAD ThreadSelf, void *pvArg);
 int                 vmR3SetHaltMethodU(PUVM pUVM, VMHALTMETHOD enmHaltMethod);
 DECLCALLBACK(int)   vmR3Destroy(PVM pVM);
 DECLCALLBACK(void)  vmR3SetErrorUV(PUVM pUVM, int rc, RT_SRC_POS_DECL, const char *pszFormat, va_list *args);
 void                vmSetErrorCopy(PVM pVM, int rc, RT_SRC_POS_DECL, const char *pszFormat, va_list args);
+DECLCALLBACK(int)   vmR3SetRuntimeError(PVM pVM, uint32_t fFlags, const char *pszErrorId, char *pszMessage);
 DECLCALLBACK(int)   vmR3SetRuntimeErrorV(PVM pVM, uint32_t fFlags, const char *pszErrorId, const char *pszFormat, va_list *pVa);
 void                vmSetRuntimeErrorCopy(PVM pVM, uint32_t fFlags, const char *pszErrorId, const char *pszFormat, va_list va);
-void                vmR3DestroyFinalBitFromEMT(PUVM pUVM);
-void                vmR3SetState(PVM pVM, VMSTATE enmStateNew);
+void                vmR3DestroyFinalBitFromEMT(PUVM pUVM, VMCPUID idCpu);
+void                vmR3SetGuruMeditation(PVM pVM);
 
-__END_DECLS
+RT_C_DECLS_END
 
 
 /** @} */
