@@ -1,10 +1,10 @@
-/* $Id: IOMAll.cpp $ */
+/* $Id: IOMAll.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * IOM - Input / Output Monitor - Any Context.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -28,8 +24,10 @@
 #include <VBox/param.h>
 #include "IOMInternal.h"
 #include <VBox/vm.h>
+#include <VBox/vmm.h>
 #include <VBox/selm.h>
 #include <VBox/trpm.h>
+#include <VBox/pdmdev.h>
 #include <VBox/pgm.h>
 #include <VBox/cpum.h>
 #include <VBox/err.h>
@@ -38,6 +36,59 @@
 
 
 
+/**
+ * Try take the EMT/IOM lock, wait in ring-3 return VERR_SEM_BUSY in R0/RC.
+ *
+ * @retval  VINF_SUCCESS on success (always in ring-3).
+ * @retval  VERR_SEM_BUSY in RC and R0 if the semaphore is busy.
+ *
+ * @param   pVM         VM handle.
+ */
+int iomLock(PVM pVM)
+{
+    Assert(pVM->cCpus == 1 || !PGMIsLockOwner(pVM));
+    int rc = PDMCritSectEnter(&pVM->iom.s.EmtLock, VERR_SEM_BUSY);
+    return rc;
+}
+
+
+/**
+ * Try take the EMT/IOM lock, no waiting.
+ *
+ * @retval  VINF_SUCCESS on success.
+ * @retval  VERR_SEM_BUSY if busy.
+ *
+ * @param   pVM         VM handle.
+ */
+int iomTryLock(PVM pVM)
+{
+    int rc = PDMCritSectTryEnter(&pVM->iom.s.EmtLock);
+    return rc;
+}
+
+
+/**
+ * Release EMT/IOM lock.
+ *
+ * @param   pVM         VM handle.
+ */
+void iomUnlock(PVM pVM)
+{
+    PDMCritSectLeave(&pVM->iom.s.EmtLock);
+}
+
+
+/**
+ * Check if this VCPU currently owns the IOM lock.
+ *
+ * @returns bool owner/not owner
+ * @param   pVM         The VM to operate on.
+ */
+VMMDECL(bool) IOMIsLockOwner(PVM pVM)
+{
+    return PDMCritSectIsOwner(&pVM->iom.s.EmtLock);
+}
+
 /**
  * Returns the contents of register or immediate data of instruction's parameter.
  *
@@ -204,8 +255,16 @@ bool iomSaveDataToReg(PDISCPUSTATE pCpu, PCOP_PARAMETER pParam, PCPUMCTXCORE pRe
  * @param   pu32Value   Where to store the value read.
  * @param   cbValue     The size of the register to read in bytes. 1, 2 or 4 bytes.
  */
-VMMDECL(int) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t cbValue)
+VMMDECL(VBOXSTRICTRC) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t cbValue)
 {
+    /* Take the IOM lock before performing any device I/O. */
+    int rc2 = iomLock(pVM);
+#ifndef IN_RING3
+    if (rc2 == VERR_SEM_BUSY)
+        return VINF_IOM_HC_IOPORT_READ;
+#endif
+    AssertRC(rc2);
+
 #ifdef VBOX_WITH_STATISTICS
     /*
      * Get the statistics record.
@@ -234,38 +293,71 @@ VMMDECL(int) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t c
     if (pRange)
     {
         /*
-         * Found a range.
+         * Found a range, get the data in case we leave the IOM lock.
          */
+        PFNIOMIOPORTIN  pfnInCallback = pRange->pfnInCallback;
 #ifndef IN_RING3
-        if (!pRange->pfnInCallback)
+        if (!pfnInCallback)
         {
-# ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-                STAM_COUNTER_INC(&pStats->CTX_MID_Z(In,ToR3));
-# endif
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
+            iomUnlock(pVM);
             return VINF_IOM_HC_IOPORT_READ;
         }
 #endif
-        /* call the device. */
+        void           *pvUser    = pRange->pvUser;
+        PPDMDEVINS      pDevIns   = pRange->pDevIns;
+        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
+
+        /*
+         * Call the device - 4 variations.
+         */
+        VBOXSTRICTRC    rcStrict;
+        if (pCritSect)
+        {
+            iomUnlock(pVM);
+            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_READ);
+            if (rcStrict != VINF_SUCCESS)
+            {
+                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
+                return rcStrict;
+            }
 #ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_START(&pStats->CTX_SUFF_Z(ProfIn), a);
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
+                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
+            }
+            else
 #endif
-        int rc = pRange->pfnInCallback(pRange->pDevIns, pRange->pvUser, Port, pu32Value, (unsigned)cbValue);
+                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
+            PDMCritSectLeave(pCritSect);
+        }
+        else
+        {
 #ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
-        if (rc == VINF_SUCCESS && pStats)
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
+                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
+            }
+            else
+#endif
+                rcStrict = pfnInCallback(pDevIns, pvUser, Port, pu32Value, (unsigned)cbValue);
+        }
+#ifdef VBOX_WITH_STATISTICS
+        if (rcStrict == VINF_SUCCESS && pStats)
             STAM_COUNTER_INC(&pStats->CTX_SUFF_Z(In));
 # ifndef IN_RING3
-        else if (rc == VINF_IOM_HC_IOPORT_READ && pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(In,ToR3));
+        else if (rcStrict == VINF_IOM_HC_IOPORT_READ && pStats)
+            STAM_COUNTER_INC(&pStats->InRZToR3);
 # endif
 #endif
-        if (rc == VERR_IOM_IOPORT_UNUSED)
+        if (rcStrict == VERR_IOM_IOPORT_UNUSED)
         {
             /* make return value */
-            rc = VINF_SUCCESS;
+            rcStrict = VINF_SUCCESS;
             switch (cbValue)
             {
                 case 1: *(uint8_t  *)pu32Value = 0xff; break;
@@ -273,11 +365,15 @@ VMMDECL(int) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t c
                 case 4: *(uint32_t *)pu32Value = UINT32_C(0xffffffff); break;
                 default:
                     AssertMsgFailed(("Invalid I/O port size %d. Port=%d\n", cbValue, Port));
+                    if (!pCritSect)
+                        iomUnlock(pVM);
                     return VERR_IOM_INVALID_IOPORT_SIZE;
             }
         }
-        Log3(("IOMIOPortRead: Port=%RTiop *pu32=%08RX32 cb=%d rc=%Rrc\n", Port, *pu32Value, cbValue, rc));
-        return rc;
+        Log3(("IOMIOPortRead: Port=%RTiop *pu32=%08RX32 cb=%d rc=%Rrc\n", Port, *pu32Value, cbValue, VBOXSTRICTRC_VAL(rcStrict)));
+        if (!pCritSect)
+            iomUnlock(pVM);
+        return rcStrict;
     }
 
 #ifndef IN_RING3
@@ -289,8 +385,9 @@ VMMDECL(int) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t c
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(In,ToR3));
+            STAM_COUNTER_INC(&pStats->InRZToR3);
 # endif
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_READ;
     }
 #endif
@@ -305,6 +402,7 @@ VMMDECL(int) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t c
     {
 # ifndef IN_RING3
         /* Ring-3 will have to create the statistics record. */
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_READ;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -322,9 +420,11 @@ VMMDECL(int) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t c
         case 4: *(uint32_t *)pu32Value = UINT32_C(0xffffffff); break;
         default:
             AssertMsgFailed(("Invalid I/O port size %d. Port=%d\n", cbValue, Port));
+            iomUnlock(pVM);
             return VERR_IOM_INVALID_IOPORT_SIZE;
     }
     Log3(("IOMIOPortRead: Port=%RTiop *pu32=%08RX32 cb=%d rc=VINF_SUCCESS\n", Port, *pu32Value, cbValue));
+    iomUnlock(pVM);
     return VINF_SUCCESS;
 }
 
@@ -344,9 +444,17 @@ VMMDECL(int) IOMIOPortRead(PVM pVM, RTIOPORT Port, uint32_t *pu32Value, size_t c
  * @param   pGCPtrDst   Pointer to the destination buffer (GC, incremented appropriately).
  * @param   pcTransfers Pointer to the number of transfer units to read, on return remaining transfer units.
  * @param   cb          Size of the transfer unit (1, 2 or 4 bytes).
- *   */
-VMMDECL(int) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRTGCUINTREG pcTransfers, unsigned cb)
+ */
+VMMDECL(VBOXSTRICTRC) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRTGCUINTREG pcTransfers, unsigned cb)
 {
+    /* Take the IOM lock before performing any device I/O. */
+    int rc2 = iomLock(pVM);
+#ifndef IN_RING3
+    if (rc2 == VERR_SEM_BUSY)
+        return VINF_IOM_HC_IOPORT_READ;
+#endif
+    AssertRC(rc2);
+
 #ifdef LOG_ENABLED
     const RTGCUINTREG cTransfers = *pcTransfers;
 #endif
@@ -380,36 +488,71 @@ VMMDECL(int) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRT
         /*
          * Found a range.
          */
+        PFNIOMIOPORTINSTRING pfnInStrCallback = pRange->pfnInStrCallback;
 #ifndef IN_RING3
-        if (!pRange->pfnInStrCallback)
+        if (!pfnInStrCallback)
         {
-# ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-                STAM_COUNTER_INC(&pStats->CTX_MID_Z(In,ToR3));
-# endif
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
+            iomUnlock(pVM);
             return VINF_IOM_HC_IOPORT_READ;
         }
 #endif
-        /* call the device. */
-#ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_START(&pStats->CTX_SUFF_Z(ProfIn), a);
-#endif
+        void           *pvUser    = pRange->pvUser;
+        PPDMDEVINS      pDevIns   = pRange->pDevIns;
+        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
 
-        int rc = pRange->pfnInStrCallback(pRange->pDevIns, pRange->pvUser, Port, pGCPtrDst, pcTransfers, cb);
+        /*
+         * Call the device - 4 variations.
+         */
+        VBOXSTRICTRC    rcStrict;
+        if (pCritSect)
+        {
+            iomUnlock(pVM);
+            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_READ);
+            if (rcStrict != VINF_SUCCESS)
+            {
+                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->InRZToR3); });
+                return rcStrict;
+            }
 #ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
-        if (rc == VINF_SUCCESS && pStats)
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
+                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
+            }
+            else
+#endif
+                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
+            PDMCritSectLeave(pCritSect);
+        }
+        else
+        {
+#ifdef VBOX_WITH_STATISTICS
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfIn), a);
+                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfIn), a);
+            }
+            else
+#endif
+                rcStrict = pfnInStrCallback(pDevIns, pvUser, Port, pGCPtrDst, pcTransfers, cb);
+        }
+
+#ifdef VBOX_WITH_STATISTICS
+        if (rcStrict == VINF_SUCCESS && pStats)
             STAM_COUNTER_INC(&pStats->CTX_SUFF_Z(In));
 # ifndef IN_RING3
-        else if (rc == VINF_IOM_HC_IOPORT_READ && pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(In, ToR3));
+        else if (rcStrict == VINF_IOM_HC_IOPORT_READ && pStats)
+            STAM_COUNTER_INC(&pStats->InRZToR3);
 # endif
 #endif
         Log3(("IOMIOPortReadStr: Port=%RTiop pGCPtrDst=%p pcTransfer=%p:{%#x->%#x} cb=%d rc=%Rrc\n",
-              Port, pGCPtrDst, pcTransfers, cTransfers, *pcTransfers, cb, rc));
-        return rc;
+              Port, pGCPtrDst, pcTransfers, cTransfers, *pcTransfers, cb, VBOXSTRICTRC_VAL(rcStrict)));
+        if (!pCritSect)
+            iomUnlock(pVM);
+        return rcStrict;
     }
 
 #ifndef IN_RING3
@@ -421,8 +564,9 @@ VMMDECL(int) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRT
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(In,ToR3));
+            STAM_COUNTER_INC(&pStats->InRZToR3);
 # endif
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_READ;
     }
 #endif
@@ -437,6 +581,7 @@ VMMDECL(int) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRT
     {
 # ifndef IN_RING3
         /* Ring-3 will have to create the statistics record. */
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_READ;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -448,6 +593,7 @@ VMMDECL(int) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRT
 
     Log3(("IOMIOPortReadStr: Port=%RTiop pGCPtrDst=%p pcTransfer=%p:{%#x->%#x} cb=%d rc=VINF_SUCCESS\n",
           Port, pGCPtrDst, pcTransfers, cTransfers, *pcTransfers, cb));
+    iomUnlock(pVM);
     return VINF_SUCCESS;
 }
 
@@ -467,9 +613,18 @@ VMMDECL(int) IOMIOPortReadString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrDst, PRT
  * @param   u32Value    The value to write.
  * @param   cbValue     The size of the register to read in bytes. 1, 2 or 4 bytes.
  */
-VMMDECL(int) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cbValue)
+VMMDECL(VBOXSTRICTRC) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cbValue)
 {
-/** @todo bird: When I get time, I'll remove the GC tree and link the GC entries to the ring-3 node. */
+    /* Take the IOM lock before performing any device I/O. */
+    int rc2 = iomLock(pVM);
+#ifndef IN_RING3
+    if (rc2 == VERR_SEM_BUSY)
+        return VINF_IOM_HC_IOPORT_WRITE;
+#endif
+    AssertRC(rc2);
+
+/** @todo bird: When I get time, I'll remove the RC/R0 trees and link the RC/R0
+ *        entries to the ring-3 node. */
 #ifdef VBOX_WITH_STATISTICS
     /*
      * Find the statistics record.
@@ -500,35 +655,70 @@ VMMDECL(int) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cb
         /*
          * Found a range.
          */
+        PFNIOMIOPORTOUT pfnOutCallback = pRange->pfnOutCallback;
 #ifndef IN_RING3
-        if (!pRange->pfnOutCallback)
+        if (!pfnOutCallback)
         {
-# ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-                STAM_COUNTER_INC(&pStats->CTX_MID_Z(Out,ToR3));
-# endif
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
+            iomUnlock(pVM);
             return VINF_IOM_HC_IOPORT_WRITE;
         }
 #endif
-        /* call the device. */
+        void           *pvUser    =  pRange->pvUser;
+        PPDMDEVINS      pDevIns   = pRange->pDevIns;
+        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
+
+        /*
+         * Call the device - 4 variations.
+         */
+        VBOXSTRICTRC    rcStrict;
+        if (pCritSect)
+        {
+            iomUnlock(pVM);
+            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_WRITE);
+            if (rcStrict != VINF_SUCCESS)
+            {
+                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
+                return rcStrict;
+            }
 #ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
+            }
+            else
 #endif
-        int rc = pRange->pfnOutCallback(pRange->pDevIns, pRange->pvUser, Port, u32Value, (unsigned)cbValue);
+                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
+            PDMCritSectLeave(pCritSect);
+        }
+        else
+        {
+#ifdef VBOX_WITH_STATISTICS
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
+            }
+            else
+#endif
+                rcStrict = pfnOutCallback(pDevIns, pvUser, Port, u32Value, (unsigned)cbValue);
+        }
 
 #ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
-        if (rc == VINF_SUCCESS && pStats)
+        if (rcStrict == VINF_SUCCESS && pStats)
             STAM_COUNTER_INC(&pStats->CTX_SUFF_Z(Out));
 # ifndef IN_RING3
-        else if (rc == VINF_IOM_HC_IOPORT_WRITE && pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(Out, ToR3));
+        else if (rcStrict == VINF_IOM_HC_IOPORT_WRITE && pStats)
+            STAM_COUNTER_INC(&pStats->OutRZToR3);
 # endif
 #endif
-        Log3(("IOMIOPortWrite: Port=%RTiop u32=%08RX32 cb=%d rc=%Rrc\n", Port, u32Value, cbValue, rc));
-        return rc;
+        Log3(("IOMIOPortWrite: Port=%RTiop u32=%08RX32 cb=%d rc=%Rrc\n", Port, u32Value, cbValue, VBOXSTRICTRC_VAL(rcStrict)));
+        if (!pCritSect)
+            iomUnlock(pVM);
+        return rcStrict;
     }
 
 #ifndef IN_RING3
@@ -540,8 +730,9 @@ VMMDECL(int) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cb
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(Out,ToR3));
+            STAM_COUNTER_INC(&pStats->OutRZToR3);
 # endif
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
     }
 #endif
@@ -557,6 +748,7 @@ VMMDECL(int) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cb
     {
 # ifndef IN_RING3
         /* R3 will have to create the statistics record. */
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -566,6 +758,7 @@ VMMDECL(int) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cb
     }
 #endif
     Log3(("IOMIOPortWrite: Port=%RTiop u32=%08RX32 cb=%d nop\n", Port, u32Value, cbValue));
+    iomUnlock(pVM);
     return VINF_SUCCESS;
 }
 
@@ -586,8 +779,16 @@ VMMDECL(int) IOMIOPortWrite(PVM pVM, RTIOPORT Port, uint32_t u32Value, size_t cb
  * @param   pcTransfers Pointer to the number of transfer units to write, on return remaining transfer units.
  * @param   cb          Size of the transfer unit (1, 2 or 4 bytes).
  *   */
-VMMDECL(int) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PRTGCUINTREG pcTransfers, unsigned cb)
+VMMDECL(VBOXSTRICTRC) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PRTGCUINTREG pcTransfers, unsigned cb)
 {
+    /* Take the IOM lock before performing any device I/O. */
+    int rc2 = iomLock(pVM);
+#ifndef IN_RING3
+    if (rc2 == VERR_SEM_BUSY)
+        return VINF_IOM_HC_IOPORT_WRITE;
+#endif
+    AssertRC(rc2);
+
 #ifdef LOG_ENABLED
     const RTGCUINTREG cTransfers = *pcTransfers;
 #endif
@@ -621,35 +822,71 @@ VMMDECL(int) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PR
         /*
          * Found a range.
          */
+        PFNIOMIOPORTOUTSTRING   pfnOutStrCallback = pRange->pfnOutStrCallback;
 #ifndef IN_RING3
-        if (!pRange->pfnOutStrCallback)
+        if (!pfnOutStrCallback)
         {
-# ifdef VBOX_WITH_STATISTICS
-            if (pStats)
-                STAM_COUNTER_INC(&pStats->CTX_MID_Z(Out,ToR3));
-# endif
+            STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
+            iomUnlock(pVM);
             return VINF_IOM_HC_IOPORT_WRITE;
         }
 #endif
-        /* call the device. */
+        void           *pvUser    = pRange->pvUser;
+        PPDMDEVINS      pDevIns   = pRange->pDevIns;
+        PPDMCRITSECT    pCritSect = pDevIns->CTX_SUFF(pCritSect);
+
+        /*
+         * Call the device - 4 variations.
+         */
+        VBOXSTRICTRC    rcStrict;
+        if (pCritSect)
+        {
+            iomUnlock(pVM);
+            rcStrict = PDMCritSectEnter(pCritSect, VINF_IOM_HC_IOPORT_WRITE);
+            if (rcStrict != VINF_SUCCESS)
+            {
+                STAM_STATS({ if (pStats) STAM_COUNTER_INC(&pStats->OutRZToR3); });
+                return rcStrict;
+            }
 #ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
+            }
+            else
 #endif
-        int rc = pRange->pfnOutStrCallback(pRange->pDevIns, pRange->pvUser, Port, pGCPtrSrc, pcTransfers, cb);
+                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
+            PDMCritSectLeave(pCritSect);
+        }
+        else
+        {
 #ifdef VBOX_WITH_STATISTICS
-        if (pStats)
-            STAM_PROFILE_ADV_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
-        if (rc == VINF_SUCCESS && pStats)
+            if (pStats)
+            {
+                STAM_PROFILE_START(&pStats->CTX_SUFF_Z(ProfOut), a);
+                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
+                STAM_PROFILE_STOP(&pStats->CTX_SUFF_Z(ProfOut), a);
+            }
+            else
+#endif
+                rcStrict = pfnOutStrCallback(pDevIns, pvUser, Port, pGCPtrSrc, pcTransfers, cb);
+        }
+
+#ifdef VBOX_WITH_STATISTICS
+        if (rcStrict == VINF_SUCCESS && pStats)
             STAM_COUNTER_INC(&pStats->CTX_SUFF_Z(Out));
 # ifndef IN_RING3
-        else if (rc == VINF_IOM_HC_IOPORT_WRITE && pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(Out, ToR3));
+        else if (rcStrict == VINF_IOM_HC_IOPORT_WRITE && pStats)
+            STAM_COUNTER_INC(&pStats->OutRZToR3);
 # endif
 #endif
-        Log3(("IOMIOPortWriteStr: Port=%RTiop pGCPtrSrc=%p pcTransfer=%p:{%#x->%#x} cb=%d rc=%Rrc\n",
-              Port, pGCPtrSrc, pcTransfers, cTransfers, *pcTransfers, cb, rc));
-        return rc;
+        Log3(("IOMIOPortWriteStr: Port=%RTiop pGCPtrSrc=%p pcTransfer=%p:{%#x->%#x} cb=%d rcStrict=%Rrc\n",
+              Port, pGCPtrSrc, pcTransfers, cTransfers, *pcTransfers, cb, VBOXSTRICTRC_VAL(rcStrict)));
+        if (!pCritSect)
+            iomUnlock(pVM);
+        return rcStrict;
     }
 
 #ifndef IN_RING3
@@ -661,8 +898,9 @@ VMMDECL(int) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PR
     {
 # ifdef VBOX_WITH_STATISTICS
         if (pStats)
-            STAM_COUNTER_INC(&pStats->CTX_MID_Z(Out,ToR3));
+            STAM_COUNTER_INC(&pStats->OutRZToR3);
 # endif
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
     }
 #endif
@@ -677,6 +915,7 @@ VMMDECL(int) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PR
     {
 # ifndef IN_RING3
         /* Ring-3 will have to create the statistics record. */
+        iomUnlock(pVM);
         return VINF_IOM_HC_IOPORT_WRITE;
 # else
         pStats = iomR3IOPortStatsCreate(pVM, Port, NULL);
@@ -688,6 +927,7 @@ VMMDECL(int) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PR
 
     Log3(("IOMIOPortWriteStr: Port=%RTiop pGCPtrSrc=%p pcTransfer=%p:{%#x->%#x} cb=%d rc=VINF_SUCCESS\n",
           Port, pGCPtrSrc, pcTransfers, cTransfers, *pcTransfers, cb));
+    iomUnlock(pVM);
     return VINF_SUCCESS;
 }
 
@@ -708,13 +948,15 @@ VMMDECL(int) IOMIOPortWriteString(PVM pVM, RTIOPORT Port, PRTGCPTR pGCPtrSrc, PR
  * @param   Port        The I/O port number.
  * @param   cb          The access size.
  */
-VMMDECL(int) IOMInterpretCheckPortIOAccess(PVM pVM, PCPUMCTXCORE pCtxCore, RTIOPORT Port, unsigned cb)
+VMMDECL(VBOXSTRICTRC) IOMInterpretCheckPortIOAccess(PVM pVM, PCPUMCTXCORE pCtxCore, RTIOPORT Port, unsigned cb)
 {
+    PVMCPU pVCpu = VMMGetCpu(pVM);
+
     /*
      * If this isn't ring-0, we have to check for I/O privileges.
      */
-    uint32_t efl = CPUMRawGetEFlags(pVM, pCtxCore);
-    uint32_t cpl = CPUMGetGuestCPL(pVM, pCtxCore);
+    uint32_t efl = CPUMRawGetEFlags(pVCpu, pCtxCore);
+    uint32_t cpl = CPUMGetGuestCPL(pVCpu, pCtxCore);
 
     if (    (    cpl > 0
              &&  X86_EFL_GET_IOPL(efl) < cpl)
@@ -727,11 +969,11 @@ VMMDECL(int) IOMInterpretCheckPortIOAccess(PVM pVM, PCPUMCTXCORE pCtxCore, RTIOP
         RTGCUINTPTR GCPtrTss;
         RTGCUINTPTR cbTss;
         bool        fCanHaveIOBitmap;
-        int rc = SELMGetTSSInfo(pVM, &GCPtrTss, &cbTss, &fCanHaveIOBitmap);
-        if (RT_FAILURE(rc))
+        int rc2 = SELMGetTSSInfo(pVM, pVCpu, &GCPtrTss, &cbTss, &fCanHaveIOBitmap);
+        if (RT_FAILURE(rc2))
         {
-            Log(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d %Rrc -> #GP(0)\n", Port, cb, rc));
-            return TRPMRaiseXcptErr(pVM, pCtxCore, X86_XCPT_GP, 0);
+            Log(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d %Rrc -> #GP(0)\n", Port, cb, rc2));
+            return TRPMRaiseXcptErr(pVCpu, pCtxCore, X86_XCPT_GP, 0);
         }
 
         if (    !fCanHaveIOBitmap
@@ -739,19 +981,19 @@ VMMDECL(int) IOMInterpretCheckPortIOAccess(PVM pVM, PCPUMCTXCORE pCtxCore, RTIOP
         {
             Log(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d cbTss=%#x fCanHaveIOBitmap=%RTbool -> #GP(0)\n",
                  Port, cb, cbTss, fCanHaveIOBitmap));
-            return TRPMRaiseXcptErr(pVM, pCtxCore, X86_XCPT_GP, 0);
+            return TRPMRaiseXcptErr(pVCpu, pCtxCore, X86_XCPT_GP, 0);
         }
 
         /*
          * Fetch the I/O bitmap offset.
          */
         uint16_t offIOPB;
-        rc = PGMPhysInterpretedRead(pVM, pCtxCore, &offIOPB, GCPtrTss + RT_OFFSETOF(VBOXTSS, offIoBitmap), sizeof(offIOPB));
-        if (rc != VINF_SUCCESS)
+        VBOXSTRICTRC rcStrict = PGMPhysInterpretedRead(pVCpu, pCtxCore, &offIOPB, GCPtrTss + RT_OFFSETOF(VBOXTSS, offIoBitmap), sizeof(offIOPB));
+        if (rcStrict != VINF_SUCCESS)
         {
             Log(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d GCPtrTss=%RGv %Rrc\n",
-                 Port, cb, GCPtrTss, rc));
-            return rc;
+                 Port, cb, GCPtrTss, VBOXSTRICTRC_VAL(rcStrict)));
+            return rcStrict;
         }
 
         /*
@@ -762,15 +1004,15 @@ VMMDECL(int) IOMInterpretCheckPortIOAccess(PVM pVM, PCPUMCTXCORE pCtxCore, RTIOP
         {
             Log(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d offTss=%#x cbTss=%#x -> #GP(0)\n",
                  Port, cb, offTss, cbTss));
-            return TRPMRaiseXcptErr(pVM, pCtxCore, X86_XCPT_GP, 0);
+            return TRPMRaiseXcptErr(pVCpu, pCtxCore, X86_XCPT_GP, 0);
         }
         uint16_t u16;
-        rc = PGMPhysInterpretedRead(pVM, pCtxCore, &u16, GCPtrTss + offTss, sizeof(u16));
-        if (rc != VINF_SUCCESS)
+        rcStrict = PGMPhysInterpretedRead(pVCpu, pCtxCore, &u16, GCPtrTss + offTss, sizeof(u16));
+        if (rcStrict != VINF_SUCCESS)
         {
             Log(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d GCPtrTss=%RGv offTss=%#x -> %Rrc\n",
-                 Port, cb, GCPtrTss, offTss, rc));
-            return rc;
+                 Port, cb, GCPtrTss, offTss, VBOXSTRICTRC_VAL(rcStrict)));
+            return rcStrict;
         }
 
         /*
@@ -780,7 +1022,7 @@ VMMDECL(int) IOMInterpretCheckPortIOAccess(PVM pVM, PCPUMCTXCORE pCtxCore, RTIOP
         {
             Log(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d u16=%#x -> #GP(0)\n",
                  Port, cb, u16, offTss));
-            return TRPMRaiseXcptErr(pVM, pCtxCore, X86_XCPT_GP, 0);
+            return TRPMRaiseXcptErr(pVCpu, pCtxCore, X86_XCPT_GP, 0);
         }
         LogFlow(("iomInterpretCheckPortIOAccess: Port=%RTiop cb=%d offTss=%#x cbTss=%#x u16=%#x -> OK\n",
                  Port, cb, u16, offTss, cbTss));
@@ -806,7 +1048,7 @@ VMMDECL(int) IOMInterpretCheckPortIOAccess(PVM pVM, PCPUMCTXCORE pCtxCore, RTIOP
  * @param   pRegFrame   Pointer to CPUMCTXCORE guest registers structure.
  * @param   pCpu        Disassembler CPU state.
  */
-VMMDECL(int) IOMInterpretIN(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
+VMMDECL(VBOXSTRICTRC) IOMInterpretIN(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
 {
 #ifdef IN_RC
     STAM_COUNTER_INC(&pVM->iom.s.StatInstIn);
@@ -823,15 +1065,15 @@ VMMDECL(int) IOMInterpretIN(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
 
     cbSize = DISGetParamSize(pCpu, &pCpu->param1);
     Assert(cbSize > 0);
-    int rc = IOMInterpretCheckPortIOAccess(pVM, pRegFrame, uPort, cbSize);
-    if (rc == VINF_SUCCESS)
+    VBOXSTRICTRC rcStrict = IOMInterpretCheckPortIOAccess(pVM, pRegFrame, uPort, cbSize);
+    if (rcStrict == VINF_SUCCESS)
     {
         /*
          * Attemp to read the port.
          */
         uint32_t u32Data = UINT32_C(0xffffffff);
-        rc = IOMIOPortRead(pVM, uPort, &u32Data, cbSize);
-        if (IOM_SUCCESS(rc))
+        rcStrict = IOMIOPortRead(pVM, uPort, &u32Data, cbSize);
+        if (IOM_SUCCESS(rcStrict))
         {
             /*
              * Store the result in the AL|AX|EAX register.
@@ -840,11 +1082,12 @@ VMMDECL(int) IOMInterpretIN(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
             AssertMsg(fRc, ("Failed to store register value!\n")); NOREF(fRc);
         }
         else
-            AssertMsg(rc == VINF_IOM_HC_IOPORT_READ || RT_FAILURE(rc), ("%Rrc\n", rc));
+            AssertMsg(rcStrict == VINF_IOM_HC_IOPORT_READ || RT_FAILURE(rcStrict), ("%Rrc\n", VBOXSTRICTRC_VAL(rcStrict)));
     }
     else
-        AssertMsg(rc == VINF_EM_RAW_GUEST_TRAP || rc == VINF_TRPM_XCPT_DISPATCHED || rc == VINF_TRPM_XCPT_DISPATCHED || RT_FAILURE(rc), ("%Rrc\n", rc));
-    return rc;
+        AssertMsg(rcStrict == VINF_EM_RAW_GUEST_TRAP || rcStrict == VINF_TRPM_XCPT_DISPATCHED || rcStrict == VINF_TRPM_XCPT_DISPATCHED || RT_FAILURE(rcStrict), ("%Rrc\n", VBOXSTRICTRC_VAL(rcStrict)));
+
+    return rcStrict;
 }
 
 
@@ -865,7 +1108,7 @@ VMMDECL(int) IOMInterpretIN(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
  * @param   pRegFrame   Pointer to CPUMCTXCORE guest registers structure.
  * @param   pCpu        Disassembler CPU state.
  */
-VMMDECL(int) IOMInterpretOUT(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
+VMMDECL(VBOXSTRICTRC) IOMInterpretOUT(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
 {
 #ifdef IN_RC
     STAM_COUNTER_INC(&pVM->iom.s.StatInstOut);
@@ -880,8 +1123,8 @@ VMMDECL(int) IOMInterpretOUT(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
     bool fRc = iomGetRegImmData(pCpu, &pCpu->param1, pRegFrame, &uPort, &cbSize);
     AssertMsg(fRc, ("Failed to get reg/imm port number!\n")); NOREF(fRc);
 
-    int rc = IOMInterpretCheckPortIOAccess(pVM, pRegFrame, uPort, cbSize);
-    if (rc == VINF_SUCCESS)
+    VBOXSTRICTRC rcStrict = IOMInterpretCheckPortIOAccess(pVM, pRegFrame, uPort, cbSize);
+    if (rcStrict == VINF_SUCCESS)
     {
         uint64_t u64Data = 0;
         fRc = iomGetRegImmData(pCpu, &pCpu->param2, pRegFrame, &u64Data, &cbSize);
@@ -890,10 +1133,10 @@ VMMDECL(int) IOMInterpretOUT(PVM pVM, PCPUMCTXCORE pRegFrame, PDISCPUSTATE pCpu)
         /*
          * Attempt to write to the port.
          */
-        rc = IOMIOPortWrite(pVM, uPort, u64Data, cbSize);
-        AssertMsg(rc == VINF_SUCCESS || rc == VINF_IOM_HC_IOPORT_WRITE || (rc >= VINF_EM_FIRST && rc <= VINF_EM_LAST) || RT_FAILURE(rc), ("%Rrc\n", rc));
+        rcStrict = IOMIOPortWrite(pVM, uPort, u64Data, cbSize);
+        AssertMsg(rcStrict == VINF_SUCCESS || rcStrict == VINF_IOM_HC_IOPORT_WRITE || (rcStrict >= VINF_EM_FIRST && rcStrict <= VINF_EM_LAST) || RT_FAILURE(rcStrict), ("%Rrc\n", VBOXSTRICTRC_VAL(rcStrict)));
     }
     else
-        AssertMsg(rc == VINF_EM_RAW_GUEST_TRAP || rc == VINF_TRPM_XCPT_DISPATCHED || rc == VINF_TRPM_XCPT_DISPATCHED || RT_FAILURE(rc), ("%Rrc\n", rc));
-    return rc;
+        AssertMsg(rcStrict == VINF_EM_RAW_GUEST_TRAP || rcStrict == VINF_TRPM_XCPT_DISPATCHED || rcStrict == VINF_TRPM_XCPT_DISPATCHED || RT_FAILURE(rcStrict), ("%Rrc\n", VBOXSTRICTRC_VAL(rcStrict)));
+    return rcStrict;
 }
