@@ -1,10 +1,10 @@
-/* $Id: MMAll.cpp 15174 2008-12-09 14:11:35Z vboxsync $ */
+/* $Id: MMAll.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * MM - Memory Manager - Any Context.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -30,6 +26,7 @@
 #include <VBox/vm.h>
 #include <VBox/log.h>
 #include <iprt/assert.h>
+#include <iprt/string.h>
 
 
 
@@ -184,6 +181,7 @@ DECLINLINE(PMMLOOKUPHYPER) mmHyperLookupRC(PVM pVM, RTRCPTR RCPtr, uint32_t *pof
                     break;
             }
             AssertMsgFailed(("enmType=%d\n", pLookup->enmType));
+            *poff = 0; /* shut up gcc */
             return NULL;
         }
 
@@ -194,6 +192,7 @@ DECLINLINE(PMMLOOKUPHYPER) mmHyperLookupRC(PVM pVM, RTRCPTR RCPtr, uint32_t *pof
     }
 
     AssertMsgFailed(("RCPtr=%RRv is not inside the hypervisor memory area!\n", RCPtr));
+    *poff = 0; /* shut up gcc */
     return NULL;
 }
 
@@ -286,7 +285,7 @@ DECLINLINE(RTR0PTR) mmHyperLookupCalcR0(PVM pVM, PMMLOOKUPHYPER pLookup, uint32_
  */
 DECLINLINE(RTRCPTR) mmHyperLookupCalcRC(PVM pVM, PMMLOOKUPHYPER pLookup, uint32_t off)
 {
-    return (RTRCPTR)((RTGCUINTPTR)pVM->mm.s.pvHyperAreaGC + pLookup->off + off);
+    return (RTRCPTR)((RTRCUINTPTR)pVM->mm.s.pvHyperAreaGC + pLookup->off + off);
 }
 
 
@@ -409,6 +408,7 @@ VMMDECL(RTRCPTR) MMHyperR3ToRC(PVM pVM, RTR3PTR R3Ptr)
 }
 
 
+#ifndef IN_RING3
 /**
  * Converts a ring-3 host context address in the Hypervisor memory region to a current context address.
  *
@@ -418,7 +418,6 @@ VMMDECL(RTRCPTR) MMHyperR3ToRC(PVM pVM, RTR3PTR R3Ptr)
  *                      You'll be damned if this is not in the HMA! :-)
  * @thread  The Emulation Thread.
  */
-#ifndef IN_RING3
 VMMDECL(void *) MMHyperR3ToCC(PVM pVM, RTR3PTR R3Ptr)
 {
     uint32_t off;
@@ -467,7 +466,7 @@ VMMDECL(RTR0PTR) MMHyperRCToR0(PVM pVM, RTRCPTR RCPtr)
     return NIL_RTR0PTR;
 }
 
-
+#ifndef IN_RC
 /**
  * Converts a raw-mode context address in the Hypervisor memory region to a current context address.
  *
@@ -477,7 +476,6 @@ VMMDECL(RTR0PTR) MMHyperRCToR0(PVM pVM, RTRCPTR RCPtr)
  *                      You'll be damned if this is not in the HMA! :-)
  * @thread  The Emulation Thread.
  */
-#ifndef IN_RC
 VMMDECL(void *) MMHyperRCToCC(PVM pVM, RTRCPTR RCPtr)
 {
     uint32_t off;
@@ -488,8 +486,7 @@ VMMDECL(void *) MMHyperRCToCC(PVM pVM, RTRCPTR RCPtr)
 }
 #endif
 
-
-
+#ifndef IN_RING3
 /**
  * Converts a current context address in the Hypervisor memory region to a ring-3 host context address.
  *
@@ -499,7 +496,6 @@ VMMDECL(void *) MMHyperRCToCC(PVM pVM, RTRCPTR RCPtr)
  *                      You'll be damned if this is not in the HMA! :-)
  * @thread  The Emulation Thread.
  */
-#ifndef IN_RING3
 VMMDECL(RTR3PTR) MMHyperCCToR3(PVM pVM, void *pv)
 {
     uint32_t off;
@@ -510,6 +506,7 @@ VMMDECL(RTR3PTR) MMHyperCCToR3(PVM pVM, void *pv)
 }
 #endif
 
+#ifndef IN_RING0
 /**
  * Converts a current context address in the Hypervisor memory region to a ring-0 host context address.
  *
@@ -519,7 +516,6 @@ VMMDECL(RTR3PTR) MMHyperCCToR3(PVM pVM, void *pv)
  *                      You'll be damned if this is not in the HMA! :-)
  * @thread  The Emulation Thread.
  */
-#ifndef IN_RING0
 VMMDECL(RTR0PTR) MMHyperCCToR0(PVM pVM, void *pv)
 {
     uint32_t off;
@@ -531,6 +527,7 @@ VMMDECL(RTR0PTR) MMHyperCCToR0(PVM pVM, void *pv)
 #endif
 
 
+#ifndef IN_RC
 /**
  * Converts a current context address in the Hypervisor memory region to a raw-mode context address.
  *
@@ -540,7 +537,6 @@ VMMDECL(RTR0PTR) MMHyperCCToR0(PVM pVM, void *pv)
  *                      You'll be damned if this is not in the HMA! :-)
  * @thread  The Emulation Thread.
  */
-#ifndef IN_RC
 VMMDECL(RTRCPTR) MMHyperCCToRC(PVM pVM, void *pv)
 {
     uint32_t off;
@@ -551,3 +547,108 @@ VMMDECL(RTRCPTR) MMHyperCCToRC(PVM pVM, void *pv)
 }
 #endif
 
+
+/**
+ * Gets the string name of a memory tag.
+ *
+ * @returns name of enmTag.
+ * @param   enmTag      The tag.
+ */
+const char *mmGetTagName(MMTAG enmTag)
+{
+    switch (enmTag)
+    {
+        #define TAG2STR(tag) case MM_TAG_##tag: return #tag
+
+        TAG2STR(CFGM);
+        TAG2STR(CFGM_BYTES);
+        TAG2STR(CFGM_STRING);
+        TAG2STR(CFGM_USER);
+
+        TAG2STR(CSAM);
+        TAG2STR(CSAM_PATCH);
+
+        TAG2STR(DBGF);
+        TAG2STR(DBGF_AS);
+        TAG2STR(DBGF_INFO);
+        TAG2STR(DBGF_LINE);
+        TAG2STR(DBGF_LINE_DUP);
+        TAG2STR(DBGF_MODULE);
+        TAG2STR(DBGF_OS);
+        TAG2STR(DBGF_STACK);
+        TAG2STR(DBGF_SYMBOL);
+        TAG2STR(DBGF_SYMBOL_DUP);
+
+        TAG2STR(EM);
+
+        TAG2STR(IOM);
+        TAG2STR(IOM_STATS);
+
+        TAG2STR(MM);
+        TAG2STR(MM_LOOKUP_GUEST);
+        TAG2STR(MM_LOOKUP_PHYS);
+        TAG2STR(MM_LOOKUP_VIRT);
+        TAG2STR(MM_PAGE);
+
+        TAG2STR(PARAV);
+
+        TAG2STR(PATM);
+        TAG2STR(PATM_PATCH);
+
+        TAG2STR(PDM);
+        TAG2STR(PDM_DEVICE);
+        TAG2STR(PDM_DEVICE_DESC);
+        TAG2STR(PDM_DEVICE_USER);
+        TAG2STR(PDM_DRIVER);
+        TAG2STR(PDM_DRIVER_DESC);
+        TAG2STR(PDM_DRIVER_USER);
+        TAG2STR(PDM_USB);
+        TAG2STR(PDM_USB_DESC);
+        TAG2STR(PDM_USB_USER);
+        TAG2STR(PDM_LUN);
+        TAG2STR(PDM_QUEUE);
+        TAG2STR(PDM_THREAD);
+        TAG2STR(PDM_ASYNC_COMPLETION);
+
+        TAG2STR(PGM);
+        TAG2STR(PGM_CHUNK_MAPPING);
+        TAG2STR(PGM_HANDLERS);
+        TAG2STR(PGM_MAPPINGS);
+        TAG2STR(PGM_PHYS);
+        TAG2STR(PGM_POOL);
+
+        TAG2STR(REM);
+
+        TAG2STR(SELM);
+
+        TAG2STR(SSM);
+
+        TAG2STR(STAM);
+
+        TAG2STR(TM);
+
+        TAG2STR(TRPM);
+
+        TAG2STR(VM);
+        TAG2STR(VM_REQ);
+
+        TAG2STR(VMM);
+
+        TAG2STR(HWACCM);
+
+        #undef TAG2STR
+
+        default:
+        {
+            AssertMsgFailed(("Unknown tag %d! forgot to add it to the switch?\n", enmTag));
+#ifdef IN_RING3
+            static char sz[48];
+            RTStrPrintf(sz, sizeof(sz), "%d", enmTag);
+            return sz;
+#else
+            return "unknown tag!";
+#endif
+        }
+    }
+}
+
