@@ -1,10 +1,10 @@
-/* $Id: PGMAll.cpp $ */
+/* $Id: PGMAll.cpp 29250 2010-05-09 17:53:58Z vboxsync $ */
 /** @file
  * PGM - Page Manager and Monitor - All context code.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -37,10 +33,11 @@
 #include <VBox/em.h>
 #include <VBox/hwaccm.h>
 #include <VBox/hwacc_vmx.h>
-#include "PGMInternal.h"
+#include "../PGMInternal.h"
 #include <VBox/vm.h>
+#include "../PGMInline.h"
 #include <iprt/assert.h>
-#include <iprt/asm.h>
+#include <iprt/asm-amd64-x86.h>
 #include <iprt/string.h>
 #include <VBox/log.h>
 #include <VBox/param.h>
@@ -402,7 +399,7 @@ VMMDECL(int) PGMTrap0eHandler(PVMCPU pVCpu, RTGCUINT uErr, PCPUMCTXCORE pRegFram
 {
     PVM pVM = pVCpu->CTX_SUFF(pVM);
 
-    LogFlow(("PGMTrap0eHandler: uErr=%RGu pvFault=%RGv eip=%04x:%RGv\n", uErr, pvFault, pRegFrame->cs, (RTGCPTR)pRegFrame->rip));
+    Log(("PGMTrap0eHandler: uErr=%RGx pvFault=%RGv eip=%04x:%RGv\n", uErr, pvFault, pRegFrame->cs, (RTGCPTR)pRegFrame->rip));
     STAM_PROFILE_START(&pVCpu->pgm.s.StatRZTrap0e, a);
     STAM_STATS({ pVCpu->pgm.s.CTX_SUFF(pStatTrap0eAttribution) = NULL; } );
 
@@ -450,19 +447,25 @@ VMMDECL(int) PGMTrap0eHandler(PVMCPU pVCpu, RTGCUINT uErr, PCPUMCTXCORE pRegFram
     /*
      * Call the worker.
      */
-    pgmLock(pVM);
-    int rc = PGM_BTH_PFN(Trap0eHandler, pVCpu)(pVCpu, uErr, pRegFrame, pvFault);
-    Assert(PGMIsLockOwner(pVM));
-    pgmUnlock(pVM);
+    bool fLockTaken = false;
+    int rc = PGM_BTH_PFN(Trap0eHandler, pVCpu)(pVCpu, uErr, pRegFrame, pvFault, &fLockTaken);
+    if (fLockTaken)
+    {
+        Assert(PGMIsLockOwner(pVM));
+        pgmUnlock(pVM);
+    }
     if (rc == VINF_PGM_SYNCPAGE_MODIFIED_PDE)
         rc = VINF_SUCCESS;
 
 # ifdef IN_RING0
     /* Note: hack alert for difficult to reproduce problem. */
-    if (    pVM->cCPUs > 1
-        &&  rc == VERR_PAGE_TABLE_NOT_PRESENT)
+    if (    rc == VERR_PAGE_NOT_PRESENT                 /* SMP only ; disassembly might fail. */
+        ||  rc == VERR_PAGE_TABLE_NOT_PRESENT           /* seen with UNI & SMP */
+        ||  rc == VERR_PAGE_DIRECTORY_PTR_NOT_PRESENT   /* seen with SMP */
+        ||  rc == VERR_PAGE_MAP_LEVEL4_NOT_PRESENT)     /* precaution */
     {
-        Log(("WARNING: Unexpected VERR_PAGE_TABLE_NOT_PRESENT for page fault at %RGv error code %x (rip=%RGv)\n", pvFault, uErr, pRegFrame->rip));
+        Log(("WARNING: Unexpected VERR_PAGE_TABLE_NOT_PRESENT (%d) for page fault at %RGv error code %x (rip=%RGv)\n", rc, pvFault, uErr, pRegFrame->rip));
+        /* Some kind of inconsistency in the SMP case; it's safe to just execute the instruction again; not sure about single VCPU VMs though. */
         rc = VINF_SUCCESS;
     }
 # endif
@@ -707,11 +710,8 @@ VMMDECL(int) PGMInvalidatePage(PVMCPU pVCpu, RTGCPTR GCPtrPage)
 #ifndef IN_RING3
     /*
      * Notify the recompiler so it can record this instruction.
-     * Failure happens when it's out of space. We'll return to HC in that case.
      */
-    rc = REMNotifyInvalidatePage(pVM, GCPtrPage);
-    if (rc != VINF_SUCCESS)
-        return rc;
+    REMNotifyInvalidatePage(pVM, GCPtrPage);
 #endif /* !IN_RING3 */
 
 
@@ -719,7 +719,7 @@ VMMDECL(int) PGMInvalidatePage(PVMCPU pVCpu, RTGCPTR GCPtrPage)
     /*
      * Check for conflicts and pending CR3 monitoring updates.
      */
-    if (!pVM->pgm.s.fMappingsFixed)
+    if (pgmMapAreMappingsFloating(&pVM->pgm.s))
     {
         if (    pgmGetMapping(pVM, GCPtrPage)
             &&  PGMGstGetPage(pVCpu, GCPtrPage, NULL, NULL) != VERR_PAGE_TABLE_NOT_PRESENT)
@@ -748,6 +748,9 @@ VMMDECL(int) PGMInvalidatePage(PVMCPU pVCpu, RTGCPTR GCPtrPage)
     pgmUnlock(pVM);
     STAM_PROFILE_STOP(&pVCpu->pgm.s.CTX_MID_Z(Stat,InvalidatePage), a);
 
+    /* Invalidate the TLB entry; might already be done by InvalidatePage (@todo) */
+    PGM_INVL_PG(pVCpu, GCPtrPage);
+
 #ifdef IN_RING3
     /*
      * Check if we have a pending update of the CR3 monitoring.
@@ -756,7 +759,7 @@ VMMDECL(int) PGMInvalidatePage(PVMCPU pVCpu, RTGCPTR GCPtrPage)
         &&  (pVCpu->pgm.s.fSyncFlags & PGM_SYNC_MONITOR_CR3))
     {
         pVCpu->pgm.s.fSyncFlags &= ~PGM_SYNC_MONITOR_CR3;
-        Assert(!pVM->pgm.s.fMappingsFixed);
+        Assert(!pVM->pgm.s.fMappingsFixed); Assert(!pVM->pgm.s.fMappingsDisabled);
     }
 
     /*
@@ -767,6 +770,14 @@ VMMDECL(int) PGMInvalidatePage(PVMCPU pVCpu, RTGCPTR GCPtrPage)
      */
     CSAMR3FlushPage(pVM, GCPtrPage);
 #endif /* IN_RING3 */
+
+    /* Ignore all irrelevant error codes. */
+    if (    rc == VERR_PAGE_NOT_PRESENT
+        ||  rc == VERR_PAGE_TABLE_NOT_PRESENT
+        ||  rc == VERR_PAGE_DIRECTORY_PTR_NOT_PRESENT
+        ||  rc == VERR_PAGE_MAP_LEVEL4_NOT_PRESENT)
+        rc = VINF_SUCCESS;
+
     return rc;
 }
 
@@ -889,8 +900,6 @@ int pgmShwSyncPaePDPtr(PVMCPU pVCpu, RTGCPTR GCPtr, PX86PDPE pGstPdpe, PX86PDPAE
     if (    !pPdpe->n.u1Present
         &&  !(pPdpe->u & X86_PDPE_PG_MASK))
     {
-        bool        fNestedPaging = HWACCMIsNestedPagingActive(pVM);
-        bool        fPaging       = !!(CPUMGetGuestCR0(pVCpu) & X86_CR0_PG);
         RTGCPTR64   GCPdPt;
         PGMPOOLKIND enmKind;
 
@@ -899,7 +908,7 @@ int pgmShwSyncPaePDPtr(PVMCPU pVCpu, RTGCPTR GCPtr, PX86PDPE pGstPdpe, PX86PDPAE
         PGMDynLockHCPage(pVM, (uint8_t *)pPdpe);
 # endif
 
-        if (fNestedPaging || !fPaging)
+        if (HWACCMIsNestedPagingActive(pVM) || !CPUMIsGuestPagingEnabled(pVCpu))
         {
             /* AMD-V nested paging or real/protected mode without paging */
             GCPdPt  = (RTGCPTR64)iPdPt << X86_PDPT_SHIFT;
@@ -1019,8 +1028,7 @@ int pgmShwSyncLongModePDPtr(PVMCPU pVCpu, RTGCPTR64 GCPtr, PX86PML4E pGstPml4e,
     PPGMPOOL       pPool         = pVM->pgm.s.CTX_SUFF(pPool);
     const unsigned iPml4         = (GCPtr >> X86_PML4_SHIFT) & X86_PML4_MASK;
     PX86PML4E      pPml4e        = pgmShwGetLongModePML4EPtr(pPGM, iPml4);
-    bool           fNestedPaging = HWACCMIsNestedPagingActive(pVM);
-    bool           fPaging       = !!(CPUMGetGuestCR0(pVCpu) & X86_CR0_PG);
+    bool           fNestedPagingOrNoGstPaging = HWACCMIsNestedPagingActive(pVM) || !CPUMIsGuestPagingEnabled(pVCpu);
     PPGMPOOLPAGE   pShwPage;
     int            rc;
 
@@ -1035,7 +1043,7 @@ int pgmShwSyncLongModePDPtr(PVMCPU pVCpu, RTGCPTR64 GCPtr, PX86PML4E pGstPml4e,
 
         Assert(pVCpu->pgm.s.CTX_SUFF(pShwPageCR3));
 
-        if (fNestedPaging || !fPaging)
+        if (fNestedPagingOrNoGstPaging)
         {
             /* AMD-V nested paging or real/protected mode without paging */
             GCPml4  = (RTGCPTR64)iPml4 << X86_PML4_SHIFT;
@@ -1075,7 +1083,7 @@ int pgmShwSyncLongModePDPtr(PVMCPU pVCpu, RTGCPTR64 GCPtr, PX86PML4E pGstPml4e,
         RTGCPTR64   GCPdPt;
         PGMPOOLKIND enmKind;
 
-        if (fNestedPaging || !fPaging)
+        if (fNestedPagingOrNoGstPaging)
         {
             /* AMD-V nested paging or real/protected mode without paging */
             GCPdPt  = (RTGCPTR64)iPdPt << X86_PDPT_SHIFT;
@@ -1130,7 +1138,7 @@ DECLINLINE(int) pgmShwGetLongModePDPtr(PVMCPU pVCpu, RTGCPTR64 GCPtr, PX86PML4E
     if (ppPml4e)
         *ppPml4e = (PX86PML4E)pPml4e;
 
-    Log4(("pgmShwGetLongModePDPtr %VGv (%VHv) %RX64\n", GCPtr, pPml4e, pPml4e->u));
+    Log4(("pgmShwGetLongModePDPtr %RGv (%RHv) %RX64\n", GCPtr, pPml4e, pPml4e->u));
 
     if (!pPml4e->n.u1Present)
         return VERR_PAGE_MAP_LEVEL4_NOT_PRESENT;
@@ -1678,10 +1686,8 @@ VMMDECL(int) PGMFlushTLB(PVMCPU pVCpu, uint64_t cr3, bool fGlobal)
         rc = PGM_BTH_PFN(MapCR3, pVCpu)(pVCpu, GCPhysCR3);
         if (RT_LIKELY(rc == VINF_SUCCESS))
         {
-            if (!pVM->pgm.s.fMappingsFixed)
-            {
+            if (pgmMapAreMappingsFloating(&pVM->pgm.s))
                 pVCpu->pgm.s.fSyncFlags &= ~PGM_SYNC_MONITOR_CR3;
-            }
         }
         else
         {
@@ -1689,7 +1695,7 @@ VMMDECL(int) PGMFlushTLB(PVMCPU pVCpu, uint64_t cr3, bool fGlobal)
             Assert(VMCPU_FF_ISPENDING(pVCpu, VMCPU_FF_PGM_SYNC_CR3_NON_GLOBAL | VMCPU_FF_PGM_SYNC_CR3));
             pVCpu->pgm.s.GCPhysCR3 = GCPhysOldCR3;
             pVCpu->pgm.s.fSyncFlags |= PGM_SYNC_MAP_CR3;
-            if (!pVM->pgm.s.fMappingsFixed)
+            if (pgmMapAreMappingsFloating(&pVM->pgm.s))
                 pVCpu->pgm.s.fSyncFlags |= PGM_SYNC_MONITOR_CR3;
         }
 
@@ -1700,13 +1706,22 @@ VMMDECL(int) PGMFlushTLB(PVMCPU pVCpu, uint64_t cr3, bool fGlobal)
     }
     else
     {
+# ifdef PGMPOOL_WITH_OPTIMIZED_DIRTY_PT
+        PPGMPOOL pPool = pVM->pgm.s.CTX_SUFF(pPool);
+        if (pPool->cDirtyPages)
+        {
+            pgmLock(pVM);
+            pgmPoolResetDirtyPages(pVM);
+            pgmUnlock(pVM);
+        }
+# endif
         /*
          * Check if we have a pending update of the CR3 monitoring.
          */
         if (pVCpu->pgm.s.fSyncFlags & PGM_SYNC_MONITOR_CR3)
         {
             pVCpu->pgm.s.fSyncFlags &= ~PGM_SYNC_MONITOR_CR3;
-            Assert(!pVM->pgm.s.fMappingsFixed);
+            Assert(!pVM->pgm.s.fMappingsFixed); Assert(!pVM->pgm.s.fMappingsDisabled);
         }
         if (fGlobal)
             STAM_COUNTER_INC(&pVCpu->pgm.s.CTX_MID_Z(Stat,FlushTLBSameCR3Global));
@@ -1743,9 +1758,9 @@ VMMDECL(int) PGMUpdateCR3(PVMCPU pVCpu, uint64_t cr3)
     LogFlow(("PGMUpdateCR3: cr3=%RX64 OldCr3=%RX64\n", cr3, pVCpu->pgm.s.GCPhysCR3));
 
     /* We assume we're only called in nested paging mode. */
-    Assert(pVM->pgm.s.fMappingsFixed);
-    Assert(!(pVCpu->pgm.s.fSyncFlags & PGM_SYNC_MONITOR_CR3));
     Assert(HWACCMIsNestedPagingActive(pVM) || pVCpu->pgm.s.enmShadowMode == PGMMODE_EPT);
+    Assert(pVM->pgm.s.fMappingsDisabled);
+    Assert(!(pVCpu->pgm.s.fSyncFlags & PGM_SYNC_MONITOR_CR3));
 
     /*
      * Remap the CR3 content and adjust the monitoring if CR3 was actually changed.
@@ -1795,7 +1810,6 @@ VMMDECL(int) PGMSyncCR3(PVMCPU pVCpu, uint64_t cr0, uint64_t cr3, uint64_t cr4,
     PVM pVM = pVCpu->CTX_SUFF(pVM);
     int rc;
 
-#ifdef PGMPOOL_WITH_MONITORING
     /*
      * The pool may have pending stuff and even require a return to ring-3 to
      * clear the whole thing.
@@ -1803,7 +1817,6 @@ VMMDECL(int) PGMSyncCR3(PVMCPU pVCpu, uint64_t cr0, uint64_t cr3, uint64_t cr4,
     rc = pgmPoolSyncCR3(pVCpu);
     if (rc != VINF_SUCCESS)
         return rc;
-#endif
 
     /*
      * We might be called when we shouldn't.
@@ -1816,6 +1829,7 @@ VMMDECL(int) PGMSyncCR3(PVMCPU pVCpu, uint64_t cr0, uint64_t cr3, uint64_t cr4,
     if (pVCpu->pgm.s.enmGuestMode <= PGMMODE_PROTECTED)
     {
         Assert((cr0 & (X86_CR0_PG | X86_CR0_PE)) != (X86_CR0_PG | X86_CR0_PE));
+        Assert(!(pVCpu->pgm.s.fSyncFlags & PGM_SYNC_CLEAR_PGM_POOL));
         VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_PGM_SYNC_CR3);
         VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_PGM_SYNC_CR3_NON_GLOBAL);
         return VINF_SUCCESS;
@@ -1857,16 +1871,19 @@ VMMDECL(int) PGMSyncCR3(PVMCPU pVCpu, uint64_t cr0, uint64_t cr3, uint64_t cr4,
             pVCpu->pgm.s.GCPhysCR3 = GCPhysCR3;
             rc = PGM_BTH_PFN(MapCR3, pVCpu)(pVCpu, GCPhysCR3);
         }
+        /* Make sure we check for pending pgm pool syncs as we clear VMCPU_FF_PGM_SYNC_CR3 later on! */
+        if (    rc == VINF_PGM_SYNC_CR3
+            ||  (pVCpu->pgm.s.fSyncFlags & PGM_SYNC_CLEAR_PGM_POOL))
+        {
+            Log(("PGMSyncCR3: pending pgm pool sync after MapCR3!\n"));
 #ifdef IN_RING3
-        if (rc == VINF_PGM_SYNC_CR3)
             rc = pgmPoolSyncCR3(pVCpu);
 #else
-        if (rc == VINF_PGM_SYNC_CR3)
-        {
-            pVCpu->pgm.s.GCPhysCR3 = GCPhysCR3Old;
-            return rc;
-        }
+            if (rc == VINF_PGM_SYNC_CR3)
+                pVCpu->pgm.s.GCPhysCR3 = GCPhysCR3Old;
+            return VINF_PGM_SYNC_CR3;
 #endif
+        }
         AssertRCReturn(rc, rc);
         AssertRCSuccessReturn(rc, VERR_INTERNAL_ERROR);
     }
@@ -1880,8 +1897,15 @@ VMMDECL(int) PGMSyncCR3(PVMCPU pVCpu, uint64_t cr0, uint64_t cr3, uint64_t cr4,
     AssertMsg(rc == VINF_SUCCESS || rc == VINF_PGM_SYNC_CR3 || RT_FAILURE(rc), ("rc=%Rrc\n", rc));
     if (rc == VINF_SUCCESS)
     {
+        if (pVCpu->pgm.s.fSyncFlags & PGM_SYNC_CLEAR_PGM_POOL)
+        {
+            /* Go back to ring 3 if a pgm pool sync is again pending. */
+            return VINF_PGM_SYNC_CR3;
+        }
+
         if (!(pVCpu->pgm.s.fSyncFlags & PGM_SYNC_ALWAYS))
         {
+            Assert(!(pVCpu->pgm.s.fSyncFlags & PGM_SYNC_CLEAR_PGM_POOL));
             VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_PGM_SYNC_CR3);
             VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_PGM_SYNC_CR3_NON_GLOBAL);
         }
@@ -1892,7 +1916,7 @@ VMMDECL(int) PGMSyncCR3(PVMCPU pVCpu, uint64_t cr0, uint64_t cr3, uint64_t cr4,
         if (pVCpu->pgm.s.fSyncFlags & PGM_SYNC_MONITOR_CR3)
         {
             pVCpu->pgm.s.fSyncFlags &= ~PGM_SYNC_MONITOR_CR3;
-            Assert(!pVM->pgm.s.fMappingsFixed);
+            Assert(!pVM->pgm.s.fMappingsFixed); Assert(!pVM->pgm.s.fMappingsDisabled);
         }
     }
 
@@ -2054,6 +2078,17 @@ VMMDECL(const char *) PGMGetModeName(PGMMODE enmMode)
 }
 
 
+/**
+ * Check if any pgm pool pages are marked dirty (not monitored)
+ *
+ * @returns bool locked/not locked
+ * @param   pVM         The VM to operate on.
+ */
+VMMDECL(bool) PGMHasDirtyPages(PVM pVM)
+{
+    return pVM->pgm.s.CTX_SUFF(pPool)->cDirtyPages != 0;
+}
+
 /**
  * Check if the PGM lock is currently taken.
  *
@@ -2078,6 +2113,17 @@ VMMDECL(bool) PGMIsLockOwner(PVM pVM)
 }
 
 
+/**
+ * Enable or disable large page usage
+ *
+ * @param   pVM             The VM to operate on.
+ * @param   fUseLargePages  Use/not use large pages
+ */
+VMMDECL(void) PGMSetLargePageUsage(PVM pVM, bool fUseLargePages)
+{
+      pVM->fUseLargePages = fUseLargePages;
+}
+
 /**
  * Acquire the PGM lock.
  *
@@ -2189,7 +2235,7 @@ VMMDECL(int) PGMDynMapGCPageOff(PVM pVM, RTGCPHYS GCPhys, void **ppv)
 #else
     PGMDynMapHCPage(pVM, HCPhys, ppv);
 #endif
-    *ppv = (void *)((uintptr_t)*ppv | (GCPhys & PAGE_OFFSET_MASK));
+    *ppv = (void *)((uintptr_t)*ppv | (uintptr_t)(GCPhys & PAGE_OFFSET_MASK));
     return VINF_SUCCESS;
 }
 
@@ -2508,7 +2554,7 @@ VMMDECL(unsigned) PGMAssertNoMappingConflicts(PVM pVM)
     unsigned cErrors = 0;
 
     /* Only applies to raw mode -> 1 VPCU */
-    Assert(pVM->cCPUs == 1);
+    Assert(pVM->cCpus == 1);
     PVMCPU pVCpu = &pVM->aCpus[0];
 
     /*
