@@ -1,10 +1,10 @@
-/* $Id: SELMAll.cpp $ */
+/* $Id: SELMAll.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * SELM All contexts.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -35,6 +31,7 @@
 #include <VBox/param.h>
 #include <iprt/assert.h>
 #include <VBox/log.h>
+#include <VBox/vmm.h>
 
 
 
@@ -54,7 +51,7 @@
  */
 VMMDECL(RTGCPTR) SELMToFlatBySel(PVM pVM, RTSEL Sel, RTGCPTR Addr)
 {
-    Assert(!CPUMIsGuestInLongMode(pVM));    /* DON'T USE! */
+    Assert(pVM->cCpus == 1 && !CPUMIsGuestInLongMode(VMMGetCpu(pVM)));    /* DON'T USE! */
 
     /** @todo check the limit. */
     X86DESC    Desc;
@@ -67,7 +64,7 @@ VMMDECL(RTGCPTR) SELMToFlatBySel(PVM pVM, RTSEL Sel, RTGCPTR Addr)
         Desc = paLDT[Sel >> X86_SEL_SHIFT];
     }
 
-    return (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
+    return (RTGCPTR)(((RTGCUINTPTR)Addr + X86DESC_BASE(Desc)) & 0xffffffff);
 }
 #endif /* !IN_RING0 */
 
@@ -89,14 +86,15 @@ VMMDECL(RTGCPTR) SELMToFlat(PVM pVM, DIS_SELREG SelReg, PCPUMCTXCORE pCtxCore, R
     PCPUMSELREGHID pHiddenSel;
     RTSEL          Sel;
     int            rc;
+    PVMCPU         pVCpu = VMMGetCpu(pVM);
 
     rc = DISFetchRegSegEx(pCtxCore, SelReg, &Sel, &pHiddenSel); AssertRC(rc);
 
     /*
      * Deal with real & v86 mode first.
      */
-    if (    CPUMIsGuestInRealMode(pVM)
-        ||  pCtxCore->eflags.Bits.u1VM)
+    if (    pCtxCore->eflags.Bits.u1VM
+        ||  CPUMIsGuestInRealMode(pVCpu))
     {
         RTGCUINTPTR uFlat = (RTGCUINTPTR)Addr & 0xffff;
         if (CPUMAreHiddenSelRegsValid(pVM))
@@ -115,8 +113,8 @@ VMMDECL(RTGCPTR) SELMToFlat(PVM pVM, DIS_SELREG SelReg, PCPUMCTXCORE pCtxCore, R
 #endif
 
     /* 64 bits mode: CS, DS, ES and SS are treated as if each segment base is 0 (Intel 64 and IA-32 Architectures Software Developer's Manual: 3.4.2.1). */
-    if (    CPUMIsGuestInLongMode(pVM)
-        &&  pCtxCore->csHid.Attr.n.u1Long)
+    if (    pCtxCore->csHid.Attr.n.u1Long
+        &&  CPUMIsGuestInLongMode(pVCpu))
     {
         switch (SelReg)
         {
@@ -156,14 +154,16 @@ VMMDECL(int) SELMToFlatEx(PVM pVM, DIS_SELREG SelReg, PCCPUMCTXCORE pCtxCore, RT
      */
     PCPUMSELREGHID pHiddenSel;
     RTSEL          Sel;
+    PVMCPU         pVCpu = VMMGetCpu(pVM);
+
     int rc = DISFetchRegSegEx(pCtxCore, SelReg, &Sel, &pHiddenSel);
     AssertRC(rc);
 
     /*
      * Deal with real & v86 mode first.
      */
-    if (    CPUMIsGuestInRealMode(pVM)
-        ||  pCtxCore->eflags.Bits.u1VM)
+    if (    pCtxCore->eflags.Bits.u1VM
+        ||  CPUMIsGuestInRealMode(pVCpu))
     {
         RTGCUINTPTR uFlat = (RTGCUINTPTR)Addr & 0xffff;
         if (ppvGC)
@@ -196,8 +196,8 @@ VMMDECL(int) SELMToFlatEx(PVM pVM, DIS_SELREG SelReg, PCCPUMCTXCORE pCtxCore, RT
         u32Limit      = pHiddenSel->u32Limit;
 
         /* 64 bits mode: CS, DS, ES and SS are treated as if each segment base is 0 (Intel 64 and IA-32 Architectures Software Developer's Manual: 3.4.2.1). */
-        if (    CPUMIsGuestInLongMode(pVM)
-            &&  pCtxCore->csHid.Attr.n.u1Long)
+        if (    pCtxCore->csHid.Attr.n.u1Long
+            &&  CPUMIsGuestInLongMode(pVCpu))
         {
             fCheckLimit = false;
             switch (SelReg)
@@ -327,6 +327,10 @@ VMMDECL(int) SELMToFlatEx(PVM pVM, DIS_SELREG SelReg, PCCPUMCTXCORE pCtxCore, RT
         /* calc address assuming straight stuff. */
         pvFlat = (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
 
+        /* Cut the address to 32 bits. */
+        Assert(!CPUMIsGuestInLongMode(pVCpu));
+        pvFlat &= 0xffffffff;
+
         u1Present     = Desc.Gen.u1Present;
         u1Granularity = Desc.Gen.u1Granularity;
         u1DescType    = Desc.Gen.u1DescType;
@@ -457,13 +461,15 @@ VMMDECL(int) SELMToFlatEx(PVM pVM, DIS_SELREG SelReg, PCCPUMCTXCORE pCtxCore, RT
  */
 VMMDECL(int) SELMToFlatBySelEx(PVM pVM, X86EFLAGS eflags, RTSEL Sel, RTGCPTR Addr, CPUMSELREGHID *pHiddenSel, unsigned fFlags, PRTGCPTR ppvGC, uint32_t *pcb)
 {
-    Assert(!CPUMIsGuestInLongMode(pVM));    /* DON'T USE! */
+    PVMCPU pVCpu = VMMGetCpu(pVM);
+
+    Assert(!CPUMIsGuestInLongMode(pVCpu));    /* DON'T USE! */
 
     /*
      * Deal with real & v86 mode first.
      */
-    if (    CPUMIsGuestInRealMode(pVM)
-        ||  eflags.Bits.u1VM)
+    if (    eflags.Bits.u1VM
+        ||  CPUMIsGuestInRealMode(pVCpu))
     {
         RTGCUINTPTR uFlat = (RTGCUINTPTR)Addr & 0xffff;
         if (ppvGC)
@@ -496,8 +502,8 @@ VMMDECL(int) SELMToFlatBySelEx(PVM pVM, X86EFLAGS eflags, RTSEL Sel, RTGCPTR Add
         u32Limit      = pHiddenSel->u32Limit;
         pvFlat        = (RTGCPTR)(pHiddenSel->u64Base + (RTGCUINTPTR)Addr);
 
-        if (   !CPUMIsGuestInLongMode(pVM)
-            || !pHiddenSel->Attr.n.u1Long)
+        if (   !pHiddenSel->Attr.n.u1Long
+            || !CPUMIsGuestInLongMode(pVCpu))
         {
             /* AMD64 manual: compatibility mode ignores the high 32 bits when calculating an effective address. */
             pvFlat &= 0xffffffff;
@@ -532,6 +538,10 @@ VMMDECL(int) SELMToFlatBySelEx(PVM pVM, X86EFLAGS eflags, RTSEL Sel, RTGCPTR Add
         /* calc address assuming straight stuff. */
         pvFlat = (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
 
+        /* Cut the address to 32 bits. */
+        Assert(!CPUMIsGuestInLongMode(pVCpu));
+        pvFlat &= 0xffffffff;
+
         u1Present     = Desc.Gen.u1Present;
         u1Granularity = Desc.Gen.u1Granularity;
         u1DescType    = Desc.Gen.u1DescType;
@@ -730,6 +740,9 @@ DECLINLINE(int) selmValidateAndConvertCSAddrStd(PVM pVM, RTSEL SelCPL, RTSEL Sel
                 if ((RTGCUINTPTR)Addr <= u32Limit)
                 {
                     *ppvFlat = (RTGCPTR)((RTGCUINTPTR)Addr + X86DESC_BASE(Desc));
+                    /* Cut the address to 32 bits. */
+                    *ppvFlat &= 0xffffffff;
+
                     if (pcBits)
                         *pcBits = Desc.Gen.u1DefBig ? 32 : 16; /** @todo GUEST64 */
                     return VINF_SUCCESS;
@@ -750,14 +763,14 @@ DECLINLINE(int) selmValidateAndConvertCSAddrStd(PVM pVM, RTSEL SelCPL, RTSEL Sel
  * address when in protected/long mode using the standard algorithm.
  *
  * @returns VBox status code.
- * @param   pVM     VM Handle.
+ * @param   pVCpu   VMCPU Handle.
  * @param   SelCPL  Current privilege level. Get this from SS - CS might be conforming!
  *                  A full selector can be passed, we'll only use the RPL part.
  * @param   SelCS   Selector part.
  * @param   Addr    Address part.
  * @param   ppvFlat Where to store the flat address.
  */
-DECLINLINE(int) selmValidateAndConvertCSAddrHidden(PVM pVM, RTSEL SelCPL, RTSEL SelCS, PCPUMSELREGHID pHidCS, RTGCPTR Addr, PRTGCPTR ppvFlat)
+DECLINLINE(int) selmValidateAndConvertCSAddrHidden(PVMCPU pVCpu, RTSEL SelCPL, RTSEL SelCS, PCPUMSELREGHID pHidCS, RTGCPTR Addr, PRTGCPTR ppvFlat)
 {
     /*
      * Check if present.
@@ -780,8 +793,8 @@ DECLINLINE(int) selmValidateAndConvertCSAddrHidden(PVM pVM, RTSEL SelCPL, RTSEL
                     )
             {
                 /* 64 bits mode: CS, DS, ES and SS are treated as if each segment base is 0 (Intel 64 and IA-32 Architectures Software Developer's Manual: 3.4.2.1). */
-                if (    CPUMIsGuestInLongMode(pVM)
-                    &&  pHidCS->Attr.n.u1Long)
+                if (    pHidCS->Attr.n.u1Long
+                    &&  CPUMIsGuestInLongMode(pVCpu))
                 {
                     *ppvFlat = Addr;
                     return VINF_SUCCESS;
@@ -828,8 +841,11 @@ DECLINLINE(int) selmValidateAndConvertCSAddrHidden(PVM pVM, RTSEL SelCPL, RTSEL
  */
 VMMDECL(int) SELMValidateAndConvertCSAddrGCTrap(PVM pVM, X86EFLAGS eflags, RTSEL SelCPL, RTSEL SelCS, RTGCPTR Addr, PRTGCPTR ppvFlat, uint32_t *pcBits)
 {
-    if (    CPUMIsGuestInRealMode(pVM)
-        ||  eflags.Bits.u1VM)
+    Assert(pVM->cCpus == 1);
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
+    if (    eflags.Bits.u1VM
+        ||  CPUMIsGuestInRealMode(pVCpu))
     {
         *pcBits = 16;
         return selmValidateAndConvertCSAddrRealMode(pVM, SelCS, NULL, Addr, ppvFlat);
@@ -854,8 +870,10 @@ VMMDECL(int) SELMValidateAndConvertCSAddrGCTrap(PVM pVM, X86EFLAGS eflags, RTSEL
  */
 VMMDECL(int) SELMValidateAndConvertCSAddr(PVM pVM, X86EFLAGS eflags, RTSEL SelCPL, RTSEL SelCS, CPUMSELREGHID *pHiddenCSSel, RTGCPTR Addr, PRTGCPTR ppvFlat)
 {
-    if (    CPUMIsGuestInRealMode(pVM)
-        ||  eflags.Bits.u1VM)
+    PVMCPU pVCpu = VMMGetCpu(pVM);
+
+    if (    eflags.Bits.u1VM
+        ||  CPUMIsGuestInRealMode(pVCpu))
         return selmValidateAndConvertCSAddrRealMode(pVM, SelCS, pHiddenCSSel, Addr, ppvFlat);
 
 #ifdef IN_RING0
@@ -865,7 +883,7 @@ VMMDECL(int) SELMValidateAndConvertCSAddr(PVM pVM, X86EFLAGS eflags, RTSEL SelCP
     if (!CPUMAreHiddenSelRegsValid(pVM))
         return selmValidateAndConvertCSAddrStd(pVM, SelCPL, SelCS, Addr, ppvFlat, NULL);
 #endif
-    return selmValidateAndConvertCSAddrHidden(pVM, SelCPL, SelCS, pHiddenCSSel, Addr, ppvFlat);
+    return selmValidateAndConvertCSAddrHidden(pVCpu, SelCPL, SelCS, pHiddenCSSel, Addr, ppvFlat);
 }
 
 
@@ -907,6 +925,7 @@ static DISCPUMODE selmGetCpuModeFromSelector(PVM pVM, RTSEL Sel)
  */
 VMMDECL(DISCPUMODE) SELMGetCpuModeFromSelector(PVM pVM, X86EFLAGS eflags, RTSEL Sel, CPUMSELREGHID *pHiddenSel)
 {
+    PVMCPU pVCpu = VMMGetCpu(pVM);
 #ifdef IN_RING0
     Assert(CPUMAreHiddenSelRegsValid(pVM));
 #else  /* !IN_RING0 */
@@ -915,15 +934,15 @@ VMMDECL(DISCPUMODE) SELMGetCpuModeFromSelector(PVM pVM, X86EFLAGS eflags, RTSEL
         /*
          * Deal with real & v86 mode first.
          */
-        if (    CPUMIsGuestInRealMode(pVM)
-            ||  eflags.Bits.u1VM)
+        if (    eflags.Bits.u1VM
+            ||  CPUMIsGuestInRealMode(pVCpu))
             return CPUMODE_16BIT;
 
         return selmGetCpuModeFromSelector(pVM, Sel);
     }
 #endif /* !IN_RING0 */
-    if (    CPUMIsGuestInLongMode(pVM)
-        &&  pHiddenSel->Attr.n.u1Long)
+    if (    pHiddenSel->Attr.n.u1Long
+        &&  CPUMIsGuestInLongMode(pVCpu))
         return CPUMODE_64BIT;
 
     /* Else compatibility or 32 bits mode. */
@@ -983,6 +1002,9 @@ void selmSetRing1Stack(PVM pVM, uint32_t ss, RTGCPTR32 esp)
  */
 VMMDECL(int) SELMGetRing1Stack(PVM pVM, uint32_t *pSS, PRTGCPTR32 pEsp)
 {
+    Assert(pVM->cCpus == 1);
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     if (pVM->selm.s.fSyncTSSRing0Stack)
     {
         RTGCPTR GCPtrTss = pVM->selm.s.GCPtrGuestTss;
@@ -995,10 +1017,11 @@ VMMDECL(int) SELMGetRing1Stack(PVM pVM, uint32_t *pSS, PRTGCPTR32 pEsp)
         bool    fTriedAlready = false;
 
 l_tryagain:
-        rc  = MMGCRamRead(pVM, &tss.ss0,  (RCPTRTYPE(void *))(GCPtrTss + RT_OFFSETOF(VBOXTSS, ss0)), sizeof(tss.ss0));
-        rc |= MMGCRamRead(pVM, &tss.esp0, (RCPTRTYPE(void *))(GCPtrTss + RT_OFFSETOF(VBOXTSS, esp0)), sizeof(tss.esp0));
+        PVBOXTSS pTss = (PVBOXTSS)(uintptr_t)GCPtrTss;
+        rc  = MMGCRamRead(pVM, &tss.ss0,  &pTss->ss0,  sizeof(tss.ss0));
+        rc |= MMGCRamRead(pVM, &tss.esp0, &pTss->esp0, sizeof(tss.esp0));
 #  ifdef DEBUG
-        rc |= MMGCRamRead(pVM, &tss.offIoBitmap, (RCPTRTYPE(void *))(GCPtrTss + RT_OFFSETOF(VBOXTSS, offIoBitmap)), sizeof(tss.offIoBitmap));
+        rc |= MMGCRamRead(pVM, &tss.offIoBitmap, &pTss->offIoBitmap, sizeof(tss.offIoBitmap));
 #  endif
 
         if (RT_FAILURE(rc))
@@ -1008,7 +1031,7 @@ l_tryagain:
                 /* Shadow page might be out of sync. Sync and try again */
                 /** @todo might cross page boundary */
                 fTriedAlready = true;
-                rc = PGMPrefetchPage(pVM, (RTGCPTR)GCPtrTss);
+                rc = PGMPrefetchPage(pVCpu, (RTGCPTR)GCPtrTss);
                 if (rc != VINF_SUCCESS)
                     return rc;
                 goto l_tryagain;
@@ -1019,7 +1042,7 @@ l_tryagain:
 
 # else /* !IN_RC */
         /* Reading too much. Could be cheaper than two seperate calls though. */
-        rc = PGMPhysSimpleReadGCPtr(pVM, &tss, GCPtrTss, sizeof(VBOXTSS));
+        rc = PGMPhysSimpleReadGCPtr(pVCpu, &tss, GCPtrTss, sizeof(VBOXTSS));
         if (RT_FAILURE(rc))
         {
             AssertReleaseMsgFailed(("Unable to read TSS structure at %08X\n", GCPtrTss));
@@ -1062,42 +1085,6 @@ VMMDECL(RTGCPTR) SELMGetGuestTSS(PVM pVM)
 }
 
 
-/**
- * Validates a CS selector.
- *
- * @returns VBox status code.
- * @param   pSelInfo    Pointer to the selector information for the CS selector.
- * @param   SelCPL      The selector defining the CPL (SS).
- */
-VMMDECL(int) SELMSelInfoValidateCS(PCSELMSELINFO pSelInfo, RTSEL SelCPL)
-{
-    /*
-     * Check if present.
-     */
-    if (pSelInfo->Raw.Gen.u1Present)
-    {
-        /*
-         * Type check.
-         */
-        if (    pSelInfo->Raw.Gen.u1DescType == 1
-            &&  (pSelInfo->Raw.Gen.u4Type & X86_SEL_TYPE_CODE))
-        {
-            /*
-             * Check level.
-             */
-            unsigned uLevel = RT_MAX(SelCPL & X86_SEL_RPL, pSelInfo->Sel & X86_SEL_RPL);
-            if (    !(pSelInfo->Raw.Gen.u4Type & X86_SEL_TYPE_CONF)
-                ?   uLevel <= pSelInfo->Raw.Gen.u2Dpl
-                :   uLevel >= pSelInfo->Raw.Gen.u2Dpl /* hope I got this right now... */
-                    )
-                return VINF_SUCCESS;
-            return VERR_INVALID_RPL;
-        }
-        return VERR_NOT_CODE_SELECTOR;
-    }
-    return VERR_SELECTOR_NOT_PRESENT;
-}
-
 #ifndef IN_RING0
 
 /**
@@ -1182,17 +1169,18 @@ VMMDECL(RTRCPTR) SELMGetHyperGDT(PVM pVM)
  * @retval  VERR_SELM_NO_TSS if we haven't got a TSS (rather unlikely).
  *
  * @param   pVM                 The VM handle.
+ * @param   pVCpu               VMCPU Handle.
  * @param   pGCPtrTss           Where to store the TSS address.
  * @param   pcbTss              Where to store the TSS size limit.
  * @param   pfCanHaveIOBitmap   Where to store the can-have-I/O-bitmap indicator. (optional)
  */
-VMMDECL(int) SELMGetTSSInfo(PVM pVM, PRTGCUINTPTR pGCPtrTss, PRTGCUINTPTR pcbTss, bool *pfCanHaveIOBitmap)
+VMMDECL(int) SELMGetTSSInfo(PVM pVM, PVMCPU pVCpu, PRTGCUINTPTR pGCPtrTss, PRTGCUINTPTR pcbTss, bool *pfCanHaveIOBitmap)
 {
     /*
      * The TR hidden register is always valid.
      */
     CPUMSELREGHID trHid;
-    RTSEL tr = CPUMGetGuestTR(pVM, &trHid);
+    RTSEL tr = CPUMGetGuestTR(pVCpu, &trHid);
     if (!(tr & X86_SEL_MASK))
         return VERR_SELM_NO_TSS;
 
@@ -1212,9 +1200,11 @@ VMMDECL(int) SELMGetTSSInfo(PVM pVM, PRTGCUINTPTR pGCPtrTss, PRTGCUINTPTR pcbTss
  * This is called by PGM.
  *
  * @param   pVM       The VM handle
+ * @param   pVCpu     The VMCPU handle
  */
-VMMDECL(void) SELMShadowCR3Changed(PVM pVM)
+VMMDECL(void) SELMShadowCR3Changed(PVM pVM, PVMCPU pVCpu)
 {
-    pVM->selm.s.Tss.cr3       = PGMGetHyperCR3(pVM);
-    pVM->selm.s.TssTrap08.cr3 = PGMGetInterRCCR3(pVM);
+    /** @todo SMP support!! */
+    pVM->selm.s.Tss.cr3       = PGMGetHyperCR3(pVCpu);
+    pVM->selm.s.TssTrap08.cr3 = PGMGetInterRCCR3(pVM, pVCpu);
 }
