@@ -1,10 +1,10 @@
-/* $Id: TMAllVirtual.cpp $ */
+/* $Id: TMAllVirtual.cpp 29250 2010-05-09 17:53:58Z vboxsync $ */
 /** @file
  * TM - Timeout Manager, Virtual Time, All Contexts.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -39,6 +35,7 @@
 #include <iprt/time.h>
 #include <iprt/assert.h>
 #include <iprt/asm.h>
+#include <iprt/asm-math.h>
 
 
 
@@ -74,6 +71,7 @@ DECLEXPORT(uint64_t) tmVirtualNanoTSRediscover(PRTTIMENANOTSDATA pData)
     //PVM pVM = (PVM)((uint8_t *)pData - RT_OFFSETOF(VM, CTXALLSUFF(s.tm.VirtualGetRawData)));
     PSUPGLOBALINFOPAGE pGip = g_pSUPGlobalInfoPage;
     AssertFatalMsgFailed(("pGip=%p u32Magic=%#x\n", pGip, VALID_PTR(pGip) ? pGip->u32Magic : 0));
+    return 0; /* gcc false positive warning */
 }
 
 
@@ -369,7 +367,7 @@ DECLINLINE(uint64_t) tmVirtualGet(PVM pVM, bool fCheckTimers)
  *          influenced by power saving (SpeedStep, PowerNow!), while the former
  *          makes use of TSC and kernel timers.
  */
-VMMDECL(uint64_t) TMVirtualGet(PVM pVM)
+VMM_INT_DECL(uint64_t) TMVirtualGet(PVM pVM)
 {
     return tmVirtualGet(pVM, true /* check timers */);
 }
@@ -386,7 +384,7 @@ VMMDECL(uint64_t) TMVirtualGet(PVM pVM)
  *
  * @remarks See TMVirtualGet.
  */
-VMMDECL(uint64_t) TMVirtualGetNoCheck(PVM pVM)
+VMM_INT_DECL(uint64_t) TMVirtualGetNoCheck(PVM pVM)
 {
     return tmVirtualGet(pVM, false /*fCheckTimers*/);
 }
@@ -405,7 +403,7 @@ DECLINLINE(uint64_t) tmVirtualSyncGetHandleCatchUpLocked(PVM pVM, uint64_t u64,
     STAM_COUNTER_INC(&pVM->tm.s.StatVirtualSyncGetLocked);
 
     /*
-     * Don't make updates untill
+     * Don't make updates until we've check the timer qeueue.
      */
     bool            fUpdatePrev = true;
     bool            fUpdateOff  = true;
@@ -744,7 +742,7 @@ DECLINLINE(uint64_t) tmVirtualSyncGetEx(PVM pVM, bool fCheckTimers)
  * @thread  EMT.
  * @remarks May set the timer and virtual sync FFs.
  */
-VMMDECL(uint64_t) TMVirtualSyncGet(PVM pVM)
+VMM_INT_DECL(uint64_t) TMVirtualSyncGet(PVM pVM)
 {
     return tmVirtualSyncGetEx(pVM, true /* check timers */);
 }
@@ -759,7 +757,7 @@ VMMDECL(uint64_t) TMVirtualSyncGet(PVM pVM)
  * @thread  EMT.
  * @remarks May set the timer and virtual sync FFs.
  */
-VMMDECL(uint64_t) TMVirtualSyncGetNoCheck(PVM pVM)
+VMM_INT_DECL(uint64_t) TMVirtualSyncGetNoCheck(PVM pVM)
 {
     return tmVirtualSyncGetEx(pVM, false /* check timers */);
 }
@@ -774,7 +772,7 @@ VMMDECL(uint64_t) TMVirtualSyncGetNoCheck(PVM pVM)
  * @thread  EMT.
  * @remarks May set the timer and virtual sync FFs.
  */
-VMMDECL(uint64_t) TMVirtualSyncGetEx(PVM pVM, bool fCheckTimers)
+VMM_INT_DECL(uint64_t) TMVirtualSyncGetEx(PVM pVM, bool fCheckTimers)
 {
     return tmVirtualSyncGetEx(pVM, fCheckTimers);
 }
@@ -786,7 +784,7 @@ VMMDECL(uint64_t) TMVirtualSyncGetEx(PVM pVM, bool fCheckTimers)
  * @return  The current lag.
  * @param   pVM     VM handle.
  */
-VMMDECL(uint64_t) TMVirtualSyncGetLag(PVM pVM)
+VMM_INT_DECL(uint64_t) TMVirtualSyncGetLag(PVM pVM)
 {
     return pVM->tm.s.offVirtualSync - pVM->tm.s.offVirtualSyncGivenUp;
 }
@@ -798,7 +796,7 @@ VMMDECL(uint64_t) TMVirtualSyncGetLag(PVM pVM)
  * @return  The current catch0up percent. 0 means running at the same speed as the virtual clock.
  * @param   pVM     VM handle.
  */
-VMMDECL(uint32_t) TMVirtualSyncGetCatchUpPct(PVM pVM)
+VMM_INT_DECL(uint32_t) TMVirtualSyncGetCatchUpPct(PVM pVM)
 {
     if (pVM->tm.s.fVirtualSyncCatchUp)
         return pVM->tm.s.u32VirtualSyncCatchUpPercentage;
@@ -812,7 +810,7 @@ VMMDECL(uint32_t) TMVirtualSyncGetCatchUpPct(PVM pVM)
  * @returns The freqency.
  * @param   pVM     VM handle.
  */
-VMMDECL(uint64_t) TMVirtualGetFreq(PVM pVM)
+VMM_INT_DECL(uint64_t) TMVirtualGetFreq(PVM pVM)
 {
     return TMCLOCK_FREQ_VIRTUAL;
 }
@@ -827,7 +825,7 @@ VMMDECL(uint64_t) TMVirtualGetFreq(PVM pVM)
 int tmVirtualPauseLocked(PVM pVM)
 {
     uint32_t c = ASMAtomicDecU32(&pVM->tm.s.cVirtualTicking);
-    AssertMsgReturn(c < pVM->cCPUs, ("%u vs %u\n", c, pVM->cCPUs), VERR_INTERNAL_ERROR);
+    AssertMsgReturn(c < pVM->cCpus, ("%u vs %u\n", c, pVM->cCpus), VERR_INTERNAL_ERROR);
     if (c == 0)
     {
         STAM_COUNTER_INC(&pVM->tm.s.StatVirtualPause);
@@ -847,7 +845,7 @@ int tmVirtualPauseLocked(PVM pVM)
 int tmVirtualResumeLocked(PVM pVM)
 {
     uint32_t c = ASMAtomicIncU32(&pVM->tm.s.cVirtualTicking);
-    AssertMsgReturn(c <= pVM->cCPUs, ("%u vs %u\n", c, pVM->cCPUs), VERR_INTERNAL_ERROR);
+    AssertMsgReturn(c <= pVM->cCpus, ("%u vs %u\n", c, pVM->cCpus), VERR_INTERNAL_ERROR);
     if (c == 1)
     {
         STAM_COUNTER_INC(&pVM->tm.s.StatVirtualResume);
@@ -869,7 +867,7 @@ int tmVirtualResumeLocked(PVM pVM)
  * @remark  There could be rounding errors here. We just do a simple integere divide
  *          without any adjustments.
  */
-VMMDECL(uint64_t) TMVirtualToNano(PVM pVM, uint64_t u64VirtualTicks)
+VMM_INT_DECL(uint64_t) TMVirtualToNano(PVM pVM, uint64_t u64VirtualTicks)
 {
     AssertCompile(TMCLOCK_FREQ_VIRTUAL == 1000000000);
     return u64VirtualTicks;
@@ -885,7 +883,7 @@ VMMDECL(uint64_t) TMVirtualToNano(PVM pVM, uint64_t u64VirtualTicks)
  * @remark  There could be rounding errors here. We just do a simple integere divide
  *          without any adjustments.
  */
-VMMDECL(uint64_t) TMVirtualToMicro(PVM pVM, uint64_t u64VirtualTicks)
+VMM_INT_DECL(uint64_t) TMVirtualToMicro(PVM pVM, uint64_t u64VirtualTicks)
 {
     AssertCompile(TMCLOCK_FREQ_VIRTUAL == 1000000000);
     return u64VirtualTicks / 1000;
@@ -901,7 +899,7 @@ VMMDECL(uint64_t) TMVirtualToMicro(PVM pVM, uint64_t u64VirtualTicks)
  * @remark  There could be rounding errors here. We just do a simple integere divide
  *          without any adjustments.
  */
-VMMDECL(uint64_t) TMVirtualToMilli(PVM pVM, uint64_t u64VirtualTicks)
+VMM_INT_DECL(uint64_t) TMVirtualToMilli(PVM pVM, uint64_t u64VirtualTicks)
 {
         AssertCompile(TMCLOCK_FREQ_VIRTUAL == 1000000000);
     return u64VirtualTicks / 1000000;
@@ -916,7 +914,7 @@ VMMDECL(uint64_t) TMVirtualToMilli(PVM pVM, uint64_t u64VirtualTicks)
  * @param   u64NanoTS       The nanosecond value ticks to convert.
  * @remark  There could be rounding and overflow errors here.
  */
-VMMDECL(uint64_t) TMVirtualFromNano(PVM pVM, uint64_t u64NanoTS)
+VMM_INT_DECL(uint64_t) TMVirtualFromNano(PVM pVM, uint64_t u64NanoTS)
 {
     AssertCompile(TMCLOCK_FREQ_VIRTUAL == 1000000000);
     return u64NanoTS;
@@ -931,7 +929,7 @@ VMMDECL(uint64_t) TMVirtualFromNano(PVM pVM, uint64_t u64NanoTS)
  * @param   u64MicroTS      The microsecond value ticks to convert.
  * @remark  There could be rounding and overflow errors here.
  */
-VMMDECL(uint64_t) TMVirtualFromMicro(PVM pVM, uint64_t u64MicroTS)
+VMM_INT_DECL(uint64_t) TMVirtualFromMicro(PVM pVM, uint64_t u64MicroTS)
 {
     AssertCompile(TMCLOCK_FREQ_VIRTUAL == 1000000000);
     return u64MicroTS * 1000;
@@ -946,7 +944,7 @@ VMMDECL(uint64_t) TMVirtualFromMicro(PVM pVM, uint64_t u64MicroTS)
  * @param   u64MilliTS      The millisecond value ticks to convert.
  * @remark  There could be rounding and overflow errors here.
  */
-VMMDECL(uint64_t) TMVirtualFromMilli(PVM pVM, uint64_t u64MilliTS)
+VMM_INT_DECL(uint64_t) TMVirtualFromMilli(PVM pVM, uint64_t u64MilliTS)
 {
     AssertCompile(TMCLOCK_FREQ_VIRTUAL == 1000000000);
     return u64MilliTS * 1000000;
