@@ -1,10 +1,10 @@
-/* $Id: CPUMGC.cpp 13960 2008-11-07 13:04:45Z vboxsync $ */
+/* $Id: CPUMGC.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * CPUM - Guest Context Code.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -37,9 +33,9 @@
 /*******************************************************************************
 *   Internal Functions                                                         *
 *******************************************************************************/
-__BEGIN_DECLS /* addressed from asm (not called so no DECLASM). */
+RT_C_DECLS_BEGIN /* addressed from asm (not called so no DECLASM). */
 DECLCALLBACK(int) cpumGCHandleNPAndGP(PVM pVM, PCPUMCTXCORE pRegFrame, uintptr_t uUser);
-__END_DECLS
+RT_C_DECLS_END
 
 
 /**
@@ -58,12 +54,14 @@ DECLCALLBACK(int) cpumGCHandleNPAndGP(PVM pVM, PCPUMCTXCORE pRegFrame, uintptr_t
     Log(("cpumGCHandleNPAndGP: eip=%RX32 uUser=%#x\n", pRegFrame->eip, uUser));
     Log(("********************************************************\n"));
 
+    PVMCPU pVCpu = VMMGetCpu0(pVM);
+
     /*
      * Update the guest cpu state.
      */
     if (uUser & CPUM_HANDLER_CTXCORE_IN_EBP)
     {
-        PCPUMCTXCORE  pGstCtxCore = (PCPUMCTXCORE)CPUMGetGuestCtxCore(pVM);
+        PCPUMCTXCORE  pGstCtxCore = (PCPUMCTXCORE)CPUMGetGuestCtxCore(pVCpu);
         PCCPUMCTXCORE pGstCtxCoreSrc = (PCPUMCTXCORE)pRegFrame->ebp;
         *pGstCtxCore = *pGstCtxCoreSrc;
     }
@@ -89,7 +87,7 @@ DECLCALLBACK(int) cpumGCHandleNPAndGP(PVM pVM, PCPUMCTXCORE pRegFrame, uintptr_t
         /* Make sure we restore the guest context from the interrupt stack frame. */
         case CPUM_HANDLER_IRET:
         {
-            PCPUMCTXCORE  pGstCtxCore = (PCPUMCTXCORE)CPUMGetGuestCtxCore(pVM);
+            PCPUMCTXCORE  pGstCtxCore = (PCPUMCTXCORE)CPUMGetGuestCtxCore(pVCpu);
             uint32_t     *pEsp = (uint32_t *)pRegFrame->esp;
 
             /* Sync general purpose registers */
