@@ -1,10 +1,10 @@
-/* $Id: VMMGC.cpp $ */
+/* $Id: VMMGC.cpp 29250 2010-05-09 17:53:58Z vboxsync $ */
 /** @file
  * VMM - Raw-mode Context.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -24,6 +20,7 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_VMM
+#include <iprt/asm-amd64-x86.h> /* for SUPGetCpuHzFromGIP */
 #include <VBox/vmm.h>
 #include <VBox/trpm.h>
 #include <VBox/pgm.h>
@@ -32,7 +29,6 @@
 #include <VBox/sup.h>
 #include <VBox/err.h>
 #include <VBox/log.h>
-#include <iprt/asm.h>
 #include <iprt/assert.h>
 #include <iprt/initterm.h>
 
@@ -175,7 +171,9 @@ VMMRCDECL(int) vmmGCLoggerFlush(PRTLOGGERRC pLogger)
 {
     PVM pVM = &g_VM;
     NOREF(pLogger);
-    return VMMGCCallHost(pVM, VMMCALLHOST_VMM_LOGGER_FLUSH, 0);
+    if (pVM->vmm.s.fRCLoggerFlushingDisabled)
+        return VINF_SUCCESS; /* fail quietly. */
+    return VMMRZCallRing3NoCpu(pVM, VMMCALLRING3_VMM_LOGGER_FLUSH, 0);
 }
 
 
@@ -189,37 +187,12 @@ VMMRCDECL(void) VMMGCLogFlushIfFull(PVM pVM)
     if (    pVM->vmm.s.pRCLoggerRC
         &&  pVM->vmm.s.pRCLoggerRC->offScratch >= (sizeof(pVM->vmm.s.pRCLoggerRC->achScratch)*3/4))
     {
-        VMMGCCallHost(pVM, VMMCALLHOST_VMM_LOGGER_FLUSH, 0);
+        if (pVM->vmm.s.fRCLoggerFlushingDisabled)
+            return; /* fail quietly. */
+        VMMRZCallRing3NoCpu(pVM, VMMCALLRING3_VMM_LOGGER_FLUSH, 0);
     }
 }
 
-/**
- * Disables the GC logger temporarily, restore with VMMGCLogRestore.
- *
- * @param   pVM             The VM handle.
- */
-VMMRCDECL(bool) VMMGCLogDisable(PVM pVM)
-{
-    bool fLog = pVM->vmm.s.pRCLoggerRC
-             && !(pVM->vmm.s.pRCLoggerRC->fFlags & RTLOGFLAGS_DISABLED);
-    if (fLog)
-        pVM->vmm.s.pRCLoggerRC->fFlags |= RTLOGFLAGS_DISABLED;
-    return fLog;
-}
-
-
-/**
- * Restores the GC logger after a call to VMMGCLogDisable.
- *
- * @param   pVM             The VM handle.
- * @param   fLog            What VMMGCLogDisable returned.
- */
-VMMRCDECL(void) VMMGCLogRestore(PVM pVM, bool fLog)
-{
-    if (fLog && pVM->vmm.s.pRCLoggerRC)
-        pVM->vmm.s.pRCLoggerRC->fFlags &= ~RTLOGFLAGS_DISABLED;
-}
-
 
 /**
  * Switches from guest context to host context.
@@ -233,25 +206,6 @@ VMMRCDECL(void) VMMGCGuestToHost(PVM pVM, int rc)
 }
 
 
-/**
- * Calls the ring-3 host code.
- *
- * @returns VBox status code of the ring-3 call.
- * @param   pVM             The VM handle.
- * @param   enmOperation    The operation.
- * @param   uArg            The argument to the operation.
- */
-VMMRCDECL(int) VMMGCCallHost(PVM pVM, VMMCALLHOST enmOperation, uint64_t uArg)
-{
-/** @todo profile this! */
-    pVM->vmm.s.enmCallHostOperation = enmOperation;
-    pVM->vmm.s.u64CallHostArg = uArg;
-    pVM->vmm.s.rcCallHost = VERR_INTERNAL_ERROR;
-    pVM->vmm.s.pfnGuestToHostRC(VINF_VMM_CALL_HOST);
-    return pVM->vmm.s.rcCallHost;
-}
-
-
 /**
  * Execute the trap testcase.
  *
@@ -371,7 +325,7 @@ static int vmmGCTest(PVM pVM, unsigned uOperation, unsigned uArg)
 
 
 /**
- * Temporary #PF trap handler for the #PF test case.
+ * Temporary \#PF trap handler for the \#PF test case.
  *
  * @returns VBox status code (appropriate for GC return).
  *          In this context RT_SUCCESS means to restart the instruction.
@@ -390,7 +344,8 @@ static DECLCALLBACK(int) vmmGCTestTmpPFHandler(PVM pVM, PCPUMCTXCORE pRegFrame)
 
 
 /**
- * Temporary #PF trap handler for the #PF test case, this one messes up the fs selector.
+ * Temporary \#PF trap handler for the \#PF test case, this one messes up the fs
+ * selector.
  *
  * @returns VBox status code (appropriate for GC return).
  *          In this context RT_SUCCESS means to restart the instruction.
