@@ -1,10 +1,10 @@
-/* $Id: VMMGCDeps.cpp 8155 2008-04-18 15:16:47Z vboxsync $ */
+/* $Id: VMMGCDeps.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VMMGC Runtime Dependencies.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2010 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,19 +13,17 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #include <iprt/string.h>
+#include <iprt/crc32.h>
 
 PFNRT g_VMMGCDeps[] =
 {
     (PFNRT)memset,
     (PFNRT)memcpy,
     (PFNRT)memchr,
-    (PFNRT)memcmp
+    (PFNRT)memcmp,
+    (PFNRT)RTCrc32
 };
 
