@@ -1,10 +1,10 @@
-/* $Id: VMMInternal.h $ */
+/* $Id: VMMInternal.h 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VMM - Internal header file.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 #ifndef ___VMMInternal_h
@@ -49,9 +45,6 @@
  * Ring-0 logging isn't 100% safe yet (thread id reuse / process exit cleanup),
  * so you have to sign up here by adding your defined(DEBUG_<userid>) to the
  * #if, or by adding VBOX_WITH_R0_LOGGING to your LocalConfig.kmk.
- *
- * You might also wish to enable the AssertMsg1/2 overrides in VMMR0.cpp when
- * enabling this.
  */
 #if defined(DEBUG_sandervl) || defined(DEBUG_frank) || defined(DOXYGEN_RUNNING)
 # define VBOX_WITH_R0_LOGGING
@@ -138,7 +131,7 @@ typedef struct VMMR0JMPBUF
     uint32_t                    ebp;
     uint32_t                    esp;
     uint32_t                    eip;
-    uint32_t                    u32Padding;
+    uint32_t                    eflags;
 #endif
 #if HC_ARCH_BITS == 64
     uint64_t                    rbx;
@@ -165,6 +158,7 @@ typedef struct VMMR0JMPBUF
     uint128_t                   xmm14;
     uint128_t                   xmm15;
 # endif
+    uint64_t                    rflags;
 #endif
     /** @} */
 
@@ -267,9 +261,12 @@ typedef struct VMM
     uint32_t                    cbRCRelLogger;
     /** Whether log flushing has been disabled or not. */
     bool                        fRCLoggerFlushingDisabled;
-    bool                        afAlignment[7]; /**< Alignment padding. */
+    bool                        afAlignment[6]; /**< Alignment padding. */
     /** @} */
 
+    /** Whether the stack guard pages have been stationed or not. */
+    bool                        fStackGuardsStationed;
+
     /** The EMT yield timer. */
     PTMTIMERR3                  pYieldTimer;
     /** The period to the next timeout when suspended or stopped.
@@ -287,6 +284,8 @@ typedef struct VMM
 
     /** @name EMT Rendezvous
      * @{ */
+    /** Semaphore to wait on upon entering ordered execution. */
+    R3PTRTYPE(PRTSEMEVENT)      pahEvtRendezvousEnterOrdered;
     /** Semaphore to wait on upon entering for one-by-one execution. */
     RTSEMEVENT                  hEvtRendezvousEnterOneByOne;
     /** Semaphore to wait on upon entering for all-at-once execution. */
@@ -313,6 +312,10 @@ typedef struct VMM
     volatile uint32_t           u32RendezvousLock;
     /** @} */
 
+#if HC_ARCH_BITS == 32
+    uint32_t                    u32Alignment; /**< Alignment padding. */
+#endif
+
     /** Buffer for storing the standard assertion message for a ring-0 assertion.
      * Used for saving the assertion message text for the release log and guru
      * meditation dump. */
@@ -363,6 +366,7 @@ typedef struct VMM
     STAMCOUNTER                 StatRZRetPATMDuplicateFn;
     STAMCOUNTER                 StatRZRetPGMChangeMode;
     STAMCOUNTER                 StatRZRetPendingRequest;
+    STAMCOUNTER                 StatRZRetPGMFlushPending;
     STAMCOUNTER                 StatRZRetPatchTPR;
     STAMCOUNTER                 StatRZCallPDMLock;
     STAMCOUNTER                 StatRZCallLogFlush;
