@@ -1,10 +1,10 @@
-/* $Id: HWACCMR0.cpp $ */
+/* $Id: HWACCMR0.cpp 29250 2010-05-09 17:53:58Z vboxsync $ */
 /** @file
  * HWACCM - Host Context Ring 0.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -25,19 +21,17 @@
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_HWACCM
 #include <VBox/hwaccm.h>
+#include <VBox/pgm.h>
 #include "HWACCMInternal.h"
 #include <VBox/vm.h>
 #include <VBox/x86.h>
 #include <VBox/hwacc_vmx.h>
 #include <VBox/hwacc_svm.h>
-#include <VBox/pgm.h>
-#include <VBox/pdm.h>
 #include <VBox/err.h>
 #include <VBox/log.h>
-#include <VBox/selm.h>
-#include <VBox/iom.h>
 #include <iprt/assert.h>
 #include <iprt/asm.h>
+#include <iprt/asm-amd64-x86.h>
 #include <iprt/cpuset.h>
 #include <iprt/memobj.h>
 #include <iprt/param.h>
@@ -89,6 +83,9 @@ static struct
         /** Host CR4 value (set by ring-0 VMX init) */
         uint64_t                    hostCR4;
 
+        /** Host EFER value (set by ring-0 VMX init) */
+        uint64_t                    hostEFER;
+
         /** VMX MSR values */
         struct
         {
@@ -112,14 +109,17 @@ static struct
     } vmx;
     struct
     {
-        /** Set by the ring-0 driver to indicate SVM is supported by the CPU. */
-        bool                        fSupported;
+        /* HWCR msr (for diagnostics) */
+        uint64_t                    msrHWCR;
 
         /** SVM revision. */
         uint32_t                    u32Rev;
 
         /** SVM feature bits from cpuid 0x8000000a */
         uint32_t                    u32Features;
+
+        /** Set by the ring-0 driver to indicate SVM is supported by the CPU. */
+        bool                        fSupported;
     } svm;
     /** Saved error from detection */
     int32_t         lLastError;
@@ -145,7 +145,8 @@ static struct
  */
 VMMR0DECL(int) HWACCMR0Init(void)
 {
-    int        rc;
+    int     rc;
+    bool    fAMDVPresent = false;
 
     memset(&HWACCMR0Globals, 0, sizeof(HWACCMR0Globals));
     HWACCMR0Globals.enmHwAccmState = HWACCMSTATE_UNINITIALIZED;
@@ -265,6 +266,7 @@ VMMR0DECL(int) HWACCMR0Init(void)
                         if (!HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
                         {
                             HWACCMR0Globals.vmx.hostCR4             = ASMGetCR4();
+                            HWACCMR0Globals.vmx.hostEFER            = ASMRdMsr(MSR_K6_EFER);
 
                             rc = RTR0MemObjAllocCont(&pScatchMemObj, 1 << PAGE_SHIFT, true /* executable R0 mapping */);
                             if (RT_FAILURE(rc))
@@ -350,6 +352,11 @@ VMMR0DECL(int) HWACCMR0Init(void)
                 int     aRc[RTCPUSET_MAX_CPUS];
                 RTCPUID idCpu = 0;
 
+                fAMDVPresent = true;
+
+                /* Query AMD features. */
+                ASMCpuId(0x8000000A, &HWACCMR0Globals.svm.u32Rev, &HWACCMR0Globals.uMaxASID, &u32Dummy, &HWACCMR0Globals.svm.u32Features);
+
                 /* We need to check if AMD-V has been properly initialized on all CPUs. Some BIOSes might do a poor job. */
                 memset(aRc, 0, sizeof(aRc));
                 rc = RTMpOnAll(HWACCMR0InitCPU, (void *)u32VendorEBX, aRc);
@@ -359,13 +366,13 @@ VMMR0DECL(int) HWACCMR0Init(void)
                 if (RT_SUCCESS(rc))
                     rc = hwaccmR0CheckCpuRcArray(aRc, RT_ELEMENTS(aRc), &idCpu);
 
-                AssertMsgRC(rc, ("HWACCMR0InitCPU failed for cpu %d with rc=%d\n", idCpu, rc));
-
+#ifndef DEBUG_bird
+                AssertMsg(rc == VINF_SUCCESS || rc == VERR_SVM_IN_USE, ("HWACCMR0InitCPU failed for cpu %d with rc=%d\n", idCpu, rc));
+#endif
                 if (RT_SUCCESS(rc))
                 {
-                    /* Query AMD features. */
-                    ASMCpuId(0x8000000A, &HWACCMR0Globals.svm.u32Rev, &HWACCMR0Globals.uMaxASID, &u32Dummy, &HWACCMR0Globals.svm.u32Features);
-
+                    /* Read the HWCR msr for diagnostics. */
+                    HWACCMR0Globals.svm.msrHWCR    = ASMRdMsr(MSR_K8_HWCR);
                     HWACCMR0Globals.svm.fSupported = true;
                 }
                 else
@@ -394,7 +401,7 @@ VMMR0DECL(int) HWACCMR0Init(void)
         HWACCMR0Globals.pfnSetupVM          = VMXR0SetupVM;
     }
     else
-    if (HWACCMR0Globals.svm.fSupported)
+    if (fAMDVPresent)
     {
         HWACCMR0Globals.pfnEnterSession     = SVMR0Enter;
         HWACCMR0Globals.pfnLeaveSession     = SVMR0Leave;
@@ -472,7 +479,7 @@ VMMR0DECL(int) HWACCMR0Term(void)
         if (!HWACCMR0Globals.vmx.fUsingSUPR0EnableVTx)
         {
             rc = RTPowerNotificationDeregister(hwaccmR0PowerCallback, 0);
-            Assert(RT_SUCCESS(rc));
+            AssertRC(rc);
         }
         else
             rc = VINF_SUCCESS;
@@ -555,22 +562,28 @@ static DECLCALLBACK(void) HWACCMR0InitCPU(RTCPUID idCpu, void *pvUser1, void *pv
         {
             /* Turn on SVM in the EFER MSR. */
             val = ASMRdMsr(MSR_K6_EFER);
-            if (!(val & MSR_K6_EFER_SVME))
-                ASMWrMsr(MSR_K6_EFER, val | MSR_K6_EFER_SVME);
-
-            /* Paranoia. */
-            val = ASMRdMsr(MSR_K6_EFER);
             if (val & MSR_K6_EFER_SVME)
             {
-                /* Restore previous value. */
-                ASMWrMsr(MSR_K6_EFER, val & ~MSR_K6_EFER_SVME);
-                paRc[idCpu] = VINF_SUCCESS;
+                paRc[idCpu] = VERR_SVM_IN_USE;
             }
             else
-                paRc[idCpu] = VERR_SVM_ILLEGAL_EFER_MSR;
+            {
+                ASMWrMsr(MSR_K6_EFER, val | MSR_K6_EFER_SVME);
+
+                /* Paranoia. */
+                val = ASMRdMsr(MSR_K6_EFER);
+                if (val & MSR_K6_EFER_SVME)
+                {
+                    /* Restore previous value. */
+                    ASMWrMsr(MSR_K6_EFER, val & ~MSR_K6_EFER_SVME);
+                    paRc[idCpu] = VINF_SUCCESS;
+                }
+                else
+                    paRc[idCpu] = VERR_SVM_ILLEGAL_EFER_MSR;
+            }
         }
         else
-            paRc[idCpu] = HWACCMR0Globals.lLastError = VERR_SVM_DISABLED;
+            paRc[idCpu] = VERR_SVM_DISABLED;
     }
     else
         AssertFailed(); /* can't happen */
@@ -887,6 +900,7 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
 
     pVM->hwaccm.s.vmx.msr.feature_ctrl      = HWACCMR0Globals.vmx.msr.feature_ctrl;
     pVM->hwaccm.s.vmx.hostCR4               = HWACCMR0Globals.vmx.hostCR4;
+    pVM->hwaccm.s.vmx.hostEFER              = HWACCMR0Globals.vmx.hostEFER;
     pVM->hwaccm.s.vmx.msr.vmx_basic_info    = HWACCMR0Globals.vmx.msr.vmx_basic_info;
     pVM->hwaccm.s.vmx.msr.vmx_pin_ctls      = HWACCMR0Globals.vmx.msr.vmx_pin_ctls;
     pVM->hwaccm.s.vmx.msr.vmx_proc_ctls     = HWACCMR0Globals.vmx.msr.vmx_proc_ctls;
@@ -900,6 +914,7 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
     pVM->hwaccm.s.vmx.msr.vmx_cr4_fixed1    = HWACCMR0Globals.vmx.msr.vmx_cr4_fixed1;
     pVM->hwaccm.s.vmx.msr.vmx_vmcs_enum     = HWACCMR0Globals.vmx.msr.vmx_vmcs_enum;
     pVM->hwaccm.s.vmx.msr.vmx_eptcaps       = HWACCMR0Globals.vmx.msr.vmx_eptcaps;
+    pVM->hwaccm.s.svm.msrHWCR               = HWACCMR0Globals.svm.msrHWCR;
     pVM->hwaccm.s.svm.u32Rev                = HWACCMR0Globals.svm.u32Rev;
     pVM->hwaccm.s.svm.u32Features           = HWACCMR0Globals.svm.u32Features;
     pVM->hwaccm.s.cpuid.u32AMDFeatureECX    = HWACCMR0Globals.cpuid.u32AMDFeatureECX;
@@ -918,7 +933,7 @@ VMMR0DECL(int) HWACCMR0InitVM(PVM pVM)
 #endif
     }
 
-    for (VMCPUID i = 0; i < pVM->cCPUs; i++)
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         PVMCPU pVCpu = &pVM->aCpus[i];
 
@@ -1003,7 +1018,7 @@ VMMR0DECL(int) HWACCMR0SetupVM(PVM pVM)
 
     ASMAtomicWriteBool(&pCpu->fInUse, true);
 
-    for (VMCPUID i = 0; i < pVM->cCPUs; i++)
+    for (VMCPUID i = 0; i < pVM->cCpus; i++)
     {
         /* On first entry we'll sync everything. */
         pVM->aCpus[i].hwaccm.s.fContextUseFlags = HWACCM_CHANGED_ALL;
@@ -1077,6 +1092,10 @@ VMMR0DECL(int) HWACCMR0Enter(PVM pVM, PVMCPU pVCpu)
         AssertRCReturn(rc, rc);
     }
 
+#ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
+    bool fStartedSet = PGMDynMapStartOrMigrateAutoSet(pVCpu);
+#endif
+
     rc  = HWACCMR0Globals.pfnEnterSession(pVM, pVCpu, pCpu);
     AssertRC(rc);
     /* We must save the host context here (VT-x) as we might be rescheduled on a different cpu after a long jump back to ring 3. */
@@ -1085,14 +1104,13 @@ VMMR0DECL(int) HWACCMR0Enter(PVM pVM, PVMCPU pVCpu)
     rc |= HWACCMR0Globals.pfnLoadGuestState(pVM, pVCpu, pCtx);
     AssertRC(rc);
 
-    /* keep track of the CPU owning the VMCS for debugging scheduling weirdness and ring-3 calls. */
-    if (RT_SUCCESS(rc))
-    {
 #ifdef VBOX_WITH_2X_4GB_ADDR_SPACE
-        PGMDynMapMigrateAutoSet(pVCpu);
+    if (fStartedSet)
+        PGMDynMapReleaseAutoSet(pVCpu);
 #endif
-    }
-    else
+
+    /* keep track of the CPU owning the VMCS for debugging scheduling weirdness and ring-3 calls. */
+    if (RT_FAILURE(rc))
         pVCpu->hwaccm.s.idEnteredCpu = NIL_RTCPUID;
     return rc;
 }
@@ -1132,6 +1150,15 @@ VMMR0DECL(int) HWACCMR0Leave(PVM pVM, PVMCPU pVCpu)
 
     rc = HWACCMR0Globals.pfnLeaveSession(pVM, pVCpu, pCtx);
 
+    /* We don't pass on invlpg information to the recompiler for nested paging guests, so we must make sure the recompiler flushes its TLB
+     * the next time it executes code.
+     */
+    if (    pVM->hwaccm.s.fNestedPaging
+        &&  CPUMIsGuestInPagedProtectedModeEx(pCtx))
+    {
+        CPUMSetChangedFlags(pVCpu, CPUM_CHANGED_GLOBAL_TLB_FLUSH);
+    }
+
     /* keep track of the CPU owning the VMCS for debugging scheduling weirdness and ring-3 calls. */
 #ifdef RT_STRICT
     if (RT_UNLIKELY(    pVCpu->hwaccm.s.idEnteredCpu != idCpu
@@ -1300,7 +1327,7 @@ VMMR0DECL(PVMCPU)  HWACCMR0GetVMCPU(PVM pVM)
     RTCPUID idHostCpu = RTMpCpuId();
 
     /** @todo optimize for large number of VCPUs when that becomes more common. */
-    for (VMCPUID idCpu = 0; idCpu < pVM->cCPUs; idCpu++)
+    for (VMCPUID idCpu = 0; idCpu < pVM->cCpus; idCpu++)
     {
         PVMCPU pVCpu = &pVM->aCpus[idCpu];
 
@@ -1380,7 +1407,7 @@ VMMR0DECL(int) HWACCMR0EnterSwitcher(PVM pVM, bool *pfVTxDisabled)
 
     if (    HWACCMR0Globals.enmHwAccmState != HWACCMSTATE_ENABLED
         ||  !HWACCMR0Globals.vmx.fSupported /* no such issues with AMD-V */
-        ||  !pVM->hwaccm.s.fGlobalInit      /* Local init implies the CPU is currently not in VMX root mode. */)
+        ||  !HWACCMR0Globals.fGlobalInit    /* Local init implies the CPU is currently not in VMX root mode. */)
         return VINF_SUCCESS;    /* nothing to do */
 
     switch(VMMGetSwitcher(pVM))
@@ -1428,7 +1455,7 @@ VMMR0DECL(int) HWACCMR0LeaveSwitcher(PVM pVM, bool fVTxDisabled)
 
     Assert(   HWACCMR0Globals.enmHwAccmState == HWACCMSTATE_ENABLED
            && HWACCMR0Globals.vmx.fSupported
-           && pVM->hwaccm.s.fGlobalInit);
+           && HWACCMR0Globals.fGlobalInit);
 
     PHWACCM_CPUINFO pCpu = HWACCMR0GetCurrentCpu();
     void           *pvPageCpu;
@@ -1684,11 +1711,11 @@ VMMR0DECL(void) HWACCMDumpRegs(PVM pVM, PVMCPU pVCpu, PCPUMCTX pCtx)
 
     Log(("FPU:\n"
         "FCW=%04x FSW=%04x FTW=%02x\n"
-        "res1=%02x FOP=%04x FPUIP=%08x CS=%04x Rsvrd1=%04x\n"
+        "FOP=%04x FPUIP=%08x CS=%04x Rsvrd1=%04x\n"
         "FPUDP=%04x DS=%04x Rsvrd2=%04x MXCSR=%08x MXCSR_MASK=%08x\n"
         ,
         pCtx->fpu.FCW, pCtx->fpu.FSW, pCtx->fpu.FTW,
-        pCtx->fpu.huh1, pCtx->fpu.FOP, pCtx->fpu.FPUIP, pCtx->fpu.CS, pCtx->fpu.Rsvrd1,
+        pCtx->fpu.FOP, pCtx->fpu.FPUIP, pCtx->fpu.CS, pCtx->fpu.Rsvrd1,
         pCtx->fpu.FPUDP, pCtx->fpu.DS, pCtx->fpu.Rsrvd2,
         pCtx->fpu.MXCSR, pCtx->fpu.MXCSR_MASK));
 
