@@ -1,10 +1,10 @@
-/* $Id: VMMSwitcher.cpp $ */
+/* $Id: VMMSwitcher.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VMM - The Virtual Machine Monitor, World Switcher(s).
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 /*******************************************************************************
@@ -51,7 +47,8 @@
 static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
 {
     NULL, /* invalid entry */
-#ifndef RT_ARCH_AMD64
+#ifdef VBOX_WITH_RAW_MODE
+# ifndef RT_ARCH_AMD64
     &vmmR3Switcher32BitTo32Bit_Def,
     &vmmR3Switcher32BitToPAE_Def,
     &vmmR3Switcher32BitToAMD64_Def,
@@ -59,13 +56,13 @@ static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
     &vmmR3SwitcherPAEToPAE_Def,
     &vmmR3SwitcherPAEToAMD64_Def,
     NULL,   //&vmmR3SwitcherPAETo32Bit_Def,
-# ifdef VBOX_WITH_HYBRID_32BIT_KERNEL
+#  ifdef VBOX_WITH_HYBRID_32BIT_KERNEL
     &vmmR3SwitcherAMD64ToPAE_Def,
-# else
+#  else
     NULL,   //&vmmR3SwitcherAMD64ToPAE_Def,
-# endif
+#  endif
     NULL    //&vmmR3SwitcherAMD64ToAMD64_Def,
-#else  /* RT_ARCH_AMD64 */
+# else  /* RT_ARCH_AMD64 */
     NULL,   //&vmmR3Switcher32BitTo32Bit_Def,
     NULL,   //&vmmR3Switcher32BitToPAE_Def,
     NULL,   //&vmmR3Switcher32BitToAMD64_Def,
@@ -75,7 +72,18 @@ static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
     &vmmR3SwitcherAMD64To32Bit_Def,
     &vmmR3SwitcherAMD64ToPAE_Def,
     NULL    //&vmmR3SwitcherAMD64ToAMD64_Def,
-#endif /* RT_ARCH_AMD64 */
+# endif /* RT_ARCH_AMD64 */
+#else  /* !VBOX_WITH_RAW_MODE */
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+#endif /* !VBOX_WITH_RAW_MODE */
 };
 
 
@@ -90,6 +98,9 @@ static PVMMSWITCHERDEF s_apSwitchers[VMMSWITCHER_MAX] =
  */
 int vmmR3SwitcherInit(PVM pVM)
 {
+#ifndef VBOX_WITH_RAW_MODE
+    return VINF_SUCCESS;
+#else
     /*
      * Calc the size.
      */
@@ -110,7 +121,7 @@ int vmmR3SwitcherInit(PVM pVM)
      * conflicts in the intermediate mapping of the code.
      */
     pVM->vmm.s.cbCoreCode = RT_ALIGN_32(cbCoreCode, PAGE_SIZE);
-    pVM->vmm.s.pvCoreCodeR3 = SUPContAlloc2(pVM->vmm.s.cbCoreCode >> PAGE_SHIFT, &pVM->vmm.s.pvCoreCodeR0, &pVM->vmm.s.HCPhysCoreCode);
+    pVM->vmm.s.pvCoreCodeR3 = SUPR3ContAlloc(pVM->vmm.s.cbCoreCode >> PAGE_SHIFT, &pVM->vmm.s.pvCoreCodeR0, &pVM->vmm.s.HCPhysCoreCode);
     int rc = VERR_NO_MEMORY;
     if (pVM->vmm.s.pvCoreCodeR3)
     {
@@ -136,7 +147,7 @@ int vmmR3SwitcherInit(PVM pVM)
                 i++;
                 pVM->vmm.s.pvCoreCodeR0 = NIL_RTR0PTR;
                 pVM->vmm.s.HCPhysCoreCode = NIL_RTHCPHYS;
-                pVM->vmm.s.pvCoreCodeR3 = SUPContAlloc2(pVM->vmm.s.cbCoreCode >> PAGE_SHIFT, &pVM->vmm.s.pvCoreCodeR0, &pVM->vmm.s.HCPhysCoreCode);
+                pVM->vmm.s.pvCoreCodeR3 = SUPR3ContAlloc(pVM->vmm.s.cbCoreCode >> PAGE_SHIFT, &pVM->vmm.s.pvCoreCodeR0, &pVM->vmm.s.HCPhysCoreCode);
                 if (!pVM->vmm.s.pvCoreCodeR3)
                     break;
                 rc = PGMR3MapIntermediate(pVM, pVM->vmm.s.pvCoreCodeR0, pVM->vmm.s.HCPhysCoreCode, cbCoreCode);
@@ -157,7 +168,7 @@ int vmmR3SwitcherInit(PVM pVM)
             {
                 LogRel(("Core code alloc attempt #%d: pvR3=%p pvR0=%p HCPhys=%RHp\n",
                         i, paBadTries[i].pvR3, paBadTries[i].pvR0, paBadTries[i].HCPhys));
-                SUPContFree(paBadTries[i].pvR3, paBadTries[i].cb >> PAGE_SHIFT);
+                SUPR3ContFree(paBadTries[i].pvR3, paBadTries[i].cb >> PAGE_SHIFT);
             }
             RTMemTmpFree(paBadTries);
         }
@@ -199,7 +210,7 @@ int vmmR3SwitcherInit(PVM pVM)
 
         /* shit */
         AssertMsgFailed(("PGMR3Map(,%RRv, %RHp, %#x, 0) failed with rc=%Rrc\n", pVM->vmm.s.pvCoreCodeRC, pVM->vmm.s.HCPhysCoreCode, cbCoreCode, rc));
-        SUPContFree(pVM->vmm.s.pvCoreCodeR3, pVM->vmm.s.cbCoreCode >> PAGE_SHIFT);
+        SUPR3ContFree(pVM->vmm.s.pvCoreCodeR3, pVM->vmm.s.cbCoreCode >> PAGE_SHIFT);
     }
     else
         VMSetError(pVM, rc, RT_SRC_POS,
@@ -210,6 +221,7 @@ int vmmR3SwitcherInit(PVM pVM)
     pVM->vmm.s.pvCoreCodeR0 = NIL_RTR0PTR;
     pVM->vmm.s.pvCoreCodeRC = 0;
     return rc;
+#endif
 }
 
 /**
@@ -220,6 +232,7 @@ int vmmR3SwitcherInit(PVM pVM)
  */
 void vmmR3SwitcherRelocate(PVM pVM, RTGCINTPTR offDelta)
 {
+#ifdef VBOX_WITH_RAW_MODE
     /*
      * Relocate all the switchers.
      */
@@ -250,6 +263,7 @@ void vmmR3SwitcherRelocate(PVM pVM, RTGCINTPTR offDelta)
     pVM->pfnVMMGCGuestToHostAsmGuestCtx = RCPtr + pSwitcher->offGCGuestToHostAsmGuestCtx;
 
 //    AssertFailed();
+#endif
 }
 
 
@@ -401,6 +415,17 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
                 break;
             }
 
+            /*
+             * Make 32-bit GC pointer given CPUMCPU offset.
+             */
+            case FIX_GC_CPUMCPU_OFF:
+            {
+                uint32_t offCPUM = *u.pu32++;
+                Assert(offCPUM < sizeof(pVM->aCpus[0].cpum));
+                *uSrc.pu32 = (uint32_t)(VM_RC_ADDR(pVM, &pVM->aCpus[0].cpum) + offCPUM);
+                break;
+            }
+
             /*
              * Make 32-bit GC pointer given VM offset.
              */
@@ -546,13 +571,13 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
             }
 
             /*
-             * Insert relative jump to specified target it SYSENTER isn't used by the host.
+             * Insert relative jump to specified target it SYSCALL isn't used by the host.
              */
             case FIX_NO_SYSCALL_JMP:
             {
                 uint32_t offTrg = *u.pu32++;
                 Assert(offTrg < pSwitcher->cbCode);
-                if (!CPUMIsHostUsingSysEnter(pVM))
+                if (!CPUMIsHostUsingSysCall(pVM))
                 {
                     *uSrc.pu8++ = 0xe9; /* jmp rel32 */
                     *uSrc.pu32++ = offTrg - (offSrc + 5);
@@ -699,7 +724,7 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
                     "   pCPUMR3     = %p\n"
                     "   GCPtrGDT    = %RGv\n"
                     "   InterCR3s   = %08RHp, %08RHp, %08RHp (32-Bit, PAE, AMD64)\n"
-                    "   HyperCR3s   = %08RHp, %08RHp, %08RHp (32-Bit, PAE, AMD64)\n"
+                    "   HyperCR3s   = %08RHp (32-Bit, PAE & AMD64)\n"
                     "   SelCS       = %04x\n"
                     "   SelDS       = %04x\n"
                     "   SelCS64     = %04x\n"
@@ -715,8 +740,7 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
                     &pVM->cpum,
                     GCPtrGDT,
                     PGMGetInter32BitCR3(pVM), PGMGetInterPaeCR3(pVM), PGMGetInterAmd64CR3(pVM),
-                    /* @todo No need for three GetHyper calls; one and the same base is used */
-                    PGMGetHyper32BitCR3(pVM), PGMGetHyperPaeCR3(pVM), PGMGetHyperAmd64CR3(pVM),
+                    PGMGetHyperCR3(VMMGetCpu(pVM)),
                     SelCS, SelDS, SelCS64, SelTSS);
 
         uint32_t offCode = 0;
@@ -766,7 +790,7 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
             else
             {
                 RTLogPrintf("  %04x: %02x '%c' (nowhere)\n",
-                            offCode, pu8CodeR3[offCode], isprint(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
+                            offCode, pu8CodeR3[offCode], RT_C_IS_PRINT(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
                 offCode++;
                 continue;
             }
@@ -798,12 +822,12 @@ static void vmmR3SwitcherGenericRelocate(PVM pVM, PVMMSWITCHERDEF pSwitcher, RTR
                 /* disas */
                 uint32_t cbInstr = 0;
                 char szDisas[256];
-                if (RT_SUCCESS(DISInstr(&Cpu, (RTUINTPTR)pu8CodeR3 + offCode, uBase - (RTUINTPTR)pu8CodeR3, &cbInstr, szDisas)))
+                if (RT_SUCCESS(DISInstr(&Cpu, (uintptr_t)pu8CodeR3 + offCode, uBase - (uintptr_t)pu8CodeR3, &cbInstr, szDisas)))
                     RTLogPrintf("  %04x: %s", offCode, szDisas); //for whatever reason szDisas includes '\n'.
                 else
                 {
                     RTLogPrintf("  %04x: %02x '%c'\n",
-                                offCode, pu8CodeR3[offCode], isprint(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
+                                offCode, pu8CodeR3[offCode], RT_C_IS_PRINT(pu8CodeR3[offCode]) ? pu8CodeR3[offCode] : ' ');
                     cbInstr = 1;
                 }
                 offCode += cbInstr;
@@ -962,6 +986,7 @@ VMMR3DECL(int) VMMR3DisableSwitcher(PVM pVM)
     return VINF_SUCCESS;
 }
 
+
 /**
  * Gets the switcher to be used for switching to GC.
  *
@@ -978,7 +1003,7 @@ VMMR3DECL(RTR0PTR) VMMR3GetHostToGuestSwitcher(PVM pVM, VMMSWITCHER enmSwitcher)
         ||  enmSwitcher >= VMMSWITCHER_MAX)
     {
         AssertMsgFailed(("Invalid input enmSwitcher=%d\n", enmSwitcher));
-        return VERR_INVALID_PARAMETER;
+        return NIL_RTR0PTR;
     }
 
     /*
@@ -990,5 +1015,5 @@ VMMR3DECL(RTR0PTR) VMMR3GetHostToGuestSwitcher(PVM pVM, VMMSWITCHER enmSwitcher)
         RTR0PTR     pbCodeR0 = (RTR0PTR)pVM->vmm.s.pvCoreCodeR0 + pVM->vmm.s.aoffSwitchers[enmSwitcher]; /** @todo fix the pvCoreCodeR0 type */
         return pbCodeR0 + pSwitcher->offR0HostToGuest;
     }
-    return (RTR0PTR)0;
+    return NIL_RTR0PTR;
 }
