@@ -1,10 +1,10 @@
-/* $Id: VMMTests.cpp $ */
+/* $Id: VMMTests.cpp 29250 2010-05-09 17:53:58Z vboxsync $ */
 /** @file
  * VMM - The Virtual Machine Monitor Core, Tests.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 //#define NO_SUPCALLR0VMM
@@ -25,9 +21,11 @@
 *   Header Files                                                               *
 *******************************************************************************/
 #define LOG_GROUP LOG_GROUP_VMM
+#include <iprt/asm-amd64-x86.h> /* for SUPGetCpuHzFromGIP */
 #include <VBox/vmm.h>
-#include <VBox/pdm.h>
+#include <VBox/pdmapi.h>
 #include <VBox/cpum.h>
+#include <VBox/dbg.h>
 #include <VBox/mm.h>
 #include <VBox/trpm.h>
 #include <VBox/selm.h>
@@ -55,24 +53,26 @@
  */
 static int vmmR3DoGCTest(PVM pVM, VMMGCOPERATION enmTestcase, unsigned uVariation)
 {
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     RTRCPTR RCPtrEP;
     int rc = PDMR3LdrGetSymbolRC(pVM, VMMGC_MAIN_MODULE_NAME, "VMMGCEntry", &RCPtrEP);
     if (RT_FAILURE(rc))
         return rc;
 
-    CPUMHyperSetCtxCore(pVM, NULL);
-    memset(pVM->vmm.s.pbEMTStackR3, 0xaa, VMM_STACK_SIZE);
-    CPUMSetHyperESP(pVM, pVM->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
-    CPUMPushHyper(pVM, uVariation);
-    CPUMPushHyper(pVM, enmTestcase);
-    CPUMPushHyper(pVM, pVM->pVMRC);
-    CPUMPushHyper(pVM, 3 * sizeof(RTRCPTR));    /* stack frame size */
-    CPUMPushHyper(pVM, RCPtrEP);                /* what to call */
-    CPUMSetHyperEIP(pVM, pVM->vmm.s.pfnCallTrampolineRC);
-    Assert(CPUMGetHyperCR3(pVM) && CPUMGetHyperCR3(pVM) == PGMGetHyperCR3(pVM));
-    rc = SUPCallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
+    CPUMHyperSetCtxCore(pVCpu, NULL);
+    memset(pVCpu->vmm.s.pbEMTStackR3, 0xaa, VMM_STACK_SIZE);
+    CPUMSetHyperESP(pVCpu, pVCpu->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
+    CPUMPushHyper(pVCpu, uVariation);
+    CPUMPushHyper(pVCpu, enmTestcase);
+    CPUMPushHyper(pVCpu, pVM->pVMRC);
+    CPUMPushHyper(pVCpu, 3 * sizeof(RTRCPTR));    /* stack frame size */
+    CPUMPushHyper(pVCpu, RCPtrEP);                /* what to call */
+    CPUMSetHyperEIP(pVCpu, pVM->vmm.s.pfnCallTrampolineRC);
+    Assert(CPUMGetHyperCR3(pVCpu) && CPUMGetHyperCR3(pVCpu) == PGMGetHyperCR3(pVCpu));
+    rc = SUPR3CallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
     if (RT_LIKELY(rc == VINF_SUCCESS))
-        rc = pVM->vmm.s.iLastGZRc;
+        rc = pVCpu->vmm.s.iLastGZRc;
     return rc;
 }
 
@@ -91,6 +91,8 @@ static int vmmR3DoGCTest(PVM pVM, VMMGCOPERATION enmTestcase, unsigned uVariatio
  */
 static int vmmR3DoTrapTest(PVM pVM, uint8_t u8Trap, unsigned uVariation, int rcExpect, uint32_t u32Eax, const char *pszFaultEIP, const char *pszDesc)
 {
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
     RTPrintf("VMM: testing 0%x / %d - %s\n", u8Trap, uVariation, pszDesc);
 
     RTRCPTR RCPtrEP;
@@ -98,19 +100,19 @@ static int vmmR3DoTrapTest(PVM pVM, uint8_t u8Trap, unsigned uVariation, int rcE
     if (RT_FAILURE(rc))
         return rc;
 
-    CPUMHyperSetCtxCore(pVM, NULL);
-    memset(pVM->vmm.s.pbEMTStackR3, 0xaa, VMM_STACK_SIZE);
-    CPUMSetHyperESP(pVM, pVM->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
-    CPUMPushHyper(pVM, uVariation);
-    CPUMPushHyper(pVM, u8Trap + VMMGC_DO_TESTCASE_TRAP_FIRST);
-    CPUMPushHyper(pVM, pVM->pVMRC);
-    CPUMPushHyper(pVM, 3 * sizeof(RTRCPTR));    /* stack frame size */
-    CPUMPushHyper(pVM, RCPtrEP);                /* what to call */
-    CPUMSetHyperEIP(pVM, pVM->vmm.s.pfnCallTrampolineRC);
-    Assert(CPUMGetHyperCR3(pVM) && CPUMGetHyperCR3(pVM) == PGMGetHyperCR3(pVM));
-    rc = SUPCallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
+    CPUMHyperSetCtxCore(pVCpu, NULL);
+    memset(pVCpu->vmm.s.pbEMTStackR3, 0xaa, VMM_STACK_SIZE);
+    CPUMSetHyperESP(pVCpu, pVCpu->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
+    CPUMPushHyper(pVCpu, uVariation);
+    CPUMPushHyper(pVCpu, u8Trap + VMMGC_DO_TESTCASE_TRAP_FIRST);
+    CPUMPushHyper(pVCpu, pVM->pVMRC);
+    CPUMPushHyper(pVCpu, 3 * sizeof(RTRCPTR));    /* stack frame size */
+    CPUMPushHyper(pVCpu, RCPtrEP);                /* what to call */
+    CPUMSetHyperEIP(pVCpu, pVM->vmm.s.pfnCallTrampolineRC);
+    Assert(CPUMGetHyperCR3(pVCpu) && CPUMGetHyperCR3(pVCpu) == PGMGetHyperCR3(pVCpu));
+    rc = SUPR3CallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
     if (RT_LIKELY(rc == VINF_SUCCESS))
-        rc = pVM->vmm.s.iLastGZRc;
+        rc = pVCpu->vmm.s.iLastGZRc;
     bool fDump = false;
     if (rc != rcExpect)
     {
@@ -122,9 +124,9 @@ static int vmmR3DoTrapTest(PVM pVM, uint8_t u8Trap, unsigned uVariation, int rcE
              &&  u8Trap != 8 /* double fault doesn't dare set TrapNo. */
              &&  u8Trap != 3 /* guest only, we're not in guest. */
              &&  u8Trap != 1 /* guest only, we're not in guest. */
-             &&  u8Trap != TRPMGetTrapNo(pVM))
+             &&  u8Trap != TRPMGetTrapNo(pVCpu))
     {
-        RTPrintf("VMM: FAILURE - Trap %#x expected %#x\n", TRPMGetTrapNo(pVM), u8Trap);
+        RTPrintf("VMM: FAILURE - Trap %#x expected %#x\n", TRPMGetTrapNo(pVCpu), u8Trap);
         fDump = true;
     }
     else if (pszFaultEIP)
@@ -133,41 +135,41 @@ static int vmmR3DoTrapTest(PVM pVM, uint8_t u8Trap, unsigned uVariation, int rcE
         int rc2 = PDMR3LdrGetSymbolRC(pVM, VMMGC_MAIN_MODULE_NAME, pszFaultEIP, &RCPtrFault);
         if (RT_FAILURE(rc2))
             RTPrintf("VMM: FAILURE - Failed to resolve symbol '%s', %Rrc!\n", pszFaultEIP, rc);
-        else if (RCPtrFault != CPUMGetHyperEIP(pVM))
+        else if (RCPtrFault != CPUMGetHyperEIP(pVCpu))
         {
-            RTPrintf("VMM: FAILURE - EIP=%08RX32 expected %RRv (%s)\n", CPUMGetHyperEIP(pVM), RCPtrFault, pszFaultEIP);
+            RTPrintf("VMM: FAILURE - EIP=%08RX32 expected %RRv (%s)\n", CPUMGetHyperEIP(pVCpu), RCPtrFault, pszFaultEIP);
             fDump = true;
         }
     }
     else if (rcExpect != VINF_SUCCESS)
     {
-        if (CPUMGetHyperSS(pVM) == SELMGetHyperDS(pVM))
-            RTPrintf("VMM: FAILURE - ss=%x expected %x\n", CPUMGetHyperSS(pVM), SELMGetHyperDS(pVM));
-        if (CPUMGetHyperES(pVM) == SELMGetHyperDS(pVM))
-            RTPrintf("VMM: FAILURE - es=%x expected %x\n", CPUMGetHyperES(pVM), SELMGetHyperDS(pVM));
-        if (CPUMGetHyperDS(pVM) == SELMGetHyperDS(pVM))
-            RTPrintf("VMM: FAILURE - ds=%x expected %x\n", CPUMGetHyperDS(pVM), SELMGetHyperDS(pVM));
-        if (CPUMGetHyperFS(pVM) == SELMGetHyperDS(pVM))
-            RTPrintf("VMM: FAILURE - fs=%x expected %x\n", CPUMGetHyperFS(pVM), SELMGetHyperDS(pVM));
-        if (CPUMGetHyperGS(pVM) == SELMGetHyperDS(pVM))
-            RTPrintf("VMM: FAILURE - gs=%x expected %x\n", CPUMGetHyperGS(pVM), SELMGetHyperDS(pVM));
-        if (CPUMGetHyperEDI(pVM) == 0x01234567)
-            RTPrintf("VMM: FAILURE - edi=%x expected %x\n", CPUMGetHyperEDI(pVM), 0x01234567);
-        if (CPUMGetHyperESI(pVM) == 0x42000042)
-            RTPrintf("VMM: FAILURE - esi=%x expected %x\n", CPUMGetHyperESI(pVM), 0x42000042);
-        if (CPUMGetHyperEBP(pVM) == 0xffeeddcc)
-            RTPrintf("VMM: FAILURE - ebp=%x expected %x\n", CPUMGetHyperEBP(pVM), 0xffeeddcc);
-        if (CPUMGetHyperEBX(pVM) == 0x89abcdef)
-            RTPrintf("VMM: FAILURE - ebx=%x expected %x\n", CPUMGetHyperEBX(pVM), 0x89abcdef);
-        if (CPUMGetHyperECX(pVM) == 0xffffaaaa)
-            RTPrintf("VMM: FAILURE - ecx=%x expected %x\n", CPUMGetHyperECX(pVM), 0xffffaaaa);
-        if (CPUMGetHyperEDX(pVM) == 0x77778888)
-            RTPrintf("VMM: FAILURE - edx=%x expected %x\n", CPUMGetHyperEDX(pVM), 0x77778888);
-        if (CPUMGetHyperEAX(pVM) == u32Eax)
-            RTPrintf("VMM: FAILURE - eax=%x expected %x\n", CPUMGetHyperEAX(pVM), u32Eax);
+        if (CPUMGetHyperSS(pVCpu) == SELMGetHyperDS(pVM))
+            RTPrintf("VMM: FAILURE - ss=%x expected %x\n", CPUMGetHyperSS(pVCpu), SELMGetHyperDS(pVM));
+        if (CPUMGetHyperES(pVCpu) == SELMGetHyperDS(pVM))
+            RTPrintf("VMM: FAILURE - es=%x expected %x\n", CPUMGetHyperES(pVCpu), SELMGetHyperDS(pVM));
+        if (CPUMGetHyperDS(pVCpu) == SELMGetHyperDS(pVM))
+            RTPrintf("VMM: FAILURE - ds=%x expected %x\n", CPUMGetHyperDS(pVCpu), SELMGetHyperDS(pVM));
+        if (CPUMGetHyperFS(pVCpu) == SELMGetHyperDS(pVM))
+            RTPrintf("VMM: FAILURE - fs=%x expected %x\n", CPUMGetHyperFS(pVCpu), SELMGetHyperDS(pVM));
+        if (CPUMGetHyperGS(pVCpu) == SELMGetHyperDS(pVM))
+            RTPrintf("VMM: FAILURE - gs=%x expected %x\n", CPUMGetHyperGS(pVCpu), SELMGetHyperDS(pVM));
+        if (CPUMGetHyperEDI(pVCpu) == 0x01234567)
+            RTPrintf("VMM: FAILURE - edi=%x expected %x\n", CPUMGetHyperEDI(pVCpu), 0x01234567);
+        if (CPUMGetHyperESI(pVCpu) == 0x42000042)
+            RTPrintf("VMM: FAILURE - esi=%x expected %x\n", CPUMGetHyperESI(pVCpu), 0x42000042);
+        if (CPUMGetHyperEBP(pVCpu) == 0xffeeddcc)
+            RTPrintf("VMM: FAILURE - ebp=%x expected %x\n", CPUMGetHyperEBP(pVCpu), 0xffeeddcc);
+        if (CPUMGetHyperEBX(pVCpu) == 0x89abcdef)
+            RTPrintf("VMM: FAILURE - ebx=%x expected %x\n", CPUMGetHyperEBX(pVCpu), 0x89abcdef);
+        if (CPUMGetHyperECX(pVCpu) == 0xffffaaaa)
+            RTPrintf("VMM: FAILURE - ecx=%x expected %x\n", CPUMGetHyperECX(pVCpu), 0xffffaaaa);
+        if (CPUMGetHyperEDX(pVCpu) == 0x77778888)
+            RTPrintf("VMM: FAILURE - edx=%x expected %x\n", CPUMGetHyperEDX(pVCpu), 0x77778888);
+        if (CPUMGetHyperEAX(pVCpu) == u32Eax)
+            RTPrintf("VMM: FAILURE - eax=%x expected %x\n", CPUMGetHyperEAX(pVCpu), u32Eax);
     }
     if (fDump)
-        VMMR3FatalDump(pVM, rc);
+        VMMR3FatalDump(pVM, pVCpu, rc);
     return rc;
 }
 
@@ -176,6 +178,8 @@ static int vmmR3DoTrapTest(PVM pVM, uint8_t u8Trap, unsigned uVariation, int rcE
 VMMR3DECL(int) VMMDoTest(PVM pVM)
 {
 #if 1
+    PVMCPU pVCpu = &pVM->aCpus[0];
+
 #ifdef NO_SUPCALLR0VMM
     RTPrintf("NO_SUPCALLR0VMM\n");
     return VINF_SUCCESS;
@@ -259,8 +263,8 @@ VMMR3DECL(int) VMMDoTest(PVM pVM)
 
         /* resume the breakpoint */
         RTPrintf("VMM: resuming hyper after breakpoint\n");
-        CPUMSetHyperEFlags(pVM, CPUMGetHyperEFlags(pVM) | X86_EFL_RF);
-        rc = VMMR3ResumeHyper(pVM);
+        CPUMSetHyperEFlags(pVCpu, CPUMGetHyperEFlags(pVCpu) | X86_EFL_RF);
+        rc = VMMR3ResumeHyper(pVM, pVCpu);
         if (rc != VINF_SUCCESS)
         {
             RTPrintf("VMM: failed to resume on hyper breakpoint, rc=%Rrc\n", rc);
@@ -276,19 +280,19 @@ VMMR3DECL(int) VMMDoTest(PVM pVM)
             return rc;
         }
 
-        RTGCUINTREG OldPc = CPUMGetHyperEIP(pVM);
+        RTGCUINTREG OldPc = CPUMGetHyperEIP(pVCpu);
         RTPrintf("%RGr=>", OldPc);
         unsigned i;
         for (i = 0; i < 8; i++)
         {
-            CPUMSetHyperEFlags(pVM, CPUMGetHyperEFlags(pVM) | X86_EFL_TF | X86_EFL_RF);
-            rc = VMMR3ResumeHyper(pVM);
+            CPUMSetHyperEFlags(pVCpu, CPUMGetHyperEFlags(pVCpu) | X86_EFL_TF | X86_EFL_RF);
+            rc = VMMR3ResumeHyper(pVM, pVCpu);
             if (rc != VINF_EM_DBG_HYPER_STEPPED)
             {
                 RTPrintf("\nVMM: failed to step on hyper breakpoint, rc=%Rrc\n", rc);
                 return rc;
             }
-            RTGCUINTREG Pc = CPUMGetHyperEIP(pVM);
+            RTGCUINTREG Pc = CPUMGetHyperEIP(pVCpu);
             RTPrintf("%RGr=>", Pc);
             if (Pc == OldPc)
             {
@@ -334,14 +338,14 @@ VMMR3DECL(int) VMMDoTest(PVM pVM)
         /*
          * Interrupt forwarding.
          */
-        CPUMHyperSetCtxCore(pVM, NULL);
-        CPUMSetHyperESP(pVM, pVM->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
-        CPUMPushHyper(pVM, 0);
-        CPUMPushHyper(pVM, VMMGC_DO_TESTCASE_HYPER_INTERRUPT);
-        CPUMPushHyper(pVM, pVM->pVMRC);
-        CPUMPushHyper(pVM, 3 * sizeof(RTRCPTR));    /* stack frame size */
-        CPUMPushHyper(pVM, RCPtrEP);                /* what to call */
-        CPUMSetHyperEIP(pVM, pVM->vmm.s.pfnCallTrampolineRC);
+        CPUMHyperSetCtxCore(pVCpu, NULL);
+        CPUMSetHyperESP(pVCpu, pVCpu->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
+        CPUMPushHyper(pVCpu, 0);
+        CPUMPushHyper(pVCpu, VMMGC_DO_TESTCASE_HYPER_INTERRUPT);
+        CPUMPushHyper(pVCpu, pVM->pVMRC);
+        CPUMPushHyper(pVCpu, 3 * sizeof(RTRCPTR));    /* stack frame size */
+        CPUMPushHyper(pVCpu, RCPtrEP);                /* what to call */
+        CPUMSetHyperEIP(pVCpu, pVM->vmm.s.pfnCallTrampolineRC);
         Log(("trampoline=%x\n", pVM->vmm.s.pfnCallTrampolineRC));
 
         /*
@@ -351,22 +355,22 @@ VMMR3DECL(int) VMMDoTest(PVM pVM)
         i = 0;
         uint64_t    tsBegin = RTTimeNanoTS();
         uint64_t    TickStart = ASMReadTSC();
-        Assert(CPUMGetHyperCR3(pVM) && CPUMGetHyperCR3(pVM) == PGMGetHyperCR3(pVM));
+        Assert(CPUMGetHyperCR3(pVCpu) && CPUMGetHyperCR3(pVCpu) == PGMGetHyperCR3(pVCpu));
         do
         {
-            rc = SUPCallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
+            rc = SUPR3CallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
             if (RT_LIKELY(rc == VINF_SUCCESS))
-                rc = pVM->vmm.s.iLastGZRc;
+                rc = pVCpu->vmm.s.iLastGZRc;
             if (RT_FAILURE(rc))
             {
                 Log(("VMM: GC returned fatal %Rra in iteration %d\n", rc, i));
-                VMMR3FatalDump(pVM, rc);
+                VMMR3FatalDump(pVM, pVCpu, rc);
                 return rc;
             }
             i++;
             if (!(i % 32))
                 Log(("VMM: iteration %d, esi=%08x edi=%08x ebx=%08x\n",
-                       i, CPUMGetHyperESI(pVM), CPUMGetHyperEDI(pVM), CPUMGetHyperEBX(pVM)));
+                       i, CPUMGetHyperESI(pVCpu), CPUMGetHyperEDI(pVCpu), CPUMGetHyperEBX(pVCpu)));
         } while (rc == VINF_EM_RAW_INTERRUPT_HYPER);
         uint64_t    TickEnd = ASMReadTSC();
         uint64_t    tsEnd = RTTimeNanoTS();
@@ -384,8 +388,8 @@ VMMR3DECL(int) VMMDoTest(PVM pVM)
         /*
          * These forced actions are not necessary for the test and trigger breakpoints too.
          */
-        VM_FF_CLEAR(pVM, VM_FF_TRPM_SYNC_IDT);
-        VM_FF_CLEAR(pVM, VM_FF_SELM_SYNC_TSS);
+        VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
+        VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_SELM_SYNC_TSS);
 
         /*
          * Profile switching.
@@ -395,27 +399,27 @@ VMMR3DECL(int) VMMDoTest(PVM pVM)
         uint64_t TickMin = ~0;
         tsBegin = RTTimeNanoTS();
         TickStart = ASMReadTSC();
-        Assert(CPUMGetHyperCR3(pVM) && CPUMGetHyperCR3(pVM) == PGMGetHyperCR3(pVM));
+        Assert(CPUMGetHyperCR3(pVCpu) && CPUMGetHyperCR3(pVCpu) == PGMGetHyperCR3(pVCpu));
         for (i = 0; i < 1000000; i++)
         {
-            CPUMHyperSetCtxCore(pVM, NULL);
-            CPUMSetHyperESP(pVM, pVM->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
-            CPUMPushHyper(pVM, 0);
-            CPUMPushHyper(pVM, VMMGC_DO_TESTCASE_NOP);
-            CPUMPushHyper(pVM, pVM->pVMRC);
-            CPUMPushHyper(pVM, 3 * sizeof(RTRCPTR));    /* stack frame size */
-            CPUMPushHyper(pVM, RCPtrEP);                /* what to call */
-            CPUMSetHyperEIP(pVM, pVM->vmm.s.pfnCallTrampolineRC);
+            CPUMHyperSetCtxCore(pVCpu, NULL);
+            CPUMSetHyperESP(pVCpu, pVCpu->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
+            CPUMPushHyper(pVCpu, 0);
+            CPUMPushHyper(pVCpu, VMMGC_DO_TESTCASE_NOP);
+            CPUMPushHyper(pVCpu, pVM->pVMRC);
+            CPUMPushHyper(pVCpu, 3 * sizeof(RTRCPTR));    /* stack frame size */
+            CPUMPushHyper(pVCpu, RCPtrEP);                /* what to call */
+            CPUMSetHyperEIP(pVCpu, pVM->vmm.s.pfnCallTrampolineRC);
 
             uint64_t TickThisStart = ASMReadTSC();
-            rc = SUPCallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
+            rc = SUPR3CallVMMR0Fast(pVM->pVMR0, VMMR0_DO_RAW_RUN, 0);
             if (RT_LIKELY(rc == VINF_SUCCESS))
-                rc = pVM->vmm.s.iLastGZRc;
+                rc = pVCpu->vmm.s.iLastGZRc;
             uint64_t TickThisElapsed = ASMReadTSC() - TickThisStart;
             if (RT_FAILURE(rc))
             {
                 Log(("VMM: GC returned fatal %Rra in iteration %d\n", rc, i));
-                VMMR3FatalDump(pVM, rc);
+                VMMR3FatalDump(pVM, pVCpu, rc);
                 return rc;
             }
             if (TickThisElapsed < TickMin)
@@ -445,19 +449,19 @@ VMMR3DECL(int) VMMDoTest(PVM pVM)
 #define SYNC_SEL(pHyperCtx, reg)                                                        \
         if (pHyperCtx->reg)                                                             \
         {                                                                               \
-            SELMSELINFO selInfo;                                                        \
-            int rc = SELMR3GetShadowSelectorInfo(pVM, pHyperCtx->reg, &selInfo);        \
-            AssertRC(rc);                                                               \
+            DBGFSELINFO selInfo;                                                        \
+            int rc2 = SELMR3GetShadowSelectorInfo(pVM, pHyperCtx->reg, &selInfo);       \
+            AssertRC(rc2);                                                              \
                                                                                         \
             pHyperCtx->reg##Hid.u64Base              = selInfo.GCPtrBase;               \
             pHyperCtx->reg##Hid.u32Limit             = selInfo.cbLimit;                 \
-            pHyperCtx->reg##Hid.Attr.n.u1Present     = selInfo.Raw.Gen.u1Present;       \
-            pHyperCtx->reg##Hid.Attr.n.u1DefBig      = selInfo.Raw.Gen.u1DefBig;        \
-            pHyperCtx->reg##Hid.Attr.n.u1Granularity = selInfo.Raw.Gen.u1Granularity;   \
-            pHyperCtx->reg##Hid.Attr.n.u4Type        = selInfo.Raw.Gen.u4Type;          \
-            pHyperCtx->reg##Hid.Attr.n.u2Dpl         = selInfo.Raw.Gen.u2Dpl;           \
-            pHyperCtx->reg##Hid.Attr.n.u1DescType    = selInfo.Raw.Gen.u1DescType;      \
-            pHyperCtx->reg##Hid.Attr.n.u1Long        = selInfo.Raw.Gen.u1Long;          \
+            pHyperCtx->reg##Hid.Attr.n.u1Present     = selInfo.u.Raw.Gen.u1Present;     \
+            pHyperCtx->reg##Hid.Attr.n.u1DefBig      = selInfo.u.Raw.Gen.u1DefBig;      \
+            pHyperCtx->reg##Hid.Attr.n.u1Granularity = selInfo.u.Raw.Gen.u1Granularity; \
+            pHyperCtx->reg##Hid.Attr.n.u4Type        = selInfo.u.Raw.Gen.u4Type;        \
+            pHyperCtx->reg##Hid.Attr.n.u2Dpl         = selInfo.u.Raw.Gen.u2Dpl;         \
+            pHyperCtx->reg##Hid.Attr.n.u1DescType    = selInfo.u.Raw.Gen.u1DescType;    \
+            pHyperCtx->reg##Hid.Attr.n.u1Long        = selInfo.u.Raw.Gen.u1Long;        \
         }
 
 /* execute the switch. */
@@ -467,6 +471,7 @@ VMMR3DECL(int) VMMDoHwAccmTest(PVM pVM)
     int      rc;
     PCPUMCTX pHyperCtx, pGuestCtx;
     RTGCPHYS CR3Phys = 0x0; /* fake address */
+    PVMCPU   pVCpu = &pVM->aCpus[0];
 
     if (!HWACCMR3IsAllowed(pVM))
     {
@@ -477,8 +482,8 @@ VMMR3DECL(int) VMMDoHwAccmTest(PVM pVM)
     /*
      * These forced actions are not necessary for the test and trigger breakpoints too.
      */
-    VM_FF_CLEAR(pVM, VM_FF_TRPM_SYNC_IDT);
-    VM_FF_CLEAR(pVM, VM_FF_SELM_SYNC_TSS);
+    VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TRPM_SYNC_IDT);
+    VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_SELM_SYNC_TSS);
 
     /* Enable mapping of the hypervisor into the shadow page table. */
     uint32_t cb;
@@ -489,15 +494,16 @@ VMMR3DECL(int) VMMDoHwAccmTest(PVM pVM)
     rc = PGMR3MappingsFix(pVM, MM_HYPER_AREA_ADDRESS, cb);
     AssertRCReturn(rc, rc);
 
-    CPUMQueryHyperCtxPtr(pVM, &pHyperCtx);
+    CPUMQueryHyperCtxPtr(pVCpu, &pHyperCtx);
 
     pHyperCtx->cr0 = X86_CR0_PE | X86_CR0_WP | X86_CR0_PG | X86_CR0_TS | X86_CR0_ET | X86_CR0_NE | X86_CR0_MP;
     pHyperCtx->cr4 = X86_CR4_PGE | X86_CR4_OSFSXR | X86_CR4_OSXMMEEXCPT;
-    PGMChangeMode(pVM, pHyperCtx->cr0, pHyperCtx->cr4, pHyperCtx->msrEFER);
-    PGMSyncCR3(pVM, pHyperCtx->cr0, CR3Phys, pHyperCtx->cr4, true);
+    PGMChangeMode(pVCpu, pHyperCtx->cr0, pHyperCtx->cr4, pHyperCtx->msrEFER);
+    PGMSyncCR3(pVCpu, pHyperCtx->cr0, CR3Phys, pHyperCtx->cr4, true);
 
-    VM_FF_CLEAR(pVM, VM_FF_TO_R3);
-    VM_FF_CLEAR(pVM, VM_FF_TIMER);
+    VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TO_R3);
+    VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TIMER);
+    VM_FF_CLEAR(pVM, VM_FF_TM_VIRTUAL_SYNC);
     VM_FF_CLEAR(pVM, VM_FF_REQUEST);
 
     /*
@@ -509,7 +515,7 @@ VMMR3DECL(int) VMMDoHwAccmTest(PVM pVM)
     {
         RTPrintf("VMM: VMMGCEntry=%RRv\n", RCPtrEP);
 
-        CPUMQueryHyperCtxPtr(pVM, &pHyperCtx);
+        CPUMQueryHyperCtxPtr(pVCpu, &pHyperCtx);
 
         /* Fill in hidden selector registers for the hypervisor state. */
         SYNC_SEL(pHyperCtx, cs);
@@ -530,33 +536,34 @@ VMMR3DECL(int) VMMDoHwAccmTest(PVM pVM)
         uint64_t TickStart = ASMReadTSC();
         for (i = 0; i < 1000000; i++)
         {
-            CPUMHyperSetCtxCore(pVM, NULL);
+            CPUMHyperSetCtxCore(pVCpu, NULL);
 
-            CPUMSetHyperESP(pVM, pVM->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
-            CPUMPushHyper(pVM, 0);
-            CPUMPushHyper(pVM, VMMGC_DO_TESTCASE_HWACCM_NOP);
-            CPUMPushHyper(pVM, pVM->pVMRC);
-            CPUMPushHyper(pVM, 3 * sizeof(RTRCPTR));    /* stack frame size */
-            CPUMPushHyper(pVM, RCPtrEP);                /* what to call */
-            CPUMSetHyperEIP(pVM, pVM->vmm.s.pfnCallTrampolineRC);
+            CPUMSetHyperESP(pVCpu, pVCpu->vmm.s.pbEMTStackBottomRC); /* Clear the stack. */
+            CPUMPushHyper(pVCpu, 0);
+            CPUMPushHyper(pVCpu, VMMGC_DO_TESTCASE_HWACCM_NOP);
+            CPUMPushHyper(pVCpu, pVM->pVMRC);
+            CPUMPushHyper(pVCpu, 3 * sizeof(RTRCPTR));    /* stack frame size */
+            CPUMPushHyper(pVCpu, RCPtrEP);                /* what to call */
+            CPUMSetHyperEIP(pVCpu, pVM->vmm.s.pfnCallTrampolineRC);
 
-            CPUMQueryHyperCtxPtr(pVM, &pHyperCtx);
-            pGuestCtx = CPUMQueryGuestCtxPtr(pVM);
+            CPUMQueryHyperCtxPtr(pVCpu, &pHyperCtx);
+            pGuestCtx = CPUMQueryGuestCtxPtr(pVCpu);
 
             /* Copy the hypervisor context to make sure we have a valid guest context. */
             *pGuestCtx = *pHyperCtx;
             pGuestCtx->cr3 = CR3Phys;
 
-            VM_FF_CLEAR(pVM, VM_FF_TO_R3);
-            VM_FF_CLEAR(pVM, VM_FF_TIMER);
+            VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TO_R3);
+            VMCPU_FF_CLEAR(pVCpu, VMCPU_FF_TIMER);
+            VM_FF_CLEAR(pVM, VM_FF_TM_VIRTUAL_SYNC);
 
             uint64_t TickThisStart = ASMReadTSC();
-            rc = SUPCallVMMR0Fast(pVM->pVMR0, VMMR0_DO_HWACC_RUN, 0);
+            rc = SUPR3CallVMMR0Fast(pVM->pVMR0, VMMR0_DO_HWACC_RUN, 0);
             uint64_t TickThisElapsed = ASMReadTSC() - TickThisStart;
             if (RT_FAILURE(rc))
             {
                 Log(("VMM: R0 returned fatal %Rrc in iteration %d\n", rc, i));
-                VMMR3FatalDump(pVM, rc);
+                VMMR3FatalDump(pVM, pVCpu, rc);
                 return rc;
             }
             if (TickThisElapsed < TickMin)
