@@ -1,10 +1,10 @@
-/* $Id: tstAnimate.cpp $ */
+/* $Id: tstAnimate.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VBox Animation Testcase / Tool.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -215,7 +211,7 @@ static DECLCALLBACK(int) scriptRun(PVM pVM, RTFILE File)
                     while (psz && *psz)
                     {
                         /* skip blanks. */
-                        while (isspace(*psz))
+                        while (RT_C_IS_SPACE(*psz))
                             psz++;
                         if (!*psz)
                             break;
@@ -234,7 +230,7 @@ static DECLCALLBACK(int) scriptRun(PVM pVM, RTFILE File)
                         {
                             /* strip end */
                             *pszEnd = '\0';
-                            while (pszEnd > psz && isspace(pszEnd[-1]))
+                            while (pszEnd > psz && RT_C_IS_SPACE(pszEnd[-1]))
                                 *--pszEnd = '\0';
 
                             /* process the line */
@@ -679,7 +675,7 @@ int main(int argc, char **argv)
 
                 case 'o':
                 {
-                    int rc = RTStrToUInt64Ex(argv[++i], NULL, 0, &offRawMem);
+                    rc = RTStrToUInt64Ex(argv[++i], NULL, 0, &offRawMem);
                     if (RT_FAILURE(rc))
                     {
                         RTPrintf("tstAnimate: Syntax error: Invalid offset given to -o.\n");
@@ -691,7 +687,7 @@ int main(int argc, char **argv)
                 case 'm':
                 {
                     char *pszNext;
-                    int rc = RTStrToUInt64Ex(argv[++i], &pszNext, 0, &cbMem);
+                    rc = RTStrToUInt64Ex(argv[++i], &pszNext, 0, &cbMem);
                     if (RT_FAILURE(rc))
                     {
                         RTPrintf("tstAnimate: Syntax error: Invalid memory size given to -m.\n");
@@ -725,7 +721,7 @@ int main(int argc, char **argv)
 
                 case 'w':
                 {
-                    int rc = RTStrToUInt32Ex(argv[++i], NULL, 0, &u32WarpDrive);
+                    rc = RTStrToUInt32Ex(argv[++i], NULL, 0, &u32WarpDrive);
                     if (RT_FAILURE(rc))
                     {
                         RTPrintf("tstAnimate: Syntax error: Invalid number given to -w.\n");
@@ -798,7 +794,7 @@ int main(int argc, char **argv)
     {
         if (FileRawMem != NIL_RTFILE)
         {
-            int rc = RTFileGetSize(FileRawMem, &cbMem);
+            rc = RTFileGetSize(FileRawMem, &cbMem);
             AssertReleaseRC(rc);
             cbMem -= offRawMem;
             cbMem &= ~(PAGE_SIZE - 1);
@@ -833,26 +829,19 @@ int main(int argc, char **argv)
         /*
          * Load memory.
          */
-        PVMREQ pReq1 = NULL;
         if (FileRawMem != NIL_RTFILE)
-            rc = VMR3ReqCall(pVM, VMCPUID_ANY, &pReq1, RT_INDEFINITE_WAIT, (PFNRT)loadMem, 3, pVM, FileRawMem, &offRawMem);
+            rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)loadMem, 3, pVM, FileRawMem, &offRawMem);
         else
-            rc = VMR3ReqCall(pVM, VMCPUID_ANY, &pReq1, RT_INDEFINITE_WAIT, (PFNRT)SSMR3Load, 5, pVM, pszSavedState, SSMAFTER_DEBUG_IT, (uintptr_t)NULL, (uintptr_t)NULL);
-        AssertReleaseRC(rc);
-        rc = pReq1->iStatus;
-        VMR3ReqFree(pReq1);
+            rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)SSMR3Load,
+                                 7, pVM, pszSavedState, (uintptr_t)NULL /*pStreamOps*/, (uintptr_t)NULL /*pvUser*/,
+                                 SSMAFTER_DEBUG_IT, (uintptr_t)NULL /*pfnProgress*/, (uintptr_t)NULL /*pvProgressUser*/);
         if (RT_SUCCESS(rc))
         {
             /*
              * Load register script.
              */
             if (FileScript != NIL_RTFILE)
-            {
-                rc = VMR3ReqCall(pVM, VMCPUID_ANY, &pReq1, RT_INDEFINITE_WAIT, (PFNRT)scriptRun, 2, pVM, FileScript);
-                AssertReleaseRC(rc);
-                rc = pReq1->iStatus;
-                VMR3ReqFree(pReq1);
-            }
+                rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)scriptRun, 2, pVM, FileScript);
             if (RT_SUCCESS(rc))
             {
                 if (fPowerOn)
@@ -876,10 +865,8 @@ int main(int argc, char **argv)
                     rc = REMR3DisasEnableStepping(pVM, true);
                     if (RT_SUCCESS(rc))
                     {
-                        rc = VMR3ReqCall(pVM, VMCPUID_ANY, &pReq1, RT_INDEFINITE_WAIT, (PFNRT)EMR3RawSetMode, 2, pVM, EMRAW_NONE);
+                        rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)EMR3RawSetMode, 2, pVM, EMRAW_NONE);
                         AssertReleaseRC(rc);
-                        VMR3ReqFree(pReq1);
-
                         DBGFR3Info(pVM, "cpumguest", "verbose", NULL);
                         if (fPowerOn)
                             rc = VMR3PowerOn(pVM);
