@@ -1,10 +1,10 @@
-/* $Id: tstVMM.cpp $ */
+/* $Id: tstVMM-HwAccm.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VMM Testcase.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -92,10 +88,8 @@ int main(int argc, char **argv)
          * Do testing.
          */
         RTPrintf(TESTCASE ": Testing...\n");
-        PVMREQ pReq1 = NULL;
-        rc = VMR3ReqCall(pVM, VMCPUID_ANY, &pReq1, RT_INDEFINITE_WAIT, (PFNRT)VMMDoHwAccmTest, 1, pVM);
+        rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)VMMDoHwAccmTest, 1, pVM);
         AssertRC(rc);
-        VMR3ReqFree(pReq1);
 
         STAMR3Dump(pVM, "*");
 
@@ -103,7 +97,7 @@ int main(int argc, char **argv)
          * Cleanup.
          */
         rc = VMR3Destroy(pVM);
-        if (!RT_SUCCESS(rc))
+        if (RT_FAILURE(rc))
         {
             RTPrintf(TESTCASE ": error: failed to destroy vm! rc=%d\n", rc);
             rcRet++;
