@@ -1,10 +1,10 @@
-/* $Id: tstVMREQ.cpp $ */
+/* $Id: tstVMREQ.cpp 28800 2010-04-27 08:22:32Z vboxsync $ */
 /** @file
  * VMM Testcase.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -13,10 +13,6 @@
  * Foundation, in version 2 as it comes in the "COPYING" file of the
  * VirtualBox OSE distribution. VirtualBox OSE is distributed in the
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -133,12 +129,8 @@ static void PassVA2(PVM pVM, const char *pszFormat, va_list va)
     void *pvVA = va2;
 #endif
 
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)PassVACallback, 5,
-                         pVM, _4K, _1G, pszFormat, pvVA);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)PassVACallback, 5, pVM, _4K, _1G, pszFormat, pvVA);
+    NOREF(rc);
 
 #if 1
     va_end(va2);
@@ -155,13 +147,9 @@ static void PassVA(PVM pVM, const char *pszFormat, ...)
     /* 1st test */
     va_list va1;
     va_start(va1, pszFormat);
-    PVMREQ pReq;
-    int rc = VMR3ReqCall(pVM, VMREQDEST_ANY, &pReq, RT_INDEFINITE_WAIT, (PFNRT)PassVACallback, 5,
-                         pVM, _4K, _1G, pszFormat, &va1);
-    if (RT_SUCCESS(rc))
-        rc = pReq->iStatus;
-    VMR3ReqFree(pReq);
+    int rc = VMR3ReqCallWait(pVM, VMCPUID_ANY, (PFNRT)PassVACallback, 5, pVM, _4K, _1G, pszFormat, &va1);
     va_end(va1);
+    NOREF(rc);
 
     /* 2nd test */
     va_list va2;
@@ -185,7 +173,7 @@ static DECLCALLBACK(int) Thread(RTTHREAD Thread, void *pvUser)
         unsigned        iReq;
         for (iReq = 0; iReq < cReqs; iReq++)
         {
-            rc = VMR3ReqAlloc(pVM, &apReq[iReq], VMREQTYPE_INTERNAL, VMREQDEST_ANY);
+            rc = VMR3ReqAlloc(pVM, &apReq[iReq], VMREQTYPE_INTERNAL, VMCPUID_ANY);
             if (RT_FAILURE(rc))
             {
                 RTPrintf(TESTCASE ": i=%d iReq=%d cReqs=%d rc=%Rrc (alloc)\n", i, iReq, cReqs, rc);
