@@ -1,4 +1,4 @@
-/* $Id: testmath.c 13840 2008-11-05 03:31:46Z vboxsync $ */
+/* $Id: testmath.c 25528 2009-12-20 23:24:59Z vboxsync $ */
 /** @file
  * Testcase for the no-crt math stuff.
  */
@@ -11,7 +11,7 @@
 # include <iprt/assert.h>
 # include <math.h>
 # undef printf
-# define printf AssertMsg2
+# define printf RTAssertMsg2Weak
 #else
 # include <stdio.h>
 # include <math.h>
@@ -277,6 +277,8 @@ extern int testmath(void)
     CHECKLL(testremainder(), 1);
 
 
+    /* Only works in extended precision, while double precesion is defaul on BSD (including Darwin) */
+    set_cw(0x37f);
     CHECK(rintl(1.0L), 1.0);
     CHECK(rintl(1.4L), 1.0);
     CHECK(rintl(1.3L), 1.0);
@@ -284,6 +286,7 @@ extern int testmath(void)
     CHECK(rintl(3123.1232L), 3123.0);
     CHECK(rint(3985.13454), 3985.0);
     CHECK(rintf(9999.999), 10000.0);
+    set_cw(0x27f);
 
     CHECK(sinl(1.0L),  0.84147098480789650664L);
     lrd = 180.0L;
